/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2016 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
SET SCAN OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

DECLARE

  --
  -- Local constants and variables
  --
  l_SERVICE CONSTANT xxas_com_service_repository.service_name%TYPE
                       := 'AbsenceEvent';

BEGIN

  xxas_com_sr_pkg.set_rest_service
    (p_service_name       => l_SERVICE
    ,p_description        => 'AbsenceEvent'
    ,p_consumes           => 'application/xml'
    ,p_produces           => 'application/xml'
    ,p_path               => 'absence-event');

  xxas_com_sr_pkg.set_rest_method
    (p_service_name       => l_SERVICE
    ,p_method_name        => 'GetAbsenceEventReasons'
    ,p_description        => 'Get the Absence Event Reasons'
    ,p_path               => 'reasons'
    ,p_methods            => 'GET'
    ,p_remove_root_ele    => true
    ,p_sql                =>
      q'[
         SELECT XMLElement
               ("data"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("AbsenceEventReason"
                   ,XMLForest
                     (rslt.abs_attendance_reason_id "id"
                     ,rslt.meaning                  "value"
                     ,rslt.meaning                  "label")
                   )
                 )
               ) "data"
         FROM  (SELECT DISTINCT paar.abs_attendance_reason_id
                               ,hl.meaning
                FROM   hr_lookups hl
                      ,per_abs_attendance_reasons paar
                      ,per_all_people_f papf
                WHERE hl.lookup_type                  = 'ABSENCE_REASON'
                AND   paar.business_group_id          = papf.business_group_id
                AND   papf.person_id                  = {requestParam.personId}
                AND   trunc(sysdate) BETWEEN papf.effective_start_date
                                     AND     papf.effective_end_date
                AND   paar.name                       = hl.lookup_code
                AND   paar.absence_attendance_type_id = {requestParam.absAttTypeId}
                AND   trunc(sysdate) BETWEEN nvl(hl.start_date_active,trunc(sysdate))
                                     AND     nvl(hl.end_date_active,trunc(sysdate))) rslt

      ]'
    );

  xxas_com_sr_pkg.set_rest_method
    (p_service_name       => l_SERVICE
    ,p_method_name        => 'AbsenceEntryForm'
    ,p_description        => 'Get Absence Entry Form'
    ,p_methods            => 'GET'
    ,p_remove_root_ele    => true
    ,p_path               => 'entry-form'
    ,p_sql                =>
      q'[
          SELECT xxas_absence_event_pkg.entry_form
                   ({requestParam.personId}
                   ,{requestParam.startTime}
                   ,{requestParam.endTime}
                   ,{requestParam.sourceBlock}
                   ,{requestParam.savedSearchId}
                   ,{requestParam.managerId}
                   ,{requestParam.defAbsTypeFlag}) "data"
          FROM   dual
      ]'
    );

  xxas_com_sr_pkg.set_rest_method
    (p_service_name       => l_SERVICE
    ,p_method_name        => 'AbsenceEntryView'
    ,p_description        => 'Get Absence Entry View'
    ,p_methods            => 'GET'
    ,p_remove_root_ele    => true
    ,p_path               => 'entry-view/{personId}/abs/{absAttendanceId}/transaction/{transAndStepId}'
    ,p_sql                =>
      q'[
          SELECT xxas_absence_event_pkg.entry_view
                   ({pathParam.personId}
                   ,{pathParam.absAttendanceId}
                   ,{pathParam.transAndStepId}
                   ,{requestParam.sourceBlock}
                   ,{requestParam.savedSearchId}
                   ,{requestParam.managerId}) "data"
          FROM   dual
      ]'
    );

  xxas_com_sr_pkg.set_rest_method
    (p_service_name       => l_SERVICE
    ,p_method_name        => 'AbsenceEntryEdit'
    ,p_description        => 'Get Absence Entry Edit'
    ,p_methods            => 'GET'
    ,p_remove_root_ele    => true
    ,p_path               => 'entry-edit/{personId}/abs/{absAttendanceId}/transaction/{transAndStepId}'
    ,p_sql                =>
      q'[
          SELECT xxas_absence_event_pkg.entry_edit
                   ({pathParam.personId}
                   ,{pathParam.absAttendanceId}
                   ,{pathParam.transAndStepId}
                   ,{requestParam.times}
                   ,{requestParam.sourceBlock}
                   ,{requestParam.savedSearchId}
                   ,{requestParam.managerId}) "data"
          FROM   dual
      ]'
    );

  xxas_com_sr_pkg.set_rest_method
    (p_service_name       => l_SERVICE
    ,p_method_name        => 'AbsenceEntryReview'
    ,p_description        => 'Get Absence Entry Review'
    ,p_methods            => 'GET'
    ,p_remove_root_ele    => true
    ,p_path               => 'entry-review/{personId}/abs/{absAttendanceId}/transaction/{transAndStepId}'
    ,p_sql                =>
      q'[
          SELECT xxas_absence_event_pkg.entry_review
                   ({pathParam.personId}
                   ,{pathParam.absAttendanceId}
                   ,{pathParam.transAndStepId}
                   ,{requestParam.sourceBlock}
                   ,{requestParam.savedSearchId}
                   ,{requestParam.managerId}) "data"
          FROM   dual
      ]'
    );

  xxas_com_sr_pkg.set_rest_method
    (p_service_name       => l_SERVICE
    ,p_method_name        => 'DeleteAbsenceEntry'
    ,p_description        => 'Deletes a Absence Entry'
    ,p_path               => 'entry/{personId}/abs/{absAttendanceId}'
    ,p_methods            => 'DELETE'
    ,p_plsql              =>
      q'[
        DECLARE
          l_resp          xxas_http_resp_obj;
          l_response_xml  XMLType;
          l_function_id   number;
        BEGIN

          -- Check access to delete the absence entry

          IF xxas_com_sec_pkg.test_instance('XXAS_DT_ABSENCE_DELETE',{pathParam.personId},{pathParam.absAttendanceId}) = 'Y' THEN

            l_function_id  := xxas_cal_util_pkg.page_slug_func_id('absence-edit');

            l_response_xml := xxas_absence_event_pkg.delete_absence_event
                                ({pathParam.absAttendanceId}
                                ,l_function_id
                                ,replace({requestParam.sourceBlock},'_','$')
                                ,{requestParam.savedSearchId}
                                ,null
                                ,{requestParam.managerId});

          END IF;

          l_resp := xxas_http_response
                      .ok(l_response_xml
                         )
                      .build();

         {response} := l_resp;

        END
      ]'
    );

END;
/
COMMIT;
EXIT;
