/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2016 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE TYPE BODY xxas_actions IS
  
-- ------------------------------------------------------------------------ *
-- Name    : xxas_actions
-- ------------------------------------------------------------------------ *
CONSTRUCTOR FUNCTION xxas_actions RETURN SELF AS RESULT
IS  

BEGIN
  self.actions := xxas_action_table(); 

  RETURN; 
END xxas_actions;

-- ------------------------------------------------------------------------ *
-- Name    : xxas_actions
-- ------------------------------------------------------------------------ *
CONSTRUCTOR FUNCTION xxas_actions
  (p_action_table IN xxas_action_table) RETURN SELF AS RESULT
IS  

BEGIN

  IF p_action_table IS NULL THEN
    self.actions := xxas_action_table();
  ELSE
    self.actions := p_action_table;
  END IF;  

  RETURN; 
END xxas_actions;

-- ------------------------------------------------------------------------ *
-- Name    : add_action
-- ------------------------------------------------------------------------ *
MEMBER PROCEDURE add_action
    (p_action_id            IN varchar2
    ,p_order_sequence       IN number   DEFAULT NULL
    ,p_function_id          IN number   DEFAULT NULL
    ,p_function_name        IN varchar2 DEFAULT NULL
    ,p_icon                 IN varchar2 DEFAULT NULL
    ,p_obj_id               IN number   DEFAULT NULL
    ,p_obj_name             IN varchar2 DEFAULT NULL
    ,p_action_title         IN varchar2 DEFAULT NULL
    ,p_action_title_message IN varchar2 DEFAULT NULL
    ,p_bulk_action_enabled  IN varchar2 DEFAULT NULL
    ,p_operation            IN varchar2
    ,p_href                 IN varchar2 DEFAULT NULL
    ,p_page_id              IN varchar2 DEFAULT NULL
    ,p_object_id            IN varchar2 DEFAULT NULL
    ,p_target               IN varchar2 DEFAULT NULL
    ,p_color                IN varchar2
    ,p_weight               IN varchar2
    ,p_confirmation_flag    IN varchar2 DEFAULT NULL
    ,p_confirmation_text    IN varchar2 DEFAULT NULL
    ,p_confirmation_message IN varchar2 DEFAULT NULL
    ,p_inline_expression    IN varchar2 DEFAULT NULL
	,p_modal_width          IN varchar2 DEFAULT NULL)   
IS
BEGIN

  actions.extend;
  actions(actions.count) :=
    xxas_action_type
      (action_id              => p_action_id
      ,order_sequence         => nvl(p_order_sequence, actions.count)
      ,function_id            => p_function_id
      ,function_name          => p_function_name
      ,icon                   => p_icon
      ,obj_id                 => p_obj_id
      ,obj_name               => p_obj_name
      ,action_title           => p_action_title
      ,action_title_message   => p_action_title_message
      ,bulk_action_enabled    => CASE WHEN p_bulk_action_enabled IS NOT NULL
                                      THEN p_bulk_action_enabled
                                      WHEN upper(p_operation) = 'LINK' OR
                                           upper(p_operation) = 'GET'
                                      THEN 'N'
                                      ELSE 'Y'
                                 END
      ,operation              => p_operation
      ,href                   => p_href
      ,page_id                => p_page_id
      ,object_id              => p_object_id
      ,target                 => p_target
      ,color                  => p_color
      ,weight                 => p_weight
      ,confirmation_flag      => p_confirmation_flag
      ,confirmation_text      => p_confirmation_text
      ,confirmation_message   => p_confirmation_message
      ,inline_expression      => p_inline_expression
	  ,modal_width            => p_modal_width);

END add_action;

-- ------------------------------------------------------------------------ *
-- Name    : get_xml
-- ------------------------------------------------------------------------ *
MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  
  SELECT XMLELEMENT
          ("actions"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("action"
              ,XMLForest
                (decode(a.function_id,null,a.action_id,a.function_id)                      "id"
                ,a.action_title                     "title"
                ,xxas_util_pkg.flag_to_boolean
                  (nvl(a.bulk_action_enabled, 'Y')) "bulkActionEnabled"
                ,a.operation                        "operation"
                ,a.href                             "href"
                ,a.page_id                          "pageId"
                ,a.object_id                        "objectId"
                ,a.target                           "target"
                ,a.inline_expression                "inlineExpression"
				,a.modal_width                      "modalWidth")
              ,XMLElement
                ("meta"
                ,XMLElement
                  ("options"
                  ,XMLForest
                    (a.color                        "color"
                    ,a.weight                       "weight"
                    ,a.icon                         "icon")
                  )
                )
              ) order by a.order_sequence
            )
          )
  INTO   l_xml   
  FROM   TABLE(actions) a;

  RETURN l_xml;
END get_xml;
  
END;
/

COMMIT;
EXIT;
