/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2015 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
SET ESCAPE '\'
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

DECLARE

  --
  -- Local constants and variables
  --
  e_no_bg_id          EXCEPTION;
  e_bg_not_exists     EXCEPTION;
  e_ff_type_not_found EXCEPTION;
  l_BG_ID   CONSTANT  NUMBER        := to_number('&1');
  l_USER_ID CONSTANT  NUMBER        := to_number('&2');
  l_START   CONSTANT  DATE          := to_date('1900-01-01','YYYY-MM-DD');
  l_END     CONSTANT  DATE          := xxas_util_pkg.eot;
  l_FF_NAME CONSTANT  VARCHAR2(80)  := 'XXAS_COM_WL_COLORS';
  l_FF_DESC CONSTANT  VARCHAR2(240) := 'Applaud Set Worklist Colors for html5';
  l_FF_TYPE CONSTANT  varchar2(30)  := 'Applaud Person';
  l_formula_text      LONG;
  l_formula_type_id   NUMBER;
  l_ff_id             NUMBER;
  l_bg_exists         VARCHAR2(1);

  --
  -- Tests if the Business Group already exists
  --
  CURSOR csr_bg_exists IS
  SELECT 'Y'
  FROM   per_business_groups_perf bg
  WHERE  bg.business_group_id = l_BG_ID;

  --
  -- Gets the FF Id
  --
  CURSOR csr_ff_id IS
  SELECT ff.formula_id
  FROM   ff_formulas_f ff
  WHERE  ff.business_group_id = l_BG_ID
  AND    ff.formula_name = l_FF_NAME;

  --
  -- Gets the Formula Type Id
  --
  CURSOR csr_ff_type IS
  SELECT fft.formula_type_id
  FROM   ff_formula_types fft
  WHERE  fft.formula_type_name = l_FF_TYPE;

BEGIN

  --
  -- Check that the Business Group Id was passed in
  --
  IF l_BG_ID IS NULL THEN
    RAISE e_no_bg_id;
  END IF;

  --
  -- Check the Business Group exists
  --
  OPEN  csr_bg_exists;
  FETCH csr_bg_exists INTO l_bg_exists;
  CLOSE csr_bg_exists;

  IF l_bg_exists IS NULL THEN
    RAISE e_bg_not_exists;
  END IF;

  --
  -- Fetch the Fast Formula Id
  --
  OPEN  csr_ff_id;
  FETCH csr_ff_id INTO l_ff_id;
  CLOSE csr_ff_id;

  --
  -- Get the Formula Type; error if it cannot be found
  --
  OPEN  csr_ff_type;
  FETCH csr_ff_type INTO l_formula_type_id;
  CLOSE csr_ff_type;

  IF l_formula_type_id IS NULL THEN
    RAISE e_ff_type_not_found;
  END IF;

  --
  -- Set the Formula Text
  --
  l_formula_text := q'[
/*==============================================================================

Formula Name: XXAS_COM_WL_COLORS
Formula Type: Applaud Person
Description:  Sets the color of Worklist Response Choices for html5

==============================================================================*/

Default for Lookup_Type is 'NONE'
Default for Lookup_Code is 'NONE'
Default for Meaning     is 'NONE'

Inputs are Lookup_Type, Lookup_Code, Meaning

Color = 'positive'
weight = 'med'
Icon = 'paper-plane'

If
   upper(Lookup_Code) like '%NOT%RESOLVED%' Or
   upper(Lookup_Code) like '%DISAPPROVED%' Or
   upper(Lookup_Code) like '%NOT%APPROVE%' Or
   upper(Lookup_Code) like '%NOT%FIXED%' Or
   upper(Lookup_Code) like '%NOT%PROCEED%' Or
   upper(Lookup_Code) like '%NOT%ACCEPT%' Or
   upper(Lookup_Code) like '%REJECT%' Or
   upper(Lookup_Code) like '%STOP%' Or
   upper(Lookup_Code) = 'N' Or
   upper(Lookup_Code) = 'NO' Or
   upper(Lookup_Code) = 'B-NO' Or
   upper(Lookup_Code) like '%TERMINATE%' Or
   upper(Lookup_Code) like '%ABORT%' Or
   upper(Lookup_Code) like '%CANCEL%' Or
   upper(Lookup_Code) like '%NOT_RECEIVED%' Or
   upper(Lookup_Code) like '%ESCALATE%' Or
   upper(Lookup_Code) like '%FAIL%' Or
   upper(Lookup_Code) like '%DECLINE%' Or
   upper(Lookup_Code) like '%ROLLBACK%' Or
   upper(Lookup_Code) like '%ROLLABCK%' Or
   upper(Lookup_Code) like '%DELETE%' Or
   upper(Lookup_Code) = 'DEL' then
(
  Color = 'assertive'
  Weight = 'med'
  Icon = 'validationerror'
)
Else If
   upper(Lookup_Code) like '%APPROVE%' Or
   upper(Lookup_Code) like '%DONE%' Or
   upper(Lookup_Code) = 'Y' Or
   upper(Lookup_Code) like '%YES%' Or
   upper(Lookup_Code) = 'OK' Or
   upper(Lookup_Code) like '%FIXED%' Or
   upper(Lookup_Code) like '%PROCEED%' Or
   upper(Lookup_Code) like '%RESOLVED%' Or
   upper(Lookup_Code) like '%ACCEPT%' Or
   upper(Lookup_Code) like '%COMPLETED%' Or
   upper(Lookup_Code) like '%FINISH%' Or
   upper(Lookup_Code) = 'PASS' then
(
  Color = 'balanced'
  Weight = 'med'
  Icon = 'tick'
)
Else IF 
   upper(Lookup_Code) like 'TRANSFER' OR
   upper(Lookup_Code) like 'RMI' OR
   upper(Lookup_Code) like 'RMIT' THEN
   (
     Color = 'energized'
	 Icon = 'randomup'
   )
ELSE IF
   upper(Lookup_Code) like '%CLOSE%' Or
   upper(Lookup_Code) like '%OK%' THEN
   (
     Color = 'positive'
	 Icon = 'close'
   )
Else IF
   upper(Lookup_Code) like 'REASSIGN' THEN
   (
     Color = 'energized'
	 Icon = 'reassign'
   )
Else IF
   upper(Lookup_Code) like 'FORWARD' THEN
   (
     Icon = 'forwardto'
   )
Else IF
  upper(Lookup_Code) like 'CONT_ACT' THEN
   (
     Icon = 'edit'
   )
   
IF upper(Lookup_Code) like 'FORWARD' THEN
 (

  Allow_Quick_Action = 'N'

 )
 
Else IF upper(Lookup_Code) like 'APPROVE_AND_FORWARD' THEN
 (

  Allow_Quick_Action = 'N'

 )
   
Return Color,Weight,Icon,Allow_Quick_Action

]';



  BEGIN

    IF l_ff_id IS NULL THEN
      --
      -- Create the Fast Formula
      --
      INSERT INTO ff_formulas_f
        (formula_id
        ,effective_start_date
        ,effective_end_date
        ,business_group_id
        ,formula_type_id
        ,formula_name
        ,description
        ,formula_text)
      VALUES
        (ff_formulas_s.nextval
        ,l_START
        ,l_END
        ,l_BG_ID
        ,l_formula_type_id
        ,l_FF_NAME
        ,l_FF_DESC
        ,l_formula_text);

    ELSE
      --
      -- Update the existing Fast Formula text
      -- (if it has multiple date-track versions the text of
      -- all versions is updated).
      --
      UPDATE ff_formulas_f
      SET    formula_text = l_formula_text
            ,description  = l_FF_DESC
      WHERE  formula_id = l_ff_id;

    END IF;

    COMMIT;

    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        RAISE;
    END;

    --
    -- Compile the Fast Formula
    --
    xxas_util_pkg.compile_ff
      (p_user_id => l_USER_ID
      ,p_ff_type => l_FF_TYPE
      ,p_ff_name => l_FF_NAME);

EXCEPTION

  WHEN e_no_bg_id THEN
    RAISE no_data_found;

  WHEN e_bg_not_exists THEN
    RAISE no_data_found;

  WHEN e_ff_type_not_found THEN
    RAISE no_data_found;

END;
/
COMMIT;
EXIT;
