/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2013 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

DECLARE

  --
  -- Local constants and variables
  --
  e_no_bg_id          EXCEPTION;
  e_bg_not_exists     EXCEPTION;
  e_ff_type_not_found EXCEPTION;
  l_BG_ID   CONSTANT  NUMBER        := to_number('&1');
  l_USER_ID CONSTANT  NUMBER        := to_number('&2');
  l_START   CONSTANT  DATE          := to_date('0001-01-01','YYYY-MM-DD');
  l_END     CONSTANT  DATE          := xxas_util_pkg.eot;
  l_FF_NAME CONSTANT  VARCHAR2(80)  := 'XXAS_EMP_RES_VIEW';
  l_FF_DESC CONSTANT  VARCHAR2(240) := 'Resume EmployeeCenter View';
  l_FF_TYPE CONSTANT  varchar2(30)  := 'Applaud Assignment';
  l_formula_text      LONG;
  l_formula_type_id   NUMBER;
  l_ff_id             NUMBER;
  l_bg_exists         VARCHAR2(1);

  --
  -- Gets the FF Id
  --
  CURSOR csr_ff_id IS
  SELECT ff.formula_id
  FROM   ff_formulas_f ff
  WHERE  ff.business_group_id = l_BG_ID
  AND    ff.formula_name = l_FF_NAME;

  --
  -- Tests if the Business Group already exists
  --
  CURSOR csr_bg_exists IS
  SELECT 'Y'
  FROM   per_business_groups_perf bg
  WHERE  bg.business_group_id = l_BG_ID;

  --
  -- Gets the Formula Type Id
  --
  CURSOR csr_ff_type IS
  SELECT fft.formula_type_id
  FROM   ff_formula_types fft
  WHERE  fft.formula_type_name = l_FF_TYPE;

BEGIN

  --
  -- Check that the Business Group Id was passed in
  --
  IF l_BG_ID IS NULL THEN
    RAISE e_no_bg_id;
  END IF;

  --
  -- Check the Business Group exists
  --
  OPEN  csr_bg_exists;
  FETCH csr_bg_exists INTO l_bg_exists;
  CLOSE csr_bg_exists;

  IF l_bg_exists IS NULL THEN
    RAISE e_bg_not_exists;
  END IF;

  --
  -- Fetch the Fast Formula Id
  --
  OPEN  csr_ff_id;
  FETCH csr_ff_id INTO l_ff_id;
  CLOSE csr_ff_id;

  --
  -- Get the Formula Type; error if it cannot be found
  --
  OPEN  csr_ff_type;
  FETCH csr_ff_type INTO l_formula_type_id;
  CLOSE csr_ff_type;

  IF l_formula_type_id IS NULL THEN
    RAISE e_ff_type_not_found;
  END IF;

  --
  -- Set the Formula Text
  --
  l_formula_text := '
/*==============================================================================

Formula Name: XXAS_EMP_RES_VIEW
Formula Type: Applaud Assignment
Description:  Resume EmployeeCenter View

==============================================================================*/

  Default for Title                      is ''Untitled''
  Default for File_Name                  is ''<no filename>''
  Default for Description                is ''No description''
  Default for File_URL                   is ''None''
  Default for Creation_Date              is ''0001/01/01 00:00:00'' (date)
  Default for Last_Update_Date           is ''0001/01/01 00:00:00'' (date)
  Default for Created_By                 is ''Unknown''
  Default for Last_Updated_By            is ''Unknown''

  Inputs are Title, File_Name, Description, File_URL,
             Creation_Date, Last_Update_Date, Created_By, Last_Updated_By

  DATE_MASK      = ''FmDd Month YYYY''
  MSG_APP        = ''XXAS''

  Attachments_Label = xxas_get_message(''XXAS'',''XXAS_EMP_RESUME'')
  Current_Field_Num = 1

  /* Set the summary text and description */
  If Title was defaulted and File_Name was not defaulted Then
    (Summary_Text = File_Name)
  Else
    (Summary_Text = Title)

  If Description was not defaulted Then
    (Rslt = xxas_emp_set_long_output
      (''SUMMARY_MESSAGE'', ''XXAS'', ''XXAS_COM_TEXT'', ''TEXT'', Description))
  Else If Title was not defaulted and File_Name was not defaulted then
    (Rslt = xxas_emp_set_long_output
      (''SUMMARY_MESSAGE'', ''XXAS'', ''XXAS_COM_TEXT'', ''TEXT'', File_Name))

  /* Set the Title field */
  If Title was not defaulted Then
    (Func_Result = xxas_emp_set_text_field
      (Current_Field_Num, xxas_get_message(''XXAS'',''XXAS_RB_EMP_EXP_JT''), Title))

  /* Set the File Name field */
  If File_Name was not defaulted Then
    (Func_Result = xxas_emp_set_text_field
      (Current_Field_Num, xxas_get_message(''XXAS'',''XXAS_EMP_FILE_NAME''), File_Name))

  /* Set the Description field */
  If Description was not defaulted Then
    (Func_Result = xxas_emp_set_text_field
      (Current_Field_Num, xxas_get_message(''XXAS'',''XXAS_RB_EMP_EXP_DESC''), Description))

  /* Set the update/delete permissions */
  Edit_Allowed = xxas_ess_mss_policy(''Y'', ''Y'')
  Update_Allowed = Edit_Allowed
  Delete_Allowed = Edit_Allowed

  Return Update_Allowed, Delete_Allowed,
         Summary_Text, Attachments_Label
';


  BEGIN

    IF l_ff_id IS NULL THEN
      --
      -- Create the Fast Formula
      --
      INSERT INTO ff_formulas_f
        (formula_id
        ,effective_start_date
        ,effective_end_date
        ,business_group_id
        ,formula_type_id
        ,formula_name
        ,description
        ,formula_text)
      VALUES
        (ff_formulas_s.nextval
        ,l_START
        ,l_END
        ,l_BG_ID
        ,l_formula_type_id
        ,l_FF_NAME
        ,l_FF_DESC
        ,l_formula_text);

    ELSE
      --
      -- Update the existing Fast Formula text
      -- (if it has multiple date-track versions the text of
      -- all versions is updated).
      --
      UPDATE ff_formulas_f
      SET    formula_text = l_formula_text
            ,description  = l_FF_DESC
      WHERE  formula_id = l_ff_id;

    END IF;

    COMMIT;

    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        RAISE;
    END;

    --
    -- Compile the Fast Formula
    --
    xxas_util_pkg.compile_ff
      (p_user_id => l_USER_ID
      ,p_ff_type => l_FF_TYPE
      ,p_ff_name => l_FF_NAME);

EXCEPTION

  WHEN e_no_bg_id THEN
    RAISE no_data_found;

  WHEN e_bg_not_exists THEN
    RAISE no_data_found;

  WHEN e_ff_type_not_found THEN
    RAISE no_data_found;

END;
/
COMMIT;
EXIT;
