/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2018 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
SET SCAN OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_gb_reports_pkg AS 

gc_log xxas_com_log_type := xxas_com_log_type('xxas_gb_reports_pkg');

--
-- Private Global Types and Constants
--
g_PACKAGE_NAME         CONSTANT varchar2(30) := 'xxas_gb_reports_pkg';
g_FORM_LAYOUT          CONSTANT varchar2(4)  := 'FORM';
g_TABULAR_LAYOUT       CONSTANT varchar2(7)  := 'TABULAR';
g_P11D_DATASOURCE_CODE CONSTANT varchar2(9)  := 'PERGBP11D';

--
-- ------------------------------------------------------------------------ *
-- Name    : log_start
-- ------------------------------------------------------------------------ *
PROCEDURE log_start
  (p_method      IN  varchar2
  ,p_parameter1  IN  varchar2 DEFAULT null
  ,p_parameter2  IN  varchar2 DEFAULT null
  ,p_parameter3  IN  varchar2 DEFAULT null
  ,p_parameter4  IN  varchar2 DEFAULT null
  ,p_parameter5  IN  varchar2 DEFAULT null
  ,p_parameter6  IN  varchar2 DEFAULT null
  ,p_parameter7  IN  varchar2 DEFAULT null
  ,p_parameter8  IN  varchar2 DEFAULT null
  ,p_parameter9  IN  varchar2 DEFAULT null
  ,p_parameter10 IN  varchar2 DEFAULT null)
IS
BEGIN
   IF gc_log.proc=1 
   THEN 
     gc_log.procedure_start_params
       (p_proc_name   =>  p_method
       ,p_parameter1  =>  p_parameter1
       ,p_parameter2  =>  p_parameter2
       ,p_parameter3  =>  p_parameter3
       ,p_parameter4  =>  p_parameter4
       ,p_parameter5  =>  p_parameter5
       ,p_parameter6  =>  p_parameter6
       ,p_parameter7  =>  p_parameter7
       ,p_parameter8  =>  p_parameter8
       ,p_parameter9  =>  p_parameter9
       ,p_parameter10 =>  p_parameter10); 
   END IF;  
END log_start; 

--
-- ------------------------------------------------------------------------ *
-- Name    : log_end
-- ------------------------------------------------------------------------ *
PROCEDURE log_end
  (p_method  IN  varchar2
  ,p_result  IN  varchar2 DEFAULT null)
IS
BEGIN
  IF gc_log.proc=1 
  THEN 
    gc_log.procedure_end_result
      (p_proc_name =>  p_method
      ,p_result    =>  p_result); 
  END IF;
END log_end;  

--
-- ------------------------------------------------------------------------ *
-- Name    : log_statement
-- ------------------------------------------------------------------------ *
PROCEDURE log_statement
  (p_method     IN  varchar2
  ,p_log_string IN  varchar2)
IS
BEGIN
  IF gc_log.stmt=1 
  THEN 
    gc_log.statement
      (p_proc_name  =>  p_method
      ,p_log_string =>  p_log_string); 
  END IF;
END log_statement;

--
-- ------------------------------------------------------------------------ *
-- Name    : p11dreport_url
-- ------------------------------------------------------------------------ *
FUNCTION p11dreport_url
  (p_assignment_id   IN number
  ,p_p11d_report_id  IN number
  ,p_template        IN varchar2
  ,p_inline          IN varchar2 DEFAULT 'N'
  ,p_rest_api_path   IN varchar2 DEFAULT  NULL ) RETURN varchar2
IS

  c_method CONSTANT varchar2(30) := 'p11dreport_url';
  
BEGIN

  log_start(c_method,p_assignment_id,p_p11d_report_id,p_template,p_inline,p_rest_api_path);

  IF p_rest_api_path IS NOT NULL THEN
    log_end(c_method);
    RETURN utl_url.unescape(xxas_com_rest_util_pkg.uri_from_path(p_rest_api_path), xxas_util_pkg.get_db_charset);
  END IF;

  IF p_assignment_id   IS NULL OR
     p_p11d_report_id  IS NULL OR
     p_template        IS NULL THEN
    log_end(c_method);
    RETURN null;
  ELSE
    log_end(c_method);
    RETURN xxas_com_rest_util_pkg.uri_from_path
              ('assignments/'
            || to_char(p_assignment_id)
            || '/p11dreports/'
            || to_char(p_p11d_report_id)
            || '/'
            || p_template
            || '?_accept=application/pdf'
            || CASE WHEN p_inline = 'Y' THEN
                 '&inline=true'
               END
            || CASE WHEN p_template is not null THEN
                 '&biPublisherDataSourceCode=' || p_template
               END
            || CASE WHEN p_template is not null THEN
                 '&actionContextId=' || p_p11d_report_id
               END);
  END IF;

END p11dreport_url;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_p11dreports_tbl
-- ------------------------------------------------------------------------ *
FUNCTION get_p11dreports_tbl
  (p_assignment_id     IN number
  ,p_action_context_id IN number DEFAULT NULL) RETURN xxas_p11d_report_varray PIPELINED
IS

  CURSOR csr_p11d_reports_data
           (cp_assignment_id IN number)
  IS
  SELECT pav.row_id
        ,pav.effective_date
        ,pav.period_name
        ,pav.action_type
        ,pav.type
        ,pav.assignment_action_id
        ,pav.pre_payment_id
        ,pav.action_status
        ,pav.status_code
        ,pav.messages_exist
        ,pav.assignment_id
        ,pav.payroll_action_id
  FROM   pay_payroll_actions a 
        ,pay_assignment_actions_v pav
  WHERE  a.report_type = 'P11D' 
  AND    a.report_qualifier = 'GB' 
  AND    a.report_category IN('P11D_ARCHIVE_SS' ,'ARCHIVE')
  AND    a.action_status = 'C' 
  AND    a.action_type   = 'X' 
  --AND a.business_group_id +0  = 202
  --:$PROFILES$.PER_BUSINESS_GROUP_ID 
  AND    pav.payroll_action_id = a.payroll_action_id
  AND    pav.assignment_id = cp_assignment_id
  ORDER BY pav.effective_date desc;

  c_method CONSTANT       varchar2(30) := 'get_p11dreports_tbl';
  e_null_params           exception;
  l_row                   xxas_p11d_report_type;
  l_template_code         xdo_templates_vl.template_code%TYPE;

BEGIN

  log_start(c_method,p_assignment_id,p_action_context_id);

  IF p_assignment_id IS NULL THEN
    RAISE e_null_params;
  END IF;

  FOR p11d_report_rec IN csr_p11d_reports_data(p_assignment_id)
  LOOP

    log_statement(c_method,'Effective Date: ' || p11d_report_rec.effective_date);
    log_statement(c_method,'Period Name: ' || p11d_report_rec.period_name);
    log_statement(c_method,'Payroll Action Id: ' || p11d_report_rec.payroll_action_id);
    log_statement(c_method,'Assignment Action Id: ' || p11d_report_rec.assignment_action_id);

    l_row := xxas_p11d_report_type
               (p11d_report_id  => p11d_report_rec.assignment_action_id
               ,template        => g_P11D_DATASOURCE_CODE
               ,summary_context => null
               ,summary_text    => p11d_report_rec.period_name
               ,summary_message => null
               ,rest_api_path   => null);
    PIPE ROW (l_row);

  END LOOP;

  log_end(c_method);

  RETURN;

EXCEPTION

  WHEN e_null_params THEN

    log_statement(c_method,'e_null_params exception');
    log_end(c_method);
    RETURN;

END get_p11dreports_tbl;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_p11d_report_details_xml
-- ------------------------------------------------------------------------ *
FUNCTION get_p11d_report_details_xml
  (p_assignment_id     IN number
  ,p_action_context_id IN number
  ,p_template          IN varchar2) RETURN XMLType
IS

  c_method CONSTANT varchar2(30) := 'get_p11d_report_details_xml';
  l_details_xml     XMLType;

BEGIN

  log_start(c_method, p_assignment_id, p_action_context_id, p_template);

  log_end(c_method);
  RETURN l_details_xml;
  
END get_p11d_report_details_xml;

--
-- ------------------------------------------------------------------------ *
-- Name    : report_run_effective_year
-- Purpose : Retrieves the Report run Effective year for the context ID
--
-- * @param p_action_context_id Optional Id of individual report
-- * @rep:scope internal
-- * @rep:displayname Get P11d Reports
-- ------------------------------------------------------------------------ *
FUNCTION report_run_effective_year
  (p_action_context_id IN number) RETURN varchar2
IS

  CURSOR csr_effective_year
           (cp_action_context_id IN number)
  IS
  SELECT to_char(pav.effective_date,'YY') --to_char(to_number(to_char(pav.effective_date,'YY')+1))
  FROM   pay_payroll_actions a 
        ,pay_assignment_actions_v pav
  WHERE  a.report_type = 'P11D' 
  AND    a.report_qualifier = 'GB' 
  AND    a.report_category IN('P11D_ARCHIVE_SS' ,'ARCHIVE')
  AND    a.action_status = 'C' 
  AND    a.action_type   = 'X'
  AND    pav.payroll_action_id = a.payroll_action_id
  and    pav.assignment_action_id = cp_action_context_id
  ORDER BY pav.effective_date desc;

  c_method CONSTANT varchar2(30) := 'report_run_effective_year';
  l_effective_year  varchar2(4);

BEGIN

  log_start(c_method, p_action_context_id);

  OPEN  csr_effective_year(p_action_context_id);
  FETCH csr_effective_year INTO l_effective_year;
  CLOSE csr_effective_year;

  log_end(c_method,l_effective_year);
  RETURN l_effective_year;
  
END report_run_effective_year;

--
-- ------------------------------------------------------------------------ *
-- Name    : p60report_url
-- ------------------------------------------------------------------------ *
FUNCTION p60report_url
  (p_assignment_id   IN number
  ,p_p60_report_id   IN number
  ,p_template        IN varchar2
  ,p_inline          IN varchar2 DEFAULT 'N'
  ,p_rest_api_path   IN varchar2 DEFAULT  NULL
  ,p_report_type     IN varchar2) RETURN varchar2
IS

  c_method CONSTANT varchar2(30) := 'p60report_url';
  
BEGIN

  log_start(c_method,p_assignment_id,p_p60_report_id,p_template,p_inline,p_rest_api_path,p_report_type);

  IF p_rest_api_path is not null THEN
    log_end(c_method);
    RETURN utl_url.unescape(xxas_com_rest_util_pkg.uri_from_path(p_rest_api_path), xxas_util_pkg.get_db_charset);
  END IF;

  IF p_assignment_id   is null OR
     p_p60_report_id   is null OR
     p_template        is null THEN
    log_end(c_method);
    RETURN null;
  ELSE
    log_end(c_method);
    RETURN xxas_com_rest_util_pkg.uri_from_path
              ('assignments/'
            || to_char(p_assignment_id)
            || '/p60reports/'
            || to_char(p_p60_report_id)
            || '/'
            || p_template
            || '?_accept=application/pdf'
            || CASE WHEN p_inline = 'Y' THEN
                 '&inline=true'
               END
            || CASE WHEN p_template is not null THEN
                 '&biPublisherTemplate=' || p_template
               END
            || CASE WHEN p_report_type is not null THEN
                 '&reportType=' || p_report_type
               END);
  END IF;

END p60report_url;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_p60reports_tbl
-- ------------------------------------------------------------------------ *
FUNCTION get_p60reports_tbl
  (p_assignment_id     IN number
  ,p_action_context_id IN number DEFAULT NULL) RETURN xxas_p60_report_varray PIPELINED
IS

  CURSOR csr_p60_reports_data
           (cp_assignment_id IN number)
  IS
  SELECT pav.row_id
        ,pav.effective_date
        ,pav.period_name
        ,pav.action_type
        ,pav.type
        ,pav.assignment_action_id
        ,pav.pre_payment_id
        ,pav.action_status
        ,pav.status_code
        ,pav.messages_exist
        ,pav.assignment_id
        ,pav.payroll_action_id
  FROM   pay_payroll_actions ppa 
        ,pay_assignment_actions_v pav
  WHERE  ppa.report_type      = 'RTI_P60_REP'
  and    ppa.report_qualifier = 'GB'
  and    ppa.action_status    = 'C'
  and    ppa.action_type      = 'X'
  --  and ppa.business_group_id =
  -- :$PROFILES$.PER_BUSINESS_GROUP_ID
  AND    pav.payroll_action_id = ppa.payroll_action_id
  AND    pav.assignment_id     = cp_assignment_id
  ORDER BY pav.effective_date desc;

  CURSOR csr_p60_reports_eoy_data
           (cp_assignment_id IN number)
  IS
  SELECT pav.row_id
        ,pav.effective_date
        ,pav.period_name
        ,pav.action_type
        ,pav.type
        ,pav.assignment_action_id
        ,pav.pre_payment_id
        ,pav.action_status
        ,pav.status_code
        ,pav.messages_exist
        ,pav.assignment_id
        ,pav.payroll_action_id
  FROM   pay_payroll_actions ppa
        ,pay_assignment_actions_v pav
  WHERE  ppa.report_type      = 'EOY'
  AND    ppa.report_qualifier = 'GB'
  AND    ppa.action_status    = 'C'
  AND    ppa.action_type      = 'X'
  --  and ppa.business_group_id = 559
  AND    pav.payroll_action_id = ppa.payroll_action_id
  AND    pav.assignment_id     = cp_assignment_id
  ORDER BY pav.effective_date desc;

  CURSOR csr_p60_templates
  IS
  SELECT xt.template_name
        ,xt.description
        ,xt.template_code
  FROM   xdo_templates_vl xt
        ,xdo_ds_definitions_b xds
  WHERE  xt.ds_app_short_name = xds.application_short_name
  AND    xt.data_source_code  = xds.data_source_code
  AND    xt.ds_app_short_name = 'PAY'
  AND    xt.data_source_code  = 'PYGBRTIP60OP'
  AND    sysdate BETWEEN xt.start_date 
                 AND     nvl(xt.end_date,sysdate)
  ORDER BY xt.template_name;

  CURSOR csr_p60_templates_eoy
  IS
  SELECT xt.template_name
        ,xt.description
        ,xt.template_code
  FROM   xdo_templates_vl xt
        ,xdo_ds_definitions_b xds
  WHERE  xt.ds_app_short_name = xds.application_short_name
  AND    xt.data_source_code  = xds.data_source_code
  AND    xt.ds_app_short_name = 'PAY'
  AND    xt.data_source_code  IN ('PAYP60PX','PAYGBP60')
  AND    sysdate BETWEEN xt.start_date 
                 AND     nvl(xt.end_date,sysdate)
  ORDER BY xt.template_name;

  c_method CONSTANT       varchar2(30) := 'get_p60reports_tbl';
  e_null_params           exception;
  l_row                   xxas_p60_report_type;

BEGIN

  log_start(c_method,p_assignment_id,p_action_context_id);

  IF p_assignment_id IS NULL THEN
    RAISE e_null_params;
  END IF;

  FOR p60_report_rec IN csr_p60_reports_data(p_assignment_id)
  LOOP

    log_statement(c_method,'Effective Date: ' || p60_report_rec.effective_date);
    log_statement(c_method,'Period Name: ' || p60_report_rec.period_name);
    log_statement(c_method,'Payroll Action Id: ' || p60_report_rec.payroll_action_id);
    log_statement(c_method,'Assignment Action Id: ' || p60_report_rec.assignment_action_id);

    FOR p60_template_rec IN csr_p60_templates
    LOOP

      log_statement(c_method,'Template Name: ' || p60_template_rec.template_name);
      log_statement(c_method,'Template Code: ' || p60_template_rec.template_code);

      IF p60_template_rec.template_name is not null AND 
         p60_template_rec.template_code is not null THEN

        l_row := xxas_p60_report_type
                   (p60_report_id   => p60_report_rec.assignment_action_id
                   ,template        => p60_template_rec.template_code
                   ,summary_context => null
                   ,summary_text    => p60_report_rec.period_name || ' - ' || p60_template_rec.template_name
                   ,summary_message => null
                   ,rest_api_path   => null
                   ,report_type     => 'RTI');
        PIPE ROW (l_row);

      END IF;

    END LOOP;

  END LOOP;

  FOR p60_report_eoy_rec IN csr_p60_reports_eoy_data(p_assignment_id)
  LOOP

    log_statement(c_method,'Effective Date: ' || p60_report_eoy_rec.effective_date);
    log_statement(c_method,'Period Name: ' || p60_report_eoy_rec.period_name);
    log_statement(c_method,'Payroll Action Id: ' || p60_report_eoy_rec.payroll_action_id);
    log_statement(c_method,'Assignment Action Id: ' || p60_report_eoy_rec.assignment_action_id);

    FOR p60_template_eoy_rec IN csr_p60_templates_eoy
    LOOP

      log_statement(c_method,'Template Name: ' || p60_template_eoy_rec.template_name);
      log_statement(c_method,'Template Code: ' || p60_template_eoy_rec.template_code);

      IF p60_template_eoy_rec.template_name is not null AND 
         p60_template_eoy_rec.template_code is not null THEN

        l_row := xxas_p60_report_type
                   (p60_report_id   => p60_report_eoy_rec.assignment_action_id
                   ,template        => p60_template_eoy_rec.template_code
                   ,summary_context => null
                   ,summary_text    => p60_report_eoy_rec.period_name || ' - ' || p60_template_eoy_rec.template_name
                   ,summary_message => null
                   ,rest_api_path   => null
                   ,report_type     => 'EOY');
        PIPE ROW (l_row);

      END IF;

    END LOOP;

  END LOOP;

  log_end(c_method);

  RETURN;

EXCEPTION

  WHEN e_null_params THEN

    log_statement(c_method,'e_null_params exception');
    log_end(c_method);
    RETURN;

END get_p60reports_tbl;

PROCEDURE portrait_shift_fields
  (p_address1 IN OUT varchar2
  ,p_address2 IN OUT varchar2
  ,p_address3 IN OUT varchar2
  ,p_region1  IN OUT varchar2
  ,p_region2  IN OUT varchar2
  ,p_town     IN OUT varchar2)
IS

  c_method CONSTANT varchar2(30) := 'portrait_shift_fields';
  field1 varchar2(240) := p_address1;
  field2 varchar2(240) := p_address2;
  field3 varchar2(240) := p_address3;
  field4 varchar2(60)  := p_town;
  field5 varchar2(120) := p_region1;

BEGIN

  log_start(c_method, p_address1, p_address2, p_address3, p_region1, p_region2, p_town);

  FOR i IN 1..6
  LOOP
    field1 := p_address1;
    p_address1 := nvl(p_address1,p_address2);
    IF field1 is null THEN
      p_address2 := null;
    END IF;

    field2 := p_address2;
    p_address2 := nvl(p_address2,p_address3);
    IF field2 is null THEN
      p_address3 := null;
    END IF;

    field3 := p_address3;
    p_address3 := nvl(p_address3,p_town);
    IF field3 is null THEN
      p_town := null;
    END IF;

    field4 := p_town;
    p_town := nvl(p_town,p_region1);
    IF field4 is null THEN
      p_region1 := null;
    END IF;

    field5 := p_region1;
    p_region1 := nvl(p_region1,p_region2);
    IF field5 is null THEN
      p_region2 := null;
    END IF;

  END LOOP;

  log_end(c_method);

END portrait_shift_fields;

FUNCTION portrait_populate_addr_fields
  (p_person_id                  IN number
  ,p_align                      IN varchar2
  ,p_expense_check_send_to_addr IN varchar2
  ,p_address1          OUT NOCOPY varchar2
  ,p_address2          OUT NOCOPY varchar2
  ,p_address3          OUT NOCOPY varchar2
  ,p_region1           OUT NOCOPY varchar2
  ,p_region2           OUT NOCOPY varchar2
  ,p_region3           OUT NOCOPY varchar2
  ,p_town              OUT NOCOPY varchar2) RETURN number 
IS

  CURSOR get_home_add 
  IS
  SELECT ltrim(rtrim(pad.address_line1)) address_line1
        ,ltrim(rtrim(pad.address_line2)) address_line2
        ,ltrim(rtrim(pad.address_line3)) address_line3
        ,ltrim(rtrim(pad.town_or_city)) town_or_city
        ,substr(pad.postal_code,1,8)
        ,country
  FROM   per_addresses pad
  WHERE  pad.person_id    = p_person_id
  AND    pad.primary_flag = 'Y'
  AND    sysdate BETWEEN nvl(pad.date_from,sysdate)
                 AND     nvl(pad.date_to,sysdate);

  CURSOR csr_country_name(p_country_code IN VARCHAR2) 
  IS
  SELECT substr(ftt.territory_short_name, 1, 35) country
  FROM   fnd_territories_tl ftt
  WHERE  ftt.territory_code = p_country_code
  AND    ftt.language = userenv('LANG');

  c_method CONSTANT  varchar2(30) := 'portrait_populate_addr_fields';
  l_address1         varchar2(240);
  l_address2         varchar2(240);
  l_address3         varchar2(240);
  l_region1          varchar2(120);
  l_region2          varchar2(120);
  l_region3          varchar2(120);
  l_town             varchar2(60);
  l_cont             varchar2(60);
  l_code             varchar2(60);
  l_country_code     varchar2(60);

BEGIN

  log_start(c_method, p_person_id, p_align);

  IF p_align = 'Y' THEN
    p_address1 := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_address2 := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_address3 := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_region1  := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_region2  := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_region3  := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_town     := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXX';
  ELSE
    -- Decide which address needs to be retrieved. 
    -- If its not the home address then it must be the office address.
    -- 
    IF p_expense_check_send_to_addr = 'H' THEN

      OPEN  get_home_add;
      FETCH get_home_add INTO l_address1
                             ,l_address2
                             ,l_address3
                             ,l_town
                             ,l_region1
                             ,l_country_code;
      CLOSE get_home_add;

      IF nvl(l_country_code, 'GB') = 'GB' THEN
        l_region2 := ' ';
      ELSE
        OPEN csr_country_name(l_country_code);
        FETCH csr_country_name INTO l_region2;
        CLOSE csr_country_name;
      END IF;

      l_region3 := ' ';
      p_address1 := substr(l_address1,1,27);
      p_address2 := substr(l_address2,1,27);
      p_address3 := substr(l_address3,1,27);
      p_region1  := substr(l_region1,1,27);
      p_region2  := substr(l_region2,1,27);
      p_region3  := substr(l_region3,1,27);
      p_town     := substr(l_town,1,27);

    ELSIF ((p_expense_check_send_to_addr != 'H') OR 
           (p_expense_check_send_to_addr is null)) THEN
      p_address1 := rpad(null,20,' ')||' '||rpad(null,23);
      p_address2 := rpad(null,20,' ')||' '||rpad(null,23);
      p_address3 := rpad(null,20,' ')||' '||rpad(null,23);
      p_town     := rpad(null,20,' ')||' '||rpad(null,23);
      p_region1  := rpad(null,20,' ')||' '||rpad(null,23);
      p_region2  := rpad(null,20,' ')||' '||rpad(null,23);
    END IF;
  END IF;

  portrait_shift_fields(p_address1   => p_address1
                       ,p_address2   => p_address2
                       ,p_address3   => p_address3
                       ,p_region1    => p_region1
                       ,p_region2    => p_region2
                       ,p_town       => p_town);

  log_end(c_method);

  IF p_address1 is not null OR
     p_address2 is not null OR
     p_address3 is not null OR
     p_town     is not null OR
     p_region1  is not null OR
     p_region2  is not null OR
     p_region3  is not null	THEN 
    RETURN 1;
  ELSE
    RETURN null;
  END IF;

END portrait_populate_addr_fields;

PROCEDURE type_le_shift_fields
  (p_address1 IN OUT varchar2
  ,p_address2 IN OUT varchar2
  ,p_address3 IN OUT varchar2
  ,p_region1  IN OUT varchar2
  ,p_region2  IN OUT varchar2
  ,p_town     IN OUT varchar2)
IS

  c_method CONSTANT varchar2(30) := 'type_le_shift_fields';
  field1 varchar2(240) := p_address1;
  field2 varchar2(240) := p_address2;
  field3 varchar2(240) := p_address3;
  field4 varchar2(60)  := p_town;
  field5 varchar2(120) := p_region1;

BEGIN

  log_start(c_method, p_address1, p_address2, p_address3, p_region1, p_region2, p_town);

  FOR i IN 1..6
  LOOP
    field1 := p_address1;
    p_address1 := nvl(p_address1,p_address2);
    IF field1 is null THEN
      p_address2 := null;
    END IF;

    field2 := p_address2;
    p_address2 := nvl(p_address2,p_address3);
    IF field2 is null THEN
      p_address3 := null;
    END IF;

    field3 := p_address3;
    p_address3 := nvl(p_address3,p_town);
    IF field3 is null THEN
      p_town := null;
    END IF;

    field4 := p_town;
    p_town := nvl(p_town,p_region1);
    IF field4 is null THEN
      p_region1 := null;
    END IF;

    field5 := p_region1;
    p_region1 := nvl(p_region1,p_region2);
    IF field5 is null THEN
      p_region2 := null;
    END IF;

  END LOOP;

  log_end(c_method);

END type_le_shift_fields;

FUNCTION type_le_populate_addr_fields
  (p_person_id                  IN number
  ,p_align                      IN varchar2
  ,p_expense_check_send_to_addr IN varchar2
  ,p_address1          OUT NOCOPY varchar2
  ,p_address2          OUT NOCOPY varchar2
  ,p_address3          OUT NOCOPY varchar2
  ,p_region1           OUT NOCOPY varchar2
  ,p_region2           OUT NOCOPY varchar2
  ,p_region3           OUT NOCOPY varchar2
  ,p_town              OUT NOCOPY varchar2) RETURN number 
IS

  CURSOR get_home_add 
  IS
  SELECT ltrim(rtrim(pad.address_line1)) address_line1
        ,ltrim(rtrim(pad.address_line2)) address_line2
        ,ltrim(rtrim(pad.address_line3)) address_line3
        ,ltrim(rtrim(pad.town_or_city)) town_or_city
        ,substr(pad.postal_code,1,8)
        ,country
  FROM   per_addresses pad
  WHERE  pad.person_id    = p_person_id
  AND    pad.primary_flag = 'Y'
  AND    sysdate BETWEEN nvl(pad.date_from,sysdate)
                 AND     nvl(pad.date_to,sysdate);

  CURSOR csr_country_name(p_country_code IN VARCHAR2) 
  IS
  SELECT substr(ftt.territory_short_name, 1, 35) country
  FROM   fnd_territories_tl ftt
  WHERE  ftt.territory_code = p_country_code
  AND    ftt.language = userenv('LANG');

  c_method CONSTANT  varchar2(30) := 'type_le_populate_addr_fields';
  l_address1         varchar2(240);
  l_address2         varchar2(240);
  l_address3         varchar2(240);
  l_region1          varchar2(120);
  l_region2          varchar2(120);
  l_region3          varchar2(120);
  l_town             varchar2(60);
  l_cont             varchar2(60);
  l_code             varchar2(60);
  l_country_code     varchar2(60);

BEGIN

  log_start(c_method, p_person_id, p_align);

  IF p_align = 'Y' THEN
    p_address1 := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_address2 := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_address3 := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_region1  := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_region2  := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_region3  := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    p_town     := 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
  ELSE
    -- Decide which address needs to be retrieved. 
    -- If its not the home address then it must be the office address.
    -- 
    IF p_expense_check_send_to_addr = 'H' THEN

      OPEN  get_home_add;
      FETCH get_home_add INTO l_address1
                             ,l_address2
                             ,l_address3
                             ,l_town
                             ,l_region1
                             ,l_country_code;
      CLOSE get_home_add;

      IF nvl(l_country_code, 'GB') = 'GB' THEN
        l_region2 := ' ';
      ELSE
        OPEN csr_country_name(l_country_code);
        FETCH csr_country_name INTO l_region2;
        CLOSE csr_country_name;
      END IF;

      l_region3 := ' ';
      p_address1 := substr(l_address1,1,27);
      p_address2 := substr(l_address2,1,27);
      p_address3 := substr(l_address3,1,27);
      p_region1  := substr(l_region1,1,27);
      p_region2  := substr(l_region2,1,27);
      p_region3  := substr(l_region3,1,27);
      p_town     := substr(l_town,1,27);

    ELSIF ((p_expense_check_send_to_addr != 'H') OR 
           (p_expense_check_send_to_addr is null)) THEN
      p_address1 := rpad(null,20)||' '||rpad(null,23);
      p_address2 := rpad(null,20)||' '||rpad(null,23);
      p_address3 := rpad(null,20)||' '||rpad(null,23);
      p_town     := rpad(null,20)||' '||rpad(null,23);
      p_region1  := rpad(null,20)||' '||rpad(null,23);
      p_region2  := rpad(null,20)||' '||rpad(null,23);
    END IF;
  END IF;

  type_le_shift_fields(p_address1   => p_address1
                      ,p_address2   => p_address2
                      ,p_address3   => p_address3
                      ,p_region1    => p_region1
                      ,p_region2    => p_region2
                      ,p_town       => p_town);

  log_end(c_method);

  IF p_address1 is not null OR
     p_address2 is not null OR
     p_address3 is not null OR
     p_town     is not null OR
     p_region1  is not null OR
     p_region2  is not null OR
     p_region3  is not null	THEN 
    RETURN 1;
  ELSE
    RETURN null;
  END IF;

END type_le_populate_addr_fields;

$IF xxas_dynamic_pkg.p60_pkg_exists $THEN
--
-- ------------------------------------------------------------------------ *
-- Name    : get_p60_report_details_xml
-- ------------------------------------------------------------------------ *
FUNCTION get_p60_report_details_xml
  (p_assignment_id     IN number
  ,p_action_context_id IN number
  ,p_template          IN varchar2
  ,p_report_type       IN varchar2) RETURN XMLType
IS

  CURSOR csr_employee_data
  IS
  SELECT /*pay_gb_rti_p60.get_column_value(:P_SORT_ORDER1,emp.action_information19) SORT_ORDER1 ,
         pay_gb_rti_p60.get_column_value(:P_SORT_ORDER2,emp.action_information19) SORT_ORDER2 ,
         pay_gb_rti_p60.get_column_value(:P_SORT_ORDER3,emp.action_information19) SORT_ORDER3 ,
         pay_gb_rti_p60.get_column_value(:P_SORT_ORDER4,emp.action_information19) SORT_ORDER4 ,
         pay_gb_rti_p60.get_column_value(:P_SORT_ORDER5,emp.action_information19) SORT_ORDER5 ,
         pay_gb_rti_p60.get_column_value(:P_SORT_ORDER6,emp.action_information19) SORT_ORDER6 ,
         decode(:P_SORT_ORDER7,'ASSIGNMENT_NUMBER',asg.action_information17,emp.action_information3) SORT_ORDER7 ,*/
         asg.action_context_id assignment_action_id
        ,header.action_information1   tax_dist_ref
        ,header.action_information2   tax_dist_name
        ,header.action_information3   employer_name
        ,header.action_information4   employer_address
        ,emp.action_information1 ni
        ,substr(emp.action_information1,1,2) ni_1
        ,substr(emp.action_information1,3,2) ni_2
        ,substr(emp.action_information1,5,2) ni_3
        ,substr(emp.action_information1,7,2) ni_4
        ,substr(emp.action_information1,9,2) ni_5
        ,emp.action_information2 emp_number
        ,upper(emp.action_information3) last_name
        ,emp.action_information4 name_and_initials
        ,emp.action_information5 forenames
        ,xxas_gb_reports_pkg.remove_null_lines(emp.action_information6 ,ppa.payroll_action_id, asg.action_context_id, 1) l_address1
        ,xxas_gb_reports_pkg.remove_null_lines(emp.action_information7 ,ppa.payroll_action_id, asg.action_context_id, 2) l_address2
        ,xxas_gb_reports_pkg.remove_null_lines(emp.action_information8 ,ppa.payroll_action_id, asg.action_context_id, 3) l_address3
        ,xxas_gb_reports_pkg.remove_null_lines(emp.action_information9 ,ppa.payroll_action_id, asg.action_context_id, 4) l_town
        ,xxas_gb_reports_pkg.remove_null_lines(emp.action_information11 ,ppa.payroll_action_id, asg.action_context_id, 5) l_region1
        ,xxas_gb_reports_pkg.remove_null_lines(emp.action_information12 ,ppa.payroll_action_id, asg.action_context_id, 6) l_region2
        ,'' l_region3
        ,emp.action_information13 dob_dd
        ,emp.action_information14 dob_mm
        ,emp.action_information15 dob_yy
        ,emp.action_information16 sex
        ,emp.action_information17 tax_year
        ,nvl(emp.action_information18,'O') expense_check
        ,emp.action_information19 people_grp_id
        ,emp.action_information20 c_business_group_id
        ,asg.action_information1 current_pay
        ,asg.action_information2 current_tax
        ,trunc(nvl(asg.action_information3,0)) student_loans
        ,trunc(nvl(asg.action_information19,0)) postgraduate_loans
        ,asg.action_information4 refund_flag
        ,asg.action_information5 tax_code
        ,asg.action_information6 w1_m1_indicator
        ,nvl(asg.action_information7,'0.00') wao_amount
        ,asg.action_information8 total_pay
        ,asg.action_information9 total_tax_deducted
        ,asg.action_information10 previous_pay
        ,asg.action_information11 previous_tax_paid
        ,asg.action_information12 smp
        ,asg.action_information13 sap
        ,asg.action_information14 spp
        ,asg.action_information15 aspp
        ,asg.action_information16 week_53_indicator
        ,asg.action_information17 employee_number
        ,substr(pay_gb_eoy_archive.get_parameter(legislative_parameters,'RETURN_TYPE'),1,20) return_type
  FROM   pay_action_information asg
        ,pay_action_information emp
        ,pay_action_information header
        ,pay_assignment_actions paa
        ,per_all_assignments_f paaf
        ,pay_payroll_actions ppa
  WHERE  paa.assignment_action_id = p_action_context_id
  AND    paa.payroll_action_id    = ppa.payroll_action_id
  AND    paa.assignment_id        = paaf.assignment_id
  AND    paaf.effective_start_date = (SELECT max(effective_start_date) 
                                      FROM   per_all_assignments_f 
                                      WHERE  assignment_id = paaf.assignment_id 
                                      AND    effective_start_date <= ppa.effective_date)
  /*AND    (:p_assignment_set_id is null
          OR EXISTS (SELECT 1 FROM hr_assignment_sets has1
                     WHERE has1.assignment_set_id = :p_assignment_set_id
                     AND has1.business_group_id = paaf.business_group_id
                     AND nvl(has1.payroll_id, paaf.payroll_id) = paaf.payroll_id
                     AND (NOT EXISTS (SELECT 1 -- chk no amendmts
                                     FROM hr_assignment_set_amendments hasa1
                                     WHERE hasa1.assignment_set_id =
                                               has1.assignment_set_id)
                         OR EXISTS (SELECT 1 -- chk include amendmts
                                    FROM hr_assignment_set_amendments hasa2
                                    WHERE hasa2.assignment_set_id =
                                               has1.assignment_set_id
                                    AND (hasa2.assignment_id = paaf.assignment_id OR hasa2.assignment_id in (select p1.assignment_id from per_all_assignments_f p1,per_all_people_f p2
                                     where p1.person_id = paaf.person_id
                                     and p2.person_id = p1.person_id and p2.PER_INFORMATION9 = 'Y' and p2.PER_INFORMATION10 ='Y'
                                    and ppa.effective_date between p1.effective_start_date and p1.effective_end_date
                                    and ppa.effective_date between p2.effective_start_date and p2.effective_end_date ))
                                    AND nvl(hasa2.include_or_exclude,'I') = 'I'
                                    
                                    )
                         OR (NOT EXISTS (SELECT 1 --chk no exlude amendmts
                                    FROM hr_assignment_set_amendments hasa3
                                    WHERE hasa3.assignment_set_id =
                                               has1.assignment_set_id
                                    AND (hasa3.assignment_id = paaf.assignment_id OR hasa3.assignment_id in (select p1.assignment_id from per_all_assignments_f p1,per_all_people_f p2
                                     where p1.person_id = paaf.person_id
                                     and p2.person_id = p1.person_id and p2.PER_INFORMATION9 = 'Y' and p2.PER_INFORMATION10 ='Y'
                                    and ppa.effective_date between p1.effective_start_date and p1.effective_end_date
                                    and ppa.effective_date between p2.effective_start_date and p2.effective_end_date ))
                                    AND nvl(hasa3.include_or_exclude,'I') = 'E')
                             AND NOT EXISTS (SELECT 1 --and chk no Inc amendmts
                                    FROM hr_assignment_set_amendments hasa4
                                    WHERE hasa4.assignment_set_id =
                                               has1.assignment_set_id
                                    AND nvl(hasa4.include_or_exclude,'I') = 'I')                             ) -- end checking exclude amendmts
                         ) -- done checking amendments
                    ) -- done asg set check when not null
           ) -- end of asg set check
						--*/
  AND    emp.action_context_id = paa.assignment_action_id
  AND    emp.action_context_type = 'AAP'
  AND    emp.action_information_category = 'GB RTI P60 EMP DET'
  AND    asg.action_context_id = paa.assignment_action_id
  AND    asg.action_context_type = 'AAP'
  AND    asg.action_information_category = 'GB RTI P60 ASG DET'
  AND    (asg.action_information20 is null
          OR (asg.action_information20 = 'Y' AND asg.action_information21 = 'Y'))
  AND    header.action_context_id = ppa.payroll_action_id
  AND    header.action_context_type = 'PA'
  AND    header.action_information_category = 'GB RTI P60 PAYROLL DET'
  AND    (EXISTS(SELECT null 
                 FROM   per_assignment_extra_info 
		         WHERE  nvl(aei_information1,'D') = 'P' 
                 AND    aei_information_category  = 'GB_P60_OUTPUT_OPTION' 
                 AND    assignment_id = paa.assignment_id
                 UNION
                 SELECT 1
                 FROM   hr_organization_information hoi
                 WHERE  hoi.org_information1 = (SELECT replace(header.action_information1,'    ','/') FROM dual)
                 AND    hoi.organization_id  = emp.action_information20
                 AND    hoi.org_information_context = 'Tax Details References'
                 AND    nvl(hoi.org_information14,'P') = 'P' 
                 AND    NOT EXISTS(SELECT 1 
                                   FROM   per_assignment_extra_info 
                                   where  nvl(aei_information1,'D') = 'E' 
                                   AND    aei_information_category  = 'GB_P60_OUTPUT_OPTION' 
                                   AND    assignment_id = paa.assignment_id)
                )
          OR (nvl(null,'No') = 'No'))
  ORDER BY 1,2,3,4,5,6,7;

  CURSOR csr_employee_addr_data(cp_expense_check     IN varchar2
                               ,cp_business_group_id IN number
                               ,cp_people_grp_id     IN number
                               ,cp_address1          IN varchar2
                               ,cp_address2          IN varchar2
                               ,cp_address3          IN varchar2
                               ,cp_town              IN varchar2
                               ,cp_region1           IN varchar2
                               ,cp_region2           IN varchar2
                               ,cp_sort_order1       IN varchar2
                               ,cp_sort_order2       IN varchar2
                               ,cp_sort_order3       IN varchar2
                               ,cp_sort_order4       IN varchar2
                               ,cp_sort_order5       IN varchar2
                               ,cp_sort_order6       IN varchar2)
  IS
  SELECT max(decode(cp_expense_check,'H' 
                   ,substr(cp_address1,0,43)
                   ,decode(fnd.application_column_name,(SELECT cp_sort_order1 FROM dual)
                          ,substr(fnd.segment_name,0,17) || ' - ' || (SELECT substr(pay_gb_rti_p60.get_column_value(cp_sort_order1,cp_people_grp_id),0,23) FROM dual)
                          ,null)
                   )
			) c_address1
        ,max(decode(cp_expense_check,'H'
                   ,substr(cp_address2,0,43)
                   ,decode(fnd.application_column_name,(SELECT cp_sort_order2 FROM dual)
                          ,substr(fnd.segment_name,0,17) || ' - ' || (SELECT substr(pay_gb_rti_p60.get_column_value(cp_sort_order2,cp_people_grp_id),0,23) FROM dual)
                          ,null)
                   )
			) c_address2
        ,max(decode(cp_expense_check,'H'
                   ,substr(cp_address3,0,43)
                   ,decode(fnd.application_column_name,(SELECT cp_sort_order3 FROM dual)
                          ,substr(fnd.segment_name,0,17) || ' - ' || (SELECT substr(pay_gb_rti_p60.get_column_value(cp_sort_order3,cp_people_grp_id),0,23) FROM dual)
                          ,null)
                   )
			) c_address3
        ,max(decode(cp_expense_check,'H'
                   ,substr(cp_town,0,43)
                   ,decode(fnd.application_column_name,(SELECT cp_sort_order4 FROM dual)
                          ,substr(fnd.segment_name,0,17) || ' - ' || (SELECT substr(pay_gb_rti_p60.get_column_value(cp_sort_order4,cp_people_grp_id),0,23) FROM dual)
                          ,null)
                   )
			) c_town
        ,max(decode(cp_expense_check,'H'
                   ,substr(cp_region1,0,43)
                   ,decode(fnd.application_column_name,(SELECT cp_sort_order5 FROM dual)
                          ,substr(fnd.segment_name,0,17) || ' - ' || (SELECT substr(pay_gb_rti_p60.get_column_value(cp_sort_order5,cp_people_grp_id),0,23) FROM dual)
                          ,null)
                   )
			) c_region1
        ,max(decode(cp_expense_check,'H'
                   ,substr(cp_region2,0,43)
                   ,decode(fnd.application_column_name,(SELECT cp_sort_order6 FROM dual)
                          ,substr(fnd.segment_name,0,17) || ' - ' || (SELECT substr(pay_gb_rti_p60.get_column_value(cp_sort_order6,cp_people_grp_id),0,23) FROM dual)
                          ,null)
                   )
			) c_region2
        ,'' c_region3
        ,max(decode(cp_expense_check,'H'
                   ,substr(cp_region1 || ' ' ||cp_region2,0,43)
                   ,decode((SELECT cp_sort_order5 FROM dual),null
                          ,decode((SELECT cp_sort_order6 FROM dual),null,'  ',substr(fnd.segment_name,0,17) || ' - ' || (SELECT substr(pay_gb_rti_p60.get_column_value(cp_sort_order6,cp_people_grp_id),0,23) FROM dual))
                          ,decode((SELECT cp_sort_order6 FROM dual),null,substr(fnd.segment_name,0,17) || ' - ' || (SELECT substr(pay_gb_rti_p60.get_column_value(cp_sort_order5,cp_people_grp_id),0,23) FROM dual)
                                 ,substr((SELECT segment_name
                                          FROM   fnd_id_flex_segments_vl
                                          WHERE  application_id BETWEEN 801 
                                                                AND     899
                                          AND    id_flex_code = 'GRP'
                                          AND    application_column_name IN (cp_sort_order5)
                                          AND    id_flex_num             = (SELECT people_group_structure
                                                                            FROM   per_business_groups
                                                                            WHERE  business_group_id = cp_business_group_id)),0,8) || '-' || (SELECT substr(pay_gb_rti_p60.get_column_value(cp_sort_order5,cp_people_grp_id),0,12) FROM dual) || ' ' || 
                                  substr((SELECT segment_name
                                          FROM   fnd_id_flex_segments_vl
                                          WHERE  application_id BETWEEN 801 
                                                                AND     899
                                          AND    id_flex_code = 'GRP'
                                          AND    application_column_name IN(cp_sort_order6)
                                          AND    id_flex_num = (SELECT people_group_structure
                                                                FROM   per_business_groups
                                                                WHERE  business_group_id = cp_business_group_id)),0,8) || '-' || (select substr(pay_gb_rti_p60.get_column_value(cp_sort_order6,cp_people_grp_id),0,12) from dual))))) c_region4
  FROM   fnd_id_flex_segments_vl fnd
  WHERE  cp_expense_check = 'H' OR 
         (fnd.application_id BETWEEN 801
                             AND     899
          AND fnd.id_flex_code = 'GRP'
          AND fnd.application_column_name IN(cp_sort_order1,cp_sort_order2
                                            ,cp_sort_order3,cp_sort_order4
                                            ,cp_sort_order5,cp_sort_order6)
          AND fnd.id_flex_num  = (SELECT people_group_structure
                                  FROM   per_business_groups
                                  WHERE  business_group_id = cp_business_group_id)	
         );

  CURSOR csr_employee_nic_data
  IS
  SELECT nic.action_information1 cp_nic
        ,trunc(nvl(nic.action_information2,0)) cp_lel
        ,trunc(nvl(nic.action_information3,0)) cp_et
        ,trunc(nvl(nic.action_information4,0)) cp_uap
        ,trunc(nvl(nic.action_information5,0)) cp_uel
        ,nic.action_information6 cp_ec
  FROM   pay_action_information nic
  WHERE  nic.action_context_id = p_action_context_id
  AND    nic.action_context_type = 'AAP'
  AND    nic.action_information_category = 'GB RTI P60 NI DET'
  AND    nic.action_information7 IN ( 'NIC1','NIC2','NIC3','NIC4')
  ORDER BY nic.action_information7;

  CURSOR csr_employee_nic1_data
  IS
  SELECT nic.action_information1 cp_nic1
        ,trunc(nvl(nic.action_information2,0)) cp_lel1
        ,trunc(nvl(nic.action_information3,0)) cp_et1
        ,trunc(nvl(nic.action_information4,0)) cp_uap1
        ,trunc(nvl(nic.action_information5,0)) cp_uel1
        ,nic.action_information6 cp_ec1
  FROM   pay_action_information nic
  WHERE  nic.action_context_id = p_action_context_id
  AND    nic.action_context_type = 'AAP'
  AND    nic.action_information_category = 'GB RTI P60 NI DET'
  AND    nic.action_information7 = 'NIC1';

  CURSOR csr_employee_nic2_data
  IS
  SELECT nic.action_information1 cp_nic2
        ,trunc(nvl(nic.action_information2,0)) cp_lel2
        ,trunc(nvl(nic.action_information3,0)) cp_et2
        ,trunc(nvl(nic.action_information4,0)) cp_uap2
        ,trunc(nvl(nic.action_information5,0)) cp_uel2
        ,nic.action_information6 cp_ec2
  FROM   pay_action_information nic
  WHERE  nic.action_context_id   = p_action_context_id
  AND    nic.action_context_type = 'AAP'
  AND    nic.action_information_category = 'GB RTI P60 NI DET'
  AND    nic.action_information7 = 'NIC2';

  CURSOR csr_employee_nic3_data
  IS
  SELECT nic.action_information1 cp_nic3
        ,trunc(nvl(nic.action_information2,0)) cp_lel3
        ,trunc(nvl(nic.action_information3,0)) cp_et3
        ,trunc(nvl(nic.action_information4,0)) cp_uap3
        ,trunc(nvl(nic.action_information5,0)) cp_uel3
        ,nic.action_information6 cp_ec3
  FROM   pay_action_information nic
  WHERE  nic.action_context_id   = p_action_context_id
  AND    nic.action_context_type = 'AAP'
  AND    nic.action_information_category = 'GB RTI P60 NI DET'
  AND    nic.action_information7 = 'NIC3';

  CURSOR csr_employee_nic4_data
  IS
  SELECT nic.action_information1 cp_nic4
        ,trunc(nvl(nic.action_information2,0)) cp_lel4
        ,trunc(nvl(nic.action_information3,0)) cp_et4
        ,trunc(nvl(nic.action_information4,0)) cp_uap4
        ,trunc(nvl(nic.action_information5,0)) cp_uel4
        ,nic.action_information6 cp_ec4
  FROM   pay_action_information nic
  where  nic.action_context_id = p_action_context_id
  and    nic.action_context_type = 'AAP'
  and    nic.action_information_category = 'GB RTI P60 NI DET'
  and    nic.action_information7 = 'NIC4';

  CURSOR csr_business_group_id
  IS
  SELECT ppa.business_group_id
  FROM   pay_payroll_actions ppa
        ,pay_assignment_actions_v pav
  WHERE  pav.payroll_action_id = ppa.payroll_action_id
  AND    pav.assignment_id     = p_assignment_id;

  l_profile_name_format VARCHAR2(4000);

  CURSOR csr_plain_portrait_main_data(cp_align IN varchar2)
  IS
  SELECT yea.permit_number permit_number
        ,yea.payroll_id payroll_id
        ,yep.expense_check_send_to_address expense_check_send_to_address
        ,null segment1s
        ,null segment2s
        ,null segment3s
        ,null segment4s
        ,null segment5s
        ,null segment6s
        ,null segment7s
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment1c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment2c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment3c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment4c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment5c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment6c
        ,null segment7c
        ,yep.assignment_action_id assig_action_id
        ,yep.assignment_id assignment_id
        ,yep.effective_end_date effective_end_date
        ,yep.person_id person_id
        ,substr(yep.national_insurance_number,1,2) ni_1
        ,substr(yep.national_insurance_number,3,2) ni_2
        ,substr(yep.national_insurance_number,5,2) ni_3
        ,substr(yep.national_insurance_number,7,2) ni_4
        ,substr(yep.national_insurance_number,9,1) ni_5
        ,upper(substr(yep.sex,1,1)) sex
        ,lpad(to_char(yea.tax_district_reference),3,'0') || '         ' || yea.tax_reference_number tax_dist_ref
        ,substr(yea.tax_district_name,1,24) tax_dist_name
        ,substr(to_char(yea.tax_year),1,4) tax_year
        ,upper(substr(yep.last_name,1,20)) last_name
        ,upper(substr(yep.middle_name,1,17)) middle_name
        ,upper(substr(yep.first_name,1,17)) forenames
        ,(CASE WHEN l_profile_name_format is null OR l_profile_name_format = 'FORMAT1' 
               THEN upper(rpad(substr((yep.last_name || ' ' || decode(yep.first_name, null, null, yep.first_name|| ' ')|| 
                          decode(substr(yep.middle_name,1,1), null, null, substr(yep.middle_name,1,1)||' ')|| yep.title),1,(36-(length(yep.assignment_number)+1))),
                          (36-(length(yep.assignment_number)+1)),' ')||' '|| yep.assignment_number)
               WHEN l_profile_name_format = 'FORMAT2'
               THEN upper(rpad(substr((decode(yep.title,null,null,yep.title||' ') || decode(yep.first_name, null, null, yep.first_name|| ' ')|| 
                          decode(substr(yep.middle_name,1,1), null, null, substr(yep.middle_name,1,1)||' ')|| yep.last_name),1,(36-(length(yep.assignment_number)+1))),
                          (36-(length(yep.assignment_number)+1)),' ')||' '|| yep.assignment_number)
               WHEN l_profile_name_format = 'FORMAT3'
               THEN upper(rpad(substr((yep.last_name || ' ' || decode(yep.title,null,null,yep.title||' ') || decode(yep.first_name, null, null, yep.first_name|| ' ')|| 
                          decode(substr(yep.middle_name,1,1), null, null, substr(yep.middle_name,1,1)||' ')),1,(36-(length(yep.assignment_number)+1))),
                          (36-(length(yep.assignment_number)+1)),' ')||' '|| yep.assignment_number)
               WHEN l_profile_name_format = 'FORMAT4' 
               THEN upper(rpad(substr((decode(yep.title,null,null,yep.title||' ') || decode(yep.first_name, null, null, yep.first_name|| ' ')|| yep.last_name),1,(36-(length(yep.assignment_number)+1))),
                          (36-(length(yep.assignment_number)+1)),' ')||' '|| yep.assignment_number)
         END) name_and_initial
        ,yea.employers_name employer_name
        ,yea.employers_address_line employer_address
        ,yep.assignment_number employee_number
        ,nvl(yep.ssp,0)/100 ssp
        ,nvl(yep.smp,0)/100 smp
        ,nvl(yep.sap,0)/100 sap
        ,nvl((yep.spp_adopt + yep.spp_birth),0)/100 spp
        ,(nvl(yep.aspp_adopt,0) + nvl(yep.aspp_birth,0))/100 aspp
        ,to_char(yep.termination_date, 'DD MM YYYY') termination_date
        ,to_char(yep.start_of_emp, 'DD MM YYYY') start_of_emp    
        ,(nvl(yep.taxable_pay,0) + nvl(yep.previous_taxable_pay,0))/100 total_pay
        ,(nvl(yep.tax_paid,0) * decode(yep.tax_refund,'R',-1,1)
          + nvl(yep.previous_tax_paid,0))/100 total_tax_deducted
        ,nvl(yep.previous_taxable_pay,0)/100 previous_pay
        ,nvl(yep.previous_tax_paid,0)/100 previous_tax_paid
        ,nvl(yep.taxable_pay,0)/100 current_pay
         /*mod(nvl(yep.taxable_pay,0),100) current_pay_pence*/
        ,nvl(yep.tax_paid,0)/100 current_tax
        ,nvl(yep.student_loans,0)/100 student_loans
         /*nvl(yep.tax_credits,0) / 100 tax_credits*/
        ,yep.tax_refund refund_flag
         /*yep.tax_code tax_code -- Bug 9282704*/
        ,decode(yep.tax_code,'NI',null,yep.tax_code) tax_code
         /*yep.w1_m1_indicator	|| '1' w1_m1_indicator*/
        ,(CASE WHEN yep.w1_m1_indicator	|| '1' = ' 1' or yep.tax_code = 'NI' 
               THEN null
               ELSE yep.w1_m1_indicator	|| '1'
		  END) w1_m1_indicator -- Bug 9282704
        ,nvl(yep.widows_and_orphans,0)/100 wao_amount
         /*mod(nvl(yep.widows_and_orphans,0)/100,100) wao_pence,*/
        ,decode(yep.week_53_indicator,'3',' X', '4','54', '6','56', '  ') week_53_indicator
        ,substr(yep.date_of_birth,1,2) dob_dd
        ,to_char(yep.date_of_birth,'MM') dob_mm
        ,substr(to_char(yep.date_of_birth,'DD-MM-YYYY'),7,4) dob_yy
  FROM   pay_gb_year_end_payrolls_v	yea
        ,pay_gb_year_end_assignments_v yep
        ,pay_people_groups ppg
  WHERE  yep.assignment_action_id = p_action_context_id
  AND    yea.payroll_action_id = yep.payroll_action_id
  AND    yea.payroll_id        = yep.payroll_id
  /*
  AND           ((yep.multiple_asg_flag IS NULL) -- if no multi asg for NI then select the asg
  OR             (yep.multiple_asg_flag = 'Y' -- if multi asg for ni but not aggrg for PAYE then select this asg only if there are non 0 values
                     AND nvl(yep.aggregated_paye_flag, 'N') = 'N'
                     AND (yep.taxable_pay > 0
                     OR              yep.tax_paid > 0
                     OR              yep.previous_taxable_pay > 0
                     OR              yep.previous_tax_paid > 0
                     OR              yep.superannuation_paid > 0
                     OR              yep.ssp > 0
                     OR              yep.smp > 0
                     OR              yep.student_loans > 0
                     OR              yep.tax_credits > 0))
  OR            ( yep.multiple_asg_flag = 'Y' and yep.aggregated_paye_flag = 'Y' and yep.eoy_primary_flag = 'Y')) -- if multi asg for NI and aggreg for PAYE then select only if primary asg for EOY , P60 will not be generated for other assignments
  */
  AND    ((nvl(yep.aggregated_paye_flag,'N') = 'N')  -- if not PAYE aggregated then select,
          OR (nvl(yep.aggregated_paye_flag,'N') = 'Y' AND nvl(yep.eoy_primary_flag,'N') = 'Y') -- if multi asg for NI and aggreg for PAYE then select only if primary asg for EOY , P60 will not be generated for other assignments
         )
  AND    (yep.taxable_pay <> 0
          OR yep.tax_paid <> 0
          OR yep.previous_taxable_pay <> 0
          OR yep.previous_tax_paid <> 0
          OR yep.ssp <> 0
          OR yep.student_loans > 0
          OR yep.smp <> 0
          OR yep.sap <> 0         -- new
          OR yep.spp_adopt <> 0   -- new
          OR yep.spp_birth <> 0   -- new
          OR yep.aspp_birth <> 0
          OR yep.aspp_adopt<>0
          OR nvl(pay_gb_eoy_archive.get_arch_str(yep.assignment_action_id,'X_REPORTABLE_NI'),'N') = 'Y'  
         )
  AND    yep.people_group_id = ppg.people_group_id (+)
  AND    (yep.termination_date is null
          OR yep.termination_date > to_date('05/04/' || yea.tax_year, 'DD/MM/YYYY'))
  AND    cp_align = 'N'
  AND    (null is null -- don't check for assignment set in this case 
          OR EXISTS (SELECT 1 FROM hr_assignment_sets has1 
                     WHERE has1.assignment_set_id = null
                     -- AND has1.business_group_id = asg.business_group_id 
                     AND nvl(has1.payroll_id,yea.payroll_id) = yea.payroll_id  
                     AND (NOT EXISTS (SELECT 1 -- chk no amendmts 
                                      FROM   hr_assignment_set_amendments hasa1 
                                      WHERE  hasa1.assignment_set_id = has1.assignment_set_id) 
                          OR EXISTS (SELECT 1 -- chk include amendmts 
                                     FROM   hr_assignment_set_amendments hasa2 
                                     WHERE  hasa2.assignment_set_id = has1.assignment_set_id 
                                     AND    hasa2.assignment_id = yep.assignment_id 
                                     AND    nvl(hasa2.include_or_exclude,'I') = 'I') 
                          OR (NOT EXISTS (SELECT 1 --chk no exlude amendmts 
                                          FROM   hr_assignment_set_amendments hasa3 
                                          WHERE  hasa3.assignment_set_id = has1.assignment_set_id 
                                          AND    hasa3.assignment_id = yep.assignment_id 
                                          AND    nvl(hasa3.include_or_exclude,'I') = 'E') 
                              AND NOT EXISTS (SELECT 1 --and chk no Inc amendmts 
                                              FROM   hr_assignment_set_amendments hasa4
                                              WHERE  hasa4.assignment_set_id = has1.assignment_set_id 
                                              AND    nvl(hasa4.include_or_exclude,'I') = 'I')) -- end checking exclude amendmts 
                         ) -- done checking amendments 
                    ) -- done asg set check when not null 
         ) -- end of asg set check
  /* Added check to generate for Employees who opted for Paper based P60 only 
      If p_restrict_out parameter is not Yes, then the query would generate data for all employees archived by the EOY process minus the records filtered by Assignment Set.
  */
  AND    ((nvl(null,'No') = 'No') OR
          EXISTS(/* Check if assignment level preference is Paper */
                 SELECT null 
                 FROM   per_assignment_extra_info 
                 WHERE  nvl(aei_information1,'D') = 'P' 
                 AND    aei_information_category = 'GB_P60_OUTPUT_OPTION' 
                 AND    assignment_id = yep.assignment_id
                 UNION
                 /* Check if Tax reference level preference is Paper and Assignment level it is not Electronic */
                 SELECT 1
                 FROM   hr_organization_information hoi
                 WHERE  hoi.organization_id = yea.business_group_id
                 AND    hoi.org_information1 = lpad(to_char(yea.tax_district_reference), 3, '0') || '/' || yea.tax_reference_number
                 AND    hoi.org_information_context = 'Tax Details References'
                 AND    nvl(hoi.org_information14,'P') = 'P' 
                 AND    NOT EXISTS(SELECT 1 
                                   FROM   per_assignment_extra_info 
                                   WHERE  nvl(aei_information1,'D') = 'E' 
                                   AND    aei_information_category = 'GB_P60_OUTPUT_OPTION' 
                                   AND    assignment_id = yep.assignment_id)
                ) -- end of exists
         )
  UNION ALL
  SELECT null permit_number
        ,999999999999999 payroll_id
        ,null expense_check_send_to_address
        ,null segment1s
        ,null segment2s
        ,null segment3s
        ,null segment4s
        ,null segment5s
        ,null segment6s
        ,null segment7s
        ,null segment1c
        ,null segment2c
        ,null segment3c
        ,null segment4c
        ,null segment5c
        ,null segment6c
        ,null segment7c
        ,999999999999999 assig_action_id
        ,999999999999999 assignment_id
        ,to_date('31/12/4712','DD/MM/YYYY') effective_end_date
        ,999999999999999 person_id
        ,'XX' ni_1
        ,'XX' ni_2
        ,'XX' ni_3
        ,'XX' ni_4
        ,'X' ni_5
        ,'X' sex
        ,'XXX    XXXXXXX' tax_dist_ref
        ,'XXXXXXXXXXXXXXXXXXXX' tax_dist_name
        ,'9999' tax_year
        ,'XXXXXXXXXXXXXXXXXXXX' last_name
        ,'XXXXXXXXXXXXXXXXX' middle_name
        ,'XXXXXXXXXXXXXXXXX' forenames
        ,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX' name_and_initial
        ,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX' employer_name
        ,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX' || ' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX' employer_address
        ,'XXXXXXXXXXXXXX' employee_number
        ,999999.99 ssp
        ,999999.99 smp
        ,999999.99 sap
        ,999999.99 spp
        ,999999.99 aspp
        ,'XX XX XXXX' termination_date
        ,'XX XX XXXX' start_of_emp
        ,999999999.99 total_pay
        ,999999999.99 total_tax_deducted
        ,999999999.99 previous_pay
        ,999999999.99 previous_tax_paid
        ,999999999.99 current_pay
         /*99 current_pay_pence,*/
        ,999999999.99 current_tax
        ,99999 student_loans
		/*9999999.99 tax_credits,*/
        ,'X' refund_flag
        ,'XXXX' tax_code
        ,'XX' w1_m1_indicator
        ,9999999.99 wao_amount
		/*99 wao_pence,*/
        ,'XX' week_53_indicator
        ,'XX' dob_dd
        ,'XX' dob_mm
        ,'XXXX' dob_yy
  FROM   fnd_lookups lku
  WHERE  cp_align = 'Y'
  AND    ROWNUM	< 4
  ORDER BY 1, 2, 3, 4, 5, 6, 7, 8, 9, 10;

  CURSOR csr_plain_portrait_ni_data(cp_align IN varchar2)
  IS
  SELECT yev.assignment_action_id assig_action_lnk
        ,yev.assignment_id assignment_id
        ,yev.effective_end_date effective_end_date
        ,yev.ni_category_code ni_category_code
        ,trunc(nvl(yev.earnings/100,0)) earnings
        ,nvl(yev.total_contributions,0)/100 total_contributions
        ,nvl(yev.employees_contributions,0)/100 employees_contributions
        ,trunc(nvl(yev.ni_able_lel,0)/100) ni_able_lel
        ,trunc(nvl(yev.ni_able_uel,0)/100) ni_able_uel
        ,trunc(nvl(yev.ni_able_et,0)/100) ni_able_et
        ,trunc(nvl(yev.ni_able_uap,0)/100) ni_able_uap
        ,nvl(yev.employers_rebate,0)/100 employers_rebate
        ,nvl(yev.employees_rebate,0)/100 employees_rebate
        ,(nvl(yev.employees_rebate,0)/100) + (nvl(yev.employers_rebate,0)/100) total_rebate
        ,yev.ni_refund ni_refund
        ,substr(yev.scon,2,8) scon
  FROM   pay_gb_year_end_values_v yev
  WHERE  cp_align = 'N'
  AND    yev.reportable <> 'N'
  AND    yev.assignment_action_id = p_action_context_id
  UNION ALL
  SELECT 999999999999999 assig_action_lnk
        ,999999999999999 assignment_id
        ,to_date('31/12/4712','DD/MM/YYYY')	effective_end_date
        ,'X' ni_category_code
        ,999999 earnings
        ,999999.99 total_contributions
        ,99999.99 employees_contributions
        ,99999 ni_able_lel
        ,99999 ni_able_uel
        ,99999 ni_able_et
        ,99999 ni_able_uap
        ,99999.99 employers_rebate
        ,99999.99 employees_rebate
        ,99999.99 total_rebate
        ,'X' ni_refund
        ,'XXXXXXXX' scon
  FROM   fnd_lookups lk1
  WHERE  cp_align = 'Y'
  AND    rownum < 5
  ORDER BY 3;

  CURSOR csr_plain_portrait_cp_data(cp_align IN varchar2)
  IS
  SELECT max(decode(rownumber,1,ni_category_code,null)) nic1
        ,max(decode(rownumber,1,ni_able_lel,null)) lel1
        ,max(decode(rownumber,1,ni_able_et,null)) et1
        ,max(decode(rownumber,1,ni_able_uap,null)) uap1
        ,max(decode(rownumber,1,ni_able_uel,null)) uel1
        ,max(decode(rownumber,1,employees_contributions,null)) ec1
        ,max(decode(rownumber,1,scon,null)) scon1
        ,max(decode(rownumber,2,ni_category_code,null)) nic2
        ,max(decode(rownumber,2,ni_able_lel,null)) lel2
        ,max(decode(rownumber,2,ni_able_et,null)) et2
        ,max(decode(rownumber,2,ni_able_uap,null)) uap2
        ,max(decode(rownumber,2,ni_able_uel,null)) uel2
        ,max(decode(rownumber,2,employees_contributions,null)) ec2
        ,max(decode(rownumber,2,scon,null)) scon2
        ,max(decode(rownumber,3,ni_category_code,null)) nic3
        ,max(decode(rownumber,3,ni_able_lel,null)) lel3
        ,max(decode(rownumber,3,ni_able_et,null)) et3
        ,max(decode(rownumber,3,ni_able_uap,null)) uap3
        ,max(decode(rownumber,3,ni_able_uel,null)) uel3
        ,max(decode(rownumber,3,employees_contributions,null)) ec3
        ,max(decode(rownumber,3,scon,null)) scon3
        ,max(decode(rownumber,4,ni_category_code,null)) nic4
        ,max(decode(rownumber,4,ni_able_lel,null)) lel4
        ,max(decode(rownumber,4,ni_able_et,null)) et4
        ,max(decode(rownumber,4,ni_able_uap,null)) uap4
        ,max(decode(rownumber,4,ni_able_uel,null)) uel4
        ,max(decode(rownumber,4,employees_contributions,null)) ec4
        ,max(decode(rownumber,4,scon,null)) scon4
  FROM   (SELECT yev.ni_category_code ni_category_code
                ,nvl(yev.employees_contributions,0)/100 employees_contributions
                ,trunc(nvl(yev.ni_able_lel,0)/100) ni_able_lel
                ,trunc(nvl(yev.ni_able_uel,0)/100) ni_able_uel
                ,trunc(nvl(yev.ni_able_uap,0)/100) ni_able_uap
                ,trunc(nvl(yev.ni_able_et,0)/100) ni_able_et
                ,substr(yev.scon,2,8) scon
                ,rownum rownumber
          FROM   pay_gb_year_end_values_v  yev
          WHERE  yev.assignment_action_id = p_action_context_id
          AND    cp_align = 'N'
          AND    yev.reportable <> 'N'
         )
  UNION ALL
  SELECT 'X' nic1
        ,9999 lel1
        ,9999 et1
        ,9999 uap1
        ,9999 uel1
        ,99999.99 ec1
        ,'XXXXXXXX' scon1
        ,null nic2
        ,null lel2
        ,null et2
        ,null uap2
        ,null uel2
        ,null ec2
        ,null scon2
        ,null nic3
        ,null lel3
        ,null et3
        ,null uap3
        ,null uel3
        ,null ec3
        ,null scon3
        ,null nic4
        ,null lel4
        ,null et4
        ,null uap4
        ,null uel4
        ,null ec4
        ,null scon4
  FROM   dual
  WHERE  cp_align = 'Y';

  CURSOR csr_type_le_main_data(cp_align IN varchar2)
  IS
  SELECT yea.permit_number permit_number
        ,yea.payroll_id payroll_id
        ,yep.expense_check_send_to_address expense_check_send_to_address
        ,null segment1s
        ,null segment2s
        ,null segment3s
        ,null segment4s
        ,null segment5s
        ,null segment6s
        ,null segment7s
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment1c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment2c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment3c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment4c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment5c
        ,decode(yep.expense_check_send_to_address, 'H',null, null) segment6c
        ,null segment7c
        ,yep.assignment_action_id assig_action_id
        ,yep.assignment_id assignment_id
        ,yep.effective_end_date effective_end_date
        ,yep.person_id person_id
        ,substr(yep.national_insurance_number,1,2) ni_1
        ,substr(yep.national_insurance_number,3,2) ni_2
        ,substr(yep.national_insurance_number,5,2) ni_3
        ,substr(yep.national_insurance_number,7,2) ni_4
        ,substr(yep.national_insurance_number,9,1) ni_5
        ,upper(substr(yep.sex,1,1)) sex
        ,lpad(to_char(yea.tax_district_reference),3,'0') || '         ' || yea.tax_reference_number tax_dist_ref
        ,substr(yea.tax_district_name,1,24) tax_dist_name
        ,substr(to_char(yea.tax_year),1,4) tax_year
        ,upper(substr(yep.last_name,1,20)) last_name
        ,upper(rpad(yep.first_name,17) || ' ' || substr(yep.middle_name,1,1)) forenames
        ,(CASE WHEN l_profile_name_format is null OR l_profile_name_format = 'FORMAT1' 
               THEN upper(rpad(substr((yep.last_name || ' ' || decode(yep.first_name, null, null, yep.first_name|| ' ')|| 
                          decode(substr(yep.middle_name,1,1), null, null, substr(yep.middle_name,1,1)||' ')|| yep.title),1,(36-(length(yep.assignment_number)+1))),
                          (36-(length(yep.assignment_number)+1)),' ')||' '|| yep.assignment_number)
               WHEN l_profile_name_format = 'FORMAT2'
               THEN upper(rpad(substr((decode(yep.title,null,null,yep.title||' ') || decode(yep.first_name, null, null, yep.first_name|| ' ')|| 
                          decode(substr(yep.middle_name,1,1), null, null, substr(yep.middle_name,1,1)||' ')|| yep.last_name),1,(36-(length(yep.assignment_number)+1))),
                          (36-(length(yep.assignment_number)+1)),' ')||' '|| yep.assignment_number)
               WHEN l_profile_name_format = 'FORMAT3'
               THEN upper(rpad(substr((yep.last_name || ' ' || decode(yep.title,null,null,yep.title||' ') || decode(yep.first_name, null, null, yep.first_name|| ' ')|| 
                          decode(substr(yep.middle_name,1,1), null, null, substr(yep.middle_name,1,1)||' ')),1,(36-(length(yep.assignment_number)+1))),
                          (36-(length(yep.assignment_number)+1)),' ')||' '|| yep.assignment_number)
               WHEN l_profile_name_format = 'FORMAT4' 
               THEN upper(rpad(substr((decode(yep.title,null,null,yep.title||' ') || decode(yep.first_name, null, null, yep.first_name|| ' ')|| yep.last_name),1,(36-(length(yep.assignment_number)+1))),
                          (36-(length(yep.assignment_number)+1)),' ')||' '|| yep.assignment_number)
         END) name_and_initials
        ,yea.employers_name employer_name
        ,yea.employers_address_line employer_address
        ,yep.assignment_number employee_number
        ,nvl(yep.ssp,0)/100 ssp
        ,nvl(yep.smp,0)/100 smp
        ,nvl(yep.sap,0)/100 sap
        ,(nvl(yep.spp_adopt,0) + nvl(yep.spp_birth,0))/100 spp
        ,(nvl(yep.aspp_adopt,0) + nvl(yep.aspp_birth,0))/100 aspp
        ,to_char(yep.termination_date, 'DD MM YYYY') termination_date
        ,to_char(yep.start_of_emp, 'DD MM YYYY') start_of_emp    
        ,(nvl(yep.taxable_pay,0) + nvl(yep.previous_taxable_pay,0))/100 total_pay
        ,(nvl(yep.tax_paid,0) * decode(yep.tax_refund,'R',-1,1)
          + nvl(yep.previous_tax_paid,0))/100 total_tax_deducted
        ,nvl(yep.previous_taxable_pay,0)/100 previous_pay
        ,nvl(yep.previous_tax_paid,0)/100 previous_tax_paid
        ,nvl(yep.taxable_pay,0)/100 current_pay
         /*mod(nvl(yep.taxable_pay,0),100) current_pay_pence*/
        ,nvl(yep.tax_paid,0)/100 current_tax
        ,nvl(yep.student_loans,0)/100 student_loans
         /*nvl(yep.tax_credits,0) / 100 tax_credits*/
        ,yep.tax_refund refund_flag
         /*yep.tax_code tax_code -- Bug 9282704*/
        ,decode(yep.tax_code,'NI',null,yep.tax_code) tax_code
         /*yep.w1_m1_indicator	|| '1' w1_m1_indicator*/
        ,(CASE WHEN yep.w1_m1_indicator	|| '1' = ' 1' or yep.tax_code = 'NI' 
               THEN null
               ELSE yep.w1_m1_indicator	|| '1'
		  END) w1_m1_indicator -- Bug 9282704
        ,nvl(yep.widows_and_orphans,0)/100 wao_amount
         /*mod(nvl(yep.widows_and_orphans,0)/100,100) wao_pence,*/
        ,decode(yep.week_53_indicator,'3',' X', '4','54', '6','56', '  ') week_53_indicator
        ,substr(yep.date_of_birth,1,2) dob_dd
        ,to_char(yep.date_of_birth,'MM') dob_mm
        ,substr(to_char(yep.date_of_birth,'DD-MM-YYYY'),7,4) dob_yy
  FROM   pay_gb_year_end_payrolls_v	yea
        ,pay_gb_year_end_assignments_v yep
        ,pay_people_groups ppg
  WHERE  yep.assignment_action_id = p_action_context_id
  AND    yea.payroll_action_id = yep.payroll_action_id
  AND    yea.payroll_id        = yep.payroll_id
  /*
  AND           ((yep.multiple_asg_flag IS NULL) -- if no multi asg for NI then select the asg
  OR             (yep.multiple_asg_flag = 'Y' -- if multi asg for ni but not aggrg for PAYE then select this asg only if there are non 0 values
                     AND nvl(yep.aggregated_paye_flag, 'N') = 'N'
                     AND (yep.taxable_pay > 0
                     OR              yep.tax_paid > 0
                     OR              yep.previous_taxable_pay > 0
                     OR              yep.previous_tax_paid > 0
                     OR              yep.superannuation_paid > 0
                     OR              yep.ssp > 0
                     OR              yep.smp > 0
                     OR              yep.student_loans > 0
                     OR              yep.tax_credits > 0))
  OR            ( yep.multiple_asg_flag = 'Y' and yep.aggregated_paye_flag = 'Y' and yep.eoy_primary_flag = 'Y')) -- if multi asg for NI and aggreg for PAYE then select only if primary asg for EOY , P60 will not be generated for other assignments
  */
  AND    ((nvl(yep.aggregated_paye_flag,'N') = 'N')  -- if not PAYE aggregated then select,
          OR (nvl(yep.aggregated_paye_flag,'N') = 'Y' AND nvl(yep.eoy_primary_flag,'N') = 'Y') -- if multi asg for NI and aggreg for PAYE then select only if primary asg for EOY , P60 will not be generated for other assignments
         )
  AND    (yep.taxable_pay <> 0
          OR yep.tax_paid <> 0
          OR yep.previous_taxable_pay <> 0
          OR yep.previous_tax_paid <> 0
          OR yep.ssp <> 0
          OR yep.student_loans > 0
          OR yep.smp <> 0
          OR yep.sap <> 0         -- new
          OR yep.spp_adopt <> 0   -- new
          OR yep.spp_birth <> 0   -- new
          OR yep.aspp_birth <> 0
          OR yep.aspp_adopt<>0
          OR nvl(pay_gb_eoy_archive.get_arch_str(yep.assignment_action_id,'X_REPORTABLE_NI'),'N') = 'Y'  
         )
  AND    yep.people_group_id = ppg.people_group_id (+)
  AND    (yep.termination_date is null
          OR yep.termination_date > to_date('05/04/' || yea.tax_year, 'DD/MM/YYYY'))
  AND    cp_align = 'N'
  AND    (null is null -- don't check for assignment set in this case 
          OR EXISTS (SELECT 1 FROM hr_assignment_sets has1 
                     WHERE has1.assignment_set_id = null
                     -- AND has1.business_group_id = asg.business_group_id 
                     AND nvl(has1.payroll_id,yea.payroll_id) = yea.payroll_id  
                     AND (NOT EXISTS (SELECT 1 -- chk no amendmts 
                                      FROM   hr_assignment_set_amendments hasa1 
                                      WHERE  hasa1.assignment_set_id = has1.assignment_set_id) 
                          OR EXISTS (SELECT 1 -- chk include amendmts 
                                     FROM   hr_assignment_set_amendments hasa2 
                                     WHERE  hasa2.assignment_set_id = has1.assignment_set_id 
                                     AND    hasa2.assignment_id = yep.assignment_id 
                                     AND    nvl(hasa2.include_or_exclude,'I') = 'I') 
                          OR (NOT EXISTS (SELECT 1 --chk no exlude amendmts 
                                          FROM   hr_assignment_set_amendments hasa3 
                                          WHERE  hasa3.assignment_set_id = has1.assignment_set_id 
                                          AND    hasa3.assignment_id = yep.assignment_id 
                                          AND    nvl(hasa3.include_or_exclude,'I') = 'E') 
                              AND NOT EXISTS (SELECT 1 --and chk no Inc amendmts 
                                              FROM   hr_assignment_set_amendments hasa4
                                              WHERE  hasa4.assignment_set_id = has1.assignment_set_id 
                                              AND    nvl(hasa4.include_or_exclude,'I') = 'I')) -- end checking exclude amendmts 
                         ) -- done checking amendments 
                    ) -- done asg set check when not null 
         ) -- end of asg set check
  /* Added check to generate for Employees who opted for Paper based P60 only 
      If p_restrict_out parameter is not Yes, then the query would generate data for all employees archived by the EOY process minus the records filtered by Assignment Set.
  */
  AND    ((nvl(null,'No') = 'No') OR
          EXISTS(/* Check if assignment level preference is Paper */
                 SELECT null 
                 FROM   per_assignment_extra_info 
                 WHERE  nvl(aei_information1,'D') = 'P' 
                 AND    aei_information_category = 'GB_P60_OUTPUT_OPTION' 
                 AND    assignment_id = yep.assignment_id
                 UNION
                 /* Check if Tax reference level preference is Paper and Assignment level it is not Electronic */
                 SELECT 1
                 FROM   hr_organization_information hoi
                 WHERE  hoi.organization_id = yea.business_group_id
                 AND    hoi.org_information1 = lpad(to_char(yea.tax_district_reference), 3, '0') || '/' || yea.tax_reference_number
                 AND    hoi.org_information_context = 'Tax Details References'
                 AND    nvl(hoi.org_information14,'P') = 'P' 
                 AND    NOT EXISTS(SELECT 1 
                                   FROM   per_assignment_extra_info 
                                   WHERE  nvl(aei_information1,'D') = 'E' 
                                   AND    aei_information_category = 'GB_P60_OUTPUT_OPTION' 
                                   AND    assignment_id = yep.assignment_id)
                ) -- end of exists
         )
  UNION ALL
  SELECT null permit_number
        ,999999999999999 payroll_id
        ,null expense_check_send_to_address
        ,null segment1s
        ,null segment2s
        ,null segment3s
        ,null segment4s
        ,null segment5s
        ,null segment6s
        ,null segment7s
        ,null segment1c
        ,null segment2c
        ,null segment3c
        ,null segment4c
        ,null segment5c
        ,null segment6c
        ,null segment7c
        ,999999999999999 assig_action_id
        ,999999999999999 assignment_id
        ,to_date('31/12/4712','DD/MM/YYYY') effective_end_date
        ,999999999999999 person_id
        ,'XX' ni_1
        ,'XX' ni_2
        ,'XX' ni_3
        ,'XX' ni_4
        ,'X' ni_5
        ,'X' sex
        ,'XXX    XXXXXXX' tax_dist_ref
        ,'XXXXXXXXXXXXXXXXXXXX' tax_dist_name
        ,'9999' tax_year
        ,'XXXXXXXXXXXXXXXXXXXX' last_name
        ,'XXXXXXXXXXXXXXXXX X' forenames
        ,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX' name_and_initials
        ,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX' employer_name
        ,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX' || ' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX' employer_address
        ,'XXXXXXXXXXXXXX' employee_number
        ,999999.99 ssp
        ,999999.99 smp
        ,999999.99 sap
        ,999999.99 spp
        ,999999.99 aspp
        ,'XX XX XXXX' termination_date
        ,'XX XX XXXX' start_of_emp
        ,999999999.99 total_pay
        ,999999999.99 total_tax_deducted
        ,999999999.99 previous_pay
        ,999999999.99 previous_tax_paid
        ,999999999.99 current_pay
         /*99 current_pay_pence,*/
        ,999999999.99 current_tax
        ,999999 student_loans
		/*9999999.99 tax_credits,*/
        ,'X' refund_flag
        ,'XXXX' tax_code
        ,'XX' w1_m1_indicator
        ,9999999.99 wao_amount
		/*99 wao_pence,*/
        ,'XX' week_53_indicator
        ,'XX' dob_dd
        ,'XX' dob_mm
        ,'XXXX' dob_yy
  FROM   fnd_lookups lku
  WHERE  cp_align = 'Y'
  AND    ROWNUM	< 4
  ORDER BY 1, 2, 3, 4, 5, 6, 7, 8, 9, 10;

  CURSOR csr_type_le_ni_data(cp_align IN varchar2)
  IS
  SELECT yev.assignment_action_id assig_action_lnk
        ,yev.assignment_id assignment_id
        ,yev.effective_end_date effective_end_date
        ,yev.ni_category_code ni_category_code
        ,trunc(nvl(yev.earnings/100,0)) earnings
        ,nvl(yev.total_contributions,0)/100 total_contributions
        ,nvl(yev.employees_contributions,0)/100 employees_contributions
        ,trunc(nvl(yev.ni_able_et,0) / 100)	ni_able_et
        ,trunc(nvl(yev.ni_able_lel,0)/100) ni_able_lel
        ,trunc(nvl(yev.ni_able_uel,0)/100) ni_able_uel
        ,trunc(nvl(yev.ni_able_eet,0)/100) ni_able_eet
        ,trunc(nvl(yev.ni_able_uap,0)/100) ni_able_uap
        ,(trunc(nvl(yev.ni_able_lel,0)/100) + trunc(nvl(yev.ni_able_et,0)/100)) lel_et
        ,nvl(yev.employers_rebate,0)/100 employers_rebate
        ,nvl(yev.employees_rebate,0)/100 employees_rebate
        ,yev.scon scon
  FROM   pay_gb_year_end_values_v yev
  WHERE  cp_align = 'N'
  AND    yev.reportable <> 'N'
  AND    yev.assignment_action_id = p_action_context_id
  UNION ALL
  SELECT 999999999999999 assig_action_lnk
        ,999999999999999 assignment_id
        ,to_date('31/12/4712','DD/MM/YYYY')	effective_end_date
        ,'X' ni_category_code
        ,999999 earnings
        ,999999999.99 total_contributions
        ,99999999.99 employees_contributions
        ,99999 ni_able_et
        ,99999 ni_able_lel
        ,99999 ni_able_uel
        ,99999 ni_able_eet
        ,99999 ni_able_uap
        ,99999 lel_et
        ,999999.99 employers_rebate
        ,999999.99 employees_rebate
        ,'XXXXXXXXX' scon
  FROM   fnd_lookups lk1
  WHERE  cp_align = 'Y'
  AND    rownum < 5
  ORDER BY 3;

  CURSOR csr_type_le_cp_data(cp_align IN varchar2)
  IS
  SELECT max(decode(rownumber,1,ni_category_code,null)) nic1
        ,max(decode(rownumber,1,ni_able_lel,null)) lel1
        ,max(decode(rownumber,1,ni_able_et,null)) et1
        ,max(decode(rownumber,1,ni_able_uap,null)) uap1
        ,max(decode(rownumber,1,ni_able_uel,null)) uel1
        ,max(decode(rownumber,1,employees_contributions,null)) ec1
        ,max(decode(rownumber,1,scon,null)) scon1
        ,max(decode(rownumber,2,ni_category_code,null)) nic2
        ,max(decode(rownumber,2,ni_able_lel,null)) lel2
        ,max(decode(rownumber,2,ni_able_et,null)) et2
        ,max(decode(rownumber,2,ni_able_uap,null)) uap2
        ,max(decode(rownumber,2,ni_able_uel,null)) uel2
        ,max(decode(rownumber,2,employees_contributions,null)) ec2
        ,max(decode(rownumber,2,scon,null)) scon2
        ,max(decode(rownumber,3,ni_category_code,null)) nic3
        ,max(decode(rownumber,3,ni_able_lel,null)) lel3
        ,max(decode(rownumber,3,ni_able_et,null)) et3
        ,max(decode(rownumber,3,ni_able_uap,null)) uap3
        ,max(decode(rownumber,3,ni_able_uel,null)) uel3
        ,max(decode(rownumber,3,employees_contributions,null)) ec3
        ,max(decode(rownumber,3,scon,null)) scon3
        ,max(decode(rownumber,4,ni_category_code,null)) nic4
        ,max(decode(rownumber,4,ni_able_lel,null)) lel4
        ,max(decode(rownumber,4,ni_able_et,null)) et4
        ,max(decode(rownumber,4,ni_able_uap,null)) uap4
        ,max(decode(rownumber,4,ni_able_uel,null)) uel4
        ,max(decode(rownumber,4,employees_contributions,null)) ec4
        ,max(decode(rownumber,4,scon,null)) scon4
  FROM   (SELECT yev.ni_category_code ni_category_code
                ,nvl(yev.employees_contributions,0)/100 employees_contributions
                ,trunc(nvl(yev.ni_able_lel,0)/100) ni_able_lel
                ,trunc(nvl(yev.ni_able_uel,0)/100) ni_able_uel
                ,trunc(nvl(yev.ni_able_uap,0)/100) ni_able_uap
                ,trunc(nvl(yev.ni_able_et,0)/100) ni_able_et
                ,substr(yev.scon,2,8) scon
                ,rownum rownumber
          FROM   pay_gb_year_end_values_v  yev
          WHERE  yev.assignment_action_id = p_action_context_id
          AND    cp_align = 'N'
          AND    yev.reportable <> 'N'
         )
  UNION ALL
  SELECT 'X' nic1
        ,99999 lel1
        ,99999 et1
        ,99999 uap1
        ,99999 uel1
        ,99999999.99 ec1
        ,null scon1
        ,null nic2
        ,null lel2
        ,null et2
        ,null uap2
        ,null uel2
        ,null ec2
        ,null scon2
        ,null nic3
        ,null lel3
        ,null et3
        ,null uap3
        ,null uel3
        ,null ec3
        ,null scon3
        ,null nic4
        ,null lel4
        ,null et4
        ,null uap4
        ,null uel4
        ,null ec4
        ,null scon4
  FROM   dual
  WHERE  cp_align = 'Y';

  c_method CONSTANT     varchar2(30) := 'get_p60_report_details_xml';
  l_details_xml         XMLType;
  l_details             varchar2(32767);
  l_business_group_id   number;
  l_employee_rec        csr_employee_data%ROWTYPE;
  l_employee_addr_rec   csr_employee_addr_data%ROWTYPE;
  l_employee_nic1_rec   csr_employee_nic1_data%ROWTYPE;
  l_employee_nic2_rec   csr_employee_nic2_data%ROWTYPE;
  l_employee_nic3_rec   csr_employee_nic3_data%ROWTYPE;
  l_employee_nic4_rec   csr_employee_nic4_data%ROWTYPE;
  l_plain_portrait_main_rec   csr_plain_portrait_main_data%ROWTYPE;
  l_plain_portrait_ni_rec     csr_plain_portrait_ni_data%ROWTYPE;
  l_plain_portrait_cp_rec     csr_plain_portrait_cp_data%ROWTYPE;
  l_get_address               number;
  l_address1                  varchar2(240);
  l_address2                  varchar2(240);
  l_address3                  varchar2(240);
  l_region1                   varchar2(120);
  l_region2                   varchar2(120);
  l_region3                   varchar2(120);
  l_town                      varchar2(60);
  l_type_le_main_rec          csr_type_le_main_data%ROWTYPE;
  l_type_le_ni_rec            csr_type_le_ni_data%ROWTYPE;
  l_type_le_cp_rec            csr_type_le_cp_data%ROWTYPE;

BEGIN

  log_start(c_method, p_assignment_id, p_action_context_id, p_template, p_report_type);

  IF p_action_context_id is not null AND
     p_assignment_id     is not null AND
     p_template          is not null AND 
     p_report_type       is not null THEN

    IF p_report_type = 'RTI' THEN

      --
      -- Store the SQL queries data onto the Details XML
      --
      l_details := '<G_HEADER>';

      OPEN  csr_employee_data;
      FETCH csr_employee_data INTO l_employee_rec;
      CLOSE csr_employee_data;

      l_details := l_details
                    || '<ASSIGNMENT_ACTION_ID>' || dbms_xmlgen.convert
                        (l_employee_rec.assignment_action_id, dbms_xmlgen.entity_encode)
                    || '</ASSIGNMENT_ACTION_ID>';

      l_details := l_details
                    || '<TAX_DIST_REF>' || dbms_xmlgen.convert
                        (l_employee_rec.tax_dist_ref, dbms_xmlgen.entity_encode)
                    || '</TAX_DIST_REF>';

      l_details := l_details
                    || '<TAX_DIST_NAME>' || dbms_xmlgen.convert
                        (l_employee_rec.tax_dist_name, dbms_xmlgen.entity_encode)
                    || '</TAX_DIST_NAME>';

      l_details := l_details
                    || '<EMPLOYER_NAME>' || dbms_xmlgen.convert
                        (l_employee_rec.employer_name, dbms_xmlgen.entity_encode)
                    || '</EMPLOYER_NAME>';

      l_details := l_details
                    || '<EMPLOYER_ADDRESS>' || dbms_xmlgen.convert
                        (l_employee_rec.employer_address, dbms_xmlgen.entity_encode)
                    || '</EMPLOYER_ADDRESS>';

      l_details := l_details
                    || '<NI_1>' || dbms_xmlgen.convert
                        (l_employee_rec.ni_1, dbms_xmlgen.entity_encode)
                    || '</NI_1>';

      l_details := l_details
                    || '<NI_2>' || dbms_xmlgen.convert
                        (l_employee_rec.ni_2, dbms_xmlgen.entity_encode)
                    || '</NI_2>';

      l_details := l_details
                    || '<NI_3>' || dbms_xmlgen.convert
                        (l_employee_rec.ni_3, dbms_xmlgen.entity_encode)
                    || '</NI_3>';

      l_details := l_details
                    || '<NI_4>' || dbms_xmlgen.convert
                        (l_employee_rec.ni_4, dbms_xmlgen.entity_encode)
                    || '</NI_4>';

      l_details := l_details
                    || '<NI_5>' || dbms_xmlgen.convert
                        (l_employee_rec.ni_5, dbms_xmlgen.entity_encode)
                    || '</NI_5>';

      l_details := l_details
                    || '<EMP_NUMBER>' || dbms_xmlgen.convert
                        (l_employee_rec.emp_number, dbms_xmlgen.entity_encode)
                    || '</EMP_NUMBER>';

      l_details := l_details
                    || '<LAST_NAME>' || dbms_xmlgen.convert
                        (l_employee_rec.last_name, dbms_xmlgen.entity_encode)
                    || '</LAST_NAME>';

      l_details := l_details
                    || '<NAME_AND_INITIALS>' || dbms_xmlgen.convert
                        (l_employee_rec.name_and_initials, dbms_xmlgen.entity_encode)
                    || '</NAME_AND_INITIALS>';

      l_details := l_details
                    || '<FORENAMES>' || dbms_xmlgen.convert
                        (l_employee_rec.forenames, dbms_xmlgen.entity_encode)
                    || '</FORENAMES>';

      l_details := l_details
                    || '<L_ADDRESS1>' || dbms_xmlgen.convert
                        (l_employee_rec.l_address1, dbms_xmlgen.entity_encode)
                    || '</L_ADDRESS1>';

      l_details := l_details
                    || '<L_ADDRESS2>' || dbms_xmlgen.convert
                        (l_employee_rec.l_address2, dbms_xmlgen.entity_encode)
                    || '</L_ADDRESS2>';

      l_details := l_details
                    || '<L_ADDRESS3>' || dbms_xmlgen.convert
                        (l_employee_rec.l_address3, dbms_xmlgen.entity_encode)
                    || '</L_ADDRESS3>';

      l_details := l_details
                    || '<L_TOWN>' || dbms_xmlgen.convert
                        (l_employee_rec.l_town, dbms_xmlgen.entity_encode)
                    || '</L_TOWN>';

      l_details := l_details
                    || '<L_REGION1>' || dbms_xmlgen.convert
                        (l_employee_rec.l_region1, dbms_xmlgen.entity_encode)
                    || '</L_REGION1>';

      l_details := l_details
                    || '<L_REGION2>' || dbms_xmlgen.convert
                        (l_employee_rec.l_region2, dbms_xmlgen.entity_encode)
                    || '</L_REGION2>';

      l_details := l_details
                    || '<L_REGION3>' || dbms_xmlgen.convert
                        (l_employee_rec.l_region3, dbms_xmlgen.entity_encode)
                    || '</L_REGION3>';

      l_details := l_details
                    || '<DOB_DD>' || dbms_xmlgen.convert
                        (l_employee_rec.dob_dd, dbms_xmlgen.entity_encode)
                    || '</DOB_DD>';

      l_details := l_details
                    || '<DOB_MM>' || dbms_xmlgen.convert
                        (l_employee_rec.dob_mm, dbms_xmlgen.entity_encode)
                    || '</DOB_MM>';

      l_details := l_details
                    || '<DOB_YY>' || dbms_xmlgen.convert
                        (l_employee_rec.dob_yy, dbms_xmlgen.entity_encode)
                    || '</DOB_YY>';

      l_details := l_details
                    || '<SEX>' || dbms_xmlgen.convert
                        (l_employee_rec.sex, dbms_xmlgen.entity_encode)
                    || '</SEX>';

      l_details := l_details
                    || '<TAX_YEAR>' || dbms_xmlgen.convert
                        (l_employee_rec.tax_year, dbms_xmlgen.entity_encode)
                    || '</TAX_YEAR>';

      l_details := l_details
                    || '<EXPENSE_CHECK>' || dbms_xmlgen.convert
                        (l_employee_rec.expense_check, dbms_xmlgen.entity_encode)
                    || '</EXPENSE_CHECK>';

      l_details := l_details
                    || '<PEOPLE_GRP_ID>' || dbms_xmlgen.convert
                        (l_employee_rec.people_grp_id, dbms_xmlgen.entity_encode)
                    || '</PEOPLE_GRP_ID>';

      l_details := l_details
                    || '<C_BUSINESS_GROUP_ID>' || dbms_xmlgen.convert
                        (l_employee_rec.c_business_group_id, dbms_xmlgen.entity_encode)
                    || '</C_BUSINESS_GROUP_ID>';

      l_details := l_details
                    || '<CURRENT_PAY>' || dbms_xmlgen.convert
                        (l_employee_rec.current_pay, dbms_xmlgen.entity_encode)
                    || '</CURRENT_PAY>';

      l_details := l_details
                    || '<CURRENT_TAX>' || dbms_xmlgen.convert
                        (l_employee_rec.current_tax, dbms_xmlgen.entity_encode)
                    || '</CURRENT_TAX>';

      l_details := l_details
                    || '<STUDENT_LOANS>' || dbms_xmlgen.convert
                        (l_employee_rec.student_loans, dbms_xmlgen.entity_encode)
                    || '</STUDENT_LOANS>';

      l_details := l_details
                    || '<POSTGRADUATE_LOANS>' || dbms_xmlgen.convert
                        (l_employee_rec.postgraduate_loans, dbms_xmlgen.entity_encode)
                    || '</POSTGRADUATE_LOANS>';

      l_details := l_details
                    || '<REFUND_FLAG>' || dbms_xmlgen.convert
                        (l_employee_rec.refund_flag, dbms_xmlgen.entity_encode)
                    || '</REFUND_FLAG>';

      l_details := l_details
                    || '<TAX_CODE>' || dbms_xmlgen.convert
                        (l_employee_rec.tax_code, dbms_xmlgen.entity_encode)
                    || '</TAX_CODE>';

      l_details := l_details
                    || '<W1_M1_INDICATOR>' || dbms_xmlgen.convert
                        (l_employee_rec.w1_m1_indicator, dbms_xmlgen.entity_encode)
                    || '</W1_M1_INDICATOR>';

      l_details := l_details
                    || '<WAO_AMOUNT>' || dbms_xmlgen.convert
                        (l_employee_rec.wao_amount, dbms_xmlgen.entity_encode)
                    || '</WAO_AMOUNT>';

      l_details := l_details
                    || '<TOTAL_PAY>' || dbms_xmlgen.convert
                        (l_employee_rec.total_pay, dbms_xmlgen.entity_encode)
                    || '</TOTAL_PAY>';

      l_details := l_details
                    || '<TOTAL_TAX_DEDUCTED>' || dbms_xmlgen.convert
                        (l_employee_rec.total_tax_deducted, dbms_xmlgen.entity_encode)
                    || '</TOTAL_TAX_DEDUCTED>';

      l_details := l_details
                    || '<PREVIOUS_PAY>' || dbms_xmlgen.convert
                        (l_employee_rec.previous_pay, dbms_xmlgen.entity_encode)
                    || '</PREVIOUS_PAY>';

      l_details := l_details
                    || '<PREVIOUS_TAX_PAID>' || dbms_xmlgen.convert
                        (l_employee_rec.previous_tax_paid, dbms_xmlgen.entity_encode)
                    || '</PREVIOUS_TAX_PAID>';

      l_details := l_details
                    || '<SMP>' || dbms_xmlgen.convert
                        (l_employee_rec.smp, dbms_xmlgen.entity_encode)
                    || '</SMP>';

      l_details := l_details
                    || '<SAP>' || dbms_xmlgen.convert
                        (l_employee_rec.sap, dbms_xmlgen.entity_encode)
                    || '</SAP>';

      l_details := l_details
                    || '<SPP>' || dbms_xmlgen.convert
                        (l_employee_rec.spp, dbms_xmlgen.entity_encode)
                    || '</SPP>';

      l_details := l_details
                    || '<ASPP>' || dbms_xmlgen.convert
                        (l_employee_rec.aspp, dbms_xmlgen.entity_encode)
                    || '</ASPP>';

      l_details := l_details
                    || '<WEEK_53_INDICATOR>' || dbms_xmlgen.convert
                        (l_employee_rec.week_53_indicator, dbms_xmlgen.entity_encode)
                    || '</WEEK_53_INDICATOR>';

      l_details := l_details
                    || '<EMPLOYEE_NUMBER>' || dbms_xmlgen.convert
                        (l_employee_rec.employee_number, dbms_xmlgen.entity_encode)
                    || '</EMPLOYEE_NUMBER>';

      l_details := l_details
                    || '<RETURN_TYPE>' || dbms_xmlgen.convert
                        (l_employee_rec.return_type, dbms_xmlgen.entity_encode)
                    || '</RETURN_TYPE>';

      OPEN  csr_employee_addr_data(cp_expense_check     => l_employee_rec.expense_check
                                  ,cp_business_group_id => l_employee_rec.c_business_group_id
                                  ,cp_people_grp_id     => l_employee_rec.people_grp_id
                                  ,cp_address1          => l_employee_rec.l_address1
                                  ,cp_address2          => l_employee_rec.l_address2
                                  ,cp_address3          => l_employee_rec.l_address3
                                  ,cp_town              => l_employee_rec.l_town
                                  ,cp_region1           => l_employee_rec.l_region1
                                  ,cp_region2           => l_employee_rec.l_region2
                                  ,cp_sort_order1       => null
                                  ,cp_sort_order2       => null
                                  ,cp_sort_order3       => null
                                  ,cp_sort_order4       => null
                                  ,cp_sort_order5       => null
                                  ,cp_sort_order6       => null);
      FETCH csr_employee_addr_data INTO l_employee_addr_rec;
      CLOSE csr_employee_addr_data;

      l_details := l_details
                    || '<C_ADDRESS1>' || dbms_xmlgen.convert
                        (l_employee_addr_rec.c_address1, dbms_xmlgen.entity_encode)
                    || '</C_ADDRESS1>';

      l_details := l_details
                    || '<C_ADDRESS2>' || dbms_xmlgen.convert
                        (l_employee_addr_rec.c_address2, dbms_xmlgen.entity_encode)
                    || '</C_ADDRESS2>';

      l_details := l_details
                    || '<C_ADDRESS3>' || dbms_xmlgen.convert
                        (l_employee_addr_rec.c_address3, dbms_xmlgen.entity_encode)
                    || '</C_ADDRESS3>';

      l_details := l_details
                    || '<C_TOWN>' || dbms_xmlgen.convert
                        (l_employee_addr_rec.c_town, dbms_xmlgen.entity_encode)
                    || '</C_TOWN>';

      l_details := l_details
                    || '<C_REGION1>' || dbms_xmlgen.convert
                        (l_employee_addr_rec.c_region1, dbms_xmlgen.entity_encode)
                    || '</C_REGION1>';

      l_details := l_details
                    || '<C_REGION2>' || dbms_xmlgen.convert
                        (l_employee_addr_rec.c_region2, dbms_xmlgen.entity_encode)
                    || '</C_REGION2>';

      l_details := l_details
                    || '<C_REGION3>' || dbms_xmlgen.convert
                        (l_employee_addr_rec.c_region3, dbms_xmlgen.entity_encode)
                    || '</C_REGION3>';

      l_details := l_details
                    || '<C_REGION4>' || dbms_xmlgen.convert
                        (l_employee_addr_rec.c_region4, dbms_xmlgen.entity_encode)
                    || '</C_REGION4>';

      OPEN  csr_employee_nic1_data;
      FETCH csr_employee_nic1_data INTO l_employee_nic1_rec;
      CLOSE csr_employee_nic1_data;

      l_details := l_details
                    || '<CP_NIC1>' || dbms_xmlgen.convert
                        (l_employee_nic1_rec.cp_nic1, dbms_xmlgen.entity_encode)
                    || '</CP_NIC1>';

      l_details := l_details
                    || '<CP_LEL1>' || dbms_xmlgen.convert
                        (l_employee_nic1_rec.cp_lel1, dbms_xmlgen.entity_encode)
                    || '</CP_LEL1>';

      l_details := l_details
                    || '<CP_ET1>' || dbms_xmlgen.convert
                        (l_employee_nic1_rec.cp_et1, dbms_xmlgen.entity_encode)
                    || '</CP_ET1>';

      l_details := l_details
                    || '<CP_UAP1>' || dbms_xmlgen.convert
                        (l_employee_nic1_rec.cp_uap1, dbms_xmlgen.entity_encode)
                    || '</CP_UAP1>';

      l_details := l_details
                    || '<CP_UEL1>' || dbms_xmlgen.convert
                        (l_employee_nic1_rec.cp_uel1, dbms_xmlgen.entity_encode)
                    || '</CP_UEL1>';

      l_details := l_details
                    || '<CP_EC1>' || dbms_xmlgen.convert
                        (l_employee_nic1_rec.cp_ec1, dbms_xmlgen.entity_encode)
                    || '</CP_EC1>';

      OPEN  csr_employee_nic2_data;
      FETCH csr_employee_nic2_data INTO l_employee_nic2_rec;
      CLOSE csr_employee_nic2_data;

      l_details := l_details
                    || '<CP_NIC2>' || dbms_xmlgen.convert
                        (l_employee_nic2_rec.cp_nic2, dbms_xmlgen.entity_encode)
                    || '</CP_NIC2>';

      l_details := l_details
                    || '<CP_LEL2>' || dbms_xmlgen.convert
                        (l_employee_nic2_rec.cp_lel2, dbms_xmlgen.entity_encode)
                    || '</CP_LEL2>';

      l_details := l_details
                    || '<CP_ET2>' || dbms_xmlgen.convert
                        (l_employee_nic2_rec.cp_et2, dbms_xmlgen.entity_encode)
                    || '</CP_ET2>';

      l_details := l_details
                    || '<CP_UAP2>' || dbms_xmlgen.convert
                        (l_employee_nic2_rec.cp_uap2, dbms_xmlgen.entity_encode)
                    || '</CP_UAP2>';

      l_details := l_details
                    || '<CP_UEL2>' || dbms_xmlgen.convert
                        (l_employee_nic2_rec.cp_uel2, dbms_xmlgen.entity_encode)
                    || '</CP_UEL2>';

      l_details := l_details
                    || '<CP_EC2>' || dbms_xmlgen.convert
                        (l_employee_nic2_rec.cp_ec2, dbms_xmlgen.entity_encode)
                    || '</CP_EC2>';

      OPEN  csr_employee_nic3_data;
      FETCH csr_employee_nic3_data INTO l_employee_nic3_rec;
      CLOSE csr_employee_nic3_data;

      l_details := l_details
                    || '<CP_NIC3>' || dbms_xmlgen.convert
                        (l_employee_nic3_rec.cp_nic3, dbms_xmlgen.entity_encode)
                    || '</CP_NIC3>';

      l_details := l_details
                    || '<CP_LEL3>' || dbms_xmlgen.convert
                        (l_employee_nic3_rec.cp_lel3, dbms_xmlgen.entity_encode)
                    || '</CP_LEL3>';

      l_details := l_details
                    || '<CP_ET3>' || dbms_xmlgen.convert
                        (l_employee_nic3_rec.cp_et3, dbms_xmlgen.entity_encode)
                    || '</CP_ET3>';

      l_details := l_details
                    || '<cp_uap3>' || dbms_xmlgen.convert
                        (l_employee_nic3_rec.cp_uap3, dbms_xmlgen.entity_encode)
                    || '</cp_uap3>';

      l_details := l_details
                    || '<CP_UEL3>' || dbms_xmlgen.convert
                        (l_employee_nic3_rec.cp_uel3, dbms_xmlgen.entity_encode)
                    || '</CP_UEL3>';

      l_details := l_details
                    || '<CP_EC3>' || dbms_xmlgen.convert
                        (l_employee_nic3_rec.cp_ec3, dbms_xmlgen.entity_encode)
                    || '</CP_EC3>';

      OPEN  csr_employee_nic4_data;
      FETCH csr_employee_nic4_data INTO l_employee_nic4_rec;
      CLOSE csr_employee_nic4_data;

      l_details := l_details
                    || '<CP_NIC4>' || dbms_xmlgen.convert
                        (l_employee_nic4_rec.cp_nic4, dbms_xmlgen.entity_encode)
                    || '</CP_NIC4>';

      l_details := l_details
                    || '<CP_LEL4>' || dbms_xmlgen.convert
                        (l_employee_nic4_rec.cp_lel4, dbms_xmlgen.entity_encode)
                    || '</CP_LEL4>';

      l_details := l_details
                    || '<CP_ET4>' || dbms_xmlgen.convert
                        (l_employee_nic4_rec.cp_et4, dbms_xmlgen.entity_encode)
                    || '</CP_ET4>';

      l_details := l_details
                    || '<CP_UAP4>' || dbms_xmlgen.convert
                        (l_employee_nic4_rec.cp_uap4, dbms_xmlgen.entity_encode)
                    || '</CP_UAP4>';

      l_details := l_details
                    || '<CP_UEL4>' || dbms_xmlgen.convert
                        (l_employee_nic4_rec.cp_uel4, dbms_xmlgen.entity_encode)
                    || '</CP_UEL4>';

      l_details := l_details
                    || '<CP_EC4>' || dbms_xmlgen.convert
                        (l_employee_nic4_rec.cp_ec4, dbms_xmlgen.entity_encode)
                    || '</CP_EC4>';

--Following tags will be added to the xml output only when template passed is E60
      IF p_template like 'PYGBEP60%' THEN
      
      l_details := l_details
                    || '<NI>' || dbms_xmlgen.convert
                        (l_employee_rec.ni, dbms_xmlgen.entity_encode)
                    || '</NI>';
                    
      l_details := l_details
                    || '<NAME_AND_INITIAL>' || dbms_xmlgen.convert
                        (l_employee_rec.name_and_initials, dbms_xmlgen.entity_encode)
                    || '</NAME_AND_INITIAL>';
                    
      l_details := l_details
                    || '<YEAR>' || dbms_xmlgen.convert
                        (l_employee_rec.tax_year, dbms_xmlgen.entity_encode)
                    || '</YEAR>';
                    
      l_details := l_details || '<LIST_G_NI_CONTRIB>';

      FOR l_nic_rec IN csr_employee_nic_data
      LOOP
        
        l_details := l_details || '<G_NI_CONTRIB>'; 
        
        l_details := l_details
                    || '<NI_CATEGORY_CODE>' || dbms_xmlgen.convert
                        (l_nic_rec.cp_nic, dbms_xmlgen.entity_encode)
                    || '</NI_CATEGORY_CODE>';

        l_details := l_details
                    || '<NI_ABLE_LEL>' || dbms_xmlgen.convert
                        (l_nic_rec.cp_lel, dbms_xmlgen.entity_encode)
                    || '</NI_ABLE_LEL>';

        l_details := l_details
                    || '<NI_ABLE_ET>' || dbms_xmlgen.convert
                        (l_nic_rec.cp_et, dbms_xmlgen.entity_encode)
                    || '</NI_ABLE_ET>';

        l_details := l_details
                    || '<NI_ABLE_UEL>' || dbms_xmlgen.convert
                        (l_nic_rec.cp_uel, dbms_xmlgen.entity_encode)
                    || '</NI_ABLE_UEL>';

        l_details := l_details
                    || '<EMPLOYEES_CONTRIBUTIONS>' || dbms_xmlgen.convert
                        (l_nic_rec.cp_ec, dbms_xmlgen.entity_encode)
                    || '</EMPLOYEES_CONTRIBUTIONS>';
                    
        l_details := l_details || '</G_NI_CONTRIB>'; 
      END LOOP;
      
      l_details := l_details || '</LIST_G_NI_CONTRIB>';

      END IF; --template check if

      l_details := l_details || '</G_HEADER>';

    ELSIF p_report_type = 'EOY' THEN

      IF p_template IN ('PYGBP60PP','PYGBP60S') THEN

        OPEN  csr_business_group_id;
        FETCH csr_business_group_id INTO l_business_group_id;
        CLOSE csr_business_group_id;

        fnd_profile.get('GB_EMPLOYEE_NAME_REPORT_FORMAT', l_profile_name_format);

        OPEN  csr_plain_portrait_main_data(cp_align => 'N');
        FETCH csr_plain_portrait_main_data INTO l_plain_portrait_main_rec;
        CLOSE csr_plain_portrait_main_data;

        --
        -- Store the SQL queries data onto the Details XML
        --
        l_details := '<PAYP60P>';
        l_details := l_details || '<LIST_G_MAIN>';
        l_details := l_details || '<G_MAIN>';

        l_details := l_details
                      || '<ASPP>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.aspp, dbms_xmlgen.entity_encode)
                      || '</ASPP>';

        l_details := l_details
                      || '<NAME_AND_INITIAL>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.name_and_initial, dbms_xmlgen.entity_encode)
                      || '</NAME_AND_INITIAL>';

        l_details := l_details
                      || '<SEGMENT1S>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment1s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT1S>';

        l_details := l_details
                      || '<SEGMENT2S>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment2s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT2S>';

        l_details := l_details
                      || '<SEGMENT3S>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment3s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT3S>';

        l_details := l_details
                      || '<SEGMENT4S>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment4s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT4S>';

        l_details := l_details
                      || '<SEGMENT5S>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment5s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT5S>';

        l_details := l_details
                      || '<SEGMENT6S>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment6s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT6S>';

        l_details := l_details
                      || '<SEGMENT7S>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment7s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT7S>';

        l_details := l_details
                      || '<SEGMENT1C>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment1c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT1C>';

        l_details := l_details
                      || '<SEGMENT2C>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment2c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT2C>';

        l_details := l_details
                      || '<SEGMENT3C>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment3c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT3C>';

        l_details := l_details
                      || '<SEGMENT4C>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment4c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT4C>';

        l_details := l_details
                      || '<SEGMENT5C>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment5c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT5C>';

        l_details := l_details
                      || '<SEGMENT6C>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment6c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT6C>';

        l_details := l_details
                      || '<SEGMENT7C>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.segment7c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT7C>';

        l_details := l_details
                      || '<SAP>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.sap, dbms_xmlgen.entity_encode)
                      || '</SAP>';

        l_details := l_details
                      || '<SPP>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.spp, dbms_xmlgen.entity_encode)
                      || '</SPP>';

        l_details := l_details
                      || '<STUDENT_LOANS>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.student_loans, dbms_xmlgen.entity_encode)
                      || '</STUDENT_LOANS>';

        l_details := l_details
                      || '<ASSIG_ACTION_ID>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.assig_action_id, dbms_xmlgen.entity_encode)
                      || '</ASSIG_ACTION_ID>';

        l_details := l_details
                      || '<MIDDLE_NAME>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.middle_name, dbms_xmlgen.entity_encode)
                      || '</MIDDLE_NAME>';

        l_details := l_details
                      || '<START_OF_EMP>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.start_of_emp, dbms_xmlgen.entity_encode)
                      || '</START_OF_EMP>';

        l_details := l_details
                      || '<PAYROLL_ID>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.payroll_id, dbms_xmlgen.entity_encode)
                      || '</PAYROLL_ID>';

        l_details := l_details
                      || '<ASSIGNMENT_ID>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.assignment_id, dbms_xmlgen.entity_encode)
                      || '</ASSIGNMENT_ID>';

        l_details := l_details
                      || '<EFFECTIVE_END_DATE>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.effective_end_date, dbms_xmlgen.entity_encode)
                      || '</EFFECTIVE_END_DATE>';

        l_details := l_details
                      || '<PERSON_ID>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.person_id, dbms_xmlgen.entity_encode)
                      || '</PERSON_ID>';

        l_details := l_details
                      || '<EXPENSE_CHECK_SEND_TO_ADDRESS>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.expense_check_send_to_address, dbms_xmlgen.entity_encode)
                      || '</EXPENSE_CHECK_SEND_TO_ADDRESS>';

        l_details := l_details
                      || '<NI_1>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.ni_1, dbms_xmlgen.entity_encode)
                      || '</NI_1>';

        l_details := l_details
                      || '<NI_2>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.ni_2, dbms_xmlgen.entity_encode)
                      || '</NI_2>';

        l_details := l_details
                      || '<NI_3>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.ni_3, dbms_xmlgen.entity_encode)
                      || '</NI_3>';

        l_details := l_details
                      || '<NI_4>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.ni_4, dbms_xmlgen.entity_encode)
                      || '</NI_4>';

        l_details := l_details
                      || '<NI_5>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.ni_5, dbms_xmlgen.entity_encode)
                      || '</NI_5>';

        l_details := l_details
                      || '<SEX>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.sex, dbms_xmlgen.entity_encode)
                      || '</SEX>';

        l_details := l_details
                      || '<TAX_DIST_REF>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.tax_dist_ref, dbms_xmlgen.entity_encode)
                      || '</TAX_DIST_REF>';

        l_details := l_details
                      || '<TAX_YEAR>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.tax_year, dbms_xmlgen.entity_encode)
                      || '</TAX_YEAR>';

        l_details := l_details
                      || '<LAST_NAME>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.last_name, dbms_xmlgen.entity_encode)
                      || '</LAST_NAME>';

        l_details := l_details
                      || '<FORENAMES>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.forenames, dbms_xmlgen.entity_encode)
                      || '</FORENAMES>';

        l_details := l_details
                      || '<EMPLOYER_NAME>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.employer_name, dbms_xmlgen.entity_encode)
                      || '</EMPLOYER_NAME>';

        l_details := l_details
                      || '<EMPLOYER_ADDRESS>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.employer_address, dbms_xmlgen.entity_encode)
                      || '</EMPLOYER_ADDRESS>';

        l_details := l_details
                      || '<SSP>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.ssp, dbms_xmlgen.entity_encode)
                      || '</SSP>';

        l_details := l_details
                      || '<SMP>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.smp, dbms_xmlgen.entity_encode)
                      || '</SMP>';

        l_details := l_details
                      || '<TERMINATION_DATE>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.termination_date, dbms_xmlgen.entity_encode)
                      || '</TERMINATION_DATE>';

        l_details := l_details
                      || '<TOTAL_PAY>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.total_pay, dbms_xmlgen.entity_encode)
                      || '</TOTAL_PAY>';

        l_details := l_details
                      || '<TOTAL_TAX_DEDUCTED>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.total_tax_deducted, dbms_xmlgen.entity_encode)
                      || '</TOTAL_TAX_DEDUCTED>';

        l_details := l_details
                      || '<PREVIOUS_PAY>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.previous_pay, dbms_xmlgen.entity_encode)
                      || '</PREVIOUS_PAY>';

        l_details := l_details
                      || '<PREVIOUS_TAX_PAID>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.previous_tax_paid, dbms_xmlgen.entity_encode)
                      || '</PREVIOUS_TAX_PAID>';

        l_details := l_details
                      || '<CURRENT_PAY>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.current_pay, dbms_xmlgen.entity_encode)
                      || '</CURRENT_PAY>';

        l_details := l_details
                      || '<CURRENT_TAX>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.current_tax, dbms_xmlgen.entity_encode)
                      || '</CURRENT_TAX>';

        l_details := l_details
                      || '<REFUND_FLAG>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.refund_flag, dbms_xmlgen.entity_encode)
                      || '</REFUND_FLAG>';

        l_details := l_details
                      || '<TAX_CODE>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.tax_code, dbms_xmlgen.entity_encode)
                      || '</TAX_CODE>';

        l_details := l_details
                      || '<WAO_AMOUNT>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.wao_amount, dbms_xmlgen.entity_encode)
                      || '</WAO_AMOUNT>';

        l_details := l_details
                      || '<WEEK_53_INDICATOR>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.week_53_indicator, dbms_xmlgen.entity_encode)
                      || '</WEEK_53_INDICATOR>';

        l_details := l_details
                      || '<EMPLOYEE_NUMBER>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.employee_number, dbms_xmlgen.entity_encode)
                      || '</EMPLOYEE_NUMBER>';

        l_details := l_details
                      || '<PERMIT_NUMBER>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.permit_number, dbms_xmlgen.entity_encode)
                      || '</PERMIT_NUMBER>';

        l_details := l_details
                      || '<W1_M1_INDICATOR>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.w1_m1_indicator, dbms_xmlgen.entity_encode)
                      || '</W1_M1_INDICATOR>';

        l_details := l_details
                      || '<DOB_DD>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.dob_dd, dbms_xmlgen.entity_encode)
                      || '</DOB_DD>';

        l_details := l_details
                      || '<DOB_MM>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.dob_mm, dbms_xmlgen.entity_encode)
                      || '</DOB_MM>';

        l_details := l_details
                      || '<DOB_YY>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.dob_yy, dbms_xmlgen.entity_encode)
                      || '</DOB_YY>';

        l_details := l_details
                      || '<TAX_DIST_NAME>' || dbms_xmlgen.convert
                          (l_plain_portrait_main_rec.tax_dist_name, dbms_xmlgen.entity_encode)
                      || '</TAX_DIST_NAME>';

        l_details := l_details || '<LIST_G_NI_CONTRIB>';

        l_details := l_details || '<G_NI_CONTRIB>';

        OPEN  csr_plain_portrait_ni_data(cp_align => 'N');
        FETCH csr_plain_portrait_ni_data INTO l_plain_portrait_ni_rec;
        CLOSE csr_plain_portrait_ni_data;

        l_details := l_details
                      || '<NI_ABLE_UAP>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.ni_able_uap, dbms_xmlgen.entity_encode)
                      || '</NI_ABLE_UAP>';

        l_details := l_details
                      || '<NI_REFUND>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.ni_refund, dbms_xmlgen.entity_encode)
                      || '</NI_REFUND>';

        l_details := l_details
                      || '<NI_ABLE_ET>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.ni_able_et, dbms_xmlgen.entity_encode)
                      || '</NI_ABLE_ET>';

        l_details := l_details
                      || '<EMPLOYEES_REBATE>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.employees_rebate, dbms_xmlgen.entity_encode)
                      || '</EMPLOYEES_REBATE>';

        l_details := l_details
                      || '<TOTAL_REBATE>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.total_rebate, dbms_xmlgen.entity_encode)
                      || '</TOTAL_REBATE>';

        l_details := l_details
                      || '<ASSIG_ACTION_LNK>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.assig_action_lnk, dbms_xmlgen.entity_encode)
                      || '</ASSIG_ACTION_LNK>';

        l_details := l_details
                      || '<NI_ABLE_LEL>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.ni_able_lel, dbms_xmlgen.entity_encode)
                      || '</NI_ABLE_LEL>';

        l_details := l_details
                      || '<NI_ABLE_UEL>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.ni_able_uel, dbms_xmlgen.entity_encode)
                      || '</NI_ABLE_UEL>';

        l_details := l_details
                      || '<EMPLOYERS_REBATE>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.employers_rebate, dbms_xmlgen.entity_encode)
                      || '</EMPLOYERS_REBATE>';

        l_details := l_details
                      || '<SCON>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.scon, dbms_xmlgen.entity_encode)
                      || '</SCON>';

        l_details := l_details
                      || '<ASSIGNMENT_ID1>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.assignment_id, dbms_xmlgen.entity_encode)
                      || '</ASSIGNMENT_ID1>';

        l_details := l_details
                      || '<EFFECTIVE_END_DATE1>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.effective_end_date, dbms_xmlgen.entity_encode)
                      || '</EFFECTIVE_END_DATE1>';

        l_details := l_details
                      || '<NI_CATEGORY_CODE>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.ni_category_code, dbms_xmlgen.entity_encode)
                      || '</NI_CATEGORY_CODE>';

        l_details := l_details
                      || '<EARNINGS>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.earnings, dbms_xmlgen.entity_encode)
                      || '</EARNINGS>';

        l_details := l_details
                      || '<TOTAL_CONTRIBUTIONS>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.total_contributions, dbms_xmlgen.entity_encode)
                      || '</TOTAL_CONTRIBUTIONS>';

        l_details := l_details
                      || '<EMPLOYEES_CONTRIBUTIONS>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.employees_contributions, dbms_xmlgen.entity_encode)
                      || '</EMPLOYEES_CONTRIBUTIONS>';

        l_details := l_details || '</G_NI_CONTRIB>';

        l_details := l_details || '</LIST_G_NI_CONTRIB>';

        OPEN  csr_plain_portrait_cp_data(cp_align => 'N');
        FETCH csr_plain_portrait_cp_data INTO l_plain_portrait_cp_rec;
          IF csr_plain_portrait_cp_data%FOUND THEN
            l_details := l_details
                          || '<CF_GET_CONTRIBUTIONS>' || dbms_xmlgen.convert
                              (1, dbms_xmlgen.entity_encode)
                          || '</CF_GET_CONTRIBUTIONS>';
          ELSE
            l_details := l_details
                          || '<CF_GET_CONTRIBUTIONS>'
                          || '</CF_GET_CONTRIBUTIONS>';
          END IF;
        CLOSE csr_plain_portrait_cp_data;

        l_details := l_details
                      || '<CP_NIC1>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.nic1, dbms_xmlgen.entity_encode)
                      || '</CP_NIC1>';

        l_details := l_details
                      || '<CP_LEL1>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.lel1, dbms_xmlgen.entity_encode)
                      || '</CP_LEL1>';

        l_details := l_details
                      || '<CP_UEL1>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.uel1, dbms_xmlgen.entity_encode)
                      || '</CP_UEL1>';

        l_details := l_details
                      || '<CP_ET1>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.et1, dbms_xmlgen.entity_encode)
                      || '</CP_ET1>';

        l_details := l_details
                      || '<CP_UAP1>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.uap1, dbms_xmlgen.entity_encode)
                      || '</CP_UAP1>';

        l_details := l_details
                      || '<CP_EC1>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.ec1, dbms_xmlgen.entity_encode)
                      || '</CP_EC1>';

        l_details := l_details
                      || '<CP_SCON1>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.scon1, dbms_xmlgen.entity_encode)
                      || '</CP_SCON1>';

        l_details := l_details
                      || '<CP_NIC2>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.nic2, dbms_xmlgen.entity_encode)
                      || '</CP_NIC2>';

        l_details := l_details
                      || '<CP_LEL2>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.lel2, dbms_xmlgen.entity_encode)
                      || '</CP_LEL2>';

        l_details := l_details
                      || '<CP_UEL2>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.uel2, dbms_xmlgen.entity_encode)
                      || '</CP_UEL2>';

        l_details := l_details
                      || '<CP_ET2>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.et2, dbms_xmlgen.entity_encode)
                      || '</CP_ET2>';

        l_details := l_details
                      || '<CP_UAP2>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.uap2, dbms_xmlgen.entity_encode)
                      || '</CP_UAP2>';

        l_details := l_details
                      || '<CP_EC2>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.ec2, dbms_xmlgen.entity_encode)
                      || '</CP_EC2>';

        l_details := l_details
                      || '<CP_SCON2>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.scon2, dbms_xmlgen.entity_encode)
                      || '</CP_SCON2>';

        l_details := l_details
                      || '<CP_NIC3>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.nic3, dbms_xmlgen.entity_encode)
                      || '</CP_NIC3>';

        l_details := l_details
                      || '<CP_LEL3>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.lel3, dbms_xmlgen.entity_encode)
                      || '</CP_LEL3>';

        l_details := l_details
                      || '<CP_UEL3>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.uel3, dbms_xmlgen.entity_encode)
                      || '</CP_UEL3>';

        l_details := l_details
                      || '<CP_ET3>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.et3, dbms_xmlgen.entity_encode)
                      || '</CP_ET3>';

        l_details := l_details
                      || '<CP_UAP3>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.uap3, dbms_xmlgen.entity_encode)
                      || '</CP_UAP3>';

        l_details := l_details
                      || '<CP_EC3>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.ec3, dbms_xmlgen.entity_encode)
                      || '</CP_EC3>';

        l_details := l_details
                      || '<CP_SCON3>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.scon3, dbms_xmlgen.entity_encode)
                      || '</CP_SCON3>';

        l_details := l_details
                      || '<CP_NIC4>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.nic4, dbms_xmlgen.entity_encode)
                      || '</CP_NIC4>';

        l_details := l_details
                      || '<CP_LEL4>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.lel4, dbms_xmlgen.entity_encode)
                      || '</CP_LEL4>';

        l_details := l_details
                      || '<CP_UEL4>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.uel4, dbms_xmlgen.entity_encode)
                      || '</CP_UEL4>';

        l_details := l_details
                      || '<CP_ET4>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.et4, dbms_xmlgen.entity_encode)
                      || '</CP_ET4>';

        l_details := l_details
                      || '<CP_UAP4>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.uap4, dbms_xmlgen.entity_encode)
                      || '</CP_UAP4>';

        l_details := l_details
                      || '<CP_EC4>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.ec4, dbms_xmlgen.entity_encode)
                      || '</CP_EC4>';

        l_details := l_details
                      || '<CP_SCON4>' || dbms_xmlgen.convert
                          (l_plain_portrait_cp_rec.scon4, dbms_xmlgen.entity_encode)
                      || '</CP_SCON4>';

        l_details := l_details
                      || '<CS_EMPLOYERS_REBATE>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.employers_rebate, dbms_xmlgen.entity_encode)
                      || '</CS_EMPLOYERS_REBATE>';

        l_details := l_details
                      || '<CS_EMPLOYEES_REBATE>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.employees_rebate, dbms_xmlgen.entity_encode)
                      || '</CS_EMPLOYEES_REBATE>';

        l_details := l_details
                      || '<CS_TOTAL_REBATE>' || dbms_xmlgen.convert
                          (l_plain_portrait_ni_rec.total_rebate, dbms_xmlgen.entity_encode)
                      || '</CS_TOTAL_REBATE>';

        l_get_address := portrait_populate_addr_fields
                           (p_person_id                  => xxas_util_pkg.get_person_from_asg(p_assignment_id,trunc(sysdate))
                           ,p_align                      => 'N'
                           ,p_expense_check_send_to_addr => l_plain_portrait_main_rec.expense_check_send_to_address
                           ,p_address1                   => l_address1
                           ,p_address2                   => l_address2
                           ,p_address3                   => l_address3
                           ,p_region1                    => l_region1
                           ,p_region2                    => l_region2
                           ,p_region3                    => l_region3
                           ,p_town                       => l_town);

        l_details := l_details
                      || '<C_GET_ADDRESS>' || dbms_xmlgen.convert
                          (l_get_address, dbms_xmlgen.entity_encode)
                      || '</C_GET_ADDRESS>';

        l_details := l_details
                      || '<C_ADDRESS1>' || dbms_xmlgen.convert
                          (l_address1, dbms_xmlgen.entity_encode)
                      || '</C_ADDRESS1>';

        l_details := l_details
                      || '<C_ADDRESS2>' || dbms_xmlgen.convert
                          (l_address2, dbms_xmlgen.entity_encode)
                      || '</C_ADDRESS2>';

        l_details := l_details
                      || '<C_ADDRESS3>' || dbms_xmlgen.convert
                          (l_address3, dbms_xmlgen.entity_encode)
                      || '</C_ADDRESS3>';

        l_details := l_details
                      || '<C_TOWN>' || dbms_xmlgen.convert
                          (l_town, dbms_xmlgen.entity_encode)
                      || '</C_TOWN>';

        l_details := l_details
                      || '<C_REGION1>' || dbms_xmlgen.convert
                          (l_region1, dbms_xmlgen.entity_encode)
                      || '</C_REGION1>';

        l_details := l_details
                      || '<C_REGION2>' || dbms_xmlgen.convert
                          (l_region2, dbms_xmlgen.entity_encode)
                      || '</C_REGION2>';

        l_details := l_details
                      || '<C_REGION3>' || dbms_xmlgen.convert
                          (l_region3, dbms_xmlgen.entity_encode)
                      || '</C_REGION3>';

        l_details := l_details
                      || '<C_SEG_NAME1>'
                      || '</C_SEG_NAME1>';

        l_details := l_details
                      || '<C_SEG_NAME2>'
                      || '</C_SEG_NAME2>';

        l_details := l_details
                      || '<C_SEG_NAME3>'
                      || '</C_SEG_NAME3>';

        l_details := l_details
                      || '<C_SEG_NAME4>'
                      || '</C_SEG_NAME4>';

        l_details := l_details
                      || '<C_SEG_NAME5>'
                      || '</C_SEG_NAME5>';

        l_details := l_details
                      || '<C_SEG_NAME6>'
                      || '</C_SEG_NAME6>';

        l_details := l_details || '</G_MAIN>';
        l_details := l_details || '</LIST_G_MAIN>';

        l_details := l_details
                      || '<C_BUSINESS_GROUP_ID>' || dbms_xmlgen.convert
                          (l_business_group_id, dbms_xmlgen.entity_encode)
                      || '</C_BUSINESS_GROUP_ID>';

        l_details := l_details || '</PAYP60P>';

      ELSIF p_template IN ('PYGBP60LE') THEN

        OPEN  csr_business_group_id;
        FETCH csr_business_group_id INTO l_business_group_id;
        CLOSE csr_business_group_id;

        fnd_profile.get('GB_EMPLOYEE_NAME_REPORT_FORMAT', l_profile_name_format);

        OPEN  csr_type_le_main_data(cp_align => 'N');
        FETCH csr_type_le_main_data INTO l_type_le_main_rec;
        CLOSE csr_type_le_main_data;

        --
        -- Store the SQL queries data onto the Details XML
        --
        l_details := '<PAYP60LE>';
        l_details := l_details || '<LIST_G_MAIN>';
        l_details := l_details || '<G_MAIN>';

        l_details := l_details
                      || '<ASPP>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.aspp, dbms_xmlgen.entity_encode)
                      || '</ASPP>';

        l_details := l_details
                      || '<SEGMENT7S>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment7s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT7S>';

        l_details := l_details
                      || '<SEGMENT5S>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment5s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT5S>';

        l_details := l_details
                      || '<SEGMENT6S>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment6s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT6S>';

        l_details := l_details
                      || '<SEGMENT6C>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment6c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT6C>';

        l_details := l_details
                      || '<SAP>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.sap, dbms_xmlgen.entity_encode)
                      || '</SAP>';

        l_details := l_details
                      || '<SPP>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.spp, dbms_xmlgen.entity_encode)
                      || '</SPP>';

        l_details := l_details
                      || '<STUDENT_LOANS>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.student_loans, dbms_xmlgen.entity_encode)
                      || '</STUDENT_LOANS>';

        l_details := l_details
                      || '<ASSIG_ACTION_ID>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.assig_action_id, dbms_xmlgen.entity_encode)
                      || '</ASSIG_ACTION_ID>';

        l_details := l_details
                      || '<NAME_AND_INITIALS>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.name_and_initials, dbms_xmlgen.entity_encode)
                      || '</NAME_AND_INITIALS>';

        l_details := l_details
                      || '<START_OF_EMP>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.start_of_emp, dbms_xmlgen.entity_encode)
                      || '</START_OF_EMP>';

        l_details := l_details
                      || '<SEGMENT7C>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment7c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT7C>';

        l_details := l_details
                      || '<SEGMENT2C>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment2c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT2C>';

        l_details := l_details
                      || '<SEGMENT3C>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment3c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT3C>';

        l_details := l_details
                      || '<SEGMENT4C>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment4c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT4C>';

        l_details := l_details
                      || '<SEGMENT5C>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment5c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT5C>';

        l_details := l_details
                      || '<SEGMENT6C>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment6c, dbms_xmlgen.entity_encode)
                      || '</SEGMENT6C>';

        l_details := l_details
                      || '<PAYROLL_ID>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.payroll_id, dbms_xmlgen.entity_encode)
                      || '</PAYROLL_ID>';

        l_details := l_details
                      || '<ASSIGNMENT_ID>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.assignment_id, dbms_xmlgen.entity_encode)
                      || '</ASSIGNMENT_ID>';

        l_details := l_details
                      || '<EFFECTIVE_END_DATE>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.effective_end_date, dbms_xmlgen.entity_encode)
                      || '</EFFECTIVE_END_DATE>';

        l_details := l_details
                      || '<PERSON_ID>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.person_id, dbms_xmlgen.entity_encode)
                      || '</PERSON_ID>';

        l_details := l_details
                      || '<NI_1>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.ni_1, dbms_xmlgen.entity_encode)
                      || '</NI_1>';

        l_details := l_details
                      || '<NI_2>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.ni_2, dbms_xmlgen.entity_encode)
                      || '</NI_2>';

        l_details := l_details
                      || '<NI_3>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.ni_3, dbms_xmlgen.entity_encode)
                      || '</NI_3>';

        l_details := l_details
                      || '<NI_4>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.ni_4, dbms_xmlgen.entity_encode)
                      || '</NI_4>';

        l_details := l_details
                      || '<NI_5>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.ni_5, dbms_xmlgen.entity_encode)
                      || '</NI_5>';

        l_details := l_details
                      || '<SEX>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.sex, dbms_xmlgen.entity_encode)
                      || '</SEX>';

        l_details := l_details
                      || '<TAX_DIST_REF>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.tax_dist_ref, dbms_xmlgen.entity_encode)
                      || '</TAX_DIST_REF>';

        l_details := l_details
                      || '<TAX_YEAR>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.tax_year, dbms_xmlgen.entity_encode)
                      || '</TAX_YEAR>';

        l_details := l_details
                      || '<FORENAMES>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.forenames, dbms_xmlgen.entity_encode)
                      || '</FORENAMES>';

        l_details := l_details
                      || '<EMPLOYER_NAME>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.employer_name, dbms_xmlgen.entity_encode)
                      || '</EMPLOYER_NAME>';

        l_details := l_details
                      || '<EMPLOYER_ADDRESS>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.employer_address, dbms_xmlgen.entity_encode)
                      || '</EMPLOYER_ADDRESS>';

        l_details := l_details
                      || '<SSP>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.ssp, dbms_xmlgen.entity_encode)
                      || '</SSP>';

        l_details := l_details
                      || '<SMP>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.smp, dbms_xmlgen.entity_encode)
                      || '</SMP>';

        l_details := l_details
                      || '<TERMINATION_DATE>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.termination_date, dbms_xmlgen.entity_encode)
                      || '</TERMINATION_DATE>';

        l_details := l_details
                      || '<TOTAL_PAY>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.total_pay, dbms_xmlgen.entity_encode)
                      || '</TOTAL_PAY>';

        l_details := l_details
                      || '<TOTAL_TAX_DEDUCTED>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.total_tax_deducted, dbms_xmlgen.entity_encode)
                      || '</TOTAL_TAX_DEDUCTED>';

        l_details := l_details
                      || '<PREVIOUS_PAY>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.previous_pay, dbms_xmlgen.entity_encode)
                      || '</PREVIOUS_PAY>';

        l_details := l_details
                      || '<PREVIOUS_TAX_PAID>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.previous_tax_paid, dbms_xmlgen.entity_encode)
                      || '</PREVIOUS_TAX_PAID>';

        l_details := l_details
                      || '<CURRENT_PAY>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.current_pay, dbms_xmlgen.entity_encode)
                      || '</CURRENT_PAY>';

        l_details := l_details
                      || '<CURRENT_TAX>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.current_tax, dbms_xmlgen.entity_encode)
                      || '</CURRENT_TAX>';

        l_details := l_details
                      || '<REFUND_FLAG>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.refund_flag, dbms_xmlgen.entity_encode)
                      || '</REFUND_FLAG>';

        l_details := l_details
                      || '<TAX_CODE>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.tax_code, dbms_xmlgen.entity_encode)
                      || '</TAX_CODE>';

        l_details := l_details
                      || '<WAO_AMOUNT>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.wao_amount, dbms_xmlgen.entity_encode)
                      || '</WAO_AMOUNT>';

        l_details := l_details
                      || '<WEEK_53_INDICATOR>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.week_53_indicator, dbms_xmlgen.entity_encode)
                      || '</WEEK_53_INDICATOR>';

        l_details := l_details
                      || '<EMPLOYEE_NUMBER>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.employee_number, dbms_xmlgen.entity_encode)
                      || '</EMPLOYEE_NUMBER>';

        l_details := l_details
                      || '<PERMIT_NUMBER>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.permit_number, dbms_xmlgen.entity_encode)
                      || '</PERMIT_NUMBER>';

        l_details := l_details
                      || '<LAST_NAME>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.last_name, dbms_xmlgen.entity_encode)
                      || '</LAST_NAME>';

        l_details := l_details
                      || '<W1_M1_INDICATOR>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.w1_m1_indicator, dbms_xmlgen.entity_encode)
                      || '</W1_M1_INDICATOR>';

        l_details := l_details
                      || '<DOB_DD>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.dob_dd, dbms_xmlgen.entity_encode)
                      || '</DOB_DD>';

        l_details := l_details
                      || '<DOB_MM>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.dob_mm, dbms_xmlgen.entity_encode)
                      || '</DOB_MM>';

        l_details := l_details
                      || '<DOB_YY>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.dob_yy, dbms_xmlgen.entity_encode)
                      || '</DOB_YY>';

        l_details := l_details
                      || '<TAX_DIST_NAME>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.tax_dist_name, dbms_xmlgen.entity_encode)
                      || '</TAX_DIST_NAME>';

        l_details := l_details
                      || '<SEGMENT2S>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment2s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT2S>';

        l_details := l_details
                      || '<SEGMENT3S>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment3s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT3S>';

        l_details := l_details
                      || '<SEGMENT4S>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment4s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT4S>';

        l_details := l_details
                      || '<SEGMENT5S>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment5s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT5S>';

        l_details := l_details
                      || '<SEGMENT6S>' || dbms_xmlgen.convert
                          (l_type_le_main_rec.segment6s, dbms_xmlgen.entity_encode)
                      || '</SEGMENT6S>';

        l_details := l_details || '<LIST_G_NI_CONTRIB>';

        l_details := l_details || '<G_NI_CONTRIB>';

        OPEN  csr_type_le_ni_data(cp_align => 'N');
        FETCH csr_type_le_ni_data INTO l_type_le_ni_rec;
        CLOSE csr_type_le_ni_data;

        l_details := l_details
                      || '<NI_ABLE_UAP>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.ni_able_uap, dbms_xmlgen.entity_encode)
                      || '</NI_ABLE_UAP>';

        l_details := l_details
                      || '<LEL_ET>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.lel_et, dbms_xmlgen.entity_encode)
                      || '</LEL_ET>';

        l_details := l_details
                      || '<EMPLOYEES_REBATE>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.employees_rebate, dbms_xmlgen.entity_encode)
                      || '</EMPLOYEES_REBATE>';

        l_details := l_details
                      || '<NI_ABLE_EET>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.ni_able_eet, dbms_xmlgen.entity_encode)
                      || '</NI_ABLE_EET>';

        l_details := l_details
                      || '<ASSIG_ACTION_LNK>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.assig_action_lnk, dbms_xmlgen.entity_encode)
                      || '</ASSIG_ACTION_LNK>';

        l_details := l_details
                      || '<NI_ABLE_ET>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.ni_able_et, dbms_xmlgen.entity_encode)
                      || '</NI_ABLE_ET>';

        l_details := l_details
                      || '<NI_ABLE_LEL>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.ni_able_lel, dbms_xmlgen.entity_encode)
                      || '</NI_ABLE_LEL>';

        l_details := l_details
                      || '<NI_ABLE_UEL>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.ni_able_uel, dbms_xmlgen.entity_encode)
                      || '</NI_ABLE_UEL>';

        l_details := l_details
                      || '<EMPLOYERS_REBATE>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.employers_rebate, dbms_xmlgen.entity_encode)
                      || '</EMPLOYERS_REBATE>';

        l_details := l_details
                      || '<SCON>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.scon, dbms_xmlgen.entity_encode)
                      || '</SCON>';

        l_details := l_details
                      || '<ASSIGNMENT_ID1>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.assignment_id, dbms_xmlgen.entity_encode)
                      || '</ASSIGNMENT_ID1>';

        l_details := l_details
                      || '<EFFECTIVE_END_DATE1>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.effective_end_date, dbms_xmlgen.entity_encode)
                      || '</EFFECTIVE_END_DATE1>';

        l_details := l_details
                      || '<NI_CATEGORY_CODE>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.ni_category_code, dbms_xmlgen.entity_encode)
                      || '</NI_CATEGORY_CODE>';

        l_details := l_details
                      || '<EARNINGS>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.earnings, dbms_xmlgen.entity_encode)
                      || '</EARNINGS>';

        l_details := l_details
                      || '<TOTAL_CONTRIBUTIONS>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.total_contributions, dbms_xmlgen.entity_encode)
                      || '</TOTAL_CONTRIBUTIONS>';

        l_details := l_details
                      || '<EMPLOYEES_CONTRIBUTIONS>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.employees_contributions, dbms_xmlgen.entity_encode)
                      || '</EMPLOYEES_CONTRIBUTIONS>';

        l_details := l_details || '</G_NI_CONTRIB>';

        l_details := l_details || '</LIST_G_NI_CONTRIB>';

        OPEN  csr_type_le_cp_data(cp_align => 'N');
        FETCH csr_type_le_cp_data INTO l_type_le_cp_rec;
          IF csr_type_le_cp_data%FOUND THEN
            l_details := l_details
                          || '<CF_GET_CONTRIBUTIONS>' || dbms_xmlgen.convert
                              (1, dbms_xmlgen.entity_encode)
                          || '</CF_GET_CONTRIBUTIONS>';
          ELSE
            l_details := l_details
                          || '<CF_GET_CONTRIBUTIONS>'
                          || '</CF_GET_CONTRIBUTIONS>';
          END IF;
        CLOSE csr_type_le_cp_data;

        l_details := l_details
                      || '<CP_NIC1>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.nic1, dbms_xmlgen.entity_encode)
                      || '</CP_NIC1>';

        l_details := l_details
                      || '<CP_LEL1>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.lel1, dbms_xmlgen.entity_encode)
                      || '</CP_LEL1>';

        l_details := l_details
                      || '<CP_UEL1>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.uel1, dbms_xmlgen.entity_encode)
                      || '</CP_UEL1>';

        l_details := l_details
                      || '<CP_ET1>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.et1, dbms_xmlgen.entity_encode)
                      || '</CP_ET1>';

        l_details := l_details
                      || '<CP_UAP1>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.uap1, dbms_xmlgen.entity_encode)
                      || '</CP_UAP1>';

        l_details := l_details
                      || '<CP_EC1>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.ec1, dbms_xmlgen.entity_encode)
                      || '</CP_EC1>';

        l_details := l_details
                      || '<CP_SCON1>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.scon1, dbms_xmlgen.entity_encode)
                      || '</CP_SCON1>';

        l_details := l_details
                      || '<CP_NIC2>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.nic2, dbms_xmlgen.entity_encode)
                      || '</CP_NIC2>';

        l_details := l_details
                      || '<CP_LEL2>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.lel2, dbms_xmlgen.entity_encode)
                      || '</CP_LEL2>';

        l_details := l_details
                      || '<CP_UEL2>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.uel2, dbms_xmlgen.entity_encode)
                      || '</CP_UEL2>';

        l_details := l_details
                      || '<CP_ET2>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.et2, dbms_xmlgen.entity_encode)
                      || '</CP_ET2>';

        l_details := l_details
                      || '<CP_UAP2>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.uap2, dbms_xmlgen.entity_encode)
                      || '</CP_UAP2>';

        l_details := l_details
                      || '<CP_EC2>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.ec2, dbms_xmlgen.entity_encode)
                      || '</CP_EC2>';

        l_details := l_details
                      || '<CP_SCON2>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.scon2, dbms_xmlgen.entity_encode)
                      || '</CP_SCON2>';

        l_details := l_details
                      || '<CP_NIC3>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.nic3, dbms_xmlgen.entity_encode)
                      || '</CP_NIC3>';

        l_details := l_details
                      || '<CP_LEL3>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.lel3, dbms_xmlgen.entity_encode)
                      || '</CP_LEL3>';

        l_details := l_details
                      || '<CP_UEL3>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.uel3, dbms_xmlgen.entity_encode)
                      || '</CP_UEL3>';

        l_details := l_details
                      || '<CP_ET3>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.et3, dbms_xmlgen.entity_encode)
                      || '</CP_ET3>';

        l_details := l_details
                      || '<CP_UAP3>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.uap3, dbms_xmlgen.entity_encode)
                      || '</CP_UAP3>';

        l_details := l_details
                      || '<CP_EC3>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.ec3, dbms_xmlgen.entity_encode)
                      || '</CP_EC3>';

        l_details := l_details
                      || '<CP_SCON3>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.scon3, dbms_xmlgen.entity_encode)
                      || '</CP_SCON3>';

        l_details := l_details
                      || '<CP_NIC4>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.nic4, dbms_xmlgen.entity_encode)
                      || '</CP_NIC4>';

        l_details := l_details
                      || '<CP_LEL4>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.lel4, dbms_xmlgen.entity_encode)
                      || '</CP_LEL4>';

        l_details := l_details
                      || '<CP_UEL4>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.uel4, dbms_xmlgen.entity_encode)
                      || '</CP_UEL4>';

        l_details := l_details
                      || '<CP_ET4>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.et4, dbms_xmlgen.entity_encode)
                      || '</CP_ET4>';

        l_details := l_details
                      || '<CP_UAP4>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.uap4, dbms_xmlgen.entity_encode)
                      || '</CP_UAP4>';

        l_details := l_details
                      || '<CP_EC4>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.ec4, dbms_xmlgen.entity_encode)
                      || '</CP_EC4>';

        l_details := l_details
                      || '<CP_SCON4>' || dbms_xmlgen.convert
                          (l_type_le_cp_rec.scon4, dbms_xmlgen.entity_encode)
                      || '</CP_SCON4>';

        l_get_address := type_le_populate_addr_fields
                           (p_person_id                  => xxas_util_pkg.get_person_from_asg(p_assignment_id,trunc(sysdate))
                           ,p_align                      => 'N'
                           ,p_expense_check_send_to_addr => l_type_le_main_rec.expense_check_send_to_address
                           ,p_address1                   => l_address1
                           ,p_address2                   => l_address2
                           ,p_address3                   => l_address3
                           ,p_region1                    => l_region1
                           ,p_region2                    => l_region2
                           ,p_region3                    => l_region3
                           ,p_town                       => l_town);

        l_details := l_details
                      || '<C_GET_ADDRESS>' || dbms_xmlgen.convert
                          (l_get_address, dbms_xmlgen.entity_encode)
                      || '</C_GET_ADDRESS>';

        l_details := l_details
                      || '<C_ADDRESS1>' || dbms_xmlgen.convert
                          (l_address1, dbms_xmlgen.entity_encode)
                      || '</C_ADDRESS1>';

        l_details := l_details
                      || '<C_ADDRESS2>' || dbms_xmlgen.convert
                          (l_address2, dbms_xmlgen.entity_encode)
                      || '</C_ADDRESS2>';

        l_details := l_details
                      || '<C_ADDRESS3>' || dbms_xmlgen.convert
                          (l_address3, dbms_xmlgen.entity_encode)
                      || '</C_ADDRESS3>';

        l_details := l_details
                      || '<C_TOWN>' || dbms_xmlgen.convert
                          (l_town, dbms_xmlgen.entity_encode)
                      || '</C_TOWN>';

        l_details := l_details
                      || '<C_REGION1>' || dbms_xmlgen.convert
                          (l_region1, dbms_xmlgen.entity_encode)
                      || '</C_REGION1>';

        l_details := l_details
                      || '<C_REGION2>' || dbms_xmlgen.convert
                          (l_region2, dbms_xmlgen.entity_encode)
                      || '</C_REGION2>';

        l_details := l_details
                      || '<C_REGION3>' || dbms_xmlgen.convert
                          (l_region3, dbms_xmlgen.entity_encode)
                      || '</C_REGION3>';

        l_details := l_details
                      || '<C_SEG_NAME1>'
                      || '</C_SEG_NAME1>';

        l_details := l_details
                      || '<C_SEG_NAME2>'
                      || '</C_SEG_NAME2>';

        l_details := l_details
                      || '<C_SEG_NAME3>'
                      || '</C_SEG_NAME3>';

        l_details := l_details
                      || '<C_SEG_NAME4>'
                      || '</C_SEG_NAME4>';

        l_details := l_details
                      || '<C_SEG_NAME5>'
                      || '</C_SEG_NAME5>';

        l_details := l_details
                      || '<C_SEG_NAME6>'
                      || '</C_SEG_NAME6>';

        l_details := l_details || '</G_MAIN>';
        l_details := l_details || '</LIST_G_MAIN>';

        l_details := l_details
                      || '<C_BUSINESS_GROUP_ID>' || dbms_xmlgen.convert
                          (l_business_group_id, dbms_xmlgen.entity_encode)
                      || '</C_BUSINESS_GROUP_ID>';

        l_details := l_details
                      || '<CS_EMPLOYEES_REBATE>' || dbms_xmlgen.convert
                          (l_type_le_ni_rec.employees_rebate, dbms_xmlgen.entity_encode)
                      || '</CS_EMPLOYEES_REBATE>';

        l_details := l_details || '</PAYP60LE>';

      END IF;

    END IF;

    IF l_details IS NOT NULL THEN
      l_details_xml := XMLType(l_details);
    END IF;

  END IF;

  log_end(c_method);
  RETURN l_details_xml;
  
END get_p60_report_details_xml;

$END

--
-- ------------------------------------------------------------------------ *
-- Name    : remove_null_lines
-- ------------------------------------------------------------------------ *
FUNCTION remove_null_lines
  (p_address_line      IN varchar2
  ,p_payroll_action_id IN varchar2
  ,p_asg_action_id     IN number
  ,p_line_number       IN number) RETURN varchar2
IS

  c_method CONSTANT varchar2(100) := 'remove_null_lines';
  l_address1        varchar2(500);
  l_address2        varchar2(500);
  l_address3        varchar2(500);
  l_town            varchar2(500);
  l_region1         varchar2(500);
  l_region2         varchar2(500);
  l_final_string    varchar2(500);
  l_count_words     number := 0;
  prev_occurance    number;
  current_occurance number;
  l_nth_substring   varchar2(500);

  CURSOR csr_addresses_rearrange_null(p_payroll_action_id IN number)
  IS
  SELECT emp.action_information6 l_address1
        ,emp.action_information7 l_address2
        ,emp.action_information8 l_address3
        ,emp.action_information9 l_town
        ,emp.action_information11 l_region1
        ,emp.action_information12 l_region2
  FROM   pay_action_information emp
        ,pay_assignment_actions paa
        ,per_all_assignments_f paaf
        ,pay_payroll_actions ppa
  WHERE  paa.payroll_action_id = p_payroll_action_id
  AND    ppa.payroll_action_id =  paa.payroll_action_id
  AND    paa.assignment_id = paaf.assignment_id
  AND    emp.action_context_id = paa.assignment_action_id
  AND    emp.action_context_type = 'AAP'
  AND    emp.action_information_category = 'GB RTI P60 EMP DET'
  AND    emp.action_context_id = p_asg_action_id;

BEGIN

  log_start(c_method,p_address_line,p_payroll_action_id,p_asg_action_id,p_line_number);

  OPEN  csr_addresses_rearrange_null(p_payroll_action_id);
  FETCH csr_addresses_rearrange_null INTO l_address1
                                         ,l_address2
                                         ,l_address3
                                         ,l_town
                                         ,l_region1
                                         ,l_region2;
  CLOSE csr_addresses_rearrange_null;

  IF l_address1 is not null AND 
     l_address2 is not null AND 
     l_address3 is not null AND 
     l_town     is not null AND 
     l_region1  is not null AND 
     l_region2  is not null THEN
    RETURN p_address_line;
  END IF;

  IF l_address1 is not null THEN
    l_final_string := l_final_string || l_address1 || '|';
    l_count_words  := l_count_words + 1;
  END IF;

  IF l_address2 is not null THEN
    l_final_string := l_final_string || l_address2 || '|';
    l_count_words  := l_count_words + 1;
  END IF;

  IF l_address3 is not null THEN
    l_final_string := l_final_string || l_address3 || '|';
    l_count_words  := l_count_words + 1;
  END IF;

  IF l_town is not null THEN
    l_final_string := l_final_string || l_town || '|';
    l_count_words  := l_count_words + 1;
  END IF;

  IF l_region1 is not null THEN
    l_final_string := l_final_string || l_region1 || '|';
    l_count_words  := l_count_words + 1;
  END IF;

  IF l_region2 is not null THEN
    l_final_string := l_final_string || l_region2 || '|';
    l_count_words  := l_count_words + 1;
  END IF;

  IF p_line_number > l_count_words THEN
    RETURN null;
  END IF;

  IF p_line_number = 1 THEN
    prev_occurance := 0;
  ELSE
    prev_occurance := INSTR(l_final_string, '|', 1, p_line_number-1);
  END IF;

  current_occurance := INSTR(l_final_string, '|', 1, p_line_number);
  l_nth_substring   := SUBSTR(l_final_string, 1 + prev_occurance, current_occurance - prev_occurance - 1 );

  log_end(c_method);
  RETURN l_nth_substring;

END remove_null_lines;

END xxas_gb_reports_pkg;
/
COMMIT;
EXIT;