/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2013 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK


CREATE OR REPLACE TYPE BODY xxas_hr_api_trxn_value IS

CONSTRUCTOR FUNCTION xxas_hr_api_trxn_value
  (p_transaction_value_id   IN NUMBER
  ,p_transaction_step_id    IN NUMBER
  ,p_data_type              IN VARCHAR2
  ,p_name                   IN VARCHAR2
  ,p_varchar2_value         IN VARCHAR2
  ,p_number_value           IN NUMBER
  ,p_date_value             IN DATE) RETURN SELF AS RESULT
IS

BEGIN

  self.transaction_value_id := p_transaction_value_id;
  self.transaction_step_id  := p_transaction_step_id;
  self.data_type            := p_data_type;
  self.name                 := p_name;
  
  IF p_data_type = 'NUMBER' THEN
    self.varchar2_value := NULL;
    self.date_value     := NULL;
    self.number_value   := p_number_value;
    
  ELSIF p_data_type = 'VARCHAR2' THEN
  
    self.varchar2_value := p_varchar2_value;
    self.date_value     := NULL;
    self.number_value   := NULL;  
  
  ELSIF p_data_type = 'DATE' THEN
    self.varchar2_value := NULL;
    self.date_value     := p_date_value;
    self.number_value   := NULL;    
  END IF;
  
  RETURN;

END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_hr_api_txn_steps IS

CONSTRUCTOR FUNCTION xxas_hr_api_txn_steps
  (p_transaction_id       IN NUMBER
  ,p_transaction_step_id  IN NUMBER) RETURN SELF AS RESULT
IS

  CURSOR csr_transaction_step IS
  SELECT hats.*
  FROM   hr_api_transaction_steps hats
  WHERE  hats.transaction_id      = p_transaction_id
  AND    hats.transaction_step_id = p_transaction_step_id;

  l_txn_step_rec  csr_transaction_step%ROWTYPE;
  
  CURSOR  csr_transaction_step_values IS
  SELECT  hatv.*
  FROM    hr_api_transaction_values hatv
  WHERE   hatv.transaction_step_id  = p_transaction_step_id;
  
  l_txn_step_values csr_transaction_step_values%ROWTYPE;
  
  l_txn_value_tbl   xxas_hr_api_trxn_value_table := xxas_hr_api_trxn_value_table();
  
BEGIN
  
  OPEN  csr_transaction_step;
  FETCH csr_transaction_step  INTO l_txn_step_rec;
  CLOSE csr_transaction_step;
  
  self.transaction_id         := p_transaction_id;
  self.transaction_step_id    := l_txn_step_rec.transaction_step_id;
  self.api_name               := l_txn_step_rec.api_name;
  self.processing_order       := l_txn_step_rec.processing_order;
  self.item_type              := l_txn_step_rec.item_type;
  self.item_key               := l_txn_step_rec.item_key;
  self.creator_person_id      := l_txn_step_rec.creator_person_id;
  
  FOR l_txn_value_rec IN csr_transaction_step_values LOOP
    
    l_txn_value_tbl.EXTEND;
    l_txn_value_tbl(l_txn_value_tbl.COUNT) := xxas_hr_api_trxn_value
                                                (p_transaction_value_id   => l_txn_value_rec.transaction_value_id
                                                ,p_transaction_step_id    => l_txn_value_rec.transaction_step_id
                                                ,p_data_type              => l_txn_value_rec.datatype
                                                ,p_name                   => l_txn_value_rec.name
                                                ,p_varchar2_value         => l_txn_value_rec.varchar2_value
                                                ,p_number_value           => l_txn_value_rec.number_value
                                                ,p_date_value             => l_txn_value_rec.date_value);
    
  END LOOP;
  
  self.transaction_values_tbl := l_txn_value_tbl;
  
  RETURN;

END;  

MEMBER FUNCTION get_xml RETURN XMLType
IS

  l_xml           XMLType;
  i               NUMBER  := 1;
  l_tag           VARCHAR2(4000);
  l_value_clob    CLOB;
  l_temp          CLOB;
  
BEGIN

  DBMS_LOB.createtemporary(l_value_clob, true);
  DBMS_LOB.createtemporary(l_temp, true);
  
  l_value_clob := TO_CLOB('<ROW>
                              <TRANSACTION_ID>'||self.transaction_id||'</TRANSACTION_ID>
                              <TRANSACTION_STEP_ID>'||self.transaction_step_id||'</TRANSACTION_STEP_ID>
                              <APPROVERS>' || self.transaction_id ||'</APPROVERS>');  
  
  
  FOR i IN 1..self.transaction_values_tbl.COUNT LOOP
    
    l_tag := SUBSTR(self.transaction_values_tbl(i).name,3,LENGTH(self.transaction_values_tbl(i).name)); --dbms_xmlgen.encode 
    
    IF self.transaction_values_tbl(i).data_type = 'NUMBER' AND transaction_values_tbl(i).number_value IS NOT NULL THEN
    
      l_temp  := TO_CLOB('<' ||dbms_xmlgen.convert(l_tag, dbms_xmlgen.entity_encode) ||'>'
                             ||self.transaction_values_tbl(i).number_value ||'</'||dbms_xmlgen.convert(l_tag, dbms_xmlgen.entity_encode)||'>');

      DBMS_LOB.APPEND(l_value_clob,l_temp);
      
    ELSIF transaction_values_tbl(i).data_type = 'VARCHAR2' AND transaction_values_tbl(i).varchar2_value IS NOT NULL THEN
    
      l_temp  := TO_CLOB('<' ||dbms_xmlgen.convert(l_tag, dbms_xmlgen.entity_encode) ||'>'
                             ||transaction_values_tbl(i).varchar2_value ||'</'||dbms_xmlgen.convert(l_tag, dbms_xmlgen.entity_encode)||'>');

                             DBMS_LOB.APPEND(l_value_clob,l_temp);    
      
    ELSIF transaction_values_tbl(i).data_type = 'DATE' AND transaction_values_tbl(i).date_value IS NOT NULL THEN
    
      l_temp  := TO_CLOB('<' ||dbms_xmlgen.convert(l_tag, dbms_xmlgen.entity_encode) ||'>'
                             ||transaction_values_tbl(i).date_value ||'</'||dbms_xmlgen.convert(l_tag, dbms_xmlgen.entity_encode)||'>');
                             
      DBMS_LOB.APPEND(l_value_clob,l_temp);    
      
    END IF;
    
  END LOOP;
  DBMS_LOB.APPEND(l_value_clob,TO_CLOB('</ROW>'));

  l_xml := XMLType(l_value_clob);

  dbms_lob.freetemporary(l_value_clob);
  dbms_lob.freetemporary(l_temp);

  RETURN l_xml;
END;

END;
/

COMMIT;
EXIT;
/