/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2014 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

-- ------------------------------------------------------------------------ *
-- Name    : xxas_http_resp_obj
-- ------------------------------------------------------------------------ * 
CREATE OR REPLACE TYPE BODY xxas_http_resp_obj IS

  -- ---------------------------------------------------------------------- *
  -- Name    : xxas_http_resp_obj
  -- ---------------------------------------------------------------------- * 
  CONSTRUCTOR FUNCTION xxas_http_resp_obj RETURN SELF AS RESULT
  IS

  BEGIN

    resp := XMLType
              ('<r charset="' || xxas_util_pkg.get_db_charset_iana || '"'
            || ' locale="' || xxas_util_pkg.iso_locale || '"'
            || '><rh></rh></r>');
    RETURN;
    
  END xxas_http_resp_obj;

  -- ---------------------------------------------------------------------- *
  -- Name    : xxas_http_resp_obj
  -- ---------------------------------------------------------------------- * 
  CONSTRUCTOR FUNCTION xxas_http_resp_obj
    (p_status IN number) RETURN SELF AS RESULT
  IS

  BEGIN

    resp := XMLType('<r status="' || to_char(p_status) || '"></r>');
    RETURN;
    
  END xxas_http_resp_obj;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_response_xml
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION get_response_xml RETURN XMLType
  IS
  
  BEGIN
  
    RETURN resp;
    
  END get_response_xml;

  -- ---------------------------------------------------------------------- *
  -- Name    : get_status
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_status RETURN number
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_number(resp, 'r/@status');
    
  END get_status;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : set_status
  -- ---------------------------------------------------------------------- *   
  MEMBER PROCEDURE set_status
    (p_status IN number)
  IS

  BEGIN
  
    IF get_status IS NULL THEN
      select insertChildXML(resp, 'r', '@status', to_char(p_status))
      INTO   resp
      FROM   dual; 
    ELSE
      SELECT updateXML(resp, 'r/@status', to_char(p_status))
      INTO   resp
      FROM   dual;
    END IF;
    
  END set_status;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_token
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_token RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string(resp, 'r/@token');
    
  END get_token;
    -- ---------------------------------------------------------------------- *
  -- Name    : set_token
  -- ---------------------------------------------------------------------- *   
  MEMBER PROCEDURE set_token
    (p_token IN varchar2)
  IS

  BEGIN
  
    IF get_token IS NULL THEN
      select insertChildXML(resp, 'r', '@token', p_token)
      INTO   resp
      FROM   dual; 
    ELSE
      SELECT updateXML(resp, 'r/@token', p_token)
      INTO   resp
      FROM   dual;
    END IF;
    
  END set_token;

  -- ---------------------------------------------------------------------- *
  -- Name    : get_content_type
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_content_type RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string(resp, 'r/@contentType');
    
  END get_content_type;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : set_content_type
  -- ---------------------------------------------------------------------- *   
  MEMBER PROCEDURE set_content_type
    (p_content_type IN varchar2)
  IS

  BEGIN
  
    IF get_content_type IS NULL THEN
      select insertChildXML(resp, 'r', '@contentType', p_content_type)
      INTO   resp
      FROM   dual; 
    ELSE
      SELECT updateXML(resp, 'r/@contentType', p_content_type)
      INTO   resp
      FROM   dual;
    END IF;
    
  END set_content_type;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_header
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_header
    (p_name IN varchar2) RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string
             (resp
             ,'r/rh/h[@n = "' || p_name || '"]/v[1]/text()');
    
  END get_header;

  -- ---------------------------------------------------------------------- *
  -- Name    : set_header
  -- ---------------------------------------------------------------------- *   
  MEMBER PROCEDURE set_header
    (p_name  IN varchar2
    ,p_value IN varchar2)
  IS

  BEGIN
  
    IF get_header(p_name) IS NULL THEN
      SELECT insertChildXML
               (resp
               ,'r/rh'
               ,'h'
               ,XMLElement
                 ("h"
                 ,XMLAttributes
                   (p_name as "n")
                 ,XMLElement
                   ("v", XMLCData(p_value))))
      INTO resp
      FROM dual;
    ELSE
      SELECT updateXML
               (resp
               ,'r/rh/h[@n = "' || p_name || '"]/v[1]/text()'
               ,XMLCData(p_value))
      INTO   resp
      FROM   dual;
    END IF;
    
  END set_header;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_character_encoding
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_character_encoding RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string(resp, 'r/@charset');
    
  END get_character_encoding;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : set_character_encoding
  -- ---------------------------------------------------------------------- *   
  MEMBER PROCEDURE set_character_encoding
    (p_charset IN varchar2)
  IS

  BEGIN
  
    IF get_content_type IS NULL THEN
      select insertChildXML(resp, 'r', '@charset', p_charset)
      INTO   resp
      FROM   dual; 
    ELSE
      SELECT updateXML(resp, 'r/@charset', p_charset)
      INTO   resp
      FROM   dual;
    END IF;
    
  END set_character_encoding;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_locale
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_locale RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string(resp, 'r/@locale');
    
  END get_locale;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : set_locale
  -- ---------------------------------------------------------------------- *   
  MEMBER PROCEDURE set_locale
    (p_locale IN varchar2)
  IS

  BEGIN
  
    IF get_locale IS NULL THEN
      select insertChildXML(resp, 'r', '@locale', p_locale)
      INTO   resp
      FROM   dual; 
    ELSE
      SELECT updateXML(resp, 'r/@locale', p_locale)
      INTO   resp
      FROM   dual;
    END IF;
    
  END set_locale;

  -- ---------------------------------------------------------------------- *
  -- Name    : get_last_modified
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION get_last_modified RETURN varchar2
  IS

  BEGIN
  
    RETURN get_header
      (p_name => 'Last-Modified');
          
  END get_last_modified;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : set_last_modified
  -- ---------------------------------------------------------------------- * 
  MEMBER PROCEDURE set_last_modified
    (p_date IN date)
  IS

  BEGIN
  
    set_header
      (p_name  => 'Last-Modified'
      ,p_value => xxas_util_pkg.date_to_httpdate(p_date));
          
  END set_last_modified;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_expires
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION get_expires RETURN varchar2
  IS

  BEGIN
  
    RETURN get_header
      (p_name => 'Expires');
          
  END get_expires;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : set_expires
  -- ---------------------------------------------------------------------- * 
  MEMBER PROCEDURE set_expires
    (p_date IN date)
  IS

  BEGIN
  
    set_header
      (p_name  => 'Expires'
      ,p_value => xxas_util_pkg.date_to_httpdate(p_date));
          
  END set_expires;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_tag
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION get_tag RETURN varchar2
  IS

  BEGIN
  
    RETURN get_header
      (p_name => 'ETag');
          
  END get_tag;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : set_tag
  -- ---------------------------------------------------------------------- * 
  MEMBER PROCEDURE set_tag
    (p_tag IN varchar2)
  IS

  BEGIN
  
    set_header
      (p_name  => 'ETag'
      ,p_value => p_tag);
          
  END set_tag;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_entity_clob
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_entity_clob RETURN CLOB
  IS

  BEGIN
  
    RETURN entity_clob;
    
  END get_entity_clob;
 
  -- ---------------------------------------------------------------------- *
  -- Name    : get_entity_blob
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_entity_blob RETURN BLOB
  IS

  BEGIN
  
    RETURN entity_blob;
    
  END get_entity_blob;
 
  -- ---------------------------------------------------------------------- *
  -- Name    : set_entity
  -- ---------------------------------------------------------------------- *   
  MEMBER PROCEDURE set_entity
    (p_entity IN XMLType)
  IS

  BEGIN
  
    entity_clob := p_entity.getCLOBVal();
    
  END set_entity;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : set_entity
  -- ---------------------------------------------------------------------- *   
  MEMBER PROCEDURE set_entity
    (p_entity IN CLOB)
  IS

  BEGIN
  
    entity_clob := p_entity;
    
  END set_entity;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : set_entity
  -- ---------------------------------------------------------------------- *   
  MEMBER PROCEDURE set_entity
    (p_entity IN BLOB)
  IS

  BEGIN
  
    entity_blob := p_entity;
    
  END set_entity;
  
END;
/

-- ------------------------------------------------------------------------ *
-- Name    : xxas_http_response_builder
-- ------------------------------------------------------------------------ * 
CREATE OR REPLACE TYPE BODY xxas_http_response_builder IS

  -- ---------------------------------------------------------------------- *
  -- Name    : xxas_http_response_builder
  -- ---------------------------------------------------------------------- * 
  CONSTRUCTOR FUNCTION xxas_http_response_builder RETURN SELF AS RESULT
  IS

  BEGIN

    resp := xxas_http_resp_obj();
  
    RETURN;
    
  END xxas_http_response_builder;

  -- ---------------------------------------------------------------------- *
  -- Name    : status
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION status
    (p_status IN number) RETURN xxas_http_response_builder
  IS

    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_status
      (p_status => p_status);
            
    RETURN xxas_http_response_builder(l_resp);
      
  END status;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : content_type
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION content_type
    (p_content_type IN varchar2) RETURN xxas_http_response_builder
  IS

    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_content_type
      (p_content_type => p_content_type);
            
    RETURN xxas_http_response_builder(l_resp);
      
  END content_type;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : header
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION header
    (p_name   IN varchar2
    ,p_value  IN varchar2) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_header
      (p_name  => p_name
      ,p_value => p_value);
      
    RETURN xxas_http_response_builder(l_resp);
    
  END header;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : last_modified
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION last_modified
    (p_date IN date) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_last_modified
      (p_date => p_date);
      
    RETURN xxas_http_response_builder(l_resp);
    
  END last_modified;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : expires
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION expires
    (p_date IN date) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_expires
      (p_date => p_date);
      
    RETURN xxas_http_response_builder(l_resp);
    
  END expires;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : tag
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION tag
    (p_tag IN varchar2) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_tag
      (p_tag => p_tag);
      
    RETURN xxas_http_response_builder(l_resp);
    
  END tag;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : entity
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION entity
    (p_entity IN XMLType) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;
    l_token varchar2(1000);

  BEGIN
  
    l_resp := self.resp;
  
    --
    -- Default the Content Type to XML if not yet specified
    --
    IF l_resp.get_content_type IS NULL THEN
      l_resp.set_content_type('application/xml');
    END IF;
    l_resp.set_entity(p_entity);

    l_token := xxas_util_pkg.extract_string(p_entity,'/data/token/text()');
    
    IF l_token IS NOT NULL THEN

      l_resp.set_token(l_token);
    
    END IF;

    RETURN xxas_http_response_builder(l_resp);
    
  END entity;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : entity
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION entity
    (p_entity       IN XMLType
    ,p_content_type IN varchar2) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_content_type(p_content_type);
    l_resp.set_entity(p_entity);
      
    RETURN xxas_http_response_builder(l_resp);
    
  END entity;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : entity
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION entity
    (p_entity       IN CLOB
    ,p_content_type IN varchar2) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_content_type(p_content_type);
    l_resp.set_entity(p_entity);
      
    RETURN xxas_http_response_builder(l_resp);
    
  END entity;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : entity_with_etag
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION entity_with_etag
    (p_entity             IN CLOB
    ,p_content_type       IN varchar2
    ,p_content_diposition IN varchar2 DEFAULT NULL) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_content_type(p_content_type);
    l_resp.set_entity(p_entity);
    l_resp.set_header('ETag', xxas_util_pkg.md5(p_entity));
    l_resp.set_header('Content-Disposition', p_content_diposition);
      
    RETURN xxas_http_response_builder(l_resp);
    
  END entity_with_etag;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : entity
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION entity
    (p_entity       IN BLOB
    ,p_content_type IN varchar2) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_content_type(p_content_type);
    l_resp.set_entity(p_entity);
      
    RETURN xxas_http_response_builder(l_resp);
    
  END entity;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : entity_with_etag
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION entity_with_etag
    (p_entity             IN BLOB
    ,p_content_type       IN varchar2
    ,p_content_diposition IN varchar2 DEFAULT NULL) RETURN xxas_http_response_builder
  IS
  
    l_resp xxas_http_resp_obj;

  BEGIN
  
    l_resp := self.resp;
  
    l_resp.set_content_type(p_content_type);
    l_resp.set_entity(p_entity);
    l_resp.set_header('ETag', xxas_util_pkg.md5(p_entity));
    l_resp.set_header('Content-Disposition', p_content_diposition);
      
    RETURN xxas_http_response_builder(l_resp);
    
  END entity_with_etag;

  -- ---------------------------------------------------------------------- *
  -- Name    : build
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION build RETURN xxas_http_resp_obj
  IS

  BEGIN
  
    RETURN resp;
    
  END build;
  
END;
/

-- ------------------------------------------------------------------------ *
-- Name    : xxas_http_response
-- ------------------------------------------------------------------------ * 
CREATE OR REPLACE TYPE BODY xxas_http_response IS

  -- ---------------------------------------------------------------------- *
  -- Name    : xxas_http_response
  -- ---------------------------------------------------------------------- * 
  CONSTRUCTOR FUNCTION xxas_http_response RETURN SELF AS RESULT
  IS

  BEGIN

    RETURN;
    
  END xxas_http_response;

  -- ---------------------------------------------------------------------- *
  -- Name    : created
  -- ---------------------------------------------------------------------- * 
  STATIC FUNCTION created
    (p_location IN varchar2) RETURN xxas_http_response_builder
  IS
  
  BEGIN

    RETURN xxas_http_response_builder()
            .status(xxas_http_resp_status_pkg.CREATED)
            .header('Location', p_location);
    
  END created;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : ok
  -- ---------------------------------------------------------------------- * 
  STATIC FUNCTION ok RETURN xxas_http_response_builder
  IS
  
  BEGIN

    RETURN xxas_http_response_builder()
             .status(xxas_http_resp_status_pkg.OK);
    
  END ok;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : ok
  -- ---------------------------------------------------------------------- * 
  STATIC FUNCTION ok
    (p_entity IN XMLType) RETURN xxas_http_response_builder
  IS
  
  BEGIN

    RETURN ok().entity(p_entity);
    
  END ok;

  -- ---------------------------------------------------------------------- *
  -- Name    : ok
  -- ---------------------------------------------------------------------- * 
  STATIC FUNCTION ok
    (p_entity       IN XMLType
    ,p_content_type IN varchar2) RETURN xxas_http_response_builder
  IS
  
  BEGIN

    RETURN ok().entity(p_entity, p_content_type);
    
  END ok;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : ok
  -- ---------------------------------------------------------------------- * 
  STATIC FUNCTION ok
    (p_entity       IN CLOB
    ,p_content_type IN varchar2) RETURN xxas_http_response_builder
  IS
  
  BEGIN

    RETURN ok().entity(p_entity, p_content_type);
    
  END ok;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : ok
  -- ---------------------------------------------------------------------- * 
  STATIC FUNCTION ok
    (p_entity       IN BLOB
    ,p_content_type IN varchar2) RETURN xxas_http_response_builder
  IS
  
  BEGIN

    RETURN ok().entity(p_entity, p_content_type);
    
  END ok;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : status
  -- ---------------------------------------------------------------------- * 
  STATIC FUNCTION status
    (p_status_code IN number) RETURN xxas_http_response_builder
  IS
  
  BEGIN

    RETURN xxas_http_response_builder().status(p_status_code);
    
  END status;
  
END;
/

COMMIT;
EXIT;
