/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2016 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK
  
-- ------------------------------------------------------------------------ *
-- Name    : xxas_item_actions_builder
-- ------------------------------------------------------------------------ * 
CREATE OR REPLACE TYPE BODY xxas_item_actions_builder IS

  -- ---------------------------------------------------------------------- *
  -- Name    : xxas_item_actions_builder
  -- ---------------------------------------------------------------------- * 
  CONSTRUCTOR FUNCTION xxas_item_actions_builder RETURN SELF AS RESULT
  IS

  BEGIN
  
    self.actions := xxas_item_action_table();
    RETURN;
    
  END xxas_item_actions_builder;

  -- ---------------------------------------------------------------------- *
  -- Name    : link_action
  -- ---------------------------------------------------------------------- *  
  MEMBER FUNCTION link_action
    (p_item_id   IN varchar2
    ,p_title     IN varchar2
    ,p_page_id   IN varchar2
    ,p_object_id IN varchar2
    ,p_target    IN varchar2
    ,p_color     IN varchar2
    ,p_icon      IN varchar2) RETURN xxas_item_actions_builder
  IS
  
    l_actions xxas_item_action_table;
  
  BEGIN
  
    l_actions := self.actions;
    
    l_actions.extend;
    l_actions(l_actions.count) :=
      xxas_item_action_type
        (item_id             => p_item_id
        ,order_sequence      => l_actions.count
        ,title               => p_title
        ,operation           => 'LINK'
        ,page_id             => p_page_id
        ,object_id           => p_object_id
        ,target              => p_target
        ,href                => null
        ,icon                => p_icon
        ,color               => p_color
        ,bulk_action_enabled => 'N');

    RETURN xxas_item_actions_builder(l_actions);
  
  END link_action;

  -- ---------------------------------------------------------------------- *
  -- Name    : post_action
  -- ---------------------------------------------------------------------- *  
  MEMBER FUNCTION post_action
    (p_item_id             IN varchar2
    ,p_title               IN varchar2
    ,p_api_path            IN varchar2
    ,p_color               IN varchar2
    ,p_icon                IN varchar2) RETURN xxas_item_actions_builder
  IS
  
    l_actions xxas_item_action_table;
  
  BEGIN
  
    l_actions := self.actions;
    
    l_actions.extend;
    l_actions(l_actions.count) :=
      xxas_item_action_type
        (item_id             => p_item_id
        ,order_sequence      => l_actions.count
        ,title               => p_title
        ,operation           => 'POST'
        ,page_id             => null
        ,object_id           => null
        ,target              => null
        ,href                => xxas_com_rest_util_pkg.uri_from_path(p_api_path)
        ,icon                => p_icon
        ,color               => p_color
        ,bulk_action_enabled => 'N');

    RETURN xxas_item_actions_builder(l_actions);
  
  END post_action;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : put_action
  -- ---------------------------------------------------------------------- *  
  MEMBER FUNCTION put_action
    (p_item_id             IN varchar2
    ,p_title               IN varchar2
    ,p_api_path            IN varchar2
    ,p_color               IN varchar2
    ,p_icon                IN varchar2) RETURN xxas_item_actions_builder
  IS
  
    l_actions xxas_item_action_table;
  
  BEGIN
  
    l_actions := self.actions;
    
    l_actions.extend;
    l_actions(l_actions.count) :=
      xxas_item_action_type
        (item_id             => p_item_id
        ,order_sequence      => l_actions.count
        ,title               => p_title
        ,operation           => 'PUT'
        ,page_id             => null
        ,object_id           => null
        ,target              => null
        ,href                => xxas_com_rest_util_pkg.uri_from_path(p_api_path)
        ,icon                => p_icon
        ,color               => p_color
        ,bulk_action_enabled => 'N');

    RETURN xxas_item_actions_builder(l_actions);
  
  END put_action;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : delete_action
  -- ---------------------------------------------------------------------- *  
  MEMBER FUNCTION delete_action
    (p_item_id             IN varchar2
    ,p_title               IN varchar2
    ,p_api_path            IN varchar2
    ,p_color               IN varchar2
    ,p_icon                IN varchar2
    ,p_bulk_action_enabled IN varchar2 DEFAULT 'N') RETURN xxas_item_actions_builder
  IS
  
    l_actions xxas_item_action_table;
  
  BEGIN
  
    l_actions := self.actions;
    
    l_actions.extend;
    l_actions(l_actions.count) :=
      xxas_item_action_type
        (item_id             => p_item_id
        ,order_sequence      => l_actions.count
        ,title               => p_title
        ,operation           => 'DELETE'
        ,page_id             => null
        ,object_id           => null
        ,target              => null
        ,href                => xxas_com_rest_util_pkg.uri_from_path(p_api_path)
        ,icon                => p_icon
        ,color               => p_color
        ,bulk_action_enabled => p_bulk_action_enabled);

    RETURN xxas_item_actions_builder(l_actions);
  
  END delete_action;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_xml
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION get_xml RETURN XMLType
  IS

    l_xml XMLType;
  
  BEGIN
  
    SELECT XMLELEMENT
            ("actions"
            ,XMLAttributes
              ('true' as "json:force-array"
              ,'http://json.org/' as "xmlns:json")
            ,XMLAgg
              (XMLElement
                ("action"
                ,XMLForest
                  (a.item_id                          "id"
                  ,a.title                            "title"
                  ,xxas_util_pkg.flag_to_boolean
                    (nvl(a.bulk_action_enabled, 'Y')) "bulkActionEnabled"
                  ,a.operation                        "operation"
                  ,a.href                             "href"
                  ,a.page_id                          "pageId"
                  ,a.object_id                        "objectId"
                  ,a.target                           "target")
                ,XMLElement
                  ("meta"
                  ,XMLElement
                    ("options"
                    ,XMLForest
                      (a.color                        "color"
                      ,a.icon                         "icon")
                    )
                  )
                ) ORDER BY a.order_sequence
              )
            )
    INTO   l_xml   
    FROM   TABLE(actions) a;

    RETURN l_xml;
      
  END get_xml;
  
END;
/

COMMIT;
EXIT;
