/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2017 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

DECLARE

  --
  -- Local constants and variables
  --
  e_no_bg_id          EXCEPTION;
  e_bg_not_exists     EXCEPTION;
  e_ff_type_not_found EXCEPTION;
  l_BG_ID   CONSTANT  NUMBER        := to_number('&1');
  l_USER_ID CONSTANT  NUMBER        := to_number('&2');
  l_START   CONSTANT  DATE          := to_date('1900-01-01','YYYY-MM-DD');
  l_END     CONSTANT  DATE          := xxas_util_pkg.eot;
  l_FF_NAME CONSTANT  VARCHAR2(80)  := 'XXAS_BK_TEAM_ENT_BALANCE';
  l_FF_DESC CONSTANT  VARCHAR2(240) := 'Seeded Team Accrual Entitlement balance';
  l_FF_TYPE CONSTANT  varchar2(30)  := 'Applaud Assignment';
  l_formula_text      LONG;
  l_formula_type_id   NUMBER;
  l_ff_id             NUMBER;
  l_bg_exists         VARCHAR2(1);

  --
  -- Tests if the Business Group already exists
  --
  CURSOR csr_bg_exists IS
  SELECT 'Y'
  FROM   per_business_groups_perf bg
  WHERE  bg.business_group_id = l_BG_ID;

  --
  -- Gets the FF Id
  --
  CURSOR csr_ff_id IS
  SELECT ff.formula_id
  FROM   ff_formulas_f ff
  WHERE  ff.business_group_id = l_BG_ID
  AND    ff.formula_name = l_FF_NAME;

  --
  -- Gets the Formula Type Id
  --
  CURSOR csr_ff_type IS
  SELECT fft.formula_type_id
  FROM   ff_formula_types fft
  WHERE  fft.formula_type_name = l_FF_TYPE;

BEGIN

  --
  -- Check that the Business Group Id was passed in
  --
  IF l_BG_ID IS NULL THEN
    RAISE e_no_bg_id;
  END IF;

  --
  -- Check the Business Group exists
  --
  OPEN  csr_bg_exists;
  FETCH csr_bg_exists INTO l_bg_exists;
  CLOSE csr_bg_exists;

  IF l_bg_exists IS NULL THEN
    RAISE e_bg_not_exists;
  END IF;

  --
  -- Fetch the Fast Formula Id
  --
  OPEN  csr_ff_id;
  FETCH csr_ff_id INTO l_ff_id;
  CLOSE csr_ff_id;

  --
  -- Get the Formula Type; error if it cannot be found
  --
  OPEN  csr_ff_type;
  FETCH csr_ff_type INTO l_formula_type_id;
  CLOSE csr_ff_type;

  IF l_formula_type_id IS NULL THEN
    RAISE e_ff_type_not_found;
  END IF;

  --
  -- Set the Formula Text
  --
  l_formula_text := q'[
  /*==============================================================================
  Formula Name: XXAS_BK_TEAM_ENT_BALANCE
  Formula Type: Applaud Assignment
  Description:  Seeded Team Accrual Entitlement balance
  ==============================================================================*/
  
  /* Defaults must be specified for all Inputs */

  Default for Person_Id_Value_in               is 0
  Default for Effective_At_Value_in            is '0001/01/01 00:00:00' (date)
  Default for Accrual_Plan_Id_Value_in         is 0
  Default for Saved_Search_Id_Value_in         is -1
  Default for Uom_name_Value_in                is ' '
  
  Inputs are Person_Id_Value_in
            ,Effective_At_Value_in
            ,Accrual_Plan_Id_Value_in
            ,Saved_Search_Id_Value_in
            ,Uom_name_Value_in
            
  Effective_At_Value = trunc(sysdate)
  If Effective_At_Value_in was not defaulted then
  (
    Effective_At_Value = Effective_At_Value_in
  )
  
  Parameters = 'pPersonId=' + to_char(Person_Id_Value_in)
             + '&'
             + 'pAccrualPlanId=' + to_char(Accrual_Plan_Id_Value_in)
             + '&'
             + 'pEffectiveDate=' + to_char(Effective_At_Value, 'YYYY-MM-DD')
             + '&'
             + 'pSavedSearchId=' + to_char(Saved_Search_Id_Value_in)
             + '&'
             + 'pUomName=' + Uom_name_Value_in
             
  If Accrual_Plan_id_value_in = 0 Then
  ( 
    /* No accrual plans */
    HTML_Text_Value = xxas_emp_set_long_output
                     ('HTML_TEXT_VALUE', 'XXAS', 'XXAS_NO_ACC_PLANS_AT'
                     ,'DATE', to_char(Effective_At_Value))
    HTML_Text_Visible = 'true'
    Accrual_Chart_Visible = 'false'
  )
  
  Accrual_Chart_Value = Parameters

  Return Effective_At_Value
        ,Accrual_Chart_Value
        ,HTML_Text_Visible
        ,Accrual_Chart_Visible

]';

  BEGIN

    IF l_ff_id IS NULL THEN
      --
      -- Create the Fast Formula
      --
      INSERT INTO ff_formulas_f
        (formula_id
        ,effective_start_date
        ,effective_end_date
        ,business_group_id
        ,formula_type_id
        ,formula_name
        ,description
        ,formula_text)
      VALUES
        (ff_formulas_s.nextval
        ,l_START
        ,l_END
        ,l_BG_ID
        ,l_formula_type_id
        ,l_FF_NAME
        ,l_FF_DESC
        ,l_formula_text);

    ELSE
      --
      -- Update the existing Fast Formula text
      -- (if it has multiple date-track versions the text of
      -- all versions is updated).
      --
      UPDATE ff_formulas_f
      SET    formula_text = l_formula_text
            ,description  = l_FF_DESC
      WHERE  formula_id = l_ff_id;

    END IF;

    COMMIT;

    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        RAISE;
    END;

    --
    -- Compile the Fast Formula
    --
    xxas_util_pkg.compile_ff
      (p_user_id => l_USER_ID
      ,p_ff_type => l_FF_TYPE
      ,p_ff_name => l_FF_NAME);

EXCEPTION

  WHEN e_no_bg_id THEN
    RAISE no_data_found;

  WHEN e_bg_not_exists THEN
    RAISE no_data_found;

  WHEN e_ff_type_not_found THEN
    RAISE no_data_found;

END;
/
COMMIT;
EXIT;
