/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2018 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_xsl_rslt_cache_pkg AS 

--
-- Private Global Types and Constants
--
g_PACKAGE_NAME CONSTANT varchar2(30) := 'xxas_xsl_rslt_cache_pkg';

-- ------------------------------------------------------------------------ *
-- Name    : get_xsl_file_id
-- ------------------------------------------------------------------------ *
FUNCTION get_xsl_file_id(p_lang_xsl_name IN varchar2
                        ,p_xsl_name      IN varchar2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN

  $ELSE
    RESULT_CACHE RELIES_ON (fnd_lobs)
  $END

IS

  c_method CONSTANT varchar2(30) := 'get_xsl_file_id';

  CURSOR c_xsl(cp_lang_xsl_name varchar2
              ,cp_xsl_name      varchar2)
  IS
  SELECT (CASE WHEN INSTR(fl.file_name, '.') > 0
               THEN SUBSTR(fl.file_name,  1, INSTR(fl.file_name, '.') - 1)
               ELSE fl.file_name
          END) file_name
        ,fl.file_id
  FROM   fnd_lobs fl
  WHERE  CASE WHEN INSTR(fl.file_name, '.') > 0
              THEN SUBSTR(fl.file_name,  1, INSTR(fl.file_name, '.') - 1)
              ELSE fl.file_name
         END IN (cp_lang_xsl_name, cp_xsl_name)
  ORDER BY file_id DESC;    
  
  l_xml  XMLTYPE;
  l_lang_xsl_file_id   number;
  l_xsl_file_id        number;
  l_flag               boolean := false;
  l_lang_flag          boolean := false;

BEGIN

  FOR xsl_rec IN c_xsl(cp_lang_xsl_name => p_lang_xsl_name
                      ,cp_xsl_name      => p_xsl_name)
  LOOP
    IF xsl_rec.file_name = p_lang_xsl_name THEN
      IF NOT l_lang_flag THEN
        l_lang_xsl_file_id := xsl_rec.file_id;
        l_lang_flag          := true;
      END IF;
    ELSIF xsl_rec.file_name = p_xsl_name THEN
      IF NOT l_flag THEN
        l_xsl_file_id := xsl_rec.file_id;
        l_flag          := true;
      END IF;		
    END IF;
  END LOOP;

  RETURN nvl(l_lang_xsl_file_id,l_xsl_file_id);

END get_xsl_file_id;

END xxas_xsl_rslt_cache_pkg;
/
COMMIT;
EXIT;
