							   /* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2017 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
SET SCAN OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_absence_event_pkg IS
--
-- Private global constant's
--
gc_log xxas_com_log_type := xxas_com_log_type('xxas_absence_event_pkg');

g_PACKAGE_NAME       CONSTANT varchar2(22) := 'xxas_absence_event_pkg';
g_HRAPI_DATETIME_FMT CONSTANT varchar2(25) := 'YYYY-MM-DD HH24:MI:SS".0"';
g_HRAPI_DT_FMT       CONSTANT varchar2(21) := 'YYYY-MM-DD HH24:MI:SS';
g_HH24MI_FMT         CONSTANT varchar2(7)  := 'HH24:MI';
g_MON_DDTH_YYYY_FMT  CONSTANT varchar2(19) := 'Dy FMMon ddth, YYYY';
g_EOT                CONSTANT date         :=  xxas_util_pkg.eot;
g_UNDER_SCR          CONSTANT varchar2(1)  := '_';
g_INSERT             CONSTANT varchar2(10) := 'CreateMode';
g_UPDATE             CONSTANT varchar2(10) := 'UpdateMode';
g_DELETE             CONSTANT varchar2(10) := 'CancelMode';
g_SSH_PROD_KEY       CONSTANT varchar2(3)  := 'SSH';
g_ABS_ACTION_ID      CONSTANT varchar2(7)  := 'ABSENCE';
g_PER                CONSTANT varchar2(3)  := 'PER';
g_ABS_EVENT          CONSTANT varchar2(7)  := 'Absence';
g_ABS_EVENT_CD       CONSTANT varchar2(1)  := 'A';
g_APPROVED           CONSTANT varchar2(8)  := 'Approved';
g_PENDING            CONSTANT varchar2(7)  := 'Pending';
g_SUCCESS            CONSTANT varchar2(30) := 'balanced';
g_WARNING            CONSTANT varchar2(30) := 'energized';
g_ERROR              CONSTANT varchar2(30) := 'assertive';
g_PLANNED            CONSTANT varchar2(7)  := 'PLANNED';
g_CONFIRMED          CONSTANT varchar2(9)  := 'CONFIRMED';
g_CAL_EVENTS         CONSTANT varchar2(10) := 'CAL_EVENTS';
g_TCM_EVENT_CD       CONSTANT varchar2(3)  := 'TCM';
g_DRAFT              CONSTANT varchar2(5)  := 'DRAFT';
g_NOTIFICATION_PGID  CONSTANT varchar2(12) := 'NOTIFICATION';
g_FLEXIBLE           CONSTANT varchar2(8)  := 'flexible';
g_WORKLIST           CONSTANT varchar2(8)  := 'worklist';
g_PEERS              CONSTANT varchar2(2)  := 'PE';
g_DR_ONLY            CONSTANT varchar2(2)  := 'DR';
gc_boolean_true      CONSTANT VARCHAR2(4)  := 'true';
gc_boolean_false     CONSTANT VARCHAR2(5)  := 'false';

TYPE g_event_r  IS RECORD
       (row_id                   varchar2(1000)
       ,person_id                number
       ,assignment_id            number
       ,formatted_name           varchar2(400)
       ,initials                 varchar2(400)
       ,type                     varchar2(30)
       ,code                     varchar2(30)
       ,tooltip_line1            varchar2(400)
       ,tooltip_line2            varchar2(400)
       ,start_date               date
       ,end_date                 date
       ,start_time               varchar2(8)
       ,end_time                 varchar2(8)
       ,proj_start_date          date
       ,proj_end_date            date
       ,proj_start_time          varchar2(5)
       ,proj_end_time            varchar2(5)
       ,title                    varchar2(400)
       ,sch_additional_info      varchar2(400)
       ,icon                     varchar2(400)
       ,color                    varchar2(30)
       ,border_color             varchar2(30)
       ,text_color               varchar2(30)
       ,bg_color                 varchar2(30)
       ,country                  varchar2(80)
       ,page_id                  varchar2(4000)
       ,object_id                varchar2(4000)
       ,target                   varchar2(4000)
       ,absence_type             varchar2(1000)
       ,absence_category         varchar2(1000)
       ,absence_reason           varchar2(1000)
       ,payment_type             varchar2(1000)
       ,period_name              varchar2(1000)
       ,status                   varchar2(1000)
       ,status_code              varchar2(100)
       ,editable                 varchar2(30)
       ,public_holiday_name      varchar2(4000)
       ,payroll_name             varchar2(4000)
       ,birthday_name            varchar2(4000)
       ,absence_days             number
       ,absence_hours            number
       ,hours_or_days            varchar2(240)
       ,course_name              varchar2(80)
       ,class_name               varchar2(80)
       ,training_event_id        number
       ,session_or_scheduled     varchar2(30)
       ,delivery_mode            varchar2(240)
       ,training_duration        number
       ,training_dur_units       varchar2(240)
       ,item_type                varchar2(8)
       ,item_key                 varchar2(240)
       ,absence_id               number
       ,transaction_id           number
       ,transaction_step_id      number
       ,show_event               varchar2(1)
       ,common_events_res_name   varchar2(1000)
       ,log_entry_id             number
       ,duration_hours           number
       ,tcm_field1               varchar2(60)
       ,tcm_field2               varchar2(60)
       ,tcm_field3               varchar2(60)
       ,tcm_field4               varchar2(60)
       ,tcm_field5               varchar2(60)
       ,current_approver_role    varchar2(320)
       ,override_approver_role   varchar2(320)
       ,cost_concat_segments     varchar2(4000)
       ,deleted_flag             varchar2(1)
       ,summation_group          varchar2(150)
       ,summation_field          varchar2(150)
       ,payment_type_id          number
       ,otl_entry_type           varchar2(30)
       ,otl_entry_transferred_to varchar2(240)
       ,otl_timecard_id          number
       ,otl_hourstype_id         number
       ,otl_absence_days         number
       ,otl_absence_hours        number
       ,otl_timecard_ovn         number);

TYPE g_del_item_r IS RECORD
       (person_id             number
       ,absence_attendance_id number);

TYPE g_event_t IS TABLE OF g_event_r INDEX BY binary_integer;
TYPE g_del_items_t IS TABLE OF g_del_item_r INDEX BY binary_integer;

g_event_tbl            g_event_t;
g_del_items_tbl        g_del_items_t;

TYPE g_factor_r  IS RECORD
       (low_range      number
       ,high_range     number
       ,color          varchar2(100)
       ,message        varchar2(100));

TYPE g_factor_t IS TABLE OF g_factor_r INDEX BY binary_integer;

g_factor_tbl            g_factor_t;

--
-- ------------------------------------------------------------------------ *
-- Name    : log_start
-- ------------------------------------------------------------------------ *
PROCEDURE log_start
  (p_method      IN  varchar2
  ,p_parameter1  IN  varchar2 DEFAULT null
  ,p_parameter2  IN  varchar2 DEFAULT null
  ,p_parameter3  IN  varchar2 DEFAULT null
  ,p_parameter4  IN  varchar2 DEFAULT null
  ,p_parameter5  IN  varchar2 DEFAULT null
  ,p_parameter6  IN  varchar2 DEFAULT null
  ,p_parameter7  IN  varchar2 DEFAULT null
  ,p_parameter8  IN  varchar2 DEFAULT null
  ,p_parameter9  IN  varchar2 DEFAULT null
  ,p_parameter10 IN  varchar2 DEFAULT null)
IS
BEGIN
   IF gc_log.proc=1 
   THEN 
     gc_log.procedure_start_params
       (p_proc_name   =>  p_method
       ,p_parameter1  =>  p_parameter1
       ,p_parameter2  =>  p_parameter2
       ,p_parameter3  =>  p_parameter3
       ,p_parameter4  =>  p_parameter4
       ,p_parameter5  =>  p_parameter5
       ,p_parameter6  =>  p_parameter6
       ,p_parameter7  =>  p_parameter7
       ,p_parameter8  =>  p_parameter8
       ,p_parameter9  =>  p_parameter9
       ,p_parameter10 =>  p_parameter10); 
   END IF;  
END log_start; 

--
-- ------------------------------------------------------------------------ *
-- Name    : log_end
-- ------------------------------------------------------------------------ *
PROCEDURE log_end
  (p_method  IN  varchar2
  ,p_result  IN  varchar2 DEFAULT null)
IS
BEGIN
  IF gc_log.proc=1 
  THEN 
    gc_log.procedure_end_result
      (p_proc_name =>  p_method
      ,p_result    =>  p_result); 
  END IF;
END log_end;  

--
-- ------------------------------------------------------------------------ *
-- Name    : log_statement
-- ------------------------------------------------------------------------ *
PROCEDURE log_statement
  (p_method     IN  varchar2
  ,p_log_string IN  varchar2)
IS
BEGIN
  IF gc_log.stmt=1 
  THEN 
    gc_log.statement
      (p_proc_name  =>  p_method
      ,p_log_string =>  p_log_string); 
  END IF;
END log_statement;

--
-- ------------------------------------------------------------------------ *
-- Name    : register_user_error
-- ------------------------------------------------------------------------ *
PROCEDURE register_user_error
  (p_package_name IN varchar2
  ,p_proc_name    IN varchar2
  ,p_message      IN varchar2)
IS
 
BEGIN

 xxas_com_api_result_pkg.register_user_error
   (p_package_name => p_package_name
   ,p_proc_name    => p_proc_name
   ,p_field_name   => null
   ,p_message      => xxas_com_msg_type
                        (xxas_util_pkg.app
                        ,'XXAS_COM_TEXT'
                        ,xxas_com_msg_token_varray
                          (xxas_com_msg_token_type
                            ('TEXT'
                            ,p_message))));
END register_user_error;

--
-- ------------------------------------------------------------------------ *
-- Name    : raise_workflow_error
-- ------------------------------------------------------------------------ *
PROCEDURE raise_workflow_error
  (l_transaction_id IN number)
IS
  l_trans_error_message varchar2(4000);
  c_method            CONSTANT varchar2(30) := 'raise_workflow_error'; 
BEGIN

  IF l_transaction_id IS NOT NULL THEN
 
    select xxas_selfservice_workflow_pkg.get_error_text(l_transaction_id) into l_trans_error_message from dual;

    IF l_trans_error_message IS NOT NULL THEN
    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT',l_trans_error_message)
                              )
                           ));               
    END IF;
  END IF;

END raise_workflow_error;

-- ------------------------------------------------------------------------ *
-- Name    : event_formula_id
-- ------------------------------------------------------------------------ *
FUNCTION event_formula_id
  (p_formula_name      IN varchar2
  ,p_business_group_id IN number
  ,p_effective_date    IN date) RETURN number
IS

  --
  -- Gets the Fast Formula Id using the current formula name.
  --
  CURSOR csr_ff IS
  SELECT ff.formula_id
  FROM   ff_formulas_f ff
  WHERE  ff.business_group_id = p_business_group_id
  AND    ff.formula_name      = upper(p_formula_name)
  AND    p_effective_date BETWEEN ff.effective_start_date 
                          AND     ff.effective_end_date;

  l_formula_id number;

BEGIN

  IF p_business_group_id is not null AND
     p_formula_name      is not null AND
     p_effective_date    is not null THEN

    OPEN  csr_ff;
    FETCH csr_ff INTO l_formula_id;
    CLOSE csr_ff;

  END IF;

  RETURN l_formula_id;

END event_formula_id;

-- ------------------------------------------------------------------------ *
-- Name    : get_event_details
-- ------------------------------------------------------------------------ *
PROCEDURE get_event_details(p_formula_name    IN varchar2
                           ,p_effective_date  IN date
                           ,p_source_block    IN varchar2
                           ,p_saved_search_id IN number   DEFAULT null
                           ,p_manager_id      IN number   DEFAULT null)
IS

  c_method CONSTANT   varchar2(30) := 'get_event_details';
  l_inputs            ff_exec.inputs_t;
  l_outputs           ff_exec.outputs_t;
  l_show_event        varchar2(1) := g_NO;
  l_formula_id        number;
  err_code            varchar2(200);
  err_msg             varchar2(200);
  l_business_group_id number := fnd_profile.value('PER_BUSINESS_GROUP_ID');

BEGIN

  log_start(c_method,p_formula_name,p_effective_date,p_source_block,p_saved_search_id);

  IF l_business_group_id is null THEN

    l_business_group_id := xxas_com_sec_pkg.get_bg_id
                             (p_object_name    => xxas_com_sec_pkg.g_PER_OBJ_NAME
                             ,p_primary_key_id => xxas_cal_util_pkg.get_per_from_username(fnd_global.user_name));

  END IF;

  IF p_formula_name   is not null AND
     p_effective_date is not null AND
     l_business_group_id is not null AND
     g_event_tbl.count > 0 THEN

    --
    -- Get the formula Id
    --
    l_formula_id := event_formula_id
                      (p_formula_name      => p_formula_name
                      ,p_business_group_id => l_business_group_id
                      ,p_effective_date    => p_effective_date);

    --
    -- Validate that a formula was found
    --
    IF l_formula_id is null THEN

      xxas_com_api_result_pkg.raise_user_error
        (p_package_name => g_PACKAGE_NAME
        ,p_proc_name    => c_method
        ,p_field_name   => null
        ,p_message      => xxas_com_msg_type
                             (xxas_util_pkg.app
                             ,'XXAS_CAL_FF_NOT_PRESENT'
                             ,xxas_com_msg_token_varray
                                (xxas_com_msg_token_type
                                   ('BG_ID'
                                   ,l_business_group_id))));

    END IF;

    FOR i IN g_event_tbl.first..g_event_tbl.last 
    LOOP

      IF g_event_tbl.exists(i) AND
         g_event_tbl(i).person_id is not null AND 
         g_event_tbl(i).assignment_id is not null AND 
         g_event_tbl(i).show_event is null THEN

        --
        -- Set inputs.
        --
        l_inputs(1).name   := 'PERSON_ID';
        l_inputs(1).value  := g_event_tbl(i).person_id;
        l_inputs(2).name   := 'BUSINESS_GROUP_ID';
        l_inputs(2).value  := null;--g_event_tbl(i).business_group_id;
        l_inputs(3).name   := 'DATE_EARNED';
        l_inputs(3).value  := fnd_date.date_to_canonical(p_effective_date);
        l_inputs(4).name   := 'EVENT_TYPE';
        l_inputs(4).value  := g_event_tbl(i).type;
        l_inputs(5).name   := 'ABSENCE_ID';
        l_inputs(5).value  := g_event_tbl(i).absence_id;
        l_inputs(6).name   := 'STATUS';
        l_inputs(6).value  := g_event_tbl(i).status;
        l_inputs(7).name   := 'ABSENCE_TYPE';
        l_inputs(7).value  := g_event_tbl(i).absence_type;
        l_inputs(8).name   := 'ABSENCE_CATEGORY';
        l_inputs(8).value  := g_event_tbl(i).absence_category;
        l_inputs(9).name   := 'ABSENCE_REASON';
        l_inputs(9).value  := g_event_tbl(i).absence_reason;
        l_inputs(10).name  := 'HOURS_OR_DAYS';
        l_inputs(10).value := g_event_tbl(i).hours_or_days;
        l_inputs(11).name  := 'DATE_PROJECTED_START';
        l_inputs(11).value := fnd_date.date_to_canonical(g_event_tbl(i).proj_start_date);
        l_inputs(12).name  := 'TIME_PROJECTED_START';
        l_inputs(12).value := g_event_tbl(i).proj_start_time;
        l_inputs(13).name  := 'DATE_PROJECTED_END';
        l_inputs(13).value := fnd_date.date_to_canonical(g_event_tbl(i).proj_end_date);
        l_inputs(14).name  := 'TIME_PROJECTED_END';
        l_inputs(14).value := g_event_tbl(i).proj_end_time;
        l_inputs(15).name  := 'DATE_START';
        l_inputs(15).value := fnd_date.date_to_canonical(g_event_tbl(i).start_date);
        l_inputs(16).name  := 'TIME_START';
        l_inputs(16).value := g_event_tbl(i).start_time;
        l_inputs(17).name  := 'DATE_END';
        l_inputs(17).value := fnd_date.date_to_canonical(g_event_tbl(i).end_date);
        l_inputs(18).name  := 'TIME_END';
        l_inputs(18).value := g_event_tbl(i).end_time;
        l_inputs(19).name  := 'ABSENCE_DAYS';
        l_inputs(19).value := g_event_tbl(i).absence_days;
        l_inputs(20).name  := 'ABSENCE_HOURS';
        l_inputs(20).value := g_event_tbl(i).absence_hours;
        l_inputs(21).name  := 'ITEM_TYPE';
        l_inputs(21).value := g_event_tbl(i).item_type;
        l_inputs(22).name  := 'ITEM_KEY';
        l_inputs(22).value := g_event_tbl(i).item_key;
        l_inputs(23).name  := 'TRANSACTION_ID';
        l_inputs(23).value := g_event_tbl(i).transaction_id;
        l_inputs(24).name  := 'TRANSACTION_STEP_ID';
        l_inputs(24).value := g_event_tbl(i).transaction_step_id;
        l_inputs(25).name  := 'ASSIGNMENT_ID';
        l_inputs(25).value := g_event_tbl(i).assignment_id;
        l_inputs(26).name  := 'PUBLIC_HOLIDAY_NAME';
        l_inputs(26).value := g_event_tbl(i).public_holiday_name;
        l_inputs(27).name  := 'PAYROLL_NAME';
        l_inputs(27).value := g_event_tbl(i).payroll_name;
        l_inputs(28).name  := 'BIRTHDAY_NAME';
        l_inputs(28).value := g_event_tbl(i).birthday_name;
        l_inputs(29).name  := 'COURSE_NAME';
        l_inputs(29).value := g_event_tbl(i).course_name;
        l_inputs(30).name  := 'CLASS_NAME';
        l_inputs(30).value := g_event_tbl(i).class_name;
        l_inputs(31).name  := 'TRAINING_DURATION';
        l_inputs(31).value := g_event_tbl(i).training_duration;
        l_inputs(32).name  := 'COUNTRY';
        l_inputs(32).value := g_event_tbl(i).country;
        l_inputs(33).name  := 'TRAINING_EVENT_ID';
        l_inputs(33).value := g_event_tbl(i).training_event_id;
        l_inputs(34).name  := 'SESSION_OR_SCHEDULED';
        l_inputs(34).value := g_event_tbl(i).session_or_scheduled;
        l_inputs(35).name  := 'DELIVERY_MODE';
        l_inputs(35).value := g_event_tbl(i).delivery_mode;
        l_inputs(36).name  := 'TRAINING_DURATION_UNITS';
        l_inputs(36).value := g_event_tbl(i).training_dur_units;
        l_inputs(37).name  := 'ELEMENT_NAME';
        l_inputs(37).value := g_event_tbl(i).payment_type;
        l_inputs(38).name  := 'LOG_ENTRY_ID';
        l_inputs(38).value := g_event_tbl(i).log_entry_id;
        l_inputs(39).name  := 'TIME_ENTRY_DURATION_HOURS';
        l_inputs(39).value := g_event_tbl(i).duration_hours;
        l_inputs(40).name  := 'STATUS_CODE';
        l_inputs(40).value := g_event_tbl(i).status_code;
        l_inputs(41).name  := 'FIELD1';
        l_inputs(41).value := g_event_tbl(i).tcm_field1;
        l_inputs(42).name  := 'FIELD2';
        l_inputs(42).value := g_event_tbl(i).tcm_field2;
        l_inputs(43).name  := 'FIELD3';
        l_inputs(43).value := g_event_tbl(i).tcm_field3;
        l_inputs(44).name  := 'FIELD4';
        l_inputs(44).value := g_event_tbl(i).tcm_field4;
        l_inputs(45).name  := 'FIELD5';
        l_inputs(45).value := g_event_tbl(i).tcm_field5;
        l_inputs(46).name  := 'OVERRIDE_APPROVER_ROLE';
        l_inputs(46).value := g_event_tbl(i).override_approver_role;
        l_inputs(47).name  := 'COST_ALLOC_CONCAT_SEGMENTS';
        l_inputs(47).value := g_event_tbl(i).cost_concat_segments;
        l_inputs(48).name  := 'SOURCE_BLOCK';
        l_inputs(48).value := p_source_block;
        l_inputs(49).name  := 'CURRENT_APPROVER_ROLE';
        l_inputs(49).value := g_event_tbl(i).current_approver_role;
        l_inputs(50).name  := 'SAVED_SEARCH_ID';
        l_inputs(50).value := p_saved_search_id;
        l_inputs(51).name  := 'PAYMENT_TYPE_ID';
        l_inputs(51).value := g_event_tbl(i).payment_type_id;
        l_inputs(52).name  := 'OTL_ENTRY_TYPE';
        l_inputs(52).value := g_event_tbl(i).otl_entry_type;
        l_inputs(53).name  := 'OTL_ENTRY_TRANSFERRED_TO';
        l_inputs(53).value := g_event_tbl(i).otl_entry_transferred_to;
        l_inputs(54).name  := 'OTL_TIMECARD_ID';
        l_inputs(54).value := g_event_tbl(i).otl_timecard_id;
        l_inputs(55).name  := 'OTL_HOURSTYPE_ID';
        l_inputs(55).value := g_event_tbl(i).otl_hourstype_id;
        l_inputs(56).name  := 'OTL_ABSENCE_DAYS';
        l_inputs(56).value := g_event_tbl(i).otl_absence_days;
        l_inputs(57).name  := 'OTL_ABSENCE_HOURS';
        l_inputs(57).value := g_event_tbl(i).otl_absence_hours;
        l_inputs(58).name  := 'OTL_TIMECARD_OVN';
        l_inputs(58).value := g_event_tbl(i).otl_timecard_ovn;
        l_inputs(58).name  := 'MANAGER_ID';
        l_inputs(58).value := p_manager_id;

        --
        -- Prepare outputs.
        --
        l_outputs(1).name  := 'SHOW_EVENT';
        l_outputs(2).name  := 'START';
        l_outputs(3).name  := 'END';
        l_outputs(4).name  := 'EVENT_EDITABLE';
        l_outputs(5).name  := 'TITLE';
        l_outputs(6).name  := 'SCHEDULER_ADDITIONAL_INFO';
        l_outputs(7).name  := 'TOOLTIP_LINE1';
        l_outputs(8).name  := 'TOOLTIP_LINE2';
        l_outputs(9).name  := 'ICON';
        l_outputs(10).name := 'COLOR';
        l_outputs(11).name := 'TEXT_COLOR';
        l_outputs(12).name := 'BORDER_COLOR';
        l_outputs(13).name := 'PAGE_ID';
        l_outputs(14).name := 'OBJECT_ID';
        l_outputs(15).name := 'TARGET';

        --
        -- Run the formula.
        --
        xxas_fast_formula_pkg.run_formula
          (p_formula_id     => l_formula_id
          ,p_effective_date => p_effective_date
          ,p_inputs         => l_inputs
          ,p_outputs        => l_outputs);

        --
        -- Retrieve the formula outputs.
        --
        g_event_tbl(i).show_event             := upper(nvl(l_outputs(1).value, l_show_event));
        g_event_tbl(i).start_date             := fnd_date.canonical_to_date(l_outputs(2).value);
        g_event_tbl(i).end_date               := fnd_date.canonical_to_date(l_outputs(3).value);
        g_event_tbl(i).editable               := substr(l_outputs(4).value, 1, 400);
        g_event_tbl(i).title                  := substr(l_outputs(5).value, 1, 400);
        g_event_tbl(i).sch_additional_info    := substr(l_outputs(6).value, 1, 400);
        g_event_tbl(i).tooltip_line1          := substr(l_outputs(7).value, 1, 400);
        g_event_tbl(i).tooltip_line2          := substr(l_outputs(8).value, 1, 400);
        g_event_tbl(i).icon                   := substr(l_outputs(9).value, 1, 400);
        g_event_tbl(i).color                  := substr(trim(l_outputs(10).value), 1, 400);
        g_event_tbl(i).text_color             := substr(trim(l_outputs(11).value), 1, 400);
        g_event_tbl(i).border_color           := substr(trim(l_outputs(12).value), 1, 400);
        g_event_tbl(i).page_id                := substr(l_outputs(13).value, 1, 400);
        g_event_tbl(i).object_id              := substr(l_outputs(14).value, 1, 400);
        g_event_tbl(i).target                 := substr(l_outputs(15).value, 1, 400);
        g_event_tbl(i).row_id                 := CASE WHEN g_event_tbl(i).row_id is null AND upper(g_event_tbl(i).page_id) <> 'NOTIFICATION'
                                                      THEN g_event_tbl(i).code || '-' || g_event_tbl(i).object_id
                                                      WHEN g_event_tbl(i).row_id is null AND upper(g_event_tbl(i).page_id) = 'NOTIFICATION'
                                                      THEN g_event_tbl(i).object_id
                                                      ELSE g_event_tbl(i).row_id
                                                 END;

      END IF;

    END LOOP;

  END IF;

  log_end(c_method);

EXCEPTION

  WHEN others THEN

      err_code := SQLCODE;
      err_msg  := SUBSTR(SQLERRM, 1, 200);

      xxas_com_api_result_pkg.raise_user_error
        (p_package_name => g_PACKAGE_NAME
        ,p_proc_name    => c_method
        ,p_field_name   => null
        ,p_message      => xxas_com_msg_type
                             (xxas_util_pkg.app
                             ,'XXAS_COM_TEXT'
                             ,xxas_com_msg_token_varray
                                (xxas_com_msg_token_type
                                   ('TEXT'
                                   ,'Error Code: ' || err_code || ' and Error Message: ' || err_msg))));

END get_event_details;

--
-- ------------------------------------------------------------------------ *
-- Name    : changed_items_data
-- ------------------------------------------------------------------------ *
FUNCTION changed_items_data(p_source_block    IN varchar2
                           ,p_saved_search_id IN number   DEFAULT null
                           ,p_manager_id      IN number   DEFAULT null) RETURN xxas_cal_event_view_varray PIPELINED
IS

  c_method  CONSTANT       varchar2(30) := 'changed_items_data';
  l_row                    xxas_cal_event_view_type;
  l_rest_provider          varchar2(4000);
  l_rest_provider_path     varchar2(4000);
  l_formula_name           varchar2(1000);

BEGIN

  log_start(c_method,p_source_block,p_saved_search_id);

  IF g_event_tbl.count > 0 THEN
 
    l_formula_name := xxas_com_sec_pkg.get_function_parameter
                        (p_function_name  => p_source_block
                        ,p_parameter_name => 'pFormulaName');

    get_event_details(p_formula_name    => l_formula_name
                     ,p_effective_date  => trunc(sysdate)
                     ,p_source_block    => p_source_block
                     ,p_saved_search_id => p_saved_search_id
                     ,p_manager_id      => p_manager_id);
    FOR i IN g_event_tbl.first..g_event_tbl.last 
    LOOP
          IF g_event_tbl.exists(i) THEN

        l_row := null;
        IF g_event_tbl(i).code IN (g_ABS_EVENT_CD) AND 
           g_event_tbl(i).show_event = 'Y' THEN
      
          -- Convert the Outputs into xxas_cal_event_view_type
          -- and PIPE the row
          --
          l_row := xxas_cal_event_view_type
                     (row_id                => g_event_tbl(i).row_id
                     ,person_id             => g_event_tbl(i).person_id
                     ,assignment_id         => g_event_tbl(i).assignment_id
                     ,summary_icon          => lower(trim(g_event_tbl(i).icon))
                     ,title                 => g_event_tbl(i).title
                     ,resource_name         => g_event_tbl(i).formatted_name
                     ,resource_initials     => g_event_tbl(i).initials
                     ,sch_additional_info   => g_event_tbl(i).sch_additional_info
                     ,event_start           => g_event_tbl(i).start_date
                     ,event_end             => g_event_tbl(i).end_date
                     ,tooltip_line1         => g_event_tbl(i).tooltip_line1
                     ,tooltip_line2         => g_event_tbl(i).tooltip_line2
                     ,color                 => g_event_tbl(i).color
                     ,border_color          => g_event_tbl(i).border_color
                     ,text_color            => g_event_tbl(i).text_color
                     ,bg_color              => g_event_tbl(i).bg_color
                     ,page_id               => g_event_tbl(i).page_id
                     ,object_id             => g_event_tbl(i).object_id
                     ,target                => g_event_tbl(i).target
                     ,type                  => g_event_tbl(i).type
                     ,absence_type          => g_event_tbl(i).absence_type
                     ,absence_category      => g_event_tbl(i).absence_category
                     ,absence_reason        => g_event_tbl(i).absence_reason
                     ,payment_type          => g_event_tbl(i).payment_type
                     ,period_name           => g_event_tbl(i).period_name
                     ,status                => g_event_tbl(i).status
                     ,editable              => g_event_tbl(i).editable
                     ,code                  => g_event_tbl(i).code
                     ,deleted_flag          => g_event_tbl(i).deleted_flag
                     ,status_code           => g_event_tbl(i).status_code
                     ,summation_group       => g_event_tbl(i).summation_group
                     ,summation_field       => g_event_tbl(i).summation_field
                     ,duration_hours        => g_event_tbl(i).duration_hours
                     ,field1                => g_event_tbl(i).tcm_field1
                     ,field2                => g_event_tbl(i).tcm_field2
                     ,field3                => g_event_tbl(i).tcm_field3
                     ,field4                => g_event_tbl(i).tcm_field4
                     ,field5                => g_event_tbl(i).tcm_field5
                     ,current_approver_role  => g_event_tbl(i).current_approver_role
                     ,override_approver_role => g_event_tbl(i).override_approver_role);

          PIPE ROW (l_row);

        END IF;

      END IF;

    END LOOP;

  END IF;
 
  log_end(c_method);

  RETURN;
  
END changed_items_data;

--
-- ------------------------------------------------------------------------ *
-- Name    : deleted_items_data
-- ------------------------------------------------------------------------ *
FUNCTION deleted_items_data(p_source_block    IN varchar2
                           ,p_saved_search_id IN number   DEFAULT null
                           ,p_manager_id      IN number   DEFAULT null) RETURN xxas_cal_event_view_varray PIPELINED
IS

  c_method  CONSTANT varchar2(30) := 'deleted_items_data';
  l_row              xxas_cal_event_view_type;
BEGIN

  log_start(c_method,p_source_block,p_saved_search_id);

  IF g_del_items_tbl.count > 0 THEN

    FOR i IN g_del_items_tbl.first..g_del_items_tbl.last 
    LOOP

      IF g_del_items_tbl.exists(i) THEN

        l_row := null;

        --
        -- Convert the Outputs into xxas_cal_event_view_type
        -- and PIPE the row
        --
        l_row := xxas_cal_event_view_type
                   (row_id                => g_ABS_EVENT_CD || '-' || to_char(g_del_items_tbl(i).person_id) || '_' || to_char(g_del_items_tbl(i).absence_attendance_id) || '___' || p_source_block || '_' || p_saved_search_id||  '_' || to_char(p_manager_id)
                   ,person_id             => null
                   ,assignment_id         => null
                   ,summary_icon          => null
                   ,title                 => null
                   ,resource_name         => null
                   ,resource_initials     => null
                   ,sch_additional_info   => null
                   ,event_start           => null
                   ,event_end             => null
                   ,tooltip_line1         => null
                   ,tooltip_line2         => null
                   ,color                 => null
                   ,border_color          => null
                   ,text_color            => null
                   ,bg_color              => null
                   ,page_id               => null
                   ,object_id             => null
                   ,target                => null
                   ,type                  => null
                   ,absence_type          => null
                   ,absence_category      => null
                   ,absence_reason        => null
                   ,payment_type          => null
                   ,period_name           => null
                   ,status                => null
                   ,editable              => null
                   ,code                  => null
                   ,deleted_flag          => null
                   ,status_code           => null
                   ,summation_group       => null
                   ,summation_field       => null
                   ,duration_hours        => null
                   ,field1                => null
                   ,field2                => null
                   ,field3                => null
                   ,field4                => null
                   ,field5                => null
                   ,current_approver_role  => null
                   ,override_approver_role => null);

        PIPE ROW (l_row);

      END IF;

    END LOOP;

  END IF;

  log_end(c_method);

  RETURN;
  
END deleted_items_data;

-- ------------------------------------------------------------------------ *
-- Name    : changed_items_xml
-- ------------------------------------------------------------------------ *
FUNCTION changed_items_xml(p_source_block    IN varchar2
                          ,p_saved_search_id IN number   DEFAULT null
                          ,p_manager_id      IN number   DEFAULT null) RETURN XMLType
IS

  c_method  CONSTANT  varchar2(30) := 'changed_items_xml';
  l_xml               XMLType;
  l_emp_calendar_flag varchar2(1);

BEGIN

  log_start(c_method,p_source_block,p_saved_search_id);
  IF g_event_tbl is not null AND 
     g_event_tbl.count > 0 THEN

    l_emp_calendar_flag := CASE WHEN instr(xxas_com_sec_pkg.get_function_parameter
                                             (p_function_name  => p_source_block
                                             ,p_parameter_name => 'pRestProvider'),'for-individual') <> 0
                                THEN 'Y'
                                ELSE 'N'
                           END;

    SELECT XMLAgg
            (XMLElement
              ("item"
              ,XMLForest
                (cal_event.resource_id   "resourceId"
                ,cal_event.row_id        "id"
                ,cal_event.type          "type"
                ,cal_event.event_type    "eventType"
                ,CASE WHEN l_emp_calendar_flag = 'Y'
                      THEN null
                      ELSE cal_event.resource_name
                 END "resourceName"
                ,cal_event.title         "title"
                ,cal_event.sch_addl_info "schedulerAdditionalInfo"
                ,cal_event.tooltip_line1 "tooltipLine1"
                ,cal_event.tooltip_line2 "tooltipLine2"
                ,CASE WHEN cal_event.event_start is null
                      THEN null
                      WHEN to_char(cal_event.event_start, g_HH24MI_FMT) = '00:00'
                      THEN to_char(cal_event.event_start, g_HRAPI_DATE_FMT)
                      ELSE to_char(cal_event.event_start, g_HRAPI_DT_FMT)
                 END "start"
                ,CASE WHEN cal_event.event_end is null
                      THEN null
                      WHEN to_char(cal_event.event_end, g_HH24MI_FMT) = '00:00'
                      THEN to_char(cal_event.event_end+1, g_HRAPI_DATE_FMT)
                      ELSE to_char(cal_event.event_end, g_HRAPI_DT_FMT)
                 END "end"
                ,(CASE WHEN nvl(cal_event.deleted_flag,'N') = 'Y' 
                       THEN xxas_util_pkg.hex_to_rgba(cal_event.color, 0.5)
                       ELSE xxas_util_pkg.hex_to_rgba(cal_event.color)
                  END
                 ) "color"
                ,cal_event.border_color  "borderColor"
                ,cal_event.text_color    "textColor"
                ,cal_event.bg_color      "backgroundColor"
                ,cal_event.icon          "icon"
                ,cal_event.page_id       "pageId"
                ,cal_event.object_id     "objectId"
                ,cal_event.target        "target"
                ,cal_event.absence_type     "absenceType"
                ,cal_event.absence_category "absenceCategory"
                ,cal_event.absence_reason   "absenceReason"
                ,cal_event.payment_type     "paymentType"
                ,cal_event.period_name      "periodName"
                ,cal_event.event_status     "eventStatus"
                ,cal_event.editable         "editable"
                ,(CASE WHEN nvl(cal_event.deleted_flag,'N') = 'Y' 
                       THEN 'true'
                       ELSE null
                  END
                 ) "deleted"
                ,CASE WHEN cal_event.events_type = g_CAL_EVENTS AND upper(cal_event.page_id) = g_NOTIFICATION_PGID AND cal_event.object_id is not null
                      THEN XMLElement
                            ("data"
                            ,xxas_com_worklist_pkg.get_actions(p_notification_id       => cal_event.object_id
                                                              ,p_from_role             => null
                                                              ,p_response_formula      => null
                                                              ,p_response_xml_function => 'xxas_cal_event_pkg.changed_items_notif_resp_xml'
                                                              ,p_source_block          => p_source_block).get_xml()
                            ,CASE WHEN l_emp_calendar_flag = 'Y'
                                  THEN null
                                  ELSE XMLElement
                                        ("image"
                                        ,XMLForest
                                          (xxas_com_rest_util_pkg.uri_from_path
                                             ('people/' || cal_event.img_person_id || '/photo') "self"
                                          ,cal_event.initials "initials"
                                          ,xxas_per_image_pkg.url
                                             (cal_event.img_person_id, cal_event.img_last_update_date, 'SMALL') "small"
                                          )
                                        )
                             END
                            ).extract('/data/*')
                      WHEN cal_event.events_type = g_CAL_EVENTS AND cal_event.code = g_TCM_EVENT_CD AND cal_event.event_status_code = 'P' AND 
                           nvl(cal_event.override_approver_role,cal_event.current_approver_role) = fnd_global.user_name
                      THEN XMLElement
                            ("data"
                            ,xxas_timecard_entry_pkg.get_pending_entry_actions()
                            ,CASE WHEN l_emp_calendar_flag = 'Y'
                                  THEN null
                                  ELSE XMLElement
                                        ("image"
                                        ,XMLForest
                                          (xxas_com_rest_util_pkg.uri_from_path
                                             ('people/' || cal_event.img_person_id || '/photo') "self"
                                          ,cal_event.initials "initials"
                                          ,xxas_per_image_pkg.url
                                             (cal_event.img_person_id, cal_event.img_last_update_date, 'SMALL') "small"
                                          )
                                        )
                             END
                            ).extract('/data/*')
                      WHEN cal_event.events_type = g_CAL_EVENTS AND cal_event.code = g_TCM_EVENT_CD AND upper(cal_event.event_status) = g_DRAFT
                      THEN XMLElement
                            ("data"
                            ,CASE WHEN nvl(cal_event.deleted_flag,'N') = 'Y'
                                  THEN xxas_timecard_entry_pkg.get_appr_draft_entry_actions()
                                  ELSE xxas_timecard_entry_pkg.get_draft_status_actions()
                             END
                            ,CASE WHEN l_emp_calendar_flag = 'Y'
                                  THEN null
                                  ELSE XMLElement
                                        ("image"
                                        ,XMLForest
                                          (xxas_com_rest_util_pkg.uri_from_path
                                             ('people/' || cal_event.img_person_id || '/photo') "self"
                                          ,cal_event.initials "initials"
                                          ,xxas_per_image_pkg.url
                                             (cal_event.img_person_id, cal_event.img_last_update_date, 'SMALL') "small"
                                          )
                                        )
                             END
                            ).extract('/data/*')
                      WHEN cal_event.events_type = g_CAL_EVENTS AND cal_event.code = g_TCM_EVENT_CD AND upper(cal_event.event_status) = upper(g_APPROVED)
                      THEN XMLElement
                            ("data"
                            ,xxas_timecard_entry_pkg.get_approved_entry_actions()
                            ,CASE WHEN l_emp_calendar_flag = 'Y'
                                  THEN null
                                  ELSE XMLElement
                                        ("image"
                                        ,XMLForest
                                          (xxas_com_rest_util_pkg.uri_from_path
                                             ('people/' || cal_event.img_person_id || '/photo') "self"
                                          ,cal_event.initials "initials"
                                          ,xxas_per_image_pkg.url
                                             (cal_event.img_person_id, cal_event.img_last_update_date, 'SMALL') "small"
                                          )
                                        )
                             END
                            ).extract('/data/*')
                      WHEN cal_event.events_type = g_CAL_EVENTS
                      THEN CASE WHEN l_emp_calendar_flag = 'Y'
                                THEN null
                                ELSE XMLElement
                                      ("image"
                                      ,XMLForest
                                        (xxas_com_rest_util_pkg.uri_from_path
                                           ('people/' || cal_event.img_person_id || '/photo') "self"
                                        ,cal_event.initials "initials"
                                        ,xxas_per_image_pkg.url
                                           (cal_event.img_person_id, cal_event.img_last_update_date, 'SMALL') "small"
                                        )
                                      )
                           END
                      ELSE null
                 END "linked"
                ,XMLElement
                  ("resourceIds"
                  ,XMLAttributes
                    ('true' as "json:force-array"
                    ,'http://json.org/' as "xmlns:json")
                  ,XMLElement
                    ("resourceId"
                    ,cal_event.resource_id)
                  ) "constraint"
                )
              ,(CASE WHEN cal_event.code = g_TCM_EVENT_CD
                     THEN XMLElement
                           ("summary"
                           ,XMLAttributes
                             ('true' as "json:force-array"
                             ,'http://json.org/' as "xmlns:json")
                             ,(CASE WHEN cal_event.summation_group is not null AND
                                         cal_event.summation_field is not null
                                    THEN XMLElement
                                          ("group"
                                          ,XMLForest
                                            ('GROUP:' || cal_event.summation_group  "id"
                                            ,(CASE WHEN upper(cal_event.summation_field) = 'DURATION'
                                                   THEN cal_event.duration_hours
                                                   WHEN upper(cal_event.summation_field) = 'USER FIELD 1'
                                                   THEN to_number(cal_event.field1)
                                                   WHEN upper(cal_event.summation_field) = 'USER FIELD 2'
                                                   THEN to_number(cal_event.field2)
                                                   WHEN upper(cal_event.summation_field) = 'USER FIELD 3'
                                                   THEN to_number(cal_event.field3)
                                                   WHEN upper(cal_event.summation_field) = 'USER FIELD 4'
                                                   THEN to_number(cal_event.field4)
                                                   WHEN upper(cal_event.summation_field) = 'USER FIELD 5'
                                                   THEN to_number(cal_event.field5)
                                                   ELSE null
                                              END
                                             ) "value"
                                            )
                                          )
                                    ELSE null
                               END
                              )
                           ,XMLElement
                             ("state"
                             ,XMLForest
                               ('STATE:' || cal_event.event_status_code  "id"
                               ,1                                        "value"
                               )
                             )
                           )
                     ELSE null
                END)
              ,(CASE WHEN cal_event.editable = 'true'
                     THEN XMLElement
                           ("meta"
                           ,XMLElement
                             ("operations"
                             ,XMLElement
                               ("PUT"
                               ,XMLElement
                                 ("href"
                                 ,xxas_com_rest_util_pkg.uri_from_path
                                    ('calendar-event/edit-entry?resourceId={resourceId}' || '&eventId={eventId}' || '&startTime={startTime}' || '&endTime={endTime}' || '&sourceBlock=' || p_source_block || '&savedSearchId=' || to_char(p_saved_search_id || '&managerId=' || p_manager_id))
                                 )
                               )
                             )
                           )
                     ELSE null
                END)
              )
            )  "data"
    INTO   l_xml
    FROM   (SELECT cali.assignment_id         resource_id
                  ,cali.row_id                row_id
                  ,DECODE(upper(cali.page_id),g_NOTIFICATION_PGID,g_WORKLIST,g_FLEXIBLE) type
                  ,cali.resource_name         resource_name
                  ,cali.resource_initials     initials
                  ,cali.sch_additional_info   sch_addl_info
                  ,cali.title                 title
                  ,cali.tooltip_line1         tooltip_line1
                  ,cali.tooltip_line2         tooltip_line2
                  ,cali.event_start           event_start
                  ,cali.event_end             event_end
                  ,cali.color                 color
                  ,cali.border_color          border_color
                  ,cali.text_color            text_color
                  ,cali.bg_color              bg_color
                  ,cali.summary_icon          icon
                  ,cali.page_id               page_id
                  ,cali.object_id             object_id
                  ,cali.target                target
                  ,img.person_id              img_person_id
                  ,img.last_update_date       img_last_update_date
                  ,g_CAL_EVENTS               events_type
                  ,cali.type                  event_type
                  ,cali.absence_type          absence_type
                  ,cali.absence_category      absence_category
                  ,cali.absence_reason        absence_reason
                  ,cali.payment_type          payment_type
                  ,cali.period_name           period_name
                  ,cali.status                event_status
                  ,cali.editable              editable
                  ,cali.code                  code
                  ,cali.deleted_flag          deleted_flag
                  ,cali.status_code           event_status_code
                  ,cali.summation_group       summation_group
                  ,cali.summation_field       summation_field
                  ,cali.duration_hours        duration_hours
                  ,cali.field1                field1
                  ,cali.field2                field2
                  ,cali.field3                field3
                  ,cali.field4                field4
                  ,cali.field5                field5
                  ,cali.current_approver_role  current_approver_role
                  ,cali.override_approver_role override_approver_role
            FROM   TABLE(xxas_absence_event_pkg.changed_items_data(p_source_block    => p_source_block
                                                                  ,p_saved_search_id => p_saved_search_id
                                                                  ,p_manager_id      => p_manager_id)) cali
                  ,xxas_per_thumbnails   img
            WHERE  cali.person_id = img.person_id(+)
           ) cal_event;

  END IF;

  log_end(c_method);
  RETURN l_xml;

END changed_items_xml;

-- ------------------------------------------------------------------------ *
-- Name    : deleted_items_xml
-- ------------------------------------------------------------------------ *
FUNCTION deleted_items_xml(p_source_block    IN varchar2
                          ,p_saved_search_id IN number   DEFAULT null
                          ,p_manager_id      IN number   DEFAULT null) RETURN XMLType
IS

  c_method CONSTANT     varchar2(30) := 'deleted_items_xml';
  l_xml               XMLType;

BEGIN

  log_start(c_method,p_source_block,p_saved_search_id);

  IF g_del_items_tbl is not null AND
     g_del_items_tbl.count > 0 THEN

    SELECT XMLAgg
            (XMLElement
              ("item"
              ,XMLForest
                (cali.row_id "id"
                )
              )
            ) "data"
    INTO   l_xml
    FROM   TABLE(xxas_absence_event_pkg.deleted_items_data(p_source_block    => p_source_block
                                                          ,p_saved_search_id => p_saved_search_id
                                                          ,p_manager_id      => p_manager_id)) cali;

  END IF;

  log_end(c_method);
  RETURN l_xml;

END deleted_items_xml;

--
-- ------------------------------------------------------------------------ *
-- Name    : store_item_into_event_tbl
-- ------------------------------------------------------------------------ *
PROCEDURE store_item_into_event_tbl(p_person_id             IN number
                                   ,p_absence_attendance_id IN number
                                   ,p_transaction_id        IN number
                                   ,p_transaction_step_id   IN number
                                   ,p_effective_date        IN date
                                   ,p_status                IN varchar2
                                   ,p_action                IN varchar2
                                   ,p_approvers             IN varchar2)
IS

  --
  -- Private constants and variables.
  --
  l_ABS_REAS_LOOKUP CONSTANT varchar2(14) := 'ABSENCE_REASON';
  l_ABS_UOM_LOOKUP  CONSTANT varchar2(13) := 'HOURS_OR_DAYS';
  l_ABS_CAT_LOOKUP  CONSTANT varchar2(16) := 'ABSENCE_CATEGORY';
  l_ABS_API_NAME    CONSTANT varchar2(21) := 'HR_LOA_SS.PROCESS_API';

  CURSOR csr_absences IS
  SELECT * FROM (
  SELECT abs.absence_attendance_id
        ,'A' status_code
        ,g_APPROVED workflow_status
        ,null item_type
        ,null item_key
        ,abt.name absence_type
        ,hrl1.meaning absence_category
        ,decode(abr.name, null, null
               ,(SELECT meaning
                 FROM   hr_lookups
                 WHERE  lookup_code = abr.name
                 AND    lookup_type = l_ABS_REAS_LOOKUP)) absence_reason
        ,decode(abt.hours_or_days, null, null
               ,(SELECT meaning
                 FROM   hr_lookups
                 WHERE  lookup_code = abt.hours_or_days
                 AND    lookup_type = l_ABS_UOM_LOOKUP)) hours_or_days
        ,abs.date_projected_start
        ,abs.time_projected_start
        ,abs.date_projected_end
        ,abs.time_projected_end
        ,abs.date_start
        ,abs.time_start
        ,abs.date_end
        ,abs.time_end
        ,abs.absence_days
        ,abs.absence_hours
        ,abs.object_version_number
        ,null transaction_id
        ,null transaction_step_id
        ,papf.person_id
        ,xxas_util_pkg.get_formatted_name
           (p_full_name   => papf.full_name
           ,p_global_name => papf.global_name
           ,p_local_name  => papf.local_name
           ,p_first_name  => papf.first_name
           ,p_last_name   => papf.last_name
           ,p_known_as    => papf.known_as) formatted_name
        ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
        ,xxas_util_pkg.get_primary_worker_asg(papf.person_id,p_effective_date) assignment_id
        ,'N' deleted_flag
  FROM   per_absence_attendances abs
        ,per_abs_attendance_types_vl abt
        ,per_abs_attendance_reasons abr
        ,hr_lookups hrl1
        ,per_all_people_f papf
  WHERE  abs.person_id = p_person_id
  AND    abs.person_id = papf.person_id
  AND    p_effective_date BETWEEN papf.effective_start_date 
                          AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y')
  AND    abs.absence_attendance_type_id = abt.absence_attendance_type_id
  AND    abs.abs_attendance_reason_id = abr.abs_attendance_reason_id(+)
  AND    abt.absence_category = hrl1.lookup_code(+)
  AND    hrl1.lookup_type(+) = l_ABS_CAT_LOOKUP
  AND    (abs.date_projected_start is not null OR abs.date_start is not null)
  AND    abs.absence_attendance_id = p_absence_attendance_id
  AND NOT EXISTS
        (SELECT null
         FROM   hr_api_transactions t
               ,hr_api_transaction_steps ts
         WHERE  t.selected_person_id = p_person_id
         AND    t.transaction_id     = p_transaction_id
         AND    t.transaction_id     = ts.transaction_id(+)
         AND    ts.transaction_step_id   = p_transaction_step_id
         AND    t.transaction_ref_table  = 'PER_ABSENCE_ATTENDANCES'
         AND    t.transaction_group      = 'ABSENCE_MGMT'
         AND    t.transaction_identifier = 'ABSENCES'
         AND    t.transaction_ref_id     = p_absence_attendance_id
         AND    t.status NOT IN ('D', 'N', 'W', 'ACTIVE', 'AC'))
  UNION ALL
  SELECT to_number(decode(xxas_xml_node_util_pkg.get_node_value(t.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', t.transaction_ref_id, 'UpdateMode', t.transaction_ref_id, 'CancelMode', t.transaction_ref_id, t.transaction_ref_id))  absence_attendance_id
        ,t.status status_code
        ,xxas_selfservice_workflow_pkg.decode_status
           (xxas_selfservice_workflow_pkg.transcode_status
              (t.status)) workflow_status
        ,t.item_type
        ,t.item_key
        ,(SELECT abt.name
          FROM   per_abs_attendance_types_vl abt
          WHERE  abt.absence_attendance_type_id =
                   xxas_util_pkg.get_step_val_num
                     (ts.transaction_step_id, 'INFORMATION5')) absence_type
        ,xxas_util_pkg.get_step_val
          (ts.transaction_step_id, 'INFORMATION6') absence_category
        ,(SELECT rea.meaning     
          FROM   per_abs_attendance_reasons paar
                ,hr_lookups                 rea
          WHERE  paar.abs_attendance_reason_id = xxas_xml_node_util_pkg.get_node_value(t.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
          AND    paar.name                     = rea.lookup_code
          AND    rea.lookup_type               = 'ABSENCE_REASON') absence_reason
        ,(SELECT hrl.meaning
          FROM   hr_lookups hrl
                ,per_abs_attendance_types_vl abt
          WHERE  hrl.lookup_type = l_ABS_UOM_LOOKUP
          AND    hrl.lookup_code = abt.hours_or_days
          AND    abt.absence_attendance_type_id =
                   xxas_util_pkg.get_step_val_num
                     (ts.transaction_step_id, 'INFORMATION5')) hours_or_days
        ,xxas_util_pkg.get_step_val_dt
          (ts.transaction_step_id, 'INFORMATION3') date_projected_start
        ,xxas_xml_node_util_pkg.get_node_value(t.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') time_projected_start
        ,xxas_util_pkg.get_step_val_dt
          (ts.transaction_step_id, 'INFORMATION4') date_projected_end
        ,xxas_xml_node_util_pkg.get_node_value(t.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') time_projected_end
        ,xxas_util_pkg.get_step_val_dt
          (ts.transaction_step_id, 'INFORMATION1') date_start
        ,xxas_xml_node_util_pkg.get_node_value(t.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') time_start
        ,xxas_util_pkg.get_step_val_dt
          (ts.transaction_step_id, 'INFORMATION2') date_end
        ,xxas_xml_node_util_pkg.get_node_value(t.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') time_end
        ,xxas_util_pkg.get_step_val_num
          (ts.transaction_step_id, 'INFORMATION8') absence_days
        ,xxas_util_pkg.get_step_val_num
          (ts.transaction_step_id, 'INFORMATION7') absence_hours
        ,null object_version_number
        ,t.transaction_id
        ,ts.transaction_step_id
        ,papf.person_id
        ,xxas_util_pkg.get_formatted_name
           (p_full_name   => papf.full_name
           ,p_global_name => papf.global_name
           ,p_local_name  => papf.local_name
           ,p_first_name  => papf.first_name
           ,p_last_name   => papf.last_name
           ,p_known_as    => papf.known_as) formatted_name
        ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
        ,xxas_util_pkg.get_primary_worker_asg(papf.person_id,p_effective_date) assignment_id
        ,(CASE WHEN xxas_xml_node_util_pkg.get_node_value(t.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode') = 'CancelMode'
               THEN 'Y'
               ELSE 'N'
          END) deleted_flag
  FROM   hr_api_transactions t
        ,hr_api_transaction_steps ts
        ,per_all_people_f papf
  WHERE  t.selected_person_id = p_person_id
  AND    t.selected_person_id = papf.person_id
  AND    p_effective_date BETWEEN papf.effective_start_date 
                          AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y')
  AND    t.transaction_id = ts.transaction_id(+)
  AND    t.transaction_ref_table = 'PER_ABSENCE_ATTENDANCES'
  AND    t.transaction_group = 'ABSENCE_MGMT'
  AND    t.transaction_identifier = 'ABSENCES'
  AND    t.transaction_ref_id is not null
  AND    t.status NOT IN ('D', 'N', 'W', 'ACTIVE', 'AC')
  AND    (xxas_util_pkg.get_step_val_dt(ts.transaction_step_id, 'INFORMATION1') is not null OR
          xxas_util_pkg.get_step_val_dt(ts.transaction_step_id, 'INFORMATION3') is not null)
  AND    t.transaction_ref_id   = p_absence_attendance_id
  AND    t.transaction_id       = p_transaction_id
  AND    ts.transaction_step_id = p_transaction_step_id);

  c_method CONSTANT  varchar2(30) := 'store_item_into_event_tbl';
  l_absence_rec      csr_absences%ROWTYPE;
  l_row_counter      number := 1;
  l_del_item_counter number := 1;

BEGIN

  log_start(c_method,p_person_id,p_absence_attendance_id,p_transaction_id,p_transaction_step_id,p_effective_date,p_status,p_action,p_approvers);

  IF p_person_id             is not null AND 
     p_absence_attendance_id is not null AND 
     p_transaction_id        is not null AND 
     p_transaction_step_id   is not null AND 
     p_effective_date        is not null AND 
     p_action                is not null AND 
     p_status                is not null AND 
     p_approvers             is not null THEN

    OPEN  csr_absences;
    FETCH csr_absences INTO l_absence_rec;
    CLOSE csr_absences;

    --
    -- For Add Absence Action for both approvers/no-approvers status(W/RI) - Add the item
    --
    IF p_action = g_INSERT THEN

      --
      -- Add this event into the g_event_tbl.
      --
      IF g_event_tbl.count > 0 THEN
        l_row_counter := g_event_tbl.last + 1;
      END IF;

      g_event_tbl(l_row_counter).row_id                 := null;
      g_event_tbl(l_row_counter).person_id              := l_absence_rec.person_id;
      g_event_tbl(l_row_counter).assignment_id          := l_absence_rec.assignment_id;
      g_event_tbl(l_row_counter).formatted_name         := l_absence_rec.formatted_name;
      g_event_tbl(l_row_counter).initials               := l_absence_rec.initials;
      g_event_tbl(l_row_counter).type                   := g_ABS_EVENT;
      g_event_tbl(l_row_counter).code                   := g_ABS_EVENT_CD;
      g_event_tbl(l_row_counter).tooltip_line1          := null;
      g_event_tbl(l_row_counter).tooltip_line2          := null;
      g_event_tbl(l_row_counter).start_date             := l_absence_rec.date_start;
      g_event_tbl(l_row_counter).end_date               := l_absence_rec.date_end;
      g_event_tbl(l_row_counter).start_time             := l_absence_rec.time_start;
      g_event_tbl(l_row_counter).end_time               := l_absence_rec.time_end;
      g_event_tbl(l_row_counter).proj_start_date        := l_absence_rec.date_projected_start;
      g_event_tbl(l_row_counter).proj_end_date          := l_absence_rec.date_projected_end;
      g_event_tbl(l_row_counter).proj_start_time        := l_absence_rec.time_projected_start;
      g_event_tbl(l_row_counter).proj_end_time          := l_absence_rec.time_projected_end;
      g_event_tbl(l_row_counter).title                  := null;
      g_event_tbl(l_row_counter).sch_additional_info    := null;
      g_event_tbl(l_row_counter).icon                   := null;
      g_event_tbl(l_row_counter).color                  := null;
      g_event_tbl(l_row_counter).border_color           := null;
      g_event_tbl(l_row_counter).text_color             := null;
      g_event_tbl(l_row_counter).bg_color               := null;
      g_event_tbl(l_row_counter).country                := null;
      g_event_tbl(l_row_counter).page_id                := null;
      g_event_tbl(l_row_counter).object_id              := null;
      g_event_tbl(l_row_counter).target                 := null;
      g_event_tbl(l_row_counter).absence_type           := l_absence_rec.absence_type;
      g_event_tbl(l_row_counter).absence_category       := l_absence_rec.absence_category;
      g_event_tbl(l_row_counter).absence_reason         := l_absence_rec.absence_reason;
      g_event_tbl(l_row_counter).payment_type           := null;
      g_event_tbl(l_row_counter).period_name            := null;
      g_event_tbl(l_row_counter).status                 := l_absence_rec.workflow_status;
      g_event_tbl(l_row_counter).status_code            := l_absence_rec.status_code;
      g_event_tbl(l_row_counter).editable               := null;
      g_event_tbl(l_row_counter).public_holiday_name    := null;
      g_event_tbl(l_row_counter).payroll_name           := null;
      g_event_tbl(l_row_counter).birthday_name          := null;
      g_event_tbl(l_row_counter).absence_days           := l_absence_rec.absence_days;
      g_event_tbl(l_row_counter).absence_hours          := l_absence_rec.absence_hours;
      g_event_tbl(l_row_counter).hours_or_days          := l_absence_rec.hours_or_days;
      g_event_tbl(l_row_counter).course_name            := null;
      g_event_tbl(l_row_counter).class_name             := null;
      g_event_tbl(l_row_counter).training_event_id      := null;
      g_event_tbl(l_row_counter).session_or_scheduled   := null;
      g_event_tbl(l_row_counter).delivery_mode          := null;
      g_event_tbl(l_row_counter).training_duration      := null;
      g_event_tbl(l_row_counter).training_dur_units     := null;
      g_event_tbl(l_row_counter).item_type              := l_absence_rec.item_type;
      g_event_tbl(l_row_counter).item_key               := l_absence_rec.item_key;
      g_event_tbl(l_row_counter).absence_id             := l_absence_rec.absence_attendance_id;
      g_event_tbl(l_row_counter).transaction_id         := l_absence_rec.transaction_id;
      g_event_tbl(l_row_counter).transaction_step_id    := l_absence_rec.transaction_step_id;
      g_event_tbl(l_row_counter).show_event             := null;
      g_event_tbl(l_row_counter).common_events_res_name := null;
      g_event_tbl(l_row_counter).log_entry_id           := null;
      g_event_tbl(l_row_counter).duration_hours         := null;
      g_event_tbl(l_row_counter).tcm_field1             := null;
      g_event_tbl(l_row_counter).tcm_field2             := null;
      g_event_tbl(l_row_counter).tcm_field3             := null;
      g_event_tbl(l_row_counter).tcm_field4             := null;
      g_event_tbl(l_row_counter).tcm_field5             := null;
      g_event_tbl(l_row_counter).current_approver_role  := null;
      g_event_tbl(l_row_counter).override_approver_role := null;
      g_event_tbl(l_row_counter).cost_concat_segments   := null;
      g_event_tbl(l_row_counter).deleted_flag           := l_absence_rec.deleted_flag;
      g_event_tbl(l_row_counter).summation_group        := null;
      g_event_tbl(l_row_counter).summation_field        := null;
      g_event_tbl(l_row_counter).payment_type_id        := null;
      g_event_tbl(l_row_counter).otl_entry_type           := null;
      g_event_tbl(l_row_counter).otl_entry_transferred_to := null;
      g_event_tbl(l_row_counter).otl_timecard_id          := null;
      g_event_tbl(l_row_counter).otl_hourstype_id         := null;
      g_event_tbl(l_row_counter).otl_absence_days         := null;
      g_event_tbl(l_row_counter).otl_absence_hours        := null;
      g_event_tbl(l_row_counter).otl_timecard_ovn         := null;

    --
    -- For Update Absence Action
    --
    ELSIF p_action = g_UPDATE THEN

      --
      -- If Approvers are present or not present then same way events should add/delete.
      --
      IF p_status = 'W' THEN

        --
        -- Add this event into the g_del_items_tbl to delete the existing event as rowid is changing
        --
        IF g_del_items_tbl.count > 0 THEN
          l_del_item_counter := g_del_items_tbl.last + 1;
        END IF;

        g_del_items_tbl(l_del_item_counter).person_id              := p_person_id;
        g_del_items_tbl(l_del_item_counter).absence_attendance_id  := p_absence_attendance_id;

        --
        -- Add this event into the g_event_tbl to add a new event because of new rowid
        --
        IF g_event_tbl.count > 0 THEN
          l_row_counter := g_event_tbl.last + 1;
        END IF;

        g_event_tbl(l_row_counter).row_id                 := null;
        g_event_tbl(l_row_counter).person_id              := l_absence_rec.person_id;
        g_event_tbl(l_row_counter).assignment_id          := l_absence_rec.assignment_id;
        g_event_tbl(l_row_counter).formatted_name         := l_absence_rec.formatted_name;
        g_event_tbl(l_row_counter).initials               := l_absence_rec.initials;
        g_event_tbl(l_row_counter).type                   := g_ABS_EVENT;
        g_event_tbl(l_row_counter).code                   := g_ABS_EVENT_CD;
        g_event_tbl(l_row_counter).tooltip_line1          := null;
        g_event_tbl(l_row_counter).tooltip_line2          := null;
        g_event_tbl(l_row_counter).start_date             := l_absence_rec.date_start;
        g_event_tbl(l_row_counter).end_date               := l_absence_rec.date_end;
        g_event_tbl(l_row_counter).start_time             := l_absence_rec.time_start;
        g_event_tbl(l_row_counter).end_time               := l_absence_rec.time_end;
        g_event_tbl(l_row_counter).proj_start_date        := l_absence_rec.date_projected_start;
        g_event_tbl(l_row_counter).proj_end_date          := l_absence_rec.date_projected_end;
        g_event_tbl(l_row_counter).proj_start_time        := l_absence_rec.time_projected_start;
        g_event_tbl(l_row_counter).proj_end_time          := l_absence_rec.time_projected_end;
        g_event_tbl(l_row_counter).title                  := null;
        g_event_tbl(l_row_counter).sch_additional_info    := null;
        g_event_tbl(l_row_counter).icon                   := null;
        g_event_tbl(l_row_counter).color                  := null;
        g_event_tbl(l_row_counter).border_color           := null;
        g_event_tbl(l_row_counter).text_color             := null;
        g_event_tbl(l_row_counter).bg_color               := null;
        g_event_tbl(l_row_counter).country                := null;
        g_event_tbl(l_row_counter).page_id                := null;
        g_event_tbl(l_row_counter).object_id              := null;
        g_event_tbl(l_row_counter).target                 := null;
        g_event_tbl(l_row_counter).absence_type           := l_absence_rec.absence_type;
        g_event_tbl(l_row_counter).absence_category       := l_absence_rec.absence_category;
        g_event_tbl(l_row_counter).absence_reason         := l_absence_rec.absence_reason;
        g_event_tbl(l_row_counter).payment_type           := null;
        g_event_tbl(l_row_counter).period_name            := null;
        g_event_tbl(l_row_counter).status                 := l_absence_rec.workflow_status;
        g_event_tbl(l_row_counter).status_code            := l_absence_rec.status_code;
        g_event_tbl(l_row_counter).editable               := null;
        g_event_tbl(l_row_counter).public_holiday_name    := null;
        g_event_tbl(l_row_counter).payroll_name           := null;
        g_event_tbl(l_row_counter).birthday_name          := null;
        g_event_tbl(l_row_counter).absence_days           := l_absence_rec.absence_days;
        g_event_tbl(l_row_counter).absence_hours          := l_absence_rec.absence_hours;
        g_event_tbl(l_row_counter).hours_or_days          := l_absence_rec.hours_or_days;
        g_event_tbl(l_row_counter).course_name            := null;
        g_event_tbl(l_row_counter).class_name             := null;
        g_event_tbl(l_row_counter).training_event_id      := null;
        g_event_tbl(l_row_counter).session_or_scheduled   := null;
        g_event_tbl(l_row_counter).delivery_mode          := null;
        g_event_tbl(l_row_counter).training_duration      := null;
        g_event_tbl(l_row_counter).training_dur_units     := null;
        g_event_tbl(l_row_counter).item_type              := l_absence_rec.item_type;
        g_event_tbl(l_row_counter).item_key               := l_absence_rec.item_key;
        g_event_tbl(l_row_counter).absence_id             := l_absence_rec.absence_attendance_id;
        g_event_tbl(l_row_counter).transaction_id         := l_absence_rec.transaction_id;
        g_event_tbl(l_row_counter).transaction_step_id    := l_absence_rec.transaction_step_id;
        g_event_tbl(l_row_counter).show_event             := null;
        g_event_tbl(l_row_counter).common_events_res_name := null;
        g_event_tbl(l_row_counter).log_entry_id           := null;
        g_event_tbl(l_row_counter).duration_hours         := null;
        g_event_tbl(l_row_counter).tcm_field1             := null;
        g_event_tbl(l_row_counter).tcm_field2             := null;
        g_event_tbl(l_row_counter).tcm_field3             := null;
        g_event_tbl(l_row_counter).tcm_field4             := null;
        g_event_tbl(l_row_counter).tcm_field5             := null;
        g_event_tbl(l_row_counter).current_approver_role  := null;
        g_event_tbl(l_row_counter).override_approver_role := null;
        g_event_tbl(l_row_counter).cost_concat_segments   := null;
        g_event_tbl(l_row_counter).deleted_flag           := l_absence_rec.deleted_flag;
        g_event_tbl(l_row_counter).summation_group        := null;
        g_event_tbl(l_row_counter).summation_field        := null;
        g_event_tbl(l_row_counter).payment_type_id        := null;
        g_event_tbl(l_row_counter).otl_entry_type           := null;
        g_event_tbl(l_row_counter).otl_entry_transferred_to := null;
        g_event_tbl(l_row_counter).otl_timecard_id          := null;
        g_event_tbl(l_row_counter).otl_hourstype_id         := null;
        g_event_tbl(l_row_counter).otl_absence_days         := null;
        g_event_tbl(l_row_counter).otl_absence_hours        := null;
        g_event_tbl(l_row_counter).otl_timecard_ovn         := null;

      ELSIF p_status = 'RI' THEN

        --
        -- Add this event into the g_event_tbl to add a new event because no need to delete existing event.
        --
        IF g_event_tbl.count > 0 THEN
          l_row_counter := g_event_tbl.last + 1;
        END IF;

        g_event_tbl(l_row_counter).row_id                 := null;
        g_event_tbl(l_row_counter).person_id              := l_absence_rec.person_id;
        g_event_tbl(l_row_counter).assignment_id          := l_absence_rec.assignment_id;
        g_event_tbl(l_row_counter).formatted_name         := l_absence_rec.formatted_name;
        g_event_tbl(l_row_counter).initials               := l_absence_rec.initials;
        g_event_tbl(l_row_counter).type                   := g_ABS_EVENT;
        g_event_tbl(l_row_counter).code                   := g_ABS_EVENT_CD;
        g_event_tbl(l_row_counter).tooltip_line1          := null;
        g_event_tbl(l_row_counter).tooltip_line2          := null;
        g_event_tbl(l_row_counter).start_date             := l_absence_rec.date_start;
        g_event_tbl(l_row_counter).end_date               := l_absence_rec.date_end;
        g_event_tbl(l_row_counter).start_time             := l_absence_rec.time_start;
        g_event_tbl(l_row_counter).end_time               := l_absence_rec.time_end;
        g_event_tbl(l_row_counter).proj_start_date        := l_absence_rec.date_projected_start;
        g_event_tbl(l_row_counter).proj_end_date          := l_absence_rec.date_projected_end;
        g_event_tbl(l_row_counter).proj_start_time        := l_absence_rec.time_projected_start;
        g_event_tbl(l_row_counter).proj_end_time          := l_absence_rec.time_projected_end;
        g_event_tbl(l_row_counter).title                  := null;
        g_event_tbl(l_row_counter).sch_additional_info    := null;
        g_event_tbl(l_row_counter).icon                   := null;
        g_event_tbl(l_row_counter).color                  := null;
        g_event_tbl(l_row_counter).border_color           := null;
        g_event_tbl(l_row_counter).text_color             := null;
        g_event_tbl(l_row_counter).bg_color               := null;
        g_event_tbl(l_row_counter).country                := null;
        g_event_tbl(l_row_counter).page_id                := null;
        g_event_tbl(l_row_counter).object_id              := null;
        g_event_tbl(l_row_counter).target                 := null;
        g_event_tbl(l_row_counter).absence_type           := l_absence_rec.absence_type;
        g_event_tbl(l_row_counter).absence_category       := l_absence_rec.absence_category;
        g_event_tbl(l_row_counter).absence_reason         := l_absence_rec.absence_reason;
        g_event_tbl(l_row_counter).payment_type           := null;
        g_event_tbl(l_row_counter).period_name            := null;
        g_event_tbl(l_row_counter).status                 := l_absence_rec.workflow_status;
        g_event_tbl(l_row_counter).status_code            := l_absence_rec.status_code;
        g_event_tbl(l_row_counter).editable               := null;
        g_event_tbl(l_row_counter).public_holiday_name    := null;
        g_event_tbl(l_row_counter).payroll_name           := null;
        g_event_tbl(l_row_counter).birthday_name          := null;
        g_event_tbl(l_row_counter).absence_days           := l_absence_rec.absence_days;
        g_event_tbl(l_row_counter).absence_hours          := l_absence_rec.absence_hours;
        g_event_tbl(l_row_counter).hours_or_days          := l_absence_rec.hours_or_days;
        g_event_tbl(l_row_counter).course_name            := null;
        g_event_tbl(l_row_counter).class_name             := null;
        g_event_tbl(l_row_counter).training_event_id      := null;
        g_event_tbl(l_row_counter).session_or_scheduled   := null;
        g_event_tbl(l_row_counter).delivery_mode          := null;
        g_event_tbl(l_row_counter).training_duration      := null;
        g_event_tbl(l_row_counter).training_dur_units     := null;
        g_event_tbl(l_row_counter).item_type              := l_absence_rec.item_type;
        g_event_tbl(l_row_counter).item_key               := l_absence_rec.item_key;
        g_event_tbl(l_row_counter).absence_id             := l_absence_rec.absence_attendance_id;
        g_event_tbl(l_row_counter).transaction_id         := l_absence_rec.transaction_id;
        g_event_tbl(l_row_counter).transaction_step_id    := l_absence_rec.transaction_step_id;
        g_event_tbl(l_row_counter).show_event             := null;
        g_event_tbl(l_row_counter).common_events_res_name := null;
        g_event_tbl(l_row_counter).log_entry_id           := null;
        g_event_tbl(l_row_counter).duration_hours         := null;
        g_event_tbl(l_row_counter).tcm_field1             := null;
        g_event_tbl(l_row_counter).tcm_field2             := null;
        g_event_tbl(l_row_counter).tcm_field3             := null;
        g_event_tbl(l_row_counter).tcm_field4             := null;
        g_event_tbl(l_row_counter).tcm_field5             := null;
        g_event_tbl(l_row_counter).current_approver_role  := null;
        g_event_tbl(l_row_counter).override_approver_role := null;
        g_event_tbl(l_row_counter).cost_concat_segments   := null;
        g_event_tbl(l_row_counter).deleted_flag           := l_absence_rec.deleted_flag;
        g_event_tbl(l_row_counter).summation_group        := null;
        g_event_tbl(l_row_counter).summation_field        := null;
        g_event_tbl(l_row_counter).payment_type_id        := null;
        g_event_tbl(l_row_counter).otl_entry_type           := null;
        g_event_tbl(l_row_counter).otl_entry_transferred_to := null;
        g_event_tbl(l_row_counter).otl_timecard_id          := null;
        g_event_tbl(l_row_counter).otl_hourstype_id         := null;
        g_event_tbl(l_row_counter).otl_absence_days         := null;
        g_event_tbl(l_row_counter).otl_absence_hours        := null;
        g_event_tbl(l_row_counter).otl_timecard_ovn         := null;

      END IF;

    ELSIF p_action = g_DELETE THEN

      --
      -- If Approvers are present
      --
      IF p_approvers = 'Y' THEN

        IF p_status = 'W' THEN

          --
          -- Add this event into the g_del_items_tbl to delete the existing event as rowid is changing
          --
          IF g_del_items_tbl.count > 0 THEN
            l_del_item_counter := g_del_items_tbl.last + 1;
          END IF;

          g_del_items_tbl(l_del_item_counter).person_id              := p_person_id;
          g_del_items_tbl(l_del_item_counter).absence_attendance_id  := p_absence_attendance_id;

          --
          -- Add this event into the g_event_tbl to add a new event because of new rowid
          --
          IF g_event_tbl.count > 0 THEN
            l_row_counter := g_event_tbl.last + 1;
          END IF;

          g_event_tbl(l_row_counter).row_id                 := null;
          g_event_tbl(l_row_counter).person_id              := l_absence_rec.person_id;
          g_event_tbl(l_row_counter).assignment_id          := l_absence_rec.assignment_id;
          g_event_tbl(l_row_counter).formatted_name         := l_absence_rec.formatted_name;
          g_event_tbl(l_row_counter).initials               := l_absence_rec.initials;
          g_event_tbl(l_row_counter).type                   := g_ABS_EVENT;
          g_event_tbl(l_row_counter).code                   := g_ABS_EVENT_CD;
          g_event_tbl(l_row_counter).tooltip_line1          := null;
          g_event_tbl(l_row_counter).tooltip_line2          := null;
          g_event_tbl(l_row_counter).start_date             := l_absence_rec.date_start;
          g_event_tbl(l_row_counter).end_date               := l_absence_rec.date_end;
          g_event_tbl(l_row_counter).start_time             := l_absence_rec.time_start;
          g_event_tbl(l_row_counter).end_time               := l_absence_rec.time_end;
          g_event_tbl(l_row_counter).proj_start_date        := l_absence_rec.date_projected_start;
          g_event_tbl(l_row_counter).proj_end_date          := l_absence_rec.date_projected_end;
          g_event_tbl(l_row_counter).proj_start_time        := l_absence_rec.time_projected_start;
          g_event_tbl(l_row_counter).proj_end_time          := l_absence_rec.time_projected_end;
          g_event_tbl(l_row_counter).title                  := null;
          g_event_tbl(l_row_counter).sch_additional_info    := null;
          g_event_tbl(l_row_counter).icon                   := null;
          g_event_tbl(l_row_counter).color                  := null;
          g_event_tbl(l_row_counter).border_color           := null;
          g_event_tbl(l_row_counter).text_color             := null;
          g_event_tbl(l_row_counter).bg_color               := null;
          g_event_tbl(l_row_counter).country                := null;
          g_event_tbl(l_row_counter).page_id                := null;
          g_event_tbl(l_row_counter).object_id              := null;
          g_event_tbl(l_row_counter).target                 := null;
          g_event_tbl(l_row_counter).absence_type           := l_absence_rec.absence_type;
          g_event_tbl(l_row_counter).absence_category       := l_absence_rec.absence_category;
          g_event_tbl(l_row_counter).absence_reason         := l_absence_rec.absence_reason;
          g_event_tbl(l_row_counter).payment_type           := null;
          g_event_tbl(l_row_counter).period_name            := null;
          g_event_tbl(l_row_counter).status                 := l_absence_rec.workflow_status;
          g_event_tbl(l_row_counter).status_code            := l_absence_rec.status_code;
          g_event_tbl(l_row_counter).editable               := null;
          g_event_tbl(l_row_counter).public_holiday_name    := null;
          g_event_tbl(l_row_counter).payroll_name           := null;
          g_event_tbl(l_row_counter).birthday_name          := null;
          g_event_tbl(l_row_counter).absence_days           := l_absence_rec.absence_days;
          g_event_tbl(l_row_counter).absence_hours          := l_absence_rec.absence_hours;
          g_event_tbl(l_row_counter).hours_or_days          := l_absence_rec.hours_or_days;
          g_event_tbl(l_row_counter).course_name            := null;
          g_event_tbl(l_row_counter).class_name             := null;
          g_event_tbl(l_row_counter).training_event_id      := null;
          g_event_tbl(l_row_counter).session_or_scheduled   := null;
          g_event_tbl(l_row_counter).delivery_mode          := null;
          g_event_tbl(l_row_counter).training_duration      := null;
          g_event_tbl(l_row_counter).training_dur_units     := null;
          g_event_tbl(l_row_counter).item_type              := l_absence_rec.item_type;
          g_event_tbl(l_row_counter).item_key               := l_absence_rec.item_key;
          g_event_tbl(l_row_counter).absence_id             := l_absence_rec.absence_attendance_id;
          g_event_tbl(l_row_counter).transaction_id         := l_absence_rec.transaction_id;
          g_event_tbl(l_row_counter).transaction_step_id    := l_absence_rec.transaction_step_id;
          g_event_tbl(l_row_counter).show_event             := null;
          g_event_tbl(l_row_counter).common_events_res_name := null;
          g_event_tbl(l_row_counter).log_entry_id           := null;
          g_event_tbl(l_row_counter).duration_hours         := null;
          g_event_tbl(l_row_counter).tcm_field1             := null;
          g_event_tbl(l_row_counter).tcm_field2             := null;
          g_event_tbl(l_row_counter).tcm_field3             := null;
          g_event_tbl(l_row_counter).tcm_field4             := null;
          g_event_tbl(l_row_counter).tcm_field5             := null;
          g_event_tbl(l_row_counter).current_approver_role  := null;
          g_event_tbl(l_row_counter).override_approver_role := null;
          g_event_tbl(l_row_counter).cost_concat_segments   := null;
          g_event_tbl(l_row_counter).deleted_flag           := l_absence_rec.deleted_flag;
          g_event_tbl(l_row_counter).summation_group        := null;
          g_event_tbl(l_row_counter).summation_field        := null;
          g_event_tbl(l_row_counter).payment_type_id        := null;
          g_event_tbl(l_row_counter).otl_entry_type           := null;
          g_event_tbl(l_row_counter).otl_entry_transferred_to := null;
          g_event_tbl(l_row_counter).otl_timecard_id          := null;
          g_event_tbl(l_row_counter).otl_hourstype_id         := null;
          g_event_tbl(l_row_counter).otl_absence_days         := null;
          g_event_tbl(l_row_counter).otl_absence_hours        := null;
          g_event_tbl(l_row_counter).otl_timecard_ovn         := null;

        ELSIF p_status = 'RI' THEN

          --
          -- Add this event into the g_event_tbl to add a new event because no need to delete existing event.
          --
          IF g_event_tbl.count > 0 THEN
            l_row_counter := g_event_tbl.last + 1;
          END IF;

          g_event_tbl(l_row_counter).row_id                 := null;
          g_event_tbl(l_row_counter).person_id              := l_absence_rec.person_id;
          g_event_tbl(l_row_counter).assignment_id          := l_absence_rec.assignment_id;
          g_event_tbl(l_row_counter).formatted_name         := l_absence_rec.formatted_name;
          g_event_tbl(l_row_counter).initials               := l_absence_rec.initials;
          g_event_tbl(l_row_counter).type                   := g_ABS_EVENT;
          g_event_tbl(l_row_counter).code                   := g_ABS_EVENT_CD;
          g_event_tbl(l_row_counter).tooltip_line1          := null;
          g_event_tbl(l_row_counter).tooltip_line2          := null;
          g_event_tbl(l_row_counter).start_date             := l_absence_rec.date_start;
          g_event_tbl(l_row_counter).end_date               := l_absence_rec.date_end;
          g_event_tbl(l_row_counter).start_time             := l_absence_rec.time_start;
          g_event_tbl(l_row_counter).end_time               := l_absence_rec.time_end;
          g_event_tbl(l_row_counter).proj_start_date        := l_absence_rec.date_projected_start;
          g_event_tbl(l_row_counter).proj_end_date          := l_absence_rec.date_projected_end;
          g_event_tbl(l_row_counter).proj_start_time        := l_absence_rec.time_projected_start;
          g_event_tbl(l_row_counter).proj_end_time          := l_absence_rec.time_projected_end;
          g_event_tbl(l_row_counter).title                  := null;
          g_event_tbl(l_row_counter).sch_additional_info    := null;
          g_event_tbl(l_row_counter).icon                   := null;
          g_event_tbl(l_row_counter).color                  := null;
          g_event_tbl(l_row_counter).border_color           := null;
          g_event_tbl(l_row_counter).text_color             := null;
          g_event_tbl(l_row_counter).bg_color               := null;
          g_event_tbl(l_row_counter).country                := null;
          g_event_tbl(l_row_counter).page_id                := null;
          g_event_tbl(l_row_counter).object_id              := null;
          g_event_tbl(l_row_counter).target                 := null;
          g_event_tbl(l_row_counter).absence_type           := l_absence_rec.absence_type;
          g_event_tbl(l_row_counter).absence_category       := l_absence_rec.absence_category;
          g_event_tbl(l_row_counter).absence_reason         := l_absence_rec.absence_reason;
          g_event_tbl(l_row_counter).payment_type           := null;
          g_event_tbl(l_row_counter).period_name            := null;
          g_event_tbl(l_row_counter).status                 := l_absence_rec.workflow_status;
          g_event_tbl(l_row_counter).status_code            := l_absence_rec.status_code;
          g_event_tbl(l_row_counter).editable               := null;
          g_event_tbl(l_row_counter).public_holiday_name    := null;
          g_event_tbl(l_row_counter).payroll_name           := null;
          g_event_tbl(l_row_counter).birthday_name          := null;
          g_event_tbl(l_row_counter).absence_days           := l_absence_rec.absence_days;
          g_event_tbl(l_row_counter).absence_hours          := l_absence_rec.absence_hours;
          g_event_tbl(l_row_counter).hours_or_days          := l_absence_rec.hours_or_days;
          g_event_tbl(l_row_counter).course_name            := null;
          g_event_tbl(l_row_counter).class_name             := null;
          g_event_tbl(l_row_counter).training_event_id      := null;
          g_event_tbl(l_row_counter).session_or_scheduled   := null;
          g_event_tbl(l_row_counter).delivery_mode          := null;
          g_event_tbl(l_row_counter).training_duration      := null;
          g_event_tbl(l_row_counter).training_dur_units     := null;
          g_event_tbl(l_row_counter).item_type              := l_absence_rec.item_type;
          g_event_tbl(l_row_counter).item_key               := l_absence_rec.item_key;
          g_event_tbl(l_row_counter).absence_id             := l_absence_rec.absence_attendance_id;
          g_event_tbl(l_row_counter).transaction_id         := l_absence_rec.transaction_id;
          g_event_tbl(l_row_counter).transaction_step_id    := l_absence_rec.transaction_step_id;
          g_event_tbl(l_row_counter).show_event             := null;
          g_event_tbl(l_row_counter).common_events_res_name := null;
          g_event_tbl(l_row_counter).log_entry_id           := null;
          g_event_tbl(l_row_counter).duration_hours         := null;
          g_event_tbl(l_row_counter).tcm_field1             := null;
          g_event_tbl(l_row_counter).tcm_field2             := null;
          g_event_tbl(l_row_counter).tcm_field3             := null;
          g_event_tbl(l_row_counter).tcm_field4             := null;
          g_event_tbl(l_row_counter).tcm_field5             := null;
          g_event_tbl(l_row_counter).current_approver_role  := null;
          g_event_tbl(l_row_counter).override_approver_role := null;
          g_event_tbl(l_row_counter).cost_concat_segments   := null;
          g_event_tbl(l_row_counter).deleted_flag           := l_absence_rec.deleted_flag;
          g_event_tbl(l_row_counter).summation_group        := null;
          g_event_tbl(l_row_counter).summation_field        := null;
          g_event_tbl(l_row_counter).payment_type_id        := null;
          g_event_tbl(l_row_counter).otl_entry_type           := null;
          g_event_tbl(l_row_counter).otl_entry_transferred_to := null;
          g_event_tbl(l_row_counter).otl_timecard_id          := null;
          g_event_tbl(l_row_counter).otl_hourstype_id         := null;
          g_event_tbl(l_row_counter).otl_absence_days         := null;
          g_event_tbl(l_row_counter).otl_absence_hours        := null;
          g_event_tbl(l_row_counter).otl_timecard_ovn         := null;

        END IF;

      --
      -- If Approvers are not present
      --
      ELSE

        --
        -- Add this event into the g_del_items_tbl to delete the existing event.
        --
        IF g_del_items_tbl.count > 0 THEN
          l_del_item_counter := g_del_items_tbl.last + 1;
        END IF;

        g_del_items_tbl(l_del_item_counter).person_id              := p_person_id;
        g_del_items_tbl(l_del_item_counter).absence_attendance_id  := p_absence_attendance_id;

      END IF;

    END IF;

  END IF;

  log_end(c_method);

END store_item_into_event_tbl;

--
-- ------------------------------------------------------------------------ *
-- Name    : launch_workflow
-- ------------------------------------------------------------------------ *
PROCEDURE launch_workflow 
  (p_person_id              IN  number
  ,p_transaction_id         IN  number
  ,p_comments_to_approver   IN  varchar2
  ,p_called_from            IN  varchar2
  ,p_ame_item_type          IN  varchar2
  ,p_ame_app_id             IN  number
  ,p_ame_trans_type         IN  varchar2
  ,p_ame_process_name       IN  varchar2
  ,p_item_key       OUT NOCOPY  varchar2
  ,p_activity_id    OUT NOCOPY  varchar2)
IS

  c_method CONSTANT     varchar2(30) := 'launch_workflow';
  l_current_per_display varchar2(1000);
  
  CURSOR csr_person_name IS
  SELECT papf.full_name
  FROM   per_all_people_f papf
  WHERE  p_person_id is not null
  AND    papf.person_id = p_person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;
  
BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  OPEN  csr_person_name;
  FETCH csr_person_name INTO l_current_per_display;
  CLOSE csr_person_name;
  
  xxas_selfservice_workflow_pkg.hr_generic_approval_prc_launch
    (p_transaction_id              => p_transaction_id
    ,p_item_type                   => p_ame_item_type
    ,p_process_name                => p_ame_process_name
    ,p_process_display_name        => xxas_com_msg_pkg.get_string
                                        ('PER|' || xxas_com_sec_pkg.get_function_parameter
                                                     (p_called_from, 'pNtfSubMsg'))
    ,p_ame_application_id          => p_ame_app_id
    ,p_ame_transaction_type        => p_ame_trans_type
    ,p_review_template_rn          => 'HR_ABS_NTF_SS'
    ,p_ntf_sub_fnd_msg             => xxas_com_sec_pkg.get_function_parameter
                                        (p_called_from, 'pNtfSubMsg') --'HR_ABS_NTF_SUB_MSG'
    ,p_current_person_display_name => l_current_per_display
    ,p_comments_to_approver        => p_comments_to_approver
    ,p_notif_fyi_details          =>  xxas_com_sec_pkg.get_function_parameter
                                        (p_called_from, 'pNtfFyiDetails')
    ,p_current_person_id           => p_person_id
    ,p_current_person_orig_sys     => g_PER
    ,x_item_key                    => p_item_key
    ,x_activity_id                 => p_activity_id);
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

END launch_workflow;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_hrapi_transaction
-- ------------------------------------------------------------------------ *
FUNCTION create_hrapi_transaction 
  (p_transaction_id        IN   number
  ,p_absence_attendance_id IN   number
  ,p_assignment_id         IN   number
  ,p_person_id             IN   number
  ,p_item_type             IN   varchar2
  ,p_item_key              IN   varchar2
  ,p_absence_trans_doc     IN   XMLType
  ,p_information1          IN   varchar2
  ,p_information2          IN   varchar2
  ,p_information3          IN   varchar2
  ,p_information4          IN   varchar2
  ,p_information5          IN   varchar2
  ,p_information6          IN   varchar2
  ,p_information7          IN   varchar2
  ,p_information8          IN   varchar2
  ,p_information9          IN   varchar2
  ,p_object_state          IN   varchar2
  ,p_called_from           IN   varchar2
  ,p_ame_process_name      IN   varchar2
  ,p_transaction_status    IN   varchar2) RETURN number
IS

  c_method CONSTANT     varchar2(30) := 'create_hrapi_transaction';
  l_transaction_id      number;
  l_transaction_step_id number;
  l_step_ovn            number;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  hr_trn_ins.set_base_key_value
   (p_transaction_id => p_transaction_id);
  
  hr_trn_ins.ins
   (p_transaction_id               => l_transaction_id
   ,p_creator_person_id            => fnd_global.employee_id
   ,p_transaction_privilege        => 'PRIVATE'
   ,p_product_code                 => g_PER
   ,p_url                          => null
   ,p_status                       => p_transaction_status --'W'
   ,p_section_display_name         => null
   ,p_function_id                  => xxas_com_sec_pkg.get_function_id(p_called_from)
   ,p_transaction_ref_table        => 'PER_ABSENCE_ATTENDANCES'
   ,p_transaction_ref_id           => p_absence_attendance_id
   ,p_transaction_type             => 'WF'
   ,p_assignment_id                => p_assignment_id
   ,p_api_addtnl_info              => null
   ,p_selected_person_id           => p_person_id
   ,p_item_type                    => p_item_type
   ,p_item_key                     => p_item_key
   ,p_transaction_effective_date   => trunc(sysdate)
   ,p_process_name                 => p_ame_process_name
   ,p_plan_id                      => null
   ,p_rptg_grp_id                  => null
   ,p_effective_date_option        => null
   ,p_validate                     => false
   ,p_creator_role                 => 'PER:' || fnd_global.employee_id
   ,p_last_update_role             => 'PER:' || fnd_global.employee_id
   ,p_parent_transaction_id        => null
   ,p_relaunch_function            => 'HR_ABS_ENTRY_PAGE_SS'
   ,p_transaction_group            => 'ABSENCE_MGMT'
   ,p_transaction_identifier       => 'ABSENCES'
   ,p_transaction_document         => p_absence_trans_doc.getCLOBVal());

  hr_trs_ins.ins
    (p_transaction_step_id          => l_transaction_step_id
    ,p_transaction_id               => p_transaction_id
    ,p_api_name                     => 'HR_PERSON_ABSENCE_SWI.PROCESS_API'
    ,p_processing_order             => 0
    ,p_item_type                    => p_item_type
    ,p_creator_person_id            => fnd_global.employee_id
    ,p_update_person_id             => -1
    ,p_object_version_number        => l_step_ovn
    ,p_validate                     => false
    ,p_object_type                  => 'ENTITY'
    ,p_object_name                  => 'oracle.apps.per.schema.server.PerAbsenceAttendancesEO'
    ,p_object_identifier            => xxas_selfservice_workflow_pkg.get_hex_key
                                         (p_absence_attendance_id)
    ,p_object_state                 => p_object_state
    ,p_pk1                          => p_absence_attendance_id
    ,p_information_category         => null
    ,p_information1                 => p_information1
    ,p_information2                 => p_information2
    ,p_information3                 => p_information3
    ,p_information4                 => p_information4
    ,p_information5                 => p_information5
    ,p_information6                 => p_information6
    ,p_information7                 => p_information7
    ,p_information8                 => p_information8
    ,p_information9                 => p_information9
    ,p_information30                => 'ATT');

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

  RETURN l_transaction_step_id;

END create_hrapi_transaction;

--
-- ------------------------------------------------------------------------ *
-- Name    : update_hrapi_transaction
-- ------------------------------------------------------------------------ *
PROCEDURE update_hrapi_transaction 
  (p_transaction_id        IN   number
  ,p_transaction_step_id   IN   number
  ,p_absence_attendance_id IN   number
  ,p_assignment_id         IN   number
  ,p_person_id             IN   number
  ,p_item_type             IN   varchar2
  ,p_item_key              IN   varchar2
  ,p_absence_trans_doc     IN   XMLType
  ,p_information1          IN   varchar2
  ,p_information2          IN   varchar2
  ,p_information3          IN   varchar2
  ,p_information4          IN   varchar2
  ,p_information5          IN   varchar2
  ,p_information6          IN   varchar2
  ,p_information7          IN   varchar2
  ,p_information8          IN   varchar2
  ,p_information9          IN   varchar2
  ,p_object_state          IN   varchar2
  ,p_called_from           IN   varchar2
  ,p_ame_process_name      IN   varchar2
  ,p_transaction_status    IN   varchar2)
IS

  CURSOR csr_trans_step_obj_ver_number IS
  SELECT hats.object_version_number
  FROM   hr_api_transaction_steps hats
  WHERE  hats.transaction_step_id = p_transaction_step_id
  AND    hats.transaction_id      = p_transaction_id;

  c_method CONSTANT     varchar2(30) := 'update_hrapi_transaction';
  l_step_ovn            number;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  hr_trn_upd.upd
   (p_transaction_id               => p_transaction_id
   ,p_transaction_privilege        => 'PRIVATE'
   ,p_product_code                 => g_PER
   ,p_url                          => null
   ,p_status                       => p_transaction_status  --'W'
   ,p_section_display_name         => null
   ,p_function_id                  => xxas_com_sec_pkg.get_function_id(p_called_from)
   ,p_transaction_ref_table        => 'PER_ABSENCE_ATTENDANCES'
   ,p_transaction_ref_id           => p_absence_attendance_id
   ,p_transaction_type             => 'WF'
   ,p_assignment_id                => p_assignment_id
   ,p_api_addtnl_info              => null
   ,p_selected_person_id           => p_person_id
   ,p_item_type                    => p_item_type
   ,p_item_key                     => p_item_key
   ,p_transaction_effective_date   => trunc(sysdate)
   ,p_process_name                 => p_ame_process_name
   ,p_plan_id                      => null
   ,p_rptg_grp_id                  => null
   ,p_effective_date_option        => null
   ,p_validate                     => false
   ,p_creator_role                 => 'PER:' || fnd_global.employee_id
   ,p_last_update_role             => 'PER:' || fnd_global.employee_id
   ,p_parent_transaction_id        => null
   ,p_relaunch_function            => 'HR_ABS_ENTRY_PAGE_SS'
   ,p_transaction_group            => 'ABSENCE_MGMT'
   ,p_transaction_identifier       => 'ABSENCES'
   ,p_transaction_document         => p_absence_trans_doc.getCLOBVal());

  OPEN  csr_trans_step_obj_ver_number;
  FETCH csr_trans_step_obj_ver_number INTO l_step_ovn;
  CLOSE csr_trans_step_obj_ver_number;

  hr_trs_upd.upd
    (p_transaction_step_id          => p_transaction_step_id
    ,p_transaction_id               => p_transaction_id
    ,p_api_name                     => 'HR_PERSON_ABSENCE_SWI.PROCESS_API'
    ,p_processing_order             => 0
    ,p_item_type                    => p_item_type
    ,p_update_person_id             => -1
    ,p_object_version_number        => l_step_ovn
    ,p_validate                     => false
    ,p_object_type                  => 'ENTITY'
    ,p_object_name                  => 'oracle.apps.per.schema.server.PerAbsenceAttendancesEO'
    ,p_object_identifier            => xxas_selfservice_workflow_pkg.get_hex_key
                                         (p_absence_attendance_id)
    ,p_object_state                 => p_object_state
    ,p_pk1                          => p_absence_attendance_id
    ,p_information_category         => null
    ,p_information1                 => p_information1
    ,p_information2                 => p_information2
    ,p_information3                 => p_information3
    ,p_information4                 => p_information4
    ,p_information5                 => p_information5
    ,p_information6                 => p_information6
    ,p_information7                 => p_information7
    ,p_information8                 => p_information8
    ,p_information9                 => p_information9
    ,p_information30                => 'ATT');

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END update_hrapi_transaction;

--
-- ------------------------------------------------------------------------ *
-- Name    : workflow_review_page
-- ------------------------------------------------------------------------ *
PROCEDURE workflow_review_page
  (p_item_type    IN varchar2
  ,p_item_key     IN varchar2
  ,p_next_result  IN varchar2 DEFAULT 'NEXT')

IS

  CURSOR c_action 
  IS
  SELECT hatv.varchar2_value action
  FROM   hr_api_transactions hat
  ,      hr_api_transaction_steps hats
  ,      hr_api_transaction_values hatv
  WHERE  hat.item_type            = p_item_type
  AND    hat.item_key             = p_item_key
  AND    hat.transaction_id       = hats.transaction_id
  AND    hats.transaction_step_id = hatv.transaction_step_id
  AND    hatv.name                = 'P_ACTIVITY_NAME';
  
  CURSOR csr_trans_det(cp_transaction_id NUMBER)
  IS
  SELECT hats.transaction_step_id
        ,hat.transaction_ref_id
        ,hat.selected_person_id
  FROM   hr_api_transactions hat
        ,hr_api_transaction_steps hats
  WHERE  hat.transaction_id = hats.transaction_id
  AND    hat.transaction_id = cp_transaction_id
  AND    hats.api_name = 'HR_PERSON_ABSENCE_SWI.PROCESS_API';

  c_method CONSTANT   varchar2(30) := 'workflow_review_page';
  l_current_activity  wf_process_activities.activity_name%TYPE;
  l_func_name         varchar2(4000);
  l_action            varchar2(30);
  l_transaction_id        NUMBER;
  l_absence_attendance_id NUMBER;
  l_transaction_step_id   NUMBER;
  l_person_id             NUMBER;

BEGIN
  log_start(c_method, p_item_type, p_item_key, p_next_result); 

  l_current_activity := xxas_selfservice_workflow_pkg.get_current_activity_name
                              (p_item_type => p_item_type
                              ,p_item_key  => p_item_key);
    
  log_statement(c_method, 'l_current_activity = ' || l_current_activity);
  
  IF l_current_activity = 'HR_LOA_SUMMARY_FCT'
  THEN
    OPEN  c_action;
    FETCH c_action INTO l_action;
    CLOSE c_action;

    log_statement(c_method, 'l_action = ' || l_action);
  
    wf_engine.CompleteActivityInternalName
      (itemtype => p_item_type
      ,itemkey  => p_item_key
      ,activity => l_current_activity
      ,result   => CASE WHEN l_action = g_INSERT
                        THEN 'NEXT'
                        WHEN l_action = g_UPDATE
                        THEN 'UPDATE_ABSENCE'
                   END);
  END IF;

  --
  -- Retrieve the function from the existing attribute
  --
  l_func_name := xxas_cal_util_pkg.get_wf_attribute_text
                   (p_item_type, p_item_key, 'P_CALLED_FROM');
  
  l_transaction_id := xxas_selfservice_workflow_pkg.get_wf_attribute_text(p_item_type,p_item_key,'TRANSACTION_ID');
        
  --l_person_id := xxas_selfservice_workflow_pkg.get_wf_attribute_text(p_item_type,p_item_key,'CREATOR_PERSON_ID');
        
  -- Get transaction step id and transaction ref id which is absence_attendance_id
  OPEN csr_trans_det(l_transaction_id);
    
  FETCH csr_trans_det INTO l_transaction_step_id,l_absence_attendance_id,l_person_id;
  
  CLOSE csr_trans_det;
    
  -- 
  -- set page attributes.
  --
  xxas_com_worklist_pkg.populate_page_attributes(p_transaction_id      => l_transaction_id
                                                ,p_transaction_step_id => l_transaction_step_id
                                                ,p_item_type           => p_item_type
                                                ,p_item_key            => p_item_key
                                                ,p_edit_page_id        => xxas_com_worklist_pkg.g_ABSENCE_EDIT_PAGE
                                                ,p_edit_object_id      => l_person_id 
                                                                            || '_' || l_absence_attendance_id 
                                                                            || '_' || l_transaction_id 
                                                                            || '_' || l_transaction_step_id
                                                ,p_edit_target         => 'modal'
                                                ,p_view_page_id        => xxas_com_worklist_pkg.g_ABSENCE_VIEW_PAGE
                                                ,p_view_object_id      => l_person_id 
                                                                            || '_' || l_absence_attendance_id 
                                                                            || '_' || l_transaction_id 
                                                                            || '_' || l_transaction_step_id
                                                ,p_view_target         => 'modal');
                                                
  
  --
  -- If the function is specified, set the following attributes if they're
  -- currently null.
  --  
  IF l_func_name is not null THEN
    IF xxas_cal_util_pkg.get_wf_attribute_text
        (p_item_type, p_item_key, 'HR_GENERIC_URL_REDIRECT') is null
    THEN
      wf_engine.setitemattrtext
        (itemtype  => p_item_type
        ,itemkey   => p_item_key
        ,aname     => 'HR_GENERIC_URL_REDIRECT'
        ,avalue    => 'OA.jsp?akRegionCode=HR_REVIEW_TOP_SS'||'&'||'akRegionApplicationId=800'||'&'||'OAFunc=' || l_func_name);
    END IF;

    IF xxas_cal_util_pkg.get_wf_attribute_text
        (p_item_type, p_item_key, 'APPROVAL_GENERIC_URL') is null
    THEN
      wf_engine.setitemattrtext
        (itemtype  => p_item_type
        ,itemkey   => p_item_key
        ,aname     => 'APPROVAL_GENERIC_URL'
        ,avalue    => 'JSP:/' || fnd_profile.VALUE('ICX_OA_HTML')
                              || '/OA.jsp?akRegionCode=HR_REVIEW_TOP_SS'||'&'||'akRegionApplicationId=800'||'&'||'NtfId=-'||'&'||'#NID-'||'&'||'retainAM=Y'||'&'||'OAFunc='
                              || l_func_name);
    END IF;
  END IF;

  l_current_activity := xxas_selfservice_workflow_pkg.get_current_activity_name
                          (p_item_type => p_item_type
                          ,p_item_key  => p_item_key);

  log_end(c_method, l_current_activity);
          
END workflow_review_page;

-- ------------------------------------------------------------------------ *
-- Name    : approval_required
-- ------------------------------------------------------------------------ *
FUNCTION approval_required RETURN boolean
IS

  c_method          CONSTANT varchar2(30) := 'approval_required';
  l_num_approvers          number;
  l_create_update_rslt_xml XMLType;
  
  CURSOR csr_count_approvers
  IS
  SELECT count(*)
  FROM   TABLE(XMLSequence(extract(l_create_update_rslt_xml
                                  ,'//Approver')));

BEGIN

    --
    -- Contains the workflow details including a list of approvers
    --
    l_create_update_rslt_xml := xxas_com_api_result_pkg.result_details;
    
    OPEN  csr_count_approvers;
    FETCH csr_count_approvers INTO l_num_approvers;
    CLOSE csr_count_approvers;
    
    RETURN (nvl(l_num_approvers, 0) > 0);

END approval_required;

-- ------------------------------------------------------------------------ *
-- Name    : go_to_review_page_xml
-- ------------------------------------------------------------------------ *
FUNCTION go_to_review_page_xml
  (p_person_id             IN number
  ,p_absence_attendance_id IN number
  ,p_transaction_id        IN number
  ,p_transaction_step_id   IN number
  ,p_source_block          IN varchar2
  ,p_saved_search_id       IN number
  ,p_manager_id            IN number DEFAULT NULL) RETURN XMLType
IS

  c_method CONSTANT varchar2(30) := 'go_to_review_page_xml';
  l_xml           XMLType;

BEGIN

  log_start(c_method,p_person_id,p_absence_attendance_id,p_transaction_id,p_transaction_step_id,p_source_block,p_saved_search_id,p_manager_id);

  l_xml := xxas_com_rest_util_pkg.app_route_redirect_response
             ('absence-review/'
              || to_char(p_person_id)
              || '_'
              || to_char(p_absence_attendance_id)
              || '_'
              || to_char(p_transaction_id)
              || '_'
              || to_char(p_transaction_step_id)
              || '_'
              || p_source_block
              || '_'
              || to_char(p_saved_search_id)
              || '_' 
              ||  p_manager_id
             ,'modal'
             ,'true'
             ,'false');

  RETURN l_xml;

END go_to_review_page_xml;

-- ------------------------------------------------------------------------ *
-- Name    : save_file_attachments
-- ------------------------------------------------------------------------ *
PROCEDURE save_file_attachments
  (p_attachments_xml       IN XMLType
  ,p_data                  IN xxas_emp_view_ui_pkg.xxas_data_table
  ,p_instance_pk1_value    IN varchar2
  ,p_instance_pk2_value    IN varchar2
  ,p_instance_pk3_value    IN varchar2
  ,p_instance_pk4_value    IN varchar2
  ,p_instance_pk5_value    IN varchar2
  ,p_entity_name           IN varchar2
  ,p_pk1_value             IN varchar2
  ,p_category_code         IN varchar2
  ,p_absence_attendance_id IN NUMBER DEFAULT NULL
  ,p_transaction_id        IN NUMBER DEFAULT NULL)
IS

  c_method CONSTANT varchar2(30) := 'save_file_attachments';
  l_old_loaded      boolean      := false;
  l_pk1_value       varchar2(400);
  l_pk2_value       varchar2(400);
  l_pk3_value       varchar2(400);
  l_pk4_value       varchar2(400);
  l_pk5_value       varchar2(400);

  CURSOR csr_attachments
  IS
  SELECT xxas_cal_util_pkg.get_attribute_value('//ATTACHED_DOCUMENT_ID/value',attach.xml) attached_document_id
        ,p_pk1_value pk1_field
        ,xxas_cal_util_pkg.get_attribute_value('//PK2_VALUE/value',attach.xml) pk2_field
        ,xxas_cal_util_pkg.get_attribute_value('//PK3_VALUE/value',attach.xml) pk3_field
        ,xxas_cal_util_pkg.get_attribute_value('//PK4_VALUE/value',attach.xml) pk4_field
        ,xxas_cal_util_pkg.get_attribute_value('//PK5_VALUE/value',attach.xml) pk5_field
        ,xxas_cal_util_pkg.get_attribute_value('//TITLE/value',attach.xml) title
        ,xxas_cal_util_pkg.get_attribute_value('//DESCRIPTION/value',attach.xml) description
        ,xxas_cal_util_pkg.get_attribute_value('//FILE_ID/value',attach.xml) file_id
  FROM   XMLTable
          ('//ATTACHMENTS/rows'
          PASSING  p_attachments_xml
          COLUMNS  xml                      XMLType           path '.'
          ) attach;

  CURSOR csr_cur_attach
    (cp_entity_name IN varchar2
    ,cp_pk1_value   IN varchar2
    ,cp_pk2_value   IN varchar2
    ,cp_pk3_value   IN varchar2
    ,cp_pk4_value   IN varchar2
    ,cp_pk5_value   IN varchar2)
  IS
  SELECT *
  FROM   table(xxas_attachment_util_pkg.attachments_tbl
                (null, cp_entity_name, cp_pk1_value, cp_pk2_value
                ,cp_pk3_value, cp_pk4_value, cp_pk5_value)) o
  WHERE  o.file_id is not null
  UNION
  -- Added for RFC cases where duplicate attachments were created.
  SELECT * 
  FROM   table(xxas_attachment_util_pkg.attachments_tbl
                (null,'PER_ABSENCE_ATTENDANCES', CASE WHEN to_Char(p_transaction_id) IS NOT NULL THEN
                                                  p_absence_attendance_id|| '_'|| p_transaction_id ELSE to_char(p_absence_attendance_id) END
                ,null, null, null, null)) o
  WHERE  o.file_id is not null
  UNION
  -- Added for update absence case duplicate attachments were created.
  SELECT * 
  FROM   table(xxas_attachment_util_pkg.attachments_tbl
                (null,'PER_ABSENCE_ATTENDANCES',p_absence_attendance_id,null, null, null, null)) o
  WHERE  o.file_id is not null;

  TYPE l_old_attach_t IS TABLE OF csr_cur_attach%ROWTYPE
                         INDEX BY binary_integer;
  l_old_attach_tbl    l_old_attach_t;
  l_exists            boolean;
  i                   number;
  l_attachments_exists boolean := false;

  FUNCTION find_pk_value
    (p_field_name IN varchar2) RETURN varchar2
  IS
    l_pk_value varchar2(400);
  BEGIN
    IF p_data.count > 0 THEN
      IF p_field_name is not null THEN
        IF p_field_name = 'INSTANCE_PK1_VALUE' THEN
          l_pk_value := p_instance_pk1_value;
        ELSIF p_field_name = 'INSTANCE_PK2_VALUE' THEN
          l_pk_value := p_instance_pk2_value;
        ELSIF p_field_name = 'INSTANCE_PK3_VALUE' THEN
          l_pk_value := p_instance_pk3_value;
        ELSIF p_field_name = 'INSTANCE_PK4_VALUE' THEN
          l_pk_value := p_instance_pk4_value;
        ELSIF p_field_name = 'INSTANCE_PK5_VALUE' THEN
          l_pk_value := p_instance_pk5_value;
        ELSIF p_data.exists(p_field_name) THEN
          l_pk_value := p_data(p_field_name);
        END IF;
      END IF;
    END IF;
    RETURN l_pk_value;
  END find_pk_value;

BEGIN

  log_start(c_method);

  IF p_attachments_xml is null THEN
    log_statement(c_method, 'p_attachments_xml is null');
    RETURN;
  END IF;

  log_statement(c_method, 'p_data.count: '            || to_char(p_data.count));
  log_statement(c_method, 'p_instance_pk1_value: '    || p_instance_pk1_value);
  log_statement(c_method, 'p_instance_pk2_value: '    || p_instance_pk2_value);
  log_statement(c_method, 'p_instance_pk3_value: '    || p_instance_pk3_value);
  log_statement(c_method, 'p_instance_pk4_value: '    || p_instance_pk4_value);
  log_statement(c_method, 'p_instance_pk5_value: '    || p_instance_pk5_value);
  log_statement(c_method, 'p_transaction_id: '        || p_transaction_id);
  log_statement(c_method, 'p_absence_attendance_id: ' || p_absence_attendance_id);

  --
  -- Loop through each attachment and create/update it
  --
  FOR csr_rec IN csr_attachments LOOP
  
    log_statement(c_method, 'csr_rec.file_id: '            || to_char(csr_rec.file_id));
    log_statement(c_method, 'csr_rec.title: '              || csr_rec.title);
    log_statement(c_method, 'csr_rec.description: '        || csr_rec.description);
    log_statement(c_method, 'csr_rec.pk1_field: '          || csr_rec.pk1_field);
    log_statement(c_method, 'csr_rec.pk2_field: '          || csr_rec.pk2_field);
    log_statement(c_method, 'csr_rec.pk3_field: '          || csr_rec.pk3_field);
    log_statement(c_method, 'csr_rec.pk4_field: '          || csr_rec.pk4_field);
    log_statement(c_method, 'csr_rec.pk5_field: '          || csr_rec.pk5_field);

    IF (csr_rec.file_id   is not null OR
        csr_rec.pk1_field is not null OR
        csr_rec.pk2_field is not null OR
        csr_rec.pk3_field is not null OR
        csr_rec.pk4_field is not null OR 
        csr_rec.pk5_field is not null) AND 
       NOT l_attachments_exists THEN
      log_statement(c_method, 'l_attachments_exists is true');
      l_attachments_exists := true;
    END IF;

    --
    -- Resolve the attachment primary keys
    --
    l_pk1_value := find_pk_value(replace(csr_rec.pk1_field,'"'));
    l_pk2_value := find_pk_value(replace(csr_rec.pk2_field,'"'));
    l_pk3_value := find_pk_value(replace(csr_rec.pk3_field,'"'));
    l_pk4_value := find_pk_value(replace(csr_rec.pk4_field,'"'));
    l_pk5_value := find_pk_value(replace(csr_rec.pk5_field,'"'));

    --
    -- Get the current attachments to work out whether to create or update
    --
    IF NOT l_old_loaded THEN
      OPEN  csr_cur_attach
        (cp_entity_name => p_entity_name
        ,cp_pk1_value   => l_pk1_value
        ,cp_pk2_value   => l_pk2_value
        ,cp_pk3_value   => l_pk3_value
        ,cp_pk4_value   => l_pk4_value
        ,cp_pk5_value   => l_pk5_value);
      FETCH csr_cur_attach BULK COLLECT INTO l_old_attach_tbl;
      CLOSE csr_cur_attach;
      l_old_loaded := true;
      log_statement(c_method, 'l_old_attach_tbl.count: ' || to_char(l_old_attach_tbl.count));
    END IF;

    --
    -- Look to find a match to see if the attachment already exists
    --
    l_exists := false;
    IF l_old_attach_tbl.count > 0 THEN
      i := l_old_attach_tbl.first;
      LOOP
        EXIT WHEN i is null;
        IF l_old_attach_tbl(i).file_id = csr_rec.file_id THEN
          l_exists := true;
          log_statement(c_method, 'l_old_attach_tbl(i).attached_document_id: ' || to_char(l_old_attach_tbl(i).attached_document_id));
          EXIT;
        END IF;
        i := l_old_attach_tbl.next(i);
       END LOOP;
    END IF;
    
    --
    -- Create or update the attachment
    --
    IF l_exists THEN
      --
      -- Update the existing attachment details
      --
      log_statement(c_method, 'l_old_attach_tbl(i).attached_document_id: ' || to_char(l_old_attach_tbl(i).attached_document_id));

      xxas_attachment_pkg.update_file_attachment
        (pAttachedDocumentId => l_old_attach_tbl(i).attached_document_id
        ,pFileData           => null /* will leave the current file untouched */
        ,pCategoryCode       => p_category_code
        ,pFileName           => null /* will leave the current file untouched */
        ,pTitle              => replace(csr_rec.title,'"')
        ,pDescription        => replace(csr_rec.description,'"')
        ,pEntityName         => p_entity_name
        ,pPk1Value           => p_transaction_id
        ,pPk2Value           => l_pk2_value
        ,pPk3Value           => l_pk3_value
        ,pPk4Value           => l_pk4_value
        ,pPk5Value           => l_pk5_value);

      log_statement(c_method, '5');
    ELSE
      --
      -- Create a new attachment
      --
      log_statement(c_method, '10');
      xxas_attachment_pkg.create_file_attachment 
        (pEntityName   => p_entity_name
        ,pPk1Value     => l_pk1_value
        ,pPk2Value     => l_pk2_value
        ,pPk3Value     => l_pk3_value
        ,pPk4Value     => l_pk4_value
        ,pPk5Value     => l_pk5_value
        ,pFileData     => null
        ,pCategoryCode => p_category_code
        ,pFileName     => null
        ,pTitle        => replace(csr_rec.title,'"')
        ,pDescription  => replace(csr_rec.description,'"')
        ,pFileId       => csr_rec.file_id);
      log_statement(c_method, '15');
    END IF;
    
  END LOOP;

  l_exists := false;
  IF l_old_attach_tbl.count > 0 THEN
    i := l_old_attach_tbl.first;
    LOOP
      EXIT WHEN i is null;
        FOR csr_rec IN csr_attachments LOOP
          IF l_old_attach_tbl(i).file_id = csr_rec.file_id THEN
            l_exists := true;
            log_statement(c_method, 'l_old_attach_tbl(i).file_id2: ' || to_char(l_old_attach_tbl(i).file_id));
            log_statement(c_method, 'l_old_attach_tbl(i).attached_document_id2: ' || to_char(l_old_attach_tbl(i).attached_document_id));
            EXIT;
          END IF;
        END LOOP;

        IF NOT l_exists THEN
          --
          -- Delete the new attachment
          --
          log_statement(c_method, '20');
          xxas_attachment_pkg.delete_file_attachment 
            (pAttachedDocumentId  => l_old_attach_tbl(i).attached_document_id);
          log_statement(c_method, '25');
        END IF;
        i := l_old_attach_tbl.next(i);
        l_exists := false;
    END LOOP;
  END IF;

  IF NOT l_attachments_exists THEN
    --
    -- Resolve the attachment primary keys
    --
    l_pk1_value := find_pk_value(replace(p_pk1_value,'"'));
    FOR csr_cur_rec IN csr_cur_attach
                         (cp_entity_name => p_entity_name
                         ,cp_pk1_value   => l_pk1_value
                         ,cp_pk2_value   => null
                         ,cp_pk3_value   => null
                         ,cp_pk4_value   => null
                         ,cp_pk5_value   => null) LOOP
      log_statement(c_method, '30');
      IF csr_cur_rec.attached_document_id is not null THEN
        log_statement(c_method, '40');
        log_statement(c_method, 'csr_cur_rec.attached_document_id: ' || csr_cur_rec.attached_document_id);
        --
        -- Delete the existing attachments
        --
        xxas_attachment_pkg.delete_file_attachment 
          (pAttachedDocumentId  => csr_cur_rec.attached_document_id);
        log_statement(c_method, '50');
      END IF;
    END LOOP;
  END IF;

  log_end(c_method);
  
END save_file_attachments;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_absence_event
-- ------------------------------------------------------------------------ *
PROCEDURE create_absence_event
  (p_person_id                IN number
  ,p_assignment_id            IN number
  ,p_date_start               IN date
  ,p_date_end                 IN date
  ,p_time_start               IN varchar2
  ,p_time_end                 IN varchar2
  ,p_duration_type            IN varchar2
  ,p_duration_days            IN number
  ,p_duration_hours           IN number
  ,p_effective_date           IN date
  ,p_called_from              IN varchar2
  ,p_leave_status             IN varchar2
  ,p_absence_type_id          IN number
  ,p_abs_attendance_reason_id IN number
  ,p_attribute_category       IN varchar2
  ,p_attribute1               IN varchar2
  ,p_attribute2               IN varchar2
  ,p_attribute3               IN varchar2
  ,p_attribute4               IN varchar2
  ,p_attribute5               IN varchar2
  ,p_attribute6               IN varchar2
  ,p_attribute7               IN varchar2
  ,p_attribute8               IN varchar2
  ,p_attribute9               IN varchar2
  ,p_attribute10              IN varchar2
  ,p_attribute11              IN varchar2
  ,p_attribute12              IN varchar2
  ,p_attribute13              IN varchar2
  ,p_attribute14              IN varchar2
  ,p_attribute15              IN varchar2
  ,p_attribute16              IN varchar2
  ,p_attribute17              IN varchar2
  ,p_attribute18              IN varchar2
  ,p_attribute19              IN varchar2
  ,p_attribute20              IN varchar2
  ,p_abs_information_category IN varchar2
  ,p_abs_information1         IN varchar2
  ,p_abs_information2         IN varchar2
  ,p_abs_information3         IN varchar2
  ,p_abs_information4         IN varchar2
  ,p_abs_information5         IN varchar2
  ,p_abs_information6         IN varchar2
  ,p_abs_information7         IN varchar2
  ,p_abs_information8         IN varchar2
  ,p_abs_information9         IN varchar2
  ,p_abs_information10        IN varchar2
  ,p_abs_information11        IN varchar2
  ,p_abs_information12        IN varchar2
  ,p_abs_information13        IN varchar2
  ,p_abs_information14        IN varchar2
  ,p_abs_information15        IN varchar2
  ,p_abs_information16        IN varchar2
  ,p_abs_information17        IN varchar2
  ,p_abs_information18        IN varchar2
  ,p_abs_information19        IN varchar2
  ,p_abs_information20        IN varchar2
  ,p_abs_information21        IN varchar2
  ,p_abs_information22        IN varchar2
  ,p_abs_information23        IN varchar2
  ,p_abs_information24        IN varchar2
  ,p_abs_information25        IN varchar2
  ,p_abs_information26        IN varchar2
  ,p_abs_information27        IN varchar2
  ,p_abs_information28        IN varchar2
  ,p_abs_information29        IN varchar2
  ,p_abs_information30        IN varchar2
  ,p_replacement_person_id    IN number
  ,p_comments_to_approver     IN varchar2
  ,p_date_notification        IN date
  ,p_ame_item_type            IN varchar2
  ,p_ame_app_id               IN number
  ,p_ame_trans_type           IN varchar2
  ,p_ame_process_name         IN varchar2
  ,p_absence_attendance_id    OUT NOCOPY number
  ,p_transaction_id           IN OUT number
  ,p_transaction_step_id      IN OUT number)

IS

  CURSOR csr_asg(cp_assignment_id IN number) IS
  SELECT paaf.*
  FROM   per_all_assignments_f paaf
  WHERE  paaf.assignment_id = cp_assignment_id
  AND    trunc(p_date_start) BETWEEN paaf.effective_start_date 
                        AND     paaf.effective_end_date
  AND    (paaf.assignment_type = 'E' 
          OR paaf.assignment_type = 'C');

  CURSOR csr_person(cp_person_id IN number) IS
  SELECT papf.*
  FROM   per_all_people_f papf
  WHERE  papf.person_id = cp_person_id
  AND    trunc(p_date_start) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y');

  CURSOR csr_login_person(cp_login_person_id IN number) IS
  SELECT papf.*
        ,paaf.supervisor_id
        ,paaf.assignment_number
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.person_id = papf.person_id
  AND    trunc(p_date_start) BETWEEN paaf.effective_start_date 
                        AND     paaf.effective_end_date
  AND    papf.person_id = cp_login_person_id
  AND    trunc(p_date_start) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    ((nvl(papf.current_employee_flag, 'N') = 'Y' AND
           paaf.assignment_type = 'E')
          OR (nvl(papf.current_npw_flag, 'N') = 'Y' AND
              paaf.assignment_type = 'C'))
  AND    nvl(paaf.primary_flag, 'N') = 'Y';

  CURSOR csr_get_emp_full_name(cp_person_id IN number) IS
  SELECT decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'),'G',papf.global_name,papf.local_name) full_name
  FROM   per_all_people_f papf
  WHERE  papf.person_id = cp_person_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y');

  CURSOR csr_get_sec_grp_leg_code(cp_person_id IN number) IS
  SELECT bg.security_group_id
        ,bg.legislation_code
  FROM   per_all_people_f papf
        ,per_business_groups_perf bg
  WHERE  papf.person_id = cp_person_id
  AND    papf.business_group_id = bg.business_group_id
  AND    trunc(p_date_start) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y');

  CURSOR csr_get_job_name(cp_job_id number) IS
  SELECT pjv.name 
  FROM   per_jobs_vl pjv
  WHERE  pjv.job_id = cp_job_id 
  AND    trunc(sysdate) BETWEEN pjv.date_from 
                        AND     nvl(pjv.date_to,trunc(sysdate));

  CURSOR csr_get_pos_name(cp_pos_id IN number) IS
  SELECT hap.name 
  FROM   hr_all_positions_f_vl hap
  WHERE  hap.position_id = cp_pos_id 
  AND    trunc(sysdate) BETWEEN nvl(hap.effective_start_date,trunc(sysdate)) 
                        AND     nvl(hap.effective_end_date,trunc(sysdate));

  CURSOR csr_get_org_info(cp_person_id IN number) IS
  SELECT hoi.org_information2
        ,hoi.org_information10
  FROM   hr_organization_information_v hoi
  WHERE  hoi.org_information_context = 'Business Group Information'
  AND    hoi.organization_id         = xxas_per_pkg.get_business_group_id(cp_person_id);

  CURSOR csr_get_abs_att_type_meaning(cp_abs_att_type_id IN number) IS
  SELECT fcl.meaning
  FROM   per_abs_attendance_types_vl paat
        ,hr_lookups fcl
  WHERE  absence_attendance_type_id = cp_abs_att_type_id
  AND    paat.absence_category      = fcl.lookup_code(+)
  AND    fcl.lookup_type(+)         = 'ABSENCE_CATEGORY';

  CURSOR csr_trans_doc(cp_person_id                IN number
                      ,cp_absence_attendance_id    IN number
                      ,cp_per_row                  IN csr_person%ROWTYPE
                      ,cp_asg_row                  IN csr_asg%ROWTYPE
                      ,cp_login_per_row            IN csr_login_person%ROWTYPE
                      ,cp_per_manager_name         IN varchar2
                      ,cp_login_per_manager_name   IN varchar2
                      ,cp_transaction_id           IN number
                      ,cp_per_sec_group_id         IN number
                      ,cp_per_leg_code             IN varchar2
                      ,cp_login_per_leg_code       IN varchar2
                      ,cp_job_name                 IN varchar2
                      ,cp_pos_name                 IN varchar2
                      ,cp_emp_generation           IN varchar2
                      ,cp_per_bg_cur_code          IN varchar2
                      ,cp_called_from              IN varchar2
                      ,cp_assignment_id            IN number
                      ,cp_effective_date           IN date
                      ,cp_leave_status             IN varchar2
                      ,cp_absence_type_id          IN number
                      ,cp_abs_attendance_reason_id IN number
                      ,cp_duration_days            IN number
                      ,cp_duration_hours           IN number
                      ,cp_date_start               IN date
                      ,cp_date_end                 IN date
                      ,cp_time_start               IN varchar2
                      ,cp_time_end                 IN varchar2
                      ,cp_attribute_category       IN varchar2
                      ,cp_attribute1               IN varchar2
                      ,cp_attribute2               IN varchar2
                      ,cp_attribute3               IN varchar2
                      ,cp_attribute4               IN varchar2
                      ,cp_attribute5               IN varchar2
                      ,cp_attribute6               IN varchar2
                      ,cp_attribute7               IN varchar2
                      ,cp_attribute8               IN varchar2
                      ,cp_attribute9               IN varchar2
                      ,cp_attribute10              IN varchar2
                      ,cp_attribute11              IN varchar2
                      ,cp_attribute12              IN varchar2
                      ,cp_attribute13              IN varchar2
                      ,cp_attribute14              IN varchar2
                      ,cp_attribute15              IN varchar2
                      ,cp_attribute16              IN varchar2
                      ,cp_attribute17              IN varchar2
                      ,cp_attribute18              IN varchar2
                      ,cp_attribute19              IN varchar2
                      ,cp_attribute20              IN varchar2
                      ,cp_abs_information_category IN varchar2
                      ,cp_abs_information1         IN varchar2
                      ,cp_abs_information2         IN varchar2
                      ,cp_abs_information3         IN varchar2
                      ,cp_abs_information4         IN varchar2
                      ,cp_abs_information5         IN varchar2
                      ,cp_abs_information6         IN varchar2
                      ,cp_abs_information7         IN varchar2
                      ,cp_abs_information8         IN varchar2
                      ,cp_abs_information9         IN varchar2
                      ,cp_abs_information10        IN varchar2
                      ,cp_abs_information11        IN varchar2
                      ,cp_abs_information12        IN varchar2
                      ,cp_abs_information13        IN varchar2
                      ,cp_abs_information14        IN varchar2
                      ,cp_abs_information15        IN varchar2
                      ,cp_abs_information16        IN varchar2
                      ,cp_abs_information17        IN varchar2
                      ,cp_abs_information18        IN varchar2
                      ,cp_abs_information19        IN varchar2
                      ,cp_abs_information20        IN varchar2
                      ,cp_abs_information21        IN varchar2
                      ,cp_abs_information22        IN varchar2
                      ,cp_abs_information23        IN varchar2
                      ,cp_abs_information24        IN varchar2
                      ,cp_abs_information25        IN varchar2
                      ,cp_abs_information26        IN varchar2
                      ,cp_abs_information27        IN varchar2
                      ,cp_abs_information28        IN varchar2
                      ,cp_abs_information29        IN varchar2
                      ,cp_abs_information30        IN varchar2
                      ,cp_replacement_person_id    IN number
                      ,cp_comments_to_approver     IN varchar2
                      ,cp_ame_item_type            IN varchar2
                      ,cp_ame_app_id               IN number
                      ,cp_ame_trans_type           IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", cp_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", cp_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", cp_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", cp_ame_item_type)
             ,XMLElement("PrsnJobId", cp_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(cp_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", cp_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), cp_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", cp_login_per_leg_code)
             ,XMLElement("PrsnMgrId", cp_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", cp_assignment_id)
             ,XMLElement("PrsnLegCode", cp_per_leg_code)
             ,XMLElement("LoginPrsnId", cp_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", g_PER)
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(cp_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,xxas_com_sec_pkg.get_function_id(cp_called_from)
               )
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", cp_transaction_id)
             ,XMLElement("PrsnLocationId", cp_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", cp_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", cp_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", cp_per_manager_name)
             ,XMLElement("PrsnOrganizationId", cp_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", cp_asg_row.position_id)
             ,XMLElement("TxnStatus", 'W')
             ,XMLElement("LoginWorkerNumber", cp_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", cp_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", cp_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", cp_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(cp_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_login_per_row.global_name, cp_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), cp_login_per_row.person_id)
             ,XMLElement("PrsnId", cp_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || cp_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", cp_ame_trans_type)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(cp_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", cp_called_from)
             ,XMLElement("LoginPrsnBgId", cp_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", cp_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", cp_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_per_row.global_name, cp_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", cp_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CreateMode'))
             ,XMLElement("pAMEAppId", cp_ame_app_id)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(cp_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (0 as "PS"
                       ,'Y' as "PK")
                       ,XMLElement("AbsenceAttendanceId", cp_absence_attendance_id)
                       ,XMLElement("BusinessGroupId", cp_asg_row.business_group_id)
                       ,XMLElement("AbsenceAttendanceTypeId", cp_absence_type_id)
                       ,XMLElement("AbsAttendanceReasonId", cp_abs_attendance_reason_id)
                       ,XMLElement("PersonId", cp_person_id)
                       ,decode(cp_duration_days, null, XMLElement("AbsenceDays", XMLAttributes('true' as "null"), cp_duration_days),XMLElement("AbsenceDays", cp_duration_days))
                       ,decode(cp_duration_hours, null, XMLElement("AbsenceHours", XMLAttributes('true' as "null"), cp_duration_hours),XMLElement("AbsenceHours", cp_duration_hours))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("DateEnd", XMLAttributes('true' as "null")), decode(cp_date_end, null, XMLElement("DateEnd", XMLAttributes('true' as "null"), cp_date_end), XMLElement("DateEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT))))
                       ,XMLElement("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_date_end, null, XMLElement("DateProjectedEnd", XMLAttributes('true' as "null"), cp_date_end), XMLElement("DateProjectedEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT))), XMLElement("DateProjectedEnd", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_date_start, null, XMLElement("DateProjectedStart", XMLAttributes('true' as "null"), cp_date_start), XMLElement("DateProjectedStart", to_char(cp_date_start, g_HRAPI_DATE_FMT))), XMLElement("DateProjectedStart", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("DateStart", XMLAttributes('true' as "null")), decode(cp_date_start, null, XMLElement("DateStart", XMLAttributes('true' as "null"), cp_date_start), XMLElement("DateStart", to_char(cp_date_start, g_HRAPI_DATE_FMT))))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("TimeEnd", XMLAttributes('true' as "null")), decode(cp_time_end, null, XMLElement("TimeEnd", XMLAttributes('true' as "null"), cp_time_end), XMLElement("TimeEnd", cp_time_end)))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_time_end, null, XMLElement("TimeProjectedEnd", XMLAttributes('true' as "null"), cp_time_end), XMLElement("TimeProjectedEnd", cp_time_end)), XMLElement("TimeProjectedEnd", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_time_start, null, XMLElement("TimeProjectedStart", XMLAttributes('true' as "null"), cp_time_start), XMLElement("TimeProjectedStart", cp_time_start)), XMLElement("TimeProjectedStart", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("TimeStart", XMLAttributes('true' as "null")), decode(cp_time_start, null, XMLElement("TimeStart", XMLAttributes('true' as "null"), cp_time_start), XMLElement("TimeStart", cp_time_start)))
                       ,decode(cp_replacement_person_id, null, XMLElement("ReplacementPersonId", XMLAttributes('true' as "null"), cp_replacement_person_id), XMLElement("ReplacementPersonId", cp_replacement_person_id))
                       ,decode
                         (cp_attribute_category, null, null
                         ,XMLElement
                           ("AttributeCategory", cp_attribute_category))
                       ,decode
                         (cp_attribute1, null, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (cp_attribute2, null, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (cp_attribute3, null, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (cp_attribute4, null, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (cp_attribute5, null, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (cp_attribute6, null, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (cp_attribute7, null, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (cp_attribute8, null, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (cp_attribute9, null, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (cp_attribute10, null, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (cp_attribute11, null, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (cp_attribute12, null, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (cp_attribute13, null, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (cp_attribute14, null, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (cp_attribute15, null, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (cp_attribute16, null, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (cp_attribute17, null, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (cp_attribute18, null, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (cp_attribute19, null, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (cp_attribute20, null, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (cp_abs_information_category, null, null
                         ,XMLElement
                           ("AbsInformationCategory", cp_abs_information_category))
                       ,decode
                         (cp_abs_information1, null, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (cp_abs_information2, null, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (cp_abs_information3, null, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (cp_abs_information4, null, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (cp_abs_information5, null, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (cp_abs_information6, null, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (cp_abs_information7, null, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (cp_abs_information8, null, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (cp_abs_information9, null, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (cp_abs_information10, null, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (cp_abs_information11, null, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (cp_abs_information12, null, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (cp_abs_information13, null, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (cp_abs_information14, null, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (cp_abs_information15, null, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (cp_abs_information16, null, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (cp_abs_information17, null, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (cp_abs_information18, null, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (cp_abs_information19, null, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (cp_abs_information20, null, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (cp_abs_information21, null, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (cp_abs_information22, null, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (cp_abs_information23, null, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (cp_abs_information24, null, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (cp_abs_information25, null, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (cp_abs_information26, null, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (cp_abs_information27, null, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (cp_abs_information28, null, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (cp_abs_information29, null, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (cp_abs_information30, null, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("LastUpdateDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("LastUpdatedBy", fnd_global.user_id)
                       ,XMLElement("LastUpdateLogin", fnd_global.login_id)
                       ,XMLElement("CreatedBy", fnd_global.user_id)
                       ,XMLElement("CreationDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"), null)
                       ,decode(cp_comments_to_approver, null, XMLElement("Comments", XMLAttributes('true' as "null"), cp_comments_to_approver), XMLElement("Comments", cp_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(cp_transaction_id))
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_rc_trans_doc(cp_person_id                IN number
                         ,cp_absence_attendance_id    IN number
                         ,cp_per_row                  IN csr_person%ROWTYPE
                         ,cp_asg_row                  IN csr_asg%ROWTYPE
                         ,cp_login_per_row            IN csr_login_person%ROWTYPE
                         ,cp_per_manager_name         IN varchar2
                         ,cp_login_per_manager_name   IN varchar2
                         ,cp_transaction_id           IN number
                         ,cp_per_sec_group_id         IN number
                         ,cp_per_leg_code             IN varchar2
                         ,cp_login_per_leg_code       IN varchar2
                         ,cp_job_name                 IN varchar2
                         ,cp_pos_name                 IN varchar2
                         ,cp_emp_generation           IN varchar2
                         ,cp_per_bg_cur_code          IN varchar2
                         ,cp_called_from              IN varchar2
                         ,cp_assignment_id            IN number
                         ,cp_effective_date           IN date
                         ,cp_leave_status             IN varchar2
                         ,cp_absence_type_id          IN number
                         ,cp_abs_attendance_reason_id IN number
                         ,cp_duration_days            IN number
                         ,cp_duration_hours           IN number
                         ,cp_date_start               IN date
                         ,cp_date_end                 IN date
                         ,cp_time_start               IN varchar2
                         ,cp_time_end                 IN varchar2
                         ,cp_attribute_category       IN varchar2
                         ,cp_attribute1               IN varchar2
                         ,cp_attribute2               IN varchar2
                         ,cp_attribute3               IN varchar2
                         ,cp_attribute4               IN varchar2
                         ,cp_attribute5               IN varchar2
                         ,cp_attribute6               IN varchar2
                         ,cp_attribute7               IN varchar2
                         ,cp_attribute8               IN varchar2
                         ,cp_attribute9               IN varchar2
                         ,cp_attribute10              IN varchar2
                         ,cp_attribute11              IN varchar2
                         ,cp_attribute12              IN varchar2
                         ,cp_attribute13              IN varchar2
                         ,cp_attribute14              IN varchar2
                         ,cp_attribute15              IN varchar2
                         ,cp_attribute16              IN varchar2
                         ,cp_attribute17              IN varchar2
                         ,cp_attribute18              IN varchar2
                         ,cp_attribute19              IN varchar2
                         ,cp_attribute20              IN varchar2
                         ,cp_abs_information_category IN varchar2
                         ,cp_abs_information1         IN varchar2
                         ,cp_abs_information2         IN varchar2
                         ,cp_abs_information3         IN varchar2
                         ,cp_abs_information4         IN varchar2
                         ,cp_abs_information5         IN varchar2
                         ,cp_abs_information6         IN varchar2
                         ,cp_abs_information7         IN varchar2
                         ,cp_abs_information8         IN varchar2
                         ,cp_abs_information9         IN varchar2
                         ,cp_abs_information10        IN varchar2
                         ,cp_abs_information11        IN varchar2
                         ,cp_abs_information12        IN varchar2
                         ,cp_abs_information13        IN varchar2
                         ,cp_abs_information14        IN varchar2
                         ,cp_abs_information15        IN varchar2
                         ,cp_abs_information16        IN varchar2
                         ,cp_abs_information17        IN varchar2
                         ,cp_abs_information18        IN varchar2
                         ,cp_abs_information19        IN varchar2
                         ,cp_abs_information20        IN varchar2
                         ,cp_abs_information21        IN varchar2
                         ,cp_abs_information22        IN varchar2
                         ,cp_abs_information23        IN varchar2
                         ,cp_abs_information24        IN varchar2
                         ,cp_abs_information25        IN varchar2
                         ,cp_abs_information26        IN varchar2
                         ,cp_abs_information27        IN varchar2
                         ,cp_abs_information28        IN varchar2
                         ,cp_abs_information29        IN varchar2
                         ,cp_abs_information30        IN varchar2
                         ,cp_replacement_person_id    IN number
                         ,cp_comments_to_approver     IN varchar2
                         ,cp_ame_item_type            IN varchar2
                         ,cp_ame_app_id               IN number
                         ,cp_ame_trans_type           IN varchar2
                         ,cp_transaction_status       IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", cp_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", cp_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", cp_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", cp_ame_item_type)
             ,XMLElement("PrsnJobId", cp_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(cp_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", cp_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), cp_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", cp_login_per_leg_code)
             ,XMLElement("PrsnMgrId", cp_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", cp_assignment_id)
             ,XMLElement("PrsnLegCode", cp_per_leg_code)
             ,XMLElement("LoginPrsnId", cp_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", g_PER)
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(cp_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,xxas_com_sec_pkg.get_function_id(cp_called_from)
               )
             ,XMLElement("TxnState", 'T')
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", cp_transaction_id)
             ,XMLElement("PrsnLocationId", cp_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", cp_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", cp_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", cp_per_manager_name)
             ,XMLElement("PrsnOrganizationId", cp_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", cp_asg_row.position_id)
             ,decode(cp_transaction_status, 'Y', XMLElement("TxnStatus", 'Y'), XMLElement("TxnStatus", 'RI'))
             ,XMLElement("LoginWorkerNumber", cp_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", cp_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", cp_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", cp_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(cp_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_login_per_row.global_name, cp_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), cp_login_per_row.person_id)
             ,XMLElement("PrsnId", cp_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || cp_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", cp_ame_trans_type)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(cp_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", cp_called_from)
             ,XMLElement("LoginPrsnBgId", cp_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", cp_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", cp_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_per_row.global_name, cp_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", cp_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CreateMode'))
             ,XMLElement("pAMEAppId", cp_ame_app_id)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(cp_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (0 as "PS"
                       ,'Y' as "PK")
                       ,XMLElement("AbsenceAttendanceId", cp_absence_attendance_id)
                       ,XMLElement("BusinessGroupId", cp_asg_row.business_group_id)
                       ,XMLElement("AbsenceAttendanceTypeId", cp_absence_type_id)
                       ,XMLElement("AbsAttendanceReasonId", cp_abs_attendance_reason_id)
                       ,XMLElement("PersonId", cp_person_id)
                       ,decode(cp_duration_days, null, XMLElement("AbsenceDays", XMLAttributes('true' as "null"), cp_duration_days),XMLElement("AbsenceDays", cp_duration_days))
                       ,decode(cp_duration_hours, null, XMLElement("AbsenceHours", XMLAttributes('true' as "null"), cp_duration_hours),XMLElement("AbsenceHours", cp_duration_hours))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("DateEnd", XMLAttributes('true' as "null")), decode(cp_date_end, null, XMLElement("DateEnd", XMLAttributes('true' as "null"), cp_date_end), XMLElement("DateEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT))))
                       ,XMLElement("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_date_end, null, XMLElement("DateProjectedEnd", XMLAttributes('true' as "null"), cp_date_end), XMLElement("DateProjectedEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT))), XMLElement("DateProjectedEnd", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_date_start, null, XMLElement("DateProjectedStart", XMLAttributes('true' as "null"), cp_date_start), XMLElement("DateProjectedStart", to_char(cp_date_start, g_HRAPI_DATE_FMT))), XMLElement("DateProjectedStart", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("DateStart", XMLAttributes('true' as "null")), decode(cp_date_start, null, XMLElement("DateStart", XMLAttributes('true' as "null"), cp_date_start), XMLElement("DateStart", to_char(cp_date_start, g_HRAPI_DATE_FMT))))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("TimeEnd", XMLAttributes('true' as "null")), decode(cp_time_end, null, XMLElement("TimeEnd", XMLAttributes('true' as "null"), cp_time_end), XMLElement("TimeEnd", cp_time_end)))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_time_end, null, XMLElement("TimeProjectedEnd", XMLAttributes('true' as "null"), cp_time_end), XMLElement("TimeProjectedEnd", cp_time_end)), XMLElement("TimeProjectedEnd", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_time_start, null, XMLElement("TimeProjectedStart", XMLAttributes('true' as "null"), cp_time_start), XMLElement("TimeProjectedStart", cp_time_start)), XMLElement("TimeProjectedStart", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("TimeStart", XMLAttributes('true' as "null")), decode(cp_time_start, null, XMLElement("TimeStart", XMLAttributes('true' as "null"), cp_time_start), XMLElement("TimeStart", cp_time_start)))
                       ,decode(cp_replacement_person_id, null, XMLElement("ReplacementPersonId", XMLAttributes('true' as "null"), cp_replacement_person_id), XMLElement("ReplacementPersonId", cp_replacement_person_id))
                       ,decode
                         (cp_attribute_category, null, null
                         ,XMLElement
                           ("AttributeCategory", cp_attribute_category))
                       ,decode
                         (cp_attribute1, null, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (cp_attribute2, null, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (cp_attribute3, null, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (cp_attribute4, null, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (cp_attribute5, null, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (cp_attribute6, null, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (cp_attribute7, null, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (cp_attribute8, null, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (cp_attribute9, null, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (cp_attribute10, null, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (cp_attribute11, null, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (cp_attribute12, null, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (cp_attribute13, null, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (cp_attribute14, null, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (cp_attribute15, null, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (cp_attribute16, null, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (cp_attribute17, null, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (cp_attribute18, null, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (cp_attribute19, null, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (cp_attribute20, null, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (cp_abs_information_category, null, null
                         ,XMLElement
                           ("AbsInformationCategory", cp_abs_information_category))
                       ,decode
                         (cp_abs_information1, null, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (cp_abs_information2, null, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (cp_abs_information3, null, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (cp_abs_information4, null, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (cp_abs_information5, null, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (cp_abs_information6, null, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (cp_abs_information7, null, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (cp_abs_information8, null, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (cp_abs_information9, null, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (cp_abs_information10, null, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (cp_abs_information11, null, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (cp_abs_information12, null, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (cp_abs_information13, null, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (cp_abs_information14, null, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (cp_abs_information15, null, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (cp_abs_information16, null, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (cp_abs_information17, null, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (cp_abs_information18, null, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (cp_abs_information19, null, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (cp_abs_information20, null, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (cp_abs_information21, null, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (cp_abs_information22, null, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (cp_abs_information23, null, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (cp_abs_information24, null, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (cp_abs_information25, null, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (cp_abs_information26, null, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (cp_abs_information27, null, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (cp_abs_information28, null, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (cp_abs_information29, null, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (cp_abs_information30, null, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("LastUpdateDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("LastUpdatedBy", fnd_global.user_id)
                       ,XMLElement("LastUpdateLogin", fnd_global.login_id)
                       ,XMLElement("CreatedBy", fnd_global.user_id)
                       ,XMLElement("CreationDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"), null)
                       ,decode(cp_comments_to_approver, null, XMLElement("Comments", XMLAttributes('true' as "null"), cp_comments_to_approver), XMLElement("Comments", cp_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA('000100000000')
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_workflow_info(cp_transaction_id IN number) IS
  SELECT hat.item_type
        ,hat.item_key
        ,hat.transaction_ref_id
        ,hat.status
  FROM   hr_api_transactions  hat
  WHERE  hat.transaction_id = cp_transaction_id;

  CURSOR csr_transaction_status(cp_transaction_id IN number) IS
  SELECT hat.status
  FROM   hr_api_transactions  hat
  WHERE  hat.transaction_id = cp_transaction_id;

  c_method CONSTANT            varchar2(30) := 'create_absence_event';
  l_asg_row                    csr_asg%ROWTYPE;
  l_per_row                    csr_person%ROWTYPE;
  l_login_per_row              csr_login_person%ROWTYPE;
  l_login_person_id            number;
  l_per_manager_name           varchar2(240);
  l_login_per_manager_name     varchar2(240);
  l_per_sec_group_id           number;
  l_per_leg_code               varchar2(150);
  l_login_per_sec_group_id     number;
  l_login_per_leg_code         varchar2(150);
  l_per_job_name               varchar2(700);
  l_per_position_name          varchar2(240);
  l_doc                        XMLType;
  l_rc_doc                     XMLType;
  l_emp_generation             varchar2(150);
  l_per_bg_cur_code            varchar2(150);
  l_duration_days              per_absence_attendances.absence_days%TYPE  := p_duration_days;
  l_duration_hours             per_absence_attendances.absence_hours%TYPE := p_duration_hours;
  l_time_start                 varchar2(5) := p_time_start;
  l_time_end                   varchar2(5) := p_time_end;
  l_workflow_info              csr_workflow_info%ROWTYPE;
  l_absence_attendance_id      number;
  l_absence_attendance_id1     number;
  l_item_type                  wf_item_activity_statuses.item_type%TYPE;
  l_item_key                   wf_item_activity_statuses.item_key%TYPE;
  l_transaction_id             number := p_transaction_id;
  l_transaction_step_id        number := p_transaction_step_id;
  l_abs_att_category           varchar2(240);
  l_return_status              varchar2(30);
  l_occurrence                 varchar2(240);
  l_object_version_number      number;
  l_msg_data                   varchar2(2000) := null;
  l_msg_count                  number := 0;
  l_api_errors                 boolean := false;
  l_activity_id                wf_item_activity_statuses.process_activity%TYPE;
  err_code                     varchar2(200);
  err_msg                      varchar2(200);
  l_attribute1                 varchar2(1000);
  l_attribute2                 varchar2(1000);
  l_attribute3                 varchar2(1000);
  l_attribute4                 varchar2(1000);
  l_attribute5                 varchar2(1000);
  l_attribute6                 varchar2(1000);
  l_attribute7                 varchar2(1000);
  l_attribute8                 varchar2(1000);
  l_attribute9                 varchar2(1000);
  l_attribute10                varchar2(1000);
  l_attribute11                varchar2(1000);
  l_attribute12                varchar2(1000);
  l_attribute13                varchar2(1000);
  l_attribute14                varchar2(1000);
  l_attribute15                varchar2(1000);
  l_attribute16                varchar2(1000);
  l_attribute17                varchar2(1000);
  l_attribute18                varchar2(1000);
  l_attribute19                varchar2(1000);
  l_attribute20                varchar2(1000);
  l_abs_information1           varchar2(1000);
  l_abs_information2           varchar2(1000);
  l_abs_information3           varchar2(1000);
  l_abs_information4           varchar2(1000);
  l_abs_information5           varchar2(1000);
  l_abs_information6           varchar2(1000);
  l_abs_information7           varchar2(1000);
  l_abs_information8           varchar2(1000);
  l_abs_information9           varchar2(1000);
  l_abs_information10          varchar2(1000);
  l_abs_information11          varchar2(1000);
  l_abs_information12          varchar2(1000);
  l_abs_information13          varchar2(1000);
  l_abs_information14          varchar2(1000);
  l_abs_information15          varchar2(1000);
  l_abs_information16          varchar2(1000);
  l_abs_information17          varchar2(1000);
  l_abs_information18          varchar2(1000);
  l_abs_information19          varchar2(1000);
  l_abs_information20          varchar2(1000);
  l_abs_information21          varchar2(1000);
  l_abs_information22          varchar2(1000);
  l_abs_information23          varchar2(1000);
  l_abs_information24          varchar2(1000);
  l_abs_information25          varchar2(1000);
  l_abs_information26          varchar2(1000);
  l_abs_information27          varchar2(1000);
  l_abs_information28          varchar2(1000);
  l_abs_information29          varchar2(1000);
  l_abs_information30          varchar2(1000);
  l_flex_vs_id                 number;
  l_temp                       varchar2(1000);
  l_transaction_status         hr_api_transactions.status%TYPE;

  CURSOR csr_get_vs_id IS
  SELECT flex_value_set_id
  FROM   fnd_flex_value_sets
  WHERE  flex_value_set_name = 'PQP_GB_OSP_PART_DAYS_VS';
  
  CURSOR csr_get_dff_info IS
  SELECT application_column_name 
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  flex_value_set_id = l_flex_vs_id
  AND    enabled_flag = 'Y'
  AND    descriptive_flex_context_code in (p_attribute_category,p_abs_information_category);

  CURSOR csr_get_dff_gdd IS
  SELECT application_column_name 
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  flex_value_set_id = l_flex_vs_id
  AND    enabled_flag = 'Y'
  AND    descriptive_flex_context_code = 'Global Data Elements';
  
  CURSOR csr_get_enabled_dff_gdd IS
  SELECT application_column_name
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  enabled_flag = 'Y'
  and    descriptive_flexfield_name in ('PER_ABS_DEVELOPER_DF','PER_ABSENCE_ATTENDANCES')
  AND    descriptive_flex_context_code = 'Global Data Elements';

BEGIN

  log_start(c_method);

  OPEN  csr_person(p_person_id);
  FETCH csr_person INTO l_per_row;
  CLOSE csr_person;

  OPEN  csr_asg(p_assignment_id);
  FETCH csr_asg INTO l_asg_row;
  CLOSE csr_asg;

  OPEN  csr_get_emp_full_name(l_asg_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_job_name(l_asg_row.job_id);
  FETCH csr_get_job_name INTO l_per_job_name;
  CLOSE csr_get_job_name;

  OPEN  csr_get_pos_name(l_asg_row.position_id);
  FETCH csr_get_pos_name INTO l_per_position_name;
  CLOSE csr_get_pos_name;

  OPEN  csr_get_sec_grp_leg_code(p_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_per_sec_group_id
                                     ,l_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  OPEN  csr_get_org_info(p_person_id);
  FETCH csr_get_org_info INTO l_emp_generation
                             ,l_per_bg_cur_code;
  CLOSE csr_get_org_info;

  l_attribute1        := p_attribute1;
  l_attribute2        := p_attribute2;
  l_attribute3        := p_attribute3;
  l_attribute4        := p_attribute4;
  l_attribute5        := p_attribute5;
  l_attribute6        := p_attribute6;
  l_attribute7        := p_attribute7;
  l_attribute8        := p_attribute8;
  l_attribute9        := p_attribute9;
  l_attribute10       := p_attribute10;
  l_attribute11       := p_attribute11;
  l_attribute12       := p_attribute12;
  l_attribute13       := p_attribute13;
  l_attribute14       := p_attribute14;
  l_attribute15       := p_attribute15;
  l_attribute16       := p_attribute16;
  l_attribute17       := p_attribute17;
  l_attribute18       := p_attribute18;
  l_attribute19       := p_attribute19;
  l_attribute20       := p_attribute20;
  
  l_abs_information1  := p_abs_information1;
  l_abs_information2  := p_abs_information2;
  l_abs_information3  := p_abs_information3;
  l_abs_information4  := p_abs_information4;
  l_abs_information5  := p_abs_information5;
  l_abs_information6  := p_abs_information6;
  l_abs_information7  := p_abs_information7;
  l_abs_information8  := p_abs_information8;
  l_abs_information9  := p_abs_information9;
  l_abs_information10 := p_abs_information10;
  l_abs_information11  := p_abs_information11;
  l_abs_information12  := p_abs_information12;
  l_abs_information13  := p_abs_information13;
  l_abs_information14  := p_abs_information14;
  l_abs_information15  := p_abs_information15;
  l_abs_information16  := p_abs_information16;
  l_abs_information17  := p_abs_information17;
  l_abs_information18  := p_abs_information18;
  l_abs_information19  := p_abs_information19;
  l_abs_information20  := p_abs_information20;
  l_abs_information21  := p_abs_information21;
  l_abs_information22  := p_abs_information22;
  l_abs_information23  := p_abs_information23;
  l_abs_information24  := p_abs_information24;
  l_abs_information25  := p_abs_information25;
  l_abs_information26  := p_abs_information26;
  l_abs_information27  := p_abs_information27;
  l_abs_information28  := p_abs_information28;
  l_abs_information29  := p_abs_information29;
  l_abs_information30  := p_abs_information30;

  OPEN csr_get_vs_id;
  FETCH csr_get_vs_id INTO l_flex_vs_id;
  CLOSE csr_get_vs_id;
  
  OPEN csr_get_dff_gdd;
  FETCH csr_get_dff_gdd INTO l_temp;
  CLOSE csr_get_dff_gdd;
  
  IF l_temp IS NULL and p_abs_information_category IS NULL THEN
    l_abs_information1  := null;
    l_abs_information2  := null;
    l_abs_information3  := null;
    l_abs_information4  := null;
    l_abs_information5  := null;
    l_abs_information6  := null;
    l_abs_information7  := null;
    l_abs_information8  := null;
    l_abs_information9  := null;
    l_abs_information10 := null;
    l_abs_information11  := null;
    l_abs_information12  := null;
    l_abs_information13  := null;
    l_abs_information14  := null;
    l_abs_information15  := null;
    l_abs_information16  := null;
    l_abs_information17  := null;
    l_abs_information18  := null;
    l_abs_information19  := null;
    l_abs_information20  := null;
    l_abs_information21  := null;
    l_abs_information22  := null;
    l_abs_information23  := null;
    l_abs_information24  := null;
    l_abs_information25  := null;
    l_abs_information26  := null;
    l_abs_information27  := null;
    l_abs_information28  := null;
    l_abs_information29  := null;
    l_abs_information30  := null;
  END IF;

  IF l_temp IS NULL AND p_attribute_category IS NULL THEN
    l_attribute1        := null;
    l_attribute2        := null;
    l_attribute3        := null;
    l_attribute4        := null;
    l_attribute5        := null;
    l_attribute6        := null;
    l_attribute7        := null;
    l_attribute8        := null;
    l_attribute9        := null;
    l_attribute10       := null;
    l_attribute11       := null;
    l_attribute12       := null;
    l_attribute13       := null;
    l_attribute14       := null;
    l_attribute15       := null;
    l_attribute16       := null;
    l_attribute17       := null;
    l_attribute18       := null;
    l_attribute19       := null;
    l_attribute20       := null;
  END IF;
  
  FOR csr_rec in csr_get_enabled_dff_gdd
  LOOP
  
  IF csr_rec.application_column_name = 'ATTRIBUTE1' THEN
      if p_attribute1 is not null THEN
        l_attribute1 := p_attribute1;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE2' THEN
      if p_attribute2 is not null THEN
        l_attribute2 := p_attribute2;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE3' THEN
      if p_attribute3 is not null THEN
        l_attribute3 := p_attribute3;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE4' THEN
      if p_attribute4 is not null THEN
        l_attribute4 := p_attribute4;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE5' THEN
      if p_attribute5 is not null THEN
        l_attribute5 := p_attribute5;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE6' THEN
      if p_attribute6 is not null THEN
        l_attribute6 := p_attribute6;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE7' THEN
      if p_attribute7 is not null THEN
        l_attribute7 := p_attribute7;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE8' THEN
      if p_attribute8 is not null THEN
        l_attribute8 := p_attribute8;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE9' THEN
      if p_attribute9 is not null THEN
        l_attribute9 := p_attribute9;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE10' THEN
      if p_attribute10 is not null THEN
        l_attribute10 := p_attribute10;
      end if;
    elsIF csr_rec.application_column_name = 'ATTRIBUTE11' THEN
      if p_attribute11 is not null THEN
        l_attribute11 := p_attribute11;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE12' THEN
      if p_attribute12 is not null THEN
        l_attribute12 := p_attribute12;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE13' THEN
      if p_attribute13 is not null THEN
        l_attribute13 := p_attribute13;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE14' THEN
      if p_attribute14 is not null THEN
        l_attribute14 := p_attribute14;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE15' THEN
      if p_attribute15 is not null THEN
        l_attribute15 := p_attribute15;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE16' THEN
      if p_attribute16 is not null THEN
        l_attribute16 := p_attribute16;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE17' THEN
      if p_attribute17 is not null THEN
        l_attribute17 := p_attribute17;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE18' THEN
      if p_attribute18 is not null THEN
        l_attribute18 := p_attribute18;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE19' THEN
      if p_attribute19 is not null THEN
        l_attribute19 := p_attribute19;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE20' THEN
      if p_attribute20 is not null THEN
        l_attribute20 := p_attribute20;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION1' THEN
      if p_abs_information1 is not null THEN
        l_abs_information1 := p_abs_information1;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION2' THEN
      if p_abs_information2 is not null THEN
        l_abs_information2 := p_abs_information2;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION3' THEN
      if p_abs_information3 is not null THEN
        l_abs_information3 := p_abs_information3;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION4' THEN
      if p_abs_information4 is not null THEN
        l_abs_information4 := p_abs_information4;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION5' THEN
      if p_abs_information5 is not null THEN
        l_abs_information5 := p_abs_information5;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION6' THEN
      if p_abs_information6 is not null THEN
        l_abs_information6 := p_abs_information6;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION7' THEN
      if p_abs_information7 is not null THEN
        l_abs_information7 := p_abs_information7;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION8' THEN
      if p_abs_information8 is not null THEN
        l_abs_information8 := p_abs_information8;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION9' THEN
      if p_abs_information9 is not null THEN
        l_abs_information9 := p_abs_information9;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION10' THEN
      if p_abs_information10 is not null THEN
        l_abs_information10 := p_abs_information10;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION11' THEN
      if p_abs_information11 is not null THEN
        l_abs_information11 := p_abs_information11;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION12' THEN
      if p_abs_information12 is not null THEN
        l_abs_information12 := p_abs_information12;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION13' THEN
      if p_abs_information13 is not null THEN
        l_abs_information13 := p_abs_information13;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION14' THEN
      if p_abs_information14 is not null THEN
        l_abs_information14 := p_abs_information14;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION15' THEN
      if p_abs_information15 is not null THEN
        l_abs_information15 := p_abs_information15;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION16' THEN
      if p_abs_information16 is not null THEN
        l_abs_information16 := p_abs_information16;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION17' THEN
      if p_abs_information17 is not null THEN
        l_abs_information17 := p_abs_information17;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION18' THEN
      if p_abs_information18 is not null THEN
        l_abs_information18 := p_abs_information18;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION19' THEN
      if p_abs_information19 is not null THEN
        l_abs_information19 := p_abs_information19;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION20' THEN
      if p_abs_information20 is not null THEN
        l_abs_information20 := p_abs_information20;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION21' THEN
      if p_abs_information21 is not null THEN
        l_abs_information21 := p_abs_information21;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION22' THEN
      if p_abs_information22 is not null THEN
        l_abs_information22 := p_abs_information22;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION23' THEN
      if p_abs_information23 is not null THEN
        l_abs_information23 := p_abs_information23;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION24' THEN
      if p_abs_information24 is not null THEN
        l_abs_information24 := p_abs_information24;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION25' THEN
      if p_abs_information25 is not null THEN
        l_abs_information25 := p_abs_information25;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION26' THEN
      if p_abs_information26 is not null THEN
        l_abs_information26 := p_abs_information26;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION27' THEN
      if p_abs_information27 is not null THEN
        l_abs_information27 := p_abs_information27;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION28' THEN
      if p_abs_information28 is not null THEN
        l_abs_information28 := p_abs_information28;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION29' THEN
      if p_abs_information29 is not null THEN
        l_abs_information29 := p_abs_information29;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION30' THEN
      if p_abs_information30 is not null THEN
        l_abs_information30 := p_abs_information30;
      end if;
    end if;
 END LOOP;
  
  FOR csr_rec in csr_get_dff_info
  LOOP
  
  IF csr_rec.application_column_name = 'ATTRIBUTE1' THEN
      if instr(l_attribute1,'.') = 0 THEN
        l_attribute1 := p_attribute1||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE2' THEN
      if instr(l_attribute2,'.') = 0 THEN
        l_attribute2 := p_attribute2||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE3' THEN
      if instr(l_attribute3,'.') = 0 THEN
        l_attribute3 := p_attribute3||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE4' THEN
      if instr(l_attribute4,'.') = 0 THEN
        l_attribute4 := p_attribute4||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE5' THEN
      if instr(l_attribute5,'.') = 0 THEN
        l_attribute5 := p_attribute5||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE6' THEN
      if instr(l_attribute6,'.') = 0 THEN
        l_attribute6 := p_attribute6||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE7' THEN
      if instr(l_attribute7,'.') = 0 THEN
        l_attribute7 := p_attribute7||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE8' THEN
      if instr(l_attribute8,'.') = 0 THEN
        l_attribute8 := p_attribute8||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE9' THEN
      if instr(l_attribute9,'.') = 0 THEN
        l_attribute9 := p_attribute9||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE10' THEN
      if instr(l_attribute10,'.') = 0 THEN
        l_attribute10 := p_attribute10||'.00';
      end if;
    elsIF csr_rec.application_column_name = 'ATTRIBUTE11' THEN
      if instr(l_attribute11,'.') = 0 THEN
        l_attribute11 := p_attribute11||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE12' THEN
      if instr(l_attribute12,'.') = 0 THEN
        l_attribute12 := p_attribute12||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE13' THEN
      if instr(l_attribute13,'.') = 0 THEN
        l_attribute13 := p_attribute13||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE14' THEN
      if instr(l_attribute14,'.') = 0 THEN
        l_attribute14 := p_attribute14||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE15' THEN
      if instr(l_attribute15,'.') = 0 THEN
        l_attribute15 := p_attribute15||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE16' THEN
      if instr(l_attribute16,'.') = 0 THEN
        l_attribute16 := p_attribute16||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE17' THEN
      if instr(l_attribute17,'.') = 0 THEN
        l_attribute17 := p_attribute17||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE18' THEN
      if instr(l_attribute18,'.') = 0 THEN
        l_attribute18 := p_attribute18||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE19' THEN
      if instr(l_attribute19,'.') = 0 THEN
        l_attribute19 := p_attribute19||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE20' THEN
      if instr(l_attribute20,'.') = 0 THEN
        l_attribute20 := p_attribute20||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION1' THEN
      if instr(l_abs_information1,'.') = 0 THEN
        l_abs_information1 := l_abs_information1||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION2' THEN
      if instr(l_abs_information2,'.') = 0 THEN
        l_abs_information2 := l_abs_information2||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION3' THEN
      if instr(l_abs_information3,'.') = 0 THEN
        l_abs_information3 := l_abs_information3||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION4' THEN
      if instr(l_abs_information4,'.') = 0 THEN
        l_abs_information4 := l_abs_information4||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION5' THEN
      if instr(l_abs_information5,'.') = 0 THEN
        l_abs_information5 := l_abs_information5||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION6' THEN
      if instr(l_abs_information6,'.') = 0 THEN
        l_abs_information6 := l_abs_information6||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION7' THEN
      if instr(l_abs_information7,'.') = 0 THEN
        l_abs_information7 := l_abs_information7||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION8' THEN
      if instr(l_abs_information8,'.') = 0 THEN
        l_abs_information8 := l_abs_information8||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION9' THEN
      if instr(l_abs_information9,'.') = 0 THEN
        l_abs_information9 := l_abs_information9||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION10' THEN
      if instr(l_abs_information10,'.') = 0 THEN
        l_abs_information10 := l_abs_information10||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION11' THEN
      if instr(l_abs_information11,'.') = 0 THEN
        l_abs_information11 := l_abs_information11||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION12' THEN
      if instr(l_abs_information12,'.') = 0 THEN
        l_abs_information12 := l_abs_information12||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION13' THEN
      if instr(l_abs_information13,'.') = 0 THEN
        l_abs_information13 := l_abs_information13||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION14' THEN
      if instr(l_abs_information14,'.') = 0 THEN
        l_abs_information14 := l_abs_information14||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION15' THEN
      if instr(l_abs_information15,'.') = 0 THEN
        l_abs_information15 := l_abs_information15||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION16' THEN
      if instr(l_abs_information16,'.') = 0 THEN
        l_abs_information16 := l_abs_information16||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION17' THEN
      if instr(l_abs_information17,'.') = 0 THEN
        l_abs_information17 := l_abs_information17||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION18' THEN
      if instr(l_abs_information18,'.') = 0 THEN
        l_abs_information18 := l_abs_information18||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION19' THEN
      if instr(l_abs_information19,'.') = 0 THEN
        l_abs_information19 := l_abs_information19||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION20' THEN
      if instr(l_abs_information20,'.') = 0 THEN
        l_abs_information20 := l_abs_information20||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION21' THEN
      if instr(l_abs_information21,'.') = 0 THEN
        l_abs_information21 := l_abs_information21||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION22' THEN
      if instr(l_abs_information22,'.') = 0 THEN
        l_abs_information22 := l_abs_information22||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION23' THEN
      if instr(l_abs_information23,'.') = 0 THEN
        l_abs_information23 := l_abs_information23||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION24' THEN
      if instr(l_abs_information24,'.') = 0 THEN
        l_abs_information24 := l_abs_information24||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION25' THEN
      if instr(l_abs_information25,'.') = 0 THEN
        l_abs_information25 := l_abs_information25||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION26' THEN
      if instr(l_abs_information26,'.') = 0 THEN
        l_abs_information26 := l_abs_information26||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION27' THEN
      if instr(l_abs_information27,'.') = 0 THEN
        l_abs_information27 := l_abs_information27||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION28' THEN
      if instr(l_abs_information28,'.') = 0 THEN
        l_abs_information28 := l_abs_information28||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION29' THEN
      if instr(l_abs_information29,'.') = 0 THEN
        l_abs_information29 := l_abs_information29||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION30' THEN
      if instr(l_abs_information30,'.') = 0 THEN
        l_abs_information30 := l_abs_information30||'.00';
      end if;
    end if;
  END LOOP;

  IF p_duration_type is not null THEN

    IF p_duration_type = 'H' THEN

      l_duration_days := null;

    ELSIF p_duration_type NOT IN (xxas_fast_formula_pkg.g_NULL_STRING,'H') THEN

      l_time_start     := null;
      l_time_end       := null;
      l_duration_hours := null;

    END IF;

  END IF;

  IF l_transaction_id is not null AND
     l_transaction_step_id is not null THEN

    OPEN  csr_workflow_info(l_transaction_id);
    FETCH csr_workflow_info INTO l_workflow_info;
    CLOSE csr_workflow_info;

    l_absence_attendance_id := l_workflow_info.transaction_ref_id;
    l_item_type             := l_workflow_info.item_type;
    l_item_key              := l_workflow_info.item_key;
    l_transaction_status    := l_workflow_info.status;

  ELSE

    --
    -- Generate the new sequences
    --
    SELECT  per_absence_attendances_s.nextval
           ,hr_api_transactions_s.nextval
    INTO    l_absence_attendance_id
           ,l_transaction_id
    FROM   dual;

    l_transaction_status := 'W';

  END IF;

  l_login_person_id := xxas_util_pkg.get_per_from_user(fnd_global.user_id);

  OPEN  csr_login_person(l_login_person_id);
  FETCH csr_login_person INTO l_login_per_row;
  CLOSE csr_login_person;

  OPEN  csr_get_emp_full_name(l_login_per_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_login_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_sec_grp_leg_code(l_login_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_login_per_sec_group_id
                                     ,l_login_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  IF l_transaction_id is not null AND 
     l_transaction_step_id is not null THEN

    OPEN  csr_rc_trans_doc(p_person_id
                          ,l_absence_attendance_id
                          ,l_per_row
                          ,l_asg_row
                          ,l_login_per_row
                          ,l_per_manager_name
                          ,l_login_per_manager_name
                          ,l_transaction_id
                          ,l_per_sec_group_id
                          ,l_per_leg_code
                          ,l_login_per_leg_code
                          ,l_per_job_name
                          ,l_per_position_name
                          ,l_emp_generation
                          ,l_per_bg_cur_code
                          ,p_called_from
                          ,p_assignment_id
                          ,p_effective_date
                          ,p_leave_status
                          ,p_absence_type_id
                          ,p_abs_attendance_reason_id
                          ,l_duration_days
                          ,l_duration_hours
                          ,p_date_start
                          ,p_date_end
                          ,l_time_start
                          ,l_time_end
                          ,p_attribute_category
                          ,l_attribute1
                          ,l_attribute2
                          ,l_attribute3
                          ,l_attribute4
                          ,l_attribute5
                          ,l_attribute6
                          ,l_attribute7
                          ,l_attribute8
                          ,l_attribute9
                          ,l_attribute10
                          ,l_attribute11
                          ,l_attribute12
                          ,l_attribute13
                          ,l_attribute14
                          ,l_attribute15
                          ,l_attribute16
                          ,l_attribute17
                          ,l_attribute18
                          ,l_attribute19
                          ,l_attribute20
                          ,p_abs_information_category
                          ,l_abs_information1
                          ,l_abs_information2
                          ,l_abs_information3
                          ,l_abs_information4
                          ,l_abs_information5
                          ,l_abs_information6
                          ,l_abs_information7
                          ,l_abs_information8
                          ,l_abs_information9
                          ,l_abs_information10
                          ,l_abs_information11
                          ,l_abs_information12
                          ,l_abs_information13
                          ,l_abs_information14
                          ,l_abs_information15
                          ,l_abs_information16
                          ,l_abs_information17
                          ,l_abs_information18
                          ,l_abs_information19
                          ,l_abs_information20
                          ,l_abs_information21
                          ,l_abs_information22
                          ,l_abs_information23
                          ,l_abs_information24
                          ,l_abs_information25
                          ,l_abs_information26
                          ,l_abs_information27
                          ,l_abs_information28
                          ,l_abs_information29
                          ,l_abs_information30
                          ,p_replacement_person_id
                          ,p_comments_to_approver
                          ,p_ame_item_type
                          ,p_ame_app_id
                          ,p_ame_trans_type
                          ,l_transaction_status);
    FETCH csr_rc_trans_doc INTO l_rc_doc;
    CLOSE csr_rc_trans_doc;

  ELSE

    OPEN  csr_trans_doc(p_person_id
                       ,l_absence_attendance_id
                       ,l_per_row
                       ,l_asg_row
                       ,l_login_per_row
                       ,l_per_manager_name
                       ,l_login_per_manager_name
                       ,l_transaction_id
                       ,l_per_sec_group_id
                       ,l_per_leg_code
                       ,l_login_per_leg_code
                       ,l_per_job_name
                       ,l_per_position_name
                       ,l_emp_generation
                       ,l_per_bg_cur_code
                       ,p_called_from
                       ,p_assignment_id
                       ,p_effective_date
                       ,p_leave_status
                       ,p_absence_type_id
                       ,p_abs_attendance_reason_id
                       ,l_duration_days
                       ,l_duration_hours
                       ,p_date_start
                       ,p_date_end
                       ,l_time_start
                       ,l_time_end
                       ,p_attribute_category
                       ,l_attribute1
                       ,l_attribute2
                       ,l_attribute3
                       ,l_attribute4
                       ,l_attribute5
                       ,l_attribute6
                       ,l_attribute7
                       ,l_attribute8
                       ,l_attribute9
                       ,l_attribute10
                       ,l_attribute11
                       ,l_attribute12
                       ,l_attribute13
                       ,l_attribute14
                       ,l_attribute15
                       ,l_attribute16
                       ,l_attribute17
                       ,l_attribute18
                       ,l_attribute19
                       ,l_attribute20
                       ,p_abs_information_category
                       ,l_abs_information1
                       ,l_abs_information2
                       ,l_abs_information3
                       ,l_abs_information4
                       ,l_abs_information5
                       ,l_abs_information6
                       ,l_abs_information7
                       ,l_abs_information8
                       ,l_abs_information9
                       ,l_abs_information10
                       ,l_abs_information11
                       ,l_abs_information12
                       ,l_abs_information13
                       ,l_abs_information14
                       ,l_abs_information15
                       ,l_abs_information16
                       ,l_abs_information17
                       ,l_abs_information18
                       ,l_abs_information19
                       ,l_abs_information20
                       ,l_abs_information21
                       ,l_abs_information22
                       ,l_abs_information23
                       ,l_abs_information24
                       ,l_abs_information25
                       ,l_abs_information26
                       ,l_abs_information27
                       ,l_abs_information28
                       ,l_abs_information29
                       ,l_abs_information30
                       ,p_replacement_person_id
                       ,p_comments_to_approver
                       ,p_ame_item_type
                       ,p_ame_app_id
                       ,p_ame_trans_type);
    FETCH csr_trans_doc INTO l_doc;
    CLOSE csr_trans_doc;

  END IF;

  OPEN  csr_get_abs_att_type_meaning(p_absence_type_id);
  FETCH csr_get_abs_att_type_meaning INTO l_abs_att_category;
  CLOSE csr_get_abs_att_type_meaning;

  IF l_transaction_id is not null AND 
     l_transaction_step_id is not null THEN

    update_hrapi_transaction(p_transaction_id        => l_transaction_id
                            ,p_transaction_step_id   => l_transaction_step_id
                            ,p_absence_attendance_id => l_absence_attendance_id
                            ,p_assignment_id         => p_assignment_id
                            ,p_person_id             => p_person_id
                            ,p_item_type             => p_ame_item_type
                            ,p_item_key              => null--l_item_key
                            ,p_absence_trans_doc     => l_rc_doc
                            ,p_information1          => CASE WHEN p_leave_status = g_PLANNED
                                                             THEN null
                                                             ELSE to_char(p_date_start, g_HRAPI_DATE_FMT)
                                                        END
                            ,p_information2          => CASE WHEN p_leave_status = g_PLANNED
                                                             THEN null
                                                             ELSE to_char(p_date_end, g_HRAPI_DATE_FMT)
                                                        END
                            ,p_information3          => CASE WHEN p_leave_status = g_PLANNED
                                                             THEN to_char(p_date_start, g_HRAPI_DATE_FMT)
                                                             ELSE null
                                                        END
                            ,p_information4          => CASE WHEN p_leave_status = g_PLANNED
                                                             THEN to_char(p_date_end, g_HRAPI_DATE_FMT)
                                                             ELSE null
                                                        END
                            ,p_information5          => p_absence_type_id
                            ,p_information6          => l_abs_att_category
                            ,p_information7          => l_duration_hours
                            ,p_information8          => l_duration_days
                            ,p_information9          => p_leave_status
                            ,p_object_state          => '0'
                            ,p_called_from           => p_called_from
                            ,p_ame_process_name      => p_ame_process_name
                            ,p_transaction_status    => l_transaction_status);

  ELSE

    l_transaction_step_id := create_hrapi_transaction(p_transaction_id        => l_transaction_id
                                                     ,p_absence_attendance_id => l_absence_attendance_id
                                                     ,p_assignment_id         => p_assignment_id
                                                     ,p_person_id             => p_person_id
                                                     ,p_item_type             => p_ame_item_type
                                                     ,p_item_key              => l_item_key
                                                     ,p_absence_trans_doc     => l_doc
                                                     ,p_information1          => CASE WHEN p_leave_status = g_PLANNED
                                                                                      THEN null
                                                                                      ELSE to_char(p_date_start, g_HRAPI_DATE_FMT)
                                                                                 END
                                                     ,p_information2          => CASE WHEN p_leave_status = g_PLANNED
                                                                                      THEN null
                                                                                      ELSE to_char(p_date_end, g_HRAPI_DATE_FMT)
                                                                                 END
                                                     ,p_information3          => CASE WHEN p_leave_status = g_PLANNED
                                                                                      THEN to_char(p_date_start, g_HRAPI_DATE_FMT)
                                                                                      ELSE null
                                                                                 END
                                                     ,p_information4          => CASE WHEN p_leave_status = g_PLANNED
                                                                                      THEN to_char(p_date_end, g_HRAPI_DATE_FMT)
                                                                                      ELSE null
                                                                                 END
                                                     ,p_information5          => p_absence_type_id
                                                     ,p_information6          => l_abs_att_category
                                                     ,p_information7          => l_duration_hours
                                                     ,p_information8          => l_duration_days
                                                     ,p_information9          => p_leave_status
                                                     ,p_object_state          => '0'
                                                     ,p_called_from           => p_called_from
                                                     ,p_ame_process_name      => p_ame_process_name
                                                     ,p_transaction_status    => l_transaction_status);

  END IF;

  --
  -- Do create API's validation
  --
  --
  -- Issue a savepoint
  --
  savepoint before_api_validation;

  l_absence_attendance_id1 := l_absence_attendance_id;

  hr_person_absence_swi.create_person_absence
    (p_validate                     => hr_api.g_true_num
    ,p_effective_date               => p_effective_date
    ,p_person_id                    => p_person_id
    ,p_business_group_id            => l_asg_row.business_group_id
    ,p_absence_attendance_type_id   => p_absence_type_id
    ,p_abs_attendance_reason_id     => p_abs_attendance_reason_id
    ,p_comments                     => p_comments_to_approver
    ,p_date_notification            => p_date_notification
    ,p_date_projected_start         => CASE WHEN p_leave_status = g_PLANNED
                                            THEN p_date_start
                                            ELSE null
                                       END
    ,p_time_projected_start         => CASE WHEN p_leave_status = g_PLANNED
                                            THEN l_time_start
                                            ELSE null
                                       END
    ,p_date_projected_end           => CASE WHEN p_leave_status = g_PLANNED
                                            THEN p_date_end
                                            ELSE null
                                       END
    ,p_time_projected_end           => CASE WHEN p_leave_status = g_PLANNED
                                            THEN l_time_end
                                            ELSE null
                                       END
    ,p_date_start                   => CASE WHEN p_leave_status = g_PLANNED
                                            THEN null
                                            ELSE p_date_start
                                       END
    ,p_time_start                   => CASE WHEN p_leave_status = g_PLANNED
                                            THEN null
                                            ELSE l_time_start
                                       END
    ,p_date_end                     => CASE WHEN p_leave_status = g_PLANNED
                                            THEN null
                                            ELSE p_date_end
                                       END
    ,p_time_end                     => CASE WHEN p_leave_status = g_PLANNED
                                            THEN null
                                            ELSE l_time_end
                                       END
    ,p_absence_days                 => l_duration_days
    ,p_absence_hours                => l_duration_hours
    ,p_authorising_person_id        => null -- how to pass p_authorising_person_id
    ,p_replacement_person_id        => p_replacement_person_id
    ,p_attribute_category           => p_attribute_category
    ,p_attribute1                   => l_attribute1
    ,p_attribute2                   => l_attribute2
    ,p_attribute3                   => l_attribute3
    ,p_attribute4                   => l_attribute4
    ,p_attribute5                   => l_attribute5
    ,p_attribute6                   => l_attribute6
    ,p_attribute7                   => l_attribute7
    ,p_attribute8                   => l_attribute8
    ,p_attribute9                   => l_attribute9
    ,p_attribute10                  => l_attribute10
    ,p_attribute11                  => l_attribute11
    ,p_attribute12                  => l_attribute12
    ,p_attribute13                  => l_attribute13
    ,p_attribute14                  => l_attribute14
    ,p_attribute15                  => l_attribute15
    ,p_attribute16                  => l_attribute16
    ,p_attribute17                  => l_attribute17
    ,p_attribute18                  => l_attribute18
    ,p_attribute19                  => l_attribute19
    ,p_attribute20                  => l_attribute20
    ,p_period_of_incapacity_id      => null
    ,p_ssp1_issued                  => null
    ,p_maternity_id                 => null
    ,p_sickness_start_date          => null
    ,p_sickness_end_date            => null
    ,p_pregnancy_related_illness    => null
    ,p_reason_for_notification_dela => null
    ,p_accept_late_notification_fla => null
    ,p_linked_absence_id            => null
    ,p_batch_id                     => null
    ,p_create_element_entry         => null
    ,p_abs_information_category     => p_abs_information_category
    ,p_abs_information1             => l_abs_information1
    ,p_abs_information2             => l_abs_information2
    ,p_abs_information3             => l_abs_information3
    ,p_abs_information4             => l_abs_information4
    ,p_abs_information5             => l_abs_information5
    ,p_abs_information6             => l_abs_information6
    ,p_abs_information7             => l_abs_information7
    ,p_abs_information8             => l_abs_information8
    ,p_abs_information9             => l_abs_information9
    ,p_abs_information10            => l_abs_information10
    ,p_abs_information11            => l_abs_information11
    ,p_abs_information12            => l_abs_information12
    ,p_abs_information13            => l_abs_information13
    ,p_abs_information14            => l_abs_information14
    ,p_abs_information15            => l_abs_information15
    ,p_abs_information16            => l_abs_information16
    ,p_abs_information17            => l_abs_information17
    ,p_abs_information18            => l_abs_information18
    ,p_abs_information19            => l_abs_information19
    ,p_abs_information20            => l_abs_information20
    ,p_abs_information21            => l_abs_information21
    ,p_abs_information22            => l_abs_information22
    ,p_abs_information23            => l_abs_information23
    ,p_abs_information24            => l_abs_information24
    ,p_abs_information25            => l_abs_information25
    ,p_abs_information26            => l_abs_information26
    ,p_abs_information27            => l_abs_information27
    ,p_abs_information28            => l_abs_information28
    ,p_abs_information29            => l_abs_information29
    ,p_abs_information30            => l_abs_information30
    ,p_absence_case_id              => null
    ,p_absence_attendance_id        => l_absence_attendance_id1
    ,p_object_version_number        => l_object_version_number
    ,p_occurrence                   => l_occurrence
    ,p_return_status                => l_return_status);

  IF l_return_status = 'E' THEN
    IF (fnd_msg_pub.count_msg > 0) THEN
      FOR i IN 1..fnd_msg_pub.count_msg
      LOOP

        fnd_msg_pub.get
         (p_msg_index     => i,
          p_encoded       => 'F',
          p_data          => l_msg_data,
          p_msg_index_out => l_msg_count
         );

        log_statement(c_method,'l_msg_data: ' || l_msg_data);

        --
        -- Skip the following warning messages
        --
        IF upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_EMP_ABS_SHORT_DURATION')) OR 
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_ABS_HOUR_LESS_DURATION')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_EMP_NOT_ENTITLED')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_RUNNING_ZERO')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_ABSENCE_OVERLAP')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_ABS_DAY_AFTER')) THEN

          null;

        ELSE

          register_user_error
            (p_package_name => gc_log.package_name
            ,p_proc_name    => c_method
            ,p_message      => l_msg_data);

          l_api_errors := true;

        END IF;

        l_msg_data   := null;
        l_msg_count  := 0;
      END LOOP;
    END IF;
  END IF;

  IF l_api_errors THEN
    xxas_com_api_result_pkg.raise_registered_user_errors;
  END IF;

  --
  -- Issue a rollback
  --
  rollback to before_api_validation;

  IF l_item_key is null THEN

    launch_workflow
      (p_person_id            => p_person_id
      ,p_transaction_id       => l_transaction_id
      ,p_comments_to_approver => p_comments_to_approver
      ,p_called_from          => p_called_from
      ,p_ame_item_type        => p_ame_item_type
      ,p_ame_app_id           => p_ame_app_id
      ,p_ame_trans_type       => p_ame_trans_type
      ,p_ame_process_name     => p_ame_process_name
      ,p_item_key             => l_item_key
      ,p_activity_id          => l_activity_id);

  END IF;

  IF l_transaction_id is not null THEN

    OPEN  csr_transaction_status(cp_transaction_id => l_transaction_id);
    FETCH csr_transaction_status INTO l_transaction_status;
    CLOSE csr_transaction_status;

  END IF;

  IF l_transaction_status is null OR 
     l_transaction_status = 'W' THEN

    hr_trn_upd.upd(p_transaction_id => l_transaction_id
                  ,p_validate       => false
                  ,p_status         => 'W'
                  ,p_item_key       => l_item_key);

  ELSE

    hr_trn_upd.upd(p_transaction_id => l_transaction_id
                  ,p_validate       => false
                  ,p_status         => l_transaction_status
                  ,p_item_key       => l_item_key);

  END IF;

  xxas_selfservice_workflow_pkg.set_item_attr_text 
    (p_item_type => p_ame_item_type
    ,p_item_key  => l_item_key
    ,p_name      => 'HR_RUNTIME_APPROVAL_REQ_FLAG'
    ,p_value     => 'YES_DYNAMIC');

  workflow_review_page
    (p_item_type   => p_ame_item_type
    ,p_item_key    => l_item_key
    ,p_next_result => 'NEXT');

  p_transaction_id        := l_transaction_id;
  p_transaction_step_id   := l_transaction_step_id;
  p_absence_attendance_id := l_absence_attendance_id;

  log_end(c_method);

EXCEPTION
  
  WHEN xxas_com_api_result_pkg.xxas_com_api_exception
  THEN
    rollback to before_api_validation;
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN hr_utility.hr_error
  THEN
    rollback to before_api_validation;
    register_user_error
      (p_package_name => gc_log.package_name
      ,p_proc_name    => c_method
      ,p_message      => hr_utility.get_message);
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN others
  THEN
    err_code := SQLCODE;
    err_msg  := SUBSTR(SQLERRM, 1, 200);

    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT'
                                 ,'Error Code: ' || err_code || ' and Error Message: ' || err_msg))));

END create_absence_event;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_absence_form_handler
-- ------------------------------------------------------------------------ *
PROCEDURE create_absence_form_handler 
  (p_function_id    IN  number
  ,p_form_data_xml  IN  XMLType
  ,p_user_submitted IN  varchar2 DEFAULT g_NO
  ,p_response_xml   OUT XMLType)

IS

  CURSOR csr_data
  IS
  SELECT r.nam field_name
        ,nvl(r.val1, nvl(r.val2, r.val3)) val
        ,r.column_value field_xml
  FROM (SELECT x.column_value.getRootElement() nam
              ,extract(x.column_value, '/*/text()').getCLOBVal() val1
              ,extract(x.column_value, '/*/*/text()').getCLOBVal() val2
              ,extract(x.column_value, '/*/*/id/text()').getCLOBVal() val3
              ,x.column_value
        FROM  TABLE(XMLSequence
                     (extract
                       (p_form_data_xml
                       ,'//*'
                     || decode('Y'
                              ,'Y', '[not(ancestor-or-self::rows) '
                                  , '[not(self::rows) ')
                     || ' and not(self::root)
                          and not(self::postType)
                          and not(self::submittedFieldId)
                          and not(self::value)
                          and not(self::id)
                          and not(self::label)
                          and not(self::description)]'))) x) r;

  CURSOR csr_date_time_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dtr.start_date,'"') start_date
        ,replace(dtr.end_date,'"') end_date
        ,replace(dtr.start_time,'"') start_time
        ,replace(dtr.end_time,'"') end_time
  FROM   XMLTable
          ('//DATE_TIME_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
                  ,start_time      varchar2(10)     path '//startTime'
                  ,end_time        varchar2(10)     path '//endTime'
          ) dtr;

  CURSOR csr_date_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dr.start_date,'"') start_date
        ,replace(dr.end_date,'"') end_date
  FROM   XMLTable
          ('//DATE_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
          ) dr;

  CURSOR csr_employee_id(cp_employees_xml IN XMLType) IS
  SELECT emp.id
  FROM   XMLTable
          ('/EMPLOYEES/value'
          PASSING  cp_employees_xml
          COLUMNS  xml  XMLType           path '.'
                  ,id   number            path '//id'
          ) emp;

  c_method            CONSTANT varchar2(30) := 'create_absence_form_handler';
  l_person_id                  number;
  l_assignment_id              number;
  l_override_assignment_id     number;
  l_transaction_id             number;
  l_transaction_step_id        number;
  l_absence_attendance_id      number;
  l_leave_status               varchar2(4000);
  l_absence_type_id            number;
  l_abs_attendance_reason_id   number;
  l_effective_date             date;
  l_date_notification          date;
  l_date_start                 date;
  l_time_start                 varchar2(5);
  l_date_end                   date;
  l_time_end                   varchar2(5);
  l_duration_type              varchar2(10);
  l_duration_days              per_absence_attendances.absence_days%TYPE;
  l_duration_hours             per_absence_attendances.absence_hours%TYPE;
  l_attribute_category         varchar2(4000);
  l_attribute1                 varchar2(4000);
  l_attribute2                 varchar2(4000);
  l_attribute3                 varchar2(4000);
  l_attribute4                 varchar2(4000);
  l_attribute5                 varchar2(4000);
  l_attribute6                 varchar2(4000);
  l_attribute7                 varchar2(4000);
  l_attribute8                 varchar2(4000);
  l_attribute9                 varchar2(4000);
  l_attribute10                varchar2(4000);
  l_attribute11                varchar2(4000);
  l_attribute12                varchar2(4000);
  l_attribute13                varchar2(4000);
  l_attribute14                varchar2(4000);
  l_attribute15                varchar2(4000);
  l_attribute16                varchar2(4000);
  l_attribute17                varchar2(4000);
  l_attribute18                varchar2(4000);
  l_attribute19                varchar2(4000);
  l_attribute20                varchar2(4000);
  l_abs_information_category   varchar2(4000);
  l_abs_information1           varchar2(4000);
  l_abs_information2           varchar2(4000);
  l_abs_information3           varchar2(4000);
  l_abs_information4           varchar2(4000);
  l_abs_information5           varchar2(4000);
  l_abs_information6           varchar2(4000);
  l_abs_information7           varchar2(4000);
  l_abs_information8           varchar2(4000);
  l_abs_information9           varchar2(4000);
  l_abs_information10          varchar2(4000);
  l_abs_information11          varchar2(4000);
  l_abs_information12          varchar2(4000);
  l_abs_information13          varchar2(4000);
  l_abs_information14          varchar2(4000);
  l_abs_information15          varchar2(4000);
  l_abs_information16          varchar2(4000);
  l_abs_information17          varchar2(4000);
  l_abs_information18          varchar2(4000);
  l_abs_information19          varchar2(4000);
  l_abs_information20          varchar2(4000);
  l_abs_information21          varchar2(4000);
  l_abs_information22          varchar2(4000);
  l_abs_information23          varchar2(4000);
  l_abs_information24          varchar2(4000);
  l_abs_information25          varchar2(4000);
  l_abs_information26          varchar2(4000);
  l_abs_information27          varchar2(4000);
  l_abs_information28          varchar2(4000);
  l_abs_information29          varchar2(4000);
  l_abs_information30          varchar2(4000);
  l_replacement_person_id      number;
  l_comments_to_approver       varchar2(4000);
  l_status                     varchar2(10);
  l_action                     varchar2(30);
  l_date_time_range_xml        XMLType;
  l_date_range_xml             XMLType;
  l_attachments_xml            XMLType;
  l_employees_xml              XMLType;
  l_dtr_values_rec             csr_date_time_range_values%ROWTYPE;
  l_dr_values_rec              csr_date_range_values%ROWTYPE;
  l_source_block               varchar2(2000);
  l_saved_search_id            number;
  l_data                       xxas_emp_view_ui_pkg.xxas_data_table;
  l_called_from                fnd_form_functions.function_name%TYPE;
  l_ame_item_type              varchar2(30);
  l_ame_app_id                 number;
  l_ame_trans_type             varchar2(30);
  l_ame_process_name           varchar2(30);
  err_code                     varchar2(200);
  err_msg                      varchar2(200);
  l_save_attachments           varchar2(10);
  l_save_attach_as_sup_docs    varchar2(10);
  l_called_view                varchar2(30);
  l_redirect_notif             varchar2(2);
  l_rule                       varchar2(30);
  l_grantee                    varchar2(30);
  l_show_review_page           varchar2(2);
  l_absence_days               number;
  l_absence_hours              number;
  l_use_formula                number;
  l_min_max_failure            varchar2(4000);
  l_warning_or_error           varchar2(4000);
  l_page_error_msg             varchar2(4000);
  l_cal_duration               per_absence_attendances.absence_days%TYPE;
  l_workflow_info              xxas_cal_util_pkg.workflow_info;
  l_current_tran_submit        hr_api_transactions.status%TYPE;
  l_trans_error_message        varchar2(4000);
BEGIN

  log_start(c_method, p_function_id, p_user_submitted);

  xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

/*  IF p_user_submitted = 'Y' THEN*/

  l_data := xxas_emp_view_ui_pkg.transform_jsonxml_form_data
              (p_form_data_xml);

  -- Retrieve the form values
  IF l_data.exists('PERSON_ID') THEN
    l_person_id := l_data('PERSON_ID');
  END IF;

  IF l_data.exists('ASSIGNMENT_ID') THEN
    l_assignment_id := l_data('ASSIGNMENT_ID');
  END IF;

  IF l_data.exists('OVERRIDE_ASSIGNMENT_ID') THEN
    l_override_assignment_id := l_data('OVERRIDE_ASSIGNMENT_ID');
  END IF;

  IF l_override_assignment_id is not null THEN
    l_assignment_id := l_override_assignment_id;
  END IF;

/*  IF l_data.exists('ABSENCE_ATTENDANCE_ID') THEN
    l_absence_attendance_id := l_data('ABSENCE_ATTENDANCE_ID');
  END IF;*/

  IF l_data.exists('TRANSACTION_ID') THEN
    l_transaction_id := l_data('TRANSACTION_ID');
  END IF;

  IF l_data.exists('TRANSACTION_STEP_ID') THEN
    l_transaction_step_id := l_data('TRANSACTION_STEP_ID');
  END IF;

  IF l_data.exists('SOURCE_BLOCK') THEN
    l_source_block := l_data('SOURCE_BLOCK');
  END IF;

  IF l_data.exists('SAVED_SEARCH_ID') THEN
    l_saved_search_id := l_data('SAVED_SEARCH_ID');
  END IF;

  IF l_data.exists('LEAVE_STATUS') THEN
    l_leave_status := l_data('LEAVE_STATUS');
  END IF;

  IF l_data.exists('ABSENCE_TYPE_ID') THEN
    l_absence_type_id := l_data('ABSENCE_TYPE_ID');
  END IF;

  IF l_data.exists('ABS_ATTENDANCE_REASON_ID') THEN
    l_abs_attendance_reason_id := l_data('ABS_ATTENDANCE_REASON_ID');
  END IF;

  IF l_data.exists('DATE_NOTIFICATION') THEN
    l_date_notification := fnd_date.canonical_to_date(l_data('DATE_NOTIFICATION'));
  END IF;

  IF l_data.exists('DURATION_TYPE') THEN
    l_duration_type := l_data('DURATION_TYPE');
  END IF;

  IF l_data.exists('DURATION_DAYS') THEN
    l_duration_days := l_data('DURATION_DAYS');
  END IF;

  IF l_data.exists('DURATION_HOURS') THEN
    l_duration_hours := l_data('DURATION_HOURS');
  END IF;
  
  IF l_data.exists('CALLED_VIEW') THEN
    l_called_view := l_data('CALLED_VIEW');
  END IF;
  
  --
  -- Convert the XML form data into the xxas_data_table structure.
  --
  IF p_form_data_xml is not null THEN

    FOR csr_rec IN csr_data LOOP
      IF csr_rec.field_name is not null THEN
        IF csr_rec.field_name = 'ATTACHMENTS' THEN
          l_attachments_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'DATE_TIME_RANGE' AND 
              (l_duration_type is not null AND
               l_duration_type IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_time_range_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'DATE_RANGE' AND
              (l_duration_type is not null AND
               l_duration_type NOT IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_range_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'EMPLOYEES' THEN
          l_employees_xml := csr_rec.field_xml;
        END IF;
      END IF;
    END LOOP;

  END IF;

  IF l_date_time_range_xml is not null THEN

    OPEN  csr_date_time_range_values(l_date_time_range_xml);
    FETCH csr_date_time_range_values INTO l_dtr_values_rec;
    CLOSE csr_date_time_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dtr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dtr_values_rec.end_date);
    l_time_start := l_dtr_values_rec.start_time;
    l_time_end   := l_dtr_values_rec.end_time;

  ELSIF l_date_range_xml is not null THEN

    OPEN  csr_date_range_values(l_date_range_xml);
    FETCH csr_date_range_values INTO l_dr_values_rec;
    CLOSE csr_date_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dr_values_rec.end_date);

  END IF;

  IF l_data.exists('ATTRIBUTE_CATEGORY') THEN
    l_attribute_category := l_data('ATTRIBUTE_CATEGORY');
  END IF;

  IF l_data.exists('ATTRIBUTE1') THEN
    l_attribute1 := l_data('ATTRIBUTE1');
  END IF;

  IF l_data.exists('ATTRIBUTE2') THEN
    l_attribute2 := l_data('ATTRIBUTE2');
  END IF;

  IF l_data.exists('ATTRIBUTE3') THEN
    l_attribute3 := l_data('ATTRIBUTE3');
  END IF;

  IF l_data.exists('ATTRIBUTE4') THEN
    l_attribute4 := l_data('ATTRIBUTE4');
  END IF;

  IF l_data.exists('ATTRIBUTE5') THEN
    l_attribute5 := l_data('ATTRIBUTE5');
  END IF;

  IF l_data.exists('ATTRIBUTE6') THEN
    l_attribute6 := l_data('ATTRIBUTE6');
  END IF;

  IF l_data.exists('ATTRIBUTE7') THEN
    l_attribute7 := l_data('ATTRIBUTE7');
  END IF;

  IF l_data.exists('ATTRIBUTE8') THEN
    l_attribute8 := l_data('ATTRIBUTE8');
  END IF;

  IF l_data.exists('ATTRIBUTE9') THEN
    l_attribute9 := l_data('ATTRIBUTE9');
  END IF;

  IF l_data.exists('ATTRIBUTE10') THEN
    l_attribute10 := l_data('ATTRIBUTE10');
  END IF;

  IF l_data.exists('ATTRIBUTE11') THEN
    l_attribute11 := l_data('ATTRIBUTE11');
  END IF;

  IF l_data.exists('ATTRIBUTE12') THEN
    l_attribute12 := l_data('ATTRIBUTE12');
  END IF;

  IF l_data.exists('ATTRIBUTE13') THEN
    l_attribute13 := l_data('ATTRIBUTE13');
  END IF;

  IF l_data.exists('ATTRIBUTE14') THEN
    l_attribute14 := l_data('ATTRIBUTE14');
  END IF;

  IF l_data.exists('ATTRIBUTE15') THEN
    l_attribute15 := l_data('ATTRIBUTE15');
  END IF;

  IF l_data.exists('ATTRIBUTE16') THEN
    l_attribute16 := l_data('ATTRIBUTE16');
  END IF;

  IF l_data.exists('ATTRIBUTE17') THEN
    l_attribute17 := l_data('ATTRIBUTE17');
  END IF;

  IF l_data.exists('ATTRIBUTE18') THEN
    l_attribute18 := l_data('ATTRIBUTE18');
  END IF;

  IF l_data.exists('ATTRIBUTE19') THEN
    l_attribute19 := l_data('ATTRIBUTE19');
  END IF;

  IF l_data.exists('ATTRIBUTE20') THEN
    l_attribute20 := l_data('ATTRIBUTE20');
  END IF;

  IF l_data.exists('ABS_INFORMATION_CATEGORY') THEN
    l_abs_information_category := l_data('ABS_INFORMATION_CATEGORY');
  END IF;

  IF l_data.exists('ABS_INFORMATION1') THEN
    l_abs_information1 := l_data('ABS_INFORMATION1');
  END IF;

  IF l_data.exists('ABS_INFORMATION2') THEN
    l_abs_information2 := l_data('ABS_INFORMATION2');
  END IF;

  IF l_data.exists('ABS_INFORMATION3') THEN
    l_abs_information3 := l_data('ABS_INFORMATION3');
  END IF;

  IF l_data.exists('ABS_INFORMATION4') THEN
    l_abs_information4 := l_data('ABS_INFORMATION4');
  END IF;

  IF l_data.exists('ABS_INFORMATION5') THEN
    l_abs_information5 := l_data('ABS_INFORMATION5');
  END IF;

  IF l_data.exists('ABS_INFORMATION6') THEN
    l_abs_information6 := l_data('ABS_INFORMATION6');
  END IF;

  IF l_data.exists('ABS_INFORMATION7') THEN
    l_abs_information7 := l_data('ABS_INFORMATION7');
  END IF;

  IF l_data.exists('ABS_INFORMATION8') THEN
    l_abs_information8 := l_data('ABS_INFORMATION8');
  END IF;

  IF l_data.exists('ABS_INFORMATION9') THEN
    l_abs_information9 := l_data('ABS_INFORMATION9');
  END IF;

  IF l_data.exists('ABS_INFORMATION10') THEN
    l_abs_information10 := l_data('ABS_INFORMATION10');
  END IF;

  IF l_data.exists('ABS_INFORMATION11') THEN
    l_abs_information11 := l_data('ABS_INFORMATION11');
  END IF;

  IF l_data.exists('ABS_INFORMATION12') THEN
    l_abs_information12 := l_data('ABS_INFORMATION12');
  END IF;

  IF l_data.exists('ABS_INFORMATION13') THEN
    l_abs_information13 := l_data('ABS_INFORMATION13');
  END IF;

  IF l_data.exists('ABS_INFORMATION14') THEN
    l_abs_information14 := l_data('ABS_INFORMATION14');
  END IF;

  IF l_data.exists('ABS_INFORMATION15') THEN
    l_abs_information15 := l_data('ABS_INFORMATION15');
  END IF;

  IF l_data.exists('ABS_INFORMATION16') THEN
    l_abs_information16 := l_data('ABS_INFORMATION16');
  END IF;

  IF l_data.exists('ABS_INFORMATION17') THEN
    l_abs_information17 := l_data('ABS_INFORMATION17');
  END IF;

  IF l_data.exists('ABS_INFORMATION18') THEN
    l_abs_information18 := l_data('ABS_INFORMATION18');
  END IF;

  IF l_data.exists('ABS_INFORMATION19') THEN
    l_abs_information19 := l_data('ABS_INFORMATION19');
  END IF;

  IF l_data.exists('ABS_INFORMATION20') THEN
    l_abs_information20 := l_data('ABS_INFORMATION20');
  END IF;

  IF l_data.exists('ABS_INFORMATION21') THEN
    l_abs_information21 := l_data('ABS_INFORMATION21');
  END IF;

  IF l_data.exists('ABS_INFORMATION22') THEN
    l_abs_information22 := l_data('ABS_INFORMATION22');
  END IF;

  IF l_data.exists('ABS_INFORMATION23') THEN
    l_abs_information23 := l_data('ABS_INFORMATION23');
  END IF;

  IF l_data.exists('ABS_INFORMATION24') THEN
    l_abs_information24 := l_data('ABS_INFORMATION24');
  END IF;

  IF l_data.exists('ABS_INFORMATION25') THEN
    l_abs_information25 := l_data('ABS_INFORMATION25');
  END IF;

  IF l_data.exists('ABS_INFORMATION26') THEN
    l_abs_information26 := l_data('ABS_INFORMATION26');
  END IF;

  IF l_data.exists('ABS_INFORMATION27') THEN
    l_abs_information27 := l_data('ABS_INFORMATION27');
  END IF;

  IF l_data.exists('ABS_INFORMATION28') THEN
    l_abs_information28 := l_data('ABS_INFORMATION28');
  END IF;

  IF l_data.exists('ABS_INFORMATION29') THEN
    l_abs_information29 := l_data('ABS_INFORMATION29');
  END IF;

  IF l_data.exists('ABS_INFORMATION30') THEN
    l_abs_information30 := l_data('ABS_INFORMATION30');
  END IF;

  IF l_data.exists('REPLACEMENT_PERSON_ID') THEN
    l_replacement_person_id := l_data('REPLACEMENT_PERSON_ID');
  END IF;

  IF l_data.exists('COMMENTS_TO_APPROVER') THEN
    l_comments_to_approver := l_data('COMMENTS_TO_APPROVER');
  END IF;

  l_effective_date := nvl(l_date_start,trunc(sysdate));

  IF l_person_id is null THEN

    IF l_employees_xml is not null THEN

      OPEN  csr_employee_id(l_employees_xml);
      FETCH csr_employee_id INTO l_person_id;
      CLOSE csr_employee_id;

    END IF;
    
    l_assignment_id := xxas_util_pkg.get_primary_worker_asg(l_person_id,nvl(l_date_start,l_effective_date));


    IF l_override_assignment_id is not null THEN
      l_assignment_id := l_override_assignment_id;
    END IF;

  END IF;

  l_called_from := xxas_com_sec_pkg.get_function_parameter
                     (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                     ,p_parameter_name => 'pCalledFrom');

  l_ame_item_type    := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pItemType');
  l_ame_app_id       := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pAMEAppId');
  l_ame_trans_type   := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pAMETranType');
  l_ame_process_name := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pProcessName');

  l_cal_duration := xxas_leave_of_absence_pkg.calculate_abs_duration_full
                      (p_business_group_id => xxas_per_pkg.get_business_group_id(l_person_id)
                      ,p_assignment_id     => l_assignment_id
                      ,p_effective_date    => l_effective_date
                      ,p_absence_type_id   => l_absence_type_id
                      ,p_date_start        => l_date_start
                      ,p_date_end          => l_date_end
                      ,p_time_start        => l_time_start
                      ,p_time_end          => l_time_end
                      ,p_absence_days      => l_absence_days
                      ,p_absence_hours     => l_absence_hours
                      ,p_use_formula       => l_use_formula
                      ,p_min_max_failure   => l_min_max_failure
                      ,p_warning_or_error  => l_warning_or_error
                      ,p_page_error_msg    => l_page_error_msg);	  

  IF l_page_error_msg is not null THEN

    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => null
      ,p_proc_name    => null
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT',l_page_error_msg)
                              )
                           ));

  END IF;

  create_absence_event(p_person_id                => l_person_id
                      ,p_assignment_id            => l_assignment_id
                      ,p_date_start               => l_date_start
                      ,p_date_end                 => l_date_end
                      ,p_time_start               => l_time_start
                      ,p_time_end                 => l_time_end
                      ,p_duration_type            => l_duration_type
                      ,p_duration_days            => l_duration_days
                      ,p_duration_hours           => l_duration_hours
                      ,p_effective_date           => l_effective_date
                      ,p_called_from              => l_called_from
                      ,p_leave_status             => l_leave_status
                      ,p_absence_type_id          => l_absence_type_id
                      ,p_abs_attendance_reason_id => l_abs_attendance_reason_id
                      ,p_attribute_category       => l_attribute_category
                      ,p_attribute1               => l_attribute1
                      ,p_attribute2               => l_attribute2
                      ,p_attribute3               => l_attribute3
                      ,p_attribute4               => l_attribute4
                      ,p_attribute5               => l_attribute5
                      ,p_attribute6               => l_attribute6
                      ,p_attribute7               => l_attribute7
                      ,p_attribute8               => l_attribute8
                      ,p_attribute9               => l_attribute9
                      ,p_attribute10              => l_attribute10
                      ,p_attribute11              => l_attribute11
                      ,p_attribute12              => l_attribute12
                      ,p_attribute13              => l_attribute13
                      ,p_attribute14              => l_attribute14
                      ,p_attribute15              => l_attribute15
                      ,p_attribute16              => l_attribute16
                      ,p_attribute17              => l_attribute17
                      ,p_attribute18              => l_attribute18
                      ,p_attribute19              => l_attribute19
                      ,p_attribute20              => l_attribute20
                      ,p_abs_information_category => l_abs_information_category
                      ,p_abs_information1         => l_abs_information1
                      ,p_abs_information2         => l_abs_information2
                      ,p_abs_information3         => l_abs_information3
                      ,p_abs_information4         => l_abs_information4
                      ,p_abs_information5         => l_abs_information5
                      ,p_abs_information6         => l_abs_information6
                      ,p_abs_information7         => l_abs_information7
                      ,p_abs_information8         => l_abs_information8
                      ,p_abs_information9         => l_abs_information9
                      ,p_abs_information10        => l_abs_information10
                      ,p_abs_information11        => l_abs_information11
                      ,p_abs_information12        => l_abs_information12
                      ,p_abs_information13        => l_abs_information13
                      ,p_abs_information14        => l_abs_information14
                      ,p_abs_information15        => l_abs_information15
                      ,p_abs_information16        => l_abs_information16
                      ,p_abs_information17        => l_abs_information17
                      ,p_abs_information18        => l_abs_information18
                      ,p_abs_information19        => l_abs_information19
                      ,p_abs_information20        => l_abs_information20
                      ,p_abs_information21        => l_abs_information21
                      ,p_abs_information22        => l_abs_information22
                      ,p_abs_information23        => l_abs_information23
                      ,p_abs_information24        => l_abs_information24
                      ,p_abs_information25        => l_abs_information25
                      ,p_abs_information26        => l_abs_information26
                      ,p_abs_information27        => l_abs_information27
                      ,p_abs_information28        => l_abs_information28
                      ,p_abs_information29        => l_abs_information29
                      ,p_abs_information30        => l_abs_information30
                      ,p_replacement_person_id    => l_replacement_person_id
                      ,p_comments_to_approver     => l_comments_to_approver
                      ,p_date_notification        => l_date_notification
                      ,p_ame_item_type            => l_ame_item_type
                      ,p_ame_app_id               => l_ame_app_id
                      ,p_ame_trans_type           => l_ame_trans_type
                      ,p_ame_process_name         => l_ame_process_name
                      ,p_absence_attendance_id    => l_absence_attendance_id
                      ,p_transaction_id           => l_transaction_id
                      ,p_transaction_step_id      => l_transaction_step_id);

   /* *************************************************************
     Check for vacation rules setup
     If vacation rules is setup then add the values
     to Transaction document. It will be used in absence user hook
     to create vacation rules.
     ************************************************************* */
     
  IF xxas_com_sec_pkg.test_instance('XXAS_ABS_VAC_RULE_SETUP',l_person_id) = 'Y' THEN
    
    IF l_data.exists('REDIRECT_NOTIF') THEN
      l_redirect_notif := l_data('REDIRECT_NOTIF');
    END IF;
  
    IF l_data.exists('RULE') THEN
      l_rule := l_data('RULE');
    END IF;
  
    IF l_data.exists('GRANTEE') THEN
      l_grantee := l_data('GRANTEE');
    END IF;

    --Check for retro absence. If not then proceed further.
    IF nvl(l_date_end,trunc(sysdate)) >= trunc(sysdate) THEN
      -- set transaction values
      hr_transaction_api.set_varchar2_value(p_transaction_step_id => l_transaction_step_id
                                           ,p_person_id           => l_person_id
                                           ,p_name                => 'P_SETUP_VAC_RULE'
                                           ,p_value               => l_redirect_notif);
      
      hr_transaction_api.set_varchar2_value(p_transaction_step_id => l_transaction_step_id
                                           ,p_person_id           => l_person_id
                                           ,p_name                => 'P_RULE'
                                           ,p_value               => l_rule);
                                           
      hr_transaction_api.set_varchar2_value(p_transaction_step_id => l_transaction_step_id
                                           ,p_person_id           => l_person_id
                                           ,p_name                => 'P_ASSIGNEE'
                                           ,p_value               => l_grantee);
    ELSIF l_redirect_notif = 'Y' THEN
      -- Log information message.
      hr_transaction_api.set_varchar2_value(p_transaction_step_id => l_transaction_step_id
                                           ,p_person_id           => l_person_id
                                           ,p_name                => 'P_SETUP_VAC_RULE'
                                           ,p_value               => 'E');
    END IF;
  END IF;

  save_file_attachments
    (p_attachments_xml    => l_attachments_xml
    ,p_data               => l_data
    ,p_instance_pk1_value => l_person_id
    ,p_instance_pk2_value => null
    ,p_instance_pk3_value => null
    ,p_instance_pk4_value => l_transaction_id
    ,p_instance_pk5_value => l_transaction_step_id
    ,p_entity_name        => 'PQH_SS_ATTACHMENT'
    ,p_pk1_value          => 'INSTANCE_PK4_VALUE'
    ,p_category_code      => 'MISC');

  -- Added for GIT#2626
  l_show_review_page := nvl(xxas_com_sec_pkg.get_function_parameter
                                   (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                                   ,p_parameter_name => 'pShowApprovalPageFlag')
                           ,'N');

  IF l_show_review_page = 'N' THEN
    xxas_emp_view_ui_pkg.register_result_details
    (p_transaction_id => l_transaction_id);
  END IF;

  l_workflow_info := xxas_cal_util_pkg.get_workflow_info
                       (p_transaction_id => l_transaction_id);

  IF l_workflow_info.item_type is not null AND
     l_workflow_info.item_key  is not null THEN

    l_current_tran_submit := xxas_selfservice_workflow_pkg.get_wf_attribute_text
                               (p_item_type      => l_workflow_info.item_type
                               ,p_item_key       => l_workflow_info.item_key
                               ,p_attribute_name => 'TRAN_SUBMIT');

    IF l_workflow_info.trans_status = 'W' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'W');

    ELSIF l_workflow_info.trans_status = 'RI' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'RI');

    ELSIF l_workflow_info.trans_status = 'Y' AND 
          l_current_tran_submit        = 'RI' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'RI');

    ELSIF l_workflow_info.trans_status = 'Y' AND 
          l_current_tran_submit        = 'Y' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'Y');

    END IF;

  END IF;

  IF NOT (l_show_review_page = 'Y' OR approval_required) THEN

    log_statement(c_method,'approval_required is false');

    l_status := xxas_xml_node_util_pkg.get_node_value(l_transaction_id, 'TxnStatus', 'Transaction/TransCtx');
    l_action := xxas_xml_node_util_pkg.get_node_value(l_transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode');

    l_save_attachments := xxas_com_sec_pkg.get_function_parameter
                            (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                            ,p_parameter_name => 'pSaveAttachments');

    l_save_attach_as_sup_docs := xxas_com_sec_pkg.get_function_parameter
                                   (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                                   ,p_parameter_name => 'pSaveAttachmentsAsSupportingDocs');

    p_response_xml := submit_emp_view_row
                        (p_effective_date          => trunc(sysdate)
                        ,p_person_id               => l_person_id
                        ,p_absence_attendance_id   => l_absence_attendance_id
                        ,p_transaction_id          => l_transaction_id
                        ,p_transaction_step_id     => l_transaction_step_id
                        ,p_status                  => l_status
                        ,p_action                  => l_action
                        ,p_approvers               => 'N'
                        ,p_source_block            => l_source_block
                        ,p_saved_search_id         => l_saved_search_id
                        ,p_save_attachments        => l_save_attachments
                        ,p_save_attach_as_sup_docs => l_save_attach_as_sup_docs);

  ELSE

    log_statement(c_method,'approval_required is true'); 

    p_response_xml := go_to_review_page_xml
                        (p_person_id             => l_person_id
                        ,p_absence_attendance_id => l_absence_attendance_id
                        ,p_transaction_id        => l_transaction_id
                        ,p_transaction_step_id   => l_transaction_step_id
                        ,p_source_block          => l_source_block
                        ,p_saved_search_id       => l_saved_search_id);

  END IF;

  --Check and raise any workflow errors
  raise_workflow_error(l_transaction_id);
  
  log_end(c_method);

EXCEPTION

  WHEN xxas_com_api_result_pkg.xxas_com_api_exception
  THEN
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN others
  THEN
    err_code := SQLCODE;
    err_msg  := SUBSTR(SQLERRM, 1, 200);

    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT'
                                 ,'Error Code: ' || err_code || ' and Error Message: ' || err_msg))));

END create_absence_form_handler;

--
-- ------------------------------------------------------------------------ *
-- Name    : update_absence_event
-- ------------------------------------------------------------------------ *
PROCEDURE update_absence_event
  (p_person_id                IN number
  ,p_assignment_id            IN number
  ,p_date_start               IN date
  ,p_date_end                 IN date
  ,p_time_start               IN varchar2
  ,p_time_end                 IN varchar2
  ,p_duration_type            IN varchar2
  ,p_duration_days            IN number
  ,p_duration_hours           IN number
  ,p_effective_date           IN date
  ,p_called_from              IN varchar2
  ,p_leave_status             IN varchar2
  ,p_absence_type_id          IN number
  ,p_abs_attendance_reason_id IN number
  ,p_attribute_category       IN varchar2
  ,p_attribute1               IN varchar2
  ,p_attribute2               IN varchar2
  ,p_attribute3               IN varchar2
  ,p_attribute4               IN varchar2
  ,p_attribute5               IN varchar2
  ,p_attribute6               IN varchar2
  ,p_attribute7               IN varchar2
  ,p_attribute8               IN varchar2
  ,p_attribute9               IN varchar2
  ,p_attribute10              IN varchar2
  ,p_attribute11              IN varchar2
  ,p_attribute12              IN varchar2
  ,p_attribute13              IN varchar2
  ,p_attribute14              IN varchar2
  ,p_attribute15              IN varchar2
  ,p_attribute16              IN varchar2
  ,p_attribute17              IN varchar2
  ,p_attribute18              IN varchar2
  ,p_attribute19              IN varchar2
  ,p_attribute20              IN varchar2
  ,p_abs_information_category IN varchar2
  ,p_abs_information1         IN varchar2
  ,p_abs_information2         IN varchar2
  ,p_abs_information3         IN varchar2
  ,p_abs_information4         IN varchar2
  ,p_abs_information5         IN varchar2
  ,p_abs_information6         IN varchar2
  ,p_abs_information7         IN varchar2
  ,p_abs_information8         IN varchar2
  ,p_abs_information9         IN varchar2
  ,p_abs_information10        IN varchar2
  ,p_abs_information11        IN varchar2
  ,p_abs_information12        IN varchar2
  ,p_abs_information13        IN varchar2
  ,p_abs_information14        IN varchar2
  ,p_abs_information15        IN varchar2
  ,p_abs_information16        IN varchar2
  ,p_abs_information17        IN varchar2
  ,p_abs_information18        IN varchar2
  ,p_abs_information19        IN varchar2
  ,p_abs_information20        IN varchar2
  ,p_abs_information21        IN varchar2
  ,p_abs_information22        IN varchar2
  ,p_abs_information23        IN varchar2
  ,p_abs_information24        IN varchar2
  ,p_abs_information25        IN varchar2
  ,p_abs_information26        IN varchar2
  ,p_abs_information27        IN varchar2
  ,p_abs_information28        IN varchar2
  ,p_abs_information29        IN varchar2
  ,p_abs_information30        IN varchar2
  ,p_replacement_person_id    IN number
  ,p_comments_to_approver     IN varchar2
  ,p_date_notification        IN date
  ,p_ame_item_type            IN varchar2
  ,p_ame_app_id               IN number
  ,p_ame_trans_type           IN varchar2
  ,p_ame_process_name         IN varchar2
  ,p_object_version_number    IN number
  ,p_absence_attendance_id    IN number
  ,p_transaction_id           IN OUT number
  ,p_transaction_step_id      IN OUT number)
  
IS

  CURSOR csr_asg(cp_assignment_id IN number) IS
  SELECT paaf.*
  FROM   per_all_assignments_f paaf
  WHERE  paaf.assignment_id = cp_assignment_id
  AND    trunc(sysdate) BETWEEN paaf.effective_start_date 
                        AND     paaf.effective_end_date
  AND    (paaf.assignment_type = 'E' 
          OR paaf.assignment_type = 'C');

  CURSOR csr_person(cp_person_id IN number) IS
  SELECT papf.*
  FROM   per_all_people_f papf
  WHERE  papf.person_id = cp_person_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y');

  CURSOR csr_login_person(cp_login_person_id IN number) IS
  SELECT papf.*
        ,paaf.supervisor_id
        ,paaf.assignment_number
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN paaf.effective_start_date 
                        AND     paaf.effective_end_date
  AND    papf.person_id = cp_login_person_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    ((nvl(papf.current_employee_flag, 'N') = 'Y' AND
           paaf.assignment_type = 'E')
          OR (nvl(papf.current_npw_flag, 'N') = 'Y' AND
              paaf.assignment_type = 'C'))
  AND    nvl(paaf.primary_flag, 'N') = 'Y';

  CURSOR csr_abs_attendance_info(cp_absence_attendance_id IN number) IS
  SELECT paa.*
  FROM   per_absence_attendances paa
  WHERE  paa.absence_attendance_id = cp_absence_attendance_id;

  CURSOR csr_get_emp_full_name(cp_person_id IN number) IS
  SELECT decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'),'G', global_name, local_name) full_name
  FROM   per_all_people_f papf
  WHERE  papf.person_id = cp_person_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y');

  CURSOR csr_get_sec_grp_leg_code(cp_person_id IN number) IS
  SELECT bg.security_group_id
        ,bg.legislation_code
  FROM   per_all_people_f papf
        ,per_business_groups_perf bg
  WHERE  papf.person_id = cp_person_id
  AND    papf.business_group_id = bg.business_group_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y');

  CURSOR csr_get_job_name(cp_job_id number) IS
  SELECT pjv.name 
  FROM   per_jobs_vl pjv
  WHERE  pjv.job_id = cp_job_id 
  AND    trunc(sysdate) BETWEEN pjv.date_from 
                        AND     nvl(pjv.date_to,trunc(sysdate));

  CURSOR csr_get_pos_name(cp_pos_id IN number) IS
  SELECT hap.name 
  FROM   hr_all_positions_f_vl hap
  WHERE  hap.position_id = cp_pos_id 
  AND    trunc(sysdate) BETWEEN nvl(hap.effective_start_date,trunc(sysdate)) 
                        AND     nvl(hap.effective_end_date,trunc(sysdate));

  CURSOR csr_get_org_info(cp_person_id IN number) IS
  SELECT hoi.org_information2
        ,hoi.org_information10
  FROM   hr_organization_information_v hoi
  WHERE  hoi.org_information_context = 'Business Group Information'
  AND    hoi.organization_id         = xxas_per_pkg.get_business_group_id(cp_person_id);

  CURSOR csr_get_abs_att_type_meaning(cp_abs_att_type_id IN number) IS
  SELECT fcl.meaning
  FROM   per_abs_attendance_types_vl paat
        ,hr_lookups fcl
  WHERE  absence_attendance_type_id = cp_abs_att_type_id
  AND    paat.absence_category      = fcl.lookup_code(+)
  AND    fcl.lookup_type(+)         = 'ABSENCE_CATEGORY';

  CURSOR csr_trans_doc(cp_person_id                IN number
                      ,cp_per_row                  IN csr_person%ROWTYPE
                      ,cp_asg_row                  IN csr_asg%ROWTYPE
                      ,cp_login_per_row            IN csr_login_person%ROWTYPE
                      ,cp_per_manager_name         IN varchar2
                      ,cp_login_per_manager_name   IN varchar2
                      ,cp_transaction_id           IN number
                      ,cp_per_sec_group_id         IN number
                      ,cp_per_leg_code             IN varchar2
                      ,cp_login_per_leg_code       IN varchar2
                      ,cp_job_name                 IN varchar2
                      ,cp_pos_name                 IN varchar2
                      ,cp_emp_generation           IN varchar2
                      ,cp_per_bg_cur_code          IN varchar2
                      ,cp_abs_row                  IN per_absence_attendances%ROWTYPE
                      ,cp_called_from              IN varchar2
                      ,cp_assignment_id            IN number
                      ,cp_effective_date           IN date
                      ,cp_leave_status             IN varchar2
                      ,cp_absence_type_id          IN number
                      ,cp_abs_attendance_reason_id IN number
                      ,cp_duration_days            IN number
                      ,cp_duration_hours           IN number
                      ,cp_date_start               IN date
                      ,cp_date_end                 IN date
                      ,cp_time_start               IN varchar2
                      ,cp_time_end                 IN varchar2
                      ,cp_attribute_category       IN varchar2
                      ,cp_attribute1               IN varchar2
                      ,cp_attribute2               IN varchar2
                      ,cp_attribute3               IN varchar2
                      ,cp_attribute4               IN varchar2
                      ,cp_attribute5               IN varchar2
                      ,cp_attribute6               IN varchar2
                      ,cp_attribute7               IN varchar2
                      ,cp_attribute8               IN varchar2
                      ,cp_attribute9               IN varchar2
                      ,cp_attribute10              IN varchar2
                      ,cp_attribute11              IN varchar2
                      ,cp_attribute12              IN varchar2
                      ,cp_attribute13              IN varchar2
                      ,cp_attribute14              IN varchar2
                      ,cp_attribute15              IN varchar2
                      ,cp_attribute16              IN varchar2
                      ,cp_attribute17              IN varchar2
                      ,cp_attribute18              IN varchar2
                      ,cp_attribute19              IN varchar2
                      ,cp_attribute20              IN varchar2
                      ,cp_abs_information_category IN varchar2
                      ,cp_abs_information1         IN varchar2
                      ,cp_abs_information2         IN varchar2
                      ,cp_abs_information3         IN varchar2
                      ,cp_abs_information4         IN varchar2
                      ,cp_abs_information5         IN varchar2
                      ,cp_abs_information6         IN varchar2
                      ,cp_abs_information7         IN varchar2
                      ,cp_abs_information8         IN varchar2
                      ,cp_abs_information9         IN varchar2
                      ,cp_abs_information10        IN varchar2
                      ,cp_abs_information11        IN varchar2
                      ,cp_abs_information12        IN varchar2
                      ,cp_abs_information13        IN varchar2
                      ,cp_abs_information14        IN varchar2
                      ,cp_abs_information15        IN varchar2
                      ,cp_abs_information16        IN varchar2
                      ,cp_abs_information17        IN varchar2
                      ,cp_abs_information18        IN varchar2
                      ,cp_abs_information19        IN varchar2
                      ,cp_abs_information20        IN varchar2
                      ,cp_abs_information21        IN varchar2
                      ,cp_abs_information22        IN varchar2
                      ,cp_abs_information23        IN varchar2
                      ,cp_abs_information24        IN varchar2
                      ,cp_abs_information25        IN varchar2
                      ,cp_abs_information26        IN varchar2
                      ,cp_abs_information27        IN varchar2
                      ,cp_abs_information28        IN varchar2
                      ,cp_abs_information29        IN varchar2
                      ,cp_abs_information30        IN varchar2
                      ,cp_replacement_person_id    IN number
                      ,cp_comments_to_approver     IN varchar2
                      ,cp_absence_attendance_id    IN number
                      ,cp_object_version_number    IN number
                      ,cp_date_end_xml             IN XMLType
                      ,cp_date_start_xml           IN XMLType
                      ,cp_date_projected_end_xml   IN XMLType
                      ,cp_date_projected_start_xml IN XMLType
                      ,cp_ame_item_type            IN varchar2
                      ,cp_ame_app_id               IN number
                      ,cp_ame_trans_type           IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", cp_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", cp_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", cp_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", cp_ame_item_type)
             ,XMLElement("PrsnJobId", cp_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(cp_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", cp_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), cp_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", cp_login_per_leg_code)
             ,XMLElement("PrsnMgrId", cp_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", cp_assignment_id)
             ,XMLElement("PrsnLegCode", cp_per_leg_code)
             ,XMLElement("LoginPrsnId", cp_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", g_PER)
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(cp_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,xxas_com_sec_pkg.get_function_id(cp_called_from)
               )
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", cp_transaction_id)
             ,XMLElement("PrsnLocationId", cp_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", cp_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", cp_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", cp_per_manager_name)
             ,XMLElement("PrsnOrganizationId", cp_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", cp_asg_row.position_id)
             ,XMLElement("TxnStatus", 'W')
             ,XMLElement("LoginWorkerNumber", cp_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", cp_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", cp_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", cp_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(cp_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_login_per_row.global_name, cp_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), cp_login_per_row.person_id)
             ,XMLElement("PrsnId", cp_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || cp_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", cp_ame_trans_type)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(cp_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", cp_called_from)
             ,XMLElement("LoginPrsnBgId", cp_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", cp_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", cp_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_per_row.global_name, cp_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", cp_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'UpdateMode'))
             ,XMLElement("pAMEAppId", cp_ame_app_id)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(cp_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (2 as "PS")
                       ,XMLElement("AbsenceAttendanceId", cp_absence_attendance_id)
                       ,decode
                         (cp_absence_type_id, cp_abs_row.absence_attendance_type_id, null
                         ,XMLElement
                           ("AbsenceAttendanceTypeId", cp_absence_type_id))
                       ,decode
                         (cp_abs_attendance_reason_id, cp_abs_row.abs_attendance_reason_id, null
                         ,XMLElement
                           ("AbsAttendanceReasonId", cp_abs_attendance_reason_id))
                       ,decode(cp_duration_days, cp_abs_row.absence_days, null, XMLElement("AbsenceDays", cp_duration_days))
                       ,decode(cp_duration_hours, cp_abs_row.absence_hours, null, XMLElement("AbsenceHours", cp_duration_hours))
                       ,cp_date_end_xml
                       ,XMLElement
                         ("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,cp_date_projected_end_xml
                       ,cp_date_projected_start_xml
                       ,cp_date_start_xml
                       ,decode
                         (decode(cp_leave_status, g_PLANNED, null, cp_time_end),cp_abs_row.time_end, null
                         ,XMLElement
                           ("TimeEnd", decode(cp_leave_status, g_PLANNED, null, cp_time_end)))
                       ,decode
                         (decode(cp_leave_status, g_PLANNED, cp_time_end, null),cp_abs_row.time_projected_end, null
                         ,XMLElement
                           ("TimeProjectedEnd", decode(cp_leave_status, g_PLANNED, cp_time_end, null)))
                       ,decode
                         (decode(cp_leave_status, g_PLANNED, cp_time_start, null),cp_abs_row.time_projected_start, null
                         ,XMLElement
                           ("TimeProjectedStart", decode(cp_leave_status, g_PLANNED, cp_time_start, null)))
                       ,decode
                         (decode(cp_leave_status, g_PLANNED, null, cp_time_start),cp_abs_row.time_start, null
                         ,XMLElement
                           ("TimeStart", decode(cp_leave_status, g_PLANNED, null, cp_time_start)))
                       ,decode
                         (cp_attribute_category, cp_abs_row.attribute_category, null
                         ,XMLElement
                           ("AttributeCategory", cp_attribute_category))
                       ,decode
                         (cp_attribute1, cp_abs_row.attribute1, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (cp_attribute2, cp_abs_row.attribute2, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (cp_attribute3, cp_abs_row.attribute3, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (cp_attribute4, cp_abs_row.attribute4, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (cp_attribute5, cp_abs_row.attribute5, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (cp_attribute6, cp_abs_row.attribute6, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (cp_attribute7, cp_abs_row.attribute7, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (cp_attribute8, cp_abs_row.attribute8, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (cp_attribute9, cp_abs_row.attribute9, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (cp_attribute10, cp_abs_row.attribute10, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (cp_attribute11, cp_abs_row.attribute11, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (cp_attribute12, cp_abs_row.attribute12, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (cp_attribute13, cp_abs_row.attribute13, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (cp_attribute14, cp_abs_row.attribute14, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (cp_attribute15, cp_abs_row.attribute15, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (cp_attribute16, cp_abs_row.attribute16, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (cp_attribute17, cp_abs_row.attribute17, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (cp_attribute18, cp_abs_row.attribute18, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (cp_attribute19, cp_abs_row.attribute19, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (cp_attribute20, cp_abs_row.attribute20, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (cp_abs_information_category, cp_abs_row.abs_information_category, null
                         ,XMLElement
                           ("AbsInformationCategory", cp_abs_information_category))
                       ,decode
                         (cp_abs_information1, cp_abs_row.abs_information1, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (cp_abs_information2, cp_abs_row.abs_information2, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (cp_abs_information3, cp_abs_row.abs_information3, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (cp_abs_information4, cp_abs_row.abs_information4, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (cp_abs_information5, cp_abs_row.abs_information5, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (cp_abs_information6, cp_abs_row.abs_information6, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (cp_abs_information7, cp_abs_row.abs_information7, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (cp_abs_information8, cp_abs_row.abs_information8, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (cp_abs_information9, cp_abs_row.abs_information9, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (cp_abs_information10, cp_abs_row.abs_information10, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (cp_abs_information11, cp_abs_row.abs_information11, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (cp_abs_information12, cp_abs_row.abs_information12, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (cp_abs_information13, cp_abs_row.abs_information13, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (cp_abs_information14, cp_abs_row.abs_information14, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (cp_abs_information15, cp_abs_row.abs_information15, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (cp_abs_information16, cp_abs_row.abs_information16, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (cp_abs_information17, cp_abs_row.abs_information17, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (cp_abs_information18, cp_abs_row.abs_information18, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (cp_abs_information19, cp_abs_row.abs_information19, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (cp_abs_information20, cp_abs_row.abs_information20, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (cp_abs_information21, cp_abs_row.abs_information21, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (cp_abs_information22, cp_abs_row.abs_information22, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (cp_abs_information23, cp_abs_row.abs_information23, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (cp_abs_information24, cp_abs_row.abs_information24, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (cp_abs_information25, cp_abs_row.abs_information25, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (cp_abs_information26, cp_abs_row.abs_information26, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (cp_abs_information27, cp_abs_row.abs_information27, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (cp_abs_information28, cp_abs_row.abs_information28, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (cp_abs_information29, cp_abs_row.abs_information29, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (cp_abs_information30, cp_abs_row.abs_information30, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("ObjectVersionNumber", cp_object_version_number)
                       ,decode
                         (cp_replacement_person_id, cp_abs_row.replacement_person_id, null
                         ,XMLElement
                           ("ReplacementPersonId", cp_replacement_person_id))
                       ,decode(cp_comments_to_approver, null, null, XMLElement("Comments", cp_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(cp_transaction_id))
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_rc_trans_doc(cp_person_id                IN number
                         ,cp_per_row                  IN csr_person%ROWTYPE
                         ,cp_asg_row                  IN csr_asg%ROWTYPE
                         ,cp_login_per_row            IN csr_login_person%ROWTYPE
                         ,cp_per_manager_name         IN varchar2
                         ,cp_login_per_manager_name   IN varchar2
                         ,cp_transaction_id           IN number
                         ,cp_per_sec_group_id         IN number
                         ,cp_per_leg_code             IN varchar2
                         ,cp_login_per_leg_code       IN varchar2
                         ,cp_job_name                 IN varchar2
                         ,cp_pos_name                 IN varchar2
                         ,cp_emp_generation           IN varchar2
                         ,cp_per_bg_cur_code          IN varchar2
                         ,cp_abs_row                  IN per_absence_attendances%ROWTYPE
                         ,cp_called_from              IN varchar2
                         ,cp_assignment_id            IN number
                         ,cp_effective_date           IN date
                         ,cp_leave_status             IN varchar2
                         ,cp_absence_type_id          IN number
                         ,cp_abs_attendance_reason_id IN number
                         ,cp_duration_days            IN number
                         ,cp_duration_hours           IN number
                         ,cp_date_start               IN date
                         ,cp_date_end                 IN date
                         ,cp_time_start               IN varchar2
                         ,cp_time_end                 IN varchar2
                         ,cp_attribute_category       IN varchar2
                         ,cp_attribute1               IN varchar2
                         ,cp_attribute2               IN varchar2
                         ,cp_attribute3               IN varchar2
                         ,cp_attribute4               IN varchar2
                         ,cp_attribute5               IN varchar2
                         ,cp_attribute6               IN varchar2
                         ,cp_attribute7               IN varchar2
                         ,cp_attribute8               IN varchar2
                         ,cp_attribute9               IN varchar2
                         ,cp_attribute10              IN varchar2
                         ,cp_attribute11              IN varchar2
                         ,cp_attribute12              IN varchar2
                         ,cp_attribute13              IN varchar2
                         ,cp_attribute14              IN varchar2
                         ,cp_attribute15              IN varchar2
                         ,cp_attribute16              IN varchar2
                         ,cp_attribute17              IN varchar2
                         ,cp_attribute18              IN varchar2
                         ,cp_attribute19              IN varchar2
                         ,cp_attribute20              IN varchar2
                         ,cp_abs_information_category IN varchar2
                         ,cp_abs_information1         IN varchar2
                         ,cp_abs_information2         IN varchar2
                         ,cp_abs_information3         IN varchar2
                         ,cp_abs_information4         IN varchar2
                         ,cp_abs_information5         IN varchar2
                         ,cp_abs_information6         IN varchar2
                         ,cp_abs_information7         IN varchar2
                         ,cp_abs_information8         IN varchar2
                         ,cp_abs_information9         IN varchar2
                         ,cp_abs_information10        IN varchar2
                         ,cp_abs_information11        IN varchar2
                         ,cp_abs_information12        IN varchar2
                         ,cp_abs_information13        IN varchar2
                         ,cp_abs_information14        IN varchar2
                         ,cp_abs_information15        IN varchar2
                         ,cp_abs_information16        IN varchar2
                         ,cp_abs_information17        IN varchar2
                         ,cp_abs_information18        IN varchar2
                         ,cp_abs_information19        IN varchar2
                         ,cp_abs_information20        IN varchar2
                         ,cp_abs_information21        IN varchar2
                         ,cp_abs_information22        IN varchar2
                         ,cp_abs_information23        IN varchar2
                         ,cp_abs_information24        IN varchar2
                         ,cp_abs_information25        IN varchar2
                         ,cp_abs_information26        IN varchar2
                         ,cp_abs_information27        IN varchar2
                         ,cp_abs_information28        IN varchar2
                         ,cp_abs_information29        IN varchar2
                         ,cp_abs_information30        IN varchar2
                         ,cp_replacement_person_id    IN number
                         ,cp_comments_to_approver     IN varchar2
                         ,cp_absence_attendance_id    IN number
                         ,cp_object_version_number    IN number
                         ,cp_date_end_xml             IN XMLType
                         ,cp_date_start_xml           IN XMLType
                         ,cp_date_projected_end_xml   IN XMLType
                         ,cp_date_projected_start_xml IN XMLType
                         ,cp_ame_item_type            IN varchar2
                         ,cp_ame_app_id               IN number
                         ,cp_ame_trans_type           IN varchar2
                         ,cp_transaction_status       IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", cp_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", cp_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", cp_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", cp_ame_item_type)
             ,XMLElement("PrsnJobId", cp_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(cp_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", cp_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), cp_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", cp_login_per_leg_code)
             ,XMLElement("PrsnMgrId", cp_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", cp_assignment_id)
             ,XMLElement("PrsnLegCode", cp_per_leg_code)
             ,XMLElement("LoginPrsnId", cp_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", g_PER)
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(cp_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,xxas_com_sec_pkg.get_function_id(cp_called_from)
               )
             ,XMLElement("TxnState", 'T')
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", cp_transaction_id)
             ,XMLElement("PrsnLocationId", cp_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", cp_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", cp_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", cp_per_manager_name)
             ,XMLElement("PrsnOrganizationId", cp_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", cp_asg_row.position_id)
             ,decode(cp_transaction_status, 'Y', XMLElement("TxnStatus", 'Y'), XMLElement("TxnStatus", 'RI'))
             ,XMLElement("LoginWorkerNumber", cp_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", cp_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", cp_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", cp_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(cp_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_login_per_row.global_name, cp_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), cp_login_per_row.person_id)
             ,XMLElement("PrsnId", cp_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || cp_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", cp_ame_trans_type)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(cp_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", cp_called_from)
             ,XMLElement("LoginPrsnBgId", cp_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", cp_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", cp_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_per_row.global_name, cp_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", cp_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'UpdateMode'))
             ,XMLElement("pAMEAppId", cp_ame_app_id)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(cp_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (2 as "PS")
                       ,XMLElement("AbsenceAttendanceId", cp_absence_attendance_id)
                       ,decode
                         (cp_absence_type_id, cp_abs_row.absence_attendance_type_id, null
                         ,XMLElement
                           ("AbsenceAttendanceTypeId", cp_absence_type_id))
                       ,decode
                         (cp_abs_attendance_reason_id, cp_abs_row.abs_attendance_reason_id, null
                         ,XMLElement
                           ("AbsAttendanceReasonId", cp_abs_attendance_reason_id))
                       ,decode(cp_duration_days, cp_abs_row.absence_days, null, XMLElement("AbsenceDays", cp_duration_days))
                       ,decode(cp_duration_hours, cp_abs_row.absence_hours, null, XMLElement("AbsenceHours", cp_duration_hours))
                       ,cp_date_end_xml
                       ,XMLElement
                         ("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,cp_date_projected_end_xml
                       ,cp_date_projected_start_xml
                       ,cp_date_start_xml  
                       ,decode
                         (decode(cp_leave_status, g_PLANNED, null, cp_time_end),cp_abs_row.time_end, null
                         ,XMLElement
                           ("TimeEnd", decode(cp_leave_status, g_PLANNED, null, cp_time_end)))
                       ,decode
                         (decode(cp_leave_status, g_PLANNED, cp_time_end, null),cp_abs_row.time_projected_end, null
                         ,XMLElement
                           ("TimeProjectedEnd", decode(cp_leave_status, g_PLANNED, cp_time_end, null)))
                       ,decode
                         (decode(cp_leave_status, g_PLANNED, cp_time_start, null),cp_abs_row.time_projected_start, null
                         ,XMLElement
                           ("TimeProjectedStart", decode(cp_leave_status, g_PLANNED, cp_time_start, null)))
                       ,decode
                         (decode(cp_leave_status, g_PLANNED, null, cp_time_start),cp_abs_row.time_start, null
                         ,XMLElement
                           ("TimeStart", decode(cp_leave_status, g_PLANNED, null, cp_time_start)))
                       ,decode
                         (cp_attribute_category, cp_abs_row.attribute_category, null
                         ,XMLElement
                           ("AttributeCategory", cp_attribute_category))
                       ,decode
                         (cp_attribute1, cp_abs_row.attribute1, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (cp_attribute2, cp_abs_row.attribute2, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (cp_attribute3, cp_abs_row.attribute3, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (cp_attribute4, cp_abs_row.attribute4, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (cp_attribute5, cp_abs_row.attribute5, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (cp_attribute6, cp_abs_row.attribute6, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (cp_attribute7, cp_abs_row.attribute7, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (cp_attribute8, cp_abs_row.attribute8, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (cp_attribute9, cp_abs_row.attribute9, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (cp_attribute10, cp_abs_row.attribute10, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (cp_attribute11, cp_abs_row.attribute11, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (cp_attribute12, cp_abs_row.attribute12, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (cp_attribute13, cp_abs_row.attribute13, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (cp_attribute14, cp_abs_row.attribute14, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (cp_attribute15, cp_abs_row.attribute15, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (cp_attribute16, cp_abs_row.attribute16, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (cp_attribute17, cp_abs_row.attribute17, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (cp_attribute18, cp_abs_row.attribute18, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (cp_attribute19, cp_abs_row.attribute19, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (cp_attribute20, cp_abs_row.attribute20, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (cp_abs_information_category, cp_abs_row.abs_information_category, null
                         ,XMLElement
                           ("AbsInformationCategory", cp_abs_information_category))
                       ,decode
                         (cp_abs_information1, cp_abs_row.abs_information1, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (cp_abs_information2, cp_abs_row.abs_information2, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (cp_abs_information3, cp_abs_row.abs_information3, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (cp_abs_information4, cp_abs_row.abs_information4, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (cp_abs_information5, cp_abs_row.abs_information5, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (cp_abs_information6, cp_abs_row.abs_information6, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (cp_abs_information7, cp_abs_row.abs_information7, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (cp_abs_information8, cp_abs_row.abs_information8, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (cp_abs_information9, cp_abs_row.abs_information9, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (cp_abs_information10, cp_abs_row.abs_information10, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (cp_abs_information11, cp_abs_row.abs_information11, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (cp_abs_information12, cp_abs_row.abs_information12, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (cp_abs_information13, cp_abs_row.abs_information13, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (cp_abs_information14, cp_abs_row.abs_information14, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (cp_abs_information15, cp_abs_row.abs_information15, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (cp_abs_information16, cp_abs_row.abs_information16, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (cp_abs_information17, cp_abs_row.abs_information17, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (cp_abs_information18, cp_abs_row.abs_information18, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (cp_abs_information19, cp_abs_row.abs_information19, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (cp_abs_information20, cp_abs_row.abs_information20, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (cp_abs_information21, cp_abs_row.abs_information21, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (cp_abs_information22, cp_abs_row.abs_information22, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (cp_abs_information23, cp_abs_row.abs_information23, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (cp_abs_information24, cp_abs_row.abs_information24, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (cp_abs_information25, cp_abs_row.abs_information25, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (cp_abs_information26, cp_abs_row.abs_information26, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (cp_abs_information27, cp_abs_row.abs_information27, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (cp_abs_information28, cp_abs_row.abs_information28, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (cp_abs_information29, cp_abs_row.abs_information29, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (cp_abs_information30, cp_abs_row.abs_information30, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("ObjectVersionNumber", cp_object_version_number)
                       ,decode
                         (cp_replacement_person_id, cp_abs_row.replacement_person_id, null
                         ,XMLElement
                           ("ReplacementPersonId", cp_replacement_person_id))
                       ,decode(cp_comments_to_approver, null, null, XMLElement("Comments", cp_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA('000100000000')
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_rc_trans_create_doc(cp_person_id                IN number
                                ,cp_absence_attendance_id    IN number
                                ,cp_per_row                  IN csr_person%ROWTYPE
                                ,cp_asg_row                  IN csr_asg%ROWTYPE
                                ,cp_login_per_row            IN csr_login_person%ROWTYPE
                                ,cp_per_manager_name         IN varchar2
                                ,cp_login_per_manager_name   IN varchar2
                                ,cp_transaction_id           IN number
                                ,cp_per_sec_group_id         IN number
                                ,cp_per_leg_code             IN varchar2
                                ,cp_login_per_leg_code       IN varchar2
                                ,cp_job_name                 IN varchar2
                                ,cp_pos_name                 IN varchar2
                                ,cp_emp_generation           IN varchar2
                                ,cp_per_bg_cur_code          IN varchar2
                                ,cp_called_from              IN varchar2
                                ,cp_assignment_id            IN number
                                ,cp_effective_date           IN date
                                ,cp_leave_status             IN varchar2
                                ,cp_absence_type_id          IN number
                                ,cp_abs_attendance_reason_id IN number
                                ,cp_duration_days            IN number
                                ,cp_duration_hours           IN number
                                ,cp_date_start               IN date
                                ,cp_date_end                 IN date
                                ,cp_time_start               IN varchar2
                                ,cp_time_end                 IN varchar2
                                ,cp_attribute_category       IN varchar2
                                ,cp_attribute1               IN varchar2
                                ,cp_attribute2               IN varchar2
                                ,cp_attribute3               IN varchar2
                                ,cp_attribute4               IN varchar2
                                ,cp_attribute5               IN varchar2
                                ,cp_attribute6               IN varchar2
                                ,cp_attribute7               IN varchar2
                                ,cp_attribute8               IN varchar2
                                ,cp_attribute9               IN varchar2
                                ,cp_attribute10              IN varchar2
                                ,cp_attribute11              IN varchar2
                                ,cp_attribute12              IN varchar2
                                ,cp_attribute13              IN varchar2
                                ,cp_attribute14              IN varchar2
                                ,cp_attribute15              IN varchar2
                                ,cp_attribute16              IN varchar2
                                ,cp_attribute17              IN varchar2
                                ,cp_attribute18              IN varchar2
                                ,cp_attribute19              IN varchar2
                                ,cp_attribute20              IN varchar2
                                ,cp_abs_information_category IN varchar2
                                ,cp_abs_information1         IN varchar2
                                ,cp_abs_information2         IN varchar2
                                ,cp_abs_information3         IN varchar2
                                ,cp_abs_information4         IN varchar2
                                ,cp_abs_information5         IN varchar2
                                ,cp_abs_information6         IN varchar2
                                ,cp_abs_information7         IN varchar2
                                ,cp_abs_information8         IN varchar2
                                ,cp_abs_information9         IN varchar2
                                ,cp_abs_information10        IN varchar2
                                ,cp_abs_information11        IN varchar2
                                ,cp_abs_information12        IN varchar2
                                ,cp_abs_information13        IN varchar2
                                ,cp_abs_information14        IN varchar2
                                ,cp_abs_information15        IN varchar2
                                ,cp_abs_information16        IN varchar2
                                ,cp_abs_information17        IN varchar2
                                ,cp_abs_information18        IN varchar2
                                ,cp_abs_information19        IN varchar2
                                ,cp_abs_information20        IN varchar2
                                ,cp_abs_information21        IN varchar2
                                ,cp_abs_information22        IN varchar2
                                ,cp_abs_information23        IN varchar2
                                ,cp_abs_information24        IN varchar2
                                ,cp_abs_information25        IN varchar2
                                ,cp_abs_information26        IN varchar2
                                ,cp_abs_information27        IN varchar2
                                ,cp_abs_information28        IN varchar2
                                ,cp_abs_information29        IN varchar2
                                ,cp_abs_information30        IN varchar2
                                ,cp_replacement_person_id    IN number
                                ,cp_comments_to_approver     IN varchar2
                                ,cp_ame_item_type            IN varchar2
                                ,cp_ame_app_id               IN number
                                ,cp_ame_trans_type           IN varchar2
                                ,cp_transaction_status       IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", cp_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", cp_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", cp_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", cp_ame_item_type)
             ,XMLElement("PrsnJobId", cp_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(cp_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", cp_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), cp_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", cp_login_per_leg_code)
             ,XMLElement("PrsnMgrId", cp_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", cp_assignment_id)
             ,XMLElement("PrsnLegCode", cp_per_leg_code)
             ,XMLElement("LoginPrsnId", cp_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", g_PER)
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(cp_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,xxas_com_sec_pkg.get_function_id(cp_called_from)
               )
             ,XMLElement("TxnState", 'T')
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", cp_transaction_id)
             ,XMLElement("PrsnLocationId", cp_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", cp_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", cp_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", cp_per_manager_name)
             ,XMLElement("PrsnOrganizationId", cp_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", cp_asg_row.position_id)
             ,decode(cp_transaction_status, 'Y', XMLElement("TxnStatus", 'Y'), XMLElement("TxnStatus", 'RI'))
             ,XMLElement("LoginWorkerNumber", cp_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", cp_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", cp_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", cp_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(cp_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_login_per_row.global_name, cp_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), cp_login_per_row.person_id)
             ,XMLElement("PrsnId", cp_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || cp_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", cp_ame_trans_type)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(cp_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", cp_called_from)
             ,XMLElement("LoginPrsnBgId", cp_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", cp_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", cp_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_per_row.global_name, cp_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", cp_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CreateMode'))
             ,XMLElement("pAMEAppId", cp_ame_app_id)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(cp_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (0 as "PS"
                       ,'Y' as "PK")
                       ,XMLElement("AbsenceAttendanceId", cp_absence_attendance_id)
                       ,XMLElement("BusinessGroupId", cp_asg_row.business_group_id)
                       ,XMLElement("AbsenceAttendanceTypeId", cp_absence_type_id)
                       ,XMLElement("AbsAttendanceReasonId", cp_abs_attendance_reason_id)
                       ,XMLElement("PersonId", cp_person_id)
                       ,decode(cp_duration_days, null, XMLElement("AbsenceDays", XMLAttributes('true' as "null"), cp_duration_days),XMLElement("AbsenceDays", cp_duration_days))
                       ,decode(cp_duration_hours, null, XMLElement("AbsenceHours", XMLAttributes('true' as "null"), cp_duration_hours),XMLElement("AbsenceHours", cp_duration_hours))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("DateEnd", XMLAttributes('true' as "null")), decode(cp_date_end, null, XMLElement("DateEnd", XMLAttributes('true' as "null"), cp_date_end), XMLElement("DateEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT))))
                       ,XMLElement("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_date_end, null, XMLElement("DateProjectedEnd", XMLAttributes('true' as "null"), cp_date_end), XMLElement("DateProjectedEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT))), XMLElement("DateProjectedEnd", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_date_start, null, XMLElement("DateProjectedStart", XMLAttributes('true' as "null"), cp_date_start), XMLElement("DateProjectedStart", to_char(cp_date_start, g_HRAPI_DATE_FMT))), XMLElement("DateProjectedStart", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("DateStart", XMLAttributes('true' as "null")), decode(cp_date_start, null, XMLElement("DateStart", XMLAttributes('true' as "null"), cp_date_start), XMLElement("DateStart", to_char(cp_date_start, g_HRAPI_DATE_FMT))))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("TimeEnd", XMLAttributes('true' as "null")), decode(cp_time_end, null, XMLElement("TimeEnd", XMLAttributes('true' as "null"), cp_time_end), XMLElement("TimeEnd", cp_time_end)))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_time_end, null, XMLElement("TimeProjectedEnd", XMLAttributes('true' as "null"), cp_time_end), XMLElement("TimeProjectedEnd", cp_time_end)), XMLElement("TimeProjectedEnd", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, decode(cp_time_start, null, XMLElement("TimeProjectedStart", XMLAttributes('true' as "null"), cp_time_start), XMLElement("TimeProjectedStart", cp_time_start)), XMLElement("TimeProjectedStart", XMLAttributes('true' as "null")))
                       ,decode(cp_leave_status, g_PLANNED, XMLElement("TimeStart", XMLAttributes('true' as "null")), decode(cp_time_start, null, XMLElement("TimeStart", XMLAttributes('true' as "null"), cp_time_start), XMLElement("TimeStart", cp_time_start)))
                       ,decode(cp_replacement_person_id, null, XMLElement("ReplacementPersonId", XMLAttributes('true' as "null"), cp_replacement_person_id), XMLElement("ReplacementPersonId", cp_replacement_person_id))
                       ,decode
                         (cp_attribute_category, null, null
                         ,XMLElement
                           ("AttributeCategory", cp_attribute_category))
                       ,decode
                         (cp_attribute1, null, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (cp_attribute2, null, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (cp_attribute3, null, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (cp_attribute4, null, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (cp_attribute5, null, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (cp_attribute6, null, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (cp_attribute7, null, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (cp_attribute8, null, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (cp_attribute9, null, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (cp_attribute10, null, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (cp_attribute11, null, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (cp_attribute12, null, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (cp_attribute13, null, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (cp_attribute14, null, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (cp_attribute15, null, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (cp_attribute16, null, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (cp_attribute17, null, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (cp_attribute18, null, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (cp_attribute19, null, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (cp_attribute20, null, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (cp_abs_information_category, null, null
                         ,XMLElement
                           ("AbsInformationCategory", cp_abs_information_category))
                       ,decode
                         (cp_abs_information1, null, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (cp_abs_information2, null, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (cp_abs_information3, null, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (cp_abs_information4, null, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (cp_abs_information5, null, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (cp_abs_information6, null, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (cp_abs_information7, null, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (cp_abs_information8, null, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (cp_abs_information9, null, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (cp_abs_information10, null, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (cp_abs_information11, null, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (cp_abs_information12, null, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (cp_abs_information13, null, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (cp_abs_information14, null, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (cp_abs_information15, null, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (cp_abs_information16, null, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (cp_abs_information17, null, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (cp_abs_information18, null, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (cp_abs_information19, null, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (cp_abs_information20, null, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (cp_abs_information21, null, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (cp_abs_information22, null, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (cp_abs_information23, null, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (cp_abs_information24, null, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (cp_abs_information25, null, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (cp_abs_information26, null, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (cp_abs_information27, null, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (cp_abs_information28, null, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (cp_abs_information29, null, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (cp_abs_information30, null, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("LastUpdateDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("LastUpdatedBy", fnd_global.user_id)
                       ,XMLElement("LastUpdateLogin", fnd_global.login_id)
                       ,XMLElement("CreatedBy", fnd_global.user_id)
                       ,XMLElement("CreationDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"), null)
                       ,decode(cp_comments_to_approver, null, XMLElement("Comments", XMLAttributes('true' as "null"), cp_comments_to_approver), XMLElement("Comments", cp_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA('000100000000')
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_workflow_info(cp_transaction_id IN number) IS
  SELECT hat.item_type
        ,hat.item_key
        ,hat.transaction_ref_id
        ,hat.status
  FROM   hr_api_transactions  hat
  WHERE  hat.transaction_id = cp_transaction_id;

  CURSOR csr_transaction_status(cp_transaction_id IN number) IS
  SELECT hat.status
  FROM   hr_api_transactions  hat
  WHERE  hat.transaction_id = cp_transaction_id;

  c_method    CONSTANT         varchar2(30) := 'update_absence_event';
  l_person_id                  number;
  l_assignment_id              number;
  l_transaction_id             number := p_transaction_id;
  l_transaction_step_id        number := p_transaction_step_id;
  l_object_version_number      number := p_object_version_number;
  l_time_start                 varchar2(5) := p_time_start;
  l_time_end                   varchar2(5) := p_time_end;
  l_duration_days              per_absence_attendances.absence_days%TYPE  := p_duration_days;
  l_duration_hours             per_absence_attendances.absence_hours%TYPE := p_duration_hours;
  l_item_type                  wf_item_activity_statuses.item_type%TYPE;
  l_item_key                   wf_item_activity_statuses.item_key%TYPE;
  l_activity_id                wf_item_activity_statuses.process_activity%TYPE;
  l_asg_row                    csr_asg%ROWTYPE;
  l_per_row                    csr_person%ROWTYPE;
  l_login_per_row              csr_login_person%ROWTYPE;
  l_abs_row                    per_absence_attendances%ROWTYPE;
  l_login_person_id            number;
  l_per_manager_name           varchar2(240);
  l_login_per_manager_name     varchar2(240);
  l_doc                        XMLType;
  l_rc_doc                     XMLType;
  l_date_end_xml               XMLType;
  l_date_start_xml             XMLType;
  l_date_projected_end_xml     XMLType;
  l_date_projected_start_xml   XMLType;
  l_per_sec_group_id           number;
  l_per_leg_code               varchar2(150);
  l_login_per_sec_group_id     number;
  l_login_per_leg_code         varchar2(150);
  l_per_job_name               varchar2(700);
  l_per_position_name          varchar2(240);
  l_emp_generation             varchar2(150);
  l_per_bg_cur_code            varchar2(150);
  l_abs_att_category           varchar2(240);
  l_msg_data                   varchar2(2000) := null;
  l_msg_count                  number := 0;
  l_api_errors                 boolean := false;
  l_return_status              varchar2(30);
  l_occurrence                 varchar2(240);
  l_absence_attendance_id1     number;
  l_called_from                fnd_form_functions.function_name%TYPE;
  l_ame_item_type              varchar2(30);
  l_ame_app_id                 number;
  l_ame_trans_type             varchar2(30);
  l_ame_process_name           varchar2(30);
  l_workflow_info              csr_workflow_info%ROWTYPE;
  err_code                     varchar2(200);
  err_msg                      varchar2(200);
  l_attribute1                 varchar2(1000);
  l_attribute2                 varchar2(1000);
  l_attribute3                 varchar2(1000);
  l_attribute4                 varchar2(1000);
  l_attribute5                 varchar2(1000);
  l_attribute6                 varchar2(1000);
  l_attribute7                 varchar2(1000);
  l_attribute8                 varchar2(1000);
  l_attribute9                 varchar2(1000);
  l_attribute10                varchar2(1000);
  l_attribute11                varchar2(1000);
  l_attribute12                varchar2(1000);
  l_attribute13                varchar2(1000);
  l_attribute14                varchar2(1000);
  l_attribute15                varchar2(1000);
  l_attribute16                varchar2(1000);
  l_attribute17                varchar2(1000);
  l_attribute18                varchar2(1000);
  l_attribute19                varchar2(1000);
  l_attribute20                varchar2(1000);
  l_abs_information1           varchar2(1000);
  l_abs_information2           varchar2(1000);
  l_abs_information3           varchar2(1000);
  l_abs_information4           varchar2(1000);
  l_abs_information5           varchar2(1000);
  l_abs_information6           varchar2(1000);
  l_abs_information7           varchar2(1000);
  l_abs_information8           varchar2(1000);
  l_abs_information9           varchar2(1000);
  l_abs_information10          varchar2(1000);
  l_abs_information11          varchar2(1000);
  l_abs_information12          varchar2(1000);
  l_abs_information13          varchar2(1000);
  l_abs_information14          varchar2(1000);
  l_abs_information15          varchar2(1000);
  l_abs_information16          varchar2(1000);
  l_abs_information17          varchar2(1000);
  l_abs_information18          varchar2(1000);
  l_abs_information19          varchar2(1000);
  l_abs_information20          varchar2(1000);
  l_abs_information21          varchar2(1000);
  l_abs_information22          varchar2(1000);
  l_abs_information23          varchar2(1000);
  l_abs_information24          varchar2(1000);
  l_abs_information25          varchar2(1000);
  l_abs_information26          varchar2(1000);
  l_abs_information27          varchar2(1000);
  l_abs_information28          varchar2(1000);
  l_abs_information29          varchar2(1000);
  l_abs_information30          varchar2(1000);
  l_flex_vs_id                 number;
  l_temp                       varchar2(1000);
  l_transaction_status         hr_api_transactions.status%TYPE;
  
  CURSOR csr_get_vs_id IS
  SELECT flex_value_set_id
  FROM   fnd_flex_value_sets
  WHERE  flex_value_set_name = 'PQP_GB_OSP_PART_DAYS_VS';
  
  CURSOR csr_get_dff_info IS
  SELECT application_column_name 
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  flex_value_set_id = l_flex_vs_id
  AND    enabled_flag = 'Y'
  AND    descriptive_flex_context_code in (p_attribute_category,p_abs_information_category);
  
  CURSOR csr_get_dff_gdd IS
  SELECT application_column_name 
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  flex_value_set_id = l_flex_vs_id
  AND    enabled_flag = 'Y'
  AND    descriptive_flex_context_code = 'Global Data Elements';

  CURSOR csr_get_enabled_dff_gdd IS
  SELECT application_column_name
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  enabled_flag = 'Y'
  and    descriptive_flexfield_name in ('PER_ABS_DEVELOPER_DF','PER_ABSENCE_ATTENDANCES')
  AND    descriptive_flex_context_code = 'Global Data Elements';

BEGIN

  log_start(c_method);

  OPEN  csr_person(p_person_id);
  FETCH csr_person INTO l_per_row;
  CLOSE csr_person;

  OPEN  csr_asg(p_assignment_id);
  FETCH csr_asg INTO l_asg_row;
  CLOSE csr_asg;

  OPEN  csr_abs_attendance_info(p_absence_attendance_id);
  FETCH csr_abs_attendance_info INTO l_abs_row;
  CLOSE csr_abs_attendance_info;

  OPEN  csr_get_emp_full_name(l_asg_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_job_name(l_asg_row.job_id);
  FETCH csr_get_job_name INTO l_per_job_name;
  CLOSE csr_get_job_name;

  OPEN  csr_get_pos_name(l_asg_row.position_id);
  FETCH csr_get_pos_name INTO l_per_position_name;
  CLOSE csr_get_pos_name;

  OPEN  csr_get_sec_grp_leg_code(p_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_per_sec_group_id
                                     ,l_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  OPEN  csr_get_org_info(p_person_id);
  FETCH csr_get_org_info INTO l_emp_generation
                             ,l_per_bg_cur_code;
  CLOSE csr_get_org_info;

  l_attribute1        := p_attribute1;
  l_attribute2        := p_attribute2;
  l_attribute3        := p_attribute3;
  l_attribute4        := p_attribute4;
  l_attribute5        := p_attribute5;
  l_attribute6        := p_attribute6;
  l_attribute7        := p_attribute7;
  l_attribute8        := p_attribute8;
  l_attribute9        := p_attribute9;
  l_attribute10       := p_attribute10;
  l_attribute11       := p_attribute11;
  l_attribute12       := p_attribute12;
  l_attribute13       := p_attribute13;
  l_attribute14       := p_attribute14;
  l_attribute15       := p_attribute15;
  l_attribute16       := p_attribute16;
  l_attribute17       := p_attribute17;
  l_attribute18       := p_attribute18;
  l_attribute19       := p_attribute19;
  l_attribute20       := p_attribute20;
  
  l_abs_information1  := p_abs_information1;
  l_abs_information2  := p_abs_information2;
  l_abs_information3  := p_abs_information3;
  l_abs_information4  := p_abs_information4;
  l_abs_information5  := p_abs_information5;
  l_abs_information6  := p_abs_information6;
  l_abs_information7  := p_abs_information7;
  l_abs_information8  := p_abs_information8;
  l_abs_information9  := p_abs_information9;
  l_abs_information10 := p_abs_information10;
  l_abs_information11  := p_abs_information11;
  l_abs_information12  := p_abs_information12;
  l_abs_information13  := p_abs_information13;
  l_abs_information14  := p_abs_information14;
  l_abs_information15  := p_abs_information15;
  l_abs_information16  := p_abs_information16;
  l_abs_information17  := p_abs_information17;
  l_abs_information18  := p_abs_information18;
  l_abs_information19  := p_abs_information19;
  l_abs_information20  := p_abs_information20;
  l_abs_information21  := p_abs_information21;
  l_abs_information22  := p_abs_information22;
  l_abs_information23  := p_abs_information23;
  l_abs_information24  := p_abs_information24;
  l_abs_information25  := p_abs_information25;
  l_abs_information26  := p_abs_information26;
  l_abs_information27  := p_abs_information27;
  l_abs_information28  := p_abs_information28;
  l_abs_information29  := p_abs_information29;
  l_abs_information30  := p_abs_information30;

  OPEN csr_get_vs_id;
  FETCH csr_get_vs_id INTO l_flex_vs_id;
  CLOSE csr_get_vs_id;
  
  OPEN csr_get_dff_gdd;
  FETCH csr_get_dff_gdd INTO l_temp;
  CLOSE csr_get_dff_gdd;
  
  IF l_temp IS NULL and p_abs_information_category IS NULL THEN
    l_abs_information1  := null;
    l_abs_information2  := null;
    l_abs_information3  := null;
    l_abs_information4  := null;
    l_abs_information5  := null;
    l_abs_information6  := null;
    l_abs_information7  := null;
    l_abs_information8  := null;
    l_abs_information9  := null;
    l_abs_information10 := null;
    l_abs_information11  := null;
    l_abs_information12  := null;
    l_abs_information13  := null;
    l_abs_information14  := null;
    l_abs_information15  := null;
    l_abs_information16  := null;
    l_abs_information17  := null;
    l_abs_information18  := null;
    l_abs_information19  := null;
    l_abs_information20  := null;
    l_abs_information21  := null;
    l_abs_information22  := null;
    l_abs_information23  := null;
    l_abs_information24  := null;
    l_abs_information25  := null;
    l_abs_information26  := null;
    l_abs_information27  := null;
    l_abs_information28  := null;
    l_abs_information29  := null;
    l_abs_information30  := null;
  END IF;

  IF l_temp IS NULL AND p_attribute_category IS NULL THEN
    l_attribute1        := null;
    l_attribute2        := null;
    l_attribute3        := null;
    l_attribute4        := null;
    l_attribute5        := null;
    l_attribute6        := null;
    l_attribute7        := null;
    l_attribute8        := null;
    l_attribute9        := null;
    l_attribute10       := null;
    l_attribute11       := null;
    l_attribute12       := null;
    l_attribute13       := null;
    l_attribute14       := null;
    l_attribute15       := null;
    l_attribute16       := null;
    l_attribute17       := null;
    l_attribute18       := null;
    l_attribute19       := null;
    l_attribute20       := null;
  END IF;
  
  FOR csr_rec in csr_get_enabled_dff_gdd
  LOOP
  
  IF csr_rec.application_column_name = 'ATTRIBUTE1' THEN
      if p_attribute1 is not null THEN
        l_attribute1 := p_attribute1;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE2' THEN
      if p_attribute2 is not null THEN
        l_attribute2 := p_attribute2;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE3' THEN
      if p_attribute3 is not null THEN
        l_attribute3 := p_attribute3;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE4' THEN
      if p_attribute4 is not null THEN
        l_attribute4 := p_attribute4;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE5' THEN
      if p_attribute5 is not null THEN
        l_attribute5 := p_attribute5;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE6' THEN
      if p_attribute6 is not null THEN
        l_attribute6 := p_attribute6;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE7' THEN
      if p_attribute7 is not null THEN
        l_attribute7 := p_attribute7;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE8' THEN
      if p_attribute8 is not null THEN
        l_attribute8 := p_attribute8;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE9' THEN
      if p_attribute9 is not null THEN
        l_attribute9 := p_attribute9;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE10' THEN
      if p_attribute10 is not null THEN
        l_attribute10 := p_attribute10;
      end if;
    elsIF csr_rec.application_column_name = 'ATTRIBUTE11' THEN
      if p_attribute11 is not null THEN
        l_attribute11 := p_attribute11;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE12' THEN
      if p_attribute12 is not null THEN
        l_attribute12 := p_attribute12;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE13' THEN
      if p_attribute13 is not null THEN
        l_attribute13 := p_attribute13;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE14' THEN
      if p_attribute14 is not null THEN
        l_attribute14 := p_attribute14;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE15' THEN
      if p_attribute15 is not null THEN
        l_attribute15 := p_attribute15;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE16' THEN
      if p_attribute16 is not null THEN
        l_attribute16 := p_attribute16;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE17' THEN
      if p_attribute17 is not null THEN
        l_attribute17 := p_attribute17;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE18' THEN
      if p_attribute18 is not null THEN
        l_attribute18 := p_attribute18;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE19' THEN
      if p_attribute19 is not null THEN
        l_attribute19 := p_attribute19;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE20' THEN
      if p_attribute20 is not null THEN
        l_attribute20 := p_attribute20;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION1' THEN
      if p_abs_information1 is not null THEN
        l_abs_information1 := p_abs_information1;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION2' THEN
      if p_abs_information2 is not null THEN
        l_abs_information2 := p_abs_information2;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION3' THEN
      if p_abs_information3 is not null THEN
        l_abs_information3 := p_abs_information3;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION4' THEN
      if p_abs_information4 is not null THEN
        l_abs_information4 := p_abs_information4;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION5' THEN
      if p_abs_information5 is not null THEN
        l_abs_information5 := p_abs_information5;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION6' THEN
      if p_abs_information6 is not null THEN
        l_abs_information6 := p_abs_information6;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION7' THEN
      if p_abs_information7 is not null THEN
        l_abs_information7 := p_abs_information7;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION8' THEN
      if p_abs_information8 is not null THEN
        l_abs_information8 := p_abs_information8;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION9' THEN
      if p_abs_information9 is not null THEN
        l_abs_information9 := p_abs_information9;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION10' THEN
      if p_abs_information10 is not null THEN
        l_abs_information10 := p_abs_information10;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION11' THEN
      if p_abs_information11 is not null THEN
        l_abs_information11 := p_abs_information11;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION12' THEN
      if p_abs_information12 is not null THEN
        l_abs_information12 := p_abs_information12;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION13' THEN
      if p_abs_information13 is not null THEN
        l_abs_information13 := p_abs_information13;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION14' THEN
      if p_abs_information14 is not null THEN
        l_abs_information14 := p_abs_information14;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION15' THEN
      if p_abs_information15 is not null THEN
        l_abs_information15 := p_abs_information15;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION16' THEN
      if p_abs_information16 is not null THEN
        l_abs_information16 := p_abs_information16;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION17' THEN
      if p_abs_information17 is not null THEN
        l_abs_information17 := p_abs_information17;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION18' THEN
      if p_abs_information18 is not null THEN
        l_abs_information18 := p_abs_information18;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION19' THEN
      if p_abs_information19 is not null THEN
        l_abs_information19 := p_abs_information19;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION20' THEN
      if p_abs_information20 is not null THEN
        l_abs_information20 := p_abs_information20;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION21' THEN
      if p_abs_information21 is not null THEN
        l_abs_information21 := p_abs_information21;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION22' THEN
      if p_abs_information22 is not null THEN
        l_abs_information22 := p_abs_information22;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION23' THEN
      if p_abs_information23 is not null THEN
        l_abs_information23 := p_abs_information23;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION24' THEN
      if p_abs_information24 is not null THEN
        l_abs_information24 := p_abs_information24;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION25' THEN
      if p_abs_information25 is not null THEN
        l_abs_information25 := p_abs_information25;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION26' THEN
      if p_abs_information26 is not null THEN
        l_abs_information26 := p_abs_information26;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION27' THEN
      if p_abs_information27 is not null THEN
        l_abs_information27 := p_abs_information27;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION28' THEN
      if p_abs_information28 is not null THEN
        l_abs_information28 := p_abs_information28;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION29' THEN
      if p_abs_information29 is not null THEN
        l_abs_information29 := p_abs_information29;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION30' THEN
      if p_abs_information30 is not null THEN
        l_abs_information30 := p_abs_information30;
      end if;
    end if;
 END LOOP;
  
  FOR csr_rec in csr_get_dff_info
  LOOP
  
  IF csr_rec.application_column_name = 'ATTRIBUTE1' THEN
      if instr(l_attribute1,'.') = 0 THEN
        l_attribute1 := p_attribute1||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE2' THEN
      if instr(l_attribute2,'.') = 0 THEN
        l_attribute2 := p_attribute2||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE3' THEN
      if instr(l_attribute3,'.') = 0 THEN
        l_attribute3 := p_attribute3||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE4' THEN
      if instr(l_attribute4,'.') = 0 THEN
        l_attribute4 := p_attribute4||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE5' THEN
      if instr(l_attribute5,'.') = 0 THEN
        l_attribute5 := p_attribute5||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE6' THEN
      if instr(l_attribute6,'.') = 0 THEN
        l_attribute6 := p_attribute6||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE7' THEN
      if instr(l_attribute7,'.') = 0 THEN
        l_attribute7 := p_attribute7||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE8' THEN
      if instr(l_attribute8,'.') = 0 THEN
        l_attribute8 := p_attribute8||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE9' THEN
      if instr(l_attribute9,'.') = 0 THEN
        l_attribute9 := p_attribute9||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE10' THEN
      if instr(l_attribute10,'.') = 0 THEN
        l_attribute10 := p_attribute10||'.00';
      end if;
    elsIF csr_rec.application_column_name = 'ATTRIBUTE11' THEN
      if instr(l_attribute11,'.') = 0 THEN
        l_attribute11 := p_attribute11||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE12' THEN
      if instr(l_attribute12,'.') = 0 THEN
        l_attribute12 := p_attribute12||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE13' THEN
      if instr(l_attribute13,'.') = 0 THEN
        l_attribute13 := p_attribute13||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE14' THEN
      if instr(l_attribute14,'.') = 0 THEN
        l_attribute14 := p_attribute14||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE15' THEN
      if instr(l_attribute15,'.') = 0 THEN
        l_attribute15 := p_attribute15||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE16' THEN
      if instr(l_attribute16,'.') = 0 THEN
        l_attribute16 := p_attribute16||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE17' THEN
      if instr(l_attribute17,'.') = 0 THEN
        l_attribute17 := p_attribute17||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE18' THEN
      if instr(l_attribute18,'.') = 0 THEN
        l_attribute18 := p_attribute18||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE19' THEN
      if instr(l_attribute19,'.') = 0 THEN
        l_attribute19 := p_attribute19||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE20' THEN
      if instr(l_attribute20,'.') = 0 THEN
        l_attribute20 := p_attribute20||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION1' THEN
      if instr(l_abs_information1,'.') = 0 THEN
        l_abs_information1 := l_abs_information1||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION2' THEN
      if instr(l_abs_information2,'.') = 0 THEN
        l_abs_information2 := l_abs_information2||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION3' THEN
      if instr(l_abs_information3,'.') = 0 THEN
        l_abs_information3 := l_abs_information3||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION4' THEN
      if instr(l_abs_information4,'.') = 0 THEN
        l_abs_information4 := l_abs_information4||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION5' THEN
      if instr(l_abs_information5,'.') = 0 THEN
        l_abs_information5 := l_abs_information5||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION6' THEN
      if instr(l_abs_information6,'.') = 0 THEN
        l_abs_information6 := l_abs_information6||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION7' THEN
      if instr(l_abs_information7,'.') = 0 THEN
        l_abs_information7 := l_abs_information7||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION8' THEN
      if instr(l_abs_information8,'.') = 0 THEN
        l_abs_information8 := l_abs_information8||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION9' THEN
      if instr(l_abs_information9,'.') = 0 THEN
        l_abs_information9 := l_abs_information9||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION10' THEN
      if instr(l_abs_information10,'.') = 0 THEN
        l_abs_information10 := l_abs_information10||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION11' THEN
      if instr(l_abs_information11,'.') = 0 THEN
        l_abs_information11 := l_abs_information11||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION12' THEN
      if instr(l_abs_information12,'.') = 0 THEN
        l_abs_information12 := l_abs_information12||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION13' THEN
      if instr(l_abs_information13,'.') = 0 THEN
        l_abs_information13 := l_abs_information13||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION14' THEN
      if instr(l_abs_information14,'.') = 0 THEN
        l_abs_information14 := l_abs_information14||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION15' THEN
      if instr(l_abs_information15,'.') = 0 THEN
        l_abs_information15 := l_abs_information15||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION16' THEN
      if instr(l_abs_information16,'.') = 0 THEN
        l_abs_information16 := l_abs_information16||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION17' THEN
      if instr(l_abs_information17,'.') = 0 THEN
        l_abs_information17 := l_abs_information17||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION18' THEN
      if instr(l_abs_information18,'.') = 0 THEN
        l_abs_information18 := l_abs_information18||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION19' THEN
      if instr(l_abs_information19,'.') = 0 THEN
        l_abs_information19 := l_abs_information19||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION20' THEN
      if instr(l_abs_information20,'.') = 0 THEN
        l_abs_information20 := l_abs_information20||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION21' THEN
      if instr(l_abs_information21,'.') = 0 THEN
        l_abs_information21 := l_abs_information21||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION22' THEN
      if instr(l_abs_information22,'.') = 0 THEN
        l_abs_information22 := l_abs_information22||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION23' THEN
      if instr(l_abs_information23,'.') = 0 THEN
        l_abs_information23 := l_abs_information23||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION24' THEN
      if instr(l_abs_information24,'.') = 0 THEN
        l_abs_information24 := l_abs_information24||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION25' THEN
      if instr(l_abs_information25,'.') = 0 THEN
        l_abs_information25 := l_abs_information25||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION26' THEN
      if instr(l_abs_information26,'.') = 0 THEN
        l_abs_information26 := l_abs_information26||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION27' THEN
      if instr(l_abs_information27,'.') = 0 THEN
        l_abs_information27 := l_abs_information27||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION28' THEN
      if instr(l_abs_information28,'.') = 0 THEN
        l_abs_information28 := l_abs_information28||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION29' THEN
      if instr(l_abs_information29,'.') = 0 THEN
        l_abs_information29 := l_abs_information29||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION30' THEN
      if instr(l_abs_information30,'.') = 0 THEN
        l_abs_information30 := l_abs_information30||'.00';
      end if;
    end if;
  END LOOP;
  
  IF p_duration_type is not null THEN

    IF p_duration_type = 'H' THEN

      l_duration_days := null;

    ELSIF p_duration_type NOT IN (xxas_fast_formula_pkg.g_NULL_STRING,'H') THEN

      l_time_start     := null;
      l_time_end       := null;
      l_duration_hours := null;

    END IF;

  END IF;

  IF l_transaction_id is not null AND 
     l_transaction_step_id is not null THEN

    OPEN  csr_workflow_info(l_transaction_id);
    FETCH csr_workflow_info INTO l_workflow_info;
    CLOSE csr_workflow_info;

    l_item_type             := l_workflow_info.item_type;
    l_item_key              := l_workflow_info.item_key;

  ELSE

    --
    -- Generate the new sequences
    --
    SELECT  hr_api_transactions_s.nextval
    INTO    l_transaction_id
    FROM   dual;

  END IF;

  l_login_person_id := xxas_util_pkg.get_per_from_user(fnd_global.user_id);

  OPEN  csr_login_person(l_login_person_id);
  FETCH csr_login_person INTO l_login_per_row;
  CLOSE csr_login_person;

  OPEN  csr_get_emp_full_name(l_login_per_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_login_per_manager_name;
  CLOSE csr_get_emp_full_name;
  
  OPEN  csr_get_sec_grp_leg_code(l_login_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_login_per_sec_group_id
                                     ,l_login_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  SELECT (CASE WHEN p_leave_status = g_PLANNED
               THEN XMLElement
                     ("DateEnd", null)
               WHEN p_leave_status = g_CONFIRMED AND NVL(to_char(p_date_end, g_HRAPI_DATE_FMT),-1) = NVL(to_char(l_abs_row.date_end, g_HRAPI_DATE_FMT),-1)
               THEN null
               ELSE XMLElement
                     ("DateEnd", to_char(p_date_end, g_HRAPI_DATE_FMT))
          END)
  INTO   l_date_end_xml 
  FROM   dual;

  SELECT (CASE WHEN p_leave_status = g_PLANNED 
               THEN XMLElement
                     ("DateStart", null)
               WHEN p_leave_status = g_CONFIRMED AND NVL(to_char(p_date_start, g_HRAPI_DATE_FMT),-1) = NVL(to_char(l_abs_row.date_start, g_HRAPI_DATE_FMT),-1)
               THEN null
               ELSE XMLElement
                     ("DateStart", to_char(p_date_start, g_HRAPI_DATE_FMT))
          END)
  INTO   l_date_start_xml 
  FROM   dual;

  SELECT (CASE WHEN p_leave_status = g_CONFIRMED 
               THEN XMLElement
                     ("DateProjectedEnd", null)
               WHEN p_leave_status = g_PLANNED AND NVL(to_char(p_date_end, g_HRAPI_DATE_FMT),-1) = NVL(to_char(l_abs_row.date_projected_end, g_HRAPI_DATE_FMT),-1)
               THEN null
               ELSE XMLElement
                     ("DateProjectedEnd", to_char(p_date_end, g_HRAPI_DATE_FMT))
          END)
  INTO   l_date_projected_end_xml 
  FROM   dual;

  SELECT (CASE WHEN p_leave_status = g_CONFIRMED 
               THEN XMLElement
                     ("DateProjectedStart", null)
               WHEN p_leave_status = g_PLANNED AND NVL(to_char(p_date_start, g_HRAPI_DATE_FMT),-1) = NVL(to_char(l_abs_row.date_projected_start, g_HRAPI_DATE_FMT),-1)
               THEN null
               ELSE XMLElement
                     ("DateProjectedStart", to_char(p_date_start, g_HRAPI_DATE_FMT))
          END)
  INTO   l_date_projected_start_xml 
  FROM   dual;

  IF l_transaction_id is not null AND 
     l_transaction_step_id is not null THEN

    OPEN  csr_workflow_info(l_transaction_id);
    FETCH csr_workflow_info INTO l_workflow_info;
    CLOSE csr_workflow_info;

    l_transaction_status    := l_workflow_info.status;

    IF l_abs_row.absence_attendance_id is not null THEN

      OPEN  csr_rc_trans_doc(p_person_id
                            ,l_per_row
                            ,l_asg_row
                            ,l_login_per_row
                            ,l_per_manager_name
                            ,l_login_per_manager_name
                            ,l_transaction_id
                            ,l_per_sec_group_id
                            ,l_per_leg_code
                            ,l_login_per_leg_code
                            ,l_per_job_name
                            ,l_per_position_name
                            ,l_emp_generation
                            ,l_per_bg_cur_code
                            ,l_abs_row
                            ,p_called_from
                            ,p_assignment_id
                            ,p_effective_date
                            ,p_leave_status
                            ,p_absence_type_id
                            ,p_abs_attendance_reason_id
                            ,l_duration_days
                            ,l_duration_hours
                            ,p_date_start
                            ,p_date_end
                            ,l_time_start
                            ,l_time_end
                            ,p_attribute_category
                            ,l_attribute1
                            ,l_attribute2
                            ,l_attribute3
                            ,l_attribute4
                            ,l_attribute5
                            ,l_attribute6
                            ,l_attribute7
                            ,l_attribute8
                            ,l_attribute9
                            ,l_attribute10
                            ,l_attribute11
                            ,l_attribute12
                            ,l_attribute13
                            ,l_attribute14
                            ,l_attribute15
                            ,l_attribute16
                            ,l_attribute17
                            ,l_attribute18
                            ,l_attribute19
                            ,l_attribute20
                            ,p_abs_information_category
                            ,l_abs_information1
                            ,l_abs_information2
                            ,l_abs_information3
                            ,l_abs_information4
                            ,l_abs_information5
                            ,l_abs_information6
                            ,l_abs_information7
                            ,l_abs_information8
                            ,l_abs_information9
                            ,l_abs_information10
                            ,l_abs_information11
                            ,l_abs_information12
                            ,l_abs_information13
                            ,l_abs_information14
                            ,l_abs_information15
                            ,l_abs_information16
                            ,l_abs_information17
                            ,l_abs_information18
                            ,l_abs_information19
                            ,l_abs_information20
                            ,l_abs_information21
                            ,l_abs_information22
                            ,l_abs_information23
                            ,l_abs_information24
                            ,l_abs_information25
                            ,l_abs_information26
                            ,l_abs_information27
                            ,l_abs_information28
                            ,l_abs_information29
                            ,l_abs_information30
                            ,p_replacement_person_id
                            ,p_comments_to_approver
                            ,p_absence_attendance_id
                            ,p_object_version_number
                            ,l_date_end_xml
                            ,l_date_start_xml
                            ,l_date_projected_end_xml
                            ,l_date_projected_start_xml
                            ,p_ame_item_type
                            ,p_ame_app_id
                            ,p_ame_trans_type
                            ,l_transaction_status);
      FETCH csr_rc_trans_doc INTO l_rc_doc;
      CLOSE csr_rc_trans_doc;

    ELSE

      OPEN  csr_rc_trans_create_doc(p_person_id
                                   ,p_absence_attendance_id
                                   ,l_per_row
                                   ,l_asg_row
                                   ,l_login_per_row
                                   ,l_per_manager_name
                                   ,l_login_per_manager_name
                                   ,l_transaction_id
                                   ,l_per_sec_group_id
                                   ,l_per_leg_code
                                   ,l_login_per_leg_code
                                   ,l_per_job_name
                                   ,l_per_position_name
                                   ,l_emp_generation
                                   ,l_per_bg_cur_code
                                   ,p_called_from
                                   ,p_assignment_id
                                   ,p_effective_date
                                   ,p_leave_status
                                   ,p_absence_type_id
                                   ,p_abs_attendance_reason_id
                                   ,l_duration_days
                                   ,l_duration_hours
                                   ,p_date_start
                                   ,p_date_end
                                   ,l_time_start
                                   ,l_time_end
                                   ,p_attribute_category
                                   ,l_attribute1
                                   ,l_attribute2
                                   ,l_attribute3
                                   ,l_attribute4
                                   ,l_attribute5
                                   ,l_attribute6
                                   ,l_attribute7
                                   ,l_attribute8
                                   ,l_attribute9
                                   ,l_attribute10
                                   ,l_attribute11
                                   ,l_attribute12
                                   ,l_attribute13
                                   ,l_attribute14
                                   ,l_attribute15
                                   ,l_attribute16
                                   ,l_attribute17
                                   ,l_attribute18
                                   ,l_attribute19
                                   ,l_attribute20
                                   ,p_abs_information_category
                                   ,l_abs_information1
                                   ,l_abs_information2
                                   ,l_abs_information3
                                   ,l_abs_information4
                                   ,l_abs_information5
                                   ,l_abs_information6
                                   ,l_abs_information7
                                   ,l_abs_information8
                                   ,l_abs_information9
                                   ,l_abs_information10
                                   ,l_abs_information11
                                   ,l_abs_information12
                                   ,l_abs_information13
                                   ,l_abs_information14
                                   ,l_abs_information15
                                   ,l_abs_information16
                                   ,l_abs_information17
                                   ,l_abs_information18
                                   ,l_abs_information19
                                   ,l_abs_information20
                                   ,l_abs_information21
                                   ,l_abs_information22
                                   ,l_abs_information23
                                   ,l_abs_information24
                                   ,l_abs_information25
                                   ,l_abs_information26
                                   ,l_abs_information27
                                   ,l_abs_information28
                                   ,l_abs_information29
                                   ,l_abs_information30
                                   ,p_replacement_person_id
                                   ,p_comments_to_approver
                                   ,p_ame_item_type
                                   ,p_ame_app_id
                                   ,p_ame_trans_type
                                   ,l_transaction_status);
      FETCH csr_rc_trans_create_doc INTO l_rc_doc;
      CLOSE csr_rc_trans_create_doc;

    END IF;

  ELSE

    l_transaction_status := 'W';

    OPEN  csr_trans_doc(p_person_id
                       ,l_per_row
                       ,l_asg_row
                       ,l_login_per_row
                       ,l_per_manager_name
                       ,l_login_per_manager_name
                       ,l_transaction_id
                       ,l_per_sec_group_id
                       ,l_per_leg_code
                       ,l_login_per_leg_code
                       ,l_per_job_name
                       ,l_per_position_name
                       ,l_emp_generation
                       ,l_per_bg_cur_code
                       ,l_abs_row
                       ,p_called_from
                       ,p_assignment_id
                       ,p_effective_date
                       ,p_leave_status
                       ,p_absence_type_id
                       ,p_abs_attendance_reason_id
                       ,l_duration_days
                       ,l_duration_hours
                       ,p_date_start
                       ,p_date_end
                       ,l_time_start
                       ,l_time_end
                       ,p_attribute_category
                       ,l_attribute1
                       ,l_attribute2
                       ,l_attribute3
                       ,l_attribute4
                       ,l_attribute5
                       ,l_attribute6
                       ,l_attribute7
                       ,l_attribute8
                       ,l_attribute9
                       ,l_attribute10
                       ,l_attribute11
                       ,l_attribute12
                       ,l_attribute13
                       ,l_attribute14
                       ,l_attribute15
                       ,l_attribute16
                       ,l_attribute17
                       ,l_attribute18
                       ,l_attribute19
                       ,l_attribute20
                       ,p_abs_information_category
                       ,l_abs_information1
                       ,l_abs_information2
                       ,l_abs_information3
                       ,l_abs_information4
                       ,l_abs_information5
                       ,l_abs_information6
                       ,l_abs_information7
                       ,l_abs_information8
                       ,l_abs_information9
                       ,l_abs_information10
                       ,l_abs_information11
                       ,l_abs_information12
                       ,l_abs_information13
                       ,l_abs_information14
                       ,l_abs_information15
                       ,l_abs_information16
                       ,l_abs_information17
                       ,l_abs_information18
                       ,l_abs_information19
                       ,l_abs_information20
                       ,l_abs_information21
                       ,l_abs_information22
                       ,l_abs_information23
                       ,l_abs_information24
                       ,l_abs_information25
                       ,l_abs_information26
                       ,l_abs_information27
                       ,l_abs_information28
                       ,l_abs_information29
                       ,l_abs_information30
                       ,p_replacement_person_id
                       ,p_comments_to_approver
                       ,p_absence_attendance_id
                       ,p_object_version_number
                       ,l_date_end_xml
                       ,l_date_start_xml
                       ,l_date_projected_end_xml
                       ,l_date_projected_start_xml
                       ,p_ame_item_type
                       ,p_ame_app_id
                       ,p_ame_trans_type);
    FETCH csr_trans_doc INTO l_doc;
    CLOSE csr_trans_doc;

  END IF;

  OPEN  csr_get_abs_att_type_meaning(p_absence_type_id);
  FETCH csr_get_abs_att_type_meaning INTO l_abs_att_category;
  CLOSE csr_get_abs_att_type_meaning;

  IF l_transaction_id is not null AND 
     l_transaction_step_id is not null THEN

    update_hrapi_transaction(p_transaction_id        => l_transaction_id
                            ,p_transaction_step_id   => l_transaction_step_id
                            ,p_absence_attendance_id => p_absence_attendance_id
                            ,p_assignment_id         => p_assignment_id
                            ,p_person_id             => p_person_id
                            ,p_item_type             => p_ame_item_type
                            ,p_item_key              => null--l_item_key
                            ,p_absence_trans_doc     => l_rc_doc
                            ,p_information1          => CASE WHEN p_leave_status = g_PLANNED
                                                             THEN null
                                                             ELSE to_char(p_date_start, g_HRAPI_DATE_FMT)
                                                        END
                            ,p_information2          => CASE WHEN p_leave_status = g_PLANNED
                                                             THEN null
                                                             ELSE to_char(p_date_end, g_HRAPI_DATE_FMT)
                                                        END
                            ,p_information3          => CASE WHEN p_leave_status = g_PLANNED
                                                             THEN to_char(p_date_start, g_HRAPI_DATE_FMT)
                                                             ELSE null
                                                        END
                            ,p_information4          => CASE WHEN p_leave_status = g_PLANNED
                                                             THEN to_char(p_date_end, g_HRAPI_DATE_FMT)
                                                             ELSE null
                                                        END
                            ,p_information5          => p_absence_type_id
                            ,p_information6          => l_abs_att_category
                            ,p_information7          => l_duration_hours
                            ,p_information8          => l_duration_days
                            ,p_information9          => p_leave_status
                            ,p_object_state          => '2'
                            ,p_called_from           => p_called_from
                            ,p_ame_process_name      => p_ame_process_name
                            ,p_transaction_status    => l_transaction_status);

  ELSE

    l_transaction_step_id := create_hrapi_transaction
                               (p_transaction_id        => l_transaction_id
                               ,p_absence_attendance_id => p_absence_attendance_id
                               ,p_assignment_id         => p_assignment_id
                               ,p_person_id             => p_person_id
                               ,p_item_type             => p_ame_item_type
                               ,p_item_key              => l_item_key
                               ,p_absence_trans_doc     => l_doc
                               ,p_information1          => CASE WHEN p_leave_status = g_PLANNED
                                                                THEN null
                                                                ELSE to_char(p_date_start, g_HRAPI_DATE_FMT)
                                                           END
                               ,p_information2          => CASE WHEN p_leave_status = g_PLANNED
                                                                THEN null
                                                                ELSE to_char(p_date_end, g_HRAPI_DATE_FMT)
                                                           END
                               ,p_information3          => CASE WHEN p_leave_status = g_PLANNED
                                                                THEN to_char(p_date_start, g_HRAPI_DATE_FMT)
                                                                ELSE null
                                                           END
                               ,p_information4          => CASE WHEN p_leave_status = g_PLANNED
                                                                THEN to_char(p_date_end, g_HRAPI_DATE_FMT)
                                                                ELSE null
                                                           END
                               ,p_information5          => p_absence_type_id
                               ,p_information6          => l_abs_att_category
                               ,p_information7          => l_duration_hours
                               ,p_information8          => l_duration_days
                               ,p_information9          => p_leave_status
                               ,p_object_state          => '2'
                               ,p_called_from           => p_called_from
                               ,p_ame_process_name      => p_ame_process_name
                               ,p_transaction_status    => l_transaction_status);

  END IF;

  --
  -- Do create API's validation
  --
  --
  -- Issue a savepoint
  --
  savepoint before_api_validation;

  IF l_abs_row.absence_attendance_id is null THEN

    l_absence_attendance_id1 := p_absence_attendance_id;

    hr_person_absence_swi.create_person_absence
      (p_validate                     => hr_api.g_true_num
      ,p_effective_date               => p_effective_date
      ,p_person_id                    => p_person_id
      ,p_business_group_id            => l_asg_row.business_group_id
      ,p_absence_attendance_type_id   => p_absence_type_id
      ,p_abs_attendance_reason_id     => p_abs_attendance_reason_id
      ,p_comments                     => p_comments_to_approver
      ,p_date_notification            => p_date_notification
      ,p_date_projected_start         => CASE WHEN p_leave_status = g_PLANNED
                                              THEN p_date_start
                                              ELSE null
                                         END
      ,p_time_projected_start         => CASE WHEN p_leave_status = g_PLANNED
                                              THEN l_time_start
                                              ELSE null
                                         END
      ,p_date_projected_end           => CASE WHEN p_leave_status = g_PLANNED
                                              THEN p_date_end
                                              ELSE null
                                         END
      ,p_time_projected_end           => CASE WHEN p_leave_status = g_PLANNED
                                              THEN l_time_end
                                              ELSE null
                                         END
      ,p_date_start                   => CASE WHEN p_leave_status = g_PLANNED
                                              THEN null
                                              ELSE p_date_start
                                         END
      ,p_time_start                   => CASE WHEN p_leave_status = g_PLANNED
                                              THEN null
                                              ELSE l_time_start
                                         END
      ,p_date_end                     => CASE WHEN p_leave_status = g_PLANNED
                                              THEN null
                                              ELSE p_date_end
                                         END
      ,p_time_end                     => CASE WHEN p_leave_status = g_PLANNED
                                              THEN null
                                              ELSE l_time_end
                                         END
      ,p_absence_days                 => l_duration_days
      ,p_absence_hours                => l_duration_hours
      ,p_authorising_person_id        => null -- how to pass p_authorising_person_id
      ,p_replacement_person_id        => p_replacement_person_id
      ,p_attribute_category           => p_attribute_category
      ,p_attribute1                   => l_attribute1
      ,p_attribute2                   => l_attribute2
      ,p_attribute3                   => l_attribute3
      ,p_attribute4                   => l_attribute4
      ,p_attribute5                   => l_attribute5
      ,p_attribute6                   => l_attribute6
      ,p_attribute7                   => l_attribute7
      ,p_attribute8                   => l_attribute8
      ,p_attribute9                   => l_attribute9
      ,p_attribute10                  => l_attribute10
      ,p_attribute11                  => l_attribute11
      ,p_attribute12                  => l_attribute12
      ,p_attribute13                  => l_attribute13
      ,p_attribute14                  => l_attribute14
      ,p_attribute15                  => l_attribute15
      ,p_attribute16                  => l_attribute16
      ,p_attribute17                  => l_attribute17
      ,p_attribute18                  => l_attribute18
      ,p_attribute19                  => l_attribute19
      ,p_attribute20                  => l_attribute20
      ,p_period_of_incapacity_id      => null
      ,p_ssp1_issued                  => null
      ,p_maternity_id                 => null
      ,p_sickness_start_date          => null
      ,p_sickness_end_date            => null
      ,p_pregnancy_related_illness    => null
      ,p_reason_for_notification_dela => null
      ,p_accept_late_notification_fla => null
      ,p_linked_absence_id            => null
      ,p_batch_id                     => null
      ,p_create_element_entry         => null
      ,p_abs_information_category     => p_abs_information_category
      ,p_abs_information1             => l_abs_information1
      ,p_abs_information2             => l_abs_information2
      ,p_abs_information3             => l_abs_information3
      ,p_abs_information4             => l_abs_information4
      ,p_abs_information5             => l_abs_information5
      ,p_abs_information6             => l_abs_information6
      ,p_abs_information7             => l_abs_information7
      ,p_abs_information8             => l_abs_information8
      ,p_abs_information9             => l_abs_information9
      ,p_abs_information10            => l_abs_information10
      ,p_abs_information11            => l_abs_information11
      ,p_abs_information12            => l_abs_information12
      ,p_abs_information13            => l_abs_information13
      ,p_abs_information14            => l_abs_information14
      ,p_abs_information15            => l_abs_information15
      ,p_abs_information16            => l_abs_information16
      ,p_abs_information17            => l_abs_information17
      ,p_abs_information18            => l_abs_information18
      ,p_abs_information19            => l_abs_information19
      ,p_abs_information20            => l_abs_information20
      ,p_abs_information21            => l_abs_information21
      ,p_abs_information22            => l_abs_information22
      ,p_abs_information23            => l_abs_information23
      ,p_abs_information24            => l_abs_information24
      ,p_abs_information25            => l_abs_information25
      ,p_abs_information26            => l_abs_information26
      ,p_abs_information27            => l_abs_information27
      ,p_abs_information28            => l_abs_information28
      ,p_abs_information29            => l_abs_information29
      ,p_abs_information30            => l_abs_information30
      ,p_absence_case_id              => null
      ,p_absence_attendance_id        => l_absence_attendance_id1
      ,p_object_version_number        => l_object_version_number
      ,p_occurrence                   => l_occurrence
      ,p_return_status                => l_return_status);

  ELSIF l_abs_row.absence_attendance_id is not null THEN

    hr_person_absence_swi.update_person_absence
      (p_validate                     => hr_api.g_true_num
      ,p_effective_date               => p_effective_date
      ,p_absence_attendance_id        => p_absence_attendance_id
      ,p_abs_attendance_reason_id     => p_abs_attendance_reason_id
      ,p_comments                     => p_comments_to_approver
      ,p_date_notification            => p_date_notification
      ,p_date_projected_start         => CASE WHEN p_leave_status = g_PLANNED
                                              THEN p_date_start
                                              ELSE null
                                         END
      ,p_time_projected_start         => CASE WHEN p_leave_status = g_PLANNED
                                              THEN l_time_start
                                              ELSE null
                                         END
      ,p_date_projected_end           => CASE WHEN p_leave_status = g_PLANNED
                                              THEN p_date_end
                                              ELSE null
                                         END
      ,p_time_projected_end           => CASE WHEN p_leave_status = g_PLANNED
                                              THEN l_time_end
                                              ELSE null
                                         END
      ,p_date_start                   => CASE WHEN p_leave_status = g_PLANNED
                                              THEN null
                                              ELSE p_date_start
                                         END
      ,p_time_start                   => CASE WHEN p_leave_status = g_PLANNED
                                              THEN null
                                              ELSE l_time_start
                                         END
      ,p_date_end                     => CASE WHEN p_leave_status = g_PLANNED
                                              THEN null
                                              ELSE p_date_end
                                         END
      ,p_time_end                     => CASE WHEN p_leave_status = g_PLANNED
                                              THEN null
                                              ELSE l_time_end
                                         END
      ,p_absence_days                 => l_duration_days
      ,p_absence_hours                => l_duration_hours
      ,p_authorising_person_id        => l_abs_row.authorising_person_id
      ,p_replacement_person_id        => p_replacement_person_id
      ,p_attribute_category           => p_attribute_category
      ,p_attribute1                   => l_attribute1
      ,p_attribute2                   => l_attribute2
      ,p_attribute3                   => l_attribute3
      ,p_attribute4                   => l_attribute4
      ,p_attribute5                   => l_attribute5
      ,p_attribute6                   => l_attribute6
      ,p_attribute7                   => l_attribute7
      ,p_attribute8                   => l_attribute8
      ,p_attribute9                   => l_attribute9
      ,p_attribute10                  => l_attribute10
      ,p_attribute11                  => l_attribute11
      ,p_attribute12                  => l_attribute12
      ,p_attribute13                  => l_attribute13
      ,p_attribute14                  => l_attribute14
      ,p_attribute15                  => l_attribute15
      ,p_attribute16                  => l_attribute16
      ,p_attribute17                  => l_attribute17
      ,p_attribute18                  => l_attribute18
      ,p_attribute19                  => l_attribute19
      ,p_attribute20                  => l_attribute20
      ,p_period_of_incapacity_id      => l_abs_row.period_of_incapacity_id
      ,p_ssp1_issued                  => l_abs_row.ssp1_issued
      ,p_maternity_id                 => l_abs_row.maternity_id
      ,p_sickness_start_date          => l_abs_row.sickness_start_date
      ,p_sickness_end_date            => l_abs_row.sickness_end_date
      ,p_pregnancy_related_illness    => l_abs_row.pregnancy_related_illness
      ,p_reason_for_notification_dela => l_abs_row.reason_for_notification_delay
      ,p_accept_late_notification_fla => l_abs_row.accept_late_notification_flag
      ,p_linked_absence_id            => l_abs_row.linked_absence_id
      ,p_batch_id                     => l_abs_row.batch_id
      ,p_abs_information_category     => p_abs_information_category
      ,p_abs_information1             => l_abs_information1
      ,p_abs_information2             => l_abs_information2
      ,p_abs_information3             => l_abs_information3
      ,p_abs_information4             => l_abs_information4
      ,p_abs_information5             => l_abs_information5
      ,p_abs_information6             => l_abs_information6
      ,p_abs_information7             => l_abs_information7
      ,p_abs_information8             => l_abs_information8
      ,p_abs_information9             => l_abs_information9
      ,p_abs_information10            => l_abs_information10
      ,p_abs_information11            => l_abs_information11
      ,p_abs_information12            => l_abs_information12
      ,p_abs_information13            => l_abs_information13
      ,p_abs_information14            => l_abs_information14
      ,p_abs_information15            => l_abs_information15
      ,p_abs_information16            => l_abs_information16
      ,p_abs_information17            => l_abs_information17
      ,p_abs_information18            => l_abs_information18
      ,p_abs_information19            => l_abs_information19
      ,p_abs_information20            => l_abs_information20
      ,p_abs_information21            => l_abs_information21
      ,p_abs_information22            => l_abs_information22
      ,p_abs_information23            => l_abs_information23
      ,p_abs_information24            => l_abs_information24
      ,p_abs_information25            => l_abs_information25
      ,p_abs_information26            => l_abs_information26
      ,p_abs_information27            => l_abs_information27
      ,p_abs_information28            => l_abs_information28
      ,p_abs_information29            => l_abs_information29
      ,p_abs_information30            => l_abs_information30
      ,p_absence_case_id              => l_abs_row.absence_case_id
      ,p_object_version_number        => l_object_version_number
      ,p_return_status                => l_return_status);

  END IF;

  log_statement('l_return_status: ',l_return_status);
  
  IF l_return_status = 'E' THEN
    IF (fnd_msg_pub.count_msg > 0) THEN
      FOR i IN 1..fnd_msg_pub.count_msg
      LOOP

        fnd_msg_pub.get
         (p_msg_index     => i,
          p_encoded       => 'F',
          p_data          => l_msg_data,
          p_msg_index_out => l_msg_count
         );

        log_statement(c_method,'l_msg_data: ' || l_msg_data);

        --
        -- Skip the following warning messages
        --
        IF upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_EMP_ABS_SHORT_DURATION')) OR 
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_ABS_HOUR_LESS_DURATION')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_EMP_NOT_ENTITLED')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_RUNNING_ZERO')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_ABSENCE_OVERLAP')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_ABS_DAY_AFTER')) THEN

          null;

        ELSE

          register_user_error
            (p_package_name => gc_log.package_name
            ,p_proc_name    => c_method
            ,p_message      => l_msg_data);

          l_api_errors := true;

        END IF;

        l_msg_data   := null;
        l_msg_count  := 0;
      END LOOP;
    END IF;
  END IF;

  IF l_api_errors THEN
    xxas_com_api_result_pkg.raise_registered_user_errors;
  END IF;

  --
  -- Issue a rollback
  --
  rollback to before_api_validation;

  IF l_item_key is null THEN

    launch_workflow
      (p_person_id            => p_person_id
      ,p_transaction_id       => l_transaction_id
      ,p_comments_to_approver => p_comments_to_approver
      ,p_called_from          => p_called_from
      ,p_ame_item_type        => p_ame_item_type
      ,p_ame_app_id           => p_ame_app_id
      ,p_ame_trans_type       => p_ame_trans_type
      ,p_ame_process_name     => p_ame_process_name
      ,p_item_key             => l_item_key
      ,p_activity_id          => l_activity_id);

  END IF;

  IF l_transaction_id is not null THEN

    OPEN  csr_transaction_status(cp_transaction_id => l_transaction_id);
    FETCH csr_transaction_status INTO l_transaction_status;
    CLOSE csr_transaction_status;

  END IF;

  IF l_transaction_status is null OR 
     l_transaction_status = 'W' THEN

    hr_trn_upd.upd(p_transaction_id => l_transaction_id
                  ,p_validate       => false
                  ,p_status         => 'W'
                  ,p_item_key       => l_item_key);

  ELSE

    hr_trn_upd.upd(p_transaction_id => l_transaction_id
                  ,p_validate       => false
                  ,p_status         => l_transaction_status
                  ,p_item_key       => l_item_key);

  END IF;

  xxas_selfservice_workflow_pkg.set_item_attr_text 
    (p_item_type => p_ame_item_type
    ,p_item_key  => l_item_key
    ,p_name      => 'HR_RUNTIME_APPROVAL_REQ_FLAG'
    ,p_value     => 'YES_DYNAMIC');

  workflow_review_page
    (p_item_type   => p_ame_item_type
    ,p_item_key    => l_item_key
    ,p_next_result => 'NEXT');

  p_transaction_id        := l_transaction_id;
  p_transaction_step_id   := l_transaction_step_id;

  log_end(c_method);

EXCEPTION
  
  WHEN xxas_com_api_result_pkg.xxas_com_api_exception
  THEN
    rollback to before_api_validation;
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN hr_utility.hr_error
  THEN
    rollback to before_api_validation;
    register_user_error
      (p_package_name => gc_log.package_name
      ,p_proc_name    => c_method
      ,p_message      => hr_utility.get_message);
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN others
  THEN
    err_code := SQLCODE;
    err_msg  := SUBSTR(SQLERRM, 1, 200);

    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT'
                                 ,'Error Code: ' || err_code || ' and Error Message: ' || err_msg))));

END update_absence_event;

--
-- ------------------------------------------------------------------------ *
-- Name    : update_absence_form_handler
-- ------------------------------------------------------------------------ *
PROCEDURE update_absence_form_handler
  (p_function_id    IN  number
  ,p_form_data_xml  IN  XMLType
  ,p_user_submitted IN  varchar2 DEFAULT g_NO
  ,p_response_xml   OUT XMLType)
  
IS

  CURSOR csr_data
  IS
  SELECT r.nam field_name
        ,nvl(r.val1, nvl(r.val2, r.val3)) val
        ,r.column_value field_xml
  FROM (SELECT x.column_value.getRootElement() nam
              ,extract(x.column_value, '/*/text()').getCLOBVal() val1
              ,extract(x.column_value, '/*/*/text()').getCLOBVal() val2
              ,extract(x.column_value, '/*/*/id/text()').getCLOBVal() val3
              ,x.column_value
        FROM  TABLE(XMLSequence
                     (extract
                       (p_form_data_xml
                       ,'//*'
                     || decode('Y'
                              ,'Y', '[not(ancestor-or-self::rows) '
                                  , '[not(self::rows) ')
                     || ' and not(self::root)
                          and not(self::postType)
                          and not(self::submittedFieldId)
                          and not(self::value)
                          and not(self::id)
                          and not(self::label)
                          and not(self::description)]'))) x) r;

  CURSOR csr_date_time_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dtr.start_date,'"') start_date
        ,replace(dtr.end_date,'"') end_date
        ,replace(dtr.start_time,'"') start_time
        ,replace(dtr.end_time,'"') end_time
  FROM   XMLTable
          ('//DATE_TIME_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
                  ,start_time      varchar2(10)     path '//startTime'
                  ,end_time        varchar2(10)     path '//endTime'
          ) dtr;

  CURSOR csr_date_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dr.start_date,'"') start_date
        ,replace(dr.end_date,'"') end_date
  FROM   XMLTable
          ('//DATE_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
          ) dr;

  c_method    CONSTANT         varchar2(30) := 'update_absence_form_handler';
  l_person_id                  number;
  l_assignment_id              number;
  l_override_assignment_id     number;
  l_absence_attendance_id      number;
  l_transaction_id             number;
  l_transaction_step_id        number;
  l_object_version_number      number;
  l_effective_date             date;
  l_date_start                 date;
  l_time_start                 varchar2(5);
  l_date_end                   date;
  l_time_end                   varchar2(5);
  l_source_block               varchar2(2000);
  l_saved_search_id            number;
  l_manager_id                 number;
  l_duration_type              varchar2(10);
  l_duration_days              per_absence_attendances.absence_days%TYPE;
  l_duration_hours             per_absence_attendances.absence_hours%TYPE;
  l_leave_status               varchar2(4000);
  l_absence_type_id            number;
  l_abs_attendance_reason_id   number;
  l_date_notification          date;
  l_attribute_category         varchar2(4000);
  l_attribute1                 varchar2(4000);
  l_attribute2                 varchar2(4000);
  l_attribute3                 varchar2(4000);
  l_attribute4                 varchar2(4000);
  l_attribute5                 varchar2(4000);
  l_attribute6                 varchar2(4000);
  l_attribute7                 varchar2(4000);
  l_attribute8                 varchar2(4000);
  l_attribute9                 varchar2(4000);
  l_attribute10                varchar2(4000);
  l_attribute11                varchar2(4000);
  l_attribute12                varchar2(4000);
  l_attribute13                varchar2(4000);
  l_attribute14                varchar2(4000);
  l_attribute15                varchar2(4000);
  l_attribute16                varchar2(4000);
  l_attribute17                varchar2(4000);
  l_attribute18                varchar2(4000);
  l_attribute19                varchar2(4000);
  l_attribute20                varchar2(4000);
  l_abs_information_category   varchar2(4000);
  l_abs_information1           varchar2(4000);
  l_abs_information2           varchar2(4000);
  l_abs_information3           varchar2(4000);
  l_abs_information4           varchar2(4000);
  l_abs_information5           varchar2(4000);
  l_abs_information6           varchar2(4000);
  l_abs_information7           varchar2(4000);
  l_abs_information8           varchar2(4000);
  l_abs_information9           varchar2(4000);
  l_abs_information10          varchar2(4000);
  l_abs_information11          varchar2(4000);
  l_abs_information12          varchar2(4000);
  l_abs_information13          varchar2(4000);
  l_abs_information14          varchar2(4000);
  l_abs_information15          varchar2(4000);
  l_abs_information16          varchar2(4000);
  l_abs_information17          varchar2(4000);
  l_abs_information18          varchar2(4000);
  l_abs_information19          varchar2(4000);
  l_abs_information20          varchar2(4000);
  l_abs_information21          varchar2(4000);
  l_abs_information22          varchar2(4000);
  l_abs_information23          varchar2(4000);
  l_abs_information24          varchar2(4000);
  l_abs_information25          varchar2(4000);
  l_abs_information26          varchar2(4000);
  l_abs_information27          varchar2(4000);
  l_abs_information28          varchar2(4000);
  l_abs_information29          varchar2(4000);
  l_abs_information30          varchar2(4000);
  l_replacement_person_id      number;
  l_comments_to_approver       varchar2(4000);
  l_called_from                fnd_form_functions.function_name%TYPE;
  l_ame_item_type              varchar2(30);
  l_ame_app_id                 number;
  l_ame_trans_type             varchar2(30);
  l_ame_process_name           varchar2(30);
  l_data                       xxas_emp_view_ui_pkg.xxas_data_table;
  l_attachments_xml            XMLType;
  l_status                     varchar2(10);
  l_action                     varchar2(30);
  l_date_time_range_xml        XMLType;
  l_date_range_xml             XMLType;
  l_dtr_values_rec             csr_date_time_range_values%ROWTYPE;
  l_dr_values_rec              csr_date_range_values%ROWTYPE;
  err_code                     varchar2(200);
  err_msg                      varchar2(200);
  l_save_attachments           varchar2(10);
  l_save_attach_as_sup_docs    varchar2(10);
  l_called_view                varchar2(10);
  l_redirect_notif             varchar2(2);
  l_redirect_notif_old         varchar2(2);
  l_rule                       varchar2(30);
  l_grantee                    varchar2(30);
  l_show_review_page           varchar2(2);
  l_absence_days               number;
  l_absence_hours              number;
  l_use_formula                number;
  l_min_max_failure            varchar2(4000);
  l_warning_or_error           varchar2(4000);
  l_page_error_msg             varchar2(4000);
  l_cal_duration               per_absence_attendances.absence_days%TYPE;
  l_workflow_info              xxas_cal_util_pkg.workflow_info;
  l_current_tran_submit        hr_api_transactions.status%TYPE;

BEGIN

  log_start(c_method, p_function_id, p_user_submitted);

  xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

  --execute immediate 'alter session set nls_date_format="DD-MON-RR"';

  l_data := xxas_emp_view_ui_pkg.transform_jsonxml_form_data
              (p_form_data_xml);

  -- Retrieve the form values
  IF l_data.exists('PERSON_ID') THEN
    l_person_id := l_data('PERSON_ID');
  END IF;

  IF l_data.exists('ASSIGNMENT_ID') THEN
    l_assignment_id := l_data('ASSIGNMENT_ID');
  END IF;

  IF l_data.exists('OVERRIDE_ASSIGNMENT_ID') THEN
    l_override_assignment_id := l_data('OVERRIDE_ASSIGNMENT_ID');
  END IF;

  IF l_override_assignment_id is not null THEN
    l_assignment_id := l_override_assignment_id;
  END IF;

  IF l_data.exists('TRANSACTION_ID') THEN
    l_transaction_id := l_data('TRANSACTION_ID');
  END IF;

  IF l_data.exists('TRANSACTION_STEP_ID') THEN
    l_transaction_step_id := l_data('TRANSACTION_STEP_ID');
  END IF;

  IF l_data.exists('SOURCE_BLOCK') THEN
    l_source_block := l_data('SOURCE_BLOCK');
  END IF;

  IF l_data.exists('SAVED_SEARCH_ID') THEN
    l_saved_search_id := l_data('SAVED_SEARCH_ID');
  END IF;

  IF l_data.exists('OBJECT_VERSION_NUMBER') THEN
    l_object_version_number := l_data('OBJECT_VERSION_NUMBER');
  END IF;

  IF l_data.exists('LEAVE_STATUS') THEN
    l_leave_status := l_data('LEAVE_STATUS');
  END IF;

  IF l_data.exists('ABSENCE_ATTENDANCE_ID') THEN
    l_absence_attendance_id := l_data('ABSENCE_ATTENDANCE_ID');
  END IF;

  IF l_data.exists('ABSENCE_TYPE_ID') THEN
    l_absence_type_id := l_data('ABSENCE_TYPE_ID');
  END IF;

  IF l_data.exists('ABS_ATTENDANCE_REASON_ID') THEN
    l_abs_attendance_reason_id := l_data('ABS_ATTENDANCE_REASON_ID');
  END IF;

  IF l_data.exists('DATE_NOTIFICATION') THEN
    l_date_notification := fnd_date.canonical_to_date(l_data('DATE_NOTIFICATION'));
  END IF;

  IF l_data.exists('DURATION_TYPE') THEN
    l_duration_type := l_data('DURATION_TYPE');
  END IF;

  IF l_data.exists('DURATION_DAYS') THEN
    l_duration_days := l_data('DURATION_DAYS');
  END IF;

  IF l_data.exists('DURATION_HOURS') THEN
    l_duration_hours := l_data('DURATION_HOURS');
  END IF;

  IF l_data.exists('MANAGER_ID') THEN
    l_manager_id := l_data('MANAGER_ID');
  END IF;
  --
  -- Convert the XML form data into the xxas_data_table structure.
  --
  IF p_form_data_xml is not null THEN

    FOR csr_rec IN csr_data LOOP
      IF csr_rec.field_name is not null THEN
        IF csr_rec.field_name = 'ATTACHMENTS' THEN
          l_attachments_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'DATE_TIME_RANGE' AND 
              (l_duration_type is not null AND
               l_duration_type IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_time_range_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'DATE_RANGE' AND
              (l_duration_type is not null AND
               l_duration_type NOT IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_range_xml := csr_rec.field_xml;
        END IF;
      END IF;
    END LOOP;

  END IF;

  IF l_date_time_range_xml is not null THEN

    OPEN  csr_date_time_range_values(l_date_time_range_xml);
    FETCH csr_date_time_range_values INTO l_dtr_values_rec;
    CLOSE csr_date_time_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dtr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dtr_values_rec.end_date);
    l_time_start := l_dtr_values_rec.start_time;
    l_time_end   := l_dtr_values_rec.end_time;

  ELSIF l_date_range_xml is not null THEN

    OPEN  csr_date_range_values(l_date_range_xml);
    FETCH csr_date_range_values INTO l_dr_values_rec;
    CLOSE csr_date_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dr_values_rec.end_date);

  END IF;

  IF l_data.exists('ATTRIBUTE_CATEGORY') THEN
    l_attribute_category := l_data('ATTRIBUTE_CATEGORY');
  END IF;

  IF l_data.exists('ATTRIBUTE1') THEN
    l_attribute1 := l_data('ATTRIBUTE1');
  END IF;

  IF l_data.exists('ATTRIBUTE2') THEN
    l_attribute2 := l_data('ATTRIBUTE2');
  END IF;

  IF l_data.exists('ATTRIBUTE3') THEN
    l_attribute3 := l_data('ATTRIBUTE3');
  END IF;

  IF l_data.exists('ATTRIBUTE4') THEN
    l_attribute4 := l_data('ATTRIBUTE4');
  END IF;

  IF l_data.exists('ATTRIBUTE5') THEN
    l_attribute5 := l_data('ATTRIBUTE5');
  END IF;

  IF l_data.exists('ATTRIBUTE6') THEN
    l_attribute6 := l_data('ATTRIBUTE6');
  END IF;

  IF l_data.exists('ATTRIBUTE7') THEN
    l_attribute7 := l_data('ATTRIBUTE7');
  END IF;

  IF l_data.exists('ATTRIBUTE8') THEN
    l_attribute8 := l_data('ATTRIBUTE8');
  END IF;

  IF l_data.exists('ATTRIBUTE9') THEN
    l_attribute9 := l_data('ATTRIBUTE9');
  END IF;

  IF l_data.exists('ATTRIBUTE10') THEN
    l_attribute10 := l_data('ATTRIBUTE10');
  END IF;

  IF l_data.exists('ATTRIBUTE11') THEN
    l_attribute11 := l_data('ATTRIBUTE11');
  END IF;

  IF l_data.exists('ATTRIBUTE12') THEN
    l_attribute12 := l_data('ATTRIBUTE12');
  END IF;

  IF l_data.exists('ATTRIBUTE13') THEN
    l_attribute13 := l_data('ATTRIBUTE13');
  END IF;

  IF l_data.exists('ATTRIBUTE14') THEN
    l_attribute14 := l_data('ATTRIBUTE14');
  END IF;

  IF l_data.exists('ATTRIBUTE15') THEN
    l_attribute15 := l_data('ATTRIBUTE15');
  END IF;

  IF l_data.exists('ATTRIBUTE16') THEN
    l_attribute16 := l_data('ATTRIBUTE16');
  END IF;

  IF l_data.exists('ATTRIBUTE17') THEN
    l_attribute17 := l_data('ATTRIBUTE17');
  END IF;

  IF l_data.exists('ATTRIBUTE18') THEN
    l_attribute18 := l_data('ATTRIBUTE18');
  END IF;

  IF l_data.exists('ATTRIBUTE19') THEN
    l_attribute19 := l_data('ATTRIBUTE19');
  END IF;

  IF l_data.exists('ATTRIBUTE20') THEN
    l_attribute20 := l_data('ATTRIBUTE20');
  END IF;

  IF l_data.exists('ABS_INFORMATION_CATEGORY') THEN
    l_abs_information_category := l_data('ABS_INFORMATION_CATEGORY');
  END IF;

  IF l_data.exists('ABS_INFORMATION1') THEN
    l_abs_information1 := l_data('ABS_INFORMATION1');
  END IF;

  IF l_data.exists('ABS_INFORMATION2') THEN
    l_abs_information2 := l_data('ABS_INFORMATION2');
  END IF;

  IF l_data.exists('ABS_INFORMATION3') THEN
    l_abs_information3 := l_data('ABS_INFORMATION3');
  END IF;

  IF l_data.exists('ABS_INFORMATION4') THEN
    l_abs_information4 := l_data('ABS_INFORMATION4');
  END IF;

  IF l_data.exists('ABS_INFORMATION5') THEN
    l_abs_information5 := l_data('ABS_INFORMATION5');
  END IF;

  IF l_data.exists('ABS_INFORMATION6') THEN
    l_abs_information6 := l_data('ABS_INFORMATION6');
  END IF;

  IF l_data.exists('ABS_INFORMATION7') THEN
    l_abs_information7 := l_data('ABS_INFORMATION7');
  END IF;

  IF l_data.exists('ABS_INFORMATION8') THEN
    l_abs_information8 := l_data('ABS_INFORMATION8');
  END IF;

  IF l_data.exists('ABS_INFORMATION9') THEN
    l_abs_information9 := l_data('ABS_INFORMATION9');
  END IF;

  IF l_data.exists('ABS_INFORMATION10') THEN
    l_abs_information10 := l_data('ABS_INFORMATION10');
  END IF;

  IF l_data.exists('ABS_INFORMATION11') THEN
    l_abs_information11 := l_data('ABS_INFORMATION11');
  END IF;

  IF l_data.exists('ABS_INFORMATION12') THEN
    l_abs_information12 := l_data('ABS_INFORMATION12');
  END IF;

  IF l_data.exists('ABS_INFORMATION13') THEN
    l_abs_information13 := l_data('ABS_INFORMATION13');
  END IF;

  IF l_data.exists('ABS_INFORMATION14') THEN
    l_abs_information14 := l_data('ABS_INFORMATION14');
  END IF;

  IF l_data.exists('ABS_INFORMATION15') THEN
    l_abs_information15 := l_data('ABS_INFORMATION15');
  END IF;

  IF l_data.exists('ABS_INFORMATION16') THEN
    l_abs_information16 := l_data('ABS_INFORMATION16');
  END IF;

  IF l_data.exists('ABS_INFORMATION17') THEN
    l_abs_information17 := l_data('ABS_INFORMATION17');
  END IF;

  IF l_data.exists('ABS_INFORMATION18') THEN
    l_abs_information18 := l_data('ABS_INFORMATION18');
  END IF;

  IF l_data.exists('ABS_INFORMATION19') THEN
    l_abs_information19 := l_data('ABS_INFORMATION19');
  END IF;

  IF l_data.exists('ABS_INFORMATION20') THEN
    l_abs_information20 := l_data('ABS_INFORMATION20');
  END IF;

  IF l_data.exists('ABS_INFORMATION21') THEN
    l_abs_information21 := l_data('ABS_INFORMATION21');
  END IF;

  IF l_data.exists('ABS_INFORMATION22') THEN
    l_abs_information22 := l_data('ABS_INFORMATION22');
  END IF;

  IF l_data.exists('ABS_INFORMATION23') THEN
    l_abs_information23 := l_data('ABS_INFORMATION23');
  END IF;

  IF l_data.exists('ABS_INFORMATION24') THEN
    l_abs_information24 := l_data('ABS_INFORMATION24');
  END IF;

  IF l_data.exists('ABS_INFORMATION25') THEN
    l_abs_information25 := l_data('ABS_INFORMATION25');
  END IF;

  IF l_data.exists('ABS_INFORMATION26') THEN
    l_abs_information26 := l_data('ABS_INFORMATION26');
  END IF;

  IF l_data.exists('ABS_INFORMATION27') THEN
    l_abs_information27 := l_data('ABS_INFORMATION27');
  END IF;

  IF l_data.exists('ABS_INFORMATION28') THEN
    l_abs_information28 := l_data('ABS_INFORMATION28');
  END IF;

  IF l_data.exists('ABS_INFORMATION29') THEN
    l_abs_information29 := l_data('ABS_INFORMATION29');
  END IF;

  IF l_data.exists('ABS_INFORMATION30') THEN
    l_abs_information30 := l_data('ABS_INFORMATION30');
  END IF;

  IF l_data.exists('REPLACEMENT_PERSON_ID') THEN
    l_replacement_person_id := l_data('REPLACEMENT_PERSON_ID');
  END IF;

  IF l_data.exists('COMMENTS_TO_APPROVER') THEN
    l_comments_to_approver := l_data('COMMENTS_TO_APPROVER');
  END IF;

  IF l_data.exists('CALLED_VIEW') THEN
    l_called_view := l_data('CALLED_VIEW');
  END IF;
  
  l_effective_date := trunc(sysdate);

  l_called_from := xxas_com_sec_pkg.get_function_parameter
                     (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                     ,p_parameter_name => 'pCalledFrom');

  l_ame_item_type    := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pItemType');
  l_ame_app_id       := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pAMEAppId');
  l_ame_trans_type   := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pAMETranType');
  l_ame_process_name := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pProcessName');

  l_cal_duration := xxas_leave_of_absence_pkg.calculate_abs_duration_full
                      (p_business_group_id => xxas_per_pkg.get_business_group_id(l_person_id)
                      ,p_assignment_id     => l_assignment_id
                      ,p_effective_date    => l_effective_date
                      ,p_absence_type_id   => l_absence_type_id
                      ,p_date_start        => l_date_start
                      ,p_date_end          => l_date_end
                      ,p_time_start        => l_time_start
                      ,p_time_end          => l_time_end
                      ,p_absence_days      => l_absence_days
                      ,p_absence_hours     => l_absence_hours
                      ,p_use_formula       => l_use_formula
                      ,p_min_max_failure   => l_min_max_failure
                      ,p_warning_or_error  => l_warning_or_error
                      ,p_page_error_msg    => l_page_error_msg);

  IF l_page_error_msg is not null THEN

    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => null
      ,p_proc_name    => null
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT',l_page_error_msg)
                              )
                           ));

  END IF;

  update_absence_event(p_person_id                => l_person_id
                      ,p_assignment_id            => l_assignment_id
                      ,p_date_start               => l_date_start
                      ,p_date_end                 => l_date_end
                      ,p_time_start               => l_time_start
                      ,p_time_end                 => l_time_end
                      ,p_duration_type            => l_duration_type
                      ,p_duration_days            => l_duration_days
                      ,p_duration_hours           => l_duration_hours
                      ,p_effective_date           => l_effective_date
                      ,p_called_from              => l_called_from
                      ,p_leave_status             => l_leave_status
                      ,p_absence_type_id          => l_absence_type_id
                      ,p_abs_attendance_reason_id => l_abs_attendance_reason_id
                      ,p_attribute_category       => l_attribute_category
                      ,p_attribute1               => l_attribute1
                      ,p_attribute2               => l_attribute2
                      ,p_attribute3               => l_attribute3
                      ,p_attribute4               => l_attribute4
                      ,p_attribute5               => l_attribute5
                      ,p_attribute6               => l_attribute6
                      ,p_attribute7               => l_attribute7
                      ,p_attribute8               => l_attribute8
                      ,p_attribute9               => l_attribute9
                      ,p_attribute10              => l_attribute10
                      ,p_attribute11              => l_attribute11
                      ,p_attribute12              => l_attribute12
                      ,p_attribute13              => l_attribute13
                      ,p_attribute14              => l_attribute14
                      ,p_attribute15              => l_attribute15
                      ,p_attribute16              => l_attribute16
                      ,p_attribute17              => l_attribute17
                      ,p_attribute18              => l_attribute18
                      ,p_attribute19              => l_attribute19
                      ,p_attribute20              => l_attribute20
                      ,p_abs_information_category => l_abs_information_category
                      ,p_abs_information1         => l_abs_information1
                      ,p_abs_information2         => l_abs_information2
                      ,p_abs_information3         => l_abs_information3
                      ,p_abs_information4         => l_abs_information4
                      ,p_abs_information5         => l_abs_information5
                      ,p_abs_information6         => l_abs_information6
                      ,p_abs_information7         => l_abs_information7
                      ,p_abs_information8         => l_abs_information8
                      ,p_abs_information9         => l_abs_information9
                      ,p_abs_information10        => l_abs_information10
                      ,p_abs_information11        => l_abs_information11
                      ,p_abs_information12        => l_abs_information12
                      ,p_abs_information13        => l_abs_information13
                      ,p_abs_information14        => l_abs_information14
                      ,p_abs_information15        => l_abs_information15
                      ,p_abs_information16        => l_abs_information16
                      ,p_abs_information17        => l_abs_information17
                      ,p_abs_information18        => l_abs_information18
                      ,p_abs_information19        => l_abs_information19
                      ,p_abs_information20        => l_abs_information20
                      ,p_abs_information21        => l_abs_information21
                      ,p_abs_information22        => l_abs_information22
                      ,p_abs_information23        => l_abs_information23
                      ,p_abs_information24        => l_abs_information24
                      ,p_abs_information25        => l_abs_information25
                      ,p_abs_information26        => l_abs_information26
                      ,p_abs_information27        => l_abs_information27
                      ,p_abs_information28        => l_abs_information28
                      ,p_abs_information29        => l_abs_information29
                      ,p_abs_information30        => l_abs_information30
                      ,p_replacement_person_id    => l_replacement_person_id
                      ,p_comments_to_approver     => l_comments_to_approver
                      ,p_date_notification        => l_date_notification
                      ,p_ame_item_type            => l_ame_item_type
                      ,p_ame_app_id               => l_ame_app_id
                      ,p_ame_trans_type           => l_ame_trans_type
                      ,p_ame_process_name         => l_ame_process_name
                      ,p_object_version_number    => l_object_version_number
                      ,p_absence_attendance_id    => l_absence_attendance_id
                      ,p_transaction_id           => l_transaction_id
                      ,p_transaction_step_id      => l_transaction_step_id);
  
  /* *************************************************************
     Check for vacation rules setup
     If vacation rules is setup then add the values
     to Transaction document. It will be used in absence user hook
     to create vacation rules.
     ************************************************************* */
     
  IF xxas_com_sec_pkg.test_instance('XXAS_ABS_VAC_RULE_SETUP',l_person_id) = 'Y' THEN
    
    IF l_data.exists('REDIRECT_NOTIF') THEN
      l_redirect_notif := l_data('REDIRECT_NOTIF');
    END IF;
  
    IF l_data.exists('REDIRECT_NOTIF_OLD') THEN
      l_redirect_notif_old := l_data('REDIRECT_NOTIF_OLD');
    END IF;
    
    IF l_data.exists('RULE') THEN
      l_rule := l_data('RULE');
    END IF;
  
    IF l_data.exists('GRANTEE') THEN
      l_grantee := l_data('GRANTEE');
    END IF;
    
    log_statement(c_method,'Before Update l_redirect_notif:' || l_redirect_notif);
    log_statement(c_method,'l_redirect_notif_old:' || l_redirect_notif_old);
    
    IF l_redirect_notif_old = 'Y' THEN
      IF l_redirect_notif = 'Y' THEN
        l_redirect_notif := 'Y'; -- Rule already exists and it is updated now. 
      ELSE 
        l_redirect_notif := 'D'; -- Rule is already existing and it is removed now. This should delete rules.
      END IF;
    ELSIF l_redirect_notif = 'Y' AND nvl(l_date_end,trunc(sysdate)) < trunc(sysdate) THEN
      l_redirect_notif := 'E'; -- New rule is created but with retro absence dates. This should raise info message.
    END IF;
    
    log_statement(c_method,'After Update l_redirect_notif:' || l_redirect_notif);
    
    -- set transaction values
    hr_transaction_api.set_varchar2_value(p_transaction_step_id => l_transaction_step_id
                                         ,p_person_id           => l_person_id
                                         ,p_name                => 'P_SETUP_VAC_RULE'
                                         ,p_value               => l_redirect_notif);
    IF l_redirect_notif = 'Y' THEN
      hr_transaction_api.set_varchar2_value(p_transaction_step_id => l_transaction_step_id
                                           ,p_person_id           => l_person_id
                                           ,p_name                => 'P_RULE'
                                           ,p_value               => l_rule);
                                           
      hr_transaction_api.set_varchar2_value(p_transaction_step_id => l_transaction_step_id
                                           ,p_person_id           => l_person_id
                                           ,p_name                => 'P_ASSIGNEE'
                                           ,p_value               => l_grantee);
    END IF;
  END IF;
  
  save_file_attachments
    (p_attachments_xml       => l_attachments_xml
    ,p_data                  => l_data
    ,p_instance_pk1_value    => l_person_id
    ,p_instance_pk2_value    => l_absence_attendance_id
    ,p_instance_pk3_value    => l_transaction_id
    ,p_instance_pk4_value    => l_transaction_step_id
    ,p_instance_pk5_value    => null
    ,p_entity_name           => 'PQH_SS_ATTACHMENT'
    ,p_pk1_value             => 'INSTANCE_PK3_VALUE'
    ,p_category_code         => 'MISC'
    ,p_absence_attendance_id => l_absence_attendance_id
    ,p_transaction_id        => l_transaction_id);

  -- Added for GIT#2626
  l_show_review_page := nvl(xxas_com_sec_pkg.get_function_parameter
                                   (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                                   ,p_parameter_name => 'pShowApprovalPageFlag')
                           ,'N');

  IF l_show_review_page = 'N' THEN
    xxas_emp_view_ui_pkg.register_result_details
    (p_transaction_id => l_transaction_id);
  END IF;

  l_workflow_info := xxas_cal_util_pkg.get_workflow_info
                       (p_transaction_id => l_transaction_id);

  IF l_workflow_info.item_type is not null AND
     l_workflow_info.item_key  is not null THEN

    l_current_tran_submit := xxas_selfservice_workflow_pkg.get_wf_attribute_text
                               (p_item_type      => l_workflow_info.item_type
                               ,p_item_key       => l_workflow_info.item_key
                               ,p_attribute_name => 'TRAN_SUBMIT');

    IF l_workflow_info.trans_status = 'W' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'W');

    ELSIF l_workflow_info.trans_status = 'RI' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'RI');

    ELSIF l_workflow_info.trans_status = 'Y' AND 
          l_current_tran_submit        = 'RI' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'RI');

    ELSIF l_workflow_info.trans_status = 'Y' AND 
          l_current_tran_submit        = 'Y' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'Y');

    END IF;

  END IF;
                           
  IF NOT (l_show_review_page = 'Y' OR approval_required) THEN

    log_statement(c_method,'approval_required is false');

    l_status := xxas_xml_node_util_pkg.get_node_value(l_transaction_id, 'TxnStatus', 'Transaction/TransCtx');
    l_action := xxas_xml_node_util_pkg.get_node_value(l_transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode');

    l_save_attachments := xxas_com_sec_pkg.get_function_parameter
                            (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                            ,p_parameter_name => 'pSaveAttachments');

    l_save_attach_as_sup_docs := xxas_com_sec_pkg.get_function_parameter
                                   (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                                   ,p_parameter_name => 'pSaveAttachmentsAsSupportingDocs');

    p_response_xml := submit_emp_view_row
                        (p_effective_date          => trunc(sysdate)
                        ,p_person_id               => l_person_id
                        ,p_absence_attendance_id   => l_absence_attendance_id
                        ,p_transaction_id          => l_transaction_id
                        ,p_transaction_step_id     => l_transaction_step_id
                        ,p_status                  => l_status
                        ,p_action                  => l_action
                        ,p_approvers               => 'N'
                        ,p_source_block            => l_source_block
                        ,p_saved_search_id         => l_saved_search_id
                        ,p_save_attachments        => l_save_attachments
                        ,p_save_attach_as_sup_docs => l_save_attach_as_sup_docs
                        ,p_manager_id              => l_manager_id);

  ELSE

    log_statement(c_method,'approval_required is true'); 
    
    p_response_xml := go_to_review_page_xml
                        (p_person_id             => l_person_id
                        ,p_absence_attendance_id => l_absence_attendance_id
                        ,p_transaction_id        => l_transaction_id
                        ,p_transaction_step_id   => l_transaction_step_id
                        ,p_source_block          => l_source_block
                        ,p_saved_search_id       => l_saved_search_id
                        ,p_manager_id              => l_manager_id);

  END IF;

  --Check and raise any workflow errors
  raise_workflow_error(l_transaction_id);
  
  log_end(c_method);

EXCEPTION
  
  WHEN xxas_com_api_result_pkg.xxas_com_api_exception
  THEN
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN others
  THEN
    err_code := SQLCODE;
    err_msg  := SUBSTR(SQLERRM, 1, 200);

    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT'
                                 ,'Error Code: ' || err_code || ' and Error Message: ' || err_msg))));

END update_absence_form_handler;

--
-- ------------------------------------------------------------------------ *
-- Name    : delete_absence_event
-- ------------------------------------------------------------------------ *
FUNCTION delete_absence_event
  (p_absence_attendance_id IN number
  ,p_function_id           IN number
  ,p_source_block          IN varchar2
  ,p_saved_search_id       IN number
  ,p_comments_to_approver  IN varchar2 DEFAULT null
  ,p_manager_id            IN number DEFAULT null) RETURN XMLType
IS

  CURSOR csr_asg(cp_person_id IN number) IS
  SELECT paaf.*
  FROM   per_all_assignments_f paaf
  WHERE  paaf.person_id = cp_person_id
  AND    trunc(sysdate) BETWEEN paaf.effective_start_date 
                        AND     paaf.effective_end_date
  AND    nvl(paaf.primary_flag, 'N') = 'Y'
  AND    (paaf.assignment_type = 'E' 
          OR paaf.assignment_type = 'C');

  CURSOR csr_person(cp_person_id IN number) IS
  SELECT papf.*
  FROM   per_all_people_f papf
  WHERE  papf.person_id = cp_person_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y');

  CURSOR csr_login_person(cp_login_person_id IN number) IS
  SELECT papf.*
        ,paaf.supervisor_id
        ,paaf.assignment_number
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN paaf.effective_start_date 
                        AND     paaf.effective_end_date
  AND    papf.person_id = cp_login_person_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    ((nvl(papf.current_employee_flag, 'N') = 'Y' AND
           paaf.assignment_type = 'E')
          OR (nvl(papf.current_npw_flag, 'N') = 'Y' AND
              paaf.assignment_type = 'C'))
  AND    nvl(paaf.primary_flag, 'N') = 'Y';

  CURSOR csr_abs_attendance_info(cp_absence_attendance_id IN number) IS
  SELECT paa.*
  FROM   per_absence_attendances paa
  WHERE  paa.absence_attendance_id = cp_absence_attendance_id;

  CURSOR csr_get_emp_full_name(cp_person_id IN number) IS
  SELECT decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'),'G', global_name, local_name) full_name
  FROM   per_all_people_f papf
  WHERE  papf.person_id = cp_person_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y');

  CURSOR csr_get_sec_grp_leg_code(cp_person_id IN number) IS
  SELECT bg.security_group_id
        ,bg.legislation_code
  FROM   per_all_people_f papf
        ,per_business_groups_perf bg
  WHERE  papf.person_id = cp_person_id
  AND    papf.business_group_id = bg.business_group_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                        AND     papf.effective_end_date
  AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
          OR nvl(papf.current_npw_flag, 'N') = 'Y');

  CURSOR csr_get_job_name(cp_job_id IN number) IS
  SELECT pjv.name 
  FROM   per_jobs_vl pjv
  WHERE  pjv.job_id = cp_job_id 
  AND    trunc(sysdate) BETWEEN pjv.date_from 
                        AND     nvl(pjv.date_to,trunc(sysdate));

  CURSOR csr_get_pos_name(cp_pos_id IN number) IS
  SELECT hap.name 
  FROM   hr_all_positions_f_vl hap
  WHERE  hap.position_id = cp_pos_id 
  AND    trunc(sysdate) BETWEEN nvl(hap.effective_start_date,trunc(sysdate)) 
                        AND     nvl(hap.effective_end_date,trunc(sysdate));

  CURSOR csr_get_org_info(cp_person_id IN number) IS
  SELECT hoi.org_information2
        ,hoi.org_information10
  FROM   hr_organization_information_v hoi
  WHERE  hoi.org_information_context = 'Business Group Information'
  AND    hoi.organization_id         = xxas_per_pkg.get_business_group_id(cp_person_id);

  CURSOR csr_get_abs_att_type_meaning(cp_abs_att_type_id IN number) IS
  SELECT fcl.meaning
  FROM   per_abs_attendance_types_vl paat
        ,hr_lookups fcl
  WHERE  absence_attendance_type_id = cp_abs_att_type_id
  AND    paat.absence_category      = fcl.lookup_code(+)
  AND    fcl.lookup_type(+)         = 'ABSENCE_CATEGORY';

  CURSOR csr_trans_doc(cp_person_id               IN number
                      ,cp_per_row                 IN csr_person%ROWTYPE
                      ,cp_asg_row                 IN csr_asg%ROWTYPE
                      ,cp_login_per_row           IN csr_login_person%ROWTYPE
                      ,cp_per_manager_name        IN varchar2
                      ,cp_login_per_manager_name  IN varchar2
                      ,cp_transaction_id          IN number
                      ,cp_per_sec_group_id        IN number
                      ,cp_per_leg_code            IN varchar2
                      ,cp_login_per_leg_code      IN varchar2
                      ,cp_job_name                IN varchar2
                      ,cp_pos_name                IN varchar2
                      ,cp_emp_generation          IN varchar2
                      ,cp_per_bg_cur_code         IN varchar2
                      ,cp_abs_row                 IN per_absence_attendances%ROWTYPE
                      ,cp_called_from             IN varchar2
                      ,cp_assignment_id           IN number
                      ,cp_effective_date          IN date
                      ,cp_date_notification       IN date
                      ,cp_object_version_number   IN number
                      ,cp_comments_to_approver    IN varchar2
                      ,cp_ame_item_type           IN varchar2
                      ,cp_ame_app_id              IN number
                      ,cp_ame_trans_type          IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", cp_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", cp_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", cp_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", cp_ame_item_type)
             ,XMLElement("PrsnJobId", cp_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(cp_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", cp_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), p_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", cp_login_per_leg_code)
             ,XMLElement("PrsnMgrId", cp_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", cp_assignment_id)
             ,XMLElement("PrsnLegCode", cp_per_leg_code)
             ,XMLElement("LoginPrsnId", cp_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", g_PER)
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(cp_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,xxas_com_sec_pkg.get_function_id(cp_called_from)
               )
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", cp_transaction_id)
             ,XMLElement("PrsnLocationId", cp_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", cp_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", cp_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", cp_per_manager_name)
             ,XMLElement("PrsnOrganizationId", cp_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", cp_asg_row.position_id)
             ,XMLElement("TxnStatus", 'W')
             ,XMLElement("LoginWorkerNumber", cp_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", cp_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", cp_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", cp_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(cp_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_login_per_row.global_name, cp_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), cp_login_per_row.person_id)
             ,XMLElement("PrsnId", cp_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || cp_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", cp_ame_trans_type)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(cp_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", cp_called_from)
             ,XMLElement("LoginPrsnBgId", cp_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", cp_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", cp_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_per_row.global_name, cp_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", p_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CancelMode'))
             ,XMLElement("pAMEAppId", cp_ame_app_id)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (2 as "PS")
                       ,XMLElement("AbsenceAttendanceId", p_absence_attendance_id)
                       ,XMLElement("DateNotification", to_char(cp_date_notification, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", cp_object_version_number)
                       ,decode(cp_comments_to_approver, null, null, XMLElement("Comments", cp_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(cp_transaction_id))
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_rc_trans_doc(cp_person_id               IN number
                         ,cp_per_row                 IN csr_person%ROWTYPE
                         ,cp_asg_row                 IN csr_asg%ROWTYPE
                         ,cp_login_per_row           IN csr_login_person%ROWTYPE
                         ,cp_per_manager_name        IN varchar2
                         ,cp_login_per_manager_name  IN varchar2
                         ,cp_transaction_id          IN number
                         ,cp_per_sec_group_id        IN number
                         ,cp_per_leg_code            IN varchar2
                         ,cp_login_per_leg_code      IN varchar2
                         ,cp_job_name                IN varchar2
                         ,cp_pos_name                IN varchar2
                         ,cp_emp_generation          IN varchar2
                         ,cp_per_bg_cur_code         IN varchar2
                         ,cp_abs_row                 IN per_absence_attendances%ROWTYPE
                         ,cp_called_from             IN varchar2
                         ,cp_assignment_id           IN number
                         ,cp_effective_date          IN date
                         ,cp_date_notification       IN date
                         ,cp_object_version_number   IN number
                         ,cp_comments_to_approver    IN varchar2
                         ,cp_ame_item_type           IN varchar2
                         ,cp_ame_app_id              IN number
                         ,cp_ame_trans_type          IN varchar2
                         ,cp_transaction_status      IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", cp_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", cp_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", cp_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", cp_ame_item_type)
             ,XMLElement("PrsnJobId", cp_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(cp_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", cp_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), p_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", cp_login_per_leg_code)
             ,XMLElement("PrsnMgrId", cp_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", cp_assignment_id)
             ,XMLElement("PrsnLegCode", cp_per_leg_code)
             ,XMLElement("LoginPrsnId", cp_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", g_PER)
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(cp_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,xxas_com_sec_pkg.get_function_id(cp_called_from)
               )
             ,XMLElement("TxnState", 'T')
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", cp_transaction_id)
             ,XMLElement("PrsnLocationId", cp_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", cp_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", cp_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", cp_per_manager_name)
             ,XMLElement("PrsnOrganizationId", cp_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", cp_asg_row.position_id)
             ,decode(cp_transaction_status, 'Y', XMLElement("TxnStatus", 'Y'), XMLElement("TxnStatus", 'RI'))
             ,XMLElement("LoginWorkerNumber", cp_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", cp_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", cp_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", cp_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(cp_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_login_per_row.global_name, cp_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), cp_login_per_row.person_id)
             ,XMLElement("PrsnId", cp_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || cp_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", cp_ame_trans_type)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(cp_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", cp_called_from)
             ,XMLElement("LoginPrsnBgId", cp_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", cp_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", cp_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', cp_per_row.global_name, cp_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", p_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CancelMode'))
             ,XMLElement("pAMEAppId", cp_ame_app_id)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (2 as "PS")
                       ,XMLElement("AbsenceAttendanceId", p_absence_attendance_id)
                       ,XMLElement("DateNotification", to_char(cp_date_notification, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", cp_object_version_number)
                       ,decode(cp_comments_to_approver, null, null, XMLElement("Comments", cp_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA('000100000000')
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_workflow_info(cp_transaction_id IN number) IS
  SELECT hat.item_type
        ,hat.item_key
        ,hat.transaction_ref_id
        ,hat.status
  FROM   hr_api_transactions  hat
  WHERE  hat.transaction_id = cp_transaction_id;

  CURSOR csr_transaction_status(cp_transaction_id IN number) IS
  SELECT hat.status
  FROM   hr_api_transactions  hat
  WHERE  hat.transaction_id = cp_transaction_id;

  c_method    CONSTANT         varchar2(30) := 'delete_absence_event';
  l_object_version_number      number;
  l_item_type                  wf_item_activity_statuses.item_type%TYPE;
  l_item_key                   wf_item_activity_statuses.item_key%TYPE;
  l_activity_id                wf_item_activity_statuses.process_activity%TYPE;
  l_asg_row                    csr_asg%ROWTYPE;
  l_per_row                    csr_person%ROWTYPE;
  l_login_per_row              csr_login_person%ROWTYPE;
  l_abs_row                    per_absence_attendances%ROWTYPE;
  l_login_person_id            number;
  l_per_manager_name           varchar2(240);
  l_login_per_manager_name     varchar2(240);
  l_doc                        XMLType;
  l_rc_doc                     XMLType;
  l_transaction_id             number;
  l_per_sec_group_id           number;
  l_per_leg_code               varchar2(150);
  l_login_per_sec_group_id     number;
  l_login_per_leg_code         varchar2(150);
  l_per_job_name               varchar2(700);
  l_per_position_name          varchar2(240);
  l_emp_generation             varchar2(150);
  l_per_bg_cur_code            varchar2(150);
  l_abs_att_category           varchar2(240);
  l_abs_dur_hours              number;
  l_abs_dur_days               number;
  l_transaction_step_id        number;
  l_msg_data                   varchar2(2000) := null;
  l_msg_count                  number := 0;
  l_api_errors                 boolean := false;
  l_return_status              varchar2(30);
  l_effective_date             date;
  l_person_id                  number;
  l_assignment_id              number;
  l_date_notification          date;
  l_response_xml               XMLType;
  l_function_id                fnd_form_functions.function_id%TYPE := p_function_id;
  l_workflow_info              csr_workflow_info%ROWTYPE;
  l_called_from                fnd_form_functions.function_name%TYPE;
  l_ame_item_type              varchar2(30);
  l_ame_app_id                 number;
  l_ame_trans_type             varchar2(30);
  l_ame_process_name           varchar2(30);
  err_code                     varchar2(200);
  err_msg                      varchar2(200);
  l_status                     varchar2(10);
  l_action                     varchar2(30);
  l_save_attachments           varchar2(10);
  l_save_attach_as_sup_docs    varchar2(10);
  l_show_review_page           varchar2(2);
  l_workflow_info2             xxas_cal_util_pkg.workflow_info;
  l_transaction_status         hr_api_transactions.status%TYPE;
  l_current_tran_submit        hr_api_transactions.status%TYPE;

BEGIN

  log_start(c_method, p_absence_attendance_id,p_function_id,p_source_block,p_comments_to_approver,p_manager_id);

  xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

  l_effective_date := trunc(sysdate);

  OPEN  csr_abs_attendance_info(p_absence_attendance_id);
  FETCH csr_abs_attendance_info INTO l_abs_row;
  CLOSE csr_abs_attendance_info;

  l_person_id := l_abs_row.person_id;
  l_object_version_number := l_abs_row.object_version_number;
  l_date_notification := l_effective_date;

  l_called_from := xxas_com_sec_pkg.get_function_parameter
                     (p_function_name  => xxas_com_sec_pkg.get_function_name(l_function_id)
                     ,p_parameter_name => 'pCalledFrom');

  l_ame_item_type    := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pItemType');
  l_ame_app_id       := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pAMEAppId');
  l_ame_trans_type   := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pAMETranType');
  l_ame_process_name := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => l_called_from
                          ,p_parameter_name => 'pProcessName');

  OPEN  csr_person(l_person_id);
  FETCH csr_person INTO l_per_row;
  CLOSE csr_person;

  OPEN  csr_asg(l_person_id);
  FETCH csr_asg INTO l_asg_row;
  CLOSE csr_asg;

  l_assignment_id := l_asg_row.assignment_id;

  OPEN  csr_get_emp_full_name(l_asg_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_job_name(l_asg_row.job_id);
  FETCH csr_get_job_name INTO l_per_job_name;
  CLOSE csr_get_job_name;

  OPEN  csr_get_pos_name(l_asg_row.position_id);
  FETCH csr_get_pos_name INTO l_per_position_name;
  CLOSE csr_get_pos_name;

  OPEN  csr_get_sec_grp_leg_code(l_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_per_sec_group_id
                                     ,l_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  OPEN  csr_get_org_info(l_person_id);
  FETCH csr_get_org_info INTO l_emp_generation
                             ,l_per_bg_cur_code;
  CLOSE csr_get_org_info;

  IF l_transaction_id is not null AND 
     l_transaction_step_id is not null THEN

    OPEN  csr_workflow_info(l_transaction_id);
    FETCH csr_workflow_info INTO l_workflow_info;
    CLOSE csr_workflow_info;

    l_item_type             := l_workflow_info.item_type;
    l_item_key              := l_workflow_info.item_key;
    l_transaction_status    := l_workflow_info.status;

  ELSE

    --
    -- Generate the new sequences
    --
    SELECT  hr_api_transactions_s.nextval
    INTO    l_transaction_id
    FROM   dual;

    l_transaction_status := 'W';

  END IF;

  l_login_person_id := xxas_util_pkg.get_per_from_user(fnd_global.user_id);

  OPEN  csr_login_person(l_login_person_id);
  FETCH csr_login_person INTO l_login_per_row;
  CLOSE csr_login_person;

  OPEN  csr_get_emp_full_name(l_login_per_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_login_per_manager_name;
  CLOSE csr_get_emp_full_name;
  
  OPEN  csr_get_sec_grp_leg_code(l_login_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_login_per_sec_group_id
                                     ,l_login_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  IF l_transaction_id is not null AND 
     l_transaction_step_id is not null THEN

    OPEN  csr_rc_trans_doc(l_person_id
                          ,l_per_row
                          ,l_asg_row
                          ,l_login_per_row
                          ,l_per_manager_name
                          ,l_login_per_manager_name
                          ,l_transaction_id
                          ,l_per_sec_group_id
                          ,l_per_leg_code
                          ,l_login_per_leg_code
                          ,l_per_job_name
                          ,l_per_position_name
                          ,l_emp_generation
                          ,l_per_bg_cur_code
                          ,l_abs_row
                          ,l_called_from
                          ,l_assignment_id
                          ,l_effective_date
                          ,l_date_notification
                          ,l_object_version_number
                          ,p_comments_to_approver
                          ,l_ame_item_type
                          ,l_ame_app_id
                          ,l_ame_trans_type
                          ,l_transaction_status);
    FETCH csr_rc_trans_doc INTO l_rc_doc;
    CLOSE csr_rc_trans_doc;

  ELSE

    OPEN  csr_trans_doc(l_person_id
                       ,l_per_row
                       ,l_asg_row
                       ,l_login_per_row
                       ,l_per_manager_name
                       ,l_login_per_manager_name
                       ,l_transaction_id
                       ,l_per_sec_group_id
                       ,l_per_leg_code
                       ,l_login_per_leg_code
                       ,l_per_job_name
                       ,l_per_position_name
                       ,l_emp_generation
                       ,l_per_bg_cur_code
                       ,l_abs_row
                       ,l_called_from
                       ,l_assignment_id
                       ,l_effective_date
                       ,l_date_notification
                       ,l_object_version_number
                       ,p_comments_to_approver
                       ,l_ame_item_type
                       ,l_ame_app_id
                       ,l_ame_trans_type);
    FETCH csr_trans_doc INTO l_doc;
    CLOSE csr_trans_doc;

  END IF;

  OPEN  csr_get_abs_att_type_meaning(l_abs_row.absence_attendance_type_id);
  FETCH csr_get_abs_att_type_meaning INTO l_abs_att_category;
  CLOSE csr_get_abs_att_type_meaning;

  IF l_transaction_id is not null AND 
     l_transaction_step_id is not null THEN

    update_hrapi_transaction(p_transaction_id        => l_transaction_id
                            ,p_transaction_step_id   => l_transaction_step_id
                            ,p_absence_attendance_id => p_absence_attendance_id
                            ,p_assignment_id         => l_assignment_id
                            ,p_person_id             => l_person_id
                            ,p_item_type             => l_ame_item_type
                            ,p_item_key              => null--l_item_key
                            ,p_absence_trans_doc     => l_rc_doc
                            ,p_information1          => to_char(l_abs_row.date_start, g_HRAPI_DATE_FMT)
                            ,p_information2          => to_char(l_abs_row.date_end, g_HRAPI_DATE_FMT)
                            ,p_information3          => to_char(l_abs_row.date_projected_start, g_HRAPI_DATE_FMT)
                            ,p_information4          => to_char(l_abs_row.date_projected_end, g_HRAPI_DATE_FMT)
                            ,p_information5          => l_abs_row.absence_attendance_type_id
                            ,p_information6          => l_abs_att_category
                            ,p_information7          => l_abs_row.absence_hours
                            ,p_information8          => l_abs_row.absence_days
                            ,p_information9          => null -- how to pass the leave status
                            ,p_object_state          => '2'
                            ,p_called_from           => l_called_from
                            ,p_ame_process_name      => l_ame_process_name
                            ,p_transaction_status    => l_transaction_status);

  ELSE

    l_transaction_step_id := create_hrapi_transaction(p_transaction_id        => l_transaction_id
                                                     ,p_absence_attendance_id => p_absence_attendance_id
                                                     ,p_assignment_id         => l_assignment_id
                                                     ,p_person_id             => l_person_id
                                                     ,p_item_type             => l_ame_item_type
                                                     ,p_item_key              => l_item_key
                                                     ,p_absence_trans_doc     => l_doc
                                                     ,p_information1          => to_char(l_abs_row.date_start, g_HRAPI_DATE_FMT)
                                                     ,p_information2          => to_char(l_abs_row.date_end, g_HRAPI_DATE_FMT)
                                                     ,p_information3          => to_char(l_abs_row.date_projected_start, g_HRAPI_DATE_FMT)
                                                     ,p_information4          => to_char(l_abs_row.date_projected_end, g_HRAPI_DATE_FMT)
                                                     ,p_information5          => l_abs_row.absence_attendance_type_id
                                                     ,p_information6          => l_abs_att_category
                                                     ,p_information7          => l_abs_row.absence_hours
                                                     ,p_information8          => l_abs_row.absence_days
                                                     ,p_information9          => null -- how to pass the leave status
                                                     ,p_object_state          => '2'
                                                     ,p_called_from           => l_called_from
                                                     ,p_ame_process_name      => l_ame_process_name
                                                     ,p_transaction_status    => l_transaction_status);

  END IF;

  IF xxas_com_sec_pkg.test_instance_rtn_boolean('XXAS_ABS_VAC_RULE_SETUP',l_person_id) THEN
    
    -- set transaction value
    hr_transaction_api.set_varchar2_value(p_transaction_step_id => l_transaction_step_id
                                         ,p_person_id           => l_person_id
                                         ,p_name                => 'P_SETUP_VAC_RULE'
                                         ,p_value               => 'D');
  END IF;
  
  --
  -- Do create API's validation
  --
  --
  -- Issue a savepoint
  --
  savepoint before_api_validation;

  hr_person_absence_swi.delete_person_absence
    (p_validate                     => hr_api.g_true_num
    ,p_absence_attendance_id        => p_absence_attendance_id
    ,p_object_version_number        => l_abs_row.object_version_number
    ,p_return_status                => l_return_status);

  IF l_return_status = 'E' THEN
    IF (fnd_msg_pub.count_msg > 0) THEN
      FOR i IN 1..fnd_msg_pub.count_msg
      LOOP

        fnd_msg_pub.get
         (p_msg_index     => i,
          p_encoded       => 'F',
          p_data          => l_msg_data,
          p_msg_index_out => l_msg_count
         );

        log_statement(c_method,'l_msg_data: ' || l_msg_data);

        --
        -- Skip the following warning messages
        --
        IF upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_EMP_ABS_SHORT_DURATION')) OR 
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_ABS_HOUR_LESS_DURATION')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_EMP_NOT_ENTITLED')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_RUNNING_ZERO')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_ABSENCE_OVERLAP')) OR  
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_ABS_DAY_AFTER')) THEN

          null;

        ELSE

          register_user_error
            (p_package_name => gc_log.package_name
            ,p_proc_name    => c_method
            ,p_message      => l_msg_data);

          l_api_errors := true;

        END IF;

        l_msg_data   := null;
        l_msg_count  := 0;
      END LOOP;
    END IF;
  END IF;

  IF l_api_errors THEN
    xxas_com_api_result_pkg.raise_registered_user_errors;
  END IF;

  --
  -- Issue a rollback
  --
  rollback to before_api_validation;

  IF l_item_key is null THEN

    launch_workflow
      (p_person_id            => l_person_id
      ,p_transaction_id       => l_transaction_id
      ,p_comments_to_approver => p_comments_to_approver
      ,p_called_from          => l_called_from
      ,p_ame_item_type        => l_ame_item_type
      ,p_ame_app_id           => l_ame_app_id
      ,p_ame_trans_type       => l_ame_trans_type
      ,p_ame_process_name     => l_ame_process_name
      ,p_item_key             => l_item_key
      ,p_activity_id          => l_activity_id);

  END IF;

  IF l_transaction_id is not null THEN

    OPEN  csr_transaction_status(cp_transaction_id => l_transaction_id);
    FETCH csr_transaction_status INTO l_transaction_status;
    CLOSE csr_transaction_status;

  END IF;

  IF l_transaction_status is null OR 
     l_transaction_status = 'W' THEN

    hr_trn_upd.upd(p_transaction_id => l_transaction_id
                  ,p_validate       => false
                  ,p_status         => 'W'
                  ,p_item_key       => l_item_key);

  ELSE

    hr_trn_upd.upd(p_transaction_id => l_transaction_id
                  ,p_validate       => false
                  ,p_status         => l_transaction_status
                  ,p_item_key       => l_item_key);

  END IF;

  xxas_selfservice_workflow_pkg.set_item_attr_text 
    (p_item_type => l_ame_item_type
    ,p_item_key  => l_item_key
    ,p_name      => 'HR_RUNTIME_APPROVAL_REQ_FLAG'
    ,p_value     => 'YES_DYNAMIC');

  workflow_review_page
    (p_item_type   => l_ame_item_type
    ,p_item_key    => l_item_key
    ,p_next_result => 'NEXT');

  -- Added for GIT#2626
  l_show_review_page := nvl(xxas_com_sec_pkg.get_function_parameter
                                   (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                                   ,p_parameter_name => 'pShowApprovalPageFlag')
                           ,'N');

  IF l_show_review_page = 'N' THEN
    xxas_emp_view_ui_pkg.register_result_details
    (p_transaction_id => l_transaction_id);
  END IF;

  l_workflow_info2 := xxas_cal_util_pkg.get_workflow_info
                       (p_transaction_id => l_transaction_id);

  IF l_workflow_info2.item_type is not null AND
     l_workflow_info2.item_key  is not null THEN

    l_current_tran_submit := xxas_selfservice_workflow_pkg.get_wf_attribute_text
                               (p_item_type      => l_workflow_info2.item_type
                               ,p_item_key       => l_workflow_info2.item_key
                               ,p_attribute_name => 'TRAN_SUBMIT');

    IF l_workflow_info2.trans_status = 'W' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info2.item_type
        ,p_item_key     => l_workflow_info2.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'W');

    ELSIF l_workflow_info2.trans_status = 'RI' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info2.item_type
        ,p_item_key     => l_workflow_info2.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'RI');

    ELSIF l_workflow_info2.trans_status = 'Y' AND 
          l_current_tran_submit         = 'RI' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info2.item_type
        ,p_item_key     => l_workflow_info2.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'RI');

    ELSIF l_workflow_info2.trans_status = 'Y' AND 
          l_current_tran_submit         = 'Y' THEN

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info2.item_type
        ,p_item_key     => l_workflow_info2.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'Y');

    END IF;

  END IF;
    
  IF NOT (l_show_review_page = 'Y' OR approval_required) THEN

    log_statement(c_method,'approval_required is false');

    l_status := xxas_xml_node_util_pkg.get_node_value(l_transaction_id, 'TxnStatus', 'Transaction/TransCtx');
    l_action := xxas_xml_node_util_pkg.get_node_value(l_transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode');

    l_save_attachments := xxas_com_sec_pkg.get_function_parameter
                            (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                            ,p_parameter_name => 'pSaveAttachments');

    l_save_attach_as_sup_docs := xxas_com_sec_pkg.get_function_parameter
                                   (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                                   ,p_parameter_name => 'pSaveAttachmentsAsSupportingDocs');

    l_response_xml := submit_emp_view_row
                        (p_effective_date          => trunc(sysdate)
                        ,p_person_id               => l_person_id
                        ,p_absence_attendance_id   => p_absence_attendance_id
                        ,p_transaction_id          => l_transaction_id
                        ,p_transaction_step_id     => l_transaction_step_id
                        ,p_status                  => l_status
                        ,p_action                  => l_action
                        ,p_approvers               => 'N'
                        ,p_source_block            => p_source_block
                        ,p_saved_search_id         => p_saved_search_id
                        ,p_save_attachments        => l_save_attachments
                        ,p_save_attach_as_sup_docs => l_save_attach_as_sup_docs
                        ,p_manager_id              => p_manager_id);

  ELSE

    log_statement(c_method,'approval_required is true');

    l_response_xml := go_to_review_page_xml
                        (p_person_id             => l_person_id
                        ,p_absence_attendance_id => p_absence_attendance_id
                        ,p_transaction_id        => l_transaction_id
                        ,p_transaction_step_id   => l_transaction_step_id
                        ,p_source_block          => p_source_block
                        ,p_saved_search_id       => p_saved_search_id
                        ,p_manager_id            => p_manager_id);

  END IF;

  --Check and raise any workflow errors
  raise_workflow_error(l_transaction_id);
  
  log_end(c_method);
	
  RETURN l_response_xml;

EXCEPTION  
  
  WHEN xxas_com_api_result_pkg.xxas_com_api_exception
  THEN
    rollback to before_api_validation;
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN hr_utility.hr_error
  THEN
    rollback to before_api_validation;
    register_user_error
      (p_package_name => gc_log.package_name
      ,p_proc_name    => c_method
      ,p_message      => hr_utility.get_message);
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN others
  THEN
    err_code := SQLCODE;
    err_msg  := SUBSTR(SQLERRM, 1, 200);

    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT'
                                 ,'Error Code: ' || err_code || ' and Error Message: ' || err_msg))));

END delete_absence_event;

--
-- ------------------------------------------------------------------------ *
-- Name    : submit_absence_event
-- ------------------------------------------------------------------------ *
PROCEDURE submit_absence_event 
  (p_function_id    IN  number
  ,p_form_data_xml  IN  XMLType
  ,p_user_submitted IN  varchar2 DEFAULT g_NO
  ,p_response_xml   OUT XMLType)
  
IS

  c_method    CONSTANT      varchar2(30) := 'submit_absence_event';
  l_person_id               number;
  l_absence_attendance_id   number;
  l_transaction_id          number;
  l_transaction_step_id     number;
  l_effective_date          date;
  err_code                  varchar2(200);
  err_msg                   varchar2(200);
  l_data                    xxas_emp_view_ui_pkg.xxas_data_table;
  l_status                  varchar2(10);
  l_action                  varchar2(30);
  l_source_block            varchar2(2000);
  l_saved_search_id         number;
  l_save_attachments        varchar2(10);
  l_save_attach_as_sup_docs varchar2(10);
  l_manager_id              number;
  l_start_date              date;

BEGIN

  log_start(c_method, p_function_id, p_user_submitted);

  xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

  l_data := xxas_emp_view_ui_pkg.transform_jsonxml_form_data
              (p_form_data_xml);

  -- Retrieve the form values
  IF l_data.exists('PERSON_ID') THEN
    l_person_id := l_data('PERSON_ID');
  END IF;

  IF l_data.exists('TRANSACTION_ID') THEN
    l_transaction_id := l_data('TRANSACTION_ID');
  END IF;

  IF l_data.exists('TRANSACTION_STEP_ID') THEN
    l_transaction_step_id := l_data('TRANSACTION_STEP_ID');
  END IF;

  IF l_data.exists('ABSENCE_ATTENDANCE_ID') THEN
    l_absence_attendance_id := l_data('ABSENCE_ATTENDANCE_ID');
  END IF;

  IF l_data.exists('SOURCE_BLOCK') THEN
    l_source_block := l_data('SOURCE_BLOCK');
  END IF;

  IF l_data.exists('SAVED_SEARCH_ID') THEN
    l_saved_search_id := l_data('SAVED_SEARCH_ID');
  END IF;

  IF l_data.exists('EFFECTIVE_DATE') THEN
    l_effective_date := fnd_date.canonical_to_date(l_data('EFFECTIVE_DATE'));
  END IF;

  IF l_data.exists('MANAGER_ID') THEN
    l_manager_id := l_data('MANAGER_ID');
  END IF;

  l_status := xxas_xml_node_util_pkg.get_node_value(l_transaction_id, 'TxnStatus', 'Transaction/TransCtx');
  l_action := xxas_xml_node_util_pkg.get_node_value(l_transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode');
  l_start_date := xxas_util_pkg.get_step_val_dt(l_transaction_step_id, 'INFORMATION1');
  l_save_attachments := xxas_com_sec_pkg.get_function_parameter
                          (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                          ,p_parameter_name => 'pSaveAttachments');

  l_save_attach_as_sup_docs := xxas_com_sec_pkg.get_function_parameter
                                 (p_function_name  => xxas_com_sec_pkg.get_function_name(p_function_id)
                                 ,p_parameter_name => 'pSaveAttachmentsAsSupportingDocs');

  p_response_xml := submit_emp_view_row 
                      (p_effective_date          => nvl(l_start_date,l_effective_date)
                      ,p_person_id               => l_person_id
                      ,p_absence_attendance_id   => l_absence_attendance_id
                      ,p_transaction_id          => l_transaction_id
                      ,p_transaction_step_id     => l_transaction_step_id
                      ,p_status                  => l_status
                      ,p_action                  => l_action
                      ,p_approvers               => 'Y'
                      ,p_source_block            => l_source_block
                      ,p_saved_search_id         => l_saved_search_id
                      ,p_save_attachments        => l_save_attachments
                      ,p_save_attach_as_sup_docs => l_save_attach_as_sup_docs
                      ,p_manager_id              => l_manager_id);

  log_end(c_method);

EXCEPTION
  
  WHEN xxas_com_api_result_pkg.xxas_com_api_exception
  THEN
    rollback to before_api_validation;
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN hr_utility.hr_error
  THEN
    rollback to before_api_validation;
    register_user_error
      (p_package_name => gc_log.package_name
      ,p_proc_name    => c_method
      ,p_message      => hr_utility.get_message);
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN others
  THEN
    err_code := SQLCODE;
    err_msg  := SUBSTR(SQLERRM, 1, 200);

    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT'
                                 ,'Error Code: ' || err_code || ' and Error Message: ' || err_msg))));

END submit_absence_event;

--
-- ------------------------------------------------------------------------ *
-- Name    : cancel_wf_emp_view_row
-- ------------------------------------------------------------------------ *
PROCEDURE cancel_wf_emp_view_row
  (p_effective_date        IN date
  ,p_person_id             IN number
  ,p_transaction_id        IN number   DEFAULT null 
  ,p_transaction_step_id   IN number   DEFAULT null)
  
IS
  c_method CONSTANT varchar2(30) := 'cancel_wf_emp_view_row';

BEGIN
  log_start(c_method,p_effective_date,p_person_id,p_transaction_id,p_transaction_step_id); 

  IF p_transaction_id is not null AND 
     p_transaction_step_id is not null THEN

    hr_trn_upd.upd(p_transaction_id => p_transaction_id
                  ,p_validate       => false
                  ,p_status         => 'D');

  END IF;

  log_end(c_method);

END cancel_wf_emp_view_row;

--
-- ------------------------------------------------------------------------ *
-- Name    : cancel_submit_emp_view_row
-- ------------------------------------------------------------------------ *
PROCEDURE cancel_submit_emp_view_row
  (p_effective_date        IN date
  ,p_person_id             IN number
  ,p_transaction_id        IN number   DEFAULT null 
  ,p_transaction_step_id   IN number   DEFAULT null)
  
IS
  c_method CONSTANT varchar2(30) := 'cancel_submit_emp_view_row';
  l_item_type       hr_api_transactions.item_type%TYPE;
  l_item_key        hr_api_transactions.item_key%TYPE;
  l_process_name    hr_api_transactions.process_name%TYPE;

  CURSOR csr_transaction_info
  IS
  SELECT hat.item_type
        ,hat.item_key
        ,hat.process_name
  FROM   hr_api_transactions hat
        ,hr_api_transaction_steps hats
  WHERE  hat.transaction_id       = p_transaction_id
  AND    hat.transaction_id       = hats.transaction_id
  AND    hats.transaction_step_id = p_transaction_step_id;

BEGIN
  log_start(c_method,p_effective_date,p_person_id,p_transaction_id,p_transaction_step_id); 

  IF p_transaction_id is not null AND 
     p_transaction_step_id is not null THEN

    hr_trn_upd.upd(p_transaction_id => p_transaction_id
                  ,p_validate       => false
                  ,p_status         => 'D');

    OPEN csr_transaction_info;
    FETCH csr_transaction_info INTO l_item_type
                                   ,l_item_key
                                   ,l_process_name;
    CLOSE csr_transaction_info;

    --
    -- Aborting the workflow process
    --
    wf_engine.AbortProcess(itemtype => l_item_type
                          ,itemkey  => l_item_key
                          ,process  => l_process_name);

  END IF;

  log_end(c_method);

END cancel_submit_emp_view_row;

--
-- ------------------------------------------------------------------------ *
-- Name    : submit_emp_view_row
-- ------------------------------------------------------------------------ *
FUNCTION submit_emp_view_row 
  (p_effective_date          IN date
  ,p_person_id               IN number
  ,p_absence_attendance_id   IN number
  ,p_transaction_id          IN number
  ,p_transaction_step_id     IN number
  ,p_status                  IN varchar2
  ,p_action                  IN varchar2
  ,p_approvers               IN varchar2
  ,p_source_block            IN varchar2
  ,p_saved_search_id         IN number
  ,p_save_attachments        IN varchar2
  ,p_save_attach_as_sup_docs IN varchar2
  ,p_manager_id              IN number  DEFAULT NULL) RETURN XMLType
IS

  --
  -- Gets attachments for given entity row
  --
  CURSOR csr_attachments IS
  SELECT a.rowid
        ,a.attached_document_id
        ,a.document_id
        ,a.seq_num
        ,a.entity_name
        ,a.column1
        ,a.pk1_value
        ,a.pk2_value
        ,a.pk3_value
        ,a.pk4_value
        ,a.pk5_value
        ,a.automatically_added_flag
        ,d.datatype_id
        ,a.category_id
        ,d.security_type
        ,d.publish_flag
        ,d.usage_type
        ,d.start_date_active
        ,d.end_date_active
        ,d.title
        ,d.description
        ,d.file_name
        ,d.media_id
  FROM   fnd_attached_documents a
        ,fnd_documents_vl d
  WHERE  a.document_id = d.document_id
  AND    a.pk1_value = p_transaction_id
  AND    a.entity_name = 'PQH_SS_ATTACHMENT';

  CURSOR get_abs_attendance_id IS
  SELECT transaction_ref_id
  FROM   hr_api_transactions
  WHERE  transaction_ref_table = 'PER_ABSENCE_ATTENDANCES'
  AND    transaction_id        = p_transaction_id;

  CURSOR c_action1(cp_item_type varchar2
                  ,cp_item_key  varchar2)
  IS
  SELECT xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode') action
  FROM   hr_api_transactions hat
  ,      hr_api_transaction_steps hats
  WHERE  hat.item_type            = cp_item_type
  AND    hat.item_key             = cp_item_key
  AND    hat.transaction_id       = hats.transaction_id
  AND    hats.api_name            = 'HR_PERSON_ABSENCE_SWI.PROCESS_API'; 
  
  CURSOR csr_trans_exists is
  SELECT 'Y'
  FROM  hr_api_transactions
  WHERE transaction_id = p_transaction_id
  AND   status <> 'AC';

  c_method CONSTANT         varchar2(30) := 'submit_emp_view_row';
  l_workflow_info           xxas_cal_util_pkg.workflow_info;
  l_attachments             csr_attachments%ROWTYPE;
  l_current_activity        wf_process_activities.activity_name%TYPE;
  l_action1                 varchar2(30);
  l_abs_attendance_id       number;
  l_save_attachments        varchar2(10);
  l_save_attach_as_sup_docs varchar2(10);
  l_response_xml            XMLType;
  l_changed_items_xml       XMLType;
  l_deleted_items_xml       XMLType;
  err_code                  varchar2(200);
  err_msg                   varchar2(200);
  l_source_block_id         number;
  l_source_block            varchar2(2000);
  l_update_action           BOOLEAN;
  l_refresh_page            BOOLEAN;
  l_page_refresh            varchar2(3);
  l_trans_exists            varchar2(2) := 'N';
  l_rest_provider           varchar2(4000);
  l_invalidate_string       varchar2(4000);
  l_invalidate_tbl          xxas_com_varchar2_table;
  l_current_tran_submit     hr_api_transactions.status%TYPE;

BEGIN

  log_start(c_method,p_effective_date,p_person_id,p_transaction_id,p_transaction_step_id,p_status,p_action,p_approvers,p_source_block,p_saved_search_id,p_manager_id); 

  xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

  --
  -- Delete the contents of the cached tables.
  --
  g_event_tbl.delete;
  g_del_items_tbl.delete;

  l_workflow_info := xxas_cal_util_pkg.get_workflow_info
                       (p_transaction_id => p_transaction_id);
  --
  -- Only update the workflow if one exists - this may happen when cancelling
  --

  IF l_workflow_info.item_type is not null AND
     l_workflow_info.item_key  is not null THEN

    l_current_tran_submit := xxas_selfservice_workflow_pkg.get_wf_attribute_text
                               (p_item_type      => l_workflow_info.item_type
                               ,p_item_key       => l_workflow_info.item_key
                               ,p_attribute_name => 'TRAN_SUBMIT');

    IF l_workflow_info.trans_status = 'RI' AND 
       l_current_tran_submit        = 'RI' THEN 

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'RI');

    ELSIF (l_workflow_info.trans_status = 'Y' AND 
           l_current_tran_submit        = 'Y') OR 
          (l_workflow_info.trans_status = 'Y' AND 
           l_current_tran_submit        = 'RI') THEN 

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'Y');

    ELSE

      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type    => l_workflow_info.item_type
        ,p_item_key     => l_workflow_info.item_key
        ,p_name         => 'TRAN_SUBMIT'
        ,p_value        => 'Y');

    END IF;
		
    xxas_selfservice_workflow_pkg.set_item_attr_text 
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_REVIEW_TEMPLATE_RN_ATTR'
      ,p_value     => 'HR_ABS_NTF_SS');
    
    /* Code changes GIT#2664
    
    xxas_selfservice_workflow_pkg.set_item_attr_text 
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_NTF_ATTACHMENTS_ATTR'
      ,p_value     => 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || p_transaction_id);
    */
    xxas_selfservice_workflow_pkg.set_item_attr_text 
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_PERZ_FUNCTION_NAME_ATTR'
      ,p_value     => xxas_xml_node_util_pkg.get_node_value(p_transaction_id, 'PerzFunctionName', 'Transaction/TransCtx'));

    xxas_selfservice_workflow_pkg.set_item_attr_text 
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_PERZ_LOCALIZATION_CODE_ATTR'
      ,p_value     => xxas_xml_node_util_pkg.get_node_value(p_transaction_id, 'PerzLocalizationCode', 'Transaction/TransCtx'));

    xxas_selfservice_workflow_pkg.set_item_attr_text 
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_PERZ_ORGANIZATION_ID_ATTR'
      ,p_value     => xxas_xml_node_util_pkg.get_node_value(p_transaction_id, 'PerzOrganizationId', 'Transaction/TransCtx'));

    l_save_attachments := NVL(NVL(p_save_attachments
                                 ,xxas_com_sec_pkg.get_function_parameter
                                    (p_function_name  => xxas_xml_node_util_pkg.get_node_value(p_transaction_id, 'PerzFunctionName', 'Transaction/TransCtx')
                                    ,p_parameter_name => 'pSaveAttachments'))
                             ,'Y');

    xxas_selfservice_workflow_pkg.set_item_attr_text 
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'SAVE_ATTACHMENT'
      ,p_value     => l_save_attachments);

    --
    -- Code changes GIT#2664
    -- This will show the attachments in notification page Summary section.
    --
    IF l_save_attachments = 'Y' THEN
     
      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type => l_workflow_info.item_type
        ,p_item_key  => l_workflow_info.item_key
        ,p_name      => 'HR_NTF_ATTACHMENTS_ATTR'
        ,p_value     => 'FND:entity=PER_ABSENCE_ATTENDANCES' || '&' || 'pk1name=AbsenceAttendanceId' ||
        '&' || 'pk1value=' || p_absence_attendance_id || '_' || p_transaction_id);
  
    ELSE
      xxas_selfservice_workflow_pkg.set_item_attr_text 
        (p_item_type => l_workflow_info.item_type
        ,p_item_key  => l_workflow_info.item_key
        ,p_name      => 'HR_NTF_ATTACHMENTS_ATTR'
        ,p_value     => 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' ||
        'pk1value=' || p_transaction_id);
    END IF;
    
    l_current_activity := xxas_selfservice_workflow_pkg.get_current_activity_name
                            (p_item_type => l_workflow_info.item_type
                            ,p_item_key  => l_workflow_info.item_key);

    log_statement(c_method, 'l_current_activity = ' || l_current_activity);
    
    -- respond to notification if it is an update action activity.
    l_update_action := xxas_com_worklist_pkg.update_action(l_workflow_info.item_type,l_workflow_info.item_key);
    
    IF l_update_action OR xxas_com_worklist_pkg.is_rfc(p_transaction_step_id) = 'Y' THEN
      l_refresh_page := TRUE;
    END IF;
    
    OPEN  c_action1(l_workflow_info.item_type
                   ,l_workflow_info.item_key);
    FETCH c_action1 INTO l_action1;
    CLOSE c_action1;

    log_statement(c_method, 'l_action1 = ' || l_action1);

    IF l_current_activity IN ('HR_INITIATOR_RFC_NTF','HR_APPROVER_RFC_NTF') THEN

      wf_engine.CompleteActivityInternalName
        (itemtype => l_workflow_info.item_type
        ,itemkey  => l_workflow_info.item_key
        ,activity => l_current_activity
        ,result   => 'RESUBMIT');

    ELSIF l_current_activity <> 'HR_REVIEW_CHANGES_JSP_FCT' AND NOT l_update_action THEN

      xxas_selfservice_workflow_pkg.complete_current_activity 
        (p_item_type => l_workflow_info.item_type
        ,p_item_key  => l_workflow_info.item_key
        ,p_result    => CASE WHEN l_action1 = g_INSERT
                        THEN 'NEXT'
                        WHEN l_action1 = g_UPDATE
                        THEN 'UPDATE_ABSENCE'
                        ELSE null
                   END);

    END IF;

    OPEN  csr_trans_exists;
    FETCH csr_trans_exists into l_trans_exists;
    CLOSE csr_trans_exists;
    
    IF l_trans_exists = 'Y' THEN

      IF l_update_action THEN
        -- update transaction table
        update hr_api_transactions set status = 'Y' WHERE transaction_id = p_transaction_id;
      ELSE
        hr_trn_upd.upd(p_transaction_id => p_transaction_id
                      ,p_validate       => false
                      ,p_status         => 'Y');
      END IF;
      
      l_save_attach_as_sup_docs := NVL(NVL(p_save_attach_as_sup_docs
                                          ,xxas_com_sec_pkg.get_function_parameter
                                             (p_function_name  => xxas_xml_node_util_pkg.get_node_value(p_transaction_id, 'PerzFunctionName', 'Transaction/TransCtx')
                                             ,p_parameter_name => 'pSaveAttachmentsAsSupportingDocs'))
                                      ,'N');

    END IF;

    -- Code changes GIT#2664
    -- Ignore flag l_save_attach_as_sup_docs
    IF l_save_attachments = 'Y' /*AND l_save_attach_as_sup_docs = 'Y'*/ THEN

      OPEN  get_abs_attendance_id;
      FETCH get_abs_attendance_id INTO l_abs_attendance_id;
      CLOSE get_abs_attendance_id;

      OPEN csr_attachments;
      LOOP
        FETCH csr_attachments INTO l_attachments;
        EXIT WHEN csr_attachments%NOTFOUND;

        fnd_attached_documents_pkg.update_row
          (x_rowid                        => l_attachments.rowid
          ,x_attached_document_id         => l_attachments.attached_document_id
          ,x_document_id                  => l_attachments.document_id
          ,x_last_update_date             => sysdate
          ,x_last_updated_by              => fnd_global.user_id
          ,x_last_update_login            => fnd_global.login_id
          ,x_seq_num                      => l_attachments.seq_num
          ,x_entity_name                  => 'PER_ABSENCE_ATTENDANCES'
          ,x_column1                      => l_attachments.column1
          ,x_pk1_value                    => CASE WHEN l_abs_attendance_id IS NOT NULL THEN l_abs_attendance_id || '_' || p_transaction_id
                                             ELSE to_char(p_absence_attendance_id) END
          ,x_pk2_value                    => l_attachments.pk2_value
          ,x_pk3_value                    => l_attachments.pk3_value
          ,x_pk4_value                    => l_attachments.pk4_value
          ,x_pk5_value                    => l_attachments.pk5_value
          ,x_automatically_added_flag     => l_attachments.automatically_added_flag
          ,x_datatype_id                  => l_attachments.datatype_id
          ,x_category_id                  => l_attachments.category_id
          ,x_security_type                => l_attachments.security_type
          ,x_publish_flag                 => l_attachments.publish_flag
          ,x_usage_type                   => l_attachments.usage_type
          ,x_start_date_active            => l_attachments.start_date_active
          ,x_end_date_active              => l_attachments.end_date_active
          ,x_language                     => userenv('LANG')
          ,x_title                        => l_attachments.title
          ,x_description                  => l_attachments.description
          ,x_file_name                    => l_attachments.file_name
          ,x_media_id                     => l_attachments.media_id);

        l_attachments := null;

      END LOOP;
      CLOSE csr_attachments;
    END IF;

  END IF;

  store_item_into_event_tbl(p_person_id             => p_person_id
                           ,p_absence_attendance_id => p_absence_attendance_id
                           ,p_transaction_id        => p_transaction_id
                           ,p_transaction_step_id   => p_transaction_step_id
                           ,p_effective_date        => p_effective_date
                           ,p_status                => p_status
                           ,p_action                => p_action
                           ,p_approvers             => p_approvers);
  
  IF l_refresh_page THEN
  
    l_response_xml := xxas_com_msg_pkg.get_wk_success_message_xml
                          (p_message     => xxas_com_msg_type(xxas_util_pkg.app, 'XXAS_COM_CHNG_SAVE', null)
                          ,p_redirect    => 'self'
                          ,p_close_modal => 'true');
                          
  ELSIF p_action = g_INSERT THEN

    --
    -- Replace $ with _ because getting source block with _ replaced with $
    --
    l_source_block      := replace(p_source_block,'$','_');
    l_source_block_id   := xxas_rslt_cache_pkg.get_function_id(l_source_block);
    l_changed_items_xml := changed_items_xml(p_source_block    => l_source_block
                                            ,p_saved_search_id => p_saved_search_id
                                            ,p_manager_id     =>  p_manager_id);
    l_deleted_items_xml := deleted_items_xml(p_source_block    => p_source_block
                                            ,p_saved_search_id => p_saved_search_id
                                            ,p_manager_id     =>  p_manager_id);
    l_rest_provider     := xxas_com_sec_pkg.get_function_parameter
                                                (p_function_name  => 'l_source_block'
                                                ,p_parameter_name => 'pRestProvider');
    l_invalidate_string := regexp_replace(l_rest_provider,'([^/]+$)');                         
    l_invalidate_tbl    := xxas_util_pkg.string_to_varchar2tbl(l_invalidate_string || fnd_global.employee_id,'*');

    -- Need to raise a info messages when vacation rules is setup and 
    -- it is retro absences.	
    IF hr_transaction_api.get_varchar2_value(p_transaction_step_id,'P_SETUP_VAC_RULE') = 'E' THEN
      
      l_response_xml := xxas_com_msg_pkg.get_info_message_xml
                          (p_message       => xxas_com_msg_type
                                                (xxas_util_pkg.app
                                                ,'XXAS_EMP_ABS_VAC_IN_PAST'
                                                ,null)
                          ,p_redirect      => null
                          ,p_target        => null
                          ,p_persist_cache => 'true'
                          ,p_block_id      => l_source_block_id
                          ,p_cre_upd_items => l_changed_items_xml
                          ,p_del_items     => l_deleted_items_xml
                          ,p_invalid_urls  => l_invalidate_tbl
                          );
    ELSE
    
      l_response_xml := xxas_com_msg_pkg.get_success_message_xml
                          (p_message       => xxas_com_msg_type
                                                (xxas_util_pkg.app
                                                ,'XXAS_ABS_CREATE_ENTRY_SUC_MSG'
                                                ,null)
                          ,p_redirect      => null
                          ,p_target        => null
                          ,p_persist_cache => 'true'
                          ,p_block_id      => l_source_block_id
                          ,p_cre_upd_items => l_changed_items_xml
                          ,p_del_items     => l_deleted_items_xml
                          ,p_invalid_urls  => l_invalidate_tbl
                          );
    END IF;
    
  ELSIF p_action = g_UPDATE THEN

    --
    -- Replace $ with _ because getting source block with _ replaced with $
    --
    l_source_block      := replace(p_source_block,'$','_');
    l_source_block_id   := xxas_rslt_cache_pkg.get_function_id(l_source_block);
    l_changed_items_xml := changed_items_xml(p_source_block    => l_source_block
                                            ,p_saved_search_id => p_saved_search_id
                                            ,p_manager_id     =>  p_manager_id);
    l_deleted_items_xml := deleted_items_xml(p_source_block    => p_source_block
                                            ,p_saved_search_id => p_saved_search_id
                                            ,p_manager_id     =>  p_manager_id);
    l_rest_provider     := xxas_com_sec_pkg.get_function_parameter
                                                (p_function_name  => 'l_source_block'
                                                ,p_parameter_name => 'pRestProvider');
    l_invalidate_string := regexp_replace(l_rest_provider,'([^/]+$)');                     
    l_invalidate_tbl    := xxas_util_pkg.string_to_varchar2tbl(l_invalidate_string || fnd_global.employee_id,'*');

    -- Need to raise a info messages when vacation rules is setup and 
    -- it is retro absences.	
    IF hr_transaction_api.get_varchar2_value(p_transaction_step_id,'P_SETUP_VAC_RULE') = 'E'
    THEN
      
      l_response_xml := xxas_com_msg_pkg.get_info_message_xml
                          (p_message       => xxas_com_msg_type
                                                (xxas_util_pkg.app
                                                ,'XXAS_EMP_ABS_VAC_IN_PAST'
                                                ,null)
                          ,p_redirect      => null
                          ,p_target        => null
                          ,p_persist_cache => 'true'
                          ,p_block_id      => l_source_block_id
                          ,p_cre_upd_items => l_changed_items_xml
                          ,p_del_items     => l_deleted_items_xml
                          ,p_invalid_urls  => l_invalidate_tbl
                          );
    ELSE
    
      l_response_xml := xxas_com_msg_pkg.get_success_message_xml
                          (p_message       => xxas_com_msg_type
                                                (xxas_util_pkg.app
                                                ,'XXAS_ABS_UPDATE_ENTRY_SUC_MSG'
                                                ,null)
                          ,p_redirect      => null
                          ,p_target        => null
                          ,p_persist_cache => 'true'
                          ,p_block_id      => l_source_block_id
                          ,p_cre_upd_items => l_changed_items_xml
                          ,p_del_items     => l_deleted_items_xml
                          ,p_invalid_urls  => l_invalidate_tbl
                          );
    END IF; 
  ELSIF p_action = g_DELETE THEN

    --
    -- Replace $ with _ because getting source block with _ replaced with $
    --
    l_source_block      := replace(p_source_block,'$','_');
    l_source_block_id   := xxas_rslt_cache_pkg.get_function_id(l_source_block);
    l_changed_items_xml := changed_items_xml(p_source_block    => l_source_block
                                            ,p_saved_search_id => p_saved_search_id
                                            ,p_manager_id     =>  p_manager_id);
    l_deleted_items_xml := deleted_items_xml(p_source_block    => p_source_block
                                            ,p_saved_search_id => p_saved_search_id
                                            ,p_manager_id     =>  p_manager_id);
    l_rest_provider     := xxas_com_sec_pkg.get_function_parameter
                                                (p_function_name  => l_source_block
                                                ,p_parameter_name => 'pRestProvider');
    l_invalidate_string := regexp_replace(l_rest_provider,'([^/]+$)');                 
    l_invalidate_tbl    := xxas_util_pkg.string_to_varchar2tbl(l_invalidate_string || fnd_global.employee_id ,'*');

   l_page_refresh      := xxas_com_sec_pkg.get_function_parameter
                                                (p_function_name  => l_source_block
                                                ,p_parameter_name => 'pPageRefresh');
	
	IF l_page_refresh = 'Y' THEN
	
        l_response_xml := xxas_com_msg_pkg.get_success_message_xml
                        (p_message       => xxas_com_msg_type
                                              (xxas_util_pkg.app
                                              ,'XXAS_ABS_DELETE_ENTRY_SUC_MSG'
                                              ,null)
                        ,p_redirect      => 'self-full'
                        ,p_target        => null
                        ,p_persist_cache => 'true'
                        ,p_block_id      => l_source_block_id
                        ,p_cre_upd_items => l_changed_items_xml
                        ,p_del_items     => l_deleted_items_xml
                        ,p_invalid_urls  => l_invalidate_tbl
                        );
	 ELSE	
                l_response_xml := xxas_com_msg_pkg.get_success_message_xml
                        (p_message       => xxas_com_msg_type
                                              (xxas_util_pkg.app
                                              ,'XXAS_ABS_DELETE_ENTRY_SUC_MSG'
                                              ,null)
                        ,p_redirect      => null
                        ,p_target        => null
                        ,p_persist_cache => 'true'
                        ,p_block_id      => l_source_block_id
                        ,p_cre_upd_items => l_changed_items_xml
                        ,p_del_items     => l_deleted_items_xml
                        ,p_invalid_urls  => l_invalidate_tbl
                        );
	END IF ;					

  END IF;
	
	


  RETURN l_response_xml;

  --
  -- Delete the contents of the cached tables.
  --
  g_event_tbl.delete;
  g_del_items_tbl.delete;

  log_end(c_method);

EXCEPTION  

  WHEN others
  THEN

    --
    -- Delete the contents of the cached tables.
    --
    g_event_tbl.delete;
    g_del_items_tbl.delete;

    err_code := SQLCODE;
    err_msg  := SUBSTR(SQLERRM, 1, 200);

    xxas_com_api_result_pkg.raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT'
                                 ,'Error Code: ' || err_code || ' and Error Message: ' || err_msg))));

END submit_emp_view_row;

--
-- ------------------------------------------------------------------------ *
-- Name    : dur_cal_ol_acr_bal_abs_handler
-- ------------------------------------------------------------------------ *
FUNCTION dur_cal_ol_acr_bal_abs_handler
  (p_request_xml IN XMLType) RETURN XMLType
IS

  CURSOR csr_peer_asgs(cp_start_date IN date
                      ,cp_end_date   IN date) IS
  SELECT distinct column_value
  FROM   TABLE(CAST(xxas_cal_util_pkg.asgs_in_hier
                      (g_PEERS
                      ,cp_start_date
                      ,cp_end_date
                      ,trunc(sysdate)
                      ,fnd_global.employee_id) AS xxas_com_num_table))
  WHERE  rownum >= 0;

  CURSOR csr_peer_person_ids(cp_peer_asg_tbl IN xxas_com_num_table) IS
  SELECT distinct xxas_util_pkg.get_person_from_asg(column_value,trunc(sysdate))
  FROM   TABLE(cp_peer_asg_tbl);

  CURSOR csr_team_asgs(cp_start_date IN date
                      ,cp_end_date   IN date) IS
  SELECT distinct column_value
  FROM   TABLE(CAST(xxas_cal_util_pkg.asgs_in_hier
                      (g_DR_ONLY
                      ,cp_start_date
                      ,cp_start_date
                      ,trunc(sysdate)
                      ,fnd_global.employee_id) AS xxas_com_num_table))
  WHERE  rownum >= 0;

  CURSOR csr_team_person_ids(cp_team_asg_tbl IN xxas_com_num_table) IS
  SELECT distinct xxas_util_pkg.get_person_from_asg(column_value,trunc(sysdate))
  FROM   TABLE(cp_team_asg_tbl);

  CURSOR csr_team_saved_sch_asgs(cp_saved_search_id IN number) IS
  SELECT /*+ cardinality(rslts 10)*/
         distinct rslts.assignment_id
  FROM   TABLE(xxas_per_search_pkg.people_tbl
                 (null
                 ,(SELECT xxas_saved_search_pkg.xml_to_obj(ss.search_parameters)
                   FROM   xxas_saved_searches_v ss
                   WHERE  ss.saved_search_id = cp_saved_search_id
                   AND    xxas_com_sec_pkg.test_instance
                            ('XXAS_DATA_SAVED_SEARCH_VIEW',cp_saved_search_id) = 'Y')
                 )
              ) rslts
  WHERE  xxas_com_sec_pkg.test_instance
           ('XXAS_DT_SAVE_SRCH_TIME_ABS'
           ,rslts.assignment_id
           ,null
           ,null
           ,null
           ,null) = 'Y';

  CURSOR csr_team_saved_sch_person_ids(cp_team_asg_tbl IN xxas_com_num_table) IS
  SELECT distinct xxas_util_pkg.get_person_from_asg(column_value,trunc(sysdate))
  FROM   TABLE(cp_team_asg_tbl);

  CURSOR csr_absence_type_value(cp_abs_type_id IN number) IS
  SELECT paat.hours_or_days
  FROM   per_abs_attendance_types_v paat
  WHERE  paat.absence_attendance_type_id = cp_abs_type_id;

  CURSOR csr_or_asg_values_xml(cp_asg_id     IN number
                              ,cp_person_id  IN number
                              ,cp_date_start IN date) IS
  SELECT XMLElement
          ("values"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("value"
              ,XMLForest
                (rslt.assignment_id "id"
                ,rslt.short_name || ', ' || xxas_cal_util_pkg.job_name(rslt.job_id) "value"
                ,rslt.short_name || ', ' || xxas_cal_util_pkg.job_name(rslt.job_id) "label"
                )
              )
            )
          )
  FROM  (SELECT paaf.assignment_id
               ,xxas_util_pkg.get_formatted_name
                  (papf.full_name
                  ,papf.global_name
                  ,papf.local_name
                  ,papf.first_name
                  ,papf.last_name
                  ,papf.known_as) short_name
                  ,paaf.job_id job_id
         FROM   per_all_people_f papf
               ,per_all_assignments_f paaf
         WHERE  papf.person_id = paaf.person_id
         AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                               AND     papf.effective_end_date
         AND    paaf.person_id = DECODE(cp_person_id,null,fnd_global.employee_id,cp_person_id)
         AND    DECODE(cp_date_start,null,trunc(sysdate),cp_date_start) BETWEEN paaf.effective_start_date
                                                                        AND     paaf.effective_end_date
         AND    paaf.assignment_id IN (SELECT distinct column_value
                                       FROM   TABLE(CAST(xxas_cal_util_pkg.per_asgs_eff_as_on_date_tbl
                                                           (DECODE(cp_date_start,null,trunc(sysdate),cp_date_start)
                                                           ,DECODE(cp_person_id,null,fnd_global.employee_id,cp_person_id)) AS xxas_com_num_table)))
        ) rslt;

  CURSOR csr_ol_absences_data(cp_person_id         IN number
                             ,cp_start_date        IN date
                             ,cp_end_date          IN date
                             ,cp_emp_calendar_flag IN varchar2
                             ,cp_mgr_calendar_flag IN varchar2) IS
  SELECT XMLElement
          ("absences"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("item"
              ,XMLForest
                ('worklist' AS "type"
                ,xxas_util_pkg.get_formatted_name
                   (p_full_name   => rslt.full_name
                   ,p_global_name => rslt.global_name
                   ,p_local_name  => rslt.local_name
                   ,p_first_name  => rslt.first_name
                   ,p_last_name   => rslt.last_name
                   ,p_known_as    => rslt.known_as) AS "sender"
                ,CASE WHEN rslt.date_end = rslt.date_start 
                      THEN to_char(rslt.date_start,'Mon dd yyyy') 
                      ELSE to_char(rslt.date_start,'Mon dd') || ' - ' || nvl(to_char(rslt.date_end,'Mon dd yyyy'),xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_ONGOING'))
                 END
                 || decode((rslt.date_end -rslt.date_start +1)
                          ,null
                          ,''
                          ,1
                          ,', 1 '|| xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_CAL_EVENT_DUR_DAY')
                          , ', ' || (rslt.date_end -rslt.date_start+1) || ' ' || xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_CAL_EVENT_DUR_DAYS')
                          )
                 || ' - '
                 || CASE WHEN cp_emp_calendar_flag = 'Y'
                         THEN xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABSENT_TYPE_DEFAULT')
                         WHEN cp_mgr_calendar_flag = 'Y'
                         THEN rslt.absence_type
                         ELSE xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABSENT_TYPE_DEFAULT')
                    END  AS "subject"
                )
              ,CASE WHEN rslt.wf_status_code IS NOT NULL 
                    THEN XMLElement
                          ("labels"
                          ,XMLAttributes
                            ('true' as "json:force-array"
                            ,'http://json.org/' as "xmlns:json")
                          ,XMLElement
                            ("label"
                            ,XMLForest(CASE rslt.wf_status_code
                                       WHEN 'PENDING' THEN
                                            xxas_rslt_cache_pkg.resolve_color_name('energized') 
                                       WHEN 'RFC' THEN
                                            xxas_rslt_cache_pkg.resolve_color_name('assertive')
                                       WHEN 'ERROR' THEN
                                            xxas_rslt_cache_pkg.resolve_color_name('assertive')
                                       WHEN 'APPROVED' THEN 
                                            xxas_rslt_cache_pkg.resolve_color_name('green') 
                                       END                                              AS "color"
                                      ,rslt.wf_status_meaning                     AS "title"
                                      )
                            )
                          )
               END
              ,XMLElement
                ("linked"
                ,XMLElement
                  ("image"
                  ,XMLForest
                    (xxas_com_rest_util_pkg.uri_from_path
                       ('people/' || rslt.img_person_id || '/photo') "self"
                    ,xxas_per_image_pkg.url
                       (rslt.img_person_id, rslt.last_update_date, 'SMALL') "small"
                    ,xxas_per_image_pkg.url
                       (rslt.img_person_id, rslt.last_update_date, 'MEDIUM') "medium"
                    ,xxas_per_image_pkg.url
                       (rslt.img_person_id, rslt.last_update_date, 'FULL_SIZE') "fullSize"
                    )
                  )
                ) "photo"
              ) ORDER BY rslt.date_start DESC
            )
          )
  FROM  (SELECT papf.person_id
               ,img.person_id img_person_id
               ,img.last_update_date
               ,(CASE WHEN xamv.hr_api_status_code is not null 
                      THEN nvl(xamv.date_start_p,xamv.date_projected_start_p)
                      ELSE nvl(xamv.date_start,xamv.date_projected_start)
                 END) date_start
               ,(CASE WHEN xamv.hr_api_status_code is not null 
                      THEN nvl(xamv.date_end_p,nvl(xamv.date_projected_end_p,cp_end_date))
                      ELSE nvl(xamv.date_end,nvl(xamv.date_projected_end,cp_end_date))
                 END) date_end
               ,(CASE WHEN xamv.hr_api_status_code is not null 
                      THEN xamv.absence_type_p
                      ELSE xamv.absence_type
                 END) absence_type
               ,xamv.hr_api_status_code status_code
               ,xamv.workflow_status_code wf_status_code
               ,xamv.workflow_status_meaning wf_status_meaning
               ,papf.full_name
               ,papf.global_name
               ,papf.local_name
               ,papf.first_name
               ,papf.last_name
               ,papf.known_as
         FROM   per_all_people_f          papf
               ,xxas_per_thumbnails       img
               ,xxas_absence_management_v xamv
         WHERE  xamv.person_id = cp_person_id
         AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                               AND     papf.effective_end_date
         AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
                 OR nvl(papf.current_npw_flag, 'N')  = 'Y')
         AND    xamv.person_id = papf.person_id
         AND    papf.person_id = img.person_id(+)
         AND    (((CASE WHEN xamv.hr_api_status_code is not null 
                        THEN nvl(xamv.date_start_p,xamv.date_projected_start_p)
                        ELSE nvl(xamv.date_start,xamv.date_projected_start)
                   END) BETWEEN cp_start_date
                        AND     cp_end_date) OR
                 ((CASE WHEN xamv.hr_api_status_code is not null 
                        THEN nvl(xamv.date_end_p,nvl(xamv.date_projected_end_p,g_EOT))
                        ELSE nvl(xamv.date_end,nvl(xamv.date_projected_end,g_EOT))
                   END) BETWEEN cp_start_date
                        AND     cp_end_date) OR
                 (cp_start_date BETWEEN (CASE WHEN xamv.hr_api_status_code is not null 
                                                THEN nvl(xamv.date_start_p,xamv.date_projected_start_p)
                                                ELSE nvl(xamv.date_start,xamv.date_projected_start)
                                           END)
                                  AND     (CASE WHEN xamv.hr_api_status_code is not null 
                                                THEN nvl(xamv.date_end_p,nvl(xamv.date_projected_end_p,g_EOT))
                                                ELSE nvl(xamv.date_end,nvl(xamv.date_projected_end,g_EOT))
                                           END)) OR
                 (cp_end_date BETWEEN (CASE WHEN xamv.hr_api_status_code is not null 
                                              THEN nvl(xamv.date_start_p,xamv.date_projected_start_p)
                                              ELSE nvl(xamv.date_start,xamv.date_projected_start)
                                         END)
                                AND     (CASE WHEN xamv.hr_api_status_code is not null 
                                              THEN nvl(xamv.date_end_p,nvl(xamv.date_projected_end_p,g_EOT))
                                              ELSE nvl(xamv.date_end,nvl(xamv.date_projected_end,g_EOT))
                                         END))
                )
        ) rslt;

  CURSOR csr_data
  IS
  SELECT r.nam field_name
        ,nvl(r.val1, nvl(r.val2, r.val3)) val
        ,r.column_value field_xml
  FROM (SELECT x.column_value.getRootElement() nam
              ,extract(x.column_value, '/*/text()').getCLOBVal() val1
              ,extract(x.column_value, '/*/*/text()').getCLOBVal() val2
              ,extract(x.column_value, '/*/*/id/text()').getCLOBVal() val3
              ,x.column_value
        FROM  TABLE(XMLSequence
                     (extract
                       (p_request_xml
                       ,'//*'
                     || decode('Y'
                              ,'Y', '[not(ancestor-or-self::rows) '
                                  , '[not(self::rows) ')
                     || ' and not(self::root)
                          and not(self::postType)
                          and not(self::submittedFieldId)
                          and not(self::value)
                          and not(self::id)
                          and not(self::label)
                          and not(self::description)]'))) x) r;

  CURSOR csr_date_time_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dtr.start_date,'"') start_date
        ,replace(dtr.end_date,'"') end_date
        ,replace(dtr.start_time,'"') start_time
        ,replace(dtr.end_time,'"') end_time
  FROM   XMLTable
          ('//DATE_TIME_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
                  ,start_time      varchar2(10)     path '//startTime'
                  ,end_time        varchar2(10)     path '//endTime'
          ) dtr;

  CURSOR csr_date_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dr.start_date,'"') start_date
        ,replace(dr.end_date,'"') end_date
  FROM   XMLTable
          ('//DATE_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
          ) dr;

  CURSOR csr_employee_id(cp_employees_xml IN XMLType) IS
  SELECT emp.id
  FROM   XMLTable
          ('/EMPLOYEES/value'
          PASSING  cp_employees_xml
          COLUMNS  xml  XMLType           path '.'
                  ,id   number            path '//id'
          ) emp;

  CURSOR csr_accrual_plan_id(cp_date_end          IN date
                            ,cp_assignment_id     IN number
                            ,cp_absence_type_id   IN number) IS
  SELECT pap.accrual_plan_id
  FROM   pay_element_entries_f        pee
        ,pay_accrual_plans_v          pap
        ,per_absence_attendance_types paat
  WHERE  pee.assignment_id      = cp_assignment_id
  AND    cp_date_end BETWEEN pee.effective_start_date
                     AND     nvl(pee.effective_end_date,cp_date_end)
  AND    pee.element_type_id = pap.accrual_plan_element_type_id
  AND    pap.business_group_id  = paat.business_group_id
  AND    paat.absence_attendance_type_id = cp_absence_type_id
  AND    pap.pto_input_value_id = paat.input_value_id
  AND    cp_date_end BETWEEN paat.date_effective 
                     AND     nvl(paat.date_end,cp_date_end)
  ORDER BY pap.accrual_plan_name;

  CURSOR csr_accrual_plan_id_values(cp_date_end          IN date
                                   ,cp_assignment_id     IN number
                                   ,cp_absence_type_id   IN number) 
  IS
  SELECT XMLElement
          ("values"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("plan"
              ,XMLForest
                (pap.accrual_plan_id "id"
                ,pap.accrual_plan_name "value"
                ,pap.accrual_plan_name "label")
                )
              )
          )
  FROM   per_all_assignments_f        rslt
        ,pay_element_entries_f        pee
        ,pay_accrual_plans_v          pap
        ,per_absence_attendance_types paat
  WHERE  rslt.assignment_id     = cp_assignment_id
  AND    rslt.assignment_id     = pee.assignment_id
  AND    cp_date_end BETWEEN rslt.effective_start_date 
                     AND     rslt.effective_end_date
  AND    cp_date_end BETWEEN pee.effective_start_date 
                     AND     pee.effective_end_date
  AND    pee.element_type_id    = pap.accrual_plan_element_type_id
  AND    pap.business_group_id  = paat.business_group_id
  AND    paat.absence_attendance_type_id = cp_absence_type_id
  AND    pap.pto_input_value_id = paat.input_value_id
  AND    cp_date_end BETWEEN paat.date_effective 
                     AND     nvl(paat.date_end,cp_date_end)
  ORDER BY pap.accrual_plan_name;

  c_method CONSTANT           varchar2(30) := 'dur_cal_ol_acr_bal_abs_handler';
  l_duration_type             varchar2(30);
  l_absence_days              per_absence_attendances.absence_days%TYPE;
  l_absence_hours             per_absence_attendances.absence_hours%TYPE;
  l_use_formula               number;
  l_min_max_failure           varchar2(4000);
  l_warning_or_error          varchar2(4000);
  l_page_error_msg            varchar2(4000);
  l_data                      xxas_emp_view_ui_pkg.xxas_data_table;
  l_business_group_id         per_absence_attendances.business_group_id%TYPE;
  l_person_id                 per_absence_attendances.person_id%TYPE;
  l_assignment_id             per_all_assignments_f.assignment_id%TYPE;
  l_effective_date            date;
  l_absence_type_id           per_absence_attendances.absence_attendance_type_id%TYPE;
  l_date_start                per_absence_attendances.date_start%TYPE;
  l_date_end                  per_absence_attendances.date_end%TYPE;
  l_time_start                per_absence_attendances.time_start%TYPE;
  l_time_end                  per_absence_attendances.time_end%TYPE;
  l_source_block              varchar2(4000);
  l_saved_search_id           number;
  l_xml_data                  XMLType;
  l_abs_dur_auto_overwrite    varchar2(100);
  l_duration_days_visible     varchar2(10);
  l_duration_hours_visible    varchar2(10);
  l_duration_days_enabled     varchar2(10);
  l_duration_hours_enabled    varchar2(10);
  l_override_assignment_id    number;
  l_ol_absences_xml           XMLType;
  l_date_time_range_xml       XMLType;
  l_date_range_xml            XMLType;
  l_employees_xml             XMLType;
  l_or_asg_values_xml         XMLType;
  l_dtr_values_rec            csr_date_time_range_values%ROWTYPE;
  l_dr_values_rec             csr_date_range_values%ROWTYPE;
  l_cal_duration              per_absence_attendances.absence_days%TYPE;
  l_peer_asg_tbl              xxas_com_num_table;
  l_team_asg_tbl              xxas_com_num_table;
  l_per_tbl                   xxas_com_num_table;
  l_emp_calendar_flag         varchar2(1);
  l_mgr_calendar_flag         varchar2(1);
  l_person_id_field           xxas_emp_ui_hidden_type;
  l_asg_id_field              xxas_emp_ui_hidden_type;
  l_bg_id_field               xxas_emp_ui_hidden_type;
  l_dur_type_field            xxas_emp_ui_hidden_type;
  l_dur_days_field            xxas_emp_ui_number_type;
  l_dur_hours_field           xxas_emp_ui_number_type;
  l_startdate_field           xxas_emp_ui_hidden_type;
  l_enddate_field             xxas_emp_ui_hidden_type;
  l_starttime_field           xxas_emp_ui_hidden_type;
  l_endtime_field             xxas_emp_ui_hidden_type;
  l_accrual_plan_id           pay_accrual_plans_v.accrual_plan_id%TYPE;
  l_proj_bal_container        xxas_emp_ui_container_type;
  l_effective_at_field        xxas_emp_ui_date_type;
  l_accr_plan_id_values_xml   XMLType;
  l_accrual_chart_parameters  varchar2(4000);
  l_leave_balance_chart_xml   XMLType;
  l_accrual_plan_name         pay_accrual_plans_v.accrual_plan_name%TYPE;
  l_accrual_plan_uom          pay_accrual_plans_v.accrual_units_of_measure%TYPE;
  l_accrual_plan_uom_name     pay_accrual_plans_v.accrual_units_of_measure_name%TYPE;
  l_start_date                date;
  l_end_date                  date;
  l_gross_total               number;
  l_entitlement               number;
  l_carryover                 number;
  l_ent_adjust                number;
  l_used_total                number;
  l_used                      number;
  l_pending                   number;
  l_used_adjust               number;
  l_remaining                 number;
  l_html_text                 varchar2(4000);
  l_html_field                xxas_emp_ui_html_type;
  l_ol_absences_per_xml       XMLType;
  l_ui                        xxas_emp_ui_view_type;
  l_function_id               fnd_form_functions.function_id%TYPE;
  l_absence_attendance_id     per_absence_attendances.absence_attendance_id%TYPE;
  l_emp_ui_fields             xxas_emp_ui_component_table := xxas_emp_ui_component_table();
  l_container                 xxas_emp_ui_container_type;
  l_container_grp_field_list  xxas_emp_ui_component_table;
  l_ff_dur_days_field         xxas_emp_ui_number_type;
  l_ff_dur_hours_field        xxas_emp_ui_number_type;
  l_ff_proj_bal_container     xxas_emp_ui_container_type;
  l_ff_overlap_abs_container  xxas_emp_ui_container_type;
  l_ff_effective_at_field     xxas_emp_ui_date_type;
  l_ff_accr_plan_id_field     xxas_emp_ui_list_type;
  l_ff_html_text_field        xxas_emp_ui_html_type;
  l_duration_days             number;
  l_duration_hours            number;

BEGIN

  log_start(c_method);

  l_data := xxas_emp_view_ui_pkg.transform_jsonxml_form_data
              (p_request_xml);

  IF l_data.exists('BUSINESS_GROUP_ID') THEN
    l_business_group_id := l_data('BUSINESS_GROUP_ID');
  END IF;

  IF l_data.exists('PERSON_ID') THEN
    l_person_id := l_data('PERSON_ID');
  END IF;

  IF l_data.exists('ASSIGNMENT_ID') THEN
    l_assignment_id := l_data('ASSIGNMENT_ID');
  END IF;

  IF l_data.exists('OVERRIDE_ASSIGNMENT_ID') THEN
    l_override_assignment_id := l_data('OVERRIDE_ASSIGNMENT_ID');
  END IF;

  l_effective_date := trunc(sysdate);

  IF l_data.exists('ABSENCE_TYPE_ID') THEN
    l_absence_type_id := l_data('ABSENCE_TYPE_ID');
  END IF;

  IF l_data.exists('SOURCE_BLOCK') THEN
    l_source_block := replace(l_data('SOURCE_BLOCK'),'$','_');
  END IF;

  IF l_data.exists('SAVED_SEARCH_ID') THEN
    l_saved_search_id := l_data('SAVED_SEARCH_ID');
  END IF;

  IF l_data.exists('ABSENCE_ATTENDANCE_ID') THEN
    l_absence_attendance_id := l_data('ABSENCE_ATTENDANCE_ID');
  END IF;

  
  IF l_data.exists('DURATION_DAYS') THEN
    l_duration_days := l_data('DURATION_DAYS');
  END IF;

  IF l_data.exists('DURATION_HOURS') THEN
   l_duration_hours := l_data('DURATION_HOURS');
  END IF;

  l_duration_type := xxas_leave_of_absence_pkg.get_absence_type_value(l_absence_type_id, 'HOURS_OR_DAYS');

  --
  -- Convert the XML form data into the xxas_data_table structure.
  --
  IF p_request_xml is not null THEN

    FOR csr_rec IN csr_data LOOP
      IF csr_rec.field_name is not null THEN
        IF csr_rec.field_name = 'DATE_TIME_RANGE' AND 
           (l_duration_type is not null AND
            l_duration_type IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_time_range_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'DATE_RANGE' AND
              (l_duration_type is not null AND
               l_duration_type NOT IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_range_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'EMPLOYEES' THEN
          l_employees_xml := csr_rec.field_xml;
        END IF;
      END IF;
    END LOOP;

  END IF;

  IF l_date_time_range_xml is not null THEN

    OPEN  csr_date_time_range_values(l_date_time_range_xml);
    FETCH csr_date_time_range_values INTO l_dtr_values_rec;
    CLOSE csr_date_time_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dtr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dtr_values_rec.end_date);
    l_time_start := l_dtr_values_rec.start_time;
    l_time_end   := l_dtr_values_rec.end_time;

  ELSIF l_date_range_xml is not null THEN

    OPEN  csr_date_range_values(l_date_range_xml);
    FETCH csr_date_range_values INTO l_dr_values_rec;
    CLOSE csr_date_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dr_values_rec.end_date);

  END IF;

  IF l_person_id is null THEN

    IF l_employees_xml is not null THEN

      OPEN  csr_employee_id(l_employees_xml);
      FETCH csr_employee_id INTO l_person_id;
      CLOSE csr_employee_id;

      l_assignment_id     := xxas_util_pkg.get_primary_worker_asg(l_person_id,NVL(l_date_start,l_effective_date));
      l_business_group_id := xxas_per_pkg.get_business_group_id(l_person_id);

    END IF;

  ELSE

    IF l_employees_xml is null THEN

      l_person_id         := null;
      l_assignment_id     := null;
      l_business_group_id := null;

    ELSE

      l_person_id         := null;
      l_assignment_id     := null;
      l_business_group_id := null;

      OPEN  csr_employee_id(l_employees_xml);
      FETCH csr_employee_id INTO l_person_id;
      CLOSE csr_employee_id;

      l_assignment_id     := xxas_util_pkg.get_primary_worker_asg(l_person_id,NVL(l_date_start,l_effective_date));
      l_business_group_id := xxas_per_pkg.get_business_group_id(l_person_id);
      
    END IF;

  END IF;

  IF l_person_id is not null THEN

    OPEN  csr_or_asg_values_xml(l_override_assignment_id
                               ,l_person_id
                               ,l_date_start);
    FETCH csr_or_asg_values_xml INTO l_or_asg_values_xml;
    CLOSE csr_or_asg_values_xml;

  END IF;

  IF l_absence_attendance_id is null THEN

    l_function_id := xxas_cal_util_pkg.page_slug_func_id('absence-new');

  ELSE

    l_function_id := xxas_cal_util_pkg.page_slug_func_id('absence-edit');

  END IF;

  l_ui := xxas_emp_view_ui_pkg.process_block_fast_formula
            (p_function_id       => l_function_id
            ,p_form_data_xml     => p_request_xml);

  IF l_ui.createfields IS NOT NULL 
  THEN
    l_emp_ui_fields := l_ui.createfields.field_list;
  END IF;

  IF l_ui.editfields IS NOT NULL 
  THEN
    l_emp_ui_fields := l_ui.editfields.field_list;
  END IF;

  FOR i IN 1..l_emp_ui_fields.COUNT
  LOOP

    log_statement(c_method,'field_id: ' || l_emp_ui_fields(i).get_id);

    IF l_emp_ui_fields(i) IS OF(xxas_emp_ui_container_type) THEN

      l_container := TREAT(l_emp_ui_fields(i) AS xxas_emp_ui_container_type);

      l_container_grp_field_list := l_container.field_list;

      IF l_container.get_id = 'BASIC_GRP' THEN

        FOR j IN 1..l_container_grp_field_list.COUNT
        LOOP

          log_statement(c_method,'BASIC_GRP - field_id: ' || l_container_grp_field_list(j).get_id);

          IF l_container_grp_field_list(j).get_id = 'DURATION_DAYS' THEN

            l_ff_dur_days_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_number_type);

          ELSIF l_container_grp_field_list(j).get_id = 'DURATION_HOURS' THEN

            l_ff_dur_hours_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_number_type);

          END IF;

        END LOOP;

      ELSIF l_container.get_id = 'PROJECTED_BALANCE' THEN

        l_ff_proj_bal_container := TREAT(l_container AS xxas_emp_ui_container_type);

        FOR j IN 1..l_container_grp_field_list.COUNT
        LOOP

          log_statement(c_method,'PROJECTED_BALANCE - field_id: ' || l_container_grp_field_list(j).get_id);

          IF l_container_grp_field_list(j).get_id = 'EFFECTIVE_AT' THEN

            l_ff_effective_at_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_date_type);

          ELSIF l_container_grp_field_list(j).get_id = 'ACCRUAL_PLAN_ID' THEN

            l_ff_accr_plan_id_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_list_type);

          ELSIF l_container_grp_field_list(j).get_id = 'HTML_TEXT' THEN

            l_ff_html_text_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_html_type);

          END IF;

        END LOOP;

      ELSIF l_container.get_id = 'OVERLAP_ABSENCES' THEN

        l_ff_overlap_abs_container := TREAT(l_container AS xxas_emp_ui_container_type);

      END IF;

    END IF;

    l_container        := null;

  END LOOP;

  IF l_date_start   is not null AND 
     l_date_end     is not null AND 
     l_source_block is not null THEN

    l_emp_calendar_flag := CASE WHEN instr(xxas_com_sec_pkg.get_function_parameter
                                             (p_function_name  => l_source_block
                                             ,p_parameter_name => 'pRestProvider'),'for-individual') <> 0
                                THEN 'Y'
                                ELSE 'N'
                           END;

    l_mgr_calendar_flag := CASE WHEN instr(xxas_com_sec_pkg.get_function_parameter
                                             (p_function_name  => l_source_block
                                             ,p_parameter_name => 'pRestProvider'),'for-team') <> 0
                                THEN 'Y'
                                ELSE 'N'
                           END;

    IF l_emp_calendar_flag = 'Y' THEN

      OPEN  csr_peer_asgs(l_date_start
                         ,l_date_end);
      FETCH csr_peer_asgs BULK COLLECT INTO l_peer_asg_tbl;
      CLOSE csr_peer_asgs;

      OPEN  csr_peer_person_ids(l_peer_asg_tbl);
      FETCH csr_peer_person_ids BULK COLLECT INTO l_per_tbl;
      CLOSE csr_peer_person_ids;

    ELSIF l_mgr_calendar_flag = 'Y' THEN

      IF l_saved_search_id is not null THEN

        OPEN  csr_team_saved_sch_asgs(l_saved_search_id);
        FETCH csr_team_saved_sch_asgs BULK COLLECT INTO l_team_asg_tbl;
        CLOSE csr_team_saved_sch_asgs;

        OPEN  csr_team_saved_sch_person_ids(l_team_asg_tbl);
        FETCH csr_team_saved_sch_person_ids BULK COLLECT INTO l_per_tbl;
        CLOSE csr_team_saved_sch_person_ids;

      ELSE

        OPEN  csr_team_asgs(l_date_start
                           ,l_date_end);
        FETCH csr_team_asgs BULK COLLECT INTO l_team_asg_tbl;
        CLOSE csr_team_asgs;

        OPEN  csr_team_person_ids(l_team_asg_tbl);
        FETCH csr_team_person_ids BULK COLLECT INTO l_per_tbl;
        CLOSE csr_team_person_ids;

      END IF;

    END IF;

    IF l_per_tbl is not null AND 
       l_per_tbl.count > 0 THEN

      FOR i in 1..l_per_tbl.count
      LOOP

        IF l_per_tbl(i) is not null THEN

          OPEN  csr_ol_absences_data(l_per_tbl(i)
                                    ,l_date_start
                                    ,l_date_end
                                    ,l_emp_calendar_flag
                                    ,l_mgr_calendar_flag);
          FETCH csr_ol_absences_data INTO l_ol_absences_per_xml;
          CLOSE csr_ol_absences_data;

          IF l_ol_absences_per_xml.extract('/absences/*') is not null THEN

            IF l_ol_absences_xml is null THEN

              SELECT XMLTYPE.appendChildXML
                      (XMLElement
                        ("items"
                        ,XMLAttributes
                          ('true' as "json:force-array"
                          ,'http://json.org/' as "xmlns:json"))
                      ,'/items'
                      ,l_ol_absences_per_xml.extract('/absences/*'))
              INTO   l_ol_absences_xml
              FROM   dual;

            ELSE

              SELECT XMLTYPE.appendChildXML
                      (l_ol_absences_xml
                      ,'/items'
                      ,l_ol_absences_per_xml.extract('/absences/*'))
              INTO   l_ol_absences_xml
              FROM   dual;

            END IF;

          END IF;

        END IF;

      END LOOP;

    END IF;

  END IF;

  IF l_duration_type is not null AND
     l_date_start    is not null AND
     l_date_end      is not null THEN

    l_cal_duration := xxas_leave_of_absence_pkg.calculate_abs_duration_full
                             (p_business_group_id => l_business_group_id
                             ,p_assignment_id     => l_assignment_id
                             ,p_effective_date    => l_effective_date
                             ,p_absence_type_id   => l_absence_type_id
                             ,p_date_start        => l_date_start
                             ,p_date_end          => l_date_end
                             ,p_time_start        => l_time_start
                             ,p_time_end          => l_time_end
                             ,p_absence_days      => l_absence_days
                             ,p_absence_hours     => l_absence_hours
                             ,p_use_formula       => l_use_formula
                             ,p_min_max_failure   => l_min_max_failure
                             ,p_warning_or_error  => l_warning_or_error
                             ,p_page_error_msg    => l_page_error_msg);

  END IF;

  IF l_duration_type is not null THEN

    l_abs_dur_auto_overwrite := fnd_profile.value('PER_ABSENCE_DURATION_AUTO_OVERWRITE');

    IF l_duration_type = 'H' THEN

      IF nvl(l_abs_dur_auto_overwrite,'N') = 'Y' THEN
        l_duration_hours_enabled := 'false';
      ELSE
        l_duration_hours_enabled := 'true';
      END IF;

      l_duration_hours_visible := 'true';
      l_duration_days_visible  := 'false';

    ELSIF l_duration_type = xxas_fast_formula_pkg.g_NULL_STRING THEN

      IF nvl(l_abs_dur_auto_overwrite,'N') = 'Y' THEN
        l_duration_days_enabled  := 'false';
        l_duration_hours_enabled := 'false';
      ELSE
        l_duration_days_enabled  := 'true';
        l_duration_hours_enabled := 'true';
      END IF;

      l_duration_days_visible  := 'true';
      l_duration_hours_visible := 'true';

    ELSE

      IF nvl(l_abs_dur_auto_overwrite,'N') = 'Y' THEN
        l_duration_days_enabled  := 'false';
      ELSE
        l_duration_days_enabled  := 'true';
      END IF;

      l_duration_days_visible  := 'true';
      l_duration_hours_visible := 'false';

    END IF;

  END IF;

  IF l_page_error_msg is null AND 
     l_date_end >= l_date_start THEN

    IF l_ff_dur_days_field is not null THEN

      l_dur_days_field := l_ff_dur_days_field;
      l_dur_days_field.set_enabled(nvl(l_ff_dur_days_field.get_enabled,l_duration_days_enabled));
      l_dur_days_field.set_visible(nvl(l_ff_dur_days_field.get_visible,l_duration_days_visible));
      l_dur_days_field.set_value(nvl(l_ff_dur_days_field.get_number_value,nvl(l_absence_days,0)));
      l_dur_days_field.set_decimal_precision(nvl(l_ff_dur_days_field.get_decimal_precision,4));

    ELSE 

      l_dur_days_field := xxas_emp_ui_number_type
                            (p_id                => 'DURATION_DAYS'
                            ,p_label             => null
                            ,p_description       => null
                            ,p_refresh_on_change => null
                            ,p_trigger_name      => null
                            ,p_trigger_method    => null
                            ,p_enabled           => l_duration_days_enabled
                            ,p_required          => null
                            ,p_visible           => l_duration_days_visible
                            ,p_hide_if           => null
                            ,p_value             => nvl(l_absence_days,0)
                            ,p_decimal_precision => 4
                            ,p_regexp            => null
                            ,p_regexp_error      => null
                            ,p_transient         => null);

    END IF;

    IF l_ff_dur_hours_field is not null THEN

      l_dur_hours_field := l_ff_dur_hours_field;
      l_dur_hours_field.set_enabled(nvl(l_ff_dur_hours_field.get_enabled,l_duration_hours_enabled));
      l_dur_hours_field.set_visible(nvl(l_ff_dur_hours_field.get_visible,l_duration_hours_visible));
      l_dur_hours_field.set_value(nvl(l_ff_dur_hours_field.get_number_value,nvl(l_absence_hours,0)));
      l_dur_hours_field.set_decimal_precision(nvl(l_ff_dur_hours_field.get_decimal_precision,4));

    ELSE 

      l_dur_hours_field := xxas_emp_ui_number_type
                             (p_id                => 'DURATION_HOURS'
                             ,p_label             => null
                             ,p_description       => null
                             ,p_refresh_on_change => null
                             ,p_trigger_name      => null
                             ,p_trigger_method    => null
                             ,p_enabled           => l_duration_hours_enabled
                             ,p_required          => null
                             ,p_visible           => l_duration_hours_visible
                             ,p_hide_if           => null
                             ,p_value             => nvl(l_absence_hours,0)
                             ,p_decimal_precision => 4
                             ,p_regexp            => null
                             ,p_regexp_error      => null
                             ,p_transient         => null);

    END IF;

  END IF;

  l_dur_type_field := xxas_emp_ui_hidden_type
                        (p_id        => 'DURATION_TYPE'
                        ,p_value     => l_duration_type
                        ,p_transient => null);

  l_person_id_field := xxas_emp_ui_hidden_type
                         (p_id        => 'PERSON_ID'
                         ,p_value     => l_person_id
                         ,p_transient => null);

  l_asg_id_field := xxas_emp_ui_hidden_type
                      (p_id        => 'ASSIGNMENT_ID'
                      ,p_value     => l_assignment_id
                      ,p_transient => null);

  l_bg_id_field := xxas_emp_ui_hidden_type
                      (p_id        => 'BUSINESS_GROUP_ID'
                      ,p_value     => l_business_group_id
                      ,p_transient => null);

  l_startdate_field := xxas_emp_ui_hidden_type
                         (p_id        => 'STARTDATE'
                         ,p_value     => to_char(l_date_start,g_HRAPI_DATE_FMT)
                         ,p_transient => null);

  l_enddate_field := xxas_emp_ui_hidden_type
                       (p_id        => 'ENDDATE'
                       ,p_value     => to_char(l_date_end,g_HRAPI_DATE_FMT)
                       ,p_transient => null);

  l_starttime_field := xxas_emp_ui_hidden_type
                         (p_id        => 'STARTTIME'
                         ,p_value     => l_time_start
                         ,p_transient => null);

  l_endtime_field := xxas_emp_ui_hidden_type
                       (p_id        => 'ENDTIME'
                       ,p_value     => l_time_end
                       ,p_transient => null);

  IF l_date_end        is not null AND 
     l_assignment_id   is not null AND 
     l_absence_type_id is not null THEN

    --
    -- Check if the absence type is linked to an accrual plan the person is enrolled onto as of the absence end date
    --
    OPEN  csr_accrual_plan_id(l_date_end
                             ,l_assignment_id
                             ,l_absence_type_id);
    FETCH csr_accrual_plan_id INTO l_accrual_plan_id;
    CLOSE csr_accrual_plan_id;

    IF l_accrual_plan_id is not null THEN

      IF l_ff_effective_at_field is not null THEN

        l_effective_at_field := l_ff_effective_at_field;
        l_effective_at_field.set_value(nvl(l_ff_effective_at_field.get_date_value('Y'),fnd_date.date_to_canonical(l_date_end)));

      ELSE
        --
        -- EFFECTIVE_AT Date field
        --
        l_effective_at_field := xxas_emp_ui_date_type
                                  (p_date_type         => 'Date'
                                  ,p_id                => 'EFFECTIVE_AT'
                                  ,p_label             => null
                                  ,p_description       => null
                                  ,p_refresh_on_change => null
                                  ,p_trigger_name      => null
                                  ,p_trigger_method    => null
                                  ,p_enabled           => null
                                  ,p_required          => null
                                  ,p_visible           => null
                                  ,p_hide_if           => null 
                                  ,p_value             => fnd_date.date_to_canonical(l_date_end)
                                  ,p_date_format       => null
                                  ,p_transient         => null
                                  ,p_post_format       => null);

      END IF;

      OPEN  csr_accrual_plan_id_values(l_date_end
                                      ,l_assignment_id
                                      ,l_absence_type_id);
      FETCH csr_accrual_plan_id_values INTO l_accr_plan_id_values_xml;
      CLOSE csr_accrual_plan_id_values;

      l_accrual_chart_parameters := 'pAssignmentId=' || to_char(l_assignment_id) || '&pEffectiveDate=' || to_char(l_date_end, 'YYYY-MM-DD') 
                                    || '&pAccrualPlanId=' || to_char(l_accrual_plan_id) || '&pProjectedAmount=' || to_char(nvl(l_absence_days,0));

      l_leave_balance_chart_xml := xxas_absence_management_pkg.leave_balance_chart(p_value => l_accrual_chart_parameters);

      xxas_absence_management_pkg.get_accrual_plan_details
        (p_person_id             => xxas_util_pkg.get_person_from_asg(l_assignment_id,l_date_end)
        ,p_assignment_id         => l_assignment_id
        ,p_effective_date        => l_date_end
        ,p_accrual_plan_id       => l_accrual_plan_id
        ,p_include_pending       => true --xxas_util_pkg.flag_to_boolean('Y')
        ,p_accrual_plan_name     => l_accrual_plan_name
        ,p_accrual_plan_uom      => l_accrual_plan_uom
        ,p_accrual_plan_uom_name => l_accrual_plan_uom_name
        ,p_start_date            => l_start_date
        ,p_end_date              => l_end_date
        ,p_gross_total           => l_gross_total
        ,p_entitlement           => l_entitlement
        ,p_carryover             => l_carryover
        ,p_entitlement_adjust    => l_ent_adjust
        ,p_used_total            => l_used_total
        ,p_used                  => l_used
        ,p_pending               => l_pending
        ,p_used_adjust           => l_used_adjust
        ,p_remaining             => l_remaining);

      --
      -- Accrual plan Balance is less than zero
      --              
      IF nvl(l_remaining,0) < 0 THEN

        l_html_text := xxas_com_msg_pkg.get_decoded_string
                         (xxas_com_msg_type
                            (xxas_util_pkg.app
                            ,'XXAS_ACC_REM_BAL_LES_THAN_ZERO'
                            ,xxas_com_msg_token_varray
                               (xxas_com_msg_token_type
                                  ('DATE'
                                  ,to_char(l_date_end, 'FMDd Mon YYYY'))
                               )
                            )
                         );

        IF l_html_text is not null THEN

          l_html_text := '<font color="' 
                         || xxas_rslt_cache_pkg.resolve_color_name('red')
                         || '">' || l_html_text || '</font>';
        END IF;

        IF l_ff_html_text_field is not null THEN

          l_html_field := l_ff_html_text_field;
          l_html_field.set_visible(nvl(l_ff_html_text_field.get_visible,'true'));
          l_html_field.set_value(nvl(l_ff_html_text_field.get_value,l_html_text));

        ELSE

          l_html_field := xxas_emp_ui_html_type
                            (p_id                => 'HTML_TEXT'
                            ,p_label             => null
                            ,p_description       => null  
                            ,p_refresh_on_change => null
                            ,p_trigger_name      => null
                            ,p_trigger_method    => null
                            ,p_enabled           => null
                            ,p_required          => null
                            ,p_visible           => 'true'
                            ,p_hide_if           => null
                            ,p_value             => l_html_text
                            ,p_transient         => null);

        END IF;

      END IF;

    ELSE

      --
      -- PROJECTED_BALANCE Container make it hidden
      --
      l_proj_bal_container := xxas_emp_ui_container_type
                                (p_id             => 'PROJECTED_BALANCE'
                                ,p_label          => null
                                ,p_description    => null
                                ,p_visible        => 'false'
                                ,p_hide_if        => null
                                ,p_review_only    => null
                                ,p_type           => null
                                ,p_trigger_name   => null
                                ,p_trigger_method => null);

    END IF;

  ELSE

    --
    -- PROJECTED_BALANCE Container make it hidden
    --
    l_proj_bal_container := xxas_emp_ui_container_type
                              (p_id             => 'PROJECTED_BALANCE'
                              ,p_label          => null
                              ,p_description    => null
                              ,p_visible        => 'false'
                              ,p_hide_if        => null
                              ,p_review_only    => null
                              ,p_type           => null
                              ,p_trigger_name   => null
                              ,p_trigger_method => null);

  END IF;

  SELECT XMLElement
          ("data"
          ,XMLElement
          ("form"
          ,XMLElement
            ("fields"
            ,XMLAttributes
              ('http://json.org/' as "xmlns:json"
              ,'true' as "json:force-array")
            ,XMLElement
              ("container"
              ,XMLForest
                ('BASIC_GRP' as "id"
                )
              ,XMLElement
                ("fields"
                ,XMLAttributes
                  ('http://json.org/' as "xmlns:json"
                  ,'true' as "json:force-array")
                ,XMLElement
                  ("field"
                  ,XMLForest
                    ('OVERRIDE_ASSIGNMENT_ID' as "id"
                    ,(CASE WHEN l_override_assignment_id is not null
                           THEN l_override_assignment_id
                           ELSE null
                      END)             as "value"
                    )
                  ,l_or_asg_values_xml
                  )
                ,l_startdate_field.get_json_friendly()
                ,l_enddate_field.get_json_friendly()
                ,l_starttime_field.get_json_friendly()
                ,l_endtime_field.get_json_friendly()
                ,l_dur_type_field.get_json_friendly()
                ,CASE WHEN l_duration_type is not null AND
                           (l_duration_type NOT IN ('H',xxas_fast_formula_pkg.g_NULL_STRING) OR 
                            l_duration_type = xxas_fast_formula_pkg.g_NULL_STRING) AND
                           (l_page_error_msg is null AND 
                            l_date_end >= l_date_start)
                      THEN l_dur_days_field.get_json_friendly()
                      WHEN  l_duration_days is not null and (l_date_start is null or l_date_end is null) then
                        XMLELement("field"
                                  ,XMLElement("id" ,'DURATION_DAYS')
                                  ,XMLElement("value" ,null)        
                                  ) 
                      ELSE null 
                 END
                ,CASE WHEN (l_duration_type is not null AND
                            l_duration_type IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) AND
                           (l_page_error_msg is null AND 
                            l_date_end >= l_date_start)
                      THEN l_dur_hours_field.get_json_friendly()
                        WHEN  l_duration_hours is not null and (l_date_start is null or l_date_end is null) then
                        XMLELement("field"
                                  ,XMLElement("id",'DURATION_HOURS')
                                  ,XMLElement("value" ,null)        
                                  ) 
                      ELSE null
                 END
                ,l_person_id_field.get_json_friendly()
                ,l_asg_id_field.get_json_friendly()
                ,l_bg_id_field.get_json_friendly()
                )
              )
            ,(CASE WHEN l_accrual_plan_id is not null
                   THEN XMLElement
                         ("container"
                         ,XMLForest
                           ('PROJECTED_BALANCE' as "id"
                           ,(CASE WHEN l_ff_proj_bal_container is not null 
                                  THEN nvl(l_ff_proj_bal_container.get_state,'input')
                                  ELSE 'input'
                             END)               as "state"
                           ,(CASE WHEN l_ff_proj_bal_container is not null 
                                  THEN nvl(l_ff_proj_bal_container.get_visible,'true')
                                  ELSE 'true'
                             END)               as"visible"
                           ,'fieldset'          as "type"
                           )
                         ,XMLElement
                           ("fields"
                           ,XMLAttributes
                             ('http://json.org/' as "xmlns:json"
                             ,'true' as "json:force-array")
                           ,l_effective_at_field.get_json_friendly()
                           ,XMLElement
                             ("field"
                             ,XMLForest
                               ('ACCRUAL_PLAN_ID' as "id"
                               ,l_accrual_plan_id as "value"
                               )
                             ,l_accr_plan_id_values_xml
                             )
                           ,(CASE WHEN l_leave_balance_chart_xml is not null
                                  THEN XMLElement
                                        ("field"
                                        ,XMLForest
                                          ('ACCRUAL_CHART' as "id"
                                          ,'item'          as "type"
                                          ,'view'          as "state"
                                          )
                                        ,l_leave_balance_chart_xml
                                        ,XMLElement
                                          ("meta"
                                          ,XMLElement
                                            ("options"
                                            ,XMLForest
                                              ('detail' AS "style"
                                              )
                                            )
                                          )
                                        )
                                  ELSE null
                             END)
                           ,(CASE WHEN l_html_text is not null
                                  THEN l_html_field.get_json_friendly()
                                  ELSE null
                             END)
                           )
                         )
                   ELSE l_proj_bal_container.get_json_friendly()
              END)
            ,(CASE WHEN l_date_start is not null AND
                        l_date_end   is not null
                   THEN XMLElement
                         ("container"
                         ,XMLForest
                           ('fieldset'              as "type"
                           ,'OVERLAP_ABSENCES'      as "id"
                           ,(CASE WHEN l_ff_overlap_abs_container is not null 
                                  THEN nvl(l_ff_overlap_abs_container.get_label,xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_EMP_ABS_EVENT_OL_ABSENCES'))
                                  ELSE xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_EMP_ABS_EVENT_OL_ABSENCES')
                             END)  as "label"
                           ,(CASE WHEN l_ff_overlap_abs_container is not null 
                                  THEN nvl(l_ff_overlap_abs_container.get_state,'input')
                                  ELSE 'input'
                             END)  as "state"
                           )
                         ,XMLElement
                           ("fields"
                           ,XMLAttributes
                             ('http://json.org/' as "xmlns:json"
                             ,'true' as "json:force-array")
                           ,XMLElement
                             ("field"
                             ,XMLForest
                               ('list'                 as "type"
                               ,'OL_ABSENCES_LI'       as "id"
                               --,xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_EMP_ABS_EVENT_OL_ABSENCES') as "label"
                               ,'view'                 as "state"
                               )
                             ,XMLElement
                               ("items"
                               ,XMLAttributes
                                 ('http://json.org/' as "xmlns:json"
                                 ,'true' as "json:force-array")
                               ,(CASE WHEN l_ol_absences_xml.extract('/items/*') is not null
                                      THEN l_ol_absences_xml.extract('/items/*')
                                      ELSE null 
                                 END)
                               )
                             ,XMLElement
                               ("meta"
                               ,XMLElement
                                 ("options"
                                 ,XMLForest
                                   ('row' AS "style"
                                   ,(CASE WHEN l_ol_absences_xml.extract('/items/*') is null
                                          THEN xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_OL_ABSENCES_NM')
                                          ELSE null
                                     END) "noDataMessage"
                                   )
                                 )
                               )
                             )
                           )
                         )
                   ELSE null
              END)
            )
          )
         ,(CASE WHEN l_date_end < l_date_start
                THEN xxas_com_msg_pkg.get_error_message_xml
                       (p_message    => xxas_com_msg_type(xxas_util_pkg.app, 'XXAS_EMP_ABS_ED', null)
                       ,p_redirect   => null
                       ,p_target     => null
                       ).extract('data/*')
                WHEN l_page_error_msg is not null
                THEN xxas_com_msg_pkg.get_error_message_xml
                       (p_message    => xxas_com_msg_type
                                          (xxas_util_pkg.app
                                          ,'XXAS_COM_TEXT'
                                          ,xxas_com_msg_token_varray
                                             (xxas_com_msg_token_type
                                                ('TEXT'
                                                ,l_page_error_msg)))
                       ,p_redirect   => null
                       ,p_target     => null
                       ).extract('data/*')
                ELSE null
           END
           ))
  INTO   l_xml_data
  FROM   dual;

  log_end(c_method);

  RETURN l_xml_data;

END dur_cal_ol_acr_bal_abs_handler;

--
-- ------------------------------------------------------------------------ *
-- Name    : show_dates_reasons_acr_bal
-- ------------------------------------------------------------------------ *
FUNCTION show_dates_reasons_acr_bal
  (p_request_xml IN XMLType) RETURN XMLType
IS

  CURSOR csr_absence_reasons(cp_abs_type_id IN number
                            ,cp_person_id   IN number) 
  IS
  SELECT XMLElement
          ("values"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("value"
              ,XMLForest
                (rslt.abs_attendance_reason_id "id"
                ,null                          "description"
                ,rslt.meaning                  "label")
              )
            )
          )
  FROM  (SELECT DISTINCT paar.abs_attendance_reason_id
                        ,hl.meaning
         FROM   hr_lookups hl
               ,per_abs_attendance_reasons paar
               ,per_all_people_f papf
         WHERE hl.lookup_type                  = 'ABSENCE_REASON'
         AND   paar.business_group_id          = papf.business_group_id
         AND   papf.person_id                  = cp_person_id
         AND   trunc(sysdate) BETWEEN papf.effective_start_date
                              AND     papf.effective_end_date
         AND   paar.name                       = hl.lookup_code
         AND   paar.absence_attendance_type_id = cp_abs_type_id
         AND   trunc(sysdate) BETWEEN nvl(hl.start_date_active,trunc(sysdate))
                              AND     nvl(hl.end_date_active,trunc(sysdate))) rslt;

  CURSOR csr_data
  IS
  SELECT r.nam field_name
        ,nvl(r.val1, nvl(r.val2, r.val3)) val
        ,r.column_value field_xml
  FROM (SELECT x.column_value.getRootElement() nam
              ,extract(x.column_value, '/*/text()').getCLOBVal() val1
              ,extract(x.column_value, '/*/*/text()').getCLOBVal() val2
              ,extract(x.column_value, '/*/*/id/text()').getCLOBVal() val3
              ,x.column_value
        FROM  TABLE(XMLSequence
                     (extract
                       (p_request_xml
                       ,'//*'
                     || decode('Y'
                              ,'Y', '[not(ancestor-or-self::rows) '
                                  , '[not(self::rows) ')
                     || ' and not(self::root)
                          and not(self::postType)
                          and not(self::submittedFieldId)
                          and not(self::value)
                          and not(self::id)
                          and not(self::label)
                          and not(self::description)]'))) x) r;

  CURSOR csr_date_time_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dtr.start_date,'"') start_date
        ,replace(dtr.end_date,'"') end_date
        ,replace(dtr.start_time,'"') start_time
        ,replace(dtr.end_time,'"') end_time
  FROM   XMLTable
          ('//DATE_TIME_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
                  ,start_time      varchar2(10)     path '//startTime'
                  ,end_time        varchar2(10)     path '//endTime'
          ) dtr;

  CURSOR csr_date_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dr.start_date,'"') start_date
        ,replace(dr.end_date,'"') end_date
  FROM   XMLTable
          ('//DATE_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
          ) dr;

  CURSOR csr_employee_id(cp_employees_xml IN XMLType) IS
  SELECT emp.id
  FROM   XMLTable
          ('/EMPLOYEES/value'
          PASSING  cp_employees_xml
          COLUMNS  xml  XMLType           path '.'
                  ,id   number            path '//id'
          ) emp;

  CURSOR csr_accrual_plan_id(cp_date_end          IN date
                            ,cp_assignment_id     IN number
                            ,cp_absence_type_id   IN number) IS
  SELECT pap.accrual_plan_id
  FROM   pay_element_entries_f        pee
        ,pay_accrual_plans_v          pap
        ,per_absence_attendance_types paat
  WHERE  pee.assignment_id      = cp_assignment_id
  AND    cp_date_end BETWEEN pee.effective_start_date
                     AND     nvl(pee.effective_end_date,cp_date_end)
  AND    pee.element_type_id = pap.accrual_plan_element_type_id
  AND    pap.business_group_id  = paat.business_group_id
  AND    paat.absence_attendance_type_id = cp_absence_type_id
  AND    pap.pto_input_value_id = paat.input_value_id
  AND    cp_date_end BETWEEN paat.date_effective 
                     AND     nvl(paat.date_end,cp_date_end)
  ORDER BY pap.accrual_plan_name;

  CURSOR csr_accrual_plan_id_values(cp_date_end          IN date
                                   ,cp_assignment_id     IN number
                                   ,cp_absence_type_id   IN number) 
  IS
  SELECT XMLElement
          ("values"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("plan"
              ,XMLForest
                (pap.accrual_plan_id "id"
                ,pap.accrual_plan_name "value"
                ,pap.accrual_plan_name "label")
                )
              )
          )
  FROM   per_all_assignments_f        rslt
        ,pay_element_entries_f        pee
        ,pay_accrual_plans_v          pap
        ,per_absence_attendance_types paat
  WHERE  rslt.assignment_id     = cp_assignment_id
  AND    rslt.assignment_id     = pee.assignment_id
  AND    cp_date_end BETWEEN rslt.effective_start_date 
                     AND     rslt.effective_end_date
  AND    cp_date_end BETWEEN pee.effective_start_date 
                     AND     pee.effective_end_date
  AND    pee.element_type_id    = pap.accrual_plan_element_type_id
  AND    pap.business_group_id  = paat.business_group_id
  AND    paat.absence_attendance_type_id = cp_absence_type_id
  AND    pap.pto_input_value_id = paat.input_value_id
  AND    cp_date_end BETWEEN paat.date_effective 
                     AND     nvl(paat.date_end,cp_date_end)
  ORDER BY pap.accrual_plan_name;

  c_method CONSTANT          varchar2(30) := 'show_dates_reasons_acr_bal';
  l_duration_type            varchar2(30);
  l_absence_days             per_absence_attendances.absence_days%TYPE;
  l_absence_hours            per_absence_attendances.absence_hours%TYPE;
  l_use_formula              number;
  l_min_max_failure          varchar2(4000);
  l_warning_or_error         varchar2(4000);
  l_page_error_msg           varchar2(4000);
  l_data                     xxas_emp_view_ui_pkg.xxas_data_table;
  l_business_group_id        per_absence_attendances.business_group_id%TYPE;
  l_person_id                per_absence_attendances.person_id%TYPE;
  l_assignment_id            per_all_assignments_f.assignment_id%TYPE;
  l_effective_date           date;
  l_absence_type_id          per_absence_attendances.absence_attendance_type_id%TYPE;
  l_date_start               per_absence_attendances.date_start%TYPE;
  l_date_end                 per_absence_attendances.date_end%TYPE;
  l_time_start               per_absence_attendances.time_start%TYPE;
  l_time_end                 per_absence_attendances.time_end%TYPE;
  l_xml_data                 XMLType;
  l_date_time_range_visible  varchar2(10);
  l_date_range_visible       varchar2(10);
  l_date_start_required      varchar2(10);
  l_date_time_range_enabled  varchar2(10);
  l_date_range_enabled       varchar2(10);
  l_duration_days_visible    varchar2(10);
  l_duration_hours_visible   varchar2(10);
  l_duration_days_enabled    varchar2(10);
  l_duration_hours_enabled   varchar2(10);
  l_values_xml               XMLType;
  l_date_time_range_xml      XMLType;
  l_date_range_xml           XMLType;
  l_employees_xml            XMLType;
  l_dtr_values_rec           csr_date_time_range_values%ROWTYPE;
  l_dr_values_rec            csr_date_range_values%ROWTYPE;
  l_abs_dur_auto_overwrite   varchar2(100);
  l_cal_duration             per_absence_attendances.absence_days%TYPE;
  l_person_id_field          xxas_emp_ui_hidden_type;
  l_asg_id_field             xxas_emp_ui_hidden_type;
  l_bg_id_field              xxas_emp_ui_hidden_type;
  l_dur_type_field           xxas_emp_ui_hidden_type;
  l_dur_days_field           xxas_emp_ui_number_type;
  l_dur_hours_field          xxas_emp_ui_number_type;
  l_dur_days_field2          xxas_emp_ui_number_type;
  l_dur_hours_field2         xxas_emp_ui_number_type;
  l_date_time_range_field    xxas_emp_ui_date_range_type;
  l_date_range_field         xxas_emp_ui_date_range_type;
  l_startdate_field          xxas_emp_ui_hidden_type;
  l_enddate_field            xxas_emp_ui_hidden_type;
  l_starttime_field          xxas_emp_ui_hidden_type;
  l_endtime_field            xxas_emp_ui_hidden_type;
  l_accrual_plan_id          pay_accrual_plans_v.accrual_plan_id%TYPE;
  l_proj_bal_container       xxas_emp_ui_container_type;
  l_effective_at_field       xxas_emp_ui_date_type;
  l_accr_plan_id_values_xml  XMLType;
  l_accrual_chart_parameters varchar2(4000);
  l_leave_balance_chart_xml  XMLType;
  l_accrual_plan_name        pay_accrual_plans_v.accrual_plan_name%TYPE;
  l_accrual_plan_uom         pay_accrual_plans_v.accrual_units_of_measure%TYPE;
  l_accrual_plan_uom_name    pay_accrual_plans_v.accrual_units_of_measure_name%TYPE;
  l_start_date               date;
  l_end_date                 date;
  l_gross_total              number;
  l_entitlement              number;
  l_carryover                number;
  l_ent_adjust               number;
  l_used_total               number;
  l_used                     number;
  l_pending                  number;
  l_used_adjust              number;
  l_remaining                number;
  l_html_text                varchar2(4000);
  l_html_field               xxas_emp_ui_html_type;
  l_ui                        xxas_emp_ui_view_type;
  l_function_id               fnd_form_functions.function_id%TYPE;
  l_absence_attendance_id     per_absence_attendances.absence_attendance_id%TYPE;
  l_emp_ui_fields             xxas_emp_ui_component_table := xxas_emp_ui_component_table();
  l_container                 xxas_emp_ui_container_type;
  l_container_grp_field_list  xxas_emp_ui_component_table;
  l_ff_dur_days_field         xxas_emp_ui_number_type;
  l_ff_dur_hours_field        xxas_emp_ui_number_type;
  l_ff_date_time_range_field  xxas_emp_ui_date_range_type;
  l_ff_date_range_field       xxas_emp_ui_date_range_type;
  l_ff_proj_bal_container     xxas_emp_ui_container_type;
  l_ff_effective_at_field     xxas_emp_ui_date_type;
  l_ff_accr_plan_id_field     xxas_emp_ui_list_type;
  l_ff_html_text_field        xxas_emp_ui_html_type;

BEGIN

  log_start(c_method);

  l_data := xxas_emp_view_ui_pkg.transform_jsonxml_form_data
              (p_request_xml);

  IF l_data.exists('BUSINESS_GROUP_ID') THEN
    l_business_group_id := l_data('BUSINESS_GROUP_ID');
  END IF;

  IF l_data.exists('PERSON_ID') THEN
    l_person_id := l_data('PERSON_ID');
  END IF;

  IF l_data.exists('ASSIGNMENT_ID') THEN
    l_assignment_id := l_data('ASSIGNMENT_ID');
  END IF;

  l_effective_date := trunc(sysdate);

  IF l_data.exists('ABSENCE_TYPE_ID') THEN
    l_absence_type_id := l_data('ABSENCE_TYPE_ID');
  END IF;

  IF l_data.exists('ABSENCE_ATTENDANCE_ID') THEN
    l_absence_attendance_id := l_data('ABSENCE_ATTENDANCE_ID');
  END IF;

  l_duration_type := xxas_leave_of_absence_pkg.get_absence_type_value(l_absence_type_id, 'HOURS_OR_DAYS');

  --
  -- Convert the XML form data into the xxas_data_table structure.
  --
  IF p_request_xml is not null THEN

    FOR csr_rec IN csr_data LOOP
      IF csr_rec.field_name is not null THEN
        IF csr_rec.field_name = 'DATE_TIME_RANGE' AND 
           (l_duration_type is not null AND
            l_duration_type IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_time_range_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'DATE_RANGE' AND
              (l_duration_type is not null AND
               l_duration_type NOT IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_range_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'EMPLOYEES' THEN
          l_employees_xml := csr_rec.field_xml;
        END IF;
      END IF;
    END LOOP;

  END IF;

  IF l_date_time_range_xml is not null THEN

    OPEN  csr_date_time_range_values(l_date_time_range_xml);
    FETCH csr_date_time_range_values INTO l_dtr_values_rec;
    CLOSE csr_date_time_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dtr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dtr_values_rec.end_date);
    l_time_start := l_dtr_values_rec.start_time;
    l_time_end   := l_dtr_values_rec.end_time;

  ELSIF l_date_range_xml is not null THEN

    OPEN  csr_date_range_values(l_date_range_xml);
    FETCH csr_date_range_values INTO l_dr_values_rec;
    CLOSE csr_date_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dr_values_rec.end_date);

  END IF;

  IF l_person_id is null THEN

    IF l_employees_xml is not null THEN

      OPEN  csr_employee_id(l_employees_xml);
      FETCH csr_employee_id INTO l_person_id;
      CLOSE csr_employee_id;

      l_assignment_id     := xxas_util_pkg.get_primary_worker_asg(l_person_id,l_effective_date);
      l_business_group_id := xxas_per_pkg.get_business_group_id(l_person_id);

    END IF;

  ELSE

    IF l_employees_xml is null THEN

      l_person_id         := null;
      l_assignment_id     := null;
      l_business_group_id := null;

    ELSE

      l_person_id         := null;
      l_assignment_id     := null;
      l_business_group_id := null;

      OPEN  csr_employee_id(l_employees_xml);
      FETCH csr_employee_id INTO l_person_id;
      CLOSE csr_employee_id;

      l_assignment_id     := xxas_util_pkg.get_primary_worker_asg(l_person_id,l_effective_date);
      l_business_group_id := xxas_per_pkg.get_business_group_id(l_person_id);
      
    END IF;

  END IF;

  IF l_absence_attendance_id is null THEN

    l_function_id := xxas_cal_util_pkg.page_slug_func_id('absence-new');

  ELSE

    l_function_id := xxas_cal_util_pkg.page_slug_func_id('absence-edit');

  END IF;

  l_ui := xxas_emp_view_ui_pkg.process_block_fast_formula
            (p_function_id       => l_function_id
            ,p_form_data_xml     => p_request_xml);

  IF l_ui.createfields IS NOT NULL 
  THEN
    l_emp_ui_fields := l_ui.createfields.field_list;
  END IF;

  IF l_ui.editfields IS NOT NULL 
  THEN
    l_emp_ui_fields := l_ui.editfields.field_list;
  END IF;

  FOR i IN 1..l_emp_ui_fields.COUNT
  LOOP

    log_statement(c_method,'field_id: ' || l_emp_ui_fields(i).get_id);

    IF l_emp_ui_fields(i) IS OF(xxas_emp_ui_container_type) THEN

      l_container := TREAT(l_emp_ui_fields(i) AS xxas_emp_ui_container_type);

      l_container_grp_field_list := l_container.field_list;

      IF l_container.get_id = 'BASIC_GRP' THEN

        FOR j IN 1..l_container_grp_field_list.COUNT
        LOOP

          log_statement(c_method,'BASIC_GRP - field_id: ' || l_container_grp_field_list(j).get_id);

          IF l_container_grp_field_list(j).get_id = 'DURATION_DAYS' THEN

            l_ff_dur_days_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_number_type);

          ELSIF l_container_grp_field_list(j).get_id = 'DURATION_HOURS' THEN

            l_ff_dur_hours_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_number_type);

          ELSIF l_container_grp_field_list(j).get_id = 'DATE_TIME_RANGE' THEN

            l_ff_date_time_range_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_date_range_type);

          ELSIF l_container_grp_field_list(j).get_id = 'DATE_RANGE' THEN

            l_ff_date_range_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_date_range_type);

          END IF;

        END LOOP;

      ELSIF l_container.get_id = 'PROJECTED_BALANCE' THEN

        l_ff_proj_bal_container := TREAT(l_container AS xxas_emp_ui_container_type);

        FOR j IN 1..l_container_grp_field_list.COUNT
        LOOP

          log_statement(c_method,'PROJECTED_BALANCE - field_id: ' || l_container_grp_field_list(j).get_id);

          IF l_container_grp_field_list(j).get_id = 'EFFECTIVE_AT' THEN

            l_ff_effective_at_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_date_type);

          ELSIF l_container_grp_field_list(j).get_id = 'ACCRUAL_PLAN_ID' THEN

            l_ff_accr_plan_id_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_list_type);

          ELSIF l_container_grp_field_list(j).get_id = 'HTML_TEXT' THEN

            l_ff_html_text_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_html_type);

          END IF;

        END LOOP;

      END IF;

    END IF;

    l_container        := null;

  END LOOP;

  IF l_duration_type is not null AND
     l_date_start    is not null AND
     l_date_end      is not null THEN

    l_cal_duration := xxas_leave_of_absence_pkg.calculate_abs_duration_full
                             (p_business_group_id => l_business_group_id
                             ,p_assignment_id     => l_assignment_id
                             ,p_effective_date    => l_effective_date
                             ,p_absence_type_id   => l_absence_type_id
                             ,p_date_start        => l_date_start
                             ,p_date_end          => l_date_end
                             ,p_time_start        => l_time_start
                             ,p_time_end          => l_time_end
                             ,p_absence_days      => l_absence_days
                             ,p_absence_hours     => l_absence_hours
                             ,p_use_formula       => l_use_formula
                             ,p_min_max_failure   => l_min_max_failure
                             ,p_warning_or_error  => l_warning_or_error
                             ,p_page_error_msg    => l_page_error_msg);

  END IF;

  IF l_duration_type is not null THEN

    l_abs_dur_auto_overwrite := fnd_profile.value('PER_ABSENCE_DURATION_AUTO_OVERWRITE');

    IF l_duration_type = 'H' THEN

      l_date_time_range_visible := 'true';
      l_date_start_required     := 'true';
      l_date_time_range_enabled := 'true';
      l_date_range_visible      := 'false';

      IF nvl(l_abs_dur_auto_overwrite,'N') = 'Y' THEN
        l_duration_hours_enabled := 'false';
      ELSE
        l_duration_hours_enabled := 'true';
      END IF;

      l_duration_hours_visible := 'true';
      l_duration_days_visible  := 'false';

    ELSIF l_duration_type = xxas_fast_formula_pkg.g_NULL_STRING THEN

      l_date_time_range_visible := 'true';
      l_date_start_required     := 'true';
      l_date_time_range_enabled := 'true';
      l_date_range_visible      := 'false';

      IF nvl(l_abs_dur_auto_overwrite,'N') = 'Y' THEN
        l_duration_days_enabled  := 'false';
        l_duration_hours_enabled := 'false';
      ELSE
        l_duration_days_enabled  := 'true';
        l_duration_hours_enabled := 'true';
      END IF;

      l_duration_days_visible  := 'true';
      l_duration_hours_visible := 'true';

    ELSE

      l_date_range_visible      := 'true';
      l_date_start_required     := 'true';
      l_date_range_enabled      := 'true';
      l_date_time_range_visible := 'false';

      IF nvl(l_abs_dur_auto_overwrite,'N') = 'Y' THEN
        l_duration_days_enabled  := 'false';
      ELSE
        l_duration_days_enabled  := 'true';
      END IF;

      l_duration_days_visible  := 'true';
      l_duration_hours_visible := 'false';

    END IF;

  END IF;

  OPEN  csr_absence_reasons(l_absence_type_id
                           ,l_person_id);
  FETCH csr_absence_reasons INTO l_values_xml;
  CLOSE csr_absence_reasons;

  IF l_ff_date_time_range_field is not null THEN

    l_date_time_range_field := l_ff_date_time_range_field;
    l_date_time_range_field.set_enabled(nvl(l_ff_date_time_range_field.get_enabled,l_date_time_range_enabled));
    l_date_time_range_field.set_visible(nvl(l_ff_date_time_range_field.get_visible,l_date_time_range_visible));
    l_date_time_range_field.set_start_date_required(nvl(l_ff_date_time_range_field.get_start_date_required,l_date_start_required));

  ELSE

    l_date_time_range_field := xxas_emp_ui_date_range_type
                                 (p_date_range_type         => 'DateTimeRange'
                                 ,p_id                      => 'DATE_TIME_RANGE'
                                 ,p_label                   => null
                                 ,p_description             => null
                                 ,p_refresh_on_change       => null
                                 ,p_trigger_name            => null
                                 ,p_trigger_method          => null
                                 ,p_enabled                 => l_date_time_range_enabled
                                 ,p_required                => null
                                 ,p_visible                 => l_date_time_range_visible
                                 ,p_hide_if                 => null
                                 ,p_value                   => (CASE WHEN l_time_start is not null AND 
                                                                          l_time_end   is not null
                                                                     THEN l_date_start || ' ' || l_time_start || '_' || l_date_end || ' ' || l_time_end
                                                                     WHEN l_time_start is not null
                                                                     THEN l_date_start || ' ' || l_time_start || '_' || l_date_end
                                                                     WHEN l_time_end is not null
                                                                     THEN l_date_start || '_' || l_date_end || ' ' || l_time_end
                                                                     WHEN l_time_start is null AND 
                                                                          l_time_end   is null
                                                                     THEN l_date_start || '_' || l_date_end
                                                                END)
                                 ,p_start_date_description  => null
                                 ,p_start_date_format       => null
                                 ,p_start_date_label        => null
                                 ,p_start_date_required     => l_date_start_required
                                 ,p_start_date_visible      => null
                                 ,p_end_date_description    => null
                                 ,p_end_date_format         => null
                                 ,p_end_date_label          => null
                                 ,p_end_date_required       => null
                                 ,p_end_date_visible        => null
                                 ,p_start_time_description  => null
                                 ,p_start_time_label        => null
                                 ,p_start_time_required     => null
                                 ,p_start_time_visible      => null
                                 ,p_end_time_description    => null
                                 ,p_end_time_label          => null
                                 ,p_end_time_required       => null
                                 ,p_end_time_visible        => null);

  END IF;

  IF l_ff_date_range_field is not null THEN

    l_date_range_field := l_ff_date_range_field;
    l_date_range_field.set_enabled(nvl(l_ff_date_range_field.get_enabled,l_date_range_enabled));
    l_date_range_field.set_visible(nvl(l_ff_date_range_field.get_visible,l_date_range_visible));
    l_date_range_field.set_start_date_required(nvl(l_ff_date_range_field.get_start_date_required,l_date_start_required));

  ELSE

    l_date_range_field := xxas_emp_ui_date_range_type
                            (p_date_range_type         => 'DateRange'
                            ,p_id                      => 'DATE_RANGE'
                            ,p_label                   => null
                            ,p_description             => null
                            ,p_refresh_on_change       => null
                            ,p_trigger_name            => null
                            ,p_trigger_method          => null
                            ,p_enabled                 => l_date_range_enabled
                            ,p_required                => null
                            ,p_visible                 => l_date_range_visible
                            ,p_hide_if                 => null
                            ,p_value                   => l_date_start || '_' || l_date_end
                            ,p_start_date_description  => null
                            ,p_start_date_format       => null
                            ,p_start_date_label        => null
                            ,p_start_date_required     => l_date_start_required
                            ,p_start_date_visible      => null
                            ,p_end_date_description    => null
                            ,p_end_date_format         => null
                            ,p_end_date_label          => null
                            ,p_end_date_required       => null
                            ,p_end_date_visible        => null
                            ,p_start_time_description  => null
                            ,p_start_time_label        => null
                            ,p_start_time_required     => null
                            ,p_start_time_visible      => null
                            ,p_end_time_description    => null
                            ,p_end_time_label          => null
                            ,p_end_time_required       => null
                            ,p_end_time_visible        => null);

  END IF;

     IF l_page_error_msg is null AND 
     l_date_end >= l_date_start THEN

    IF l_ff_dur_days_field is not null THEN

      l_dur_days_field := l_ff_dur_days_field;
      l_dur_days_field.set_enabled(nvl(l_ff_dur_days_field.get_enabled,l_duration_days_enabled));
      l_dur_days_field.set_visible(nvl(l_ff_dur_days_field.get_visible,l_duration_days_visible));
     l_dur_days_field.set_value(nvl(l_ff_dur_days_field.get_number_value,nvl(l_absence_days,0)));
      l_dur_days_field.set_decimal_precision(nvl(l_ff_dur_days_field.get_decimal_precision,4));

    ELSE

      l_dur_days_field := xxas_emp_ui_number_type
                            (p_id                => 'DURATION_DAYS'
                            ,p_label             => null
                            ,p_description       => null
                            ,p_refresh_on_change => null
                            ,p_trigger_name      => null
                            ,p_trigger_method    => null
                            ,p_enabled           => l_duration_days_enabled
                            ,p_required          => null
                            ,p_visible           => l_duration_days_visible
                            ,p_hide_if           => null
                            ,p_value             => nvl(l_absence_days,0)
                            ,p_decimal_precision => 4
                            ,p_regexp            => null
                            ,p_regexp_error      => null
                            ,p_transient         => null);

    END IF;

    IF l_ff_dur_hours_field is not null THEN

      l_dur_hours_field := l_ff_dur_hours_field;
      l_dur_hours_field.set_enabled(nvl(l_ff_dur_hours_field.get_enabled,l_duration_hours_enabled));
      l_dur_hours_field.set_visible(nvl(l_ff_dur_hours_field.get_visible,l_duration_hours_visible));
      l_dur_hours_field.set_value(nvl(l_ff_dur_hours_field.get_number_value,nvl(l_absence_hours,0)));
      l_dur_hours_field.set_decimal_precision(nvl(l_ff_dur_hours_field.get_decimal_precision,4));

    ELSE

      l_dur_hours_field := xxas_emp_ui_number_type
                             (p_id                => 'DURATION_HOURS'
                             ,p_label             => null
                             ,p_description       => null
                             ,p_refresh_on_change => null
                             ,p_trigger_name      => null
                             ,p_trigger_method    => null
                             ,p_enabled           => l_duration_hours_enabled
                             ,p_required          => null
                             ,p_visible           => l_duration_hours_visible
                             ,p_hide_if           => null
                             ,p_value             => nvl(l_absence_hours,0)
                             ,p_decimal_precision => 4
                             ,p_regexp            => null
                             ,p_regexp_error      => null
                             ,p_transient         => null);

    END IF;

  END IF;

  IF l_ff_dur_days_field is not null THEN

    l_dur_days_field2 := l_ff_dur_days_field;
    l_dur_days_field2.set_enabled(nvl(l_ff_dur_days_field.get_enabled,l_duration_days_enabled));
    l_dur_days_field2.set_visible(nvl(l_ff_dur_days_field.get_visible,l_duration_days_visible));
    l_dur_days_field2.set_decimal_precision(nvl(l_ff_dur_days_field.get_decimal_precision,4));

  ELSE

    l_dur_days_field2 := xxas_emp_ui_number_type
                           (p_id                => 'DURATION_DAYS'
                           ,p_label             => null
                           ,p_description       => null
                           ,p_refresh_on_change => null
                           ,p_trigger_name      => null
                           ,p_trigger_method    => null
                           ,p_enabled           => l_duration_days_enabled
                           ,p_required          => null
                           ,p_visible           => l_duration_days_visible
                           ,p_hide_if           => null
                           ,p_value             => null
                           ,p_decimal_precision => 4
                           ,p_regexp            => null
                           ,p_regexp_error      => null
                           ,p_transient         => null);

  END IF;

  IF l_ff_dur_hours_field is not null THEN

    l_dur_hours_field2 := l_ff_dur_hours_field;
    l_dur_hours_field2.set_enabled(nvl(l_ff_dur_hours_field.get_enabled,l_duration_hours_enabled));
    l_dur_hours_field2.set_visible(nvl(l_ff_dur_hours_field.get_visible,l_duration_hours_visible));
    l_dur_hours_field2.set_decimal_precision(nvl(l_ff_dur_hours_field.get_decimal_precision,4));

  ELSE

    l_dur_hours_field2 := xxas_emp_ui_number_type
                            (p_id                => 'DURATION_HOURS'
                            ,p_label             => null
                            ,p_description       => null
                            ,p_refresh_on_change => null
                            ,p_trigger_name      => null
                            ,p_trigger_method    => null
                            ,p_enabled           => l_duration_hours_enabled
                            ,p_required          => null
                            ,p_visible           => l_duration_hours_visible
                            ,p_hide_if           => null
                            ,p_value             => null
                            ,p_decimal_precision => 4
                            ,p_regexp            => null
                            ,p_regexp_error      => null
                            ,p_transient         => null);

  END IF;

  l_dur_type_field := xxas_emp_ui_hidden_type
                        (p_id        => 'DURATION_TYPE'
                        ,p_value     => l_duration_type
                        ,p_transient => null);

  l_person_id_field := xxas_emp_ui_hidden_type
                         (p_id        => 'PERSON_ID'
                         ,p_value     => l_person_id
                         ,p_transient => null);

  l_asg_id_field := xxas_emp_ui_hidden_type
                      (p_id        => 'ASSIGNMENT_ID'
                      ,p_value     => l_assignment_id
                      ,p_transient => null);

  l_bg_id_field := xxas_emp_ui_hidden_type
                      (p_id        => 'BUSINESS_GROUP_ID'
                      ,p_value     => l_business_group_id
                      ,p_transient => null);

  l_startdate_field := xxas_emp_ui_hidden_type
                         (p_id        => 'STARTDATE'
                         ,p_value     => to_char(l_date_start,g_HRAPI_DATE_FMT)
                         ,p_transient => null);

  l_enddate_field := xxas_emp_ui_hidden_type
                       (p_id        => 'ENDDATE'
                       ,p_value     => to_char(l_date_end,g_HRAPI_DATE_FMT)
                       ,p_transient => null);

  l_starttime_field := xxas_emp_ui_hidden_type
                         (p_id        => 'STARTTIME'
                         ,p_value     => l_time_start
                         ,p_transient => null);

  l_endtime_field := xxas_emp_ui_hidden_type
                       (p_id        => 'ENDTIME'
                       ,p_value     => l_time_end
                       ,p_transient => null);

  IF l_date_end        is not null AND 
     l_assignment_id   is not null AND 
     l_absence_type_id is not null THEN

    --
    -- Check if the absence type is linked to an accrual plan the person is enrolled onto as of the absence end date
    --
    OPEN  csr_accrual_plan_id(l_date_end
                             ,l_assignment_id
                             ,l_absence_type_id);
    FETCH csr_accrual_plan_id INTO l_accrual_plan_id;
    CLOSE csr_accrual_plan_id;

    IF l_accrual_plan_id is not null THEN

      IF l_ff_effective_at_field is not null THEN

        l_effective_at_field := l_ff_effective_at_field;
        l_effective_at_field.set_value(nvl(l_ff_effective_at_field.get_date_value('Y'),fnd_date.date_to_canonical(l_date_end)));

      ELSE
        --
        -- EFFECTIVE_AT Date field
        --
        l_effective_at_field := xxas_emp_ui_date_type
                                  (p_date_type         => 'Date'
                                  ,p_id                => 'EFFECTIVE_AT'
                                  ,p_label             => null
                                  ,p_description       => null
                                  ,p_refresh_on_change => null
                                  ,p_trigger_name      => null
                                  ,p_trigger_method    => null
                                  ,p_enabled           => null
                                  ,p_required          => null
                                  ,p_visible           => null
                                  ,p_hide_if           => null 
                                  ,p_value             => fnd_date.date_to_canonical(l_date_end)
                                  ,p_date_format       => null
                                  ,p_transient         => null
                                  ,p_post_format       => null);

      END IF;

      OPEN  csr_accrual_plan_id_values(l_date_end
                                      ,l_assignment_id
                                      ,l_absence_type_id);
      FETCH csr_accrual_plan_id_values INTO l_accr_plan_id_values_xml;
      CLOSE csr_accrual_plan_id_values;

      l_accrual_chart_parameters := 'pAssignmentId=' || to_char(l_assignment_id) || '&pEffectiveDate=' || to_char(l_date_end, 'YYYY-MM-DD') 
                                    || '&pAccrualPlanId=' || to_char(l_accrual_plan_id) || '&pProjectedAmount=' || to_char(nvl(l_absence_days,0));

      l_leave_balance_chart_xml := xxas_absence_management_pkg.leave_balance_chart(p_value => l_accrual_chart_parameters);

      xxas_absence_management_pkg.get_accrual_plan_details
        (p_person_id             => xxas_util_pkg.get_person_from_asg(l_assignment_id,l_date_end)
        ,p_assignment_id         => l_assignment_id
        ,p_effective_date        => l_date_end
        ,p_accrual_plan_id       => l_accrual_plan_id
        ,p_include_pending       => true --xxas_util_pkg.flag_to_boolean('Y')
        ,p_accrual_plan_name     => l_accrual_plan_name
        ,p_accrual_plan_uom      => l_accrual_plan_uom
        ,p_accrual_plan_uom_name => l_accrual_plan_uom_name
        ,p_start_date            => l_start_date
        ,p_end_date              => l_end_date
        ,p_gross_total           => l_gross_total
        ,p_entitlement           => l_entitlement
        ,p_carryover             => l_carryover
        ,p_entitlement_adjust    => l_ent_adjust
        ,p_used_total            => l_used_total
        ,p_used                  => l_used
        ,p_pending               => l_pending
        ,p_used_adjust           => l_used_adjust
        ,p_remaining             => l_remaining);

      --
      -- Accrual plan Balance is less than zero
      --              
      IF nvl(l_remaining,0) < 0 THEN

        l_html_text := xxas_com_msg_pkg.get_decoded_string
                         (xxas_com_msg_type
                            (xxas_util_pkg.app
                            ,'XXAS_ACC_REM_BAL_LES_THAN_ZERO'
                            ,xxas_com_msg_token_varray
                               (xxas_com_msg_token_type
                                  ('DATE'
                                  ,to_char(l_date_end, 'FMDd Mon YYYY'))
                               )
                            )
                         );

        IF l_html_text is not null THEN

          l_html_text := '<font color="' 
                         || xxas_rslt_cache_pkg.resolve_color_name('red')
                         || '">' || l_html_text || '</font>';
        END IF;

        IF l_ff_html_text_field is not null THEN

          l_html_field := l_ff_html_text_field;
          l_html_field.set_visible(nvl(l_ff_html_text_field.get_visible,'true'));
          l_html_field.set_value(nvl(l_ff_html_text_field.get_value,l_html_text));

        ELSE

          l_html_field := xxas_emp_ui_html_type
                            (p_id                => 'HTML_TEXT'
                            ,p_label             => null
                            ,p_description       => null  
                            ,p_refresh_on_change => null
                            ,p_trigger_name      => null
                            ,p_trigger_method    => null
                            ,p_enabled           => null
                            ,p_required          => null
                            ,p_visible           => 'true'
                            ,p_hide_if           => null
                            ,p_value             => l_html_text
                            ,p_transient         => null);

        END IF;

      END IF;

    ELSE

      --
      -- PROJECTED_BALANCE Container make it hidden
      --
      l_proj_bal_container := xxas_emp_ui_container_type
                                (p_id             => 'PROJECTED_BALANCE'
                                ,p_label          => null
                                ,p_description    => null
                                ,p_visible        => 'false'
                                ,p_hide_if        => null
                                ,p_review_only    => null
                                ,p_type           => null
                                ,p_trigger_name   => null
                                ,p_trigger_method => null);

    END IF;

  ELSE

    --
    -- PROJECTED_BALANCE Container make it hidden
    --
    l_proj_bal_container := xxas_emp_ui_container_type
                              (p_id             => 'PROJECTED_BALANCE'
                              ,p_label          => null
                              ,p_description    => null
                              ,p_visible        => 'false'
                              ,p_hide_if        => null
                              ,p_review_only    => null
                              ,p_type           => null
                              ,p_trigger_name   => null
                              ,p_trigger_method => null);

  END IF;
  
  SELECT XMLElement
          ("data"
          /*,XMLAttributes
            ('http://json.org/' as "xmlns:json"
            ,'true' as "json:force-array")*/
          ,XMLElement
          ("form"
          ,XMLElement
            ("fields"
            ,XMLAttributes
              ('http://json.org/' as "xmlns:json"
              ,'true' as "json:force-array")
            ,XMLElement
              ("container"
              ,XMLForest
                ('BASIC_GRP' as "id"
                )
              ,XMLElement
                ("fields"
                ,XMLAttributes
                  ('http://json.org/' as "xmlns:json"
                  ,'true' as "json:force-array")
                ,l_startdate_field.get_json_friendly()
                ,l_enddate_field.get_json_friendly()
                ,l_starttime_field.get_json_friendly()
                ,l_endtime_field.get_json_friendly()
                ,XMLElement
                  ("field"
                  ,XMLForest
                    ('ABS_ATTENDANCE_REASON_ID' as "id"
                    )
                  ,l_values_xml
                  )
                ,CASE WHEN (l_duration_type is not null AND
                            l_duration_type IN ('H',xxas_fast_formula_pkg.g_NULL_STRING))
                      THEN l_date_time_range_field.get_json_friendly()
                      WHEN (l_duration_type is not null AND
                            l_duration_type NOT IN ('H',xxas_fast_formula_pkg.g_NULL_STRING))
                      THEN l_date_range_field.get_json_friendly()
                      ELSE null
                 END
                ,CASE WHEN (l_duration_type is not null AND
                            l_duration_type IN ('H',xxas_fast_formula_pkg.g_NULL_STRING))
                      THEN l_date_range_field.get_json_friendly()
                      WHEN (l_duration_type is not null AND
                            l_duration_type NOT IN ('H',xxas_fast_formula_pkg.g_NULL_STRING))
                      THEN l_date_time_range_field.get_json_friendly()
                      ELSE null
                 END
                ,l_dur_type_field.get_json_friendly()
                ,CASE WHEN l_duration_type is not null AND
                           (l_duration_type NOT IN ('H',xxas_fast_formula_pkg.g_NULL_STRING) OR 
                            l_duration_type = xxas_fast_formula_pkg.g_NULL_STRING) AND
                           (l_page_error_msg is null AND 
                                  l_date_end >= l_date_start)
                      THEN l_dur_days_field.get_json_friendly()
                      ELSE l_dur_days_field2.get_json_friendly()
                 END
                ,CASE WHEN (l_duration_type is not null AND
                            l_duration_type IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) AND
                           (l_page_error_msg is null AND 
                             l_date_end >= l_date_start)
                      THEN l_dur_hours_field.get_json_friendly()
                      ELSE l_dur_hours_field2.get_json_friendly()
                 END
                ,l_person_id_field.get_json_friendly()
                ,l_asg_id_field.get_json_friendly()
                ,l_bg_id_field.get_json_friendly()
                )
              )
            ,(CASE WHEN l_accrual_plan_id is not null
                   THEN XMLElement
                         ("container"
                         ,XMLForest
                           ('PROJECTED_BALANCE' as "id"
                           ,(CASE WHEN l_ff_proj_bal_container is not null 
                                  THEN nvl(l_ff_proj_bal_container.get_state,'input')
                                  ELSE 'input'
                             END)               as "state"
                           ,(CASE WHEN l_ff_proj_bal_container is not null 
                                  THEN nvl(l_ff_proj_bal_container.get_visible,'true')
                                  ELSE 'true'
                             END)               as "visible"
                           ,'fieldset'          as "type"
                           )
                         ,XMLElement
                           ("fields"
                           ,XMLAttributes
                             ('http://json.org/' as "xmlns:json"
                             ,'true' as "json:force-array")
                           ,l_effective_at_field.get_json_friendly()
                           ,XMLElement
                             ("field"
                             ,XMLForest
                               ('ACCRUAL_PLAN_ID' as "id"
                               ,l_accrual_plan_id as "value"
                               )
                             ,l_accr_plan_id_values_xml
                             )
                           ,(CASE WHEN l_leave_balance_chart_xml is not null
                                  THEN XMLElement
                                        ("field"
                                        ,XMLForest
                                          ('ACCRUAL_CHART' as "id"
                                          ,'item'          as "type"
                                          ,'view'          as "state"
                                          )
                                        ,l_leave_balance_chart_xml
                                        ,XMLElement
                                          ("meta"
                                          ,XMLElement
                                            ("options"
                                            ,XMLForest
                                              ('detail' AS "style"
                                              )
                                            )
                                          )
                                        )
                                  ELSE null
                             END)
                           ,(CASE WHEN l_html_text is not null
                                  THEN l_html_field.get_json_friendly()
                                  ELSE null
                             END)
                           )
                         )
                   ELSE l_proj_bal_container.get_json_friendly()
              END)
            )
          )
         ,(CASE WHEN l_date_end < l_date_start
                THEN xxas_com_msg_pkg.get_error_message_xml
                       (p_message    => xxas_com_msg_type(xxas_util_pkg.app, 'XXAS_EMP_ABS_ED', null)
                       ,p_redirect   => null
                       ,p_target     => null
                       ).extract('data/*')
                WHEN l_page_error_msg is not null
                THEN xxas_com_msg_pkg.get_error_message_xml
                       (p_message    => xxas_com_msg_type
                                          (xxas_util_pkg.app
                                          ,'XXAS_COM_TEXT'
                                          ,xxas_com_msg_token_varray
                                             (xxas_com_msg_token_type
                                                ('TEXT'
                                                ,l_page_error_msg)))
                       ,p_redirect   => null
                       ,p_target     => null
                       ).extract('data/*')
                ELSE null
           END
           ))
  INTO   l_xml_data
  FROM   dual;

  log_end(c_method);

  RETURN l_xml_data;

END show_dates_reasons_acr_bal;

--
-- ------------------------------------------------------------------------ *
-- Name    : project_bal_accr_plan_handler
-- ------------------------------------------------------------------------ *
FUNCTION project_bal_accr_plan_handler
  (p_request_xml IN XMLType) RETURN XMLType
IS

  CURSOR csr_data
  IS
  SELECT r.nam field_name
        ,nvl(r.val1, nvl(r.val2, r.val3)) val
        ,r.column_value field_xml
  FROM (SELECT x.column_value.getRootElement() nam
              ,extract(x.column_value, '/*/text()').getCLOBVal() val1
              ,extract(x.column_value, '/*/*/text()').getCLOBVal() val2
              ,extract(x.column_value, '/*/*/id/text()').getCLOBVal() val3
              ,x.column_value
        FROM  TABLE(XMLSequence
                     (extract
                       (p_request_xml
                       ,'//*'
                     || decode('Y'
                              ,'Y', '[not(ancestor-or-self::rows) '
                                  , '[not(self::rows) ')
                     || ' and not(self::root)
                          and not(self::postType)
                          and not(self::submittedFieldId)
                          and not(self::value)
                          and not(self::id)
                          and not(self::label)
                          and not(self::description)]'))) x) r;

  CURSOR csr_date_time_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dtr.start_date,'"') start_date
        ,replace(dtr.end_date,'"') end_date
        ,replace(dtr.start_time,'"') start_time
        ,replace(dtr.end_time,'"') end_time
  FROM   XMLTable
          ('//DATE_TIME_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
                  ,start_time      varchar2(10)     path '//startTime'
                  ,end_time        varchar2(10)     path '//endTime'
          ) dtr;

  CURSOR csr_date_range_values(p_xml IN XMLType)
  IS
  SELECT replace(dr.start_date,'"') start_date
        ,replace(dr.end_date,'"') end_date
  FROM   XMLTable
          ('//DATE_RANGE/value'
          PASSING  p_xml
          COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path '//startDate'
                  ,end_date        varchar2(30)     path '//endDate'
          ) dr;

  CURSOR csr_accrual_plan_id(cp_date_end          IN date
                            ,cp_assignment_id     IN number
                            ,cp_absence_type_id   IN number) IS
  SELECT pap.accrual_plan_id
  FROM   pay_element_entries_f        pee
        ,pay_accrual_plans_v            pap
        ,per_absence_attendance_types paat
  WHERE  pee.assignment_id      = cp_assignment_id
  AND    cp_date_end BETWEEN pee.effective_start_date
                     AND     nvl(pee.effective_end_date,cp_date_end)
  AND    pee.element_type_id = pap.accrual_plan_element_type_id
  AND    pap.business_group_id  = paat.business_group_id
  AND    paat.absence_attendance_type_id = cp_absence_type_id
  AND    pap.pto_input_value_id = paat.input_value_id
  AND    cp_date_end BETWEEN paat.date_effective 
                     AND     nvl(paat.date_end,cp_date_end)
  ORDER BY pap.accrual_plan_name;

  CURSOR csr_accrual_plan_id_values(cp_date_end          IN date
                                   ,cp_assignment_id     IN number
                                   ,cp_absence_type_id   IN number) 
  IS
  SELECT XMLElement
          ("values"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("plan"
              ,XMLForest
                (pap.accrual_plan_id "id"
                ,pap.accrual_plan_name "value"
                ,pap.accrual_plan_name "label")
                )
              )
          )
  FROM   per_all_assignments_f        rslt
        ,pay_element_entries_f        pee
        ,pay_accrual_plans_v          pap
        ,per_absence_attendance_types paat
  WHERE  rslt.assignment_id     = cp_assignment_id
  AND    rslt.assignment_id     = pee.assignment_id
  AND    cp_date_end BETWEEN rslt.effective_start_date 
                     AND     rslt.effective_end_date
  AND    cp_date_end BETWEEN pee.effective_start_date 
                     AND     pee.effective_end_date
  AND    pee.element_type_id    = pap.accrual_plan_element_type_id
  AND    pap.business_group_id  = paat.business_group_id
  AND    paat.absence_attendance_type_id = cp_absence_type_id
  AND    pap.pto_input_value_id = paat.input_value_id
  AND    cp_date_end BETWEEN paat.date_effective 
                     AND     nvl(paat.date_end,cp_date_end)
  ORDER BY pap.accrual_plan_name;

  c_method CONSTANT           varchar2(30) := 'project_bal_accr_plan_handler';
  l_duration_type             varchar(30);
  l_data                      xxas_emp_view_ui_pkg.xxas_data_table;
  l_business_group_id         per_absence_attendances.business_group_id%TYPE;
  l_person_id                 per_absence_attendances.person_id%TYPE;
  l_assignment_id             per_all_assignments_f.assignment_id%TYPE;
  l_absence_type_id           per_absence_attendances.absence_attendance_type_id%TYPE;
  l_date_start                per_absence_attendances.date_start%TYPE;
  l_date_end                  per_absence_attendances.date_end%TYPE;
  l_time_start                per_absence_attendances.time_start%TYPE;
  l_time_end                  per_absence_attendances.time_end%TYPE;
  l_absence_days              per_absence_attendances.absence_days%TYPE;
  l_effective_at              date;
  l_xml_data                  XMLType;
  l_date_time_range_xml       XMLType;
  l_date_range_xml            XMLType;
  l_dtr_values_rec            csr_date_time_range_values%ROWTYPE;
  l_dr_values_rec             csr_date_range_values%ROWTYPE;
  l_accrual_plan_id           pay_accrual_plans_v.accrual_plan_id%TYPE;
  l_proj_bal_container        xxas_emp_ui_container_type;
  l_effective_at_field        xxas_emp_ui_date_type;
  l_accr_plan_id_values_xml   XMLType;
  l_accrual_chart_parameters  varchar2(4000);
  l_leave_balance_chart_xml   XMLType;
  l_accrual_plan_name         pay_accrual_plans_v.accrual_plan_name%TYPE;
  l_accrual_plan_uom          pay_accrual_plans_v.accrual_units_of_measure%TYPE;
  l_accrual_plan_uom_name     pay_accrual_plans_v.accrual_units_of_measure_name%TYPE;
  l_start_date                date;
  l_end_date                  date;
  l_gross_total               number;
  l_entitlement               number;
  l_carryover                 number;
  l_ent_adjust                number;
  l_used_total                number;
  l_used                      number;
  l_pending                   number;
  l_used_adjust               number;
  l_remaining                 number;
  l_html_text                 varchar2(4000);
  l_html_field                xxas_emp_ui_html_type;
  l_ui                        xxas_emp_ui_view_type;
  l_function_id               fnd_form_functions.function_id%TYPE;
  l_absence_attendance_id     per_absence_attendances.absence_attendance_id%TYPE;
  l_emp_ui_fields             xxas_emp_ui_component_table := xxas_emp_ui_component_table();
  l_container                 xxas_emp_ui_container_type;
  l_container_grp_field_list  xxas_emp_ui_component_table;
  l_ff_proj_bal_container     xxas_emp_ui_container_type;
  l_ff_effective_at_field     xxas_emp_ui_date_type;
  l_ff_accr_plan_id_field     xxas_emp_ui_list_type;
  l_ff_html_text_field        xxas_emp_ui_html_type;

BEGIN

  log_start(c_method);

  l_data := xxas_emp_view_ui_pkg.transform_jsonxml_form_data
              (p_request_xml);

  IF l_data.exists('BUSINESS_GROUP_ID') THEN
    l_business_group_id := l_data('BUSINESS_GROUP_ID');
  END IF;

  IF l_data.exists('PERSON_ID') THEN
    l_person_id := l_data('PERSON_ID');
  END IF;

  IF l_data.exists('ASSIGNMENT_ID') THEN
    l_assignment_id := l_data('ASSIGNMENT_ID');
  END IF;

  IF l_data.exists('EFFECTIVE_AT') THEN
    l_effective_at := fnd_date.canonical_to_date(l_data('EFFECTIVE_AT'));
  END IF;

  IF l_data.exists('ABSENCE_TYPE_ID') THEN
    l_absence_type_id := l_data('ABSENCE_TYPE_ID');
  END IF;

  IF l_data.exists('DURATION_DAYS') THEN
    l_absence_days := l_data('DURATION_DAYS');
  END IF;

  IF l_data.exists('ACCRUAL_PLAN_ID') THEN
    l_accrual_plan_id := l_data('ACCRUAL_PLAN_ID');
  END IF;

  IF l_data.exists('ABSENCE_ATTENDANCE_ID') THEN
    l_absence_attendance_id := l_data('ABSENCE_ATTENDANCE_ID');
  END IF;

  l_duration_type := xxas_leave_of_absence_pkg.get_absence_type_value(l_absence_type_id, 'HOURS_OR_DAYS');

  --
  -- Convert the XML form data into the xxas_data_table structure.
  --
  IF p_request_xml is not null THEN

    FOR csr_rec IN csr_data LOOP
      IF csr_rec.field_name is not null THEN
        IF csr_rec.field_name = 'DATE_TIME_RANGE' AND 
           (l_duration_type is not null AND
            l_duration_type IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_time_range_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'DATE_RANGE' AND
              (l_duration_type is not null AND
               l_duration_type NOT IN ('H',xxas_fast_formula_pkg.g_NULL_STRING)) THEN
          l_date_range_xml := csr_rec.field_xml;
        END IF;
      END IF;
    END LOOP;

  END IF;

  IF l_date_time_range_xml is not null THEN

    OPEN  csr_date_time_range_values(l_date_time_range_xml);
    FETCH csr_date_time_range_values INTO l_dtr_values_rec;
    CLOSE csr_date_time_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dtr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dtr_values_rec.end_date);
    l_time_start := l_dtr_values_rec.start_time;
    l_time_end   := l_dtr_values_rec.end_time;

  ELSIF l_date_range_xml is not null THEN

    OPEN  csr_date_range_values(l_date_range_xml);
    FETCH csr_date_range_values INTO l_dr_values_rec;
    CLOSE csr_date_range_values;

    l_date_start := fnd_date.canonical_to_date(l_dr_values_rec.start_date);
    l_date_end   := fnd_date.canonical_to_date(l_dr_values_rec.end_date);

  END IF;

  IF l_accrual_plan_id is null AND
     l_effective_at    is not null AND 
     l_assignment_id   is not null AND 
     l_absence_type_id is not null THEN

    --
    -- Check if the absence type is linked to an accrual plan the person is enrolled onto as of the absence end date
    --
    OPEN  csr_accrual_plan_id(l_effective_at
                             ,l_assignment_id
                             ,l_absence_type_id);
    FETCH csr_accrual_plan_id INTO l_accrual_plan_id;
    CLOSE csr_accrual_plan_id;

  END IF;

  IF l_absence_attendance_id is null THEN

    l_function_id := xxas_cal_util_pkg.page_slug_func_id('absence-new');

  ELSE

    l_function_id := xxas_cal_util_pkg.page_slug_func_id('absence-edit');

  END IF;

  l_ui := xxas_emp_view_ui_pkg.process_block_fast_formula
            (p_function_id       => l_function_id
            ,p_form_data_xml     => p_request_xml);

  IF l_ui.createfields IS NOT NULL 
  THEN
    l_emp_ui_fields := l_ui.createfields.field_list;
  END IF;

  IF l_ui.editfields IS NOT NULL 
  THEN
    l_emp_ui_fields := l_ui.editfields.field_list;
  END IF;

  FOR i IN 1..l_emp_ui_fields.COUNT
  LOOP

    log_statement(c_method,'field_id: ' || l_emp_ui_fields(i).get_id);

    IF l_emp_ui_fields(i) IS OF(xxas_emp_ui_container_type) THEN

      l_container := TREAT(l_emp_ui_fields(i) AS xxas_emp_ui_container_type);

      l_container_grp_field_list := l_container.field_list;

      IF l_container.get_id = 'PROJECTED_BALANCE' THEN

        l_ff_proj_bal_container := TREAT(l_container AS xxas_emp_ui_container_type);

        FOR j IN 1..l_container_grp_field_list.COUNT
        LOOP

          log_statement(c_method,'PROJECTED_BALANCE - field_id: ' || l_container_grp_field_list(j).get_id);

          IF l_container_grp_field_list(j).get_id = 'EFFECTIVE_AT' THEN

            l_ff_effective_at_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_date_type);

          ELSIF l_container_grp_field_list(j).get_id = 'ACCRUAL_PLAN_ID' THEN

            l_ff_accr_plan_id_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_list_type);

          ELSIF l_container_grp_field_list(j).get_id = 'HTML_TEXT' THEN

            l_ff_html_text_field := TREAT(l_container_grp_field_list(j) AS xxas_emp_ui_html_type);

          END IF;

        END LOOP;

      END IF;

    END IF;

    l_container        := null;

  END LOOP;

  IF l_accrual_plan_id is not null THEN

    IF l_ff_effective_at_field is not null THEN

      l_effective_at_field := l_ff_effective_at_field;
      l_effective_at_field.set_value(fnd_date.date_to_canonical(l_effective_at));

    ELSE
      --
      -- EFFECTIVE_AT Date field
      --
      l_effective_at_field := xxas_emp_ui_date_type
                                (p_date_type         => 'Date'
                                ,p_id                => 'EFFECTIVE_AT'
                                ,p_label             => null
                                ,p_description       => null
                                ,p_refresh_on_change => null
                                ,p_trigger_name      => null
                                ,p_trigger_method    => null
                                ,p_enabled           => null
                                ,p_required          => null
                                ,p_visible           => null
                                ,p_hide_if           => null 
                                ,p_value             => fnd_date.date_to_canonical(l_effective_at)
                                ,p_date_format       => null
                                ,p_transient         => null
                                ,p_post_format       => null);

    END IF;

    OPEN  csr_accrual_plan_id_values(l_effective_at
                                    ,l_assignment_id
                                    ,l_absence_type_id);
    FETCH csr_accrual_plan_id_values INTO l_accr_plan_id_values_xml;
    CLOSE csr_accrual_plan_id_values;

    l_accrual_chart_parameters := 'pAssignmentId=' || to_char(l_assignment_id) || '&pEffectiveDate=' || to_char(l_effective_at, 'YYYY-MM-DD') 
                                  || '&pAccrualPlanId=' || to_char(l_accrual_plan_id) || '&pProjectedAmount=' || to_char(nvl(l_absence_days,0));

    l_leave_balance_chart_xml := xxas_absence_management_pkg.leave_balance_chart(p_value => l_accrual_chart_parameters);

    xxas_absence_management_pkg.get_accrual_plan_details
      (p_person_id             => xxas_util_pkg.get_person_from_asg(l_assignment_id,l_effective_at)
      ,p_assignment_id         => l_assignment_id
      ,p_effective_date        => l_effective_at
      ,p_accrual_plan_id       => l_accrual_plan_id
      ,p_include_pending       => true --xxas_util_pkg.flag_to_boolean('Y')
      ,p_accrual_plan_name     => l_accrual_plan_name
      ,p_accrual_plan_uom      => l_accrual_plan_uom
      ,p_accrual_plan_uom_name => l_accrual_plan_uom_name
      ,p_start_date            => l_start_date
      ,p_end_date              => l_end_date
      ,p_gross_total           => l_gross_total
      ,p_entitlement           => l_entitlement
      ,p_carryover             => l_carryover
      ,p_entitlement_adjust    => l_ent_adjust
      ,p_used_total            => l_used_total
      ,p_used                  => l_used
      ,p_pending               => l_pending
      ,p_used_adjust           => l_used_adjust
      ,p_remaining             => l_remaining);

    --
    -- Accrual plan Balance is less than zero
    --            
    IF nvl(l_remaining,0) < 0 THEN

      l_html_text := xxas_com_msg_pkg.get_decoded_string
                       (xxas_com_msg_type
                          (xxas_util_pkg.app
                          ,'XXAS_ACC_REM_BAL_LES_THAN_ZERO'
                          ,xxas_com_msg_token_varray
                             (xxas_com_msg_token_type
                                ('DATE'
                                ,to_char(l_effective_at, 'FMDd Mon YYYY'))
                             )
                          )
                       );

        IF l_html_text is not null THEN

          l_html_text := '<font color="' 
                         || xxas_rslt_cache_pkg.resolve_color_name('red')
                         || '">' || l_html_text || '</font>';
        END IF;

        IF l_ff_html_text_field is not null THEN

          l_html_field := l_ff_html_text_field;
          l_html_field.set_visible(nvl(l_ff_html_text_field.get_visible,'true'));
          l_html_field.set_value(nvl(l_ff_html_text_field.get_value,l_html_text));

        ELSE

          l_html_field := xxas_emp_ui_html_type
                            (p_id                => 'HTML_TEXT'
                            ,p_label             => null
                            ,p_description       => null  
                            ,p_refresh_on_change => null
                            ,p_trigger_name      => null
                            ,p_trigger_method    => null
                            ,p_enabled           => null
                            ,p_required          => null
                            ,p_visible           => 'true'
                            ,p_hide_if           => null
                            ,p_value             => l_html_text
                            ,p_transient         => null);

        END IF;

    END IF;

  ELSE

    --
    -- PROJECTED_BALANCE Container make it hidden
    --
    l_proj_bal_container := xxas_emp_ui_container_type
                              (p_id             => 'PROJECTED_BALANCE'
                              ,p_label          => null
                              ,p_description    => null
                              ,p_visible        => 'false'
                              ,p_hide_if        => null
                              ,p_review_only    => null
                              ,p_type           => null
                              ,p_trigger_name   => null
                              ,p_trigger_method => null);

  END IF;

  SELECT XMLElement
          ("data"
          ,XMLElement
            ("form"
            ,XMLElement
              ("fields"
              ,XMLAttributes
                ('http://json.org/' as "xmlns:json"
                ,'true' as "json:force-array")
              ,(CASE WHEN l_accrual_plan_id is not null
                     THEN XMLElement
                           ("container"
                           ,XMLForest
                             ('PROJECTED_BALANCE' as "id"
                           ,(CASE WHEN l_ff_proj_bal_container is not null 
                                  THEN nvl(l_ff_proj_bal_container.get_state,'input')
                                  ELSE 'input'
                             END)                 as "state"
                           ,(CASE WHEN l_ff_proj_bal_container is not null 
                                  THEN nvl(l_ff_proj_bal_container.get_visible,'true')
                                  ELSE 'true'
                             END)                 as "visible"
                             ,'fieldset'          as "type"
                             )
                           ,XMLElement
                             ("fields"
                             ,XMLAttributes
                               ('http://json.org/' as "xmlns:json"
                               ,'true' as "json:force-array")
                             ,l_effective_at_field.get_json_friendly()
                             ,XMLElement
                               ("field"
                               ,XMLForest
                                 ('ACCRUAL_PLAN_ID' as "id"
                                 ,l_accrual_plan_id as "value"
                                 )
                               ,l_accr_plan_id_values_xml
                               )
                             ,(CASE WHEN l_leave_balance_chart_xml is not null
                                    THEN XMLElement
                                          ("field"
                                          ,XMLForest
                                            ('ACCRUAL_CHART' as "id"
                                            ,'item'          as "type"
                                            ,'view'          as "state"
                                            )
                                          ,l_leave_balance_chart_xml
                                          ,XMLElement
                                            ("meta"
                                            ,XMLElement
                                              ("options"
                                              ,XMLForest
                                                ('detail' AS "style"
                                                )
                                              )
                                            )
                                          )
                                    ELSE null
                               END)
                             ,(CASE WHEN l_html_text is not null
                                    THEN l_html_field.get_json_friendly()
                                    ELSE null
                               END)
                             )
                           )
                     ELSE l_proj_bal_container.get_json_friendly()
                END)
            )
          ))
  INTO   l_xml_data
  FROM   dual;

  log_end(c_method);

  RETURN l_xml_data;

END project_bal_accr_plan_handler;

--
-- ------------------------------------------------------------------------ *
-- Name    : entry_form
-- ------------------------------------------------------------------------ *
FUNCTION entry_form
  (p_person_id       IN number
  ,p_start_time      IN varchar2
  ,p_end_time        IN varchar2
  ,p_source_block    IN varchar2
  ,p_saved_search_id IN number
  ,p_manager_id      IN number
  ,p_def_abs_type_flag   IN varchar2) RETURN XMLType
IS

  c_method CONSTANT  varchar2(30) := 'entry_form';
  l_cal_event        xxas_cal_event;
  l_form_data_xml    XMLType;

BEGIN

  log_start(c_method,p_person_id,p_start_time,p_end_time,p_source_block,p_saved_search_id,p_manager_id,p_def_abs_type_flag);

  xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

l_cal_event := xxas_cal_event
                   (p_person_id             => p_person_id
                   ,p_resource_id           => nvl(xxas_util_pkg.get_primary_worker_asg(p_person_id,trunc(SYSDATE))
                                                  ,xxas_util_pkg.get_primary_worker_asg(p_person_id,fnd_date.canonical_to_date(p_start_time)))
                   ,p_start_time            => fnd_date.canonical_to_date(p_start_time)
                   ,p_end_time              => fnd_date.canonical_to_date(p_end_time)
                   ,p_action_id             => g_ABS_ACTION_ID
                   ,p_source_block          => p_source_block
                   ,p_saved_search_id       => p_saved_search_id
                   ,p_manager_id            => p_manager_id
                   ,p_def_abs_type_flag     => p_def_abs_type_flag);

  l_form_data_xml := l_cal_event.get_xml();

  RETURN l_form_data_xml;

END entry_form;

--
-- ------------------------------------------------------------------------ *
-- Name    : entry_view
-- ------------------------------------------------------------------------ *
FUNCTION entry_view
  (p_person_id           IN number
  ,p_abs_attendance_id   IN number
  ,p_trans_and_step_id   IN varchar2
  ,p_source_block        IN varchar2
  ,p_saved_search_id     IN number
  ,p_manager_id          IN number) RETURN XMLType
IS

  c_method CONSTANT     varchar2(30) := 'entry_view';
  l_cal_event_view      xxas_cal_event_view;
  l_view_data_xml       XMLType;
  l_varchar2tbl         xxas_com_varchar2_table;
  l_transaction_id      number;
  l_transaction_step_id number;
  l_keys                xxas_com_keys_type;

BEGIN

  log_start(c_method,p_person_id,p_abs_attendance_id,p_trans_and_step_id,p_source_block,p_saved_search_id,p_manager_id);

  xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

  IF p_trans_and_step_id is not null THEN
  
    l_keys := xxas_emp_menu_pkg.object_id_to_keys
                (p_object_id        => p_trans_and_step_id
                ,p_object_id_format => '{TRANSACTION_ID}_{TRANSACTION_STEP_ID}');

    l_transaction_id      := to_number(l_keys.get_key('TRANSACTION_ID'));
    l_transaction_step_id := to_number(l_keys.get_key('TRANSACTION_STEP_ID'));

  END IF;

  l_cal_event_view := xxas_cal_event_view
                        (p_person_id           => p_person_id
                        ,p_resource_id         => xxas_util_pkg.get_primary_worker_asg(p_person_id,trunc(sysdate))
                        ,p_abs_attendance_id   => p_abs_attendance_id
                        ,p_log_entry_id        => null
                        ,p_transaction_id      => l_transaction_id
                        ,p_transaction_step_id => l_transaction_step_id
                        ,p_action_id           => g_ABS_ACTION_ID
                        ,p_source_block        => p_source_block
                        ,p_saved_search_id     => p_saved_search_id
                        ,p_manager_id          => p_manager_id);

  l_view_data_xml := l_cal_event_view.get_xml();

  RETURN l_view_data_xml;

END entry_view;

--
-- ------------------------------------------------------------------------ *
-- Name    : entry_edit
-- ------------------------------------------------------------------------ *
FUNCTION entry_edit
  (p_person_id           IN number
  ,p_abs_attendance_id   IN number
  ,p_trans_and_step_id   IN varchar2
  ,p_times               IN varchar2
  ,p_source_block        IN varchar2
  ,p_saved_search_id     IN number
  ,p_manager_id          IN number) RETURN XMLType
IS

  c_method CONSTANT     varchar2(30) := 'entry_edit';
  l_cal_event_edit      xxas_cal_event_edit;
  l_cal_event_drag      xxas_cal_event_drag;
  l_edit_data_xml       XMLType;
  l_trans_tbl           xxas_com_varchar2_table;
  l_times_tbl           xxas_com_varchar2_table;
  l_transaction_id      number;
  l_transaction_step_id number;
  l_start_time          varchar2(30);
  l_end_time            varchar2(30);
  l_keys                xxas_com_keys_type;

BEGIN

  log_start(c_method,p_person_id,p_abs_attendance_id,p_trans_and_step_id,p_times,p_source_block,p_saved_search_id,p_manager_id);

  xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

  IF p_trans_and_step_id is not null THEN
  
    l_keys := xxas_emp_menu_pkg.object_id_to_keys
                (p_object_id        => p_trans_and_step_id
                ,p_object_id_format => '{TRANSACTION_ID}_{TRANSACTION_STEP_ID}');

    l_transaction_id      := to_number(l_keys.get_key('TRANSACTION_ID'));
    l_transaction_step_id := to_number(l_keys.get_key('TRANSACTION_STEP_ID'));

  END IF;

  l_times_tbl := xxas_util_pkg.string_to_varchar2tbl(p_times,'*');

  IF l_times_tbl.count = 0 THEN

    l_start_time := null;
    l_end_time   := null;

  ELSE

    l_start_time := l_times_tbl(1);
    l_end_time   := l_times_tbl(2);

  END IF;

  IF l_start_time is not null AND 
     l_end_time   is not null THEN

    l_cal_event_drag := xxas_cal_event_drag
                          (p_person_id           => p_person_id
                          ,p_resource_id         => xxas_util_pkg.get_primary_worker_asg(p_person_id,trunc(sysdate))
                          ,p_abs_attendance_id   => p_abs_attendance_id
                          ,p_log_entry_id        => null
                          ,p_transaction_id      => l_transaction_id
                          ,p_transaction_step_id => l_transaction_step_id
                          ,p_action_id           => g_ABS_ACTION_ID
                          ,p_start_time          => fnd_date.canonical_to_date(l_start_time)
                          ,p_end_time            => fnd_date.canonical_to_date(l_end_time)
                          ,p_source_block        => p_source_block
                          ,p_saved_search_id     => p_saved_search_id
                          ,p_manager_id          => p_manager_id);

    l_edit_data_xml := l_cal_event_drag.get_xml();

  ELSE

    l_cal_event_edit := xxas_cal_event_edit
                          (p_person_id           => p_person_id
                          ,p_resource_id         => xxas_util_pkg.get_primary_worker_asg(p_person_id,trunc(sysdate))
                          ,p_abs_attendance_id   => p_abs_attendance_id
                          ,p_log_entry_id        => null
                          ,p_transaction_id      => l_transaction_id
                          ,p_transaction_step_id => l_transaction_step_id
                          ,p_action_id           => g_ABS_ACTION_ID
                          ,p_source_block        => p_source_block
                          ,p_saved_search_id     => p_saved_search_id
                          ,p_manager_id          => p_manager_id);

    l_edit_data_xml := l_cal_event_edit.get_xml();

  END IF;

  RETURN l_edit_data_xml;

END entry_edit;

--
-- ------------------------------------------------------------------------ *
-- Name    : entry_review
-- ------------------------------------------------------------------------ *
FUNCTION entry_review
  (p_person_id           IN number
  ,p_abs_attendance_id   IN number
  ,p_trans_and_step_id   IN varchar2
  ,p_source_block        IN varchar2
  ,p_saved_search_id     IN number
  ,p_manager_id          IN number DEFAULT NULL) RETURN XMLType
IS

  c_method CONSTANT     varchar2(30) := 'entry_review';
  l_cal_event_view      xxas_cal_event_review;
  l_view_data_xml       XMLType;
  l_varchar2tbl         xxas_com_varchar2_table;
  l_transaction_id      number;
  l_transaction_step_id number;
  l_keys                xxas_com_keys_type;

BEGIN

  log_start(c_method,p_person_id,p_abs_attendance_id,p_trans_and_step_id,p_source_block,p_saved_search_id,p_manager_id);

  xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

  IF p_trans_and_step_id is not null THEN
  
    l_keys := xxas_emp_menu_pkg.object_id_to_keys
                (p_object_id        => p_trans_and_step_id
                ,p_object_id_format => '{TRANSACTION_ID}_{TRANSACTION_STEP_ID}');

    l_transaction_id      := to_number(l_keys.get_key('TRANSACTION_ID'));
    l_transaction_step_id := to_number(l_keys.get_key('TRANSACTION_STEP_ID'));

  END IF;

  l_cal_event_view := xxas_cal_event_review
                        (p_person_id           => p_person_id
                        ,p_resource_id         => xxas_util_pkg.get_primary_worker_asg(p_person_id,trunc(sysdate))
                        ,p_abs_attendance_id   => p_abs_attendance_id
                        ,p_log_entry_id        => null
                        ,p_transaction_id      => l_transaction_id
                        ,p_transaction_step_id => l_transaction_step_id
                        ,p_action_id           => g_ABS_ACTION_ID
                        ,p_source_block        => p_source_block
                        ,p_saved_search_id     => p_saved_search_id
                        ,p_manager_id          => p_manager_id);

  l_view_data_xml := l_cal_event_view.get_xml();

  RETURN l_view_data_xml;

END entry_review;

--
-- ------------------------------------------------------------------------ *
-- Name    : edit_entry_response
-- ------------------------------------------------------------------------ *
FUNCTION edit_entry_response
  (p_resource_id     IN varchar2
  ,p_keys            IN xxas_com_keys_type
  ,p_start_time      IN varchar2
  ,p_end_time        IN varchar2
  ,p_source_block    IN varchar2
  ,p_saved_search_id IN number
  ,p_manager_id      IN number DEFAULT NULL) RETURN XMLType
IS

  c_method    CONSTANT varchar2(30) := 'edit_entry_response';
  l_cal_event_drag     xxas_cal_event_drag;
  l_form_data_xml      XMLType;
  l_response_xml       XMLType;

BEGIN

  log_start(c_method,p_resource_id,p_start_time,p_end_time,p_source_block,p_saved_search_id,p_manager_id);

  IF p_resource_id is not null AND 
     p_keys        is not null AND
     p_start_time  is not null AND
     p_end_time    is not null THEN

    --
    -- Same resource event edited - Drag operation
    --
    IF p_keys.get_key('PERSON_ID') is not null AND 
       p_keys.get_key('PERSON_ID') = xxas_util_pkg.get_person_from_asg(to_number(p_resource_id),trunc(sysdate)) THEN

      xxas_util_pkg.user_has_access_check(g_SSH_PROD_KEY);

      l_response_xml := xxas_com_rest_util_pkg.app_route_redirect_response
                          ('absence-edit/'
                           || to_char(p_keys.get_key('PERSON_ID'))
                           || '_'
                           || to_char(p_keys.get_key('ABSENCE_ATTENDANCE_ID'))
                           || '_'
                           || to_char(p_keys.get_key('TRANSACTION_ID'))
                           || '_'
                           || to_char(p_keys.get_key('TRANSACTION_STEP_ID'))
                           || '_'
                           || p_start_time || '*' || p_end_time
                           || '_'
                           || p_source_block
                           || '_'
                           || p_saved_search_id
                           || '_'
                           || p_manager_id
                          ,'modal'
                          ,'true'
                          ,'false');

      /*
      l_cal_event_drag := xxas_cal_event_drag
                            (p_resource_id         => p_resource_id
                            ,p_abs_attendance_id   => p_keys.get_key('ABSENCE_ATTENDANCE_ID')
                            ,p_log_entry_id        => null
                            ,p_transaction_id      => p_keys.get_key('TRANSACTION_ID')
                            ,p_transaction_step_id => p_keys.get_key('TRANSACTION_STEP_ID')
                            ,p_action_id           => g_ABS_ACTION_ID
                            ,p_start_time          => fnd_date.canonical_to_date(p_start_time)
                            ,p_end_time            => fnd_date.canonical_to_date(p_end_time));

      l_form_data_xml := l_cal_event_drag.get_xml();

      update_absence_event
        (p_function_id    => xxas_cal_util_pkg.page_slug_func_id('absence-edit')
        ,p_form_data_xml  => l_form_data_xml
        ,p_user_submitted => 'Y'
        ,p_response_xml   => l_response_xml);*/

      RETURN l_response_xml;

    --
    -- Different resource event edited - Drop operation
    --
    ELSIF p_keys.get_key('PERSON_ID') is not null AND 
          p_keys.get_key('PERSON_ID') <> p_resource_id THEN

      null;

    END IF;

  END IF;

  log_end(c_method);

  RETURN null;
  
END edit_entry_response;

--
-- ------------------------------------------------------------------------ *
-- Name    : ol_absence_entries
-- ------------------------------------------------------------------------ *
FUNCTION ol_absence_entries
  (p_params_tbl IN varchar2) RETURN XMLType
IS

  CURSOR csr_peer_asgs(cp_start_date IN date 
                      ,cp_end_date   IN date) IS
  SELECT distinct column_value
  FROM   TABLE(CAST(xxas_cal_util_pkg.asgs_in_hier
                      (g_PEERS
                      ,cp_start_date
                      ,cp_end_date
                      ,trunc(sysdate)
                      ,fnd_global.employee_id) AS xxas_com_num_table))
  WHERE  rownum >= 0;

  CURSOR csr_peer_person_ids(cp_peer_asg_tbl IN xxas_com_num_table) IS
  SELECT distinct xxas_util_pkg.get_person_from_asg(column_value,trunc(sysdate))
  FROM   TABLE(cp_peer_asg_tbl);

  CURSOR csr_team_asgs(cp_start_date IN date
                      ,cp_end_date   IN date) IS
  SELECT distinct column_value
  FROM   TABLE(CAST(xxas_cal_util_pkg.asgs_in_hier
                      (g_DR_ONLY
                      ,cp_start_date
                      ,cp_start_date
                      ,trunc(sysdate)
                      ,fnd_global.employee_id) AS xxas_com_num_table))
  WHERE  rownum >= 0;

  CURSOR csr_team_person_ids(cp_team_asg_tbl IN xxas_com_num_table) IS
  SELECT distinct xxas_util_pkg.get_person_from_asg(column_value,trunc(sysdate))
  FROM   TABLE(cp_team_asg_tbl);

  CURSOR csr_team_saved_sch_asgs(cp_saved_search_id IN number) IS
  SELECT /*+ cardinality(rslts 10)*/
         distinct rslts.assignment_id
  FROM   TABLE(xxas_per_search_pkg.people_tbl
                 (null
                 ,(SELECT xxas_saved_search_pkg.xml_to_obj(ss.search_parameters)
                   FROM   xxas_saved_searches_v ss
                   WHERE  ss.saved_search_id = cp_saved_search_id
                   AND    xxas_com_sec_pkg.test_instance
                            ('XXAS_DATA_SAVED_SEARCH_VIEW',cp_saved_search_id) = 'Y')
                 )
              ) rslts
  WHERE  xxas_com_sec_pkg.test_instance
           ('XXAS_DT_SAVE_SRCH_TIME_ABS'
           ,rslts.assignment_id
           ,null
           ,null
           ,null
           ,null) = 'Y';

  CURSOR csr_team_saved_sch_person_ids(cp_team_asg_tbl IN xxas_com_num_table) IS
  SELECT distinct xxas_util_pkg.get_person_from_asg(column_value,trunc(sysdate))
  FROM   TABLE(cp_team_asg_tbl);

  CURSOR csr_ol_absences_data(cp_person_id         IN number
                             ,cp_start_date        IN date
                             ,cp_end_date          IN date
                             ,cp_emp_calendar_flag IN varchar2
                             ,cp_mgr_calendar_flag IN varchar2) IS
  SELECT XMLElement
          ("absences"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("item"
              ,XMLForest
                ('worklist' AS "type"
                ,xxas_util_pkg.get_formatted_name
                   (p_full_name   => rslt.full_name
                   ,p_global_name => rslt.global_name
                   ,p_local_name  => rslt.local_name
                   ,p_first_name  => rslt.first_name
                   ,p_last_name   => rslt.last_name
                   ,p_known_as    => rslt.known_as) AS "sender"
                ,CASE WHEN rslt.date_end = rslt.date_start 
                      THEN to_char(rslt.date_start,'Mon dd yyyy') 
                      ELSE to_char(rslt.date_start,'Mon dd') || ' - ' || nvl(to_char(rslt.date_end,'Mon dd yyyy'),xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_ONGOING'))
                 END
                 || decode((rslt.date_end -rslt.date_start +1)
                          ,null
                          ,''
                          ,1
                          ,', 1 '|| xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_CAL_EVENT_DUR_DAY')
                          , ', ' || (rslt.date_end -rslt.date_start+1) || ' ' || xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_CAL_EVENT_DUR_DAYS')
                          )
                 || ' - '
                 || CASE WHEN cp_emp_calendar_flag = 'Y'
                         THEN xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABSENT_TYPE_DEFAULT')
                         WHEN cp_mgr_calendar_flag = 'Y'
                         THEN rslt.absence_type
                         ELSE xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABSENT_TYPE_DEFAULT')
                    END  AS "subject"
                )
              ,CASE WHEN rslt.wf_status_code IS NOT NULL 
                    THEN XMLElement
                          ("labels"
                          ,XMLAttributes
                            ('true' as "json:force-array"
                            ,'http://json.org/' as "xmlns:json")
                          ,XMLElement
                            ("label"
                            ,XMLForest(CASE rslt.wf_status_code
                                       WHEN 'PENDING' THEN
                                            xxas_rslt_cache_pkg.resolve_color_name('energized') 
                                       WHEN 'RFC' THEN
                                            xxas_rslt_cache_pkg.resolve_color_name('assertive')
                                       WHEN 'ERROR' THEN
                                            xxas_rslt_cache_pkg.resolve_color_name('assertive')
                                       WHEN 'APPROVED' THEN 
                                            xxas_rslt_cache_pkg.resolve_color_name('green') 
                                       END                                        AS "color"
                                      ,rslt.wf_status_meaning                     AS "title"
                                      )
                            )
                          )
               END
              ,XMLElement
                ("linked"
                ,XMLElement
                  ("image"
                  ,XMLForest
                    (xxas_com_rest_util_pkg.uri_from_path
                       ('people/' || rslt.img_person_id || '/photo') "self"
                    ,xxas_per_image_pkg.url
                       (rslt.img_person_id, rslt.last_update_date, 'SMALL') "small"
                    ,xxas_per_image_pkg.url
                       (rslt.img_person_id, rslt.last_update_date, 'MEDIUM') "medium"
                    ,xxas_per_image_pkg.url
                       (rslt.img_person_id, rslt.last_update_date, 'FULL_SIZE') "fullSize"
                    )
                  )
                ) "photo"
              ) ORDER BY rslt.date_start DESC
            )
          )
  FROM  (SELECT papf.person_id
               ,img.person_id img_person_id
               ,img.last_update_date
               ,(CASE WHEN xamv.hr_api_status_code is not null 
                      THEN nvl(xamv.date_start_p,xamv.date_projected_start_p)
                      ELSE nvl(xamv.date_start,xamv.date_projected_start)
                 END) date_start
               ,(CASE WHEN xamv.hr_api_status_code is not null 
                      THEN nvl(xamv.date_end_p,nvl(xamv.date_projected_end_p,cp_end_date))
                      ELSE nvl(xamv.date_end,nvl(xamv.date_projected_end,cp_end_date))
                 END) date_end
               ,(CASE WHEN xamv.hr_api_status_code is not null 
                      THEN xamv.absence_type_p
                      ELSE xamv.absence_type
                 END) absence_type
               ,xamv.hr_api_status_code status_code
               ,xamv.workflow_status_code wf_status_code
               ,xamv.workflow_status_meaning wf_status_meaning
               ,papf.full_name
               ,papf.global_name
               ,papf.local_name
               ,papf.first_name
               ,papf.last_name
               ,papf.known_as
         FROM   per_all_people_f          papf
               ,xxas_per_thumbnails       img
               ,xxas_absence_management_v xamv
         WHERE  xamv.person_id = cp_person_id
         AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                               AND     papf.effective_end_date
         AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
                 OR nvl(papf.current_npw_flag, 'N')  = 'Y')
         AND    xamv.person_id = papf.person_id
         AND    papf.person_id = img.person_id(+)
         AND    (((CASE WHEN xamv.hr_api_status_code is not null 
                        THEN nvl(xamv.date_start_p,xamv.date_projected_start_p)
                        ELSE nvl(xamv.date_start,xamv.date_projected_start)
                   END) BETWEEN cp_start_date
                        AND     cp_end_date) OR
                 ((CASE WHEN xamv.hr_api_status_code is not null 
                        THEN nvl(xamv.date_end_p,nvl(xamv.date_projected_end_p,cp_end_date))
                        ELSE nvl(xamv.date_end,nvl(xamv.date_projected_end,cp_end_date))
                   END) BETWEEN cp_start_date
                        AND     cp_end_date) OR
                 (cp_start_date BETWEEN (CASE WHEN xamv.hr_api_status_code is not null 
                                              THEN nvl(xamv.date_start_p,xamv.date_projected_start_p)
                                              ELSE nvl(xamv.date_start,xamv.date_projected_start)
                                         END)
                                AND     (CASE WHEN xamv.hr_api_status_code is not null 
                                              THEN nvl(xamv.date_end_p,nvl(xamv.date_projected_end_p,cp_end_date))
                                              ELSE nvl(xamv.date_end,nvl(xamv.date_projected_end,cp_end_date))
                                         END)) OR
                 (cp_end_date BETWEEN (CASE WHEN xamv.hr_api_status_code is not null 
                                            THEN nvl(xamv.date_start_p,xamv.date_projected_start_p)
                                            ELSE nvl(xamv.date_start,xamv.date_projected_start)
                                       END)
                              AND     (CASE WHEN xamv.hr_api_status_code is not null 
                                            THEN nvl(xamv.date_end_p,nvl(xamv.date_projected_end_p,cp_end_date))
                                            ELSE nvl(xamv.date_end,nvl(xamv.date_projected_end,cp_end_date))
                                       END))
                )
        ) rslt;

  c_method  CONSTANT varchar2(30) := 'ol_absence_entries';
  l_varchar2tbl      xxas_com_varchar2_table;
  l_person_id        number;
  l_start_date_time  date;
  l_end_date_time    date;
  l_start_date       date;
  l_end_date         date;
  l_start_time       varchar2(5);
  l_end_time         varchar2(5);
  l_source_block     varchar2(4000);
  l_saved_search_id  number;
  l_ol_absences_xml  XMLType;
  l_peer_asg_tbl     xxas_com_num_table;
  l_team_asg_tbl     xxas_com_num_table;
  l_per_tbl          xxas_com_num_table;
  l_emp_calendar_flag varchar2(1);
  l_mgr_calendar_flag varchar2(1);
  l_keys              xxas_com_keys_type;
  l_ol_absences_per_xml XMLType;

BEGIN

  log_start(c_method,p_params_tbl);


  IF p_params_tbl is not null THEN

    l_keys := xxas_emp_menu_pkg.object_id_to_keys
                (p_object_id        => p_params_tbl
                ,p_object_id_format => '{PERSON_ID}_{START_DATE_TIME}_{END_DATE_TIME}_{SOURCE_BLOCK}_{SAVED_SEARCH_ID}');

    l_person_id       := to_number(l_keys.get_key('PERSON_ID'));
    l_start_date_time := fnd_date.canonical_to_date(l_keys.get_key('START_DATE_TIME'));
    l_end_date_time   := fnd_date.canonical_to_date(l_keys.get_key('END_DATE_TIME'));
    l_source_block    := replace(l_keys.get_key('SOURCE_BLOCK'),'$','_');
    l_saved_search_id := to_number(l_keys.get_key('SAVED_SEARCH_ID'));

    l_start_date      := trunc(l_start_date_time);
    l_end_date        := trunc(l_end_date_time);
    /*l_start_time      := to_char(l_start_date_time,g_HH24MI_FMT);
    l_end_time        := to_char(l_end_date_time,g_HH24MI_FMT);*/

    IF l_start_date is not null AND 
       l_end_date is not null AND 
       l_source_block is not null THEN

      l_emp_calendar_flag := CASE WHEN instr(xxas_com_sec_pkg.get_function_parameter
                                               (p_function_name  => l_source_block
                                               ,p_parameter_name => 'pRestProvider'),'for-individual') <> 0
                                  THEN 'Y'
                                  ELSE 'N'
                             END;

      l_mgr_calendar_flag := CASE WHEN instr(xxas_com_sec_pkg.get_function_parameter
                                               (p_function_name  => l_source_block
                                               ,p_parameter_name => 'pRestProvider'),'for-team') <> 0
                                  THEN 'Y'
                                  ELSE 'N'
                             END;

      IF l_emp_calendar_flag = 'Y' THEN

        OPEN  csr_peer_asgs(l_start_date
                           ,l_end_date);
        FETCH csr_peer_asgs BULK COLLECT INTO l_peer_asg_tbl;
        CLOSE csr_peer_asgs;

        OPEN  csr_peer_person_ids(l_peer_asg_tbl);
        FETCH csr_peer_person_ids BULK COLLECT INTO l_per_tbl;
        CLOSE csr_peer_person_ids;

      ELSIF l_mgr_calendar_flag = 'Y' THEN

        IF l_saved_search_id is not null THEN

          OPEN  csr_team_saved_sch_asgs(l_saved_search_id);
          FETCH csr_team_saved_sch_asgs BULK COLLECT INTO l_team_asg_tbl;
          CLOSE csr_team_saved_sch_asgs;

          OPEN  csr_team_saved_sch_person_ids(l_team_asg_tbl);
          FETCH csr_team_saved_sch_person_ids BULK COLLECT INTO l_per_tbl;
          CLOSE csr_team_saved_sch_person_ids;

        ELSE

          OPEN  csr_team_asgs(l_start_date
                             ,l_end_date);
          FETCH csr_team_asgs BULK COLLECT INTO l_team_asg_tbl;
          CLOSE csr_team_asgs;

          OPEN  csr_team_person_ids(l_team_asg_tbl);
          FETCH csr_team_person_ids BULK COLLECT INTO l_per_tbl;
          CLOSE csr_team_person_ids;

        END IF;

      END IF;

      IF l_per_tbl is not null AND 
         l_per_tbl.count > 0 THEN

        FOR i in 1..l_per_tbl.count
        LOOP

          IF l_per_tbl(i) is not null THEN

            OPEN  csr_ol_absences_data(l_per_tbl(i)
                                      ,l_start_date
                                      ,l_end_date
                                      ,l_emp_calendar_flag
                                      ,l_mgr_calendar_flag);
            FETCH csr_ol_absences_data INTO l_ol_absences_per_xml;
            CLOSE csr_ol_absences_data;

            IF l_ol_absences_per_xml.extract('/absences/*') is not null THEN

              IF l_ol_absences_xml is null THEN

                SELECT XMLTYPE.appendChildXML
                        (XMLElement
                          ("items"
                          ,XMLAttributes
                            ('true' as "json:force-array"
                            ,'http://json.org/' as "xmlns:json"))
                        ,'/items'
                        ,l_ol_absences_per_xml.extract('/absences/*'))
                INTO   l_ol_absences_xml
                FROM   dual;

              ELSE

                SELECT XMLTYPE.appendChildXML
                        (l_ol_absences_xml
                        ,'/items'
                        ,l_ol_absences_per_xml.extract('/absences/*'))
                INTO   l_ol_absences_xml
                FROM   dual;

              END IF;

            END IF;

          END IF;

        END LOOP;

      END IF;

    END IF;

  END IF;

  RETURN l_ol_absences_xml;

  log_end(c_method);

END ol_absence_entries;

-- ------------------------------------------------------------------------ *
-- Name    : datetimerange_date_times
-- ------------------------------------------------------------------------ *
FUNCTION datetimerange_date_times
  (p_date_time_range   IN     varchar2
  ,p_date_start        IN OUT varchar2
  ,p_date_end          IN OUT varchar2
  ,p_time_start        IN OUT varchar2
  ,p_time_end          IN OUT varchar2) RETURN number
IS

  c_method  CONSTANT varchar2(30) := 'datetimerange_date_times';
  l_varchar2tbl      xxas_com_varchar2_table;

BEGIN

  log_start(c_method,p_date_time_range);

  IF p_date_time_range is not null THEN

    l_varchar2tbl := xxas_util_pkg.string_to_varchar2tbl(p_date_time_range,'_');

  END IF;

  IF l_varchar2tbl.count = 2 THEN

    p_date_start := l_varchar2tbl(1);
    p_date_end   := l_varchar2tbl(2);
    p_time_start := to_char(fnd_date.canonical_to_date(l_varchar2tbl(1)),'HH24:SS');
    p_time_end   := to_char(fnd_date.canonical_to_date(l_varchar2tbl(2)),'HH24:SS');

    RETURN 0;

  ELSIF l_varchar2tbl.count = 1 THEN

    p_date_start := l_varchar2tbl(1);
    p_time_start := to_char(fnd_date.canonical_to_date(l_varchar2tbl(1)),'HH24:SS');

    RETURN 0;

  END IF;

  RETURN -1;

  log_end(c_method);

END datetimerange_date_times;

-- ------------------------------------------------------------------------ *
-- Name    : get_range_dates
-- ------------------------------------------------------------------------ *
FUNCTION get_range_dates
  (p_range      IN     varchar2
  ,p_date_start IN OUT varchar2
  ,p_date_end   IN OUT varchar2) RETURN number
IS

  c_method  CONSTANT varchar2(30) := 'get_range_dates';
  l_varchar2tbl      xxas_com_varchar2_table;
  l_date_start       date;
  l_date_end         date;
  l_date_format_flag boolean;

BEGIN

  log_start(c_method,p_range);

  IF p_range is not null THEN

    l_varchar2tbl := xxas_util_pkg.string_to_varchar2tbl(p_range,'_');

  END IF;

  IF l_varchar2tbl.count = 2 THEN

    p_date_start := l_varchar2tbl(1);
    p_date_end   := l_varchar2tbl(2);

    BEGIN

      l_date_start       := trunc(to_date(p_date_start,'YYYY-MM-DD HH24:MI'));
      l_date_end         := trunc(to_date(p_date_end,'YYYY-MM-DD HH24:MI'));
      l_date_format_flag := true;

    EXCEPTION

      WHEN others THEN
        l_date_format_flag := false;

    END;

    IF l_date_format_flag THEN
      RETURN 0;
    ELSE
      RETURN -1;
    END IF;

  ELSIF l_varchar2tbl.count = 1 THEN

    p_date_start := l_varchar2tbl(1);

    RETURN 0;

  END IF;

  RETURN -1;

  log_end(c_method);

END get_range_dates;

-- ------------------------------------------------------------------------ *
-- Name    : get_factor_message
-- ------------------------------------------------------------------------ *
FUNCTION get_factor_message
  (p_factor IN number) RETURN varchar2
IS

  c_method  CONSTANT varchar2(30) := 'get_factor_message';
  l_message          varchar2(100);

BEGIN

  log_start(c_method,p_factor);

  IF p_factor is not null AND 
     g_factor_tbl.count > 0 THEN

    FOR i IN g_factor_tbl.first..g_factor_tbl.last 
    LOOP

      IF g_factor_tbl.exists(i) THEN

        IF p_factor BETWEEN g_factor_tbl(i).low_range
                    AND     nvl(g_factor_tbl(i).high_range,p_factor) THEN

          l_message := xxas_com_msg_pkg.get_message(xxas_util_pkg.app,g_factor_tbl(i).message);
          EXIT;

        END IF;

      END IF;

    END LOOP;

  END IF;

  RETURN l_message;

  log_end(c_method);

END get_factor_message;

-- ------------------------------------------------------------------------ *
-- Name    : get_factor_color
-- ------------------------------------------------------------------------ *
FUNCTION get_factor_color
  (p_factor IN number) RETURN varchar2
IS

  c_method  CONSTANT varchar2(30) := 'get_factor_color';
  l_color            varchar2(100);

BEGIN

  log_start(c_method,p_factor);

  IF p_factor is not null AND 
     g_factor_tbl.count > 0 THEN

    FOR i IN g_factor_tbl.first..g_factor_tbl.last 
    LOOP

      IF g_factor_tbl.exists(i) THEN

        IF p_factor BETWEEN g_factor_tbl(i).low_range
                    AND     nvl(g_factor_tbl(i).high_range,p_factor) THEN

          l_color := xxas_rslt_cache_pkg.resolve_color_name(g_factor_tbl(i).color);
          EXIT;

        END IF;

      END IF;

    END LOOP;

  END IF;

  RETURN nvl(l_color,xxas_rslt_cache_pkg.resolve_color_name('grey'));

  log_end(c_method);

END get_factor_color;

--
-- ------------------------------------------------------------------------ *
-- Name    : absence_sickness_rates_chart
-- ------------------------------------------------------------------------ *
FUNCTION absence_sickness_rates_chart
  (p_function_id         IN number
  ,p_function_name       IN varchar2
  ,p_function_parameters IN varchar2           DEFAULT NULL
  ,p_object_id           IN number             DEFAULT NULL
  ,p_obj_name            IN varchar2           DEFAULT NULL
  ,p_keys                IN xxas_com_keys_type DEFAULT NULL) RETURN XMLType
IS

  CURSOR csr_team_person_ids(cp_manager_person_id IN number) IS
  SELECT distinct xxas_util_pkg.get_person_from_asg(column_value,trunc(sysdate))
  FROM   TABLE(CAST(xxas_hierarchy_pkg.asgs_in_hier
                      (g_DR_ONLY
                      ,trunc(sysdate)
                      ,cp_manager_person_id) AS xxas_com_num_table))
  WHERE  rownum >= 0;

  CURSOR csr_team_saved_sch_person_ids(cp_saved_search_id IN number) IS
  SELECT /*+ cardinality(rslts 10)*/
         distinct rslts.person_id
  FROM   TABLE(xxas_per_search_pkg.people_tbl
                 (null
                 ,(SELECT xxas_saved_search_pkg.xml_to_obj(ss.search_parameters)
                   FROM   xxas_saved_searches_v ss
                   WHERE  ss.saved_search_id = cp_saved_search_id
                   AND    xxas_com_sec_pkg.test_instance
                            ('XXAS_DATA_SAVED_SEARCH_VIEW',cp_saved_search_id) = 'Y')
                 )
              ) rslts
  WHERE  xxas_com_sec_pkg.test_instance
           ('XXAS_DT_SAVE_SRCH_TIME_ABS'
           ,rslts.assignment_id
           ,null
           ,null
           ,null
           ,null) = 'Y';

  c_method  CONSTANT       varchar2(30) := 'absence_sickness_rates_chart';
  l_chart_xml              XMLType;
  l_mgr_person_id          number := p_keys.get_key('PERSON_ID');
  l_saved_search_id        number := p_keys.get_key('SAVED_SEARCH_ID');
  l_team_per_tbl           xxas_com_num_table;
  l_bradford_factor_label  varchar2(100) := xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_SICK_RATE_BRAD_FACTOR');
  l_employee_label         varchar2(100) := xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_SICK_RATE_EMPLOYEE');
  l_duration_label         varchar2(100) := xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_SICK_RATE_12_MONTHS');
  l_function_parameters    varchar2(4000);
  l_factor_threshold_range varchar2(100);
  l_factor_message         fnd_new_messages.message_name%TYPE;
  l_factor_color           varchar2(100);
  l_varchar2tbl            xxas_com_varchar2_table;
  l_row_counter            number := 1;

BEGIN

  log_start(c_method,p_function_id,p_function_name,p_function_parameters,p_object_id,p_obj_name);

  --
  -- Delete the contents of the cached tables.
  --
  g_factor_tbl.delete;

  IF p_function_parameters is not null THEN

    l_function_parameters := p_function_parameters;

  ELSIF p_function_name is not null THEN

    l_function_parameters := xxas_com_sec_pkg.get_function_parameters(p_function_name => p_function_name);

  END IF;

  --
  -- Extract different parameters associated for Factors like Threshold Range, Color, Message
  --
  IF l_function_parameters is not null THEN

    l_factor_threshold_range := null;
    l_factor_message         := null;
    l_factor_color           := null;

    FOR i IN 1..10 LOOP

      l_factor_threshold_range := xxas_com_sec_pkg.get_parameter_value(l_function_parameters,'pFactorThresholdRange'|| to_char(i));
      l_factor_message         := xxas_com_sec_pkg.get_parameter_value(l_function_parameters,'pFactorMessage'|| to_char(i));
      l_factor_color           := xxas_com_sec_pkg.get_parameter_value(l_function_parameters,'pFactorColor'|| to_char(i));


      IF l_factor_threshold_range is not null THEN

        l_varchar2tbl := xxas_util_pkg.string_to_varchar2tbl(l_factor_threshold_range,'-');

        --
        -- Add this factor into the g_factor_tbl.
        --
        IF g_factor_tbl.count > 0 THEN
          l_row_counter := g_factor_tbl.last + 1;
        END IF;

        IF l_varchar2tbl.count = 2 THEN

          g_factor_tbl(l_row_counter).low_range  := l_varchar2tbl(1);
          g_factor_tbl(l_row_counter).high_range := l_varchar2tbl(2);

        ELSIF l_varchar2tbl.count = 1 THEN

          g_factor_tbl(l_row_counter).low_range  := l_varchar2tbl(1);
          g_factor_tbl(l_row_counter).high_range := null;

        END IF;

        g_factor_tbl(l_row_counter).message  := l_factor_message;
        g_factor_tbl(l_row_counter).color    := l_factor_color;

      END IF;

      l_factor_threshold_range := null;
      l_factor_message         := null;
      l_factor_color           := null;

    END LOOP;

  END IF;

  IF l_mgr_person_id   is not null AND
     l_saved_search_id is not null THEN

    OPEN  csr_team_saved_sch_person_ids(l_saved_search_id);
    FETCH csr_team_saved_sch_person_ids BULK COLLECT INTO l_team_per_tbl;
    CLOSE csr_team_saved_sch_person_ids;

  ELSIF l_mgr_person_id is not null THEN

    OPEN  csr_team_person_ids(l_mgr_person_id);
    FETCH csr_team_person_ids BULK COLLECT INTO l_team_per_tbl;
    CLOSE csr_team_person_ids;

  END IF;

  IF l_team_per_tbl is not null THEN

    SELECT XMLElement
           ("chart"
           ,XMLElement("type", 'BarChart')
           ,XMLElement
             ("data"
             ,XMLElement
               ("cols"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLElement
                 ("column"
                 ,XMLForest
                   (l_employee_label "label"
                   ,'string'         "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   (l_bradford_factor_label "label"
                   ,'number'                "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('annotation' "role"
                   ,'string'     "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('style'  "role"
                   ,'string' "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('tooltip' "role"
                   ,'string'  "type")
                 )
               )
             ,XMLElement
              ("rows"
              ,XMLAttributes
                ('true' as "json:force-array"
                ,'http://json.org/' as "xmlns:json")
              ,XMLAgg
                (XMLElement
                  ("row"
                 ,XMLElement
                   ("c"
                    ,XMLAttributes
                     ('true' as "json:force-array"
                     ,'http://json.org/' as "xmlns:json")
                   ,XMLElement
                     ("cv"
                     ,XMLElement
                       ("v", rslt.formatted_name)
                     )
                   ,XMLElement
                     ("cv"
                     ,XMLElement
                       ("v", rslt.factor)
                     )
                   ,XMLElement
                     ("cv"
                     ,XMLElement
                       ("v", to_char(rslt.factor))
                     )
                   ,XMLElement
                     ("cv"
                     ,XMLElement
                       ("v", xxas_absence_event_pkg.get_factor_color(rslt.factor))
                     )
                   ,XMLElement
                     ("cv"
                     ,XMLElement
                       ("v",xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_SICK_RATE_TOT_SPELLS') || ' ' || to_char(rslt.instances) || '
                            '|| xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_SICK_RATE_TOT_SICK') || ' ' || to_char(rslt.total) || '
                            '|| xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_SICK_RATE_TOT_PE_SPEL') || ' ' || to_char(rslt.pend_instances) || '
                            '|| xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_SICK_RATE_TOT_PE_SICK') || ' ' || to_char(rslt.pend_total) || '
                            '|| xxas_absence_event_pkg.get_factor_message(rslt.factor)
                       )
                     )
                   )
                  )
                )
              )
           )
           ,XMLElement
             ("options"
             ,XMLElement("title", null)
             ,XMLElement
               ("legend"
               ,XMLElement("position" , 'none')
               ,XMLElement("maxLines" , 3)
               )
             ,XMLElement
               ("annotations"
               ,XMLElement("alwaysOutside" , 'true')
               )
             ,XMLElement
               ("animation"
               ,XMLElement("startup" , 'true')
               ,XMLElement("duration" , 1000)
               ,XMLElement("easing", 'out')
               )
             ,XMLElement
               ("hAxis"
               ,XMLElement("title" , l_duration_label)
               ,XMLElement("format" , '0')
               )
             ,XMLElement("isStacked", 'true')
             )
            ) "data"
  INTO l_chart_xml
  FROM (SELECT person_id
              ,instances
              ,pend_instances
              ,total
              ,pend_total
              ,factor
              ,xxas_cal_util_pkg.formatted_name_from_per_id(person_id,trunc(sysdate)) formatted_name
        FROM   (SELECT xamv.person_id
                      ,COUNT((CASE WHEN xamv.transaction_id is null AND xamv.transaction_step_id is null
                                   THEN xamv.absence_attendance_id
                                   ELSE null
                              END)) instances
                      ,COUNT((CASE WHEN xamv.transaction_id is not null AND xamv.transaction_step_id is not null
                                   THEN xamv.absence_attendance_id_p
                                   ELSE null
                              END)) pend_instances
                      ,SUM((xamv.date_end - xamv.date_start) + 1) total
                      ,SUM((xamv.date_end_p - xamv.date_start_p) + 1) pend_total
                      ,((COUNT((CASE WHEN xamv.transaction_id is not null AND xamv.transaction_step_id is not null
                                     THEN xamv.absence_attendance_id_p
                                     WHEN xamv.transaction_id is null AND xamv.transaction_step_id is null
                                     THEN xamv.absence_attendance_id
                                     ELSE null
                                END))
                         * COUNT((CASE WHEN xamv.transaction_id is not null AND xamv.transaction_step_id is not null
                                       THEN xamv.absence_attendance_id_p
                                       WHEN xamv.transaction_id is null AND xamv.transaction_step_id is null
                                       THEN xamv.absence_attendance_id
                                       ELSE null
                                  END))
                        ) * SUM((CASE WHEN xamv.transaction_id is not null AND xamv.transaction_step_id is not null
                                       THEN (xamv.date_end_p - xamv.date_start_p) + 1
                                       WHEN xamv.transaction_id is null AND xamv.transaction_step_id is null
                                       THEN (xamv.date_end - xamv.date_start) + 1
                                       ELSE 0
                                  END))
                       ) factor
                FROM   xxas_absence_management_v xamv
                WHERE  xamv.person_id IN (SELECT /*+ cardinality(rslts 10) */ rslts.column_value
                                          FROM   TABLE(l_team_per_tbl) rslts
                                          WHERE  rownum >= 0)
                AND    ((l_saved_search_id is null) OR 
                        (l_saved_search_id is not null AND xxas_com_sec_pkg.test_instance('XXAS_DT_ABSENCE_LIST'
                                                                                         ,(CASE WHEN xamv.transaction_id is not null AND xamv.transaction_step_id is not null
                                                                                                THEN xamv.absence_attendance_id_p
                                                                                                WHEN xamv.transaction_id is null AND xamv.transaction_step_id is null
                                                                                                THEN xamv.absence_attendance_id
                                                                                                ELSE null
                                                                                           END)
                                                                                         ,xamv.transaction_id
                                                                                         ,xamv.transaction_step_id
                                                                                         ) = 'Y'))
                AND    ((xamv.transaction_id      is null AND
                         xamv.transaction_step_id is null AND
                         xamv.date_start >= add_months(trunc(sysdate),-12) AND
                         xamv.date_end   <= trunc(sysdate) AND
                         xamv.absence_category_code = 'S') OR 
                        (xamv.transaction_id      is not null AND
                         xamv.transaction_step_id is not null AND 
                         xamv.date_start_p >= add_months(trunc(sysdate),-12) AND
                         xamv.date_end_p   <= trunc(sysdate) AND
                         xamv.absence_category_code_p = 'S')
                       )
                AND    xamv.person_id <> fnd_global.employee_id
                GROUP BY xamv.person_id
                ORDER BY 6 desc
               )
        WHERE rownum <=15
       ) rslt;

  END IF;

  --
  -- Delete the contents of the cached tables.
  --
  g_factor_tbl.delete;

  log_end(c_method);

  RETURN l_chart_xml;

END absence_sickness_rates_chart;

--
-- ------------------------------------------------------------------------ *
-- Name    : is_absence_present
-- ------------------------------------------------------------------------ *
FUNCTION is_absence_present
  (p_absence_attendance_id IN number) RETURN varchar2
IS

  CURSOR csr_absence_attendance_id IS
  SELECT absence_attendance_id
  FROM   per_absence_attendances
  WHERE  absence_attendance_id = p_absence_attendance_id;

  c_method       CONSTANT  varchar2(30) := 'is_absence_present';
  l_abs_present_flag       varchar2(1);
  l_absence_attendance_id  number;

BEGIN

  log_start(c_method,p_absence_attendance_id);

  IF p_absence_attendance_id is not null THEN

    OPEN  csr_absence_attendance_id;
    FETCH csr_absence_attendance_id INTO l_absence_attendance_id;
    CLOSE csr_absence_attendance_id;

    IF l_absence_attendance_id is not null THEN
      l_abs_present_flag := 'Y';
    ELSE
      l_abs_present_flag := 'N';
    END IF;

  END IF;

  log_end(c_method,nvl(l_abs_present_flag,'N'));

  RETURN nvl(l_abs_present_flag,'N');

END is_absence_present;

-- ------------------------------------------------------------------------ *
-- Name    : accr_plan_enr_per_as_of_date
-- ------------------------------------------------------------------------ *
FUNCTION accr_plan_enr_per_as_of_date
  (p_assignment_id     IN         number
  ,p_effective_date    IN         varchar2
  ,p_absence_type_id   IN         number
  ,p_accrual_plan_id   OUT NOCOPY number) RETURN NUMBER
IS

  c_method  CONSTANT  varchar2(30) := 'accr_plan_enr_per_as_of_date';
  l_accrual_plan_id   pay_accrual_plans_v.accrual_plan_id%TYPE;
  l_effective_date    date;
  l_date_format_flag  boolean;

  CURSOR csr_accrual_plan_id
  IS
  SELECT pap.accrual_plan_id
        --,pap.accrual_plan_name
        --,pap.accrual_units_of_measure accrual_plan_uom
  FROM   pay_element_entries_f        pee
        ,pay_accrual_plans_v          pap
        ,per_absence_attendance_types paat
        ,per_all_assignments_f        paaf
  WHERE  paaf.assignment_id = p_assignment_id
  AND    trunc(sysdate) BETWEEN paaf.effective_start_date 
                        AND     paaf.effective_end_date
  AND    paaf.assignment_type = 'E'
  AND    paaf.primary_flag = 'Y'
  AND    pee.assignment_id      = paaf.assignment_id
  AND    l_effective_date BETWEEN pee.effective_start_date
                          AND     nvl(pee.effective_end_date,l_effective_date)
  AND    pee.element_type_id    = pap.accrual_plan_element_type_id
  AND    pap.business_group_id  = paat.business_group_id
  AND    paat.absence_attendance_type_id = p_absence_type_id
  AND    pap.pto_input_value_id = paat.input_value_id
  AND    l_effective_date BETWEEN paat.date_effective
                          AND     nvl(paat.date_end,l_effective_date)
  ORDER BY pap.accrual_plan_name
          ,nvl(paaf.primary_flag, 'N') DESC;
  

BEGIN

  log_start(c_method,p_assignment_id,p_effective_date,p_absence_type_id);
  IF p_effective_date = 'XXASNULLVAL' THEN
    RETURN 1;
  ELSE

    BEGIN

      l_effective_date   := trunc(to_date(p_effective_date,'YYYY-MM-DD HH24:MI'));
      l_date_format_flag := true;

    EXCEPTION

      WHEN others THEN
        l_date_format_flag := false;

    END;

    IF NOT l_date_format_flag THEN
      RETURN 1;
    END IF;

    IF p_assignment_id   is not null AND
       p_effective_date  is not null AND
       p_absence_type_id is not null THEN

      OPEN  csr_accrual_plan_id;
      FETCH csr_accrual_plan_id INTO l_accrual_plan_id;
      CLOSE csr_accrual_plan_id;

      IF l_accrual_plan_id IS NOT NULL then

        p_accrual_plan_id   := l_accrual_plan_id;
        log_end(c_method,l_accrual_plan_id);
        RETURN 0;

      ELSE

        log_end(c_method,l_accrual_plan_id);
        RETURN 1;

      END IF;

    END IF;

  END IF;

  log_end(c_method,l_accrual_plan_id);
  RETURN 1;

END accr_plan_enr_per_as_of_date;

-- ------------------------------------------------------------------------ *
-- Name    : accr_plan_details_as_of_date
-- ------------------------------------------------------------------------ *
FUNCTION accr_plan_details_as_of_date
  (p_assignment_id          IN         number
  ,p_effective_date         IN         varchar2
  ,p_accrual_plan_id        IN         number
  ,p_include_pending        IN         varchar2
  ,p_accrual_plan_name      OUT NOCOPY varchar2
  ,p_accrual_plan_uom       OUT NOCOPY varchar2
  ,p_accrual_plan_uom_name  OUT NOCOPY varchar2
  ,p_start_date             OUT NOCOPY date
  ,p_end_date               OUT NOCOPY date
  ,p_gross_total            OUT NOCOPY number
  ,p_entitlement            OUT NOCOPY number
  ,p_carryover              OUT NOCOPY number
  ,p_entitlement_adjust     OUT NOCOPY number
  ,p_used_total             OUT NOCOPY number
  ,p_used                   OUT NOCOPY number
  ,p_pending                OUT NOCOPY number
  ,p_used_adjust            OUT NOCOPY number
  ,p_remaining              OUT NOCOPY number) RETURN NUMBER
IS

  c_method  CONSTANT      varchar2(30) := 'accr_plan_details_as_of_date';
  l_accrual_plan_id       pay_accrual_plans_v.accrual_plan_id%TYPE;
  l_accrual_plan_name     pay_accrual_plans_v.accrual_plan_name%TYPE;
  l_accrual_plan_uom      pay_accrual_plans_v.accrual_units_of_measure%TYPE;
  l_accrual_plan_uom_name pay_accrual_plans_v.accrual_units_of_measure_name%TYPE;
  l_start_date            date;
  l_end_date              date;
  l_gross_total           number;
  l_entitlement           number;
  l_carryover             number;
  l_ent_adjust            number;
  l_used_total            number;
  l_used                  number;
  l_pending               number;
  l_used_adjust           number;
  l_remaining             number;
  l_effective_date        date;
  l_date_format_flag      boolean;

BEGIN

  log_start(c_method,p_assignment_id,p_effective_date,p_accrual_plan_id);

  IF p_effective_date <> 'XXASNULLVAL' THEN

    BEGIN

      l_effective_date   := trunc(to_date(p_effective_date,'YYYY-MM-DD HH24:MI'));
      l_date_format_flag := true;

    EXCEPTION

      WHEN others THEN
        l_date_format_flag := false;

    END;

    IF NOT l_date_format_flag THEN
      RETURN 1;
    END IF;

    IF p_assignment_id   is not null AND
       p_effective_date  is not null AND
       p_accrual_plan_id is not null THEN

      xxas_absence_management_pkg.get_accrual_plan_details
        (p_person_id             => xxas_util_pkg.get_person_from_asg(p_assignment_id,l_effective_date)
        ,p_assignment_id         => p_assignment_id
        ,p_effective_date        => l_effective_date
        ,p_accrual_plan_id       => p_accrual_plan_id
        ,p_include_pending       => (CASE WHEN p_include_pending = 'Y'
                                          THEN true
                                          ELSE false
                                     END)
        ,p_accrual_plan_name     => l_accrual_plan_name
        ,p_accrual_plan_uom      => l_accrual_plan_uom
        ,p_accrual_plan_uom_name => l_accrual_plan_uom_name
        ,p_start_date            => l_start_date
        ,p_end_date              => l_end_date
        ,p_gross_total           => l_gross_total
        ,p_entitlement           => l_entitlement
        ,p_carryover             => l_carryover
        ,p_entitlement_adjust    => l_ent_adjust
        ,p_used_total            => l_used_total
        ,p_used                  => l_used
        ,p_pending               => l_pending
        ,p_used_adjust           => l_used_adjust
        ,p_remaining             => l_remaining);

      p_accrual_plan_name      := l_accrual_plan_name;
      p_accrual_plan_uom       := l_accrual_plan_uom;
      p_accrual_plan_uom_name  := l_accrual_plan_uom_name;
      p_start_date             := l_start_date;
      p_end_date               := l_end_date;
      p_gross_total            := l_gross_total;
      p_entitlement            := l_entitlement;
      p_carryover              := l_carryover;
      p_entitlement_adjust     := l_ent_adjust;
      p_used_total             := l_used_total;
      p_used                   := l_used;
      p_pending                := l_pending;
      p_used_adjust            := l_used_adjust;
      p_remaining              := l_remaining;

      RETURN 0;

    END IF;

  ELSE

    RETURN 1;

  END IF;

  log_end(c_method);
  RETURN 1;

END accr_plan_details_as_of_date;

-- ------------------------------------------------------------------------ *
-- Name    : get_vacation_rules_xml
-- ------------------------------------------------------------------------ *
FUNCTION get_vacation_rules_xml(p_person_id  IN NUMBER
                               ,p_date_start IN DATE
                               ,p_date_end   IN DATE) RETURN XMLType
IS 

  CURSOR csr_vac_rules
  IS
  SELECT rule_id
        ,action_argument
  FROM   wf_routing_rules wrr
  WHERE  wrr.begin_date = p_date_start
  AND    nvl(wrr.end_date,trunc(sysdate))  = nvl(p_date_end,trunc(sysdate)) + 1
  AND    wrr.role = xxas_util_pkg.get_username_from_per(p_person_id,sysdate)
  AND    wrr.action = 'FORWARD'
  AND    wrr.message_type IS NULL;
  
  CURSOR csr_worklist_access
  IS
  SELECT fg.grant_guid
        ,fg.grantee_key
  FROM   fnd_grants fg
  WHERE  fg.start_date = p_date_start
  AND    nvl(fg.end_date,sysdate) = nvl(p_date_end,trunc(sysdate)) + 1
  AND    fg.grantee_type = 'USER'
  AND    fg.parameter1 = xxas_util_pkg.get_username_from_per(p_person_id,sysdate)
  AND    fg.menu_id = xxas_rslt_cache_pkg.menu_name_to_id('FND_WF_WORKLIST')
  AND    fg.instance_type = 'SET'
  AND    fg.instance_set_id = xxas_rslt_cache_pkg.get_instance_set_id('WL_PROXY_ACCESS')
  AND    fg.object_id = nvl(xxas_rslt_cache_pkg.object_id('NOTIFICATIONS'), -1);
 
  CURSOR csr_vac_xml(cp_rule_type IN VARCHAR2
                    ,cp_assignee  IN VARCHAR2)
  IS
  SELECT XMLFOREST('Y'          as "VACATION_RULES_PERM"
                  ,'Y'          as "REDIRECT_NOTIF"
                  ,'Y'          as "REDIRECT_NOTIF_OLD"
                  ,cp_rule_type as "RULE"
                  ,cp_assignee  as "GRANTEE")
  FROM   dual;
  
  l_rule_id  NUMBER;
  l_assignee VARCHAR2(100);
  l_grant_uid VARCHAR2(4000);
  l_xml       XMLTYPE;
  
BEGIN
  
  --insert into test1 values('p_person_id:' || p_person_id ||' p_date_start:' || p_date_start || ' p_date_end:' || p_date_end);
  OPEN csr_vac_rules;
  FETCH csr_vac_rules INTO l_rule_id,l_assignee;
  CLOSE csr_vac_rules;
  --insert into test1 values('l_rule_id:' || l_rule_id || ' l_assignee:' || l_assignee);
  
  IF l_rule_id IS NOT NULL THEN
    
    OPEN csr_vac_xml('VACATION',l_assignee);
    FETCH csr_vac_xml INTO l_xml;
    CLOSE csr_vac_xml;
  
  END IF;
  
  IF l_xml IS NULL THEN
    
    OPEN csr_worklist_access;
    FETCH csr_worklist_access INTO l_grant_uid,l_assignee;
    CLOSE csr_worklist_access;
    
    IF l_grant_uid IS NOT NULL THEN
      OPEN csr_vac_xml('WORKLIST_ACCESS',l_assignee);
      FETCH csr_vac_xml INTO l_xml;
      CLOSE csr_vac_xml;
    END IF;
  END IF;
  
  IF l_xml IS NULL THEN
    l_xml := XMLType('<VACATION_RULES_PERM>Y</VACATION_RULES_PERM>');
  END IF;
  RETURN l_xml;
  
END get_vacation_rules_xml;

-- ------------------------------------------------------------------------ *
-- Name    : team_absencences_list_int
-- ------------------------------------------------------------------------ *
FUNCTION team_absencences_list_int
  (p_absence_type       IN varchar2
  ,p_due_days           IN number
  ,p_saved_search_id    IN number
  ,p_auth_function      IN varchar2
  ,p_person_id          IN number) RETURN xxas_absence_data_varray PIPELINED
IS

  c_method  CONSTANT    varchar2(30) := 'team_absencences_list_int';
  l_abs_query           varchar2(32767);
  l_abs_select_query    varchar2(32767); 
  l_auth_func_predicate varchar2(32767);
  l_abs_row             xxas_absence_data;
  l_abs_rec             sys_refcursor;

BEGIN

  log_start(c_method,p_absence_type,p_due_days,p_saved_search_id,p_auth_function,p_person_id);

  l_auth_func_predicate := xxas_com_sec_pkg.get_security_predicate(p_auth_function);

  l_abs_select_query := 'SELECT xxas_absence_data
                                  (rslt.absence_attendance_id
                                  ,rslt.person_id
                                  ,rslt.transaction_id
                                  ,rslt.transaction_step_id
                                  ,rslt.date_start
                                  ,rslt.date_end
                                  ,rslt.workflow_status_meaning
                                  ,rslt.absence_type
                                  ,rslt.workflow_status_code
                                  ,rslt.notification_id
                                  ,rslt.absence_days
                                  ,rslt.absence_hours)
                         FROM ';

  IF p_absence_type is null THEN

    l_abs_select_query := l_abs_select_query || ' (SELECT abs.absence_attendance_id
                            ,person_id
                            ,null TRANSACTION_ID
                            ,null TRANSACTION_STEP_ID
                            ,abs.date_start
                            ,abs.date_end
                            ,null workflow_status_meaning
                            ,paat.name absence_type
                            ,null workflow_status_code
					        ,null notification_id
							,abs.absence_days
							,abs.absence_hours						 
							FROM   per_absence_attendances abs,
							PER_ABS_ATTENDANCE_TYPES_VL paat
							WHERE  abs.PERSON_ID IN 
							(
								select '|| p_person_id ||' from dual
								union all
								SELECT xxas_util_pkg.get_person_from_asg(rslts.column_value,trunc(sysdate)) 
								FROM TABLE(CAST(xxas_hierarchy_pkg.asgs_in_hier (''AR'' ,trunc(sysdate) ,'|| p_person_id ||') AS xxas_com_num_table)) rslts WHERE rownum >= 0
							)	
							and abs.absence_attendance_type_id = paat.absence_attendance_type_id
					        UNION
							SELECT 
								to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''AbsenceAttendanceId'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow''))  absence_attendance_id
								,hat.selected_person_id person_id
								,hat.TRANSACTION_ID
								,hats.TRANSACTION_STEP_ID
								,fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''DateStart'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'')) date_start
								,fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''DateEnd'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow''))   date_end
								,xxas_selfservice_workflow_pkg.decode_status(xxas_selfservice_workflow_pkg.transcode_status(hat.status))    workflow_status_meaning
								,(SELECT paat.name
									FROM   per_abs_attendance_types_vl paat
									WHERE  to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''AbsenceAttendanceTypeId'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'')) = absence_attendance_type_id) absence_type
								,xxas_selfservice_workflow_pkg.transcode_status(hat.status)    workflow_status_code
								,(select max(wn.notification_id) from wf_notifications wn, fnd_user fu  
									WHERE     hat.item_key        = wn.item_key
									AND    hat.item_type       = wn.message_type
									AND    wn.recipient_role   = fu.user_name
									AND    fu.employee_id      = '|| p_person_id ||'
									AND    wn.status           = ''OPEN'') notification_id
								,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''AbsenceDays'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'')) 	absence_days
								,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''AbsenceHours'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'')) absence_hours				   
								FROM HR_API_TRANSACTIONS hat,
								HR_API_TRANSACTION_STEPS hats
								WHERE 
									hat.selected_person_id IN 
									(
										select '|| p_person_id ||' from dual
										union all
										SELECT xxas_util_pkg.get_person_from_asg(rslts.column_value,trunc(sysdate)) 
										FROM TABLE(CAST(xxas_hierarchy_pkg.asgs_in_hier (''AR'' ,trunc(sysdate) ,'|| p_person_id ||') AS xxas_com_num_table)) rslts WHERE rownum >= 0
									)
								AND hat.transaction_id = hats.transaction_id
								AND    hat.status          NOT IN (''D'', ''N'', ''W'', ''ACTIVE'', ''AC'')
								AND    hats.api_name            = ''HR_PERSON_ABSENCE_SWI.PROCESS_API'' 
							) rslt ';
  
  ELSE

    l_abs_select_query := l_abs_select_query || ' (SELECT to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''AbsenceAttendanceId'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow''))  absence_attendance_id
						    ,hat.selected_person_id person_id
						    ,hat.TRANSACTION_ID
						    ,hats.TRANSACTION_STEP_ID
						    ,fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''DateStart'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'')) date_start
						    ,fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''DateEnd'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow''))   date_end
						    ,xxas_selfservice_workflow_pkg.decode_status
						     (xxas_selfservice_workflow_pkg.transcode_status
						     (hat.status))    workflow_status_meaning
						    ,(SELECT paat.name
						    	FROM   per_abs_attendance_types_vl paat
						    	WHERE  to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''AbsenceAttendanceTypeId'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'')) = absence_attendance_type_id) absence_type
						    ,xxas_selfservice_workflow_pkg.transcode_status
						    	(hat.status)    workflow_status_code
						    ,(select max(wn.notification_id) from wf_notifications wn, fnd_user fu  
								WHERE   hat.item_key        = wn.item_key
						    	 AND    hat.item_type       = wn.message_type
						    	 AND    wn.recipient_role   = fu.user_name
	                 AND    fu.employee_id      = '|| p_person_id ||'
						    	 AND    wn.status           = ''OPEN'') notification_id
						    ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''AbsenceDays'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'')) absence_days
						    ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, ''AbsenceHours'', ''Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'')) absence_hours				   
							FROM HR_API_TRANSACTIONS hat,
							HR_API_TRANSACTION_STEPS hats,
							wf_notifications wn,
							fnd_user fu
							WHERE hat.selected_person_id IN 
							(
								select '|| p_person_id ||' from dual
								union all
								SELECT xxas_util_pkg.get_person_from_asg(rslts.column_value,trunc(sysdate)) 
								FROM TABLE(CAST(xxas_hierarchy_pkg.asgs_in_hier (''AR'' ,trunc(sysdate) ,'|| p_person_id ||') AS xxas_com_num_table)) rslts WHERE rownum >= 0
							)
							AND hat.transaction_id = hats.transaction_id
							AND    hat.status NOT IN (''D'', ''N'', ''W'', ''ACTIVE'', ''AC'')
							AND    hats.api_name  = ''HR_PERSON_ABSENCE_SWI.PROCESS_API''
							AND    hat.item_key        = wn.item_key
              AND    hat.item_type       = wn.message_type
              AND    wn.recipient_role   = fu.user_name
              AND    fu.employee_id      = '|| p_person_id ||'
              AND    wn.status           = ''OPEN''							
							AND    ' || '''' || p_absence_type || '''' || ' = ''PENDING'' 
							AND xxas_selfservice_workflow_pkg.transcode_status(hat.status) = ''PENDING''
                  ) rslt ';

  END IF;

  l_abs_select_query := l_abs_select_query || ' WHERE ' || l_auth_func_predicate;

  l_abs_select_query := l_abs_select_query || ' AND ' || '( ' || '''' || p_absence_type || '''' || ' = ' || '''PENDING''' || ' OR '||  
                   '( ' || '''' || p_absence_type || '''' || ' is null AND ' || ' ((trunc(sysdate) BETWEEN rslt.date_start AND nvl(rslt.date_end,sysdate)) OR '||
                   '(rslt.date_start > trunc(sysdate) AND trunc(sysdate) + nvl(' || p_due_days || ',30) >= rslt.date_start)) ' ||
                   ') ' ||
                  ') ';

  IF p_saved_search_id is null THEN

    l_abs_select_query := l_abs_select_query || ' AND ' || ' rslt.person_id IN (SELECT reportees.column_value
                                       FROM   TABLE(xxas_com_hier_pkg.drs_for_per_tbl( ' || p_person_id || ',SYSDATE)) reportees
                                      ) ';

  ELSE

    l_abs_select_query := l_abs_select_query || ' AND ' || ' EXISTS (SELECT 1 
                            FROM   TABLE(xxas_per_search_pkg.people_tbl
                                           (null
                                           ,( SELECT xxas_saved_search_pkg.xml_to_obj(ss.search_parameters)
                                              FROM   xxas_saved_searches_v ss
                                              WHERE  ss.saved_search_id = ' || p_saved_search_id || '
                                              AND    xxas_com_sec_pkg.test_instance
                                                       (''XXAS_DATA_SAVED_SEARCH_VIEW'', ' || p_saved_search_id || ') = ''Y''
                                            )
                                           )
			                            ) rslts
                            WHERE  rslt.person_id IN (SELECT reportees.column_value
                                                      FROM   TABLE(xxas_com_hier_pkg.drs_for_per_tbl(rslts.person_id,SYSDATE)) reportees
                                                     )
                          ) ';

  END IF;

  OPEN l_abs_rec FOR l_abs_select_query;
  LOOP
    FETCH l_abs_rec INTO l_abs_row;
    EXIT WHEN l_abs_rec%NOTFOUND;
    PIPE ROW (l_abs_row);
  END LOOP;

  log_end(c_method);
  RETURN;

END team_absencences_list_int;

-- ------------------------------------------------------------------------ *
-- Name    : team_absencences_list
-- ------------------------------------------------------------------------ *
FUNCTION team_absencences_list
  (p_absence_type       IN varchar2
  ,p_due_days           IN number
  ,p_saved_search_id    IN number
  ,p_auth_function      IN varchar2
  ,p_person_id          IN number) RETURN XMLType
IS

  CURSOR csr_xml_data
  IS
  SELECT XMLElement
          ("data"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAGG(XMLElement
                   ("item"
                   ,XMLFOREST
                     ('worklist'                                            AS  "type"
                     ,rslt.notification_id                                  AS  "id"
                     ,rslt.absence_attendance_id                            AS  "absence_attendance_id"
                     ,xxas_perf_util_pkg.get_formatted_name(rslt.person_id) AS  "sender"
                     ,CASE  WHEN rslt.date_end = rslt.date_start THEN
                                 to_char(rslt.date_start,'Mon dd yyyy') 
                            ELSE 
                                 to_char(rslt.date_start,'Mon dd') 
                                 || ' - ' 
                                 || nvl(to_char(rslt.date_end,'Mon dd yyyy'),xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_ABS_ONGOING'))
                            END
                            || decode(rslt.absence_days
                                     ,null
                                     ,''
                                     ,1
                                     ,', 1 ' || xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_CAL_EVENT_DUR_DAY')
                                     , ', '  || (rslt.absence_days) || ' ' || xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_CAL_EVENT_DUR_DAYS')
                                     )
                            || decode(rslt.absence_hours
                                     ,null
                                     ,''
                                     ,1
                                     ,', 1 ' || xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_COM_HOUR')
                                     , ', '  || (rslt.absence_hours) || ' ' || xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_COM_HOURS')
                                     )
                            || ' - ' || rslt.absence_type                       AS "subject"
                     ,'#/absence-view/' || rslt.person_id || '_' || rslt.absence_attendance_id || '_'
                      || rslt.transaction_id || '_' || rslt.transaction_step_id || '__'  AS "href"
                     ,'modal'                                               AS "target"
                     )
                   ,CASE WHEN rslt.workflow_status_code IS NOT NULL 
                         THEN XMLElement
                               ("labels"
                               ,XMLAttributes
                                 ('true' as "json:force-array"
                                 ,'http://json.org/' as "xmlns:json")
                               ,XMLElement
                                 ("label"
                                 ,XMLForest
                                   (CASE rslt.workflow_status_code
                                         WHEN 'PENDING' THEN
                                                 xxas_rslt_cache_pkg.resolve_color_name('energized') 
                                         WHEN 'RFC' THEN
                                                 xxas_rslt_cache_pkg.resolve_color_name('assertive')
                                         WHEN 'ERROR' THEN
                                                 xxas_rslt_cache_pkg.resolve_color_name('assertive')
                                         WHEN 'APPROVED' THEN 
                                                 xxas_rslt_cache_pkg.resolve_color_name('green') 
                                    END                                              AS "color"
                                   ,rslt.workflow_status_meaning                     AS "title"
                                   )
                                 )
                               )
                    END
                   ,XMLELEMENT
                     ("linked"
                     ,CASE p_absence_type
                           WHEN 'PENDING' THEN
                                xxas_com_worklist_pkg.get_actions(rslt.notification_id,xxas_util_pkg.get_username_from_per(p_person_id,trunc(SYSDATE)))
                      END
                     ,XMLElement
                       ("image"
                       ,XMLElement
                         ("self"
                         ,xxas_com_rest_util_pkg.uri_from_path('people/' || rslt.person_id || '/photo'))
                        ,XMLForest
                          (xxas_perf_util_pkg.get_initials(rslt.person_id,rslt.date_start) "initials"
                          ,xxas_per_image_pkg.url(img.person_id, img.last_update_date, 'SMALL') "small"
                          ,xxas_per_image_pkg.url(img.person_id,img.last_update_date, 'MEDIUM') "medium"
                          ,xxas_per_image_pkg.url(img.person_id, img.last_update_date, 'FULL_SIZE') "fullSize"
                          )
                       )
                     )
                   )
                 ORDER BY date_start DESC)
          )
  FROM  TABLE(xxas_absence_event_pkg.team_absencences_list_int
                (p_absence_type       => p_absence_type
                ,p_due_days           => p_due_days
                ,p_saved_search_id    => p_saved_search_id
                ,p_auth_function      => p_auth_function
                ,p_person_id          => p_person_id)) rslt
       ,xxas_per_thumbnails img
  WHERE rslt.person_id = img.person_id (+);

  c_method  CONSTANT      varchar2(30) := 'team_absencences_list';
  l_absence_data_varray   xxas_absence_data_varray;
  l_xml_data              XMLType;

BEGIN

  log_start(c_method,p_absence_type,p_due_days,p_saved_search_id,p_auth_function,p_person_id);

  OPEN  csr_xml_data;
  FETCH csr_xml_data INTO l_xml_data;
  CLOSE csr_xml_data;

  log_end(c_method);
  RETURN l_xml_data;
  
END team_absencences_list;

END xxas_absence_event_pkg;
/
COMMIT;
EXIT;
