/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2016 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_absence_management_pkg IS

gc_log xxas_com_log_type := xxas_com_log_type('xxas_absence_management_pkg');

g_PACKAGE_NAME       CONSTANT varchar2(30):= 'xxas_absence_management_pkg';
g_PROD_KEY           CONSTANT varchar2(3)  := 'SSH';
g_HRAPI_DATETIME_FMT CONSTANT varchar2(30) := 'YYYY-MM-DD HH24:MI:SS".0"';
g_HRAPI_DATE_FMT     CONSTANT varchar2(30) := 'YYYY-MM-DD';
g_ITEM_TYPE                   varchar2(30);
g_AME_APP_ID                  number;
g_AME_TRANS_TYPE              varchar2(30);
g_PROCESS_NAME                varchar2(30);
c_insert             CONSTANT varchar2(30) := 'CreateMode';
c_update             CONSTANT varchar2(30) := 'UpdateMode';
c_delete             CONSTANT varchar2(30) := 'CancelMode';

TYPE workflow_info IS RECORD(item_type varchar2(10)
                            ,item_key  varchar2(240));

--
-- ------------------------------------------------------------------------ *
-- Name    : log_start
-- ------------------------------------------------------------------------ *
PROCEDURE log_start
  (p_method      IN  varchar2
  ,p_parameter1  IN  varchar2 DEFAULT null
  ,p_parameter2  IN  varchar2 DEFAULT null
  ,p_parameter3  IN  varchar2 DEFAULT null
  ,p_parameter4  IN  varchar2 DEFAULT null
  ,p_parameter5  IN  varchar2 DEFAULT null
  ,p_parameter6  IN  varchar2 DEFAULT null
  ,p_parameter7  IN  varchar2 DEFAULT null
  ,p_parameter8  IN  varchar2 DEFAULT null
  ,p_parameter9  IN  varchar2 DEFAULT null
  ,p_parameter10 IN  varchar2 DEFAULT null)
IS
BEGIN
   IF gc_log.proc=1
   THEN
     gc_log.procedure_start_params
       (p_proc_name   =>  p_method
       ,p_parameter1  =>  p_parameter1
       ,p_parameter2  =>  p_parameter2
       ,p_parameter3  =>  p_parameter3
       ,p_parameter4  =>  p_parameter4
       ,p_parameter5  =>  p_parameter5
       ,p_parameter6  =>  p_parameter6
       ,p_parameter7  =>  p_parameter7
       ,p_parameter8  =>  p_parameter8
       ,p_parameter9  =>  p_parameter9
       ,p_parameter10 =>  p_parameter10);
   END IF;
END log_start;

--
-- ------------------------------------------------------------------------ *
-- Name    : log_end
-- ------------------------------------------------------------------------ *
PROCEDURE log_end
  (p_method  IN  varchar2
  ,p_result  IN  varchar2 DEFAULT null)
IS
BEGIN
  IF gc_log.proc=1
  THEN
    gc_log.procedure_end_result
      (p_proc_name =>  p_method
      ,p_result    =>  p_result);
  END IF;
END log_end;

--
-- ------------------------------------------------------------------------ *
-- Name    : log_statement
-- ------------------------------------------------------------------------ *
PROCEDURE log_statement
  (p_method     IN  varchar2
  ,p_log_string IN  varchar2)
IS
BEGIN
  IF gc_log.stmt=1
  THEN
    gc_log.statement
      (p_proc_name  =>  p_method
      ,p_log_string =>  p_log_string);
  END IF;
END log_statement;

--
-- ------------------------------------------------------------------------ *
-- Name    : register_user_error
-- ------------------------------------------------------------------------ *
PROCEDURE register_user_error
  (p_package_name IN varchar2
  ,p_proc_name    IN varchar2
  ,p_message      IN varchar2)
IS

BEGIN

 xxas_com_api_result_pkg.register_user_error
   (p_package_name => p_package_name
   ,p_proc_name    => p_proc_name
   ,p_field_name   => null
   ,p_message      => xxas_com_msg_type
                        (xxas_util_pkg.app
                        ,'XXAS_COM_TEXT'
                        ,xxas_com_msg_token_varray
                          (xxas_com_msg_token_type
                            ('TEXT'
                            ,p_message))));
END register_user_error;

-- ------------------------------------------------------------------------ *
-- Name    : approval_required
-- ------------------------------------------------------------------------ *
FUNCTION approval_required RETURN boolean
IS

  l_PROC          CONSTANT varchar2(30) := 'approval_required';
  l_num_approvers          number;
  l_create_update_rslt_xml XMLType;

  CURSOR csr_count_approvers
  IS
  SELECT count(*)
  FROM   TABLE(XMLSequence(extract(l_create_update_rslt_xml
                                  ,'//Approver')));

BEGIN

    --
    -- Contains the workflow details including a list of approvers
    --
    l_create_update_rslt_xml := xxas_com_api_result_pkg.result_details;

    OPEN  csr_count_approvers;
    FETCH csr_count_approvers INTO l_num_approvers;
    CLOSE csr_count_approvers;

    RETURN (nvl(l_num_approvers, 0) > 0);

END approval_required;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_wf_attribute_text
-- ------------------------------------------------------------------------ *
FUNCTION get_wf_attribute_text
  (p_item_type      IN varchar2
  ,p_item_key       IN varchar2
  ,p_attribute_name IN varchar2) RETURN varchar2
IS

BEGIN

  IF p_item_type      IS NOT null AND
     p_item_key       IS NOT null AND
     p_attribute_name IS NOT null THEN
    RETURN wf_engine.GetItemAttrText
            (p_item_type, p_item_key, p_attribute_name, true);
  ELSE
    RETURN null;
  END IF;

END get_wf_attribute_text;

--
-- ------------------------------------------------------------------------ *
-- Name    : launch_workflow
-- ------------------------------------------------------------------------ *
PROCEDURE launch_workflow
  (p_person_id              IN  number
  ,p_transaction_id         IN  number
  ,p_comments_to_approver   IN  varchar2
  ,p_item_key       OUT NOCOPY  varchar2
  ,p_activity_id    OUT NOCOPY  varchar2)
IS

  c_method CONSTANT     varchar2(30) := 'launch_workflow';
  l_current_per_display varchar2(1000);

  CURSOR csr_person_name IS
  SELECT papf.full_name
  FROM   per_all_people_f papf
  WHERE  p_person_id IS NOT null
  AND    papf.person_id = p_person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;

  OPEN  csr_person_name;
  FETCH csr_person_name INTO l_current_per_display;
  CLOSE csr_person_name;

  xxas_selfservice_workflow_pkg.hr_generic_approval_prc_launch
    (p_transaction_id              => p_transaction_id
    ,p_item_type                   => g_ITEM_TYPE
    ,p_process_name                => g_PROCESS_NAME
    ,p_process_display_name        => xxas_com_msg_pkg.get_string
                                        ('PER|' || xxas_com_sec_pkg.get_function_parameter
                                                     (xxas_com_sec_pkg.get_function_parameter
                                                        (p_function_name  => xxas_emp_view_context_pkg.data_function
                                                        ,p_parameter_name => 'pCalledFrom'), 'pNtfSubMsg'))
    ,p_ame_application_id          => g_AME_APP_ID
    ,p_ame_transaction_type        => g_AME_TRANS_TYPE
    ,p_review_template_rn          => 'HR_ABS_NTF_SS'
    ,p_ntf_sub_fnd_msg             => xxas_com_sec_pkg.get_function_parameter
                                        (xxas_com_sec_pkg.get_function_parameter
                                           (p_function_name  => xxas_emp_view_context_pkg.data_function
                                           ,p_parameter_name => 'pCalledFrom'), 'pNtfSubMsg') --'HR_ABS_NTF_SUB_MSG'
    ,p_current_person_display_name => l_current_per_display
    ,p_comments_to_approver        => p_comments_to_approver
    ,p_notif_fyi_details          => 'Y'
    ,p_current_person_id           => p_person_id
    ,p_current_person_orig_sys     => 'PER'
    ,x_item_key                    => p_item_key
    ,x_activity_id                 => p_activity_id);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END launch_workflow;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_hrapi_transaction
-- ------------------------------------------------------------------------ *
FUNCTION create_hrapi_transaction
  (p_transaction_id        IN   number
  ,p_absence_attendance_id IN   number
  ,p_assignment_id         IN   number
  ,p_person_id             IN   number
  ,p_item_type             IN   varchar2
  ,p_item_key              IN   varchar2
  ,p_absence_trans_doc     IN   XMLType
  ,p_information1          IN   varchar2
  ,p_information2          IN   varchar2
  ,p_information3          IN   varchar2
  ,p_information4          IN   varchar2
  ,p_information5          IN   varchar2
  ,p_information6          IN   varchar2
  ,p_information7          IN   varchar2
  ,p_information8          IN   varchar2
  ,p_information9          IN   varchar2
  ,p_object_state          IN   varchar2) RETURN number
IS

  c_method CONSTANT     varchar2(30) := 'create_hrapi_transaction';
  l_transaction_id      number;
  l_transaction_step_id number;
  l_step_ovn            number;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;

  hr_trn_ins.set_base_key_value
   (p_transaction_id => p_transaction_id);

  hr_trn_ins.ins
   (p_transaction_id               => l_transaction_id
   ,p_creator_person_id            => fnd_global.employee_id
   ,p_transaction_privilege        => 'PRIVATE'
   ,p_product_code                 => 'PER'
   ,p_url                          => null
   ,p_status                       => 'W'
   ,p_section_display_name         => null
   ,p_function_id                  => fnd_function.get_function_id(xxas_com_sec_pkg.get_function_parameter
                                                                     (p_function_name  => xxas_emp_view_context_pkg.data_function
                                                                     ,p_parameter_name => 'pCalledFrom'))
   ,p_transaction_ref_table        => 'PER_ABSENCE_ATTENDANCES'
   ,p_transaction_ref_id           => p_absence_attendance_id
   ,p_transaction_type             => 'WF'
   ,p_assignment_id                => p_assignment_id
   ,p_api_addtnl_info              => null
   ,p_selected_person_id           => p_person_id
   ,p_item_type                    => p_item_type
   ,p_item_key                     => p_item_key
   ,p_transaction_effective_date   => trunc(sysdate)
   ,p_process_name                 => g_PROCESS_NAME
   ,p_plan_id                      => null
   ,p_rptg_grp_id                  => null
   ,p_effective_date_option        => null
   ,p_validate                     => false
   ,p_creator_role                 => 'PER:' || fnd_global.employee_id
   ,p_last_update_role             => 'PER:' || fnd_global.employee_id
   ,p_parent_transaction_id        => null
   ,p_relaunch_function            => 'HR_ABS_ENTRY_PAGE_SS'
   ,p_transaction_group            => 'ABSENCE_MGMT'
   ,p_transaction_identifier       => 'ABSENCES'
   ,p_transaction_document         => p_absence_trans_doc.getCLOBVal());

  hr_trs_ins.ins
    (p_transaction_step_id          => l_transaction_step_id
    ,p_transaction_id               => p_transaction_id
    ,p_api_name                     => 'HR_PERSON_ABSENCE_SWI.PROCESS_API'
    ,p_processing_order             => 0
    ,p_item_type                    => p_item_type
    ,p_creator_person_id            => fnd_global.employee_id
    ,p_update_person_id             => -1
    ,p_object_version_number        => l_step_ovn
    ,p_validate                     => false
    ,p_object_type                  => 'ENTITY'
    ,p_object_name                  => 'oracle.apps.per.schema.server.PerAbsenceAttendancesEO'
    ,p_object_identifier            => xxas_selfservice_workflow_pkg.get_hex_key
                                         (p_absence_attendance_id)
    ,p_object_state                 => p_object_state
    ,p_pk1                          => p_absence_attendance_id
    ,p_information_category         => null
    ,p_information1                 => p_information1
    ,p_information2                 => p_information2
    ,p_information3                 => p_information3
    ,p_information4                 => p_information4
    ,p_information5                 => p_information5
    ,p_information6                 => p_information6
    ,p_information7                 => p_information7
    ,p_information8                 => p_information8
    ,p_information9                 => p_information9
    ,p_information30                => 'ATT');

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

  RETURN l_transaction_step_id;

END create_hrapi_transaction;

--
-- ------------------------------------------------------------------------ *
-- Name    : update_hrapi_transaction
-- ------------------------------------------------------------------------ *
PROCEDURE update_hrapi_transaction
  (p_transaction_id        IN   number
  ,p_transaction_step_id   IN   number
  ,p_absence_attendance_id IN   number
  ,p_assignment_id         IN   number
  ,p_person_id             IN   number
  ,p_item_type             IN   varchar2
  ,p_item_key              IN   varchar2
  ,p_absence_trans_doc     IN   XMLType
  ,p_information1          IN   varchar2
  ,p_information2          IN   varchar2
  ,p_information3          IN   varchar2
  ,p_information4          IN   varchar2
  ,p_information5          IN   varchar2
  ,p_information6          IN   varchar2
  ,p_information7          IN   varchar2
  ,p_information8          IN   varchar2
  ,p_information9          IN   varchar2
  ,p_object_state          IN   varchar2)
IS

  CURSOR csr_trans_step_obj_ver_number IS
  SELECT hats.object_version_number
  FROM   hr_api_transaction_steps hats
  WHERE  hats.transaction_step_id = p_transaction_step_id
  AND    hats.transaction_id      = p_transaction_id;

  c_method CONSTANT     varchar2(30) := 'update_hrapi_transaction';
  l_step_ovn            number;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;

  hr_trn_upd.upd
   (p_transaction_id               => p_transaction_id
   ,p_creator_person_id            => CASE WHEN xxas_com_worklist_pkg.is_resubmit(p_transaction_step_id) = 'Y' THEN
                                        p_person_id
									  ELSE
									    fnd_global.employee_id
								      END
   ,p_transaction_privilege        => 'PRIVATE'
   ,p_product_code                 => 'PER'
   ,p_url                          => null
   ,p_status                       => 'W'
   ,p_section_display_name         => null
   ,p_function_id                  => fnd_function.get_function_id(xxas_com_sec_pkg.get_function_parameter
                                                                     (p_function_name  => xxas_emp_view_context_pkg.data_function
                                                                     ,p_parameter_name => 'pCalledFrom'))
   ,p_transaction_ref_table        => 'PER_ABSENCE_ATTENDANCES'
   ,p_transaction_ref_id           => p_absence_attendance_id
   ,p_transaction_type             => 'WF'
   ,p_assignment_id                => p_assignment_id
   ,p_api_addtnl_info              => null
   ,p_selected_person_id           => p_person_id
   ,p_item_type                    => p_item_type
   ,p_item_key                     => p_item_key
   ,p_transaction_effective_date   => trunc(sysdate)
   ,p_process_name                 => g_PROCESS_NAME
   ,p_plan_id                      => null
   ,p_rptg_grp_id                  => null
   ,p_effective_date_option        => null
   ,p_validate                     => false
   ,p_creator_role                 => 'PER:' || fnd_global.employee_id
   ,p_last_update_role             => 'PER:' || fnd_global.employee_id
   ,p_parent_transaction_id        => null
   ,p_relaunch_function            => 'HR_ABS_ENTRY_PAGE_SS'
   ,p_transaction_group            => 'ABSENCE_MGMT'
   ,p_transaction_identifier       => 'ABSENCES'
   ,p_transaction_document         => p_absence_trans_doc.getCLOBVal());

  OPEN  csr_trans_step_obj_ver_number;
  FETCH csr_trans_step_obj_ver_number INTO l_step_ovn;
  CLOSE csr_trans_step_obj_ver_number;

  hr_trs_upd.upd
    (p_transaction_step_id          => p_transaction_step_id
    ,p_transaction_id               => p_transaction_id
    ,p_api_name                     => 'HR_PERSON_ABSENCE_SWI.PROCESS_API'
    ,p_processing_order             => 0
    ,p_item_type                    => p_item_type
    ,p_creator_person_id            => CASE WHEN xxas_com_worklist_pkg.is_resubmit(p_transaction_step_id) = 'Y' THEN
                                        p_person_id
									  ELSE
									    fnd_global.employee_id
								      END
    ,p_update_person_id             => -1
    ,p_object_version_number        => l_step_ovn
    ,p_validate                     => false
    ,p_object_type                  => 'ENTITY'
    ,p_object_name                  => 'oracle.apps.per.schema.server.PerAbsenceAttendancesEO'
    ,p_object_identifier            => xxas_selfservice_workflow_pkg.get_hex_key
                                         (p_absence_attendance_id)
    ,p_object_state                 => p_object_state
    ,p_pk1                          => p_absence_attendance_id
    ,p_information_category         => null
    ,p_information1                 => p_information1
    ,p_information2                 => p_information2
    ,p_information3                 => p_information3
    ,p_information4                 => p_information4
    ,p_information5                 => p_information5
    ,p_information6                 => p_information6
    ,p_information7                 => p_information7
    ,p_information8                 => p_information8
    ,p_information9                 => p_information9
    ,p_information30                => 'ATT');

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END update_hrapi_transaction;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_emp_view_row
-- ------------------------------------------------------------------------ *
PROCEDURE create_emp_view_row
  (p_effective_date           IN     date
  ,p_assignment_id            IN     number
  ,p_absence_type_id          IN     number
  ,p_override_assignment_id   IN     number   DEFAULT NULL
  ,p_abs_attendance_reason_id IN     number   DEFAULT null
  ,p_date_notification        IN     date     DEFAULT null
  ,p_date_projected_start     IN     date     DEFAULT null
  ,p_time_projected_start     IN     varchar2 DEFAULT null
  ,p_date_projected_end       IN     date     DEFAULT null
  ,p_time_projected_end       IN     varchar2 DEFAULT null
  ,p_date_start               IN     date     DEFAULT null
  ,p_time_start               IN     varchar2 DEFAULT null
  ,p_date_end                 IN     date     DEFAULT null
  ,p_time_end                 IN     varchar2 DEFAULT null
  ,p_absence_days             IN     number   DEFAULT null
  ,p_absence_hours            IN     number   DEFAULT null
  ,p_authorising_person_id    IN     number   DEFAULT null
  ,p_replacement_person_id    IN     number   DEFAULT null
  ,p_attribute_category       IN     varchar2 DEFAULT null
  ,p_attribute1               IN     varchar2 DEFAULT null
  ,p_attribute2               IN     varchar2 DEFAULT null
  ,p_attribute3               IN     varchar2 DEFAULT null
  ,p_attribute4               IN     varchar2 DEFAULT null
  ,p_attribute5               IN     varchar2 DEFAULT null
  ,p_attribute6               IN     varchar2 DEFAULT null
  ,p_attribute7               IN     varchar2 DEFAULT null
  ,p_attribute8               IN     varchar2 DEFAULT null
  ,p_attribute9               IN     varchar2 DEFAULT null
  ,p_attribute10              IN     varchar2 DEFAULT null
  ,p_attribute11              IN     varchar2 DEFAULT null
  ,p_attribute12              IN     varchar2 DEFAULT null
  ,p_attribute13              IN     varchar2 DEFAULT null
  ,p_attribute14              IN     varchar2 DEFAULT null
  ,p_attribute15              IN     varchar2 DEFAULT null
  ,p_attribute16              IN     varchar2 DEFAULT null
  ,p_attribute17              IN     varchar2 DEFAULT null
  ,p_attribute18              IN     varchar2 DEFAULT null
  ,p_attribute19              IN     varchar2 DEFAULT null
  ,p_attribute20              IN     varchar2 DEFAULT null
  ,p_abs_information_category IN     varchar2 DEFAULT null
  ,p_abs_information1         IN     varchar2 DEFAULT null
  ,p_abs_information2         IN     varchar2 DEFAULT null
  ,p_abs_information3         IN     varchar2 DEFAULT null
  ,p_abs_information4         IN     varchar2 DEFAULT null
  ,p_abs_information5         IN     varchar2 DEFAULT null
  ,p_abs_information6         IN     varchar2 DEFAULT null
  ,p_abs_information7         IN     varchar2 DEFAULT null
  ,p_abs_information8         IN     varchar2 DEFAULT null
  ,p_abs_information9         IN     varchar2 DEFAULT null
  ,p_abs_information10        IN     varchar2 DEFAULT null
  ,p_abs_information11        IN     varchar2 DEFAULT null
  ,p_abs_information12        IN     varchar2 DEFAULT null
  ,p_abs_information13        IN     varchar2 DEFAULT null
  ,p_abs_information14        IN     varchar2 DEFAULT null
  ,p_abs_information15        IN     varchar2 DEFAULT null
  ,p_abs_information16        IN     varchar2 DEFAULT null
  ,p_abs_information17        IN     varchar2 DEFAULT null
  ,p_abs_information18        IN     varchar2 DEFAULT null
  ,p_abs_information19        IN     varchar2 DEFAULT null
  ,p_abs_information20        IN     varchar2 DEFAULT null
  ,p_abs_information21        IN     varchar2 DEFAULT null
  ,p_abs_information22        IN     varchar2 DEFAULT null
  ,p_abs_information23        IN     varchar2 DEFAULT null
  ,p_abs_information24        IN     varchar2 DEFAULT null
  ,p_abs_information25        IN     varchar2 DEFAULT null
  ,p_abs_information26        IN     varchar2 DEFAULT null
  ,p_abs_information27        IN     varchar2 DEFAULT null
  ,p_abs_information28        IN     varchar2 DEFAULT null
  ,p_abs_information29        IN     varchar2 DEFAULT null
  ,p_abs_information30        IN     varchar2 DEFAULT null
  ,p_leave_status             IN     varchar2 DEFAULT 'CONFIRMED'
  ,p_comments_to_approver     IN     varchar2 DEFAULT null
  ,p_transaction_id           IN OUT number
  ,p_transaction_step_id      IN OUT number)

IS

  CURSOR csr_asg IS
  SELECT paaf.*
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.assignment_id = NVL(p_override_assignment_id,p_assignment_id)
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

  CURSOR csr_person IS
  SELECT papf.*
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.assignment_id = NVL(p_override_assignment_id,p_assignment_id)
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

  CURSOR csr_login_person(p_login_person_id IN number) IS  --What If multiple assignments are there ??
  SELECT papf.*, paaf.supervisor_id, paaf.assignment_number
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.primary_flag = 'Y'
  AND    papf.person_id = p_login_person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

  CURSOR csr_get_emp_full_name(p_person_id number) IS
  SELECT decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'),'G', global_name, local_name) full_name
  FROM   per_all_people_f
  WHERE  person_id = p_person_id
  AND    trunc(sysdate) BETWEEN
         effective_start_date AND effective_end_date;

  CURSOR csr_get_sec_grp_leg_code(p_person_id number) IS
  SELECT bg.security_group_id
        ,bg.legislation_code
  FROM   per_all_people_f papf
        ,per_business_groups_perf bg
  WHERE  papf.person_id = p_person_id
  AND    papf.business_group_id = bg.business_group_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date and papf.effective_end_date;

  CURSOR csr_get_job_name(p_job_id number) IS
  SELECT name
  FROM   per_jobs_vl
  WHERE  job_id = p_job_id
  AND    trunc(sysdate) BETWEEN date_from AND nvl(date_to,trunc(sysdate));

  CURSOR csr_get_pos_name(p_pos_id number) IS
  SELECT name
  FROM   hr_all_positions_f_vl
  WHERE  position_id = p_pos_id
  AND    trunc(sysdate) BETWEEN nvl(effective_start_date,trunc(sysdate)) AND nvl(effective_end_date,trunc(sysdate));

  CURSOR csr_get_org_info(p_person_id number) IS
  SELECT org_information2
        ,org_information10
  FROM   hr_organization_information_v
  WHERE  org_information_context = 'Business Group Information'
  AND    organization_id         = xxas_per_pkg.get_business_group_id(p_person_id);

  CURSOR csr_get_abs_att_type_meaning(p_abs_att_type_id number) IS
  SELECT fcl.meaning
  FROM   per_abs_attendance_types_vl paat
        ,hr_lookups fcl
  WHERE  absence_attendance_type_id = p_abs_att_type_id
  AND    paat.absence_category    = fcl.lookup_code(+)
  AND    fcl.lookup_type(+)       = 'ABSENCE_CATEGORY';

  CURSOR csr_trans_doc(p_person_id                 IN number
                      ,p_absence_attendance_id     IN number
                      ,l_per_row                   IN csr_person%ROWTYPE
                      ,l_asg_row                   IN csr_asg%ROWTYPE
                      ,l_login_per_row             IN csr_login_person%ROWTYPE
                      ,p_per_manager_name          IN varchar2
                      ,p_login_per_manager_name    IN varchar2
                      ,p_transaction_id            IN number
                      ,p_per_sec_group_id          IN number
                      ,p_per_leg_code              IN varchar2
                      ,p_login_per_leg_code        IN varchar2
                      ,p_job_name                  IN varchar2
                      ,p_pos_name                  IN varchar2
                      ,p_emp_generation            IN varchar2
                      ,p_per_bg_cur_code           IN varchar2
                      ,p_called_from               IN varchar2
                      ,cp_date_projected_start     IN date
                      ,cp_time_projected_start     IN varchar2
                      ,cp_date_projected_end       IN date
                      ,cp_time_projected_end       IN varchar2
                      ,cp_date_start               IN date
                      ,cp_time_start               IN varchar2
                      ,cp_date_end                 IN date
                      ,cp_time_end                 IN varchar2
                      ,cp_absence_days             IN number
                      ,cp_absence_hours            IN number
					  ,cp_attribute1               IN varchar2
                      ,cp_attribute2               IN varchar2
                      ,cp_attribute3               IN varchar2
                      ,cp_attribute4               IN varchar2
                      ,cp_attribute5               IN varchar2
                      ,cp_attribute6               IN varchar2
                      ,cp_attribute7               IN varchar2
                      ,cp_attribute8               IN varchar2
                      ,cp_attribute9               IN varchar2
                      ,cp_attribute10              IN varchar2
                      ,cp_attribute11              IN varchar2
                      ,cp_attribute12              IN varchar2
                      ,cp_attribute13              IN varchar2
                      ,cp_attribute14              IN varchar2
                      ,cp_attribute15              IN varchar2
                      ,cp_attribute16              IN varchar2
                      ,cp_attribute17              IN varchar2
                      ,cp_attribute18              IN varchar2
                      ,cp_attribute19              IN varchar2
                      ,cp_attribute20              IN varchar2
                      ,cp_abs_information_category IN varchar2
                      ,cp_abs_information1         IN varchar2
                      ,cp_abs_information2         IN varchar2
                      ,cp_abs_information3         IN varchar2
                      ,cp_abs_information4         IN varchar2
                      ,cp_abs_information5         IN varchar2
                      ,cp_abs_information6         IN varchar2
                      ,cp_abs_information7         IN varchar2
                      ,cp_abs_information8         IN varchar2
                      ,cp_abs_information9         IN varchar2
                      ,cp_abs_information10        IN varchar2
                      ,cp_abs_information11        IN varchar2
                      ,cp_abs_information12        IN varchar2
                      ,cp_abs_information13        IN varchar2
                      ,cp_abs_information14        IN varchar2
                      ,cp_abs_information15        IN varchar2
                      ,cp_abs_information16        IN varchar2
                      ,cp_abs_information17        IN varchar2
                      ,cp_abs_information18        IN varchar2
                      ,cp_abs_information19        IN varchar2
                      ,cp_abs_information20        IN varchar2
                      ,cp_abs_information21        IN varchar2
                      ,cp_abs_information22        IN varchar2
                      ,cp_abs_information23        IN varchar2
                      ,cp_abs_information24        IN varchar2
                      ,cp_abs_information25        IN varchar2
                      ,cp_abs_information26        IN varchar2
                      ,cp_abs_information27        IN varchar2
                      ,cp_abs_information28        IN varchar2
                      ,cp_abs_information29        IN varchar2
                      ,cp_abs_information30        IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", p_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", l_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", p_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", g_ITEM_TYPE)
             ,XMLElement("PrsnJobId", l_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(l_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", l_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), p_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", p_login_per_leg_code)
             ,XMLElement("PrsnMgrId", l_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", NVL(p_override_assignment_id,p_assignment_id))
             ,XMLElement("PrsnLegCode", p_per_leg_code)
             ,XMLElement("LoginPrsnId", l_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", 'PER')
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(p_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,fnd_function.get_function_id(p_called_from)
               )
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", p_transaction_id)
             ,XMLElement("PrsnLocationId", l_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", p_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", p_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", p_per_manager_name)
             ,XMLElement("PrsnOrganizationId", l_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", l_asg_row.position_id)
             ,XMLElement("TxnStatus", 'W')
             ,XMLElement("LoginWorkerNumber", l_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", p_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", p_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", l_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(p_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_login_per_row.global_name, l_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), l_login_per_row.person_id)
             ,XMLElement("PrsnId", p_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || p_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", g_AME_TRANS_TYPE)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(p_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", p_called_from)
             ,XMLElement("LoginPrsnBgId", l_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", p_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", l_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_per_row.global_name, l_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", p_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CreateMode'))
             ,XMLElement("pAMEAppId", g_AME_APP_ID)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (0 as "PS"
                       ,'Y' as "PK")
                       ,XMLElement("AbsenceAttendanceId", p_absence_attendance_id)
                       ,XMLElement("BusinessGroupId", l_asg_row.business_group_id)
                       ,XMLElement("AbsenceAttendanceTypeId", p_absence_type_id)
                       ,XMLElement("AbsAttendanceReasonId", p_abs_attendance_reason_id)
                       ,XMLElement("PersonId", p_person_id)
                       ,decode(cp_absence_days, null, XMLElement("AbsenceDays", XMLAttributes('true' as "null"), cp_absence_days), XMLElement("AbsenceDays", cp_absence_days))
                       ,decode(cp_absence_hours, null, XMLElement("AbsenceHours", XMLAttributes('true' as "null"), cp_absence_hours), XMLElement("AbsenceHours", cp_absence_hours))
                       ,decode(cp_date_end, null, XMLElement("DateEnd", XMLAttributes('true' as "null"), cp_date_end), XMLElement("DateEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT)))
                       ,XMLElement("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,decode(cp_date_projected_end, null, XMLElement("DateProjectedEnd", XMLAttributes('true' as "null"), cp_date_projected_end), XMLElement("DateProjectedEnd", to_char(cp_date_projected_end, g_HRAPI_DATE_FMT)))
                       ,decode(cp_date_projected_start, null, XMLElement("DateProjectedStart", XMLAttributes('true' as "null"), cp_date_projected_start), XMLElement("DateProjectedStart", to_char(cp_date_projected_start, g_HRAPI_DATE_FMT)))
                       ,decode(cp_date_start, null, XMLElement("DateStart", XMLAttributes('true' as "null"), cp_date_start), XMLElement("DateStart", to_char(cp_date_start, g_HRAPI_DATE_FMT)))
                       ,decode(cp_time_end, null, XMLElement("TimeEnd", XMLAttributes('true' as "null"), cp_time_end), XMLElement("TimeEnd", cp_time_end))
                       ,decode(cp_time_projected_end, null, XMLElement("TimeProjectedEnd", XMLAttributes('true' as "null"), cp_time_projected_end), XMLElement("TimeProjectedEnd", cp_time_projected_end))
                       ,decode(cp_time_projected_start, null, XMLElement("TimeProjectedStart", XMLAttributes('true' as "null"), cp_time_projected_start), XMLElement("TimeProjectedStart", cp_time_projected_start))
                       ,decode(cp_time_start, null, XMLElement("TimeStart", XMLAttributes('true' as "null"), cp_time_start), XMLElement("TimeStart", cp_time_start))
                       ,decode(p_replacement_person_id, null, XMLElement("ReplacementPersonId", XMLAttributes('true' as "null"), p_replacement_person_id), XMLElement("ReplacementPersonId", p_replacement_person_id))
					   ,decode
                         (p_attribute_category, null, null
                         ,XMLElement
                           ("AttributeCategory", p_attribute_category))
                       ,decode
                         (p_attribute1, null, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (p_attribute2, null, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (p_attribute3, null, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (p_attribute4, null, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (p_attribute5, null, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (p_attribute6, null, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (p_attribute7, null, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (p_attribute8, null, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (p_attribute9, null, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (p_attribute10, null, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (p_attribute11, null, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (p_attribute12, null, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (p_attribute13, null, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (p_attribute14, null, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (p_attribute15, null, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (p_attribute16, null, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (p_attribute17, null, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (p_attribute18, null, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (p_attribute19, null, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (p_attribute20, null, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (p_abs_information_category, null, null
                         ,XMLElement
                           ("AbsInformationCategory", cp_abs_information_category))
                       ,decode
                         (p_abs_information1, null, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (p_abs_information2, null, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (p_abs_information3, null, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (p_abs_information4, null, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (p_abs_information5, null, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (p_abs_information6, null, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (p_abs_information7, null, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (p_abs_information8, null, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (p_abs_information9, null, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (p_abs_information10, null, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (p_abs_information11, null, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (p_abs_information12, null, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (p_abs_information13, null, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (p_abs_information14, null, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (p_abs_information15, null, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (p_abs_information16, null, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (p_abs_information17, null, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (p_abs_information18, null, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (p_abs_information19, null, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (p_abs_information20, null, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (p_abs_information21, null, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (p_abs_information22, null, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (p_abs_information23, null, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (p_abs_information24, null, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (p_abs_information25, null, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (p_abs_information26, null, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (p_abs_information27, null, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (p_abs_information28, null, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (p_abs_information29, null, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (p_abs_information30, null, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("LastUpdateDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("LastUpdatedBy", fnd_global.user_id)
                       ,XMLElement("LastUpdateLogin", fnd_global.login_id)
                       ,XMLElement("CreatedBy", fnd_global.user_id)
                       ,XMLElement("CreationDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"), null)
                       ,decode(p_comments_to_approver, null, XMLElement("Comments", XMLAttributes('true' as "null"), p_comments_to_approver), XMLElement("Comments", p_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_transaction_id))
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_rc_trans_doc(p_person_id                 IN number
                         ,p_absence_attendance_id     IN number
                         ,l_per_row                   IN csr_person%ROWTYPE
                         ,l_asg_row                   IN csr_asg%ROWTYPE
                         ,l_login_per_row             IN csr_login_person%ROWTYPE
                         ,p_per_manager_name          IN varchar2
                         ,p_login_per_manager_name    IN varchar2
                         ,p_transaction_id            IN number
                         ,p_per_sec_group_id          IN number
                         ,p_per_leg_code              IN varchar2
                         ,p_login_per_leg_code        IN varchar2
                         ,p_job_name                  IN varchar2
                         ,p_pos_name                  IN varchar2
                         ,p_emp_generation            IN varchar2
                         ,p_per_bg_cur_code           IN varchar2
                         ,p_called_from               IN varchar2
                         ,cp_date_projected_start     IN date
                         ,cp_time_projected_start     IN varchar2
                         ,cp_date_projected_end       IN date
                         ,cp_time_projected_end	      IN varchar2
                         ,cp_date_start	              IN date
                         ,cp_time_start	              IN varchar2
                         ,cp_date_end	              IN date
                         ,cp_time_end                 IN varchar2
                         ,cp_absence_days             IN number
                         ,cp_absence_hours            IN number
                         ,cp_attribute1               IN varchar2
                         ,cp_attribute2               IN varchar2
                         ,cp_attribute3               IN varchar2
                         ,cp_attribute4               IN varchar2
                         ,cp_attribute5               IN varchar2
                         ,cp_attribute6               IN varchar2
                         ,cp_attribute7               IN varchar2
                         ,cp_attribute8               IN varchar2
                         ,cp_attribute9               IN varchar2
                         ,cp_attribute10              IN varchar2
                         ,cp_attribute11              IN varchar2
                         ,cp_attribute12              IN varchar2
                         ,cp_attribute13              IN varchar2
                         ,cp_attribute14              IN varchar2
                         ,cp_attribute15              IN varchar2
                         ,cp_attribute16              IN varchar2
                         ,cp_attribute17              IN varchar2
                         ,cp_attribute18              IN varchar2
                         ,cp_attribute19              IN varchar2
                         ,cp_attribute20              IN varchar2
                         ,cp_abs_information_category IN varchar2
                         ,cp_abs_information1         IN varchar2
                         ,cp_abs_information2         IN varchar2
                         ,cp_abs_information3         IN varchar2
                         ,cp_abs_information4         IN varchar2
                         ,cp_abs_information5         IN varchar2
                         ,cp_abs_information6         IN varchar2
                         ,cp_abs_information7         IN varchar2
                         ,cp_abs_information8         IN varchar2
                         ,cp_abs_information9         IN varchar2
                         ,cp_abs_information10        IN varchar2
                         ,cp_abs_information11        IN varchar2
                         ,cp_abs_information12        IN varchar2
                         ,cp_abs_information13        IN varchar2
                         ,cp_abs_information14        IN varchar2
                         ,cp_abs_information15        IN varchar2
                         ,cp_abs_information16        IN varchar2
                         ,cp_abs_information17        IN varchar2
                         ,cp_abs_information18        IN varchar2
                         ,cp_abs_information19        IN varchar2
                         ,cp_abs_information20        IN varchar2
                         ,cp_abs_information21        IN varchar2
                         ,cp_abs_information22        IN varchar2
                         ,cp_abs_information23        IN varchar2
                         ,cp_abs_information24        IN varchar2
                         ,cp_abs_information25        IN varchar2
                         ,cp_abs_information26        IN varchar2
                         ,cp_abs_information27        IN varchar2
                         ,cp_abs_information28        IN varchar2
                         ,cp_abs_information29        IN varchar2
                         ,cp_abs_information30        IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", p_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", l_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", p_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", g_ITEM_TYPE)
             ,XMLElement("PrsnJobId", l_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(l_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", l_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), p_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", p_login_per_leg_code)
             ,XMLElement("PrsnMgrId", l_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", NVL(p_override_assignment_id,p_assignment_id))
             ,XMLElement("PrsnLegCode", p_per_leg_code)
             ,XMLElement("LoginPrsnId", l_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", 'PER')
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(p_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,fnd_function.get_function_id(p_called_from)
               )
             ,XMLElement("TxnState", 'T')
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", p_transaction_id)
             ,XMLElement("PrsnLocationId", l_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", p_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", p_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", p_per_manager_name)
             ,XMLElement("PrsnOrganizationId", l_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", l_asg_row.position_id)
             ,XMLElement("TxnStatus", 'RI')
             ,XMLElement("LoginWorkerNumber", l_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", p_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", p_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", l_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(p_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_login_per_row.global_name, l_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), l_login_per_row.person_id)
             ,XMLElement("PrsnId", p_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || p_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", g_AME_TRANS_TYPE)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(p_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", p_called_from)
             ,XMLElement("LoginPrsnBgId", l_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", p_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", l_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_per_row.global_name, l_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", p_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CreateMode'))
             ,XMLElement("pAMEAppId", g_AME_APP_ID)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (0 as "PS"
                       ,'Y' as "PK")
                       ,XMLElement("AbsenceAttendanceId", p_absence_attendance_id)
                       ,XMLElement("BusinessGroupId", l_asg_row.business_group_id)
                       ,XMLElement("AbsenceAttendanceTypeId", p_absence_type_id)
                       ,XMLElement("AbsAttendanceReasonId", p_abs_attendance_reason_id)
                       ,XMLElement("PersonId", p_person_id)
                       ,decode(cp_absence_days, null, XMLElement("AbsenceDays", XMLAttributes('true' as "null"), cp_absence_days), XMLElement("AbsenceDays", cp_absence_days))
                       ,decode(cp_absence_hours, null, XMLElement("AbsenceHours", XMLAttributes('true' as "null"), cp_absence_hours), XMLElement("AbsenceHours", cp_absence_hours))
                       ,decode(cp_date_end, null, XMLElement("DateEnd", XMLAttributes('true' as "null"), cp_date_end), XMLElement("DateEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT)))
                       ,XMLElement("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,decode(cp_date_projected_end, null, XMLElement("DateProjectedEnd", XMLAttributes('true' as "null"), cp_date_projected_end), XMLElement("DateProjectedEnd", to_char(cp_date_projected_end, g_HRAPI_DATE_FMT)))
                       ,decode(cp_date_projected_start, null, XMLElement("DateProjectedStart", XMLAttributes('true' as "null"), cp_date_projected_start), XMLElement("DateProjectedStart", to_char(cp_date_projected_start, g_HRAPI_DATE_FMT)))
                       ,decode(cp_date_start, null, XMLElement("DateStart", XMLAttributes('true' as "null"), cp_date_start), XMLElement("DateStart", to_char(cp_date_start, g_HRAPI_DATE_FMT)))
                       ,decode(cp_time_end, null, XMLElement("TimeEnd", XMLAttributes('true' as "null"), cp_time_end), XMLElement("TimeEnd", cp_time_end))
                       ,decode(cp_time_projected_end, null, XMLElement("TimeProjectedEnd", XMLAttributes('true' as "null"), cp_time_projected_end), XMLElement("TimeProjectedEnd", cp_time_projected_end))
                       ,decode(cp_time_projected_start, null, XMLElement("TimeProjectedStart", XMLAttributes('true' as "null"), cp_time_projected_start), XMLElement("TimeProjectedStart", cp_time_projected_start))
                       ,decode(cp_time_start, null, XMLElement("TimeStart", XMLAttributes('true' as "null"), cp_time_start), XMLElement("TimeStart", cp_time_start))
                       ,decode(p_replacement_person_id, null, XMLElement("ReplacementPersonId", XMLAttributes('true' as "null"), p_replacement_person_id), XMLElement("ReplacementPersonId", p_replacement_person_id))
					   ,decode
                         (p_attribute_category, null, null
                         ,XMLElement
                           ("AttributeCategory", p_attribute_category))
                       ,decode
                         (p_attribute1, null, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (p_attribute2, null, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (p_attribute3, null, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (p_attribute4, null, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (p_attribute5, null, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (p_attribute6, null, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (p_attribute7, null, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (p_attribute8, null, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (p_attribute9, null, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (p_attribute10, null, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (p_attribute11, null, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (p_attribute12, null, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (p_attribute13, null, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (p_attribute14, null, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (p_attribute15, null, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (p_attribute16, null, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (p_attribute17, null, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (p_attribute18, null, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (p_attribute19, null, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (p_attribute20, null, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (p_abs_information_category, null, null
                         ,XMLElement
                           ("AbsInformationCategory", p_abs_information_category))
                       ,decode
                         (p_abs_information1, null, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (p_abs_information2, null, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (p_abs_information3, null, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (p_abs_information4, null, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (p_abs_information5, null, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (p_abs_information6, null, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (p_abs_information7, null, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (p_abs_information8, null, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (p_abs_information9, null, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (p_abs_information10, null, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (p_abs_information11, null, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (p_abs_information12, null, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (p_abs_information13, null, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (p_abs_information14, null, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (p_abs_information15, null, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (p_abs_information16, null, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (p_abs_information17, null, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (p_abs_information18, null, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (p_abs_information19, null, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (p_abs_information20, null, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (p_abs_information21, null, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (p_abs_information22, null, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (p_abs_information23, null, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (p_abs_information24, null, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (p_abs_information25, null, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (p_abs_information26, null, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (p_abs_information27, null, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (p_abs_information28, null, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (p_abs_information29, null, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (p_abs_information30, null, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("LastUpdateDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("LastUpdatedBy", fnd_global.user_id)
                       ,XMLElement("LastUpdateLogin", fnd_global.login_id)
                       ,XMLElement("CreatedBy", fnd_global.user_id)
                       ,XMLElement("CreationDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"), null)
                       ,decode(p_comments_to_approver, null, XMLElement("Comments", XMLAttributes('true' as "null"), p_comments_to_approver), XMLElement("Comments", p_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA('000100000000')
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_workflow_info IS
  SELECT hat.item_type
        ,hat.item_key
        ,hat.transaction_ref_id
  FROM   hr_api_transactions  hat
  WHERE  hat.transaction_id = p_transaction_id;

  l_flex_vs_id       number;

  CURSOR csr_get_vs_id IS
  SELECT flex_value_set_id
  FROM   fnd_flex_value_sets
  WHERE  flex_value_set_name = 'PQP_GB_OSP_PART_DAYS_VS';

  CURSOR csr_get_dff_info IS
  SELECT application_column_name
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  flex_value_set_id = l_flex_vs_id
  AND    enabled_flag = 'Y'
  AND    descriptive_flex_context_code in (p_attribute_category, p_abs_information_category);

  CURSOR csr_get_dff_gdd IS
  SELECT application_column_name
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  flex_value_set_id = l_flex_vs_id
  AND    enabled_flag = 'Y'
  AND    descriptive_flex_context_code = 'Global Data Elements';

  CURSOR csr_get_enabled_dff_gdd IS
  SELECT application_column_name
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  enabled_flag = 'Y'
  and    descriptive_flexfield_name in ('PER_ABS_DEVELOPER_DF','PER_ABSENCE_ATTENDANCES')
  AND    descriptive_flex_context_code = 'Global Data Elements';


  c_method    CONSTANT varchar2(30) := 'create_emp_view_row';

  c_person_id CONSTANT number       := xxas_emp_view_util_pkg.get_person_id
                                         (p_assignment_id  => NVL(p_override_assignment_id,p_assignment_id)
                                         ,p_effective_date => p_effective_date);

  l_absence_attendance_id      number;
  l_object_version_number      number;
  l_absence_days               number       := p_absence_days;
  l_absence_hours              number       := p_absence_hours;
  l_date_projected_start       date         := p_date_projected_start;
  l_time_projected_start       varchar2(10) := p_time_projected_start;
  l_date_projected_end         date         := p_date_projected_end;
  l_time_projected_end         varchar2(10) := p_time_projected_end;
  l_date_start                 date         := p_date_start;
  l_time_start                 varchar2(10) := p_time_start;
  l_date_end                   date         := p_date_end;
  l_time_end                   varchar2(10) := p_time_end;
  l_hours_or_days              varchar2(30);
  l_item_type                  wf_item_activity_statuses.item_type%TYPE;
  l_item_key                   wf_item_activity_statuses.item_key%TYPE;
  l_activity_id                wf_item_activity_statuses.process_activity%TYPE;
  l_asg_row                    csr_asg%ROWTYPE;
  l_per_row                    csr_person%ROWTYPE;
  l_login_per_row              csr_login_person%ROWTYPE;
  l_login_person_id            number;
  l_per_manager_name           varchar2(240);
  l_login_per_manager_name     varchar2(240);
  l_doc                        XMLType;
  l_rc_doc                     XMLType;
  l_transaction_id             number;
  l_per_sec_group_id           number;
  l_per_leg_code               varchar2(150);
  l_login_per_sec_group_id     number;
  l_login_per_leg_code         varchar2(150);
  l_per_job_name               varchar2(700);
  l_per_position_name          varchar2(240);
  l_emp_generation             varchar2(150);
  l_per_bg_cur_code            varchar2(150);
  l_abs_att_category           varchar2(240);
  l_return                     number;
  l_msg_data                   varchar2(2000) := null;
  l_msg_count                  number := 0;
  l_api_errors                 boolean := false;
  l_return_status              varchar2(30);
  l_absence_attendance_id1     number;
  l_occurrence                 varchar2(240);
  l_called_from                fnd_form_functions.function_name%TYPE;
  l_workflow_info              csr_workflow_info%ROWTYPE;
  l_attribute1                 varchar2(1000);
  l_attribute2                 varchar2(1000);
  l_attribute3                 varchar2(1000);
  l_attribute4                 varchar2(1000);
  l_attribute5                 varchar2(1000);
  l_attribute6                 varchar2(1000);
  l_attribute7                 varchar2(1000);
  l_attribute8                 varchar2(1000);
  l_attribute9                 varchar2(1000);
  l_attribute10                varchar2(1000);
  l_attribute11                varchar2(1000);
  l_attribute12                varchar2(1000);
  l_attribute13                varchar2(1000);
  l_attribute14                varchar2(1000);
  l_attribute15                varchar2(1000);
  l_attribute16                varchar2(1000);
  l_attribute17                varchar2(1000);
  l_attribute18                varchar2(1000);
  l_attribute19                varchar2(1000);
  l_attribute20                varchar2(1000);
  l_abs_information1           varchar2(1000);
  l_abs_information2           varchar2(1000);
  l_abs_information3           varchar2(1000);
  l_abs_information4           varchar2(1000);
  l_abs_information5           varchar2(1000);
  l_abs_information6           varchar2(1000);
  l_abs_information7           varchar2(1000);
  l_abs_information8           varchar2(1000);
  l_abs_information9           varchar2(1000);
  l_abs_information10          varchar2(1000);
  l_abs_information11          varchar2(1000);
  l_abs_information12          varchar2(1000);
  l_abs_information13          varchar2(1000);
  l_abs_information14          varchar2(1000);
  l_abs_information15          varchar2(1000);
  l_abs_information16          varchar2(1000);
  l_abs_information17          varchar2(1000);
  l_abs_information18          varchar2(1000);
  l_abs_information19          varchar2(1000);
  l_abs_information20          varchar2(1000);
  l_abs_information21          varchar2(1000);
  l_abs_information22          varchar2(1000);
  l_abs_information23          varchar2(1000);
  l_abs_information24          varchar2(1000);
  l_abs_information25          varchar2(1000);
  l_abs_information26          varchar2(1000);
  l_abs_information27          varchar2(1000);
  l_abs_information28          varchar2(1000);
  l_abs_information29          varchar2(1000);
  l_abs_information30          varchar2(1000);
  l_temp                       varchar2(1000);

BEGIN
  log_start(c_method, p_effective_date, p_assignment_id, p_absence_type_id, p_abs_attendance_reason_id, p_date_start, p_date_end, p_attribute_category, p_override_assignment_id, p_transaction_id, p_transaction_step_id);

  xxas_util_pkg.user_has_access_check(g_PROD_KEY);

  log_statement(c_method, 'p_effective_date = ' || to_char(p_effective_date));
  log_statement(c_method, 'p_date_start = ' || to_char(p_date_start));
  log_statement(c_method, 'p_date_end = ' || to_char(p_date_end));
  log_statement(c_method, 'p_time_start = ' || to_char(p_time_start));
  log_statement(c_method, 'p_time_end = ' || to_char(p_time_end));

  g_ITEM_TYPE          := xxas_emp_view_util_pkg.get_item_type;
  g_AME_APP_ID         := xxas_emp_view_util_pkg.get_ame_application_id;
  g_AME_TRANS_TYPE     := xxas_emp_view_util_pkg.get_ame_transaction_type;
  g_PROCESS_NAME       := xxas_emp_view_util_pkg.get_process_name;

  log_statement(c_method,'g_ITEM_TYPE: ' || g_ITEM_TYPE);
  log_statement(c_method,'g_AME_APP_ID: ' || g_AME_APP_ID);
  log_statement(c_method,'g_AME_TRANS_TYPE: ' || g_AME_TRANS_TYPE);
  log_statement(c_method,'g_PROCESS_NAME: ' || g_PROCESS_NAME);

  l_hours_or_days := xxas_leave_of_absence_pkg.get_absence_type_value
                       (p_absence_type_id => p_absence_type_id
                       ,p_column_name     => 'HOURS_OR_DAYS');

  IF l_hours_or_days = 'H' THEN
    l_absence_days := null;
  ELSIF l_hours_or_days <> 'XXASNODATA' THEN
    l_absence_hours := null;
  END IF;

  IF p_leave_status = 'CONFIRMED' THEN

    l_date_projected_start := null;
    l_time_projected_start := null;
    l_date_projected_end   := null;
    l_time_projected_end   := null;

  ELSE

    l_date_start := null;
    l_time_start := null;
    l_date_end   := null;
    l_time_end   := null;

  END IF;

  OPEN  csr_person;
  FETCH csr_person INTO l_per_row;
  CLOSE csr_person;

  OPEN  csr_asg;
  FETCH csr_asg INTO l_asg_row;
  CLOSE csr_asg;

  OPEN  csr_get_emp_full_name(l_asg_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_job_name(l_asg_row.job_id);
  FETCH csr_get_job_name INTO l_per_job_name;
  CLOSE csr_get_job_name;

  OPEN  csr_get_pos_name(l_asg_row.position_id);
  FETCH csr_get_pos_name INTO l_per_position_name;
  CLOSE csr_get_pos_name;

  OPEN  csr_get_sec_grp_leg_code(c_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_per_sec_group_id
                                     ,l_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  OPEN  csr_get_org_info(c_person_id);
  FETCH csr_get_org_info INTO l_emp_generation
                             ,l_per_bg_cur_code;
  CLOSE csr_get_org_info;

  IF p_transaction_id is not null AND
     p_transaction_step_id is not null THEN

    OPEN  csr_workflow_info;
    FETCH csr_workflow_info INTO l_workflow_info;
    CLOSE csr_workflow_info;

    l_transaction_id        := p_transaction_id;
    l_absence_attendance_id := l_workflow_info.transaction_ref_id;
    l_item_type             := l_workflow_info.item_type;
    l_item_key              := l_workflow_info.item_key;

  ELSE

    --
    -- Generate the new sequences
    --
    SELECT  per_absence_attendances_s.nextval
           ,hr_api_transactions_s.nextval
    INTO    l_absence_attendance_id
           ,l_transaction_id
    FROM   dual;

  END IF;

  l_login_person_id := xxas_util_pkg.get_per_from_user(fnd_global.user_id);

  OPEN  csr_login_person(l_login_person_id);
  FETCH csr_login_person INTO l_login_per_row;
  CLOSE csr_login_person;

  OPEN  csr_get_emp_full_name(l_login_per_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_login_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_sec_grp_leg_code(l_login_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_login_per_sec_group_id
                                     ,l_login_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  l_attribute1        := p_attribute1;
  l_attribute2        := p_attribute2;
  l_attribute3        := p_attribute3;
  l_attribute4        := p_attribute4;
  l_attribute5        := p_attribute5;
  l_attribute6        := p_attribute6;
  l_attribute7        := p_attribute7;
  l_attribute8        := p_attribute8;
  l_attribute9        := p_attribute9;
  l_attribute10       := p_attribute10;
  l_attribute11       := p_attribute11;
  l_attribute12       := p_attribute12;
  l_attribute13       := p_attribute13;
  l_attribute14       := p_attribute14;
  l_attribute15       := p_attribute15;
  l_attribute16       := p_attribute16;
  l_attribute17       := p_attribute17;
  l_attribute18       := p_attribute18;
  l_attribute19       := p_attribute19;
  l_attribute20       := p_attribute20;

  l_abs_information1  := p_abs_information1;
  l_abs_information2  := p_abs_information2;
  l_abs_information3  := p_abs_information3;
  l_abs_information4  := p_abs_information4;
  l_abs_information5  := p_abs_information5;
  l_abs_information6  := p_abs_information6;
  l_abs_information7  := p_abs_information7;
  l_abs_information8  := p_abs_information8;
  l_abs_information9  := p_abs_information9;
  l_abs_information10 := p_abs_information10;
  l_abs_information11  := p_abs_information11;
  l_abs_information12  := p_abs_information12;
  l_abs_information13  := p_abs_information13;
  l_abs_information14  := p_abs_information14;
  l_abs_information15  := p_abs_information15;
  l_abs_information16  := p_abs_information16;
  l_abs_information17  := p_abs_information17;
  l_abs_information18  := p_abs_information18;
  l_abs_information19  := p_abs_information19;
  l_abs_information20  := p_abs_information20;
  l_abs_information21  := p_abs_information21;
  l_abs_information22  := p_abs_information22;
  l_abs_information23  := p_abs_information23;
  l_abs_information24  := p_abs_information24;
  l_abs_information25  := p_abs_information25;
  l_abs_information26  := p_abs_information26;
  l_abs_information27  := p_abs_information27;
  l_abs_information28  := p_abs_information28;
  l_abs_information29  := p_abs_information29;
  l_abs_information30  := p_abs_information30;


  OPEN csr_get_vs_id;
  FETCH csr_get_vs_id INTO l_flex_vs_id;
  CLOSE csr_get_vs_id;

  OPEN csr_get_dff_gdd;
  FETCH csr_get_dff_gdd INTO l_temp;
  CLOSE csr_get_dff_gdd;

  IF l_temp IS NULL and p_abs_information_category IS NULL THEN
    l_abs_information1  := null;
    l_abs_information2  := null;
    l_abs_information3  := null;
    l_abs_information4  := null;
    l_abs_information5  := null;
    l_abs_information6  := null;
    l_abs_information7  := null;
    l_abs_information8  := null;
    l_abs_information9  := null;
    l_abs_information10 := null;
    l_abs_information11  := null;
    l_abs_information12  := null;
    l_abs_information13  := null;
    l_abs_information14  := null;
    l_abs_information15  := null;
    l_abs_information16  := null;
    l_abs_information17  := null;
    l_abs_information18  := null;
    l_abs_information19  := null;
    l_abs_information20  := null;
    l_abs_information21  := null;
    l_abs_information22  := null;
    l_abs_information23  := null;
    l_abs_information24  := null;
    l_abs_information25  := null;
    l_abs_information26  := null;
    l_abs_information27  := null;
    l_abs_information28  := null;
    l_abs_information29  := null;
    l_abs_information30  := null;
  END IF;

  IF l_temp IS NULL AND p_attribute_category IS NULL THEN
    l_attribute1        := null;
    l_attribute2        := null;
    l_attribute3        := null;
    l_attribute4        := null;
    l_attribute5        := null;
    l_attribute6        := null;
    l_attribute7        := null;
    l_attribute8        := null;
    l_attribute9        := null;
    l_attribute10       := null;
    l_attribute11       := null;
    l_attribute12       := null;
    l_attribute13       := null;
    l_attribute14       := null;
    l_attribute15       := null;
    l_attribute16       := null;
    l_attribute17       := null;
    l_attribute18       := null;
    l_attribute19       := null;
    l_attribute20       := null;
  END IF;

  FOR csr_rec in csr_get_enabled_dff_gdd
  LOOP

  IF csr_rec.application_column_name = 'ATTRIBUTE1' THEN
      if p_attribute1 is not null THEN
        l_attribute1 := p_attribute1;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE2' THEN
      if p_attribute2 is not null THEN
        l_attribute2 := p_attribute2;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE3' THEN
      if p_attribute3 is not null THEN
        l_attribute3 := p_attribute3;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE4' THEN
      if p_attribute4 is not null THEN
        l_attribute4 := p_attribute4;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE5' THEN
      if p_attribute5 is not null THEN
        l_attribute5 := p_attribute5;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE6' THEN
      if p_attribute6 is not null THEN
        l_attribute6 := p_attribute6;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE7' THEN
      if p_attribute7 is not null THEN
        l_attribute7 := p_attribute7;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE8' THEN
      if p_attribute8 is not null THEN
        l_attribute8 := p_attribute8;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE9' THEN
      if p_attribute9 is not null THEN
        l_attribute9 := p_attribute9;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE10' THEN
      if p_attribute10 is not null THEN
        l_attribute10 := p_attribute10;
      end if;
	elsIF csr_rec.application_column_name = 'ATTRIBUTE11' THEN
	  if p_attribute11 is not null THEN
        l_attribute11 := p_attribute11;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE12' THEN
      if p_attribute12 is not null THEN
        l_attribute12 := p_attribute12;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE13' THEN
      if p_attribute13 is not null THEN
        l_attribute13 := p_attribute13;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE14' THEN
      if p_attribute14 is not null THEN
        l_attribute14 := p_attribute14;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE15' THEN
      if p_attribute15 is not null THEN
        l_attribute15 := p_attribute15;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE16' THEN
      if p_attribute16 is not null THEN
        l_attribute16 := p_attribute16;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE17' THEN
      if p_attribute17 is not null THEN
        l_attribute17 := p_attribute17;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE18' THEN
      if p_attribute18 is not null THEN
        l_attribute18 := p_attribute18;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE19' THEN
      if p_attribute19 is not null THEN
        l_attribute19 := p_attribute19;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE20' THEN
      if p_attribute20 is not null THEN
        l_attribute20 := p_attribute20;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION1' THEN
      if p_abs_information1 is not null THEN
        l_abs_information1 := p_abs_information1;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION2' THEN
      if p_abs_information2 is not null THEN
        l_abs_information2 := p_abs_information2;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION3' THEN
      if p_abs_information3 is not null THEN
        l_abs_information3 := p_abs_information3;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION4' THEN
      if p_abs_information4 is not null THEN
        l_abs_information4 := p_abs_information4;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION5' THEN
      if p_abs_information5 is not null THEN
        l_abs_information5 := p_abs_information5;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION6' THEN
      if p_abs_information6 is not null THEN
        l_abs_information6 := p_abs_information6;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION7' THEN
      if p_abs_information7 is not null THEN
        l_abs_information7 := p_abs_information7;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION8' THEN
      if p_abs_information8 is not null THEN
        l_abs_information8 := p_abs_information8;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION9' THEN
      if p_abs_information9 is not null THEN
        l_abs_information9 := p_abs_information9;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION10' THEN
      if p_abs_information10 is not null THEN
        l_abs_information10 := p_abs_information10;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION11' THEN
      if p_abs_information11 is not null THEN
        l_abs_information11 := p_abs_information11;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION12' THEN
      if p_abs_information12 is not null THEN
        l_abs_information12 := p_abs_information12;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION13' THEN
      if p_abs_information13 is not null THEN
        l_abs_information13 := p_abs_information13;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION14' THEN
      if p_abs_information14 is not null THEN
        l_abs_information14 := p_abs_information14;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION15' THEN
      if p_abs_information15 is not null THEN
        l_abs_information15 := p_abs_information15;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION16' THEN
      if p_abs_information16 is not null THEN
        l_abs_information16 := p_abs_information16;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION17' THEN
      if p_abs_information17 is not null THEN
        l_abs_information17 := p_abs_information17;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION18' THEN
      if p_abs_information18 is not null THEN
        l_abs_information18 := p_abs_information18;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION19' THEN
      if p_abs_information19 is not null THEN
        l_abs_information19 := p_abs_information19;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION20' THEN
      if p_abs_information20 is not null THEN
        l_abs_information20 := p_abs_information20;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION21' THEN
      if p_abs_information21 is not null THEN
        l_abs_information21 := p_abs_information21;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION22' THEN
      if p_abs_information22 is not null THEN
        l_abs_information22 := p_abs_information22;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION23' THEN
      if p_abs_information23 is not null THEN
        l_abs_information23 := p_abs_information23;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION24' THEN
      if p_abs_information24 is not null THEN
        l_abs_information24 := p_abs_information24;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION25' THEN
      if p_abs_information25 is not null THEN
        l_abs_information25 := p_abs_information25;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION26' THEN
      if p_abs_information26 is not null THEN
        l_abs_information26 := p_abs_information26;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION27' THEN
      if p_abs_information27 is not null THEN
        l_abs_information27 := p_abs_information27;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION28' THEN
      if p_abs_information28 is not null THEN
        l_abs_information28 := p_abs_information28;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION29' THEN
      if p_abs_information29 is not null THEN
        l_abs_information29 := p_abs_information29;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION30' THEN
      if p_abs_information30 is not null THEN
        l_abs_information30 := p_abs_information30;
      end if;
    end if;
 END LOOP;

  FOR csr_rec in csr_get_dff_info
  LOOP

  IF csr_rec.application_column_name = 'ATTRIBUTE1' THEN
      if instr(l_attribute1,'.') = 0 THEN
        l_attribute1 := p_attribute1||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE2' THEN
      if instr(l_attribute2,'.') = 0 THEN
        l_attribute2 := p_attribute2||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE3' THEN
      if instr(l_attribute3,'.') = 0 THEN
        l_attribute3 := p_attribute3||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE4' THEN
      if instr(l_attribute4,'.') = 0 THEN
        l_attribute4 := p_attribute4||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE5' THEN
      if instr(l_attribute5,'.') = 0 THEN
        l_attribute5 := p_attribute5||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE6' THEN
      if instr(l_attribute6,'.') = 0 THEN
        l_attribute6 := p_attribute6||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE7' THEN
      if instr(l_attribute7,'.') = 0 THEN
        l_attribute7 := p_attribute7||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE8' THEN
      if instr(l_attribute8,'.') = 0 THEN
        l_attribute8 := p_attribute8||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE9' THEN
      if instr(l_attribute9,'.') = 0 THEN
        l_attribute9 := p_attribute9||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE10' THEN
      if instr(l_attribute10,'.') = 0 THEN
        l_attribute10 := p_attribute10||'.00';
      end if;
	elsIF csr_rec.application_column_name = 'ATTRIBUTE11' THEN
	  if instr(l_attribute11,'.') = 0 THEN
        l_attribute11 := p_attribute11||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE12' THEN
      if instr(l_attribute12,'.') = 0 THEN
        l_attribute12 := p_attribute12||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE13' THEN
      if instr(l_attribute13,'.') = 0 THEN
        l_attribute13 := p_attribute13||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE14' THEN
      if instr(l_attribute14,'.') = 0 THEN
        l_attribute14 := p_attribute14||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE15' THEN
      if instr(l_attribute15,'.') = 0 THEN
        l_attribute15 := p_attribute15||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE16' THEN
      if instr(l_attribute16,'.') = 0 THEN
        l_attribute16 := p_attribute16||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE17' THEN
      if instr(l_attribute17,'.') = 0 THEN
        l_attribute17 := p_attribute17||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE18' THEN
      if instr(l_attribute18,'.') = 0 THEN
        l_attribute18 := p_attribute18||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE19' THEN
      if instr(l_attribute19,'.') = 0 THEN
        l_attribute19 := p_attribute19||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE20' THEN
      if instr(l_attribute20,'.') = 0 THEN
        l_attribute20 := p_attribute20||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION1' THEN
      if instr(l_abs_information1,'.') = 0 THEN
        l_abs_information1 := l_abs_information1||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION2' THEN
      if instr(l_abs_information2,'.') = 0 THEN
        l_abs_information2 := l_abs_information2||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION3' THEN
      if instr(l_abs_information3,'.') = 0 THEN
        l_abs_information3 := l_abs_information3||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION4' THEN
      if instr(l_abs_information4,'.') = 0 THEN
        l_abs_information4 := l_abs_information4||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION5' THEN
      if instr(l_abs_information5,'.') = 0 THEN
        l_abs_information5 := l_abs_information5||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION6' THEN
      if instr(l_abs_information6,'.') = 0 THEN
        l_abs_information6 := l_abs_information6||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION7' THEN
      if instr(l_abs_information7,'.') = 0 THEN
        l_abs_information7 := l_abs_information7||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION8' THEN
      if instr(l_abs_information8,'.') = 0 THEN
        l_abs_information8 := l_abs_information8||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION9' THEN
      if instr(l_abs_information9,'.') = 0 THEN
        l_abs_information9 := l_abs_information9||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION10' THEN
      if instr(l_abs_information10,'.') = 0 THEN
        l_abs_information10 := l_abs_information10||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION11' THEN
      if instr(l_abs_information11,'.') = 0 THEN
        l_abs_information11 := l_abs_information11||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION12' THEN
      if instr(l_abs_information12,'.') = 0 THEN
        l_abs_information12 := l_abs_information12||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION13' THEN
      if instr(l_abs_information13,'.') = 0 THEN
        l_abs_information13 := l_abs_information13||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION14' THEN
      if instr(l_abs_information14,'.') = 0 THEN
        l_abs_information14 := l_abs_information14||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION15' THEN
      if instr(l_abs_information15,'.') = 0 THEN
        l_abs_information15 := l_abs_information15||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION16' THEN
      if instr(l_abs_information16,'.') = 0 THEN
        l_abs_information16 := l_abs_information16||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION17' THEN
      if instr(l_abs_information17,'.') = 0 THEN
        l_abs_information17 := l_abs_information17||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION18' THEN
      if instr(l_abs_information18,'.') = 0 THEN
        l_abs_information18 := l_abs_information18||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION19' THEN
      if instr(l_abs_information19,'.') = 0 THEN
        l_abs_information19 := l_abs_information19||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION20' THEN
      if instr(l_abs_information20,'.') = 0 THEN
        l_abs_information20 := l_abs_information20||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION21' THEN
      if instr(l_abs_information21,'.') = 0 THEN
        l_abs_information21 := l_abs_information21||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION22' THEN
      if instr(l_abs_information22,'.') = 0 THEN
        l_abs_information22 := l_abs_information22||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION23' THEN
      if instr(l_abs_information23,'.') = 0 THEN
        l_abs_information23 := l_abs_information23||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION24' THEN
      if instr(l_abs_information24,'.') = 0 THEN
        l_abs_information24 := l_abs_information24||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION25' THEN
      if instr(l_abs_information25,'.') = 0 THEN
        l_abs_information25 := l_abs_information25||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION26' THEN
      if instr(l_abs_information26,'.') = 0 THEN
        l_abs_information26 := l_abs_information26||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION27' THEN
      if instr(l_abs_information27,'.') = 0 THEN
        l_abs_information27 := l_abs_information27||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION28' THEN
      if instr(l_abs_information28,'.') = 0 THEN
        l_abs_information28 := l_abs_information28||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION29' THEN
      if instr(l_abs_information29,'.') = 0 THEN
        l_abs_information29 := l_abs_information29||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION30' THEN
      if instr(l_abs_information30,'.') = 0 THEN
        l_abs_information30 := l_abs_information30||'.00';
      end if;
    end if;
 END LOOP;

  l_called_from := xxas_com_sec_pkg.get_function_parameter
                     (p_function_name  => xxas_emp_view_context_pkg.data_function
                     ,p_parameter_name => 'pCalledFrom');

  IF p_transaction_id is not null AND
     p_transaction_step_id is not null THEN

    OPEN  csr_rc_trans_doc(c_person_id
                          ,l_absence_attendance_id
                          ,l_per_row
                          ,l_asg_row
                          ,l_login_per_row
                          ,l_per_manager_name
                          ,l_login_per_manager_name
                          ,l_transaction_id
                          ,l_per_sec_group_id
                          ,l_per_leg_code
                          ,l_login_per_leg_code
                          ,l_per_job_name
                          ,l_per_position_name
                          ,l_emp_generation
                          ,l_per_bg_cur_code
                          ,l_called_from
                          ,l_date_projected_start
                          ,l_time_projected_start
                          ,l_date_projected_end
                          ,l_time_projected_end
                          ,l_date_start
                          ,l_time_start
                          ,l_date_end
                          ,l_time_end
                          ,l_absence_days
                          ,l_absence_hours
						  ,l_attribute1
                          ,l_attribute2
                          ,l_attribute3
                          ,l_attribute4
                          ,l_attribute5
                          ,l_attribute6
                          ,l_attribute7
                          ,l_attribute8
                          ,l_attribute9
                          ,l_attribute10
                          ,l_attribute11
                          ,l_attribute12
                          ,l_attribute13
                          ,l_attribute14
                          ,l_attribute15
                          ,l_attribute16
                          ,l_attribute17
                          ,l_attribute18
                          ,l_attribute19
                          ,l_attribute20
                          ,p_abs_information_category
                          ,l_abs_information1
                          ,l_abs_information2
                          ,l_abs_information3
                          ,l_abs_information4
                          ,l_abs_information5
                          ,l_abs_information6
                          ,l_abs_information7
                          ,l_abs_information8
                          ,l_abs_information9
                          ,l_abs_information10
                          ,l_abs_information11
                          ,l_abs_information12
                          ,l_abs_information13
                          ,l_abs_information14
                          ,l_abs_information15
                          ,l_abs_information16
                          ,l_abs_information17
                          ,l_abs_information18
                          ,l_abs_information19
                          ,l_abs_information20
                          ,l_abs_information21
                          ,l_abs_information22
                          ,l_abs_information23
                          ,l_abs_information24
                          ,l_abs_information25
                          ,l_abs_information26
                          ,l_abs_information27
                          ,l_abs_information28
                          ,l_abs_information29
                          ,l_abs_information30
						  );
    FETCH csr_rc_trans_doc INTO l_rc_doc;
    CLOSE csr_rc_trans_doc;

  ELSE

    OPEN  csr_trans_doc(c_person_id
                       ,l_absence_attendance_id
                       ,l_per_row
                       ,l_asg_row
                       ,l_login_per_row
                       ,l_per_manager_name
                       ,l_login_per_manager_name
                       ,l_transaction_id
                       ,l_per_sec_group_id
                       ,l_per_leg_code
                       ,l_login_per_leg_code
                       ,l_per_job_name
                       ,l_per_position_name
                       ,l_emp_generation
                       ,l_per_bg_cur_code
                       ,l_called_from
                       ,l_date_projected_start
                       ,l_time_projected_start
                       ,l_date_projected_end
                       ,l_time_projected_end
                       ,l_date_start
                       ,l_time_start
                       ,l_date_end
                       ,l_time_end
                       ,l_absence_days
                       ,l_absence_hours
					   ,l_attribute1
                       ,l_attribute2
                       ,l_attribute3
                       ,l_attribute4
                       ,l_attribute5
                       ,l_attribute6
                       ,l_attribute7
                       ,l_attribute8
                       ,l_attribute9
                       ,l_attribute10
                       ,l_attribute11
                       ,l_attribute12
                       ,l_attribute13
                       ,l_attribute14
                       ,l_attribute15
                       ,l_attribute16
                       ,l_attribute17
                       ,l_attribute18
                       ,l_attribute19
                       ,l_attribute20
                       ,p_abs_information_category
                       ,l_abs_information1
                       ,l_abs_information2
                       ,l_abs_information3
                       ,l_abs_information4
                       ,l_abs_information5
                       ,l_abs_information6
                       ,l_abs_information7
                       ,l_abs_information8
                       ,l_abs_information9
                       ,l_abs_information10
                       ,l_abs_information11
                       ,l_abs_information12
                       ,l_abs_information13
                       ,l_abs_information14
                       ,l_abs_information15
                       ,l_abs_information16
                       ,l_abs_information17
                       ,l_abs_information18
                       ,l_abs_information19
                       ,l_abs_information20
                       ,l_abs_information21
                       ,l_abs_information22
                       ,l_abs_information23
                       ,l_abs_information24
                       ,l_abs_information25
                       ,l_abs_information26
                       ,l_abs_information27
                       ,l_abs_information28
                       ,l_abs_information29
                       ,l_abs_information30);
    FETCH csr_trans_doc INTO l_doc;
    CLOSE csr_trans_doc;

  END IF;

  OPEN  csr_get_abs_att_type_meaning(p_absence_type_id);
  FETCH csr_get_abs_att_type_meaning INTO l_abs_att_category;
  CLOSE csr_get_abs_att_type_meaning;

/*  l_abs_dur_hours := calculate_absence_dur_hours(p_business_group_id => l_per_row.business_group_id
                                                ,p_assignment_id     => NVL(p_override_assignment_id,p_assignment_id)
                                                ,p_effective_date    => p_effective_date
                                                ,p_absence_type_id   => p_absence_type_id
                                                ,p_date_start        => l_date_start
                                                ,p_date_end          => l_date_end
                                                ,p_time_start        => l_time_start
                                                ,p_time_end          => l_time_end);

  l_abs_dur_days := calculate_absence_dur_days(p_business_group_id => l_per_row.business_group_id
                                              ,p_assignment_id     => NVL(p_override_assignment_id,p_assignment_id)
                                              ,p_effective_date    => p_effective_date
                                              ,p_absence_type_id   => p_absence_type_id
                                              ,p_date_start        => l_date_start
                                              ,p_date_end          => l_date_end
                                              ,p_time_start        => l_time_start
                                              ,p_time_end          => l_time_end); -- commented because already calculating duration through Fast Formula*/

  IF p_transaction_id is not null AND
     p_transaction_step_id is not null THEN

    update_hrapi_transaction(p_transaction_id        => p_transaction_id
                            ,p_transaction_step_id   => p_transaction_step_id
                            ,p_absence_attendance_id => l_absence_attendance_id
                            ,p_assignment_id         => NVL(p_override_assignment_id,p_assignment_id)
                            ,p_person_id             => c_person_id
                            ,p_item_type             => g_ITEM_TYPE
                            ,p_item_key              => null--l_item_key
                            ,p_absence_trans_doc     => l_rc_doc
                            ,p_information1          => to_char(l_date_start, g_HRAPI_DATE_FMT)
                            ,p_information2          => to_char(l_date_end, g_HRAPI_DATE_FMT)
                            ,p_information3          => to_char(l_date_projected_start, g_HRAPI_DATE_FMT)
                            ,p_information4          => to_char(l_date_projected_end, g_HRAPI_DATE_FMT)
                            ,p_information5          => p_absence_type_id
                            ,p_information6          => l_abs_att_category
                            ,p_information7          => l_absence_hours
                            ,p_information8          => l_absence_days
                            ,p_information9          => p_leave_status
                            ,p_object_state          => '0');

  ELSE

    p_transaction_step_id := create_hrapi_transaction(p_transaction_id        => l_transaction_id
                                                     ,p_absence_attendance_id => l_absence_attendance_id
                                                     ,p_assignment_id         => NVL(p_override_assignment_id,p_assignment_id)
                                                     ,p_person_id             => c_person_id
                                                     ,p_item_type             => g_ITEM_TYPE
                                                     ,p_item_key              => l_item_key
                                                     ,p_absence_trans_doc     => l_doc
                                                     ,p_information1          => to_char(l_date_start, g_HRAPI_DATE_FMT)
                                                     ,p_information2          => to_char(l_date_end, g_HRAPI_DATE_FMT)
                                                     ,p_information3          => to_char(l_date_projected_start, g_HRAPI_DATE_FMT)
                                                     ,p_information4          => to_char(l_date_projected_end, g_HRAPI_DATE_FMT)
                                                     ,p_information5          => p_absence_type_id
                                                     ,p_information6          => l_abs_att_category
                                                     ,p_information7          => l_absence_hours
                                                     ,p_information8          => l_absence_days
                                                     ,p_information9          => p_leave_status
                                                     ,p_object_state          => '0');

    p_transaction_id := l_transaction_id;

  END IF;

  --
  -- Do create API's validation
  --
  --
  -- Issue a savepoint
  --
  savepoint before_api_validation;

  l_absence_attendance_id1 := l_absence_attendance_id;

  hr_person_absence_swi.create_person_absence
    (p_validate                     => hr_api.g_true_num
    ,p_effective_date               => p_effective_date
    ,p_person_id                    => c_person_id
    ,p_business_group_id            => l_asg_row.business_group_id
    ,p_absence_attendance_type_id   => p_absence_type_id
    ,p_abs_attendance_reason_id     => p_abs_attendance_reason_id
    ,p_comments                     => p_comments_to_approver
    ,p_date_notification            => p_date_notification
    ,p_date_projected_start         => l_date_projected_start
    ,p_time_projected_start         => l_time_projected_start
    ,p_date_projected_end           => l_date_projected_end
    ,p_time_projected_end           => l_time_projected_end
    ,p_date_start                   => l_date_start
    ,p_time_start                   => l_time_start
    ,p_date_end                     => l_date_end
    ,p_time_end                     => l_time_end
    ,p_absence_days                 => l_absence_days
    ,p_absence_hours                => l_absence_hours
    ,p_authorising_person_id        => p_authorising_person_id
    ,p_replacement_person_id        => p_replacement_person_id
    ,p_attribute_category           => p_attribute_category
    ,p_attribute1                   => l_attribute1
    ,p_attribute2                   => l_attribute2
    ,p_attribute3                   => l_attribute3
    ,p_attribute4                   => l_attribute4
    ,p_attribute5                   => l_attribute5
    ,p_attribute6                   => l_attribute6
    ,p_attribute7                   => l_attribute7
    ,p_attribute8                   => l_attribute8
    ,p_attribute9                   => l_attribute9
    ,p_attribute10                  => l_attribute10
    ,p_attribute11                  => l_attribute11
    ,p_attribute12                  => l_attribute12
    ,p_attribute13                  => l_attribute13
    ,p_attribute14                  => l_attribute14
    ,p_attribute15                  => l_attribute15
    ,p_attribute16                  => l_attribute16
    ,p_attribute17                  => l_attribute17
    ,p_attribute18                  => l_attribute18
    ,p_attribute19                  => l_attribute19
    ,p_attribute20                  => l_attribute20
    ,p_period_of_incapacity_id      => null
    ,p_ssp1_issued                  => null
    ,p_maternity_id                 => null
    ,p_sickness_start_date          => null
    ,p_sickness_end_date            => null
    ,p_pregnancy_related_illness    => null
    ,p_reason_for_notification_dela => null
    ,p_accept_late_notification_fla => null
    ,p_linked_absence_id            => null
    ,p_batch_id                     => null
    ,p_create_element_entry         => null
    ,p_abs_information_category     => p_abs_information_category
    ,p_abs_information1             => l_abs_information1
    ,p_abs_information2             => l_abs_information2
    ,p_abs_information3             => l_abs_information3
    ,p_abs_information4             => l_abs_information4
    ,p_abs_information5             => l_abs_information5
    ,p_abs_information6             => l_abs_information6
    ,p_abs_information7             => l_abs_information7
    ,p_abs_information8             => l_abs_information8
    ,p_abs_information9             => l_abs_information9
    ,p_abs_information10            => l_abs_information10
    ,p_abs_information11            => l_abs_information11
    ,p_abs_information12            => l_abs_information12
    ,p_abs_information13            => l_abs_information13
    ,p_abs_information14            => l_abs_information14
    ,p_abs_information15            => l_abs_information15
    ,p_abs_information16            => l_abs_information16
    ,p_abs_information17            => l_abs_information17
    ,p_abs_information18            => l_abs_information18
    ,p_abs_information19            => l_abs_information19
    ,p_abs_information20            => l_abs_information20
    ,p_abs_information21            => l_abs_information21
    ,p_abs_information22            => l_abs_information22
    ,p_abs_information23            => l_abs_information23
    ,p_abs_information24            => l_abs_information24
    ,p_abs_information25            => l_abs_information25
    ,p_abs_information26            => l_abs_information26
    ,p_abs_information27            => l_abs_information27
    ,p_abs_information28            => l_abs_information28
    ,p_abs_information29            => l_abs_information29
    ,p_abs_information30            => l_abs_information30
    ,p_absence_case_id              => null
    ,p_absence_attendance_id        => l_absence_attendance_id1
    ,p_object_version_number        => l_object_version_number
    ,p_occurrence                   => l_occurrence
    ,p_return_status                => l_return_status);

  IF l_return_status = 'E' THEN
    IF (fnd_msg_pub.count_msg > 0) THEN
      FOR i IN 1..fnd_msg_pub.count_msg
      LOOP

        fnd_msg_pub.get
         (p_msg_index     => i,
          p_encoded       => 'F',
          p_data          => l_msg_data,
          p_msg_index_out => l_msg_count
         );

        log_statement('l_msg_data: ',l_msg_data);

        --
        -- Skip the following warning messages
        --
        IF upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_EMP_ABS_SHORT_DURATION')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_ABS_HOUR_LESS_DURATION')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_EMP_NOT_ENTITLED')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_RUNNING_ZERO')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_ABSENCE_OVERLAP')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_ABS_DAY_AFTER')) THEN

          null;

        ELSE

          register_user_error
            (p_package_name => gc_log.package_name
            ,p_proc_name    => c_method
            ,p_message      => l_msg_data);

          l_api_errors := true;

        END IF;

        l_msg_data   := null;
        l_msg_count  := 0;
      END LOOP;
    END IF;
  END IF;

  IF l_api_errors THEN
    xxas_com_api_result_pkg.raise_registered_user_errors;
  END IF;

  --
  -- Issue a rollback
  --
  rollback to before_api_validation;

  IF l_item_key is null THEN

    launch_workflow
      (p_person_id            => c_person_id
      ,p_transaction_id       => l_transaction_id
      ,p_comments_to_approver => p_comments_to_approver
      ,p_item_key             => l_item_key
      ,p_activity_id          => l_activity_id);

  END IF;

  hr_trn_upd.upd(p_transaction_id => l_transaction_id
                ,p_validate       => false
                ,p_status         => 'W'
                ,p_item_key       => l_item_key);

  xxas_emp_view_util_pkg.workflow_goto_review_page
      (p_item_type => g_ITEM_TYPE
      ,p_item_key  => l_item_key);

  --
  -- set page attributes.
  --
  xxas_com_worklist_pkg.populate_page_attributes(p_transaction_id      => p_transaction_id
                                                ,p_transaction_step_id => p_transaction_step_id
                                                ,p_item_type           => g_ITEM_TYPE
                                                ,p_item_key            => l_item_key
                                                ,p_edit_page_id        => xxas_com_worklist_pkg.g_FLEXI_EDIT_PAGE
                                                ,p_edit_object_id      => xxas_com_sec_pkg.get_function_id(xxas_emp_view_context_pkg.securing_function)
                                                                           || '_' || NVL(p_override_assignment_id,p_assignment_id)
                                                                           || '_' || null      -- pass null for create
                                                                           || '_' || p_transaction_id
                                                                           || '_' || p_transaction_step_id
                                                                           || '_' || '_' || '_'
                                                                           || null
                                                                           || '_' || null        -- this is null as we dont have page id
                                                ,p_edit_target         => 'modal'
                                                ,p_view_page_id        => xxas_com_worklist_pkg.g_FLEXI_VIEW_PAGE
                                                ,p_view_object_id      => xxas_com_sec_pkg.get_function_id(xxas_emp_view_context_pkg.securing_function)
                                                                           || '_' || NVL(p_override_assignment_id,p_assignment_id)
                                                                           || '_' || null      -- pass null for create
                                                                           || '_' || p_transaction_id
                                                                           || '_' || p_transaction_step_id
                                                                           || '_' || '_' || '_'
                                                                           || null
                                                                           || '_' || null        -- this is null as we dont have page id
                                                ,p_view_target         => 'modal');

  log_end(c_method, p_transaction_id);

EXCEPTION

  WHEN xxas_com_api_result_pkg.xxas_com_api_exception
  THEN
    rollback to before_api_validation;
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN hr_utility.hr_error
  THEN
    rollback to before_api_validation;
    register_user_error
      (p_package_name => gc_log.package_name
      ,p_proc_name    => c_method
      ,p_message      => hr_utility.get_message);
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

END create_emp_view_row;

--
-- ------------------------------------------------------------------------ *
-- Name    : update_emp_view_row
-- ------------------------------------------------------------------------ *
PROCEDURE update_emp_view_row
  (p_effective_date           IN     date
  ,p_assignment_id            IN     number
  ,p_absence_attendance_id    IN     number   DEFAULT null
  --,p_absence_attendance_id1   IN     number   DEFAULT null
  ,p_object_version_number    IN     number
  ,p_absence_type_id          IN     number   DEFAULT null
  ,p_override_assignment_id   IN     NUMBER   DEFAULT NULL
  ,p_abs_attendance_reason_id IN     number   DEFAULT null
  ,p_date_notification        IN     date     DEFAULT null
  ,p_date_projected_start     IN     date     DEFAULT null
  ,p_time_projected_start     IN     varchar2 DEFAULT null
  ,p_date_projected_end       IN     date     DEFAULT null
  ,p_time_projected_end	      IN     varchar2 DEFAULT null
  ,p_date_start               IN     date     DEFAULT null
  ,p_time_start               IN     varchar2 DEFAULT null
  ,p_date_end                 IN     date     DEFAULT null
  ,p_time_end     	          IN     varchar2 DEFAULT null
  ,p_absence_days             IN     number   DEFAULT null
  ,p_absence_hours            IN     number   DEFAULT null
  ,p_replacement_person_id    IN     number   DEFAULT null
  ,p_attribute_category       IN     varchar2 DEFAULT null
  ,p_attribute1               IN     varchar2 DEFAULT null
  ,p_attribute2               IN     varchar2 DEFAULT null
  ,p_attribute3               IN     varchar2 DEFAULT null
  ,p_attribute4               IN     varchar2 DEFAULT null
  ,p_attribute5               IN     varchar2 DEFAULT null
  ,p_attribute6               IN     varchar2 DEFAULT null
  ,p_attribute7               IN     varchar2 DEFAULT null
  ,p_attribute8               IN     varchar2 DEFAULT null
  ,p_attribute9               IN     varchar2 DEFAULT null
  ,p_attribute10              IN     varchar2 DEFAULT null
  ,p_attribute11              IN     varchar2 DEFAULT null
  ,p_attribute12              IN     varchar2 DEFAULT null
  ,p_attribute13              IN     varchar2 DEFAULT null
  ,p_attribute14              IN     varchar2 DEFAULT null
  ,p_attribute15              IN     varchar2 DEFAULT null
  ,p_attribute16              IN     varchar2 DEFAULT null
  ,p_attribute17              IN     varchar2 DEFAULT null
  ,p_attribute18              IN     varchar2 DEFAULT null
  ,p_attribute19              IN     varchar2 DEFAULT null
  ,p_attribute20              IN     varchar2 DEFAULT null
  ,p_abs_information_category IN     varchar2 DEFAULT null
  ,p_abs_information1         IN     varchar2 DEFAULT null
  ,p_abs_information2         IN     varchar2 DEFAULT null
  ,p_abs_information3         IN     varchar2 DEFAULT null
  ,p_abs_information4         IN     varchar2 DEFAULT null
  ,p_abs_information5         IN     varchar2 DEFAULT null
  ,p_abs_information6         IN     varchar2 DEFAULT null
  ,p_abs_information7         IN     varchar2 DEFAULT null
  ,p_abs_information8         IN     varchar2 DEFAULT null
  ,p_abs_information9         IN     varchar2 DEFAULT null
  ,p_abs_information10        IN     varchar2 DEFAULT null
  ,p_abs_information11        IN     varchar2 DEFAULT null
  ,p_abs_information12        IN     varchar2 DEFAULT null
  ,p_abs_information13        IN     varchar2 DEFAULT null
  ,p_abs_information14        IN     varchar2 DEFAULT null
  ,p_abs_information15        IN     varchar2 DEFAULT null
  ,p_abs_information16        IN     varchar2 DEFAULT null
  ,p_abs_information17        IN     varchar2 DEFAULT null
  ,p_abs_information18        IN     varchar2 DEFAULT null
  ,p_abs_information19        IN     varchar2 DEFAULT null
  ,p_abs_information20        IN     varchar2 DEFAULT null
  ,p_abs_information21        IN     varchar2 DEFAULT null
  ,p_abs_information22        IN     varchar2 DEFAULT null
  ,p_abs_information23        IN     varchar2 DEFAULT null
  ,p_abs_information24        IN     varchar2 DEFAULT null
  ,p_abs_information25        IN     varchar2 DEFAULT null
  ,p_abs_information26        IN     varchar2 DEFAULT null
  ,p_abs_information27        IN     varchar2 DEFAULT null
  ,p_abs_information28        IN     varchar2 DEFAULT null
  ,p_abs_information29        IN     varchar2 DEFAULT null
  ,p_abs_information30        IN     varchar2 DEFAULT null
  ,p_leave_status             IN     varchar2 DEFAULT 'CONFIRMED'
  ,p_comments_to_approver     IN     varchar2 DEFAULT null
  ,p_transaction_id           IN OUT number
  ,p_transaction_step_id      IN OUT number)

IS

  CURSOR csr_asg IS
  SELECT paaf.*
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.assignment_id = NVL(p_override_assignment_id,p_assignment_id)
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

  CURSOR csr_person IS
  SELECT papf.*
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.assignment_id = NVL(p_override_assignment_id,p_assignment_id)
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

  CURSOR csr_login_person(p_login_person_id IN number) IS  --What If multiple assignments are there ??
  SELECT papf.*, paaf.supervisor_id, paaf.assignment_number
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.primary_flag = 'Y'
  AND    papf.person_id = p_login_person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

  CURSOR csr_abs_attendance_info IS
  SELECT *
  FROM   per_absence_attendances
  WHERE  absence_attendance_id = p_absence_attendance_id;

  CURSOR csr_get_emp_full_name(p_person_id number) IS
  SELECT decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'),'G', global_name, local_name) full_name
  FROM   per_all_people_f
  WHERE  person_id = p_person_id
  AND    trunc(sysdate) BETWEEN
         effective_start_date AND effective_end_date;

  CURSOR csr_get_sec_grp_leg_code(p_person_id number) IS
  SELECT bg.security_group_id
        ,bg.legislation_code
  FROM   per_all_people_f papf
        ,per_business_groups_perf bg
  WHERE  papf.person_id = p_person_id
  AND    papf.business_group_id = bg.business_group_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date and papf.effective_end_date;

  CURSOR csr_get_job_name(p_job_id number) IS
  SELECT name
  FROM   per_jobs_vl
  WHERE  job_id = p_job_id
  AND    trunc(sysdate) BETWEEN date_from AND nvl(date_to,trunc(sysdate));

  CURSOR csr_get_pos_name(p_pos_id number) IS
  SELECT name
  FROM   hr_all_positions_f_vl
  WHERE  position_id = p_pos_id
  AND    trunc(sysdate) BETWEEN nvl(effective_start_date,trunc(sysdate)) AND nvl(effective_end_date,trunc(sysdate));

  CURSOR csr_get_org_info(p_person_id number) IS
  SELECT org_information2
        ,org_information10
  FROM   hr_organization_information_v
  WHERE  org_information_context = 'Business Group Information'
  AND    organization_id         = xxas_per_pkg.get_business_group_id(p_person_id);

  CURSOR csr_get_abs_att_type_meaning(p_abs_att_type_id number) IS
  SELECT fcl.meaning
  FROM   per_abs_attendance_types_vl paat
        ,hr_lookups fcl
  WHERE  absence_attendance_type_id = p_abs_att_type_id
  AND    paat.absence_category    = fcl.lookup_code(+)
  AND    fcl.lookup_type(+)       = 'ABSENCE_CATEGORY';

  CURSOR csr_trans_doc(p_person_id                 IN number
                      ,l_per_row                   IN csr_person%ROWTYPE
                      ,l_asg_row                   IN csr_asg%ROWTYPE
                      ,l_login_per_row             IN csr_login_person%ROWTYPE
                      ,p_per_manager_name          IN varchar2
                      ,p_login_per_manager_name    IN varchar2
                      ,p_transaction_id            IN number
                      ,p_per_sec_group_id          IN number
                      ,p_per_leg_code              IN varchar2
                      ,p_login_per_leg_code        IN varchar2
                      ,p_job_name                  IN varchar2
                      ,p_pos_name                  IN varchar2
                      ,p_emp_generation            IN varchar2
                      ,p_per_bg_cur_code           IN varchar2
                      ,l_abs_row                   IN per_absence_attendances%ROWTYPE
                      ,p_called_from               IN varchar2
                      ,cp_date_projected_start     IN date
                      ,cp_time_projected_start     IN varchar2
                      ,cp_date_projected_end       IN date
                      ,cp_time_projected_end       IN varchar2
                      ,cp_date_start               IN date
                      ,cp_time_start               IN varchar2
                      ,cp_date_end                 IN date
                      ,cp_time_end                 IN varchar2
                      ,cp_absence_days             IN number
                      ,cp_absence_hours            IN number
					  ,cp_attribute1               IN varchar2
                      ,cp_attribute2               IN varchar2
                      ,cp_attribute3               IN varchar2
                      ,cp_attribute4               IN varchar2
                      ,cp_attribute5               IN varchar2
                      ,cp_attribute6               IN varchar2
                      ,cp_attribute7               IN varchar2
                      ,cp_attribute8               IN varchar2
                      ,cp_attribute9               IN varchar2
                      ,cp_attribute10              IN varchar2
                      ,cp_attribute11              IN varchar2
                      ,cp_attribute12              IN varchar2
                      ,cp_attribute13              IN varchar2
                      ,cp_attribute14              IN varchar2
                      ,cp_attribute15              IN varchar2
                      ,cp_attribute16              IN varchar2
                      ,cp_attribute17              IN varchar2
                      ,cp_attribute18              IN varchar2
                      ,cp_attribute19              IN varchar2
                      ,cp_attribute20              IN varchar2
                      ,cp_abs_information_category IN varchar2
                      ,cp_abs_information1         IN varchar2
                      ,cp_abs_information2         IN varchar2
                      ,cp_abs_information3         IN varchar2
                      ,cp_abs_information4         IN varchar2
                      ,cp_abs_information5         IN varchar2
                      ,cp_abs_information6         IN varchar2
                      ,cp_abs_information7         IN varchar2
                      ,cp_abs_information8         IN varchar2
                      ,cp_abs_information9         IN varchar2
                      ,cp_abs_information10        IN varchar2
                      ,cp_abs_information11        IN varchar2
                      ,cp_abs_information12        IN varchar2
                      ,cp_abs_information13        IN varchar2
                      ,cp_abs_information14        IN varchar2
                      ,cp_abs_information15        IN varchar2
                      ,cp_abs_information16        IN varchar2
                      ,cp_abs_information17        IN varchar2
                      ,cp_abs_information18        IN varchar2
                      ,cp_abs_information19        IN varchar2
                      ,cp_abs_information20        IN varchar2
                      ,cp_abs_information21        IN varchar2
                      ,cp_abs_information22        IN varchar2
                      ,cp_abs_information23        IN varchar2
                      ,cp_abs_information24        IN varchar2
                      ,cp_abs_information25        IN varchar2
                      ,cp_abs_information26        IN varchar2
                      ,cp_abs_information27        IN varchar2
                      ,cp_abs_information28        IN varchar2
                      ,cp_abs_information29        IN varchar2
                      ,cp_abs_information30        IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", p_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", l_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", p_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", g_ITEM_TYPE)
             ,XMLElement("PrsnJobId", l_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(l_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", l_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), p_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", p_login_per_leg_code)
             ,XMLElement("PrsnMgrId", l_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", NVL(p_override_assignment_id,p_assignment_id))
             ,XMLElement("PrsnLegCode", p_per_leg_code)
             ,XMLElement("LoginPrsnId", l_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", 'PER')
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(p_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,fnd_function.get_function_id(p_called_from)
               )
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", p_transaction_id)
             ,XMLElement("PrsnLocationId", l_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", p_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", p_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", p_per_manager_name)
             ,XMLElement("PrsnOrganizationId", l_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", l_asg_row.position_id)
             ,XMLElement("TxnStatus", 'W')
             ,XMLElement("LoginWorkerNumber", l_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", p_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", p_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", l_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(p_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_login_per_row.global_name, l_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), l_login_per_row.person_id)
             ,XMLElement("PrsnId", p_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || p_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", g_AME_TRANS_TYPE)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(p_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", p_called_from)
             ,XMLElement("LoginPrsnBgId", l_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", p_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", l_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_per_row.global_name, l_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", p_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'UpdateMode'))
             ,XMLElement("pAMEAppId", g_AME_APP_ID)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (2 as "PS")
                       ,XMLElement("AbsenceAttendanceId", p_absence_attendance_id)
                       ,decode
                         (p_absence_type_id, l_abs_row.absence_attendance_type_id, null
                         ,XMLElement
                           ("AbsenceAttendanceTypeId", p_absence_type_id))
                       ,decode
                         (p_abs_attendance_reason_id, l_abs_row.abs_attendance_reason_id, null
                         ,XMLElement
                           ("AbsAttendanceReasonId", p_abs_attendance_reason_id))
                       ,decode
                         (cp_absence_days,l_abs_row.absence_days, null
                         ,XMLElement
                           ("AbsenceDays", cp_absence_days))
                       ,decode
                         (cp_absence_hours,l_abs_row.absence_hours, null
                         ,XMLElement
                           ("AbsenceHours", cp_absence_hours))
                       ,decode
                         (to_char(cp_date_end, g_HRAPI_DATE_FMT), to_char(l_abs_row.date_end, g_HRAPI_DATE_FMT), null
                         ,XMLElement
                           ("DateEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT)))
                       ,decode
                         (to_char(sysdate, g_HRAPI_DATETIME_FMT), to_char(l_abs_row.date_notification, g_HRAPI_DATETIME_FMT), null
                         ,XMLElement
                           ("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT)))
                       ,decode
                         (to_char(cp_date_projected_end, g_HRAPI_DATE_FMT), to_char(l_abs_row.date_projected_end, g_HRAPI_DATE_FMT), null
                         ,XMLElement
                           ("DateProjectedEnd", to_char(cp_date_projected_end, g_HRAPI_DATE_FMT)))
                       ,decode
                         (to_char(cp_date_projected_start, g_HRAPI_DATE_FMT), to_char(l_abs_row.date_projected_start, g_HRAPI_DATE_FMT), null
                         ,XMLElement
                           ("DateProjectedStart", to_char(cp_date_projected_start, g_HRAPI_DATE_FMT)))
                       ,decode
                         (to_char(cp_date_start, g_HRAPI_DATE_FMT), to_char(l_abs_row.date_start, g_HRAPI_DATE_FMT), null
                         ,XMLElement
                           ("DateStart", to_char(cp_date_start, g_HRAPI_DATE_FMT)))
                       ,decode
                         (cp_time_end,l_abs_row.time_end, null
                         ,XMLElement
                           ("TimeEnd", cp_time_end))
                       ,decode
                         (cp_time_projected_end,l_abs_row.time_projected_end, null
                         ,XMLElement
                           ("TimeProjectedEnd", cp_time_projected_end))
                       ,decode
                         (cp_time_projected_start,l_abs_row.time_projected_start, null
                         ,XMLElement
                           ("TimeProjectedStart", cp_time_projected_start))
                       ,decode
                         (cp_time_start,l_abs_row.time_start, null
                         ,XMLElement
                           ("TimeStart", cp_time_start))
                       ,decode
                         (p_attribute_category, l_abs_row.attribute_category, null
                         ,XMLElement
                           ("AttributeCategory", p_attribute_category))
                       ,decode
                         (p_attribute1, l_abs_row.attribute1, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (p_attribute2, l_abs_row.attribute2, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (p_attribute3, l_abs_row.attribute3, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (p_attribute4, l_abs_row.attribute4, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (p_attribute5, l_abs_row.attribute5, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (p_attribute6, l_abs_row.attribute6, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (p_attribute7, l_abs_row.attribute7, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (p_attribute8, l_abs_row.attribute8, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (p_attribute9, l_abs_row.attribute9, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (p_attribute10, l_abs_row.attribute10, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (p_attribute11, l_abs_row.attribute11, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (p_attribute12, l_abs_row.attribute12, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (p_attribute13, l_abs_row.attribute13, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (p_attribute14, l_abs_row.attribute14, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (p_attribute15, l_abs_row.attribute15, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (p_attribute16, l_abs_row.attribute16, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (p_attribute17, l_abs_row.attribute17, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (p_attribute18, l_abs_row.attribute18, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (p_attribute19, l_abs_row.attribute19, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (p_attribute20, l_abs_row.attribute20, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (p_abs_information_category, l_abs_row.abs_information_category, null
                         ,XMLElement
                           ("AbsInformationCategory", p_abs_information_category))
                       ,decode
                         (p_abs_information1, l_abs_row.abs_information1, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (p_abs_information2, l_abs_row.abs_information2, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (p_abs_information3, l_abs_row.abs_information3, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (p_abs_information4, l_abs_row.abs_information4, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (p_abs_information5, l_abs_row.abs_information5, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (p_abs_information6, l_abs_row.abs_information6, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (p_abs_information7, l_abs_row.abs_information7, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (p_abs_information8, l_abs_row.abs_information8, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (p_abs_information9, l_abs_row.abs_information9, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (p_abs_information10, l_abs_row.abs_information10, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (p_abs_information11, l_abs_row.abs_information11, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (p_abs_information12, l_abs_row.abs_information12, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (p_abs_information13, l_abs_row.abs_information13, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (p_abs_information14, l_abs_row.abs_information14, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (p_abs_information15, l_abs_row.abs_information15, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (p_abs_information16, l_abs_row.abs_information16, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (p_abs_information17, l_abs_row.abs_information17, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (p_abs_information18, l_abs_row.abs_information18, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (p_abs_information19, l_abs_row.abs_information19, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (p_abs_information20, l_abs_row.abs_information20, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (p_abs_information21, l_abs_row.abs_information21, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (p_abs_information22, l_abs_row.abs_information22, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (p_abs_information23, l_abs_row.abs_information23, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (p_abs_information24, l_abs_row.abs_information24, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (p_abs_information25, l_abs_row.abs_information25, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (p_abs_information26, l_abs_row.abs_information26, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (p_abs_information27, l_abs_row.abs_information27, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (p_abs_information28, l_abs_row.abs_information28, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (p_abs_information29, l_abs_row.abs_information29, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (p_abs_information30, l_abs_row.abs_information30, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("ObjectVersionNumber", p_object_version_number)
                       ,decode
                         (p_replacement_person_id, l_abs_row.replacement_person_id, null
                         ,XMLElement
                           ("ReplacementPersonId", p_replacement_person_id))
                       ,decode(p_comments_to_approver, null, null, XMLElement("Comments", p_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_transaction_id))
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_rc_trans_doc(p_person_id                 IN number
                         ,l_per_row                   IN csr_person%ROWTYPE
                         ,l_asg_row                   IN csr_asg%ROWTYPE
                         ,l_login_per_row             IN csr_login_person%ROWTYPE
                         ,p_per_manager_name          IN varchar2
                         ,p_login_per_manager_name    IN varchar2
                         ,p_transaction_id            IN number
                         ,p_per_sec_group_id          IN number
                         ,p_per_leg_code              IN varchar2
                         ,p_login_per_leg_code        IN varchar2
                         ,p_job_name                  IN varchar2
                         ,p_pos_name                  IN varchar2
                         ,p_emp_generation            IN varchar2
                         ,p_per_bg_cur_code           IN varchar2
                         ,l_abs_row                   IN per_absence_attendances%ROWTYPE
                         ,p_called_from               IN varchar2
                         ,cp_date_projected_start     IN date
                         ,cp_time_projected_start     IN varchar2
                         ,cp_date_projected_end       IN date
                         ,cp_time_projected_end	      IN varchar2
                         ,cp_date_start	              IN date
                         ,cp_time_start	              IN varchar2
                         ,cp_date_end	              IN date
                         ,cp_time_end                 IN varchar2
                         ,cp_absence_days             IN number
                         ,cp_absence_hours            IN number
                         ,cp_attribute1               IN varchar2
                         ,cp_attribute2               IN varchar2
                         ,cp_attribute3               IN varchar2
                         ,cp_attribute4               IN varchar2
                         ,cp_attribute5               IN varchar2
                         ,cp_attribute6               IN varchar2
                         ,cp_attribute7               IN varchar2
                         ,cp_attribute8               IN varchar2
                         ,cp_attribute9               IN varchar2
                         ,cp_attribute10              IN varchar2
                         ,cp_attribute11              IN varchar2
                         ,cp_attribute12              IN varchar2
                         ,cp_attribute13              IN varchar2
                         ,cp_attribute14              IN varchar2
                         ,cp_attribute15              IN varchar2
                         ,cp_attribute16              IN varchar2
                         ,cp_attribute17              IN varchar2
                         ,cp_attribute18              IN varchar2
                         ,cp_attribute19              IN varchar2
                         ,cp_attribute20              IN varchar2
                         ,cp_abs_information_category IN varchar2
                         ,cp_abs_information1         IN varchar2
                         ,cp_abs_information2         IN varchar2
                         ,cp_abs_information3         IN varchar2
                         ,cp_abs_information4         IN varchar2
                         ,cp_abs_information5         IN varchar2
                         ,cp_abs_information6         IN varchar2
                         ,cp_abs_information7         IN varchar2
                         ,cp_abs_information8         IN varchar2
                         ,cp_abs_information9         IN varchar2
                         ,cp_abs_information10        IN varchar2
                         ,cp_abs_information11        IN varchar2
                         ,cp_abs_information12        IN varchar2
                         ,cp_abs_information13        IN varchar2
                         ,cp_abs_information14        IN varchar2
                         ,cp_abs_information15        IN varchar2
                         ,cp_abs_information16        IN varchar2
                         ,cp_abs_information17        IN varchar2
                         ,cp_abs_information18        IN varchar2
                         ,cp_abs_information19        IN varchar2
                         ,cp_abs_information20        IN varchar2
                         ,cp_abs_information21        IN varchar2
                         ,cp_abs_information22        IN varchar2
                         ,cp_abs_information23        IN varchar2
                         ,cp_abs_information24        IN varchar2
                         ,cp_abs_information25        IN varchar2
                         ,cp_abs_information26        IN varchar2
                         ,cp_abs_information27        IN varchar2
                         ,cp_abs_information28        IN varchar2
                         ,cp_abs_information29        IN varchar2
                         ,cp_abs_information30        IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", p_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", l_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", p_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", g_ITEM_TYPE)
             ,XMLElement("PrsnJobId", l_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(l_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", l_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), p_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", p_login_per_leg_code)
             ,XMLElement("PrsnMgrId", l_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", NVL(p_override_assignment_id,p_assignment_id))
             ,XMLElement("PrsnLegCode", p_per_leg_code)
             ,XMLElement("LoginPrsnId", l_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", 'PER')
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(p_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,fnd_function.get_function_id(p_called_from)
               )
             ,XMLElement("TxnState", 'T')
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", p_transaction_id)
             ,XMLElement("PrsnLocationId", l_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", p_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", p_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", p_per_manager_name)
             ,XMLElement("PrsnOrganizationId", l_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", l_asg_row.position_id)
             ,XMLElement("TxnStatus", 'RI')
             ,XMLElement("LoginWorkerNumber", l_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", p_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", p_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", l_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(p_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_login_per_row.global_name, l_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), l_login_per_row.person_id)
             ,XMLElement("PrsnId", p_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || p_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", g_AME_TRANS_TYPE)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(p_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", p_called_from)
             ,XMLElement("LoginPrsnBgId", l_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", p_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", l_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_per_row.global_name, l_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", p_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'UpdateMode'))
             ,XMLElement("pAMEAppId", g_AME_APP_ID)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (2 as "PS")
                       ,XMLElement("AbsenceAttendanceId", p_absence_attendance_id)
                       ,decode
                         (p_absence_type_id, l_abs_row.absence_attendance_type_id, null
                         ,XMLElement
                           ("AbsenceAttendanceTypeId", p_absence_type_id))
                       ,decode
                         (p_abs_attendance_reason_id, l_abs_row.abs_attendance_reason_id, null
                         ,XMLElement
                           ("AbsAttendanceReasonId", p_abs_attendance_reason_id))
                       ,decode
                         (cp_absence_days,l_abs_row.absence_days, null
                         ,XMLElement
                           ("AbsenceDays", cp_absence_days))
                       ,decode
                         (cp_absence_hours,l_abs_row.absence_hours, null
                         ,XMLElement
                           ("AbsenceHours", cp_absence_hours))
                       ,decode
                         (to_char(cp_date_end, g_HRAPI_DATE_FMT), to_char(l_abs_row.date_end, g_HRAPI_DATE_FMT), null
                         ,XMLElement
                           ("DateEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT)))
                       ,decode
                         (to_char(sysdate, g_HRAPI_DATETIME_FMT), to_char(l_abs_row.date_notification, g_HRAPI_DATETIME_FMT), null
                         ,XMLElement
                           ("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT)))
                       ,decode
                         (to_char(cp_date_projected_end, g_HRAPI_DATE_FMT), to_char(l_abs_row.date_projected_end, g_HRAPI_DATE_FMT), null
                         ,XMLElement
                           ("DateProjectedEnd", to_char(cp_date_projected_end, g_HRAPI_DATE_FMT)))
                       ,decode
                         (to_char(cp_date_projected_start, g_HRAPI_DATE_FMT), to_char(l_abs_row.date_projected_start, g_HRAPI_DATE_FMT), null
                         ,XMLElement
                           ("DateProjectedStart", to_char(cp_date_projected_start, g_HRAPI_DATE_FMT)))
                       ,decode
                         (to_char(cp_date_start, g_HRAPI_DATE_FMT), to_char(l_abs_row.date_start, g_HRAPI_DATE_FMT), null
                         ,XMLElement
                           ("DateStart", to_char(cp_date_start, g_HRAPI_DATE_FMT)))
                       ,decode
                         (cp_time_end,l_abs_row.time_end, null
                         ,XMLElement
                           ("TimeEnd", cp_time_end))
                       ,decode
                         (cp_time_projected_end,l_abs_row.time_projected_end, null
                         ,XMLElement
                           ("TimeProjectedEnd", cp_time_projected_end))
                       ,decode
                         (cp_time_projected_start,l_abs_row.time_projected_start, null
                         ,XMLElement
                           ("TimeProjectedStart", cp_time_projected_start))
                       ,decode
                         (cp_time_start,l_abs_row.time_start, null
                         ,XMLElement
                           ("TimeStart", cp_time_start))
                       ,decode
                         (p_attribute_category, l_abs_row.attribute_category, null
                         ,XMLElement
                           ("AttributeCategory", p_attribute_category))
                       ,decode
                         (p_attribute1, l_abs_row.attribute1, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (p_attribute2, l_abs_row.attribute2, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (p_attribute3, l_abs_row.attribute3, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (p_attribute4, l_abs_row.attribute4, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (p_attribute5, l_abs_row.attribute5, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (p_attribute6, l_abs_row.attribute6, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (p_attribute7, l_abs_row.attribute7, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (p_attribute8, l_abs_row.attribute8, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (p_attribute9, l_abs_row.attribute9, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (p_attribute10, l_abs_row.attribute10, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (p_attribute11, l_abs_row.attribute11, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (p_attribute12, l_abs_row.attribute12, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (p_attribute13, l_abs_row.attribute13, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (p_attribute14, l_abs_row.attribute14, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (p_attribute15, l_abs_row.attribute15, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (p_attribute16, l_abs_row.attribute16, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (p_attribute17, l_abs_row.attribute17, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (p_attribute18, l_abs_row.attribute18, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (p_attribute19, l_abs_row.attribute19, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (p_attribute20, l_abs_row.attribute20, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (p_abs_information_category, l_abs_row.abs_information_category, null
                         ,XMLElement
                           ("AbsInformationCategory", p_abs_information_category))
                       ,decode
                         (p_abs_information1, l_abs_row.abs_information1, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (p_abs_information2, l_abs_row.abs_information2, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (p_abs_information3, l_abs_row.abs_information3, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (p_abs_information4, l_abs_row.abs_information4, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (p_abs_information5, l_abs_row.abs_information5, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (p_abs_information6, l_abs_row.abs_information6, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (p_abs_information7, l_abs_row.abs_information7, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (p_abs_information8, l_abs_row.abs_information8, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (p_abs_information9, l_abs_row.abs_information9, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (p_abs_information10, l_abs_row.abs_information10, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (p_abs_information11, l_abs_row.abs_information11, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (p_abs_information12, l_abs_row.abs_information12, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (p_abs_information13, l_abs_row.abs_information13, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (p_abs_information14, l_abs_row.abs_information14, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (p_abs_information15, l_abs_row.abs_information15, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (p_abs_information16, l_abs_row.abs_information16, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (p_abs_information17, l_abs_row.abs_information17, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (p_abs_information18, l_abs_row.abs_information18, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (p_abs_information19, l_abs_row.abs_information19, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (p_abs_information20, l_abs_row.abs_information20, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (p_abs_information21, l_abs_row.abs_information21, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (p_abs_information22, l_abs_row.abs_information22, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (p_abs_information23, l_abs_row.abs_information23, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (p_abs_information24, l_abs_row.abs_information24, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (p_abs_information25, l_abs_row.abs_information25, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (p_abs_information26, l_abs_row.abs_information26, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (p_abs_information27, l_abs_row.abs_information27, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (p_abs_information28, l_abs_row.abs_information28, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (p_abs_information29, l_abs_row.abs_information29, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (p_abs_information30, l_abs_row.abs_information30, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("ObjectVersionNumber", p_object_version_number)
                       ,decode
                         (p_replacement_person_id, l_abs_row.replacement_person_id, null
                         ,XMLElement
                           ("ReplacementPersonId", p_replacement_person_id))
                       ,decode(p_comments_to_approver, null, null, XMLElement("Comments", p_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA('000100000000')
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_rc_trans_create_doc(p_person_id                 IN number
                                ,p_absence_attendance_id     IN number
                                ,l_per_row                   IN csr_person%ROWTYPE
                                ,l_asg_row                   IN csr_asg%ROWTYPE
                                ,l_login_per_row             IN csr_login_person%ROWTYPE
                                ,p_per_manager_name          IN varchar2
                                ,p_login_per_manager_name    IN varchar2
                                ,p_transaction_id            IN number
                                ,p_per_sec_group_id          IN number
                                ,p_per_leg_code              IN varchar2
                                ,p_login_per_leg_code        IN varchar2
                                ,p_job_name                  IN varchar2
                                ,p_pos_name                  IN varchar2
                                ,p_emp_generation            IN varchar2
                                ,p_per_bg_cur_code           IN varchar2
                                ,p_called_from               IN varchar2
                                ,cp_date_projected_start     IN date
                                ,cp_time_projected_start     IN varchar2
                                ,cp_date_projected_end       IN date
                                ,cp_time_projected_end       IN varchar2
                                ,cp_date_start               IN date
                                ,cp_time_start               IN varchar2
                                ,cp_date_end                 IN date
                                ,cp_time_end                 IN varchar2
                                ,cp_absence_days             IN number
                                ,cp_absence_hours            IN number
                                ,cp_attribute1               IN varchar2
                                ,cp_attribute2               IN varchar2
                                ,cp_attribute3               IN varchar2
                                ,cp_attribute4               IN varchar2
                                ,cp_attribute5               IN varchar2
                                ,cp_attribute6               IN varchar2
                                ,cp_attribute7               IN varchar2
                                ,cp_attribute8               IN varchar2
                                ,cp_attribute9               IN varchar2
                                ,cp_attribute10              IN varchar2
                                ,cp_attribute11              IN varchar2
                                ,cp_attribute12              IN varchar2
                                ,cp_attribute13              IN varchar2
                                ,cp_attribute14              IN varchar2
                                ,cp_attribute15              IN varchar2
                                ,cp_attribute16              IN varchar2
                                ,cp_attribute17              IN varchar2
                                ,cp_attribute18              IN varchar2
                                ,cp_attribute19              IN varchar2
                                ,cp_attribute20              IN varchar2
                                ,cp_abs_information_category IN varchar2
                                ,cp_abs_information1         IN varchar2
                                ,cp_abs_information2         IN varchar2
                                ,cp_abs_information3         IN varchar2
                                ,cp_abs_information4         IN varchar2
                                ,cp_abs_information5         IN varchar2
                                ,cp_abs_information6         IN varchar2
                                ,cp_abs_information7         IN varchar2
                                ,cp_abs_information8         IN varchar2
                                ,cp_abs_information9         IN varchar2
                                ,cp_abs_information10        IN varchar2
                                ,cp_abs_information11        IN varchar2
                                ,cp_abs_information12        IN varchar2
                                ,cp_abs_information13        IN varchar2
                                ,cp_abs_information14        IN varchar2
                                ,cp_abs_information15        IN varchar2
                                ,cp_abs_information16        IN varchar2
                                ,cp_abs_information17        IN varchar2
                                ,cp_abs_information18        IN varchar2
                                ,cp_abs_information19        IN varchar2
                                ,cp_abs_information20        IN varchar2
                                ,cp_abs_information21        IN varchar2
                                ,cp_abs_information22        IN varchar2
                                ,cp_abs_information23        IN varchar2
                                ,cp_abs_information24        IN varchar2
                                ,cp_abs_information25        IN varchar2
                                ,cp_abs_information26        IN varchar2
                                ,cp_abs_information27        IN varchar2
                                ,cp_abs_information28        IN varchar2
                                ,cp_abs_information29        IN varchar2
                                ,cp_abs_information30        IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", p_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", l_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", p_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", g_ITEM_TYPE)
             ,XMLElement("PrsnJobId", l_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(l_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", l_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), p_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", p_login_per_leg_code)
             ,XMLElement("PrsnMgrId", l_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", NVL(p_override_assignment_id,p_assignment_id))
             ,XMLElement("PrsnLegCode", p_per_leg_code)
             ,XMLElement("LoginPrsnId", l_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", 'PER')
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(p_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,fnd_function.get_function_id(p_called_from)
               )
             ,XMLElement("TxnState", 'T')
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", p_transaction_id)
             ,XMLElement("PrsnLocationId", l_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", p_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", p_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", p_per_manager_name)
             ,XMLElement("PrsnOrganizationId", l_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", l_asg_row.position_id)
             ,XMLElement("TxnStatus", 'RI')
             ,XMLElement("LoginWorkerNumber", l_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", p_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", p_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", l_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(p_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_login_per_row.global_name, l_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), l_login_per_row.person_id)
             ,XMLElement("PrsnId", p_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || p_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", g_AME_TRANS_TYPE)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(p_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", p_called_from)
             ,XMLElement("LoginPrsnBgId", l_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", p_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", l_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_per_row.global_name, l_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", p_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CreateMode'))
             ,XMLElement("pAMEAppId", g_AME_APP_ID)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (0 as "PS"
                       ,'Y' as "PK")
                       ,XMLElement("AbsenceAttendanceId", p_absence_attendance_id)
                       ,XMLElement("BusinessGroupId", l_asg_row.business_group_id)
                       ,XMLElement("AbsenceAttendanceTypeId", p_absence_type_id)
                       ,XMLElement("AbsAttendanceReasonId", p_abs_attendance_reason_id)
                       ,XMLElement("PersonId", p_person_id)
                       ,decode(cp_absence_days, null, XMLElement("AbsenceDays", XMLAttributes('true' as "null"), cp_absence_days), XMLElement("AbsenceDays", cp_absence_days))
                       ,decode(cp_absence_hours, null, XMLElement("AbsenceHours", XMLAttributes('true' as "null"), cp_absence_hours), XMLElement("AbsenceHours", cp_absence_hours))
                       ,decode(cp_date_end, null, XMLElement("DateEnd", XMLAttributes('true' as "null"), cp_date_end), XMLElement("DateEnd", to_char(cp_date_end, g_HRAPI_DATE_FMT)))
                       ,XMLElement("DateNotification", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,decode(cp_date_projected_end, null, XMLElement("DateProjectedEnd", XMLAttributes('true' as "null"), cp_date_projected_end), XMLElement("DateProjectedEnd", to_char(cp_date_projected_end, g_HRAPI_DATE_FMT)))
                       ,decode(cp_date_projected_start, null, XMLElement("DateProjectedStart", XMLAttributes('true' as "null"), cp_date_projected_start), XMLElement("DateProjectedStart", to_char(cp_date_projected_start, g_HRAPI_DATE_FMT)))
                       ,decode(cp_date_start, null, XMLElement("DateStart", XMLAttributes('true' as "null"), cp_date_start), XMLElement("DateStart", to_char(cp_date_start, g_HRAPI_DATE_FMT)))
                       ,decode(cp_time_end, null, XMLElement("TimeEnd", XMLAttributes('true' as "null"), cp_time_end), XMLElement("TimeEnd", cp_time_end))
                       ,decode(cp_time_projected_end, null, XMLElement("TimeProjectedEnd", XMLAttributes('true' as "null"), cp_time_projected_end), XMLElement("TimeProjectedEnd", cp_time_projected_end))
                       ,decode(cp_time_projected_start, null, XMLElement("TimeProjectedStart", XMLAttributes('true' as "null"), cp_time_projected_start), XMLElement("TimeProjectedStart", cp_time_projected_start))
                       ,decode(cp_time_start, null, XMLElement("TimeStart", XMLAttributes('true' as "null"), cp_time_start), XMLElement("TimeStart", cp_time_start))
                       ,decode(p_replacement_person_id, null, XMLElement("ReplacementPersonId", XMLAttributes('true' as "null"), p_replacement_person_id), XMLElement("ReplacementPersonId", p_replacement_person_id))
					   ,decode
                         (p_attribute_category, null, null
                         ,XMLElement
                           ("AttributeCategory", p_attribute_category))
                       ,decode
                         (p_attribute1, null, null
                         ,XMLElement
                           ("Attribute1", cp_attribute1))
                       ,decode
                         (p_attribute2, null, null
                         ,XMLElement
                           ("Attribute2", cp_attribute2))
                       ,decode
                         (p_attribute3, null, null
                         ,XMLElement
                           ("Attribute3", cp_attribute3))
                       ,decode
                         (p_attribute4, null, null
                         ,XMLElement
                           ("Attribute4", cp_attribute4))
                       ,decode
                         (p_attribute5, null, null
                         ,XMLElement
                           ("Attribute5", cp_attribute5))
                       ,decode
                         (p_attribute6, null, null
                         ,XMLElement
                           ("Attribute6", cp_attribute6))
                       ,decode
                         (p_attribute7, null, null
                         ,XMLElement
                           ("Attribute7", cp_attribute7))
                       ,decode
                         (p_attribute8, null, null
                         ,XMLElement
                           ("Attribute8", cp_attribute8))
                       ,decode
                         (p_attribute9, null, null
                         ,XMLElement
                           ("Attribute9", cp_attribute9))
                       ,decode
                         (p_attribute10, null, null
                         ,XMLElement
                           ("Attribute10", cp_attribute10))
                       ,decode
                         (p_attribute11, null, null
                         ,XMLElement
                           ("Attribute11", cp_attribute11))
                       ,decode
                         (p_attribute12, null, null
                         ,XMLElement
                           ("Attribute12", cp_attribute12))
                       ,decode
                         (p_attribute13, null, null
                         ,XMLElement
                           ("Attribute13", cp_attribute13))
                       ,decode
                         (p_attribute14, null, null
                         ,XMLElement
                           ("Attribute14", cp_attribute14))
                       ,decode
                         (p_attribute15, null, null
                         ,XMLElement
                           ("Attribute15", cp_attribute15))
                       ,decode
                         (p_attribute16, null, null
                         ,XMLElement
                           ("Attribute16", cp_attribute16))
                       ,decode
                         (p_attribute17, null, null
                         ,XMLElement
                           ("Attribute17", cp_attribute17))
                       ,decode
                         (p_attribute18, null, null
                         ,XMLElement
                           ("Attribute18", cp_attribute18))
                       ,decode
                         (p_attribute19, null, null
                         ,XMLElement
                           ("Attribute19", cp_attribute19))
                       ,decode
                         (p_attribute20, null, null
                         ,XMLElement
                           ("Attribute20", cp_attribute20))
                       ,decode
                         (p_abs_information_category, null, null
                         ,XMLElement
                           ("AbsInformationCategory", p_abs_information_category))
                       ,decode
                         (p_abs_information1, null, null
                         ,XMLElement
                           ("AbsInformation1", cp_abs_information1))
                       ,decode
                         (p_abs_information2, null, null
                         ,XMLElement
                           ("AbsInformation2", cp_abs_information2))
                       ,decode
                         (p_abs_information3, null, null
                         ,XMLElement
                           ("AbsInformation3", cp_abs_information3))
                       ,decode
                         (p_abs_information4, null, null
                         ,XMLElement
                           ("AbsInformation4", cp_abs_information4))
                       ,decode
                         (p_abs_information5, null, null
                         ,XMLElement
                           ("AbsInformation5", cp_abs_information5))
                       ,decode
                         (p_abs_information6, null, null
                         ,XMLElement
                           ("AbsInformation6", cp_abs_information6))
                       ,decode
                         (p_abs_information7, null, null
                         ,XMLElement
                           ("AbsInformation7", cp_abs_information7))
                       ,decode
                         (p_abs_information8, null, null
                         ,XMLElement
                           ("AbsInformation8", cp_abs_information8))
                       ,decode
                         (p_abs_information9, null, null
                         ,XMLElement
                           ("AbsInformation9", cp_abs_information9))
                       ,decode
                         (p_abs_information10, null, null
                         ,XMLElement
                           ("AbsInformation10", cp_abs_information10))
                       ,decode
                         (p_abs_information11, null, null
                         ,XMLElement
                           ("AbsInformation11", cp_abs_information11))
                       ,decode
                         (p_abs_information12, null, null
                         ,XMLElement
                           ("AbsInformation12", cp_abs_information12))
                       ,decode
                         (p_abs_information13, null, null
                         ,XMLElement
                           ("AbsInformation13", cp_abs_information13))
                       ,decode
                         (p_abs_information14, null, null
                         ,XMLElement
                           ("AbsInformation14", cp_abs_information14))
                       ,decode
                         (p_abs_information15, null, null
                         ,XMLElement
                           ("AbsInformation15", cp_abs_information15))
                       ,decode
                         (p_abs_information16, null, null
                         ,XMLElement
                           ("AbsInformation16", cp_abs_information16))
                       ,decode
                         (p_abs_information17, null, null
                         ,XMLElement
                           ("AbsInformation17", cp_abs_information17))
                       ,decode
                         (p_abs_information18, null, null
                         ,XMLElement
                           ("AbsInformation18", cp_abs_information18))
                       ,decode
                         (p_abs_information19, null, null
                         ,XMLElement
                           ("AbsInformation19", cp_abs_information19))
                       ,decode
                         (p_abs_information20, null, null
                         ,XMLElement
                           ("AbsInformation20", cp_abs_information20))
                       ,decode
                         (p_abs_information21, null, null
                         ,XMLElement
                           ("AbsInformation21", cp_abs_information21))
                       ,decode
                         (p_abs_information22, null, null
                         ,XMLElement
                           ("AbsInformation22", cp_abs_information22))
                       ,decode
                         (p_abs_information23, null, null
                         ,XMLElement
                           ("AbsInformation23", cp_abs_information23))
                       ,decode
                         (p_abs_information24, null, null
                         ,XMLElement
                           ("AbsInformation24", cp_abs_information24))
                       ,decode
                         (p_abs_information25, null, null
                         ,XMLElement
                           ("AbsInformation25", cp_abs_information25))
                       ,decode
                         (p_abs_information26, null, null
                         ,XMLElement
                           ("AbsInformation26", cp_abs_information26))
                       ,decode
                         (p_abs_information27, null, null
                         ,XMLElement
                           ("AbsInformation27", cp_abs_information27))
                       ,decode
                         (p_abs_information28, null, null
                         ,XMLElement
                           ("AbsInformation28", cp_abs_information28))
                       ,decode
                         (p_abs_information29, null, null
                         ,XMLElement
                           ("AbsInformation29", cp_abs_information29))
                       ,decode
                         (p_abs_information30, null, null
                         ,XMLElement
                           ("AbsInformation30", cp_abs_information30))
                       ,XMLElement("LastUpdateDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("LastUpdatedBy", fnd_global.user_id)
                       ,XMLElement("LastUpdateLogin", fnd_global.login_id)
                       ,XMLElement("CreatedBy", fnd_global.user_id)
                       ,XMLElement("CreationDate", to_char(sysdate, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"), null)
                       ,decode(p_comments_to_approver, null, XMLElement("Comments", XMLAttributes('true' as "null"), p_comments_to_approver), XMLElement("Comments", p_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA('000100000000')
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_nls_data_format
  IS
  SELECT value
  FROM   nls_session_parameters nsp
  WHERE  nsp.parameter = 'NLS_DATE_FORMAT';

  CURSOR csr_workflow_info IS
  SELECT hat.item_type
        ,hat.item_key
        ,hat.transaction_ref_id
  FROM   hr_api_transactions  hat
  WHERE  hat.transaction_id = p_transaction_id;

  l_flex_vs_id       number;
  l_temp                       varchar2(1000);

  CURSOR csr_get_vs_id IS
  SELECT flex_value_set_id
  FROM   fnd_flex_value_sets
  WHERE  flex_value_set_name = 'PQP_GB_OSP_PART_DAYS_VS';

  CURSOR csr_get_dff_info IS
  SELECT application_column_name
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  flex_value_set_id = l_flex_vs_id
  AND    enabled_flag = 'Y'
  AND    descriptive_flex_context_code in (p_attribute_category, p_abs_information_category);

  CURSOR csr_get_dff_gdd IS
  SELECT application_column_name
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  flex_value_set_id = l_flex_vs_id
  AND    enabled_flag = 'Y'
  AND    descriptive_flex_context_code = 'Global Data Elements';

  CURSOR csr_get_enabled_dff_gdd IS
  SELECT application_column_name
  FROM   FND_DESCR_FLEX_COL_USAGE_VL
  WHERE  enabled_flag = 'Y'
  and    descriptive_flexfield_name in ('PER_ABS_DEVELOPER_DF','PER_ABSENCE_ATTENDANCES')
  AND    descriptive_flex_context_code = 'Global Data Elements';

  c_method    CONSTANT varchar2(30) := 'update_emp_view_row';

  c_person_id CONSTANT number       := xxas_emp_view_util_pkg.get_person_id
                                         (p_assignment_id  => NVL(p_override_assignment_id,p_assignment_id)
                                         ,p_effective_date => p_effective_date);

  l_object_version_number      number       := p_object_version_number;
  l_absence_days               number       := p_absence_days;
  l_absence_hours              number       := p_absence_hours;
  l_date_projected_start       date         := p_date_projected_start;
  l_time_projected_start       varchar2(10) := p_time_projected_start;
  l_date_projected_end         date         := p_date_projected_end;
  l_time_projected_end         varchar2(10) := p_time_projected_end;
  l_date_start                 date         := p_date_start;
  l_time_start                 varchar2(10) := p_time_start;
  l_date_end                   date         := p_date_end;
  l_time_end                   varchar2(10) := p_time_end;
  l_hours_or_days              varchar2(30);
  l_item_type                  wf_item_activity_statuses.item_type%TYPE;
  l_item_key                   wf_item_activity_statuses.item_key%TYPE;
  l_activity_id                wf_item_activity_statuses.process_activity%TYPE;
  l_asg_row                    csr_asg%ROWTYPE;
  l_per_row                    csr_person%ROWTYPE;
  l_login_per_row              csr_login_person%ROWTYPE;
  l_abs_row                    per_absence_attendances%ROWTYPE;
  l_login_person_id            number;
  l_per_manager_name           varchar2(240);
  l_login_per_manager_name     varchar2(240);
  l_doc                        XMLType;
  l_rc_doc                     XMLType;
  l_transaction_id             number;
  l_per_sec_group_id           number;
  l_per_leg_code               varchar2(150);
  l_login_per_sec_group_id     number;
  l_login_per_leg_code         varchar2(150);
  l_per_job_name               varchar2(700);
  l_per_position_name          varchar2(240);
  l_emp_generation             varchar2(150);
  l_per_bg_cur_code            varchar2(150);
  l_abs_att_category           varchar2(240);
  l_msg_data                   varchar2(2000) := null;
  l_msg_count                  number := 0;
  l_api_errors                 boolean := false;
  l_return_status              varchar2(30);
  l_called_from                fnd_form_functions.function_name%TYPE;
  l_workflow_info              csr_workflow_info%ROWTYPE;
  l_attribute1                 varchar2(1000);
  l_attribute2                 varchar2(1000);
  l_attribute3                 varchar2(1000);
  l_attribute4                 varchar2(1000);
  l_attribute5                 varchar2(1000);
  l_attribute6                 varchar2(1000);
  l_attribute7                 varchar2(1000);
  l_attribute8                 varchar2(1000);
  l_attribute9                 varchar2(1000);
  l_attribute10                varchar2(1000);
  l_attribute11                varchar2(1000);
  l_attribute12                varchar2(1000);
  l_attribute13                varchar2(1000);
  l_attribute14                varchar2(1000);
  l_attribute15                varchar2(1000);
  l_attribute16                varchar2(1000);
  l_attribute17                varchar2(1000);
  l_attribute18                varchar2(1000);
  l_attribute19                varchar2(1000);
  l_attribute20                varchar2(1000);
  l_abs_information1           varchar2(1000);
  l_abs_information2           varchar2(1000);
  l_abs_information3           varchar2(1000);
  l_abs_information4           varchar2(1000);
  l_abs_information5           varchar2(1000);
  l_abs_information6           varchar2(1000);
  l_abs_information7           varchar2(1000);
  l_abs_information8           varchar2(1000);
  l_abs_information9           varchar2(1000);
  l_abs_information10          varchar2(1000);
  l_abs_information11          varchar2(1000);
  l_abs_information12          varchar2(1000);
  l_abs_information13          varchar2(1000);
  l_abs_information14          varchar2(1000);
  l_abs_information15          varchar2(1000);
  l_abs_information16          varchar2(1000);
  l_abs_information17          varchar2(1000);
  l_abs_information18          varchar2(1000);
  l_abs_information19          varchar2(1000);
  l_abs_information20          varchar2(1000);
  l_abs_information21          varchar2(1000);
  l_abs_information22          varchar2(1000);
  l_abs_information23          varchar2(1000);
  l_abs_information24          varchar2(1000);
  l_abs_information25          varchar2(1000);
  l_abs_information26          varchar2(1000);
  l_abs_information27          varchar2(1000);
  l_abs_information28          varchar2(1000);
  l_abs_information29          varchar2(1000);
  l_abs_information30          varchar2(1000);
  l_absence_attendance_id1     number;
  l_occurrence                 varchar2(240);

BEGIN

  log_start(c_method, p_effective_date, p_assignment_id, p_absence_attendance_id, p_absence_attendance_id, p_object_version_number, p_absence_type_id, p_abs_attendance_reason_id, p_override_assignment_id, p_transaction_id, p_transaction_step_id);

  xxas_util_pkg.user_has_access_check(g_PROD_KEY);

  g_ITEM_TYPE          := xxas_emp_view_util_pkg.get_item_type;
  g_AME_APP_ID         := xxas_emp_view_util_pkg.get_ame_application_id;
  g_AME_TRANS_TYPE     := xxas_emp_view_util_pkg.get_ame_transaction_type;
  g_PROCESS_NAME       := xxas_emp_view_util_pkg.get_process_name;

  log_statement(c_method,'g_ITEM_TYPE: ' || g_ITEM_TYPE);
  log_statement(c_method,'g_AME_APP_ID: ' || g_AME_APP_ID);
  log_statement(c_method,'g_AME_TRANS_TYPE: ' || g_AME_TRANS_TYPE);
  log_statement(c_method,'g_PROCESS_NAME: ' || g_PROCESS_NAME);

  l_hours_or_days := xxas_leave_of_absence_pkg.get_absence_type_value
                       (p_absence_type_id => p_absence_type_id
                       ,p_column_name     => 'HOURS_OR_DAYS');

  IF l_hours_or_days = 'H' THEN
    l_absence_days := null;
  ELSIF l_hours_or_days <> 'XXASNODATA' THEN
    l_absence_hours := null;
  END IF;

  IF p_leave_status = 'CONFIRMED' THEN

    l_date_projected_start := null;
    l_time_projected_start := null;
    l_date_projected_end   := null;
    l_time_projected_end   := null;

  ELSE

    l_date_start := null;
    l_time_start := null;
    l_date_end   := null;
    l_time_end   := null;

  END IF;

  --execute immediate 'alter session set nls_date_format="DD-MON-RR"';

  OPEN  csr_person;
  FETCH csr_person INTO l_per_row;
  CLOSE csr_person;

  OPEN  csr_asg;
  FETCH csr_asg INTO l_asg_row;
  CLOSE csr_asg;

  OPEN  csr_abs_attendance_info;
  FETCH csr_abs_attendance_info INTO l_abs_row;
  CLOSE csr_abs_attendance_info;

  OPEN  csr_get_emp_full_name(l_asg_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_job_name(l_asg_row.job_id);
  FETCH csr_get_job_name INTO l_per_job_name;
  CLOSE csr_get_job_name;

  OPEN  csr_get_pos_name(l_asg_row.position_id);
  FETCH csr_get_pos_name INTO l_per_position_name;
  CLOSE csr_get_pos_name;

  OPEN  csr_get_sec_grp_leg_code(c_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_per_sec_group_id
                                     ,l_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  OPEN  csr_get_org_info(c_person_id);
  FETCH csr_get_org_info INTO l_emp_generation
                             ,l_per_bg_cur_code;
  CLOSE csr_get_org_info;

  IF p_transaction_id is not null AND
     p_transaction_step_id is not null THEN

    l_transaction_id      := p_transaction_id;

    OPEN  csr_workflow_info;
    FETCH csr_workflow_info INTO l_workflow_info;
    CLOSE csr_workflow_info;

    l_item_type             := l_workflow_info.item_type;
    l_item_key              := l_workflow_info.item_key;

  ELSE

    --
    -- Generate the new sequences
    --
    SELECT hr_api_transactions_s.nextval
    INTO   l_transaction_id
    FROM   dual;

  END IF;

  l_login_person_id := xxas_util_pkg.get_per_from_user(fnd_global.user_id);

  OPEN  csr_login_person(l_login_person_id);
  FETCH csr_login_person INTO l_login_per_row;
  CLOSE csr_login_person;

  OPEN  csr_get_emp_full_name(l_login_per_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_login_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_sec_grp_leg_code(l_login_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_login_per_sec_group_id
                                     ,l_login_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  l_attribute1         := p_attribute1;
  l_attribute2         := p_attribute2;
  l_attribute3         := p_attribute3;
  l_attribute4         := p_attribute4;
  l_attribute5         := p_attribute5;
  l_attribute6         := p_attribute6;
  l_attribute7         := p_attribute7;
  l_attribute8         := p_attribute8;
  l_attribute9         := p_attribute9;
  l_attribute10        := p_attribute10;
  l_attribute11        := p_attribute11;
  l_attribute12        := p_attribute12;
  l_attribute13        := p_attribute13;
  l_attribute14        := p_attribute14;
  l_attribute15        := p_attribute15;
  l_attribute16        := p_attribute16;
  l_attribute17        := p_attribute17;
  l_attribute18        := p_attribute18;
  l_attribute19        := p_attribute19;
  l_attribute20        := p_attribute20;

  l_abs_information1   := p_abs_information1;
  l_abs_information2   := p_abs_information2;
  l_abs_information3   := p_abs_information3;
  l_abs_information4   := p_abs_information4;
  l_abs_information5   := p_abs_information5;
  l_abs_information6   := p_abs_information6;
  l_abs_information7   := p_abs_information7;
  l_abs_information8   := p_abs_information8;
  l_abs_information9   := p_abs_information9;
  l_abs_information10  := p_abs_information10;
  l_abs_information11  := p_abs_information11;
  l_abs_information12  := p_abs_information12;
  l_abs_information13  := p_abs_information13;
  l_abs_information14  := p_abs_information14;
  l_abs_information15  := p_abs_information15;
  l_abs_information16  := p_abs_information16;
  l_abs_information17  := p_abs_information17;
  l_abs_information18  := p_abs_information18;
  l_abs_information19  := p_abs_information19;
  l_abs_information20  := p_abs_information20;
  l_abs_information21  := p_abs_information21;
  l_abs_information22  := p_abs_information22;
  l_abs_information23  := p_abs_information23;
  l_abs_information24  := p_abs_information24;
  l_abs_information25  := p_abs_information25;
  l_abs_information26  := p_abs_information26;
  l_abs_information27  := p_abs_information27;
  l_abs_information28  := p_abs_information28;
  l_abs_information29  := p_abs_information29;
  l_abs_information30  := p_abs_information30;


  OPEN csr_get_vs_id;
  FETCH csr_get_vs_id INTO l_flex_vs_id;
  CLOSE csr_get_vs_id;

  OPEN csr_get_dff_gdd;
  FETCH csr_get_dff_gdd INTO l_temp;
  CLOSE csr_get_dff_gdd;

  IF l_temp IS NULL and p_abs_information_category IS NULL THEN
    l_abs_information1  := null;
    l_abs_information2  := null;
    l_abs_information3  := null;
    l_abs_information4  := null;
    l_abs_information5  := null;
    l_abs_information6  := null;
    l_abs_information7  := null;
    l_abs_information8  := null;
    l_abs_information9  := null;
    l_abs_information10 := null;
    l_abs_information11  := null;
    l_abs_information12  := null;
    l_abs_information13  := null;
    l_abs_information14  := null;
    l_abs_information15  := null;
    l_abs_information16  := null;
    l_abs_information17  := null;
    l_abs_information18  := null;
    l_abs_information19  := null;
    l_abs_information20  := null;
    l_abs_information21  := null;
    l_abs_information22  := null;
    l_abs_information23  := null;
    l_abs_information24  := null;
    l_abs_information25  := null;
    l_abs_information26  := null;
    l_abs_information27  := null;
    l_abs_information28  := null;
    l_abs_information29  := null;
    l_abs_information30  := null;
  END IF;

  IF l_temp IS NULL AND p_attribute_category IS NULL THEN
    l_attribute1        := null;
    l_attribute2        := null;
    l_attribute3        := null;
    l_attribute4        := null;
    l_attribute5        := null;
    l_attribute6        := null;
    l_attribute7        := null;
    l_attribute8        := null;
    l_attribute9        := null;
    l_attribute10       := null;
    l_attribute11       := null;
    l_attribute12       := null;
    l_attribute13       := null;
    l_attribute14       := null;
    l_attribute15       := null;
    l_attribute16       := null;
    l_attribute17       := null;
    l_attribute18       := null;
    l_attribute19       := null;
    l_attribute20       := null;
  END IF;

  FOR csr_rec in csr_get_enabled_dff_gdd
  LOOP

  IF csr_rec.application_column_name = 'ATTRIBUTE1' THEN
      if p_attribute1 is not null THEN
        l_attribute1 := p_attribute1;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE2' THEN
      if p_attribute2 is not null THEN
        l_attribute2 := p_attribute2;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE3' THEN
      if p_attribute3 is not null THEN
        l_attribute3 := p_attribute3;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE4' THEN
      if p_attribute4 is not null THEN
        l_attribute4 := p_attribute4;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE5' THEN
      if p_attribute5 is not null THEN
        l_attribute5 := p_attribute5;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE6' THEN
      if p_attribute6 is not null THEN
        l_attribute6 := p_attribute6;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE7' THEN
      if p_attribute7 is not null THEN
        l_attribute7 := p_attribute7;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE8' THEN
      if p_attribute8 is not null THEN
        l_attribute8 := p_attribute8;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE9' THEN
      if p_attribute9 is not null THEN
        l_attribute9 := p_attribute9;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE10' THEN
      if p_attribute10 is not null THEN
        l_attribute10 := p_attribute10;
      end if;
	elsIF csr_rec.application_column_name = 'ATTRIBUTE11' THEN
	  if p_attribute11 is not null THEN
        l_attribute11 := p_attribute11;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE12' THEN
      if p_attribute12 is not null THEN
        l_attribute12 := p_attribute12;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE13' THEN
      if p_attribute13 is not null THEN
        l_attribute13 := p_attribute13;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE14' THEN
      if p_attribute14 is not null THEN
        l_attribute14 := p_attribute14;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE15' THEN
      if p_attribute15 is not null THEN
        l_attribute15 := p_attribute15;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE16' THEN
      if p_attribute16 is not null THEN
        l_attribute16 := p_attribute16;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE17' THEN
      if p_attribute17 is not null THEN
        l_attribute17 := p_attribute17;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE18' THEN
      if p_attribute18 is not null THEN
        l_attribute18 := p_attribute18;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE19' THEN
      if p_attribute19 is not null THEN
        l_attribute19 := p_attribute19;
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE20' THEN
      if p_attribute20 is not null THEN
        l_attribute20 := p_attribute20;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION1' THEN
      if p_abs_information1 is not null THEN
        l_abs_information1 := p_abs_information1;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION2' THEN
      if p_abs_information2 is not null THEN
        l_abs_information2 := p_abs_information2;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION3' THEN
      if p_abs_information3 is not null THEN
        l_abs_information3 := p_abs_information3;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION4' THEN
      if p_abs_information4 is not null THEN
        l_abs_information4 := p_abs_information4;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION5' THEN
      if p_abs_information5 is not null THEN
        l_abs_information5 := p_abs_information5;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION6' THEN
      if p_abs_information6 is not null THEN
        l_abs_information6 := p_abs_information6;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION7' THEN
      if p_abs_information7 is not null THEN
        l_abs_information7 := p_abs_information7;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION8' THEN
      if p_abs_information8 is not null THEN
        l_abs_information8 := p_abs_information8;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION9' THEN
      if p_abs_information9 is not null THEN
        l_abs_information9 := p_abs_information9;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION10' THEN
      if p_abs_information10 is not null THEN
        l_abs_information10 := p_abs_information10;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION11' THEN
      if p_abs_information11 is not null THEN
        l_abs_information11 := p_abs_information11;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION12' THEN
      if p_abs_information12 is not null THEN
        l_abs_information12 := p_abs_information12;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION13' THEN
      if p_abs_information13 is not null THEN
        l_abs_information13 := p_abs_information13;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION14' THEN
      if p_abs_information14 is not null THEN
        l_abs_information14 := p_abs_information14;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION15' THEN
      if p_abs_information15 is not null THEN
        l_abs_information15 := p_abs_information15;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION16' THEN
      if p_abs_information16 is not null THEN
        l_abs_information16 := p_abs_information16;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION17' THEN
      if p_abs_information17 is not null THEN
        l_abs_information17 := p_abs_information17;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION18' THEN
      if p_abs_information18 is not null THEN
        l_abs_information18 := p_abs_information18;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION19' THEN
      if p_abs_information19 is not null THEN
        l_abs_information19 := p_abs_information19;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION20' THEN
      if p_abs_information20 is not null THEN
        l_abs_information20 := p_abs_information20;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION21' THEN
      if p_abs_information21 is not null THEN
        l_abs_information21 := p_abs_information21;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION22' THEN
      if p_abs_information22 is not null THEN
        l_abs_information22 := p_abs_information22;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION23' THEN
      if p_abs_information23 is not null THEN
        l_abs_information23 := p_abs_information23;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION24' THEN
      if p_abs_information24 is not null THEN
        l_abs_information24 := p_abs_information24;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION25' THEN
      if p_abs_information25 is not null THEN
        l_abs_information25 := p_abs_information25;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION26' THEN
      if p_abs_information26 is not null THEN
        l_abs_information26 := p_abs_information26;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION27' THEN
      if p_abs_information27 is not null THEN
        l_abs_information27 := p_abs_information27;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION28' THEN
      if p_abs_information28 is not null THEN
        l_abs_information28 := p_abs_information28;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION29' THEN
      if p_abs_information29 is not null THEN
        l_abs_information29 := p_abs_information29;
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION30' THEN
      if p_abs_information30 is not null THEN
        l_abs_information30 := p_abs_information30;
      end if;
    end if;
 END LOOP;

  FOR csr_rec in csr_get_dff_info
  LOOP

  IF csr_rec.application_column_name = 'ATTRIBUTE1' THEN
      if instr(l_attribute1,'.') = 0 THEN
        l_attribute1 := p_attribute1||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE2' THEN
      if instr(l_attribute2,'.') = 0 THEN
        l_attribute2 := p_attribute2||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE3' THEN
      if instr(l_attribute3,'.') = 0 THEN
        l_attribute3 := p_attribute3||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE4' THEN
      if instr(l_attribute4,'.') = 0 THEN
        l_attribute4 := p_attribute4||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE5' THEN
      if instr(l_attribute5,'.') = 0 THEN
        l_attribute5 := p_attribute5||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE6' THEN
      if instr(l_attribute6,'.') = 0 THEN
        l_attribute6 := p_attribute6||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE7' THEN
      if instr(l_attribute7,'.') = 0 THEN
        l_attribute7 := p_attribute7||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE8' THEN
      if instr(l_attribute8,'.') = 0 THEN
        l_attribute8 := p_attribute8||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE9' THEN
      if instr(l_attribute9,'.') = 0 THEN
        l_attribute9 := p_attribute9||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE10' THEN
      if instr(l_attribute10,'.') = 0 THEN
        l_attribute10 := p_attribute10||'.00';
      end if;
	elsIF csr_rec.application_column_name = 'ATTRIBUTE11' THEN
	  if instr(l_attribute11,'.') = 0 THEN
        l_attribute11 := p_attribute11||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE12' THEN
      if instr(l_attribute12,'.') = 0 THEN
        l_attribute12 := p_attribute12||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE13' THEN
      if instr(l_attribute13,'.') = 0 THEN
        l_attribute13 := p_attribute13||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE14' THEN
      if instr(l_attribute14,'.') = 0 THEN
        l_attribute14 := p_attribute14||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE15' THEN
      if instr(l_attribute15,'.') = 0 THEN
        l_attribute15 := p_attribute15||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE16' THEN
      if instr(l_attribute16,'.') = 0 THEN
        l_attribute16 := p_attribute16||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE17' THEN
      if instr(l_attribute17,'.') = 0 THEN
        l_attribute17 := p_attribute17||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE18' THEN
      if instr(l_attribute18,'.') = 0 THEN
        l_attribute18 := p_attribute18||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE19' THEN
      if instr(l_attribute19,'.') = 0 THEN
        l_attribute19 := p_attribute19||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ATTRIBUTE20' THEN
      if instr(l_attribute20,'.') = 0 THEN
        l_attribute20 := p_attribute20||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION1' THEN
      if instr(l_abs_information1,'.') = 0 THEN
        l_abs_information1 := l_abs_information1||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION2' THEN
      if instr(l_abs_information2,'.') = 0 THEN
        l_abs_information2 := l_abs_information2||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION3' THEN
      if instr(l_abs_information3,'.') = 0 THEN
        l_abs_information3 := l_abs_information3||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION4' THEN
      if instr(l_abs_information4,'.') = 0 THEN
        l_abs_information4 := l_abs_information4||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION5' THEN
      if instr(l_abs_information5,'.') = 0 THEN
        l_abs_information5 := l_abs_information5||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION6' THEN
      if instr(l_abs_information6,'.') = 0 THEN
        l_abs_information6 := l_abs_information6||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION7' THEN
      if instr(l_abs_information7,'.') = 0 THEN
        l_abs_information7 := l_abs_information7||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION8' THEN
      if instr(l_abs_information8,'.') = 0 THEN
        l_abs_information8 := l_abs_information8||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION9' THEN
      if instr(l_abs_information9,'.') = 0 THEN
        l_abs_information9 := l_abs_information9||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION10' THEN
      if instr(l_abs_information10,'.') = 0 THEN
        l_abs_information10 := l_abs_information10||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION11' THEN
      if instr(l_abs_information11,'.') = 0 THEN
        l_abs_information11 := l_abs_information11||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION12' THEN
      if instr(l_abs_information12,'.') = 0 THEN
        l_abs_information12 := l_abs_information12||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION13' THEN
      if instr(l_abs_information13,'.') = 0 THEN
        l_abs_information13 := l_abs_information13||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION14' THEN
      if instr(l_abs_information14,'.') = 0 THEN
        l_abs_information14 := l_abs_information14||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION15' THEN
      if instr(l_abs_information15,'.') = 0 THEN
        l_abs_information15 := l_abs_information15||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION16' THEN
      if instr(l_abs_information16,'.') = 0 THEN
        l_abs_information16 := l_abs_information16||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION17' THEN
      if instr(l_abs_information17,'.') = 0 THEN
        l_abs_information17 := l_abs_information17||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION18' THEN
      if instr(l_abs_information18,'.') = 0 THEN
        l_abs_information18 := l_abs_information18||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION19' THEN
      if instr(l_abs_information19,'.') = 0 THEN
        l_abs_information19 := l_abs_information19||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION20' THEN
      if instr(l_abs_information20,'.') = 0 THEN
        l_abs_information20 := l_abs_information20||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION21' THEN
      if instr(l_abs_information21,'.') = 0 THEN
        l_abs_information21 := l_abs_information21||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION22' THEN
      if instr(l_abs_information22,'.') = 0 THEN
        l_abs_information22 := l_abs_information22||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION23' THEN
      if instr(l_abs_information23,'.') = 0 THEN
        l_abs_information23 := l_abs_information23||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION24' THEN
      if instr(l_abs_information24,'.') = 0 THEN
        l_abs_information24 := l_abs_information24||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION25' THEN
      if instr(l_abs_information25,'.') = 0 THEN
        l_abs_information25 := l_abs_information25||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION26' THEN
      if instr(l_abs_information26,'.') = 0 THEN
        l_abs_information26 := l_abs_information26||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION27' THEN
      if instr(l_abs_information27,'.') = 0 THEN
        l_abs_information27 := l_abs_information27||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION28' THEN
      if instr(l_abs_information28,'.') = 0 THEN
        l_abs_information28 := l_abs_information28||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION29' THEN
      if instr(l_abs_information29,'.') = 0 THEN
        l_abs_information29 := l_abs_information29||'.00';
      end if;
    elsif csr_rec.application_column_name = 'ABS_INFORMATION30' THEN
      if instr(l_abs_information30,'.') = 0 THEN
        l_abs_information30 := l_abs_information30||'.00';
      end if;
    end if;
  END LOOP;

  l_called_from := xxas_com_sec_pkg.get_function_parameter
                     (p_function_name  => xxas_emp_view_context_pkg.data_function
                     ,p_parameter_name => 'pCalledFrom');

  IF p_transaction_id is not null AND
     p_transaction_step_id is not null THEN

    IF l_abs_row.absence_attendance_id is not null THEN

      OPEN  csr_rc_trans_doc(c_person_id
                            ,l_per_row
                            ,l_asg_row
                            ,l_login_per_row
                            ,l_per_manager_name
                            ,l_login_per_manager_name
                            ,l_transaction_id
                            ,l_per_sec_group_id
                            ,l_per_leg_code
                            ,l_login_per_leg_code
                            ,l_per_job_name
                            ,l_per_position_name
                            ,l_emp_generation
                            ,l_per_bg_cur_code
                            ,l_abs_row
                            ,l_called_from
                            ,l_date_projected_start
                            ,l_time_projected_start
                            ,l_date_projected_end
                            ,l_time_projected_end
                            ,l_date_start
                            ,l_time_start
                            ,l_date_end
                            ,l_time_end
                            ,l_absence_days
                            ,l_absence_hours
                            ,l_attribute1
                            ,l_attribute2
                            ,l_attribute3
                            ,l_attribute4
                            ,l_attribute5
                            ,l_attribute6
                            ,l_attribute7
                            ,l_attribute8
                            ,l_attribute9
                            ,l_attribute10
                            ,l_attribute11
                            ,l_attribute12
                            ,l_attribute13
                            ,l_attribute14
                            ,l_attribute15
                            ,l_attribute16
                            ,l_attribute17
                            ,l_attribute18
                            ,l_attribute19
                            ,l_attribute20
                            ,p_abs_information_category
                            ,l_abs_information1
                            ,l_abs_information2
                            ,l_abs_information3
                            ,l_abs_information4
                            ,l_abs_information5
                            ,l_abs_information6
                            ,l_abs_information7
                            ,l_abs_information8
                            ,l_abs_information9
                            ,l_abs_information10
                            ,l_abs_information11
                            ,l_abs_information12
                            ,l_abs_information13
                            ,l_abs_information14
                            ,l_abs_information15
                            ,l_abs_information16
                            ,l_abs_information17
                            ,l_abs_information18
                            ,l_abs_information19
                            ,l_abs_information20
                            ,l_abs_information21
                            ,l_abs_information22
                            ,l_abs_information23
                            ,l_abs_information24
                            ,l_abs_information25
                            ,l_abs_information26
                            ,l_abs_information27
                            ,l_abs_information28
                            ,l_abs_information29
                            ,l_abs_information30);
      FETCH csr_rc_trans_doc INTO l_rc_doc;
      CLOSE csr_rc_trans_doc;

    ELSE

      OPEN  csr_rc_trans_create_doc(c_person_id
                                   ,p_absence_attendance_id
                                   ,l_per_row
                                   ,l_asg_row
                                   ,l_login_per_row
                                   ,l_per_manager_name
                                   ,l_login_per_manager_name
                                   ,l_transaction_id
                                   ,l_per_sec_group_id
                                   ,l_per_leg_code
                                   ,l_login_per_leg_code
                                   ,l_per_job_name
                                   ,l_per_position_name
                                   ,l_emp_generation
                                   ,l_per_bg_cur_code
                                   ,l_called_from
                                   ,l_date_projected_start
                                   ,l_time_projected_start
                                   ,l_date_projected_end
                                   ,l_time_projected_end
                                   ,l_date_start
                                   ,l_time_start
                                   ,l_date_end
                                   ,l_time_end
                                   ,l_absence_days
                                   ,l_absence_hours
                                   ,l_attribute1
                                   ,l_attribute2
                                   ,l_attribute3
                                   ,l_attribute4
                                   ,l_attribute5
                                   ,l_attribute6
                                   ,l_attribute7
                                   ,l_attribute8
                                   ,l_attribute9
                                   ,l_attribute10
                                   ,l_attribute11
                                   ,l_attribute12
                                   ,l_attribute13
                                   ,l_attribute14
                                   ,l_attribute15
                                   ,l_attribute16
                                   ,l_attribute17
                                   ,l_attribute18
                                   ,l_attribute19
                                   ,l_attribute20
                                   ,p_abs_information_category
                                   ,l_abs_information1
                                   ,l_abs_information2
                                   ,l_abs_information3
                                   ,l_abs_information4
                                   ,l_abs_information5
                                   ,l_abs_information6
                                   ,l_abs_information7
                                   ,l_abs_information8
                                   ,l_abs_information9
                                   ,l_abs_information10
                                   ,l_abs_information11
                                   ,l_abs_information12
                                   ,l_abs_information13
                                   ,l_abs_information14
                                   ,l_abs_information15
                                   ,l_abs_information16
                                   ,l_abs_information17
                                   ,l_abs_information18
                                   ,l_abs_information19
                                   ,l_abs_information20
                                   ,l_abs_information21
                                   ,l_abs_information22
                                   ,l_abs_information23
                                   ,l_abs_information24
                                   ,l_abs_information25
                                   ,l_abs_information26
                                   ,l_abs_information27
                                   ,l_abs_information28
                                   ,l_abs_information29
                                   ,l_abs_information30);
      FETCH csr_rc_trans_create_doc INTO l_rc_doc;
      CLOSE csr_rc_trans_create_doc;

    END IF;

  ELSE

    OPEN  csr_trans_doc(c_person_id
                       ,l_per_row
                       ,l_asg_row
                       ,l_login_per_row
                       ,l_per_manager_name
                       ,l_login_per_manager_name
                       ,l_transaction_id
                       ,l_per_sec_group_id
                       ,l_per_leg_code
                       ,l_login_per_leg_code
                       ,l_per_job_name
                       ,l_per_position_name
                       ,l_emp_generation
                       ,l_per_bg_cur_code
                       ,l_abs_row
                       ,l_called_from
                       ,l_date_projected_start
                       ,l_time_projected_start
                       ,l_date_projected_end
                       ,l_time_projected_end
                       ,l_date_start
                       ,l_time_start
                       ,l_date_end
                       ,l_time_end
                       ,l_absence_days
                       ,l_absence_hours
                       ,l_attribute1
                       ,l_attribute2
                       ,l_attribute3
                       ,l_attribute4
                       ,l_attribute5
                       ,l_attribute6
                       ,l_attribute7
                       ,l_attribute8
                       ,l_attribute9
                       ,l_attribute10
                       ,l_attribute11
                       ,l_attribute12
                       ,l_attribute13
                       ,l_attribute14
                       ,l_attribute15
                       ,l_attribute16
                       ,l_attribute17
                       ,l_attribute18
                       ,l_attribute19
                       ,l_attribute20
                       ,p_abs_information_category
                       ,l_abs_information1
                       ,l_abs_information2
                       ,l_abs_information3
                       ,l_abs_information4
                       ,l_abs_information5
                       ,l_abs_information6
                       ,l_abs_information7
                       ,l_abs_information8
                       ,l_abs_information9
                       ,l_abs_information10
                       ,l_abs_information11
                       ,l_abs_information12
                       ,l_abs_information13
                       ,l_abs_information14
                       ,l_abs_information15
                       ,l_abs_information16
                       ,l_abs_information17
                       ,l_abs_information18
                       ,l_abs_information19
                       ,l_abs_information20
                       ,l_abs_information21
                       ,l_abs_information22
                       ,l_abs_information23
                       ,l_abs_information24
                       ,l_abs_information25
                       ,l_abs_information26
                       ,l_abs_information27
                       ,l_abs_information28
                       ,l_abs_information29
                       ,l_abs_information30);
    FETCH csr_trans_doc INTO l_doc;
    CLOSE csr_trans_doc;

  END IF;

  OPEN  csr_get_abs_att_type_meaning(p_absence_type_id);
  FETCH csr_get_abs_att_type_meaning INTO l_abs_att_category;
  CLOSE csr_get_abs_att_type_meaning;

/*  l_abs_dur_hours := calculate_absence_dur_hours(p_business_group_id => l_per_row.business_group_id
                                                ,p_assignment_id     => NVL(p_override_assignment_id,p_assignment_id)
                                                ,p_effective_date    => p_effective_date
                                                ,p_absence_type_id   => p_absence_type_id
                                                ,p_date_start        => l_date_start
                                                ,p_date_end          => l_date_end
                                                ,p_time_start        => l_time_start
                                                ,p_time_end          => l_time_end);

  l_abs_dur_days := calculate_absence_dur_days(p_business_group_id => l_per_row.business_group_id
                                              ,p_assignment_id     => NVL(p_override_assignment_id,p_assignment_id)
                                              ,p_effective_date    => p_effective_date
                                              ,p_absence_type_id   => p_absence_type_id
                                              ,p_date_start        => l_date_start
                                              ,p_date_end          => l_date_end
                                              ,p_time_start        => l_time_start
                                              ,p_time_end          => l_time_end); -- commented because already calculating duration through Fast Formula*/

  IF p_transaction_id is not null AND
     p_transaction_step_id is not null THEN

    update_hrapi_transaction(p_transaction_id        => p_transaction_id
                            ,p_transaction_step_id   => p_transaction_step_id
                            ,p_absence_attendance_id => p_absence_attendance_id
                            ,p_assignment_id         => NVL(p_override_assignment_id,p_assignment_id)
                            ,p_person_id             => c_person_id
                            ,p_item_type             => g_ITEM_TYPE
                            ,p_item_key              => null--l_item_key
                            ,p_absence_trans_doc     => l_rc_doc
                            ,p_information1          => to_char(l_date_start, g_HRAPI_DATE_FMT)
                            ,p_information2          => to_char(l_date_end, g_HRAPI_DATE_FMT)
                            ,p_information3          => to_char(l_date_projected_start, g_HRAPI_DATE_FMT)
                            ,p_information4          => to_char(l_date_projected_end, g_HRAPI_DATE_FMT)
                            ,p_information5          => p_absence_type_id
                            ,p_information6          => l_abs_att_category
                            ,p_information7          => l_absence_hours
                            ,p_information8          => l_absence_days
                            ,p_information9          => p_leave_status
                            ,p_object_state          => '2');

  ELSE

    p_transaction_step_id := create_hrapi_transaction(p_transaction_id        => l_transaction_id
                                                     ,p_absence_attendance_id => p_absence_attendance_id
                                                     ,p_assignment_id         => NVL(p_override_assignment_id,p_assignment_id)
                                                     ,p_person_id             => c_person_id
                                                     ,p_item_type             => g_ITEM_TYPE
                                                     ,p_item_key              => l_item_key
                                                     ,p_absence_trans_doc     => l_doc
                                                     ,p_information1          => to_char(l_date_start, g_HRAPI_DATE_FMT)
                                                     ,p_information2          => to_char(l_date_end, g_HRAPI_DATE_FMT)
                                                     ,p_information3          => to_char(l_date_projected_start, g_HRAPI_DATE_FMT)
                                                     ,p_information4          => to_char(l_date_projected_end, g_HRAPI_DATE_FMT)
                                                     ,p_information5          => p_absence_type_id
                                                     ,p_information6          => l_abs_att_category
                                                     ,p_information7          => l_absence_hours
                                                     ,p_information8          => l_absence_days
                                                     ,p_information9          => p_leave_status
                                                     ,p_object_state          => '2');

  END IF;

  --
  -- Do create API's validation
  --
  --
  -- Issue a savepoint
  --
  savepoint before_api_validation;

  IF l_abs_row.absence_attendance_id is null THEN

    l_absence_attendance_id1 := p_absence_attendance_id;

    hr_person_absence_swi.create_person_absence
      (p_validate                     => hr_api.g_true_num
      ,p_effective_date               => p_effective_date
      ,p_person_id                    => c_person_id
      ,p_business_group_id            => l_asg_row.business_group_id
      ,p_absence_attendance_type_id   => p_absence_type_id
      ,p_abs_attendance_reason_id     => p_abs_attendance_reason_id
      ,p_comments                     => p_comments_to_approver
      ,p_date_notification            => p_date_notification
      ,p_date_projected_start         => l_date_projected_start
      ,p_time_projected_start         => l_time_projected_start
      ,p_date_projected_end           => l_date_projected_end
      ,p_time_projected_end           => l_time_projected_end
      ,p_date_start                   => l_date_start
      ,p_time_start                   => l_time_start
      ,p_date_end                     => l_date_end
      ,p_time_end                     => l_time_end
      ,p_absence_days                 => l_absence_days
      ,p_absence_hours                => l_absence_hours
      ,p_authorising_person_id        => null
      ,p_replacement_person_id        => p_replacement_person_id
      ,p_attribute_category           => p_attribute_category
      ,p_attribute1                   => l_attribute1
      ,p_attribute2                   => l_attribute2
      ,p_attribute3                   => l_attribute3
      ,p_attribute4                   => l_attribute4
      ,p_attribute5                   => l_attribute5
      ,p_attribute6                   => l_attribute6
      ,p_attribute7                   => l_attribute7
      ,p_attribute8                   => l_attribute8
      ,p_attribute9                   => l_attribute9
      ,p_attribute10                  => l_attribute10
      ,p_attribute11                  => l_attribute11
      ,p_attribute12                  => l_attribute12
      ,p_attribute13                  => l_attribute13
      ,p_attribute14                  => l_attribute14
      ,p_attribute15                  => l_attribute15
      ,p_attribute16                  => l_attribute16
      ,p_attribute17                  => l_attribute17
      ,p_attribute18                  => l_attribute18
      ,p_attribute19                  => l_attribute19
      ,p_attribute20                  => l_attribute20
      ,p_period_of_incapacity_id      => null
      ,p_ssp1_issued                  => null
      ,p_maternity_id                 => null
      ,p_sickness_start_date          => null
      ,p_sickness_end_date            => null
      ,p_pregnancy_related_illness    => null
      ,p_reason_for_notification_dela => null
      ,p_accept_late_notification_fla => null
      ,p_linked_absence_id            => null
      ,p_batch_id                     => null
      ,p_create_element_entry         => null
      ,p_abs_information_category     => p_abs_information_category
      ,p_abs_information1             => l_abs_information1
      ,p_abs_information2             => l_abs_information2
      ,p_abs_information3             => l_abs_information3
      ,p_abs_information4             => l_abs_information4
      ,p_abs_information5             => l_abs_information5
      ,p_abs_information6             => l_abs_information6
      ,p_abs_information7             => l_abs_information7
      ,p_abs_information8             => l_abs_information8
      ,p_abs_information9             => l_abs_information9
      ,p_abs_information10            => l_abs_information10
      ,p_abs_information11            => l_abs_information11
      ,p_abs_information12            => l_abs_information12
      ,p_abs_information13            => l_abs_information13
      ,p_abs_information14            => l_abs_information14
      ,p_abs_information15            => l_abs_information15
      ,p_abs_information16            => l_abs_information16
      ,p_abs_information17            => l_abs_information17
      ,p_abs_information18            => l_abs_information18
      ,p_abs_information19            => l_abs_information19
      ,p_abs_information20            => l_abs_information20
      ,p_abs_information21            => l_abs_information21
      ,p_abs_information22            => l_abs_information22
      ,p_abs_information23            => l_abs_information23
      ,p_abs_information24            => l_abs_information24
      ,p_abs_information25            => l_abs_information25
      ,p_abs_information26            => l_abs_information26
      ,p_abs_information27            => l_abs_information27
      ,p_abs_information28            => l_abs_information28
      ,p_abs_information29            => l_abs_information29
      ,p_abs_information30            => l_abs_information30
      ,p_absence_case_id              => null
      ,p_absence_attendance_id        => l_absence_attendance_id1
      ,p_object_version_number        => l_object_version_number
      ,p_occurrence                   => l_occurrence
      ,p_return_status                => l_return_status);

  ELSIF l_abs_row.absence_attendance_id is not null THEN

    xxas_util_pkg.set_date_format('DD-MON-YYYY');

	hr_person_absence_swi.update_person_absence
      (p_validate                     => hr_api.g_true_num
      ,p_effective_date               => p_effective_date
      ,p_absence_attendance_id        => p_absence_attendance_id
      ,p_abs_attendance_reason_id     => p_abs_attendance_reason_id
      ,p_comments                     => p_comments_to_approver
      ,p_date_notification            => p_date_notification
      ,p_date_projected_start         => l_date_projected_start
      ,p_time_projected_start         => l_time_projected_start
      ,p_date_projected_end           => l_date_projected_end
      ,p_time_projected_end           => l_time_projected_end
      ,p_date_start                   => l_date_start
      ,p_time_start                   => l_time_start
      ,p_date_end                     => l_date_end
      ,p_time_end                     => l_time_end
      ,p_absence_days                 => l_absence_days
      ,p_absence_hours                => l_absence_hours
      ,p_authorising_person_id        => l_abs_row.authorising_person_id
      ,p_replacement_person_id        => p_replacement_person_id
      ,p_attribute_category           => p_attribute_category
      ,p_attribute1                   => l_attribute1
      ,p_attribute2                   => l_attribute2
      ,p_attribute3                   => l_attribute3
      ,p_attribute4                   => l_attribute4
      ,p_attribute5                   => l_attribute5
      ,p_attribute6                   => l_attribute6
      ,p_attribute7                   => l_attribute7
      ,p_attribute8                   => l_attribute8
      ,p_attribute9                   => l_attribute9
      ,p_attribute10                  => l_attribute10
      ,p_attribute11                  => l_attribute11
      ,p_attribute12                  => l_attribute12
      ,p_attribute13                  => l_attribute13
      ,p_attribute14                  => l_attribute14
      ,p_attribute15                  => l_attribute15
      ,p_attribute16                  => l_attribute16
      ,p_attribute17                  => l_attribute17
      ,p_attribute18                  => l_attribute18
      ,p_attribute19                  => l_attribute19
      ,p_attribute20                  => l_attribute20
      ,p_period_of_incapacity_id      => l_abs_row.period_of_incapacity_id
      ,p_ssp1_issued                  => l_abs_row.ssp1_issued
      ,p_maternity_id                 => l_abs_row.maternity_id
      ,p_sickness_start_date          => l_abs_row.sickness_start_date
      ,p_sickness_end_date            => l_abs_row.sickness_end_date
      ,p_pregnancy_related_illness    => l_abs_row.pregnancy_related_illness
      ,p_reason_for_notification_dela => l_abs_row.reason_for_notification_delay
      ,p_accept_late_notification_fla => l_abs_row.accept_late_notification_flag
      ,p_linked_absence_id            => l_abs_row.linked_absence_id
      ,p_batch_id                     => l_abs_row.batch_id
      ,p_abs_information_category     => p_abs_information_category
      ,p_abs_information1             => l_abs_information1
      ,p_abs_information2             => l_abs_information2
      ,p_abs_information3             => l_abs_information3
      ,p_abs_information4             => l_abs_information4
      ,p_abs_information5             => l_abs_information5
      ,p_abs_information6             => l_abs_information6
      ,p_abs_information7             => l_abs_information7
      ,p_abs_information8             => l_abs_information8
      ,p_abs_information9             => l_abs_information9
      ,p_abs_information10            => l_abs_information10
      ,p_abs_information11            => l_abs_information11
      ,p_abs_information12            => l_abs_information12
      ,p_abs_information13            => l_abs_information13
      ,p_abs_information14            => l_abs_information14
      ,p_abs_information15            => l_abs_information15
      ,p_abs_information16            => l_abs_information16
      ,p_abs_information17            => l_abs_information17
      ,p_abs_information18            => l_abs_information18
      ,p_abs_information19            => l_abs_information19
      ,p_abs_information20            => l_abs_information20
      ,p_abs_information21            => l_abs_information21
      ,p_abs_information22            => l_abs_information22
      ,p_abs_information23            => l_abs_information23
      ,p_abs_information24            => l_abs_information24
      ,p_abs_information25            => l_abs_information25
      ,p_abs_information26            => l_abs_information26
      ,p_abs_information27            => l_abs_information27
      ,p_abs_information28            => l_abs_information28
      ,p_abs_information29            => l_abs_information29
      ,p_abs_information30            => l_abs_information30
      ,p_absence_case_id              => l_abs_row.absence_case_id
      ,p_object_version_number        => l_object_version_number
      ,p_return_status                => l_return_status);

	  xxas_util_pkg.set_date_format('YYYY/MM/DD HH24:MI:SS');

  END IF;

  log_statement('l_return_status: ',l_return_status);
  IF l_return_status = 'E' THEN
    IF (fnd_msg_pub.count_msg > 0) THEN
      FOR i IN 1..fnd_msg_pub.count_msg
      LOOP

        fnd_msg_pub.get
         (p_msg_index     => i,
          p_encoded       => 'F',
          p_data          => l_msg_data,
          p_msg_index_out => l_msg_count
         );

        log_statement('l_msg_data: ',l_msg_data);

        --
        -- Skip the following warning messages
        --
        IF upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_EMP_ABS_SHORT_DURATION')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_ABS_HOUR_LESS_DURATION')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_EMP_NOT_ENTITLED')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_RUNNING_ZERO')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_ABSENCE_OVERLAP')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_ABS_DAY_AFTER')) THEN

          null;

        ELSE

          register_user_error
            (p_package_name => gc_log.package_name
            ,p_proc_name    => c_method
            ,p_message      => l_msg_data);

          l_api_errors := true;

        END IF;

        l_msg_data   := null;
        l_msg_count  := 0;
      END LOOP;
    END IF;
  END IF;

  IF l_api_errors THEN
    xxas_com_api_result_pkg.raise_registered_user_errors;
  END IF;

  --
  -- Issue a rollback
  --
  rollback to before_api_validation;


  IF l_item_key is null THEN

    launch_workflow
      (p_person_id            => c_person_id
      ,p_transaction_id       => l_transaction_id
      ,p_comments_to_approver => p_comments_to_approver
      ,p_item_key             => l_item_key
      ,p_activity_id          => l_activity_id);

  END IF;

  hr_trn_upd.upd(p_transaction_id => l_transaction_id
                ,p_validate       => false
                ,p_status         => 'W'
                ,p_item_key       => l_item_key);

  xxas_emp_view_util_pkg.workflow_goto_review_page
      (p_item_type => g_ITEM_TYPE
      ,p_item_key  => l_item_key);

  p_transaction_id := l_transaction_id;

  -- populate page attributes.
  xxas_com_worklist_pkg.populate_page_attributes(p_transaction_id      => p_transaction_id
                                                ,p_transaction_step_id => p_transaction_step_id
                                                ,p_item_type           => g_ITEM_TYPE
                                                ,p_item_key            => l_item_key
                                                ,p_edit_page_id        => xxas_com_worklist_pkg.g_FLEXI_EDIT_PAGE
                                                ,p_edit_object_id      => xxas_com_sec_pkg.get_function_id(xxas_emp_view_context_pkg.securing_function)
                                                                            || '_' || NVL(p_override_assignment_id,p_assignment_id)
                                                                            || '_' || p_absence_attendance_id      -- pass null for create
                                                                            || '_' || p_transaction_id
                                                                            || '_' || p_transaction_step_id
                                                                            || '_' || '_' || '_'
                                                                            || l_object_version_number
                                                                            || '_' || null        -- this is null as we dont have page id
                                                ,p_edit_target         => 'modal'
                                                ,p_view_page_id        => xxas_com_worklist_pkg.g_FLEXI_VIEW_PAGE
                                                ,p_view_object_id      => xxas_com_sec_pkg.get_function_id(xxas_emp_view_context_pkg.securing_function)
                                                                            || '_' || NVL(p_override_assignment_id,p_assignment_id)
                                                                            || '_' || p_absence_attendance_id      -- pass null for create
                                                                            || '_' || p_transaction_id
                                                                            || '_' || p_transaction_step_id
                                                                            || '_' || '_' || '_'
                                                                            || l_object_version_number
                                                                            || '_' || null
                                                ,p_view_target         => 'modal');

  log_end(c_method, p_transaction_id);

EXCEPTION

  WHEN xxas_com_api_result_pkg.xxas_com_api_exception
  THEN
    rollback to before_api_validation;
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN hr_utility.hr_error
  THEN
    rollback to before_api_validation;
    register_user_error
      (p_package_name => gc_log.package_name
      ,p_proc_name    => c_method
      ,p_message      => hr_utility.get_message);
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

END update_emp_view_row;

--
-- ------------------------------------------------------------------------ *
-- Name    : delete_emp_view_row
-- ------------------------------------------------------------------------ *
PROCEDURE delete_emp_view_row
  (p_effective_date           IN  date
  ,p_assignment_id            IN  number
  ,p_absence_attendance_id    IN  number   DEFAULT null
  ,p_date_notification        IN  date     DEFAULT SYSDATE
  ,p_comments_to_approver     IN  varchar2 DEFAULT null
  ,p_leave_status             IN  varchar2 DEFAULT null
  ,p_object_version_number    IN  number
  ,p_transaction_id           IN OUT number
  ,p_transaction_step_id      IN OUT number)
IS

  CURSOR csr_asg IS
  SELECT paaf.*
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.assignment_id = p_assignment_id
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

  CURSOR csr_person IS
  SELECT papf.*
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.assignment_id = p_assignment_id
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

  CURSOR csr_login_person(p_login_person_id IN number) IS  --What If multiple assignments are there ??
  SELECT papf.*, paaf.supervisor_id, paaf.assignment_number
  FROM   per_all_assignments_f paaf,
         per_all_people_f papf
  WHERE  paaf.person_id = papf.person_id
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.primary_flag = 'Y'
  AND    papf.person_id = p_login_person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;

  CURSOR csr_abs_attendance_info IS
  SELECT *
  FROM   per_absence_attendances
  WHERE  absence_attendance_id = p_absence_attendance_id;

  CURSOR csr_get_emp_full_name(p_person_id number) IS
  SELECT decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'),'G', global_name, local_name) full_name
  FROM   per_all_people_f
  WHERE  person_id = p_person_id
  AND    trunc(sysdate) BETWEEN
         effective_start_date AND effective_end_date;

  CURSOR csr_get_sec_grp_leg_code(p_person_id number) IS
  SELECT bg.security_group_id
        ,bg.legislation_code
  FROM   per_all_people_f papf
        ,per_business_groups_perf bg
  WHERE  papf.person_id = p_person_id
  AND    papf.business_group_id = bg.business_group_id
  AND    trunc(sysdate) BETWEEN papf.effective_start_date and papf.effective_end_date;

  CURSOR csr_get_job_name(p_job_id number) IS
  SELECT name
  FROM   per_jobs_vl
  WHERE  job_id = p_job_id
  AND    trunc(sysdate) BETWEEN date_from AND nvl(date_to,trunc(sysdate));

  CURSOR csr_get_pos_name(p_pos_id number) IS
  SELECT name
  FROM   hr_all_positions_f_vl
  WHERE  position_id = p_pos_id
  AND    trunc(sysdate) BETWEEN nvl(effective_start_date,trunc(sysdate)) AND nvl(effective_end_date,trunc(sysdate));

  CURSOR csr_get_org_info(p_person_id number) IS
  SELECT org_information2
        ,org_information10
  FROM   hr_organization_information_v
  WHERE  org_information_context = 'Business Group Information'
  AND    organization_id         = xxas_per_pkg.get_business_group_id(p_person_id);

  CURSOR csr_get_abs_att_type_meaning(p_abs_att_type_id number) IS
  SELECT fcl.meaning
  FROM   per_abs_attendance_types_vl paat
        ,hr_lookups fcl
  WHERE  absence_attendance_type_id = p_abs_att_type_id
  AND    paat.absence_category    = fcl.lookup_code(+)
  AND    fcl.lookup_type(+)       = 'ABSENCE_CATEGORY';

  CURSOR csr_trans_doc(p_person_id               IN number
                      ,l_per_row                 IN csr_person%ROWTYPE
                      ,l_asg_row                 IN csr_asg%ROWTYPE
                      ,l_login_per_row           IN csr_login_person%ROWTYPE
                      ,p_per_manager_name        IN varchar2
                      ,p_login_per_manager_name  IN varchar2
                      ,p_transaction_id          IN number
                      ,p_per_sec_group_id        IN number
                      ,p_per_leg_code            IN varchar2
                      ,p_login_per_leg_code      IN varchar2
                      ,p_job_name                IN varchar2
                      ,p_pos_name                IN varchar2
                      ,p_emp_generation          IN varchar2
                      ,p_per_bg_cur_code         IN varchar2
                      ,l_abs_row                 IN per_absence_attendances%ROWTYPE
                      ,p_called_from             IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", p_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", l_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", p_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", g_ITEM_TYPE)
             ,XMLElement("PrsnJobId", l_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(l_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", l_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), p_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", p_login_per_leg_code)
             ,XMLElement("PrsnMgrId", l_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", p_assignment_id)
             ,XMLElement("PrsnLegCode", p_per_leg_code)
             ,XMLElement("LoginPrsnId", l_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", 'PER')
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(p_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,fnd_function.get_function_id(p_called_from)
               )
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", p_transaction_id)
             ,XMLElement("PrsnLocationId", l_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", p_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", p_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", p_per_manager_name)
             ,XMLElement("PrsnOrganizationId", l_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", l_asg_row.position_id)
             ,XMLElement("TxnStatus", 'W')
             ,XMLElement("LoginWorkerNumber", l_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", p_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", p_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", l_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(p_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_login_per_row.global_name, l_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), l_login_per_row.person_id)
             ,XMLElement("PrsnId", p_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || p_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", g_AME_TRANS_TYPE)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(p_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", p_called_from)
             ,XMLElement("LoginPrsnBgId", l_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", p_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", l_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_per_row.global_name, l_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", p_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CancelMode'))
             ,XMLElement("pAMEAppId", g_AME_APP_ID)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (2 as "PS")
                       ,XMLElement("AbsenceAttendanceId", p_absence_attendance_id)
                       ,XMLElement("DateNotification", to_char(p_date_notification, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", p_object_version_number)
                       ,decode(p_comments_to_approver, null, null, XMLElement("Comments", p_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_transaction_id))
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_rc_trans_doc(p_person_id               IN number
                         ,l_per_row                 IN csr_person%ROWTYPE
                         ,l_asg_row                 IN csr_asg%ROWTYPE
                         ,l_login_per_row           IN csr_login_person%ROWTYPE
                         ,p_per_manager_name        IN varchar2
                         ,p_login_per_manager_name  IN varchar2
                         ,p_transaction_id          IN number
                         ,p_per_sec_group_id        IN number
                         ,p_per_leg_code            IN varchar2
                         ,p_login_per_leg_code      IN varchar2
                         ,p_job_name                IN varchar2
                         ,p_pos_name                IN varchar2
                         ,p_emp_generation          IN varchar2
                         ,p_per_bg_cur_code         IN varchar2
                         ,l_abs_row                 IN per_absence_attendances%ROWTYPE
                         ,p_called_from             IN varchar2) IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("TransactionGroup", 'ABSENCE_MGMT')
             ,XMLElement("PrsnJobName", p_job_name)
             ,XMLElement("PrsnAsgFlag", 'Y')
             ,XMLElement("LoginPrsnNpwFlag", 'A')
             ,XMLElement("PrsnBgId", l_per_row.business_group_id)
             ,XMLElement("EmployeeGeneration", p_emp_generation)
             ,XMLElement("PrsnNpwFlag", 'A')
             ,XMLElement("ItemType", g_ITEM_TYPE)
             ,XMLElement("PrsnJobId", l_asg_row.job_id)
             ,XMLElement("AsgStartDate", XMLAttributes('d' as "dataType"), to_char(l_asg_row.effective_start_date, g_HRAPI_DATE_FMT))
             ,XMLElement("PrsnGradeId", l_asg_row.grade_id)
             ,XMLElement("TransactionRefId", XMLAttributes('n' as "dataType"), p_absence_attendance_id)
             ,XMLElement("LoginPrsnLegCode", p_login_per_leg_code)
             ,XMLElement("PrsnMgrId", l_asg_row.supervisor_id)
             ,XMLElement("PrsnAssignmentId", p_assignment_id)
             ,XMLElement("PrsnLegCode", p_per_leg_code)
             ,XMLElement("LoginPrsnId", l_login_per_row.person_id)
             ,XMLElement("LoginPrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("pNtfSubMsg", 'HR_ABS_NTF_SUB_MSG')
             ,XMLElement("ProductCode", 'PER')
             ,XMLElement("EffectiveDate", XMLAttributes('d' as "dataType"), to_char(p_effective_date, g_HRAPI_DATE_FMT))
             ,XMLElement("LoginPrsnEmpFlag", 'A')
             ,XMLElement("LoginPrsnType", 'E')
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,fnd_function.get_function_id(p_called_from)
               )
             ,XMLElement("TxnState", 'T')
             ,XMLElement("ReviewTemplateRNAttr", 'HR_ABS_NTF_SS')
             ,XMLElement("TransactionId", p_transaction_id)
             ,XMLElement("PrsnLocationId", l_asg_row.location_id)
             ,XMLElement("PrsnKflexStructCode", 'PEOPLE_GROUP_FLEXFIELD')
             ,XMLElement("LoginPrsnMgrName", p_login_per_manager_name)
             ,XMLElement("PrsnContextSet", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnPositionName", p_pos_name)
             ,XMLElement("PrsnEmpFlag", 'A')
             ,XMLElement("PrsnMgrName", p_per_manager_name)
             ,XMLElement("PrsnOrganizationId", l_asg_row.organization_id)
             ,XMLElement("PrsnPositionId", l_asg_row.position_id)
             ,XMLElement("TxnStatus", 'RI')
             ,XMLElement("LoginWorkerNumber", l_login_per_row.assignment_number)
             ,XMLElement("PrsnBgCurrencyCode", p_per_bg_cur_code)
             ,XMLElement("HeaderType", 'PER_HEADER')
             ,XMLElement("SSHR_WF_BASED", XMLAttributes('b' as "dataType"), 'true')
             ,XMLElement("PrsnSecurityGroupId", p_per_sec_group_id)
             ,XMLElement("LoginPrsnMgrId", l_login_per_row.supervisor_id)
             ,XMLElement("PrsnType", 'E')
             ,XMLElement("PerzOrganizationId", xxas_per_pkg.get_business_group_id(p_person_id))
             ,XMLElement("LoginPrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_login_per_row.global_name, l_login_per_row.local_name))
             ,XMLElement("CreatorPrsnId", XMLAttributes('n' as "dataType"), l_login_per_row.person_id)
             ,XMLElement("PrsnId", p_person_id)
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement("NtfAttachAttr", 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || p_transaction_id)
             ,XMLElement("pApprovalReqd", 'YD')
             ,XMLElement("pAMETranType", g_AME_TRANS_TYPE)
             ,XMLElement("RelaunchFunction", 'HR_ABS_ENTRY_PAGE_SS')
             ,XMLElement("PerzLocalizationCode", xxas_per_pkg.get_legislation_code(p_person_id))
             ,XMLElement("TransactionRefTable", 'PER_ABSENCE_ATTENDANCES')
             ,XMLElement("TransactionIdentifier", 'ABSENCES')
             ,XMLElement("pCalledFrom", p_called_from)
             ,XMLElement("LoginPrsnBgId", l_login_per_row.business_group_id)
             ,XMLElement("PerzFunctionName", p_called_from)
             ,XMLElement("ProcessName", 'HR_GENERIC_APPROVAL_PRC')
             ,XMLElement("PrsnPayrollId", l_asg_row.payroll_id)
             ,XMLElement("PrsnName", decode(fnd_profile.value('HR_LOCAL_OR_GLOBAL_NAME_FORMAT'), 'G', l_per_row.global_name, l_per_row.local_name))
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('AbsenceParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("AbsenceAttdId", p_absence_attendance_id)
               ,XMLElement("AbsenceAction", 'CancelMode'))
             ,XMLElement("pAMEAppId", g_AME_APP_ID)
             ,XMLElement("SSHR_REVIEW_FLOW_MODE", 'WF_REVIEW_PAGE')
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
               ,'HR_PERSON_ABSENCE_SWI.PROCESS_API')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1044362310593' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,30 as "pcid")
                 ,XMLElement("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerAbsenceAttendancesEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(p_absence_attendance_id))
                   ,XMLElement
                     ("PerAbsenceAttendancesEORow"
                     ,XMLAttributes
                       (2 as "PS")
                       ,XMLElement("AbsenceAttendanceId", p_absence_attendance_id)
                       ,XMLElement("DateNotification", to_char(p_date_notification, g_HRAPI_DATETIME_FMT))
                       ,XMLElement("ObjectVersionNumber", p_object_version_number)
                       ,decode(p_comments_to_approver, null, null, XMLElement("Comments", p_comments_to_approver))
                       ,XMLElement("LinkedAbsenceIdPerAbsenceAttendancesEO", null)
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                   ,XMLCDATA('000100000000')
                   ,XMLElement
                     ("TransactionsEORow"
                     ,XMLAttributes
                       (0 as "PS")
                     )
                   )
                 )
               )
             )
           )
  FROM   dual;

  CURSOR csr_workflow_info IS
  SELECT hat.item_type
        ,hat.item_key
        ,hat.transaction_ref_id
  FROM   hr_api_transactions  hat
  WHERE  hat.transaction_id = p_transaction_id;

  c_method    CONSTANT varchar2(30) := 'delete_emp_view_row';

  c_person_id CONSTANT number       := xxas_emp_view_util_pkg.get_person_id
                                         (p_assignment_id  => p_assignment_id
                                         ,p_effective_date => p_effective_date);

  l_object_version_number     number := p_object_version_number;

  l_item_type                  wf_item_activity_statuses.item_type%TYPE;
  l_item_key                   wf_item_activity_statuses.item_key%TYPE;
  l_activity_id                wf_item_activity_statuses.process_activity%TYPE;
  l_asg_row                    csr_asg%ROWTYPE;
  l_per_row                    csr_person%ROWTYPE;
  l_login_per_row              csr_login_person%ROWTYPE;
  l_abs_row                    per_absence_attendances%ROWTYPE;
  l_login_person_id            number;
  l_per_manager_name           varchar2(240);
  l_login_per_manager_name     varchar2(240);
  l_doc                        XMLType;
  l_rc_doc                     XMLType;
  l_transaction_id             number;
  l_per_sec_group_id           number;
  l_per_leg_code               varchar2(150);
  l_login_per_sec_group_id     number;
  l_login_per_leg_code         varchar2(150);
  l_per_job_name               varchar2(700);
  l_per_position_name          varchar2(240);
  l_emp_generation             varchar2(150);
  l_per_bg_cur_code            varchar2(150);
  l_abs_att_category           varchar2(240);
  l_transaction_step_id        number;
  l_msg_data                   varchar2(2000) := null;
  l_msg_count                  number := 0;
  l_api_errors                 boolean := false;
  l_return_status              varchar2(30);
  l_called_from                fnd_form_functions.function_name%TYPE;
  l_workflow_info              csr_workflow_info%ROWTYPE;

BEGIN
  log_start(c_method, p_effective_date, p_assignment_id, p_absence_attendance_id, p_object_version_number, p_transaction_id, p_transaction_step_id);

  xxas_util_pkg.user_has_access_check(g_PROD_KEY);

  g_ITEM_TYPE          := xxas_emp_view_util_pkg.get_item_type;
  g_AME_APP_ID         := xxas_emp_view_util_pkg.get_ame_application_id;
  g_AME_TRANS_TYPE     := xxas_emp_view_util_pkg.get_ame_transaction_type;
  g_PROCESS_NAME       := xxas_emp_view_util_pkg.get_process_name;

  log_statement(c_method,'g_ITEM_TYPE: ' || g_ITEM_TYPE);
  log_statement(c_method,'g_AME_APP_ID: ' || g_AME_APP_ID);
  log_statement(c_method,'g_AME_TRANS_TYPE: ' || g_AME_TRANS_TYPE);
  log_statement(c_method,'g_PROCESS_NAME: ' || g_PROCESS_NAME);

  OPEN  csr_person;
  FETCH csr_person INTO l_per_row;
  CLOSE csr_person;

  OPEN  csr_asg;
  FETCH csr_asg INTO l_asg_row;
  CLOSE csr_asg;

  OPEN  csr_abs_attendance_info;
  FETCH csr_abs_attendance_info INTO l_abs_row;
  CLOSE csr_abs_attendance_info;

  OPEN  csr_get_emp_full_name(l_asg_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_job_name(l_asg_row.job_id);
  FETCH csr_get_job_name INTO l_per_job_name;
  CLOSE csr_get_job_name;

  OPEN  csr_get_pos_name(l_asg_row.position_id);
  FETCH csr_get_pos_name INTO l_per_position_name;
  CLOSE csr_get_pos_name;

  OPEN  csr_get_sec_grp_leg_code(c_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_per_sec_group_id
                                     ,l_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  OPEN  csr_get_org_info(c_person_id);
  FETCH csr_get_org_info INTO l_emp_generation
                             ,l_per_bg_cur_code;
  CLOSE csr_get_org_info;


  IF p_transaction_id is not null AND
     p_transaction_step_id is not null THEN

    OPEN  csr_workflow_info;
    FETCH csr_workflow_info INTO l_workflow_info;
    CLOSE csr_workflow_info;

    l_transaction_id        := p_transaction_id;
    l_item_type             := l_workflow_info.item_type;
    l_item_key              := l_workflow_info.item_key;

  ELSE

    --
    -- Generate the new sequences
    --
    SELECT  hr_api_transactions_s.nextval
    INTO    l_transaction_id
    FROM   dual;

  END IF;

  l_login_person_id := xxas_util_pkg.get_per_from_user(fnd_global.user_id);

  OPEN  csr_login_person(l_login_person_id);
  FETCH csr_login_person INTO l_login_per_row;
  CLOSE csr_login_person;

  OPEN  csr_get_emp_full_name(l_login_per_row.supervisor_id);
  FETCH csr_get_emp_full_name INTO l_login_per_manager_name;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_sec_grp_leg_code(l_login_person_id);
  FETCH csr_get_sec_grp_leg_code INTO l_login_per_sec_group_id
                                     ,l_login_per_leg_code;
  CLOSE csr_get_sec_grp_leg_code;

  l_called_from := xxas_com_sec_pkg.get_function_parameter
                     (p_function_name  => xxas_emp_view_context_pkg.data_function
                     ,p_parameter_name => 'pCalledFrom');

  IF p_transaction_id is not null AND
     p_transaction_step_id is not null THEN

    OPEN  csr_rc_trans_doc(c_person_id
                          ,l_per_row
                          ,l_asg_row
                          ,l_login_per_row
                          ,l_per_manager_name
                          ,l_login_per_manager_name
                          ,l_transaction_id
                          ,l_per_sec_group_id
                          ,l_per_leg_code
                          ,l_login_per_leg_code
                          ,l_per_job_name
                          ,l_per_position_name
                          ,l_emp_generation
                          ,l_per_bg_cur_code
                          ,l_abs_row
                          ,l_called_from);
    FETCH csr_rc_trans_doc INTO l_rc_doc;
    CLOSE csr_rc_trans_doc;

  ELSE

    OPEN  csr_trans_doc(c_person_id
                       ,l_per_row
                       ,l_asg_row
                       ,l_login_per_row
                       ,l_per_manager_name
                       ,l_login_per_manager_name
                       ,l_transaction_id
                       ,l_per_sec_group_id
                       ,l_per_leg_code
                       ,l_login_per_leg_code
                       ,l_per_job_name
                       ,l_per_position_name
                       ,l_emp_generation
                       ,l_per_bg_cur_code
                       ,l_abs_row
                       ,l_called_from);
    FETCH csr_trans_doc INTO l_doc;
    CLOSE csr_trans_doc;

  END IF;

  OPEN  csr_get_abs_att_type_meaning(l_abs_row.absence_attendance_type_id);
  FETCH csr_get_abs_att_type_meaning INTO l_abs_att_category;
  CLOSE csr_get_abs_att_type_meaning;

/*  l_abs_dur_hours := calculate_absence_dur_hours(p_business_group_id => l_per_row.business_group_id
                                                ,p_assignment_id     => p_assignment_id
                                                ,p_effective_date    => trunc(sysdate)
                                                ,p_absence_type_id   => l_abs_row.absence_attendance_type_id
                                                ,p_date_start        => l_abs_row.date_start
                                                ,p_date_end          => l_abs_row.date_end
                                                ,p_time_start        => l_abs_row.time_start
                                                ,p_time_end          => l_abs_row.time_end);

  l_abs_dur_days := calculate_absence_dur_days(p_business_group_id => l_per_row.business_group_id
                                              ,p_assignment_id     => p_assignment_id
                                              ,p_effective_date    => trunc(sysdate)
                                              ,p_absence_type_id   => l_abs_row.absence_attendance_type_id
                                              ,p_date_start        => l_abs_row.date_start
                                              ,p_date_end          => l_abs_row.date_end
                                              ,p_time_start        => l_abs_row.time_start
                                              ,p_time_end          => l_abs_row.time_end); -- commented because already calculating duration from existing data*/

  IF p_transaction_id is not null AND
     p_transaction_step_id is not null THEN

    update_hrapi_transaction(p_transaction_id        => p_transaction_id
                            ,p_transaction_step_id   => p_transaction_step_id
                            ,p_absence_attendance_id => p_absence_attendance_id
                            ,p_assignment_id         => p_assignment_id
                            ,p_person_id             => c_person_id
                            ,p_item_type             => g_ITEM_TYPE
                            ,p_item_key              => null--l_item_key
                            ,p_absence_trans_doc     => l_rc_doc
                            ,p_information1          => to_char(l_abs_row.date_start, g_HRAPI_DATE_FMT)
                            ,p_information2          => to_char(l_abs_row.date_end, g_HRAPI_DATE_FMT)
                            ,p_information3          => to_char(l_abs_row.date_projected_start, g_HRAPI_DATE_FMT)
                            ,p_information4          => to_char(l_abs_row.date_projected_end, g_HRAPI_DATE_FMT)
                            ,p_information5          => l_abs_row.absence_attendance_type_id
                            ,p_information6          => l_abs_att_category
                            ,p_information7          => l_abs_row.absence_hours
                            ,p_information8          => l_abs_row.absence_days
                            ,p_information9          => p_leave_status
                            ,p_object_state          => '2');

  ELSE

    l_transaction_step_id := create_hrapi_transaction(p_transaction_id        => l_transaction_id
                                                     ,p_absence_attendance_id => p_absence_attendance_id
                                                     ,p_assignment_id         => p_assignment_id
                                                     ,p_person_id             => c_person_id
                                                     ,p_item_type             => g_ITEM_TYPE
                                                     ,p_item_key              => l_item_key
                                                     ,p_absence_trans_doc     => l_doc
                                                     ,p_information1          => to_char(l_abs_row.date_start, g_HRAPI_DATE_FMT)
                                                     ,p_information2          => to_char(l_abs_row.date_end, g_HRAPI_DATE_FMT)
                                                     ,p_information3          => to_char(l_abs_row.date_projected_start, g_HRAPI_DATE_FMT)
                                                     ,p_information4          => to_char(l_abs_row.date_projected_end, g_HRAPI_DATE_FMT)
                                                     ,p_information5          => l_abs_row.absence_attendance_type_id
                                                     ,p_information6          => l_abs_att_category
                                                     ,p_information7          => l_abs_row.absence_hours
                                                     ,p_information8          => l_abs_row.absence_days
                                                     ,p_information9          => p_leave_status
                                                     ,p_object_state          => '2');

  END IF;

  --
  -- Do create API's validation
  --
  --
  -- Issue a savepoint
  --
  savepoint before_api_validation;

  hr_person_absence_swi.delete_person_absence
    (p_validate                     => hr_api.g_true_num
    ,p_absence_attendance_id        => p_absence_attendance_id
    ,p_object_version_number        => l_object_version_number
    ,p_return_status                => l_return_status);

  IF l_return_status = 'E' THEN
    IF (fnd_msg_pub.count_msg > 0) THEN
      FOR i IN 1..fnd_msg_pub.count_msg
      LOOP

        fnd_msg_pub.get
         (p_msg_index     => i,
          p_encoded       => 'F',
          p_data          => l_msg_data,
          p_msg_index_out => l_msg_count
         );

        log_statement('l_msg_data: ',l_msg_data);

        --
        -- Skip the following warning messages
        --
        IF upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_EMP_ABS_SHORT_DURATION')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_ABS_HOUR_LESS_DURATION')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_EMP_NOT_ENTITLED')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_RUNNING_ZERO')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_ABSENCE_OVERLAP')) OR
           upper(l_msg_data) = upper(xxas_com_msg_pkg.get_string('PER|' || 'HR_LOA_DET_ABS_DAY_AFTER')) THEN

          null;

        ELSE

          register_user_error
            (p_package_name => gc_log.package_name
            ,p_proc_name    => c_method
            ,p_message      => l_msg_data);

          l_api_errors := true;

        END IF;

        l_msg_data   := null;
        l_msg_count  := 0;
      END LOOP;
    END IF;
  END IF;

  IF l_api_errors THEN
    xxas_com_api_result_pkg.raise_registered_user_errors;
  END IF;

  --
  -- Issue a rollback
  --
  rollback to before_api_validation;

  IF l_item_key is null THEN

    launch_workflow
      (p_person_id            => c_person_id
      ,p_transaction_id       => l_transaction_id
      ,p_comments_to_approver => p_comments_to_approver
      ,p_item_key             => l_item_key
      ,p_activity_id          => l_activity_id);

    p_transaction_id      := l_transaction_id;
    p_transaction_step_id := l_transaction_step_id;

  END IF;

  hr_trn_upd.upd(p_transaction_id             => l_transaction_id
                ,p_validate                   => false
                ,p_status                     => 'W'
                ,p_item_key                   => l_item_key);

  xxas_emp_view_util_pkg.workflow_goto_review_page
      (p_item_type => g_ITEM_TYPE
      ,p_item_key  => l_item_key);

  log_end(c_method, l_transaction_id);

EXCEPTION

  WHEN xxas_com_api_result_pkg.xxas_com_api_exception
  THEN
    rollback to before_api_validation;
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

  WHEN hr_utility.hr_error
  THEN
    rollback to before_api_validation;
    register_user_error
      (p_package_name => gc_log.package_name
      ,p_proc_name    => c_method
      ,p_message      => hr_utility.get_message);
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

END delete_emp_view_row;

--
-- ------------------------------------------------------------------------ *
-- Name    : cancel_wf_emp_view_row
-- ------------------------------------------------------------------------ *
PROCEDURE cancel_wf_emp_view_row
  (p_effective_date        IN date
  ,p_assignment_id         IN number
  ,p_absence_attendance_id IN number   DEFAULT null
  ,p_transaction_id        IN number   DEFAULT null
  ,p_transaction_step_id   IN number   DEFAULT null)

IS
  c_method CONSTANT varchar2(30) := 'cancel_wf_emp_view_row';

BEGIN
  log_start(c_method, p_effective_date, p_assignment_id ,p_absence_attendance_id ,p_transaction_id ,p_transaction_step_id);

  IF p_transaction_id IS NOT null AND
     p_transaction_step_id IS NOT null THEN

    hr_trn_upd.upd(p_transaction_id             => p_transaction_id
                  ,p_validate                   => false
                  ,p_status                     => 'D');

  END IF;

  log_end(c_method);

END cancel_wf_emp_view_row;

--
-- ------------------------------------------------------------------------ *
-- Name    : cancel_submit_emp_view_row
-- ------------------------------------------------------------------------ *
PROCEDURE cancel_submit_emp_view_row
  (p_effective_date        IN date
  ,p_assignment_id         IN number
  ,p_absence_attendance_id IN number   DEFAULT null
  ,p_transaction_id        IN number   DEFAULT null
  ,p_transaction_step_id   IN number   DEFAULT null)

IS
  c_method CONSTANT varchar2(30) := 'cancel_submit_emp_view_row';
  l_item_type       hr_api_transactions.item_type%TYPE;
  l_item_key        hr_api_transactions.item_key%TYPE;
  l_process_name    hr_api_transactions.process_name%TYPE;

  CURSOR csr_transaction_info
  IS
  SELECT hat.item_type
        ,hat.item_key
        ,hat.process_name
  FROM   hr_api_transactions hat
        ,hr_api_transaction_steps hats
  WHERE  hat.transaction_id       = p_transaction_id
  AND    hat.transaction_id       = hats.transaction_id
  AND    hats.transaction_step_id = p_transaction_step_id;

BEGIN
  log_start(c_method, p_effective_date, p_assignment_id ,p_absence_attendance_id ,p_transaction_id ,p_transaction_step_id);

  IF p_transaction_id IS NOT null AND
     p_transaction_step_id IS NOT null THEN

    hr_trn_upd.upd(p_transaction_id             => p_transaction_id
                  ,p_validate                   => false
                  ,p_status                     => 'D');

    OPEN csr_transaction_info;
    FETCH csr_transaction_info INTO l_item_type
                                   ,l_item_key
                                   ,l_process_name;
    CLOSE csr_transaction_info;

    --
    -- Aborting the workflow process
    --
    wf_engine.AbortProcess(itemtype => l_item_type
                          ,itemkey  => l_item_key
                          ,process  => l_process_name);

  END IF;

  log_end(c_method);

END cancel_submit_emp_view_row;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_workflow_info
-- ------------------------------------------------------------------------ *
FUNCTION get_workflow_info
  (p_transaction_id IN number) RETURN workflow_info
IS
  CURSOR c_workflow_info
  IS
  SELECT hat.item_type
  ,      hat.item_key
  FROM   hr_api_transactions hat
  WHERE  hat.transaction_id = p_transaction_id;

  l_workflow_info workflow_info;

BEGIN
  OPEN  c_workflow_info;
  FETCH c_workflow_info INTO l_workflow_info;
  CLOSE c_workflow_info;

  RETURN l_workflow_info;

END get_workflow_info;

--
-- ------------------------------------------------------------------------ *
-- Name    : submit_emp_view_row
-- ------------------------------------------------------------------------ *
PROCEDURE submit_emp_view_row
  (p_effective_date           IN  date
  ,p_assignment_id            IN  number
  ,p_absence_attendance_id    IN  number   DEFAULT null
  ,p_transaction_id           IN  number   DEFAULT null
  ,p_transaction_step_id      IN  number   DEFAULT null)
IS
  c_method CONSTANT varchar2(30) := 'submit_emp_view_row';

  --
  -- Gets attachments for given entity row
  --
  CURSOR csr_attachments IS
  SELECT a.rowid
        ,a.attached_document_id
        ,a.document_id
        ,a.seq_num
        ,a.entity_name
        ,a.column1
        ,a.pk1_value
        ,a.pk2_value
        ,a.pk3_value
        ,a.pk4_value
        ,a.pk5_value
        ,a.automatically_added_flag
        ,d.datatype_id
        ,a.category_id
        ,d.security_type
        ,d.publish_flag
        ,d.usage_type
        ,d.start_date_active
        ,d.end_date_active
        ,d.title
        ,d.description
        ,d.file_name
        ,d.media_id
  FROM   fnd_attached_documents a
        ,fnd_documents_vl d
  WHERE  a.document_id = d.document_id
  AND    a.pk1_value = p_transaction_id
  AND    a.entity_name = 'PQH_SS_ATTACHMENT';


  CURSOR get_abs_attendance_id IS
  SELECT transaction_ref_id
  FROM   hr_api_transactions
  WHERE  transaction_ref_table = 'PER_ABSENCE_ATTENDANCES'
  AND    transaction_id        = p_transaction_id;

  CURSOR c_action1(cp_item_type varchar2
                  ,cp_item_key  varchar2)
  IS
  SELECT xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode') action
  FROM   hr_api_transactions hat
  ,      hr_api_transaction_steps hats
  WHERE  hat.item_type            = cp_item_type
  AND    hat.item_key             = cp_item_key
  AND    hat.transaction_id       = hats.transaction_id
  AND    hats.api_name            = 'HR_PERSON_ABSENCE_SWI.PROCESS_API';

  CURSOR csr_trans_exists is
  SELECT 'Y'
  FROM  hr_api_transactions
  WHERE transaction_id = p_transaction_id;

  l_workflow_info           workflow_info;
  l_attachments             csr_attachments%ROWTYPE;
  l_current_activity        wf_process_activities.activity_name%TYPE;
  l_action1                 varchar2(30);
  l_abs_attendance_id       number;
  l_save_attachments        varchar2(10);
  l_save_attach_as_sup_docs varchar2(10);
  l_update_action           BOOLEAN;
  l_trans_exists            varchar2(2) := 'N';
  l_trans_error_message     varchar2(4000);
  
BEGIN
  log_start(c_method, p_effective_date, p_assignment_id ,p_absence_attendance_id ,p_transaction_id ,p_transaction_step_id);

  xxas_util_pkg.user_has_access_check(g_PROD_KEY);

  l_workflow_info := get_workflow_info
                       (p_transaction_id => p_transaction_id);
  --
  -- Only update the workflow if one exists - this may happen when cancelling
  --

  IF l_workflow_info.item_type IS NOT null AND
     l_workflow_info.item_key  IS NOT null THEN

    xxas_selfservice_workflow_pkg.set_item_attr_text
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_REVIEW_TEMPLATE_RN_ATTR'
      ,p_value     => 'HR_ABS_NTF_SS');

    xxas_selfservice_workflow_pkg.set_item_attr_text
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_NTF_ATTACHMENTS_ATTR'
      ,p_value     => 'FND:entity=PQH_SS_ATTACHMENT' || '&' || 'pk1name=TransactionId' || '&' || 'pk1value=' || p_transaction_id);

    xxas_selfservice_workflow_pkg.set_item_attr_text
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_PERZ_FUNCTION_NAME_ATTR'
      ,p_value     => xxas_xml_node_util_pkg.get_node_value(p_transaction_id, 'PerzFunctionName', 'Transaction/TransCtx'));

    xxas_selfservice_workflow_pkg.set_item_attr_text
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_PERZ_LOCALIZATION_CODE_ATTR'
      ,p_value     => xxas_xml_node_util_pkg.get_node_value(p_transaction_id, 'PerzLocalizationCode', 'Transaction/TransCtx'));

    xxas_selfservice_workflow_pkg.set_item_attr_text
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'HR_PERZ_ORGANIZATION_ID_ATTR'
      ,p_value     => xxas_xml_node_util_pkg.get_node_value(p_transaction_id, 'PerzOrganizationId', 'Transaction/TransCtx'));

    l_save_attachments := NVL(NVL(xxas_com_sec_pkg.get_function_parameter
                                    (p_function_name  => xxas_emp_view_context_pkg.data_function
                                    ,p_parameter_name => 'pSaveAttachments')
                                 ,xxas_com_sec_pkg.get_function_parameter
                                    (p_function_name  => xxas_emp_view_util_pkg.get_called_from_function
                                    ,p_parameter_name => 'pSaveAttachments'))
                             ,'Y');

    xxas_selfservice_workflow_pkg.set_item_attr_text
      (p_item_type => l_workflow_info.item_type
      ,p_item_key  => l_workflow_info.item_key
      ,p_name      => 'SAVE_ATTACHMENT'
      ,p_value     => l_save_attachments);

    l_current_activity := xxas_selfservice_workflow_pkg.get_current_activity_name
                            (p_item_type => l_workflow_info.item_type
                            ,p_item_key  => l_workflow_info.item_key);

    log_statement(c_method, 'l_current_activity = ' || l_current_activity);

    OPEN  c_action1(l_workflow_info.item_type
                   ,l_workflow_info.item_key);
    FETCH c_action1 INTO l_action1;
    CLOSE c_action1;

    log_statement(c_method, 'l_action1 = ' || l_action1);

	-- respond to notification if it is an update action activity.
    l_update_action := xxas_com_worklist_pkg.update_action(l_workflow_info.item_type,l_workflow_info.item_key);

	log_statement(c_method, 'After update action.');

    IF l_current_activity IN ('HR_INITIATOR_RFC_NTF','HR_APPROVER_RFC_NTF','HR_APPROVER_RFC_NTF') THEN

      wf_engine.CompleteActivityInternalName
        (itemtype => l_workflow_info.item_type
        ,itemkey  => l_workflow_info.item_key
        ,activity => l_current_activity
        ,result   => 'RESUBMIT');

    ELSIF l_current_activity <> 'HR_REVIEW_CHANGES_JSP_FCT' AND NOT l_update_action THEN

	  log_statement(c_method, 'completing current activity');

      xxas_selfservice_workflow_pkg.complete_current_activity
        (p_item_type => l_workflow_info.item_type
        ,p_item_key  => l_workflow_info.item_key
        ,p_result    => CASE WHEN l_action1 = c_insert
                        THEN 'NEXT'
                        WHEN l_action1 = c_update
                        THEN 'UPDATE_ABSENCE'
                        ELSE null
                   END);

    END IF;

	log_statement(c_method, 'p_transaction_id: ' || p_transaction_id);

    OPEN  csr_trans_exists;
    FETCH csr_trans_exists into l_trans_exists;
    CLOSE csr_trans_exists;

    IF l_trans_exists = 'Y' THEN

      xxas_emp_view_ui_pkg.register_result_details
        (p_transaction_id => p_transaction_id);
       
      select xxas_selfservice_workflow_pkg.get_error_text(p_transaction_id) into l_trans_error_message from dual;
    
      IF l_trans_error_message IS NOT NULL THEN
      xxas_com_api_result_pkg.raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_field_name   => null
      ,p_message      => xxas_com_msg_type
                           (xxas_util_pkg.app
                           ,'XXAS_COM_TEXT'
                           ,xxas_com_msg_token_varray
                              (xxas_com_msg_token_type
                                 ('TEXT',l_trans_error_message)
                              )
                           ));               
      END IF;
            
   	  IF approval_required THEN
        IF l_update_action THEN
          -- update transaction table
          update hr_api_transactions set status = 'Y' where transaction_id = p_transaction_id;
        ELSE
          hr_trn_upd.upd(p_transaction_id             => p_transaction_id
                        ,p_validate                   => false
                        ,p_status                     => 'Y');
        END IF;
	    log_statement(c_method, 'Debug approval required');
	  END IF;

    END IF;

    l_save_attach_as_sup_docs := NVL(NVL(xxas_com_sec_pkg.get_function_parameter
                                           (p_function_name  => xxas_emp_view_context_pkg.data_function
                                           ,p_parameter_name => 'pSaveAttachmentsAsSupportingDocs')
                                        ,xxas_com_sec_pkg.get_function_parameter
                                           (p_function_name  => xxas_emp_view_util_pkg.get_called_from_function
                                           ,p_parameter_name => 'pSaveAttachmentsAsSupportingDocs'))
                                    ,'N');

    log_statement(c_method, 'l_save_attach_as_sup_docs: ' || l_save_attach_as_sup_docs);

    IF l_save_attachments = 'Y' AND l_save_attach_as_sup_docs = 'Y' THEN

      OPEN  get_abs_attendance_id;
      FETCH get_abs_attendance_id INTO l_abs_attendance_id;
      CLOSE get_abs_attendance_id;

      OPEN csr_attachments;
      LOOP
        FETCH csr_attachments INTO l_attachments;
        EXIT WHEN csr_attachments%NOTFOUND;

        fnd_attached_documents_pkg.update_row
          (x_rowid                        => l_attachments.rowid
          ,x_attached_document_id         => l_attachments.attached_document_id
          ,x_document_id                  => l_attachments.document_id
          ,x_last_update_date             => sysdate
          ,x_last_updated_by              => fnd_global.user_id
          ,x_last_update_login            => fnd_global.login_id
          ,x_seq_num                      => l_attachments.seq_num
          ,x_entity_name                  => 'PER_ABSENCE_ATTENDANCES'
          ,x_column1                      => l_attachments.column1
          ,x_pk1_value                    => l_abs_attendance_id || '_' || p_transaction_id
          ,x_pk2_value                    => l_attachments.pk2_value
          ,x_pk3_value                    => l_attachments.pk3_value
          ,x_pk4_value                    => l_attachments.pk4_value
          ,x_pk5_value                    => l_attachments.pk5_value
          ,x_automatically_added_flag     => l_attachments.automatically_added_flag
          ,x_datatype_id                  => l_attachments.datatype_id
          ,x_category_id                  => l_attachments.category_id
          ,x_security_type                => l_attachments.security_type
          ,x_publish_flag                 => l_attachments.publish_flag
          ,x_usage_type                   => l_attachments.usage_type
          ,x_start_date_active            => l_attachments.start_date_active
          ,x_end_date_active              => l_attachments.end_date_active
          ,x_language                     => userenv('LANG')
          ,x_title                        => l_attachments.title
          ,x_description                  => l_attachments.description
          ,x_file_name                    => l_attachments.file_name
          ,x_media_id                     => l_attachments.media_id);

        l_attachments := null;

      END LOOP;
      CLOSE csr_attachments;
    END IF;

  END IF;

  log_end(c_method);

END submit_emp_view_row;

--
-- ------------------------------------------------------------------------ *
-- Name    : workflow_review_page
-- ------------------------------------------------------------------------ *
PROCEDURE workflow_review_page
  (p_item_type    IN varchar2
  ,p_item_key     IN varchar2
  ,p_next_result  IN varchar2 DEFAULT 'NEXT')

IS
  c_method CONSTANT varchar2(30) := 'workflow_review_page';

  l_current_activity  wf_process_activities.activity_name%TYPE;
  l_func_name             VARCHAR2(4000);
  l_assignment_id         NUMBER;
  l_transaction_id        NUMBER;
  l_absence_attendance_id NUMBER;
  l_transaction_step_id   NUMBER;

  CURSOR csr_trans_det(cp_transaction_id NUMBER)
  IS
  SELECT hats.transaction_step_id
        ,hat.transaction_ref_id
  FROM   hr_api_transactions hat
        ,hr_api_transaction_steps hats
  WHERE  hat.transaction_id = hats.transaction_id
  AND    hat.transaction_id = cp_transaction_id
  AND    hats.api_name = 'HR_PERSON_ABSENCE_SWI.PROCESS_API';

BEGIN

  log_start(c_method, p_item_type, p_item_key, p_next_result);

  --
  -- Retrieve the function from the existing attribute
  --
  l_func_name := get_wf_attribute_text
                   (p_item_type, p_item_key, 'P_CALLED_FROM');

  --
  -- If the function is specified, set the following attributes if they're
  -- currently null.
  --
  IF l_func_name IS NOT null THEN
    IF get_wf_attribute_text
        (p_item_type, p_item_key, 'HR_GENERIC_URL_REDIRECT') IS null
    THEN
      wf_engine.setitemattrtext
        (itemtype  => p_item_type
        ,itemkey   => p_item_key
        ,aname     => 'HR_GENERIC_URL_REDIRECT'
        ,avalue    => 'OA.jsp?akRegionCode=HR_REVIEW_TOP_SS'||'&'||'akRegionApplicationId=800'||'&'||'OAFunc=' || l_func_name);
    END IF;

    IF get_wf_attribute_text
        (p_item_type, p_item_key, 'APPROVAL_GENERIC_URL') IS null
    THEN
      wf_engine.setitemattrtext
        (itemtype  => p_item_type
        ,itemkey   => p_item_key
        ,aname     => 'APPROVAL_GENERIC_URL'
        ,avalue    => 'JSP:/' || fnd_profile.VALUE('ICX_OA_HTML')
                              || '/OA.jsp?akRegionCode=HR_REVIEW_TOP_SS'||'&'||'akRegionApplicationId=800'||'&'||'NtfId=-'||'&'||'#NID-'||'&'||'retainAM=Y'||'&'||'OAFunc='
                              || l_func_name);
    END IF;
  END IF;

  l_current_activity := xxas_selfservice_workflow_pkg.get_current_activity_name
                          (p_item_type => p_item_type
                          ,p_item_key  => p_item_key);

  log_end(c_method, l_current_activity);

END workflow_review_page;


--
-- ------------------------------------------------------------------------ *
-- Name    : workflow_goto_review_page
-- ------------------------------------------------------------------------ *
PROCEDURE workflow_goto_review_page
  (p_item_type    IN varchar2
  ,p_item_key     IN varchar2
  ,p_next_result  IN varchar2)
  
IS
  c_method CONSTANT varchar2(30) := 'workflow_goto_review_page';

  CURSOR c_action 
  IS
  SELECT hatv.varchar2_value action
  FROM   hr_api_transactions hat
  ,      hr_api_transaction_steps hats
  ,      hr_api_transaction_values hatv
  WHERE  hat.item_type            = p_item_type
  AND    hat.item_key             = p_item_key
  AND    hat.transaction_id       = hats.transaction_id
  AND    hats.transaction_step_id = hatv.transaction_step_id
  AND    hatv.name                = 'P_ACTIVITY_NAME';

  l_action                 varchar2(30);
  l_current_activity_name  wf_process_activities.activity_name%TYPE;
   
BEGIN
  log_start(c_method, p_item_type, p_item_key);
  
  l_current_activity_name := xxas_selfservice_workflow_pkg.get_current_activity_name
                              (p_item_type => p_item_type
                              ,p_item_key  => p_item_key);
    
  log_statement(c_method, 'l_current_activity = ' || l_current_activity_name);
  
  IF l_current_activity_name = 'HR_LOA_SUMMARY_FCT'
  THEN
    OPEN  c_action;
    FETCH c_action INTO l_action;
    CLOSE c_action;

    log_statement(c_method, 'l_action = ' || l_action);
  
    wf_engine.CompleteActivityInternalName
      (itemtype => p_item_type
      ,itemkey  => p_item_key
      ,activity => l_current_activity_name
      ,result   => CASE WHEN l_action = c_insert
                        THEN 'NEXT'
                        WHEN l_action = c_update
                        THEN 'UPDATE_ABSENCE'
                   END);
  END IF;

  workflow_review_page
    (p_item_type   => p_item_type
    ,p_item_key    => p_item_key
    ,p_next_result => p_next_result);
  
  log_end(c_method);
 
END workflow_goto_review_page;

-- ------------------------------------------------------------------------ *
-- Name    : get_absence_type_value
-- ------------------------------------------------------------------------ *
FUNCTION get_absence_type_value
  (p_absence_type_id IN number
  ,p_column_name     IN varchar2) RETURN varchar2
IS
  c_method CONSTANT    varchar2(30)    := 'get_absence_type_value';

  l_value              varchar2(32000) := xxas_fast_formula_pkg.g_NULL_STRING;
  e_invalid_identifier EXCEPTION;
  PRAGMA EXCEPTION_INIT(e_invalid_identifier, -00904);
  e_too_many_rows      EXCEPTION;
  PRAGMA EXCEPTION_INIT(e_too_many_rows, -01422);

BEGIN
  log_start(c_method, p_absence_type_id, p_column_name);

  IF p_absence_type_id  IS NOT null AND
     p_column_name      IS NOT null THEN

    log_statement(c_method, 5);

    EXECUTE IMMEDIATE
         ' SELECT paat.' || p_column_name
      || ' FROM   per_abs_attendance_types_v paat'
      || ' WHERE  paat.absence_attendance_type_id = :p_absence_attendance_type_id'
    INTO  l_value
    USING p_absence_type_id;

  END IF;

  log_end(c_method, nvl(l_value, xxas_fast_formula_pkg.g_NULL_STRING));

  RETURN nvl(l_value, xxas_fast_formula_pkg.g_NULL_STRING);

EXCEPTION

  WHEN e_invalid_identifier THEN
    log_end(c_method, 900);
    RETURN xxas_fast_formula_pkg.g_ERROR_STRING;

  WHEN e_too_many_rows THEN
    log_end(c_method,905);
    RETURN xxas_fast_formula_pkg.g_ERROR_STRING;

END get_absence_type_value;

-- ------------------------------------------------------------------------ *
-- Name    : get_absence_reason_id
-- ------------------------------------------------------------------------ *
FUNCTION get_absence_reason_id
  (p_business_group_id IN number
  ,p_absence_type_id   IN number
  ,p_reason_code       IN varchar2) RETURN number
IS
  c_method CONSTANT    varchar2(30) := 'get_absence_reason_id';

  CURSOR csr_absence_reason IS
  SELECT abr.abs_attendance_reason_id
  FROM   per_abs_attendance_reasons abr
  WHERE  abr.business_group_id = p_business_group_id
  AND    abr.absence_attendance_type_id = p_absence_type_id
  AND    abr.name = p_reason_code;

  l_return number;

BEGIN
  log_start(c_method, p_business_group_id, p_absence_type_id, p_reason_code);

  IF p_business_group_id IS NOT null AND
     p_absence_type_id   IS NOT null AND
     p_reason_code       IS NOT null THEN

    log_statement(c_method, 5);

    OPEN  csr_absence_reason;
    FETCH csr_absence_reason INTO l_return;
    CLOSE csr_absence_reason;

  END IF;

  log_end(c_method, nvl(l_return, -1));
  RETURN nvl(l_return, -1);

END get_absence_reason_id;

-- ------------------------------------------------------------------------ *
-- Name    : calculate_absence_duration
-- ------------------------------------------------------------------------ *
PROCEDURE calculate_absence_duration
  (p_business_group_id IN            number
  ,p_assignment_id     IN            number
  ,p_effective_date    IN            date
  ,p_absence_type_id   IN            number
  ,p_date_start        IN            date
  ,p_date_end          IN            date
  ,p_time_start        IN            varchar2 DEFAULT null
  ,p_time_end          IN            varchar2 DEFAULT null
  ,p_absence_days         OUT NOCOPY number
  ,p_absence_hours        OUT NOCOPY number
  ,p_use_formula          OUT NOCOPY number
  ,p_min_max_failure      OUT NOCOPY varchar2
  ,p_warning_or_error     OUT NOCOPY varchar2
  ,p_page_error_msg       OUT NOCOPY varchar2)
IS
  c_method  CONSTANT varchar2(30) := 'calculate_absence_duration';

  l_absence_days     number;
  l_absence_hours    number;
  l_use_formula      number;
  l_min_max_failure  varchar2(4000);
  l_warning_or_error varchar2(4000);
  l_page_error_msg   varchar2(4000);

BEGIN

  log_start
    (c_method, p_business_group_id, p_assignment_id, p_effective_date
    ,p_absence_type_id, p_date_start, p_date_end, p_time_start, p_time_end);

  IF p_business_group_id IS NOT null AND
     p_assignment_id     IS NOT null AND
     p_effective_date    IS NOT null AND
     p_absence_type_id   IS NOT null AND
     p_date_start        IS NOT null AND
     p_date_end          IS NOT null THEN

    log_statement(c_method, 5);

    hr_loa_ss.calculate_absence_duration
      (p_absence_attendance_type_id => p_absence_type_id
      ,p_business_group_id          => p_business_group_id
      ,p_effective_date             => p_effective_date
      ,p_person_id                  => xxas_com_asg_pkg.get_person_from_asg
                                        (p_assignment_id)
      ,p_date_start                 => p_date_start
      ,p_date_end                   => p_date_end
      ,p_time_start                 => p_time_start
      ,p_time_end                   => p_time_end
      ,p_absence_days               => l_absence_days
      ,p_absence_hours              => l_absence_hours
      ,p_use_formula                => l_use_formula
      ,p_min_max_failure            => l_min_max_failure
      ,p_warning_or_error           => l_warning_or_error
      ,p_page_error_msg	            => l_page_error_msg);


    log_statement(c_method, 'l_absence_days:  '    || to_char(l_absence_days));
    log_statement(c_method, 'l_absence_hours: '    || to_char(l_absence_hours));
    log_statement(c_method, 'l_use_formula: '      || to_char(l_use_formula));
    log_statement(c_method, 'l_min_max_failure: '  || l_min_max_failure);
    log_statement(c_method, 'l_warning_or_error: ' || l_warning_or_error);
    log_statement(c_method, 'l_page_error_msg: '   || l_page_error_msg);

  END IF;

  p_absence_days     := l_absence_days;
  p_absence_hours    := l_absence_hours;
  p_use_formula      := l_use_formula;
  p_min_max_failure  := l_min_max_failure;
  p_warning_or_error := l_warning_or_error;
  p_page_error_msg   := l_page_error_msg;

END calculate_absence_duration;

-- ------------------------------------------------------------------------ *
-- Name    : calculate_abs_duration_full
-- ------------------------------------------------------------------------ *
FUNCTION calculate_abs_duration_full
  (p_business_group_id IN            number
  ,p_assignment_id     IN            number
  ,p_effective_date    IN            date
  ,p_absence_type_id   IN            number
  ,p_date_start        IN            date
  ,p_date_end          IN            date
  ,p_time_start        IN            varchar2 DEFAULT null
  ,p_time_end          IN            varchar2 DEFAULT null
  ,p_absence_days         OUT NOCOPY number
  ,p_absence_hours        OUT NOCOPY number
  ,p_use_formula          OUT NOCOPY number
  ,p_min_max_failure      OUT NOCOPY varchar2
  ,p_warning_or_error     OUT NOCOPY varchar2
  ,p_page_error_msg       OUT NOCOPY varchar2) RETURN number
IS
  l_absence_days     number;
  l_absence_hours    number;

BEGIN

  calculate_absence_duration
    (p_business_group_id => p_business_group_id
    ,p_assignment_id     => p_assignment_id
    ,p_effective_date    => p_effective_date
    ,p_absence_type_id   => p_absence_type_id
    ,p_date_start        => p_date_start
    ,p_date_end          => p_date_end
    ,p_time_start        => p_time_start
    ,p_time_end          => p_time_end
    ,p_absence_days      => l_absence_days
    ,p_absence_hours     => l_absence_hours
    ,p_use_formula       => p_use_formula
    ,p_min_max_failure   => p_min_max_failure
    ,p_warning_or_error  => p_warning_or_error
    ,p_page_error_msg	   => p_page_error_msg);

  RETURN nvl(l_absence_hours, nvl(l_absence_days, 0));

END calculate_abs_duration_full;

-- ------------------------------------------------------------------------ *
-- Name    : calculate_absence_duration
-- ------------------------------------------------------------------------ *
FUNCTION calculate_absence_duration
  (p_business_group_id IN number
  ,p_assignment_id     IN number
  ,p_effective_date    IN date
  ,p_absence_type_id   IN number
  ,p_date_start        IN date
  ,p_date_end          IN date
  ,p_time_start        IN varchar2 DEFAULT null
  ,p_time_end          IN varchar2 DEFAULT null) RETURN number
IS
  l_absence_days     number;
  l_absence_hours    number;
  l_use_formula      number;
  l_min_max_failure  varchar2(4000);
  l_warning_or_error varchar2(4000);
  l_page_error_msg   varchar2(4000);

BEGIN

  calculate_absence_duration
    (p_business_group_id => p_business_group_id
    ,p_assignment_id     => p_assignment_id
    ,p_effective_date    => p_effective_date
    ,p_absence_type_id   => p_absence_type_id
    ,p_date_start        => p_date_start
    ,p_date_end          => p_date_end
    ,p_time_start        => p_time_start
    ,p_time_end          => p_time_end
    ,p_absence_days      => l_absence_days
    ,p_absence_hours     => l_absence_hours
    ,p_use_formula       => l_use_formula
    ,p_min_max_failure   => l_min_max_failure
    ,p_warning_or_error  => l_warning_or_error
    ,p_page_error_msg	   => l_page_error_msg);

  RETURN nvl(l_absence_hours, nvl(l_absence_days, 0));

END calculate_absence_duration;

-- ------------------------------------------------------------------------ *
-- Name    : calculate_absence_dur_days
-- ------------------------------------------------------------------------ *
FUNCTION calculate_absence_dur_days
  (p_business_group_id IN number
  ,p_assignment_id     IN number
  ,p_effective_date    IN date
  ,p_absence_type_id   IN number
  ,p_date_start        IN date
  ,p_date_end          IN date
  ,p_time_start        IN varchar2 DEFAULT null
  ,p_time_end          IN varchar2 DEFAULT null) RETURN number
IS

  l_absence_days     number;
  l_absence_hours    number;
  l_use_formula      number;
  l_min_max_failure  varchar2(4000);
  l_warning_or_error varchar2(4000);
  l_page_error_msg   varchar2(4000);

BEGIN

  calculate_absence_duration
    (p_business_group_id => p_business_group_id
    ,p_assignment_id     => p_assignment_id
    ,p_effective_date    => p_effective_date
    ,p_absence_type_id   => p_absence_type_id
    ,p_date_start        => p_date_start
    ,p_date_end          => p_date_end
    ,p_time_start        => p_time_start
    ,p_time_end          => p_time_end
    ,p_absence_days      => l_absence_days
    ,p_absence_hours     => l_absence_hours
    ,p_use_formula       => l_use_formula
    ,p_min_max_failure   => l_min_max_failure
    ,p_warning_or_error  => l_warning_or_error
    ,p_page_error_msg	   => l_page_error_msg);

  RETURN nvl(l_absence_days, 0);

END calculate_absence_dur_days;

-- ------------------------------------------------------------------------ *
-- Name    : calculate_absence_dur_hours
-- ------------------------------------------------------------------------ *
FUNCTION calculate_absence_dur_hours
  (p_business_group_id IN number
  ,p_assignment_id     IN number
  ,p_effective_date    IN date
  ,p_absence_type_id   IN number
  ,p_date_start        IN date
  ,p_date_end          IN date
  ,p_time_start        IN varchar2 DEFAULT null
  ,p_time_end          IN varchar2 DEFAULT null) RETURN number
IS

  l_absence_days     number;
  l_absence_hours    number;
  l_use_formula      number;
  l_min_max_failure  varchar2(4000);
  l_warning_or_error varchar2(4000);
  l_page_error_msg   varchar2(4000);

BEGIN

  calculate_absence_duration
    (p_business_group_id => p_business_group_id
    ,p_assignment_id     => p_assignment_id
    ,p_effective_date    => p_effective_date
    ,p_absence_type_id   => p_absence_type_id
    ,p_date_start        => p_date_start
    ,p_date_end          => p_date_end
    ,p_time_start        => p_time_start
    ,p_time_end          => p_time_end
    ,p_absence_days      => l_absence_days
    ,p_absence_hours     => l_absence_hours
    ,p_use_formula       => l_use_formula
    ,p_min_max_failure   => l_min_max_failure
    ,p_warning_or_error  => l_warning_or_error
    ,p_page_error_msg	   => l_page_error_msg);

  RETURN nvl(l_absence_hours, 0);

END calculate_absence_dur_hours;

-- ------------------------------------------------------------------------ *
-- Name    : off_sick_ad
-- ------------------------------------------------------------------------ *
FUNCTION off_sick_ad
  (p_function_id         IN number
  ,p_function_name       IN varchar2
  ,p_function_parameters IN varchar2           DEFAULT null
  ,p_object_id           IN number             DEFAULT null
  ,p_obj_name            IN varchar2           DEFAULT null
  ,p_keys                IN xxas_com_keys_type DEFAULT null) RETURN XMLType
IS

  l_PROC      CONSTANT varchar2(30) := 'off_sick_ad';
  l_person_id          number;
  l_normal_hours       per_all_assignments_f.normal_hours%TYPE;
  l_time_normal_start  per_all_assignments_f.time_normal_start%TYPE;
  l_time_normal_finish per_all_assignments_f.time_normal_finish%TYPE;
  l_frequency          per_all_assignments_f.frequency%TYPE;
  l_xml                XMLType;

  --
  -- Gets a dynamic ad
  --
  CURSOR csr_xml
  IS
  SELECT XMLElement
            ("item"
            ,XMLForest
              ('advert' as "type"
              ,'Absent since '
              || to_char(nvl(abs.date_start, abs.date_projected_start)
                        ,'FMDd Mon YYYY')
              || '. '
              || 'If the absence has ended, please close it now.' "line2"
              ,'Close now' "buttonText"
              ,'#/absence-edit/'
                || l_person_id
                || '_'
                || abs.absence_attendance_id
                || '_'
                || to_char(nvl(abs.date_start, abs.date_projected_start)
                          ,'YYYY-MM-DD')
                || ' '
                || nvl(abs.time_start, nvl(abs.time_projected_start
                                          ,l_time_normal_start))
                || ':00'
                || '_'
                || to_char(trunc(sysdate) - 1, 'YYYY-MM-DD')
                || ' '
                || l_time_normal_finish || ':00' "link"
              ,'100' "textWidth"
              ,'0' "textPosX"
              ,'0' "textMargBottom"
              ,'top' "textYAlign"
              ,'left' "textAlign"
              ,'true' "textPad"
              ,'false' "margNo"
              ,'rgba(255,255,255,0)' "textBgColor"
              ,'rgba(255,255,255,1.0)' "textColor"
              ,xxas_com_sec_pkg.get_media_url || 'off-sick-ad.jpg' as "image")
			      )
            AS "data"
  FROM  per_absence_attendances abs
       ,per_abs_attendance_types_vl abt
  WHERE abs.person_id = l_person_id
  AND   abs.absence_attendance_type_id = abt.absence_attendance_type_id
  AND  (upper(abt.name) LIKE '%SICK%'
     OR abt.absence_category = 'S')
  AND   abs.date_end IS null
  ORDER BY abs.date_start;

BEGIN

  l_person_id := to_number(p_keys.get_key('PERSON_ID'));

  IF l_person_id IS NOT null THEN

    xxas_com_asg_pkg.get_normal_conditions
      (p_assignment_id      => xxas_com_asg_pkg.get_primary_work_asg
                                 (p_person_id      => l_person_id
                                 ,p_effective_date => trunc(sysdate))
      ,p_effective_date     => trunc(sysdate)
      ,p_normal_hours       => l_normal_hours
      ,p_time_normal_start  => l_time_normal_start
      ,p_time_normal_finish => l_time_normal_finish
      ,p_frequency          => l_frequency);

    OPEN  csr_xml;
    FETCH csr_xml INTO l_xml;
    CLOSE csr_xml;

  END IF;

  RETURN l_xml;

END off_sick_ad;

-- ------------------------------------------------------------------------ *
-- Name    : accrual_plan_repeater
-- ------------------------------------------------------------------------ *
FUNCTION accrual_plan_repeater
  (p_function_id         IN number
  ,p_function_name       IN varchar2
  ,p_function_parameters IN varchar2           DEFAULT null
  ,p_object_id           IN number             DEFAULT null
  ,p_obj_name            IN varchar2           DEFAULT null
  ,p_keys                IN xxas_com_keys_type DEFAULT null) RETURN xxas_block_table
IS

  l_person_id number;
  l_func_id   number;
  l_ele_set   pay_element_sets.element_set_name%TYPE;
  l_func_name fnd_form_functions.function_name%TYPE;
  l_block_tbl xxas_block_table;

  --
  -- Gets the person's active worker assignments
  --
  CURSOR csr_plans
    (cp_person_id        IN number
    ,cp_element_set_name IN varchar2)
  IS
  SELECT paaf.person_id
        ,paaf.assignment_id
        ,pap.accrual_plan_id
        ,pap.accrual_plan_name
        ,pap.accrual_units_of_measure uom
  FROM   pay_element_entries_f pee
        ,pay_accrual_plans_v pap
        ,per_all_assignments_f paaf
  WHERE  paaf.person_id = cp_person_id
  AND    paaf.assignment_id = pee.assignment_id
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.assignment_type = 'E'
  AND    paaf.primary_flag = 'Y'
  AND    trunc(sysdate) BETWEEN
         pee.effective_start_date AND pee.effective_end_date
  AND    pee.element_type_id = pap.accrual_plan_element_type_id
  AND   (cp_element_set_name IS null
     OR (cp_element_set_name IS NOT null AND
         pap.accrual_plan_element_type_id IN
           (SELECT petr.element_type_id
            FROM   pay_element_type_rules petr
                  ,pay_element_sets pes
            WHERE  upper(pes.element_set_name) = upper(cp_element_set_name)
            AND    pes.element_set_id = petr.element_set_id
            AND    petr.include_or_exclude = 'I')))
  ORDER BY pap.accrual_plan_name
          ,nvl(paaf.primary_flag, 'N') DESC;

BEGIN

  l_person_id := p_keys.get_key('PERSON_ID');
  l_func_name := xxas_com_sec_pkg.get_parameter_value
                   (p_function_parameters, 'pBlockToRepeat');
  IF l_func_name IS null THEN
    l_func_name := xxas_com_sec_pkg.get_parameter_value
                     (p_function_parameters, 'pFunctionToRepeat');
  END IF;
  l_func_id   := xxas_com_sec_pkg.get_function_id(l_func_name);

  IF l_person_id IS NOT null AND l_func_id IS NOT null THEN

    l_block_tbl := xxas_block_table();

    l_ele_set := xxas_com_sec_pkg.get_parameter_value
                   (p_function_parameters, 'pElementSetName');

    FOR csr_rec IN csr_plans(l_person_id, l_ele_set) LOOP
      l_block_tbl.extend(1);
      l_block_tbl(l_block_tbl.count) :=
        xxas_block_type
          (sequence    => l_block_tbl.count
          ,function_id => l_func_id
          ,block_title => csr_rec.accrual_plan_name
          ,description => null
          ,active_block_flag => null
          ,keys        => xxas_com_keys_type
                           (xxas_com_named_value_table
                             (xxas_com_named_value_type
                               ('PERSON_ID', csr_rec.person_id)
                             ,xxas_com_named_value_type
                               ('ASSIGNMENT_ID', csr_rec.assignment_id)
                             ,xxas_com_named_value_type
                               ('ACCRUAL_PLAN_ID', csr_rec.accrual_plan_id))));
    END LOOP;

  END IF;

  RETURN l_block_tbl;

END accrual_plan_repeater;

--
-- ------------------------------------------------------------------------ *
-- Name    : r12_pending_absences
-- ------------------------------------------------------------------------ *
FUNCTION r12_pending_absences
  (p_person_id       IN number
  ,p_accrual_plan_id IN number
  ,p_start_date      IN date
  ,p_end_date        IN date) RETURN number
IS

  l_pending number;

  CURSOR csr_get_pending
  IS
   SELECT sum(nvl(nvl(abs.absence_days_p, abs.absence_hours_p), 0) -
          nvl(nvl(abs.absence_days, abs.absence_hours), 0)) pending_amount
   FROM   xxas_absence_management_v abs
         ,per_absence_attendance_types abt
         ,pay_net_calculation_rules    ncr
   WHERE  abs.person_id = p_person_id
   AND    nvl(abs.absence_type_id_p, abs.absence_type_id) = abt.absence_attendance_type_id
   AND    (abs.absence_days_p IS NOT null OR abs.absence_hours_p IS NOT null)
   AND    abt.input_value_id = ncr.input_value_id
   AND    ((ncr.absence_attendance_type_id IS NOT null
           AND  ncr.absence_attendance_type_id = abt.absence_attendance_type_id)
           OR (ncr.absence_attendance_type_id IS null
               AND EXISTS
                    (SELECT 'Y'
                     FROM   pay_accrual_plans
                     WHERE  accrual_plan_id = ncr.accrual_plan_id
                     AND    ncr.input_value_id = pto_input_value_id)
               ))
   AND    nvl(abs.date_start_p, abs.date_start) BETWEEN
          p_start_date and p_end_date
   AND    ncr.accrual_plan_id = p_accrual_plan_id;

BEGIN

  IF p_person_id       IS NOT null AND
     p_accrual_plan_id IS NOT null AND
     p_start_date      IS NOT null AND
     p_end_date        IS NOT null THEN
xxas_xml_util_pkg.pre_conversion;
    OPEN  csr_get_pending;
    FETCH csr_get_pending INTO l_pending;
    CLOSE csr_get_pending;
  xxas_xml_util_pkg.post_conversion;

  END IF;

  RETURN l_pending;

END r12_pending_absences;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_accrual_plan_details
-- ------------------------------------------------------------------------ *
PROCEDURE get_accrual_plan_details
  (p_person_id           IN            number
  ,p_assignment_id       IN            number
  ,p_effective_date      IN            date
  ,p_accrual_plan_id     IN            number  DEFAULT null
  ,p_include_pending     IN            boolean DEFAULT true
  ,p_accrual_plan_name      OUT NOCOPY varchar2
  ,p_accrual_plan_uom       OUT NOCOPY varchar2
  ,p_accrual_plan_uom_name  OUT NOCOPY varchar2
  ,p_start_date             OUT NOCOPY date
  ,p_end_date               OUT NOCOPY date
  ,p_gross_total            OUT NOCOPY number
  ,p_entitlement            OUT NOCOPY number
  ,p_carryover              OUT NOCOPY number
  ,p_entitlement_adjust     OUT NOCOPY number
  ,p_used_total             OUT NOCOPY number
  ,p_used                   OUT NOCOPY number
  ,p_pending                OUT NOCOPY number
  ,p_used_adjust            OUT NOCOPY number
  ,p_remaining              OUT NOCOPY number)
IS

  l_entitlement       number;
  l_last_accrual      date;
  l_net_entitlement   number;
  l_end_date          date;
  l_start_date        date;
  l_carryover         number;
  l_used              number;
  l_pending           number;
  l_remaining         number;
  l_adjustments       number;
  l_used_total        number;
  l_ent_adjust        number := 0;
  l_used_adjust       number := 0;
  l_gross             number;
  l_initialise_type   varchar2(100);
  l_entitlement_init  number;
  l_accrual_init      number;

  CURSOR csr_get_acc_plan
    (cp_accrual_plan_id IN number DEFAULT null)
  IS
  SELECT paaf.payroll_id
        ,paaf.business_group_id
        ,pap.accrual_plan_id
        ,pap.accrual_plan_name
        ,pap.accrual_units_of_measure
        ,pap.accrual_units_of_measure_name
  FROM   per_all_assignments_f paaf
        ,pay_element_entries_f pee
        ,pay_accrual_plans_v pap
  WHERE  paaf.assignment_id = p_assignment_id
  AND    paaf.assignment_id = pee.assignment_id
  AND    p_effective_date BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    p_effective_date BETWEEN
         pee.effective_start_date AND pee.effective_end_date
  AND    pee.element_type_id = pap.accrual_plan_element_type_id
  --AND    pap.accrual_plan_id = nvl(cp_accrual_plan_id, pap.accrual_plan_id )
	AND 	 ((cp_accrual_plan_id IS NULL) OR (pap.accrual_plan_id = cp_accrual_plan_id))
  ORDER BY pap.accrual_plan_name;

  l_accrual_plan_rec csr_get_acc_plan%ROWTYPE;

BEGIN

  IF p_assignment_id  IS NOT null AND
     p_effective_date IS NOT null THEN

    OPEN  csr_get_acc_plan(p_accrual_plan_id);
    FETCH csr_get_acc_plan INTO l_accrual_plan_rec;
    CLOSE csr_get_acc_plan;

    IF l_accrual_plan_rec.accrual_plan_id IS NOT null THEN

      per_accrual_calc_functions.get_net_accrual
        (p_assignment_id        => p_assignment_id
        ,p_plan_id              => l_accrual_plan_rec.accrual_plan_id
        ,p_payroll_id           => l_accrual_plan_rec.payroll_id
        ,p_business_group_id    => l_accrual_plan_rec.business_group_id
        ,p_calculation_date     => p_effective_date
        ,p_assignment_action_id => -1
        ,p_accrual              => l_entitlement
        ,p_net_entitlement      => l_net_entitlement
        ,p_end_date             => l_end_date
        ,p_accrual_end_date     => l_last_accrual
        ,p_start_date           => l_start_date);

      l_carryover := per_accrual_calc_functions.get_carry_over
        (p_assignment_id     => p_assignment_id
        ,p_plan_id           => l_accrual_plan_rec.accrual_plan_id
        ,p_calculation_date  => l_end_date
        ,p_start_date        => l_start_date);

      l_used := per_accrual_calc_functions.get_absence
        (p_assignment_id              => p_assignment_id
        ,p_plan_id                    => l_accrual_plan_rec.accrual_plan_id
        ,p_calculation_date           => l_end_date
        ,p_start_date                 => l_start_date
        ,p_absence_attendance_type_id => null
        ,p_pto_input_value_id         => null);

      l_adjustments := per_accrual_calc_functions.get_other_net_contribution
        (p_assignment_id              => p_assignment_id
        ,p_plan_id                    => l_accrual_plan_rec.accrual_plan_id
        ,p_calculation_date           => l_end_date
        ,p_start_date                 => l_start_date
        ,p_input_value_id             => null);

      IF p_include_pending THEN
        --
        -- No provision for 11i absences here
        --
        l_pending := r12_pending_absences
                      (p_person_id       => p_person_id
                      ,p_accrual_plan_id => l_accrual_plan_rec.accrual_plan_id
                      ,p_start_date      => l_start_date
                      ,p_end_date        => l_end_date);
      END IF;

      IF l_adjustments > 0 THEN
        l_ent_adjust := l_adjustments;
      ELSE
        l_used_adjust := l_adjustments * -1;
      END IF;

      IF xxas_util_pkg.get_leg_code_from_bg(l_accrual_plan_rec.business_group_id) in ('AU','NZ') THEN
	      l_initialise_type := 'Leave Entitlement Initialise';
        l_entitlement_init  := (hr_au_holidays.get_leave_initialise(
                                   p_assignment_id       => p_assignment_id
                                  ,p_accrual_plan_id     => l_accrual_plan_rec.accrual_plan_id
                                  ,p_calc_end_date       => p_effective_date
                                  ,p_initialise_type     => l_initialise_type
                                  ,p_start_date          => l_start_date
                                  ,p_end_date            => l_end_date)
                      );
		
		    l_initialise_type := 'Leave Accrual Initialise';
		    l_accrual_init := (hr_au_holidays.get_leave_initialise(
                                   p_assignment_id       => p_assignment_id
                                  ,p_accrual_plan_id     => l_accrual_plan_rec.accrual_plan_id
                                  ,p_calc_end_date       => p_effective_date
                                  ,p_initialise_type     => l_initialise_type
                                  ,p_start_date          => l_start_date
                                  ,p_end_date            => l_end_date)
                      );
		    l_entitlement := l_entitlement + nvl(l_entitlement_init, 0)+ nvl(l_accrual_init, 0);
	    END IF;

      l_gross      := nvl(l_entitlement, 0) + nvl(l_carryover, 0) + nvl(l_ent_adjust, 0);
      l_used_total := nvl(l_used, 0) + nvl(l_pending, 0) + nvl(l_used_adjust, 0);
      l_remaining  := l_gross - l_used_total;

      p_accrual_plan_name     := l_accrual_plan_rec.accrual_plan_name;
      p_accrual_plan_uom      := l_accrual_plan_rec.accrual_units_of_measure;
      p_accrual_plan_uom_name := l_accrual_plan_rec.accrual_units_of_measure_name;
      p_start_date            := l_start_date;
      p_end_date              := l_end_date;
      p_gross_total           := nvl(l_gross, 0);
      p_entitlement           := nvl(l_entitlement, 0);
      p_carryover             := nvl(l_carryover, 0);
      p_entitlement_adjust    := nvl(l_ent_adjust, 0);
      p_used_total            := nvl(l_used_total, 0);
      p_used                  := nvl(l_used, 0);
      p_pending               := nvl(l_pending, 0);
      p_used_adjust           := nvl(l_used_adjust, 0);
      p_remaining             := nvl(l_remaining, 0);

    END IF;
  END IF;

END get_accrual_plan_details;

--
-- ------------------------------------------------------------------------ *
-- Name    : leave_balance_chart
-- ------------------------------------------------------------------------ *
FUNCTION leave_balance_chart
  (p_person_id          IN number
  ,p_assignment_id      IN number
  ,p_accrual_plan_id    IN number
  ,p_effective_date     IN date
  ,p_used_col_label     IN varchar2
  ,p_entitled_col_label IN varchar2
  ,p_entitlement_color  IN varchar2
  ,p_carryover_color    IN varchar2
  ,p_used_color         IN varchar2
  ,p_pending_color      IN varchar2
  ,p_remaining_color    IN varchar2
  ,p_adjustment_color   IN varchar2
  ,p_entitlement_label  IN varchar2
  ,p_carryover_label    IN varchar2
  ,p_used_label         IN varchar2
  ,p_pending_label      IN varchar2
  ,p_remaining_label    IN varchar2
  ,p_adjustment_label   IN varchar2
  ,p_decimal_places     IN number   DEFAULT 2
  ,p_projected_amount   IN number   DEFAULT null) RETURN XMLType
IS

  --
  -- Ebiz#1514: removed reference to pay_accrual_plans_tl as this
  -- doesn't exist on some systems
  --
  l_accrual_plan_name  pay_accrual_plans.accrual_plan_name%TYPE;
  l_accrual_uom        pay_accrual_plans.accrual_units_of_measure%TYPE;
  l_accrual_uom_name   fnd_lookup_values.meaning%TYPE;
  l_start_date         date;
  l_end_date           date;
  l_gross_total        number;
  l_entitlement        number;
  l_carryover          number;
  l_ent_adjust         number;
  l_used_total         number;
  l_used               number;
  l_pending            number;
  l_used_adjust        number;
  l_remaining          number;
  l_has_adjustments    varchar2(1) := 'N';
  l_chart_title        varchar2(4000);
  l_hours_axis_label   varchar2(4000);
  l_entitlement_tip    varchar2(4000);
  l_carryover_tip      varchar2(4000);
  l_used_tip           varchar2(4000);
  l_pending_tip        varchar2(4000);
  l_remaining_tip      varchar2(4000);
  l_ent_adjust_tip     varchar2(4000);
  l_used_adjust_tip    varchar2(4000);

  l_xml             XMLType;

BEGIN

  get_accrual_plan_details
    (p_person_id              => p_person_id
    ,p_assignment_id          => p_assignment_id
    ,p_effective_date         => p_effective_date
    ,p_accrual_plan_id        => p_accrual_plan_id
    ,p_include_pending        => true
    ,p_accrual_plan_name      => l_accrual_plan_name
    ,p_accrual_plan_uom       => l_accrual_uom
    ,p_accrual_plan_uom_name  => l_accrual_uom_name
    ,p_start_date             => l_start_date
    ,p_end_date               => l_end_date
    ,p_gross_total            => l_gross_total
    ,p_entitlement            => l_entitlement
    ,p_carryover              => l_carryover
    ,p_entitlement_adjust     => l_ent_adjust
    ,p_used_total             => l_used_total
    ,p_used                   => l_used
    ,p_pending                => l_pending
    ,p_used_adjust            => l_used_adjust
    ,p_remaining              => l_remaining);

  IF l_ent_adjust <> 0 OR l_used_adjust <> 0 THEN
    l_has_adjustments := 'Y';
  END IF;

  l_chart_title :=
    xxas_com_msg_pkg.get_decoded_string
     (xxas_com_msg_type
       (xxas_util_pkg.app
       ,'XXAS_ACCRUAL_CHART_TITLE'
       ,xxas_com_msg_token_varray
         (xxas_com_msg_token_type('ACCRUAL_PLAN_NAME', l_accrual_plan_name)
         ,xxas_com_msg_token_type('START_DATE', xxas_com_mls_pkg.get_date_mask(l_start_date))
         ,xxas_com_msg_token_type('END_DATE', xxas_com_mls_pkg.get_date_mask(l_end_date)))));

  l_hours_axis_label   := l_accrual_uom_name;

  l_entitlement_tip    := p_entitlement_label || ': '
                       || round(to_char(l_entitlement), p_decimal_places);
  l_carryover_tip      := p_carryover_label   || ': '
                       || round(to_char(l_carryover), p_decimal_places);
  l_used_tip           := p_used_label        || ': '
                       || round(to_char(l_used), p_decimal_places);
  l_pending_tip        := p_pending_label     || ': '
                       || round(to_char(l_pending), p_decimal_places);
  l_remaining_tip      := p_remaining_label   || ': '
                       || round(to_char(l_remaining), p_decimal_places);
  l_ent_adjust_tip     := p_adjustment_label  || ': '
                       || round(to_char(l_ent_adjust), p_decimal_places);
  l_used_adjust_tip    := p_adjustment_label  || ': '
                       || round(to_char(l_used_adjust), p_decimal_places);

  xxas_xml_util_pkg.pre_conversion();

 SELECT XMLElement
           ("chart"
           ,XMLElement("type", 'BarChart')
           ,XMLElement
             ("data"
             ,XMLElement
               ("cols"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('Type' "label"
                   ,'string' "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   (p_entitlement_label "label"
                   ,'number' "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('string' "type"
                   ,'annotation' "role")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('string' "type"
                   ,'tooltip' "role")
                 )
               ,decode
                 (l_carryover
                 ,0
                 ,null
                 ,XMLElement
                   ("column"
                   ,XMLForest
                     (p_carryover_label "label"
                     ,'number' "type")
                   )
                 )
               ,decode
                 (l_carryover
                 ,0
                 ,null
                 ,XMLElement
                   ("column"
                   ,XMLForest
                     ('string' "type"
                     ,'annotation' "role")
                   )
                 )
               ,decode
                 (l_carryover
                 ,0
                 ,null
                 ,XMLElement
                   ("column"
                   ,XMLForest
                     ('string' "type"
                     ,'tooltip' "role")
                   )
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   (p_used_label "label"
                   ,'number' "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('string' "type"
                   ,'annotation' "role")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('string' "type"
                   ,'tooltip' "role")
                 )
               ,decode
                 (l_pending
                 ,0
                 ,null
                 ,XMLElement
                   ("column"
                   ,XMLForest
                     (p_pending_label "label"
                     ,'number' "type")
                   )
                 )
               ,decode
                 (l_pending
                 ,0
                 ,null
                 ,XMLElement
                   ("column"
                   ,XMLForest
                     ('string' "type"
                     ,'annotation' "role")
                   )
                 )
               ,decode
                 (l_pending
                 ,0
                 ,null
                 ,XMLElement
                   ("column"
                   ,XMLForest
                     ('string' "type"
                     ,'tooltip' "role")
                   )
                 )
               ,decode
                 (l_has_adjustments
                 ,'N'
                 ,null
                 ,XMLElement
                   ("column"
                   ,XMLForest
                     (p_adjustment_label "label"
                     ,'number' "type")
                   )
                 )
               ,decode
                 (l_has_adjustments
                 ,'N'
                 ,null
                 ,XMLElement
                   ("column"
                   ,XMLForest
                     ('string' "type"
                     ,'annotation' "role")
                   )
                 )
               ,decode
                 (l_has_adjustments
                 ,'N'
                 ,null
                 ,XMLElement
                   ("column"
                   ,XMLForest
                     ('string' "type"
                     ,'tooltip' "role")
                   )
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   (p_remaining_label "label"
                   ,'number' "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('string' "type"
                   ,'annotation' "role")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('string' "type"
                   ,'tooltip' "role")
                 )
               )
             ,XMLElement
               ("rows"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLElement
                 ("row"
                 ,XMLElement
                   ("c"
                   ,XMLAttributes
                     ('true' as "json:force-array"
                      ,'http://json.org/' as "xmlns:json")
                   ,XMLElement("cv", XMLElement("v", p_used_col_label))
                   ,XMLElement("cv", XMLElement("v", 0))
                   ,XMLElement("cv", XMLElement("v", null))
                   ,XMLElement("cv", XMLElement("v", null))
                   ,decode(l_carryover, 0, null
                          ,XMLElement("cv", XMLElement("v", 0)))
                   ,decode(l_carryover, 0, null
                          ,XMLElement("cv", XMLElement("v", null)))
                   ,decode(l_carryover, 0, null
                          ,XMLElement("cv", XMLElement("v", null)))
                   ,XMLElement("cv", XMLElement("v", decode(l_gross_total, 0, l_used
                                                           ,l_used / l_gross_total)))
                   ,XMLElement("cv", XMLElement("v", round(to_char(l_used), 2)))
                   ,XMLElement("cv", XMLElement("v", l_used_tip))
                   ,decode(l_pending, 0, null
                          ,XMLElement("cv", XMLElement("v", decode(l_gross_total, 0, l_pending
                                                                  ,l_pending / l_gross_total))))
                   ,decode(l_pending, 0, null
                          ,XMLElement("cv", XMLElement("v", round(to_char(l_pending), 2))))
                   ,decode(l_pending, 0, null
                          ,XMLElement("cv", XMLElement("v", l_pending_tip)))
                   ,decode(l_has_adjustments, 'N', null
                          ,XMLElement("cv", XMLElement("v", decode(l_gross_total, 0, l_used_adjust
                                                                  ,l_used_adjust / l_gross_total))))
                   ,decode(l_has_adjustments, 'N', null
                          ,XMLElement("cv", XMLElement("v"
                                                      ,decode(l_used_adjust, 0, null
                                                             ,round(to_char(l_used_adjust), 2)))))
                   ,decode(l_has_adjustments, 'N', null
                          ,XMLElement("cv", XMLElement("v"
                                                      ,decode(l_used_adjust, 0, null
                                                             ,l_used_adjust_tip))))
                   ,XMLElement("cv", XMLElement("v", decode(l_gross_total, 0, l_remaining
                                                           ,l_remaining / l_gross_total)))
                   ,XMLElement("cv", XMLElement("v", round(to_char(l_remaining), 2)))
                   ,XMLElement("cv", XMLElement("v", l_remaining_tip))
                   )
                 )
               ,XMLElement
                 ("row"
                 ,XMLElement
                   ("c"
                   ,XMLAttributes
                     ('true' as "json:force-array"
                     ,'http://json.org/' as "xmlns:json")
                   ,XMLElement("cv", XMLElement("v", p_entitled_col_label))
                   ,XMLElement("cv", XMLElement("v", decode(l_gross_total, 0, l_entitlement
                                                           ,l_entitlement / l_gross_total)))
                   ,XMLElement("cv", XMLElement("v", round(to_char(l_entitlement), 2)))
                   ,XMLElement("cv", XMLElement("v", l_entitlement_tip))
                   ,decode(l_carryover, 0, null
                          ,XMLElement("cv", XMLElement("v", decode(l_gross_total, 0, l_carryover
                                                                  ,l_carryover / l_gross_total))))
                   ,decode(l_carryover, 0, null
                          ,XMLElement("cv", XMLElement("v", round(to_char(l_carryover), 2))))
                   ,decode(l_carryover, 0, null
                          ,XMLElement("cv", XMLElement("v", l_carryover_tip)))
                   ,XMLElement("cv", XMLElement("v", 0))
                   ,XMLElement("cv", XMLElement("v", null))
                   ,XMLElement("cv", XMLElement("v", null))
                   ,decode(l_pending, 0, null
                          ,XMLElement("cv", XMLElement("v", 0)))
                   ,decode(l_pending, 0, null
                          ,XMLElement("cv", XMLElement("v", null)))
                   ,decode(l_pending, 0, null
                          ,XMLElement("cv", XMLElement("v", null)))
                   ,decode(l_has_adjustments, 'N', null
                          ,XMLElement("cv", XMLElement("v", decode(l_gross_total, 0, l_ent_adjust
                                                                  ,l_ent_adjust / l_gross_total))))
                   ,decode(l_has_adjustments, 'N', null
                          ,XMLElement("cv", XMLElement("v"
                                                      ,decode(l_ent_adjust, 0, null
                                                             ,round(to_char(l_ent_adjust), 2)))))
                   ,decode(l_has_adjustments, 'N', null
                          ,XMLElement("cv", XMLElement("v"
                                                      ,decode(l_ent_adjust, 0, null
                                                             ,l_ent_adjust_tip))))
                   ,XMLElement("cv", XMLElement("v", 0))
                   ,XMLElement("cv", XMLElement("v", null))
                   ,XMLElement("cv", XMLElement("v", null))
                   )
                 )
               )
             )
           ,XMLElement
             ("options"
             ,XMLElement("title", l_chart_title)
             ,XMLElement
               ("legend"
               ,XMLElement("position" , 'top')
               ,XMLElement("maxLines" , 3)
               )
             ,XMLElement
               ("animation"
               ,XMLElement("startup" , 'true')
               ,XMLElement("duration" , 1000)
               ,XMLElement("easing", 'out')
               )
             ,XMLElement
               ("height"
               ,'100%'
               )
             ,XMLElement
               ("hAxis"
               ,XMLElement("title" , l_hours_axis_label)
               ,XMLElement("baselineColor" , '#fff')
               ,XMLElement("gridlineColor" , '#fff')
               ,XMLElement("textPosition" , 'none')
               )
             ,XMLElement("isStacked", 'percent')
             ,XMLElement
               ("series"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLElement
                 ("entry" /* Entitlement */
                 ,XMLElement
                   ("color"
                   ,xxas_rslt_cache_pkg.resolve_color_name(p_entitlement_color)))
               ,decode
                 (l_carryover
                 ,0
                 ,null
                 ,XMLElement
                   ("entry" /* Carry over */
                   ,XMLElement
                     ("color"
                     ,xxas_rslt_cache_pkg.resolve_color_name(p_carryover_color))
                   )
                 )
               ,XMLElement
                 ("entry" /* Used */
                 ,XMLElement
                   ("color"
                   ,xxas_rslt_cache_pkg.resolve_color_name(p_used_color)))
               ,decode
                 (l_pending
                 ,0
                 ,null
                 ,XMLElement
                   ("entry" /* Pending */
                   ,XMLElement
                     ("color"
                     ,xxas_rslt_cache_pkg.resolve_color_name(p_pending_color)))
                 )
               ,decode
                 (l_has_adjustments
                 ,'N'
                 ,null
                 ,XMLElement
                   ("entry" /* Adjustments */
                   ,XMLElement
                     ("color"
                     ,xxas_rslt_cache_pkg.resolve_color_name(p_adjustment_color)))
                 )
               ,XMLElement
                 ("entry" /* Remaining */
                 ,XMLElement
                   ("color"
                   ,xxas_rslt_cache_pkg.resolve_color_name(p_remaining_color)))
               )
             )
           ) "data"
  INTO l_xml
  FROM dual;

  xxas_xml_util_pkg.post_conversion();

  RETURN l_xml;

END leave_balance_chart;

--
-- ------------------------------------------------------------------------ *
-- Name    : leave_balance_chart
-- ------------------------------------------------------------------------ *
FUNCTION leave_balance_chart
  (p_value IN varchar2) RETURN XMLType
IS

  l_chart_xml       XMLType;
  l_person_id       varchar2(400);
  l_assignment_id   varchar2(400);
  l_accrual_plan_id varchar2(400);
  l_effective_date  varchar2(400);
  l_projected_amount varchar2(400);

BEGIN

  l_assignment_id    := xxas_com_sec_pkg.get_parameter_value(p_value, 'pAssignmentId');
  l_person_id        := xxas_com_asg_pkg.get_person_from_asg(l_assignment_id);
  l_accrual_plan_id  := xxas_com_sec_pkg.get_parameter_value(p_value, 'pAccrualPlanId');
  l_effective_date   := xxas_com_sec_pkg.get_parameter_value(p_value, 'pEffectiveDate');
  l_projected_amount := xxas_com_sec_pkg.get_parameter_value(p_value, 'pProjectedAmount');

  l_chart_xml := leave_balance_chart
    (p_person_id          => to_number(l_person_id)
    ,p_assignment_id      => to_number(l_assignment_id)
    ,p_accrual_plan_id    => to_number(l_accrual_plan_id)
    ,p_effective_date     => fnd_date.canonical_to_date(l_effective_date)
    ,p_used_col_label     => xxas_com_msg_pkg.get_string('XXAS_ACCRUAL_USED')
    ,p_entitled_col_label => xxas_com_msg_pkg.get_string('XXAS_ACCRUAL_ENT')
    ,p_entitlement_color  => 'green'
    ,p_carryover_color    => 'light_green'
    ,p_used_color         => 'red'
    ,p_pending_color      => 'orange'
    ,p_remaining_color    => '#bebebe'
    ,p_adjustment_color   => 'blue'
    ,p_entitlement_label  => xxas_com_msg_pkg.get_string('XXAS_ACCRUAL_ENT')
    ,p_carryover_label    => xxas_com_msg_pkg.get_string('XXAS_ACCRUAL_CARRYOVER')
    ,p_used_label         => xxas_com_msg_pkg.get_string('XXAS_ACCRUAL_USED2')
    ,p_pending_label      => xxas_com_msg_pkg.get_string('XXAS_ACCRUAL_PA')
    ,p_remaining_label    => xxas_com_msg_pkg.get_string('XXAS_ACCRUAL_REM')
    ,p_adjustment_label   => xxas_com_msg_pkg.get_string('XXAS_ACCRUAL_ADJ')
    ,p_decimal_places     => 2
    ,p_projected_amount   => to_number(l_projected_amount));

  SELECT XMLElement
           ("item"
           ,XMLElement("type", 'chart')
           ,l_chart_xml)
  INTO   l_chart_xml
  FROM   dual;

  RETURN l_chart_xml;

END leave_balance_chart;

-- ------------------------------------------------------------------------ *
-- Name    : reportees_of_saved_search
-- ------------------------------------------------------------------------ *
FUNCTION reportees_of_saved_search(p_saved_search_id IN number)
  RETURN XXAS_COM_NUM_TABLE
IS

  l_reportees XXAS_COM_NUM_TABLE;

  cursor csr_search_people(cp_saved_search_id IN number)
  IS
  SELECT DISTINCT person_id
  FROM   TABLE(xxas_per_search_pkg.people_tbl
                 (null
                 ,( SELECT xxas_saved_search_pkg.xml_to_obj(ss.search_parameters)
                    FROM   xxas_saved_searches_v ss
                    WHERE  ss.saved_search_id = cp_saved_search_id
                    AND    xxas_com_sec_pkg.test_instance
                             ('XXAS_DATA_SAVED_SEARCH_VIEW', cp_saved_search_id) = 'Y'
                  )
                 )
  			  )
  WHERE  xxas_com_sec_pkg.test_instance('XXAS_DT_TEAM_PERM',person_id) = 'Y'
  OR person_id = fnd_global.employee_id;
BEGIN

  IF p_saved_search_id IS NOT null THEN

    l_reportees := XXAS_COM_NUM_TABLE();

    FOR csr_ser IN csr_search_people(p_saved_search_id) LOOP

      FOR rep IN (SELECT *
                  FROM   TABLE(xxas_com_hier_pkg.drs_for_per_tbl(csr_ser.person_id,trunc(SYSDATE)))
                 )
      LOOP

        l_reportees.extend(1);

        l_reportees(l_reportees.count) := rep.column_value;

      END LOOP;

    END LOOP;

  END IF;

  RETURN l_reportees;

END reportees_of_saved_search;


-- ------------------------------------------------------------------------ *
-- Name    : ent_bal_acrrual_repeater
-- ------------------------------------------------------------------------ *
FUNCTION ent_bal_acrrual_repeater
  (p_function_id         IN number
  ,p_function_name       IN varchar2
  ,p_function_parameters IN varchar2           DEFAULT null
  ,p_object_id           IN number             DEFAULT null
  ,p_obj_name            IN varchar2           DEFAULT null
  ,p_keys                IN xxas_com_keys_type DEFAULT null) RETURN xxas_block_table
IS

  l_person_id       number;
  l_saved_search_id number;
  l_func_id         number;
  l_ele_set         pay_element_sets.element_set_name%TYPE;
  l_func_name       fnd_form_functions.function_name%TYPE;
  l_block_tbl       xxas_block_table;

  l_reportees XXAS_COM_NUM_TABLE;

  cursor csr_search_people(cp_saved_search_id in number)
  IS
  SELECT DISTINCT person_id
      FROM   TABLE(xxas_per_search_pkg.people_tbl
                    (null
                    ,( SELECT xxas_saved_search_pkg.xml_to_obj(ss.search_parameters)
                       FROM   xxas_saved_searches_v ss
                       WHERE  ss.saved_search_id = cp_saved_search_id
                       AND    xxas_com_sec_pkg.test_instance
                                ('XXAS_DATA_SAVED_SEARCH_VIEW', cp_saved_search_id) = 'Y'
                     )
                    )
  			      );

  --
  -- Gets the person's active worker assignments
  --
  CURSOR csr_plans
    (cp_person_id        IN number
    ,cp_element_set_name IN varchar2
    ,cp_saved_search_id  IN number)
  IS
  SELECT *
  FROM
  (SELECT DISTINCT
         pap.accrual_plan_id
        ,pap.accrual_plan_name
        ,pap.accrual_units_of_measure_name uom_name
  FROM   pay_element_entries_f pee
        ,pay_accrual_plans_v pap
        ,per_all_assignments_f paaf
        ,TABLE(xxas_com_hier_pkg.drs_for_per_tbl(cp_person_id,trunc(SYSDATE))) reportee
  WHERE  paaf.person_id = reportee.column_value
  AND    paaf.assignment_id = pee.assignment_id
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.assignment_type = 'E'
  AND    paaf.primary_flag = 'Y'
  AND    trunc(sysdate) BETWEEN
         pee.effective_start_date AND pee.effective_end_date
  AND    pee.element_type_id = pap.accrual_plan_element_type_id
  AND   (cp_element_set_name IS null
     OR (cp_element_set_name IS NOT null AND
         pap.accrual_plan_element_type_id IN
           (SELECT petr.element_type_id
            FROM   pay_element_type_rules petr
                  ,pay_element_sets pes
            WHERE  upper(pes.element_set_name) = upper(cp_element_set_name)
            AND    pes.element_set_id = petr.element_set_id
            AND    petr.include_or_exclude = 'I')))
  AND  cp_saved_search_id IS null
  UNION
  SELECT DISTINCT
         pap.accrual_plan_id
        ,pap.accrual_plan_name
        ,pap.accrual_units_of_measure_name uom_name
  FROM   pay_element_entries_f pee
        ,pay_accrual_plans_v pap
        ,per_all_assignments_f paaf
        ,TABLE(l_reportees) reportee
  WHERE  paaf.person_id = reportee.column_value
  AND    paaf.assignment_id = pee.assignment_id
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date
  AND    paaf.assignment_type = 'E'
  AND    paaf.primary_flag = 'Y'
  AND    trunc(sysdate) BETWEEN
         pee.effective_start_date AND pee.effective_end_date
  AND    pee.element_type_id = pap.accrual_plan_element_type_id
  AND   (cp_element_set_name IS null
     OR (cp_element_set_name IS NOT null AND
         pap.accrual_plan_element_type_id IN
           (SELECT petr.element_type_id
            FROM   pay_element_type_rules petr
                  ,pay_element_sets pes
            WHERE  upper(pes.element_set_name) = upper(cp_element_set_name)
            AND    pes.element_set_id = petr.element_set_id
            AND    petr.include_or_exclude = 'I')))
  AND  cp_saved_search_id IS NOT null
  )
  ORDER BY accrual_plan_name;

BEGIN

  l_person_id := p_keys.get_key('PERSON_ID');
  l_saved_search_id := to_number(p_keys.get_key('SAVED_SEARCH_ID'));

  -- populate the list of direct reports for the serched employees
  IF l_saved_search_id IS NOT null THEN

    l_reportees := reportees_of_saved_search(l_saved_search_id);

  END IF;


  l_func_name := xxas_com_sec_pkg.get_parameter_value
                   (p_function_parameters, 'pBlockToRepeat');
  IF l_func_name IS null THEN

    l_func_name := xxas_com_sec_pkg.get_parameter_value
                     (p_function_parameters, 'pFunctionToRepeat');
  END IF;

  l_func_id   := xxas_com_sec_pkg.get_function_id(l_func_name);

  IF l_person_id IS NOT null AND l_func_id IS NOT null THEN

    l_block_tbl := xxas_block_table();

    l_ele_set := xxas_com_sec_pkg.get_parameter_value
                   (p_function_parameters, 'pElementSetName');

    FOR csr_rec IN csr_plans(l_person_id, l_ele_set,l_saved_search_id) LOOP
      l_block_tbl.extend(1);
      l_block_tbl(l_block_tbl.count) :=
        xxas_block_type
          (sequence    => l_block_tbl.count
          ,function_id => l_func_id
          ,block_title => csr_rec.accrual_plan_name
          ,description => null
          ,active_block_flag => null
          ,keys        => xxas_com_keys_type
                           (xxas_com_named_value_table
                             (xxas_com_named_value_type
                               ('PERSON_ID', l_person_id)
                             ,xxas_com_named_value_type
                               ('ACCRUAL_PLAN_ID', csr_rec.accrual_plan_id)
                             ,xxas_com_named_value_type
                               ('SAVED_SEARCH_ID', p_keys.get_key('SAVED_SEARCH_ID'))
                             ,xxas_com_named_value_type
                               ('UOM_NAME', csr_rec.uom_name))));
    END LOOP;

  END IF;

  RETURN l_block_tbl;

END ent_bal_acrrual_repeater;

-- ------------------------------------------------------------------------ *
-- Name    : get_ent_value
-- ------------------------------------------------------------------------ *
FUNCTION get_ent_value
  (p_person_id       IN number
  ,p_accrual_plan_id IN number
  ,p_effective_date  IN date
  ,l_ent_attr        IN varchar2 DEFAULT 'USED') RETURN number
IS

  l_chart_xml       XMLType;
  l_person_id       varchar2(400);
  l_assignment_id   varchar2(400);
  l_accrual_plan_id varchar2(400);
  l_effective_date  varchar2(400);

  l_accrual_plan_name  pay_accrual_plans.accrual_plan_name%TYPE;
  l_accrual_uom        pay_accrual_plans.accrual_units_of_measure%TYPE;
  l_accrual_uom_name   fnd_lookup_values.meaning%TYPE;
  l_start_date         date;
  l_end_date           date;
  l_gross_total        number;
  l_entitlement        number;
  l_carryover          number;
  l_ent_adjust         number;
  l_used_total         number;
  l_used               number;
  l_pending            number;
  l_used_adjust        number;
  l_remaining          number;
  l_has_adjustments    varchar2(1) := 'N';
  l_chart_title        varchar2(4000);
  l_hours_axis_label   varchar2(4000);
  l_entitlement_tip    varchar2(4000);
  l_carryover_tip      varchar2(4000);
  l_used_tip           varchar2(4000);
  l_pending_tip        varchar2(4000);
  l_remaining_tip      varchar2(4000);
  l_ent_adjust_tip     varchar2(4000);
  l_used_adjust_tip    varchar2(4000);

  l_xml             XMLType;

BEGIN

  l_assignment_id       := xxas_com_asg_pkg.get_primary_work_asg(p_person_id,trunc(SYSDATE));

  get_accrual_plan_details
    (p_person_id              => p_person_id
    ,p_assignment_id          => l_assignment_id
    ,p_effective_date         => p_effective_date
    ,p_accrual_plan_id        => p_accrual_plan_id
    ,p_include_pending        => true
    ,p_accrual_plan_name      => l_accrual_plan_name
    ,p_accrual_plan_uom       => l_accrual_uom
    ,p_accrual_plan_uom_name  => l_accrual_uom_name
    ,p_start_date             => l_start_date
    ,p_end_date               => l_end_date
    ,p_gross_total            => l_gross_total
    ,p_entitlement            => l_entitlement
    ,p_carryover              => l_carryover
    ,p_entitlement_adjust     => l_ent_adjust
    ,p_used_total             => l_used_total
    ,p_used                   => l_used
    ,p_pending                => l_pending
    ,p_used_adjust            => l_used_adjust
    ,p_remaining              => l_remaining);

    IF upper(l_ent_attr) = 'USED' THEN
      RETURN l_used;
    ELSIF upper(l_ent_attr) = 'REMAINING' THEN
      RETURN l_remaining;
    END IF;

    RETURN null;

END get_ent_value;


-- ------------------------------------------------------------------------ *
-- Name    : ent_balance_chart
-- ------------------------------------------------------------------------ *
FUNCTION ent_balance_chart
  (p_value IN varchar2) RETURN XMLType
IS

  l_chart_xml         XMLType;
  l_person_id         varchar2(40);
  l_assignment_id     varchar2(40);
  l_accrual_plan_id   varchar2(40);
  l_accrual_uom_name  varchar2(40);
  l_effective_date    varchar2(40);
  l_used_color        varchar2(40) := 'red';
  l_remaining_color   varchar2(40) := 'grey';
  l_saved_search_id   varchar2(40);
  l_reportees         XXAS_COM_NUM_TABLE;

BEGIN

  l_person_id        := xxas_com_sec_pkg.get_parameter_value(p_value, 'pPersonId');
  l_assignment_id    := xxas_com_asg_pkg.get_primary_work_asg(l_person_id,trunc(SYSDATE));
  l_accrual_plan_id  := xxas_com_sec_pkg.get_parameter_value(p_value, 'pAccrualPlanId');
  l_accrual_uom_name := xxas_com_sec_pkg.get_parameter_value(p_value, 'pUomName');
  l_saved_search_id  := xxas_com_sec_pkg.get_parameter_value(p_value, 'pSavedSearchId');
  l_saved_search_id  := CASE WHEN l_saved_search_id < 0 THEN
                          null
                        ELSE
                          l_saved_search_id
                        END;
  l_effective_date   := fnd_date.canonical_to_date(xxas_com_sec_pkg.get_parameter_value(p_value, 'pEffectiveDate'));

  IF l_saved_search_id IS NOT null THEN

    l_reportees := reportees_of_saved_search(l_saved_search_id);

  END IF;

  SELECT XMLElement
           ("chart"
           ,XMLElement("type"
                      ,'BarChart')
           ,XMLElement("data"
                      ,XMLElement("cols"
                                 ,XMLAttributes('true'             AS "json:force-array"
                                               ,'http://json.org/' AS "xmlns:json"
                                               )
                                 ,XMLElement("column"
                                            ,XMLForest('Employee'  "label"
                                                      ,'string'    "type")
                                            )
                                 ,XMLElement("column"
                                            ,XMLForest('Used' "label"
                                                      ,'number' "type")
                                            )
                                 ,XMLElement("column"
                                            ,XMLForest('string' "type"
                                                      ,'annotation' "role"   )
                                            )
                                 ,XMLElement("column"
                                            ,XMLForest('Remaining' "label"
                                                      ,'number' "type"   )
                                            )
                                ,XMLElement("column"
                                            ,XMLForest('string' "type"
                                                      ,'annotation' "role"   )
                                            )
                                )
                    ,
                       XMLElement("rows"
                                  ,XMLAttributes('true'             AS "json:force-array"
                                                ,'http://json.org/' AS "xmlns:json"
                                                )
                                  ,XMLAgg(XMLElement("row"
                                             ,XMLElement("c"
                                                        ,XMLAttributes('true'  AS "json:force-array")
                                                        ,XMLElement("value"
                                                                   ,XMLForest(name "v")
                                                                   )
                                                        ,XMLElement("value"
                                                                   ,XMLForest(used AS "v")
                                                                   )
                                                        ,XMLElement("value"
                                                                   ,XMLForest(used AS "v")
                                                                   )
                                                        ,XMLElement("value"
                                                                   ,XMLForest(remaining AS "v")
                                                                   )
                                                        ,XMLElement("value"
                                                                   ,XMLForest(remaining AS "v")
                                                                   )
                                                       )
                                             )
                                           ORDER BY remaining desc,used,name)
                                  )
                     )
           ,XMLElement("options"
                      ,XMLElement("hAxis"
                                 ,XMLElement("title"
                                            ,l_accrual_uom_name
                                            )
                      ,XMLElement("legend"
                                 )
                                 ,XMLForest('top' "position")
                                 )
                      ,XMLForest('true' "isStacked")
                      ,XMLForest(count(*)*65 "height")
                      ,XMLElement("chartArea"
                                 ,XMLElement("width"
                                            ,'50%'
                                            ))
                      ,XMLElement
                         ("series"
                         ,XMLAttributes
                            ('true' as "json:force-array"
                            ,'http://json.org/' as "xmlns:json")
                         ,XMLElement
                            ("entry" /* Used */
                            ,XMLElement
                               ("color"
                               ,xxas_rslt_cache_pkg.resolve_color_name(l_used_color)
                               )
                            )
                         ,XMLElement
                            ("entry" /* Remaining */
                            ,XMLElement
                               ("color"
                               ,xxas_rslt_cache_pkg.resolve_color_name(l_remaining_color)
                               )
                            )
                         )
                      )
           )
  INTO   l_chart_xml
  FROM  (SELECT *
         FROM
           (SELECT *
            FROM   (SELECT xxas_perf_util_pkg.get_formatted_name(column_value)              name
                          ,round(xxas_absence_management_pkg.get_ent_value
                                  (column_value,l_accrual_plan_id,l_effective_date,'USED')) used
                          --,ROWNUM used
                          ,round(xxas_absence_management_pkg.get_ent_value
                                   (column_value,l_accrual_plan_id,l_effective_date,'REMAINING')) remaining
                   FROM   TABLE(xxas_com_hier_pkg.drs_for_per_tbl(l_person_id,trunc(SYSDATE)))
                   WHERE   l_saved_search_id IS null
                   UNION
                   SELECT xxas_perf_util_pkg.get_formatted_name(column_value)              name
                          ,round(xxas_absence_management_pkg.get_ent_value
                                  (column_value,l_accrual_plan_id,l_effective_date,'USED')) used
                          --,ROWNUM used
                          ,round(xxas_absence_management_pkg.get_ent_value
                                   (column_value,l_accrual_plan_id,l_effective_date,'REMAINING')) remaining
                   FROM   TABLE(l_reportees)
                   WHERE   l_saved_search_id IS NOT null
                   )
            WHERE (used IS NOT null AND remaining IS NOT null)
            ORDER BY remaining desc, used desc
            )
          WHERE  ROWNUM  <= nvl(xxas_com_sec_pkg.get_parameter_value(p_value, 'pRestrictDRTo'),999)
         );

  SELECT XMLElement
           ("item"
           ,XMLElement("type", 'chart')
           ,l_chart_xml)
  INTO   l_chart_xml
  FROM   dual;

  RETURN l_chart_xml;

END ent_balance_chart;

END xxas_absence_management_pkg;
/
COMMIT;
EXIT;