/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2013 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK
SET SCAN OFF


CREATE OR REPLACE TYPE BODY xxas_emp_ui_component_type IS
  
MEMBER PROCEDURE set_attribute
  (p_name  IN VARCHAR2
  ,p_value IN VARCHAR2)
IS
BEGIN
  IF attributes IS NULL
  THEN
    attributes := xxas_emp_ui_attribute_table();
  END IF;

  FOR i IN 1..attributes.COUNT
  LOOP  
    IF attributes(i).name = p_name
    THEN 
      attributes(i).value := p_value;
      RETURN;
    END IF;
    
  END LOOP;
  
  attributes.EXTEND;
  attributes(attributes.COUNT) := xxas_emp_ui_attribute_type
                                   (name  => p_name
                                   ,value => p_value); 
END set_attribute;


MEMBER FUNCTION get_attribute
  (p_name IN VARCHAR2) RETURN VARCHAR2

IS
BEGIN
  
  IF attributes IS NOT NULL
  THEN
    FOR i IN 1..attributes.COUNT
    LOOP  
      IF attributes(i).name = p_name
      THEN 
        RETURN attributes(i).value;
      END IF;
    END LOOP;
  END IF;
  
  RETURN NULL;

END get_attribute;

MEMBER PROCEDURE set_id
  (p_value IN VARCHAR2)
IS 
BEGIN
   set_attribute('id', p_value);
END;

MEMBER FUNCTION get_id RETURN VARCHAR2
IS 
BEGIN
   RETURN get_attribute('id');
END;
  
MEMBER FUNCTION is_id
  (p_attribute IN xxas_emp_ui_attribute_type) RETURN BOOLEAN
IS
BEGIN
  RETURN p_attribute.name = 'id';
END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_emp_ui_valuefield_type IS

MEMBER PROCEDURE set_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('value', p_value);
END;

MEMBER FUNCTION get_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('value');
END;

MEMBER FUNCTION get_clob_value RETURN CLOB
IS 
BEGIN
   RETURN to_clob(self.get_attribute('value'));
END;

MEMBER PROCEDURE set_old_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('old_value', p_value);
END;

MEMBER FUNCTION get_old_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('old_value');
END;

MEMBER PROCEDURE set_value_generator
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('value_generator', p_value);
END;

MEMBER FUNCTION get_value_generator RETURN VARCHAR2
IS 

  l_generator VARCHAR2(400);
  l_type      VARCHAR2(400);
  l_value     VARCHAR2(400);

BEGIN

  IF self.get_value IS NOT NULL THEN
    RETURN null;
  END IF;

  l_generator := self.get_attribute('value_generator');
  
  IF l_generator IS NOT NULL THEN
  
    l_type  := substr(l_generator, 1, instr(l_generator, ':') - 1);
    l_value := substr(l_generator, instr(l_generator, ':') + 1);
    
    RETURN xxas_com_rest_util_pkg.uri_from_path
             ('key-generator/' || l_type || '/' || l_value);
  ELSE
    RETURN null;  
  END IF;
   
END;

MEMBER PROCEDURE set_fixed_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('fixed_value', p_value);
END;

MEMBER FUNCTION get_fixed_value RETURN VARCHAR2
IS

  CURSOR csr_profile_value
    (cp_profile_name IN VARCHAR2)
  IS
  SELECT fnd_profile.value(cp_profile_name)
  FROM dual;

  l_fixed_value   VARCHAR2(400);
  l_type          VARCHAR2(400);
  l_value         VARCHAR2(400);
  l_profile_value VARCHAR2(4000);

BEGIN

  IF self.get_value IS NOT NULL THEN
    RETURN null;
  END IF;

  l_fixed_value := self.get_attribute('fixed_value');
  
  IF l_fixed_value IS NOT NULL THEN
  
    l_type  := substr(l_fixed_value, 1, instr(l_fixed_value, ':') - 1);
    l_value := substr(l_fixed_value, instr(l_fixed_value, ':') + 1);
    
    IF l_type IS NOT NULL AND
       l_value IS NOT NULL AND 
       l_type = 'PROFILE' THEN

      OPEN  csr_profile_value(l_value);
      FETCH csr_profile_value INTO l_profile_value;  
      CLOSE csr_profile_value;

      RETURN l_profile_value;

    END IF;

  ELSE
    RETURN null;  
  END IF;
   
END;

MEMBER PROCEDURE set_transient
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('transient', p_value);
END;

MEMBER FUNCTION get_transient RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('transient');
END;

MEMBER FUNCTION is_transient RETURN BOOLEAN
IS
BEGIN
  RETURN lower(self.get_attribute('transient')) = 'true';
END;

OVERRIDING MEMBER FUNCTION get_obj_xml RETURN XMLType
IS

  l_xml XMLTYPE;

BEGIN

  IF self.is_transient THEN
    RETURN null;
  ELSE
    --
    -- Using an XMLType Constructor rather than evalname in
    -- XMLElement because of 10gR2 bug
    --
    SELECT XMLType
             ('<' || self.get_id || '>'
           || dbms_xmlgen.convert(self.get_value, dbms_xmlgen.entity_encode)
           || '</' || self.get_id || '>')                          
    INTO   l_xml   
    FROM   dual;

    RETURN l_xml;
  END IF;
END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_emp_ui_hidden_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_hidden_type
  (p_ui_hidden_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Hidden' 
                  PASSING p_ui_hidden_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          value_generator     VARCHAR2(4000) path '@valueGenerator',
                          fixed_value         VARCHAR2(4000) path '@fixedValue');            
  l_data c_data%ROWTYPE;
  
BEGIN
  
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_value_generator(l_data.value_generator);
  self.set_fixed_value(l_data.fixed_value);

  RETURN; 
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_hidden_type
  (p_id        IN VARCHAR2
  ,p_value     IN VARCHAR2
  ,p_transient IN VARCHAR2) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_value(p_value);
  self.set_transient(p_transient);
  
  RETURN;          
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
             ("Hidden"
             ,XMLATTRIBUTES
                (self.get_id        AS "id"
                ,self.get_value     AS "value"
                ,self.get_transient AS "transient"))
           ,'//Hidden[not(@value)]'
           ,'@value'
           ,'')                            
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN



  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('hidden'       AS "type"
               ,self.get_id    AS "id"
               ,'hidden'       AS "state"
			          ,CASE WHEN self.get_value IS NULL AND
                          self.get_fixed_value IS NOT NULL
                     THEN self.get_fixed_value
                     ELSE self.get_value 
                END AS "value"
			             ,CASE WHEN self.get_old_value IS NULL AND
                          self.get_fixed_value IS NOT NULL
                     THEN self.get_fixed_value
                     ELSE self.get_old_value 
                END AS "oldValue"
               )
             ,CASE WHEN self.get_value IS NULL AND
                        self.get_value_generator IS NOT NULL
                   THEN
                    XMLElement
                      ("links"
                      ,XMLElement
                        ("valueGenerator"
                        ,XMLElement
                          ("href"
                          ,self.get_value_generator))
                      )
              END
             )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_emp_ui_field_type IS


MEMBER PROCEDURE set_label
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('label'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('label');
END;

MEMBER PROCEDURE set_description
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('description'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_description RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('description');
END;

MEMBER PROCEDURE set_refresh_on_change
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('refresh_on_change', lower(p_value));
END;

MEMBER FUNCTION get_refresh_on_change RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('refresh_on_change'));
END;

MEMBER PROCEDURE set_trigger_name
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('trigger_name', p_value);
END;

MEMBER FUNCTION get_trigger_name RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('trigger_name');
END;

MEMBER PROCEDURE set_trigger_method
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('trigger_method', p_value);
END;

MEMBER FUNCTION get_trigger_method RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('trigger_method');
END;

MEMBER PROCEDURE set_enabled
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('enabled', lower(p_value));
END;

MEMBER FUNCTION get_enabled RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('enabled'));
END;

MEMBER FUNCTION get_state RETURN VARCHAR2
IS 
BEGIN

  IF nvl(self.get_visible, 'true') = 'false' THEN
    RETURN 'hidden';
  ELSIF nvl(self.get_enabled, 'true') = 'false' THEN
    RETURN 'view';
  ELSE
    RETURN 'input';
  END IF;

END;

MEMBER PROCEDURE set_required
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('required', lower(p_value));
END;

MEMBER FUNCTION get_required RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('required'));
END;

MEMBER PROCEDURE set_visible
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('visible', lower(p_value));
END;

MEMBER FUNCTION get_visible RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('visible'));
END;

MEMBER PROCEDURE set_hide_if
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('hide_if', p_value);
END;

MEMBER FUNCTION get_hide_if RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('hide_if');
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_submit_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_submit_type
  (p_ui_submit_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Submit' 
                  PASSING p_ui_submit_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name        VARCHAR2(4000) path '@triggerName',
                          trigger_method      VARCHAR2(4000) path '@triggerMethod',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible', 
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          weight              VARCHAR2(4000) path '@weight',
                          color               VARCHAR2(4000) path '@color',
                          style               VARCHAR2(4000) path '@style',
                          icon                VARCHAR2(4000) path '@icon',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_weight(l_data.weight);
  self.set_color(l_data.color);
  self.set_style(l_data.style);
  self.set_icon(l_data.icon);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_submit_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2
  ,p_trigger_name      VARCHAR2
  ,p_trigger_method    VARCHAR2 
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_weight            VARCHAR2
  ,p_color             VARCHAR2
  ,p_style             VARCHAR2
  ,p_icon              VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_weight(p_weight);
  self.set_color(p_color);
  self.set_style(p_style);
  self.set_icon(p_icon);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
  
  RETURN;          
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_weight
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('weight', p_value);
END;

MEMBER FUNCTION get_weight RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('weight');
END;

MEMBER PROCEDURE set_color
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('color', p_value);
END;

MEMBER FUNCTION get_color RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('color');
END;

MEMBER PROCEDURE set_style
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('style', p_value);
END;

MEMBER FUNCTION get_style RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('style');
END;

MEMBER PROCEDURE set_icon
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('icon', p_value);
END;

MEMBER FUNCTION get_icon RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('icon');
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
              ("Submit"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,self.get_transient           AS "transient"
                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                  END  AS "enableFormOnServerWait"))
           ,'//Submit[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('submit'                     AS "type"
               ,self.get_id                  AS "id"
               ,self.get_label               AS "label"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf"
               ,self.get_state               AS "state"
               ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                END  AS "enableFormOnServerWait")
             ,CASE WHEN self.get_trigger_name is not null AND 
                        self.get_trigger_method is not null THEN
                XMLElement
                 ("links"
                 ,XMLElement
                   ("trigger"
                   ,XMLElement
                     ("href"
                     ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                     )
                   ,XMLElement
                     ("method"
                     ,self.get_trigger_method
                     )
                   )
                 )
              END
             ,CASE WHEN self.get_weight IS NOT NULL OR
                        self.get_color  IS NOT NULL OR
                        self.get_style  IS NOT NULL OR
                        self.get_icon   IS NOT NULL 
                   THEN
                     XMLElement
                       ("meta"
                       ,XMLElement
                         ("options"
                         ,XMLForest
                           (self.get_weight AS "weight"
                           ,self.get_color  AS "color"
                           ,self.get_style  AS "style"
                           ,self.get_icon   AS "icon")
                         )
                       )
              END
             )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_html_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_html_type
  (p_ui_html_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Html' 
                  PASSING p_ui_html_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name        VARCHAR2(4000) path '@triggerName',
                          trigger_method      VARCHAR2(4000) path '@triggerMethod',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible',
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_html_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2
  ,p_trigger_name      VARCHAR2
  ,p_trigger_method    VARCHAR2
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
   
  RETURN;          
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER FUNCTION get_html_state RETURN VARCHAR2
IS 
BEGIN

  IF nvl(self.get_visible, 'true') = 'false' THEN
    RETURN 'hidden';
  ELSIF nvl(self.get_enabled, 'false') = 'false' THEN
    RETURN 'view';
  ELSE
    RETURN 'input';
  END IF;

END;

OVERRIDING MEMBER PROCEDURE set_value
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('value'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

OVERRIDING MEMBER PROCEDURE set_old_value
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('old_value'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
              ("Html"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,self.get_transient           AS "transient"
                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                  END  AS "enableFormOnServerWait"))
           ,'//Html[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
          ("field"
          ,XMLForest
            ('html'                       AS "type"
            ,self.get_id                  AS "id"
            ,self.get_label               AS "label"
            ,self.get_html_state          AS "state"
            ,self.get_value               AS "value"
          --  ,self.get_old_value           AS "oldValue"
            ,self.get_hide_if             AS "hideIf"
            ,self.get_description         AS "description"
            ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
             END  AS "enableFormOnServerWait")
          ,CASE WHEN self.get_trigger_name is not null AND 
                     self.get_trigger_method is not null THEN
             XMLElement
              ("links"
              ,XMLElement
                ("trigger"
                ,XMLElement
                  ("href"
                  ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                  )
                ,XMLElement
                  ("method"
                  ,self.get_trigger_method
                  )
                )
              )
           END
          ,CASE WHEN self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			        XMLElement("oldValue", null)
			          WHEN self.get_old_value IS NOT NULL THEN
				      XMLElement("oldValue", self.get_old_value)
           END
          )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_item_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_item_type
  (p_ui_item_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Item' 
                  PASSING p_ui_item_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',
                          visible             VARCHAR2(4000) path '@visible',
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          value               VARCHAR2(4000) path '@value',
                          item_function       VARCHAR2(4000) path '@itemFunction',
                          style               VARCHAR2(4000) path '@style',
                          trigger_name        VARCHAR2(4000) path '@triggerName',
                          trigger_method      VARCHAR2(4000) path '@triggerMethod',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_value(l_data.value);
  self.set_item_function(l_data.item_function);
  self.set_style(l_data.style);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled('false');
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_item_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_value             VARCHAR2
  ,p_item_function     VARCHAR2
  ,p_style             VARCHAR2
  ,p_trigger_name      VARCHAR2
  ,p_trigger_method    VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_value(p_value);
  self.set_item_function(p_item_function);
  self.set_style(p_style);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled('false');
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);

  RETURN;          
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_item_function
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('item_function', p_value);
END;

MEMBER FUNCTION get_item_function RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('item_function');
END;

MEMBER PROCEDURE set_style
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('style', p_value);
END;

MEMBER FUNCTION get_style RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('style');
END;

MEMBER FUNCTION get_item_xml RETURN XMLTYPE
IS

  e_parse_err EXCEPTION;
  PRAGMA exception_init(e_parse_err, -31011);
  l_xml XMLTYPE;

BEGIN

  IF self.get_item_function IS NOT NULL THEN
    l_xml := NULL;
      EXECUTE IMMEDIATE
            ' DECLARE '
         || ' BEGIN '
         || ' :xml_output := ' || self.get_item_function
         || '   (p_value => :value); '
         || ' END; '
      USING out l_xml
           ,    self.get_value;

  ELSE
    IF self.get_value IS NOT NULL THEN
      l_xml := XMLType(self.get_value);
    END IF;
  END IF;

  RETURN l_xml;
  
EXCEPTION
  WHEN e_parse_err THEN
   RAISE;

END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
              ("Item"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                  END  AS "enableFormOnServerWait"))
           ,'//Item[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('item'                       AS "type"
               ,self.get_id                  AS "id"
               ,self.get_label               AS "label"
               ,self.get_state               AS "state"
               ,self.get_description         AS "description"
               ,self.get_hide_if             AS "hideIf"
               ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                END  AS "enableFormOnServerWait"
               )
             ,CASE WHEN self.get_trigger_name is not null AND 
                        self.get_trigger_method is not null THEN
                XMLElement
                 ("links"
                 ,XMLElement
                   ("trigger"
                   ,XMLElement
                     ("href"
                     ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                     )
                   ,XMLElement
                     ("method"
                     ,self.get_trigger_method
                     )
                   )
                 )
              END
             ,self.get_item_xml
             ,CASE WHEN self.get_style IS NOT NULL
                   THEN
                     XMLElement
                       ("meta"
                       ,XMLElement
                         ("options"
                         ,XMLForest
                           (lower(self.get_style) AS "style")
                         )
                       )
              END
             )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_listitem_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_listitem_type
  (p_ui_listitem_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/ListItem' 
                  PASSING p_ui_listitem_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',
                          visible             VARCHAR2(4000) path '@visible',
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          value               VARCHAR2(4000) path '@value',
                          style               VARCHAR2(4000) path '@style',
                          list_function       VARCHAR2(4000) path '@listFunction',
                          list_provider       VARCHAR2(4000) path '@listProvider',
                          nodata_message      VARCHAR2(4000) path '@noDataMessage',
                          trigger_name        VARCHAR2(4000) path '@triggerName',
                          trigger_method      VARCHAR2(4000) path '@triggerMethod',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_value(l_data.value);
  self.set_style(l_data.style);
  self.set_list_function(l_data.list_function);
  self.set_list_provider(l_data.list_provider);
  self.set_nodata_message(l_data.nodata_message);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled('false');
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_listitem_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_value             VARCHAR2
  ,p_style             VARCHAR2
  ,p_list_function     VARCHAR2
  ,p_list_provider     VARCHAR2
  ,p_nodata_message    VARCHAR2
  ,p_trigger_name      VARCHAR2
  ,p_trigger_method    VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_value(p_value);
  self.set_style(p_style);
  self.set_list_function(p_list_function);
  self.set_list_provider(p_list_provider);
  self.set_nodata_message(p_nodata_message);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled('false');
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);

  RETURN;          
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;


MEMBER PROCEDURE set_list_function
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('list_function', p_value);
END;

MEMBER FUNCTION get_list_function RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('list_function');
END;

MEMBER PROCEDURE set_list_provider
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('list_provider', p_value);
END;

MEMBER FUNCTION get_list_provider RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('list_provider');
END;

MEMBER FUNCTION get_list_provider_type RETURN VARCHAR2
IS 
  l_provider VARCHAR2(4000);
  l_type     VARCHAR2(400);
  
BEGIN

  l_provider := self.get_attribute('list_provider');
  l_type     := substr(l_provider, 1, instr(l_provider, ':') - 1);

  RETURN l_type;  

END;

MEMBER FUNCTION is_provider_rest RETURN BOOLEAN
IS 
  l_type     VARCHAR2(400);
  
BEGIN

  l_type := self.get_list_provider_type;

  RETURN (l_type = 'REST');  

END;

MEMBER FUNCTION is_provider_rest_flag RETURN VARCHAR2
IS 
  
BEGIN

  IF self.is_provider_rest THEN
    RETURN 'Y';
  ELSE
    RETURN 'N';
  END IF;

END;

MEMBER FUNCTION get_rest_path RETURN VARCHAR2
IS 
  l_provider VARCHAR2(4000);
  l_type     VARCHAR2(400);
  l_value    VARCHAR2(4000);
  
BEGIN

  l_provider := self.get_attribute('list_provider');
  l_type     := substr(l_provider, 1, instr(l_provider, ':') - 1);
  l_value    := substr(l_provider, instr(l_provider, ':') + 1);
  
  IF l_type = 'REST' THEN
    RETURN l_value;
  END IF;
  
  RETURN NULL;

END;

MEMBER FUNCTION get_rest_href RETURN VARCHAR2
IS 

  l_path    VARCHAR2(4000);
  
BEGIN

  l_path := self.get_rest_path;
  IF l_path IS NOT NULL THEN
    RETURN xxas_com_rest_util_pkg.uri_from_path(l_path);  
  END IF;
  
  RETURN NULL;

END;

MEMBER PROCEDURE set_style
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('style', p_value);
END;

MEMBER FUNCTION get_style RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('style');
END;

MEMBER PROCEDURE set_nodata_message
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('nodata_message'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_nodata_message RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('nodata_message');
END;

MEMBER FUNCTION get_items_xml RETURN XMLTYPE
IS

  e_parse_err EXCEPTION;
  PRAGMA exception_init(e_parse_err, -31011);
  l_xml XMLTYPE;

BEGIN

  IF self.get_list_function IS NOT NULL THEN
    l_xml := NULL;
      EXECUTE IMMEDIATE
            ' DECLARE '
         || ' BEGIN '
         || ' :xml_output := ' || self.get_list_function
         || '   (:value); '
         || ' END; '
      USING out l_xml
           ,    self.get_value;
  ELSE
    IF self.get_value IS NOT NULL THEN
      l_xml := XMLType(self.get_value);
    END IF;
  END IF;

  RETURN l_xml;
  
EXCEPTION
  WHEN e_parse_err THEN
   RAISE;

END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
              ("Item"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                  END  AS "enableFormOnServerWait"))
           ,'//Item[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  l_items_xml XMLTYPE := self.get_items_xml;
  l_items_xml_root varchar2(400);

BEGIN

  IF l_items_xml is not null THEN
   l_items_xml_root := l_items_xml.getRootElement();
  END IF;
  
  IF self.is_provider_rest_flag = 'Y' THEN 

    SELECT XMLElement
            ("items"
            ,XMLElement
              ("href"
              ,self.get_rest_href
              )
            )
    INTO l_items_xml
    FROM dual;

  ELSIF l_items_xml_root = 'items' THEN

    null; -- Directly take l_items_xml

  ELSE

    SELECT XMLElement
            ("items"
            ,XMLAttributes
              ('http://json.org/' as "xmlns:json"
              ,'true' as "json:force-array")
            ,l_items_xml)
    INTO l_items_xml
    FROM dual;

  END IF;

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('list'                       AS "type"
               ,self.get_id                  AS "id"
               ,self.get_label               AS "label"
               ,self.get_state               AS "state"
               ,self.get_description         AS "description"
               ,self.get_value               AS "value"
               ,self.get_hide_if             AS "hideIf"
               ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                END  AS "enableFormOnServerWait")
             ,l_items_xml
             ,CASE WHEN self.get_trigger_name is not null AND 
                        self.get_trigger_method is not null THEN
                XMLElement
                 ("links"
                 ,XMLElement
                   ("trigger"
                   ,XMLElement
                     ("href"
                     ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                     )
                   ,XMLElement
                     ("method"
                     ,self.get_trigger_method
                     )
                   )
                 )
              END
             ,CASE WHEN self.get_style IS NOT NULL
                   THEN
                     XMLElement
                       ("meta"
                       ,XMLElement
                         ("options"
                         ,XMLForest
                           (lower(self.get_style) AS "style"
                           ,(CASE WHEN l_items_xml.extract('/items/*') is null AND
                                       self.get_nodata_message is not null
                                  THEN self.get_nodata_message
                                  ELSE null
                             END) "noDataMessage"
                           )
                         )
                       )
              END
             )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_link_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_link_type
  (p_ui_link_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Link' 
                  PASSING p_ui_link_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name        VARCHAR2(4000) path '@triggerName',
                          trigger_method      VARCHAR2(4000) path '@triggerMethod',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible',
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          page_id             VARCHAR2(4000) path '@pageId',
                          object_id           VARCHAR2(4000) path '@objectId',
                          target              VARCHAR2(4000) path '@target',
						  modal_width         VARCHAR2(4000) path '@modalWidth',
													link_text           VARCHAR2(4000) path '@linkText',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait' );            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_page_id(l_data.page_id);
  self.set_object_id(l_data.object_id);
  self.set_target(l_data.target);
  self.set_link_text(l_data.link_text);
  self.set_modal_width(l_data.modal_width);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);	

  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_link_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2
  ,p_trigger_name      VARCHAR2
  ,p_trigger_method    VARCHAR2
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_page_id           VARCHAR2
  ,p_object_id         VARCHAR2
  ,p_target            VARCHAR2
  ,p_modal_width       VARCHAR2
  ,p_link_text         VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_page_id(p_page_id);
  self.set_object_id(p_object_id);
  self.set_target(p_target);
  self.set_modal_width(p_modal_width);
  self.set_link_text(p_link_text);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);

  RETURN;          
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_page_id
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('page_id', p_value);
END;

MEMBER FUNCTION get_page_id RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('page_id');
END;

MEMBER PROCEDURE set_object_id
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('object_id', p_value);
END;

MEMBER FUNCTION get_object_id RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('object_id');
END;

MEMBER PROCEDURE set_target
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('target', p_value);
END;

MEMBER FUNCTION get_target RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('target');
END;

MEMBER PROCEDURE set_link_text
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('link_text', p_value);
END;

MEMBER FUNCTION get_link_text RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('link_text');
END;

MEMBER PROCEDURE set_modal_width
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('modal_width', p_value);
END;

MEMBER FUNCTION get_modal_width RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('modal_width');
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
              ("Link"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,self.get_transient           AS "transient"
								 ,NVL(self.get_link_text,self.get_label) AS "linkText"
                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                  END  AS "enableFormOnServerWait"))
           ,'//Html[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
          ("field"
          ,XMLForest
            ('link'                       AS "type"
            ,self.get_label               AS "label"
            ,self.get_visible             AS "visible"
            ,self.get_state               AS "state"
            ,'LINK'                       AS "operation"
            ,self.get_value               AS "href"
            ,self.get_page_id             AS "pageId"
            ,self.get_object_id           AS "objectId"
            ,self.get_target              AS "target"
            ,self.get_modal_width         AS "modalWidth"
            ,self.get_hide_if             AS "hideIf"
						,NVL(self.get_link_text, self.get_label) AS "linkText"
            ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
             END  AS "enableFormOnServerWait")
          ,CASE WHEN self.get_trigger_name is not null AND 
                     self.get_trigger_method is not null THEN
             XMLElement
              ("links"
              ,XMLElement
                ("trigger"
                ,XMLElement
                  ("href"
                  ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                  )
                ,XMLElement
                  ("method"
                  ,self.get_trigger_method
                  )
                )
              )
            END
          )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_textinput_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_textinput_type
  (p_ui_textinput_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/TextInput' 
                  PASSING p_ui_textinput_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name        VARCHAR2(4000) path '@triggerName',
                          trigger_method      VARCHAR2(4000) path '@triggerMethod',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible', 
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          max_chars           VARCHAR2(4000) path '@maxChars',
                          regexp              VARCHAR2(4000) path '@regexp',
                          regexp_error        VARCHAR2(4000) path '@regexpError',
                          empty_value         VARCHAR2(4000) path '@emptyValue',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
						  autocomplete_hint   VARCHAR2(4000) path '@autocompleteHint',
                          no_autocomplete     VARCHAR2(4000) path '@disableAutocomplete',
                          no_autocorrect      VARCHAR2(4000) path '@disableAutocorrect',
                          no_autocapitalize   VARCHAR2(4000) path '@disableAutocapitalize',
                          no_spell_check      VARCHAR2(4000) path '@disableSpellcheck',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_max_chars(l_data.max_chars);
  self.set_regexp(l_data.regexp);
  self.set_regexp_error(l_data.regexp_error);
  self.set_empty_value(l_data.empty_value);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_autocomplete_hint(l_data.autocomplete_hint);
  self.set_no_autocomplete(l_data.no_autocomplete);
  self.set_no_autocorrect(l_data.no_autocorrect);
  self.set_no_autocapitalize(l_data.no_autocapitalize);
  self.set_no_spell_check(l_data.no_spell_check);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_textinput_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2
  ,p_trigger_name      VARCHAR2
  ,p_trigger_method    VARCHAR2
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2
  ,p_hide_if           VARCHAR2  
  ,p_max_chars         NUMBER
  ,p_regexp            VARCHAR2
  ,p_regexp_error      VARCHAR2
  ,p_empty_value       VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_autocomplete_hint VARCHAR2
  ,p_no_autocomplete   VARCHAR2
  ,p_no_autocorrect    VARCHAR2
  ,p_no_autocapitalize VARCHAR2
  ,p_no_spell_check    VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_max_chars(p_max_chars);
  self.set_regexp(p_regexp);
  self.set_regexp_error(p_regexp_error);
  self.set_empty_value(p_empty_value);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_autocomplete_hint(p_autocomplete_hint);
  self.set_no_autocomplete(p_no_autocomplete);
  self.set_no_autocorrect(p_no_autocorrect);
  self.set_no_autocapitalize(p_no_autocapitalize);
  self.set_no_spell_check(p_no_spell_check);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
   
  RETURN;          
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_max_chars
  (p_value IN NUMBER)
IS 
BEGIN
   self.set_attribute('max_chars', p_value);
END;

MEMBER FUNCTION get_max_chars RETURN NUMBER
IS 
BEGIN
   RETURN self.get_attribute('max_chars');
END;

MEMBER PROCEDURE set_empty_value
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('empty_value'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_empty_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('empty_value');
END;

MEMBER PROCEDURE set_regexp
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('regexp', p_value);
END;

MEMBER FUNCTION get_regexp RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('regexp');
END;

MEMBER PROCEDURE set_regexp_error
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('regexp_error', p_value);
END;

MEMBER FUNCTION get_regexp_error RETURN VARCHAR2
IS
BEGIN
   RETURN self.get_attribute('regexp_error');
END;

MEMBER PROCEDURE set_autocomplete_hint
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('autocomplete_hint', lower(p_value));
END;

MEMBER FUNCTION get_autocomplete_hint RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('autocomplete_hint'));
END;

MEMBER PROCEDURE set_no_autocomplete
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('no_autocomplete', lower(p_value));
END;

MEMBER FUNCTION get_no_autocomplete RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('no_autocomplete'));
END;

MEMBER PROCEDURE set_no_autocorrect
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('no_autocorrect', lower(p_value));
END;

MEMBER FUNCTION get_no_autocorrect RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('no_autocorrect'));
END;

MEMBER PROCEDURE set_no_autocapitalize
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('no_autocapitalize', lower(p_value));
END;

MEMBER FUNCTION get_no_autocapitalize RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('no_autocapitalize'));
END;

MEMBER PROCEDURE set_no_spell_check
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('no_spell_check', lower(p_value));
END;

MEMBER FUNCTION get_no_spell_check RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('no_spell_check'));
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
              ("TextInput"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_max_chars           AS "maxChars"
                 ,self.get_empty_value         AS "emptyValue"
                 ,self.get_value               AS "value"
                 ,self.get_transient           AS "transient"
                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                  END  AS "enableFormOnServerWait"))
           ,'//TextInput[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               (CASE WHEN self.get_max_chars IS NOT NULL AND
                          self.get_max_chars > 150 THEN
                     'textarea'
                ELSE
                     'text'
                END                          AS "type"
               ,self.get_id                  AS "id"
               --,self.get_value               AS "value"
               --,self.get_old_value           AS "oldValue"
               ,self.get_label               AS "label"
               ,self.get_description         AS "description"   
               ,self.get_refresh_on_change   AS "refreshOnChange"
               ,self.get_state               AS "state"
               ,self.get_required            AS "required"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf"
               ,self.get_max_chars           AS "maxlength"
               ,CASE WHEN self.get_max_chars IS NOT NULL THEN
                  replace(xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_MAXLEN')
                         ,'&MAX_LENGTH'
                         ,'{maxLength}')
                END                          AS "maxLengthError"
               ,self.get_regexp              AS "pattern"
               ,CASE WHEN self.get_regexp IS NOT NULL THEN
                  nvl(self.get_regexp_error
                     ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_INVALID'))
                END                          AS "validationError"
               ,self.get_empty_value         AS "placeholder"
			   ,self.get_autocomplete_hint   AS "autocompleteHint"
               ,self.get_no_autocomplete     AS "disableAutocomplete"
               ,self.get_no_autocorrect      AS "disableAutocorrect"
               ,self.get_no_autocapitalize   AS "disableAutocapitalize"
               ,self.get_no_spell_check      AS "disableSpellcheck"
               ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                END  AS "enableFormOnServerWait")
			  ,CASE when self.get_value = xxas_mss_util_pkg.get_g_number THEN
			     XMLElement("value", null)
			   when self.get_value IS NOT NULL THEN
				 XMLElement("value", self.get_value)
               END
              ,CASE when self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			     XMLElement("oldValue", null)
			   when self.get_old_value IS NOT NULL THEN
				 XMLElement("oldValue", self.get_old_value)
               END			   
             ,CASE WHEN self.get_trigger_name is not null AND 
                        self.get_trigger_method is not null THEN
                XMLElement
                 ("links"
                 ,XMLElement
                   ("trigger"
                   ,XMLElement
                     ("href"
                     ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                     )
                   ,XMLElement
                     ("method"
                     ,self.get_trigger_method
                     )
                   )
                 )
              END
             )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_textarea_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_textarea_type
  (p_ui_textarea_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/TextArea' 
                  PASSING p_ui_textarea_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible', 
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          max_chars           VARCHAR2(4000) path '@maxChars',
                          regexp              VARCHAR2(4000) path '@regexp',
                          regexp_error        VARCHAR2(4000) path '@regexpError',
                          empty_value         VARCHAR2(4000) path '@emptyValue',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          rich_text           VARCHAR2(4000) path '@richText',
                          toolbar             VARCHAR2(4000) path '@toolbar',
						  autocomplete_hint   VARCHAR2(4000) path '@autocompleteHint',
                          no_autocomplete     VARCHAR2(4000) path '@disableAutocomplete',
                          no_autocorrect      VARCHAR2(4000) path '@disableAutocorrect',
                          no_autocapitalize   VARCHAR2(4000) path '@disableAutocapitalize',
                          no_spell_check      VARCHAR2(4000) path '@disableSpellcheck',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_max_chars(l_data.max_chars);
  self.set_regexp(l_data.regexp);
  self.set_regexp_error(l_data.regexp_error);
  self.set_empty_value(l_data.empty_value);
  self.set_value(l_data.value);
  self.set_hide_if(l_data.hide_if);
  self.set_transient(l_data.transient);
  self.set_rich_text(l_data.rich_text);
  self.set_toolbar(l_data.toolbar);
  self.set_autocomplete_hint(l_data.autocomplete_hint);
  self.set_no_autocomplete(l_data.no_autocomplete);
  self.set_no_autocorrect(l_data.no_autocorrect);
  self.set_no_autocapitalize(l_data.no_autocapitalize);
  self.set_no_spell_check(l_data.no_spell_check);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_textarea_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2 
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_max_chars         NUMBER
  ,p_regexp            VARCHAR2
  ,p_regexp_error      VARCHAR2
  ,p_empty_value       VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_rich_text         VARCHAR2
  ,p_toolbar           VARCHAR2 DEFAULT NULL
  ,p_autocomplete_hint VARCHAR2
  ,p_no_autocomplete   VARCHAR2
  ,p_no_autocorrect    VARCHAR2
  ,p_no_autocapitalize VARCHAR2
  ,p_no_spell_check    VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_max_chars(p_max_chars);
  self.set_regexp(p_regexp);
  self.set_regexp_error(p_regexp_error);
  self.set_empty_value(p_empty_value);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_rich_text(p_rich_text);
  self.set_toolbar(p_toolbar);
  self.set_autocomplete_hint(p_autocomplete_hint);
  self.set_no_autocomplete(p_no_autocomplete);
  self.set_no_autocorrect(p_no_autocorrect);
  self.set_no_autocapitalize(p_no_autocapitalize);
  self.set_no_spell_check(p_no_spell_check);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
   
  RETURN;          
END;

OVERRIDING MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

OVERRIDING MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_rich_text
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('rich_text'
    ,lower(p_value));
END;

MEMBER FUNCTION get_rich_text RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('rich_text'));
END;

MEMBER PROCEDURE set_toolbar
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('toolbar'
    ,lower(p_value));
END;

MEMBER FUNCTION get_toolbar RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('toolbar'));
END;

MEMBER FUNCTION get_toolbar_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
   SELECT XMLElement
           ("data"
           ,XMLAttributes
             ('true' as "json:force-array"
             ,'http://json.org/' as "xmlns:json")
           ,XMLAgg
              (
                CASE WHEN lower(column_value)='orderedlist'  THEN XMLElement ("item",XMLForest( 'ordered' as "list"))
                     WHEN lower(column_value)='bulletedlist' THEN XMLElement ("item",XMLForest( 'bullet' as "list"))
                     WHEN lower(column_value)='subscript'    THEN XMLElement ("item",XMLForest( 'sub' as "script"))
                     WHEN lower(column_value)='superscript'  THEN XMLElement ("item",XMLForest( 'super' as "script"))
                     WHEN lower(column_value)='directionrtl' THEN XMLElement ("item",XMLForest( 'rtl' as "direction"))
                     WHEN lower(column_value)='leftindent'   THEN XMLElement ("item",XMLForest( '-1' as "indent"))
                     WHEN lower(column_value)='rightindent'  THEN XMLElement ("item",XMLForest( '+1' as "indent"))
                     ELSE XMLElement
                             ("item",lower(column_value))
                END
              )
           ) 
     INTO l_xml
     FROM TABLE(xxas_util_pkg.string_to_varchar2tbl(self.get_toolbar()));
   RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  l_val varchar2(4000);

BEGIN

  --
  -- Only support values up to 4000 characters in length
  --
  IF length(self.get_value) >= 4000 THEN
    NULL; --l_val := '(cannot be displayed because it exceeds 4000 characters)';
  ELSE
    l_val := self.get_value;
  END IF;

  SELECT insertChildXML
           (XMLELEMENT
              ("TextArea"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,l_val                        AS "value"
                 ,self.get_max_chars           AS "maxChars"
                 ,self.get_empty_value         AS "emptyValue"
                 ,self.get_transient           AS "transient"
                 ,self.get_rich_text           AS "richText"
                 ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait"))
           ,'//TextArea[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml     XMLTYPE;
  l_val     clob;    -- varchar2(4000);
  l_old_val varchar2(4000);
BEGIN

  --
  -- If richText is not true then support values up to 4000 characters in length 
  -- 
  IF NVL(self.get_max_chars,0) > 4000 THEN
    l_val := self.get_value;
  ELSE 
    IF length(self.get_value) >= 4000 THEN
      NULL; --l_val := '(cannot be displayed because it exceeds 4000 characters)';
    ELSE
      l_val := self.get_value;
    END IF;
  END IF;

  --
  -- Only support values up to 4000 characters in length
  --
  IF length(self.get_old_value) >= 4000 THEN
    NULL; --l_old_val := '(cannot be displayed because it exceeds 4000 characters)';
  ELSE
    l_old_val := self.get_old_value;
  END IF;

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               (CASE WHEN self.get_rich_text='true' 
                     THEN 'html' 
                     ELSE 'textarea'
                END                          AS "type"
               ,self.get_id                  AS "id"
               ,self.get_clob_value          AS "value"
             --  ,l_old_val                    AS "oldValue"
               ,self.get_label               AS "label"
               ,self.get_description         AS "description"   
               ,self.get_refresh_on_change   AS "refreshOnChange"
               ,self.get_state               AS "state"
               ,self.get_required            AS "required"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf"
               ,self.get_max_chars           AS "maxlength"
               ,CASE WHEN self.get_max_chars IS NOT NULL THEN
                  replace(xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_MAXLEN')
                         ,'&MAX_LENGTH'
                         ,'{maxLength}')
                END                          AS "maxLengthError"
               ,self.get_regexp              AS "pattern"
               ,CASE WHEN self.get_regexp IS NOT NULL THEN
                  nvl(self.get_regexp_error
                     ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_INVALID'))
                END                          AS "validationError"
               ,self.get_empty_value         AS "placeholder"
               ,self.get_rich_text           AS "richText"
               ,CASE WHEN self.get_toolbar IS NOT NULL THEN
                  extract(self.get_toolbar_xml(),'/data/*')
                END                          AS "toolbar"
			   ,self.get_autocomplete_hint   AS "autocompleteHint"
               ,self.get_no_autocomplete     AS "disableAutocomplete"
               ,self.get_no_autocorrect      AS "disableAutocorrect"
               ,self.get_no_autocapitalize   AS "disableAutocapitalize"
               ,self.get_no_spell_check      AS "disableSpellcheck"
               ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait")
               ,CASE when l_old_val = xxas_mss_util_pkg.get_g_number THEN
			          XMLElement("oldValue", null)
			          when l_old_val IS NOT NULL THEN
				        XMLElement("oldValue", l_old_val)
                END)
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_address_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_address_type
  (p_ui_address_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Address' 
                  PASSING p_ui_address_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible', 
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          max_chars           VARCHAR2(4000) path '@maxChars',
                          regexp              VARCHAR2(4000) path '@regexp',
                          regexp_error        VARCHAR2(4000) path '@regexpError',
                          empty_value         VARCHAR2(4000) path '@emptyValue',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          service_name        VARCHAR2(4000) path '@serviceName',
                          google_place_type   VARCHAR2(4000) path '@googlePlaceType',
                          post_to_href        VARCHAR2(4000) path '@postToHref',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_max_chars(l_data.max_chars);
  self.set_regexp(l_data.regexp);
  self.set_regexp_error(l_data.regexp_error);
  self.set_empty_value(l_data.empty_value);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_service_name(l_data.service_name);
  self.set_google_place_type(l_data.google_place_type);
  self.set_post_to_href(l_data.post_to_href);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_address_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2 
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_max_chars         NUMBER
  ,p_regexp            VARCHAR2
  ,p_regexp_error      VARCHAR2
  ,p_empty_value       VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_service_name      VARCHAR2
  ,p_google_place_type VARCHAR2
  ,p_post_to_href      VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_max_chars(p_max_chars);
  self.set_regexp(p_regexp);
  self.set_regexp_error(p_regexp_error);
  self.set_empty_value(p_empty_value);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_service_name(p_service_name);
  self.set_google_place_type(p_google_place_type);
  self.set_post_to_href(p_post_to_href);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
   
  RETURN;          
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN

  SELECT insertChildXML
           (XMLELEMENT
              ("Address"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,self.get_max_chars           AS "maxChars"
                 ,self.get_empty_value         AS "emptyValue"
                 ,self.get_transient           AS "transient"
                 ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait"))
           ,'//Address[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

OVERRIDING MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_service_name
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('service_name'
    ,p_value);
END;

MEMBER FUNCTION get_service_name RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('service_name');
END;

MEMBER PROCEDURE set_google_place_type
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('google_place_type'
    ,p_value);
END;

MEMBER FUNCTION get_google_place_type RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('google_place_type');
END;

MEMBER PROCEDURE set_post_to_href
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('post_to_href'
    ,p_value);
END;

MEMBER FUNCTION get_post_to_href RETURN VARCHAR2
IS 
  l_post_to_href varchar2(4000);
BEGIN
   l_post_to_href := self.get_attribute('post_to_href');
   IF l_post_to_href IS NOT NULL THEN
     IF lower(substr(l_post_to_href, 1, 4)) <> 'http' THEN
       l_post_to_href :=
         xxas_com_rest_util_pkg.uri_from_path(l_post_to_href);
     END IF;     
   END IF;
   RETURN l_post_to_href;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLElement("links"
                        ,XMLElement("postTo"
                                   ,XMLElement("href"
                                              ,self.get_post_to_href
                                              )
                                    )
                        )
             ,XMLForest
               ('location'                   AS "type"
               ,self.get_id                  AS "id"
               ,self.get_value               AS "value"
               ,self.get_old_value           AS "oldValue"
               ,self.get_label               AS "label"
               ,self.get_description         AS "description"   
               ,self.get_refresh_on_change   AS "refreshOnChange"
               ,self.get_state               AS "state"
               ,self.get_required            AS "required"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf"
               ,self.get_max_chars           AS "maxlength"
               ,CASE WHEN self.get_max_chars IS NOT NULL THEN
                  replace(xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_MAXLEN')
                         ,'&MAX_LENGTH'
                         ,'{maxLength}')
                END                          AS "maxLengthError"
               ,self.get_regexp              AS "pattern"
               ,CASE WHEN self.get_regexp IS NOT NULL THEN
                  nvl(self.get_regexp_error
                     ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_INVALID'))
                END                          AS "validationError"
               ,self.get_empty_value         AS "placeholder"
               ,self.get_service_name        AS "serviceName"
               ,self.get_google_place_type   AS "googlePlaceType"
			   ,self.get_autocomplete_hint   AS "autocompleteHint"
               ,self.get_no_autocomplete     AS "disableAutocomplete"
               ,self.get_no_autocorrect      AS "disableAutocorrect"
               ,self.get_no_autocapitalize   AS "disableAutocapitalize"
               ,self.get_no_spell_check      AS "disableSpellcheck"
               ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait"))
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_password_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_password_type
  (p_ui_password_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Password' 
                  PASSING p_ui_password_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible', 
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          max_chars           VARCHAR2(4000) path '@maxChars',
                          regexp              VARCHAR2(4000) path '@regexp',
                          regexp_error        VARCHAR2(4000) path '@regexpError',
                          empty_value         VARCHAR2(4000) path '@emptyValue',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_max_chars(l_data.max_chars);
  self.set_regexp(l_data.regexp);
  self.set_regexp_error(l_data.regexp_error);
  self.set_empty_value(l_data.empty_value);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_password_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2 
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_max_chars         NUMBER
  ,p_regexp            VARCHAR2
  ,p_regexp_error      VARCHAR2
  ,p_empty_value       VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_max_chars(p_max_chars);
  self.set_regexp(p_regexp);
  self.set_regexp_error(p_regexp_error);
  self.set_empty_value(p_empty_value);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
   
  RETURN;          
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN

  SELECT insertChildXML
           (XMLELEMENT
              ("Password"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,self.get_max_chars           AS "maxChars"
                 ,self.get_empty_value         AS "emptyValue"
                 ,self.get_transient           AS "transient"
                 ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait"))
           ,'//Password[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

OVERRIDING MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('password'                   AS "type"
               ,self.get_id                  AS "id"
               ,self.get_value               AS "value"
            --   ,self.get_old_value           AS "oldValue"
               ,self.get_label               AS "label"
               ,self.get_description         AS "description"   
               ,self.get_refresh_on_change   AS "refreshOnChange"
               ,self.get_state               AS "state"
               ,self.get_required            AS "required"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf"
               ,self.get_max_chars           AS "maxlength"
               ,CASE WHEN self.get_max_chars IS NOT NULL THEN
                  replace(xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_MAXLEN')
                         ,'&MAX_LENGTH'
                         ,'{maxLength}')
                END                          AS "maxLengthError"
               ,self.get_regexp              AS "pattern"
               ,CASE WHEN self.get_regexp IS NOT NULL THEN
                  nvl(self.get_regexp_error
                     ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_INVALID'))
                END                          AS "validationError"
               ,self.get_empty_value         AS "placeholder"
			   ,self.get_autocomplete_hint   AS "autocompleteHint"
               ,nvl(self.get_no_autocomplete, 'true')   AS "disableAutocomplete"
               ,nvl(self.get_no_autocorrect, 'true')    AS "disableAutocorrect"
               ,nvl(self.get_no_autocapitalize, 'true') AS "disableAutocapitalize"
               ,nvl(self.get_no_spell_check, 'true')    AS "disableSpellcheck"
               ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait")
               ,CASE WHEN self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			            XMLElement("oldValue", null)
			            WHEN self.get_old_value IS NOT NULL THEN
				          XMLElement("oldValue", self.get_old_value)
                END)
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_color_picker_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_color_picker_type
  (p_ui_color_picker_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/ColorPicker' 
                  PASSING p_ui_color_picker_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible', 
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          max_chars           VARCHAR2(4000) path '@maxChars',
                          regexp              VARCHAR2(4000) path '@regexp',
                          regexp_error        VARCHAR2(4000) path '@regexpError',
                          empty_value         VARCHAR2(4000) path '@emptyValue',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          show_palette        VARCHAR2(4000) path '@showPalette',
                          show_picker         VARCHAR2(4000) path '@showPicker',
                          show_alpha          VARCHAR2(4000) path '@showAlpha',
                          preferred_format    VARCHAR2(4000) path '@preferredFormat',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait'
                          );

  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_max_chars(l_data.max_chars);
  self.set_regexp(l_data.regexp);
  self.set_regexp_error(l_data.regexp_error);
  self.set_empty_value(l_data.empty_value);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_show_palette(l_data.show_palette);
  self.set_show_picker(l_data.show_picker);
  self.set_show_alpha(l_data.show_alpha);
  self.set_preferred_format(l_data.preferred_format);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_color_picker_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2 
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_max_chars         NUMBER
  ,p_regexp            VARCHAR2
  ,p_regexp_error      VARCHAR2
  ,p_empty_value       VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_show_palette      VARCHAR2
  ,p_show_picker       VARCHAR2
  ,p_show_alpha        VARCHAR2
  ,p_preferred_format  VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT

IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_max_chars(p_max_chars);
  self.set_regexp(p_regexp);
  self.set_regexp_error(p_regexp_error);
  self.set_empty_value(p_empty_value);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_show_palette(p_show_palette);
  self.set_show_picker(p_show_picker);
  self.set_show_alpha(p_show_alpha);
  self.set_preferred_format(p_preferred_format);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
  
  RETURN;          
END;

OVERRIDING MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

OVERRIDING MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_show_palette
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('show_color_palette'
    ,lower(p_value));
END;

MEMBER FUNCTION get_show_palette RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('show_color_palette'));
END;

MEMBER PROCEDURE set_show_picker
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('show_color_picker'
    ,lower(p_value));
END;

MEMBER FUNCTION get_show_picker RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('show_color_picker'));
END;

MEMBER PROCEDURE set_show_alpha
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('show_color_alpha'
    ,lower(p_value));
END;

MEMBER FUNCTION get_show_alpha RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('show_color_alpha'));
END;

MEMBER PROCEDURE set_preferred_format
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('color_preferred_format'
    ,p_value);
END;

MEMBER FUNCTION get_preferred_format RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('color_preferred_format');
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN

  SELECT insertChildXML
           (XMLELEMENT
              ("ColorPicker"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,self.get_max_chars           AS "maxChars"
                 ,self.get_empty_value         AS "emptyValue"
                 ,self.get_transient           AS "transient"
                 ,self.get_show_palette        AS "showPalette"
                 ,self.get_show_picker         AS "showPicker"
                 ,self.get_show_alpha          AS "showAlpha"
                 ,self.get_preferred_format    AS "preferredFormat"
                 ,self.get_transient           AS "transient"
                 ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait"
                 ))
           ,'//ColorPicker[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml               XMLTYPE;
  l_palette_only      boolean;
  l_palette_only_flag varchar2(1);

BEGIN

  l_palette_only := (nvl(upper(self.get_show_palette), 'TRUE') = 'TRUE' AND
                     nvl(upper(self.get_show_picker), 'TRUE') = 'FALSE');
  l_palette_only_flag := xxas_util_pkg.boolean_to_flag(l_palette_only);

  SELECT XMLELEMENT
             ("field"
             ,XMLElement
               ("options"
               ,XMLForest
                 (decode(l_palette_only_flag, 'Y', 'false', 'true') "showInput"
                 ,decode(nvl(upper(self.get_required), 'FALSE'), 'TRUE', 'false', 'true') "allowEmpty"
                 ,self.get_show_alpha "showAlpha"
                 ,nvl(self.get_show_palette, 'true') "showPalette"
                 ,decode(l_palette_only_flag, 'Y', 'true', 'false') "showPaletteOnly"
                 ,decode(l_palette_only_flag, 'Y', 'true', 'false') "hideAfterPaletteSelect"
                 ,decode(l_palette_only_flag, 'Y', null, xxas_com_msg_pkg.get_string('XXAS_COM_CHOOSE')) "chooseText"
                 ,decode(l_palette_only_flag, 'Y', null, xxas_com_msg_pkg.get_string('XXAS_RB_COM_CAN')) "cancelText"
                 ,decode(l_palette_only_flag, 'Y', null, nvl(self.get_preferred_format, 'hex')) "preferredFormat"
                 )
               ,(SELECT XMLElement
                         ("palette"
                         ,XMLAgg
                            (XMLElement("color", hrl.description) ORDER BY hrl.description)
                          )
                 FROM     hr_lookups hrl
                 WHERE    hrl.lookup_type = 'XXAS_COM_COLOR_PALETTE')
               )
             ,XMLForest
               ('color-picker'               AS "type"
               ,self.get_id                  AS "id"
               ,self.get_value               AS "value"
           --    ,self.get_old_value           AS "oldValue"
               ,self.get_label               AS "label"
               ,self.get_description         AS "description"   
               ,self.get_refresh_on_change   AS "refreshOnChange"
               ,self.get_state               AS "state"
               ,self.get_required            AS "required"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf"
               ,self.get_max_chars           AS "maxlength"
               ,CASE WHEN self.get_max_chars IS NOT NULL THEN
                  replace(xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_MAXLEN')
                         ,'&MAX_LENGTH'
                         ,'{maxLength}')
                END                          AS "maxLengthError"
               ,self.get_regexp              AS "pattern"
               ,CASE WHEN self.get_regexp IS NOT NULL THEN
                  nvl(self.get_regexp_error
                     ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_INVALID'))
                END                          AS "validationError"
               ,self.get_empty_value         AS "placeholder"
			   ,self.get_autocomplete_hint   AS "autocompleteHint"
               ,nvl(self.get_no_autocomplete, 'true')   AS "disableAutocomplete"
               ,nvl(self.get_no_autocorrect, 'true')    AS "disableAutocorrect"
               ,nvl(self.get_no_autocapitalize, 'true') AS "disableAutocapitalize"
               ,nvl(self.get_no_spell_check, 'true')    AS "disableSpellcheck"
               ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait")
               ,CASE WHEN self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			            XMLElement("oldValue", null)
			            WHEN self.get_old_value IS NOT NULL THEN
				          XMLElement("oldValue", self.get_old_value)
                END)
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_icon_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_icon_type
  (p_ui_icon_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Icon' 
                  PASSING p_ui_icon_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible', 
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          max_chars           VARCHAR2(4000) path '@maxChars',
                          regexp              VARCHAR2(4000) path '@regexp',
                          regexp_error        VARCHAR2(4000) path '@regexpError',
                          empty_value         VARCHAR2(4000) path '@emptyValue',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_max_chars(l_data.max_chars);
  self.set_regexp(l_data.regexp);
  self.set_regexp_error(l_data.regexp_error);
  self.set_empty_value(l_data.empty_value);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_icon_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2 
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_max_chars         NUMBER
  ,p_regexp            VARCHAR2
  ,p_regexp_error      VARCHAR2
  ,p_empty_value       VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_max_chars(p_max_chars);
  self.set_regexp(p_regexp);
  self.set_regexp_error(p_regexp_error);
  self.set_empty_value(p_empty_value);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
   
  RETURN;          
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN

  SELECT insertChildXML
           (XMLELEMENT
              ("Icon"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,self.get_max_chars           AS "maxChars"
                 ,self.get_empty_value         AS "emptyValue"
                 ,self.get_transient           AS "transient"
                 ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait"))
           ,'//Icon[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

OVERRIDING MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('icon'                       AS "type"
               ,self.get_id                  AS "id"
               ,self.get_value               AS "value"
            --   ,self.get_old_value           AS "oldValue"
               ,self.get_label               AS "label"
               ,self.get_description         AS "description"   
               ,self.get_refresh_on_change   AS "refreshOnChange"
               ,self.get_state               AS "state"
               ,self.get_required            AS "required"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf"
               ,self.get_max_chars           AS "maxlength"
               ,CASE WHEN self.get_max_chars IS NOT NULL THEN
                  replace(xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_MAXLEN')
                         ,'&MAX_LENGTH'
                         ,'{maxLength}')
                END                          AS "maxLengthError"
               ,self.get_regexp              AS "pattern"
               ,CASE WHEN self.get_regexp IS NOT NULL THEN
                  nvl(self.get_regexp_error
                     ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_INVALID'))
                END                          AS "validationError"
               ,self.get_empty_value         AS "placeholder"
			   ,self.get_autocomplete_hint   AS "autocompleteHint"
               ,nvl(self.get_no_autocomplete, 'true')   AS "disableAutocomplete"
               ,nvl(self.get_no_autocorrect, 'true')    AS "disableAutocorrect"
               ,nvl(self.get_no_autocapitalize, 'true') AS "disableAutocapitalize"
               ,nvl(self.get_no_spell_check, 'true')    AS "disableSpellcheck"
               ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait")
               ,CASE WHEN self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			            XMLElement("oldValue", null)
			            WHEN self.get_old_value IS NOT NULL THEN
				          XMLElement("oldValue", self.get_old_value)
                END)
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_email_type IS

CONSTRUCTOR FUNCTION xxas_emp_ui_email_type
  (p_ui_email_xml IN XMLTYPE) RETURN SELF AS RESULT
IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Email' 
                  PASSING p_ui_email_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible', 
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          max_chars           VARCHAR2(4000) path '@maxChars',
                          regexp              VARCHAR2(4000) path '@regexp',
                          regexp_error        VARCHAR2(4000) path '@regexpError',
                          empty_value         VARCHAR2(4000) path '@emptyValue',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          disableVerification VARCHAR2(4000) path '@disableVerification',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');
  l_data c_data%ROWTYPE;
  
BEGIN
   
  OPEN  c_data;
  FETCH c_data INTO l_data;  
  CLOSE c_data;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_max_chars(l_data.max_chars);
  self.set_regexp(l_data.regexp);
  self.set_regexp_error(l_data.regexp_error);
  self.set_empty_value(l_data.empty_value);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_disableVerification(l_data.disableVerification);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  RETURN;
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_email_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2 
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_max_chars         NUMBER
  ,p_regexp            VARCHAR2
  ,p_regexp_error      VARCHAR2
  ,p_empty_value       VARCHAR2
  ,p_value             VARCHAR2
  ,p_transient         VARCHAR2
  ,p_disableVerification VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_max_chars(p_max_chars);
  self.set_regexp(p_regexp);
  self.set_regexp_error(p_regexp_error);
  self.set_empty_value(p_empty_value);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_disableVerification(p_disableVerification);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
  RETURN;          
END;

OVERRIDING MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

OVERRIDING MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_disableVerification
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('disableVerification'
    ,lower(p_value));
END;

MEMBER FUNCTION get_disableVerification RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('disableVerification'));
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN

  SELECT insertChildXML
           (XMLELEMENT
              ("Email"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_enabled             AS "enabled"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_value               AS "value"
                 ,self.get_max_chars           AS "maxChars"
                 ,self.get_empty_value         AS "emptyValue"
                 ,self.get_transient           AS "transient"
                 ,self.get_disableVerification AS "disableVerification"
                 ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait"))
           ,'//Email[not(@value)]'
           ,'@value'
           ,'')                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('email'                      AS "type"
               ,self.get_id                  AS "id"
               ,self.get_value               AS "value"
            --   ,self.get_old_value           AS "oldValue"
               ,self.get_label               AS "label"
               ,self.get_description         AS "description"   
               ,self.get_refresh_on_change   AS "refreshOnChange"
               ,self.get_state               AS "state"
               ,self.get_required            AS "required"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf"
               ,self.get_max_chars           AS "maxlength"
               ,CASE WHEN self.get_max_chars IS NOT NULL THEN
                  replace(xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_MAXLEN')
                         ,'&MAX_LENGTH'
                         ,'{maxLength}')
                END                          AS "maxLengthError"
               ,replace(xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_EMAIL_SUG')
                       ,'&SUGGESTED_EMAIL'
                       ,'{alternativeEmail}')
                                             AS "emailSuggestionError"
               ,self.get_regexp              AS "pattern"
               ,CASE WHEN self.get_regexp IS NOT NULL THEN
                  nvl(self.get_regexp_error
                     ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_INVALID'))
                END                          AS "validationError"
               ,self.get_empty_value         AS "placeholder"
			   ,self.get_autocomplete_hint   AS "autocompleteHint"
               ,self.get_no_autocomplete     AS "disableAutocomplete"
               ,self.get_no_autocorrect      AS "disableAutocorrect"
               ,nvl(self.get_no_autocapitalize, 'true')    AS "disableAutocapitalize"          
               ,nvl(self.get_no_spell_check, 'true')       AS "disableSpellcheck"
               ,nvl(self.get_disableVerification, 'false') AS "disableVerification"
               ,self.get_enable_form_on_server_wait AS "enableFormOnServerWait")
               ,CASE WHEN self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			            XMLElement("oldValue", null)
			            WHEN self.get_old_value IS NOT NULL THEN
				          XMLElement("oldValue", self.get_old_value)
                END)
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_attachment_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_attachment_type
  (p_ui_attachment_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Attachment' 
                  PASSING p_ui_attachment_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name        VARCHAR2(4000) path '@triggerName',
                          trigger_method      VARCHAR2(4000) path '@triggerMethod',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible',
                          value               VARCHAR2(4000) path '@value',
                          transient           VARCHAR2(4000) path '@transient',
                          empty_value         VARCHAR2(4000) path '@emptyValue',
                          max_attachments     VARCHAR2(4000) path '@maxAttachments',
                          attachment_id_field VARCHAR2(4000) path '@attachmentIdField',
                          entity_name         VARCHAR2(4000) path '@entityName',
                          pk1_field           VARCHAR2(4000) path '@pk1Field',
                          pk2_field           VARCHAR2(4000) path '@pk2Field',
                          pk3_field           VARCHAR2(4000) path '@pk3Field',
                          pk4_field           VARCHAR2(4000) path '@pk4Field',
                          pk5_field           VARCHAR2(4000) path '@pk5Field',
                          pk1_value           VARCHAR2(4000) path '@pk1Value',
                          pk2_value           VARCHAR2(4000) path '@pk2Value',
                          pk3_value           VARCHAR2(4000) path '@pk3Value',
                          pk4_value           VARCHAR2(4000) path '@pk4Value',
                          pk5_value           VARCHAR2(4000) path '@pk5Value',
                          category_code       VARCHAR2(4000) path '@categoryCode',
                          title               VARCHAR2(4000) path '@title',
                          attach_description  VARCHAR2(4000) path '@attachDescription',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
  
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_empty_value(l_data.empty_value);
  self.set_max_attachments(l_data.max_attachments);
  self.set_attachment_id_field(l_data.attachment_id_field);
  self.set_entity_name(l_data.entity_name);
  self.set_pk1_field(l_data.pk1_field);
  self.set_pk2_field(l_data.pk2_field);
  self.set_pk3_field(l_data.pk3_field);
  self.set_pk4_field(l_data.pk4_field);
  self.set_pk5_field(l_data.pk5_field);
  self.set_pk1_value(l_data.pk1_value);
  self.set_pk2_value(l_data.pk2_value);
  self.set_pk3_value(l_data.pk3_value);
  self.set_pk4_value(l_data.pk4_value);
  self.set_pk5_value(l_data.pk5_value);
  self.set_category_code(l_data.category_code);
  self.set_title(l_data.title);
  self.set_attach_description(l_data.attach_description);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);

  RETURN; 
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_attachment_type
  (p_id                  VARCHAR2
  ,p_label               VARCHAR2
  ,p_description         VARCHAR2    
  ,p_refresh_on_change   VARCHAR2
  ,p_trigger_name        VARCHAR2
  ,p_trigger_method      VARCHAR2
  ,p_enabled             VARCHAR2
  ,p_required            VARCHAR2
  ,p_visible             VARCHAR2 
  ,p_value               VARCHAR2
  ,p_transient           VARCHAR2
  ,p_empty_value         VARCHAR2
  ,p_max_attachments     NUMBER
  ,p_attachment_id_field VARCHAR2
  ,p_entity_name         VARCHAR2
  ,p_pk1_field           VARCHAR2
  ,p_pk2_field           VARCHAR2
  ,p_pk3_field           VARCHAR2
  ,p_pk4_field           VARCHAR2
  ,p_pk5_field           VARCHAR2
  ,p_pk1_value           VARCHAR2
  ,p_pk2_value           VARCHAR2
  ,p_pk3_value           VARCHAR2
  ,p_pk4_value           VARCHAR2
  ,p_pk5_value           VARCHAR2
  ,p_category_code       VARCHAR2
  ,p_title               VARCHAR2
  ,p_attach_description  VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_empty_value(p_empty_value);
  self.set_max_attachments(p_max_attachments);
  self.set_attachment_id_field(p_attachment_id_field);
  self.set_entity_name(p_entity_name);
  self.set_pk1_field(p_pk1_field);
  self.set_pk2_field(p_pk2_field);
  self.set_pk3_field(p_pk3_field);
  self.set_pk4_field(p_pk4_field);
  self.set_pk5_field(p_pk5_field);
  self.set_pk1_value(p_pk1_value);
  self.set_pk2_value(p_pk2_value);
  self.set_pk3_value(p_pk3_value);
  self.set_pk4_value(p_pk4_value);
  self.set_pk5_value(p_pk5_value);
  self.set_category_code(p_category_code);
  self.set_title(p_title);
  self.set_attach_description(p_attach_description);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
  
  RETURN;          
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_empty_value
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('empty_value'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_empty_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('empty_value');
END;

MEMBER PROCEDURE set_max_attachments
  (p_value IN NUMBER)
IS 
BEGIN
   self.set_attribute('max_attachments', p_value);
END;

MEMBER FUNCTION get_max_attachments RETURN NUMBER
IS 
BEGIN
   RETURN self.get_attribute('max_attachments');
END;

MEMBER PROCEDURE set_attachment_id_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('attachment_id_field', p_value);
END;

MEMBER FUNCTION get_attachment_id_field RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('attachment_id_field');
END;

MEMBER PROCEDURE set_entity_name
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('entity_name', p_value);
END;

MEMBER FUNCTION get_entity_name RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('entity_name');
END;

MEMBER PROCEDURE set_pk1_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk1_field', p_value);
END;

MEMBER FUNCTION get_pk1_field RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk1_field');
END;

MEMBER PROCEDURE set_pk2_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk2_field', p_value);
END;

MEMBER FUNCTION get_pk2_field RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk2_field');
END;

MEMBER PROCEDURE set_pk3_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk3_field', p_value);
END;

MEMBER FUNCTION get_pk3_field RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk3_field');
END;

MEMBER PROCEDURE set_pk4_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk4_field', p_value);
END;

MEMBER FUNCTION get_pk4_field RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk4_field');
END;

MEMBER PROCEDURE set_pk5_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk5_field', p_value);
END;

MEMBER FUNCTION get_pk5_field RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk5_field');
END;

MEMBER PROCEDURE set_pk1_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk1_value', p_value);
END;

MEMBER FUNCTION get_pk1_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk1_value');
END;

MEMBER PROCEDURE set_pk2_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk2_value', p_value);
END;

MEMBER FUNCTION get_pk2_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk2_value');
END;

MEMBER PROCEDURE set_pk3_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk3_value', p_value);
END;

MEMBER FUNCTION get_pk3_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk3_value');
END;

MEMBER PROCEDURE set_pk4_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk4_value', p_value);
END;

MEMBER FUNCTION get_pk4_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk4_value');
END;

MEMBER PROCEDURE set_pk5_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('pk5_value', p_value);
END;

MEMBER FUNCTION get_pk5_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('pk5_value');
END;

MEMBER PROCEDURE set_category_code
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('category_code', p_value);
END;

MEMBER FUNCTION get_category_code RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('category_code');
END;

MEMBER PROCEDURE set_title
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('title'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_title RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('title');
END;

MEMBER PROCEDURE set_attach_description
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('attach_description'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_attach_description RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('attach_description');
END;

MEMBER PROCEDURE set_existing_files
  (p_files_tbl IN xxas_com_num_table)
IS

BEGIN

  self.files_tbl := p_files_tbl;

END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
             ("Attachment"
             ,XMLATTRIBUTES
                (self.get_id                  AS "id"
                ,self.get_label               AS "label"
                ,self.get_description         AS "description"  
                ,self.get_refresh_on_change   AS "refreshOnChange"
                ,self.get_enabled             AS "enabled"
                ,self.get_required            AS "required"
                ,self.get_visible             AS "visible"
                ,self.get_value               AS "value"
                ,self.get_transient           AS "transient"
                ,self.get_empty_value         AS "emptyValue"
                ,self.get_max_attachments     AS "maxAttachments"
                ,self.get_entity_name         AS "entityName"
                ,self.get_pk1_field           AS "pk1Field"
                ,self.get_pk2_field           AS "pk2Field"
                ,self.get_pk3_field           AS "pk3Field"
                ,self.get_pk4_field           AS "pk4Field"
                ,self.get_pk5_field           AS "pk5Field"
                ,self.get_pk1_value           AS "pk1Value"
                ,self.get_pk2_value           AS "pk2Value"
                ,self.get_pk3_value           AS "pk3Value"
                ,self.get_pk4_value           AS "pk4Value"
                ,self.get_pk5_value           AS "pk5Value"
                ,self.get_category_code       AS "categoryCode"
                ,self.get_title               AS "title"
                ,self.get_attach_description  AS "attachDescription"
                ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                 END  AS "enableFormOnServerWait"))
           ,'//Attachment[not(@value)]'
           ,'@value'
           ,'')                            
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_container         xxas_emp_ui_container_type;
  l_multirow          xxas_emp_ui_multirow_type;
  l_is_first          boolean := true;
  l_need_single_field boolean;
  l_first_file_id     number;
  
  CURSOR csr_attachments
    (cp_attachment_data_xml IN XMLType)
  IS
  SELECT *
  FROM  XMLTable
          ('//Attachment'
          PASSING (cp_attachment_data_xml)
          COLUMNS attached_document_id    NUMBER         path 'AttachedDocumentId'
                 ,sequence_number         VARCHAR2(4000) path 'SequenceNumber'
                 ,entity_name             VARCHAR2(4000) path 'EntityName'
                 ,pk1_value               VARCHAR2(4000) path 'Pk1Value'
                 ,pk2_value               VARCHAR2(4000) path 'Pk2Value'
                 ,pk3_value               VARCHAR2(4000) path 'Pk3Value'
                 ,pk4_value               VARCHAR2(4000) path 'Pk4Value'
                 ,pk5_value               VARCHAR2(4000) path 'Pk5Value'
                 ,category_code           VARCHAR2(4000) path 'CategoryCode'
                 ,category_name           VARCHAR2(4000) path 'CategoryName'
                 ,document_id             NUMBER         path 'DocumentId'
                 ,file_name               VARCHAR2(4000) path 'FileName'
                 ,title                   VARCHAR2(4000) path 'Title'
                 ,description             VARCHAR2(4000) path 'Description'
                 ,file_id                 NUMBER         path 'FileId'
                 ,mime_type               VARCHAR2(4000) path 'MimeType'
                 ,lob_format              VARCHAR2(4000) path 'LobFormat'
                 ,file_url                VARCHAR2(4000) path 'FileUrl'
                 ,creation_date           DATE           path 'CreationDate'
                 ,created_by              VARCHAR2(4000) path 'CreatedBy'
                 ,last_update_date        DATE           path 'LastUpdateDate'
                 ,last_updated_by         VARCHAR2(4000) path 'LastUpdatedBy'
           ) a;

  FUNCTION create_file_field
    (p_file_id IN number
    ,p_required IN varchar2) RETURN xxas_emp_ui_file_type
  IS
    l_new_file xxas_emp_ui_file_type;
  BEGIN
    l_new_file := xxas_emp_ui_file_type
        (p_id                 => 'FILE_ID'
        ,p_label              => xxas_com_msg_pkg.get_string('XXAS_EMP_FILE')
        ,p_description        => NULL
        ,p_refresh_on_change  => 'false'
        ,p_trigger_name       => self.get_trigger_name
        ,p_trigger_method     => self.get_trigger_method
        ,p_enabled            => self.get_enabled
        ,p_required           => p_required
        ,p_visible            => self.get_visible
        ,p_hide_if            => NULL
        ,p_value              => p_file_id
        ,p_empty_value        => NULL
        ,p_file_info_provider => NULL
        ,p_button_text        => xxas_com_msg_pkg.get_string('XXAS_EMP_CHOOSE_FILE')
        ,p_file_types         => NULL
        ,p_post_new_to_href   => NULL);
    RETURN l_new_file;
  END create_file_field;

  FUNCTION create_attachment_row
    (p_file_id IN number) RETURN xxas_emp_ui_view_type
  IS
    l_new_row      xxas_emp_ui_view_type;
    l_createfields xxas_emp_ui_createfields_type;
  BEGIN
    l_createfields := xxas_emp_ui_createfields_type(null);
    l_createfields.field_list.EXTEND;
    l_createfields.field_list
      (l_createfields.field_list.COUNT) :=
        create_file_field(p_file_id,'true');
            
    l_new_row := xxas_emp_ui_view_type(null);
    l_new_row.set_delete_allowed('true');
    l_new_row.createfields := l_createfields;
    
    RETURN l_new_row;
  END create_attachment_row;
           
BEGIN

  IF lower(self.get_visible) = 'false' OR
     lower(self.get_enabled) = 'false' THEN
    RETURN null;
  END IF;

  --
  -- Convert legacy "Attachment" fields to newer multirow fields
  --
  l_container :=
    xxas_emp_ui_container_type
      (p_id             => 'ATTACHMENTS_GROUP'
      ,p_label          => self.get_label
      ,p_description    => self.get_description
      ,p_visible        => 'true'
      ,p_hide_if        => NULL
      ,p_review_only    => 'false'
      ,p_type           => NULL
      ,p_trigger_name   => self.get_trigger_name
      ,p_trigger_method => self.get_trigger_method);
      
  l_multirow :=
    xxas_emp_ui_multirow_type
      (p_id                         => 'ATTACHMENTS'
      ,p_label                      => NULL
      ,p_description                => NULL
      ,p_visible                    => 'true'
      ,p_hide_if                    => NULL
      ,p_create_allowed             => self.get_enabled
      ,p_create_label               => NULL
      ,p_min_rows                   => NULL
	  ,p_min_rows_error_message       => NULL
	  ,p_max_rows_error_message       => NULL
      ,p_max_rows                   => NULL
      ,p_ui_xdb_uri                 => NULL
      ,p_formula_name               => NULL
      ,p_data_callback              => NULL
      ,p_include_empty_row          => 'false'
      ,p_trigger_name               => self.get_trigger_name
      ,p_trigger_method             => self.get_trigger_method
      ,p_to_be_in_single_row        => NULL
      ,p_force_single_row           => NULL);

  l_multirow.new_row := create_attachment_row(null);

  --
  -- If attachments are required or only one is allowed
  -- show a single file field outside the multirow
  --
  l_need_single_field :=
    (lower(self.get_required) = 'true' OR
     nvl(self.get_max_attachments, 2) = 1);

  IF self.files_tbl IS NOT NULL AND
     self.files_tbl.count > 0
  THEN
    FOR i IN self.files_tbl.first..self.files_tbl.last LOOP
      IF l_is_first AND l_need_single_field THEN
        l_first_file_id := self.files_tbl(i);
      ELSE
        l_multirow.add_row(create_attachment_row(self.files_tbl(i)));      
      END IF;
      l_is_first := false;
    END LOOP;  
  END IF;    

  --
  -- Add in the entity and category
  --
  l_container.field_list.EXTEND;
  l_container.field_list(l_container.field_list.COUNT) := 
    xxas_emp_ui_hidden_type
      (p_id        => 'ENTITY_NAME'
      ,p_value     => self.get_entity_name
      ,p_transient => NULL);
  
  l_container.field_list.EXTEND;
  l_container.field_list(l_container.field_list.COUNT) := 
    xxas_emp_ui_hidden_type
      (p_id        => 'CATEGORY_CODE'
      ,p_value     => self.get_category_code
      ,p_transient => NULL);
      
  IF self.get_title IS NOT NULL THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := 
      xxas_emp_ui_hidden_type
        (p_id        => 'TITLE'
        ,p_value     => self.get_title
        ,p_transient => NULL);  
  END IF;
  
  IF self.get_attach_description IS NOT NULL THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := 
      xxas_emp_ui_hidden_type
        (p_id        => 'ATTACH_DESCRIPTION'
        ,p_value     => self.get_attach_description
        ,p_transient => NULL);  
  END IF;

  IF self.get_pk1_field IS NOT NULL THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := 
      xxas_emp_ui_hidden_type
        (p_id        => 'PK1_FIELD'
        ,p_value     => self.get_pk1_field
        ,p_transient => NULL);  
  END IF;
  
  IF self.get_pk2_field IS NOT NULL THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := 
      xxas_emp_ui_hidden_type
        (p_id        => 'PK2_FIELD'
        ,p_value     => self.get_pk2_field
        ,p_transient => NULL);  
  END IF;
  
  IF self.get_pk3_field IS NOT NULL THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := 
      xxas_emp_ui_hidden_type
        (p_id        => 'PK3_FIELD'
        ,p_value     => self.get_pk3_field
        ,p_transient => NULL);  
  END IF;
  
  IF self.get_pk4_field IS NOT NULL THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := 
      xxas_emp_ui_hidden_type
        (p_id        => 'PK4_FIELD'
        ,p_value     => self.get_pk4_field
        ,p_transient => NULL);  
  END IF;
  
  IF self.get_pk5_field IS NOT NULL THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := 
      xxas_emp_ui_hidden_type
        (p_id        => 'PK5_FIELD'
        ,p_value     => self.get_pk5_field
        ,p_transient => NULL);  
  END IF;
  
  IF l_need_single_field AND nvl(self.get_max_attachments, 2) <> 1 THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := 
      create_file_field(l_first_file_id,'true');
  ELSIF l_need_single_field AND nvl(self.get_max_attachments, 2) = 1 THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := 
      create_file_field(l_first_file_id,'false');
  END IF;
  
  IF nvl(self.get_max_attachments, 2) > 1 THEN
    l_container.field_list.EXTEND;
    l_container.field_list(l_container.field_list.COUNT) := l_multirow;
  END IF;

  RETURN l_container.get_json_friendly;

END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_file_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_file_type
  (p_ui_file_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/File' 
                  PASSING p_ui_file_xml
                  COLUMNS id                 VARCHAR2(4000) path '@id',
                          label              VARCHAR2(4000) path '@label',
                          description        VARCHAR2(4000) path '@description',   
                          refresh_on_change  VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name       VARCHAR2(4000) path '@triggerName',
                          trigger_method     VARCHAR2(4000) path '@triggerMethod',
                          enabled            VARCHAR2(4000) path '@enabled',
                          required           VARCHAR2(4000) path '@required',
                          visible            VARCHAR2(4000) path '@visible',
                          hide_if            VARCHAR2(4000) path '@hideIf',
                          value              VARCHAR2(4000) path '@value',
                          transient          VARCHAR2(4000) path '@transient',
                          empty_value        VARCHAR2(4000) path '@emptyValue',
                          file_info_provider VARCHAR2(4000) path '@fileInfoProvider',
                          button_text        VARCHAR2(4000) path '@buttonText',
                          file_types         VARCHAR2(4000) path '@fileTypes',
                          post_new_to_href   VARCHAR2(4000) path '@postNewToHref',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
  
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_empty_value(l_data.empty_value);
  self.set_file_info_provider(l_data.file_info_provider);
  self.set_button_text(l_data.button_text);
  self.set_file_types(l_data.file_types);
  self.set_post_new_to_href(l_data.post_new_to_href);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
        
  set_file_info;
  
  RETURN; 
END;
    
CONSTRUCTOR FUNCTION xxas_emp_ui_file_type
  (p_id                  VARCHAR2
  ,p_label               VARCHAR2
  ,p_description         VARCHAR2    
  ,p_refresh_on_change   VARCHAR2
  ,p_trigger_name        VARCHAR2
  ,p_trigger_method      VARCHAR2
  ,p_enabled             VARCHAR2
  ,p_required            VARCHAR2 
  ,p_visible             VARCHAR2
  ,p_hide_if             VARCHAR2
  ,p_value               VARCHAR2
  ,p_empty_value         VARCHAR2
  ,p_file_info_provider  VARCHAR2
  ,p_button_text         VARCHAR2
  ,p_file_types          VARCHAR2
  ,p_post_new_to_href    VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
IS
  
BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_value(p_value);
  self.set_empty_value(p_empty_value);
  self.set_file_info_provider(p_file_info_provider);
  self.set_button_text(p_button_text);
  self.set_file_types(p_file_types);
  self.set_post_new_to_href(p_post_new_to_href);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
  set_file_info;
  
  RETURN; 
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

OVERRIDING MEMBER PROCEDURE set_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('value', p_value);
   set_file_info;
END;

MEMBER PROCEDURE set_empty_value
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('empty_value'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_empty_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('empty_value');
END;

MEMBER PROCEDURE set_file_info_provider
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('file_info_provider', p_value);
END;

MEMBER FUNCTION get_file_info_provider RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('file_info_provider');
END;

MEMBER PROCEDURE set_button_text
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('button_text'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_button_text RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('button_text');
END;

MEMBER PROCEDURE set_file_types
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('file_types'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_file_types RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('file_types');
END;

MEMBER PROCEDURE set_file_info
IS 

  CURSOR c_file_info
    (cp_file_id IN NUMBER)
  IS
  SELECT fl.file_name
        ,xxas_com_rest_util_pkg.uri_from_path
          ('/documents/' || fl.file_id)
        ,fnd_gfm.construct_download_url
           (fnd_web_config.gfm_agent, fl.file_id)
        ,fl.file_content_type as fileMimeType
  FROM   fnd_lobs fl
  WHERE  fl.file_id = cp_file_id;

BEGIN

  --
  -- Auto-fetch file info from cursor
  --
  IF self.get_value IS NOT NULL THEN
    IF self.get_file_info_provider IS NULL THEN
      OPEN  c_file_info(self.get_value);
      FETCH c_file_info INTO file_name
                            ,self_href
                            ,file_href
                            ,fileMimeType;
      CLOSE c_file_info;
    ELSE
      EXECUTE IMMEDIATE
            ' DECLARE '
         || ' BEGIN '
         || self.get_file_info_provider
         || ' (p_file_id       => :file_id '
         || ' ,p_file_name     => :file_name '
         || ' ,p_self_href     => :self_href '
         || ' ,p_file_href     => :file_href ); '
         || ' END; '
      USING     self.get_value
           ,OUT file_name
           ,OUT self_href
           ,OUT file_href;
    END IF;
  END IF;

END;

MEMBER PROCEDURE set_post_new_to_href
  (p_value IN VARCHAR2)
IS 
BEGIN
    self.set_attribute('post_new_to_href', p_value);
END;

MEMBER FUNCTION get_post_new_to_href RETURN VARCHAR2
IS 
BEGIN
  RETURN xxas_com_rest_util_pkg.uri_from_path
           (nvl
             (self.get_attribute('post_new_to_href')
             ,'documents'));
END;

MEMBER FUNCTION get_post_new_to_href2 RETURN VARCHAR2
IS 
BEGIN
  RETURN xxas_com_rest_util_pkg.uri_from_path
           (nvl
             (self.get_attribute('post_new_to_href')
             ,'documents?fileName={fileName}&fileMimeType={fileMimeType}'));
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
             ("File"
             ,XMLATTRIBUTES
                (self.get_id                  AS "id"
                ,self.get_label               AS "label"
                ,self.get_description         AS "description"  
                ,self.get_refresh_on_change   AS "refreshOnChange"
                ,self.get_enabled             AS "enabled"
                ,self.get_required            AS "required"
                ,self.get_visible             AS "visible"
                ,self.get_value               AS "value"
                ,self.get_transient           AS "transient"
                ,self.get_empty_value         AS "emptyValue"
                ,self.file_name               AS "fileName"
                ,self.self_href               AS "self"
                ,self.file_href               AS "documentHref"
                ,self.get_post_new_to_href    AS "postNewToHref"
                ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                 END  AS "enableFormOnServerWait"))
           ,'//File[not(@value)]'
           ,'@value'
           ,'')
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('upload'                     AS "type"
               ,self.get_id                  AS "id"
               ,self.get_label               AS "label"
               ,self.get_description         AS "description"  
               ,self.get_button_text         AS "buttonText"
               ,self.get_refresh_on_change   AS "refreshOnChange"
               ,self.get_file_types          AS "fileTypes"
               ,self.get_state               AS "state"
               ,self.get_required            AS "required"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf"
               ,self.get_value               AS "value"
            --   ,self.get_old_value           AS "oldValue"
               ,self.file_name               AS "fileName"
               ,self.file_href               AS "fileHref"
               ,self.fileMimeType            AS "fileMimeType"
               ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                END  AS "enableFormOnServerWait")
             ,XMLElement
               ("operations"
               ,XMLElement
                 ("POST"
                 ,XMLElement
                   ("href"
                   ,self.get_post_new_to_href2)
                 )
               )
             ,CASE WHEN self.get_trigger_name is not null AND 
                        self.get_trigger_method is not null THEN
                XMLElement
                 ("links"
                 ,XMLElement
                   ("trigger"
                   ,XMLElement
                     ("href"
                     ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                     )
                   ,XMLElement
                     ("method"
                     ,self.get_trigger_method
                     )
                   )
                 )
              END
             ,CASE WHEN self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			            XMLElement("oldValue", null)
			            WHEN self.get_old_value IS NOT NULL THEN
				          XMLElement("oldValue", self.get_old_value)
              END
             )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_number_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_number_type
  (p_ui_number_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Number' 
                  PASSING p_ui_number_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name        VARCHAR2(4000) path '@triggerName',
                          trigger_method      VARCHAR2(4000) path '@triggerMethod',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible',
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          value               VARCHAR2(4000) path '@value',
                          decimal_precision   NUMBER         path '@decimalPrecision',
                          regexp              VARCHAR2(4000) path '@regexp',
                          regexp_error        VARCHAR2(4000) path '@regexpError',
                          transient           VARCHAR2(4000) path '@transient',
						  number_format       VARCHAR2(4000) path '@numberFormat',
              enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait'
);            
  l_data c_data%ROWTYPE;
  
BEGIN
  
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_value(l_data.value);
  self.set_number_format(l_data.number_format);
  self.set_decimal_precision(l_data.decimal_precision);
  self.set_regexp(l_data.regexp);
  self.set_regexp_error(l_data.regexp_error);
  self.set_transient(l_data.transient);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  RETURN; 
END;
  
CONSTRUCTOR FUNCTION xxas_emp_ui_number_type
  (p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2
  ,p_trigger_name      VARCHAR2
  ,p_trigger_method    VARCHAR2 
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2
  ,p_visible           VARCHAR2 
  ,p_hide_if           VARCHAR2
  ,p_value             VARCHAR2
  ,p_decimal_precision NUMBER
  ,p_regexp            VARCHAR2
  ,p_regexp_error      VARCHAR2
  ,p_transient         VARCHAR2
  ,p_number_format     VARCHAR2 DEFAULT NULL
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_value(p_value);
  self.set_number_format(p_number_format);
  self.set_decimal_precision(p_decimal_precision);
  self.set_regexp(p_regexp);
  self.set_regexp_error(p_regexp_error);
  self.set_transient(p_transient);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
   
  RETURN;          
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_number_format
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('number_format',p_value);
END;

MEMBER FUNCTION get_number_format RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('number_format');
END;

MEMBER PROCEDURE set_decimal_precision
  (p_value IN NUMBER)
IS 
BEGIN
   self.set_attribute('decimal_precision', p_value);
END;

MEMBER FUNCTION get_decimal_precision RETURN NUMBER
IS 
BEGIN
   RETURN nvl(self.get_attribute('decimal_precision'), 0);
END;

MEMBER PROCEDURE set_regexp
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('regexp', p_value);
END;

MEMBER FUNCTION get_regexp RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('regexp');
END;

MEMBER PROCEDURE set_regexp_error
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('regexp_error', p_value);
END;

MEMBER FUNCTION get_regexp_error RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('regexp_error');
END;

MEMBER FUNCTION get_number_value RETURN VARCHAR2
IS

  l_number number;

BEGIN

   IF self.get_value IS NULL THEN
     RETURN NULL;
   END IF;
   
   --
   -- First attempt to convert numbers implictly (no mask) and then
   -- canonical numbers
   --
   BEGIN
     l_number := to_number(self.get_value);
   EXCEPTION
     WHEN others THEN
       BEGIN
         l_number := fnd_number.canonical_to_number(self.get_value);
       EXCEPTION
         WHEN others THEN
           NULL;
       END;
   END;

   IF l_number IS NULL THEN
     RETURN NULL;
   ELSE
     RETURN fnd_number.number_to_canonical(l_number);
   END IF;   

END;

MEMBER FUNCTION get_number_old_value RETURN VARCHAR2
IS

  l_number number;

BEGIN

   IF self.get_old_value IS NULL THEN
     RETURN NULL;
   END IF;
   
   --
   -- First attempt to convert numbers implictly (no mask) and then
   -- canonical numbers
   --
   BEGIN
     l_number := to_number(self.get_old_value);
   EXCEPTION
     WHEN others THEN
       BEGIN
         l_number := fnd_number.canonical_to_number(self.get_old_value);
       EXCEPTION
         WHEN others THEN
           NULL;
       END;
   END;

   IF l_number IS NULL THEN
     RETURN NULL;
   ELSE
     RETURN fnd_number.number_to_canonical(l_number);
   END IF;   

END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT insertChildXML
           (XMLELEMENT
             ("Number"
             ,XMLATTRIBUTES
                (self.get_id                  AS "id"
                ,self.get_label               AS "label"
                ,self.get_description         AS "description"  
                ,self.get_refresh_on_change   AS "refreshOnChange"
                ,self.get_enabled             AS "enabled"
                ,self.get_required            AS "required"
                ,self.get_visible             AS "visible"
                ,self.get_number_value        AS "value"
                ,self.get_decimal_precision   AS "decimalPrecision"
                ,self.get_transient           AS "transient"
                ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                END  AS "enableFormOnServerWait"))
           ,'//Number[not(@value)]'
           ,'@value'
           ,'')                            
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('number'                       AS "type"
               ,self.get_id                    AS "id"
            --   ,self.get_number_value        AS "value"
            --   ,self.get_number_old_value    AS "oldValue"
               ,self.get_label                 AS "label"
               ,self.get_description           AS "description"
			   ,xxas_com_mls_pkg.get_number_mask
                  (self.get_number_format)     AS "numberFormat"
               ,self.get_refresh_on_change     AS "refreshOnChange"
               ,self.get_state                 AS "state"
               ,self.get_required              AS "required"
               ,self.get_visible               AS "visible"
               ,self.get_hide_if               AS "hideIf"
               ,self.get_decimal_precision     AS "decimalPrecision"
               ,self.get_regexp                AS "pattern"               
               ,CASE WHEN self.get_regexp IS NOT NULL THEN
                  nvl(self.get_regexp_error
                     ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_FIELD_INVALID'))
                END                            AS "validationError"
               ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                END  AS "enableFormOnServerWait"
               )
			 ,CASE WHEN self.get_value = xxas_mss_util_pkg.get_g_number THEN
			         XMLElement("value", null)
			       WHEN self.get_value IS NOT NULL THEN
			         XMLElement("value", self.get_value)
              END
             ,CASE WHEN self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			         XMLElement("oldValue", null)
			       WHEN self.get_old_value IS NOT NULL THEN 
			         XMLElement("oldValue", self.get_old_value)
              END	
             ,CASE WHEN self.get_trigger_name is not null AND 
                        self.get_trigger_method is not null THEN
                XMLElement
                 ("links"
                 ,XMLElement
                   ("trigger"
                   ,XMLElement
                     ("href"
                     ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                     )
                   ,XMLElement
                     ("method"
                     ,self.get_trigger_method
                     )
                   )
                 )
              END
             )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_date_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_date_type
  (p_ui_date_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/*' 
                  PASSING p_ui_date_xml
                  COLUMNS id                  VARCHAR2(4000) path '@id',
                          label               VARCHAR2(4000) path '@label',
                          description         VARCHAR2(4000) path '@description',   
                          refresh_on_change   VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name        VARCHAR2(4000) path '@triggerName',
                          trigger_method      VARCHAR2(4000) path '@triggerMethod',
                          enabled             VARCHAR2(4000) path '@enabled',
                          required            VARCHAR2(4000) path '@required',
                          visible             VARCHAR2(4000) path '@visible',
                          hide_if             VARCHAR2(4000) path '@hideIf',
                          value               VARCHAR2(4000) path '@value',
                          date_format         VARCHAR2(4000) path '@dateFormat',
                          transient           VARCHAR2(4000) path '@transient',
                          post_format         VARCHAR2(4000) path '@postFormat',
                          min                 VARCHAR2(4000) path '@min',
                          max                 VARCHAR2(4000) path '@max',
                          disable_days        VARCHAR2(4000) path '@disableDays',
                          disable_dates       VARCHAR2(4000) path '@disableDates',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
  
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  SELECT CASE WHEN existsNode(p_ui_date_xml, 'Date')     = 1
              THEN 'Date'
              WHEN existsNode(p_ui_date_xml, 'DateTime') = 1 
              THEN 'DateTime' 
              WHEN existsNode(p_ui_date_xml, 'Time')     = 1
              THEN 'Time'
         END
  INTO   date_type
  FROM   dual;
  
  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_value(l_data.value);
  self.set_date_format(l_data.date_format);
  self.set_transient(l_data.transient);
  self.set_post_format(l_data.post_format);
  self.set_min(l_data.min);
  self.set_max(l_data.max);
  self.set_disable_days(l_data.disable_days);
  self.set_disable_dates(l_data.disable_dates);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);

  RETURN; 
END;
  

CONSTRUCTOR FUNCTION xxas_emp_ui_date_type
  (p_date_type         VARCHAR2
  ,p_id                VARCHAR2
  ,p_label             VARCHAR2
  ,p_description       VARCHAR2    
  ,p_refresh_on_change VARCHAR2
  ,p_trigger_name      VARCHAR2
  ,p_trigger_method    VARCHAR2 
  ,p_enabled           VARCHAR2
  ,p_required          VARCHAR2 
  ,p_visible           VARCHAR2
  ,p_hide_if           VARCHAR2
  ,p_value             VARCHAR2
  ,p_date_format       VARCHAR2
  ,p_transient         VARCHAR2
  ,p_post_format       VARCHAR2
  ,p_min               VARCHAR2 DEFAULT NULL
  ,p_max               VARCHAR2 DEFAULT NULL
  ,p_disable_days      VARCHAR2 DEFAULT NULL
  ,p_disable_dates     VARCHAR2 DEFAULT NULL
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
IS

BEGIN

  date_type := p_date_type;

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_value(p_value);
  self.set_date_format(p_date_format);
  self.set_transient(p_transient);
  self.set_post_format(p_post_format);
  self.set_min(p_min);
  self.set_max(p_max);  
  self.set_disable_days(p_disable_days);
  self.set_disable_dates(p_disable_dates);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);

  RETURN;          

END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_date_format
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('date_format'
    ,p_value);
END;

MEMBER FUNCTION get_date_format RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('date_format');
END;

MEMBER PROCEDURE set_post_format
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('post_format'
    ,p_value);
END;

MEMBER FUNCTION get_post_format RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('post_format');
END;

MEMBER FUNCTION get_formatted_value RETURN VARCHAR2
IS

  l_date date;

BEGIN

   IF self.get_value IS NULL THEN
     RETURN NULL;
   END IF;
   
   BEGIN
     l_date := fnd_date.canonical_to_date(self.get_value);
   EXCEPTION
     WHEN others THEN
       NULL;
   END;
   
   IF l_date IS NULL THEN
     RETURN NULL;
   END IF;
   
   IF date_type = 'Date' THEN
     RETURN to_char(l_date, nvl(self.get_date_format, 'FMDd Month YYYY'));
   ELSIF date_type = 'DateTime' THEN
     RETURN to_char(l_date, nvl(self.get_date_format, 'FMDd Month YYYY HH24:MI'));
   ELSIF date_type = 'Time' THEN
     RETURN to_char(l_date, nvl(self.get_date_format, 'HH24:MI')); 
   END IF;     

END;

MEMBER FUNCTION get_date_value(p_is_html5 IN varchar2 DEFAULT 'N') RETURN VARCHAR2
IS

  l_date date;

BEGIN

   IF self.get_value IS NULL THEN
     RETURN NULL;
   END IF;
   
   IF self.get_value = xxas_mss_util_pkg.get_g_number THEN
     RETURN xxas_mss_util_pkg.get_g_number;
   END IF;
   
   --
   -- First attempt to convert canonical dates and then
   -- dates implicitly converted to varchar2
   --
   BEGIN
     l_date := to_date(self.get_value);
   EXCEPTION
     WHEN others THEN
       BEGIN
         l_date := fnd_date.canonical_to_date(self.get_value);
       EXCEPTION
         WHEN others THEN
           NULL;
       END;
   END;

   IF l_date IS NULL THEN 
     --
     -- Times are handled as canonical date/times or just time
     -- varchar2s
     --     
     IF date_type = 'Time' THEN
       RETURN self.get_value;
     ELSE
       RETURN NULL;
     END IF;
   END IF;
   
   IF date_type = 'Date' THEN
     IF p_is_html5 = 'Y' THEN
       IF self.get_post_format is not null THEN
         RETURN to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
       ELSE
         RETURN to_char(l_date, 'YYYY-MM-DD');
       END IF;
     ELSE
       RETURN to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
     END IF;
   ELSIF date_type = 'DateTime' THEN
     IF p_is_html5 = 'Y' THEN
       IF self.get_post_format is not null THEN
         RETURN to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
       ELSE
         RETURN to_char(l_date, 'YYYY-MM-DD HH24:MI:SS');
       END IF;
     ELSE
       RETURN to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
     END IF;
   ELSIF date_type = 'Time' THEN
     RETURN to_char(l_date, 'HH24:MI'); 
   END IF;     

END;

MEMBER FUNCTION get_date_old_value(p_is_html5 IN varchar2 DEFAULT 'N') RETURN VARCHAR2
IS

  l_date date;

BEGIN

   IF self.get_old_value IS NULL THEN
     RETURN NULL;
   END IF;
   
   IF self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
     RETURN xxas_mss_util_pkg.get_g_number;
   END IF;
   
   --
   -- First attempt to convert canonical dates and then
   -- dates implicitly converted to varchar2
   --
   BEGIN
     l_date := to_date(self.get_old_value);
   EXCEPTION
     WHEN others THEN
       BEGIN
         l_date := fnd_date.canonical_to_date(self.get_old_value);
       EXCEPTION
         WHEN others THEN
           NULL;
       END;
   END;

   IF l_date IS NULL THEN 
     --
     -- Times are handled as canonical date/times or just time
     -- varchar2s
     --     
     IF date_type = 'Time' THEN
       RETURN self.get_old_value;
     ELSE
       RETURN NULL;
     END IF;
   END IF;
   
   IF date_type = 'Date' THEN
     IF p_is_html5 = 'Y'THEN
       IF self.get_post_format is not null THEN
         RETURN to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
       ELSE
         RETURN to_char(l_date, 'YYYY-MM-DD');
       END IF;
     ELSE
       RETURN to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
     END IF;
   ELSIF date_type = 'DateTime' THEN
     IF p_is_html5 = 'Y' THEN
       IF self.get_post_format is not null THEN
         RETURN to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
       ELSE
         RETURN to_char(l_date, 'YYYY-MM-DD HH24:MI:SS');
       END IF;
     ELSE
       RETURN to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
     END IF;
   ELSIF date_type = 'Time' THEN
     RETURN to_char(l_date, 'HH24:MI'); 
   END IF;     

END;

MEMBER PROCEDURE set_min
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('min'
    ,p_value);
END;

MEMBER FUNCTION get_min RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('min');
END;

MEMBER PROCEDURE set_max
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('max'
    ,p_value);
END;

MEMBER FUNCTION get_max RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('max');
END;

MEMBER PROCEDURE set_disable_days
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('disableDays'
    ,lower(p_value));
END;

MEMBER FUNCTION get_disable_days RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('disableDays'));
END;

MEMBER FUNCTION get_disable_days_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
   SELECT XMLElement
           ("days"
           ,XMLAttributes
             ('true' as "json:force-array"
             ,'http://json.org/' as "xmlns:json")
           ,XMLAgg
              (
                XMLElement("item",lower(column_value))
              )
           ) 
     INTO l_xml
     FROM TABLE(xxas_util_pkg.string_to_varchar2tbl(self.get_disable_days()));
   RETURN l_xml;
END;

MEMBER PROCEDURE set_disable_dates
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('disableDates'
    ,lower(p_value));
END;

MEMBER FUNCTION get_disable_dates RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('disableDates'));
END;

MEMBER FUNCTION get_disable_dates_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
   SELECT XMLElement
           ("dates"
           ,XMLAttributes
             ('true' as "json:force-array"
             ,'http://json.org/' as "xmlns:json")
           ,XMLAgg
              (
                XMLElement("item",lower(column_value))
              )
           ) 
     INTO l_xml
     FROM TABLE(xxas_util_pkg.string_to_varchar2tbl(self.get_disable_dates()));
   RETURN l_xml;
END;

MEMBER FUNCTION get_disable_days_dates_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
   SELECT XMLElement
           ("disabledDate"           
           ,CASE WHEN self.get_disable_days IS NOT NULL THEN self.get_disable_days_xml()
            END
           ,CASE WHEN self.get_disable_dates IS NOT NULL THEN self.get_disable_dates_xml()
            END
           ) 
     INTO l_xml
     FROM DUAL;
   RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT CASE WHEN date_type = 'Date'
              THEN insertChildXML
                     (XMLELEMENT
                       ("Date"
                       ,XMLATTRIBUTES
                          (self.get_id                  AS "id"
                          ,self.get_label               AS "label"
                          ,self.get_description         AS "description"   
                          ,self.get_refresh_on_change   AS "refreshOnChange"
                          ,self.get_enabled             AS "enabled"
                          ,self.get_required            AS "required"
                          ,self.get_visible             AS "visible"
                          ,self.get_date_value          AS "value"
                          ,self.get_formatted_value     AS "formattedValue"
                          ,self.get_transient           AS "transient"
                          ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                      nvl(self.get_enable_form_on_server_wait,'true')
                                ELSE
                                      self.get_enable_form_on_server_wait
                           END  AS "enableFormOnServerWait"))
                     ,'//Date[not(@value)]'
                     ,'@value'
                     ,'')
              WHEN date_type = 'DateTime'
              THEN insertChildXML
                     (XMLELEMENT
                       ("DateTime"
                       ,XMLATTRIBUTES
                          (self.get_id                  AS "id"
                          ,self.get_label               AS "label"
                          ,self.get_description         AS "description" 
                          ,self.get_refresh_on_change   AS "refreshOnChange"
                          ,self.get_enabled             AS "enabled"
                          ,self.get_required            AS "required"
                          ,self.get_visible             AS "visible"
                          ,self.get_date_value          AS "value"
                          ,self.get_formatted_value     AS "formattedValue"
                          ,self.get_transient           AS "transient"
                          ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                      nvl(self.get_enable_form_on_server_wait,'true')
                                ELSE
                                      self.get_enable_form_on_server_wait
                           END  AS "enableFormOnServerWait"))
                     ,'//DateTime[not(@value)]'
                     ,'@value'
                     ,'')
              WHEN date_type = 'Time'
              THEN insertChildXML
                     (XMLELEMENT
                       ("Time"
                       ,XMLATTRIBUTES
                          (self.get_id                  AS "id"
                          ,self.get_label               AS "label"
                          ,self.get_description         AS "description"   
                          ,self.get_refresh_on_change   AS "refreshOnChange"
                          ,self.get_enabled             AS "enabled"
                          ,self.get_required            AS "required"
                          ,self.get_visible             AS "visible"
                          ,self.get_date_value          AS "value"
                          ,self.get_formatted_value     AS "formattedValue"
                          ,self.get_transient           AS "transient"
                          ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                      nvl(self.get_enable_form_on_server_wait,'true')
                                ELSE
                                      self.get_enable_form_on_server_wait
                           END  AS "enableFormOnServerWait"))
                     ,'//Time[not(@value)]'
                     ,'@value'
                     ,'')
         END                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  l_time varchar2(10) := 'HH:MM';

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               (CASE WHEN date_type = 'Date'
                     THEN 'date'
                     WHEN date_type = 'DateTime'
                     THEN 'datetime-local'
                     WHEN date_type = 'Time'
                     THEN 'time'
                END                                AS "type"
               ,self.get_id                        AS "id"
               --,self.get_date_value('Y')           AS "value"
               --,self.get_date_old_value('Y')       AS "oldValue"
               ,self.get_label                     AS "label"
               ,self.get_description               AS "description"  
               ,xxas_com_mls_pkg.get_iso_date_mask
                  (self.get_date_format)           AS "dateFormat"
               ,self.get_post_format               AS "postFormat"
               ,CASE WHEN (date_type = 'Date' OR date_type = 'DateTime')
                     THEN xxas_com_mls_pkg.get_date_mask_tip
							(self.get_date_format)
                     WHEN date_type = 'Time'
                     THEN l_time
                END 				               AS "placeholder"
               ,self.get_refresh_on_change         AS "refreshOnChange"
               ,self.get_state                     AS "state"
               ,self.get_required                  AS "required"
               ,self.get_visible                   AS "visible"
               ,self.get_hide_if                   AS "hideIf"
               ,self.get_min                       AS "min"
               ,self.get_max                       AS "max"
               ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                      nvl(self.get_enable_form_on_server_wait,'true')
                                ELSE
                                      self.get_enable_form_on_server_wait
                           END  AS "enableFormOnServerWait")
			 ,CASE when self.get_date_value('Y') = xxas_mss_util_pkg.get_g_number THEN
			     XMLElement("value", null)
			   when self.get_date_value('Y') IS NOT NULL THEN
				 XMLElement("value", self.get_date_value('Y'))
               END
              ,CASE when self.get_date_old_value('Y') = xxas_mss_util_pkg.get_g_number THEN
			     XMLElement("oldValue", null)
			   when self.get_date_old_value('Y') IS NOT NULL THEN
				 XMLElement("oldValue", self.get_date_old_value('Y'))
               END
             ,CASE WHEN date_type = 'Date' THEN
                xxas_com_mls_pkg.calendars_xml
              END
             ,self.get_disable_days_dates_xml() AS "disableDates" 
             ,CASE WHEN self.get_trigger_name is not null AND 
                        self.get_trigger_method is not null THEN
                XMLElement
                 ("links"
                 ,XMLElement
                   ("trigger"
                   ,XMLElement
                     ("href"
                     ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                     )
                   ,XMLElement
                     ("method"
                     ,self.get_trigger_method
                     )
                   )
                 )
              END
             )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_emp_ui_date_range_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_date_range_type
  (p_ui_date_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/*' 
                  PASSING p_ui_date_xml
                  COLUMNS id                       VARCHAR2(4000)          path '@id',
                          label                    VARCHAR2(4000)          path '@label',
                          description              VARCHAR2(4000)          path '@description',
                          refresh_on_change        VARCHAR2(4000)          path '@refreshOnChange',
                          trigger_name             VARCHAR2(4000)          path '@triggerName',
                          trigger_method           VARCHAR2(4000)          path '@triggerMethod',
                          enabled                  VARCHAR2(4000)          path '@enabled',
                          required                 VARCHAR2(4000)          path '@required',
                          visible                  VARCHAR2(4000)          path '@visible',
                          hide_if                  VARCHAR2(4000)          path '@hideIf',
                          value                    VARCHAR2(4000)          path '@value',
                          start_date_description   VARCHAR2(4000)          path '@startDateDescription',
                          start_date_format        VARCHAR2(4000)          path '@startDateFormat',
                          start_date_label         VARCHAR2(4000)          path '@startDateLabel',
                          start_date_required      VARCHAR2(4000)          path '@startDateRequired',
                          start_date_visible       VARCHAR2(4000)          path '@startDateVisible',
                          end_date_description     VARCHAR2(4000)          path '@endDateDescription',
                          end_date_format          VARCHAR2(4000)          path '@endDateFormat',
                          end_date_label           VARCHAR2(4000)          path '@endDateLabel',
                          end_date_required        VARCHAR2(4000)          path '@endDateRequired',
                          end_date_visible         VARCHAR2(4000)          path '@endDateVisible',
                          start_time_description   VARCHAR2(4000)          path '@startTimeDescription',
                          start_time_label         VARCHAR2(4000)          path '@startTimeLabel',
                          start_time_required      VARCHAR2(4000)          path '@startTimeRequired',
                          start_time_visible       VARCHAR2(4000)          path '@startTimeVisible',
                          end_time_description     VARCHAR2(4000)          path '@endTimeDescription',
                          end_time_label           VARCHAR2(4000)          path '@endTimeLabel',
                          end_time_required        VARCHAR2(4000)          path '@endTimeRequired',
                          end_time_visible         VARCHAR2(4000)          path '@endTimeVisible',
                          start_date_min           VARCHAR2(4000)          path '@startDateMin',
                          start_date_max           VARCHAR2(4000)          path '@startDateMax',
                          start_date_disable_days  VARCHAR2(4000)          path '@startDateDisableDays',
                          start_date_disable_dates VARCHAR2(4000)          path '@startDateDisableDates',
                          end_date_min             VARCHAR2(4000)          path '@endDateMin',
                          end_date_max             VARCHAR2(4000)          path '@endDateMax',
                          end_date_disable_days    VARCHAR2(4000)          path '@endDateDisableDays',
                          end_date_disable_dates   VARCHAR2(4000)          path '@endDateDisableDates',
                          start_date_enabled       VARCHAR2(4000)          path '@startDateEnabled',
                          end_date_enabled         VARCHAR2(4000)          path '@endDateEnabled',
                          start_time_enabled       VARCHAR2(4000)          path '@startTimeEnabled',
                          end_time_enabled         VARCHAR2(4000)          path '@endTimeEnabled',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait'
                          );            
  l_data c_data%ROWTYPE;

BEGIN

  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;

  SELECT CASE WHEN existsNode(p_ui_date_xml, 'DateRange')     = 1
              THEN 'DateRange'
              WHEN existsNode(p_ui_date_xml, 'DateTimeRange') = 1 
              THEN 'DateTimeRange' 
              WHEN existsNode(p_ui_date_xml, 'TimeRange')     = 1
              THEN 'TimeRange'
         END
  INTO   date_range_type
  FROM   dual;

  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_value(l_data.value);
  self.set_start_date_description(l_data.start_date_description);
  self.set_start_date_format(l_data.start_date_format);
  self.set_start_date_label(l_data.start_date_label);
  self.set_start_date_required(l_data.start_date_required);
  self.set_start_date_visible(l_data.start_date_visible);
  self.set_end_date_description(l_data.end_date_description);
  self.set_end_date_format(l_data.end_date_format);
  self.set_end_date_label(l_data.end_date_label);
  self.set_end_date_required(l_data.end_date_required);
  self.set_end_date_visible(l_data.end_date_visible);
  self.set_start_time_description(l_data.start_time_description);
  self.set_start_time_label(l_data.start_time_label);
  self.set_start_time_required(l_data.start_time_required);
  self.set_start_time_visible(l_data.start_time_visible);
  self.set_end_time_description(l_data.end_time_description);
  self.set_end_time_label(l_data.end_time_label);
  self.set_end_time_required(l_data.end_time_required);
  self.set_end_time_visible(l_data.end_time_visible);
  self.set_start_date_min(l_data.start_date_min);
  self.set_start_date_max(l_data.start_date_max);
  self.set_start_date_disable_days(l_data.start_date_disable_days);
  self.set_start_date_disable_dates(l_data.start_date_disable_dates);
  self.set_end_date_min(l_data.end_date_min);
  self.set_end_date_max(l_data.end_date_max);
  self.set_end_date_disable_days(l_data.end_date_disable_days);
  self.set_end_date_disable_dates(l_data.end_date_disable_dates);
  self.set_start_date_enabled(l_data.start_date_enabled);
  self.set_end_date_enabled(l_data.end_date_enabled);
  self.set_start_time_enabled(l_data.start_time_enabled);
  self.set_end_time_enabled(l_data.end_time_enabled);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);

  RETURN;

END;

CONSTRUCTOR FUNCTION xxas_emp_ui_date_range_type
  (p_date_range_type          VARCHAR2
  ,p_id                       VARCHAR2
  ,p_label                    VARCHAR2
  ,p_description              VARCHAR2
  ,p_refresh_on_change        VARCHAR2
  ,p_trigger_name             VARCHAR2
  ,p_trigger_method           VARCHAR2
  ,p_enabled                  VARCHAR2
  ,p_required                 VARCHAR2
  ,p_visible                  VARCHAR2
  ,p_hide_if                  VARCHAR2
  ,p_value                    VARCHAR2
  ,p_start_date_description   VARCHAR2
  ,p_start_date_format        VARCHAR2
  ,p_start_date_label         VARCHAR2
  ,p_start_date_required      VARCHAR2
  ,p_start_date_visible       VARCHAR2
  ,p_end_date_description     VARCHAR2
  ,p_end_date_format          VARCHAR2
  ,p_end_date_label           VARCHAR2
  ,p_end_date_required        VARCHAR2
  ,p_end_date_visible         VARCHAR2
  ,p_start_time_description   VARCHAR2
  ,p_start_time_label         VARCHAR2
  ,p_start_time_required      VARCHAR2
  ,p_start_time_visible       VARCHAR2
  ,p_end_time_description     VARCHAR2
  ,p_end_time_label           VARCHAR2
  ,p_end_time_required        VARCHAR2
  ,p_end_time_visible         VARCHAR2
  ,p_start_date_min           VARCHAR2 DEFAULT NULL
  ,p_start_date_max           VARCHAR2 DEFAULT NULL
  ,p_start_date_disable_days  VARCHAR2 DEFAULT NULL
  ,p_start_date_disable_dates VARCHAR2 DEFAULT NULL
  ,p_end_date_min             VARCHAR2 DEFAULT NULL
  ,p_end_date_max             VARCHAR2 DEFAULT NULL
  ,p_end_date_disable_days    VARCHAR2 DEFAULT NULL
  ,p_end_date_disable_dates   VARCHAR2 DEFAULT NULL
  ,p_start_date_enabled       VARCHAR2 DEFAULT NULL
  ,p_end_date_enabled         VARCHAR2 DEFAULT NULL
  ,p_start_time_enabled       VARCHAR2 DEFAULT NULL
  ,p_end_time_enabled         VARCHAR2 DEFAULT NULL
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
IS

BEGIN

  date_range_type := p_date_range_type;

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_value(p_value);
  self.set_start_date_description(p_start_date_description);
  self.set_start_date_format(p_start_date_format);
  self.set_start_date_label(p_start_date_label);
  self.set_start_date_required(p_start_date_required);
  self.set_start_date_visible(p_start_date_visible);
  self.set_end_date_description(p_end_date_description);
  self.set_end_date_format(p_end_date_format);
  self.set_end_date_label(p_end_date_label);
  self.set_end_date_required(p_end_date_required);
  self.set_end_date_visible(p_end_date_visible);
  self.set_start_time_description(p_start_time_description);
  self.set_start_time_label(p_start_time_label);
  self.set_start_time_required(p_start_time_required);
  self.set_start_time_visible(p_start_time_visible);
  self.set_end_time_description(p_end_time_description);
  self.set_end_time_label(p_end_time_label);
  self.set_end_time_required(p_end_time_required);
  self.set_end_time_visible(p_end_time_visible);
  self.set_start_date_min(p_start_date_min);
  self.set_start_date_max(p_start_date_max);
  self.set_start_date_disable_days(p_start_date_disable_days);
  self.set_start_date_disable_dates(p_start_date_disable_dates);
  self.set_end_date_min(p_end_date_min);
  self.set_end_date_max(p_end_date_max);
  self.set_end_date_disable_days(p_end_date_disable_days);
  self.set_end_date_disable_dates(p_end_date_disable_dates);
  self.set_start_date_enabled(p_start_date_enabled);
  self.set_end_date_enabled(p_end_date_enabled);
  self.set_start_time_enabled(p_start_time_enabled);
  self.set_end_time_enabled(p_end_time_enabled);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);

  RETURN;          

END;

OVERRIDING MEMBER PROCEDURE set_enabled
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('enabled', lower(p_value));
   set_start_date_enabled(p_value);
   set_end_date_enabled(p_value);
   set_start_time_enabled(p_value);
   set_end_time_enabled(p_value);
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_start_date_description
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('start_date_description'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_start_date_description RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('start_date_description');
END;

MEMBER PROCEDURE set_start_date_format
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('start_date_format', p_value);
END;

MEMBER FUNCTION get_start_date_format RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('start_date_format');
END;

MEMBER PROCEDURE set_start_date_label
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('start_date_label'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_start_date_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('start_date_label');
END;

MEMBER PROCEDURE set_start_date_required
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('start_date_required', lower(p_value));
END;

MEMBER FUNCTION get_start_date_required RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('start_date_required'));
END;

MEMBER PROCEDURE set_start_date_visible
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('start_date_visible', lower(p_value));
END;

MEMBER FUNCTION get_start_date_visible RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('start_date_visible'));
END;

MEMBER FUNCTION get_start_date_state RETURN VARCHAR2
IS 
BEGIN

  IF nvl(self.get_start_date_visible, 'true') = 'false' THEN
    RETURN 'hidden';
  ELSIF nvl(self.get_start_date_enabled, 'true') = 'false' THEN
    RETURN 'view';
  ELSE
    RETURN 'input';
  END IF;

END;

MEMBER PROCEDURE set_end_date_description
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('end_date_description'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_end_date_description RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('end_date_description');
END;

MEMBER PROCEDURE set_end_date_format
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('end_date_format', p_value);
END;

MEMBER FUNCTION get_end_date_format RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('end_date_format');
END;

MEMBER PROCEDURE set_end_date_label
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('end_date_label'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_end_date_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('end_date_label');
END;

MEMBER PROCEDURE set_end_date_required
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('end_date_required', lower(p_value));
END;

MEMBER FUNCTION get_end_date_required RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('end_date_required'));
END;

MEMBER PROCEDURE set_end_date_visible
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('end_date_visible', lower(p_value));
END;

MEMBER FUNCTION get_end_date_visible RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('end_date_visible'));
END;

MEMBER FUNCTION get_end_date_state RETURN VARCHAR2
IS 
BEGIN

  IF nvl(self.get_end_date_visible, 'true') = 'false' THEN
    RETURN 'hidden';
  ELSIF nvl(self.get_end_date_enabled, 'true') = 'false' THEN
    RETURN 'view';
  ELSE
    RETURN 'input';
  END IF;

END;

MEMBER PROCEDURE set_start_time_description
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('start_time_description'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_start_time_description RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('start_time_description');
END;

MEMBER PROCEDURE set_start_time_label
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('start_time_label'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_start_time_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('start_time_label');
END;

MEMBER PROCEDURE set_start_time_required
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('start_time_required', lower(p_value));
END;

MEMBER FUNCTION get_start_time_required RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('start_time_required'));
END;

MEMBER PROCEDURE set_start_time_visible
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('start_time_visible', lower(p_value));
END;

MEMBER FUNCTION get_start_time_visible RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('start_time_visible'));
END;

MEMBER FUNCTION get_start_time_state RETURN VARCHAR2
IS 
BEGIN

  IF nvl(self.get_start_time_visible, 'true') = 'false' THEN
    RETURN 'hidden';
  ELSIF nvl(self.get_start_time_enabled, 'true') = 'false' THEN
    RETURN 'view';
  ELSE
    RETURN 'input';
  END IF;

END;

MEMBER PROCEDURE set_end_time_description
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('end_time_description'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_end_time_description RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('end_time_description');
END;

MEMBER PROCEDURE set_end_time_label
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('end_time_label'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_end_time_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('end_time_label');
END;

MEMBER PROCEDURE set_end_time_required
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('end_time_required', lower(p_value));
END;

MEMBER FUNCTION get_end_time_required RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('end_time_required'));
END;

MEMBER PROCEDURE set_end_time_visible
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('end_time_visible', lower(p_value));
END;

MEMBER FUNCTION get_end_time_visible RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('end_time_visible'));
END;

MEMBER FUNCTION get_end_time_state RETURN VARCHAR2
IS 
BEGIN

  IF nvl(self.get_end_time_visible, 'true') = 'false' THEN
    RETURN 'hidden';
  ELSIF nvl(self.get_end_time_enabled, 'true') = 'false' THEN
    RETURN 'view';
  ELSE
    RETURN 'input';
  END IF;

END;

MEMBER PROCEDURE set_date_format
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('date_format'
    ,p_value);
END;

MEMBER PROCEDURE set_start_date_min
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('start_date_min'
    ,p_value);
END;

MEMBER FUNCTION get_start_date_min RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('start_date_min');
END;

MEMBER PROCEDURE set_start_date_max
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('start_date_max'
    ,p_value);
END;

MEMBER FUNCTION get_start_date_max RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('start_date_max');
END;

MEMBER PROCEDURE set_start_date_disable_days
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('start_date_disable_days'
    ,lower(p_value));
END;

MEMBER FUNCTION get_start_date_disable_days RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('start_date_disable_days'));
END;

MEMBER FUNCTION get_start_date_disable_dys_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
  SELECT XMLElement
          ("days"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("item"
              ,lower(column_value)
              )
            )
          ) 
  INTO   l_xml
  FROM   TABLE(xxas_util_pkg.string_to_varchar2tbl(self.get_start_date_disable_days()));

  RETURN l_xml;
END;

MEMBER PROCEDURE set_start_date_disable_dates
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('start_date_disable_dates'
    ,lower(p_value));
END;

MEMBER FUNCTION get_start_date_disable_dates RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('start_date_disable_dates'));
END;

MEMBER FUNCTION get_start_date_disable_dts_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
  SELECT XMLElement
          ("dates"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("item"
              ,lower(column_value)
              )
            )
          )
  INTO   l_xml
  FROM   TABLE(xxas_util_pkg.string_to_varchar2tbl(self.get_start_date_disable_dates()));

  RETURN l_xml;
END;

MEMBER FUNCTION get_start_date_dis_dys_dts_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
  SELECT XMLElement
          ("disabledDate"
          ,CASE WHEN self.get_start_date_disable_days is not null 
                THEN self.get_start_date_disable_dys_xml()
           END
          ,CASE WHEN self.get_start_date_disable_dates is not null 
                THEN self.get_start_date_disable_dts_xml()
           END
          )
  INTO   l_xml
  FROM   dual;

  RETURN l_xml;
END;

MEMBER PROCEDURE set_end_date_min
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('end_date_min'
    ,p_value);
END;

MEMBER FUNCTION get_end_date_min RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('end_date_min');
END;

MEMBER PROCEDURE set_end_date_max
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('end_date_max'
    ,p_value);
END;

MEMBER FUNCTION get_end_date_max RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('end_date_max');
END;

MEMBER PROCEDURE set_end_date_disable_days
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('end_date_disable_days'
    ,lower(p_value));
END;

MEMBER FUNCTION get_end_date_disable_days RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('end_date_disable_days'));
END;

MEMBER FUNCTION get_end_date_disable_dys_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
  SELECT XMLElement
          ("days"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("item"
              ,lower(column_value)
              )
            )
          ) 
  INTO   l_xml
  FROM   TABLE(xxas_util_pkg.string_to_varchar2tbl(self.get_end_date_disable_days()));

  RETURN l_xml;
END;

MEMBER PROCEDURE set_end_date_disable_dates
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('end_date_disable_dates'
    ,lower(p_value));
END;

MEMBER FUNCTION get_end_date_disable_dates RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('end_date_disable_dates'));
END;

MEMBER FUNCTION get_end_date_disable_dts_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
  SELECT XMLElement
          ("dates"
          ,XMLAttributes
            ('true' as "json:force-array"
            ,'http://json.org/' as "xmlns:json")
          ,XMLAgg
            (XMLElement
              ("item"
              ,lower(column_value)
              )
            )
          )
  INTO   l_xml
  FROM   TABLE(xxas_util_pkg.string_to_varchar2tbl(self.get_end_date_disable_dates()));

  RETURN l_xml;
END;

MEMBER FUNCTION get_end_date_dis_dys_dts_xml RETURN XMLType
IS 
  l_xml XMLType;
BEGIN
  SELECT XMLElement
          ("disabledDate"
          ,CASE WHEN self.get_end_date_disable_days is not null 
                THEN self.get_end_date_disable_dys_xml()
           END
          ,CASE WHEN self.get_end_date_disable_dates is not null 
                THEN self.get_end_date_disable_dts_xml()
           END
          )
  INTO   l_xml
  FROM   dual;

  RETURN l_xml;
END;

MEMBER PROCEDURE set_start_date_enabled
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('start_date_enabled', lower(p_value));
END;

MEMBER FUNCTION get_start_date_enabled RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('start_date_enabled'));
END;

MEMBER PROCEDURE set_end_date_enabled
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('end_date_enabled', lower(p_value));
END;

MEMBER FUNCTION get_end_date_enabled RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('end_date_enabled'));
END;

MEMBER PROCEDURE set_start_time_enabled
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('start_time_enabled', lower(p_value));
END;

MEMBER FUNCTION get_start_time_enabled RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('start_time_enabled'));
END;

MEMBER PROCEDURE set_end_time_enabled
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('end_time_enabled', lower(p_value));
END;

MEMBER FUNCTION get_end_time_enabled RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('end_time_enabled'));
END;

MEMBER FUNCTION get_date_format RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('date_format');
END;

MEMBER FUNCTION get_date_value(p_is_html5 IN varchar2 DEFAULT 'N') RETURN xxas_com_varchar2_table
IS

  l_date            date;
  l_varchar_tbl     xxas_com_varchar2_table;
  l_varchar_ret_tbl xxas_com_varchar2_table := xxas_com_varchar2_table();

BEGIN

   IF self.get_value IS NULL THEN
     RETURN l_varchar_ret_tbl;
   END IF;
   
   --
   -- First attempt to convert canonical dates and then
   -- dates implicitly converted to varchar2
   --
   l_varchar_tbl := xxas_util_pkg.string_to_varchar2tbl(self.get_value,'_');

   FOR i in 1..l_varchar_tbl.COUNT
   LOOP

     BEGIN

       l_date := to_date(l_varchar_tbl(i));

     EXCEPTION
       WHEN others THEN
       BEGIN
         l_date := fnd_date.canonical_to_date(l_varchar_tbl(i));
       EXCEPTION
         WHEN others THEN
           NULL;
       END;
     END;

     IF l_date IS NULL THEN 
       --
       -- Times are handled as canonical date/times or just time
       -- varchar2s
       --     
       IF date_range_type = 'TimeRange' THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := l_varchar_tbl(i);
       ELSIF date_range_type = 'DateTimeRange' THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := l_varchar_tbl(i);
       ELSE
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := null;
       END IF;

     ELSIF date_range_type = 'DateRange' THEN
       IF p_is_html5 = 'Y'THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'YYYY-MM-DD');
       ELSE
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
       END IF;
     ELSIF date_range_type = 'DateTimeRange' THEN
       IF p_is_html5 = 'Y' THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'YYYY-MM-DD HH24:MI:SS');
       ELSE
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
       END IF;
     ELSIF date_range_type = 'TimeRange' THEN
       l_varchar_ret_tbl.extend();
       l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'HH24:MI'); 
     END IF;

   END LOOP;   

   RETURN l_varchar_ret_tbl;

END;

MEMBER FUNCTION get_date_old_value(p_is_html5 IN varchar2 DEFAULT 'N') RETURN xxas_com_varchar2_table
IS

  l_date            date;
  l_varchar_tbl     xxas_com_varchar2_table;
  l_varchar_ret_tbl xxas_com_varchar2_table := xxas_com_varchar2_table();

BEGIN

   IF self.get_old_value IS NULL THEN
     RETURN l_varchar_ret_tbl;
   END IF;
   
   --
   -- First attempt to convert canonical dates and then
   -- dates implicitly converted to varchar2
   --
   l_varchar_tbl := xxas_util_pkg.string_to_varchar2tbl(self.get_old_value,'_');

   FOR i in 1..l_varchar_tbl.COUNT
   LOOP

     BEGIN

       l_date := to_date(l_varchar_tbl(i));

     EXCEPTION
       WHEN others THEN
       BEGIN
         l_date := fnd_date.canonical_to_date(l_varchar_tbl(i));
       EXCEPTION
         WHEN others THEN
           NULL;
       END;
     END;

     IF l_date IS NULL THEN 
       --
       -- Times are handled as canonical date/times or just time
       -- varchar2s
       --     
       IF date_range_type = 'TimeRange' THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := l_varchar_tbl(i);
       ELSIF date_range_type = 'DateTimeRange' THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := l_varchar_tbl(i);
       ELSE
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := null;
       END IF;

     ELSIF date_range_type = 'DateRange' THEN
       IF p_is_html5 = 'Y'THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'YYYY-MM-DD');
       ELSE
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
       END IF;
     ELSIF date_range_type = 'DateTimeRange' THEN
       IF p_is_html5 = 'Y' THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'YYYY-MM-DD HH24:MI:SS');
       ELSE
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'YYYY/MM/DD HH24:MI:SS');
       END IF;
     ELSIF date_range_type = 'TimeRange' THEN
       l_varchar_ret_tbl.extend();
       l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, 'HH24:MI'); 
     END IF;

   END LOOP;   

   RETURN l_varchar_ret_tbl;

END;

MEMBER FUNCTION get_formatted_value RETURN xxas_com_varchar2_table
IS

  l_date            date;
  l_varchar_tbl     xxas_com_varchar2_table;
  l_varchar_ret_tbl xxas_com_varchar2_table := xxas_com_varchar2_table();

BEGIN

   IF self.get_value IS NULL THEN
     RETURN l_varchar_ret_tbl;
   END IF;
   
   BEGIN

     l_varchar_tbl := xxas_util_pkg.string_to_varchar2tbl(self.get_value,'_');

     FOR i in 1..l_varchar_tbl.COUNT
     LOOP

       l_date := fnd_date.canonical_to_date(l_varchar_tbl(i));

       IF l_date is null THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := null;
       ELSIF date_range_type = 'DateRange' THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, nvl(self.get_date_format, 'FMDd Month YYYY'));
       ELSIF date_range_type = 'DateTimeRange' THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, nvl(self.get_date_format, 'FMDd Month YYYY HH24:MI'));
       ELSIF date_range_type = 'TimeRange' THEN
         l_varchar_ret_tbl.extend();
         l_varchar_ret_tbl(l_varchar_ret_tbl.count) := to_char(l_date, nvl(self.get_date_format, 'HH24:MI')); 
       END IF;

       l_date := null;

     END LOOP;

   EXCEPTION
     WHEN others THEN
       NULL;
   END;

   RETURN l_varchar_ret_tbl;

END;

MEMBER FUNCTION get_values_xml(p_values IN xxas_com_varchar2_table) RETURN XMLType
IS 

  e_parse_err   EXCEPTION;
  PRAGMA        exception_init(e_parse_err, -31011);
  l_xml         XMLtype;
  l_varchar_tbl xxas_com_varchar2_table;
  l_start_time  varchar2(5);
  l_end_time    varchar2(5);

BEGIN

  l_varchar_tbl := p_values;

  IF l_varchar_tbl is not null AND 
     l_varchar_tbl.count = 2 THEN

    IF date_range_type = 'DateRange' THEN

      SELECT XMLElement
              ("value"
              ,XMLForest
                (l_varchar_tbl(1)  AS "startDate"
                ,l_varchar_tbl(2)  AS "endDate")
              )
      INTO   l_xml
      FROM   dual;

    ELSIF date_range_type = 'DateTimeRange' THEN

      BEGIN

        IF to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI') = '00:00' AND 
           to_char(fnd_date.canonical_to_date(l_varchar_tbl(2)),'HH24:MI') = '00:00' THEN

          l_start_time := null;
          l_end_time   := null;

        ELSE

          l_start_time := to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI');
          l_end_time   := to_char(fnd_date.canonical_to_date(l_varchar_tbl(2)),'HH24:MI');

        END IF;

        SELECT XMLElement
                ("value"
                ,XMLForest
                  (to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'YYYY-MM-DD')  AS "startDate"
                  ,to_char(fnd_date.canonical_to_date(l_varchar_tbl(2)),'YYYY-MM-DD')  AS "endDate"
                  ,l_start_time     AS "startTime"
                  ,l_end_time       AS "endTime")
                )
        INTO   l_xml
        FROM   dual;

      EXCEPTION

        WHEN others THEN

          l_start_time := l_varchar_tbl(1);
          l_end_time   := l_varchar_tbl(2);

          SELECT XMLElement
                  ("value"
                  ,XMLForest
                    (l_start_time AS "startTime"
                    ,l_end_time   AS "endTime")
                  )
          INTO   l_xml
          FROM   dual;

      END;

    ELSIF date_range_type = 'TimeRange' THEN

      IF l_varchar_tbl(1) = '00:00' AND 
         l_varchar_tbl(2) = '00:00' THEN

        l_start_time := null;
        l_end_time   := null;

      ELSE

        l_start_time := l_varchar_tbl(1);
        l_end_time   := l_varchar_tbl(2);

      END IF;

      SELECT XMLElement
              ("value"
              ,XMLForest
                (l_start_time  AS "startTime"
                ,l_end_time    AS "endTime")
              )
      INTO   l_xml
      FROM   dual;

    END IF;

  --
  -- Cases where only Start Date or End Date are specified
  --
  ELSIF l_varchar_tbl is not null AND 
        l_varchar_tbl.count = 1 THEN

    --
    -- For End Date in format like _<End_Date>
    --
    IF instr(self.get_value,'_',1) = 1 THEN

      IF date_range_type = 'DateRange' THEN

        SELECT XMLElement
                ("value"
                ,XMLForest
                  (l_varchar_tbl(1)  AS "endDate")
                )
        INTO   l_xml
        FROM   dual;

      ELSIF date_range_type = 'DateTimeRange' THEN

        BEGIN

          IF to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI') = '00:00' THEN

            l_end_time   := null;

          ELSE

            l_end_time   := to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI');

          END IF;

          SELECT XMLElement
                  ("value"
                  ,XMLForest
                    (to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'YYYY-MM-DD')  AS "endDate"
                    ,l_end_time       AS "endTime")
                  )
          INTO   l_xml
          FROM   dual;

        EXCEPTION

          WHEN others THEN

            l_end_time := l_varchar_tbl(1);

            SELECT XMLElement
                    ("value"
                    ,XMLForest
                      (l_end_time   AS "endTime")
                    )
            INTO   l_xml
            FROM   dual;

        END;

      ELSIF date_range_type = 'TimeRange' THEN

        IF l_varchar_tbl(1) = '00:00' THEN

          l_end_time   := null;

        ELSE

          l_end_time   := l_varchar_tbl(1);

        END IF;

        SELECT XMLElement
                ("value"
                ,XMLForest
                  (l_end_time    AS "endTime")
                )
        INTO   l_xml
        FROM   dual;

      END IF;

    ELSE

      IF date_range_type = 'DateRange' THEN

        SELECT XMLElement
                ("value"
                ,XMLForest
                  (l_varchar_tbl(1)  AS "startDate")
                )
        INTO   l_xml
        FROM   dual;

      ELSIF date_range_type = 'DateTimeRange' THEN

        BEGIN

          IF to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI') = '00:00' THEN

            l_start_time := null;

          ELSE

            l_start_time := to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI');

          END IF;

          SELECT XMLElement
                  ("value"
                  ,XMLForest
                    (to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'YYYY-MM-DD')  AS "startDate"
                    ,l_start_time     AS "startTime")
                  )
          INTO   l_xml
          FROM   dual;

        EXCEPTION

          WHEN others THEN

            l_start_time := l_varchar_tbl(1);

            SELECT XMLElement
                    ("value"
                    ,XMLForest
                      (l_start_time   AS "startTime")
                    )
            INTO   l_xml
            FROM   dual;

        END;

      ELSIF date_range_type = 'TimeRange' THEN

        IF l_varchar_tbl(1) = '00:00' THEN

          l_start_time := null;

        ELSE

          l_start_time := l_varchar_tbl(1);

        END IF;

        SELECT XMLElement
                ("value"
                ,XMLForest
                  (l_start_time  AS "startTime")
                )
        INTO   l_xml
        FROM   dual;

      END IF;

    END IF;

  END IF;

  RETURN l_xml;
  
EXCEPTION
  WHEN e_parse_err THEN
   RAISE;

END;

MEMBER FUNCTION get_old_values_xml(p_values IN xxas_com_varchar2_table) RETURN XMLType
IS 

  e_parse_err   EXCEPTION;
  PRAGMA        exception_init(e_parse_err, -31011);
  l_xml         XMLtype;
  l_varchar_tbl xxas_com_varchar2_table;
  l_start_time  varchar2(5);
  l_end_time    varchar2(5);

BEGIN

  l_varchar_tbl := p_values;

  IF l_varchar_tbl is not null AND 
     l_varchar_tbl.count = 2 THEN

    IF date_range_type = 'DateRange' THEN

      SELECT XMLElement
              ("oldValue"
              ,XMLForest
                (l_varchar_tbl(1)  AS "startDate"
                ,l_varchar_tbl(2)  AS "endDate")
              )
      INTO   l_xml
      FROM   dual;

    ELSIF date_range_type = 'DateTimeRange' THEN

      BEGIN

        IF to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI') = '00:00' AND 
           to_char(fnd_date.canonical_to_date(l_varchar_tbl(2)),'HH24:MI') = '00:00' THEN

          l_start_time := null;
          l_end_time   := null;

        ELSE

          l_start_time := to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI');
          l_end_time   := to_char(fnd_date.canonical_to_date(l_varchar_tbl(2)),'HH24:MI');

        END IF;

        SELECT XMLElement
                ("oldValue"
                ,XMLForest
                  (to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'YYYY-MM-DD')  AS "startDate"
                  ,to_char(fnd_date.canonical_to_date(l_varchar_tbl(2)),'YYYY-MM-DD')  AS "endDate"
                  ,l_start_time     AS "startTime"
                  ,l_end_time       AS "endTime")
                )
        INTO   l_xml
        FROM   dual;

      EXCEPTION

        WHEN others THEN

          l_start_time := l_varchar_tbl(1);
          l_end_time   := l_varchar_tbl(2);

          SELECT XMLElement
                  ("value"
                  ,XMLForest
                    (l_start_time AS "startTime"
                    ,l_end_time   AS "endTime")
                  )
          INTO   l_xml
          FROM   dual;

      END;

    ELSIF date_range_type = 'TimeRange' THEN

      IF l_varchar_tbl(1) = '00:00' AND 
         l_varchar_tbl(2) = '00:00' THEN

        l_start_time := null;
        l_end_time   := null;

      ELSE

        l_start_time := l_varchar_tbl(1);
        l_end_time   := l_varchar_tbl(2);

      END IF;

      SELECT XMLElement
              ("oldValue"
              ,XMLForest
                (l_start_time  AS "startTime"
                ,l_end_time    AS "endTime")
              )
      INTO   l_xml
      FROM   dual;

    END IF;

  --
  -- Cases where only Start Date or End Date are specified
  --
  ELSIF l_varchar_tbl is not null AND 
        l_varchar_tbl.count = 1 THEN

    --
    -- For End Date in format like _<End_Date>
    --
    IF instr(self.get_value,'_',1) = 1 THEN

      IF date_range_type = 'DateRange' THEN

        SELECT XMLElement
                ("oldValue"
                ,XMLForest
                  (l_varchar_tbl(1)  AS "endDate")
                )
        INTO   l_xml
        FROM   dual;

      ELSIF date_range_type = 'DateTimeRange' THEN

        BEGIN

          IF to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI') = '00:00' THEN

            l_end_time   := null;

          ELSE

            l_end_time   := to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI');

          END IF;

          SELECT XMLElement
                  ("oldValue"
                  ,XMLForest
                    (to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'YYYY-MM-DD')  AS "endDate"
                    ,l_end_time       AS "endTime")
                  )
          INTO   l_xml
          FROM   dual;

        EXCEPTION

          WHEN others THEN

          l_end_time := l_varchar_tbl(1);

          SELECT XMLElement
                  ("value"
                  ,XMLForest
                    (l_end_time   AS "endTime")
                  )
          INTO   l_xml
          FROM   dual;

        END;

      ELSIF date_range_type = 'TimeRange' THEN

        IF l_varchar_tbl(1) = '00:00' THEN

          l_end_time   := null;

        ELSE

          l_end_time   := l_varchar_tbl(1);

        END IF;

        SELECT XMLElement
                ("oldValue"
                ,XMLForest
                  (l_end_time    AS "endTime")
                )
        INTO   l_xml
        FROM   dual;

      END IF;

    ELSE

      IF date_range_type = 'DateRange' THEN

        SELECT XMLElement
                ("oldValue"
                ,XMLForest
                  (l_varchar_tbl(1)  AS "startDate")
                )
        INTO   l_xml
        FROM   dual;

      ELSIF date_range_type = 'DateTimeRange' THEN

        BEGIN

          IF to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI') = '00:00' THEN

            l_start_time := null;

          ELSE

            l_start_time := to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'HH24:MI');

          END IF;

          SELECT XMLElement
                  ("oldValue"
                  ,XMLForest
                    (to_char(fnd_date.canonical_to_date(l_varchar_tbl(1)),'YYYY-MM-DD')  AS "startDate"
                    ,l_start_time     AS "startTime")
                  )
          INTO   l_xml
          FROM   dual;

        EXCEPTION

          WHEN others THEN

            l_start_time := l_varchar_tbl(1);

            SELECT XMLElement
                    ("value"
                    ,XMLForest
                      (l_start_time   AS "startTime")
                    )
            INTO   l_xml
            FROM   dual;

        END;

      ELSIF date_range_type = 'TimeRange' THEN

        IF l_varchar_tbl(1) = '00:00' THEN

          l_start_time := null;

        ELSE

          l_start_time := l_varchar_tbl(1);

        END IF;

        SELECT XMLElement
                ("oldValue"
                ,XMLForest
                  (l_start_time  AS "startTime")
                )
        INTO   l_xml
        FROM   dual;

      END IF;

    END IF;

  END IF;

  RETURN l_xml;
  
EXCEPTION
  WHEN e_parse_err THEN
   RAISE;

END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT CASE WHEN date_range_type = 'DateRange'
              THEN insertChildXML
                     (XMLELEMENT
                       ("DateRange"
                       ,XMLATTRIBUTES
                          (self.get_id                  AS "id"
                          ,self.get_label               AS "label"
                          ,self.get_description         AS "description"
                          ,self.get_refresh_on_change   AS "refreshOnChange"
                          ,self.get_enabled             AS "enabled"
                          ,self.get_required            AS "required"
                          ,self.get_visible             AS "visible"
                          /*,self.get_date_value          AS "value"
                          ,self.get_formatted_value     AS "formattedValue"*/
                          ,self.get_transient           AS "transient"
                          ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                      nvl(self.get_enable_form_on_server_wait,'true')
                                ELSE
                                      self.get_enable_form_on_server_wait
                           END  AS "enableFormOnServerWait"))
                     ,'//DateRange[not(@value)]'
                     ,'@value'
                     ,'')
              WHEN date_range_type = 'DateTimeRange'
              THEN insertChildXML
                     (XMLELEMENT
                       ("DateTimeRange"
                       ,XMLATTRIBUTES
                          (self.get_id                  AS "id"
                          ,self.get_label               AS "label"
                          ,self.get_description         AS "description" 
                          ,self.get_refresh_on_change   AS "refreshOnChange"
                          ,self.get_enabled             AS "enabled"
                          ,self.get_required            AS "required"
                          ,self.get_visible             AS "visible"
                          /*,self.get_date_value          AS "value"
                          ,self.get_formatted_value     AS "formattedValue"*/
                          ,self.get_transient           AS "transient"
                          ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                      nvl(self.get_enable_form_on_server_wait,'true')
                                ELSE
                                      self.get_enable_form_on_server_wait
                           END  AS "enableFormOnServerWait"))
                     ,'//DateTimeRange[not(@value)]'
                     ,'@value'
                     ,'')
              WHEN date_range_type = 'TimeRange'
              THEN insertChildXML
                     (XMLELEMENT
                       ("TimeRange"
                       ,XMLATTRIBUTES
                          (self.get_id                  AS "id"
                          ,self.get_label               AS "label"
                          ,self.get_description         AS "description"   
                          ,self.get_refresh_on_change   AS "refreshOnChange"
                          ,self.get_enabled             AS "enabled"
                          ,self.get_required            AS "required"
                          ,self.get_visible             AS "visible"
                          /*,self.get_date_value          AS "value"
                          ,self.get_formatted_value     AS "formattedValue"*/
                          ,self.get_transient           AS "transient"
                          ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                      nvl(self.get_enable_form_on_server_wait,'true')
                                ELSE
                                      self.get_enable_form_on_server_wait
                           END  AS "enableFormOnServerWait"))
                     ,'//TimeRange[not(@value)]'
                     ,'@value'
                     ,'')
         END                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  l_time varchar2(10) := 'HH:MM';

BEGIN

  IF date_range_type = 'DateRange' THEN

    SELECT XMLELEMENT
            ("field"
            ,XMLForest
              ('dateTimeRange'                    AS "type"
              ,self.get_id                        AS "id"
              ,self.get_label                     AS "label"
              ,self.get_state                     AS "state"
              ,self.get_refresh_on_change         AS "refreshOnChange"
              ,self.get_required                  AS "required"
              ,self.get_enabled                   AS "enabled"
              ,self.get_visible                   AS "visible"
              ,self.get_hide_if                   AS "hideIf"
              ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                      nvl(self.get_enable_form_on_server_wait,'true')
                                ELSE
                                      self.get_enable_form_on_server_wait
                           END  AS "enableFormOnServerWait")
            ,XMLElement
              ("startDate"
              ,XMLForest
                (xxas_com_mls_pkg.get_date_mask_tip
                   (self.get_start_date_format)     AS "placeholder"
                ,self.get_start_date_description    AS "description"
                ,xxas_com_mls_pkg.get_iso_date_mask
                   (self.get_start_date_format)     AS "dateFormat"
                ,self.get_start_date_label          AS "label"
                ,self.get_start_date_required       AS "required"
                ,self.get_start_date_visible        AS "visible"
                ,self.get_start_date_state          AS "state"
                ,self.get_start_date_min            AS "min"
                ,self.get_start_date_max            AS "max")
              ,self.get_start_date_dis_dys_dts_xml() AS "disableDates"
              ,xxas_com_mls_pkg.calendars_xml
              )
            ,XMLElement
              ("endDate"
              ,XMLForest
                (xxas_com_mls_pkg.get_date_mask_tip
                   (self.get_end_date_format)     AS "placeholder"
                ,self.get_end_date_description    AS "description"
                ,xxas_com_mls_pkg.get_iso_date_mask
                   (self.get_end_date_format)     AS "dateFormat"
                ,self.get_end_date_label          AS "label"
                ,self.get_end_date_required       AS "required"
                ,self.get_end_date_visible        AS "visible"
                ,self.get_end_date_state          AS "state"
                ,self.get_end_date_min            AS "min"
                ,self.get_end_date_max            AS "max")
              ,self.get_end_date_dis_dys_dts_xml() AS "disableDates"
              ,xxas_com_mls_pkg.calendars_xml
              )
            ,self.get_values_xml(self.get_date_value('Y'))
            ,self.get_old_values_xml(self.get_date_old_value('Y'))
            ,CASE WHEN self.get_trigger_name is not null AND 
                       self.get_trigger_method is not null THEN
               XMLElement
                ("links"
                ,XMLElement
                  ("trigger"
                  ,XMLElement
                    ("href"
                    ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                    )
                  ,XMLElement
                    ("method"
                    ,self.get_trigger_method
                    )
                  )
                )
             END
            )
    INTO   l_xml   
    FROM   dual;

  ELSIF date_range_type = 'DateTimeRange' THEN

    SELECT XMLELEMENT
            ("field"
            ,XMLForest
              ('dateTimeRange'                    AS "type"
              ,self.get_id                        AS "id"
              ,self.get_label                     AS "label"
              ,self.get_state                     AS "state"
              ,self.get_refresh_on_change         AS "refreshOnChange"
              ,self.get_required                  AS "required"
              ,self.get_enabled                   AS "enabled"
              ,self.get_visible                   AS "visible"
              ,self.get_hide_if                   AS "hideIf"
              ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                            nvl(self.get_enable_form_on_server_wait,'true')
                    ELSE
                            self.get_enable_form_on_server_wait
               END  AS "enableFormOnServerWait")
            ,XMLElement
              ("startDate"
              ,XMLForest
                (xxas_com_mls_pkg.get_date_mask_tip
                   (self.get_start_date_format)     AS "placeholder"
                ,self.get_start_date_description    AS "description"
                ,xxas_com_mls_pkg.get_iso_date_mask
                   (self.get_start_date_format)     AS "dateFormat"
                ,self.get_start_date_label          AS "label"
                ,self.get_start_date_required       AS "required"
                ,self.get_start_date_visible        AS "visible"
                ,self.get_start_date_state          AS "state"
                ,self.get_start_date_min            AS "min"
                ,self.get_start_date_max            AS "max")
              ,self.get_start_date_dis_dys_dts_xml() AS "disableDates"
              ,xxas_com_mls_pkg.calendars_xml
              )
            ,XMLElement
              ("endDate"
              ,XMLForest
                (xxas_com_mls_pkg.get_date_mask_tip
                   (self.get_end_date_format)     AS "placeholder"
                ,self.get_end_date_description    AS "description"
                ,xxas_com_mls_pkg.get_iso_date_mask
                   (self.get_end_date_format)     AS "dateFormat"
                ,self.get_end_date_label          AS "label"
                ,self.get_end_date_required       AS "required"
                ,self.get_end_date_visible        AS "visible"
                ,self.get_end_date_state          AS "state"
                ,self.get_end_date_min            AS "min"
                ,self.get_end_date_max            AS "max")
              ,self.get_end_date_dis_dys_dts_xml() AS "disableDates"
              ,xxas_com_mls_pkg.calendars_xml
              )
            ,XMLElement
              ("startTime"
              ,XMLForest
                (l_time						        AS "placeholder"
                ,self.get_start_time_description    AS "description"
                ,self.get_start_time_label          AS "label"
                ,self.get_start_time_required       AS "required"
                ,self.get_start_time_visible        AS "visible"
                ,self.get_start_time_state          AS "state")
              )
            ,XMLElement
              ("endTime"
              ,XMLForest
                (l_time						      AS "placeholder"
                ,self.get_end_time_description    AS "description"
                ,self.get_end_time_label          AS "label"
                ,self.get_end_time_required       AS "required"
                ,self.get_end_time_visible        AS "visible"
                ,self.get_end_time_state          AS "state"
                ,xxas_com_msg_pkg.get_string
					          ('XXAS_EMP_END_TIME_ERROR')   AS "minTimeError")
              )
            ,self.get_values_xml(self.get_date_value('Y'))
            ,self.get_old_values_xml(self.get_date_old_value('Y'))
            ,CASE WHEN self.get_trigger_name is not null AND 
                       self.get_trigger_method is not null THEN
               XMLElement
                ("links"
                ,XMLElement
                  ("trigger"
                  ,XMLElement
                    ("href"
                    ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                    )
                  ,XMLElement
                    ("method"
                    ,self.get_trigger_method
                    )
                  )
                )
             END
            )
    INTO   l_xml   
    FROM   dual;

  ELSIF date_range_type = 'TimeRange' THEN

    SELECT XMLELEMENT
            ("field"
            ,XMLForest
              ('dateTimeRange'                    AS "type"
              ,self.get_id                        AS "id"
              ,self.get_label                     AS "label"
              ,self.get_state                     AS "state"
              ,self.get_refresh_on_change         AS "refreshOnChange"
              ,self.get_required                  AS "required"
              ,self.get_enabled                   AS "enabled"
              ,self.get_visible                   AS "visible"
              ,self.get_hide_if                   AS "hideIf"
              ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
               END  AS "enableFormOnServerWait")
            ,XMLElement
              ("startTime"
              ,XMLForest
                (l_time						        AS "placeholder"
                ,self.get_start_time_description    AS "description"
                ,self.get_start_time_label          AS "label"
                ,self.get_start_time_required       AS "required"
                ,self.get_start_time_visible        AS "visible"
                ,self.get_start_time_state          AS "state")
              )
            ,XMLElement
              ("endTime"
              ,XMLForest
                (l_time                           AS "placeholder"
                ,self.get_end_time_description    AS "description"
                ,self.get_end_time_label          AS "label"
                ,self.get_end_time_required       AS "required"
                ,self.get_end_time_visible        AS "visible"
                ,self.get_end_time_state          AS "state")
              )
            ,self.get_values_xml(self.get_date_value('Y'))
            ,self.get_old_values_xml(self.get_date_old_value('Y'))
            ,CASE WHEN self.get_trigger_name is not null AND 
                       self.get_trigger_method is not null THEN
               XMLElement
                ("links"
                ,XMLElement
                  ("trigger"
                  ,XMLElement
                    ("href"
                    ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                    )
                  ,XMLElement
                    ("method"
                    ,self.get_trigger_method
                    )
                  )
                )
             END
            )
    INTO   l_xml   
    FROM   dual;

  END IF;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_divider_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_divider_type
  (p_ui_divider_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/*' 
                  PASSING p_ui_divider_xml
                  COLUMNS id      VARCHAR2(4000) path '@id',
                          visible VARCHAR2(4000) path '@visible',
                          hide_if VARCHAR2(4000) path '@hideIf');            
  l_data c_data%ROWTYPE;
  
BEGIN
  
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  
  RETURN; 
END;
  

CONSTRUCTOR FUNCTION xxas_emp_ui_divider_type
  (p_id      VARCHAR2
  ,p_visible VARCHAR2
  ,p_hide_if VARCHAR2) RETURN SELF AS RESULT
IS

BEGIN

  self.set_id(p_id);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);

  RETURN;          

END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;

BEGIN
  SELECT XMLELEMENT
          ("Divider"
          ,XMLATTRIBUTES(self.get_id                AS "id"
                        ,self.get_visible           AS "visible"))
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

BEGIN

  RETURN self.get_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT
             ("field"
             ,XMLForest
               ('line'                       AS "type"
               ,self.get_id                  AS "id"
               ,self.get_visible             AS "visible"
               ,self.get_hide_if             AS "hideIf")
             )
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_value_type IS 

CONSTRUCTOR FUNCTION xxas_emp_ui_value_type
  (p_ui_value_xml IN XMLTYPE) RETURN SELF AS RESULT

IS

  CURSOR c_data
  IS
  SELECT id
  ,      value
  ,      description
  ,      old_value
  FROM   XMLTABLE('/Value'
                  PASSING p_ui_value_xml
                  COLUMNS id          VARCHAR2(4000) path 'ID',
                          value       VARCHAR2(4000) path 'Value',
                          description VARCHAR2(4000) path 'Description',
                          old_value   VARCHAR2(4000) path 'oldValue');

BEGIN
  OPEN  c_data;
  FETCH c_data INTO id, value, description, old_value;
  CLOSE c_data;

  RETURN;
END;

MEMBER FUNCTION get_xml RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN
  SELECT XMLELEMENT("Value"
                   ,XMLELEMENT("ID", id)
                   ,XMLELEMENT("Value", value)
                   ,XMLELEMENT("Description", description))
  INTO   l_xml
  FROM   dual;

  RETURN l_xml;
END;

MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT("value"
                   ,XMLELEMENT("id", id)
                   ,XMLELEMENT("value", value)
                   ,XMLELEMENT("description", description))
  INTO   l_xml
  FROM   dual;

  RETURN l_xml;
  
END;

MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT("value"
                   ,XMLELEMENT("id", id)
                   ,XMLELEMENT("label", value)
                   ,XMLELEMENT("description", description)
                   ,XMLELEMENT("oldValue", old_value))
  INTO   l_xml
  FROM   dual;

  RETURN l_xml;
  
END;

MEMBER FUNCTION get_json_friendly_cb
  (p_positive_boolean_value IN varchar2) RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  IF nvl(id, value) = p_positive_boolean_value THEN

    SELECT XMLELEMENT("true"
                     ,nvl(id, value))
    INTO   l_xml
    FROM   dual;
  
  ELSE

    SELECT XMLELEMENT("false"
                     ,nvl(id, value))
    INTO   l_xml
    FROM   dual;
  
  END IF;

  RETURN l_xml;
  
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_valueset_type IS

CONSTRUCTOR FUNCTION xxas_emp_ui_valueset_type
  (p_ui_valueset_xml IN XMLTYPE) RETURN SELF AS RESULT

IS

  CURSOR c_data
  IS
  SELECT xxas_emp_ui_value_type(value) value
  FROM   XMLTABLE('/ValueSet/*'
                  PASSING p_ui_valueset_xml
                  COLUMNS value    XMLTYPE       path '/Value');

BEGIN
  value_table := xxas_emp_ui_value_table();

  FOR l_data IN c_data
  LOOP
    value_table.EXTEND;
    value_table(value_table.COUNT) := l_data.value;
  END LOOP;

  RETURN;
END;

MEMBER FUNCTION get_xml RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT("ValueSet"
                   ,XMLAGG(f.get_xml()))
  INTO   l_xml
  FROM   TABLE(value_table) f;

  RETURN l_xml;
    
END;

MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT("data"
                   ,XMLAGG(f.get_form_schema()))
  INTO   l_xml
  FROM   TABLE(value_table) f;

  RETURN l_xml;
END;

MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT("values"
                   ,XMLAttributes
                     ('true' as "json:force-array"
                     ,'http://json.org/' as "xmlns:json")
                   ,XMLAGG(f.get_json_friendly()))
  INTO   l_xml
  FROM   TABLE(value_table) f;

  RETURN l_xml;

END;

MEMBER FUNCTION get_json_friendly_cb
  (p_positive_boolean_value IN VARCHAR2) RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLELEMENT("values"
                   ,XMLAGG(f.get_json_friendly_cb(p_positive_boolean_value)))
  INTO   l_xml
  FROM   TABLE(value_table) f;

  RETURN l_xml;

END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_emp_ui_list_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_list_type
  (p_ui_list_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/*' 
                  PASSING p_ui_list_xml
                  COLUMNS id                     VARCHAR2(4000) path '@id',
                          label                  VARCHAR2(4000) path '@label',
                          description            VARCHAR2(4000) path '@description',   
                          refresh_on_change      VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name           VARCHAR2(4000) path '@triggerName',
                          trigger_method         VARCHAR2(4000) path '@triggerMethod',
                          enabled                VARCHAR2(4000) path '@enabled',
                          required               VARCHAR2(4000) path '@required',
                          visible                VARCHAR2(4000) path '@visible',
                          hide_if                VARCHAR2(4000) path '@hideIf',
                          list_provider          VARCHAR2(4000) path '@listProvider',
                          values_in_payload      VARCHAR2(4000) path '@valuesInPayload',
                          longlist               VARCHAR2(4000) path '@longlist',
                          show_as_radio          VARCHAR2(4000) path '@showAsRadio',
                          show_as_boolean        VARCHAR2(4000) path '@showAsBoolean',
                          positive_boolean_value VARCHAR2(4000) path '@positiveBooleanValue',
                          multiple_selection     VARCHAR2(4000) path '@multipleSelection',
                          style                  VARCHAR2(4000) path '@style',
                          allow_new_values       VARCHAR2(4000) path '@allowNewValues',
                          empty_value            VARCHAR2(4000) path '@emptyValue',
                          value                  VARCHAR2(4000) path '@value',
                          transient              VARCHAR2(4000) path '@transient',
                          rule                   VARCHAR2(4000) path '@rule',
                          message                VARCHAR2(4000) path '@message',
                          value_set              XMLTYPE        path '/ValueSet',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');            
  l_data c_data%ROWTYPE;
  
BEGIN
  
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  SELECT CASE WHEN existsNode(p_ui_list_xml, 'Poplist') = 1
              THEN 'Poplist'
              WHEN existsNode(p_ui_list_xml, 'Combolist') = 1 
              THEN 'Combolist' 
         END
  INTO   list_type
  FROM   dual;
  
  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_list_provider(l_data.list_provider);
  self.set_values_in_payload(l_data.values_in_payload);
  self.set_longlist(l_data.longlist);
  self.set_show_as_radio(l_data.show_as_radio);
  self.set_show_as_boolean(l_data.show_as_boolean);
  self.set_positive_boolean_value(l_data.positive_boolean_value);
  self.set_multiple_selection(l_data.multiple_selection);
  self.set_style(l_data.style);
  self.set_allow_new_values(l_data.allow_new_values);
  self.set_empty_value(l_data.empty_value);
  self.set_value(l_data.value);
  self.set_transient(l_data.transient);
  self.set_rule(l_data.rule);
  self.set_message(l_data.message);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
  
  value_set := xxas_emp_ui_valueset_type(l_data.value_set);

  RETURN; 
END;
  

CONSTRUCTOR FUNCTION xxas_emp_ui_list_type
  (p_list_type              VARCHAR2
  ,p_id                     VARCHAR2
  ,p_label                  VARCHAR2
  ,p_description            VARCHAR2    
  ,p_refresh_on_change      VARCHAR2
  ,p_trigger_name           VARCHAR2
  ,p_trigger_method         VARCHAR2 
  ,p_enabled                VARCHAR2
  ,p_required               VARCHAR2
  ,p_visible                VARCHAR2 
  ,p_hide_if                VARCHAR2
  ,p_list_provider          VARCHAR2
  ,p_values_in_payload      VARCHAR2
  ,p_longlist               VARCHAR2
  ,p_show_as_radio          VARCHAR2
  ,p_show_as_boolean        VARCHAR2
  ,p_positive_boolean_value VARCHAR2
  ,p_multiple_selection     VARCHAR2
  ,p_style                  VARCHAR2
  ,p_allow_new_values       VARCHAR2
  ,p_empty_value            VARCHAR2
  ,p_value                  VARCHAR2
  ,p_transient              VARCHAR2
  ,p_rule                   VARCHAR2
  ,p_message                VARCHAR2
  ,p_value_set              xxas_emp_ui_valueset_type
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
IS

BEGIN

  list_type := p_list_type;
    
  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_list_provider(p_list_provider);
  self.set_values_in_payload(p_values_in_payload);
  self.set_longlist(p_longlist);
  self.set_show_as_radio(p_show_as_radio);
  self.set_show_as_boolean(p_show_as_boolean);
  self.set_positive_boolean_value(p_positive_boolean_value);
  self.set_multiple_selection(p_multiple_selection);
  self.set_style(p_style);
  self.set_allow_new_values(p_allow_new_values);
  self.set_empty_value(p_empty_value);
  self.set_value(p_value);
  self.set_transient(p_transient);
  self.set_rule(p_rule);
  self.set_message(p_message);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
  
  value_set := p_value_set;
   
  RETURN;          
END;

MEMBER PROCEDURE set_rule
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('rule',p_value);
END;

MEMBER FUNCTION get_rule RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('rule');
END;

MEMBER PROCEDURE set_message
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('message',p_value);
END;

MEMBER FUNCTION get_message RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('message');
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_list_provider
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('list_provider', p_value);
END;

MEMBER FUNCTION get_list_provider RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('list_provider');
END;

MEMBER PROCEDURE set_values_in_payload
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('values_in_payload', lower(p_value));
END;

MEMBER FUNCTION get_values_in_payload RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('values_in_payload'));
END;

MEMBER PROCEDURE set_longlist
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('longlist', lower(p_value));
END;

MEMBER FUNCTION get_longlist RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('longlist'));
END;

MEMBER PROCEDURE set_show_as_radio
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('show_as_radio', lower(p_value));
END;

MEMBER FUNCTION get_show_as_radio RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('show_as_radio'));
END;

MEMBER PROCEDURE set_show_as_boolean
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('show_as_boolean', lower(p_value));
END;

MEMBER FUNCTION get_show_as_boolean RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('show_as_boolean'));
END;

MEMBER PROCEDURE set_positive_boolean_value
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('positive_boolean_value', p_value);
END;

MEMBER FUNCTION get_positive_boolean_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('positive_boolean_value');
END;

MEMBER PROCEDURE set_multiple_selection
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('multiple_selection', lower(p_value));
END;

MEMBER FUNCTION get_multiple_selection RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('multiple_selection'));
END;

MEMBER PROCEDURE set_style
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('style', p_value);
END;

MEMBER FUNCTION get_style RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('style');
END;

MEMBER PROCEDURE set_allow_new_values
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('allow_new_values', lower(p_value));
END;

MEMBER FUNCTION get_allow_new_values RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('allow_new_values'));
END;

MEMBER PROCEDURE set_empty_value
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('empty_value'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_empty_value RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('empty_value');
END;

MEMBER FUNCTION get_list_provider_type RETURN VARCHAR2
IS 
  l_provider VARCHAR2(4000);
  l_type     VARCHAR2(400);
  
BEGIN

  l_provider := self.get_attribute('list_provider');
  l_type     := substr(l_provider, 1, instr(l_provider, ':') - 1);

  RETURN l_type;  

END;

MEMBER FUNCTION is_provider_value_set RETURN BOOLEAN
IS 
  l_type     VARCHAR2(400);
  
BEGIN

  l_type := self.get_list_provider_type;

  RETURN (l_type = 'VALUE_SET_ID' OR l_type = 'VALUE_SET_NAME');  

END;

MEMBER FUNCTION is_provider_lookup RETURN BOOLEAN
IS 
  l_type     VARCHAR2(400);
  
BEGIN

  l_type := self.get_list_provider_type;

  RETURN (l_type = 'LOOKUP_TYPE');  

END;

MEMBER FUNCTION is_provider_rest RETURN BOOLEAN
IS 
  l_type     VARCHAR2(400);
  
BEGIN

  l_type := self.get_list_provider_type;

  RETURN (l_type = 'REST');  

END;

MEMBER FUNCTION is_provider_rest_flag RETURN VARCHAR2
IS 
  
BEGIN

  IF self.is_provider_rest THEN
    RETURN 'Y';
  ELSE
    RETURN 'N';
  END IF;

END;

MEMBER FUNCTION get_value_set_id RETURN NUMBER
IS 
  l_provider VARCHAR2(400);
  l_type     VARCHAR2(400);
  l_value    VARCHAR2(400);
  l_vset_id  NUMBER;
  
BEGIN

  l_provider := self.get_attribute('list_provider');
  l_type     := substr(l_provider, 1, instr(l_provider, ':') - 1);
  l_value    := substr(l_provider, instr(l_provider, ':') + 1);
  
  IF l_type = 'VALUE_SET_ID' THEN
    l_vset_id := to_number(l_value);
  ELSIF l_type = 'VALUE_SET_NAME' THEN
    l_vset_id :=
      xxas_com_vset_pkg.get_vset_id_by_name
        (l_value);
  END IF;
  
  RETURN l_vset_id;

END;

MEMBER FUNCTION get_lookup_type RETURN VARCHAR2
IS 
  l_provider VARCHAR2(4000);
  l_type     VARCHAR2(400);
  l_value    VARCHAR2(4000);
  
BEGIN

  l_provider := self.get_attribute('list_provider');
  l_type     := substr(l_provider, 1, instr(l_provider, ':') - 1);
  l_value    := substr(l_provider, instr(l_provider, ':') + 1);
  
  IF l_type = 'LOOKUP_TYPE' THEN
    RETURN l_value;
  END IF;
  
  RETURN NULL;

END;

MEMBER FUNCTION get_rest_path RETURN VARCHAR2
IS 
  l_provider VARCHAR2(4000);
  l_type     VARCHAR2(400);
  l_value    VARCHAR2(4000);
  
BEGIN

  l_provider := self.get_attribute('list_provider');
  l_type     := substr(l_provider, 1, instr(l_provider, ':') - 1);
  l_value    := substr(l_provider, instr(l_provider, ':') + 1);
  
  IF l_type = 'REST' THEN
    RETURN l_value;
  END IF;
  
  RETURN NULL;

END;

MEMBER FUNCTION get_rest_href RETURN VARCHAR2
IS 

  l_path    VARCHAR2(4000);
  
BEGIN

  l_path := self.get_rest_path;
  IF l_path IS NOT NULL THEN
    RETURN xxas_com_rest_util_pkg.uri_from_path(l_path);  
  END IF;
  
  RETURN NULL;

END;

MEMBER FUNCTION get_longlist_href RETURN VARCHAR2
IS 

  l_path    VARCHAR2(4000);
  l_vset_id NUMBER;
  
BEGIN

  l_path := self.get_rest_path;
  IF l_path IS NULL THEN
    l_vset_id := self.get_value_set_id;
    IF l_vset_id IS NOT NULL THEN
      l_path := 'value-sets/by-id/' || l_vset_id || '?q={searchText}';
    END IF;
  END IF;
  
  IF l_path IS NOT NULL THEN
    RETURN xxas_com_rest_util_pkg.uri_from_path(l_path);  
  ELSE
    RETURN NULL;
  END IF;

END;

MEMBER FUNCTION get_values_xml RETURN XMLType
IS 

  e_parse_err EXCEPTION;
  PRAGMA exception_init(e_parse_err, -31011);
  l_xml XMLTYPE;

BEGIN

  IF self.get_multiple_selection IS NOT NULL AND
     self.get_multiple_selection = 'true' AND
     self.get_value              IS NOT NULL THEN
    l_xml := XMLType(self.get_value);
  END IF;

  RETURN l_xml;
  
EXCEPTION
  WHEN e_parse_err THEN
   RAISE;

END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE
IS
  l_xml XMLTYPE;

BEGIN
  SELECT CASE WHEN list_type = 'Poplist'
              THEN insertChildXML
                     (insertChildXML
                       (XMLELEMENT
                          ("Poplist"
                          ,XMLATTRIBUTES
                             (self.get_id                     AS "id"
                             ,self.get_label                  AS "label"
                             ,self.get_description            AS "description"   
                             ,self.get_refresh_on_change      AS "refreshOnChange"
                             ,self.get_enabled                AS "enabled"
                             ,self.get_required               AS "required"
                             ,self.get_visible                AS "visible"
                             ,decode(self.is_provider_rest_flag
                                    ,'Y'
                                    ,self.get_rest_href
                                    ,self.get_list_provider)  AS "listProvider"
                             ,self.get_values_in_payload      AS "valuesInPayload"
                             ,self.get_show_as_boolean        AS "showAsBoolean"
                             ,self.get_positive_boolean_value AS "positiveBooleanValue"
                             ,self.get_allow_new_values       AS "allowNewValues"
                             ,self.get_value                  AS "value"
                             ,self.get_transient              AS "transient"
                             ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                        nvl(self.get_enable_form_on_server_wait,'true')
                                  ELSE
                                        self.get_enable_form_on_server_wait
                              END  AS "enableFormOnServerWait")
                          ,value_set.get_xml())
                         ,'//Poplist[not(@positiveBooleanValue)]'
                         ,'@positiveBooleanValue'
                         ,'') 
                       ,'//Poplist[not(@value)]'
                       ,'@value'
                       ,'') 
              WHEN list_type = 'Combolist'
              THEN insertChildXML
                     (insertChildXML
                       (XMLELEMENT
                          ("Combolist"
                          ,XMLATTRIBUTES
                             (self.get_id                     AS "id"
                             ,self.get_label                  AS "label"
                             ,self.get_description            AS "description"   
                             ,self.get_refresh_on_change      AS "refreshOnChange"
                             ,self.get_enabled                AS "enabled"
                             ,self.get_required               AS "required"
                             ,self.get_visible                AS "visible"
                             ,decode(self.is_provider_rest_flag
                                    ,'Y'
                                    ,self.get_rest_href
                                    ,self.get_list_provider)  AS "listProvider"
                             ,self.get_values_in_payload      AS "valuesInPayload"
                             ,self.get_longlist               AS "longlist"
                             ,self.get_show_as_boolean        AS "showAsBoolean"
                             ,self.get_positive_boolean_value AS "positiveBooleanValue"
                             ,self.get_allow_new_values       AS "allowNewValues"
                             ,self.get_value                  AS "value"
                             ,self.get_transient              AS "transient"
                             ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                        nvl(self.get_enable_form_on_server_wait,'true')
                                  ELSE
                                        self.get_enable_form_on_server_wait
                              END  AS "enableFormOnServerWait")
                          ,value_set.get_xml())
                       ,'//Combolist[not(@positiveBooleanValue)]'
                       ,'@positiveBooleanValue'
                       ,'')
                     ,'//Combolist[not(@value)]'
                     ,'@value'
                     ,'')
         END                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS
  l_xml XMLTYPE;

BEGIN
  SELECT CASE WHEN list_type = 'Poplist'
              THEN insertChildXML
                     (insertChildXML
                       (XMLELEMENT
                          ("Poplist"
                          ,XMLATTRIBUTES
                             (self.get_id                     AS "id"
                             ,self.get_label                  AS "label"
                             ,self.get_description            AS "description"   
                             ,self.get_refresh_on_change      AS "refreshOnChange"
                             ,self.get_enabled                AS "enabled"
                             ,self.get_required               AS "required"
                             ,self.get_visible                AS "visible"
                             ,decode(self.is_provider_rest_flag
                                    ,'Y'
                                    ,self.get_rest_href
                                    ,self.get_list_provider)  AS "listProvider"
                             ,self.get_values_in_payload      AS "valuesInPayload"
                             ,self.get_show_as_boolean        AS "showAsBoolean"
                             ,self.get_positive_boolean_value AS "positiveBooleanValue"
                             ,self.get_allow_new_values       AS "allowNewValues"
                             ,self.get_value                  AS "value"
                             ,self.get_transient              AS "transient")
                          ,value_set.get_form_schema())
                         ,'//Poplist[not(@positiveBooleanValue)]'
                         ,'@positiveBooleanValue'
                         ,'') 
                       ,'//Poplist[not(@value)]'
                       ,'@value'
                       ,'') 
              WHEN list_type = 'Combolist'
              THEN insertChildXML
                     (insertChildXML
                       (XMLELEMENT
                          ("Combolist"
                          ,XMLATTRIBUTES
                             (self.get_id                     AS "id"
                             ,self.get_label                  AS "label"
                             ,self.get_description            AS "description"   
                             ,self.get_refresh_on_change      AS "refreshOnChange"
                             ,self.get_enabled                AS "enabled"
                             ,self.get_required               AS "required"
                             ,self.get_visible                AS "visible"
                             ,decode(self.is_provider_rest_flag
                                    ,'Y'
                                    ,self.get_rest_href
                                    ,self.get_list_provider)  AS "listProvider"
                             ,self.get_values_in_payload      AS "valuesInPayload"
                             ,self.get_longlist               AS "longlist"
                             ,self.get_show_as_boolean        AS "showAsBoolean"
                             ,self.get_positive_boolean_value AS "positiveBooleanValue"
                             ,self.get_allow_new_values       AS "allowNewValues"
                             ,self.get_value                  AS "value"
                             ,self.get_transient              AS "transient")
                          ,value_set.get_form_schema())
                       ,'//Combolist[not(@positiveBooleanValue)]'
                       ,'@positiveBooleanValue'
                       ,'')
                     ,'//Combolist[not(@value)]'
                     ,'@value'
                     ,'')
         END                               
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml        XMLTYPE;
  l_is_boolean boolean;

BEGIN

  l_is_boolean := (upper(nvl(self.get_show_as_boolean, 'FALSE')) = 'TRUE');
  
  IF l_is_boolean THEN
  
    SELECT XMLELEMENT
               ("field"
               ,XMLForest
                 ('checkbox'                   AS "type"
                 ,self.get_id                  AS "id"
               --  ,self.get_value               AS "value"
               --  ,self.get_old_value           AS "oldValue"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"  
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_state               AS "state"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_hide_if             AS "hideIf"
                 ,CASE WHEN self.get_rule is not null AND 
                            self.get_message is not null THEN
                            XMLElement
                            ("validationRule"
                               ,XMLAttributes
                               ('http://json.org/' as "xmlns:json"
                                ,'true' as "json:force-array")
                               ,XMLForest
                                ( self.get_rule  as "rule"
                                 ,self.get_message  as "message"
                                )
                             )                  
                    END      AS "validationRule"
                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                  END  AS "enableFormOnServerWait")
               ,value_set.get_json_friendly_cb(self.get_positive_boolean_value)
			   ,CASE when self.get_value = xxas_mss_util_pkg.get_g_number THEN
			     XMLElement("value", null)
			   when self.get_value IS NOT NULL THEN
				 XMLElement("value", self.get_value)
               END
              ,CASE when self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			     XMLElement("oldValue", null)
			   when self.get_old_value IS NOT NULL THEN
				 XMLElement("oldValue", self.get_old_value)
               END	
               ,CASE WHEN self.get_trigger_name is not null AND 
                          self.get_trigger_method is not null THEN
                  XMLElement
                   ("links"
                   ,XMLElement
                     ("trigger"
                     ,XMLElement
                       ("href"
                       ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                       )
                     ,XMLElement
                       ("method"
                       ,self.get_trigger_method
                       )
                     )
                   )
                END
               )
    INTO   l_xml   
    FROM   dual;
  
  ELSIF nvl(self.get_multiple_selection, 'false') <> 'true' THEN

    SELECT XMLELEMENT
               ("field"
               ,XMLForest
                 (CASE WHEN self.get_show_as_radio = 'true' THEN
                            'radio'
                       ELSE 'select-single'
                  END AS "type"
                 ,self.get_id                  AS "id"
                -- ,self.get_value               AS "value"
                -- ,self.get_old_value           AS "oldValue"
                 ,self.get_label               AS "label"
                 ,CASE WHEN nvl(self.get_longlist, 'false') = 'true'
                       THEN
                         xxas_com_msg_pkg.get_string('XXAS_EMP_NO_RESULTS')                    
                  END                          AS "noDataMessage"
				 ,xxas_com_msg_pkg.get_string('XXAS_EMP_NO_TEXT') 
											   AS "noTextMessage"
				 ,xxas_com_msg_pkg.get_string('XXAS_EMP_SEARCH_IN_PROGRESS') 
											   AS "searchInProgressMessage"							   
                 ,CASE WHEN nvl(self.get_longlist, 'false') = 'true'
                       THEN
                         self.get_empty_value
                       ELSE
                         xxas_com_msg_pkg.get_string('XXAS_EMP_UI_SELECT_ONE')                       
                  END                          AS "placeholder"
                 ,CASE WHEN nvl(self.get_longlist, 'false') = 'true'
                       THEN self.get_allow_new_values
                  END                          AS "allowNewValues"
				 ,CASE WHEN (nvl(self.get_longlist, 'false') = 'true' and nvl(self.get_allow_new_values, 'false') = 'true')
					   THEN xxas_com_msg_pkg.get_string('XXAS_EMP_CREATE_VALUE')
				  END						   AS "createMessage"
                 ,self.get_description         AS "description"  
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_state               AS "state"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_hide_if             AS "hideIf"
                 ,self.get_style               AS "style"
                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                  END  AS "enableFormOnServerWait")
			   ,CASE WHEN self.get_value = xxas_mss_util_pkg.get_g_number THEN
			           XMLElement("value", null)
			         WHEN self.get_value IS NOT NULL THEN
			 	       XMLElement("value", self.get_value)
                END
               ,CASE WHEN self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			           XMLElement("oldValue", null)
			         WHEN self.get_old_value IS NOT NULL THEN
			 	       XMLElement("oldValue", self.get_old_value)
                END	
               ,value_set.get_json_friendly()
               ,CASE WHEN self.get_trigger_name is not null AND 
                          self.get_trigger_method is not null AND 
                          nvl(self.get_longlist, 'false') = 'false' 
                     THEN XMLElement
                           ("links"
                           ,XMLElement
                             ("trigger"
                             ,XMLElement
                               ("href"
                               ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                               )
                             ,XMLElement
                               ("method"
                               ,self.get_trigger_method
                               )
                             )
                           )
                     WHEN self.get_trigger_name is not null AND 
                          self.get_trigger_method is not null AND 
                          nvl(self.get_longlist, 'false') = 'true'
                     THEN XMLElement
                           ("links"
                           ,XMLElement
                             ("trigger"
                             ,XMLElement
                               ("href"
                               ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                               )
                             ,XMLElement
                               ("method"
                               ,self.get_trigger_method
                               )
                             )
                           ,XMLElement
                             ("values"
                             ,XMLElement
                               ("href"
                               ,self.get_longlist_href)
                             )
                           )
                     WHEN NOT(self.get_trigger_name is not null AND 
                              self.get_trigger_method is not null) AND 
                          nvl(self.get_longlist, 'false') = 'false'
                     THEN null
                     WHEN NOT(self.get_trigger_name is not null AND 
                              self.get_trigger_method is not null) AND 
                          nvl(self.get_longlist, 'false') = 'true'
                     THEN XMLElement
                           ("links"
                           ,XMLElement
                             ("values"
                             ,XMLElement
                               ("href"
                               ,self.get_longlist_href)
                             )
                           )
                END
               )
    INTO   l_xml   
    FROM   dual;
    
  ELSE
  
    SELECT XMLELEMENT
               ("field"
               ,XMLForest
                 ('select-multiple'            AS "type"
                 ,self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,CASE WHEN nvl(self.get_longlist, 'false') = 'true'
                       THEN
                         xxas_com_msg_pkg.get_string('XXAS_EMP_NO_RESULTS')                    
                  END                          AS "noDataMessage"
				 ,xxas_com_msg_pkg.get_string('XXAS_EMP_NO_TEXT') 
											   AS "noTextMessage"
                 ,xxas_com_msg_pkg.get_string('XXAS_EMP_SEARCH_IN_PROGRESS') 
											   AS "searchInProgressMessage"
				 ,CASE WHEN nvl(self.get_longlist, 'false') = 'true'
                       THEN
                         self.get_empty_value
                       ELSE
                         xxas_com_msg_pkg.get_string('XXAS_EMP_UI_SELECT_MORE')                        
                  END                          AS "placeholder"
                 ,self.get_description         AS "description"  
                 ,self.get_refresh_on_change   AS "refreshOnChange"
                 ,self.get_state               AS "state"
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_hide_if             AS "hideIf"
                 ,self.get_style               AS "style"
                 ,self.get_allow_new_values    AS "allowNewValues"
				         ,CASE WHEN nvl(self.get_allow_new_values, 'false') = 'true'
                       THEN xxas_com_msg_pkg.get_string('XXAS_EMP_CREATE_VALUE')
				         END						   AS "createMessage"
                ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                END  AS "enableFormOnServerWait")
               ,self.get_values_xml
               ,value_set.get_json_friendly()
               ,CASE WHEN self.get_trigger_name is not null AND 
                          self.get_trigger_method is not null AND 
                          nvl(self.get_longlist, 'false') = 'false' 
                     THEN XMLElement
                           ("links"
                           ,XMLElement
                             ("trigger"
                             ,XMLElement
                               ("href"
                               ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                               )
                             ,XMLElement
                               ("method"
                               ,self.get_trigger_method
                               )
                             )
                           )
                     WHEN self.get_trigger_name is not null AND 
                          self.get_trigger_method is not null AND 
                          nvl(self.get_longlist, 'false') = 'true'
                     THEN XMLElement
                           ("links"
                           ,XMLElement
                             ("trigger"
                             ,XMLElement
                               ("href"
                               ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                               )
                             ,XMLElement
                               ("method"
                               ,self.get_trigger_method
                               )
                             )
                           ,XMLElement
                             ("values"
                             ,XMLElement
                               ("href"
                               ,self.get_longlist_href)
                             )
                           )
                     WHEN NOT(self.get_trigger_name is not null AND 
                              self.get_trigger_method is not null) AND 
                          nvl(self.get_longlist, 'false') = 'false'
                     THEN null
                     WHEN NOT(self.get_trigger_name is not null AND 
                              self.get_trigger_method is not null) AND 
                          nvl(self.get_longlist, 'false') = 'true'
                     THEN XMLElement
                           ("links"
                           ,XMLElement
                             ("values"
                             ,XMLElement
                               ("href"
                               ,self.get_longlist_href)
                             )
                           )
                END
               )
    INTO   l_xml   
    FROM   dual;
  
  END IF;

  RETURN l_xml;

END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_container_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_container_type
  (p_ui_container_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Container' 
                  PASSING p_ui_container_xml
                  COLUMNS id                         VARCHAR2(4000) path '@id',
                          label                      VARCHAR2(4000) path '@label',
                          description                VARCHAR2(4000) path '@description',
                          visible                    VARCHAR2(4000) path '@visible',
                          hide_if                    VARCHAR2(4000) path '@hideIf',
                          container_type             VARCHAR2(4000) path '@type',
                          review_only                VARCHAR2(4000) path '@reviewOnly',
                          trigger_name               VARCHAR2(4000) path '@triggerName',
                          trigger_method             VARCHAR2(4000) path '@triggerMethod',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');
  
  CURSOR c_fields
  IS
  SELECT CASE WHEN existsNode(field, '/Container')  = 1
              THEN xxas_emp_ui_container_type(field)
              WHEN existsNode(field, '/MultiRow')  = 1
              THEN xxas_emp_ui_multirow_type(field)
              WHEN existsNode(field,'/Rating')  = 1
              THEN xxas_emp_ui_rating_type(field)
              WHEN existsNode(field, '/Flexfield')  = 1
              THEN xxas_emp_ui_flexfield_type(field)
              WHEN existsNode(field, '/OLMTest')  = 1
              THEN xxas_emp_ui_olmtest_type(field)
              WHEN existsNode(field, '/OLMQuestion')  = 1
              THEN xxas_emp_ui_olmquestion_type(field)
              WHEN existsNode(field, '/TextInput')  = 1
              THEN xxas_emp_ui_textinput_type(field)
              WHEN existsNode(field, '/Submit')  = 1
              THEN xxas_emp_ui_submit_type(field)
              WHEN existsNode(field, '/Html')  = 1
              THEN xxas_emp_ui_html_type(field)
              WHEN existsNode(field, '/Item')  = 1
              THEN xxas_emp_ui_item_type(field)
              WHEN existsNode(field, '/ListItem')  = 1
              THEN xxas_emp_ui_listitem_type(field)
              WHEN existsNode(field, '/Link')  = 1
              THEN xxas_emp_ui_link_type(field)
              WHEN existsNode(field, '/Address')  = 1
              THEN xxas_emp_ui_address_type(field)
              WHEN existsNode(field, '/Password')  = 1
              THEN xxas_emp_ui_password_type(field)
              WHEN existsNode(field, '/ColorPicker')  = 1
              THEN xxas_emp_ui_color_picker_type(field)
              WHEN existsNode(field, '/Icon')  = 1
              THEN xxas_emp_ui_icon_type(field)
              WHEN existsNode(field, '/Email')  = 1
              THEN xxas_emp_ui_email_type(field)
              WHEN existsNode(field, '/TextArea')  = 1
              THEN xxas_emp_ui_textarea_type(field)
              WHEN existsNode(field, '/Number')     = 1
              THEN xxas_emp_ui_number_type(field)
              WHEN existsNode(field, '/Attachment') = 1
              THEN xxas_emp_ui_attachment_type(field) 
              WHEN existsNode(field, '/File') = 1
              THEN xxas_emp_ui_file_type(field) 
              WHEN existsNode(field, '/Hidden') = 1
              THEN xxas_emp_ui_hidden_type(field) 
              WHEN existsNode(field, '/Date')       = 1 OR  
                   existsNode(field, '/DateTime')   = 1 OR 
                   existsNode(field, '/Time')       = 1 
              THEN xxas_emp_ui_date_type(field)  
              WHEN existsNode(field, '/DateRange')       = 1 OR  
                   existsNode(field, '/DateTimeRange')   = 1 OR 
                   existsNode(field, '/TimeRange')       = 1 
              THEN xxas_emp_ui_date_range_type(field)
              WHEN existsNode(field, '/Divider') = 1
              THEN xxas_emp_ui_divider_type(field) 
              WHEN existsNode(field, '/Poplist')    = 1 OR
                   existsNode(field, '/Combolist')  = 1
              THEN xxas_emp_ui_list_type(field)
         END field
  FROM   XMLTABLE('/Container/*' 
                  PASSING p_ui_container_xml
                  COLUMNS field       XMLTYPE       path '.');
  
  l_data c_data%ROWTYPE;

BEGIN
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_review_only(l_data.review_only);
  self.set_type(l_data.container_type);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
    
  field_list := xxas_emp_ui_component_table();
  
  FOR csr_rec IN c_fields
  LOOP
    field_list.EXTEND;
    field_list(field_list.COUNT) := csr_rec.field;
  END LOOP;

  RETURN; 
END;

CONSTRUCTOR FUNCTION xxas_emp_ui_container_type
  (p_id             IN VARCHAR2
  ,p_label          IN VARCHAR2
  ,p_description    IN VARCHAR2
  ,p_visible        IN VARCHAR2
  ,p_hide_if        IN VARCHAR2
  ,p_review_only    IN VARCHAR2
  ,p_type           IN VARCHAR2
  ,p_trigger_name   IN VARCHAR2
  ,p_trigger_method IN VARCHAR2
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_review_only(p_review_only);
  self.set_type(p_type);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
  
  field_list := xxas_emp_ui_component_table();
  
  RETURN;
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_review_only
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('review_only', lower(p_value));
END;

MEMBER FUNCTION get_review_only RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('review_only'));
END;

MEMBER PROCEDURE set_type
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('container_type', p_value);
END;

MEMBER FUNCTION get_type RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('container_type');
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  
BEGIN
  
  SELECT XMLAGG(f.get_xml())
  INTO   l_xml
  FROM   TABLE(field_list) f;
  
  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS
  l_xml XMLTYPE;
  
BEGIN
  SELECT XMLELEMENT("Container"
                   ,XMLATTRIBUTES(self.get_id                AS "id"
                                 ,self.get_label             AS "label"
                                 ,self.get_description       AS "description"
                                 ,self.get_visible           AS "visible"
                                 ,self.get_review_only       AS "reviewOnly"))
  INTO   l_xml   
  FROM   dual;
  
  FOR i IN 1..field_list.COUNT
  LOOP
    SELECT APPENDCHILDXML(l_xml
                         ,'/Container'
                         ,field_list(i).get_form_schema())
             
    INTO   l_xml
    FROM   dual;
  END LOOP;
  
  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  
BEGIN
  
  SELECT XMLELEMENT("field"
                   ,XMLForest
                     (nvl(self.get_type, 'fieldset') AS "type"
                     ,self.get_id                    AS "id"
                     ,self.get_label                 AS "label"
                     ,self.get_description           AS "description"
                     ,self.get_state                 AS "state"
                     ,self.get_visible               AS "visible"
                     ,self.get_hide_if               AS "hideIf"
                     ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     END  AS "enableFormOnServerWait")
                   ,XMLElement
                     ("fields"
                     ,XMLAttributes
                       ('true' as "json:force-array"
                       ,'http://json.org/' as "xmlns:json")
                     ,XMLAGG(f.get_json_friendly()))
                   ,CASE WHEN self.get_trigger_name is not null AND 
                              self.get_trigger_method is not null THEN
                      XMLElement
                       ("links"
                       ,XMLElement
                         ("trigger"
                         ,XMLElement
                           ("href"
                           ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                           )
                         ,XMLElement
                           ("method"
                           ,self.get_trigger_method
                           )
                         )
                       )
                    END
                   )
  INTO   l_xml
  FROM   TABLE(field_list) f;
  
  RETURN l_xml;

END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_flexfield_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_flexfield_type
  (p_ui_flexfield_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Flexfield' 
                  PASSING p_ui_flexfield_xml
                  COLUMNS id                         VARCHAR2(250) path '@id',
                          application_id             NUMBER        path '@applicationId',
                          application_short_code     VARCHAR2(30)  path '@applicationShortCode',
                          flex_code                  VARCHAR2(4)   path '@flexCode',
                          flex_structure_code        VARCHAR2(30)  path '@flexStructureCode',
                          descriptive_flexfield_name VARCHAR2(40)  path '@descriptiveFlexfieldName',
                          descriptive_flex_context   VARCHAR2(30)  path '@descriptiveFlexContextCode',
                          column_name_prefix         VARCHAR2(30)  path '@columnNamePrefix');
  
  l_data c_data%ROWTYPE;
  
BEGIN
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  
  application_id             := l_data.application_id;
  application_short_code     := l_data.application_short_code;
  flex_code                  := l_data.flex_code;
  flex_structure_code        := l_data.flex_structure_code;
  descriptive_flexfield_name := l_data.descriptive_flexfield_name;
  descriptive_flex_context   := l_data.descriptive_flex_context;
  column_name_prefix         := l_data.column_name_prefix;
    
  field_list := xxas_emp_ui_component_table();
  
  --
  -- Derive the application id if not specified
  --
  IF application_id IS NULL AND application_short_code IS NOT NULL THEN
    application_id := xxas_util_pkg.get_app_id(application_short_code);
  END IF;
  
  RETURN; 
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  
BEGIN
  SELECT XMLELEMENT("Flexfield"
                   ,XMLATTRIBUTES(self.get_id                AS "id"
                                 ,application_id             AS "applicationId"
                                 ,flex_code                  AS "flexCode"
                                 ,flex_structure_code        AS "flexStructureCode"
                                 ,descriptive_flexfield_name AS "descriptiveFlexfieldName"
                                 ,descriptive_flex_context   AS "descriptiveFlexContextCode"
                                 ,column_name_prefix         AS "columnNamePrefix"))
  INTO   l_xml   
  FROM   dual;
  
  FOR i IN 1..field_list.COUNT
  LOOP
    SELECT APPENDCHILDXML(l_xml
                         ,'/Flexfield'
                         ,field_list(i).get_xml())
             
    INTO   l_xml
    FROM   dual;
  END LOOP;
  
  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS
  l_xml XMLTYPE;

BEGIN

  SELECT XMLAgg(f.get_form_schema())
  INTO   l_xml
  FROM   TABLE(field_list) f;
  
  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLAgg(f.get_json_friendly())
  INTO   l_xml
  FROM   TABLE(field_list) f;
  
  RETURN l_xml;

END;

OVERRIDING MEMBER FUNCTION get_obj_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  
BEGIN

  SELECT XMLAgg(f.get_obj_xml())
  INTO   l_xml
  FROM   TABLE(field_list) f;
  
  RETURN l_xml;
END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_olmtest_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_olmtest_type
  (p_ui_olmtest_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/OLMTest' 
                  PASSING p_ui_olmtest_xml
                  COLUMNS id                         VARCHAR2(4000) path '@id',
                          value                      VARCHAR2(4000) path '@value'); 
 
  l_data c_data%ROWTYPE;
  
BEGIN
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id   (l_data.id);
  self.set_value(l_data.value);
    
  field_list := xxas_emp_ui_component_table();
  
  RETURN; 
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE
IS 
BEGIN
  RETURN null;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS
BEGIN
  RETURN null;
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLAgg(f.get_json_friendly())
  INTO   l_xml
  FROM   TABLE(field_list) f;
  
  RETURN l_xml;

END;

OVERRIDING MEMBER FUNCTION get_obj_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  
BEGIN

  SELECT XMLAgg(f.get_obj_xml())
  INTO   l_xml
  FROM   TABLE(field_list) f;
  
  RETURN l_xml;
END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_emp_ui_olmquestion_type IS

CONSTRUCTOR FUNCTION xxas_emp_ui_olmquestion_type
  (p_ui_olmquestion_xml IN XMLTYPE) RETURN SELF AS RESULT
IS

  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/OLMQuestion' 
                  PASSING p_ui_olmquestion_xml
                  COLUMNS id                         VARCHAR2(4000) path '@id',
                          question_ref_id            VARCHAR2(4000) path '@questionRefId',
                          test_ref_id                VARCHAR2(4000) path '@testRefId',
                          value                      VARCHAR2(4000) path '@value',
                          trigger_name               VARCHAR2(4000) path '@triggerName',
                          trigger_method             VARCHAR2(4000) path '@triggerMethod',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait'); 
  l_data c_data%ROWTYPE;

BEGIN

  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;

  self.set_id   (l_data.id);
  self.set_question_ref_id(l_data.question_ref_id);
  self.set_test_ref_id(l_data.test_ref_id);
  self.set_value(l_data.value);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);

  field_list := xxas_emp_ui_component_table();

  RETURN;

END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_question_id
  (p_value IN NUMBER)
IS 
BEGIN
   self.question_id := p_value;
END;

MEMBER FUNCTION get_question_id RETURN NUMBER
IS 
BEGIN
   RETURN self.question_id;
END;

MEMBER PROCEDURE set_test_id
  (p_value IN NUMBER)
IS 
BEGIN
   self.test_id := p_value;
END;

MEMBER FUNCTION get_test_id RETURN NUMBER
IS 
BEGIN
   RETURN self.test_id;
END;

MEMBER PROCEDURE set_question_ref_id
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('question_ref_id', p_value);
END;

MEMBER FUNCTION get_question_ref_id RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('question_ref_id');
END;

MEMBER PROCEDURE set_test_ref_id
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('test_ref_id', p_value);
END;

MEMBER FUNCTION get_test_ref_id RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('test_ref_id');
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE
IS 
BEGIN
  RETURN null;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS
BEGIN
  RETURN null;
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml           XMLTYPE;
  l_response_type ota_response_types.type_flag%TYPE;
  l_textarea      xxas_emp_ui_textarea_type;
  l_number_type   xxas_emp_ui_number_type;
  l_list_type     xxas_emp_ui_list_type;
  l_values        xxas_emp_ui_value_table;  
  l_boolean_true  CONSTANT VARCHAR2(4)  := 'true';
  l_boolean_false CONSTANT VARCHAR2(5)  := 'false';
  l_TRUE            VARCHAR2(30) := 'True';
  l_FALSE           VARCHAR2(30) := 'False';

  CURSOR csr_question_info
    (cp_question_id IN number
    ,cp_test_id     IN number)
  IS
  SELECT ots.test_id
        ,decode
          (nvl(ots.mandatory_flag, 'N')
          ,'Y', l_boolean_true
          ,l_boolean_false) mandatory_flag
        ,oq.question_id
        ,oq.text
        ,ort.response_type_id
        ,ort.type_flag response_type_flag
  FROM  ota_test_sections ots
       ,ota_test_questions otq
       ,ota_questions oq
       ,ota_response_types ort
  WHERE trunc(sysdate) BETWEEN
        nvl(ots.start_date_active, trunc(sysdate)) AND
        nvl(ots.end_date_active, trunc(sysdate))
  AND   ots.test_id = cp_test_id
  AND   ots.test_id = otq.test_id
  AND   ots.test_section_id = otq.section_id
  AND   oq.question_id = cp_question_id
  AND   otq.question_id = oq.question_id
  AND   trunc(sysdate) BETWEEN
        nvl(oq.start_date_active, trunc(sysdate)) AND
        nvl(oq.end_date_active, trunc(sysdate))
  AND   oq.question_id = ort.question_id
  ORDER BY ots.sequence, otq.sequence;

  CURSOR csr_resp_values
    (cp_response_type_id IN number)
  IS
  SELECT orv.response_value_id
        ,orv.text
  FROM   ota_response_values orv
  WHERE  orv.response_type_id = cp_response_type_id
  AND    trunc(sysdate) BETWEEN
         nvl(orv.start_date_active, trunc(sysdate)) AND
         nvl(orv.end_date_active, trunc(sysdate))
  ORDER BY orv.sequence;

  l_question_info csr_question_info%ROWTYPE;

BEGIN

  IF self.question_id IS NULL OR self.test_id IS NULL THEN
    RETURN NULL;
  END IF;

  OPEN  csr_question_info(self.question_id
                         ,self.test_id);
  FETCH csr_question_info INTO l_question_info;
  CLOSE csr_question_info;

  l_response_type := l_question_info.response_type_flag;


  IF l_response_type = 'F' OR
     l_response_type = 'X'
  THEN
    --
    -- F = Fill-in-the-Blank(Text Response)
    -- X =  Free text (Not Scored)
    --
    l_textarea := xxas_emp_ui_textarea_type
        (p_id                => self.get_id
        ,p_label             => l_question_info.text
        ,p_description       => null
        ,p_refresh_on_change => l_boolean_false
        ,p_enabled           => l_boolean_true
        ,p_required          => l_question_info.mandatory_flag
        ,p_visible           => l_boolean_true 
        ,p_hide_if           => null
        ,p_max_chars         => 4000
        ,p_regexp            => null
        ,p_regexp_error      => null
        ,p_empty_value       => null
        ,p_value             => self.get_value
        ,p_transient         => null
        ,p_rich_text         => l_boolean_false
		,p_autocomplete_hint => NULL
        ,p_no_autocomplete   => null
        ,p_no_autocorrect    => null
        ,p_no_autocapitalize => null
        ,p_no_spell_check    => null
        ,p_enable_form_on_server_wait => CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                                    nvl(self.get_enable_form_on_server_wait,'true')
                                         END
        );
        
    RETURN l_textarea.get_json_friendly();
  
  ELSIF l_response_type = 'B'
  THEN
    --
    -- B = Fill-in-the-Blank(Numeric Response)
    --
    l_number_type :=
       xxas_emp_ui_number_type
        (p_id                => self.get_id
        ,p_label             => l_question_info.text
        ,p_description       => null
        ,p_refresh_on_change => l_boolean_false
        ,p_trigger_name      => self.get_trigger_name
        ,p_trigger_method    => self.get_trigger_method
        ,p_enabled           => l_boolean_true
        ,p_required          => l_question_info.mandatory_flag
        ,p_visible           => l_boolean_true 
        ,p_hide_if           => null
        ,p_value             => self.get_value
        ,p_decimal_precision => null
        ,p_regexp            => null
        ,p_regexp_error      => null
        ,p_transient         => null
		,p_number_format     => null
    ,p_enable_form_on_server_wait => CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                                    nvl(self.get_enable_form_on_server_wait,'true')
                                     END
    );

    RETURN l_number_type.get_json_friendly();
        
  ELSIF l_response_type = 'T' OR
        l_response_type = 'S'
  THEN
    --
    -- T = True or False
    -- S = Multiple Choice(Single Correct Response)
    --
    l_values := xxas_emp_ui_value_table();
    
    --
    -- Populate the values
    --        
    FOR resp_rec IN csr_resp_values(l_question_info.response_type_id)
    LOOP
      l_values.EXTEND;
      IF l_response_type = 'T' THEN
        IF resp_rec.text = 'T' THEN
          l_values(l_values.COUNT) :=
            xxas_emp_ui_value_type(resp_rec.text, l_TRUE, null, null);             
        ELSE
          l_values(l_values.COUNT) :=
            xxas_emp_ui_value_type(resp_rec.text, l_FALSE, null, null);                         
        END IF;
      ELSE
        l_values(l_values.COUNT) :=
          xxas_emp_ui_value_type(resp_rec.text, resp_rec.text, null, null);
      END IF;              
    END LOOP;
    
    l_list_type :=
      xxas_emp_ui_list_type
        (p_list_type              => 'Poplist'
        ,p_id                     => self.get_id
        ,p_label                  => l_question_info.text
        ,p_description            => null
        ,p_refresh_on_change      => l_boolean_false
        ,p_trigger_name           => self.get_trigger_name
        ,p_trigger_method         => self.get_trigger_method
        ,p_enabled                => l_boolean_true
        ,p_required               => l_question_info.mandatory_flag
        ,p_visible                => l_boolean_true
        ,p_hide_if                => null
        ,p_list_provider          => null
        ,p_values_in_payload      => l_boolean_true
        ,p_longlist               => l_boolean_false
        ,p_show_as_radio          => l_boolean_false
        ,p_show_as_boolean        => l_boolean_false
        ,p_positive_boolean_value => null
        ,p_multiple_selection     => l_boolean_false
        ,p_style                  => null
        ,p_allow_new_values       => l_boolean_false
        ,p_empty_value            => null
        ,p_value                  => self.get_value
        ,p_transient              => null
        ,p_value_set              => xxas_emp_ui_valueset_type(l_values)
        ,p_enable_form_on_server_wait => CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                                    nvl(self.get_enable_form_on_server_wait,'true')
                                         END
        );

    RETURN l_list_type.get_json_friendly();
        
  END IF;

  RETURN NULL;

END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_createfields_type is
  
CONSTRUCTOR FUNCTION xxas_emp_ui_createfields_type
  (p_ui_createfields_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  
  CURSOR c_data
  IS
  SELECT CASE WHEN existsNode(field, '/Stepper')  = 1
              THEN xxas_emp_ui_stepper_type(field)
              WHEN existsNode(field, '/Container')  = 1
              THEN xxas_emp_ui_container_type(field)
              WHEN existsNode(field, '/MultiRow')  = 1
              THEN xxas_emp_ui_multirow_type(field)
              WHEN existsNode(field,'/Rating')  = 1
              THEN xxas_emp_ui_rating_type(field)
              WHEN existsNode(field, '/Flexfield')  = 1
              THEN xxas_emp_ui_flexfield_type(field)
              WHEN existsNode(field, '/OLMTest')  = 1
              THEN xxas_emp_ui_olmtest_type(field)
              WHEN existsNode(field, '/OLMQuestion')  = 1
              THEN xxas_emp_ui_olmquestion_type(field)
              WHEN existsNode(field, '/TextInput')  = 1
              THEN xxas_emp_ui_textinput_type(field)
              WHEN existsNode(field, '/Submit')  = 1
              THEN xxas_emp_ui_submit_type(field)
              WHEN existsNode(field, '/Html')  = 1
              THEN xxas_emp_ui_html_type(field)
              WHEN existsNode(field, '/Item')  = 1
              THEN xxas_emp_ui_item_type(field)
              WHEN existsNode(field, '/ListItem')  = 1
              THEN xxas_emp_ui_listitem_type(field)
              WHEN existsNode(field, '/Link')  = 1
              THEN xxas_emp_ui_link_type(field)
              WHEN existsNode(field, '/Address')  = 1
              THEN xxas_emp_ui_address_type(field)
              WHEN existsNode(field, '/Password')  = 1
              THEN xxas_emp_ui_password_type(field)
              WHEN existsNode(field, '/ColorPicker')  = 1
              THEN xxas_emp_ui_color_picker_type(field)
              WHEN existsNode(field, '/Icon')  = 1
              THEN xxas_emp_ui_icon_type(field)
              WHEN existsNode(field, '/Email')  = 1
              THEN xxas_emp_ui_email_type(field)
              WHEN existsNode(field, '/TextArea')  = 1
              THEN xxas_emp_ui_textarea_type(field)
              WHEN existsNode(field, '/Number')     = 1
              THEN xxas_emp_ui_number_type(field)
              WHEN existsNode(field, '/Attachment') = 1
              THEN xxas_emp_ui_attachment_type(field)
              WHEN existsNode(field, '/File') = 1
              THEN xxas_emp_ui_file_type(field)               
              WHEN existsNode(field, '/Hidden') = 1
              THEN xxas_emp_ui_hidden_type(field) 
              WHEN existsNode(field, '/Date')       = 1 OR  
                   existsNode(field, '/DateTime')   = 1 OR 
                   existsNode(field, '/Time')       = 1 
              THEN xxas_emp_ui_date_type(field) 
              WHEN existsNode(field, '/DateRange')       = 1 OR  
                   existsNode(field, '/DateTimeRange')   = 1 OR 
                   existsNode(field, '/TimeRange')       = 1 
              THEN xxas_emp_ui_date_range_type(field)
              WHEN existsNode(field, '/Divider') = 1
              THEN xxas_emp_ui_divider_type(field) 
              WHEN existsNode(field, '/Poplist')    = 1 OR
                   existsNode(field, '/Combolist')  = 1
              THEN xxas_emp_ui_list_type(field)
         END field
  FROM   XMLTABLE('/CreateFields/*' 
                  PASSING p_ui_createfields_xml
                  COLUMNS field       XMLTYPE       path '.');
                  
  l_object_type XMLType;
                  
BEGIN
  field_list := xxas_emp_ui_component_table(); 

  FOR l_data IN c_data
  LOOP
    field_list.EXTEND;
    field_list(field_list.COUNT) := l_data.field;
  END LOOP;
     
  RETURN; 
END;

MEMBER FUNCTION has_fields RETURN BOOLEAN
IS
  
BEGIN
  RETURN (field_list IS NOT NULL AND field_list.count > 0);
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  
BEGIN
  
  SELECT XMLELEMENT("CreateFields"
                   ,XMLAGG(f.get_xml()))
  INTO   l_xml
  FROM   TABLE(field_list) f;

  RETURN l_xml;
END;
  
OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS
  l_xml XMLTYPE;

BEGIN

  SELECT XMLAGG(f.get_form_schema())
  INTO   l_xml
  FROM   TABLE(field_list) f;

  RETURN l_xml;
  
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS
  l_xml XMLTYPE;

BEGIN

  SELECT XMLAGG(f.get_json_friendly())
  INTO   l_xml
  FROM   TABLE(field_list) f;

  RETURN l_xml;
  
END;
  
OVERRIDING MEMBER FUNCTION get_obj_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  
BEGIN
  
  SELECT XMLAGG(f.get_obj_xml())
  INTO   l_xml
  FROM   TABLE(field_list) f;

  RETURN l_xml;
END;
  
END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_editfields_type is
  
CONSTRUCTOR FUNCTION xxas_emp_ui_editfields_type
  (p_ui_editfields_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  
  CURSOR c_data
  IS
  SELECT CASE WHEN existsNode(field, '/Stepper')  = 1
              THEN xxas_emp_ui_stepper_type(field)
              WHEN existsNode(field, '/Container')  = 1
              THEN xxas_emp_ui_container_type(field)
              WHEN existsNode(field, '/MultiRow')  = 1
              THEN xxas_emp_ui_multirow_type(field)
              WHEN existsNode(field,'/Rating')  = 1
              THEN xxas_emp_ui_rating_type(field)
              WHEN existsNode(field, '/Flexfield')  = 1
              THEN xxas_emp_ui_flexfield_type(field)
              WHEN existsNode(field, '/OLMTest')  = 1
              THEN xxas_emp_ui_olmtest_type(field)
              WHEN existsNode(field, '/OLMQuestion')  = 1
              THEN xxas_emp_ui_olmquestion_type(field)
              WHEN existsNode(field, '/TextInput')  = 1
              THEN xxas_emp_ui_textinput_type(field)
              WHEN existsNode(field, '/Submit')  = 1
              THEN xxas_emp_ui_submit_type(field)
              WHEN existsNode(field, '/Html')  = 1
              THEN xxas_emp_ui_html_type(field)
              WHEN existsNode(field, '/Item')  = 1
              THEN xxas_emp_ui_item_type(field)
              WHEN existsNode(field, '/ListItem')  = 1
              THEN xxas_emp_ui_listitem_type(field)
              WHEN existsNode(field, '/Link')  = 1
              THEN xxas_emp_ui_link_type(field)
              WHEN existsNode(field, '/Address')  = 1
              THEN xxas_emp_ui_address_type(field)
              WHEN existsNode(field, '/Password')  = 1
              THEN xxas_emp_ui_password_type(field)
              WHEN existsNode(field, '/ColorPicker')  = 1
              THEN xxas_emp_ui_color_picker_type(field)
              WHEN existsNode(field, '/Icon')  = 1
              THEN xxas_emp_ui_icon_type(field)
              WHEN existsNode(field, '/Email')  = 1
              THEN xxas_emp_ui_email_type(field)
              WHEN existsNode(field, '/TextArea')  = 1
              THEN xxas_emp_ui_textarea_type(field)
              WHEN existsNode(field, '/Number')     = 1
              THEN xxas_emp_ui_number_type(field)
              WHEN existsNode(field, '/Attachment') = 1
              THEN xxas_emp_ui_attachment_type(field) 
              WHEN existsNode(field, '/File') = 1
              THEN xxas_emp_ui_file_type(field) 
              WHEN existsNode(field, '/Hidden') = 1
              THEN xxas_emp_ui_hidden_type(field) 
              WHEN existsNode(field, '/Date')       = 1 OR  
                   existsNode(field, '/DateTime')   = 1 OR 
                   existsNode(field, '/Time')       = 1 
              THEN xxas_emp_ui_date_type(field) 
              WHEN existsNode(field, '/DateRange')       = 1 OR  
                   existsNode(field, '/DateTimeRange')   = 1 OR 
                   existsNode(field, '/TimeRange')       = 1 
              THEN xxas_emp_ui_date_range_type(field)
              WHEN existsNode(field, '/Divider') = 1
              THEN xxas_emp_ui_divider_type(field) 
              WHEN existsNode(field, '/Poplist')    = 1 OR
                   existsNode(field, '/Combolist')  = 1
              THEN xxas_emp_ui_list_type(field)
         END field
  FROM   XMLTABLE('/EditFields/*' 
                  PASSING p_ui_editfields_xml
                  COLUMNS field       XMLTYPE       path '.');
                  
  l_object_type XMLType;
                  
BEGIN
  field_list := xxas_emp_ui_component_table();  
  
  FOR l_data IN c_data
  LOOP
    field_list.EXTEND;
    field_list(field_list.COUNT) := l_data.field;
  END LOOP;
     
  RETURN; 
END;

MEMBER FUNCTION has_fields RETURN BOOLEAN
IS
  
BEGIN
  RETURN (field_list IS NOT NULL AND field_list.count > 0);
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  
BEGIN
  
  SELECT XMLELEMENT("EditFields"
                   ,XMLAGG(f.get_xml()))
  INTO   l_xml
  FROM   TABLE(field_list) f;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  
BEGIN
  
  SELECT XMLAGG(f.get_form_schema())
  INTO   l_xml
  FROM   TABLE(field_list) f;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;

BEGIN

  SELECT XMLAGG(f.get_json_friendly())
  INTO   l_xml
  FROM   TABLE(field_list) f;

  RETURN l_xml;

END;
  
OVERRIDING MEMBER FUNCTION get_obj_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  
BEGIN
  
  SELECT XMLAGG(f.get_obj_xml())
  INTO   l_xml
  FROM   TABLE(field_list) f;

  RETURN l_xml;
END;
  
END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_view_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_view_type
  (p_ui_view_xml IN XMLTYPE) RETURN SELF AS RESULT

IS

  CURSOR c_data
  IS
  SELECT id
  ,      object_type
  ,      enabled
  ,      delete_allowed
  ,      delete_label
  ,      next_label
  ,      show_submit
  ,      submit_label
  ,      submit_button_icon
  ,      submit_button_color
  ,      form_help
  ,      review_help
  ,      post_to_href
  ,      client_autosave
  ,      client_autosave_reset_text
  ,      client_autosave_reset_btn
  ,      form_close_warning  
  ,      server_autosave
  ,      server_autosave_timeout
  ,      server_autosave_message
  ,      send_basic_auth
  ,      multi_factor_auth
  ,      send_token
  ,      basic_auth_un_field
  ,      basic_auth_pw_field
  ,      token_field
  ,      show_changes
  ,      xxas_emp_ui_createfields_type(createfields) createfields
  ,      xxas_emp_ui_editfields_type(editfields) editfields
  ,      CASE WHEN geofence_list IS NOT NULL THEN xxas_emp_geofence_list_type(geofence_list) END geofence_list
  ,      title
  ,      default_form_handler
  ,      is_stepper
  ,      enable_recaptcha
  FROM   XMLTABLE('/View' 
                  PASSING p_ui_view_xml
                  COLUMNS id                         VARCHAR2(250)  path '@id',
                          object_type                VARCHAR2(400)  path '@objectType',
                          enabled                    VARCHAR2(30)   path '@enabled',
                          delete_allowed             VARCHAR2(30)   path '@deleteAllowed',
                          delete_label               VARCHAR2(400)  path '@deleteLabel',
                          next_label                 VARCHAR2(400)  path '@nextLabel',
                          show_submit                VARCHAR2(400)  path '@showSubmit',
                          submit_label               VARCHAR2(400)  path '@submitLabel',
                          submit_button_icon         VARCHAR2(400)  path '@submitButtonIcon',
                          submit_button_color        VARCHAR2(400)  path '@submitButtonColor',
                          form_help                  VARCHAR2(4000) path '@formHelp',
                          review_help                VARCHAR2(4000) path '@reviewHelp',
                          post_to_href               VARCHAR2(4000) path '@postToHref',
                          client_autosave            VARCHAR2(4000) path '@clientAutosave',
                          client_autosave_reset_text VARCHAR2(4000) path '@clientAutosaveResetText',
                          client_autosave_reset_btn  VARCHAR2(4000) path '@clientAutosaveResetBtn',
                          form_close_warning         VARCHAR2(4000) path '@formCloseWarning',
                          server_autosave            VARCHAR2(4000) path '@autosave',
                          server_autosave_timeout    VARCHAR2(4000) path '@autosaveTimeout',
                          server_autosave_message    VARCHAR2(4000) path '@autosaveMessage',
                          send_basic_auth            VARCHAR2(4000) path '@sendBasicAuth',
                          multi_factor_auth          VARCHAR2(4000) path '@multiFactorAuthenticated',
                          send_token                 VARCHAR2(4000) path '@sendToken',
                          basic_auth_un_field        VARCHAR2(4000) path '@basicAuthUsernameField',
                          basic_auth_pw_field        VARCHAR2(4000) path '@basicAuthPasswordField',
                          token_field                VARCHAR2(4000) path '@tokenField',
                          show_changes               VARCHAR2(4000) path '@showChanges',
                          createfields               XMLTYPE        path 'CreateFields',
                          editfields                 XMLTYPE        path 'EditFields',
                          geofence_list              XMLTYPE        path 'GeoFenceList',
                          title                      VARCHAR2(255)  path '@title',
                          default_form_handler       VARCHAR2(255)  path '@defaultFormHandler',
						  is_stepper                 VARCHAR2(30)   path '@isStepper',
                          enable_recaptcha           VARCHAR2(4000) path '@enableRecaptcha');
                 
  l_data c_data%ROWTYPE;
                 
BEGIN

  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id                         (l_data.id);
  self.set_object_type                (nvl(l_data.object_type, 'ROW'));
  self.set_enabled                    (l_data.enabled);
  self.set_delete_allowed             (l_data.delete_allowed);
  self.set_delete_label               (l_data.delete_label);
  self.set_next_label                 (l_data.next_label);
  self.set_show_submit                (l_data.show_submit);
  self.set_submit_label               (l_data.submit_label);
  self.set_submit_button_icon         (l_data.submit_button_icon);
  self.set_submit_button_color        (l_data.submit_button_color);
  self.set_form_help                  (l_data.form_help);
  self.set_review_help                (l_data.review_help);
  self.set_post_to_href               (l_data.post_to_href);
  self.set_client_autosave            (l_data.client_autosave);
  self.set_client_autosave_reset_text (l_data.client_autosave_reset_text);
  self.set_client_autosave_reset_btn  (l_data.client_autosave_reset_btn);
  self.set_form_close_warning         (l_data.form_close_warning);
  self.set_server_autosave            (l_data.server_autosave);
  self.set_server_autosave_timeout    (l_data.server_autosave_timeout);
  self.set_server_autosave_message    (l_data.server_autosave_message);
  self.set_send_basic_auth            (l_data.send_basic_auth);
  self.set_multi_factor_auth          (l_data.multi_factor_auth); 
  self.set_send_token                 (l_data.send_token);
  self.set_basic_auth_un_field        (l_data.basic_auth_un_field);
  self.set_basic_auth_pw_field        (l_data.basic_auth_pw_field);
  self.set_token_field                (l_data.token_field);
  self.set_show_changes               (l_data.show_changes);
  self.createfields                 := l_data.createfields;
  self.editfields                   := l_data.editfields;  
  self.geofence_list                := l_data.geofence_list;  
  self.title                        := l_data.title;
  self.set_is_stepper                 (l_data.is_stepper);
  self.set_enable_recaptcha           (l_data.enable_recaptcha);
  RETURN; 
END;

MEMBER PROCEDURE set_is_stepper
  (p_value IN VARCHAR2)
IS
BEGIN
   self.set_attribute('is_stepper', lower(p_value));
END;

MEMBER FUNCTION get_is_stepper RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('is_stepper');
END;

MEMBER PROCEDURE set_title
  (p_value IN VARCHAR2)
IS
BEGIN
   self.set_attribute('title', p_value);
END;

MEMBER FUNCTION get_title RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('title');
END;

MEMBER PROCEDURE set_default_form_handler
  (p_value IN VARCHAR2)
IS
BEGIN
   self.set_attribute('default_form_handler', p_value);
END;

MEMBER FUNCTION get_default_form_handler RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('default_form_handler');
END;

MEMBER PROCEDURE set_object_type
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('object_type', p_value);
END;

MEMBER FUNCTION get_object_type RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('object_type');
END;

MEMBER PROCEDURE set_enabled
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('enabled', lower(p_value));
END;

MEMBER FUNCTION get_enabled RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('enabled'));
END;

MEMBER PROCEDURE set_delete_allowed
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('delete_allowed', lower(p_value));
END;

MEMBER FUNCTION get_delete_allowed RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('delete_allowed'));
END;

MEMBER PROCEDURE set_delete_label
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('delete_label'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));

END;

MEMBER FUNCTION get_delete_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('delete_label');
END;

MEMBER PROCEDURE set_next_label
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('next_label', p_value);
END;

MEMBER FUNCTION get_next_label RETURN VARCHAR2
IS 
BEGIN
   RETURN nvl(self.get_attribute('next_label')
             ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_NEXT'));
END;

MEMBER PROCEDURE set_show_submit
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('show_submit', lower(p_value));
END;

MEMBER FUNCTION get_show_submit RETURN VARCHAR2
IS 
BEGIN
   RETURN nvl(lower(self.get_attribute('show_submit')), 'true');
END;

MEMBER PROCEDURE set_submit_label
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('submit_label'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_submit_label RETURN VARCHAR2
IS 
BEGIN
   RETURN nvl(self.get_attribute('submit_label')
             ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_SUBMIT'));
END;

MEMBER PROCEDURE set_submit_button_icon
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('submit_button_icon'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_submit_button_icon RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('submit_button_icon');
END;

MEMBER PROCEDURE set_submit_button_color
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('submit_button_color'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_submit_button_color RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('submit_button_color');
END;

MEMBER PROCEDURE set_form_help
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('form_help'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_form_help RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('form_help');
END;

MEMBER PROCEDURE set_review_help
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('review_help'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_review_help RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('review_help');
END;

MEMBER PROCEDURE set_post_to_href
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('post_to_href', p_value);
END;

MEMBER FUNCTION get_post_to_href RETURN VARCHAR2
IS 
  l_post_to_href varchar2(4000);
BEGIN
   l_post_to_href := self.get_attribute('post_to_href');
   IF l_post_to_href IS NOT NULL THEN
     IF lower(substr(l_post_to_href, 1, 4)) <> 'http' THEN
       l_post_to_href :=
         xxas_com_rest_util_pkg.uri_from_path(l_post_to_href);
     END IF;     
   END IF;
   RETURN l_post_to_href;
END;

MEMBER PROCEDURE set_client_autosave
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('client_autosave', lower(p_value));
END;

MEMBER FUNCTION get_client_autosave RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('client_autosave'));
END;

MEMBER PROCEDURE set_client_autosave_reset_text
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('client_autosave_reset_text', p_value);
END;

MEMBER FUNCTION get_client_autosave_reset_text RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('client_autosave_reset_text');
END;

MEMBER PROCEDURE set_client_autosave_reset_btn
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('client_autosave_reset_btn', p_value);
END;

MEMBER FUNCTION get_client_autosave_reset_btn RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('client_autosave_reset_btn');
END;

MEMBER PROCEDURE set_form_close_warning
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('form_close_warning', p_value);
END;

MEMBER FUNCTION get_form_close_warning RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('form_close_warning');
END;

MEMBER PROCEDURE set_server_autosave
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('server_autosave', lower(p_value));
END;

MEMBER FUNCTION get_server_autosave RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('server_autosave'));
END;

MEMBER PROCEDURE set_server_autosave_timeout
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('server_autosave_timeout', p_value);
END;

MEMBER FUNCTION get_server_autosave_timeout RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('server_autosave_timeout');
END;

MEMBER PROCEDURE set_server_autosave_message
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute
    ('server_autosave_message'
    ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_server_autosave_message RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('server_autosave_message');
END;

MEMBER PROCEDURE set_send_basic_auth
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('send_basic_auth', lower(p_value));
END;

MEMBER FUNCTION get_send_basic_auth RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('send_basic_auth'));
END;

MEMBER PROCEDURE set_multi_factor_auth
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('multi_factor_auth', lower(p_value));
END;

MEMBER FUNCTION get_multi_factor_auth RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('multi_factor_auth'));
END;

MEMBER PROCEDURE set_send_token
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('send_token', lower(p_value));
END;

MEMBER FUNCTION get_send_token RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('send_token'));
END;

MEMBER PROCEDURE set_basic_auth_un_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('basic_auth_un_field', p_value);
END;

MEMBER FUNCTION get_basic_auth_un_field RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('basic_auth_un_field');
END;

MEMBER PROCEDURE set_basic_auth_pw_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('basic_auth_pw_field', p_value);
END;

MEMBER FUNCTION get_basic_auth_pw_field RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('basic_auth_pw_field');
END;

MEMBER PROCEDURE set_token_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('token_field', p_value);
END;

MEMBER FUNCTION get_token_field RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('token_field');
END;

MEMBER PROCEDURE set_show_changes
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('show_changes', p_value);
END;

MEMBER FUNCTION get_show_changes RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('show_changes');
END;

MEMBER PROCEDURE set_enable_recaptcha
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('enable_recaptcha', lower(p_value));
END;

MEMBER FUNCTION get_enable_recaptcha RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('enable_recaptcha'));
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  
BEGIN
  SELECT XMLELEMENT("View"
                   ,XMLATTRIBUTES(self.get_id                    AS "id"
                                 ,self.get_object_type           AS "objectType"
                                 ,self.get_enabled               AS "enabled"
                                 ,self.get_delete_allowed        AS "deleteAllowed"
                                 ,self.get_delete_label          AS "deleteLabel"
                                 ,self.get_next_label            AS "nextLabel"
                                 ,self.get_submit_label          AS "submitLabel"
                                 ,self.get_submit_button_icon    AS "submitButtonIcon"
                                 ,self.get_submit_button_color   AS "submitButtonColor"
                                 ,self.get_form_help             AS "formHelp"
                                 ,self.get_review_help           AS "reviewHelp"
                                 ,self.get_title                 AS "title")
                   ,createfields.get_xml()
                   ,editfields.get_xml())
  INTO   l_xml   
  FROM   dual;

  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS

  l_xml XMLType;

BEGIN

  SELECT XMLELEMENT("Row"
                   ,XMLATTRIBUTES(self.get_delete_allowed        AS "deleteAllowed"
                                 ,self.get_delete_label          AS "deleteLabel"
                                 ,self.get_next_label            AS "nextLabel"
                                 ,self.get_submit_label          AS "submitLabel"
                                 ,self.get_submit_button_icon    AS "submitButtonIcon"
                                 ,self.get_submit_button_color   AS "submitButtonColor"
                                 ,self.get_form_help             AS "formHelp"
                                 ,self.get_review_help           AS "reviewHelp")
                   ,createfields.get_form_schema())
  INTO   l_xml
  FROM   dual;

  RETURN l_xml;

END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml                      XMLType;
  l_use_client_autosave      boolean := true;
  l_use_client_autosave_flag varchar2(1);
  l_is_stepper               boolean := false;
  l_is_stepper_flag          varchar2(1);
  l_recaptcha_enabled_flag   varchar2(1);
BEGIN

  --
  -- Default client autosave handling: disabled if using server autosave or
  -- if it's a login form
  --
  IF self.get_client_autosave IS NOT NULL THEN
    l_use_client_autosave := (upper(self.get_client_autosave) = 'TRUE');
  ELSE
    IF self.get_server_autosave IS NOT NULL OR
       self.get_send_basic_auth IS NOT NULL THEN
       l_use_client_autosave := false;
    END IF;
  END IF;

  l_use_client_autosave_flag := xxas_util_pkg.boolean_to_flag(l_use_client_autosave);

  IF self.get_is_stepper IS NOT NULL THEN
    l_is_stepper := (upper(self.get_is_stepper) = 'Y');
  END IF;
  
  l_is_stepper_flag := xxas_util_pkg.boolean_to_flag(l_is_stepper);
  l_recaptcha_enabled_flag := xxas_captcha_pkg.captcha_enabled_flag();
  
  SELECT XMLElement("form"
                   ,XMLElement("description", self.get_form_help)
                   ,XMLElement("links"
                              ,XMLElement("self"
                                         ,XMLElement("href"
                                                    ,self.get_post_to_href
                                                     )
                                          )
                              )
                   ,XMLElement("meta"
                              ,XMLElement("options"
                                         ,XMLForest
                                           (CASE WHEN l_use_client_autosave_flag = 'Y'
                                                 THEN 'true'
                                                 ELSE 'false'
                                            END "enableClientAutoSave"
                                           ,CASE WHEN l_use_client_autosave_flag = 'Y'
                                                 THEN nvl(self.get_client_autosave_reset_text
                                                         ,xxas_com_msg_pkg.get_string('XXAS_COM_FORM_AUTOSAVE'))
                                                 ELSE null
                                            END "clientAutoSaveResetText"
                                           ,CASE WHEN l_use_client_autosave_flag = 'Y'
                                                 THEN nvl(self.get_client_autosave_reset_btn
                                                         ,xxas_com_msg_pkg.get_string('XXAS_COM_FORM_AUTOSAVE_RESET'))
                                                 ELSE null
                                            END "clientAutoSaveResetButton"
                                           ,CASE WHEN l_use_client_autosave_flag = 'Y'
                                                 THEN nvl(self.get_form_close_warning
                                                         ,xxas_com_msg_pkg.get_string('XXAS_COM_FORM_CLOSE_WARN'))
                                                 ELSE null
                                            END "formCloseWarning"
                                           ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_REQ_FIELD') "requiredFieldError"
                                           ,self.get_server_autosave "autoSave"
                                           ,self.get_server_autosave_timeout "autoSaveTimeout"
                                           ,self.get_server_autosave_message "autoSaveMessage"
                                           ,self.get_send_basic_auth "sendBasicAuth"
                                           ,CASE WHEN l_recaptcha_enabled_flag = 'Y' AND nvl(self.get_enable_recaptcha,'false') = 'true'
                                                 THEN 'true'
                                                 ELSE null
                                            END "enableReCaptcha"
                                           ,CASE WHEN l_recaptcha_enabled_flag = 'Y' AND nvl(self.get_enable_recaptcha,'false') = 'true'
                                                 THEN xxas_captcha_pkg.get_site_key()
                                                 ELSE null
                                            END "reCaptchaSiteKey"                                       
                                           ,self.get_multi_factor_auth "multiFactorAuthenticated"
                                           ,self.get_send_token      "sendToken"
                                           ,self.get_basic_auth_un_field "basicAuthUsernameField"
                                           ,self.get_basic_auth_pw_field "basicAuthPasswordField"
                                           ,self.get_token_field         "tokenField"
                                           ,nvl(self.get_show_changes,'false')        "showChanges"
                                           ,CASE WHEN nvl(self.get_show_changes,'false') = 'true'
                                                 THEN xxas_com_msg_pkg.get_string('XXAS_COM_FORM_CHANGES_ONLY_LBL')
                                                 ELSE null
                                            END "changesOnlyLabel"
                                           ,CASE WHEN nvl(self.get_show_changes,'false') = 'true'
                                                 THEN xxas_com_msg_pkg.get_string('XXAS_COM_FORM_CHANGES_ALL_LBL')
                                                 ELSE null
                                            END "changesAllLabel"
                                           ,CASE WHEN nvl(self.get_show_changes,'false') = 'true'
                                                 THEN xxas_com_msg_pkg.get_string('XXAS_COM_FORM_PREVIOUS_LBL')
                                                 ELSE null
                                            END "previousLabel"
                                           ,CASE WHEN l_is_stepper_flag = 'N'
				                                         THEN 'true'
                                                 ELSE 'false'
											                      END "verticalStepper"
                                           )
                                        ,CASE WHEN self.get_show_submit = 'true' THEN
                                         XMLForest
                                                (self.get_submit_button_icon "submitButtonIcon"
                                                ,self.get_submit_button_color "submitButtonColor"
                                                ,self.get_submit_label "submitButton"
                                                 )
                                            END 
                                        ,CASE WHEN geofence_list IS NOT NULL THEN
                                          XMLElement
                                            ("geoFencing"
                                            ,geofence_list.get_json_friendly()
                                            ,XMLElement
                                               ("states"
                                               ,XMLElement
                                                  ("gpsNotDetected"
                                                 ,XMLForest(xxas_com_msg_pkg.get_string('XXAS_CICO_GEO_NO_GPS_TITLE') "title")
                                                 ,XMLForest(xxas_com_msg_pkg.get_string('XXAS_CICO_GEO_NO_GPS') "description")
                                                 ,XMLForest('https://cdrive.applaudsolutions.com/images/location-not-detected.png' "img")
                                                  )
                                               ,XMLElement
                                                  ("distanceAway"
                                                 ,XMLForest(xxas_com_msg_pkg.get_string('XXAS_CICO_GEO_FAR_AWAY_TITLE') "title")
                                                 ,XMLForest(xxas_com_msg_pkg.get_string('XXAS_CICO_GEO_FAR_AWAY') "description")
                                                 ,XMLForest('https://cdrive.applaudsolutions.com/images/location-outside.png' "img")
                                                  )
                                                ,XMLElement
                                                  ("mockLocation"
                                                 ,XMLForest(xxas_com_msg_pkg.get_string('XXAS_CICO_GEO_SPOOF_TITLE') "title")
                                                 ,XMLForest(xxas_com_msg_pkg.get_string('XXAS_CICO_GEO_SPOOF') "description")
                                                 ,XMLForest('https://cdrive.applaudsolutions.com/images/spoof-location.png' "img")
                                                  )
                                               )
											  ,XMLForest(xxas_com_msg_pkg.get_string('XXAS_CICO_GEO_ENABLE_GPS') "enableGpsButton")
                                             )
                                         END
                                         )
                              )
                   ,CASE WHEN l_is_stepper_flag = 'N' 
				                 THEN XMLElement("fields"
                               ,XMLAttributes
                                  ('true' as "json:force-array"
                                  ,'http://json.org/' as "xmlns:json")
                               ,createfields.get_json_friendly()
                               ,editfields.get_json_friendly())
						             ELSE
						                  XMLElement("steps"
                               ,XMLAttributes
                                  ('true' as "json:force-array"
                                  ,'http://json.org/' as "xmlns:json")
                               ,createfields.get_json_friendly()
                               ,editfields.get_json_friendly())
					END 
                   )
  INTO   l_xml
  FROM   dual;

  RETURN l_xml;

END;

MEMBER FUNCTION get_json_friendly_row RETURN XMLTYPE
IS

  l_xml XMLType;
  l_is_stepper               boolean := false;
  l_is_stepper_flag          varchar2(1);

BEGIN

  IF self.get_is_stepper IS NOT NULL THEN
    l_is_stepper := (upper(self.get_is_stepper) = 'Y');
  END IF;
  
  l_is_stepper_flag := xxas_util_pkg.boolean_to_flag(l_is_stepper);

  SELECT XMLElement("row"
                   ,XMLElement("meta"
                              ,XMLElement("options"
                                         ,XMLForest
                                           (self.get_delete_allowed AS "deleteAllowed"
                                           ,self.get_delete_label   AS "deleteLabel")
                                         )
                              )
                   ,CASE WHEN l_is_stepper_flag = 'N' 
				                 THEN XMLElement("fields"
                                ,XMLAttributes
                                 ('true' as "json:force-array"
                                 ,'http://json.org/' as "xmlns:json")
                                ,createfields.get_json_friendly())
                         ELSE
						                  XMLElement("steps"
                               ,XMLAttributes
                                  ('true' as "json:force-array"
                                  ,'http://json.org/' as "xmlns:json")
                               ,createfields.get_json_friendly())
                    END
                   )
  INTO   l_xml
  FROM   dual;
  
  RETURN l_xml;

END;

OVERRIDING MEMBER FUNCTION get_obj_xml RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  
BEGIN

  --
  -- Using an XMLType Constructor rather than evalname in
  -- XMLElement because of 10gR2 bug
  --
  SELECT XMLType('<' || self.get_object_type || '/>')
  INTO   l_xml   
  FROM   dual;
  
  SELECT APPENDCHILDXML(l_xml
                       ,'/' || self.get_object_type
                       ,createfields.get_obj_xml())
           
  INTO   l_xml
  FROM   dual;
  
  RETURN l_xml;

END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_multirow_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_multirow_type
  (p_ui_multirow_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/MultiRow' 
                  PASSING p_ui_multirow_xml
                  COLUMNS id                         VARCHAR2(4000) path '@id',
                          label                      VARCHAR2(4000) path '@label',
                          description                VARCHAR2(4000) path '@description',
                          visible                    VARCHAR2(4000) path '@visible',
                          hide_if                    VARCHAR2(4000) path '@hideIf',
                          create_allowed             VARCHAR2(4000) path '@createAllowed',
                          create_label               VARCHAR2(4000) path '@createLabel',
                          delete_label               VARCHAR2(4000) path '@deleteLabel',
                          min_rows                   VARCHAR2(4000) path '@minRows',
                          min_rows_error_message     VARCHAR2(4000) path '@minRowsErrorMessage',
                          max_rows_error_message     VARCHAR2(4000) path '@maxRowsErrorMessage',
                          max_rows                   VARCHAR2(4000) path '@maxRows',
                          xdb_uri                    VARCHAR2(4000) path '@xdbUri',
                          formula_name               VARCHAR2(4000) path '@formulaName',
                          data_callback              VARCHAR2(4000) path '@dataCallback',
                          include_empty_row          VARCHAR2(4000) path '@includeEmptyRow',
                          trigger_name               VARCHAR2(4000) path '@triggerName',
                          trigger_method             VARCHAR2(4000) path '@triggerMethod',
                          to_be_in_single_row        VARCHAR2(4000) path '@toBeInSingleRow',
                          force_single_row           VARCHAR2(4000) path '@forceSingleRow',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');


  l_data c_data%ROWTYPE;

BEGIN
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_create_allowed(l_data.create_allowed);
  self.set_create_label(l_data.create_label);
  self.set_delete_label(l_data.delete_label);
  self.set_min_rows(l_data.min_rows);
  self.set_min_rows_error_message(l_data.min_rows_error_message);
  self.set_max_rows_error_message(l_data.max_rows_error_message);
  self.set_max_rows(l_data.max_rows);
  self.set_ui_xdb_uri(l_data.xdb_uri);
  self.set_formula_name(l_data.formula_name);
  self.set_data_callback(l_data.data_callback);
  self.set_include_empty_row(l_data.include_empty_row);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);
  self.set_to_be_in_single_row(l_data.to_be_in_single_row);
  self.set_force_single_row(l_data.force_single_row);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);
    
  view_table := xxas_emp_ui_view_table();

  RETURN; 
END;

CONSTRUCTOR FUNCTION xxas_emp_ui_multirow_type
  (p_id                      IN VARCHAR2
  ,p_label                   IN VARCHAR2
  ,p_description             IN VARCHAR2
  ,p_visible                 IN VARCHAR2
  ,p_hide_if                 IN VARCHAR2
  ,p_create_allowed          IN VARCHAR2
  ,p_create_label            IN VARCHAR2
  ,p_delete_label            IN VARCHAR2 DEFAULT 'DELETE'
  ,p_min_rows                IN VARCHAR2
  ,p_min_rows_error_message  IN VARCHAR2
  ,p_max_rows                IN VARCHAR2
  ,p_max_rows_error_message  IN VARCHAR2
  ,p_ui_xdb_uri              IN VARCHAR2
  ,p_formula_name            IN VARCHAR2
  ,p_data_callback           IN VARCHAR2
  ,p_include_empty_row       IN VARCHAR2
  ,p_trigger_name            IN VARCHAR2
  ,p_trigger_method          IN VARCHAR2
  ,p_to_be_in_single_row     IN VARCHAR2
  ,p_force_single_row        IN VARCHAR2 DEFAULT NULL
  ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_create_allowed(p_create_allowed);
  self.set_create_label(p_create_label);
  self.set_delete_label(p_delete_label);
  self.set_min_rows(p_min_rows);
  self.set_min_rows_error_message(p_min_rows_error_message);
  self.set_max_rows_error_message(p_max_rows_error_message);
  self.set_max_rows(p_max_rows);
  self.set_ui_xdb_uri(p_ui_xdb_uri);
  self.set_formula_name(p_formula_name);
  self.set_data_callback(p_data_callback);
  self.set_include_empty_row(p_include_empty_row);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method);
  self.set_to_be_in_single_row(p_to_be_in_single_row);
  self.set_force_single_row(p_force_single_row);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);

  view_table := xxas_emp_ui_view_table();
  
  RETURN;          
END;

MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;

MEMBER PROCEDURE set_ui_xdb_uri
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('ui_xdb_uri', p_value);
END;

MEMBER FUNCTION get_ui_xdb_uri RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('ui_xdb_uri');
END;

MEMBER PROCEDURE set_create_allowed
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('create_allowed', lower(p_value));
END;

MEMBER FUNCTION get_create_allowed RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('create_allowed'));
END;

MEMBER PROCEDURE set_create_label
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('create_label'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER PROCEDURE set_delete_label
  (p_value IN VARCHAR2)
IS
BEGIN
   self.set_attribute
     ('delete_label'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_create_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('create_label');
END;

MEMBER FUNCTION get_delete_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('delete_label');
END;

MEMBER PROCEDURE set_min_rows
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('min_rows'
     ,p_value);
END;

MEMBER FUNCTION get_min_rows RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('min_rows');
END;

MEMBER PROCEDURE set_max_rows
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('max_rows'
     ,p_value);
END;

MEMBER FUNCTION get_min_rows_error_message RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('min_rows_error_message');
END;

MEMBER PROCEDURE set_min_rows_error_message
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('min_rows_error_message'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_max_rows_error_message RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('max_rows_error_message');
END;

MEMBER PROCEDURE set_max_rows_error_message
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('max_rows_error_message'
     ,xxas_com_msg_pkg.get_string_from_bind_exp(p_value));
END;

MEMBER FUNCTION get_max_rows RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('max_rows');
END;

MEMBER PROCEDURE set_formula_name
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('formula_name', p_value);
END;

MEMBER FUNCTION get_formula_name RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('formula_name');
END;

MEMBER PROCEDURE set_data_callback
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('data_callback', p_value);
END;

MEMBER FUNCTION get_data_callback RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('data_callback');
END;

MEMBER PROCEDURE set_include_empty_row
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('include_empty_row', lower(p_value));
END;

MEMBER FUNCTION get_include_empty_row RETURN VARCHAR2
IS 
BEGIN

   IF view_table.COUNT > 0 THEN
     RETURN 'false';
   ELSE
     RETURN lower(self.get_attribute('include_empty_row'));
   END IF;
END;

MEMBER PROCEDURE set_to_be_in_single_row
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('to_be_in_single_row', lower(p_value));
END;

MEMBER FUNCTION get_to_be_in_single_row RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('to_be_in_single_row');
END;


MEMBER PROCEDURE set_force_single_row
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('force_single_row', lower(p_value) ); 
END;

MEMBER FUNCTION get_force_single_row RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('force_single_row');
END;



MEMBER PROCEDURE add_row
  (p_emp_ui_view IN xxas_emp_ui_view_type)
IS 
BEGIN

  view_table.EXTEND;
  view_table(view_table.COUNT) := p_emp_ui_view;
  
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  l_ui  xxas_emp_ui_view_type;
  
BEGIN
  SELECT XMLELEMENT("MultiRow"
                   ,XMLATTRIBUTES(self.get_id                AS "id"
                                 ,self.get_label             AS "label"
                                 ,self.get_create_allowed    AS "createAllowed"
                                 ,self.get_create_label      AS "createLabel"
                                 ,self.get_description       AS "description"
                                 ,self.get_visible           AS "visible"
                                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                                            nvl(self.get_enable_form_on_server_wait,'true')
                                  END  AS "enableFormOnServerWait"))
  INTO   l_xml   
  FROM   dual;
  
  IF view_table.COUNT > 0 THEN
    FOR i IN 1..view_table.COUNT
    LOOP
      l_ui := view_table(i);
      SELECT APPENDCHILDXML(l_xml
                           ,'/MultiRow'
                           ,l_ui.get_xml())
               
      INTO   l_xml
      FROM   dual;
    END LOOP;
  END IF;
  
  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS
  l_xml XMLTYPE;
  l_ui  xxas_emp_ui_view_type;
  
BEGIN
  SELECT XMLELEMENT("MultiRow"
                   ,XMLATTRIBUTES(self.get_id                AS "id"
                                 ,self.get_label             AS "label"
                                 ,self.get_create_allowed    AS "createAllowed"
                                 ,self.get_create_label      AS "createLabel"
                                 ,self.get_description       AS "description"
                                 ,self.get_visible           AS "visible"))
  INTO   l_xml   
  FROM   dual;
  
  IF view_table.COUNT > 0 THEN
    FOR i IN 1..view_table.COUNT
    LOOP
      l_ui := view_table(i);
      SELECT APPENDCHILDXML(l_xml
                           ,'/MultiRow'
                           ,l_ui.get_form_schema())
               
      INTO   l_xml
      FROM   dual;
    END LOOP;
  END IF;
  
  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  l_ui  xxas_emp_ui_view_type;
  
BEGIN

  SELECT XMLELEMENT("field"
                   ,XMLForest
                     ('multirow'                                                                 AS "type"
                     ,self.get_id                                                                AS "id"
                     ,self.get_label                                                             AS "label"
                     ,self.get_create_allowed                                                    AS "createAllowed"
                     ,self.get_create_label                                                      AS "createLabel"
					           ,self.get_delete_label                                                      AS "deleteLabel"
                     ,self.get_min_rows                                                          AS "minRows"
                     ,self.get_min_rows_error_message                                            AS "minRowsErrorMessage"
                     ,self.get_max_rows_error_message                                            AS "maxRowsErrorMessage"
                     ,self.get_max_rows                                                          AS "maxRows"
                     ,self.get_description                                                       AS "description"
                     ,self.get_state                                                             AS "state"
                     ,self.get_hide_if                                                           AS "hideIf"
                     ,extract(self.new_row.get_json_friendly_row(), 'row/*')                     AS "newRow"
                     ,self.get_include_empty_row                                                 AS "addNewRow"
                     ,self.get_to_be_in_single_row                                               AS "toBeInSingleRow"
                     ,decode(self.get_force_single_row,NULL,'true',self.get_force_single_row)    AS "forceSingleRow"
                     ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     END  AS "enableFormOnServerWait"
                     )
                   ,XMLElement
                     ("rows"
                     ,XMLAttributes
                       ('true' as "json:force-array"
                       ,'http://json.org/' as "xmlns:json")
                     ,XMLAGG(f.get_json_friendly_row()))
                   ,CASE WHEN self.get_trigger_name is not null AND 
                              self.get_trigger_method is not null THEN
                      XMLElement
                       ("links"
                       ,XMLElement
                         ("trigger"
                         ,XMLElement
                           ("href"
                           ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                           )
                         ,XMLElement
                           ("method"
                           ,self.get_trigger_method
                           )
                         )
                       )
                    END
                   )
  INTO   l_xml
  FROM   TABLE(view_table) f;
  
  RETURN l_xml;

END;

OVERRIDING MEMBER FUNCTION get_obj_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  l_ui  xxas_emp_ui_view_type;
  
BEGIN

  --
  -- Using an XMLType Constructor rather than evalname in
  -- XMLElement because of 10gR2 bug
  --
  SELECT XMLType('<' || self.get_id || '/>')
  INTO   l_xml   
  FROM   dual;
  
  IF view_table.COUNT > 0 THEN
    FOR i IN 1..view_table.COUNT
    LOOP
      l_ui := view_table(i);
      SELECT APPENDCHILDXML(l_xml
                           ,'/' || self.get_id
                           ,l_ui.get_obj_xml())
               
      INTO   l_xml
      FROM   dual;
    END LOOP;
  END IF;
  
  RETURN l_xml;
END;

END;
/

-----------------------------------------------------
--           xxas_emp_ui_rating_type
-----------------------------------------------------
CREATE OR REPLACE TYPE BODY xxas_emp_ui_rating_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_rating_type
  (p_ui_rating_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Rating' 
                  PASSING p_ui_rating_xml
                  COLUMNS id                         VARCHAR2(4000) path '@id',
                          label                      VARCHAR2(4000) path '@label',
                          description                VARCHAR2(4000) path '@description',   
                          refresh_on_change          VARCHAR2(4000) path '@refreshOnChange',
                          trigger_name               VARCHAR2(4000) path '@triggerName',
                          trigger_method             VARCHAR2(4000) path '@triggerMethod',                          
                          visible                    VARCHAR2(4000) path '@visible',
                          hide_if                    VARCHAR2(4000) path '@hideIf',
                          enabled                    VARCHAR2(4000) path '@enabled',
                          required                   VARCHAR2(4000) path '@required',
                          icon                       VARCHAR2(4000) path '@icon',
                          multi_color                VARCHAR2(4000) path '@multiColor',
                          icon_color                 VARCHAR2(4000) path '@iconColor',
                          min                        VARCHAR2(4000) path '@min',
                          max                        VARCHAR2(4000) path '@max',
                          min_label                  VARCHAR2(4000) path '@minLabel',
                          max_label                  VARCHAR2(4000) path '@maxLabel',
                          icon_steps                 VARCHAR2(4000) path '@iconSteps',
                          value                      VARCHAR2(4000) path '@value',
                          list_provider              VARCHAR2(4000) path '@listProvider',
                          show_as_select_single      VARCHAR2(4000) path '@showAsSelectSingle',
                          enable_form_on_server_wait VARCHAR2(4000) path '@enableFormOnServerWait');
  l_data c_data%ROWTYPE;

BEGIN
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_refresh_on_change(l_data.refresh_on_change);
  self.set_trigger_name(l_data.trigger_name);
  self.set_trigger_method(l_data.trigger_method);  
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_enabled(l_data.enabled);
  self.set_required(l_data.required);
  self.set_icon(l_data.icon);
  self.set_multi_color(l_data.multi_color);
  self.set_icon_color(l_data.icon_color);
  self.set_min(l_data.min);
  self.set_max(l_data.max);
  self.set_min_label(l_data.min_label);
  self.set_max_label(l_data.max_label);
  self.set_icon_steps(l_data.icon_steps);
  self.set_value(l_data.value);
  self.set_list_provider(l_data.list_provider);
  self.set_select_single(l_data.show_as_select_single);
  self.set_enable_form_on_server_wait(l_data.enable_form_on_server_wait);

  RETURN; 
END;

CONSTRUCTOR FUNCTION xxas_emp_ui_rating_type
    (p_id                     IN VARCHAR2
    ,p_label                  IN VARCHAR2
    ,p_description            IN VARCHAR2    
    ,p_refresh_on_change      IN VARCHAR2
    ,p_trigger_name           IN VARCHAR2
    ,p_trigger_method         IN VARCHAR2     
    ,p_visible                IN VARCHAR2
    ,p_hide_if                IN VARCHAR2
    ,p_enabled                IN VARCHAR2
    ,p_required               IN VARCHAR2
    ,p_icon                   IN VARCHAR2
    ,p_icon_steps             IN VARCHAR2
    ,p_multi_color            IN VARCHAR2
    ,p_icon_color             IN VARCHAR2
    ,p_min                    IN VARCHAR2
    ,p_max                    IN VARCHAR2
    ,p_min_label              IN VARCHAR2
    ,p_max_label              IN VARCHAR2
    ,p_value                  IN VARCHAR2
    ,p_list_provider          IN VARCHAR2
    ,p_show_as_select_single  IN VARCHAR2
    ,p_enable_form_on_server_wait VARCHAR2 DEFAULT NULL) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_refresh_on_change(p_refresh_on_change);
  self.set_trigger_name(p_trigger_name);
  self.set_trigger_method(p_trigger_method); 
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_enabled(p_enabled);
  self.set_required(p_required);
  self.set_icon(p_icon);
  self.set_multi_color(p_multi_color);
  self.set_icon_color(p_icon_color);
  self.set_min(p_min);
  self.set_max(p_max);
  self.set_min_label(p_min_label);
  self.set_max_label(p_max_label);
  self.set_icon_steps(p_icon_steps);
  self.set_value(p_value);
  self.set_list_provider(p_list_provider);
  self.set_select_single(p_show_as_select_single);
  self.set_enable_form_on_server_wait(p_enable_form_on_server_wait);
  
  RETURN;          
END;

OVERRIDING MEMBER PROCEDURE set_enable_form_on_server_wait
  (p_value IN VARCHAR2)
IS 
BEGIN
  self.set_attribute('enable_form_on_server_wait',p_value);
END;

OVERRIDING MEMBER FUNCTION get_enable_form_on_server_wait RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('enable_form_on_server_wait');
END;


MEMBER PROCEDURE set_icon
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('icon', p_value);
END;

MEMBER FUNCTION get_icon RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('icon');
END;

MEMBER PROCEDURE set_multi_color
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('multi_color', p_value);
END;

MEMBER FUNCTION get_multi_color RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('multi_color');
END;

MEMBER PROCEDURE set_icon_color
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('icon_color', p_value);
END;

MEMBER FUNCTION get_icon_color RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('icon_color');
END;

MEMBER PROCEDURE set_min
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('min',(p_value));
END;

MEMBER FUNCTION get_min RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('min');
END;

MEMBER PROCEDURE set_max
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('max'
     ,p_value);
END;

MEMBER FUNCTION get_max RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('max');
END;

MEMBER PROCEDURE set_max_label
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('max_label'
     ,p_value);
END;

MEMBER FUNCTION get_max_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('max_label');
END;

MEMBER PROCEDURE set_min_label
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute
     ('min_label'
     ,p_value);
END;

MEMBER FUNCTION get_min_label RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('min_label');
END;

MEMBER PROCEDURE set_icon_steps
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('icon_steps', p_value);
END;


MEMBER FUNCTION get_icon_steps RETURN VARCHAR2
IS 
BEGIN
   RETURN self.get_attribute('icon_steps');
END;

MEMBER PROCEDURE set_select_single
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('showAsSelectSingle', lower(p_value));
END;

MEMBER FUNCTION get_select_single RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('showAsSelectSingle'));
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
 
BEGIN
 SELECT  insertChildXML(
           XMLELEMENT
             ("Rating"
              ,XMLATTRIBUTES
                 (self.get_id                  AS "id"
                 ,self.get_label               AS "label"
                 ,self.get_description         AS "description"   
                 ,self.get_refresh_on_change   AS "refreshOnChange"                 
                 ,self.get_required            AS "required"
                 ,self.get_visible             AS "visible"
                 ,self.get_icon                AS "icon"
                 ,self.get_multi_color         AS "multiColor"
                 ,decode(nvl(self.get_multi_color
                            ,'false')
                        ,'false'
                        ,self.get_icon_color)  AS "iconColor"
                 ,self.get_min                 AS "min"
                 ,self.get_max                 AS "max"
                 ,self.get_min_label           AS "minLabel"
                 ,self.get_max_label           AS "maxLabel"
                 ,self.get_value               AS "value"
                 ,self.get_select_single       AS "showAsSelectSingle"
                 ,decode(self.is_provider_rest_flag
                        ,'Y'
                        ,self.get_rest_href)
                                               AS "listProvider"
                 ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
                  END  AS "enableFormOnServerWait"
                )
              ,(SELECT XMLELEMENT("iconSteps"
                                 ,XMLAgg
                                    (XMLELEMENT
                                      ("Value"
                                      ,XMLFOREST(label AS "id"
                                                ,label AS "label"
                                                ,color AS "color")
                                      )  
                                    ORDER BY id
                                    )
                                  )
                FROM  XMLTABLE('/values/*'
                               passing value_set.get_xml()
                               COLUMNS
                               id    VARCHAR2(1000) path 'ID',
                               label VARCHAR2(1000) path 'Value',
                               color VARCHAR2(1000) path 'Description'
                              )
             
               )
             )
           ,'//Rating[not(@value)]'
           ,'@value'
           ,'')                           
  INTO   l_xml   
  FROM   dual;
  
  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS
  l_xml XMLTYPE;
  l_ui  xxas_emp_ui_view_type;
  
BEGIN
  
  RETURN self.get_xml;
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;
BEGIN
 
  SELECT XMLELEMENT
           ("field"
           ,XMLForest
              ('rating'                           AS "type"
              ,self.get_id                        AS "id"
              ,self.get_label                     AS "label"
              ,self.get_description               AS "description"  
              ,self.get_refresh_on_change         AS "refreshOnChange"              
              ,self.get_required                  AS "required"
              ,self.get_state                     AS "state"
              ,self.get_icon                      AS "icon"
              ,self.get_multi_color               AS "multiColor"
              ,decode(nvl(self.get_multi_color
                            ,'false')
                     ,'false'
                     ,self.get_icon_color)        AS "iconColor"
              ,self.get_min                       AS "min"
              ,self.get_max                       AS "max"
              ,self.get_min_label                 AS "minLabel"
              ,self.get_max_label                 AS "maxLabel"
              ,self.get_value                     AS "value"
            --  ,self.get_old_value                 AS "oldValue"
              ,self.get_select_single             AS "showAsSelectSingle"
              ,decode(self.is_provider_rest_flag
                     ,'Y'
                     ,self.get_rest_href)         AS "listProvider"                   
              ,CASE WHEN self.get_trigger_name is not null AND self.get_trigger_method is not null THEN
                          nvl(self.get_enable_form_on_server_wait,'true')
                     ELSE
                          self.get_enable_form_on_server_wait
               END  AS "enableFormOnServerWait"
              )
            ,CASE WHEN self.get_old_value = xxas_mss_util_pkg.get_g_number THEN
			            XMLElement("oldValue", null)
			            WHEN self.get_old_value IS NOT NULL THEN
				          XMLElement("oldValue", self.get_old_value)
               END
            ,CASE WHEN self.get_trigger_name is not null AND 
                          self.get_trigger_method is not null THEN
                  XMLElement
                   ("links"
                   ,XMLElement
                     ("trigger"
                     ,XMLElement
                       ("href"
                       ,xxas_com_rest_util_pkg.uri_from_path('form-field-triggers/' || self.get_trigger_name)
                       )
                     ,XMLElement
                       ("method"
                       ,self.get_trigger_method
                       )
                     )
                   )
                END    
            ,(SELECT XMLELEMENT("iconSteps"
                               ,XMLAgg
                                  (XMLELEMENT
                                    ("value"
                                    ,XMLFOREST(id AS "id"
                                              ,label AS "label"
                                              ,decode(nvl(self.get_multi_color,'false')
                                                     ,'true'
                                                     ,color) 
                                                     AS "color"
                                              )
                                    )  
                                  ORDER BY id
                                  )
                                )
              FROM  XMLTABLE('/values/*'
                             passing value_set.get_json_friendly()
                             COLUMNS
                             id    VARCHAR2(1000) path 'id',
                             label VARCHAR2(1000) path 'label',
                             color VARCHAR2(1000) path 'description'
                            )
             
             )
           )
  INTO   l_xml
  FROM   dual;
  
  RETURN l_xml;

END;

END;

/

-----------------------------------------------------
--           xxas_emp_geofence_type
-----------------------------------------------------
CREATE OR REPLACE TYPE BODY xxas_emp_geofence_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_geofence_type
  (p_geofence_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/GeoFence' 
                  PASSING p_geofence_xml
                  COLUMNS latitude   VARCHAR2(4000) path '@latitude',
                          longitude  VARCHAR2(4000) path '@longitude',
                          perimeter  VARCHAR2(4000) path '@perimeter');
  l_data c_data%ROWTYPE;

BEGIN
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_latitude(l_data.latitude);
  self.set_longitude(l_data.longitude);
  self.set_perimeter(l_data.perimeter);

  RETURN; 
END;

MEMBER PROCEDURE set_latitude
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.latitude := p_value;
END;

MEMBER FUNCTION get_latitude RETURN VARCHAR2
IS 
BEGIN
   RETURN self.latitude;
END;

MEMBER PROCEDURE set_longitude
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.longitude := p_value;
END;

MEMBER FUNCTION get_longitude RETURN VARCHAR2
IS 
BEGIN
   RETURN self.longitude;
END;

MEMBER PROCEDURE set_perimeter
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.perimeter := p_value;
END;

MEMBER FUNCTION get_perimeter RETURN VARCHAR2
IS 
BEGIN
   RETURN self.perimeter;
END;

MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  
BEGIN

  SELECT XMLElement
             ("geoFence"
             ,XMLForest(self.get_latitude "latitude")
             ,XMLForest(self.get_longitude "longitude")
             ,XMLElement
               ("distance"
               ,XMLForest(self.get_perimeter "value")
               ,XMLForest('KM' "measurement")
               )
             )
           
  INTO   l_xml
  FROM   dual;
  
  RETURN l_xml;

END;

END;

/

-----------------------------------------------------
--           xxas_emp_geofence_list_type
-----------------------------------------------------
CREATE OR REPLACE TYPE BODY xxas_emp_geofence_list_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_geofence_list_type
  (p_geofence_list_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_geofence
  IS
  SELECT CASE WHEN existsNode(geofence, '/GeoFence')  = 1
              THEN xxas_emp_geofence_type(geofence)
         END geofence
  FROM   XMLTABLE('/GeoFenceList/*' 
                  PASSING p_geofence_list_xml 
                  COLUMNS geofence XMLTYPE       path '.');

BEGIN

  geofence_list := xxas_emp_geofence_table();
  FOR csr_rec IN c_geofence
  LOOP
    geofence_list.EXTEND;
    geofence_list(geofence_list.COUNT) := csr_rec.geofence;
  END LOOP;

  RETURN; 
END;

MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  
BEGIN
  IF geofence_list.COUNT > 0 THEN
    SELECT XMLElement
             ("coordinates"
             ,XMLAttributes
               ('true' as "json:force-array"
               ,'http://json.org/' as "xmlns:json")
             ,XMLAGG(g.get_json_friendly())
             )
    INTO   l_xml
    FROM   TABLE(geofence_list) g;
  END IF;
  
  RETURN l_xml;

END;

END;

/

CREATE OR REPLACE TYPE BODY xxas_emp_ui_stepper_type IS
  
CONSTRUCTOR FUNCTION xxas_emp_ui_stepper_type
  (p_ui_stepper_xml IN XMLTYPE) RETURN SELF AS RESULT

IS
  CURSOR c_data
  IS
  SELECT *
  FROM   XMLTABLE('/Stepper' 
                  PASSING p_ui_stepper_xml
                  COLUMNS id                         VARCHAR2(4000) path '@id',
                          label                      VARCHAR2(4000) path '@title',
                          description                VARCHAR2(4000) path '@description',
                          visible                    VARCHAR2(4000) path '@visible',
                          hide_if                    VARCHAR2(4000) path '@hideIf',
                          parent_field               VARCHAR2(4000) path '@postAsParentFields',
						  continue_label             VARCHAR2(4000) path '@continueButtonLabel',
						  back_label                 VARCHAR2(4000) path '@backButtonLabel',
						  continue_icon              VARCHAR2(4000) path '@continueButtonIcon',
						  continue_button_color      VARCHAR2(4000) path '@continueButtonColor',
						  back_icon                  VARCHAR2(4000) path '@backButtonIcon');
  
  CURSOR c_fields
  IS
  SELECT CASE WHEN existsNode(field, '/Stepper')  = 1
              THEN xxas_emp_ui_stepper_type(field)
			  WHEN existsNode(field, '/Container')  = 1
              THEN xxas_emp_ui_container_type(field)
              WHEN existsNode(field, '/MultiRow')  = 1
              THEN xxas_emp_ui_multirow_type(field)
              WHEN existsNode(field,'/Rating')  = 1
              THEN xxas_emp_ui_rating_type(field)
              WHEN existsNode(field, '/Flexfield')  = 1
              THEN xxas_emp_ui_flexfield_type(field)
              WHEN existsNode(field, '/OLMTest')  = 1
              THEN xxas_emp_ui_olmtest_type(field)
              WHEN existsNode(field, '/OLMQuestion')  = 1
              THEN xxas_emp_ui_olmquestion_type(field)
              WHEN existsNode(field, '/TextInput')  = 1
              THEN xxas_emp_ui_textinput_type(field)
              WHEN existsNode(field, '/Submit')  = 1
              THEN xxas_emp_ui_submit_type(field)
              WHEN existsNode(field, '/Html')  = 1
              THEN xxas_emp_ui_html_type(field)
              WHEN existsNode(field, '/Item')  = 1
              THEN xxas_emp_ui_item_type(field)
              WHEN existsNode(field, '/ListItem')  = 1
              THEN xxas_emp_ui_listitem_type(field)
              WHEN existsNode(field, '/Link')  = 1
              THEN xxas_emp_ui_link_type(field)
              WHEN existsNode(field, '/Address')  = 1
              THEN xxas_emp_ui_address_type(field)
              WHEN existsNode(field, '/Password')  = 1
              THEN xxas_emp_ui_password_type(field)
              WHEN existsNode(field, '/ColorPicker')  = 1
              THEN xxas_emp_ui_color_picker_type(field)
              WHEN existsNode(field, '/Icon')  = 1
              THEN xxas_emp_ui_icon_type(field)
              WHEN existsNode(field, '/Email')  = 1
              THEN xxas_emp_ui_email_type(field)
              WHEN existsNode(field, '/TextArea')  = 1
              THEN xxas_emp_ui_textarea_type(field)
              WHEN existsNode(field, '/Number')     = 1
              THEN xxas_emp_ui_number_type(field)
              WHEN existsNode(field, '/Attachment') = 1
              THEN xxas_emp_ui_attachment_type(field) 
              WHEN existsNode(field, '/File') = 1
              THEN xxas_emp_ui_file_type(field) 
              WHEN existsNode(field, '/Hidden') = 1
              THEN xxas_emp_ui_hidden_type(field) 
              WHEN existsNode(field, '/Date')       = 1 OR  
                   existsNode(field, '/DateTime')   = 1 OR 
                   existsNode(field, '/Time')       = 1 
              THEN xxas_emp_ui_date_type(field)  
              WHEN existsNode(field, '/DateRange')       = 1 OR  
                   existsNode(field, '/DateTimeRange')   = 1 OR 
                   existsNode(field, '/TimeRange')       = 1 
              THEN xxas_emp_ui_date_range_type(field)
              WHEN existsNode(field, '/Divider') = 1
              THEN xxas_emp_ui_divider_type(field) 
              WHEN existsNode(field, '/Poplist')    = 1 OR
                   existsNode(field, '/Combolist')  = 1
              THEN xxas_emp_ui_list_type(field)
         END field
  FROM   XMLTABLE('/Stepper/*' 
                  PASSING p_ui_stepper_xml
                  COLUMNS field       XMLTYPE       path '.');
  
  l_data c_data%ROWTYPE;

BEGIN
  OPEN  c_data;
  FETCH c_data INTO l_data;
  CLOSE c_data;
  
  self.set_id(l_data.id);
  self.set_label(l_data.label);
  self.set_description(l_data.description);
  self.set_visible(l_data.visible);
  self.set_hide_if(l_data.hide_if);
  self.set_parent_field(l_data.parent_field);
  self.set_continue_label(l_data.continue_label);
  self.set_back_label(l_data.back_label);
  self.set_continue_icon(l_data.continue_icon);
  self.set_continue_button_color(l_data.continue_button_color);
  self.set_back_icon(l_data.back_icon);
    
  field_list := xxas_emp_ui_component_table();
  
  FOR csr_rec IN c_fields
  LOOP
    field_list.EXTEND;
    field_list(field_list.COUNT) := csr_rec.field;
  END LOOP;

  RETURN; 
END;

CONSTRUCTOR FUNCTION xxas_emp_ui_stepper_type
  (p_id                     IN VARCHAR2
  ,p_label                  IN VARCHAR2
  ,p_description            IN VARCHAR2
  ,p_visible                IN VARCHAR2
  ,p_hide_if                IN VARCHAR2
  ,p_parent_field           IN VARCHAR2
  ,p_continue_label         IN VARCHAR2
  ,p_back_label             IN VARCHAR2
  ,p_continue_icon          IN VARCHAR2
  ,p_continue_button_color  IN VARCHAR2
  ,p_back_icon              IN VARCHAR2) RETURN SELF AS RESULT
  
IS

BEGIN

  self.set_id(p_id);
  self.set_label(p_label);
  self.set_description(p_description);
  self.set_visible(p_visible);
  self.set_hide_if(p_hide_if);
  self.set_parent_field(p_parent_field);
  self.set_continue_label(p_continue_label);
  self.set_back_label(p_back_label);
  self.set_continue_icon(p_continue_icon);
  self.set_continue_button_color(p_continue_button_color);
  self.set_back_icon(p_back_icon);
  
  field_list := xxas_emp_ui_component_table();
  
  RETURN;
END;

MEMBER PROCEDURE set_parent_field
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('parent_field', lower(p_value));
END;

MEMBER FUNCTION get_parent_field RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('parent_field'));
END;

MEMBER PROCEDURE set_continue_label
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('continue_label', lower(p_value));
END;

MEMBER FUNCTION get_continue_label RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('continue_label'));
END;

MEMBER PROCEDURE set_back_label
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('back_label', lower(p_value));
END;

MEMBER FUNCTION get_back_label RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('back_label'));
END;

MEMBER PROCEDURE set_continue_icon
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('continue_icon', lower(p_value));
END;

MEMBER FUNCTION get_continue_icon RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('continue_icon'));
END;

MEMBER PROCEDURE set_continue_button_color
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('continue_button_color', lower(p_value));
END;

MEMBER FUNCTION get_continue_button_color RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('continue_button_color'));
END;

MEMBER PROCEDURE set_back_icon
  (p_value IN VARCHAR2)
IS 
BEGIN
   self.set_attribute('back_icon', lower(p_value));
END;

MEMBER FUNCTION get_back_icon RETURN VARCHAR2
IS 
BEGIN
   RETURN lower(self.get_attribute('back_icon'));
END;

OVERRIDING MEMBER FUNCTION get_xml RETURN XMLTYPE

IS
  l_xml XMLTYPE;
  
BEGIN
  
  SELECT XMLAGG(f.get_xml())
  INTO   l_xml
  FROM   TABLE(field_list) f;
  
  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_form_schema RETURN XMLTYPE
IS
  l_xml XMLTYPE;
  
BEGIN
  SELECT XMLELEMENT("Stepper"
                   ,XMLATTRIBUTES(self.get_id                AS "id"
                                 ,self.get_label             AS "label"
                                 ,self.get_description       AS "description"
                                 ,self.get_visible           AS "visible"
                                 ,self.get_parent_field      AS "postAsParentFields"))
  INTO   l_xml   
  FROM   dual;
  
  FOR i IN 1..field_list.COUNT
  LOOP
    SELECT APPENDCHILDXML(l_xml
                         ,'/Stepper'
                         ,field_list(i).get_form_schema())
             
    INTO   l_xml
    FROM   dual;
  END LOOP;
  
  RETURN l_xml;
END;

OVERRIDING MEMBER FUNCTION get_json_friendly RETURN XMLTYPE
IS

  l_xml XMLTYPE;
  
BEGIN
  
  SELECT XMLELEMENT("field"
                   ,XMLForest
                     ('fieldset'                     AS "type"
                     ,self.get_id                    AS "id"
                     ,self.get_label                 AS "label"
                     ,self.get_description           AS "description"
                     ,self.get_state                 AS "state"
                     ,self.get_visible               AS "visible"
                     ,self.get_hide_if               AS "hideIf"
                     ,self.get_parent_field      AS "postAsParentFields")
                   ,XMLElement
                     ("fields"
                     ,XMLAttributes
                       ('true' as "json:force-array"
                       ,'http://json.org/' as "xmlns:json")
                     ,XMLAGG(f.get_json_friendly()))
                   ,XMLElement
                       ("meta"
                       ,XMLElement
                         ("options"
                         ,XMLForest
						              (nvl(self.get_continue_label,'Continue')            "continueButtonLabel",
						              self.get_continue_icon                              "continueButtonIcon",
						              nvl(self.get_continue_button_color,'positive')      "continueButtonColor",
						              nvl(self.get_back_label,'Back')                     "backButtonLabel",
						              self.get_back_icon                                  "backButtonIcon"
						              )
                         )
                       )
                   )
  INTO   l_xml
  FROM   TABLE(field_list) f;
  
  RETURN l_xml;

END;

END;
/

COMMIT;
EXIT;
