/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2014 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
SET SCAN OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_vacancy_pkg IS

g_PACKAGE_NAME       CONSTANT varchar2(30) := 'xxas_vacancy_pkg';
g_HRAPI_DATETIME_FMT CONSTANT varchar2(30) := 'YYYY-MM-DD HH24:MI:SS".0"';
g_HRAPI_DATE_FMT     CONSTANT varchar2(30) := 'YYYY-MM-DD';
g_ITEM_TYPE          CONSTANT varchar2(30) := 'HRSSA';
g_PROCESS_NAME       CONSTANT varchar2(30) := 'HR_GENERIC_APPROVAL_PRC';
g_AME_APP_ID         CONSTANT number       :=  800;
g_AME_TRANS_TYPE     CONSTANT varchar2(30) := 'IRCVACAPPROVAL';

gc_log               xxas_com_log_type := xxas_com_log_type(g_PACKAGE_NAME);
-- ------------------------------------------------------------------------ *
-- Name    : rec_mem_exists
-- ------------------------------------------------------------------------ *
FUNCTION rec_mem_exists (p_rec_team_member_id IN NUMBER) RETURN number IS
  l_exists number DEFAULT 0;
  CURSOR c_exists IS
     SELECT 2
       FROM irc_rec_team_members
      WHERE rec_team_member_id = p_rec_team_member_id;
BEGIN
  OPEN c_exists;
  FETCH c_exists
       INTO l_exists;
  CLOSE c_exists;
  RETURN l_exists;
END rec_mem_exists;
--
-- ------------------------------------------------------------------------ *
-- Name    : log_start
-- ------------------------------------------------------------------------ *
PROCEDURE log_start
  (p_method      IN  VARCHAR2
  ,p_parameter1  IN  VARCHAR2 DEFAULT NULL
  ,p_parameter2  IN  VARCHAR2 DEFAULT NULL
  ,p_parameter3  IN  VARCHAR2 DEFAULT NULL
  ,p_parameter4  IN  VARCHAR2 DEFAULT NULL
  ,p_parameter5  IN  VARCHAR2 DEFAULT NULL
  ,p_parameter6  IN  VARCHAR2 DEFAULT NULL
  ,p_parameter7  IN  VARCHAR2 DEFAULT NULL
  ,p_parameter8  IN  VARCHAR2 DEFAULT NULL
  ,p_parameter9  IN  VARCHAR2 DEFAULT NULL
  ,p_parameter10 IN  VARCHAR2 DEFAULT NULL)
IS
BEGIN
   IF gc_log.proc=1 
   THEN 
     gc_log.procedure_start_params
       (p_proc_name   =>  p_method
       ,p_parameter1  =>  p_parameter1
       ,p_parameter2  =>  p_parameter2
       ,p_parameter3  =>  p_parameter3
       ,p_parameter4  =>  p_parameter4
       ,p_parameter5  =>  p_parameter5
       ,p_parameter6  =>  p_parameter6
       ,p_parameter7  =>  p_parameter7
       ,p_parameter8  =>  p_parameter8
       ,p_parameter9  =>  p_parameter9
       ,p_parameter10 =>  p_parameter10); 
   END IF;  
END log_start; 

--
-- ------------------------------------------------------------------------ *
-- Name    : log_end
-- ------------------------------------------------------------------------ *
PROCEDURE log_end
  (p_method  IN  VARCHAR2
  ,p_result  IN  VARCHAR2 DEFAULT NULL)
IS
BEGIN
  IF gc_log.proc=1 
  THEN 
    gc_log.procedure_end_result
      (p_proc_name =>  p_method
      ,p_result    =>  p_result); 
  END IF;
END log_end;  

--
-- ------------------------------------------------------------------------ *
-- Name    : log_statement
-- ------------------------------------------------------------------------ *
PROCEDURE log_statement
  (p_method     IN  VARCHAR2
  ,p_log_string IN  VARCHAR2)
IS
BEGIN
  IF gc_log.stmt=1 
  THEN 
    gc_log.statement
      (p_proc_name  =>  p_method
      ,p_log_string =>  p_log_string); 
  END IF;
END log_statement;

--
-- ------------------------------------------------------------------------ *
-- Name    : raise_user_error
-- ------------------------------------------------------------------------ *
PROCEDURE raise_user_error
  (p_package_name IN VARCHAR2
  ,p_proc_name    IN VARCHAR2
  ,p_message      IN VARCHAR2)
IS
 
BEGIN

 xxas_com_api_result_pkg.raise_user_error
   (p_package_name => p_package_name
   ,p_proc_name    => p_proc_name
   ,p_field_name   => NULL
   ,p_message      => xxas_com_msg_type
                        (xxas_util_pkg.app
                        ,'XXAS_COM_TEXT'
                        ,xxas_com_msg_token_varray
                          (xxas_com_msg_token_type
                            ('TEXT'
                            ,p_message))));                                                     
END raise_user_error;

--
-- ------------------------------------------------------------------------ *
-- Name    : anydata_to_obj
-- ------------------------------------------------------------------------ *
FUNCTION anydata_to_obj 
  (p_anydata IN ANYDATA) RETURN xxas_per_all_vacancy
IS

  c_method CONSTANT VARCHAR2(30) := 'anydata_to_obj';
  l_obj             xxas_per_all_vacancy;
  l_ret_val         number;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  l_ret_val := p_anydata.getObject(l_obj);
   
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_obj;

END anydata_to_obj;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_business_group_id
-- ------------------------------------------------------------------------ *
FUNCTION get_business_group_id
  (p_vacancy_id IN number) RETURN number
IS

  c_method CONSTANT  VARCHAR2(30) := 'get_business_group_id';
  l_business_group_id NUMBER;
  
  CURSOR csr_bg_id IS
  SELECT vac.business_group_id
  FROM   per_all_vacancies vac
  WHERE  vac.vacancy_id = p_vacancy_id;
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
   
  --
  -- Get the BG from the vacancy
  --
  IF p_vacancy_id IS NOT NULL THEN
    OPEN  csr_bg_id;
    FETCH csr_bg_id INTO l_business_group_id;
    CLOSE csr_bg_id;
  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_business_group_id;
  
END get_business_group_id;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_ui_xdb_uri
-- ------------------------------------------------------------------------ *
FUNCTION get_ui_xdb_uri RETURN varchar2
IS

  c_method CONSTANT VARCHAR2(30)       := 'get_ui_xdb_uri';
  c_seeded CONSTANT VARCHAR2(240)      := '/public/xxas/emp/view/vacancy.xml';
  c_custom CONSTANT VARCHAR2(240)      := '/public/xxas/custom/emp/view/vacancy.xml';
  c_html_seeded CONSTANT VARCHAR2(240) := '/public/xxas/emp/view/htmlvacancy.xml';
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
   
  --
  -- Get the schema, either the custom one or the seeded one
  --
  IF xxas_emp_view_ui_pkg.emp_ui_view_exists(c_custom) THEN
      IF gc_log.proc=1 THEN gc_log.procedure_end(c_method, 900); END IF;  
      RETURN c_custom;
  ELSIF xxas_com_ctx_pkg.client_gte_version_flag('5') = 'Y' THEN
    IF gc_log.proc=1 THEN gc_log.procedure_end(c_method, 905); END IF;
    RETURN c_html_seeded;
  ELSE
    IF gc_log.proc=1 THEN gc_log.procedure_end(c_method, 905); END IF;  
      RETURN c_seeded;
  END IF;

END get_ui_xdb_uri;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_view_schema
-- ------------------------------------------------------------------------ *
FUNCTION get_view_schema
  (p_person_id          IN number
  ,p_business_group_id  IN number
  ,p_view_data_xml      IN XMLType
  ,p_refresh_event      IN varchar2 DEFAULT NULL
  ,p_refresh_field_name IN varchar2 DEFAULT NULL) RETURN xxas_emp_ui_view_type
IS

  c_method CONSTANT VARCHAR2(30) := 'get_view_schema';
  l_view_ff         xxas_fast_formula_pkg.g_ff_r;
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
   
  --
  -- Get the starting View FF, either the custom one or the seeded one
  --
  l_view_ff := xxas_fast_formula_pkg.init_person_ff
    (p_person_id         => p_person_id
    ,p_formula_name      => 'XXX_XXAS_EMP_VAC_VIEW'
    ,p_effective_date    => trunc(sysdate)
    ,p_business_group_id => p_business_group_id);
    
  IF l_view_ff.formula_id IS NULL THEN
    --
    -- If a custom formula isn't available, use the seeded one
    --
    l_view_ff := xxas_fast_formula_pkg.init_person_ff
      (p_person_id         => p_person_id
      ,p_formula_name      => 'XXAS_EMP_VAC_VIEW'
      ,p_effective_date    => trunc(sysdate)
      ,p_business_group_id => p_business_group_id);
  END IF;
    
  --
  -- Get the View Schema
  --
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN xxas_emp_view_ui_pkg.get_view_schema
    (p_ui_xdb_uri         => get_ui_xdb_uri
    ,p_view_data_xml      => p_view_data_xml
    ,p_start_ff           => l_view_ff
    ,p_refresh_event      => p_refresh_event
    ,p_refresh_field_name => p_refresh_field_name
    ,p_hide_null_fields   => true);
  
END get_view_schema;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_view_schema_xml
-- ------------------------------------------------------------------------ *
FUNCTION get_view_schema_xml 
  (p_person_id          IN number
  ,p_business_group_id  IN number
  ,p_view_data_xml      IN XMLType
  ,p_refresh_event      IN varchar2 DEFAULT NULL
  ,p_refresh_field_name IN varchar2 DEFAULT NULL) RETURN XMLType
IS

  c_method CONSTANT VARCHAR2(30) := 'get_view_schema_xml';
  l_schema          xxas_emp_ui_view_type;
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  l_schema := get_view_schema
    (p_person_id          => p_person_id
    ,p_business_group_id  => p_business_group_id
    ,p_view_data_xml      => p_view_data_xml
    ,p_refresh_event      => p_refresh_event
    ,p_refresh_field_name => p_refresh_field_name);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;
  RETURN l_schema.get_form_schema();

END get_view_schema_xml;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_view_schema_xml
-- ------------------------------------------------------------------------ *
FUNCTION get_view_schema_xml 
  (p_vacancy            IN xxas_per_all_vacancy
  ,p_refresh_event      IN varchar2             DEFAULT NULL
  ,p_refresh_field_name IN varchar2             DEFAULT NULL) RETURN XMLType
IS

  c_method CONSTANT VARCHAR2(30) := 'get_view_schema_xml';
  l_vacancy_xml     XMLType      := obj_to_xml(p_vacancy);
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN get_view_schema_xml
    (p_person_id          => p_vacancy.manager_id
    ,p_business_group_id  => p_vacancy.business_group_id
    ,p_view_data_xml      => l_vacancy_xml
    ,p_refresh_event      => p_refresh_event
    ,p_refresh_field_name => p_refresh_field_name);

END get_view_schema_xml;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_form_schema
-- ------------------------------------------------------------------------ *
FUNCTION get_form_schema
  (p_vacancy_id         IN number
  ,p_person_id          IN number
  ,p_business_group_id  IN number
  ,p_form_data_xml      IN XMLType
  ,p_refresh_event      IN varchar2 DEFAULT NULL
  ,p_refresh_field_name IN varchar2 DEFAULT NULL
  ,p_validate           IN boolean  DEFAULT false) RETURN xxas_emp_ui_view_type
IS

  c_method CONSTANT VARCHAR2(30) := 'get_form_schema';
  l_edit_ff         xxas_fast_formula_pkg.g_ff_r;
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;

  --
  -- Get the starting Edit FF, either the custom one or the seeded one
  --
  l_edit_ff := xxas_fast_formula_pkg.init_person_ff
    (p_person_id         => p_person_id
    ,p_formula_name      => 'XXX_XXAS_EMP_VAC_EDIT'
    ,p_effective_date    => trunc(sysdate)
    ,p_business_group_id => p_business_group_id);
    
  IF l_edit_ff.formula_id IS NULL THEN
    --
    -- If a custom formula isn't available, use the seeded one
    --
    l_edit_ff := xxas_fast_formula_pkg.init_person_ff
      (p_person_id         => p_person_id
      ,p_formula_name      => 'XXAS_EMP_VAC_EDIT'
      ,p_effective_date    => trunc(sysdate)
      ,p_business_group_id => p_business_group_id);
  END IF;
    
  --
  -- Get the Form Schema
  --
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN xxas_emp_view_ui_pkg.get_form_schema
    (p_ui_xdb_uri         => get_ui_xdb_uri
    ,p_form_data_xml      => p_form_data_xml
    ,p_start_ff           => l_edit_ff
    ,p_refresh_event      => p_refresh_event
    ,p_refresh_field_name => p_refresh_field_name
    ,p_validate           => p_validate
    ,p_instance_pk1_value => to_char(p_vacancy_id));

END get_form_schema;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_form_schema_xml
-- ------------------------------------------------------------------------ *
FUNCTION get_form_schema_xml 
  (p_vacancy_id         IN number
  ,p_person_id          IN number
  ,p_business_group_id  IN number
  ,p_form_data_xml      IN XMLType
  ,p_refresh_event      IN varchar2 DEFAULT NULL
  ,p_refresh_field_name IN varchar2 DEFAULT NULL) RETURN XMLType
IS

  c_method CONSTANT VARCHAR2(30) := 'get_form_schema_xml';
  l_schema          xxas_emp_ui_view_type;
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  l_schema := get_form_schema
    (p_vacancy_id         => p_vacancy_id
    ,p_person_id          => p_person_id
    ,p_business_group_id  => p_business_group_id
    ,p_form_data_xml      => p_form_data_xml
    ,p_refresh_event      => p_refresh_event
    ,p_refresh_field_name => p_refresh_field_name
    ,p_validate           => false);
    
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;
  RETURN l_schema.get_form_schema();

END get_form_schema_xml;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_form_schema_xml
-- ------------------------------------------------------------------------ *
FUNCTION get_form_schema_xml 
  (p_vacancy            IN xxas_per_all_vacancy
  ,p_refresh_event      IN varchar2             DEFAULT NULL
  ,p_refresh_field_name IN varchar2             DEFAULT NULL) RETURN XMLType
IS

  c_method CONSTANT VARCHAR2(30) := 'get_form_schema_xml';
  l_vacancy_xml     XMLType      := obj_to_xml(p_vacancy);
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN get_form_schema_xml
    (p_vacancy_id         => p_vacancy.vacancy_id
    ,p_person_id          => p_vacancy.manager_id
    ,p_business_group_id  => p_vacancy.business_group_id
    ,p_form_data_xml      => l_vacancy_xml
    ,p_refresh_event      => p_refresh_event
    ,p_refresh_field_name => p_refresh_field_name);

END get_form_schema_xml;

--
-- ------------------------------------------------------------------------ *
-- Name    : commit_form_data
-- ------------------------------------------------------------------------ *
FUNCTION commit_form_data 
  (p_vacancy_id    IN number
  ,p_form_data_xml IN XMLType) RETURN varchar2
IS

  c_method CONSTANT VARCHAR2(30) := 'commit_form_data';
  l_vacancy         xxas_per_all_vacancy;
  l_vacancy_xml     XMLType;
  l_schema          xxas_emp_ui_view_type;
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  --
  -- Get the vacancy as it currently is
  --
  l_vacancy     := vacancy_to_obj(p_vacancy_id);
  l_vacancy_xml := obj_to_xml(l_vacancy);

  --
  -- Convert the form data into a completed schema
  --
  l_schema := get_form_schema
    (p_vacancy_id        => p_vacancy_id
    ,p_person_id         => l_vacancy.manager_id
    ,p_business_group_id => l_vacancy.business_group_id
    ,p_form_data_xml     => p_form_data_xml
    ,p_validate          => true);
    
  --
  -- Copy the form to the vacancy object
  --
  xxas_emp_view_ui_pkg.copy_form_to_data
    (p_source_schema   => l_schema
    ,p_target_data_xml => l_vacancy_xml);
    
  --
  -- Get the Updated Object
  --
  l_vacancy := xml_to_obj(l_vacancy_xml);


 /* --
  -- Pre-commit customer hook
  --
  DECLARE
  
  BEGIN
    EXECUTE IMMEDIATE
          ' DECLARE '
       || ' BEGIN '
       || ' xxx_xxas_vacancy_pkg.pre_commit '
       || ' (p_vacancy => :vacancy); '
       || ' END; '
    USING IN OUT l_vacancy;
   
  EXCEPTION
    WHEN others THEN
      null;

  END; */
  
  submit_vacancy(l_vacancy);
    
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN null; /* Intention here is to return a message object */

EXCEPTION
  WHEN xxas_com_api_result_pkg.xxas_com_api_exception THEN
    RAISE xxas_com_api_result_pkg.xxas_com_api_exception;

END commit_form_data;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_href
-- ------------------------------------------------------------------------ *
FUNCTION get_href 
  (p_person_id  IN NUMBER
  ,p_vacancy_id IN NUMBER) RETURN varchar2
IS

  c_method CONSTANT VARCHAR2(30) := 'get_href';
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_person_id, p_vacancy_id);
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN xxas_com_rest_util_pkg.uri_from_path
          ('/people/'
          || to_char(p_person_id)
          || '/vacancies/'
          || to_char(p_vacancy_id));

END get_href;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_applicants_href
-- ------------------------------------------------------------------------ *
FUNCTION get_applicants_href 
  (p_vacancy_id IN NUMBER) RETURN varchar2
IS

  c_method CONSTANT VARCHAR2(30) := 'get_applicants_href';
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_vacancy_id);
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN xxas_com_rest_util_pkg.uri_from_path
          ('/vacancies/'
          || to_char(p_vacancy_id)
          || '/applicants');

END get_applicants_href;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_applicants_export_href
-- ------------------------------------------------------------------------ *
FUNCTION get_applicants_export_href 
  (p_vacancy_id IN NUMBER) RETURN varchar2
IS

  c_method CONSTANT VARCHAR2(30) := 'get_applicants_export_href';
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_vacancy_id);
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN xxas_com_rest_util_pkg.uri_from_path
          ('/vacancies/'
          || to_char(p_vacancy_id)
          || '/applicants'
          || '/{applicantId}'
          || '?includeViewSchema=true?mode=download_blob&_accept=application/pdf');

END get_applicants_export_href;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_offers_href
-- ------------------------------------------------------------------------ *
FUNCTION get_offers_href 
  (p_vacancy_id IN NUMBER) RETURN varchar2
IS

  c_method CONSTANT VARCHAR2(30) := 'get_offers_href';
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_vacancy_id);
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN xxas_com_rest_util_pkg.uri_from_path
          ('/vacancies/'
          || to_char(p_vacancy_id)
          || '/offers');

END get_offers_href;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_ad_hoc_comms_href
-- ------------------------------------------------------------------------ *
FUNCTION get_ad_hoc_comms_href 
  (p_vacancy_id IN NUMBER) RETURN varchar2
IS

  c_method CONSTANT VARCHAR2(30) := 'get_ad_hoc_comms_href';
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_vacancy_id);
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN xxas_com_rest_util_pkg.uri_from_path
          ('/vacancies/'
          || to_char(p_vacancy_id)
          || '/applicant-adhoc-comms');

END get_ad_hoc_comms_href;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_view_schema_href
-- ------------------------------------------------------------------------ *
FUNCTION get_view_schema_href 
  (p_person_id  IN NUMBER
  ,p_vacancy_id IN NUMBER) RETURN varchar2
IS

  c_method CONSTANT VARCHAR2(30) := 'get_view_schema_href';
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_person_id, p_vacancy_id);
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN xxas_com_rest_util_pkg.uri_from_path
          ('/people/'
          || to_char(p_person_id)
          || '/vacancies/'
          || to_char(p_vacancy_id)
          || '/view-schema');

END get_view_schema_href;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_form_schema_href
-- ------------------------------------------------------------------------ *
FUNCTION get_form_schema_href 
  (p_person_id  IN NUMBER
  ,p_vacancy_id IN NUMBER) RETURN varchar2
IS

  c_method CONSTANT VARCHAR2(30) := 'get_form_schema_href';
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_person_id, p_vacancy_id);
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN xxas_com_rest_util_pkg.uri_from_path
          ('/people/'
          || to_char(p_person_id)
          || '/vacancies/'
          || to_char(p_vacancy_id)
          || '/form-schema');

END get_form_schema_href;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_apl_statuses
-- ------------------------------------------------------------------------ *
PROCEDURE get_apl_statuses 
  (p_vacancy_id   IN            NUMBER
  ,p_num_active      OUT NOCOPY NUMBER
  ,p_num_term        OUT NOCOPY NUMBER
  ,p_num_interview1  OUT NOCOPY NUMBER
  ,p_num_interview2  OUT NOCOPY NUMBER
  ,p_num_offer       OUT NOCOPY NUMBER
  ,p_num_accepted    OUT NOCOPY NUMBER)
IS

  c_method CONSTANT VARCHAR2(30) := 'get_apl_statuses';

  CURSOR   csr_apl_statuses
  IS  
  SELECT   s.num_active
          ,s.num_term
          ,s.num_interview1
          ,s.num_interview2
          ,s.num_offer
          ,s.num_accepted
  FROM    (SELECT   nvl(extractvalue(a2.thexml, '//status[@code="ACTIVE_APL"]'), 0) num_active
                   ,nvl(extractvalue(a2.thexml, '//status[@code="TERM_APL"]'), 0) num_term
                   ,nvl(extractvalue(a2.thexml, '//status[@code="INTERVIEW1"]'), 0) num_interview1
                   ,nvl(extractvalue(a2.thexml, '//status[@code="INTERVIEW2"]'), 0) num_interview2
                   ,nvl(extractvalue(a2.thexml, '//status[@code="OFFER"]'), 0) num_offer
                   ,nvl(extractvalue(a2.thexml, '//status[@code="ACCEPTED"]'), 0) num_accepted
           FROM    (SELECT  XMLAgg
                              (XMLElement
                                ("status"
                                ,XMLAttributes
                                  (a1.per_system_status "code")
                                ,count(*)
                                )
                              ) thexml
                   FROM     (SELECT   CASE WHEN paaf.effective_end_date < trunc(sysdate) THEN
                                        'TERM_APL'
																				WHEN paaf.effective_end_date = paaf.effective_start_date THEN
                                        'TERM_APL'
                                           ELSE past.per_system_status
                                      END per_system_status
                             FROM     per_all_assignments_f paaf
                                     ,per_assignment_status_types past
                             WHERE    paaf.vacancy_id = p_vacancy_id
                             AND      paaf.assignment_type = 'A'
                             AND      paaf.assignment_status_type_id = past.assignment_status_type_id
                             AND      paaf.effective_start_date =
                                         (SELECT max(paaf2.effective_start_date)
                                           FROM   per_all_assignments_f paaf2
                                           WHERE  paaf2.assignment_id = paaf.assignment_id)) a1
                   GROUP BY a1.per_system_status) a2) s;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_vacancy_id);
  END IF;
  
  OPEN  csr_apl_statuses;
  FETCH csr_apl_statuses INTO p_num_active
                             ,p_num_term
                             ,p_num_interview1
                             ,p_num_interview2
                             ,p_num_offer
                             ,p_num_accepted;
  CLOSE csr_apl_statuses;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

END get_apl_statuses;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_apl_interested
-- ------------------------------------------------------------------------ *
PROCEDURE get_apl_interested 
  (p_vacancy_id   IN             NUMBER
  ,p_num_not_set      OUT NOCOPY NUMBER
  ,p_num_no           OUT NOCOPY NUMBER
  ,p_num_unsure       OUT NOCOPY NUMBER
  ,p_num_yes          OUT NOCOPY NUMBER)
IS

  c_method CONSTANT VARCHAR2(30) := 'get_apl_interested';

  CURSOR csr_apl_interested
  IS  
  SELECT nvl(extractvalue(i2.thexml, '//status[@interested="0"]'), 0) num_not_set
        ,nvl(extractvalue(i2.thexml, '//status[@interested="1"]'), 0) num_no
        ,nvl(extractvalue(i2.thexml, '//status[@interested="2"]'), 0) num_unsure
        ,nvl(extractvalue(i2.thexml, '//status[@interested="3"]'), 0) num_yes
  FROM  (SELECT XMLAgg
                  (XMLElement
                    ("status"
                    ,XMLAttributes
                      (i.interested "interested")
                    ,count(*)
                    )
                  ) thexml
          FROM  (SELECT nvl
                         (xxas_applicant_pkg.get_interested
                           (paaf.ass_attribute1, paaf.ass_attribute2, paaf.ass_attribute3
                           ,paaf.ass_attribute4, paaf.ass_attribute5, paaf.ass_attribute6, paaf.ass_attribute7
                           ,paaf.ass_attribute8, paaf.ass_attribute9, paaf.ass_attribute10, paaf.ass_attribute11
                           ,paaf.ass_attribute12, paaf.ass_attribute13, paaf.ass_attribute14, paaf.ass_attribute15
                           ,paaf.ass_attribute16, paaf.ass_attribute17, paaf.ass_attribute18, paaf.ass_attribute19
                           ,paaf.ass_attribute20, paaf.ass_attribute21, paaf.ass_attribute22, paaf.ass_attribute23
                           ,paaf.ass_attribute24, paaf.ass_attribute25, paaf.ass_attribute26, paaf.ass_attribute27
                           ,paaf.ass_attribute28, paaf.ass_attribute29, paaf.ass_attribute30), 0) interested
                 FROM   per_all_assignments_f paaf
                 WHERE  paaf.assignment_type = 'A'
                 AND    trunc(sysdate) BETWEEN
                        paaf.effective_start_date AND paaf.effective_end_date
                 AND    paaf.vacancy_id = p_vacancy_id) i
          GROUP BY i.interested) i2;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_vacancy_id);
  END IF;
  
  OPEN  csr_apl_interested;
  FETCH csr_apl_interested INTO p_num_not_set
                               ,p_num_no
                               ,p_num_unsure
                               ,p_num_yes;
  CLOSE csr_apl_interested;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  
END get_apl_interested;

--
-- ------------------------------------------------------------------------ *
-- Name    : populate_tasks
-- ------------------------------------------------------------------------ *
PROCEDURE populate_tasks 
  (p_vacancy IN OUT NOCOPY xxas_per_all_vacancy)
IS

  c_method CONSTANT VARCHAR2(30) := 'populate_tasks';
  l_tasks  xxas_vacancy_task_tbl;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;

  l_tasks := xxas_vacancy_tasks_pkg.get_tasks(p_vacancy);
  p_vacancy.vacancy_tasks := l_tasks;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

END populate_tasks;

--
-- ------------------------------------------------------------------------ *
-- Name    : post_processing
-- ------------------------------------------------------------------------ *
PROCEDURE post_processing 
  (p_vacancy IN OUT NOCOPY xxas_per_all_vacancy)
IS

  c_method CONSTANT VARCHAR2(30) := 'post_processing';
  e_no_vac          EXCEPTION;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  IF p_vacancy IS NULL OR p_vacancy.vacancy_id IS NULL THEN
    RAISE e_no_vac;
  END IF;
  
  get_apl_statuses 
    (p_vacancy_id     => p_vacancy.vacancy_id
    ,p_num_active     => p_vacancy.num_active
    ,p_num_term       => p_vacancy.num_term
    ,p_num_interview1 => p_vacancy.num_interview1
    ,p_num_interview2 => p_vacancy.num_interview2
    ,p_num_offer      => p_vacancy.num_offer
    ,p_num_accepted   => p_vacancy.num_accepted);
    
  p_vacancy.num_total := p_vacancy.num_active +
                         p_vacancy.num_term +
                         p_vacancy.num_interview1 +
                         p_vacancy.num_interview2 +
                         p_vacancy.num_offer +
                         p_vacancy.num_accepted;
                         
  get_apl_interested 
    (p_vacancy_id     => p_vacancy.vacancy_id
    ,p_num_not_set    => p_vacancy.num_not_set
    ,p_num_no         => p_vacancy.num_no
    ,p_num_unsure     => p_vacancy.num_unsure
    ,p_num_yes        => p_vacancy.num_yes);
    
  populate_tasks
    (p_vacancy        => p_vacancy);
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

EXCEPTION

  WHEN e_no_vac THEN
    IF gc_log.proc=1 THEN gc_log.procedure_end(c_method, 900); END IF;  
  
END post_processing;
--
-- ------------------------------------------------------------------------ *
-- Name    : populate_page_attributes
-- ------------------------------------------------------------------------ *
PROCEDURE populate_page_attributes
  (p_vacancy_id     IN NUMBER
  ,p_transaction_id IN NUMBER)
IS

  c_method CONSTANT     VARCHAR2(100) := 'populate_page_attributes';
  l_item_key            VARCHAR2(100);
  l_transaction_step_id NUMBER;
  l_person_id           NUMBER;

  CURSOR csr_get_trans_details IS
  SELECT hat.item_key,
         hats.transaction_step_id,
         hat.creator_person_id
  FROM   hr_api_transactions hat,
         hr_api_transaction_steps hats
  WHERE  hat.transaction_id = p_transaction_id
  AND    hats.transaction_id = hat.transaction_id
  ORDER BY hats.transaction_step_id asc;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;

  --
  -- Get item key and transaction step id
  --
  IF p_transaction_id IS NOT NULL THEN
    OPEN  csr_get_trans_details;
    FETCH csr_get_trans_details INTO l_item_key,l_transaction_step_id,l_person_id;
    CLOSE csr_get_trans_details;

    -- 
    -- set page attributes.
    --
    xxas_com_worklist_pkg.populate_page_attributes(p_transaction_id      => p_transaction_id
                                                  ,p_transaction_step_id => l_transaction_step_id
                                                  ,p_item_type           => g_item_type
                                                  ,p_item_key            => l_item_key
                                                  ,p_edit_page_id        => xxas_com_worklist_pkg.g_VACANCY_EDIT_PAGE
                                                  ,p_edit_object_id      => p_vacancy_id
                                                  ,p_edit_target         => 'modal'
                                                  ,p_view_page_id        => xxas_com_worklist_pkg.g_VACANCY_VIEW_PAGE
                                                  ,p_view_object_id      => p_vacancy_id
                                                                            || '_' || l_person_id
                                                                            || '_' || p_transaction_id
                                                  ,p_view_target         => 'modal'
                                                  ,p_modal_width         => 'large');    
    
  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

END populate_page_attributes;
--
--
-- ------------------------------------------------------------------------ *
-- Name    : obj_to_hrapi_doc
-- ------------------------------------------------------------------------ *
FUNCTION obj_to_hrapi_doc 
  (p_vacancy IN xxas_per_all_vacancy) RETURN XMLType
IS

  c_method         CONSTANT VARCHAR2(30) := 'obj_to_hrapi_doc';
  l_doc                     XMLType;
  l_vacancy_tbl             xxas_per_all_vacancy_tbl;
  l_currency                varchar2(400);
  l_vacancy_name_is_auto    varchar2(400);
  l_budget_measurement_type varchar2(400);
  l_number_of_openings      number;
  l_location_default        varchar2(400);
  l_organization_default    varchar2(400);
  
  CURSOR csr_trans_doc IS
  SELECT XMLElement
           ("Transaction"
           ,XMLElement
             ("TransCtx"
             ,XMLElement("pAMETranType", 'IRCVACAPPROVAL')
             ,XMLElement
               ("pNtfSubMsg"
               ,CASE WHEN v.transaction_type = 'INSERT' THEN
                          'IRC_VACANCY_APPROVAL_NEW'
                     WHEN v.transaction_type = 'UPDATE' THEN
                          'IRC_VACANCY_APPROVAL_UPDATE'
                END
                )
             ,XMLElement
               ("TransactionIdentifier"
               ,CASE WHEN v.transaction_type = 'INSERT' THEN
                          'CREATE_VACANCY'
                     WHEN v.transaction_type = 'UPDATE' THEN
                          'EDIT_VACANCY'
                END
                )
             ,XMLElement("TransactionId", v.transaction_id)
             ,XMLElement
               ("CreatorPrsnId"
               ,XMLAttributes
                 ('n' as "dataType")
               ,xxas_util_pkg.get_per_from_user(v.created_by)
               )
             ,XMLElement
               ("RelaunchFunction"
               ,CASE WHEN v.transaction_type = 'INSERT' THEN
                          'IRC_VAC_NEW_SFL_LAUNCH'
                     WHEN v.transaction_type = 'UPDATE' THEN
                          'IRC_VAC_EDIT_SFL_LAUNCH'
                END
                )
             ,XMLElement("LoginPrsnBgId", v.business_group_id)
             ,XMLElement("ProcessName", g_PROCESS_NAME)
             ,XMLElement
               ("EffectiveDate"
               ,XMLAttributes
                 ('d' as "dataType")
               ,to_char(sysdate, g_HRAPI_DATETIME_FMT)
               )
             ,XMLElement
               ("pCalledFrom"
               ,CASE WHEN v.transaction_type = 'INSERT' THEN
                          'IRC_VAC_DETS_NEW'
                     WHEN v.transaction_type = 'UPDATE' THEN
                          'IRC_VAC_POST_EDIT'
                END
               )
             ,XMLElement
               ("pCalledId"
               ,XMLAttributes('n' as "dataType")
               ,CASE WHEN v.transaction_type = 'INSERT' THEN
                          fnd_function.get_function_id('IRC_VAC_DETS_NEW')
                     WHEN v.transaction_type = 'UPDATE' THEN
                          fnd_function.get_function_id('IRC_VAC_POST_EDIT')
                END
               )
             ,XMLElement("TransactionType", 'WF')
             ,XMLElement
               ("CNode"
               ,XMLAttributes
                 ('VacancyParams' as "name"
                 ,'Ht' as "type")
               ,XMLElement("vacancyNameAuto", l_vacancy_name_is_auto)
               ,XMLElement("loggedInPersonBgId", v.business_group_id)
               ,XMLElement
                 ("organizationId"
                 ,XMLAttributes
                   ('n' as "dataType")
                 ,v.organization_id)
               ,XMLElement("organizationDefault", l_organization_default)
               ,XMLElement("numberOfOpenings", to_char(l_number_of_openings, 'FM999999999990D00', 'NLS_NUMERIC_CHARACTERS = ''.,'''))
               ,XMLElement("currency", l_currency)
               ,decode
                 (v.transaction_type, 'INSERT'
                 ,XMLElement("wfProcess", 'IRC_VACANCY_NEW_V2')
                 ,null
                 )
               ,XMLElement
                 ("locationId"
                 ,XMLAttributes
                   ('n' as "dataType")
                 ,v.location_id)
               ,decode
                 (v.transaction_type, 'INSERT'
                 ,XMLElement("launchPage", 'IRC_VAC_NEW_LAUNCH_WF')
                 ,null
                 )
               ,XMLElement("EffectiveDate", to_char(sysdate, g_HRAPI_DATE_FMT))
               ,XMLElement("VacancyId", v.vacancy_id)
               ,decode
                 (v.transaction_type, 'INSERT'
                 ,XMLElement("approvalProcess", 'IRC_VACANCY_NEW')
                 ,null
                 )
               ,XMLElement("locationDefault", l_location_default)
               ,XMLElement("vacancyName", v.vacancy_name)
               ,XMLElement
                 ("multipleBusinessGroups"
                 ,XMLAttributes('b' as "dataType")
                 ,'false')
               ,XMLElement("budgetMeasurementType", l_budget_measurement_type)
               ,XMLElement
                 ("ActionType"
                 ,CASE WHEN v.transaction_type = 'INSERT' THEN
                            'createVacancy'
                       WHEN v.transaction_type = 'UPDATE' THEN
                            'editVacancy'
                  END
                 )                 
               ,XMLElement("dmlMode", v.transaction_type)
               ,XMLElement("loggedInPersonId", fnd_global.employee_id)
               ,XMLElement("dataState", 'complete')
               )
             ,XMLForest
               (800 as "pAMEAppId"
               ,'YD' as "pApprovalReqd"
               ,'IRC' as "TransactionGroup")
             ,decode
               (v.comments_to_approver, null, null
               ,XMLElement
                 ("pApprComments", v.comments_to_approver))
             ,XMLElement("ReviewTemplateRNAttr", 'IRC_VACANCY_APPROVE_NOTIF_STK')
             ,XMLElement
               ("TransactionRefId"
               ,XMLAttributes
                  ('n' as "dataType")
               ,v.vacancy_id
               )
             ,XMLForest
               (v.vacancy_name as "ApiAddtnlInfo"
               ,'PER_ALL_VACANCIES' as "TransactionRefTable"
               ,v.hr_api_status_code as "TxnStatus"
               ,fnd_global.employee_id as "LoginPrsnId")
             ,XMLElement
               ("ApprNtfSubCallBack"
               ,CASE WHEN v.transaction_type = 'INSERT' THEN
                          'irc_approvals.getNotifSubjectForCreate'
                     WHEN v.transaction_type = 'UPDATE' THEN
                          'irc_approvals.getNotifSubjectForEdit'
                END
               )
             ,XMLElement("ProductCode", 'IRC')
             ,XMLElement("TxnFinalizeApi", 'PER_VACANCY_SWI.FINALIZE_TRANSACTION')
             ,XMLElement("ItemType", g_ITEM_TYPE)
             )
           ,XMLElement
             ("EoApiMap"
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.irc.schema.server.IrcAgencyVacanciesEO' as "Name")
               ,'irc_agency_vacancies_swi.process_api')
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.irc.schema.server.IrcRecTeamMembersEO' as "Name")
               ,'irc_rec_team_members_swi.process_api')
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.VacancyCompetenceElementsEO' as "Name")
               ,'HR_COMPETENCE_ELEMENT_SWI.PROCESS_API')
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerAllVacanciesEO' as "Name")
               ,'per_vacancy_swi.process_api')
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerRecruitmentActivitiesEO' as "Name")
               ,'per_recruitment_activity_swi.process_api')
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerRequisitionsEO' as "Name")
               ,'per_requisitions_swi.process_api')
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.per.schema.server.PerRecruitmentActivityForEO' as "Name")
               ,'per_rec_activity_for_swi.process_api')
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.irc.schema.server.IrcVacancySearchCriteriaEO' as "Name")
               ,'irc_search_criteria_swi.process_vacancy_api')
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.irc.schema.server.IrcVariableCompElementsEO' as "Name")
               ,'irc_variable_comp_element_swi.process_api')
             ,XMLElement
               ("EO"
               ,XMLAttributes('oracle.apps.irc.schema.server.IrcPostingContentsVlEO' as "Name")
               ,'irc_posting_content_swi.process_api')
             )
           ,XMLElement
             ("TransCache"
             ,XMLElement
               ("AM"
               ,XMLAttributes('1038941583453' as "MomVer")
               ,XMLElement("cd")
               ,XMLElement
                 ("TXN"
                 ,XMLAttributes
                   (0 as "Def"
                   ,0 as "New"
                   ,2 as "Lok"
                   ,316 as "pcid")
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.irc.schema.server.IrcPostingContentsVlEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(v.primary_posting_id))
                   ,XMLElement
                     ("IrcPostingContentsVlEORow"
                     ,XMLAttributes
                       (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                       ,'Y' as "PK"
                       ,'Y' as "CI")
                     ,decode(v.primary_posting_id, null
                            ,XMLElement("PostingContentId", XMLAttributes('true' as "null"), v.primary_posting_id)
                            ,XMLElement("PostingContentId", v.primary_posting_id))
                     ,decode(v.display_manager_info, null
                            ,XMLElement("DisplayManagerInfo", XMLAttributes('true' as "null"))
                            ,XMLElement("DisplayManagerInfo", xxas_util_pkg.boolean_to_flag(v.display_manager_info)))
                     ,decode(v.display_recruiter_info, null
                            ,XMLElement("DisplayRecruiterInfo", XMLAttributes('true' as "null"))
                            ,XMLElement("DisplayRecruiterInfo", xxas_util_pkg.boolean_to_flag(v.display_recruiter_info)))
                     ,decode(v.posting_name, null
                            ,XMLElement("Name", XMLAttributes('true' as "null"), v.posting_name)
                            ,XMLElement("Name", v.posting_name))
                     ,decode(v.posting_org_name, null
                            ,XMLElement("OrgName", XMLAttributes('true' as "null"), v.posting_org_name)
                            ,XMLElement("OrgName", v.posting_org_name))
                     ,decode(dbms_lob.substr(v.posting_org_description, 1, 1), null
                            ,XMLElement("OrgDescription", XMLAttributes('true' as "null"), v.posting_org_description)
                            ,XMLElement("OrgDescription", v.posting_org_description))
                     ,decode(v.posting_job_title, null
                            ,XMLElement("JobTitle", XMLAttributes('true' as "null"), v.posting_job_title)
                            ,XMLElement("JobTitle", v.posting_job_title))
                     ,decode(dbms_lob.substr(v.posting_brief_description, 1, 1), null
                            ,XMLElement("BriefDescription", XMLAttributes('true' as "null"), v.posting_brief_description)
                            ,XMLElement("BriefDescription", v.posting_brief_description))
                     ,decode(dbms_lob.substr(v.posting_detailed_description, 1, 1), null
                            ,XMLElement("DetailedDescription", XMLAttributes('true' as "null"), v.posting_detailed_description)
                            ,XMLElement("DetailedDescription", v.posting_detailed_description))
                     ,decode(dbms_lob.substr(v.posting_job_requirements, 1, 1), null
                            ,XMLElement("JobRequirements", XMLAttributes('true' as "null"), v.posting_job_requirements)
                            ,XMLElement("JobRequirements", v.posting_job_requirements))
                     ,decode(dbms_lob.substr(v.posting_additional_details, 1, 1), null
                            ,XMLElement("AdditionalDetails", XMLAttributes('true' as "null"), v.posting_additional_details)
                            ,XMLElement("AdditionalDetails", v.posting_additional_details))
                     ,decode(dbms_lob.substr(v.posting_how_to_apply, 1, 1), null
                            ,XMLElement("HowToApply", XMLAttributes('true' as "null"), v.posting_how_to_apply)
                            ,XMLElement("HowToApply", v.posting_how_to_apply))
                     ,decode(dbms_lob.substr(v.image_url, 1, 1), null
                            ,XMLElement("ImageUrl", XMLAttributes('true' as "null"), v.image_url)
                            ,XMLElement("ImageUrl", v.image_url))
                     ,decode(dbms_lob.substr(v.image_url_alt, 1, 1), null
                            ,XMLElement("ImageUrlAlt", XMLAttributes('true' as "null"), v.image_url_alt)
                            ,XMLElement("ImageUrlAlt", v.image_url_alt))
                     ,decode(v.ipc_attribute_category, null
                            ,XMLElement("AttributeCategory", XMLAttributes('true' as "null"), v.ipc_attribute_category)
                            ,XMLElement("AttributeCategory", v.ipc_attribute_category))
                     ,decode(v.ipc_attribute1, null
                            ,XMLElement("Attribute1", XMLAttributes('true' as "null"), v.ipc_attribute1)
                            ,XMLElement("Attribute1", v.ipc_attribute1))
                     ,decode(v.ipc_attribute2, null
                            ,XMLElement("Attribute2", XMLAttributes('true' as "null"), v.ipc_attribute2)
                            ,XMLElement("Attribute2", v.ipc_attribute2))
                     ,decode(v.ipc_attribute3, null
                            ,XMLElement("Attribute3", XMLAttributes('true' as "null"), v.ipc_attribute3)
                            ,XMLElement("Attribute3", v.ipc_attribute3))
                     ,decode(v.ipc_attribute4, null
                            ,XMLElement("Attribute4", XMLAttributes('true' as "null"), v.ipc_attribute4)
                            ,XMLElement("Attribute4", v.ipc_attribute4))
                     ,decode(v.ipc_attribute5, null
                            ,XMLElement("Attribute5", XMLAttributes('true' as "null"), v.ipc_attribute5)
                            ,XMLElement("Attribute5", v.ipc_attribute5))
                     ,decode(v.ipc_attribute6, null
                            ,XMLElement("Attribute6", XMLAttributes('true' as "null"), v.ipc_attribute6)
                            ,XMLElement("Attribute6", v.ipc_attribute6))
                     ,decode(v.ipc_attribute7, null
                            ,XMLElement("Attribute7", XMLAttributes('true' as "null"), v.ipc_attribute7)
                            ,XMLElement("Attribute7", v.ipc_attribute7))
                     ,decode(v.ipc_attribute8, null
                            ,XMLElement("Attribute8", XMLAttributes('true' as "null"), v.ipc_attribute8)
                            ,XMLElement("Attribute8", v.ipc_attribute8))
                     ,decode(v.ipc_attribute9, null
                            ,XMLElement("Attribute9", XMLAttributes('true' as "null"), v.ipc_attribute9)
                            ,XMLElement("Attribute9", v.ipc_attribute9))
                     ,decode(v.ipc_attribute10, null
                            ,XMLElement("Attribute10", XMLAttributes('true' as "null"), v.ipc_attribute10)
                            ,XMLElement("Attribute10", v.ipc_attribute10))
                     ,decode(v.ipc_attribute11, null
                            ,XMLElement("Attribute11", XMLAttributes('true' as "null"), v.ipc_attribute11)
                            ,XMLElement("Attribute11", v.ipc_attribute11))
                     ,decode(v.ipc_attribute12, null
                            ,XMLElement("Attribute12", XMLAttributes('true' as "null"), v.ipc_attribute12)
                            ,XMLElement("Attribute12", v.ipc_attribute12))
                     ,decode(v.ipc_attribute13, null
                            ,XMLElement("Attribute13", XMLAttributes('true' as "null"), v.ipc_attribute13)
                            ,XMLElement("Attribute13", v.ipc_attribute13))
                     ,decode(v.ipc_attribute14, null
                            ,XMLElement("Attribute14", XMLAttributes('true' as "null"), v.ipc_attribute14)
                            ,XMLElement("Attribute14", v.ipc_attribute14))
                     ,decode(v.ipc_attribute15, null
                            ,XMLElement("Attribute15", XMLAttributes('true' as "null"), v.ipc_attribute15)
                            ,XMLElement("Attribute15", v.ipc_attribute15))
                     ,decode(v.ipc_attribute16, null
                            ,XMLElement("Attribute16", XMLAttributes('true' as "null"), v.ipc_attribute16)
                            ,XMLElement("Attribute16", v.ipc_attribute16))
                     ,decode(v.ipc_attribute17, null
                            ,XMLElement("Attribute17", XMLAttributes('true' as "null"), v.ipc_attribute17)
                            ,XMLElement("Attribute17", v.ipc_attribute17))
                     ,decode(v.ipc_attribute18, null
                            ,XMLElement("Attribute18", XMLAttributes('true' as "null"), v.ipc_attribute18)
                            ,XMLElement("Attribute18", v.ipc_attribute18))
                     ,decode(v.ipc_attribute19, null
                            ,XMLElement("Attribute19", XMLAttributes('true' as "null"), v.ipc_attribute19)
                            ,XMLElement("Attribute19", v.ipc_attribute19))
                     ,decode(v.ipc_attribute20, null
                            ,XMLElement("Attribute20", XMLAttributes('true' as "null"), v.ipc_attribute20)
                            ,XMLElement("Attribute20", v.ipc_attribute20))
                     ,decode(v.ipc_attribute21, null
                            ,XMLElement("Attribute21", XMLAttributes('true' as "null"), v.ipc_attribute21)
                            ,XMLElement("Attribute21", v.ipc_attribute21))
                     ,decode(v.ipc_attribute22, null
                            ,XMLElement("Attribute22", XMLAttributes('true' as "null"), v.ipc_attribute22)
                            ,XMLElement("Attribute22", v.ipc_attribute22))
                     ,decode(v.ipc_attribute23, null
                            ,XMLElement("Attribute23", XMLAttributes('true' as "null"), v.ipc_attribute23)
                            ,XMLElement("Attribute23", v.ipc_attribute23))
                     ,decode(v.ipc_attribute24, null
                            ,XMLElement("Attribute24", XMLAttributes('true' as "null"), v.ipc_attribute24)
                            ,XMLElement("Attribute24", v.ipc_attribute24))
                     ,decode(v.ipc_attribute25, null
                            ,XMLElement("Attribute25", XMLAttributes('true' as "null"), v.ipc_attribute25)
                            ,XMLElement("Attribute25", v.ipc_attribute25))
                     ,decode(v.ipc_attribute26, null
                            ,XMLElement("Attribute26", XMLAttributes('true' as "null"), v.ipc_attribute26)
                            ,XMLElement("Attribute26", v.ipc_attribute26))
                     ,decode(v.ipc_attribute27, null
                            ,XMLElement("Attribute27", XMLAttributes('true' as "null"), v.ipc_attribute27)
                            ,XMLElement("Attribute27", v.ipc_attribute27))
                     ,decode(v.ipc_attribute28, null
                            ,XMLElement("Attribute28", XMLAttributes('true' as "null"), v.ipc_attribute28)
                            ,XMLElement("Attribute28", v.ipc_attribute28))
                     ,decode(v.ipc_attribute29, null
                            ,XMLElement("Attribute29", XMLAttributes('true' as "null"), v.ipc_attribute29)
                            ,XMLElement("Attribute29", v.ipc_attribute29))
                     ,decode(v.ipc_attribute30, null
                            ,XMLElement("Attribute30", XMLAttributes('true' as "null"), v.ipc_attribute30)
                            ,XMLElement("Attribute30", v.ipc_attribute30))
                     ,XMLElement("IpcInformationCategory", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation1", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation2", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation3", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation4", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation5", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation6", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation7", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation8", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation9", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation10", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation11", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation12", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation13", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation14", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation15", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation16", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation17", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation18", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation19", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation20", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation21", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation22", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation23", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation24", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation25", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation26", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation27", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation28", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation29", XMLAttributes('true' as "null"))
                     ,XMLElement("IpcInformation30", XMLAttributes('true' as "null"))
                     ,decode(v.ipc_last_update_date, null
                            ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"), to_char(v.ipc_last_update_date, g_HRAPI_DATETIME_FMT))
                            ,XMLElement("LastUpdateDate", to_char(v.ipc_last_update_date, g_HRAPI_DATETIME_FMT)))
                     ,decode(v.ipc_last_updated_by, null
                            ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"), v.ipc_last_updated_by)
                            ,XMLElement("LastUpdatedBy", v.ipc_last_updated_by))
                     ,decode(v.ipc_last_update_login, null
                            ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"), v.ipc_last_update_login)
                            ,XMLElement("LastUpdateLogin", v.ipc_last_update_login))
                      ,decode(v.ipc_created_by, null
                            ,XMLElement("CreatedBy", XMLAttributes('true' as "null"), v.ipc_created_by)
                            ,XMLElement("CreatedBy", v.ipc_created_by))
                     ,decode(v.ipc_creation_date, null
                            ,XMLElement("CreationDate", XMLAttributes('true' as "null"), to_char(v.ipc_creation_date, g_HRAPI_DATETIME_FMT))
                            ,XMLElement("CreationDate", to_char(v.ipc_creation_date, g_HRAPI_DATETIME_FMT)))
                     ,decode(v.ipc_object_version_number, null
                            ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"), v.ipc_object_version_number)
                            ,XMLElement("ObjectVersionNumber", v.ipc_object_version_number))
                     ,decode(dbms_lob.substr(v.benefit_info, 1, 1), null
                            ,XMLElement("BenefitInfo", XMLAttributes('true' as "null"), v.benefit_info)
                            ,XMLElement("BenefitInfo", v.benefit_info))
                     ,XMLElement("DisplayLookup", 'YES_NO')
                     ,XMLElement
                       ("CEO"
                       ,(SELECT
                          XMLAgg
                           (XMLElement
                             ("EO"
                             ,XMLAttributes
                               ('oracle.apps.per.schema.server.PerRecruitmentActivitiesEO' as "Name")
                             ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(pra.recruitment_activity_id))
                             ,XMLElement
                               ("PerRecruitmentActivitiesEORow"
                               ,XMLAttributes
                                 (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                                 ,'Y' as "PK"
                                 ,'Y' as "CI")
                               ,decode(pra.recruitment_activity_id, null
                                      ,XMLElement("RecruitmentActivityId", XMLAttributes('true' as "null"), pra.recruitment_activity_id)
                                      ,XMLElement("RecruitmentActivityId", pra.recruitment_activity_id))
                               ,decode(v.business_group_id, null
                                      ,XMLElement("BusinessGroupId", XMLAttributes('true' as "null"), v.business_group_id)
                                      ,XMLElement("BusinessGroupId", v.business_group_id))
                               ,XMLElement("AuthorisingPersonId", XMLAttributes('true' as "null"))
                               ,XMLElement("RunByOrganizationId", XMLAttributes('true' as "null"))
                               ,XMLElement("InternalContactPersonId", XMLAttributes('true' as "null"))
                               ,XMLElement("ParentRecruitmentActivityId", XMLAttributes('true' as "null"))
                               ,XMLElement("CurrencyCode", XMLAttributes('true' as "null"))
                               ,decode(pra.date_start, null
                                      ,XMLElement("DateStart", XMLAttributes('true' as "null"))
                                      ,XMLElement("DateStart", to_char(pra.date_start, g_HRAPI_DATE_FMT)))
                               ,decode(pra.name, null
                                      ,XMLElement("Name", XMLAttributes('true' as "null"), pra.name)
                                      ,XMLElement("Name", pra.name))
                               ,XMLElement("ActualCost", XMLAttributes('true' as "null"))
                               ,XMLElement("Comments", XMLAttributes('true' as "null"))
                               ,XMLElement("ContactTelephoneNumber", XMLAttributes('true' as "null"))
                               ,XMLElement("DateClosing", XMLAttributes('true' as "null"))
                               ,decode(pra.date_end, null
                                      ,XMLElement("DateEnd", XMLAttributes('true' as "null"))
                                      ,XMLElement("DateEnd", to_char(pra.date_end, g_HRAPI_DATE_FMT)))                               
                               ,XMLElement("ExternalContact", XMLAttributes('true' as "null"))
                               ,XMLElement("PlannedCost", XMLAttributes('true' as "null"))
                               ,XMLElement("Type", XMLAttributes('true' as "null"))
                               ,XMLElement("AttributeCategory", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute1", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute2", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute3", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute4", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute5", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute6", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute7", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute8", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute9", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute10", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute11", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute12", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute13", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute14", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute15", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute16", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute17", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute18", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute19", XMLAttributes('true' as "null"))
                               ,XMLElement("Attribute20", XMLAttributes('true' as "null"))
                               ,decode(pra.last_update_date, null
                                      ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"), to_char(pra.last_update_date, g_HRAPI_DATETIME_FMT))
                                      ,XMLElement("LastUpdateDate", to_char(pra.last_update_date, g_HRAPI_DATETIME_FMT)))
                               ,decode(pra.last_updated_by, null
                                      ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"), pra.last_updated_by)
                                      ,XMLElement("LastUpdatedBy", pra.last_updated_by))
                               ,decode(pra.last_update_login, null
                                      ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"), pra.last_update_login)
                                      ,XMLElement("LastUpdateLogin", pra.last_update_login))
                                ,decode(pra.created_by, null
                                      ,XMLElement("CreatedBy", XMLAttributes('true' as "null"), pra.created_by)
                                      ,XMLElement("CreatedBy", pra.created_by))
                               ,decode(pra.creation_date, null
                                      ,XMLElement("CreationDate", XMLAttributes('true' as "null"), to_char(pra.creation_date, g_HRAPI_DATETIME_FMT))
                                      ,XMLElement("CreationDate", to_char(pra.creation_date, g_HRAPI_DATETIME_FMT)))                            
                               ,decode(pra.posting_content_id, null
                                      ,XMLElement("PostingContentId", XMLAttributes('true' as "null"), pra.posting_content_id)
                                      ,XMLElement("PostingContentId", pra.posting_content_id))     
                               ,XMLElement("Status", XMLAttributes('true' as "null"))
                               ,XMLElement("InternalPosting", 'Y')
                               ,XMLElement("ExternalPosting", 'Y')
                               ,decode(pra.object_version_number, null
                                      ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"), pra.object_version_number)
                                      ,XMLElement("ObjectVersionNumber", pra.object_version_number))                                         
                               ,decode(pra.recruiting_site_id, null
                                      ,XMLElement("RecruitingSiteId", XMLAttributes('true' as "null"), pra.recruiting_site_id)
                                      ,XMLElement("RecruitingSiteId", pra.recruiting_site_id))
                               ,XMLElement("RecruitingSiteResponse", XMLAttributes('true' as "null"))
                               ,XMLElement("TypeLookupType", 'REC_TYPE')
                               ,XMLElement("StatusLookupType", 'REC_STATUS')
                               ,XMLElement("LastPostedDate", XMLAttributes('true' as "null"))
                               ,XMLElement("DatesEditable", 'Y')
                               ,XMLElement("DateStartEditable", 'DateStartEditable')
                               ,XMLElement("DateEndEditable", 'DateEndEditable')    
                               )
                             )
                           )
                          FROM TABLE (v.recruitment_activities) pra
                        )
                       )
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                     ('oracle.apps.per.schema.server.PerRequisitionsEO' as "Name")
                   ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(v.requisition_id))
                   ,XMLElement
                     ("PerRequisitionsEORow"
                     ,XMLAttributes
                       (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                       ,'Y' as "PK"
                       ,'Y' as "CI")
                     ,decode(v.requisition_id, null
                            ,XMLElement("RequisitionId", XMLAttributes('true' as "null"), v.requisition_id)
                            ,XMLElement("RequisitionId", v.requisition_id))
                     ,decode(v.business_group_id, null
                            ,XMLElement("BusinessGroupId", XMLAttributes('true' as "null"), v.business_group_id)
                            ,XMLElement("BusinessGroupId", v.business_group_id))                                                      
                     ,XMLElement("PersonId", XMLAttributes('true' as "null"))
                     ,decode(v.date_from, null
                            ,XMLElement("DateFrom", XMLAttributes('true' as "null"))
                            ,XMLElement("DateFrom", to_char(v.date_from, g_HRAPI_DATE_FMT)))
                     ,decode(v.vacancy_name, null
                            ,XMLElement("Name", XMLAttributes('true' as "null"), v.vacancy_name)
                            ,XMLElement("Name", v.vacancy_name)) 
                     ,XMLElement("Comments", XMLAttributes('true' as "null"))
                     ,decode(v.date_to, null
                            ,XMLElement("DateTo", XMLAttributes('true' as "null"))
                            ,XMLElement("DateTo", to_char(v.date_to, g_HRAPI_DATE_FMT)))
                     ,XMLElement("Description", XMLAttributes('true' as "null"))
                     ,XMLElement("AttributeCategory", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute1", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute2", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute3", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute4", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute5", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute6", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute7", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute8", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute9", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute10", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute11", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute12", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute13", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute14", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute15", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute16", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute17", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute18", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute19", XMLAttributes('true' as "null"))
                     ,XMLElement("Attribute20", XMLAttributes('true' as "null"))
                     ,decode(v.last_update_date, null
                            ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"), to_char(v.last_update_date, g_HRAPI_DATETIME_FMT))
                            ,XMLElement("LastUpdateDate", to_char(v.last_update_date, g_HRAPI_DATETIME_FMT)))
                     ,decode(v.last_updated_by, null
                            ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"), v.last_updated_by)
                            ,XMLElement("LastUpdatedBy", v.last_updated_by))
                     ,decode(v.last_update_login, null
                            ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"), v.last_update_login)
                            ,XMLElement("LastUpdateLogin", v.last_update_login))
                      ,decode(v.created_by, null
                            ,XMLElement("CreatedBy", XMLAttributes('true' as "null"), v.created_by)
                            ,XMLElement("CreatedBy", v.created_by))
                     ,decode(v.creation_date, null
                            ,XMLElement("CreationDate", XMLAttributes('true' as "null"), to_char(v.creation_date, g_HRAPI_DATETIME_FMT))
                            ,XMLElement("CreationDate", to_char(v.creation_date, g_HRAPI_DATETIME_FMT)))
                     ,decode(v.req_object_version_number, null
                            ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"), v.req_object_version_number)
                            ,XMLElement("ObjectVersionNumber", v.req_object_version_number))
                     ,XMLElement
                       ("CEO"
                       ,XMLElement
                         ("EO"
                         ,XMLAttributes
                           ('oracle.apps.per.schema.server.PerAllVacanciesEO' as "Name")
                         ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(v.vacancy_id))
                         ,XMLElement
                           ("PerAllVacanciesEORow"
                           ,XMLAttributes
                             (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                             ,'Y' as "PK"
                             ,'Y' as "CI")
                           ,decode(v.vacancy_id, null
                                  ,XMLElement("VacancyId", XMLAttributes('true' as "null"))
                                  ,XMLElement("VacancyId", v.vacancy_id))
                           ,decode(v.business_group_id, null
                                  ,XMLElement("BusinessGroupId", XMLAttributes('true' as "null"))
                                  ,XMLElement("BusinessGroupId", v.business_group_id))
                           ,decode(v.position_id, null
                                  ,XMLElement("PositionId", XMLAttributes('true' as "null"))
                                  ,XMLElement("PositionId", v.position_id))
                           ,decode(v.job_id, null
                                  ,XMLElement("JobId", XMLAttributes('true' as "null"))
                                  ,XMLElement("JobId", v.job_id))
                           ,decode(v.grade_id, null
                                  ,XMLElement("GradeId", XMLAttributes('true' as "null"))
                                  ,XMLElement("GradeId", v.grade_id))
                           ,decode(v.organization_id, null
                                  ,XMLElement("OrganizationId", XMLAttributes('true' as "null"))
                                  ,XMLElement("OrganizationId", v.organization_id))
                           ,decode(v.requisition_id, null
                                  ,XMLElement("RequisitionId", XMLAttributes('true' as "null"))
                                  ,XMLElement("RequisitionId", v.requisition_id))
                           ,decode(v.people_group_id, null
                                  ,XMLElement("PeopleGroupId", XMLAttributes('true' as "null"))
                                  ,XMLElement("PeopleGroupId", v.people_group_id))                                  
                           ,decode(v.location_id, null
                                  ,XMLElement("LocationId", XMLAttributes('true' as "null"))
                                  ,XMLElement("LocationId", v.location_id))                                  
                           ,decode(v.recruiter_id, null
                                  ,XMLElement("RecruiterId", XMLAttributes('true' as "null"))
                                  ,XMLElement("RecruiterId", v.recruiter_id))                                  
                           ,decode(v.date_from, null
                                  ,XMLElement("DateFrom", XMLAttributes('true' as "null"))
                                  ,XMLElement("DateFrom", to_char(v.date_from, g_HRAPI_DATE_FMT)))                                
                           ,decode(v.vacancy_name, null
                                  ,XMLElement("Name", XMLAttributes('true' as "null"))
                                  ,XMLElement("Name", v.vacancy_name))                                  
                           ,decode(v.comments, null
                                  ,XMLElement("Comments", XMLAttributes('true' as "null"))
                                  ,XMLElement("Comments", v.comments))                                  
                           ,decode(v.date_to, null
                                  ,XMLElement("DateTo", XMLAttributes('true' as "null"))
                                  ,XMLElement("DateTo", to_char(v.date_to, g_HRAPI_DATE_FMT)))                                    
                           ,decode(v.description, null
                                  ,XMLElement("Description", XMLAttributes('true' as "null"))
                                  ,XMLElement("Description", v.description))                                      
                           ,decode(v.number_of_openings, null
                                  ,XMLElement("NumberOfOpenings", XMLAttributes('true' as "null"))
                                  ,XMLElement("NumberOfOpenings", v.number_of_openings))  
                           ,decode(v.status_code, null
                                  ,XMLElement("Status", XMLAttributes('true' as "null"))
                                  ,XMLElement("Status", v.status_code))  
                           ,decode(v.budget_type_code, null
                                  ,XMLElement("BudgetMeasurementType", XMLAttributes('true' as "null"))
                                  ,XMLElement("BudgetMeasurementType", v.budget_type_code))
                           ,decode(v.budget_value, null
                                  ,XMLElement("BudgetMeasurementValue", XMLAttributes('true' as "null"))
                                  ,XMLElement("BudgetMeasurementValue", v.budget_value))  
                           ,decode(v.attribute_category, null
                                  ,XMLElement("AttributeCategory", XMLAttributes('true' as "null"))
                                  ,XMLElement("AttributeCategory", v.attribute_category))
                           ,decode(v.attribute1, null
                                  ,XMLElement("Attribute1", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute1", v.attribute1))
                           ,decode(v.attribute2, null
                                  ,XMLElement("Attribute2", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute2", v.attribute2))
                           ,decode(v.attribute3, null
                                  ,XMLElement("Attribute3", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute3", v.attribute3))
                           ,decode(v.attribute4, null
                                  ,XMLElement("Attribute4", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute4", v.attribute4))
                           ,decode(v.attribute5, null
                                  ,XMLElement("Attribute5", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute5", v.attribute5))
                           ,decode(v.attribute6, null
                                  ,XMLElement("Attribute6", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute6", v.attribute6))
                           ,decode(v.attribute7, null
                                  ,XMLElement("Attribute7", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute7", v.attribute7))
                           ,decode(v.attribute8, null
                                  ,XMLElement("Attribute8", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute8", v.attribute8))
                           ,decode(v.attribute9, null
                                  ,XMLElement("Attribute9", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute9", v.attribute9))
                           ,decode(v.attribute10, null
                                  ,XMLElement("Attribute10", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute10", v.attribute10))
                           ,decode(v.attribute11, null
                                  ,XMLElement("Attribute11", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute11", v.attribute11))
                           ,decode(v.attribute12, null
                                  ,XMLElement("Attribute12", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute12", v.attribute12))
                           ,decode(v.attribute13, null
                                  ,XMLElement("Attribute13", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute13", v.attribute13))
                           ,decode(v.attribute14, null
                                  ,XMLElement("Attribute14", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute14", v.attribute14))
                           ,decode(v.attribute15, null
                                  ,XMLElement("Attribute15", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute15", v.attribute15))
                           ,decode(v.attribute16, null
                                  ,XMLElement("Attribute16", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute16", v.attribute16))
                           ,decode(v.attribute17, null
                                  ,XMLElement("Attribute17", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute17", v.attribute17))
                           ,decode(v.attribute18, null
                                  ,XMLElement("Attribute18", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute18", v.attribute18))
                           ,decode(v.attribute19, null
                                  ,XMLElement("Attribute19", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute19", v.attribute19))
                           ,decode(v.attribute20, null
                                  ,XMLElement("Attribute20", XMLAttributes('true' as "null"))
                                  ,XMLElement("Attribute20", v.attribute20))
                           ,decode(v.pg_id_flex_num, null
                                  ,XMLElement("PgIdFlexNum", XMLAttributes('true' as "null"))
                                  ,XMLElement("PgIdFlexNum", v.pg_id_flex_num))
                           ,decode(v.segment1, null
                                  ,XMLElement("Segment1", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment1", v.segment1))
                           ,decode(v.segment2, null
                                  ,XMLElement("Segment2", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment2", v.segment2))
                           ,decode(v.segment3, null
                                  ,XMLElement("Segment3", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment3", v.segment3))
                           ,decode(v.segment4, null
                                  ,XMLElement("Segment4", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment4", v.segment4))
                           ,decode(v.segment5, null
                                  ,XMLElement("Segment5", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment5", v.segment5))
                           ,decode(v.segment6, null
                                  ,XMLElement("Segment6", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment6", v.segment6))
                           ,decode(v.segment7, null
                                  ,XMLElement("Segment7", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment7", v.segment7))
                           ,decode(v.segment8, null
                                  ,XMLElement("Segment8", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment8", v.segment8))
                           ,decode(v.segment9, null
                                  ,XMLElement("Segment9", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment9", v.segment9))
                           ,decode(v.segment10, null
                                  ,XMLElement("Segment10", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment10", v.segment10))
                           ,decode(v.segment11, null
                                  ,XMLElement("Segment11", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment11", v.segment11))
                           ,decode(v.segment12, null
                                  ,XMLElement("Segment12", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment12", v.segment12))
                           ,decode(v.segment13, null
                                  ,XMLElement("Segment13", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment13", v.segment13))
                           ,decode(v.segment14, null
                                  ,XMLElement("Segment14", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment14", v.segment14))
                           ,decode(v.segment15, null
                                  ,XMLElement("Segment15", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment15", v.segment15))
                           ,decode(v.segment16, null
                                  ,XMLElement("Segment16", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment16", v.segment16))
                           ,decode(v.segment17, null
                                  ,XMLElement("Segment17", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment17", v.segment17))
                           ,decode(v.segment18, null
                                  ,XMLElement("Segment18", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment18", v.segment18))
                           ,decode(v.segment19, null
                                  ,XMLElement("Segment19", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment19", v.segment19))
                           ,decode(v.segment20, null
                                  ,XMLElement("Segment20", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment20", v.segment20))
                           ,decode(v.segment21, null
                                  ,XMLElement("Segment21", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment21", v.segment21))
                           ,decode(v.segment22, null
                                  ,XMLElement("Segment22", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment22", v.segment22))
                           ,decode(v.segment23, null
                                  ,XMLElement("Segment23", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment23", v.segment23))
                           ,decode(v.segment24, null
                                  ,XMLElement("Segment24", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment24", v.segment24))
                           ,decode(v.segment25, null
                                  ,XMLElement("Segment25", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment25", v.segment25))
                           ,decode(v.segment26, null
                                  ,XMLElement("Segment26", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment26", v.segment26))
                           ,decode(v.segment27, null
                                  ,XMLElement("Segment27", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment27", v.segment27))
                           ,decode(v.segment28, null
                                  ,XMLElement("Segment28", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment28", v.segment28))
                           ,decode(v.segment29, null
                                  ,XMLElement("Segment29", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment29", v.segment29))
                           ,decode(v.segment30, null
                                  ,XMLElement("Segment30", XMLAttributes('true' as "null"))
                                  ,XMLElement("Segment30", v.segment30))
                           ,decode(v.last_update_date, null
                                  ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"))
                                  ,XMLElement("LastUpdateDate", to_char(v.last_update_date, g_HRAPI_DATETIME_FMT)))
                           ,decode(v.last_updated_by, null
                                  ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"))
                                  ,XMLElement("LastUpdatedBy", v.last_updated_by))
                           ,decode(v.last_update_login, null
                                  ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"))
                                  ,XMLElement("LastUpdateLogin", v.last_update_login))
                           ,decode(v.created_by, null
                                  ,XMLElement("CreatedBy", XMLAttributes('true' as "null"))
                                  ,XMLElement("CreatedBy", v.created_by))
                           ,decode(v.creation_date, null
                                  ,XMLElement("CreationDate", XMLAttributes('true' as "null"))
                                  ,XMLElement("CreationDate", to_char(v.creation_date, g_HRAPI_DATETIME_FMT)))
                           ,decode(v.vacancy_category_code, null
                                  ,XMLElement("VacancyCategory", XMLAttributes('true' as "null"))
                                  ,XMLElement("VacancyCategory", v.vacancy_category_code))
                           ,decode(v.manager_id, null
                                  ,XMLElement("ManagerId", XMLAttributes('true' as "null"))
                                  ,XMLElement("ManagerId", v.manager_id))
                           ,decode(v.security_method, null
                                  ,XMLElement("SecurityMethod", XMLAttributes('true' as "null"))
                                  ,XMLElement("SecurityMethod", v.security_method))
                           ,decode(v.object_version_number, null
                                  ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"))
                                  ,XMLElement("ObjectVersionNumber", v.object_version_number))
                           ,XMLElement("ManagerDefaultLocationCode", XMLAttributes('true' as "null"))
                           ,XMLElement("ManagerDefaultLocationId", XMLAttributes('true' as "null"))
                           ,XMLElement("PositionDefaultLocationCode", XMLAttributes('true' as "null"))
                           ,XMLElement("PositionDefaultLocationId", XMLAttributes('true' as "null"))
                           ,XMLElement("StatusLookupType", 'VACANCY_STATUS')
                           ,XMLElement("BudgetMeasurementTypeLookupType", 'BUDGET_MEASUREMENT_TYPE')
                           ,XMLElement("SecurityMethodLookupType", 'IRC_SECURITY_METHOD')
                           ,XMLElement("CompetenceType", 'VACANCY')
                           ,XMLElement("VacancyCategoryType", 'VACANCY_CATEGORY')
                           ,XMLElement("nextPressed", 'Y')
                           ,decode(v.primary_posting_id, null
                                  ,XMLElement("PrimaryPostingId", XMLAttributes('true' as "null"))
                                  ,XMLElement("PrimaryPostingId", v.primary_posting_id))
                           ,decode(v.assessment_id, null
                                  ,XMLElement("AssessmentId", XMLAttributes('true' as "null"))
                                  ,XMLElement("AssessmentId", v.assessment_id))
                           ,XMLElement("CommentsToApprovers", XMLAttributes('true' as "null"))
                           ,XMLElement
                             ("CEO"
                             ,XMLElement
                               ("EO"
                               ,XMLAttributes
                                 ('oracle.apps.irc.schema.server.IrcVacancySearchCriteriaEO' as "Name")
                               ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(v.search_criteria_id))
                               ,XMLElement
                                 ("IrcVacancySearchCriteriaEORow"
                                 ,XMLAttributes
                                   (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                                   ,'Y' as "PK"
                                   ,'Y' as "CI")
                                 ,decode(v.search_criteria_id, null
                                        ,XMLElement("SearchCriteriaId", XMLAttributes('true' as "null"))
                                        ,XMLElement("SearchCriteriaId", v.search_criteria_id))
                                 ,decode(v.vacancy_id, null
                                        ,XMLElement("ObjectId", XMLAttributes('true' as "null"))
                                        ,XMLElement("ObjectId", v.vacancy_id))
                                 ,XMLElement("ObjectType", 'VACANCY')
                                 ,XMLElement("Location", XMLAttributes('true' as "null"))
                                 ,decode(v.employee_flag, null
                                        ,XMLElement("Employee", XMLAttributes('true' as "null"))
                                        ,XMLElement("Employee", v.employee_flag))                                   
                                 ,decode(v.contractor_flag, null
                                        ,XMLElement("Contractor", XMLAttributes('true' as "null"))
                                        ,XMLElement("Contractor", v.contractor_flag))                                   
                                 ,decode(v.employment_category_code, null
                                        ,XMLElement("EmploymentCategory", XMLAttributes('true' as "null"))
                                        ,XMLElement("EmploymentCategory", v.employment_category_code))
                                 ,XMLElement("Keywords", XMLAttributes('true' as "null"))
                                 ,decode(v.travel_percentage, null
                                        ,XMLElement("TravelPercentage", XMLAttributes('true' as "null"))
                                        ,XMLElement("TravelPercentage", v.travel_percentage)) 
                                 ,decode(v.min_salary, null
                                        ,XMLElement("MinSalary", XMLAttributes('true' as "null"))
                                        ,XMLElement("MinSalary", v.min_salary)) 
                                 ,decode(v.max_salary, null
                                        ,XMLElement("MaxSalary", XMLAttributes('true' as "null"))
                                        ,XMLElement("MaxSalary", v.max_salary))
                                 ,decode(v.salary_currency, null
                                        ,XMLElement("SalaryCurrency", XMLAttributes('true' as "null"))
                                        ,XMLElement("SalaryCurrency", v.salary_currency))
                                 ,decode(v.professional_area_code, null
                                        ,XMLElement("ProfessionalArea", XMLAttributes('true' as "null"))
                                        ,XMLElement("ProfessionalArea", v.professional_area_code))
                                 ,decode(v.work_at_home_code, null
                                        ,XMLElement("WorkAtHome", XMLAttributes('true' as "null"))
                                        ,XMLElement("WorkAtHome", v.work_at_home_code))
                                 ,decode(v.min_qual_level_id, null
                                        ,XMLElement("MinQualLevel", XMLAttributes('true' as "null"))
                                        ,XMLElement("MinQualLevel", v.min_qual_level_id)) 
                                 ,decode(v.max_qual_level_id, null
                                        ,XMLElement("MaxQualLevel", XMLAttributes('true' as "null"))
                                        ,XMLElement("MaxQualLevel", v.max_qual_level_id))                                         
                                 ,decode(v.isc_attribute_category, null
                                        ,XMLElement("AttributeCategory", XMLAttributes('true' as "null"))
                                        ,XMLElement("AttributeCategory", v.isc_attribute_category))
                                 ,decode(v.isc_attribute1, null
                                        ,XMLElement("Attribute1", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute1", v.isc_attribute1))
                                 ,decode(v.isc_attribute2, null
                                        ,XMLElement("Attribute2", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute2", v.isc_attribute2))
                                 ,decode(v.isc_attribute3, null
                                        ,XMLElement("Attribute3", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute3", v.isc_attribute3))
                                 ,decode(v.isc_attribute4, null
                                        ,XMLElement("Attribute4", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute4", v.isc_attribute4))
                                 ,decode(v.isc_attribute5, null
                                        ,XMLElement("Attribute5", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute5", v.isc_attribute5))
                                 ,decode(v.isc_attribute6, null
                                        ,XMLElement("Attribute6", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute6", v.isc_attribute6))
                                 ,decode(v.isc_attribute7, null
                                        ,XMLElement("Attribute7", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute7", v.isc_attribute7))
                                 ,decode(v.isc_attribute8, null
                                        ,XMLElement("Attribute8", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute8", v.isc_attribute8))
                                 ,decode(v.isc_attribute9, null
                                        ,XMLElement("Attribute9", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute9", v.isc_attribute9))
                                 ,decode(v.isc_attribute10, null
                                        ,XMLElement("Attribute10", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute10", v.isc_attribute10))
                                 ,decode(v.isc_attribute11, null
                                        ,XMLElement("Attribute11", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute11", v.isc_attribute11))
                                 ,decode(v.isc_attribute12, null
                                        ,XMLElement("Attribute12", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute12", v.isc_attribute12))
                                 ,decode(v.isc_attribute13, null
                                        ,XMLElement("Attribute13", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute13", v.isc_attribute13))
                                 ,decode(v.isc_attribute14, null
                                        ,XMLElement("Attribute14", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute14", v.isc_attribute14))
                                 ,decode(v.isc_attribute15, null
                                        ,XMLElement("Attribute15", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute15", v.isc_attribute15))
                                 ,decode(v.isc_attribute16, null
                                        ,XMLElement("Attribute16", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute16", v.isc_attribute16))
                                 ,decode(v.isc_attribute17, null
                                        ,XMLElement("Attribute17", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute17", v.isc_attribute17))
                                 ,decode(v.isc_attribute18, null
                                        ,XMLElement("Attribute18", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute18", v.isc_attribute18))
                                 ,decode(v.isc_attribute19, null
                                        ,XMLElement("Attribute19", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute19", v.isc_attribute19))
                                 ,decode(v.isc_attribute20, null
                                        ,XMLElement("Attribute20", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute20", v.isc_attribute20))
                                 ,decode(v.isc_attribute21, null
                                        ,XMLElement("Attribute21", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute21", v.isc_attribute21))
                                 ,decode(v.isc_attribute22, null
                                        ,XMLElement("Attribute22", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute22", v.isc_attribute22))
                                 ,decode(v.isc_attribute23, null
                                        ,XMLElement("Attribute23", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute23", v.isc_attribute23))
                                 ,decode(v.isc_attribute24, null
                                        ,XMLElement("Attribute24", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute24", v.isc_attribute24))
                                 ,decode(v.isc_attribute25, null
                                        ,XMLElement("Attribute25", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute25", v.isc_attribute25))
                                 ,decode(v.isc_attribute26, null
                                        ,XMLElement("Attribute26", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute26", v.isc_attribute26))
                                 ,decode(v.isc_attribute27, null
                                        ,XMLElement("Attribute27", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute27", v.isc_attribute27))
                                 ,decode(v.isc_attribute28, null
                                        ,XMLElement("Attribute28", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute28", v.isc_attribute28))
                                 ,decode(v.isc_attribute29, null
                                        ,XMLElement("Attribute29", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute29", v.isc_attribute29))
                                 ,decode(v.isc_attribute30, null
                                        ,XMLElement("Attribute30", XMLAttributes('true' as "null"))
                                        ,XMLElement("Attribute30", v.isc_attribute30))
                                 ,XMLElement("IscInformationCategory", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation1", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation2", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation3", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation4", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation5", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation6", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation7", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation8", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation9", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation10", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation11", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation12", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation13", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation14", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation15", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation16", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation17", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation18", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation19", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation20", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation21", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation22", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation23", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation24", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation25", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation26", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation27", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation28", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation29", XMLAttributes('true' as "null"))
                                 ,XMLElement("IscInformation30", XMLAttributes('true' as "null"))
                                 ,decode(v.last_update_date, null
                                        ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"))
                                        ,XMLElement("LastUpdateDate", to_char(v.last_update_date, g_HRAPI_DATETIME_FMT)))
                                 ,decode(v.last_updated_by, null
                                        ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"))
                                        ,XMLElement("LastUpdatedBy", v.last_updated_by))
                                 ,decode(v.last_update_login, null
                                        ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"))
                                        ,XMLElement("LastUpdateLogin", v.last_update_login))
                                 ,decode(v.created_by, null
                                        ,XMLElement("CreatedBy", XMLAttributes('true' as "null"))
                                        ,XMLElement("CreatedBy", v.created_by))
                                 ,decode(v.creation_date, null
                                        ,XMLElement("CreationDate", XMLAttributes('true' as "null"))
                                        ,XMLElement("CreationDate", to_char(v.creation_date, g_HRAPI_DATETIME_FMT)))
                                 ,decode(v.isc_object_version_number, null
                                        ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"))
                                        ,XMLElement("ObjectVersionNumber", v.isc_object_version_number))
                                 ,decode(v.salary_period_code, null
                                        ,XMLElement("SalaryPeriod", XMLAttributes('true' as "null"))
                                        ,XMLElement("SalaryPeriod", v.salary_period_code))
                                 ,XMLElement("QualificationTypeCategory", 'REC')
                                 ,XMLElement("ProfAreaLookup", 'IRC_PROFESSIONAL_AREA')
                                 ,XMLElement("EmpCatLookup", 'IRC_EMP_CAT')
                                 ,XMLElement("WorkLookup", 'IRC_WORK_AT_HOME')
                                 ,XMLElement("TravelLookup", 'IRC_TRAVEL_PERCENTAGE')
                                 ,XMLElement("SalaryLookup", 'PAY_BASIS')
                                 ,XMLElement("ConLookup", 'YES_NO')
                                 ,XMLElement("EmpLookup", 'YES_NO')
                                 )
                               )                              
                            ,(SELECT
                               XMLAgg
                                (XMLElement
                                  ("EO"
                                  ,XMLAttributes
                                    ('oracle.apps.per.schema.server.PerRecruitmentActivityForEO' as "Name")
                                  ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(praf.recruitment_activity_for_id))
                                  ,XMLElement
                                    ("PerRecruitmentActivityForEORow"
                                    ,XMLAttributes
                                      (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                                      ,'Y' as "PK"
                                      ,'Y' as "CI")
                                    ,decode(praf.recruitment_activity_for_id, null
                                           ,XMLElement("RecruitmentActivityForId", XMLAttributes('true' as "null"))
                                           ,XMLElement("RecruitmentActivityForId", praf.recruitment_activity_for_id))
                                    ,decode(v.business_group_id, null
                                           ,XMLElement("BusinessGroupId", XMLAttributes('true' as "null"))
                                           ,XMLElement("BusinessGroupId", v.business_group_id))
                                    ,decode(v.vacancy_id, null
                                           ,XMLElement("VacancyId", XMLAttributes('true' as "null"))
                                           ,XMLElement("VacancyId", v.vacancy_id))
                                    ,decode(praf.recruitment_activity_id, null
                                           ,XMLElement("RecruitmentActivityId", XMLAttributes('true' as "null"))
                                           ,XMLElement("RecruitmentActivityId", praf.recruitment_activity_id))
                                    ,decode(praf.last_update_date, null
                                           ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateDate", to_char(praf.last_update_date, g_HRAPI_DATETIME_FMT)))
                                    ,decode(praf.last_updated_by, null
                                           ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdatedBy", praf.last_updated_by))
                                    ,decode(praf.last_update_login, null
                                           ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateLogin", praf.last_update_login))
                                     ,decode(praf.created_by, null
                                           ,XMLElement("CreatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreatedBy", praf.created_by))
                                    ,decode(praf.creation_date, null
                                           ,XMLElement("CreationDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreationDate", to_char(praf.creation_date, g_HRAPI_DATETIME_FMT)))                            
                                    ,decode(praf.for_object_version_number, null
                                           ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"))
                                           ,XMLElement("ObjectVersionNumber", praf.for_object_version_number))   
                                    )
                                  )
                                )
                               FROM TABLE (v.recruitment_activities) praf
                             )
                            ,(SELECT
                               XMLAgg
                                (XMLElement
                                  ("EO"
                                  ,XMLAttributes
                                    ('oracle.apps.per.schema.server.VacancyCompetenceElementsEO' as "Name")
                                  ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(pce.competence_element_id))
                                  ,XMLElement
                                    ("VacancyCompetenceElementsEORow"
                                    ,XMLAttributes
                                      (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                                      ,'Y' as "PK"
                                      ,'Y' as "CI")
                                    ,decode(pce.competence_element_id, null
                                           ,XMLElement("CompetenceElementId", XMLAttributes('true' as "null"))
                                           ,XMLElement("CompetenceElementId", pce.competence_element_id))                                      
                                    ,decode(v.business_group_id, null
                                           ,XMLElement("BusinessGroupId", XMLAttributes('true' as "null"))
                                           ,XMLElement("BusinessGroupId", v.business_group_id))                                          
                                    ,decode(pce.object_version_number, null
                                           ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"))
                                           ,XMLElement("ObjectVersionNumber", pce.object_version_number))
                                    ,decode(pce.type_code, null
                                           ,XMLElement("Type", XMLAttributes('true' as "null"))
                                           ,XMLElement("Type", pce.type_code))
                                    ,decode(pce.competence_id, null
                                           ,XMLElement("CompetenceId", XMLAttributes('true' as "null"))
                                           ,XMLElement("CompetenceId", pce.competence_id))
                                    ,decode(pce.proficiency_level_id, null
                                           ,XMLElement("ProficiencyLevelId", XMLAttributes('true' as "null"))
                                           ,XMLElement("ProficiencyLevelId", pce.proficiency_level_id))
                                    ,decode(pce.high_proficiency_level_id, null
                                           ,XMLElement("HighProficiencyLevelId", XMLAttributes('true' as "null"))
                                           ,XMLElement("HighProficiencyLevelId", pce.high_proficiency_level_id))
                                    ,decode(pce.mandatory_code, null
                                           ,XMLElement("Mandatory", XMLAttributes('true' as "null"))
                                           ,XMLElement("Mandatory", pce.mandatory_code))
                                    ,decode(pce.attribute_category, null
                                           ,XMLElement("AttributeCategory", XMLAttributes('true' as "null"))
                                           ,XMLElement("AttributeCategory", pce.attribute_category))
                                    ,decode(pce.attribute1, null
                                           ,XMLElement("Attribute1", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute1", pce.attribute1))
                                    ,decode(pce.attribute2, null
                                           ,XMLElement("Attribute2", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute2", pce.attribute2))
                                    ,decode(pce.attribute3, null
                                           ,XMLElement("Attribute3", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute3", pce.attribute3))
                                    ,decode(pce.attribute4, null
                                           ,XMLElement("Attribute4", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute4", pce.attribute4))
                                    ,decode(pce.attribute5, null
                                           ,XMLElement("Attribute5", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute5", pce.attribute5))
                                    ,decode(pce.attribute6, null
                                           ,XMLElement("Attribute6", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute6", pce.attribute6))
                                    ,decode(pce.attribute7, null
                                           ,XMLElement("Attribute7", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute7", pce.attribute7))
                                    ,decode(pce.attribute8, null
                                           ,XMLElement("Attribute8", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute8", pce.attribute8))
                                    ,decode(pce.attribute9, null
                                           ,XMLElement("Attribute9", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute9", pce.attribute9))
                                    ,decode(pce.attribute10, null
                                           ,XMLElement("Attribute10", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute10", pce.attribute10))
                                    ,decode(pce.attribute11, null
                                           ,XMLElement("Attribute11", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute11", pce.attribute11))
                                    ,decode(pce.attribute12, null
                                           ,XMLElement("Attribute12", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute12", pce.attribute12))
                                    ,decode(pce.attribute13, null
                                           ,XMLElement("Attribute13", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute13", pce.attribute13))
                                    ,decode(pce.attribute14, null
                                           ,XMLElement("Attribute14", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute14", pce.attribute14))
                                    ,decode(pce.attribute15, null
                                           ,XMLElement("Attribute15", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute15", pce.attribute15))
                                    ,decode(pce.attribute16, null
                                           ,XMLElement("Attribute16", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute16", pce.attribute16))
                                    ,decode(pce.attribute17, null
                                           ,XMLElement("Attribute17", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute17", pce.attribute17))
                                    ,decode(pce.attribute18, null
                                           ,XMLElement("Attribute18", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute18", pce.attribute18))
                                    ,decode(pce.attribute19, null
                                           ,XMLElement("Attribute19", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute19", pce.attribute19))
                                    ,decode(pce.attribute20, null
                                           ,XMLElement("Attribute20", XMLAttributes('true' as "null"))
                                           ,XMLElement("Attribute20", pce.attribute20))
                                    ,decode(pce.last_update_date, null
                                           ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateDate", to_char(pce.last_update_date, g_HRAPI_DATETIME_FMT)))
                                    ,decode(pce.last_updated_by, null
                                           ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdatedBy", pce.last_updated_by))
                                    ,decode(pce.last_update_login, null
                                           ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateLogin", pce.last_update_login))
                                     ,decode(pce.created_by, null
                                           ,XMLElement("CreatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreatedBy", pce.created_by))
                                    ,decode(pce.creation_date, null
                                           ,XMLElement("CreationDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreationDate", to_char(pce.creation_date, g_HRAPI_DATETIME_FMT)))                            
                                    ,decode(v.vacancy_id, null
                                           ,XMLElement("ObjectId", XMLAttributes('true' as "null"))
                                           ,XMLElement("ObjectId", v.vacancy_id))
                                    ,XMLElement("ObjectName", 'VACANCY')
                                    ,XMLElement("MandatoryLookupType", 'IRC_SKILL_DESIRABILITY')
                                    ,XMLElement("CertMethodLookupType", 'CERTIFICATION_METHOD')
                                    ,XMLElement("CompTypeLookup", 'COMPETENCE_TYPE')
                                    ,XMLElement("ProfSourceLookupType", 'PROFICIENCY_SOURCE')
                                    ,XMLElement("TypeLookup", 'COMPETENCE_ELEMENT_TYPE')
                                    ,XMLElement("DurationUnitLookupType", 'ELAPSE_DURATION')
                                    ,XMLElement("DatetrackUpdate", 'CORRECTION')
                                    )
                                  )
                                )
                               FROM TABLE (v.competence_elements) pce
                             )
                            /*,(SELECT
                               XMLAgg
                                (XMLElement
                                  ("EO"
                                  ,XMLAttributes
                                    ('oracle.apps.irc.schema.server.IrcRecTeamMembersEO' as "Name")

                                  ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(null))
                                  ,XMLElement
                                    ("IrcRecTeamMembersEORow"
                                    ,XMLAttributes
                                      (0 as "PS", 'Y' as "PK", 'Y' as "CI")
                                    ,XMLElement("RecTeamMemberId", XMLAttributes('true' as "null"))
                                    ,XMLElement("PersonId", itm.person_id)
                                    ,XMLElement("PartyId", XMLAttributes('true' as "null"))
                                    ,XMLElement("VacancyId", v.vacancy_id)
                                    ,decode(itm.start_date, null
                                           ,XMLElement("StartDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("StartDate", to_char(itm.start_date, g_HRAPI_DATE_FMT)))
                                    ,decode(itm.end_date, null
                                           ,XMLElement("EndDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("EndDate", to_char(itm.end_date, g_HRAPI_DATE_FMT)))
                                    ,decode(itm.update_allowed, null
                                           ,XMLElement("UpdateAllowed", 'N')
                                           ,XMLElement("UpdateAllowed", itm.update_allowed))
                                    ,decode(itm.delete_allowed, null
                                           ,XMLElement("DeleteAllowed", 'N')
                                           ,XMLElement("DeleteAllowed", itm.delete_allowed))
                                   ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"))   
                                   ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"))   
                                   ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"))   
                                   ,XMLElement("CreatedBy", XMLAttributes('true' as "null"))   
                                   ,XMLElement("CreationDate", XMLAttributes('true' as "null"))  
                                   ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"))                                          
                                    ,XMLElement
                                      ("Security"
                                      ,CASE WHEN NVL(itm.update_allowed,'N') = 'Y' AND NVL(itm.delete_allowed,'N') = 'Y' THEN
                                                 'UD'
                                            WHEN NVL(itm.update_allowed,'N') = 'Y' AND NVL(itm.delete_allowed,'N') = 'N' THEN
                                                 'U'
                                            ELSE
                                                 'V'
                                       END)
                                    ,XMLElement("UpdateAllowedLookup", 'YES_NO')
                                    ,XMLElement("DeleteAllowedLookup", 'YES_NO')
                                    ,XMLElement
                                      ("IRTMEffectiveStartDate"
                                      ,(SELECT to_char(papf1.effective_start_date, g_HRAPI_DATE_FMT)
                                        FROM   per_all_people_f papf1
                                        WHERE  trunc(sysdate) BETWEEN
                                               papf1.effective_start_date AND papf1.effective_end_date
                                        AND    nvl(itm.person_id, papf1.person_id) = papf1.person_id
                                        AND    nvl(itm.party_id, papf1.party_id) = papf1.party_id
                                        AND   (itm.person_id IS NOT NULL OR itm.party_id IS NOT NULL))
                                      )
                                    ,XMLElement
                                      ("IRTMEffectiveEndDate"
                                      ,(SELECT to_char(papf2.effective_end_date, g_HRAPI_DATE_FMT)
                                        FROM   per_all_people_f papf2
                                        WHERE  trunc(sysdate) BETWEEN
                                               papf2.effective_start_date AND papf2.effective_end_date
                                        AND    nvl(itm.person_id, papf2.person_id) = papf2.person_id
                                        AND    nvl(itm.party_id, papf2.party_id) = papf2.party_id
                                        AND   (itm.person_id IS NOT NULL OR itm.party_id IS NOT NULL))
                                      )
                                    ,XMLElement("SecurityLookupType", 'IRC_RECRUITMENT_TEAM_SECURITY')
                                    )
                                  )
                                )
                                 FROM TABLE(v.team_members) itm, irc_rec_team_members irtm
                                        WHERE itm.vacancy_id = irtm.vacancy_id
                                        AND itm.person_id <> irtm.person_id
                                        AND itm.rec_team_member_id = irtm.rec_team_member_id
                                             )*/
                             ,(SELECT
                               XMLAgg
                                (XMLElement
                                  ("EO"
                                  ,XMLAttributes
                                    ('oracle.apps.irc.schema.server.IrcRecTeamMembersEO' as "Name")
                                  ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(itm.REC_TEAM_MEMBER_ID))
                                  ,XMLElement
                                    ("IrcRecTeamMembersEORow"
                                   /* ,XMLAttributes
                                      (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                                      ,'Y' as "PK"
                                      ,'Y' as "CI")*/
                                    ,XMLAttributes
                                      (3 as "PS")
                                    ,XMLElement("RecTeamMemberId",itm.REC_TEAM_MEMBER_ID )
                                    
                                    
                                    )
                                  )
                                )FROM (  SELECT *
                                             FROM irc_rec_team_members
                                             WHERE vacancy_id = p_vacancy.vacancy_id
                                             AND (rec_team_member_id,person_id) NOT IN (
                                                   SELECT itms.rec_team_member_id,itms.person_id
                                                   FROM TABLE (p_vacancy.team_members) itms )
                                                   )itm)
                            ,(SELECT
                               XMLAgg
                                (XMLElement
                                  ("EO"
                                  ,XMLAttributes
                                    ('oracle.apps.irc.schema.server.IrcRecTeamMembersEO' as "Name")
                                  ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(itm.rec_team_member_id))
                                  ,XMLElement
                                    ("IrcRecTeamMembersEORow"
                                   /* ,XMLAttributes
                                      (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                                      ,'Y' as "PK"
                                      ,'Y' as "CI")*/
                                    ,XMLAttributes
                                      (rec_mem_exists(itm.rec_team_member_id) as "PS"
                                      ,'Y' as "PK"
                                      ,'Y' as "CI")
                                    ,decode(itm.rec_team_member_id, null
                                           ,XMLElement("RecTeamMemberId", XMLAttributes('true' as "null"))
                                           ,XMLElement("RecTeamMemberId", itm.rec_team_member_id))
                                    ,decode(itm.person_id, null
                                           ,XMLElement("PersonId", XMLAttributes('true' as "null"))
                                           ,XMLElement("PersonId", itm.person_id))
                                    ,decode(itm.party_id, null
                                           ,XMLElement("PartyId", XMLAttributes('true' as "null"))
                                           ,XMLElement("PartyId", itm.party_id))
                                    ,decode(v.vacancy_id, null
                                           ,XMLElement("VacancyId", XMLAttributes('true' as "null"))
                                           ,XMLElement("VacancyId", v.vacancy_id))
                                    ,decode(itm.start_date, null
                                           ,XMLElement("StartDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("StartDate", to_char(itm.start_date, g_HRAPI_DATE_FMT)))
                                    ,decode(itm.end_date, null
                                           ,XMLElement("EndDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("EndDate", to_char(itm.end_date, g_HRAPI_DATE_FMT)))
                                    ,decode(itm.update_allowed, null
                                           ,XMLElement("UpdateAllowed", 'N')
                                           ,XMLElement("UpdateAllowed", itm.update_allowed))
                                    ,decode(itm.delete_allowed, null
                                           ,XMLElement("DeleteAllowed", 'N')
                                           ,XMLElement("DeleteAllowed", itm.delete_allowed))
                                    ,decode(itm.last_update_date, null
                                           ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateDate", to_char(itm.last_update_date, g_HRAPI_DATETIME_FMT)))
                                    ,decode(itm.last_updated_by, null
                                           ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdatedBy", itm.last_updated_by))
                                    ,decode(itm.last_update_login, null
                                           ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateLogin", itm.last_update_login))
                                    ,decode(itm.created_by, null
                                           ,XMLElement("CreatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreatedBy", itm.created_by))
                                    ,decode(itm.creation_date, null
                                           ,XMLElement("CreationDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreationDate", to_char(itm.creation_date, g_HRAPI_DATETIME_FMT)))
                                    ,decode(itm.object_version_number, null
                                           ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"))
                                           ,XMLElement("ObjectVersionNumber", itm.object_version_number))
                                    ,XMLElement
                                      ("Security"
                                      ,CASE WHEN NVL(itm.update_allowed,'N') = 'Y' AND NVL(itm.delete_allowed,'N') = 'Y' THEN
                                                 'UD'
                                            WHEN NVL(itm.update_allowed,'N') = 'Y' AND NVL(itm.delete_allowed,'N') = 'N' THEN
                                                 'U'
                                            ELSE
                                                 'V'
                                       END)
                                    ,XMLElement("UpdateAllowedLookup", 'YES_NO')
                                    ,XMLElement("DeleteAllowedLookup", 'YES_NO')
                                    ,XMLElement
                                      ("IRTMEffectiveStartDate"
                                      ,(SELECT to_char(papf1.effective_start_date, g_HRAPI_DATE_FMT)
                                        FROM   per_all_people_f papf1
                                        WHERE  trunc(sysdate) BETWEEN
                                               papf1.effective_start_date AND papf1.effective_end_date
                                        AND    nvl(itm.person_id, papf1.person_id) = papf1.person_id
                                        AND    nvl(itm.party_id, papf1.party_id) = papf1.party_id
                                        AND   (itm.person_id IS NOT NULL OR itm.party_id IS NOT NULL))
                                      )
                                    ,XMLElement
                                      ("IRTMEffectiveEndDate"
                                      ,(SELECT to_char(papf2.effective_end_date, g_HRAPI_DATE_FMT)
                                        FROM   per_all_people_f papf2
                                        WHERE  trunc(sysdate) BETWEEN
                                               papf2.effective_start_date AND papf2.effective_end_date
                                        AND    nvl(itm.person_id, papf2.person_id) = papf2.person_id
                                        AND    nvl(itm.party_id, papf2.party_id) = papf2.party_id
                                        AND   (itm.person_id IS NOT NULL OR itm.party_id IS NOT NULL))
                                      )
                                    ,XMLElement("SecurityLookupType", 'IRC_RECRUITMENT_TEAM_SECURITY')
                                    )
                                  )
                                )
                               FROM   TABLE (v.team_members) itm 

/* TABLE (v.team_members) itm
where itm.rec_team_member_id is null
OR 
EXISTS (select 1 from irc_rec_team_members irtm where itm.vacancy_id = irtm.vacancy_id
and itm.rec_team_member_id = irtm.rec_team_member_id
and itm.person_id = irtm.person_id) */

                             )
                            ,(SELECT
                               XMLAgg
                                (XMLElement
                                  ("EO"
                                  ,XMLAttributes
                                    ('oracle.apps.irc.schema.server.IrcAgencyVacanciesEO' as "Name")
                                  ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(iav.agency_vacancy_id))
                                  ,XMLElement
                                    ("IrcAgencyVacanciesEORow"
                                    ,XMLAttributes
                                      (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                                      ,'Y' as "PK"
                                      ,'Y' as "CI")
                                    ,decode(iav.agency_vacancy_id, null
                                           ,XMLElement("AgencyVacancyId", XMLAttributes('true' as "null"))
                                           ,XMLElement("AgencyVacancyId", iav.agency_vacancy_id))
                                    ,decode(iav.agency_id, null
                                           ,XMLElement("AgencyId", XMLAttributes('true' as "null"))
                                           ,XMLElement("AgencyId", iav.agency_id))
                                    ,decode(iav.vacancy_id, null
                                           ,XMLElement("VacancyId", XMLAttributes('true' as "null"))
                                           ,XMLElement("VacancyId", iav.vacancy_id))
                                    ,decode(iav.start_date, null
                                           ,XMLElement("StartDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("StartDate", to_char(iav.start_date, g_HRAPI_DATE_FMT)))
                                    ,decode(iav.end_date, null
                                           ,XMLElement("EndDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("EndDate", to_char(iav.end_date, g_HRAPI_DATE_FMT)))
                                    ,decode(iav.max_allowed_applicants, null
                                           ,XMLElement("MaxAllowedApplicants", XMLAttributes('true' as "null"))
                                           ,XMLElement("MaxAllowedApplicants", iav.max_allowed_applicants))
                                    ,decode(iav.manage_applicants_allowed, null
                                           ,XMLElement("ManageApplicantsAllowed", XMLAttributes('true' as "null"))
                                           ,XMLElement("ManageApplicantsAllowed", xxas_util_pkg.boolean_to_flag(iav.manage_applicants_allowed)))
                                    ,decode(iav.attribute_category, null
                                           ,XMLElement("AttributeCategory", XMLAttributes('true' as "null"))
                                           ,XMLElement("AttributeCategory", iav.attribute_category))
                                    ,decode(iav.attribute1, null
                                          ,XMLElement("Attribute1", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute1", iav.attribute1))
                                    ,decode(iav.attribute2, null
                                          ,XMLElement("Attribute2", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute2", iav.attribute2))
                                    ,decode(iav.attribute3, null
                                          ,XMLElement("Attribute3", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute3", iav.attribute3))
                                    ,decode(iav.attribute4, null
                                          ,XMLElement("Attribute4", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute4", iav.attribute4))
                                    ,decode(iav.attribute5, null
                                          ,XMLElement("Attribute5", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute5", iav.attribute5))
                                    ,decode(iav.attribute6, null
                                          ,XMLElement("Attribute6", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute6", iav.attribute6))
                                    ,decode(iav.attribute7, null
                                          ,XMLElement("Attribute7", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute7", iav.attribute7))
                                    ,decode(iav.attribute8, null
                                          ,XMLElement("Attribute8", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute8", iav.attribute8))
                                    ,decode(iav.attribute9, null
                                          ,XMLElement("Attribute9", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute9", iav.attribute9))
                                    ,decode(iav.attribute10, null
                                          ,XMLElement("Attribute10", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute10", iav.attribute10))
                                    ,decode(iav.attribute11, null
                                          ,XMLElement("Attribute11", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute11", iav.attribute11))
                                    ,decode(iav.attribute12, null
                                          ,XMLElement("Attribute12", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute12", iav.attribute12))
                                    ,decode(iav.attribute13, null
                                          ,XMLElement("Attribute13", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute13", iav.attribute13))
                                    ,decode(iav.attribute14, null
                                          ,XMLElement("Attribute14", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute14", iav.attribute14))
                                    ,decode(iav.attribute15, null
                                          ,XMLElement("Attribute15", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute15", iav.attribute15))
                                    ,decode(iav.attribute16, null
                                          ,XMLElement("Attribute16", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute16", iav.attribute16))
                                    ,decode(iav.attribute17, null
                                          ,XMLElement("Attribute17", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute17", iav.attribute17))
                                    ,decode(iav.attribute18, null
                                          ,XMLElement("Attribute18", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute18", iav.attribute18))
                                    ,decode(iav.attribute19, null
                                          ,XMLElement("Attribute19", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute19", iav.attribute19))
                                    ,decode(iav.attribute20, null
                                          ,XMLElement("Attribute20", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute20", iav.attribute20))
                                    ,decode(iav.attribute21, null
                                          ,XMLElement("Attribute21", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute21", iav.attribute21))
                                    ,decode(iav.attribute22, null
                                          ,XMLElement("Attribute22", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute22", iav.attribute22))
                                    ,decode(iav.attribute23, null
                                          ,XMLElement("Attribute23", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute23", iav.attribute23))
                                    ,decode(iav.attribute24, null
                                          ,XMLElement("Attribute24", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute24", iav.attribute24))
                                    ,decode(iav.attribute25, null
                                          ,XMLElement("Attribute25", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute25", iav.attribute25))
                                    ,decode(iav.attribute26, null
                                          ,XMLElement("Attribute26", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute26", iav.attribute26))
                                    ,decode(iav.attribute27, null
                                          ,XMLElement("Attribute27", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute27", iav.attribute27))
                                    ,decode(iav.attribute28, null
                                          ,XMLElement("Attribute28", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute28", iav.attribute28))
                                    ,decode(iav.attribute29, null
                                          ,XMLElement("Attribute29", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute29", iav.attribute29))
                                    ,decode(iav.attribute30, null
                                          ,XMLElement("Attribute30", XMLAttributes('true' as "null"))
                                          ,XMLElement("Attribute30", iav.attribute30))
                                    ,decode(iav.object_version_number, null
                                           ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"))
                                           ,XMLElement("ObjectVersionNumber", iav.object_version_number))
                                    ,decode(iav.last_update_date, null
                                           ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateDate", to_char(iav.last_update_date, g_HRAPI_DATETIME_FMT)))
                                    ,decode(iav.last_updated_by, null
                                           ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdatedBy", iav.last_updated_by))
                                    ,decode(iav.last_update_login, null
                                           ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateLogin", iav.last_update_login))
                                    ,decode(iav.created_by, null
                                           ,XMLElement("CreatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreatedBy", iav.created_by))
                                    ,decode(iav.creation_date, null
                                           ,XMLElement("CreationDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreationDate", to_char(iav.creation_date, g_HRAPI_DATETIME_FMT)))
                                    )
                                  )
                                )
                               FROM TABLE (v.agency_vacancies) iav
                             )
                            ,(SELECT
                               XMLAgg
                                (XMLElement
                                  ("EO"
                                  ,XMLAttributes
                                    ('oracle.apps.irc.schema.server.IrcVariableCompElementsEO' as "Name")
                                  ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key_2nd_char(v.vacancy_id, ivce.variable_comp_code))
                                  ,XMLElement
                                    ("IrcVariableCompElementsEORow"
                                    ,XMLAttributes
                                      (decode(v.transaction_type, 'INSERT', 0, 2) as "PS"
                                      ,'Y' as "PK"
                                      ,'Y' as "CI")
                                    ,decode(v.vacancy_id, null
                                           ,XMLElement("VacancyId", XMLAttributes('true' as "null"))
                                           ,XMLElement("VacancyId", v.vacancy_id))
                                    ,decode(ivce.last_update_date, null
                                           ,XMLElement("LastUpdateDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateDate", to_char(ivce.last_update_date, g_HRAPI_DATETIME_FMT)))
                                    ,decode(ivce.last_updated_by, null
                                           ,XMLElement("LastUpdatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdatedBy", ivce.last_updated_by))
                                    ,decode(ivce.last_update_login, null
                                           ,XMLElement("LastUpdateLogin", XMLAttributes('true' as "null"))
                                           ,XMLElement("LastUpdateLogin", ivce.last_update_login))
                                    ,decode(ivce.created_by, null
                                           ,XMLElement("CreatedBy", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreatedBy", ivce.created_by))
                                    ,decode(ivce.creation_date, null
                                           ,XMLElement("CreationDate", XMLAttributes('true' as "null"))
                                           ,XMLElement("CreationDate", to_char(ivce.creation_date, g_HRAPI_DATETIME_FMT)))
                                    ,decode(ivce.object_version_number, null
                                           ,XMLElement("ObjectVersionNumber", XMLAttributes('true' as "null"))
                                           ,XMLElement("ObjectVersionNumber", ivce.object_version_number))                                           
                                    ,decode(ivce.variable_comp_code, null
                                           ,XMLElement("VariableCompLookup", XMLAttributes('true' as "null"))
                                           ,XMLElement("VariableCompLookup", ivce.variable_comp_code))
                                    ,XMLElement("VariableCompLookupType", 'IRC_VARIABLE_COMP_ELEMENT')
                                    )
                                  )
                                )
                               FROM TABLE (v.variable_comp_elements) ivce
                             )
                            )
                           )
                         )
                       )
                     )
                   )
                 ,XMLElement
                   ("EO"
                   ,XMLAttributes
                      ('oracle.apps.per.schema.server.TransactionsEO' as "Name")
                    ,XMLCDATA(xxas_selfservice_workflow_pkg.get_hex_key(v.transaction_id))
                    ,XMLElement
                      ("TransactionsEORow"
                      ,XMLAttributes
                        (0 as "PS")
                      )
                   )
                 )
               )
             )
           )
  FROM   TABLE(l_vacancy_tbl) v;
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  --
  -- Get the vacancy defaults
  --
  irc_get_vacancy_defaults.get_bg_defaults
    (p_business_group_id       => p_vacancy.business_group_id
    ,p_currency                => l_currency
    ,p_vacancy_name_is_auto    => l_vacancy_name_is_auto
    ,p_budget_measurement_type => l_budget_measurement_type
    ,p_number_of_openings      => l_number_of_openings
    ,p_location_default        => l_location_default
    ,p_organization_default    => l_organization_default);

  l_vacancy_tbl := xxas_per_all_vacancy_tbl(p_vacancy);
  
  OPEN  csr_trans_doc;
  FETCH csr_trans_doc INTO l_doc;
  CLOSE csr_trans_doc;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_doc;

END obj_to_hrapi_doc;

--
-- ------------------------------------------------------------------------ *
-- Name    : trans_to_obj
-- ------------------------------------------------------------------------ *
FUNCTION trans_to_obj 
  (p_transaction_id IN number) RETURN xxas_per_all_vacancy
IS

  c_method CONSTANT VARCHAR2(30) := 'trans_to_obj';
  l_obj             xxas_per_all_vacancy;
  l_hrapi           hr_api_transactions%ROWTYPE;
  
  --
  -- Gets recruitment activities for
  --
  CURSOR csr_rec_for IS
  SELECT praf.recruitment_activity_for_id
        ,praf.recruitment_activity_id
        ,praf.object_version_number
        ,praf.last_update_date
        ,praf.last_updated_by
        ,praf.last_update_login
        ,praf.created_by
        ,praf.creation_date
  FROM   XMLTable
           ('//PerRecruitmentActivityForEORow'
           PASSING (XMLType(l_hrapi.transaction_document))
           COLUMNS recruitment_activity_for_id     NUMBER          path 'RecruitmentActivityForId'
                  ,recruitment_activity_id         NUMBER          path 'RecruitmentActivityId'
                  ,object_version_number           NUMBER          path 'ObjectVersionNumber'
                  ,last_update_date                VARCHAR2(30)    path 'LastUpdateDate'
                  ,last_updated_by                 NUMBER          path 'LastUpdatedBy'
                  ,last_update_login               NUMBER          path 'LastUpdateLogin'
                  ,created_by                      NUMBER          path 'CreatedBy'
                  ,creation_date                   VARCHAR2(30)    path 'CreationDate'
            ) praf;
  
  --
  -- Creates an Object from the transaction table
  --
  CURSOR csr_trans_to_obj IS
  SELECT xxas_per_all_vacancy
           (v.vacancy_id
           ,xxas_vacancy_pkg.get_href
             (v.manager_id, v.vacancy_id) --href          
           ,v.name
           ,xxas_util_pkg.flag_to_boolean(xxas_com_sec_pkg.test_instance('XXAS_DATA_TRANS_NO_PEND_VIEW', to_char(p_transaction_id)))  --edit_allowed
           ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_EDIT') --edit_label
           ,xxas_util_pkg.flag_to_boolean(xxas_com_sec_pkg.test_instance('XXAS_DATA_TRANS_NO_PEND_DEL', to_char(p_transaction_id)))  --delete_allowed
           ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_DEL') --delete_label
           ,0 --num_total
           ,0 --num_active
           ,0 --num_term
           ,0 --num_interview1
           ,0 --num_interview2
           ,0 --num_offer
           ,0 --num_accepted
           ,0 --num_not_set
           ,0 --num_no
           ,0 --num_unsure
           ,0 --num_yes
           ,xxas_vacancy_pkg.get_view_schema_href
             (v.manager_id, v.vacancy_id) --view_schema_href
           ,xxas_vacancy_pkg.get_form_schema_href
             (v.manager_id, v.vacancy_id) --form_schema_href
           ,xxas_vacancy_pkg.get_applicants_href(v.vacancy_id) --applicants_href
           ,xxas_vacancy_pkg.get_applicants_export_href(v.vacancy_id) --applicants_export_href
           ,xxas_vacancy_pkg.get_offers_href(v.vacancy_id) --offers_href
           ,xxas_vacancy_pkg.get_ad_hoc_comms_href(v.vacancy_id) --ad_hoc_comms_href           
           ,p_transaction_id
           ,v.transaction_type
           ,l_hrapi.status
           ,xxas_selfservice_workflow_pkg.transcode_status(l_hrapi.status)
           ,xxas_selfservice_workflow_pkg.decode_status
             (xxas_selfservice_workflow_pkg.transcode_status(l_hrapi.status))
           ,l_hrapi.item_type
           ,l_hrapi.item_key
           ,v.business_group_id
           ,bg.name
           ,v.position_id
           ,pos.name
           ,v.job_id
           ,pj.name
           ,v.grade_id
           ,grd.name
           ,v.organization_id
           ,org.name
           ,v.requisition_id
           ,v.req_object_version_number
           ,v.people_group_id
           ,v.location_id
           ,loc.location_code
           ,v.recruiter_id
           ,xxas_xml_util_pkg.oafstring_to_date(v.date_from)
           ,xxas_xml_util_pkg.oafstring_to_date(v.date_to)
           ,v.description
           ,v.number_of_openings
           ,v.status
           ,xxas_util_pkg.decode_lookup('VACANCY_STATUS', to_char(v.status))
           ,CASE xxas_selfservice_workflow_pkg.transcode_status(l_hrapi.status)
              WHEN 'SFL'      THEN '#999999'
              WHEN 'RFC'      THEN '#FF0000'
              WHEN 'REVIEW'   THEN '#999999'
              WHEN 'PENDING'  THEN '#FF9900'
              WHEN 'ERROR'    THEN '#CC0000'
              WHEN 'APPROVED' THEN '#499324'
            END --status_color
           ,null --request_id
           ,null --program_application_id
           ,null --program_id
           ,null --program_update_date
           ,v.attribute_category
           ,v.attribute1
           ,v.attribute2
           ,v.attribute3
           ,v.attribute4
           ,v.attribute5
           ,v.attribute6
           ,v.attribute7
           ,v.attribute8
           ,v.attribute9
           ,v.attribute10
           ,v.attribute11
           ,v.attribute12
           ,v.attribute13
           ,v.attribute14
           ,v.attribute15
           ,v.attribute16
           ,v.attribute17
           ,v.attribute18
           ,v.attribute19
           ,v.attribute20
           ,xxas_xml_util_pkg.oafstring_to_date(v.last_update_date)
           ,v.last_updated_by
           ,v.last_update_login
           ,nvl(v.created_by, basev.created_by)
           ,xxas_xml_util_pkg.oafstring_to_date(v.creation_date)
           ,v.vacancy_category
           ,xxas_util_pkg.decode_lookup('VACANCY_CATEGORY', v.vacancy_category)
           ,v.budget_measurement_type
           ,xxas_util_pkg.decode_lookup('BUDGET_MEASUREMENT_TYPE', v.budget_measurement_type)
           ,v.budget_measurement_value
           ,v.manager_id
           ,v.security_method
           ,v.object_version_number
           ,v.assessment_id
           ,v.primary_posting_id
           ,null --comments
           ,v.comments_to_approver
           ,xxas_util_pkg.flag_to_boolean(v.display_manager_info)
           ,xxas_util_pkg.flag_to_boolean(v.display_recruiter_info)
           ,v.posting_name
           ,v.posting_org_name
           ,case when dbms_lob.getlength(extract(v.xml, '//IrcPostingContentsVlEORow/OrgDescription/text()').getClobVal()) > 4000 then 
                  null
                 else
                  extractvalue(v.xml, '//IrcPostingContentsVlEORow/OrgDescription/text()')
            end  --org_description
           ,v.posting_job_title
           ,case when dbms_lob.getlength(extract(v.xml, '//IrcPostingContentsVlEORow/BriefDescription/text()').getClobVal()) > 4000 then 
                  null
                 else
                  extractvalue(v.xml, '//IrcPostingContentsVlEORow/BriefDescription/text()')
            end  --brief_description
           ,case when dbms_lob.getlength(extract(v.xml, '//IrcPostingContentsVlEORow/DetailedDescription/text()').getClobVal()) > 4000 then 
                  null
                 else
                  extractvalue(v.xml, '//IrcPostingContentsVlEORow/DetailedDescription/text()')
            end  --detailed_description
           ,case when dbms_lob.getlength(extract(v.xml, '//IrcPostingContentsVlEORow/JobRequirements/text()').getClobVal()) > 4000 then 
                  null
                 else
                  extractvalue(v.xml, '//IrcPostingContentsVlEORow/JobRequirements/text()')
            end  --job_requirements
           ,case when dbms_lob.getlength(extract(v.xml, '//IrcPostingContentsVlEORow/AdditionalDetails/text()').getClobVal()) > 4000 then 
                  null
                 else
                  extractvalue(v.xml, '//IrcPostingContentsVlEORow/AdditionalDetails/text()')
            end  --additional_details
           ,extractvalue(v.xml, '//IrcPostingContentsVlEORow/HowToApply/text()') --how_to_apply
           ,extractvalue(v.xml, '//IrcPostingContentsVlEORow/ImageUrl/text()') --image_url
           ,extractvalue(v.xml, '//IrcPostingContentsVlEORow/ImageUrlAlt/text()') --image_url_alt
           ,extractvalue(v.xml, '//IrcPostingContentsVlEORow/BenefitInfo/text()') --benefit_info
           ,v.ipc_attribute_category
           ,v.ipc_attribute1
           ,v.ipc_attribute2
           ,v.ipc_attribute3
           ,v.ipc_attribute4
           ,v.ipc_attribute5
           ,v.ipc_attribute6
           ,v.ipc_attribute7
           ,v.ipc_attribute8
           ,v.ipc_attribute9
           ,v.ipc_attribute10
           ,v.ipc_attribute11
           ,v.ipc_attribute12
           ,v.ipc_attribute13
           ,v.ipc_attribute14
           ,v.ipc_attribute15
           ,v.ipc_attribute16
           ,v.ipc_attribute17
           ,v.ipc_attribute18
           ,v.ipc_attribute19
           ,v.ipc_attribute20
           ,v.ipc_attribute21
           ,v.ipc_attribute22
           ,v.ipc_attribute23
           ,v.ipc_attribute24
           ,v.ipc_attribute25
           ,v.ipc_attribute26
           ,v.ipc_attribute27
           ,v.ipc_attribute28
           ,v.ipc_attribute29
           ,v.ipc_attribute30
           ,xxas_xml_util_pkg.oafstring_to_date(v.ipc_last_update_date)
           ,v.ipc_last_updated_by
           ,v.ipc_last_update_login
           ,v.ipc_created_by
           ,xxas_xml_util_pkg.oafstring_to_date(v.ipc_creation_date)
           ,v.ipc_object_version_number
           ,null --recruiter_full_name
           ,null --recruiter_email
           ,null --recruiter_work_telephone
           ,null --manager_full_name
           ,null --manager_email
           ,null --manager_work_telephone
           ,v.search_criteria_id
           ,v.employee_flag
           ,v.contractor_flag
           ,v.employment_category
           ,xxas_util_pkg.decode_lookup('IRC_EMP_CAT', v.employment_category)
           ,v.travel_percentage
           ,xxas_util_pkg.decode_lookup
             ('IRC_TRAVEL_PERCENTAGE'
             ,fnd_number.number_to_canonical(v.travel_percentage))
           ,v.min_salary
           ,v.max_salary
           ,v.salary_currency
           ,v.salary_period
           ,xxas_util_pkg.decode_lookup('PAY_BASIS', v.salary_period)
           ,v.professional_area
           ,xxas_util_pkg.decode_lookup('IRC_PROFESSIONAL_AREA', v.professional_area)
           ,v.work_at_home
           ,xxas_util_pkg.decode_lookup('IRC_WORK_AT_HOME', v.work_at_home)
           ,v.min_qual_level
           ,xxas_qualifications_pkg.decode_qualification_type(v.min_qual_level)
           ,v.max_qual_level
           ,xxas_qualifications_pkg.decode_qualification_type(v.max_qual_level)
           ,v.isc_attribute_category
           ,v.isc_attribute1
           ,v.isc_attribute2
           ,v.isc_attribute3
           ,v.isc_attribute4
           ,v.isc_attribute5
           ,v.isc_attribute6
           ,v.isc_attribute7
           ,v.isc_attribute8
           ,v.isc_attribute9
           ,v.isc_attribute10
           ,v.isc_attribute11
           ,v.isc_attribute12
           ,v.isc_attribute13
           ,v.isc_attribute14
           ,v.isc_attribute15
           ,v.isc_attribute16
           ,v.isc_attribute17
           ,v.isc_attribute18
           ,v.isc_attribute19
           ,v.isc_attribute20
           ,v.isc_attribute21
           ,v.isc_attribute22
           ,v.isc_attribute23
           ,v.isc_attribute24
           ,v.isc_attribute25
           ,v.isc_attribute26
           ,v.isc_attribute27
           ,v.isc_attribute28
           ,v.isc_attribute29
           ,v.isc_attribute30
           ,v.isc_object_version_number
           ,v.pg_id_flex_num
           ,v.segment1
           ,v.segment2
           ,v.segment3
           ,v.segment4
           ,v.segment5
           ,v.segment6
           ,v.segment7
           ,v.segment8
           ,v.segment9
           ,v.segment10
           ,v.segment11
           ,v.segment12
           ,v.segment13
           ,v.segment14
           ,v.segment15
           ,v.segment16
           ,v.segment17
           ,v.segment18
           ,v.segment19
           ,v.segment20
           ,v.segment21
           ,v.segment22
           ,v.segment23
           ,v.segment24
           ,v.segment25
           ,v.segment26
           ,v.segment27
           ,v.segment28
           ,v.segment29
           ,v.segment30
           ,cast
             (multiset
               (SELECT xxas_irc_variable_comp_ele
                         (ivce.vacancy_id
                         ,ivce.variable_comp_lookup
                         ,xxas_util_pkg.decode_lookup('IRC_VARIABLE_COMP_ELEMENT', ivce.variable_comp_lookup)
                         ,ivce.object_version_number
                         ,xxas_xml_util_pkg.oafstring_to_date(ivce.last_update_date)
                         ,ivce.last_updated_by
                         ,ivce.last_update_login
                         ,ivce.created_by
                         ,xxas_xml_util_pkg.oafstring_to_date(ivce.creation_date))
                FROM   XMLTable
                         ('//IrcVariableCompElementsEORow'
                         PASSING v.xml
                         COLUMNS vacancy_id                      NUMBER          path 'VacancyId'
                                ,variable_comp_lookup            VARCHAR2(30)    path 'VariableCompLookup'
                                ,object_version_number           NUMBER          path 'ObjectVersionNumber'
                                ,last_update_date                VARCHAR2(30)    path 'LastUpdateDate'
                                ,last_updated_by                 NUMBER          path 'LastUpdatedBy'
                                ,last_update_login               NUMBER          path 'LastUpdateLogin'
                                ,created_by                      NUMBER          path 'CreatedBy'
                                ,creation_date                   VARCHAR2(30)    path 'CreationDate'
                          ) ivce
             )
             AS xxas_irc_variable_comp_ele_tbl)
           ,cast
             (multiset
               (SELECT xxas_per_competence_ele
                         (pce.competence_element_id
                         ,pce.object_version_number
                         ,pce.type
                         ,xxas_util_pkg.decode_lookup('COMPETENCE_ELEMENT_TYPE', pce.type)
                         ,pce.competence_id
                         ,pc.name
                         ,pce.proficiency_level_id
                         ,prlmin.name
                         ,pce.high_proficiency_level_id
                         ,prlmax.name
                         ,pce.mandatory
                         ,xxas_util_pkg.decode_lookup('IRC_SKILL_DESIRABILITY', pce.mandatory)
                         ,pce.attribute_category
                         ,pce.attribute1
                         ,pce.attribute2
                         ,pce.attribute3
                         ,pce.attribute4
                         ,pce.attribute5
                         ,pce.attribute6
                         ,pce.attribute7
                         ,pce.attribute8
                         ,pce.attribute9
                         ,pce.attribute10
                         ,pce.attribute11
                         ,pce.attribute12
                         ,pce.attribute13
                         ,pce.attribute14
                         ,pce.attribute15
                         ,pce.attribute16
                         ,pce.attribute17
                         ,pce.attribute18
                         ,pce.attribute19
                         ,pce.attribute20
                         ,xxas_xml_util_pkg.oafstring_to_date(pce.last_update_date)
                         ,pce.last_updated_by
                         ,pce.last_update_login
                         ,pce.created_by
                         ,xxas_xml_util_pkg.oafstring_to_date(pce.creation_date))
                FROM   XMLTable
                         ('//VacancyCompetenceElementsEORow'
                         PASSING v.xml
                         COLUMNS competence_element_id           NUMBER          path 'CompetenceElementId'
                                ,object_version_number           NUMBER          path 'ObjectVersionNumber'
                                ,type                            VARCHAR2(30)    path 'Type'
                                ,competence_id                   NUMBER          path 'CompetenceId'
                                ,proficiency_level_id            NUMBER          path 'ProficiencyLevelId'
                                ,high_proficiency_level_id       NUMBER          path 'HighProficiencyLevelId'
                                ,mandatory                       VARCHAR2(30)    path 'Mandatory'
                                ,attribute_category              VARCHAR2(30)    path 'AttributeCategory'
                                ,attribute1                      VARCHAR2(150)   path 'Attribute1'
                                ,attribute2                      VARCHAR2(150)   path 'Attribute2'
                                ,attribute3                      VARCHAR2(150)   path 'Attribute3'
                                ,attribute4                      VARCHAR2(150)   path 'Attribute4'
                                ,attribute5                      VARCHAR2(150)   path 'Attribute5'
                                ,attribute6                      VARCHAR2(150)   path 'Attribute6'
                                ,attribute7                      VARCHAR2(150)   path 'Attribute7'
                                ,attribute8                      VARCHAR2(150)   path 'Attribute8'
                                ,attribute9                      VARCHAR2(150)   path 'Attribute9'
                                ,attribute10                     VARCHAR2(150)   path 'Attribute10'
                                ,attribute11                     VARCHAR2(150)   path 'Attribute11'
                                ,attribute12                     VARCHAR2(150)   path 'Attribute12'
                                ,attribute13                     VARCHAR2(150)   path 'Attribute13'
                                ,attribute14                     VARCHAR2(150)   path 'Attribute14'
                                ,attribute15                     VARCHAR2(150)   path 'Attribute15'
                                ,attribute16                     VARCHAR2(150)   path 'Attribute16'
                                ,attribute17                     VARCHAR2(150)   path 'Attribute17'
                                ,attribute18                     VARCHAR2(150)   path 'Attribute18'
                                ,attribute19                     VARCHAR2(150)   path 'Attribute19'
                                ,attribute20                     VARCHAR2(150)   path 'Attribute20'
                                ,last_update_date                VARCHAR2(30)    path 'LastUpdateDate'
                                ,last_updated_by                 NUMBER          path 'LastUpdatedBy'
                                ,last_update_login               NUMBER          path 'LastUpdateLogin'
                                ,created_by                      NUMBER          path 'CreatedBy'
                                ,creation_date                   VARCHAR2(30)    path 'CreationDate'
                          ) pce
                      ,per_competences pc
                      ,per_rating_levels_vl prlmin
                      ,per_rating_levels_vl prlmax
                WHERE  pce.competence_id = pc.competence_id(+)
                AND    pce.proficiency_level_id = prlmin.rating_level_id(+)
                AND    pce.high_proficiency_level_id = prlmax.rating_level_id(+)
             )
             AS xxas_per_competence_ele_tbl)
           ,cast
             (multiset
               (SELECT xxas_irc_agency_vacancy
                         (iav.agency_vacancy_id
                         ,iav.agency_id
                         ,pov.vendor_name
                         ,iav.vacancy_id
                         ,iav.start_date
                         ,iav.end_date
                         ,iav.max_allowed_applicants
                         ,xxas_util_pkg.flag_to_boolean(iav.manage_applicants_allowed)
                         ,iav.attribute_category
                         ,iav.attribute1
                         ,iav.attribute2
                         ,iav.attribute3
                         ,iav.attribute4
                         ,iav.attribute5
                         ,iav.attribute6
                         ,iav.attribute7
                         ,iav.attribute8
                         ,iav.attribute9
                         ,iav.attribute10
                         ,iav.attribute11
                         ,iav.attribute12
                         ,iav.attribute13
                         ,iav.attribute14
                         ,iav.attribute15
                         ,iav.attribute16
                         ,iav.attribute17
                         ,iav.attribute18
                         ,iav.attribute19
                         ,iav.attribute20
                         ,iav.attribute21
                         ,iav.attribute22
                         ,iav.attribute23
                         ,iav.attribute24
                         ,iav.attribute25
                         ,iav.attribute26
                         ,iav.attribute27
                         ,iav.attribute28
                         ,iav.attribute29
                         ,iav.attribute30
                         ,iav.object_version_number
                         ,xxas_xml_util_pkg.oafstring_to_date(iav.last_update_date)
                         ,iav.last_updated_by
                         ,iav.last_update_login
                         ,iav.created_by
                         ,xxas_xml_util_pkg.oafstring_to_date(iav.creation_date))
                FROM   XMLTable
                         ('//IrcAgencyVacanciesEORow'
                         PASSING v.xml
                         COLUMNS agency_vacancy_id               NUMBER          path 'AgencyVacancyId'
                                ,agency_id                       NUMBER          path 'AgencyId'
                                ,vacancy_id                      NUMBER          path 'VacancyId'
                                ,start_date                      DATE            path 'StartDate'
                                ,end_date                        DATE            path 'EndDate'
                                ,max_allowed_applicants          NUMBER          path 'MaxAllowedApplicants'
                                ,manage_applicants_allowed       VARCHAR2(30)    path 'ManageApplicantsAllowed'
                                ,attribute_category              VARCHAR2(30)    path 'AttributeCategory'
                                ,attribute1                      VARCHAR2(150)   path 'Attribute1'
                                ,attribute2                      VARCHAR2(150)   path 'Attribute2'
                                ,attribute3                      VARCHAR2(150)   path 'Attribute3'
                                ,attribute4                      VARCHAR2(150)   path 'Attribute4'
                                ,attribute5                      VARCHAR2(150)   path 'Attribute5'
                                ,attribute6                      VARCHAR2(150)   path 'Attribute6'
                                ,attribute7                      VARCHAR2(150)   path 'Attribute7'
                                ,attribute8                      VARCHAR2(150)   path 'Attribute8'
                                ,attribute9                      VARCHAR2(150)   path 'Attribute9'
                                ,attribute10                     VARCHAR2(150)   path 'Attribute10'
                                ,attribute11                     VARCHAR2(150)   path 'Attribute11'
                                ,attribute12                     VARCHAR2(150)   path 'Attribute12'
                                ,attribute13                     VARCHAR2(150)   path 'Attribute13'
                                ,attribute14                     VARCHAR2(150)   path 'Attribute14'
                                ,attribute15                     VARCHAR2(150)   path 'Attribute15'
                                ,attribute16                     VARCHAR2(150)   path 'Attribute16'
                                ,attribute17                     VARCHAR2(150)   path 'Attribute17'
                                ,attribute18                     VARCHAR2(150)   path 'Attribute18'
                                ,attribute19                     VARCHAR2(150)   path 'Attribute19'
                                ,attribute20                     VARCHAR2(150)   path 'Attribute20'
                                ,attribute21                     VARCHAR2(150)   path 'Attribute21'
                                ,attribute22                     VARCHAR2(150)   path 'Attribute22'
                                ,attribute23                     VARCHAR2(150)   path 'Attribute23'
                                ,attribute24                     VARCHAR2(150)   path 'Attribute24'
                                ,attribute25                     VARCHAR2(150)   path 'Attribute25'
                                ,attribute26                     VARCHAR2(150)   path 'Attribute26'
                                ,attribute27                     VARCHAR2(150)   path 'Attribute27'
                                ,attribute28                     VARCHAR2(150)   path 'Attribute28'
                                ,attribute29                     VARCHAR2(150)   path 'Attribute29'
                                ,attribute30                     VARCHAR2(150)   path 'Attribute30'
                                ,object_version_number           NUMBER          path 'ObjectVersionNumber'
                                ,last_update_date                VARCHAR2(30)    path 'LastUpdateDate'
                                ,last_updated_by                 NUMBER          path 'LastUpdatedBy'
                                ,last_update_login               NUMBER          path 'LastUpdateLogin'
                                ,created_by                      NUMBER          path 'CreatedBy'
                                ,creation_date                   VARCHAR2(30)    path 'CreationDate'
                          ) iav
                      ,po_vendors pov
                WHERE  iav.agency_id = pov.vendor_id(+)
             )
             AS xxas_irc_agency_vacancy_tbl)
           ,cast
             (multiset
               (SELECT xxas_irc_rec_team_member
                         (itm.rec_team_member_id
                         ,itm.person_id
                         ,itm.party_id
                         ,itm.vacancy_id
                         ,itm.job_id
                         ,itm.start_date
                         ,itm.end_date
                         ,itm.update_allowed
                         ,itm.delete_allowed
                         ,itm.object_version_number
                         ,xxas_xml_util_pkg.oafstring_to_date(itm.last_update_date)
                         ,itm.last_updated_by
                         ,itm.last_update_login
                         ,itm.created_by
                         ,xxas_xml_util_pkg.oafstring_to_date(itm.creation_date))
                FROM   XMLTable
                         ('//IrcRecTeamMembersEORow'
                         PASSING v.xml
                         COLUMNS rec_team_member_id              NUMBER          path 'RecTeamMemberId'
                                ,party_id                        NUMBER          path 'PartyId'
                                ,vacancy_id                      NUMBER          path 'VacancyId'
                                ,job_id                          NUMBER          path 'JobId'
                                ,start_date                      DATE            path 'StartDate'
                                ,end_date                        DATE            path 'EndDate'
                                ,update_allowed                  VARCHAR2(30)    path 'UpdateAllowed'
                                ,delete_allowed                  VARCHAR2(30)    path 'DeleteAllowed'
                                ,object_version_number           NUMBER          path 'ObjectVersionNumber'
                                ,person_id                       NUMBER          path 'PersonId'
                                ,last_update_date                VARCHAR2(30)    path 'LastUpdateDate'
                                ,last_updated_by                 NUMBER          path 'LastUpdatedBy'
                                ,last_update_login               NUMBER          path 'LastUpdateLogin'
                                ,created_by                      NUMBER          path 'CreatedBy'
                                ,creation_date                   VARCHAR2(30)    path 'CreationDate'
                          ) itm
             )
             AS xxas_irc_rec_team_member_tbl)
           ,cast
             (multiset
               (SELECT xxas_per_rec_activity
                         (pra.recruitment_activity_id
                         ,null --recruitment_activity_for_id
                         ,v.vacancy_id
                         ,xxas_xml_util_pkg.oafstring_to_date(pra.date_start)
                         ,xxas_xml_util_pkg.oafstring_to_date(pra.date_end)
                         ,pra.name
                         ,pra.posting_content_id
                         ,pra.recruiting_site_id
                         ,irs.site_name
                         ,pra.object_version_number
                         ,null --for_object_version_number
                         ,xxas_xml_util_pkg.oafstring_to_date(pra.last_update_date)
                         ,pra.last_updated_by
                         ,pra.last_update_login
                         ,pra.created_by
                         ,xxas_xml_util_pkg.oafstring_to_date(pra.creation_date))
                FROM   XMLTable
                         ('//PerRecruitmentActivitiesEORow'
                         PASSING v.xml
                         COLUMNS recruitment_activity_id         NUMBER          path 'RecruitmentActivityId'
                                ,date_start                      VARCHAR2(30)    path 'DateStart'
                                ,date_end                        VARCHAR2(30)    path 'DateEnd'                               
                                ,name                            VARCHAR2(30)    path 'Name'
                                ,posting_content_id              NUMBER          path 'PostingContentId' 
                                ,recruiting_site_id              NUMBER          path 'RecruitingSiteId' 
                                ,object_version_number           NUMBER          path 'ObjectVersionNumber'
                                ,last_update_date                VARCHAR2(30)    path 'LastUpdateDate'
                                ,last_updated_by                 NUMBER          path 'LastUpdatedBy'
                                ,last_update_login               NUMBER          path 'LastUpdateLogin'
                                ,created_by                      NUMBER          path 'CreatedBy'
                                ,creation_date                   VARCHAR2(30)    path 'CreationDate'
                          ) pra
                      ,irc_all_recruiting_sites_vl irs
                WHERE  pra.recruiting_site_id = irs.recruiting_site_id(+)
             )
             AS xxas_per_rec_activity_tbl)
           ,xxas_vacancy_task_tbl()
           ,cast
             (multiset
               (SELECT a.attached_document_id
                         ,a.seq_num sequence_number
                         ,a.entity_name
                         ,a.pk1_value
                         ,a.pk2_value
                         ,a.pk3_value
                         ,a.pk4_value
                         ,a.pk5_value
                         ,c.name category_code
                         ,c.user_name category_name
                         ,d.document_id
                         ,d.file_name
                         ,d.title
                         ,d.description
                         ,l.file_id
                         ,l.file_content_type mime_type
                         ,l.file_format lob_format
                         ,fnd_gfm.construct_download_url(fnd_web_config.gfm_agent, l.file_id) file_url
                         ,null wp_link
						 ,a.creation_date
                         ,xxas_wf_role_util_pkg.get_display_name_from_user(a.created_by) created_by
                         ,a.last_update_date
                         ,xxas_wf_role_util_pkg.get_display_name_from_user(a.last_updated_by) last_updated_by
                FROM   fnd_attached_documents a
                      ,fnd_documents_vl d
                      ,fnd_lobs l
                      ,fnd_document_categories_vl c
                      ,fnd_document_datatypes_vl t
                WHERE  t.name = 'FILE'
                AND    d.media_id = l.file_id
                AND    a.document_id = d.document_id
                AND    a.category_id = c.category_id
                AND    d.datatype_id = t.datatype_id
                AND    a.entity_name IN('IRC_EXT_VAC_APPROVED', 'IRC_INT_VAC_APPROVED','IRC_EXT_VAC','IRC_INT_VAC')
                AND    v.vacancy_id IS NOT NULL 
                AND    a.pk1_value IS NOT NULL 
                AND    a.pk1_value = to_char(v.vacancy_id)
             )
             AS xxas_attachment_table)
           ,xxas_selfservice_workflow_pkg.get_approvers(p_transaction_id,g_AME_APP_ID,g_AME_TRANS_TYPE)
           ) vacancy
  FROM   XMLTable
          ('/Transaction'
          PASSING  XMLType(l_hrapi.transaction_document)
          COLUMNS  xml                             XMLType         path '.'
                  ,vacancy_id                      NUMBER          path '//PerAllVacanciesEORow/VacancyId'
                  ,name                            VARCHAR2(30)    path '//PerAllVacanciesEORow/Name'
                  ,transaction_type                VARCHAR2(30)    path 'TransCtx/CNode/dmlMode'
                  ,business_group_id               NUMBER          path '//PerAllVacanciesEORow/BusinessGroupId'
                  ,position_id                     NUMBER          path '//PerAllVacanciesEORow/PositionId'
                  ,job_id                          NUMBER          path '//PerAllVacanciesEORow/JobId'
                  ,grade_id                        NUMBER          path '//PerAllVacanciesEORow/GradeId'
                  ,organization_id                 NUMBER          path '//PerAllVacanciesEORow/OrganizationId'
                  ,requisition_id                  NUMBER          path '//PerAllVacanciesEORow/RequisitionId'
                  ,req_object_version_number       NUMBER          path '//PerRequisitionsEORow/ObjectVersionNumber'
                  ,people_group_id                 NUMBER          path '//PerAllVacanciesEORow/PeopleGroupId'
                  ,location_id                     NUMBER          path '//PerAllVacanciesEORow/LocationId'
                  ,recruiter_id                    NUMBER          path '//PerAllVacanciesEORow/RecruiterId'
                  ,date_from                       VARCHAR2(30)    path '//PerAllVacanciesEORow/DateFrom'
                  ,date_to                         VARCHAR2(30)    path '//PerAllVacanciesEORow/DateTo'
                  ,description                     VARCHAR2(80)    path '//PerAllVacanciesEORow/Description'
                  ,number_of_openings              NUMBER          path '//PerAllVacanciesEORow/NumberOfOpenings'
                  ,status                          VARCHAR2(30)    path '//PerAllVacanciesEORow/Status'
                  ,attribute_category              VARCHAR2(30)    path '//PerAllVacanciesEORow/AttributeCategory'
                  ,attribute1                      VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute1'
                  ,attribute2                      VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute2'
                  ,attribute3                      VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute3'
                  ,attribute4                      VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute4'
                  ,attribute5                      VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute5'
                  ,attribute6                      VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute6'
                  ,attribute7                      VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute7'
                  ,attribute8                      VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute8'
                  ,attribute9                      VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute9'
                  ,attribute10                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute10'
                  ,attribute11                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute11'
                  ,attribute12                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute12'
                  ,attribute13                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute13'
                  ,attribute14                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute14'
                  ,attribute15                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute15'
                  ,attribute16                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute16'
                  ,attribute17                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute17'
                  ,attribute18                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute18'
                  ,attribute19                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute19'
                  ,attribute20                     VARCHAR2(150)   path '//PerAllVacanciesEORow/Attribute20'
                  ,last_update_date                VARCHAR2(30)    path '//PerAllVacanciesEORow/LastUpdateDate'
                  ,last_updated_by                 NUMBER          path '//PerAllVacanciesEORow/LastUpdatedBy'
                  ,last_update_login               NUMBER          path '//PerAllVacanciesEORow/LastUpdateLogin'
                  ,created_by                      NUMBER          path '//PerAllVacanciesEORow/CreatedBy'
                  ,creation_date                   VARCHAR2(30)    path '//PerAllVacanciesEORow/CreationDate'
                  ,vacancy_category                VARCHAR2(30)    path '//PerAllVacanciesEORow/VacancyCategory'
                  ,budget_measurement_type         VARCHAR2(30)    path '//PerAllVacanciesEORow/BudgetMeasurementType'
                  ,budget_measurement_value        NUMBER          path '//PerAllVacanciesEORow/BudgetMeasurementValue'
                  ,manager_id                      NUMBER          path '//PerAllVacanciesEORow/ManagerId'
                  ,security_method                 VARCHAR2(30)    path '//PerAllVacanciesEORow/SecurityMethod'
                  ,object_version_number           NUMBER          path '//PerAllVacanciesEORow/ObjectVersionNumber'
                  ,assessment_id                   NUMBER          path '//PerAllVacanciesEORow/AssessmentId'
                  ,primary_posting_id              NUMBER          path '//PerAllVacanciesEORow/PrimaryPostingId'                  
                  ,comments                        VARCHAR2(4000)  path '//PerAllVacanciesEORow/Comments'
                  ,comments_to_approver            VARCHAR2(4000)  path 'TransCtx/pApprComments'                  
                  ,display_manager_info            VARCHAR2(30)    path '//IrcPostingContentsVlEORow/DisplayManagerInfo'
                  ,display_recruiter_info          VARCHAR2(30)    path '//IrcPostingContentsVlEORow/DisplayRecruiterInfo'
                  ,posting_name                    VARCHAR2(240)   path '//IrcPostingContentsVlEORow/Name'
                  ,posting_org_name                VARCHAR2(240)   path '//IrcPostingContentsVlEORow/OrgName'
                  ,posting_job_title               VARCHAR2(240)   path '//IrcPostingContentsVlEORow/JobTitle'
                  ,ipc_attribute_category          VARCHAR2(30)    path '//IrcPostingContentsVlEORow/AttributeCategory'
                  ,ipc_attribute1                  VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute1'
                  ,ipc_attribute2                  VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute2'
                  ,ipc_attribute3                  VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute3'
                  ,ipc_attribute4                  VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute4'
                  ,ipc_attribute5                  VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute5'
                  ,ipc_attribute6                  VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute6'
                  ,ipc_attribute7                  VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute7'
                  ,ipc_attribute8                  VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute8'
                  ,ipc_attribute9                  VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute9'
                  ,ipc_attribute10                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute10'
                  ,ipc_attribute11                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute11'
                  ,ipc_attribute12                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute12'
                  ,ipc_attribute13                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute13'
                  ,ipc_attribute14                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute14'
                  ,ipc_attribute15                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute15'
                  ,ipc_attribute16                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute16'
                  ,ipc_attribute17                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute17'
                  ,ipc_attribute18                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute18'
                  ,ipc_attribute19                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute19'
                  ,ipc_attribute20                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute20'
                  ,ipc_attribute21                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute21'
                  ,ipc_attribute22                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute22'
                  ,ipc_attribute23                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute23'
                  ,ipc_attribute24                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute24'
                  ,ipc_attribute25                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute25'
                  ,ipc_attribute26                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute26'
                  ,ipc_attribute27                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute27'
                  ,ipc_attribute28                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute28'
                  ,ipc_attribute29                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute29'
                  ,ipc_attribute30                 VARCHAR2(150)   path '//IrcPostingContentsVlEORow/Attribute30'
                  ,ipc_last_update_date            VARCHAR2(30)    path '//IrcPostingContentsVlEORow/LastUpdateDate'
                  ,ipc_last_updated_by             NUMBER          path '//IrcPostingContentsVlEORow/LastUpdatedBy'
                  ,ipc_last_update_login           NUMBER          path '//IrcPostingContentsVlEORow/LastUpdateLogin'
                  ,ipc_created_by                  NUMBER          path '//IrcPostingContentsVlEORow/CreatedBy'
                  ,ipc_creation_date               VARCHAR2(30)    path '//IrcPostingContentsVlEORow/CreationDate'
                  ,ipc_object_version_number       NUMBER          path '//IrcPostingContentsVlEORow/ObjectVersionNumber'
                  ,search_criteria_id              NUMBER          path '//IrcVacancySearchCriteriaEORow/SearchCriteriaId'
                  ,employee_flag                   VARCHAR2(30)    path '//IrcVacancySearchCriteriaEORow/Employee'
                  ,contractor_flag                 VARCHAR2(30)    path '//IrcVacancySearchCriteriaEORow/Contractor'
                  ,employment_category             VARCHAR2(30)    path '//IrcVacancySearchCriteriaEORow/EmploymentCategory'
                  ,travel_percentage               VARCHAR2(30)    path '//IrcVacancySearchCriteriaEORow/TravelPercentage'
                  ,min_salary                      NUMBER(15,2)    path '//IrcVacancySearchCriteriaEORow/MinSalary'
                  ,max_salary                      NUMBER          path '//IrcVacancySearchCriteriaEORow/MaxSalary'
                  ,salary_currency                 VARCHAR2(30)    path '//IrcVacancySearchCriteriaEORow/SalaryCurrency'
                  ,salary_period                   VARCHAR2(30)    path '//IrcVacancySearchCriteriaEORow/SalaryPeriod'
                  ,professional_area               VARCHAR2(30)    path '//IrcVacancySearchCriteriaEORow/ProfessionalArea'
                  ,work_at_home                    VARCHAR2(30)    path '//IrcVacancySearchCriteriaEORow/WorkAtHome'
                  ,min_qual_level                  NUMBER          path '//IrcVacancySearchCriteriaEORow/MinQualLevel'
                  ,max_qual_level                  NUMBER          path '//IrcVacancySearchCriteriaEORow/MaxQualLevel'
                  ,isc_attribute_category          VARCHAR2(30)    path '//IrcVacancySearchCriteriaEORow/AttributeCategory'
                  ,isc_attribute1                  VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute1'
                  ,isc_attribute2                  VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute2'
                  ,isc_attribute3                  VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute3'
                  ,isc_attribute4                  VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute4'
                  ,isc_attribute5                  VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute5'
                  ,isc_attribute6                  VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute6'
                  ,isc_attribute7                  VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute7'
                  ,isc_attribute8                  VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute8'
                  ,isc_attribute9                  VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute9'
                  ,isc_attribute10                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute10'
                  ,isc_attribute11                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute11'
                  ,isc_attribute12                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute12'
                  ,isc_attribute13                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute13'
                  ,isc_attribute14                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute14'
                  ,isc_attribute15                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute15'
                  ,isc_attribute16                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute16'
                  ,isc_attribute17                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute17'
                  ,isc_attribute18                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute18'
                  ,isc_attribute19                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute19'
                  ,isc_attribute20                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute20'
                  ,isc_attribute21                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute21'
                  ,isc_attribute22                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute22'
                  ,isc_attribute23                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute23'
                  ,isc_attribute24                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute24'
                  ,isc_attribute25                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute25'
                  ,isc_attribute26                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute26'
                  ,isc_attribute27                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute27'
                  ,isc_attribute28                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute28'
                  ,isc_attribute29                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute29'
                  ,isc_attribute30                 VARCHAR2(150)   path '//IrcVacancySearchCriteriaEORow/Attribute30'
                  ,isc_object_version_number       NUMBER          path '//IrcVacancySearchCriteriaEORow/ObjectVersionNumber'
                  ,pg_id_flex_num                  NUMBER          path '//PerAllVacanciesEORow/PgIdFlexNum'
                  ,segment1                        VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment1'
                  ,segment2                        VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment2'
                  ,segment3                        VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment3'
                  ,segment4                        VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment4'
                  ,segment5                        VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment5'
                  ,segment6                        VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment6'
                  ,segment7                        VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment7'
                  ,segment8                        VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment8'
                  ,segment9                        VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment9'
                  ,segment10                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment10'
                  ,segment11                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment11'
                  ,segment12                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment12'
                  ,segment13                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment13'
                  ,segment14                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment14'
                  ,segment15                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment15'
                  ,segment16                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment16'
                  ,segment17                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment17'
                  ,segment18                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment18'
                  ,segment19                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment19'
                  ,segment20                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment20'
                  ,segment21                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment21'
                  ,segment22                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment22'
                  ,segment23                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment23'
                  ,segment24                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment24'
                  ,segment25                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment25'
                  ,segment26                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment26'
                  ,segment27                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment27'
                  ,segment28                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment28'
                  ,segment29                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment29'
                  ,segment30                       VARCHAR2(60)    path '//PerAllVacanciesEORow/Segment30'
                  ) v
        ,per_all_vacancies basev
        ,per_jobs_vl pj
        ,hr_all_positions_f_vl pos
        ,hr_locations_all_tl loc -- Selecting from _tl because geometry causing issues on 10g 
        ,hr_all_organization_units_vl org
        ,per_business_groups_perf bg
        ,per_grades_vl grd 
  WHERE  v.vacancy_id = basev.vacancy_id(+)
  AND    v.job_id = pj.job_id(+)
  AND    v.position_id = pos.position_id(+)
  AND    trunc(sysdate) BETWEEN
         pos.effective_start_date(+) AND pos.effective_end_date(+)
  AND    v.location_id = loc.location_id(+)
  AND    loc.language(+) = userenv('LANG')
  AND    v.organization_id = org.organization_id(+)
  AND    v.business_group_id = bg.business_group_id(+)
  AND    v.grade_id = grd.grade_id(+);

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_transaction_id);
  END IF;
  
  IF p_transaction_id IS NOT NULL THEN
  
    --l_hrapi := xxas_selfservice_workflow_pkg.get_transaction(p_transaction_id);

    SELECT hat.* 
    INTO   l_hrapi
    FROM   hr_api_transactions hat
    WHERE  hat.transaction_id  = p_transaction_id;

    OPEN  csr_trans_to_obj;
    FETCH csr_trans_to_obj INTO l_obj;
    CLOSE csr_trans_to_obj;
    
    --
    -- Populate recruitment activity rows with 'recruitment activity for' rows
    --
    IF l_obj IS NOT NULL AND
       l_obj.recruitment_activities IS NOT NULL AND
       l_obj.recruitment_activities.count > 0 THEN
       FOR csr_rec IN csr_rec_for LOOP
         FOR i IN l_obj.recruitment_activities.first..l_obj.recruitment_activities.last LOOP
           IF csr_rec.recruitment_activity_id = l_obj.recruitment_activities(i).recruitment_activity_id THEN
             l_obj.recruitment_activities(i).recruitment_activity_for_id := csr_rec.recruitment_activity_for_id;
             l_obj.recruitment_activities(i).for_object_version_number := csr_rec.object_version_number;
           END IF;
         END LOOP;
       END LOOP;
    END IF;
    
    post_processing(l_obj);
    
  END IF;
   
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_obj;

END trans_to_obj;

--
-- ------------------------------------------------------------------------ *
-- Name    : base_to_obj
-- ------------------------------------------------------------------------ *
FUNCTION base_to_obj 
  (p_vacancy_id IN number) RETURN xxas_per_all_vacancy
IS

  c_method CONSTANT VARCHAR2(30) := 'base_to_obj';
  l_obj             xxas_per_all_vacancy;
  
  --
  -- Creates an Object from the base table
  --
  CURSOR csr_base_to_obj IS
  SELECT xxas_per_all_vacancy
           (v.vacancy_id
           ,xxas_vacancy_pkg.get_href
             (v.manager_id, v.vacancy_id) --href          
           ,v.name
           ,xxas_util_pkg.flag_to_boolean(xxas_com_sec_pkg.test_instance('XXAS_DATA_VAC_UPD', to_char(v.vacancy_id)))  --edit_allowed
           ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_EDIT') --edit_label
           ,xxas_util_pkg.flag_to_boolean(xxas_com_sec_pkg.test_instance('XXAS_DATA_VAC_DEL', to_char(v.vacancy_id)))  --delete_allowed
           ,xxas_com_msg_pkg.get_string('XXAS_RB_COM_DEL') --delete_label
           ,0 --num_total
           ,0 --num_active
           ,0 --num_term
           ,0 --num_interview1
           ,0 --num_interview2
           ,0 --num_offer
           ,0 --num_accepted
           ,0 --num_not_set
           ,0 --num_no
           ,0 --num_unsure
           ,0 --num_yes
           ,xxas_vacancy_pkg.get_view_schema_href
             (v.manager_id, v.vacancy_id) --view_schema_href
           ,xxas_vacancy_pkg.get_form_schema_href
             (v.manager_id, v.vacancy_id) --form_schema_href
           ,xxas_vacancy_pkg.get_applicants_href(v.vacancy_id) --applicants_href
           ,xxas_vacancy_pkg.get_applicants_export_href(v.vacancy_id) --applicants_export_href
           ,xxas_vacancy_pkg.get_offers_href(v.vacancy_id) --offers_href
           ,xxas_vacancy_pkg.get_ad_hoc_comms_href(v.vacancy_id) --ad_hoc_comms_href
           ,null --transaction_id
           ,'UPDATE' --transaction_type
           ,null --hr_api_status_code
           ,null --workflow_status_code
           ,null --workflow_status_meaning
           ,null --item_type
           ,null --item_key
           ,v.business_group_id
           ,bg.name
           ,v.position_id
           ,pos.name
           ,v.job_id
           ,pj.name
           ,v.grade_id
           ,grd.name
           ,v.organization_id
           ,org.name
           ,v.requisition_id
           ,r.object_version_number
           ,v.people_group_id
           ,v.location_id
           ,loc.location_code
           ,v.recruiter_id
           ,v.date_from
           ,v.date_to
           ,v.description
           ,v.number_of_openings
           ,v.status
           ,xxas_util_pkg.decode_lookup('VACANCY_STATUS', v.status)
           ,'#499324' --status_color
           ,v.request_id
           ,v.program_application_id
           ,v.program_id
           ,v.program_update_date
           ,v.attribute_category
           ,v.attribute1
           ,v.attribute2
           ,v.attribute3
           ,v.attribute4
           ,v.attribute5
           ,v.attribute6
           ,v.attribute7
           ,v.attribute8
           ,v.attribute9
           ,v.attribute10
           ,v.attribute11
           ,v.attribute12
           ,v.attribute13
           ,v.attribute14
           ,v.attribute15
           ,v.attribute16
           ,v.attribute17
           ,v.attribute18
           ,v.attribute19
           ,v.attribute20
           ,v.last_update_date
           ,v.last_updated_by
           ,v.last_update_login
           ,v.created_by
           ,v.creation_date
           ,v.vacancy_category
           ,xxas_util_pkg.decode_lookup('VACANCY_CATEGORY', v.vacancy_category)
           ,v.budget_measurement_type
           ,xxas_util_pkg.decode_lookup('BUDGET_MEASUREMENT_TYPE', v.budget_measurement_type)
           ,v.budget_measurement_value
           ,v.manager_id
           ,v.security_method
           ,v.object_version_number
           ,v.assessment_id
           ,v.primary_posting_id
           ,null --comments
           ,null --comments_to_approver
           ,xxas_util_pkg.flag_to_boolean(ipc.display_manager_info)
           ,xxas_util_pkg.flag_to_boolean(ipc.display_recruiter_info)
           ,ipc.name
           ,ipc.org_name
           ,ipc.org_description
           ,ipc.job_title
           ,ipc.brief_description
           ,ipc.detailed_description
           ,ipc.job_requirements
           ,ipc.additional_details
           ,ipc.how_to_apply
           ,ipc.image_url
           ,ipc.image_url_alt
           ,ipc.benefit_info
           ,ipc.attribute_category
           ,ipc.attribute1
           ,ipc.attribute2
           ,ipc.attribute3
           ,ipc.attribute4
           ,ipc.attribute5
           ,ipc.attribute6
           ,ipc.attribute7
           ,ipc.attribute8
           ,ipc.attribute9
           ,ipc.attribute10
           ,ipc.attribute11
           ,ipc.attribute12
           ,ipc.attribute13
           ,ipc.attribute14
           ,ipc.attribute15
           ,ipc.attribute16
           ,ipc.attribute17
           ,ipc.attribute18
           ,ipc.attribute19
           ,ipc.attribute20
           ,ipc.attribute21
           ,ipc.attribute22
           ,ipc.attribute23
           ,ipc.attribute24
           ,ipc.attribute25
           ,ipc.attribute26
           ,ipc.attribute27
           ,ipc.attribute28
           ,ipc.attribute29
           ,ipc.attribute30
           ,ipc.last_update_date
           ,ipc.last_updated_by
           ,ipc.last_update_login
           ,ipc.created_by
           ,ipc.creation_date
           ,ipc.object_version_number
           ,ipc.recruiter_full_name
           ,ipc.recruiter_email
           ,ipc.recruiter_work_telephone
           ,ipc.manager_full_name
           ,ipc.manager_email
           ,ipc.manager_work_telephone
           ,isc.search_criteria_id
           ,isc.employee
           ,isc.contractor
           ,isc.employment_category
           ,xxas_util_pkg.decode_lookup('IRC_EMP_CAT', isc.employment_category)
           ,isc.travel_percentage
           ,xxas_util_pkg.decode_lookup
             ('IRC_TRAVEL_PERCENTAGE'
             ,fnd_number.number_to_canonical(isc.travel_percentage))
           ,isc.min_salary
           ,isc.max_salary
           ,isc.salary_currency
           ,isc.salary_period
           ,xxas_util_pkg.decode_lookup('PAY_BASIS', isc.salary_period)
           ,isc.professional_area
           ,xxas_util_pkg.decode_lookup('IRC_PROFESSIONAL_AREA', isc.professional_area)
           ,isc.work_at_home
           ,xxas_util_pkg.decode_lookup('IRC_WORK_AT_HOME', isc.work_at_home)
           ,isc.min_qual_level
           ,xxas_qualifications_pkg.decode_qualification_type(isc.min_qual_level)
           ,isc.max_qual_level
           ,xxas_qualifications_pkg.decode_qualification_type(isc.max_qual_level)
           ,isc.attribute_category
           ,isc.attribute1
           ,isc.attribute2
           ,isc.attribute3
           ,isc.attribute4
           ,isc.attribute5
           ,isc.attribute6
           ,isc.attribute7
           ,isc.attribute8
           ,isc.attribute9
           ,isc.attribute10
           ,isc.attribute11
           ,isc.attribute12
           ,isc.attribute13
           ,isc.attribute14
           ,isc.attribute15
           ,isc.attribute16
           ,isc.attribute17
           ,isc.attribute18
           ,isc.attribute19
           ,isc.attribute20
           ,isc.attribute21
           ,isc.attribute22
           ,isc.attribute23
           ,isc.attribute24
           ,isc.attribute25
           ,isc.attribute26
           ,isc.attribute27
           ,isc.attribute28
           ,isc.attribute29
           ,isc.attribute30
           ,isc.object_version_number
           ,ppg.id_flex_num
           ,ppg.segment1
           ,ppg.segment2
           ,ppg.segment3
           ,ppg.segment4
           ,ppg.segment5
           ,ppg.segment6
           ,ppg.segment7
           ,ppg.segment8
           ,ppg.segment9
           ,ppg.segment10
           ,ppg.segment11
           ,ppg.segment12
           ,ppg.segment13
           ,ppg.segment14
           ,ppg.segment15
           ,ppg.segment16
           ,ppg.segment17
           ,ppg.segment18
           ,ppg.segment19
           ,ppg.segment20
           ,ppg.segment21
           ,ppg.segment22
           ,ppg.segment23
           ,ppg.segment24
           ,ppg.segment25
           ,ppg.segment26
           ,ppg.segment27
           ,ppg.segment28
           ,ppg.segment29
           ,ppg.segment30
           ,cast
             (multiset
               (SELECT xxas_irc_variable_comp_ele
                         (ivce.vacancy_id
                         ,ivce.variable_comp_lookup
                         ,hrlvc.meaning
                         ,ivce.object_version_number
                         ,ivce.last_update_date
                         ,ivce.last_updated_by
                         ,ivce.last_update_login
                         ,ivce.created_by
                         ,ivce.creation_date)
                FROM   irc_variable_comp_elements ivce
                      ,hr_lookups hrlvc
                WHERE  ivce.vacancy_id = v.vacancy_id
                AND    hrlvc.lookup_type = 'IRC_VARIABLE_COMP_ELEMENT'
                AND    hrlvc.lookup_code = ivce.variable_comp_lookup
             )
             AS xxas_irc_variable_comp_ele_tbl)
           ,cast
             (multiset
               (SELECT xxas_per_competence_ele
                         (pce.competence_element_id
                         ,pce.object_version_number
                         ,pce.type
                         ,hrlct.meaning
                         ,pce.competence_id
                         ,pc.name
                         ,pce.proficiency_level_id
                         ,prlmin.name
                         ,pce.high_proficiency_level_id
                         ,prlmax.name
                         ,pce.mandatory
                         ,hrlmand.meaning
                         ,pce.attribute_category
                         ,pce.attribute1
                         ,pce.attribute2
                         ,pce.attribute3
                         ,pce.attribute4
                         ,pce.attribute5
                         ,pce.attribute6
                         ,pce.attribute7
                         ,pce.attribute8
                         ,pce.attribute9
                         ,pce.attribute10
                         ,pce.attribute11
                         ,pce.attribute12
                         ,pce.attribute13
                         ,pce.attribute14
                         ,pce.attribute15
                         ,pce.attribute16
                         ,pce.attribute17
                         ,pce.attribute18
                         ,pce.attribute19
                         ,pce.attribute20
                         ,pce.last_update_date
                         ,pce.last_updated_by
                         ,pce.last_update_login
                         ,pce.created_by
                         ,pce.creation_date)
                FROM   per_competence_elements pce
                      ,hr_lookups hrlct
                      ,hr_lookups hrlmand
                      ,per_competences pc
                      ,per_rating_levels_vl prlmin
                      ,per_rating_levels_vl prlmax
                      ,per_rating_levels_vl prldes
                WHERE  pce.object_id = v.vacancy_id
                AND    pce.object_name = 'VACANCY'
                AND    hrlct.lookup_type = 'COMPETENCE_ELEMENT_TYPE'
                AND    hrlct.lookup_code = pce.type
                AND    hrlmand.lookup_type = 'IRC_SKILL_DESIRABILITY'
                AND    hrlmand.lookup_code = pce.mandatory
                AND    pce.competence_id = pc.competence_id(+)
                AND    pce.proficiency_level_id = prlmin.rating_level_id(+)
                AND    pce.high_proficiency_level_id = prlmax.rating_level_id(+)
                AND    pce.rating_level_id = prldes.rating_level_id(+)
             )
             AS xxas_per_competence_ele_tbl)
           ,cast
             (multiset
               (SELECT xxas_irc_agency_vacancy
                         (iav.agency_vacancy_id
                         ,iav.agency_id
                         ,pov.vendor_name
                         ,iav.vacancy_id
                         ,iav.start_date
                         ,iav.end_date
                         ,iav.max_allowed_applicants
                         ,xxas_util_pkg.flag_to_boolean(iav.manage_applicants_allowed)
                         ,iav.attribute_category
                         ,iav.attribute1
                         ,iav.attribute2
                         ,iav.attribute3
                         ,iav.attribute4
                         ,iav.attribute5
                         ,iav.attribute6
                         ,iav.attribute7
                         ,iav.attribute8
                         ,iav.attribute9
                         ,iav.attribute10
                         ,iav.attribute11
                         ,iav.attribute12
                         ,iav.attribute13
                         ,iav.attribute14
                         ,iav.attribute15
                         ,iav.attribute16
                         ,iav.attribute17
                         ,iav.attribute18
                         ,iav.attribute19
                         ,iav.attribute20
                         ,iav.attribute21
                         ,iav.attribute22
                         ,iav.attribute23
                         ,iav.attribute24
                         ,iav.attribute25
                         ,iav.attribute26
                         ,iav.attribute27
                         ,iav.attribute28
                         ,iav.attribute29
                         ,iav.attribute30
                         ,iav.object_version_number
                         ,iav.last_update_date
                         ,iav.last_updated_by
                         ,iav.last_update_login
                         ,iav.created_by
                         ,iav.creation_date)
                FROM   irc_agency_vacancies iav
                      ,po_vendors pov
                WHERE  iav.vacancy_id = v.vacancy_id
                AND    iav.agency_id = pov.vendor_id
             )
             AS xxas_irc_agency_vacancy_tbl)
           ,cast
             (multiset
               (SELECT xxas_irc_rec_team_member
                         (itm.rec_team_member_id
                         ,itm.person_id
                         ,itm.party_id
                         ,itm.vacancy_id
                         ,itm.job_id
                         ,itm.start_date
                         ,itm.end_date
                         ,itm.update_allowed
                         ,itm.delete_allowed
                         ,itm.object_version_number
                         ,itm.last_update_date
                         ,itm.last_updated_by
                         ,itm.last_update_login
                         ,itm.created_by
                         ,itm.creation_date)
                FROM   irc_rec_team_members itm
                WHERE  itm.vacancy_id = v.vacancy_id
             )
             AS xxas_irc_rec_team_member_tbl)
           ,cast
             (multiset
               (SELECT xxas_per_rec_activity
                         (pra.recruitment_activity_id
                         ,praf.recruitment_activity_for_id
                         ,v.vacancy_id
                         ,pra.date_start
                         ,pra.date_end
                         ,pra.name
                         ,pra.posting_content_id
                         ,pra.recruiting_site_id
                         ,irs.site_name
                         ,pra.object_version_number
                         ,praf.object_version_number
                         ,pra.last_update_date
                         ,pra.last_updated_by
                         ,pra.last_update_login
                         ,pra.created_by
                         ,pra.creation_date)
                FROM   per_recruitment_activity_for praf
                      ,per_recruitment_activities pra
                      ,irc_all_recruiting_sites_vl irs
                WHERE  praf.vacancy_id = v.vacancy_id
                AND    praf.recruitment_activity_id = pra.recruitment_activity_id
                AND    pra.recruiting_site_id = irs.recruiting_site_id
             )
             AS xxas_per_rec_activity_tbl)
           ,xxas_vacancy_task_tbl()
           ,cast
             (multiset
               (SELECT a.attached_document_id
                         ,a.seq_num sequence_number
                         ,a.entity_name
                         ,a.pk1_value
                         ,a.pk2_value
                         ,a.pk3_value
                         ,a.pk4_value
                         ,a.pk5_value
                         ,c.name category_code
                         ,c.user_name category_name
                         ,d.document_id
                         ,d.file_name
                         ,d.title
                         ,d.description
                         ,l.file_id
                         ,l.file_content_type mime_type
                         ,l.file_format lob_format
                         ,fnd_gfm.construct_download_url(fnd_web_config.gfm_agent, l.file_id) file_url
                         ,null wp_link
						 ,a.creation_date
                         ,xxas_wf_role_util_pkg.get_display_name_from_user(a.created_by) created_by
                         ,a.last_update_date
                         ,xxas_wf_role_util_pkg.get_display_name_from_user(a.last_updated_by) last_updated_by
                FROM   fnd_attached_documents a
                      ,fnd_documents_vl d
                      ,fnd_lobs l
                      ,fnd_document_categories_vl c
                      ,fnd_document_datatypes_vl t
                WHERE  t.name = 'FILE'
                AND    d.media_id = l.file_id
                AND    a.document_id = d.document_id
                AND    a.category_id = c.category_id
                AND    d.datatype_id = t.datatype_id
                AND    a.entity_name IN('IRC_EXT_VAC_APPROVED', 'IRC_INT_VAC_APPROVED')
                AND    v.vacancy_id IS NOT NULL 
                AND    a.pk1_value IS NOT NULL 
                AND    a.pk1_value = to_char(v.vacancy_id)
             )
             AS xxas_attachment_table)
           ,xxas_approver_tbl()
           ) vacancy
  FROM   per_all_vacancies v
        ,per_requisitions r
        ,irc_posting_contents_vl ipc
        ,irc_search_criteria isc
        ,per_jobs_vl pj
        ,hr_all_positions_f_vl pos
        ,hr_locations_all_tl loc /* Selecting from _tl because geometry causing issues on 10g */
        ,hr_all_organization_units_vl org
        ,per_business_groups_perf bg
        ,per_grades_vl grd
        ,pay_people_groups ppg
  WHERE  v.vacancy_id = p_vacancy_id
  AND    v.requisition_id = r.requisition_id
  AND    v.primary_posting_id = ipc.posting_content_id(+)
  AND    v.vacancy_id = isc.object_id(+)
  AND    isc.object_type(+) = 'VACANCY'
  AND    v.job_id = pj.job_id(+)
  AND    v.position_id = pos.position_id(+)
  AND    trunc(sysdate) BETWEEN
         pos.effective_start_date(+) AND pos.effective_end_date(+)
  AND    v.location_id = loc.location_id(+)
  AND    loc.language(+) = userenv('LANG')
  AND    v.organization_id = org.organization_id(+)
  AND    v.business_group_id = bg.business_group_id(+)
  AND    v.grade_id = grd.grade_id(+)
  AND    v.people_group_id = ppg.people_group_id(+);

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_vacancy_id);
  END IF;
  
  IF p_vacancy_id IS NOT NULL THEN
    OPEN  csr_base_to_obj;
    FETCH csr_base_to_obj INTO l_obj;
    CLOSE csr_base_to_obj;
    post_processing(l_obj);
  END IF;
   
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_obj;

END base_to_obj;

--
-- ------------------------------------------------------------------------ *
-- Name    : vacancy_to_obj
-- ------------------------------------------------------------------------ *
FUNCTION vacancy_to_obj 
  (p_vacancy_id IN number) RETURN xxas_per_all_vacancy
IS

  c_method CONSTANT VARCHAR2(30) := 'vacancy_to_obj';
  l_transaction_id  number;
  l_obj             xxas_per_all_vacancy;

  CURSOR csr_trans_id IS
  SELECT hrapi.transaction_id
  FROM   hr_api_transactions hrapi
  WHERE  hrapi.transaction_ref_id = p_vacancy_id
  AND    hrapi.transaction_ref_table = 'PER_ALL_VACANCIES'
  AND    hrapi.status NOT IN ('D', 'N', 'W', 'ACTIVE', 'AC');
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  IF p_vacancy_id IS NOT NULL THEN
    OPEN  csr_trans_id;
    FETCH csr_trans_id INTO l_transaction_id;
    CLOSE csr_trans_id;
    IF l_transaction_id IS NOT NULL THEN
      l_obj := trans_to_obj(l_transaction_id);
    ELSE
      l_obj := base_to_obj(p_vacancy_id);
    END IF;
  END IF;
   
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_obj;

END vacancy_to_obj;

--
-- ------------------------------------------------------------------------ *
-- Name    : xml_to_obj
-- ------------------------------------------------------------------------ *
FUNCTION xml_to_obj 
  (p_xml IN XMLType) RETURN xxas_per_all_vacancy
IS

  c_method CONSTANT VARCHAR2(30) := 'xml_to_obj';
  l_obj             xxas_per_all_vacancy;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;

  IF p_xml IS NOT NULL THEN
    xxas_xml_util_pkg.pre_conversion;
    p_xml.toObject(l_obj);
    xxas_xml_util_pkg.post_conversion;
  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_obj;

END xml_to_obj;

--
-- ------------------------------------------------------------------------ *
-- Name    : obj_to_xml
-- ------------------------------------------------------------------------ *
FUNCTION obj_to_xml 
  (p_vacancy IN xxas_per_all_vacancy) RETURN XMLType
IS

  c_method CONSTANT VARCHAR2(30) := 'obj_to_xml';
  l_xml             XMLType;
  
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  
  IF p_vacancy IS NOT NULL THEN
    l_xml := xxas_xml_util_pkg.obj_to_xml
               (p_object              => AnyData.ConvertObject(p_vacancy)
               ,p_anydata_to_obj_func => g_PACKAGE_NAME || '.anydata_to_obj');
  END IF;
   
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_xml;

END obj_to_xml;

--
-- ------------------------------------------------------------------------ *
-- Name    : default_rec_team
-- ------------------------------------------------------------------------ *
PROCEDURE default_rec_team 
  (p_xml IN OUT XMLType)
IS

  c_method CONSTANT VARCHAR2(30) := 'default_rec_team';

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  p_xml := XMLType('<newroot/>');
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  
END default_rec_team;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_per_vacancy
-- ------------------------------------------------------------------------ *
FUNCTION create_per_vacancy 
  (p_vacancy IN xxas_per_all_vacancy) RETURN number
IS

  c_method CONSTANT VARCHAR2(30) := 'create_per_vacancy';
  l_vacancy_id      NUMBER;
  l_ovn             NUMBER;
  l_inv_pos_warn    BOOLEAN;
  l_inv_job_warn    BOOLEAN;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  per_vacancy_api.create_vacancy
    (p_validate                  => false
    ,p_effective_date            => trunc(sysdate)
    ,p_requisition_id            => p_vacancy.requisition_id
    ,p_date_from                 => p_vacancy.date_from
    ,p_name                      => p_vacancy.vacancy_name
    ,p_security_method           => p_vacancy.security_method
    ,p_business_group_id         => p_vacancy.business_group_id
    ,p_position_id               => p_vacancy.position_id
    ,p_job_id                    => p_vacancy.job_id
    ,p_grade_id                  => p_vacancy.grade_id
    ,p_organization_id           => p_vacancy.organization_id
    ,p_people_group_id           => p_vacancy.people_group_id
    ,p_location_id               => p_vacancy.location_id
    ,p_recruiter_id              => p_vacancy.recruiter_id
    ,p_date_to                   => p_vacancy.date_to
    ,p_description               => p_vacancy.description
    ,p_number_of_openings        => p_vacancy.number_of_openings
    ,p_status                    => p_vacancy.status_code
    ,p_budget_measurement_type   => p_vacancy.budget_type_code
    ,p_budget_measurement_value  => p_vacancy.budget_value
    ,p_vacancy_category          => p_vacancy.vacancy_category_code
    ,p_manager_id                => p_vacancy.manager_id
    ,p_primary_posting_id        => p_vacancy.primary_posting_id
    ,p_assessment_id             => p_vacancy.assessment_id
    ,p_attribute_category        => p_vacancy.attribute_category
    ,p_attribute1                => p_vacancy.attribute1
    ,p_attribute2                => p_vacancy.attribute2
    ,p_attribute3                => p_vacancy.attribute3
    ,p_attribute4                => p_vacancy.attribute4
    ,p_attribute5                => p_vacancy.attribute5
    ,p_attribute6                => p_vacancy.attribute6
    ,p_attribute7                => p_vacancy.attribute7
    ,p_attribute8                => p_vacancy.attribute8
    ,p_attribute9                => p_vacancy.attribute9
    ,p_attribute10               => p_vacancy.attribute10
    ,p_attribute11               => p_vacancy.attribute11
    ,p_attribute12               => p_vacancy.attribute12
    ,p_attribute13               => p_vacancy.attribute13
    ,p_attribute14               => p_vacancy.attribute14
    ,p_attribute15               => p_vacancy.attribute15
    ,p_attribute16               => p_vacancy.attribute16
    ,p_attribute17               => p_vacancy.attribute17
    ,p_attribute18               => p_vacancy.attribute18
    ,p_attribute19               => p_vacancy.attribute19
    ,p_attribute20               => p_vacancy.attribute20
    ,p_object_version_number     => l_ovn
    ,p_vacancy_id                => l_vacancy_id
    ,p_inv_pos_grade_warning     => l_inv_pos_warn
    ,p_inv_job_grade_warning     => l_inv_job_warn);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_vacancy_id;
  
END create_per_vacancy;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_agency_vacancy
-- ------------------------------------------------------------------------ *
FUNCTION create_agency_vacancy 
  (p_vacancy_id     IN number
  ,p_agency_vacancy IN xxas_irc_agency_vacancy) RETURN NUMBER
IS

  c_method CONSTANT   VARCHAR2(30) := 'create_agency_vacancy';
  l_agency_vacancy_id NUMBER;
  l_ovn               NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  irc_agency_vacancies_api.create_agency_vacancy
    (p_validate                  => false
    ,p_agency_id                 => p_agency_vacancy.agency_id
    ,p_vacancy_id                => p_vacancy_id
    ,p_start_date                => p_agency_vacancy.start_date
    ,p_end_date                  => p_agency_vacancy.end_date
    ,p_max_allowed_applicants    => p_agency_vacancy.max_allowed_applicants
    ,p_manage_applicants_allowed => p_agency_vacancy.manage_applicants_allowed
    ,p_attribute_category        => p_agency_vacancy.attribute_category
    ,p_attribute1                => p_agency_vacancy.attribute1
    ,p_attribute2                => p_agency_vacancy.attribute2
    ,p_attribute3                => p_agency_vacancy.attribute3
    ,p_attribute4                => p_agency_vacancy.attribute4
    ,p_attribute5                => p_agency_vacancy.attribute5
    ,p_attribute6                => p_agency_vacancy.attribute6
    ,p_attribute7                => p_agency_vacancy.attribute7
    ,p_attribute8                => p_agency_vacancy.attribute8
    ,p_attribute9                => p_agency_vacancy.attribute9
    ,p_attribute10               => p_agency_vacancy.attribute10
    ,p_attribute11               => p_agency_vacancy.attribute11
    ,p_attribute12               => p_agency_vacancy.attribute12
    ,p_attribute13               => p_agency_vacancy.attribute13
    ,p_attribute14               => p_agency_vacancy.attribute14
    ,p_attribute15               => p_agency_vacancy.attribute15
    ,p_attribute16               => p_agency_vacancy.attribute16
    ,p_attribute17               => p_agency_vacancy.attribute17
    ,p_attribute18               => p_agency_vacancy.attribute18
    ,p_attribute19               => p_agency_vacancy.attribute19
    ,p_attribute20               => p_agency_vacancy.attribute20
    ,p_attribute21               => p_agency_vacancy.attribute21
    ,p_attribute22               => p_agency_vacancy.attribute22
    ,p_attribute23               => p_agency_vacancy.attribute23
    ,p_attribute24               => p_agency_vacancy.attribute24
    ,p_attribute25               => p_agency_vacancy.attribute25
    ,p_attribute26               => p_agency_vacancy.attribute26
    ,p_attribute27               => p_agency_vacancy.attribute27
    ,p_attribute28               => p_agency_vacancy.attribute28
    ,p_attribute29               => p_agency_vacancy.attribute29
    ,p_attribute30               => p_agency_vacancy.attribute30
    ,p_object_version_number     => l_ovn
    ,p_agency_vacancy_id         => l_agency_vacancy_id);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_agency_vacancy_id;
  
END create_agency_vacancy;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_agency_vacancies
-- ------------------------------------------------------------------------ *
PROCEDURE create_agency_vacancies
  (p_vacancy_id       IN     number
  ,p_agency_vacancies IN OUT xxas_irc_agency_vacancy_tbl)
IS

  c_method CONSTANT   VARCHAR2(30) := 'create_agency_vacancies';
  l_agency_vacancy_id NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  --
  -- Create all agency vacancies
  --
  IF p_agency_vacancies IS NOT NULL AND
     p_agency_vacancies.count > 0 THEN
    FOR i IN p_agency_vacancies.first..p_agency_vacancies.last LOOP
      l_agency_vacancy_id := create_agency_vacancy
        (p_vacancy_id     => p_vacancy_id
        ,p_agency_vacancy => p_agency_vacancies(i));
      p_agency_vacancies(i).agency_vacancy_id := l_agency_vacancy_id;
    END LOOP;
  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  
END create_agency_vacancies;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_vacancy
-- ------------------------------------------------------------------------ *
FUNCTION create_vacancy 
  (p_vacancy IN xxas_per_all_vacancy) RETURN number
IS

  c_method CONSTANT VARCHAR2(30) := 'create_vacancy';
  l_vacancy_id      NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;

  --
  -- Call the Oracle APIs to create the new vacancy
  --
  l_vacancy_id := create_per_vacancy(p_vacancy);
   
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_vacancy_id;

END create_vacancy;

--
-- ------------------------------------------------------------------------ *
-- Name    : launch_workflow
-- ------------------------------------------------------------------------ *
PROCEDURE launch_workflow 
  (p_vacancy     IN            xxas_per_all_vacancy
  ,p_item_key       OUT NOCOPY varchar2
  ,p_activity_id    OUT NOCOPY varchar2)
IS

  c_method CONSTANT     varchar2(30) := 'launch_workflow';
  l_current_per_display varchar2(1000);
  
  CURSOR csr_person_name
    (cp_person_id IN number) IS
  SELECT papf.full_name
  FROM   per_all_people_f papf
  WHERE  cp_person_id IS NOT NULL
  AND    papf.person_id = cp_person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;
  
BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  OPEN  csr_person_name(fnd_global.employee_id);
  FETCH csr_person_name INTO l_current_per_display;
  CLOSE csr_person_name;
  l_current_per_display := l_current_per_display || '(' || p_vacancy.vacancy_name || ')';
  
  xxas_selfservice_workflow_pkg.hr_generic_approval_prc_launch
    (p_transaction_id              => p_vacancy.transaction_id
    ,p_item_type                   => p_vacancy.item_type
    ,p_process_name                => g_PROCESS_NAME
    ,p_process_display_name        => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                                xxas_com_msg_pkg.get_string('PER|IRC_VACANCY_APPROVAL_NEW')
                                           WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                                 xxas_com_msg_pkg.get_string('PER|IRC_VACANCY_APPROVAL_UPDATE')
                                      END
    ,p_ame_application_id          => g_AME_APP_ID
    ,p_ame_transaction_type        => g_AME_TRANS_TYPE
    ,p_review_template_rn          => 'IRC_VACANCY_APPROVE_NOTIF_STK'
    ,p_ntf_sub_fnd_msg             => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                           'IRC_VACANCY_APPROVAL_NEW'
                                      WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                           'IRC_VACANCY_APPROVAL_UPDATE'
                                      END
    ,p_current_person_display_name => l_current_per_display
    ,p_comments_to_approver        => p_vacancy.comments_to_approver
    ,x_item_key                    => p_item_key
    ,x_activity_id                 => p_activity_id);
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

END launch_workflow;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_hrapi_transaction
-- ------------------------------------------------------------------------ *
PROCEDURE create_hrapi_transaction 
  (p_vacancy IN xxas_per_all_vacancy)
IS

  c_method CONSTANT     varchar2(30) := 'create_hrapi_transaction';
  l_transaction_id      number;
  l_transaction_step_id number;
  l_step_ovn            number;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;
  

  hr_trn_ins.set_base_key_value
   (p_transaction_id => p_vacancy.transaction_id);
  
  hr_trn_ins.ins
   (p_transaction_id               => l_transaction_id
   ,p_creator_person_id            => fnd_global.employee_id
   ,p_transaction_privilege        => 'PRIVATE'
   ,p_product_code                 => 'IRC'
   ,p_url                          => null
   ,p_status                       => p_vacancy.hr_api_status_code
   ,p_section_display_name         => null
   ,p_function_id                  => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                                fnd_function.get_function_id('IRC_VAC_DETS_NEW')
                                           WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                                fnd_function.get_function_id('IRC_VAC_POST_EDIT')
                                      END
   ,p_transaction_ref_table        => 'PER_ALL_VACANCIES'
   ,p_transaction_ref_id           => p_vacancy.vacancy_id
   ,p_transaction_type             => 'WF'
   ,p_assignment_id                => null
   ,p_api_addtnl_info              => p_vacancy.vacancy_name
   ,p_selected_person_id           => null --TODO, try manager id
   ,p_item_type                    => p_vacancy.item_type
   ,p_item_key                     => p_vacancy.item_key
   ,p_transaction_effective_date   => trunc(sysdate)
   ,p_process_name                 => g_PROCESS_NAME
   ,p_plan_id                      => null
   ,p_rptg_grp_id                  => null
   ,p_effective_date_option        => null
   ,p_validate                     => false
   ,p_creator_role                 => 'PER:' || fnd_global.employee_id
   ,p_last_update_role             => 'PER:' || fnd_global.employee_id
   ,p_parent_transaction_id        => null
   ,p_relaunch_function            => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                                'IRC_VAC_NEW_SFL_LAUNCH'
                                           WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                                'IRC_VAC_EDIT_SFL_LAUNCH'
                                      END
   ,p_transaction_group            => 'IRC'
   ,p_transaction_identifier       => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                                'CREATE_VACANCY'
                                           WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                                'EDIT_VACANCY'
                                      END
   ,p_transaction_document         => obj_to_hrapi_doc(p_vacancy).getCLOBVal());

  --
  -- Create a row for the posting content step
  --
  hr_trs_ins.ins
    (p_transaction_step_id          => l_transaction_step_id
    ,p_transaction_id               => p_vacancy.transaction_id
    ,p_api_name                     => 'irc_posting_content_swi.process_api'
    ,p_processing_order             => 0
    ,p_item_type                    => p_vacancy.item_type
    ,p_creator_person_id            => fnd_global.employee_id
    ,p_update_person_id             => -1
    ,p_object_version_number        => l_step_ovn
    ,p_validate                     => false
    ,p_object_type                  => 'ENTITY'
    ,p_object_name                  => 'oracle.apps.irc.schema.server.IrcPostingContentsVlEO'
    ,p_object_identifier            => xxas_selfservice_workflow_pkg.get_hex_key
                                         (p_vacancy.primary_posting_id)
    ,p_object_state                 => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                                 '0'
                                            WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                                 '2'
                                       END
    ,p_pk1                          => p_vacancy.primary_posting_id);
   
  --
  -- Create a row for the requisitions step
  --
  hr_trs_ins.ins
    (p_transaction_step_id          => l_transaction_step_id
    ,p_transaction_id               => p_vacancy.transaction_id
    ,p_api_name                     => 'per_requisitions_swi.process_api'
    ,p_processing_order             => 0
    ,p_item_type                    => p_vacancy.item_type
    ,p_creator_person_id            => fnd_global.employee_id
    ,p_update_person_id             => -1
    ,p_object_version_number        => l_step_ovn
    ,p_validate                     => false
    ,p_object_type                  => 'ENTITY'
    ,p_object_name                  => 'oracle.apps.per.schema.server.PerRequisitionsEO'
    ,p_object_identifier            => xxas_selfservice_workflow_pkg.get_hex_key
                                         (p_vacancy.requisition_id)
    ,p_object_state                 => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                                 '0'
                                            WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                                 '2'
                                       END
    ,p_pk1                          => p_vacancy.requisition_id);  
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END create_hrapi_transaction;

--
-- ------------------------------------------------------------------------ *
-- Name    : update_hrapi_transaction
-- ------------------------------------------------------------------------ *
PROCEDURE update_hrapi_transaction 
  (p_vacancy IN xxas_per_all_vacancy)
IS

  c_method CONSTANT varchar2(30) := 'update_hrapi_transaction';

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;

  hr_trn_upd.upd
   (p_transaction_id               => p_vacancy.transaction_id
   ,p_status                       => p_vacancy.hr_api_status_code
   ,p_function_id                  => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                                fnd_function.get_function_id('IRC_VAC_DETS_NEW')
                                           WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                                fnd_function.get_function_id('IRC_VAC_POST_EDIT')
                                      END
   ,p_api_addtnl_info              => p_vacancy.vacancy_name
   ,p_selected_person_id           => null --TODO, try manager id
   ,p_item_key                     => p_vacancy.item_key
   ,p_validate                     => false
   ,p_last_update_role             => 'PER:' || fnd_global.employee_id
   ,p_relaunch_function            => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                                'IRC_VAC_NEW_SFL_LAUNCH'
                                           WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                                'IRC_VAC_EDIT_SFL_LAUNCH'
                                      END
   ,p_transaction_identifier       => CASE WHEN p_vacancy.transaction_type = 'INSERT' THEN
                                                'CREATE_VACANCY'
                                           WHEN p_vacancy.transaction_type = 'UPDATE' THEN
                                                'EDIT_VACANCY'
                                      END
   ,p_transaction_document         => obj_to_hrapi_doc(p_vacancy).getCLOBVal());

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

END update_hrapi_transaction;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_empty_vacancy
-- ------------------------------------------------------------------------ *
FUNCTION create_empty_vacancy 
  (p_person_id            IN number
  ,p_copy_from_vacancy_id IN number DEFAULT null) RETURN xxas_per_all_vacancy
IS

  c_method         CONSTANT varchar2(30) := 'create_empty_vacancy';
  l_copy_from_vac           xxas_per_all_vacancy;
  
  l_currency                varchar2(400);
  l_vacancy_name_is_auto    varchar2(400);
  l_budget_measurement_type varchar2(400);
  l_number_of_openings      number;
  l_location_default        varchar2(400);
  l_organization_default    varchar2(400);
  
  l_requisition_id          number;
  l_vacancy_id              number;
  l_transaction_id          number;
  l_posting_content_id      number;
  l_search_criteria_id      number;
  l_empty_vacancy           xxas_per_all_vacancy;
  
  l_business_group_id       number;
  l_vacancy_name            per_all_vacancies.name%TYPE;
  l_organization_id         number;
  l_location_id             number;
  l_trash_number            number;
  l_oldfile_id              number;
  l_newfile_id              number;
  i                         number := 0;
  l_file_name               varchar2(100);
  l_file_data               blob;

  l_attachments             xxas_attachment_table;
  l_competences             xxas_per_competence_ele_tbl;
  l_rec_activity            xxas_per_rec_activity_tbl;
  l_team_member             xxas_irc_rec_team_member_tbl;
  l_agency_vacancy          xxas_irc_agency_vacancy_tbl;
  l_variable_comp_ele       xxas_irc_variable_comp_ele_tbl;
  l_recruitment_activity_id per_recruitment_activities.recruitment_activity_id%TYPE;

  CURSOR csr_org_loc
    (cp_person_id IN number) IS
  SELECT paaf.organization_id
        ,paaf.location_id
  FROM   per_all_assignments_f paaf
  WHERE  cp_person_id IS NOT NULL
  AND    paaf.person_id = cp_person_id
  AND    paaf.assignment_type IN ('E', 'C')
  AND    paaf.primary_flag = 'Y'
  AND    trunc(sysdate) BETWEEN
         paaf.effective_start_date AND paaf.effective_end_date;

  -- attachment
  CURSOR csr_file_data 
  IS 
  SELECT file_name
        ,file_data
        ,file_id
  FROM   fnd_lobs
  WHERE  file_id IN (SELECT d.media_id
                     FROM   fnd_attached_documents a
                           ,fnd_documents d
                     WHERE  a.document_id = d.document_id
                     AND    a.attached_document_id IN (SELECT attached_document_id
                                                       FROM   fnd_attached_documents
                                                       WHERE  pk1_value IS NOT NULL
                                                       AND    pk1_value = to_char(p_copy_from_vacancy_id)
                                                       AND    entity_name IN('IRC_EXT_VAC', 'IRC_EXT_VAC_APPROVED'
                                                                            ,'IRC_INT_VAC', 'IRC_INT_VAC_APPROVED')));

  CURSOR csr_attachments 
  IS
  SELECT xxas_attachment_type 
           (null
           ,a.seq_num
           ,decode (a.entity_name,'IRC_EXT_VAC_APPROVED','IRC_EXT_VAC','IRC_INT_VAC') 
           ,to_char (l_vacancy_id)  
           ,null
           ,null
           ,null
           ,null
           ,c.name  
           ,c.user_name  
           ,d.document_id
           ,d.file_name
           ,d.title
           ,d.description
           ,l_newfile_id
           ,l.file_content_type 
           ,l.file_format 
           ,fnd_gfm.construct_download_url(fnd_web_config.gfm_agent, l.file_id)  
		   ,null
           ,sysdate
           ,fnd_global.user_id
           ,sysdate
           ,fnd_global.user_id
           ) cattachments
  FROM   fnd_attached_documents a
        ,fnd_documents_vl d
        ,fnd_lobs l
        ,fnd_document_categories_vl c
        ,fnd_document_datatypes_vl t
  WHERE  t.name = 'FILE'
  AND    d.media_id = l.file_id
  AND    L.FILE_ID = l_oldfile_id
  AND    a.document_id = d.document_id
  AND    a.category_id = c.category_id
  AND    d.datatype_id = t.datatype_id
  AND    a.entity_name IN('IRC_EXT_VAC', 'IRC_EXT_VAC_APPROVED'
                         ,'IRC_INT_VAC', 'IRC_INT_VAC_APPROVED')
  AND    pk1_value IS NOT NULL
  AND    pk1_value = to_char(p_copy_from_vacancy_id);


  -- recruitment activity
  CURSOR csr_per_rec_activity 
  IS
  SELECT xxas_per_rec_activity
           (per_recruitment_activities_s.nextval
           ,per_recruitment_activity_for_s.nextval
           ,l_vacancy_id
           ,trunc(sysdate)
           ,null
           ,pra.name
           ,l_posting_content_id
           ,pra.recruiting_site_id
           ,irs.site_name
           ,1--pra.object_version_number
           ,1--praf.object_version_number
           ,trunc(sysdate)
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,trunc(sysdate)
           ) rec_activity
  FROM   per_recruitment_activity_for praf
        ,per_recruitment_activities pra
        ,irc_all_recruiting_sites_vl irs
  WHERE  praf.vacancy_id              = p_copy_from_vacancy_id
  AND    praf.recruitment_activity_id = pra.recruitment_activity_id
  AND    pra.recruiting_site_id       = irs.recruiting_site_id;

  -- team member
  CURSOR csr_team_member 
  IS
  SELECT xxas_irc_rec_team_member
           (irc_rec_team_members_s.nextval
           ,itm.person_id
           ,itm.party_id
           ,l_vacancy_id
           ,itm.job_id
           ,trunc(sysdate)
           ,null
           ,itm.update_allowed
           ,itm.delete_allowed
           ,1
           ,trunc(sysdate)
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,trunc(sysdate)
           ) team_member
  FROM   irc_rec_team_members itm
  WHERE  itm.vacancy_id = p_copy_from_vacancy_id;

  -- agency vacancy
  CURSOR csr_agency_vacancy 
  IS 
  SELECT xxas_irc_agency_vacancy
           (irc_agency_vacancies_s.nextval
           ,iav.agency_id
           ,pov.vendor_name
           ,l_vacancy_id
           ,trunc(sysdate)
           ,null
           ,iav.max_allowed_applicants
           ,xxas_util_pkg.flag_to_boolean(iav.manage_applicants_allowed)
           ,iav.attribute_category
           ,iav.attribute1
           ,iav.attribute2
           ,iav.attribute3
           ,iav.attribute4
           ,iav.attribute5
           ,iav.attribute6
           ,iav.attribute7
           ,iav.attribute8
           ,iav.attribute9
           ,iav.attribute10
           ,iav.attribute11
           ,iav.attribute12
           ,iav.attribute13
           ,iav.attribute14
           ,iav.attribute15
           ,iav.attribute16
           ,iav.attribute17
           ,iav.attribute18
           ,iav.attribute19
           ,iav.attribute20
           ,iav.attribute21
           ,iav.attribute22
           ,iav.attribute23
           ,iav.attribute24
           ,iav.attribute25
           ,iav.attribute26
           ,iav.attribute27
           ,iav.attribute28
           ,iav.attribute29
           ,iav.attribute30
           ,1
           ,trunc(sysdate)
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,trunc(sysdate)
           ) agency_vacancy
  FROM   irc_agency_vacancies iav
        ,po_vendors pov
  WHERE  iav.vacancy_id = p_copy_from_vacancy_id
  AND    iav.agency_id  = pov.vendor_id;

  --irc_variable_comp_ele
  CURSOR csr_variable_comp_ele
  IS
  SELECT xxas_irc_variable_comp_ele
           (l_vacancy_id
           ,ivce.variable_comp_lookup
           ,hrlvc.meaning
           ,1
           ,trunc(sysdate)
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,trunc(sysdate)
           ) variable_comp_ele
  FROM   irc_variable_comp_elements ivce
        ,hr_lookups hrlvc
  WHERE  ivce.vacancy_id   = p_copy_from_vacancy_id
  AND    hrlvc.lookup_type = 'IRC_VARIABLE_COMP_ELEMENT'
  AND    hrlvc.lookup_code = ivce.variable_comp_lookup;


  -- for competences
  CURSOR csr_competences 
  IS
  SELECT xxas_per_competence_ele
           (per_competence_elements_s.nextval
           ,1
           ,pce.type
           ,hrlct.meaning
           ,pce.competence_id
           ,pc.name
           ,pce.proficiency_level_id
           ,prlmin.name
           ,pce.high_proficiency_level_id
           ,prlmax.name
           ,pce.mandatory
           ,hrlmand.meaning
           ,pce.attribute_category
           ,pce.attribute1
           ,pce.attribute2
           ,pce.attribute3
           ,pce.attribute4
           ,pce.attribute5
           ,pce.attribute6
           ,pce.attribute7
           ,pce.attribute8
           ,pce.attribute9
           ,pce.attribute10
           ,pce.attribute11
           ,pce.attribute12
           ,pce.attribute13
           ,pce.attribute14
           ,pce.attribute15
           ,pce.attribute16
           ,pce.attribute17
           ,pce.attribute18
           ,pce.attribute19
           ,pce.attribute20
           ,trunc(sysdate)
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,trunc(sysdate)
           ) competence
  FROM   per_competence_elements pce
        ,hr_lookups hrlct
        ,hr_lookups hrlmand
        ,per_competences pc
        ,per_rating_levels_vl prlmin
        ,per_rating_levels_vl prlmax
        ,per_rating_levels_vl prldes
  WHERE  pce.object_id                 = p_copy_from_vacancy_id
  AND    pce.object_name               = 'VACANCY'
  AND    hrlct.lookup_type             = 'COMPETENCE_ELEMENT_TYPE'
  AND    hrlct.lookup_code             = pce.type
  AND    hrlmand.lookup_type           = 'IRC_SKILL_DESIRABILITY'
  AND    hrlmand.lookup_code           = pce.mandatory
  AND    pce.competence_id             = pc.competence_id(+)
  AND    pce.proficiency_level_id      = prlmin.rating_level_id(+)
  AND    pce.high_proficiency_level_id = prlmax.rating_level_id(+)
  AND    pce.rating_level_id           = prldes.rating_level_id(+);

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_copy_from_vacancy_id);
  END IF;
  
  --
  -- Get the copy from vacancy object if one is specified
  --
  IF p_copy_from_vacancy_id IS NOT NULL THEN
    l_copy_from_vac := vacancy_to_obj(p_copy_from_vacancy_id);
  END IF;
  
  --
  -- Generate the new sequences
  --
  SELECT per_vacancies_s.nextval
        ,hr_api_transactions_s.nextval
        ,per_requisitions_s.nextval
        ,irc_posting_contents_s.nextval
        ,irc_search_criteria_s.nextval
  INTO   l_vacancy_id
        ,l_transaction_id
        ,l_requisition_id
        ,l_posting_content_id
        ,l_search_criteria_id
  FROM   dual;
  
  --
  -- Set the business group
  --
  l_business_group_id := xxas_per_pkg.get_business_group_id(p_person_id);  
  
  --
  -- Get the vacancy defaults
  --
  irc_get_vacancy_defaults.get_bg_defaults
    (p_business_group_id       => l_business_group_id
    ,p_currency                => l_currency
    ,p_vacancy_name_is_auto    => l_vacancy_name_is_auto
    ,p_budget_measurement_type => l_budget_measurement_type
    ,p_number_of_openings      => l_number_of_openings
    ,p_location_default        => l_location_default
    ,p_organization_default    => l_organization_default);
  
  --
  -- Default the vacancy name
  --
  IF l_vacancy_name_is_auto = 'N' THEN
    IF l_copy_from_vac.vacancy_name IS NOT NULL THEN
      --l_vacancy_name := substr(l_copy_from_vac.vacancy_name || ' - Copy' ,1, 400);
      l_vacancy_name := substr(l_copy_from_vac.vacancy_name ,1, 20) || '- Copy';
    ELSE
      l_vacancy_name := 'New Vacancy';
    END IF;
  ELSE
    l_vacancy_name := 'IRC' || l_vacancy_id;
  END IF;
  
  --
  -- Default the organization Id
  --
  IF l_copy_from_vac.organization_id IS NOT NULL THEN
    l_organization_id := l_copy_from_vac.organization_id;
  ELSE
    OPEN  csr_org_loc
            (cp_person_id => CASE WHEN l_organization_default = 'MANAGER'
                                  THEN p_person_id
                                  ELSE fnd_global.employee_id
                             END);
    FETCH csr_org_loc INTO l_organization_id
                          ,l_location_id;
    CLOSE csr_org_loc;
  END IF;
  
  --
  -- Default the location Id
  --
  IF l_copy_from_vac.location_id IS NOT NULL THEN
    l_location_id := l_copy_from_vac.location_id;
  ELSE
    IF l_organization_default <> l_location_default THEN
      OPEN  csr_org_loc
              (cp_person_id => CASE WHEN l_location_default = 'MANAGER'
                                    THEN p_person_id
                                    ELSE fnd_global.employee_id
                               END);
      FETCH csr_org_loc INTO l_trash_number
                            ,l_location_id;
      CLOSE csr_org_loc;
    END IF;
  END IF;


  -- For REC Activity 
  l_rec_activity := xxas_per_rec_activity_tbl();
 
  IF p_copy_from_vacancy_id IS NOT NULL THEN
    IF l_rec_activity.count > 0 THEN
      l_rec_activity.DELETE;
    END IF;

    --
    -- Get all per rec activity details
    --
    FOR csr_rec_act IN csr_per_rec_activity LOOP
      l_rec_activity.extend;
      l_rec_activity(l_rec_activity.count) := csr_rec_act.rec_activity;
      l_recruitment_activity_id := l_rec_activity(l_rec_activity.count).recruitment_activity_id;
      l_rec_activity(l_rec_activity.count).name := substr(l_vacancy_name,1,30-length(l_recruitment_activity_id)) || l_recruitment_activity_id;
    END LOOP;

  END IF;

  -- For Team Member
  l_team_member := xxas_irc_rec_team_member_tbl();

  IF p_copy_from_vacancy_id IS NOT NULL THEN
    IF l_team_member.count > 0 THEN
      l_team_member.DELETE;
    END IF;

    --
    -- Get all team details
    --
    FOR csr_rec_tm IN csr_team_member LOOP
      l_team_member.extend;
      l_team_member(l_team_member.count) := csr_rec_tm.team_member;  
    END LOOP;
  END IF;
 
  -- For Agency Vacancy
  l_agency_vacancy := xxas_irc_agency_vacancy_tbl();
 
  IF p_copy_from_vacancy_id IS NOT NULL THEN
    IF l_agency_vacancy.count > 0 THEN
      l_agency_vacancy.DELETE;
    END IF;
  
    --
    -- Get all agency vac dettails
    --
    FOR csr_rec_vac IN csr_agency_vacancy LOOP
      l_agency_vacancy.extend;
      l_agency_vacancy(l_agency_vacancy.count) := csr_rec_vac.agency_vacancy;  
    END LOOP;
  END IF;

  -- For Variable Component Element
  l_variable_comp_ele := xxas_irc_variable_comp_ele_tbl();

  IF p_copy_from_vacancy_id IS NOT NULL THEN
    IF l_variable_comp_ele.count > 0 then
      l_variable_comp_ele.DELETE;
    END IF;
    --
    -- Get all variable_comp_ele
    --
    FOR csr_rec_comp IN csr_variable_comp_ele LOOP
      l_variable_comp_ele.extend;
      l_variable_comp_ele(l_variable_comp_ele.count) := csr_rec_comp.variable_comp_ele;  
    END LOOP;
  END IF;

  -- For Competencies
  l_competences := xxas_per_competence_ele_tbl();

  IF p_copy_from_vacancy_id is not null then
    IF l_competences.count > 0 THEN
      l_competences.DELETE;
    END IF;
    --
    -- Get all competences
    --
    FOR csr_rec IN csr_competences LOOP
      l_competences.extend;
      l_competences(l_competences.count) := csr_rec.competence;  
    END LOOP;
  END IF;
 
  -- For Attachments

  l_attachments:= xxas_attachment_table();

  OPEN csr_file_data;
  LOOP
    FETCH csr_file_data 
    INTO  l_file_name
         ,l_file_data
         ,l_oldfile_id;
    EXIT WHEN csr_file_data%NOTFOUND;

    l_newfile_id  := xxas_util_pkg.insert_fnd_lob(l_file_name, l_file_data);

    l_attachments.extend;

    OPEN  csr_attachments;
    FETCH csr_attachments 
    INTO  l_attachments(l_attachments.count);
    CLOSE csr_attachments;
 
  END LOOP;

  IF csr_file_data%isopen THEN 
   CLOSE csr_file_data;
  END IF;

  --
  -- Create the empty vacancy
  --
  l_empty_vacancy := xxas_per_all_vacancy
    (vacancy_id                     => l_vacancy_id
    ,href                           => xxas_vacancy_pkg.get_href(p_person_id, l_vacancy_id)
    ,vacancy_name                   => l_vacancy_name
    ,edit_allowed                   => xxas_util_pkg.flag_to_boolean('Y')  --edit_allowed
    ,edit_label                     => xxas_com_msg_pkg.get_string('XXAS_RB_COM_EDIT')
    ,delete_allowed                 => xxas_util_pkg.flag_to_boolean('Y')  --delete_allowed
    ,delete_label                   => xxas_com_msg_pkg.get_string('XXAS_RB_COM_DEL')
    ,num_total                      => 0
    ,num_active                     => 0
    ,num_term                       => 0
    ,num_interview1                 => 0
    ,num_interview2                 => 0
    ,num_offer                      => 0
    ,num_accepted                   => 0
    ,num_not_set                    => 0
    ,num_no                         => 0
    ,num_unsure                     => 0
    ,num_yes                        => 0
    ,view_schema_href               => get_view_schema_href(p_person_id, l_vacancy_id)
    ,form_schema_href               => get_form_schema_href(p_person_id, l_vacancy_id)
    ,applicants_href                => get_applicants_href(l_vacancy_id)
    ,applicants_export_href         => get_applicants_export_href(l_vacancy_id)
    ,offers_href                    => get_offers_href(l_vacancy_id)
    ,ad_hoc_comms_href              => get_ad_hoc_comms_href(l_vacancy_id)    
    ,transaction_id                 => l_transaction_id
    ,transaction_type               => 'INSERT'
    ,hr_api_status_code             => 'S'
    ,workflow_status_code           => 'REVIEW'
    ,workflow_status_meaning        => xxas_selfservice_workflow_pkg.decode_status('REVIEW')
    ,item_type                      => g_ITEM_TYPE
    ,item_key                       => null
    ,business_group_id              => l_business_group_id
    ,business_group_name            => xxas_util_pkg.get_org_name(l_business_group_id)
    ,position_id                    => l_copy_from_vac.position_id
    ,position_name                  => l_copy_from_vac.position_name
    ,job_id                         => l_copy_from_vac.job_id
    ,job_name                       => l_copy_from_vac.job_name
    ,grade_id                       => l_copy_from_vac.grade_id
    ,grade_name                     => l_copy_from_vac.grade_name
    ,organization_id                => l_organization_id
    ,organization_name              => xxas_util_pkg.get_org_name(l_organization_id)
    ,requisition_id                 => l_requisition_id
    ,req_object_version_number      => 1
    ,people_group_id                => l_copy_from_vac.people_group_id
    ,location_id                    => l_location_id
    ,location_name                  => xxas_util_pkg.get_loc_name(l_location_id)
    ,recruiter_id                   => l_copy_from_vac.recruiter_id
    ,date_from                      => trunc(sysdate)
    ,date_to                        => null
    ,description                    => l_copy_from_vac.description
    ,number_of_openings             => nvl(l_copy_from_vac.number_of_openings, l_number_of_openings)
    ,status_code                    => 'UNAPPROVED'
    ,status_meaning                 => xxas_util_pkg.decode_lookup('VACANCY_STATUS', 'UNAPPROVED')
    ,status_color                   => '#333333'
    ,request_id                     => null
    ,program_application_id         => null
    ,program_id                     => null
    ,program_update_date            => null
    ,attribute_category             => l_copy_from_vac.attribute_category
    ,attribute1                     => l_copy_from_vac.attribute1
    ,attribute2                     => l_copy_from_vac.attribute2
    ,attribute3                     => l_copy_from_vac.attribute3
    ,attribute4                     => l_copy_from_vac.attribute4
    ,attribute5                     => l_copy_from_vac.attribute5
    ,attribute6                     => l_copy_from_vac.attribute6
    ,attribute7                     => l_copy_from_vac.attribute7
    ,attribute8                     => l_copy_from_vac.attribute8
    ,attribute9                     => l_copy_from_vac.attribute9
    ,attribute10                    => l_copy_from_vac.attribute10
    ,attribute11                    => l_copy_from_vac.attribute11
    ,attribute12                    => l_copy_from_vac.attribute12
    ,attribute13                    => l_copy_from_vac.attribute13
    ,attribute14                    => l_copy_from_vac.attribute14
    ,attribute15                    => l_copy_from_vac.attribute15
    ,attribute16                    => l_copy_from_vac.attribute16
    ,attribute17                    => l_copy_from_vac.attribute17
    ,attribute18                    => l_copy_from_vac.attribute18
    ,attribute19                    => l_copy_from_vac.attribute19
    ,attribute20                    => l_copy_from_vac.attribute20
    ,last_update_date               => sysdate
    ,last_updated_by                => fnd_global.user_id
    ,last_update_login              => fnd_global.login_id
    ,created_by                     => fnd_global.user_id
    ,creation_date                  => sysdate
    ,vacancy_category_code          => l_copy_from_vac.vacancy_category_code
    ,vacancy_category_meaning       => xxas_util_pkg.decode_lookup('VACANCY_CATEGORY', l_copy_from_vac.vacancy_category_code)
    ,budget_type_code               => nvl(l_copy_from_vac.budget_type_code, l_budget_measurement_type)
    ,budget_type_meaning            => xxas_util_pkg.decode_lookup
                                        ('BUDGET_MEASUREMENT_TYPE'
                                        ,nvl(l_copy_from_vac.budget_type_code, l_budget_measurement_type))
    ,budget_value                   => nvl(l_copy_from_vac.budget_value, l_number_of_openings)
    ,manager_id                     => p_person_id
    ,security_method                => nvl(l_copy_from_vac.security_method, 'B')
    ,object_version_number          => 1
    ,assessment_id                  => l_copy_from_vac.assessment_id
    ,primary_posting_id             => l_posting_content_id
    ,comments                       => l_copy_from_vac.comments
    ,comments_to_approver           => null
    ,display_manager_info           => nvl(l_copy_from_vac.display_manager_info, 'N')
    ,display_recruiter_info         => nvl(l_copy_from_vac.display_recruiter_info, 'N')
    ,posting_name                   => l_vacancy_name
    ,posting_org_name               => l_copy_from_vac.posting_org_name
    ,posting_org_description        => l_copy_from_vac.posting_org_description
    ,posting_job_title              => l_copy_from_vac.posting_job_title
    ,posting_brief_description      => l_copy_from_vac.posting_brief_description
    ,posting_detailed_description   => l_copy_from_vac.posting_detailed_description
    ,posting_job_requirements       => l_copy_from_vac.posting_job_requirements
    ,posting_additional_details     => l_copy_from_vac.posting_additional_details
    ,posting_how_to_apply           => l_copy_from_vac.posting_how_to_apply
    ,image_url                      => l_copy_from_vac.image_url
    ,image_url_alt                  => l_copy_from_vac.image_url_alt
    ,benefit_info                   => l_copy_from_vac.benefit_info
    ,ipc_attribute_category         => l_copy_from_vac.ipc_attribute_category
    ,ipc_attribute1                 => l_copy_from_vac.ipc_attribute1
    ,ipc_attribute2                 => l_copy_from_vac.ipc_attribute2
    ,ipc_attribute3                 => l_copy_from_vac.ipc_attribute3
    ,ipc_attribute4                 => l_copy_from_vac.ipc_attribute4
    ,ipc_attribute5                 => l_copy_from_vac.ipc_attribute5
    ,ipc_attribute6                 => l_copy_from_vac.ipc_attribute6
    ,ipc_attribute7                 => l_copy_from_vac.ipc_attribute7
    ,ipc_attribute8                 => l_copy_from_vac.ipc_attribute8
    ,ipc_attribute9                 => l_copy_from_vac.ipc_attribute9
    ,ipc_attribute10                => l_copy_from_vac.ipc_attribute10
    ,ipc_attribute11                => l_copy_from_vac.ipc_attribute11
    ,ipc_attribute12                => l_copy_from_vac.ipc_attribute12
    ,ipc_attribute13                => l_copy_from_vac.ipc_attribute13
    ,ipc_attribute14                => l_copy_from_vac.ipc_attribute14
    ,ipc_attribute15                => l_copy_from_vac.ipc_attribute15
    ,ipc_attribute16                => l_copy_from_vac.ipc_attribute16
    ,ipc_attribute17                => l_copy_from_vac.ipc_attribute17
    ,ipc_attribute18                => l_copy_from_vac.ipc_attribute18
    ,ipc_attribute19                => l_copy_from_vac.ipc_attribute19
    ,ipc_attribute20                => l_copy_from_vac.ipc_attribute20
    ,ipc_attribute21                => l_copy_from_vac.ipc_attribute21
    ,ipc_attribute22                => l_copy_from_vac.ipc_attribute22
    ,ipc_attribute23                => l_copy_from_vac.ipc_attribute23
    ,ipc_attribute24                => l_copy_from_vac.ipc_attribute24
    ,ipc_attribute25                => l_copy_from_vac.ipc_attribute25
    ,ipc_attribute26                => l_copy_from_vac.ipc_attribute26
    ,ipc_attribute27                => l_copy_from_vac.ipc_attribute27
    ,ipc_attribute28                => l_copy_from_vac.ipc_attribute28
    ,ipc_attribute29                => l_copy_from_vac.ipc_attribute29
    ,ipc_attribute30                => l_copy_from_vac.ipc_attribute30
    ,ipc_last_update_date           => sysdate
    ,ipc_last_updated_by            => fnd_global.user_id
    ,ipc_last_update_login          => fnd_global.login_id
    ,ipc_created_by                 => fnd_global.user_id
    ,ipc_creation_date              => sysdate
    ,ipc_object_version_number      => 1
    ,recruiter_full_name            => l_copy_from_vac.recruiter_full_name
    ,recruiter_email                => l_copy_from_vac.recruiter_email
    ,recruiter_work_telephone       => l_copy_from_vac.recruiter_work_telephone
    ,manager_full_name              => l_copy_from_vac.manager_full_name
    ,manager_email                  => l_copy_from_vac.manager_email
    ,manager_work_telephone         => l_copy_from_vac.manager_work_telephone
    ,search_criteria_id             => l_search_criteria_id
    ,employee_flag                  => l_copy_from_vac.employee_flag
    ,contractor_flag                => l_copy_from_vac.contractor_flag
    ,employment_category_code       => l_copy_from_vac.employment_category_code
    ,employment_category_meaning    => l_copy_from_vac.employment_category_meaning
    ,travel_percentage              => l_copy_from_vac.travel_percentage
    ,travel_percentage_meaning      => l_copy_from_vac.travel_percentage_meaning
    ,min_salary                     => l_copy_from_vac.min_salary
    ,max_salary                     => l_copy_from_vac.max_salary
    ,salary_currency                => nvl(l_copy_from_vac.salary_currency, l_currency)
    ,salary_period_code             => l_copy_from_vac.salary_period_code
    ,salary_period_meaning          => l_copy_from_vac.salary_period_meaning
    ,professional_area_code         => l_copy_from_vac.professional_area_code
    ,professional_area_meaning      => l_copy_from_vac.professional_area_meaning
    ,work_at_home_code              => l_copy_from_vac.work_at_home_code
    ,work_at_home_meaning           => l_copy_from_vac.work_at_home_meaning
    ,min_qual_level_id              => l_copy_from_vac.min_qual_level_id
    ,min_qual_level_name            => l_copy_from_vac.min_qual_level_name
    ,max_qual_level_id              => l_copy_from_vac.max_qual_level_id
    ,max_qual_level_name            => l_copy_from_vac.max_qual_level_name
    ,isc_attribute_category         => l_copy_from_vac.isc_attribute_category
    ,isc_attribute1                 => l_copy_from_vac.isc_attribute1
    ,isc_attribute2                 => l_copy_from_vac.isc_attribute2
    ,isc_attribute3                 => l_copy_from_vac.isc_attribute3
    ,isc_attribute4                 => l_copy_from_vac.isc_attribute4
    ,isc_attribute5                 => l_copy_from_vac.isc_attribute5
    ,isc_attribute6                 => l_copy_from_vac.isc_attribute6
    ,isc_attribute7                 => l_copy_from_vac.isc_attribute7
    ,isc_attribute8                 => l_copy_from_vac.isc_attribute8
    ,isc_attribute9                 => l_copy_from_vac.isc_attribute9
    ,isc_attribute10                => l_copy_from_vac.isc_attribute10
    ,isc_attribute11                => l_copy_from_vac.isc_attribute11
    ,isc_attribute12                => l_copy_from_vac.isc_attribute12
    ,isc_attribute13                => l_copy_from_vac.isc_attribute13
    ,isc_attribute14                => l_copy_from_vac.isc_attribute14
    ,isc_attribute15                => l_copy_from_vac.isc_attribute15
    ,isc_attribute16                => l_copy_from_vac.isc_attribute16
    ,isc_attribute17                => l_copy_from_vac.isc_attribute17
    ,isc_attribute18                => l_copy_from_vac.isc_attribute18
    ,isc_attribute19                => l_copy_from_vac.isc_attribute19
    ,isc_attribute20                => l_copy_from_vac.isc_attribute20
    ,isc_attribute21                => l_copy_from_vac.isc_attribute21
    ,isc_attribute22                => l_copy_from_vac.isc_attribute22
    ,isc_attribute23                => l_copy_from_vac.isc_attribute23
    ,isc_attribute24                => l_copy_from_vac.isc_attribute24
    ,isc_attribute25                => l_copy_from_vac.isc_attribute25
    ,isc_attribute26                => l_copy_from_vac.isc_attribute26
    ,isc_attribute27                => l_copy_from_vac.isc_attribute27
    ,isc_attribute28                => l_copy_from_vac.isc_attribute28
    ,isc_attribute29                => l_copy_from_vac.isc_attribute29
    ,isc_attribute30                => l_copy_from_vac.isc_attribute30
    ,isc_object_version_number      => 1
    ,pg_id_flex_num                 => l_copy_from_vac.pg_id_flex_num
    ,segment1                       => l_copy_from_vac.segment1
    ,segment2                       => l_copy_from_vac.segment2
    ,segment3                       => l_copy_from_vac.segment3
    ,segment4                       => l_copy_from_vac.segment4
    ,segment5                       => l_copy_from_vac.segment5
    ,segment6                       => l_copy_from_vac.segment6
    ,segment7                       => l_copy_from_vac.segment7
    ,segment8                       => l_copy_from_vac.segment8
    ,segment9                       => l_copy_from_vac.segment9
    ,segment10                      => l_copy_from_vac.segment10
    ,segment11                      => l_copy_from_vac.segment11
    ,segment12                      => l_copy_from_vac.segment12
    ,segment13                      => l_copy_from_vac.segment13
    ,segment14                      => l_copy_from_vac.segment14
    ,segment15                      => l_copy_from_vac.segment15
    ,segment16                      => l_copy_from_vac.segment16
    ,segment17                      => l_copy_from_vac.segment17
    ,segment18                      => l_copy_from_vac.segment18
    ,segment19                      => l_copy_from_vac.segment19
    ,segment20                      => l_copy_from_vac.segment20
    ,segment21                      => l_copy_from_vac.segment21
    ,segment22                      => l_copy_from_vac.segment22
    ,segment23                      => l_copy_from_vac.segment23
    ,segment24                      => l_copy_from_vac.segment24
    ,segment25                      => l_copy_from_vac.segment25
    ,segment26                      => l_copy_from_vac.segment26
    ,segment27                      => l_copy_from_vac.segment27
    ,segment28                      => l_copy_from_vac.segment28
    ,segment29                      => l_copy_from_vac.segment29
    ,segment30                      => l_copy_from_vac.segment30
    ,variable_comp_elements         => l_variable_comp_ele -- xxas_irc_variable_comp_ele_tbl()
    ,competence_elements            => l_competences       -- xxas_per_competence_ele_tbl()
    ,agency_vacancies               => l_agency_vacancy    -- xxas_irc_agency_vacancy_tbl()
    ,team_members                   => l_team_member       --xxas_irc_rec_team_member_tbl()
    ,recruitment_activities         => l_rec_activity      --xxas_per_rec_activity_tbl()
    ,vacancy_tasks                  => xxas_vacancy_task_tbl()
    ,attachments                    => l_attachments
    ,approvers                      => xxas_approver_tbl());
  
  create_hrapi_transaction(l_empty_vacancy);

  l_empty_vacancy.approvers :=  xxas_selfservice_workflow_pkg.get_approvers(l_empty_vacancy.transaction_id,g_AME_APP_ID,g_AME_TRANS_TYPE);  
	
  IF p_copy_from_vacancy_id IS NOT NULL THEN
    save_attachments(l_attachments,l_vacancy_id);
  END IF;
	
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_empty_vacancy;

END create_empty_vacancy;

--
-- ------------------------------------------------------------------------ *
-- Name    : save_attachments
-- ------------------------------------------------------------------------ *
PROCEDURE save_attachments
  (p_attachments IN OUT xxas_attachment_table
  ,p_vacancy_id  IN number)
IS

  l_attached_document_id       number;
  l_delete_attach_document_id  number;
  l_flag                       boolean := false;
  c_method      CONSTANT VARCHAR2(30) := 'save_attachments';
  l_attachments xxas_attachment_table := xxas_attachment_table();

  CURSOR csr_get_vacancy_attachments
    (cp_pk1_value IN varchar2) IS
  SELECT attached_document_id
  FROM   fnd_attached_documents
  WHERE  pk1_value = cp_pk1_value
  AND    entity_name IN('IRC_EXT_VAC', 'IRC_EXT_VAC_APPROVED'
                       ,'IRC_INT_VAC', 'IRC_INT_VAC_APPROVED');

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method);
  END IF;

  l_attachments := p_attachments;

  IF l_attachments.count > 0 THEN

    FOR i IN 1..l_attachments.count LOOP
      IF l_attachments(i).attached_document_id IS NULL THEN

        xxas_attachment_util_pkg.create_file_attachment
         (p_entity_name          => l_attachments(i).entity_name
         ,p_pk1_value            => l_attachments(i).pk1_value
         ,p_pk2_value            => l_attachments(i).pk2_value
         ,p_pk3_value            => l_attachments(i).pk3_value
         ,p_pk4_value            => l_attachments(i).pk4_value
         ,p_pk5_value            => l_attachments(i).pk5_value
         ,p_file_id              => l_attachments(i).file_id
         ,p_category_code        => l_attachments(i).category_code
         ,p_file_name            => l_attachments(i).file_name
         ,p_title                => l_attachments(i).title
         ,p_description          => l_attachments(i).description
         ,p_attached_document_id => l_attached_document_id);

        l_attachments(i).attached_document_id := l_attached_document_id;

      ELSIF l_attachments(i).attached_document_id IS NOT NULL THEN

        xxas_attachment_util_pkg.update_file_attachment
         (p_attached_document_id => l_attachments(i).attached_document_id
         ,p_file_id              => l_attachments(i).file_id
         ,p_category_code        => l_attachments(i).category_code
         ,p_file_name            => l_attachments(i).file_name
         ,p_title                => l_attachments(i).title
         ,p_description          => l_attachments(i).description);

      END IF;

      l_attached_document_id := NULL;
    END LOOP;

  END IF;

  IF l_attachments.count > 0 THEN

    FOR csr_vac_attachment IN csr_get_vacancy_attachments(to_char(p_vacancy_id)) LOOP

      FOR j IN 1..l_attachments.count LOOP

        IF csr_vac_attachment.attached_document_id = l_attachments(j).attached_document_id THEN

          l_flag := true;

        ELSE

          l_flag := false;
          l_delete_attach_document_id := csr_vac_attachment.attached_document_id;

        END IF;

        EXIT WHEN l_flag;

      END LOOP;

      IF NOT l_flag THEN

        xxas_attachment_util_pkg.delete_file_attachment(l_delete_attach_document_id);

      END IF;

      l_delete_attach_document_id := NULL;
      l_flag := false;

    END LOOP;

  END IF;

  p_attachments := l_attachments;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END save_attachments;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_key_flex_field_id
-- ------------------------------------------------------------------------ *
FUNCTION get_key_flex_field_id
  (p_vacancy IN xxas_per_all_vacancy) RETURN number
IS
  l_key_flex_field_id      NUMBER;
  l_concat_segments_out    VARCHAR2(4000);
  l_id_flex_code           VARCHAR2(30);
  l_application_id         fnd_application.application_id%type;
  l_application_short_name fnd_application.application_short_name%type;
  c_method      CONSTANT VARCHAR2(30) := 'get_people_group_id';

  CURSOR csr_get_key_flex_details(p_id_flex_num IN number) IS
  SELECT fifs.application_id
        ,fifs.id_flex_code
  FROM   fnd_id_flex_structures_vl fifs
  WHERE  fifs.id_flex_num = p_id_flex_num;

  CURSOR csr_get_appl_short_name(p_application_id IN number) IS
  SELECT application_short_name
  FROM   fnd_application
  WHERE  application_id = p_application_id;

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method);
  END IF;

    OPEN csr_get_key_flex_details(p_vacancy.pg_id_flex_num);
    FETCH csr_get_key_flex_details INTO l_application_id,l_id_flex_code;
    CLOSE csr_get_key_flex_details;

    OPEN csr_get_appl_short_name(l_application_id);
    FETCH csr_get_appl_short_name INTO l_application_short_name;
    CLOSE csr_get_appl_short_name;

    hr_kflex_utility.ins_or_sel_keyflex_comb
      (p_appl_short_name      => l_application_short_name
      ,p_flex_code            => l_id_flex_code
      ,p_flex_num             => p_vacancy.pg_id_flex_num
      ,p_segment1             => p_vacancy.segment1
      ,p_segment2             => p_vacancy.segment2
      ,p_segment3             => p_vacancy.segment3
      ,p_segment4             => p_vacancy.segment4
      ,p_segment5             => p_vacancy.segment5
      ,p_segment6             => p_vacancy.segment6
      ,p_segment7             => p_vacancy.segment7
      ,p_segment8             => p_vacancy.segment8
      ,p_segment9             => p_vacancy.segment9
      ,p_segment10            => p_vacancy.segment10
      ,p_segment11            => p_vacancy.segment11
      ,p_segment12            => p_vacancy.segment12
      ,p_segment13            => p_vacancy.segment13
      ,p_segment14            => p_vacancy.segment14
      ,p_segment15            => p_vacancy.segment15
      ,p_segment16            => p_vacancy.segment16
      ,p_segment17            => p_vacancy.segment17
      ,p_segment18            => p_vacancy.segment18
      ,p_segment19            => p_vacancy.segment19
      ,p_segment20            => p_vacancy.segment20
      ,p_segment21            => p_vacancy.segment21
      ,p_segment22            => p_vacancy.segment22
      ,p_segment23            => p_vacancy.segment23
      ,p_segment24            => p_vacancy.segment24
      ,p_segment25            => p_vacancy.segment25
      ,p_segment26            => p_vacancy.segment26
      ,p_segment27            => p_vacancy.segment27
      ,p_segment28            => p_vacancy.segment28
      ,p_segment29            => p_vacancy.segment29
      ,p_segment30            => p_vacancy.segment30
      ,p_concat_segments_in   => NULL
      ,p_ccid                 => l_key_flex_field_id
      ,p_concat_segments_out  => l_concat_segments_out);

  RETURN l_key_flex_field_id;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END get_key_flex_field_id;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_rec_team_member
-- ------------------------------------------------------------------------ *
FUNCTION create_rec_team_member 
  (p_vacancy_id      IN number
  ,p_rec_team_member IN xxas_irc_rec_team_member) RETURN NUMBER
IS

  c_method CONSTANT    VARCHAR2(30) := 'create_rec_team_member';
  l_rec_team_member_id NUMBER;
  l_ovn                NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  irc_rec_team_members_api.create_rec_team_member
    (p_validate              => false
    ,p_person_id             => p_rec_team_member.person_id
    ,p_vacancy_id            => p_vacancy_id
    ,p_job_id                => p_rec_team_member.job_id
    ,p_start_date            => p_rec_team_member.start_date
    ,p_end_date              => p_rec_team_member.end_date
    ,p_update_allowed        => p_rec_team_member.update_allowed
    ,p_delete_allowed        => p_rec_team_member.delete_allowed
    ,p_rec_team_member_id    => l_rec_team_member_id
    ,p_object_version_number => l_ovn
	);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_rec_team_member_id;
  
END create_rec_team_member;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_rec_team_members
-- ------------------------------------------------------------------------ *
PROCEDURE create_rec_team_members
  (p_vacancy_id       IN     number
  ,p_rec_team_members IN OUT xxas_irc_rec_team_member_tbl)
IS

  c_method CONSTANT    VARCHAR2(30) := 'create_rec_team_members';
  l_rec_team_member_id NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  --
  -- Create all recruitment team members
  --
  IF p_rec_team_members IS NOT NULL AND
     p_rec_team_members.count > 0 THEN
    FOR i IN p_rec_team_members.first..p_rec_team_members.last LOOP
      l_rec_team_member_id := create_rec_team_member
        (p_vacancy_id      => p_vacancy_id
        ,p_rec_team_member => p_rec_team_members(i));
      p_rec_team_members(i).rec_team_member_id := l_rec_team_member_id;
    END LOOP;
  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  
END create_rec_team_members;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_posting_content
-- ------------------------------------------------------------------------ *
FUNCTION create_posting_content 
  (p_posting_content IN xxas_per_all_vacancy) RETURN NUMBER
IS

  c_method CONSTANT    VARCHAR2(30) := 'create_posting_content';
  l_posting_content_id NUMBER;
  l_ovn                NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  irc_posting_content_api.create_posting_content
    (p_validate                     => false
    ,p_display_manager_info         => xxas_util_pkg.boolean_to_flag(p_posting_content.display_manager_info)
    ,p_display_recruiter_info       => xxas_util_pkg.boolean_to_flag(p_posting_content.display_recruiter_info)
    ,p_language_code                => NULL
    ,p_name                         => p_posting_content.posting_name
    ,p_org_name                     => p_posting_content.posting_org_name
    ,p_org_description              => p_posting_content.posting_org_description
    ,p_job_title                    => p_posting_content.posting_job_title
    ,p_brief_description            => p_posting_content.posting_brief_description
    ,p_detailed_description         => p_posting_content.posting_detailed_description
    ,p_job_requirements             => p_posting_content.posting_job_requirements
    ,p_additional_details           => p_posting_content.posting_additional_details
    ,p_how_to_apply                 => p_posting_content.posting_how_to_apply
    ,p_benefit_info                 => p_posting_content.benefit_info
    ,p_image_url                    => p_posting_content.image_url
    ,p_alt_image_url                => p_posting_content.image_url_alt
    ,p_attribute_category           => p_posting_content.ipc_attribute_category
    ,p_attribute1                   => p_posting_content.ipc_attribute1
    ,p_attribute2                   => p_posting_content.ipc_attribute2
    ,p_attribute3                   => p_posting_content.ipc_attribute3
    ,p_attribute4                   => p_posting_content.ipc_attribute4
    ,p_attribute5                   => p_posting_content.ipc_attribute5
    ,p_attribute6                   => p_posting_content.ipc_attribute6
    ,p_attribute7                   => p_posting_content.ipc_attribute7
    ,p_attribute8                   => p_posting_content.ipc_attribute8
    ,p_attribute9                   => p_posting_content.ipc_attribute9
    ,p_attribute10                  => p_posting_content.ipc_attribute10
    ,p_attribute11                  => p_posting_content.ipc_attribute11
    ,p_attribute12                  => p_posting_content.ipc_attribute12
    ,p_attribute13                  => p_posting_content.ipc_attribute13
    ,p_attribute14                  => p_posting_content.ipc_attribute14
    ,p_attribute15                  => p_posting_content.ipc_attribute15
    ,p_attribute16                  => p_posting_content.ipc_attribute16
    ,p_attribute17                  => p_posting_content.ipc_attribute17
    ,p_attribute18                  => p_posting_content.ipc_attribute18
    ,p_attribute19                  => p_posting_content.ipc_attribute19
    ,p_attribute20                  => p_posting_content.ipc_attribute20
    ,p_attribute21                  => p_posting_content.ipc_attribute21
    ,p_attribute22                  => p_posting_content.ipc_attribute22
    ,p_attribute23                  => p_posting_content.ipc_attribute23
    ,p_attribute24                  => p_posting_content.ipc_attribute24
    ,p_attribute25                  => p_posting_content.ipc_attribute25
    ,p_attribute26                  => p_posting_content.ipc_attribute26
    ,p_attribute27                  => p_posting_content.ipc_attribute27
    ,p_attribute28                  => p_posting_content.ipc_attribute28
    ,p_attribute29                  => p_posting_content.ipc_attribute29
    ,p_attribute30                  => p_posting_content.ipc_attribute30
    ,p_ipc_information_category     => NULL
    ,p_ipc_information1             => NULL
    ,p_ipc_information2             => NULL
    ,p_ipc_information3             => NULL
    ,p_ipc_information4             => NULL
    ,p_ipc_information5             => NULL
    ,p_ipc_information6             => NULL
    ,p_ipc_information7             => NULL
    ,p_ipc_information8             => NULL
    ,p_ipc_information9             => NULL
    ,p_ipc_information10            => NULL
    ,p_ipc_information11            => NULL
    ,p_ipc_information12            => NULL
    ,p_ipc_information13            => NULL
    ,p_ipc_information14            => NULL
    ,p_ipc_information15            => NULL
    ,p_ipc_information16            => NULL
    ,p_ipc_information17            => NULL
    ,p_ipc_information18            => NULL
    ,p_ipc_information19            => NULL
    ,p_ipc_information20            => NULL
    ,p_ipc_information21            => NULL
    ,p_ipc_information22            => NULL
    ,p_ipc_information23            => NULL
    ,p_ipc_information24            => NULL
    ,p_ipc_information25            => NULL
    ,p_ipc_information26            => NULL
    ,p_ipc_information27            => NULL
    ,p_ipc_information28            => NULL
    ,p_ipc_information29            => NULL
    ,p_ipc_information30            => NULL
    ,p_date_approved                => SYSDATE
    ,p_posting_content_id           => l_posting_content_id
    ,p_object_version_number        => l_ovn
	);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_posting_content_id;
  
END create_posting_content;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_requisition
-- ------------------------------------------------------------------------ *
FUNCTION create_requisition 
  (p_requisition IN xxas_per_all_vacancy) RETURN NUMBER
IS

  c_method CONSTANT     VARCHAR2(30) := 'create_requisition';
  l_requisition_id NUMBER;
  l_ovn                 NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  per_requisitions_api.create_requisition
    (p_validate                     => false
    ,p_business_group_id            => p_requisition.business_group_id
    ,p_date_from                    => p_requisition.date_from
    ,p_name                         => p_requisition.vacancy_name
    ,p_person_id                    => NULL
    ,p_comments                     => p_requisition.comments
    ,p_date_to                      => p_requisition.date_to
    ,p_description                  => p_requisition.description
    ,p_attribute_category           => NULL
    ,p_attribute1                   => NULL
    ,p_attribute2                   => NULL
    ,p_attribute3                   => NULL
    ,p_attribute4                   => NULL
    ,p_attribute5                   => NULL
    ,p_attribute6                   => NULL
    ,p_attribute7                   => NULL
    ,p_attribute8                   => NULL
    ,p_attribute9                   => NULL
    ,p_attribute10                  => NULL
    ,p_attribute11                  => NULL
    ,p_attribute12                  => NULL
    ,p_attribute13                  => NULL
    ,p_attribute14                  => NULL
    ,p_attribute15                  => NULL
    ,p_attribute16                  => NULL
    ,p_attribute17                  => NULL
    ,p_attribute18                  => NULL
    ,p_attribute19                  => NULL
    ,p_attribute20                  => NULL
    ,p_attribute21                  => NULL
    ,p_attribute22                  => NULL
    ,p_attribute23                  => NULL
    ,p_attribute24                  => NULL
    ,p_attribute25                  => NULL
    ,p_attribute26                  => NULL
    ,p_attribute27                  => NULL
    ,p_attribute28                  => NULL
    ,p_attribute29                  => NULL
    ,p_attribute30                  => NULL
    ,p_requisition_id               => l_requisition_id
    ,p_object_version_number        => l_ovn
	);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_requisition_id;
  
END create_requisition;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_rec_activity
-- ------------------------------------------------------------------------ *
FUNCTION create_rec_activity 
  (p_business_group_id IN number
  ,p_rec_activity      IN xxas_per_rec_activity) RETURN NUMBER
IS

  c_method CONSTANT     VARCHAR2(30) := 'create_rec_activity';
  l_rec_activity_id     NUMBER;
  l_ovn                 NUMBER;
  l_status              VARCHAR2(100);

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  per_recruitment_activity_api.create_recruitment_activity
    (p_validate                     => false
    ,p_business_group_id            => p_business_group_id
    ,p_date_start                   => p_rec_activity.date_start
    ,p_name                         => p_rec_activity.name
    ,p_authorising_person_id        => NULL
    ,p_run_by_organization_id       => NULL
    ,p_internal_contact_person_id   => NULL
    ,p_parent_recruitment_activity  => NULL
    ,p_currency_code                => NULL
    ,p_actual_cost                  => NULL
    ,p_comments                     => NULL
    ,p_contact_telephone_number     => NULL
    ,p_date_closing                 => NULL
    ,p_date_end                     => p_rec_activity.date_end
    ,p_external_contact             => NULL
    ,p_planned_cost                 => NULL
    ,p_recruiting_site_id           => p_rec_activity.recruiting_site_id
    ,p_recruiting_site_response     => NULL
    ,p_last_posted_date             => NULL
    ,p_type                         => NULL
    ,p_attribute_category           => NULL
    ,p_attribute1                   => NULL
    ,p_attribute2                   => NULL
    ,p_attribute3                   => NULL
    ,p_attribute4                   => NULL
    ,p_attribute5                   => NULL
    ,p_attribute6                   => NULL
    ,p_attribute7                   => NULL
    ,p_attribute8                   => NULL
    ,p_attribute9                   => NULL
    ,p_attribute10                  => NULL
    ,p_attribute11                  => NULL
    ,p_attribute12                  => NULL
    ,p_attribute13                  => NULL
    ,p_attribute14                  => NULL
    ,p_attribute15                  => NULL
    ,p_attribute16                  => NULL
    ,p_attribute17                  => NULL
    ,p_attribute18                  => NULL
    ,p_attribute19                  => NULL
    ,p_attribute20                  => NULL
    ,p_posting_content_id           => p_rec_activity.posting_content_id
    ,p_status                       => l_status
    ,p_object_version_number        => l_ovn
    ,p_recruitment_activity_id      => l_rec_activity_id
	);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_rec_activity_id;
  
END create_rec_activity;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_rec_activities
-- ------------------------------------------------------------------------ *
PROCEDURE create_rec_activities
  (p_business_group_id  IN     number
  ,p_posting_content_id IN     number
  ,p_rec_activities     IN OUT xxas_per_rec_activity_tbl)
IS

  c_method CONSTANT VARCHAR2(30) := 'create_rec_activities';
  l_rec_activity_id NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  --
  -- Create all recruitment team members
  --
  IF p_rec_activities IS NOT NULL AND
     p_rec_activities.count > 0 THEN
    FOR i IN p_rec_activities.first..p_rec_activities.last LOOP
      p_rec_activities(i).posting_content_id := p_posting_content_id;
      l_rec_activity_id := create_rec_activity
        (p_business_group_id => p_business_group_id
        ,p_rec_activity      => p_rec_activities(i));
      p_rec_activities(i).recruitment_activity_id := l_rec_activity_id;
    END LOOP;
  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  
END create_rec_activities;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_vacancy_criteria
-- ------------------------------------------------------------------------ *
FUNCTION create_vacancy_criteria 
  (p_vacancy_id      IN number
  ,p_vacancy_criteria IN xxas_per_all_vacancy) RETURN NUMBER
IS

  c_method CONSTANT    VARCHAR2(30) := 'create_vacancy_criteria';
  l_search_criteria_id NUMBER;
  l_ovn                NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  irc_search_criteria_api.create_vacancy_criteria
    (p_validate                     => false
    ,p_vacancy_id                   => p_vacancy_id
    ,p_effective_date               => sysdate
    ,p_location                     => NULL
    ,p_employee                     => p_vacancy_criteria.employee_flag
    ,p_contractor                   => p_vacancy_criteria.contractor_flag
    ,p_employment_category          => p_vacancy_criteria.employment_category_code
    ,p_keywords                     => NULL
    ,p_travel_percentage            => p_vacancy_criteria.travel_percentage
    ,p_min_salary                   => p_vacancy_criteria.min_salary
    ,p_max_salary                   => p_vacancy_criteria.max_salary
    ,p_salary_currency              => p_vacancy_criteria.salary_currency
    ,p_salary_period                => p_vacancy_criteria.salary_period_code
    ,p_professional_area            => p_vacancy_criteria.professional_area_code
    ,p_work_at_home                 => p_vacancy_criteria.work_at_home_code
    ,p_min_qual_level               => p_vacancy_criteria.min_qual_level_id
    ,p_max_qual_level               => p_vacancy_criteria.max_qual_level_id
    ,p_description                  => NULL
    ,p_attribute_category           => p_vacancy_criteria.isc_attribute_category
    ,p_attribute1                   => p_vacancy_criteria.isc_attribute1
    ,p_attribute2                   => p_vacancy_criteria.isc_attribute2
    ,p_attribute3                   => p_vacancy_criteria.isc_attribute3
    ,p_attribute4                   => p_vacancy_criteria.isc_attribute4
    ,p_attribute5                   => p_vacancy_criteria.isc_attribute5
    ,p_attribute6                   => p_vacancy_criteria.isc_attribute6
    ,p_attribute7                   => p_vacancy_criteria.isc_attribute7
    ,p_attribute8                   => p_vacancy_criteria.isc_attribute8
    ,p_attribute9                   => p_vacancy_criteria.isc_attribute9
    ,p_attribute10                  => p_vacancy_criteria.isc_attribute10
    ,p_attribute11                  => p_vacancy_criteria.isc_attribute11
    ,p_attribute12                  => p_vacancy_criteria.isc_attribute12
    ,p_attribute13                  => p_vacancy_criteria.isc_attribute13
    ,p_attribute14                  => p_vacancy_criteria.isc_attribute14
    ,p_attribute15                  => p_vacancy_criteria.isc_attribute15
    ,p_attribute16                  => p_vacancy_criteria.isc_attribute16
    ,p_attribute17                  => p_vacancy_criteria.isc_attribute17
    ,p_attribute18                  => p_vacancy_criteria.isc_attribute18
    ,p_attribute19                  => p_vacancy_criteria.isc_attribute19
    ,p_attribute20                  => p_vacancy_criteria.isc_attribute20
    ,p_attribute21                  => p_vacancy_criteria.isc_attribute21
    ,p_attribute22                  => p_vacancy_criteria.isc_attribute22
    ,p_attribute23                  => p_vacancy_criteria.isc_attribute23
    ,p_attribute24                  => p_vacancy_criteria.isc_attribute24
    ,p_attribute25                  => p_vacancy_criteria.isc_attribute25
    ,p_attribute26                  => p_vacancy_criteria.isc_attribute26
    ,p_attribute27                  => p_vacancy_criteria.isc_attribute27
    ,p_attribute28                  => p_vacancy_criteria.isc_attribute28
    ,p_attribute29                  => p_vacancy_criteria.isc_attribute29
    ,p_attribute30                  => p_vacancy_criteria.isc_attribute30
    ,p_isc_information_category     => NULL
    ,p_isc_information1             => NULL
    ,p_isc_information2             => NULL
    ,p_isc_information3             => NULL
    ,p_isc_information4             => NULL
    ,p_isc_information5             => NULL
    ,p_isc_information6             => NULL
    ,p_isc_information7             => NULL
    ,p_isc_information8             => NULL
    ,p_isc_information9             => NULL
    ,p_isc_information10            => NULL
    ,p_isc_information11            => NULL
    ,p_isc_information12            => NULL
    ,p_isc_information13            => NULL
    ,p_isc_information14            => NULL
    ,p_isc_information15            => NULL
    ,p_isc_information16            => NULL
    ,p_isc_information17            => NULL
    ,p_isc_information18            => NULL
    ,p_isc_information19            => NULL
    ,p_isc_information20            => NULL
    ,p_isc_information21            => NULL
    ,p_isc_information22            => NULL
    ,p_isc_information23            => NULL
    ,p_isc_information24            => NULL
    ,p_isc_information25            => NULL
    ,p_isc_information26            => NULL
    ,p_isc_information27            => NULL
    ,p_isc_information28            => NULL
    ,p_isc_information29            => NULL
    ,p_isc_information30            => NULL
    ,p_object_version_number        => l_ovn
    ,p_search_criteria_id           => l_search_criteria_id
	);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_search_criteria_id;
  
END create_vacancy_criteria;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_variable_compensation
-- ------------------------------------------------------------------------ *
FUNCTION create_variable_compensation 
  (p_vacancy_id            IN number
  ,p_variable_compensation IN xxas_irc_variable_comp_ele) RETURN NUMBER
IS

  c_method CONSTANT VARCHAR2(30) := 'create_variable_compensation';
  l_ovn             NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  irc_variable_comp_element_api.create_variable_compensation
    (p_validate                     => false
    ,p_vacancy_id                   => p_vacancy_id
    ,p_variable_comp_lookup         => p_variable_compensation.variable_comp_code
    ,p_effective_date               => sysdate
    ,p_object_version_number        => l_ovn
	);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_ovn;
  
END create_variable_compensation;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_variable_compensations
-- ------------------------------------------------------------------------ *
PROCEDURE create_variable_compensations
  (p_vacancy_id             IN number
  ,p_variable_compensations IN xxas_irc_variable_comp_ele_tbl)
IS

  c_method CONSTANT VARCHAR2(30) := 'create_variable_compensations';
  l_ovn             NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  --
  -- Create all recruitment team members
  --
  IF p_variable_compensations IS NOT NULL AND
     p_variable_compensations.count > 0 THEN
    FOR i IN p_variable_compensations.first..p_variable_compensations.last LOOP
      l_ovn := create_variable_compensation
        (p_vacancy_id            => p_vacancy_id
        ,p_variable_compensation => p_variable_compensations(i));
    END LOOP;
  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  
END create_variable_compensations;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_vacancy_comp_element
-- ------------------------------------------------------------------------ *
FUNCTION create_vacancy_comp_element 
  (p_vacancy_id            IN number
  ,p_business_group_id     IN number
  ,p_vacancy_comptence_ele IN xxas_per_competence_ele) RETURN NUMBER
IS

  c_method CONSTANT          VARCHAR2(30) := 'create_vacancy_comp_element';
  l_vac_comptence_ele_id     NUMBER;
  l_ovn                      NUMBER;
  l_status                   VARCHAR2(100);

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  hr_competence_element_api.create_competence_element
    (p_validate                     => false
    ,p_competence_element_id        => l_vac_comptence_ele_id
    ,p_object_version_number        => l_ovn
    ,p_type                         => NVL(p_vacancy_comptence_ele.type_code,'REQUIREMENT')
    ,p_business_group_id            => p_business_group_id
    ,p_enterprise_id                => NULL
    ,p_competence_id                => p_vacancy_comptence_ele.competence_id
    ,p_proficiency_level_id         => p_vacancy_comptence_ele.proficiency_level_id
    ,p_high_proficiency_level_id    => p_vacancy_comptence_ele.high_proficiency_level_id
    ,p_weighting_level_id           => NULL
    ,p_rating_level_id              => NULL
    ,p_person_id                    => NULL
    ,p_job_id                       => NULL
    ,p_valid_grade_id               => NULL
    ,p_position_id                  => NULL
    ,p_organization_id              => NULL
    ,p_parent_competence_element_id => NULL
    ,p_activity_version_id          => NULL
    ,p_assessment_id                => NULL
    ,p_assessment_type_id           => NULL
    ,p_mandatory                    => p_vacancy_comptence_ele.mandatory_code
    ,p_effective_date_from          => NULL
    ,p_effective_date_to            => NULL
    ,p_group_competence_type        => NULL
    ,p_competence_type              => NULL
    ,p_normal_elapse_duration       => NULL
    ,p_normal_elapse_duration_unit  => NULL
    ,p_sequence_number              => NULL
    ,p_source_of_proficiency_level  => NULL
    ,p_line_score                   => NULL
    ,p_certification_date           => NULL
    ,p_certification_method         => NULL
    ,p_next_certification_date      => NULL
    ,p_comments                     => NULL
    ,p_attribute_category           => p_vacancy_comptence_ele.attribute_category
    ,p_attribute1                   => p_vacancy_comptence_ele.attribute1
    ,p_attribute2                   => p_vacancy_comptence_ele.attribute2
    ,p_attribute3                   => p_vacancy_comptence_ele.attribute3
    ,p_attribute4                   => p_vacancy_comptence_ele.attribute4
    ,p_attribute5                   => p_vacancy_comptence_ele.attribute5
    ,p_attribute6                   => p_vacancy_comptence_ele.attribute6
    ,p_attribute7                   => p_vacancy_comptence_ele.attribute7
    ,p_attribute8                   => p_vacancy_comptence_ele.attribute8
    ,p_attribute9                   => p_vacancy_comptence_ele.attribute9
    ,p_attribute10                  => p_vacancy_comptence_ele.attribute10
    ,p_attribute11                  => p_vacancy_comptence_ele.attribute11
    ,p_attribute12                  => p_vacancy_comptence_ele.attribute12
    ,p_attribute13                  => p_vacancy_comptence_ele.attribute13
    ,p_attribute14                  => p_vacancy_comptence_ele.attribute14
    ,p_attribute15                  => p_vacancy_comptence_ele.attribute15
    ,p_attribute16                  => p_vacancy_comptence_ele.attribute16
    ,p_attribute17                  => p_vacancy_comptence_ele.attribute17
    ,p_attribute18                  => p_vacancy_comptence_ele.attribute18
    ,p_attribute19                  => p_vacancy_comptence_ele.attribute19
    ,p_attribute20                  => p_vacancy_comptence_ele.attribute20
    ,p_effective_date               => sysdate
    ,p_object_id                    => p_vacancy_id
    ,p_object_name                  => 'VACANCY'
    ,p_party_id                     => NULL
    ,p_appr_line_score              => NULL
    ,p_status                       => l_status
	);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_vac_comptence_ele_id;
  
END create_vacancy_comp_element;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_vacancy_comp_elements
-- ------------------------------------------------------------------------ *
PROCEDURE create_vacancy_comp_elements
  (p_vacancy_id                 IN     number
  ,p_business_group_id          IN     number
  ,p_vacancy_comptence_elements IN OUT xxas_per_competence_ele_tbl)
IS

  c_method CONSTANT      VARCHAR2(30) := 'create_vacancy_comp_elements';
  l_vac_comptence_ele_id NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  --
  -- Create all recruitment team members
  --
  IF p_vacancy_comptence_elements IS NOT NULL AND
     p_vacancy_comptence_elements.count > 0 THEN
    FOR i IN p_vacancy_comptence_elements.first..p_vacancy_comptence_elements.last LOOP
      l_vac_comptence_ele_id := create_vacancy_comp_element
        (p_vacancy_id            => p_vacancy_id
        ,p_business_group_id     => p_business_group_id
        ,p_vacancy_comptence_ele => p_vacancy_comptence_elements(i));
      p_vacancy_comptence_elements(i).competence_element_id := l_vac_comptence_ele_id;
    END LOOP;
  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  
END create_vacancy_comp_elements;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_rec_activity_for
-- ------------------------------------------------------------------------ *
FUNCTION create_rec_activity_for 
  (p_vacancy_id        IN number
  ,p_business_group_id IN number
  ,p_rec_activity_for  IN xxas_per_rec_activity) RETURN NUMBER
IS

  c_method CONSTANT     VARCHAR2(30) := 'create_rec_activity_for';
  l_rec_activity_for_id NUMBER;
  l_ovn                 NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  per_rec_activity_for_api.create_rec_activity_for
    (p_validate                     => false
    ,p_rec_activity_for_id          => l_rec_activity_for_id
    ,p_business_group_id            => p_business_group_id
    ,p_vacancy_id                   => p_vacancy_id
    ,p_rec_activity_id              => p_rec_activity_for.recruitment_activity_id
    ,p_object_version_number        => l_ovn
	);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  RETURN l_rec_activity_for_id;
  
END create_rec_activity_for;

--
-- ------------------------------------------------------------------------ *
-- Name    : create_rec_activities_for
-- ------------------------------------------------------------------------ *
PROCEDURE create_rec_activities_for
  (p_vacancy_id         IN     number
  ,p_business_group_id  IN     number
  ,p_rec_activities_for IN OUT xxas_per_rec_activity_tbl)
IS

  c_method CONSTANT     VARCHAR2(30) := 'create_rec_activities_for';
  l_rec_activity_for_id NUMBER;

BEGIN

  IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;
  
  --
  -- Create all recruitment team members
  --
  IF p_rec_activities_for IS NOT NULL AND
     p_rec_activities_for.count > 0 THEN
    FOR i IN p_rec_activities_for.first..p_rec_activities_for.last LOOP
      l_rec_activity_for_id := create_rec_activity_for
        (p_vacancy_id        => p_vacancy_id
        ,p_business_group_id => p_business_group_id
        ,p_rec_activity_for  => p_rec_activities_for(i));
      p_rec_activities_for(i).recruitment_activity_for_id := l_rec_activity_for_id;
    END LOOP;
  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  
  
END create_rec_activities_for;

--
-- ------------------------------------------------------------------------ *
-- Name    : copy_attachments_for_edit
-- ------------------------------------------------------------------------ *
PROCEDURE copy_attachments_for_edit 
  (p_vacancy_id IN number)
IS

  c_method CONSTANT varchar2(30)         := 'copy_attachments_for_edit';

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;

  fnd_attached_documents2_pkg.copy_attachments(X_from_entity_name  => 'IRC_INT_VAC_APPROVED'
                                              ,X_from_pk1_value    => p_vacancy_id
                                              ,X_to_entity_name    => 'IRC_INT_VAC'
                                              ,X_to_pk1_value      => p_vacancy_id
                                              ,X_created_by        => fnd_global.user_id
                                              ,X_last_update_login => fnd_global.login_id);

  fnd_attached_documents2_pkg.copy_attachments(X_from_entity_name  => 'IRC_EXT_VAC_APPROVED'
                                              ,X_from_pk1_value    => p_vacancy_id
                                              ,X_to_entity_name    => 'IRC_EXT_VAC'
                                              ,X_to_pk1_value      => p_vacancy_id
                                              ,X_created_by        => fnd_global.user_id
                                              ,X_last_update_login => fnd_global.login_id);

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END copy_attachments_for_edit;

--
-- ------------------------------------------------------------------------ *
-- Name    : copy_attachments
-- ------------------------------------------------------------------------ *
PROCEDURE copy_attachments 
  (p_vacancy_id IN number)
IS

  c_method CONSTANT varchar2(30)         := 'copy_attachments';

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;

  fnd_attached_documents2_pkg.delete_attachments(X_entity_name          => 'IRC_EXT_VAC_APPROVED'
                                                ,X_pk1_value            => p_vacancy_id
                                                ,X_delete_document_flag => 'Y');

  fnd_attached_documents2_pkg.delete_attachments(X_entity_name          => 'IRC_INT_VAC_APPROVED'
                                                ,X_pk1_value            => p_vacancy_id
                                                ,X_delete_document_flag => 'Y');

  fnd_attached_documents2_pkg.copy_attachments(X_from_entity_name  => 'IRC_INT_VAC'
                                              ,X_from_pk1_value    => p_vacancy_id
                                              ,X_to_entity_name    => 'IRC_INT_VAC_APPROVED'
                                              ,X_to_pk1_value      => p_vacancy_id
                                              ,X_created_by        => fnd_global.user_id
                                              ,X_last_update_login => fnd_global.login_id);

  fnd_attached_documents2_pkg.copy_attachments(X_from_entity_name  => 'IRC_EXT_VAC'
                                              ,X_from_pk1_value    => p_vacancy_id
                                              ,X_to_entity_name    => 'IRC_EXT_VAC_APPROVED'
                                              ,X_to_pk1_value      => p_vacancy_id
                                              ,X_created_by        => fnd_global.user_id
                                              ,X_last_update_login => fnd_global.login_id);

  fnd_attached_documents2_pkg.delete_attachments(X_entity_name          => 'IRC_EXT_VAC'
                                                ,X_pk1_value            => p_vacancy_id
                                                ,X_delete_document_flag => 'Y');

  fnd_attached_documents2_pkg.delete_attachments(X_entity_name          => 'IRC_INT_VAC'
                                                ,X_pk1_value            => p_vacancy_id
                                                ,X_delete_document_flag => 'Y');

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END copy_attachments;


--
-- ------------------------------------------------------------------------ *
-- Name    : submit_vacancy
-- ------------------------------------------------------------------------ *
PROCEDURE submit_vacancy 
  (p_vacancy IN xxas_per_all_vacancy)
IS

  CURSOR csr_get_trans_data(p_transaction_id IN NUMBER) IS
  SELECT hat.*
  FROM   hr_api_transactions hat
  WHERE  hat.transaction_id  = p_transaction_id;

  c_method CONSTANT varchar2(30)         := 'submit_vacancy';
  l_vacancy         xxas_per_all_vacancy := p_vacancy;
  l_cur_vacancy     xxas_per_all_vacancy;
  l_activity_id     number;
  l_vacancy_id      number;
  l_posting_content_id number;
  l_requisition_id  number;
  l_ret_vacancy_id  number;
  l_search_criteria_id number;
  l_create          boolean              := false;
  l_hrapi           hr_api_transactions%ROWTYPE := NULL;
  l_approvers_tbl   xxas_approver_tbl := xxas_approver_tbl();

BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start(c_method);
  END IF;

  IF l_vacancy.transaction_id IS NULL THEN
    --
    -- Generate a new transaction id
    --
    SELECT hr_api_transactions_s.nextval
    INTO   l_vacancy.transaction_id
    FROM   dual;

    l_vacancy.item_type               := g_ITEM_TYPE;
    l_create                          := true;
    
  END IF;
  
  --
  -- Change the HR API status to 'Y'
  --
  l_vacancy.hr_api_status_code      := 'Y';
  l_vacancy.workflow_status_code    := xxas_selfservice_workflow_pkg.transcode_status
                                        (l_vacancy.hr_api_status_code);
  l_vacancy.workflow_status_meaning := xxas_selfservice_workflow_pkg.decode_status
                                        (l_vacancy.workflow_status_code);
  save_attachments(l_vacancy.attachments
                  ,l_vacancy.vacancy_id);

  IF l_vacancy.transaction_type = 'UPDATE' THEN

    --
    -- Copy Approved attachments to Unapproved
    --
    copy_attachments_for_edit(l_vacancy.vacancy_id);
    
  END IF;

  IF l_vacancy.pg_id_flex_num IS NOT NULL THEN
    l_vacancy.people_group_id := get_key_flex_field_id(l_vacancy);
  END IF;

  l_cur_vacancy := l_vacancy;

  --
  -- Do create API's validation
  --
  IF l_cur_vacancy.transaction_type = 'INSERT' THEN

    l_vacancy_id         := l_cur_vacancy.vacancy_id;
    --
    -- Issue a savepoint
    --
    savepoint before_api_validation;
    --
    -- irc_posting_content_api.create_posting_content API validation
    --
    l_posting_content_id := create_posting_content(l_cur_vacancy);
    l_cur_vacancy.primary_posting_id := l_posting_content_id;
    --
    -- per_recruitment_activity_api.create_recruitment_activity API validation
    --
    create_rec_activities(l_cur_vacancy.business_group_id, l_posting_content_id, l_cur_vacancy.recruitment_activities);
    --
    -- per_requisitions_api.create_requisition API validation
    --
    l_requisition_id     := create_requisition(l_cur_vacancy);
    l_cur_vacancy.requisition_id := l_requisition_id;
    --
    -- per_vacancy_api.create_vacancy API validation
    --
    l_ret_vacancy_id     := create_vacancy(l_cur_vacancy);
    l_cur_vacancy.vacancy_id := l_ret_vacancy_id;
    --
    -- irc_search_criteria_swi.create_vacancy_criteria API validation
    --
    l_search_criteria_id := create_vacancy_criteria(l_ret_vacancy_id, l_cur_vacancy);
    l_cur_vacancy.search_criteria_id := l_search_criteria_id;
    --
    -- per_rec_activity_for_api.create_rec_activity_for API validation
    --
    create_rec_activities_for(l_ret_vacancy_id, l_cur_vacancy.business_group_id, l_cur_vacancy.recruitment_activities);
    --
    -- hr_competence_element_api.create_competence_element API validation
    --
    create_vacancy_comp_elements(l_ret_vacancy_id, l_cur_vacancy.business_group_id, l_cur_vacancy.competence_elements);
    --
    -- irc_rec_team_members_api.create_rec_team_member API validation
    --
    FOR i IN 1..l_cur_vacancy.team_members.count LOOP

        l_cur_vacancy.team_members(i).update_allowed        := NVL(l_cur_vacancy.team_members(i).update_allowed,'N');
        l_cur_vacancy.team_members(i).delete_allowed        := NVL(l_cur_vacancy.team_members(i).delete_allowed,'N');

    END LOOP;
    create_rec_team_members(l_ret_vacancy_id, l_cur_vacancy.team_members);
    --
    -- irc_agency_vacancies_api.create_agency_vacancy API validation
    --
    create_agency_vacancies(l_ret_vacancy_id, l_cur_vacancy.agency_vacancies);
    --
    -- irc_variable_comp_element_api.create_variable_compensation API validation
    --
    create_variable_compensations(l_ret_vacancy_id, l_cur_vacancy.variable_comp_elements);
    --
    -- Issue a rollback
    --
    rollback to before_api_validation;

  END IF;

  launch_workflow
    (p_vacancy     => l_vacancy
    ,p_item_key    => l_vacancy.item_key
    ,p_activity_id => l_activity_id);
  
  --
  -- Update the transaction table
  --
  IF l_create THEN
    create_hrapi_transaction(l_vacancy);
  ELSE
    update_hrapi_transaction(l_vacancy);
  END IF;

  xxas_selfservice_workflow_pkg.complete_current_activity
    (p_item_type => l_vacancy.item_type
    ,p_item_key  => l_vacancy.item_key
    ,p_result    => null);

  OPEN  csr_get_trans_data(l_vacancy.transaction_id);
  FETCH csr_get_trans_data INTO l_hrapi;
  CLOSE csr_get_trans_data;

  --
  -- Checking for the Transaction Document to see if still transaction is there/For No approvers/Auto Approvers - Approver is same as creator person
  --
  IF l_hrapi.transaction_document IS NULL THEN

    copy_attachments(l_vacancy.vacancy_id);

  ELSIF l_vacancy.approvers.count = 0 THEN

    copy_attachments(l_vacancy.vacancy_id);

  ELSIF l_vacancy.approvers.count = 1 THEN

    l_approvers_tbl := l_vacancy.approvers;
    IF l_approvers_tbl(1).origsys_id = xxas_util_pkg.get_per_from_user(fnd_global.user_id) THEN
      copy_attachments(l_vacancy.vacancy_id);
    END IF;

  END IF;

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

EXCEPTION  
  
  WHEN hr_utility.hr_error
  THEN
    IF l_vacancy.transaction_type = 'INSERT' THEN
	  rollback to before_api_validation;
    END IF;

    raise_user_error
      (p_package_name => g_PACKAGE_NAME
      ,p_proc_name    => c_method
      ,p_message      => hr_utility.get_message);

END submit_vacancy;

--
-- ------------------------------------------------------------------------ *
-- Name    : delete_vacancy
-- ------------------------------------------------------------------------ *
PROCEDURE delete_vacancy 
  (p_vacancy_id IN number)
IS

  c_method CONSTANT VARCHAR2(30) := 'delete_vacancy';
  
  CURSOR csr_trans_id
  IS
  SELECT t.transaction_id
  FROM   hr_api_transactions t
  WHERE  t.transaction_ref_id = p_vacancy_id
  AND    t.transaction_ref_table = 'PER_ALL_VACANCIES';
  
  l_trans_id number;

BEGIN

  IF gc_log.proc=1 THEN 
    gc_log.procedure_start_params(c_method, p_vacancy_id);
  END IF;

  IF p_vacancy_id IS NOT NULL THEN
  
    OPEN  csr_trans_id;
    FETCH csr_trans_id INTO l_trans_id;
    CLOSE csr_trans_id;
    
    IF l_trans_id IS NOT NULL THEN
      hr_transaction_swi.delete_transaction
        (p_transaction_id => l_trans_id);
    END IF;
  END IF;
  
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

END delete_vacancy;

--
-- ------------------------------------------------------------------------ *
-- Name    : default_posting
-- ------------------------------------------------------------------------ *
PROCEDURE default_posting 
  (p_vacancy IN OUT NOCOPY xxas_per_all_vacancy)
IS

  c_method      CONSTANT VARCHAR2(30) := 'default_posting';
  l_org_name             irc_default_postings_vl.org_name%TYPE;
  l_org_description      irc_default_postings_vl.org_description%TYPE;
  l_job_title            irc_default_postings_vl.job_title%TYPE;
  l_brief_description    irc_default_postings_vl.brief_description%TYPE;
  l_detailed_description irc_default_postings_vl.detailed_description%TYPE;
  l_job_requirements     irc_default_postings_vl.job_requirements%TYPE;
  l_additional_details   irc_default_postings_vl.additional_details%TYPE;
  l_how_to_apply         irc_default_postings_vl.how_to_apply%TYPE;
  l_image_url            irc_default_postings_vl.image_url%TYPE;
  l_image_url_alt        irc_default_postings_vl.image_url_alt%TYPE;  
  
  CURSOR csr_pos_def
    (cp_position_id IN number)
  IS
  SELECT dp.org_name
        ,dp.org_description
        ,dp.job_title
        ,dp.brief_description
        ,dp.detailed_description
        ,dp.job_requirements
        ,dp.additional_details
        ,dp.how_to_apply
        ,dp.image_url
        ,dp.image_url_alt
  FROM   irc_default_postings_vl dp
  WHERE  dp.position_id = cp_position_id;
  
  
  CURSOR csr_job_def
    (cp_job_id IN number)
  IS
  SELECT dp.org_name
        ,dp.org_description
        ,dp.job_title
        ,dp.brief_description
        ,dp.detailed_description
        ,dp.job_requirements
        ,dp.additional_details
        ,dp.how_to_apply
        ,dp.image_url
        ,dp.image_url_alt
  FROM   irc_default_postings_vl dp
  WHERE  dp.job_id = cp_job_id;
  
  CURSOR csr_org_def
    (cp_organization_id IN number)
  IS
  SELECT dp.org_name
        ,dp.org_description
        ,dp.job_title
        ,dp.brief_description
        ,dp.detailed_description
        ,dp.job_requirements
        ,dp.additional_details
        ,dp.how_to_apply
        ,dp.image_url
        ,dp.image_url_alt
  FROM   irc_default_postings_vl dp
  WHERE  dp.organization_id = cp_organization_id;
  
BEGIN

  IF gc_log.proc=1 THEN 
    gc_log.procedure_start(c_method);
  END IF;

  p_vacancy.posting_org_name             := NULL;
  p_vacancy.posting_org_description      := NULL;
  p_vacancy.posting_job_title            := NULL;
  p_vacancy.posting_brief_description    := NULL;
  p_vacancy.posting_detailed_description := NULL;
  p_vacancy.posting_job_requirements     := NULL;
  p_vacancy.posting_additional_details   := NULL;
  p_vacancy.posting_how_to_apply         := NULL;
  p_vacancy.image_url                    := NULL;
  p_vacancy.image_url_alt                := NULL;
  
 
  --
  -- Position level
  --
  IF p_vacancy.position_id IS NOT NULL THEN
    OPEN  csr_pos_def(p_vacancy.position_id);
    FETCH csr_pos_def INTO p_vacancy.posting_org_name
                          ,p_vacancy.posting_org_description
                          ,p_vacancy.posting_job_title
                          ,p_vacancy.posting_brief_description
                          ,p_vacancy.posting_detailed_description
                          ,p_vacancy.posting_job_requirements
                          ,p_vacancy.posting_additional_details
                          ,p_vacancy.posting_how_to_apply
                          ,p_vacancy.image_url
                          ,p_vacancy.image_url_alt;  
						
    CLOSE csr_pos_def;
  END IF;
  
  --
  -- Job level
  --
  IF p_vacancy.job_id IS NOT NULL THEN
    OPEN  csr_job_def(p_vacancy.job_id);
    FETCH csr_job_def INTO l_org_name
                          ,l_org_description
                          ,l_job_title
                          ,l_brief_description
                          ,l_detailed_description
                          ,l_job_requirements
                          ,l_additional_details
                          ,l_how_to_apply
                          ,l_image_url
                          ,l_image_url_alt;  
    CLOSE csr_job_def;
    --
    -- Set out parameters if not already set at lower level
    --
    IF p_vacancy.posting_org_name IS NULL THEN
      p_vacancy.posting_org_name := l_org_name;
    END IF;
    IF nullif(length(p_vacancy.posting_org_description),0) IS NULL THEN
      p_vacancy.posting_org_description := l_org_description;
    END IF;
    IF p_vacancy.posting_job_title IS NULL THEN
      p_vacancy.posting_job_title := l_job_title;
    END IF;
    IF nullif(length(p_vacancy.posting_brief_description),0) IS NULL THEN
      p_vacancy.posting_brief_description := l_brief_description;
    END IF;
    IF nullif(length(p_vacancy.posting_detailed_description) ,0)IS NULL THEN
      p_vacancy.posting_detailed_description := l_detailed_description;
    END IF;
    IF nullif(length(p_vacancy.posting_job_requirements),0) IS NULL THEN
      p_vacancy.posting_job_requirements := l_job_requirements;
    END IF;
    IF nullif(length(p_vacancy.posting_additional_details),0) IS NULL THEN
      p_vacancy.posting_additional_details := l_additional_details;
    END IF;
    IF nullif(length(p_vacancy.posting_how_to_apply),0) IS NULL THEN
      p_vacancy.posting_how_to_apply := l_how_to_apply;
    END IF;
    IF nullif(length(p_vacancy.image_url),0) IS NULL THEN
      p_vacancy.image_url := l_image_url;
    END IF;
    IF nullif(length(p_vacancy.image_url_alt),0) IS NULL THEN
      p_vacancy.image_url_alt := l_image_url_alt;
    END IF;
  END IF;
  
  --
  -- Org level
  --
  IF p_vacancy.organization_id IS NOT NULL THEN
    OPEN  csr_org_def(p_vacancy.organization_id);
    FETCH csr_org_def INTO l_org_name
                          ,l_org_description
                          ,l_job_title
                          ,l_brief_description
                          ,l_detailed_description
                          ,l_job_requirements
                          ,l_additional_details
                          ,l_how_to_apply
                          ,l_image_url
                          ,l_image_url_alt; 
    CLOSE csr_org_def;
    --
    -- Set out parameters if not already set at lower level
    --
    IF p_vacancy.posting_org_name IS NULL THEN
      p_vacancy.posting_org_name := l_org_name;
    END IF;
    IF nullif(length(p_vacancy.posting_org_description),0) IS NULL THEN
      p_vacancy.posting_org_description := l_org_description;
    END IF;
    IF p_vacancy.posting_job_title IS NULL THEN
      p_vacancy.posting_job_title := l_job_title;
    END IF;
    IF nullif(length(p_vacancy.posting_brief_description),0) IS NULL THEN
      p_vacancy.posting_brief_description := l_brief_description;
    END IF;
    IF nullif(length(p_vacancy.posting_detailed_description),0) IS NULL THEN
      p_vacancy.posting_detailed_description := l_detailed_description;
    END IF;
    IF nullif(length(p_vacancy.posting_job_requirements),0) IS NULL THEN
      p_vacancy.posting_job_requirements := l_job_requirements;
    END IF;
    IF nullif(length(p_vacancy.posting_additional_details),0) IS NULL THEN
      p_vacancy.posting_additional_details := l_additional_details;
    END IF;
    IF nullif(length(p_vacancy.posting_how_to_apply),0) IS NULL THEN
      p_vacancy.posting_how_to_apply := l_how_to_apply;
    END IF;
    IF nullif(length(p_vacancy.image_url),0) IS NULL THEN
      p_vacancy.image_url := l_image_url;
    END IF;
    IF nullif(length(p_vacancy.image_url_alt),0) IS NULL THEN
      p_vacancy.image_url_alt := l_image_url_alt;
    END IF;
  END IF;
  
  --
  -- BG level
  --
  IF p_vacancy.business_group_id IS NOT NULL THEN
    OPEN  csr_org_def(p_vacancy.business_group_id);
    FETCH csr_org_def INTO l_org_name
                          ,l_org_description
                          ,l_job_title
                          ,l_brief_description
                          ,l_detailed_description
                          ,l_job_requirements
                          ,l_additional_details
                          ,l_how_to_apply
                          ,l_image_url
                          ,l_image_url_alt;
    CLOSE csr_org_def;
    --
    -- Set out parameters if not already set at lower level
    --
    IF p_vacancy.posting_org_name IS NULL THEN
      p_vacancy.posting_org_name := l_org_name;
    END IF;
    IF nullif(length(p_vacancy.posting_org_description),0) IS NULL THEN
      p_vacancy.posting_org_description := l_org_description;
    END IF;
    IF p_vacancy.posting_job_title IS NULL THEN
      p_vacancy.posting_job_title := l_job_title;
    END IF;
    IF nullif(length(p_vacancy.posting_brief_description),0) IS NULL THEN
      p_vacancy.posting_brief_description := l_brief_description;
    END IF;
    IF nullif(length(p_vacancy.posting_detailed_description),0) IS NULL THEN
      p_vacancy.posting_detailed_description := l_detailed_description;
    END IF;
    IF nullif(length(p_vacancy.posting_job_requirements),0) IS NULL THEN
      p_vacancy.posting_job_requirements := l_job_requirements;
    END IF;
    IF nullif(length(p_vacancy.posting_additional_details),0) IS NULL THEN
      p_vacancy.posting_additional_details := l_additional_details;
    END IF;
    IF nullif(length(p_vacancy.posting_how_to_apply),0) IS NULL THEN
      p_vacancy.posting_how_to_apply := l_how_to_apply;
    END IF;
    IF nullif(length(p_vacancy.image_url),0) IS NULL THEN
      p_vacancy.image_url := l_image_url;
    END IF;
    IF nullif(length(p_vacancy.image_url_alt),0) IS NULL THEN
      p_vacancy.image_url_alt := l_image_url_alt;
    END IF;
  END IF;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

END default_posting;

--
-- ------------------------------------------------------------------------ *
-- Name    : default_competences
-- ------------------------------------------------------------------------ *
PROCEDURE default_competences 
  (p_vacancy IN OUT NOCOPY xxas_per_all_vacancy)
IS

  c_method      CONSTANT  VARCHAR2(30) := 'default_competences'; 
  l_competence_element_id number;
  
  CURSOR csr_competences
  IS
  SELECT xxas_per_competence_ele
           (null
           ,1
           ,pce.type
           ,xxas_util_pkg.decode_lookup('COMPETENCE_ELEMENT_TYPE', pce.type)
           ,pce.competence_id
           ,pc.name
           ,pce.proficiency_level_id
           ,prlmin.name
           ,pce.high_proficiency_level_id
           ,prlmax.name
           ,pce.mandatory
           ,xxas_util_pkg.decode_lookup('IRC_SKILL_DESIRABILITY', pce.mandatory)
           ,pce.attribute_category
           ,pce.attribute1
           ,pce.attribute2
           ,pce.attribute3
           ,pce.attribute4
           ,pce.attribute5
           ,pce.attribute6
           ,pce.attribute7
           ,pce.attribute8
           ,pce.attribute9
           ,pce.attribute10
           ,pce.attribute11
           ,pce.attribute12
           ,pce.attribute13
           ,pce.attribute14
           ,pce.attribute15
           ,pce.attribute16
           ,pce.attribute17
           ,pce.attribute18
           ,pce.attribute19
           ,pce.attribute20
           ,sysdate
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,sysdate) competence
  FROM   per_competence_elements pce
        ,hr_lookups hrlct
        ,hr_lookups hrlmand
        ,per_competences pc
        ,per_rating_levels_vl prlmin
        ,per_rating_levels_vl prlmax
        ,per_rating_levels_vl prldes
  WHERE ((pce.position_id IS NOT NULL AND
          pce.position_id = nvl(p_vacancy.position_id, pce.position_id))
      OR (pce.job_id IS NOT NULL AND
          pce.job_id = nvl(p_vacancy.job_id, pce.job_id))
      OR (pce.organization_id IS NOT NULL AND
          pce.organization_id = nvl(p_vacancy.organization_id, pce.organization_id))
      OR (pce.enterprise_id IS NOT NULL AND
          pce.enterprise_id = nvl(p_vacancy.business_group_id, pce.enterprise_id))
        )
  AND    trunc(sysdate) BETWEEN
         nvl(pce.effective_date_from, trunc(sysdate)) AND
         nvl(pce.effective_date_to,   trunc(sysdate))
  AND    hrlct.lookup_type = 'COMPETENCE_ELEMENT_TYPE'
  AND    hrlct.lookup_code = pce.type
  AND    hrlmand.lookup_type = 'IRC_SKILL_DESIRABILITY'
  AND    hrlmand.lookup_code = pce.mandatory
  AND    pce.competence_id = pc.competence_id(+)
  AND    pce.proficiency_level_id = prlmin.rating_level_id(+)
  AND    pce.high_proficiency_level_id = prlmax.rating_level_id(+)
  AND    pce.rating_level_id = prldes.rating_level_id(+)
  ORDER BY pce.mandatory DESC, pc.name;
  
  l_competences xxas_per_competence_ele_tbl;
  
BEGIN

  IF gc_log.proc=1 THEN 
    gc_log.procedure_start(c_method);
  END IF;
  
  IF p_vacancy IS NULL THEN
    RETURN;
  END IF;
  
  l_competences := p_vacancy.competence_elements;
  
  --
  -- Clear existing competences
  --
  IF l_competences.count > 0 THEN
    l_competences.DELETE;
  END IF;
  
  --
  -- Get all competences
  --
  FOR csr_rec IN csr_competences LOOP
    l_competences.extend;
    l_competences(l_competences.count) := csr_rec.competence;  
    SELECT per_competence_elements_s.nextval
    INTO   l_competence_element_id
    FROM   dual;
    l_competences(l_competences.count).competence_element_id := l_competence_element_id;
  END LOOP;
  
  p_vacancy.competence_elements := l_competences;
  
  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;  

END default_competences;

-- ------------------------------------------------------------------------ *
-- Name    : populate_approvers_cb
-- ------------------------------------------------------------------------ *
PROCEDURE populate_approvers_cb
  (p_instance_pk1_value IN            varchar2
  ,p_instance_pk2_value IN            varchar2
  ,p_instance_pk3_value IN            varchar2
  ,p_instance_pk4_value IN            varchar2  
  ,p_instance_pk5_value IN            varchar2 
  ,p_event              IN            varchar2
  ,p_ui                 IN OUT NOCOPY xxas_emp_ui_view_type)
IS
  
  l_old_vacancy     xxas_per_all_vacancy;
  l_new_vacancy     xxas_per_all_vacancy;
  l_cur_vacancy_xml XMLType;
  l_new_vacancy_xml XMLType;
  l_rec_team_members xxas_irc_rec_team_member_tbl;
  l_rec_team_id        Number;
  l_rec_team_member_id Number;
  l_party_id           Number;
  l_update_allowed     Varchar2(30);
  l_delete_allowed     Varchar2(30);
  c_method      CONSTANT VARCHAR2(30) := 'populate_approvers_cb';
  
  CURSOR csr_party_id
    (cp_person_id IN number)
  IS
  SELECT papf.party_id
  FROM   per_all_people_f papf
  WHERE  papf.person_id = cp_person_id
  AND    trunc(sysdate) BETWEEN
         papf.effective_start_date AND papf.effective_end_date;
BEGIN

  IF gc_log.proc=1 THEN
    gc_log.procedure_start_params(c_method, p_instance_pk1_value, p_instance_pk2_value, p_instance_pk3_value, p_instance_pk4_value, p_instance_pk5_value, p_event);
  END IF;

  --
  -- Get the vacancy from the table as it current is
  --
  l_old_vacancy := xxas_vacancy_pkg.vacancy_to_obj(to_number(p_instance_pk1_value));
  l_cur_vacancy_xml := xxas_vacancy_pkg.obj_to_xml(l_old_vacancy);

  --
  -- Copy the latest form data to the vacancy object
  --
  xxas_emp_view_ui_pkg.copy_form_to_data
    (p_source_schema   => p_ui
    ,p_target_data_xml => l_cur_vacancy_xml);

  --
  -- Get the Updated Object
  --
  l_new_vacancy := xxas_vacancy_pkg.xml_to_obj(l_cur_vacancy_xml);


  -- For Team Member
  l_rec_team_members := xxas_irc_rec_team_member_tbl();
  l_rec_team_members := l_new_vacancy.team_members;

  IF l_rec_team_members.count > 0 THEN
    --
    -- Get all team details
    --
    FOR i IN 1..l_rec_team_members.count LOOP
      l_rec_team_member_id := l_rec_team_members(i).rec_team_member_id;
      l_update_allowed     := l_rec_team_members(i).update_allowed;
      l_delete_allowed     := l_rec_team_members(i).delete_allowed;
      IF l_rec_team_member_id IS NULL THEN

        SELECT irc_rec_team_members_s.nextval
        INTO   l_rec_team_id
        FROM   dual;

        OPEN  csr_party_id(l_rec_team_members(i).person_id);
        FETCH csr_party_id INTO l_party_id;
        CLOSE csr_party_id;

        l_new_vacancy.team_members(i).update_allowed        := NVL(l_update_allowed,'N');
        l_new_vacancy.team_members(i).delete_allowed        := NVL(l_delete_allowed,'N');
        l_new_vacancy.team_members(i).rec_team_member_id    := l_rec_team_id;
       -- l_new_vacancy.team_members(i).party_id              := l_party_id;
        l_new_vacancy.team_members(i).object_version_number := 1;
        l_new_vacancy.team_members(i).last_update_date      := sysdate;
        l_new_vacancy.team_members(i).last_updated_by       := fnd_global.user_id;
        l_new_vacancy.team_members(i).last_update_login     := fnd_global.login_id;
        l_new_vacancy.team_members(i).created_by            := fnd_global.user_id;
        l_new_vacancy.team_members(i).creation_date         := sysdate;

      END IF;

    END LOOP;
  END IF;
 -- Commented as this is handled in obj_to_hrapi_doc
  --
  -- Update the transaction
  --
  l_new_vacancy.hr_api_status_code      := 'S';
  l_new_vacancy.workflow_status_code    := xxas_selfservice_workflow_pkg.transcode_status
                                            (l_new_vacancy.hr_api_status_code);
  l_new_vacancy.workflow_status_meaning := xxas_selfservice_workflow_pkg.decode_status
                                            (l_new_vacancy.workflow_status_code);

  IF l_new_vacancy.transaction_id IS NULL THEN
    --
    -- Generate a new transaction id for the Update operation
    --
    SELECT hr_api_transactions_s.nextval
    INTO   l_new_vacancy.transaction_id
    FROM   dual;

    l_new_vacancy.item_type        := g_ITEM_TYPE;
    l_new_vacancy.transaction_type := 'UPDATE';
    create_hrapi_transaction(l_new_vacancy);

  ELSIF l_new_vacancy.transaction_id IS NOT NULL AND l_new_vacancy.transaction_type = 'UPDATE' THEN
    --
    -- Update Transaction for Update operation
    --
    l_new_vacancy.item_type        := g_ITEM_TYPE;
    update_hrapi_transaction(l_new_vacancy);

  ELSE
    --
    -- Update Transaction for Insert operation
    --
    l_new_vacancy.item_type        := g_ITEM_TYPE;
    l_new_vacancy.transaction_type := 'INSERT';
    update_hrapi_transaction(l_new_vacancy);

  END IF;

  --
  -- Default the approvers
  --
  l_new_vacancy.approvers :=  xxas_selfservice_workflow_pkg.get_approvers(l_new_vacancy.transaction_id,g_AME_APP_ID,g_AME_TRANS_TYPE);

  IF l_new_vacancy.approvers IS NULL THEN
    l_new_vacancy.approvers := xxas_approver_tbl();
  END IF;

  --
  -- Copy the updated data back to the vacancy
  --
  l_new_vacancy_xml := xxas_vacancy_pkg.obj_to_xml(l_new_vacancy);

  --
  -- Copy back the XXX Form Data
  --
  xxas_emp_view_ui_pkg.copy_form_to_custom_data(p_source_schema   => p_ui
                                               ,p_target_data_xml => l_new_vacancy_xml);

  p_ui := xxas_vacancy_pkg.get_form_schema
            (p_vacancy_id         => l_new_vacancy.vacancy_id
            ,p_person_id          => l_new_vacancy.manager_id
            ,p_business_group_id  => l_new_vacancy.business_group_id
            ,p_form_data_xml      => l_new_vacancy_xml);
  
 /* xxas_emp_view_ui_pkg.copy_data_to_form
    (p_source_data_xml => l_new_vacancy_xml
    ,p_target_schema   => p_ui); */

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END populate_approvers_cb;

-- ------------------------------------------------------------------------ *
-- Name    : vacancy_ctx
-- ------------------------------------------------------------------------ *
PROCEDURE vacancy_ctx
  (p_object_id IN            varchar2
  ,p_keys      IN OUT NOCOPY xxas_com_keys_type)
IS

  l_PROC CONSTANT varchar2(30) := 'vacancy_ctx';

BEGIN

  RETURN;
  
END vacancy_ctx;	

-- ------------------------------------------------------------------------ *
-- Name    : vacancy_and_logon_per_ctx
-- ------------------------------------------------------------------------ *
PROCEDURE vacancy_and_logon_per_ctx
  (p_object_id IN            varchar2
  ,p_keys      IN OUT NOCOPY xxas_com_keys_type)
IS

  l_PROC CONSTANT varchar2(30) := 'vacancy_and_logon_per_ctx';

BEGIN

  IF p_object_id IS NOT NULL THEN
    p_keys.set_key('VACANCY_ID', p_object_id);
    p_keys.set_key('PERSON_ID', fnd_global.employee_id);
  END IF;
  
END vacancy_and_logon_per_ctx;

-- ------------------------------------------------------------------------ *
-- Name    : vacancy_posting_name
-- ------------------------------------------------------------------------ *
FUNCTION vacancy_posting_name
  (p_vacancy_id IN number) RETURN varchar2
IS

  l_PROC CONSTANT varchar2(30) := 'vacancy_posting_name';
  
  CURSOR csr_posting_name IS
  SELECT p.name
  FROM   per_all_vacancies v
        ,irc_posting_contents_vl p
  WHERE  v.vacancy_Id = p_vacancy_id
  AND    v.primary_posting_id = p.posting_content_id;

  l_posting_name irc_posting_contents_vl.name%TYPE;
  
BEGIN

  IF p_vacancy_id IS NULL THEN
    RETURN null;
  END IF;
  
  OPEN  csr_posting_name;
  FETCH csr_posting_name INTO l_posting_name;
  CLOSE csr_posting_name;
  
  RETURN l_posting_name;
  
END vacancy_posting_name;
-- ------------------------------------------------------------------------ *
-- Name    : vacancy_change_cb
-- ------------------------------------------------------------------------ *
PROCEDURE vacancy_change_cb
  (p_instance_pk1_value IN            varchar2
  ,p_instance_pk2_value IN            varchar2
  ,p_instance_pk3_value IN            varchar2
  ,p_instance_pk4_value IN            varchar2  
  ,p_instance_pk5_value IN            varchar2 
  ,p_event              IN            varchar2
  ,p_ui                 IN OUT NOCOPY xxas_emp_ui_view_type)
IS

  c_method           CONSTANT varchar2(30) := 'vacancy_change_cb';
  l_old_vacancy               xxas_per_all_vacancy;
  l_new_vacancy               xxas_per_all_vacancy;
  l_cur_vacancy_xml           XMLType;
  l_new_vacancy_xml           XMLType;
  l_posting_brief_description CLOB;
  l_posting_how_to_apply      CLOB;
  l_doc_root                  varchar2(400);
  l_node_xpath                varchar2(400);

BEGIN

  IF gc_log.proc=1 THEN 
    gc_log.procedure_start(c_method);
  END IF;
  --
  -- Get the vacancy from the table as it current is
  --
  l_old_vacancy := xxas_vacancy_pkg.vacancy_to_obj(to_number(p_instance_pk1_value));

  l_cur_vacancy_xml := xxas_vacancy_pkg.obj_to_xml(l_old_vacancy);

  --
  -- Copy the latest form data to the vacancy object
  --
  xxas_emp_view_ui_pkg.copy_form_to_data
    (p_source_schema   => p_ui
    ,p_target_data_xml => l_cur_vacancy_xml);

    
  --
  -- Get the Updated Object
  --
  l_new_vacancy := xxas_vacancy_pkg.xml_to_obj(l_cur_vacancy_xml);

  --
  -- Default the posting content
  --
  default_posting             (l_new_vacancy);
  --default_competences(l_new_vacancy);
--default_competences(l_new_vacancy);




  --
  -- Copy the updated data back to the vacancy
  --
  l_new_vacancy_xml := xxas_vacancy_pkg.obj_to_xml(l_new_vacancy);


  l_doc_root := l_new_vacancy_xml.getRootElement();
  log_statement(c_method,'l_doc_root: '||l_doc_root);
  l_node_xpath  := '/' || l_doc_root;


  p_ui := xxas_vacancy_pkg.get_form_schema
            (p_vacancy_id         => l_new_vacancy.vacancy_id
            ,p_person_id          => l_new_vacancy.manager_id
            ,p_business_group_id  => l_new_vacancy.business_group_id
            ,p_form_data_xml      => l_new_vacancy_xml);

 /* xxas_emp_view_ui_pkg.copy_data_to_form
    (p_source_data_xml => l_new_vacancy_xml
    ,p_target_schema   => p_ui); */

  IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END vacancy_change_cb;

-- ------------------------------------------------------------------------ *
-- Name    : is_vacancy_auto
-- ------------------------------------------------------------------------ *
FUNCTION is_vacancy_auto RETURN number IS
  CURSOR c_get_irec_auto IS
     SELECT ORG_INFORMATION1
       FROM hr_organization_information
      WHERE organization_id = fnd_profile.value ( 'PER_BUSINESS_GROUP_ID' )
        AND ORG_INFORMATION_CONTEXT = 'BG Recruitment';
  l_auto varchar2(10);
BEGIN
  OPEN c_get_irec_auto;
  FETCH c_get_irec_auto
       INTO l_auto;
  CLOSE c_get_irec_auto;
  IF l_auto = 'ID' THEN
    RETURN 0;
  ELSE
    RETURN 1;
  END IF;
EXCEPTION
  WHEN others THEN
    RETURN - 1;
END is_vacancy_auto;
-- ------------------------------------------------------------------------ *
-- Name    : is_grade_valid
-- ------------------------------------------------------------------------ *
FUNCTION is_grade_valid (p_grade_id IN number,
        p_position_id IN number) RETURN varchar2 IS
  CURSOR c_is_grade_valid IS
     SELECT 'Y'
       FROM per_valid_grades pvg
      WHERE pvg.grade_id = p_grade_id
        AND pvg.position_id = p_position_id;
  l_valid varchar2(1) DEFAULT 'Y';
BEGIN
  OPEN c_is_grade_valid;
  FETCH c_is_grade_valid
       INTO l_valid;
  IF c_is_grade_valid%NOTFOUND THEN
    l_valid := 'N';
  END IF;
  CLOSE c_is_grade_valid;
  RETURN l_valid;
END is_grade_valid;
-- ------------------------------------------------------------------------ *
-- Name    : get_position_defaults
-- ------------------------------------------------------------------------ *
FUNCTION get_position_defaults (p_position_id IN number,
        p_org_id IN OUT number,
        p_location_id IN OUT number,
        p_job_id IN OUT number,
        p_grade_id IN OUT number) RETURN number IS
  CURSOR c_get_position_rec IS
     SELECT *
       FROM hr_all_positions_f
      WHERE position_id = p_position_id
        AND trunc ( sysdate )
            BETWEEN effective_start_date
                AND effective_end_date;
  l_position_rec c_get_position_rec%ROWTYPE;
BEGIN
  OPEN c_get_position_rec;
  FETCH c_get_position_rec
       INTO l_position_rec;
  CLOSE c_get_position_rec;
  p_org_id := nvl ( l_position_rec.organization_id,
                    p_org_id );
  p_location_id := nvl ( l_position_rec.location_id,
                         p_location_id );
  p_job_id := nvl ( l_position_rec.job_id,
                    p_job_id );
  RETURN 0;
EXCEPTION
  WHEN others THEN
    RETURN - 1;
END get_position_defaults;

-- ------------------------------------------------------------------------ *
-- Name    : get_vacancy_applicant_status
-- ------------------------------------------------------------------------ *
FUNCTION get_vacancy_applicant_status(p_vacancy_id   IN NUMBER,
                                      p_asg_status   IN VARCHAR2) RETURN NUMBER
IS
l_num   number;
CURSOR csr_get_status
IS
SELECT count(*)
from (SELECT   CASE WHEN paaf.effective_end_date < trunc(sysdate) THEN
              'TERM_APL'
              WHEN paaf.effective_end_date = paaf.effective_start_date THEN
              'TERM_APL'
              ELSE past.per_system_status
         END per_system_status
FROM     per_all_assignments_f paaf
        ,per_assignment_status_types past
WHERE    paaf.vacancy_id = p_vacancy_id
AND      paaf.assignment_type = 'A'
AND      paaf.assignment_status_type_id = past.assignment_status_type_id
AND      paaf.effective_start_date =
                 (SELECT max(paaf2.effective_start_date)
                  FROM   per_all_assignments_f paaf2
                  WHERE  paaf2.assignment_id = paaf.assignment_id)
)
where per_system_status = p_asg_status;

CURSOR csr_get_int_status
IS
SELECT count(*)
from (SELECT   CASE WHEN paaf.effective_end_date < trunc(sysdate) THEN
              'TERM_APL'
              WHEN paaf.effective_end_date = paaf.effective_start_date THEN
              'TERM_APL'
              ELSE past.per_system_status
         END per_system_status
FROM     per_all_assignments_f paaf
        ,per_assignment_status_types past
WHERE    paaf.vacancy_id = p_vacancy_id
AND      paaf.assignment_type = 'A'
AND      paaf.assignment_status_type_id = past.assignment_status_type_id
AND      paaf.effective_start_date =
                 (SELECT max(paaf2.effective_start_date)
                  FROM   per_all_assignments_f paaf2
                  WHERE  paaf2.assignment_id = paaf.assignment_id)
)
where per_system_status like 'INTERVIEW%';

CURSOR csr_get_all_status
IS
SELECT count(*)
from (SELECT   CASE WHEN paaf.effective_end_date < trunc(sysdate) THEN
              'TERM_APL'
              WHEN paaf.effective_end_date = paaf.effective_start_date THEN
              'TERM_APL'
              ELSE past.per_system_status
         END per_system_status
FROM     per_all_assignments_f paaf
        ,per_assignment_status_types past
WHERE    paaf.vacancy_id = p_vacancy_id
AND      paaf.assignment_type = 'A'
AND      paaf.assignment_status_type_id = past.assignment_status_type_id
AND      paaf.effective_start_date =
                 (SELECT max(paaf2.effective_start_date)
                  FROM   per_all_assignments_f paaf2
                  WHERE  paaf2.assignment_id = paaf.assignment_id)
);
--where per_system_status <> 'TERM_APL';

BEGIN
  IF p_asg_status = 'ALL' 
  THEN
     OPEN csr_get_all_status;
     FETCH csr_get_all_status INTO l_num;
     CLOSE csr_get_all_status;  
  ELSIF p_asg_status = 'INTERVIEW' 
  THEN
     OPEN csr_get_int_status;
     FETCH csr_get_int_status INTO l_num;
     CLOSE csr_get_int_status;  
  ELSE
     OPEN csr_get_status;
     FETCH csr_get_status INTO l_num;
     CLOSE csr_get_status;
  END IF;
  return l_num;
END;

-- ------------------------------------------------------------------------ *
-- Name    : subnav_header
-- ------------------------------------------------------------------------ *
FUNCTION subnav_header
  (p_subnav_menu_id IN number
  ,p_keys           IN xxas_com_keys_type) RETURN varchar2
IS

  l_PROC CONSTANT  varchar2(30) := 'subnav_header';
  l_vacancy_id     number;
  l_person_id      number;
  l_transaction_id number;
  l_path           varchar2(400);

BEGIN

  IF p_keys IS NOT NULL THEN
    l_person_id      := p_keys.get_key('PERSON_ID');
    l_vacancy_id     := p_keys.get_key('VACANCY_ID');
    l_transaction_id := p_keys.get_key('TRANSACTION_ID');
  END IF;

  IF l_person_id > 0 and l_vacancy_id > 0 
  THEN
    l_path := 'vacancy/' ||l_vacancy_id||'/'||l_person_id || '/menu-context';
  END IF;

  RETURN l_path;

END subnav_header;

-- ------------------------------------------------------------------------ *
-- Name    : assignment_status_value_set
-- ------------------------------------------------------------------------ *
FUNCTION assignment_status_value_set
  (p_applicant_asg_id IN number) RETURN xxas_emp_ui_valueset_type
IS

  l_PROC CONSTANT  varchar2(30) := 'assignment_status_value_set';
  l_values         xxas_emp_ui_value_table := xxas_emp_ui_value_table();
  l_value          xxas_emp_ui_value_type;
  
  CURSOR csr_rec_values
  IS
  SELECT assignment_status_type_id id,
         assignment_status value
  FROM   xxas_vacancy_applicants_v
  WHERE  assignment_id = p_applicant_asg_id;
  
BEGIN

  IF p_applicant_asg_id is not null THEN
     
     FOR csr_rec in csr_rec_values
     LOOP
        l_value := xxas_emp_ui_value_type
                           (csr_rec.id
                           ,csr_rec.value
                           ,csr_rec.value
                           ,null);
              l_values.EXTEND;
              l_values(l_values.COUNT) := l_value;
     END LOOP;
     return xxas_emp_ui_valueset_type(l_values);
  END IF;
  return null;
END assignment_status_value_set;

-- ------------------------------------------------------------------------ *
-- Name    : interested_value_set
-- ------------------------------------------------------------------------ *
FUNCTION interested_value_set RETURN xmltype
IS
  l_xml     xmltype;
BEGIN

  SELECT XMLElement
                                      ("values",
                                       XMLAttributes('true' AS "json:force-array",
                                                     'http://json.org/' AS "xmlns:json"),
                                         XMLAgg
                                          (XMLElement
                                           ("value",
                                             XMLForest(v.id "id", 
                                                       CASE v.meaning
                                                            WHEN 'Interested' THEN 'Yes'
                                                            WHEN 'Unsure' THEN 'Maybe' 
                                                            WHEN 'Not Specified' THEN 'Unscreened'
                                                            ELSE v.meaning
                                                        END "label",
                                                        CASE v.meaning
                                                             WHEN 'Interested' THEN 'balanced'
                                                             WHEN 'Unsure' THEN 'energized' 
                                                             WHEN 'Not Specified' THEN '#C0C0C0'
                                                             WHEN 'No' THEN 'assertive'
                                                             ELSE '#C0C0C0'
                                                         END "color"
                                                        )
                                             )
                                           )
                                       )   into l_xml
                            FROM   TABLE(xxas_com_vset_pkg.get_values
                                             ((select flex_value_set_id 
                                               from   fnd_flex_value_sets
                                               where  flex_value_set_name = 'Applaud Applicant Interested'), null,null, null)) v;

  return l_xml;
END interested_value_set;

PROCEDURE post_create_vacancy
  (p_function_id    IN   NUMBER
  ,p_form_data_xml  IN   XMLType
  ,p_user_submitted IN   VARCHAR2 DEFAULT 'N'
  ,p_response_xml   OUT  XMLType)
IS
  c_method                  CONSTANT VARCHAR2(30) := 'post_create_vacancy';

  l_data                    xxas_emp_view_ui_pkg.xxas_data_table;
  l_person_id               NUMBER;
  l_vacancy_id              NUMBER;
  l_transaction_id          NUMBER;
  l_vacancy_name            per_all_vacancies.name%TYPE;
  l_date_from               Varchar2(20);
  l_date_to                 varchar2(20);
  l_no_of_openings          NUMBER;
  l_budget_value            number;
  l_bg_id                   NUMBER;
  l_manager_id              NUMBER;
  l_position_id             NUMBER;
  l_job_id                  NUMBER;
  l_org_id                  NUMBER;
  l_location_id             NUMBER;
  l_grade_id                NUMBER;
  l_prof_area               varchar2(100);
  l_emp_flag                varchar2(1);
  l_cont_flag               varchar2(1);
  l_emp_cat                 varchar2(80);

  l_attribute_category      varchar2(4000);
  l_attribute1              varchar2(4000);
  l_attribute2              varchar2(4000);
  l_attribute3              varchar2(4000);
  l_attribute4              varchar2(4000);
  l_attribute5              varchar2(4000);
  l_attribute6              varchar2(4000);
  l_attribute7              varchar2(4000);
  l_attribute8              varchar2(4000);
  l_attribute9              varchar2(4000);
  l_attribute10             varchar2(4000);
  l_attribute11             varchar2(4000);
  l_attribute12             varchar2(4000);
  l_attribute13             varchar2(4000);
  l_attribute14             varchar2(4000);
  l_attribute15             varchar2(4000);
  l_attribute16             varchar2(4000);
  l_attribute17             varchar2(4000);
  l_attribute18             varchar2(4000);
  l_attribute19             varchar2(4000);
  l_attribute20             varchar2(4000);
  l_attribute21             varchar2(4000);
  l_attribute22             varchar2(4000);
  l_attribute23             varchar2(4000);
  l_attribute24             varchar2(4000);
  l_attribute25             varchar2(4000);
  l_attribute26             varchar2(4000);
  l_attribute27             varchar2(4000);
  l_attribute28             varchar2(4000);
  l_attribute29             varchar2(4000);
  l_attribute30             varchar2(4000); 

  l_min_sal                 number;
  l_max_sal                 number;
  l_sal_currency            varchar2(20);
  l_budget_type_code        varchar2(20);
  l_sal_per_code            varchar2(20);

  l_travel_per              number;
  l_work_at_home            varchar2(30);
  l_min_qual_level          varchar2(10);
  l_max_qual_level          varchar2(10);
  l_assessment_id           number;
  l_sec_method              varchar2(100);
  l_recruiter_id            number;

  l_posting_name            varchar2(1000);
  l_posting_job_title       varchar2(1000);
  l_posting_org_name        varchar2(1000);
  l_posting_org_desc        varchar2(4000);
  l_posting_brief_desc      varchar2(4000);
  l_posting_det_desc        varchar2(4000);
  l_posting_job_req         varchar2(4000);
  l_posting_addl_det        varchar2(4000);
  l_posting_how_to_apply    varchar2(4000);
  l_image_url               varchar2(400);
  l_display_rec_info        varchar2(10);

  l_ipc_attribute_category  varchar2(400);
  l_ipc_attribute1          varchar2(400);
  l_ipc_attribute2          varchar2(400);
  l_ipc_attribute3          varchar2(400);
  l_ipc_attribute4          varchar2(400);
  l_ipc_attribute5          varchar2(400);
  l_ipc_attribute6          varchar2(400);
  l_ipc_attribute7          varchar2(400);
  l_ipc_attribute8          varchar2(400);
  l_ipc_attribute9          varchar2(400);
  l_ipc_attribute10         varchar2(400);
  l_ipc_attribute11         varchar2(400);
  l_ipc_attribute12         varchar2(400);
  l_ipc_attribute13         varchar2(400);
  l_ipc_attribute14         varchar2(400);
  l_ipc_attribute15         varchar2(400);
  l_ipc_attribute16         varchar2(400);
  l_ipc_attribute17         varchar2(400);
  l_ipc_attribute18         varchar2(400);
  l_ipc_attribute19         varchar2(400);
  l_ipc_attribute20         varchar2(400);
  l_ipc_attribute21         varchar2(400);
  l_ipc_attribute22         varchar2(400);
  l_ipc_attribute23         varchar2(400);
  l_ipc_attribute24         varchar2(400);
  l_ipc_attribute25         varchar2(400);
  l_ipc_attribute26         varchar2(400);
  l_ipc_attribute27         varchar2(400);
  l_ipc_attribute28         varchar2(400);
  l_ipc_attribute29         varchar2(400);
  l_ipc_attribute30         varchar2(400);

  l_recruiter_full_name     varchar2(400);
  l_recruiter_email         varchar2(400);
  l_recruiter_work_telephone varchar2(400);

  l_isc_attribute_category  varchar2(400);
  l_isc_attribute1          varchar2(400);
  l_isc_attribute2          varchar2(400);
  l_isc_attribute3          varchar2(400);
  l_isc_attribute4          varchar2(400);
  l_isc_attribute5          varchar2(400);
  l_isc_attribute6          varchar2(400);
  l_isc_attribute7          varchar2(400);
  l_isc_attribute8          varchar2(400);
  l_isc_attribute9          varchar2(400);
  l_isc_attribute10         varchar2(400);
  l_isc_attribute11         varchar2(400);
  l_isc_attribute12         varchar2(400);
  l_isc_attribute13         varchar2(400);
  l_isc_attribute14         varchar2(400);
  l_isc_attribute15         varchar2(400);
  l_isc_attribute16         varchar2(400);
  l_isc_attribute17         varchar2(400);
  l_isc_attribute18         varchar2(400);
  l_isc_attribute19         varchar2(400);
  l_isc_attribute20         varchar2(400);
  l_isc_attribute21         varchar2(400);
  l_isc_attribute22         varchar2(400);
  l_isc_attribute23         varchar2(400);
  l_isc_attribute24         varchar2(400);
  l_isc_attribute25         varchar2(400);
  l_isc_attribute26         varchar2(400);
  l_isc_attribute27         varchar2(400);
  l_isc_attribute28         varchar2(400);
  l_isc_attribute29         varchar2(400);
  l_isc_attribute30         varchar2(400);

  l_id_flex_num             varchar2(100);
  l_segment1                varchar2(100);
  l_segment2                varchar2(100);
  l_segment3                varchar2(100);
  l_segment4                varchar2(100);
  l_segment5                varchar2(100);
  l_segment6                varchar2(100);
  l_segment7                varchar2(100);
  l_segment8                varchar2(100);
  l_segment9                varchar2(100);
  l_segment10               varchar2(100);
  l_segment11               varchar2(100);
  l_segment12               varchar2(100);
  l_segment13               varchar2(100);
  l_segment14               varchar2(100);
  l_segment15               varchar2(100);
  l_segment16               varchar2(100);
  l_segment17               varchar2(100);
  l_segment18               varchar2(100);
  l_segment19               varchar2(100);
  l_segment20               varchar2(100);
  l_segment21               varchar2(100);
  l_segment22               varchar2(100);
  l_segment23               varchar2(100);
  l_segment24               varchar2(100);
  l_segment25               varchar2(100);
  l_segment26               varchar2(100);
  l_segment27               varchar2(100);
  l_segment28               varchar2(100);
  l_segment29               varchar2(100);
  l_segment30               varchar2(100);

  l_attachments             xxas_attachment_table;
  l_competences             xxas_per_competence_ele_tbl;
  l_rec_activity            xxas_per_rec_activity_tbl;
  l_team_member             xxas_irc_rec_team_member_tbl;
  l_agency_vacancy          xxas_irc_agency_vacancy_tbl;
  l_variable_comp_ele       xxas_irc_variable_comp_ele_tbl;
  l_tasks                   xxas_vacancy_task_tbl;

  l_bg_name                 varchar2(400);
  l_pos_name                varchar2(400);
  l_job_name                varchar2(400);
  l_grade_name              varchar2(400);
  l_org_name                varchar2(400);
  l_location_code           varchar2(400);

  l_requisition_id          number;
  l_posting_content_id      number;
  l_search_criteria_id      number;
  l_vacancy                 xxas_per_all_vacancy;

  l_competence_xml          XMLType;
  l_var_comp_ele_xml        XMLType;
  l_team_members_xml        XMLType;
  l_agency_vac_xml          XMLType;
  l_rec_activities_xml      XMLType;
  l_attachments_xml         XMLType;

  CURSOR csr_data
  IS
  SELECT r.nam field_name
        ,nvl(r.val1, nvl(r.val2, r.val3)) val
        ,r.column_value field_xml
  FROM (SELECT x.column_value.getRootElement() nam
              ,extract(x.column_value, '/*/text()').getCLOBVal() val1
              ,extract(x.column_value, '/*/*/text()').getCLOBVal() val2
              ,extract(x.column_value, '/*/*/id/text()').getCLOBVal() val3
              ,x.column_value
        FROM  TABLE(XMLSequence
                     (extract
                       (p_form_data_xml
                       ,'//*'
                     || decode('Y'
                              ,'Y', '[not(ancestor-or-self::rows) '
                                  , '[not(self::rows) ')
                     || ' and not(self::root)
                          and not(self::postType)
                          and not(self::submittedFieldId)
                          and not(self::value)
                          and not(self::id)
                          and not(self::label)
                          and not(self::description)]'))) x) r;

  CURSOR csr_get_bg_name(p_bg_id NUMBER)
  IS
  SELECT name 
  FROM per_business_groups_perf
  WHERE business_group_id = p_bg_id;

  CURSOR csr_get_pos_name(p_pos_id NUMBER)
  IS
  SELECT name 
  FROM hr_all_positions_f_vl
  WHERE position_id = p_pos_id
  AND   trunc(sysdate) BETWEEN
        effective_start_date(+) AND effective_end_date(+);

  CURSOR csr_get_job_name(p_job_id NUMBER)
  IS
  SELECT name 
  FROM per_jobs_vl
  WHERE job_id = p_job_id;

  CURSOR csr_get_grade_name(p_grade_id NUMBER)
  IS
  SELECT name 
  FROM per_grades_vl
  WHERE grade_id = p_grade_id;

  CURSOR csr_get_org_name(p_org_id NUMBER)
  IS
  SELECT name 
  FROM hr_all_organization_units_vl
  WHERE organization_id = p_org_id;

  CURSOR csr_get_loc_code(p_loc_id NUMBER)
  IS
  SELECT location_code 
  FROM hr_locations_all_tl
  WHERE location_id = p_loc_id
  AND   language(+) = userenv('LANG');

  CURSOR csr_get_emp_full_name(p_person_id NUMBER) IS
  SELECT full_name, email_address, work_telephone
  FROM   per_all_people_f
  WHERE  person_id = p_person_id
  AND    trunc(sysdate) BETWEEN 
         effective_start_date AND effective_end_date;

  CURSOR csr_get_ids(p_transaction_id NUMBER)
  IS
  SELECT xxas_xml_node_util_pkg.get_node_value(hat.transaction_id,'RequisitionId','//PerAllVacanciesEORow/'),
         xxas_xml_node_util_pkg.get_node_value(hat.transaction_id,'PrimaryPostingId','//PerAllVacanciesEORow/'),
         xxas_xml_node_util_pkg.get_node_value(hat.transaction_id,'SearchCriteriaId','//IrcVacancySearchCriteriaEORow/')
  FROM   hr_api_transactions hat
  WHERE  hat.transaction_id = p_transaction_id;

  CURSOR csr_var_comp IS
  SELECT replace(x.var_comp_code,'"') var_comp_code
  FROM   XMLTable
          ('//VARIABLE_COMP_ELEMENTS/rows'
          PASSING  l_var_comp_ele_xml
          COLUMNS  xml             XMLType          path '.'
                  ,var_comp_code   varchar2(30)     path '//VARIABLE_COMP_CODE/value/id'
          )x;

  CURSOR csr_per_comp IS
  SELECT x.comp_ele_id               comp_ele_id,
        replace(x.type_code,'"')     type_code,
        x.competence_id              competence_id,
		x.prof_level_id              prof_level_id,
        x.high_prof_level_id         high_prof_level_id,
		replace(x.mand_code,'"')     mand_code,
        replace(x.att_cat,'"')       att_cat,
		replace(x.att1,'"')          att1,
        replace(x.att2,'"')          att2,
		replace(x.att3,'"')          att3,
        replace(x.att4,'"')          att4,
		replace(x.att5,'"')          att5,
        replace(x.att6,'"')          att6,
		replace(x.att7,'"')          att7,
        replace(x.att8,'"')          att8,
		replace(x.att9,'"')          att9,
        replace(x.att10,'"')         att10,
		replace(x.att11,'"')         att11,
        replace(x.att12,'"')         att12,
		replace(x.att13,'"')         att13,
        replace(x.att14,'"')         att14,
		replace(x.att15,'"')         att15,
        replace(x.att16,'"')         att16,
		replace(x.att17,'"')         att17,
        replace(x.att18,'"')         att18,
		replace(x.att19,'"')         att19,
        replace(x.att20,'"')         att20
 FROM xmltable('//COMPETENCE_ELEMENTS/rows'
          PASSING l_competence_xml
		  COLUMNS  xml             XMLType          path '.'
		          ,comp_ele_id     number           path 'COMPETENCE_ELEMENT_ID/value'
		          ,type_code       VARCHAR2(100)    path 'TYPE_CODE/value'
				  ,competence_id   number           path 'COMPETENCE_ID/value/id'
				  ,prof_level_id   number           path 'PROFICIENCY_LEVEL_ID/value/id'
				  ,high_prof_level_id number        path 'HIGH_PROFICIENCY_LEVEL_ID/value/id'
				  ,mand_code       varchar2(10)     path 'MANDATORY_CODE/value/id'
				  ,att_cat         varchar2(100)    path 'ATTRIBUTE_CATEGORY/value'
				  ,att1            varchar2(100)    path 'ATTRIBUTE1/value'
				  ,att2            varchar2(100)    path 'ATTRIBUTE2/value'
				  ,att3            varchar2(100)    path 'ATTRIBUTE3/value'
				  ,att4            varchar2(100)    path 'ATTRIBUTE4/value'
				  ,att5            varchar2(100)    path 'ATTRIBUTE5/value'
				  ,att6            varchar2(100)    path 'ATTRIBUTE6/value'
				  ,att7            varchar2(100)    path 'ATTRIBUTE7/value'
				  ,att8            varchar2(100)    path 'ATTRIBUTE8/value'
				  ,att9            varchar2(100)    path 'ATTRIBUTE9/value'
				  ,att10           varchar2(100)    path 'ATTRIBUTE10/value'
				  ,att11           varchar2(100)    path 'ATTRIBUTE11/value'
				  ,att12           varchar2(100)    path 'ATTRIBUTE12/value'
				  ,att13           varchar2(100)    path 'ATTRIBUTE13/value'
				  ,att14           varchar2(100)    path 'ATTRIBUTE14/value'
				  ,att15           varchar2(100)    path 'ATTRIBUTE15/value'
				  ,att16           varchar2(100)    path 'ATTRIBUTE16/value'
				  ,att17           varchar2(100)    path 'ATTRIBUTE17/value'
				  ,att18           varchar2(100)    path 'ATTRIBUTE18/value'
				  ,att19           varchar2(100)    path 'ATTRIBUTE19/value'
				  ,att20           varchar2(100)    path 'ATTRIBUTE20/value'
		    )x;

  CURSOR csr_agency_vacancy IS
  SELECT x.agency_id                                           agency_id
        ,x.max_allowed_app                                     max_allowed_app
        ,fnd_date.canonical_to_date(replace(x.start_date,'"')) start_date
        ,fnd_date.canonical_to_date(replace(x.end_date,'"'))   end_date
        ,replace(x.manage_app_allowed,'"')                     manage_app_allowed
  FROM xmltable('//AGENCY_VACANCIES/rows'
          PASSING l_agency_vac_xml
		  COLUMNS  xml             XMLType          path '.'
                  ,agency_id       number           path 'AGENCY_ID/value/id'
                  ,max_allowed_app number           path 'MAX_ALLOWED_APPLICANTS/value'
                  ,start_date      varchar2(30)     path 'START_DATE/value'
                  ,end_date        varchar2(30)     path 'END_DATE/value'
                  ,manage_app_allowed  varchar2(10) path 'MANAGE_APPLICANTS_ALLOWED/value/id'                 
          )x;

  CURSOR csr_team_member IS
  SELECT x.person_id                                           person_id
        ,fnd_date.canonical_to_date(replace(x.start_date,'"')) start_date
        ,fnd_date.canonical_to_date(replace(x.end_date,'"'))   end_date
        ,replace(x.update_allowed,'"')                         update_allowed
        ,replace(x.delete_allowed,'"')                         delete_allowed
  FROM xmltable('//TEAM_MEMBERS/rows'
          PASSING l_team_members_xml
		  COLUMNS  xml             XMLType          path '.'
                  ,person_id       number           path 'PERSON_ID/value/id'
                  ,start_date      varchar2(30)     path 'START_DATE/value'
                  ,end_date        varchar2(30)     path 'END_DATE/value'
                  ,update_allowed  varchar2(10)     path 'UPDATE_ALLOWED/value/id'                 
                  ,delete_allowed  varchar2(10)     path 'DELETE_ALLOWED/value/id'
          )x;

  CURSOR csr_rec_activity IS
  SELECT trunc(sysdate) start_date
        ,fnd_date.canonical_to_date(replace(x.end_date,'"'))   end_date
        ,replace(x.rec_site_name,'"')                          site_name
  FROM xmltable('//RECRUITMENT_ACTIVITIES/rows'
          PASSING l_rec_activities_xml
		  COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path 'DATE_START/value'
                  ,end_date        varchar2(30)     path 'DATE_END/value'
                  ,rec_site_name   varchar2(400)    path 'RECRUITING_SITE_NAME/value'
          )x;
          
  CURSOR csr_vacancy_attachments IS
  SELECT replace(x.title,'"')                       title
        ,replace(x.description,'"')                 description
        ,x.file_id                                  file_id
        ,replace(x.entity_name,'"')                 entity_name
  FROM xmltable('//ATTACHMENTS/rows'
          PASSING l_attachments_xml
		  COLUMNS  xml             XMLType          path '.'
                  ,title           varchar2(100)    path 'TITLE/value'
                  ,description     varchar2(1000)   path 'DESCRIPTION/value'
                  ,file_id         number           path 'FILE_ID/value' 
                  ,entity_name     varchar2(100)     path 'ENTITY_NAME/value'           
          )x;

  CURSOR csr_attchment_exists
  IS
  Select 'Y'
  from fnd_attached_documents
  where entity_name in ('IRC_EXT_VAC_APPROVED', 'IRC_INT_VAC_APPROVED','IRC_EXT_VAC','IRC_INT_VAC')
  and  pk1_value = to_char(l_vacancy_id);

  l_var_comp_ele   xxas_irc_variable_comp_ele;
  l_per_comp_ele   XXAS_PER_COMPETENCE_ELE;
  l_irc_agency_vac XXAS_IRC_AGENCY_VACANCY;
  l_irc_team_mem   XXAS_IRC_REC_TEAM_MEMBER;
  l_per_rec_act    XXAS_PER_REC_ACTIVITY;
  l_attach_typ     xxas_attachment_type;
  l_recruitment_activity_id  per_recruitment_activities.recruitment_activity_id%TYPE;
  l_exists         varchar2(5) := 'N';

BEGIN
IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;

l_data := xxas_emp_view_ui_pkg.transform_jsonxml_form_data
              (p_form_data_xml);

  log_statement(c_method,'form data'||substr(p_form_data_xml.getstringval(),1,3800));
  log_statement(c_method,'form data'||substr(p_form_data_xml.getstringval(),3801,7600));
  --
  -- Retrieve the form values
  --

  IF l_data.exists('PERSON_ID') THEN
    l_person_id := l_data('PERSON_ID');
  END IF;

  IF l_data.exists('VACANCY_ID') THEN
    l_vacancy_id := l_data('VACANCY_ID');
  END IF;

  IF l_data.exists('TRANSACTION_ID') THEN
    l_transaction_id := l_data('TRANSACTION_ID');
  END IF;

  IF l_data.exists('VACANCY_NAME') THEN
    l_vacancy_name := l_data('VACANCY_NAME');
  END IF;

  IF l_data.exists('DATE_FROM') THEN
    l_date_from := l_data('DATE_FROM');
  END IF;

  IF l_data.exists('DATE_TO') THEN
    l_date_to := l_data('DATE_TO');
  END IF;

  IF l_data.exists('NUMBER_OF_OPENINGS') THEN
    l_no_of_openings := l_data('NUMBER_OF_OPENINGS');
  END IF;

  IF l_data.exists('BUDGET_VALUE') THEN
    l_budget_value := l_data('BUDGET_VALUE');
  END IF;

  IF l_data.exists('BUSINESS_GROUP_ID') THEN
    l_bg_id := l_data('BUSINESS_GROUP_ID');
  END IF;

  IF l_data.exists('MANAGER_ID') THEN
    l_manager_id := l_data('MANAGER_ID');
  END IF;

  IF l_data.exists('POSITION_ID') THEN
    l_position_id := l_data('POSITION_ID');
  END IF;

  IF l_data.exists('JOB_ID') THEN
    l_job_id := l_data('JOB_ID');
  END IF;

  IF l_data.exists('LOCATION_ID') THEN
    l_location_id := l_data('LOCATION_ID');
  END IF;

  IF l_data.exists('ORGANIZATION_ID') THEN
    l_org_id := l_data('ORGANIZATION_ID');
  END IF;

  IF l_data.exists('GRADE_ID') THEN
    l_grade_id := l_data('GRADE_ID');
  END IF;

  IF l_data.exists('PROFESSIONAL_AREA_CODE') THEN
    l_prof_area := l_data('PROFESSIONAL_AREA_CODE');
  END IF;

  IF l_data.exists('EMPLOYEE_FLAG') THEN
    l_emp_flag := l_data('EMPLOYEE_FLAG');
  END IF;

  IF l_data.exists('CONTRACTOR_FLAG') THEN
    l_cont_flag := l_data('CONTRACTOR_FLAG');
  END IF;

  IF l_data.exists('EMPLOYMENT_CATEGORY') THEN
    l_emp_cat := l_data('EMPLOYMENT_CATEGORY');
  END IF;

  IF l_data.exists('ATTRIBUTE_CATEGORY') THEN
    l_attribute_category := l_data('ATTRIBUTE_CATEGORY');
  END IF;

  IF l_data.exists('ATTRIBUTE1') THEN
    l_attribute1 := l_data('ATTRIBUTE1');
  END IF;

  IF l_data.exists('ATTRIBUTE2') THEN
    l_attribute2 := l_data('ATTRIBUTE2');
  END IF;

  IF l_data.exists('ATTRIBUTE3') THEN
    l_attribute3 := l_data('ATTRIBUTE3');
  END IF;

  IF l_data.exists('ATTRIBUTE4') THEN
    l_attribute4 := l_data('ATTRIBUTE4');
  END IF;

  IF l_data.exists('ATTRIBUTE5') THEN
    l_attribute5 := l_data('ATTRIBUTE5');
  END IF;

  IF l_data.exists('ATTRIBUTE6') THEN
    l_attribute6 := l_data('ATTRIBUTE6');
  END IF;

  IF l_data.exists('ATTRIBUTE7') THEN
    l_attribute7 := l_data('ATTRIBUTE7');
  END IF;

  IF l_data.exists('ATTRIBUTE8') THEN
    l_attribute8 := l_data('ATTRIBUTE8');
  END IF;

  IF l_data.exists('ATTRIBUTE9') THEN
    l_attribute9 := l_data('ATTRIBUTE9');
  END IF;

  IF l_data.exists('ATTRIBUTE10') THEN
    l_attribute10 := l_data('ATTRIBUTE10');
  END IF;

  IF l_data.exists('ATTRIBUTE11') THEN
    l_attribute11 := l_data('ATTRIBUTE11');
  END IF;

  IF l_data.exists('ATTRIBUTE12') THEN
    l_attribute12 := l_data('ATTRIBUTE12');
  END IF;

  IF l_data.exists('ATTRIBUTE13') THEN
    l_attribute13 := l_data('ATTRIBUTE13');
  END IF;

  IF l_data.exists('ATTRIBUTE14') THEN
    l_attribute14 := l_data('ATTRIBUTE14');
  END IF;

  IF l_data.exists('ATTRIBUTE15') THEN
    l_attribute15 := l_data('ATTRIBUTE15');
  END IF;

  IF l_data.exists('ATTRIBUTE16') THEN
    l_attribute16 := l_data('ATTRIBUTE16');
  END IF;

  IF l_data.exists('ATTRIBUTE17') THEN
    l_attribute17 := l_data('ATTRIBUTE17');
  END IF;

  IF l_data.exists('ATTRIBUTE18') THEN
    l_attribute18 := l_data('ATTRIBUTE18');
  END IF;

  IF l_data.exists('ATTRIBUTE19') THEN
    l_attribute19 := l_data('ATTRIBUTE19');
  END IF;

  IF l_data.exists('ATTRIBUTE20') THEN
    l_attribute20 := l_data('ATTRIBUTE20');
  END IF;

  IF l_data.exists('ATTRIBUTE21') THEN
    l_attribute21 := l_data('ATTRIBUTE21');
  END IF;

  IF l_data.exists('ATTRIBUTE22') THEN
    l_attribute22 := l_data('ATTRIBUTE22');
  END IF;

  IF l_data.exists('ATTRIBUTE23') THEN
    l_attribute23 := l_data('ATTRIBUTE23');
  END IF;

  IF l_data.exists('ATTRIBUTE24') THEN
    l_attribute24 := l_data('ATTRIBUTE24');
  END IF;

  IF l_data.exists('ATTRIBUTE25') THEN
    l_attribute25 := l_data('ATTRIBUTE25');
  END IF;

  IF l_data.exists('ATTRIBUTE26') THEN
    l_attribute26 := l_data('ATTRIBUTE26');
  END IF;

  IF l_data.exists('ATTRIBUTE27') THEN
    l_attribute27 := l_data('ATTRIBUTE27');
  END IF;

  IF l_data.exists('ATTRIBUTE28') THEN
    l_attribute28 := l_data('ATTRIBUTE28');
  END IF;

  IF l_data.exists('ATTRIBUTE29') THEN
    l_attribute29 := l_data('ATTRIBUTE29');
  END IF;

  IF l_data.exists('ATTRIBUTE30') THEN
    l_attribute30 := l_data('ATTRIBUTE30');
  END IF;

  IF l_data.exists('MIN_SALARY') THEN
    l_min_sal := l_data('MIN_SALARY');
  END IF;

  IF l_data.exists('MAX_SALARY') THEN
    l_max_sal := l_data('MAX_SALARY');
  END IF;

  IF l_data.exists('SALARY_CURRENCY') THEN
    l_sal_currency := l_data('SALARY_CURRENCY');
  END IF;

  IF l_data.exists('BUDGET_TYPE_CODE') THEN
    l_budget_type_code := l_data('BUDGET_TYPE_CODE');
  END IF;

  IF l_data.exists('SALARY_PERIOD_CODE') THEN
    l_sal_per_code := l_data('SALARY_PERIOD_CODE');
  END IF;

  IF l_data.exists('TRAVEL_PERCENTAGE') THEN
    l_travel_per := l_data('TRAVEL_PERCENTAGE');
  END IF;

  IF l_data.exists('WORK_AT_HOME') THEN
    l_work_at_home := l_data('WORK_AT_HOME');
  END IF;

  IF l_data.exists('MIN_QUAL_LEVEL_ID') THEN
    l_min_qual_level := l_data('MIN_QUAL_LEVEL_ID');
  END IF;

  IF l_data.exists('MAX_QUAL_LEVEL_ID') THEN
    l_max_qual_level := l_data('MAX_QUAL_LEVEL_ID');
  END IF;

  IF l_data.exists('ASSESSMENT_ID') THEN
    l_assessment_id := l_data('ASSESSMENT_ID');
  END IF;

  IF l_data.exists('SECURITY_METHOD') THEN
    l_sec_method := l_data('SECURITY_METHOD');
  END IF;

  IF l_data.exists('RECRUITER_ID') THEN
    l_recruiter_id := l_data('RECRUITER_ID');
  END IF;

  IF l_data.exists('POSTING_NAME') THEN
    l_posting_name := l_data('POSTING_NAME');
  END IF;

  IF l_data.exists('POSTING_JOB_TITLE') THEN
    l_posting_job_title := l_data('POSTING_JOB_TITLE');
  END IF;

  IF l_data.exists('POSTING_ORG_NAME') THEN
    l_posting_org_name := l_data('POSTING_ORG_NAME');
  END IF;

  IF l_data.exists('POSTING_ORG_DESCRIPTION') THEN
    l_posting_org_desc := l_data('POSTING_ORG_DESCRIPTION');
  END IF;

  IF l_data.exists('POSTING_BRIEF_DESCRIPTION') THEN
    l_posting_brief_desc := l_data('POSTING_BRIEF_DESCRIPTION');
  END IF;

  IF l_data.exists('POSTING_DETAILED_DESCRIPTION') THEN
    l_posting_det_desc := l_data('POSTING_DETAILED_DESCRIPTION');
  END IF;

  IF l_data.exists('POSTING_JOB_REQUIREMENTS') THEN
    l_posting_job_req := l_data('POSTING_JOB_REQUIREMENTS');
  END IF;

  IF l_data.exists('POSTING_ADDITIONAL_DETAILS') THEN
    l_posting_addl_det := l_data('POSTING_ADDITIONAL_DETAILS');
  END IF;

  IF l_data.exists('POSTING_HOW_TO_APPLY') THEN
    l_posting_how_to_apply := l_data('POSTING_HOW_TO_APPLY');
  END IF;

  IF l_data.exists('IMAGE_URL') THEN
    l_image_url := l_data('IMAGE_URL');
  END IF;

  IF l_data.exists('DISPLAY_RECRUITER_INFO') THEN
    l_display_rec_info := l_data('DISPLAY_RECRUITER_INFO');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE_CATEGORY') THEN
    l_ipc_attribute_category := l_data('IPC_ATTRIBUTE_CATEGORY');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE1') THEN
    l_ipc_attribute1 := l_data('IPC_ATTRIBUTE1');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE2') THEN
    l_ipc_attribute2 := l_data('IPC_ATTRIBUTE2');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE3') THEN
    l_ipc_attribute3 := l_data('IPC_ATTRIBUTE3');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE4') THEN
    l_ipc_attribute4 := l_data('IPC_ATTRIBUTE4');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE5') THEN
    l_ipc_attribute5 := l_data('IPC_ATTRIBUTE5');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE6') THEN
    l_ipc_attribute6 := l_data('IPC_ATTRIBUTE6');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE7') THEN
    l_ipc_attribute7 := l_data('IPC_ATTRIBUTE7');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE8') THEN
    l_ipc_attribute8 := l_data('IPC_ATTRIBUTE8');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE9') THEN
    l_ipc_attribute9 := l_data('IPC_ATTRIBUTE9');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE10') THEN
    l_ipc_attribute10 := l_data('IPC_ATTRIBUTE10');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE11') THEN
    l_ipc_attribute11 := l_data('IPC_ATTRIBUTE11');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE12') THEN
    l_ipc_attribute12 := l_data('IPC_ATTRIBUTE12');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE13') THEN
    l_ipc_attribute13 := l_data('IPC_ATTRIBUTE13');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE14') THEN
    l_ipc_attribute14 := l_data('IPC_ATTRIBUTE14');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE15') THEN
    l_ipc_attribute15 := l_data('IPC_ATTRIBUTE15');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE16') THEN
    l_ipc_attribute16 := l_data('IPC_ATTRIBUTE16');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE17') THEN
    l_ipc_attribute17 := l_data('IPC_ATTRIBUTE17');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE18') THEN
    l_ipc_attribute18 := l_data('IPC_ATTRIBUTE18');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE19') THEN
    l_ipc_attribute19 := l_data('IPC_ATTRIBUTE19');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE20') THEN
    l_ipc_attribute20 := l_data('IPC_ATTRIBUTE20');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE21') THEN
    l_ipc_attribute21 := l_data('IPC_ATTRIBUTE21');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE22') THEN
    l_ipc_attribute22 := l_data('IPC_ATTRIBUTE22');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE23') THEN
    l_ipc_attribute23 := l_data('IPC_ATTRIBUTE23');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE24') THEN
    l_ipc_attribute24 := l_data('IPC_ATTRIBUTE24');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE25') THEN
    l_ipc_attribute25 := l_data('IPC_ATTRIBUTE25');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE26') THEN
    l_ipc_attribute26 := l_data('IPC_ATTRIBUTE26');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE27') THEN
    l_ipc_attribute27 := l_data('IPC_ATTRIBUTE27');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE28') THEN
    l_ipc_attribute28 := l_data('IPC_ATTRIBUTE28');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE29') THEN
    l_ipc_attribute29 := l_data('IPC_ATTRIBUTE29');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE30') THEN
    l_ipc_attribute30 := l_data('IPC_ATTRIBUTE30');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE_CATEGORY') THEN
    l_isc_attribute_category := l_data('ISC_ATTRIBUTE_CATEGORY');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE1') THEN
    l_isc_attribute1 := l_data('ISC_ATTRIBUTE1');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE2') THEN
    l_isc_attribute2 := l_data('ISC_ATTRIBUTE2');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE3') THEN
    l_isc_attribute3 := l_data('ISC_ATTRIBUTE3');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE4') THEN
    l_isc_attribute4 := l_data('ISC_ATTRIBUTE4');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE5') THEN
    l_isc_attribute5 := l_data('ISC_ATTRIBUTE5');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE6') THEN
    l_isc_attribute6 := l_data('ISC_ATTRIBUTE6');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE7') THEN
    l_isc_attribute7 := l_data('ISC_ATTRIBUTE7');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE8') THEN
    l_isc_attribute8 := l_data('ISC_ATTRIBUTE8');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE9') THEN
    l_isc_attribute9 := l_data('ISC_ATTRIBUTE9');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE10') THEN
    l_isc_attribute10 := l_data('ISC_ATTRIBUTE10');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE11') THEN
    l_isc_attribute11 := l_data('ISC_ATTRIBUTE11');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE12') THEN
    l_isc_attribute12 := l_data('ISC_ATTRIBUTE12');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE13') THEN
    l_isc_attribute13 := l_data('ISC_ATTRIBUTE13');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE14') THEN
    l_isc_attribute14 := l_data('ISC_ATTRIBUTE14');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE15') THEN
    l_isc_attribute15 := l_data('ISC_ATTRIBUTE15');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE16') THEN
    l_isc_attribute16 := l_data('ISC_ATTRIBUTE16');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE17') THEN
    l_isc_attribute17 := l_data('ISC_ATTRIBUTE17');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE18') THEN
    l_isc_attribute18 := l_data('ISC_ATTRIBUTE18');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE19') THEN
    l_isc_attribute19 := l_data('ISC_ATTRIBUTE19');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE20') THEN
    l_isc_attribute20 := l_data('ISC_ATTRIBUTE20');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE21') THEN
    l_isc_attribute21 := l_data('ISC_ATTRIBUTE21');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE22') THEN
    l_isc_attribute22 := l_data('ISC_ATTRIBUTE22');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE23') THEN
    l_isc_attribute23 := l_data('ISC_ATTRIBUTE23');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE24') THEN
    l_isc_attribute24 := l_data('ISC_ATTRIBUTE24');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE25') THEN
    l_isc_attribute25 := l_data('ISC_ATTRIBUTE25');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE26') THEN
    l_isc_attribute26 := l_data('ISC_ATTRIBUTE26');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE27') THEN
    l_isc_attribute27 := l_data('ISC_ATTRIBUTE27');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE28') THEN
    l_isc_attribute28 := l_data('ISC_ATTRIBUTE28');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE29') THEN
    l_isc_attribute29 := l_data('ISC_ATTRIBUTE29');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE30') THEN
    l_isc_attribute30 := l_data('ISC_ATTRIBUTE30');
  END IF;  

  IF l_data.exists('ID_FLEX_NUM') THEN
    l_id_flex_num := l_data('ID_FLEX_NUM');
  END IF;  

  IF l_data.exists('SEGMENT1') THEN
    l_segment1 := l_data('SEGMENT1');
  END IF;

  IF l_data.exists('SEGMENT2') THEN
    l_segment2 := l_data('SEGMENT2');
  END IF;

  IF l_data.exists('SEGMENT3') THEN
    l_segment3 := l_data('SEGMENT3');
  END IF;

  IF l_data.exists('SEGMENT4') THEN
    l_segment4 := l_data('SEGMENT4');
  END IF;

  IF l_data.exists('SEGMENT5') THEN
    l_segment5 := l_data('SEGMENT5');
  END IF;

  IF l_data.exists('SEGMENT6') THEN
    l_segment6 := l_data('SEGMENT6');
  END IF;

  IF l_data.exists('SEGMENT7') THEN
    l_segment7 := l_data('SEGMENT7');
  END IF;

  IF l_data.exists('SEGMENT8') THEN
    l_segment8 := l_data('SEGMENT8');
  END IF;

  IF l_data.exists('SEGMENT9') THEN
    l_segment9 := l_data('SEGMENT9');
  END IF;

  IF l_data.exists('SEGMENT10') THEN
    l_segment10 := l_data('SEGMENT10');
  END IF;

  IF l_data.exists('SEGMENT11') THEN
    l_segment11 := l_data('SEGMENT11');
  END IF;

  IF l_data.exists('SEGMENT12') THEN
    l_segment12 := l_data('SEGMENT12');
  END IF;

  IF l_data.exists('SEGMENT13') THEN
    l_segment13 := l_data('SEGMENT13');
  END IF;

  IF l_data.exists('SEGMENT14') THEN
    l_segment14 := l_data('SEGMENT14');
  END IF;

  IF l_data.exists('SEGMENT15') THEN
    l_segment15 := l_data('SEGMENT15');
  END IF;

  IF l_data.exists('SEGMENT16') THEN
    l_segment16 := l_data('SEGMENT16');
  END IF;

  IF l_data.exists('SEGMENT17') THEN
    l_segment17 := l_data('SEGMENT17');
  END IF;

  IF l_data.exists('SEGMENT18') THEN
    l_segment18 := l_data('SEGMENT18');
  END IF;

  IF l_data.exists('SEGMENT19') THEN
    l_segment19 := l_data('SEGMENT19');
  END IF;

  IF l_data.exists('SEGMENT20') THEN
    l_segment20 := l_data('SEGMENT20');
  END IF;

  IF l_data.exists('SEGMENT21') THEN
    l_segment21 := l_data('SEGMENT21');
  END IF;

  IF l_data.exists('SEGMENT22') THEN
    l_segment22 := l_data('SEGMENT22');
  END IF;

  IF l_data.exists('SEGMENT23') THEN
    l_segment23 := l_data('SEGMENT23');
  END IF;

  IF l_data.exists('SEGMENT24') THEN
    l_segment24 := l_data('SEGMENT24');
  END IF;

  IF l_data.exists('SEGMENT25') THEN
    l_segment25 := l_data('SEGMENT25');
  END IF;

  IF l_data.exists('SEGMENT26') THEN
    l_segment26 := l_data('SEGMENT26');
  END IF;

  IF l_data.exists('SEGMENT27') THEN
    l_segment27 := l_data('SEGMENT27');
  END IF;

  IF l_data.exists('SEGMENT28') THEN
    l_segment28 := l_data('SEGMENT28');
  END IF;

  IF l_data.exists('SEGMENT29') THEN
    l_segment29 := l_data('SEGMENT29');
  END IF;

  IF l_data.exists('SEGMENT30') THEN
    l_segment30 := l_data('SEGMENT30');
  END IF;

--  l_bg_id := xxas_per_pkg.get_business_group_id(l_person_id);

  OPEN  csr_get_emp_full_name(l_person_id);
  FETCH csr_get_emp_full_name INTO l_recruiter_full_name, l_recruiter_email, l_recruiter_work_telephone;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_bg_name(l_bg_id);
  FETCH csr_get_bg_name INTO l_bg_name;
  CLOSE csr_get_bg_name;

  OPEN  csr_get_pos_name(l_position_id);
  FETCH csr_get_pos_name INTO l_pos_name;
  CLOSE csr_get_pos_name;

  OPEN  csr_get_job_name(l_job_id);
  FETCH csr_get_job_name INTO l_job_name;
  CLOSE csr_get_job_name;

  OPEN  csr_get_grade_name(l_grade_id);
  FETCH csr_get_grade_name INTO l_grade_name;
  CLOSE csr_get_grade_name;

  OPEN  csr_get_org_name(l_org_id);
  FETCH csr_get_org_name INTO l_org_name;
  CLOSE csr_get_org_name;

  OPEN  csr_get_loc_code(l_location_id);
  FETCH csr_get_loc_code INTO l_location_code;
  CLOSE csr_get_loc_code;

  OPEN csr_get_ids(l_transaction_id);
  FETCH csr_get_ids INTO l_requisition_id, l_posting_content_id, l_search_criteria_id;
  CLOSE csr_get_ids;

  l_person_id  := l_manager_id;

  IF p_form_data_xml is not null THEN

    FOR csr_rec IN csr_data LOOP
      IF csr_rec.field_name is not null THEN
        IF csr_rec.field_name = 'COMPETENCE_ELEMENTS' THEN
           l_competence_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'VARIABLE_COMP_ELEMENTS' THEN 
           l_var_comp_ele_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'TEAM_MEMBERS' THEN
           l_team_members_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'AGENCY_VACANCIES' THEN
           l_agency_vac_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'RECRUITMENT_ACTIVITIES' THEN
           l_rec_activities_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'ATTACHMENTS' THEN
           l_attachments_xml := csr_rec.field_xml;
        END IF;
      END IF;
    END LOOP;

  END IF;

  -- For REC Activity 
  l_rec_activity := xxas_per_rec_activity_tbl();

  IF l_rec_activity.count > 0 THEN
    l_rec_activity.DELETE;
  END IF;

  --
  -- Get all per rec activity details
  --
  FOR csr_rec_act IN csr_rec_activity LOOP

    SELECT  xxas_per_rec_activity
            (per_recruitment_activities_s.nextval
            ,per_recruitment_activity_for_s.nextval
            ,l_vacancy_id
            ,csr_rec_act.start_date
            ,csr_rec_act.end_date
            ,null
            ,l_posting_content_id
            ,irs.recruiting_site_id
            ,csr_rec_act.site_name
            ,1
            ,1
            ,trunc(sysdate)
            ,fnd_global.user_id
            ,fnd_global.login_id
            ,fnd_global.user_id
            ,trunc(sysdate)
              )
       INTO l_per_rec_act
    FROM irc_all_recruiting_sites_vl irs
  WHERE  irs.site_name                = csr_rec_act.site_name;

    l_rec_activity.extend;
    l_rec_activity(l_rec_activity.count) := l_per_rec_act;
    l_recruitment_activity_id := l_rec_activity(l_rec_activity.count).recruitment_activity_id;
    l_rec_activity(l_rec_activity.count).name := substr(l_vacancy_name,1,30-length(l_recruitment_activity_id)) || l_recruitment_activity_id;
  END LOOP;

  -- For Team Member
  l_team_member := xxas_irc_rec_team_member_tbl();

  IF l_team_member.count > 0 THEN
    l_team_member.DELETE;
  END IF;

  --
  -- Get all team details
  --
  FOR csr_rec_tm IN csr_team_member LOOP

    SELECT xxas_irc_rec_team_member
           (irc_rec_team_members_s.nextval
           ,csr_rec_tm.person_id
           ,pap.party_id
           ,l_vacancy_id
           ,null
           ,csr_rec_tm.start_date
           ,csr_rec_tm.end_date
           ,csr_rec_tm.update_allowed
           ,csr_rec_tm.delete_allowed
           ,1
           ,trunc(sysdate)
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,trunc(sysdate)
         )
        INTO l_irc_team_mem
    FROM per_all_people_f pap
    WHERE pap.person_id = csr_rec_tm.person_id
    AND   trunc(sysdate) between pap.effective_start_date and pap.effective_end_date;

    l_team_member.extend;
    l_team_member(l_team_member.count) := l_irc_team_mem;  
  END LOOP;

  -- For Agency Vacancy
  l_agency_vacancy := xxas_irc_agency_vacancy_tbl();

  IF l_agency_vacancy.count > 0 THEN
    l_agency_vacancy.DELETE;
  END IF;

  --
  -- Get all agency vac dettails
  --
  FOR csr_rec_vac IN csr_agency_vacancy LOOP

    SELECT xxas_irc_agency_vacancy
            (irc_agency_vacancies_s.nextval
            ,csr_rec_vac.agency_id
            ,pov.vendor_name
            ,l_vacancy_id
            ,csr_rec_vac.start_date
            ,csr_rec_vac.end_date
            ,csr_rec_vac.max_allowed_app
            ,null -- nvl(csr_rec_vac.manage_app_allowed,'N')
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,1
            ,trunc(sysdate)
            ,fnd_global.user_id
            ,fnd_global.login_id
            ,fnd_global.user_id
            ,trunc(sysdate)
            ) INTO l_irc_agency_vac
    FROM po_vendors pov
  WHERE  csr_rec_vac.agency_id  = pov.vendor_id;

    l_agency_vacancy.extend;
    l_agency_vacancy(l_agency_vacancy.count) := l_irc_agency_vac;  
  END LOOP;

  -- For Variable Component Element
  l_variable_comp_ele := xxas_irc_variable_comp_ele_tbl();

  IF l_variable_comp_ele.count > 0 then
    l_variable_comp_ele.DELETE;
  END IF;
  --
  -- Get all variable_comp_ele
  --
  FOR csr_rec_comp IN csr_var_comp LOOP
    SELECT xxas_irc_variable_comp_ele
           (l_vacancy_id
           ,csr_rec_comp.var_comp_code
           ,hrlvc.meaning
           ,1
           ,trunc(sysdate)
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,trunc(sysdate)
           ) variable_comp_ele into l_var_comp_ele
    FROM   hr_lookups hrlvc
    WHERE  hrlvc.lookup_type = 'IRC_VARIABLE_COMP_ELEMENT'
    AND    hrlvc.lookup_code = csr_rec_comp.var_comp_code;

    l_variable_comp_ele.extend;
    l_variable_comp_ele(l_variable_comp_ele.count) := l_var_comp_ele;  
  END LOOP;

  -- For Competencies
  l_competences := xxas_per_competence_ele_tbl();

  IF l_competences.count > 0 THEN
    l_competences.DELETE;
  END IF;
  --
  -- Get all competences
  --
  FOR csr_rec IN csr_per_comp LOOP

    SELECT xxas_per_competence_ele
          (nvl(csr_rec.comp_ele_id,per_competence_elements_s.nextval)
          ,1
          ,csr_rec.type_code
          ,xxas_util_pkg.decode_lookup('COMPETENCE_ELEMENT_TYPE', csr_rec.type_code)
          ,csr_rec.competence_id
          ,pc.name
          ,csr_rec.prof_level_id
          ,null
          ,csr_rec.high_prof_level_id
          ,null
          ,csr_rec.mand_code
          ,xxas_util_pkg.decode_lookup('IRC_SKILL_DESIRABILITY', csr_rec.mand_code)
          ,csr_rec.att_cat
          ,csr_rec.att1
          ,csr_rec.att2
          ,csr_rec.att3
          ,csr_rec.att4
          ,csr_rec.att5
          ,csr_rec.att6
          ,csr_rec.att7
          ,csr_rec.att8
          ,csr_rec.att9
          ,csr_rec.att10
          ,csr_rec.att11
          ,csr_rec.att12
          ,csr_rec.att13
          ,csr_rec.att14
          ,csr_rec.att15
          ,csr_rec.att16
          ,csr_rec.att17
          ,csr_rec.att18
          ,csr_rec.att19
          ,csr_rec.att20
          ,trunc(sysdate)
          ,fnd_global.user_id
          ,fnd_global.login_id
          ,fnd_global.user_id
          ,trunc(sysdate)
          )
      INTO l_per_comp_ele
    FROM  per_competences pc
 --        ,per_rating_levels_vl prlmin
 --        ,per_rating_levels_vl prlmax
    WHERE  csr_rec.competence_id = pc.competence_id(+);
 --   AND    csr_rec.prof_level_id = prlmin.rating_level_id(+)
 --   AND    csr_rec.high_prof_level_id = prlmax.rating_level_id(+); 

    l_competences.extend;
    l_competences(l_competences.count) := l_per_comp_ele;  
  END LOOP;

  -- For Attachments

  l_attachments:= xxas_attachment_table();

  FOR csr_rec_attach in csr_vacancy_attachments
  LOOP
      SELECT xxas_attachment_type 
           (null
           ,null
           ,csr_rec_attach.entity_name 
           ,to_char (l_vacancy_id)  
           ,null
           ,null
           ,null
           ,null
           ,'MISC'  
           ,null  
           ,null
           ,null
           ,csr_rec_attach.title
           ,csr_rec_attach.description
           ,csr_rec_attach.file_id
           ,null
           ,null
           ,null
		       ,null
           ,sysdate
           ,fnd_global.user_id
           ,sysdate
           ,fnd_global.user_id
           ) INTO l_attach_typ
      FROM DUAL;
      
      l_attachments.extend;
      l_attachments(l_attachments.count) := l_attach_typ;  

  END LOOP;
    
  l_vacancy := xxas_per_all_vacancy(
            vacancy_id                     => l_vacancy_id
           ,href                           => null         
           ,vacancy_name                   => l_vacancy_name
           ,edit_allowed                   => xxas_util_pkg.flag_to_boolean(xxas_com_sec_pkg.test_instance('XXAS_DATA_TRANS_NO_PEND_VIEW', to_char(l_transaction_id)))  --edit_allowed
           ,edit_label                     => xxas_com_msg_pkg.get_string('XXAS_RB_COM_EDIT') --edit_label
           ,delete_allowed                 => xxas_util_pkg.flag_to_boolean(xxas_com_sec_pkg.test_instance('XXAS_DATA_TRANS_NO_PEND_DEL', to_char(l_transaction_id)))  --delete_allowed
           ,delete_label                   => xxas_com_msg_pkg.get_string('XXAS_RB_COM_DEL') --delete_label
           ,num_total                      => 0 --num_total
           ,num_active                     => 0 --num_active
           ,num_term                       => 0 --num_term
           ,num_interview1                 => 0 --num_interview1
           ,num_interview2                 => 0 --num_interview2
           ,num_offer                      => 0 --num_offer
           ,num_accepted                   => 0 --num_accepted
           ,num_not_set                    => 0 --num_not_set
           ,num_no                         => 0 --num_no
           ,num_unsure                     => 0 --num_unsure
           ,num_yes                        => 0 --num_yes
           ,view_schema_href               => null --view_schema_href
           ,form_schema_href               => null --form_schema_href
           ,applicants_href                => xxas_vacancy_pkg.get_applicants_href(l_vacancy_id) --applicants_href
           ,applicants_export_href         => xxas_vacancy_pkg.get_applicants_export_href(l_vacancy_id) --applicants_export_href
           ,offers_href                    => xxas_vacancy_pkg.get_offers_href(l_vacancy_id) --offers_href
           ,ad_hoc_comms_href              => xxas_vacancy_pkg.get_ad_hoc_comms_href(l_vacancy_id) --ad_hoc_comms_href           
           ,transaction_id                 => l_transaction_id
           ,transaction_type               => 'INSERT'   --'UPDATE'
           ,hr_api_status_code             => 'S'
           ,workflow_status_code           => xxas_selfservice_workflow_pkg.transcode_status('REVIEW')
           ,workflow_status_meaning        => xxas_selfservice_workflow_pkg.decode_status(xxas_selfservice_workflow_pkg.transcode_status('REVIEW'))
           ,item_type                      => g_item_type
           ,item_key                       => null
           ,business_group_id              => l_bg_id
           ,business_group_name            => l_bg_name
           ,position_id                    => l_position_id
           ,position_name                  => l_pos_name
           ,job_id                         => l_job_id
           ,job_name                       => l_job_name
           ,grade_id                       => l_grade_id
           ,grade_name                     => l_grade_name
           ,organization_id                => l_org_id
           ,organization_name              => l_org_name
           ,requisition_id                 => l_requisition_id
           ,req_object_version_number      => 1
           ,people_group_id                => null
           ,location_id                    => l_location_id
           ,location_name                  => l_location_code
           ,recruiter_id                   => l_recruiter_id
           ,date_from                      => nvl(xxas_xml_util_pkg.oafstring_to_date(l_date_from),trunc(sysdate))
           ,date_to                        => xxas_xml_util_pkg.oafstring_to_date(l_date_to)
           ,description                    => null
           ,number_of_openings             => l_budget_value   --l_no_of_openings
           ,status_code                    => 'UNAPPROVED'
           ,status_meaning                 => xxas_util_pkg.decode_lookup('VACANCY_STATUS', 'UNAPPROVED')
           ,status_color                   => null --status_color
           ,request_id                     => null --request_id
           ,program_application_id         => null --program_application_id
           ,program_id                     => null --program_id
           ,program_update_date            => null --program_update_date
           ,attribute_category             => l_attribute_category
           ,attribute1                     => l_attribute1
           ,attribute2                     => l_attribute2
           ,attribute3                     => l_attribute3
           ,attribute4                     => l_attribute4
           ,attribute5                     => l_attribute5
           ,attribute6                     => l_attribute6
           ,attribute7                     => l_attribute7
           ,attribute8                     => l_attribute8
           ,attribute9                     => l_attribute9
           ,attribute10                    => l_attribute10
           ,attribute11                    => l_attribute11
           ,attribute12                    => l_attribute12
           ,attribute13                    => l_attribute13
           ,attribute14                    => l_attribute14
           ,attribute15                    => l_attribute15
           ,attribute16                    => l_attribute16
           ,attribute17                    => l_attribute17
           ,attribute18                    => l_attribute18
           ,attribute19                    => l_attribute19
           ,attribute20                    => l_attribute20
           ,last_update_date               => trunc(sysdate)
           ,last_updated_by                => fnd_global.user_id
           ,last_update_login              => fnd_global.login_id
           ,created_by                     => fnd_global.user_id
           ,creation_date                  => trunc(sysdate)
           ,vacancy_category_code          => null
           ,vacancy_category_meaning       => null
           ,budget_type_code               => l_budget_type_code 
           ,budget_type_meaning            => xxas_util_pkg.decode_lookup('BUDGET_MEASUREMENT_TYPE', l_budget_type_code)
           ,budget_value                   => l_budget_value
           ,manager_id                     => l_person_id
           ,security_method                => l_sec_method  --'B'   --security_method
           ,object_version_number          => 1
           ,assessment_id                  => l_assessment_id
           ,primary_posting_id             => l_posting_content_id
           ,comments                       => null --comments
           ,comments_to_approver           => null
           ,display_manager_info           => nvl(xxas_util_pkg.flag_to_boolean(l_display_rec_info),'N')
           ,display_recruiter_info         => nvl(xxas_util_pkg.flag_to_boolean(l_display_rec_info),'N')
           ,posting_name                   => l_posting_name
           ,posting_org_name               => l_posting_org_name
		   ,posting_org_description        => l_posting_org_desc
		   ,posting_job_title              => l_posting_job_title
		   ,posting_brief_description      => l_posting_brief_desc
		   ,posting_detailed_description   => l_posting_det_desc
		   ,posting_job_requirements       => l_posting_job_req
		   ,posting_additional_details     => l_posting_addl_det
		   ,posting_how_to_apply           => l_posting_how_to_apply
		   ,image_url                      => l_image_url
		   ,image_url_alt                  => null
		   ,benefit_info                   => null   -- benefit_info
		   ,ipc_attribute_category         => l_ipc_attribute_category
		   ,ipc_attribute1                 => l_ipc_attribute1
		   ,ipc_attribute2                 => l_ipc_attribute2
		   ,ipc_attribute3                 => l_ipc_attribute3
		   ,ipc_attribute4                 => l_ipc_attribute4
		   ,ipc_attribute5                 => l_ipc_attribute5
		   ,ipc_attribute6                 => l_ipc_attribute6
		   ,ipc_attribute7                 => l_ipc_attribute7
		   ,ipc_attribute8                 => l_ipc_attribute8
		   ,ipc_attribute9                 => l_ipc_attribute9
		   ,ipc_attribute10                => l_ipc_attribute10
		   ,ipc_attribute11                => l_ipc_attribute11
		   ,ipc_attribute12                => l_ipc_attribute12
		   ,ipc_attribute13                => l_ipc_attribute13
		   ,ipc_attribute14                => l_ipc_attribute14
		   ,ipc_attribute15                => l_ipc_attribute15
		   ,ipc_attribute16                => l_ipc_attribute16
		   ,ipc_attribute17                => l_ipc_attribute17
		   ,ipc_attribute18                => l_ipc_attribute18
		   ,ipc_attribute19                => l_ipc_attribute19
		   ,ipc_attribute20                => l_ipc_attribute20
		   ,ipc_attribute21                => l_ipc_attribute21
		   ,ipc_attribute22                => l_ipc_attribute22
		   ,ipc_attribute23                => l_ipc_attribute23
		   ,ipc_attribute24                => l_ipc_attribute24
		   ,ipc_attribute25                => l_ipc_attribute25
		   ,ipc_attribute26                => l_ipc_attribute26
		   ,ipc_attribute27                => l_ipc_attribute27
		   ,ipc_attribute28                => l_ipc_attribute28
		   ,ipc_attribute29                => l_ipc_attribute29
		   ,ipc_attribute30                => l_ipc_attribute30
		   ,ipc_last_update_date           => sysdate
		   ,ipc_last_updated_by            => fnd_global.user_id
		   ,ipc_last_update_login          => fnd_global.login_id
		   ,ipc_created_by                 => fnd_global.user_id
		   ,ipc_creation_date              => sysdate
		   ,ipc_object_version_number      => 1
		   ,recruiter_full_name            => l_recruiter_full_name
		   ,recruiter_email                => l_recruiter_email
		   ,recruiter_work_telephone       => l_recruiter_work_telephone
		   ,manager_full_name              => l_recruiter_full_name
		   ,manager_email                  => l_recruiter_email
		   ,manager_work_telephone         => l_recruiter_work_telephone
		   ,search_criteria_id             => l_search_criteria_id
		   ,employee_flag                  => l_emp_flag
		   ,contractor_flag                => l_cont_flag
		   ,employment_category_code       => l_emp_cat
		   ,employment_category_meaning    => null
		   ,travel_percentage              => l_travel_per
		   ,travel_percentage_meaning      => null
		   ,min_salary                     => l_min_sal
		   ,max_salary                     => l_max_sal
		   ,salary_currency                => l_sal_currency
		   ,salary_period_code             => l_sal_per_code
		   ,salary_period_meaning          => null
		   ,professional_area_code         => l_prof_area
		   ,professional_area_meaning      => null
		   ,work_at_home_code              => l_work_at_home
		   ,work_at_home_meaning           => null
		   ,min_qual_level_id              => l_min_qual_level
		   ,min_qual_level_name            => null
		   ,max_qual_level_id              => l_max_qual_level
		   ,max_qual_level_name            => null
		   ,isc_attribute_category         => l_isc_attribute_category
		   ,isc_attribute1                 => l_isc_attribute1
		   ,isc_attribute2                 => l_isc_attribute2
		   ,isc_attribute3                 => l_isc_attribute3
		   ,isc_attribute4                 => l_isc_attribute4
		   ,isc_attribute5                 => l_isc_attribute5
		   ,isc_attribute6                 => l_isc_attribute6
		   ,isc_attribute7                 => l_isc_attribute7
		   ,isc_attribute8                 => l_isc_attribute8
		   ,isc_attribute9                 => l_isc_attribute9
		   ,isc_attribute10                => l_isc_attribute10
		   ,isc_attribute11                => l_isc_attribute11
		   ,isc_attribute12                => l_isc_attribute12
		   ,isc_attribute13                => l_isc_attribute13
		   ,isc_attribute14                => l_isc_attribute14
		   ,isc_attribute15                => l_isc_attribute15
		   ,isc_attribute16                => l_isc_attribute16
		   ,isc_attribute17                => l_isc_attribute17
		   ,isc_attribute18                => l_isc_attribute18
		   ,isc_attribute19                => l_isc_attribute19
		   ,isc_attribute20                => l_isc_attribute20
		   ,isc_attribute21                => l_isc_attribute21
		   ,isc_attribute22                => l_isc_attribute22
		   ,isc_attribute23                => l_isc_attribute23
		   ,isc_attribute24                => l_isc_attribute24
		   ,isc_attribute25                => l_isc_attribute25
		   ,isc_attribute26                => l_isc_attribute26
		   ,isc_attribute27                => l_isc_attribute27
		   ,isc_attribute28                => l_isc_attribute28
		   ,isc_attribute29                => l_isc_attribute29
		   ,isc_attribute30                => l_isc_attribute30
		   ,isc_object_version_number      => 1
		   ,pg_id_flex_num                 => l_id_flex_num
		   ,segment1                       => l_segment1
		   ,segment2                       => l_segment2
		   ,segment3                       => l_segment3
		   ,segment4                       => l_segment4
		   ,segment5                       => l_segment5
		   ,segment6                       => l_segment6
		   ,segment7                       => l_segment7
		   ,segment8                       => l_segment8
		   ,segment9                       => l_segment9
		   ,segment10                      => l_segment10
		   ,segment11                      => l_segment11
		   ,segment12                      => l_segment12
		   ,segment13                      => l_segment13
		   ,segment14                      => l_segment14
		   ,segment15                      => l_segment15
		   ,segment16                      => l_segment16
		   ,segment17                      => l_segment17
		   ,segment18                      => l_segment18
		   ,segment19                      => l_segment19
		   ,segment20                      => l_segment20
		   ,segment21                      => l_segment21
		   ,segment22                      => l_segment22
		   ,segment23                      => l_segment23
		   ,segment24                      => l_segment24
		   ,segment25                      => l_segment25
		   ,segment26                      => l_segment26
		   ,segment27                      => l_segment27
		   ,segment28                      => l_segment28
		   ,segment29                      => l_segment29
		   ,segment30                      => l_segment30
		   ,variable_comp_elements         => l_variable_comp_ele
		   ,competence_elements            => l_competences
		   ,agency_vacancies               => l_agency_vacancy
		   ,team_members                   => l_team_member
		   ,recruitment_activities         => l_rec_activity
		   ,vacancy_tasks                  => l_tasks
		   ,attachments                    => l_attachments
		   ,approvers                      => xxas_selfservice_workflow_pkg.get_approvers(l_transaction_id,g_AME_APP_ID,g_AME_TRANS_TYPE)
       );

--  default_posting(l_vacancy);  

  OPEN csr_attchment_exists;
  FETCH csr_attchment_exists into l_exists;
  CLOSE csr_attchment_exists;
  IF l_exists = 'N' THEN
     save_attachments(l_vacancy.attachments
                     ,l_vacancy.vacancy_id);
  END IF;
  update_hrapi_transaction(l_vacancy);

  p_response_xml := xxas_com_rest_util_pkg.app_route_redirect_response
                        (p_app_route         => 'review-vacancy/' ||
                                                to_char(l_vacancy_id) || '_' || to_char(l_transaction_id)
                        ,p_target            => 'modal'
                        ,p_persist_cache     => 'true'
                        ,p_delete_form_cache => 'false');

IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END post_create_vacancy;

-- ------------------------------------------------------------------------ *
-- Name    : review_form_handler
-- ------------------------------------------------------------------------ *
PROCEDURE review_form_handler
  (p_function_id    IN   NUMBER
  ,p_form_data_xml  IN   XMLType
  ,p_user_submitted IN   VARCHAR2 DEFAULT 'N'
  ,p_response_xml   OUT  XMLType)
IS

c_method CONSTANT VARCHAR2(30) := 'review_form_handler';

l_data                    xxas_emp_view_ui_pkg.xxas_data_table;
l_transaction_id          number;
l_vacancy                 xxas_per_all_vacancy;
l_vacancy_id              number;
l_comments_to_approver    varchar2(3000);

BEGIN

IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;

l_data := xxas_emp_view_ui_pkg.transform_jsonxml_form_data
              (p_form_data_xml);

  --
  -- Retrieve the form values
  --
  IF l_data.exists('TRANSACTION_ID') THEN
    l_transaction_id := l_data('TRANSACTION_ID');
  END IF;

  IF l_data.exists('VACANCY_ID') THEN
    l_vacancy_id := l_data('VACANCY_ID');
  END IF;

  IF l_data.exists('COMMENTS_TO_APPROVER') THEN
    l_comments_to_approver := l_data('COMMENTS_TO_APPROVER');
  END IF;
  
  l_vacancy := trans_to_obj(l_transaction_id);
  l_vacancy.comments_to_approver := l_comments_to_approver;
  
  log_statement(c_method,'name : '||l_vacancy.vacancy_name);
  log_statement(c_method,'transaction_id : '||l_vacancy.transaction_id);  
  submit_vacancy(l_vacancy);

  -- 
  -- set page attributes.
  --
  populate_page_attributes(l_vacancy_id,l_vacancy.transaction_id);  
  --
  -- Return success
  --
    p_response_xml := xxas_com_rest_util_pkg.app_route_redirect_response
                            ('vacancy-list'
                            ,null
                            ,null
                            );
                        
IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END review_form_handler;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_vacancy_workflow_details
-- ------------------------------------------------------------------------ *
FUNCTION get_vacancy_workflow_details
  (p_transaction_id IN NUMBER
  ,p_include_fyi    IN VARCHAR2 DEFAULT 'Y') RETURN XMLType
IS

  l_xml XMLTYPE;

BEGIN

  IF p_transaction_id IS NULL THEN
    RETURN null;
  END IF;

  SELECT XMLAgg
          (XMLElement
            ("item"
            ,XMLForest
              (a.origsys || ':' || a.origsys_id "id"
              ,'people' "type"
              ,a.display_name "name"
              ,decode(a.origsys, 'PER', 'person-home', null) "pageId"
              ,decode(a.origsys, 'PER', a.origsys_id, null) "objectId"
             )
           ,xxas_selfservice_workflow_pkg.get_appr_img_xml(a.origsys_id,a.origsys)
           ,XMLELEMENT
              ("chips"
              ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
              ,XMLELEMENT
              ("chip"
                ,XMLForest
                   (CASE WHEN xxas_util_pkg.encode_fnd_lookup
                           ('AME_APPROVER_CATEGORY', a.approver_category) = 'F'
                    THEN
                      xxas_util_pkg.decode_fnd_lookup
                        ('AME_APPROVER_CATEGORY', 'F')
                    WHEN a.approval_status_code = 'RFC' THEN
                      a.approval_status
                   || ': '
                   || xxas_selfservice_workflow_pkg.get_rfc_comments(p_transaction_id)
                    ELSE
                      a.approval_status
                    END	"title"
                    ,CASE  WHEN a.approval_status_code = 'ERROR'
                    THEN 'alert'
                    WHEN a.approval_status_code = 'APPROVED'
                    THEN 'check'
                    WHEN a.approval_status_code = 'RFC'
                    THEN 'step-backward'
                    WHEN a.approval_status_code = 'SFL'
                    THEN 'content-save'
                    WHEN a.approval_status_code = 'REVIEW'
                    THEN 'microscope'
                    WHEN a.approval_status_code = 'PENDING'
                    THEN 'timer-sand'
                    END  "icon"
                    ,CASE WHEN xxas_util_pkg.encode_fnd_lookup
                           ('AME_APPROVER_CATEGORY', a.approver_category) = 'F'
                    THEN 'positive'
                    WHEN a.approval_status_code = 'ERROR'
                    THEN 'assertive'
                    WHEN a.approval_status_code = 'APPROVED'
                    THEN 'balanced'
                    WHEN a.approval_status_code = 'RFC'
                    THEN 'assertive'
                    WHEN a.approval_status_code = 'SFL'
                    THEN 'positive'
                    WHEN a.approval_status_code = 'REVIEW'
                    THEN 'energized'
                    WHEN a.approval_status_code = 'PENDING'
                    THEN 'energized'
                    END  "color"
                    )
               )
             )             
           ) ORDER BY a.line_number
         ) "data"
  INTO   l_xml
  FROM   TABLE(xxas_selfservice_workflow_pkg.get_approvers
                (p_transaction_id
                ,g_AME_APP_ID
                ,g_AME_TRANS_TYPE)) a
  WHERE  (nvl(p_include_fyi, 'Y') = 'Y' OR
         (nvl(p_include_fyi, 'Y') = 'N' AND
          xxas_util_pkg.encode_fnd_lookup
            ('AME_APPROVER_CATEGORY', a.approver_category) <> 'F'))
  /*ORDER BY a.line_number*/;
  
  RETURN l_xml;
  
END get_vacancy_workflow_details;

--
-- ------------------------------------------------------------------------ *
-- Name    : post_update_vacancy
-- ------------------------------------------------------------------------ *
PROCEDURE post_update_vacancy
  (p_function_id    IN   NUMBER
  ,p_form_data_xml  IN   XMLType
  ,p_user_submitted IN   VARCHAR2 DEFAULT 'N'
  ,p_response_xml   OUT  XMLType)
IS
  c_method                  CONSTANT VARCHAR2(30) := 'post_update_vacancy';

  l_data                    xxas_emp_view_ui_pkg.xxas_data_table;
  l_person_id               NUMBER;
  l_vacancy_id              NUMBER;
  l_transaction_id          NUMBER;
  l_vacancy_name            per_all_vacancies.name%TYPE;
  l_date_from               Varchar2(20);
  l_date_to                 varchar2(20);
  l_no_of_openings          NUMBER;
  l_budget_value            number;
  l_bg_id                   NUMBER;
  l_manager_id              NUMBER;
  l_position_id             NUMBER;
  l_job_id                  NUMBER;
  l_org_id                  NUMBER;
  l_location_id             NUMBER;
  l_grade_id                NUMBER;
  l_prof_area               varchar2(100);
  l_emp_flag                varchar2(1);
  l_cont_flag               varchar2(1);
  l_emp_cat                 varchar2(80);

  l_attribute_category      varchar2(4000);
  l_attribute1              varchar2(4000);
  l_attribute2              varchar2(4000);
  l_attribute3              varchar2(4000);
  l_attribute4              varchar2(4000);
  l_attribute5              varchar2(4000);
  l_attribute6              varchar2(4000);
  l_attribute7              varchar2(4000);
  l_attribute8              varchar2(4000);
  l_attribute9              varchar2(4000);
  l_attribute10             varchar2(4000);
  l_attribute11             varchar2(4000);
  l_attribute12             varchar2(4000);
  l_attribute13             varchar2(4000);
  l_attribute14             varchar2(4000);
  l_attribute15             varchar2(4000);
  l_attribute16             varchar2(4000);
  l_attribute17             varchar2(4000);
  l_attribute18             varchar2(4000);
  l_attribute19             varchar2(4000);
  l_attribute20             varchar2(4000);
  l_attribute21             varchar2(4000);
  l_attribute22             varchar2(4000);
  l_attribute23             varchar2(4000);
  l_attribute24             varchar2(4000);
  l_attribute25             varchar2(4000);
  l_attribute26             varchar2(4000);
  l_attribute27             varchar2(4000);
  l_attribute28             varchar2(4000);
  l_attribute29             varchar2(4000);
  l_attribute30             varchar2(4000); 

  l_min_sal                 number;
  l_max_sal                 number;
  l_sal_currency            varchar2(20);
  l_budget_type_code        varchar2(20);
  l_sal_per_code            varchar2(20);

  l_travel_per              number;
  l_work_at_home            varchar2(30);
  l_min_qual_level          varchar2(10);
  l_max_qual_level          varchar2(10);
  l_assessment_id           number;
  l_sec_method              varchar2(100);
  l_recruiter_id            number;

  l_posting_name            varchar2(1000);
  l_posting_job_title       varchar2(1000);
  l_posting_org_name        varchar2(1000);
  l_posting_org_desc        varchar2(4000);
  l_posting_brief_desc      varchar2(4000);
  l_posting_det_desc        varchar2(4000);
  l_posting_job_req         varchar2(4000);
  l_posting_addl_det        varchar2(4000);
  l_posting_how_to_apply    varchar2(4000);
  l_image_url               varchar2(400);
  l_display_rec_info        varchar2(10);

  l_ipc_attribute_category  varchar2(400);
  l_ipc_attribute1          varchar2(400);
  l_ipc_attribute2          varchar2(400);
  l_ipc_attribute3          varchar2(400);
  l_ipc_attribute4          varchar2(400);
  l_ipc_attribute5          varchar2(400);
  l_ipc_attribute6          varchar2(400);
  l_ipc_attribute7          varchar2(400);
  l_ipc_attribute8          varchar2(400);
  l_ipc_attribute9          varchar2(400);
  l_ipc_attribute10         varchar2(400);
  l_ipc_attribute11         varchar2(400);
  l_ipc_attribute12         varchar2(400);
  l_ipc_attribute13         varchar2(400);
  l_ipc_attribute14         varchar2(400);
  l_ipc_attribute15         varchar2(400);
  l_ipc_attribute16         varchar2(400);
  l_ipc_attribute17         varchar2(400);
  l_ipc_attribute18         varchar2(400);
  l_ipc_attribute19         varchar2(400);
  l_ipc_attribute20         varchar2(400);
  l_ipc_attribute21         varchar2(400);
  l_ipc_attribute22         varchar2(400);
  l_ipc_attribute23         varchar2(400);
  l_ipc_attribute24         varchar2(400);
  l_ipc_attribute25         varchar2(400);
  l_ipc_attribute26         varchar2(400);
  l_ipc_attribute27         varchar2(400);
  l_ipc_attribute28         varchar2(400);
  l_ipc_attribute29         varchar2(400);
  l_ipc_attribute30         varchar2(400);

  l_recruiter_full_name     varchar2(400);
  l_recruiter_email         varchar2(400);
  l_recruiter_work_telephone varchar2(400);

  l_isc_attribute_category  varchar2(400);
  l_isc_attribute1          varchar2(400);
  l_isc_attribute2          varchar2(400);
  l_isc_attribute3          varchar2(400);
  l_isc_attribute4          varchar2(400);
  l_isc_attribute5          varchar2(400);
  l_isc_attribute6          varchar2(400);
  l_isc_attribute7          varchar2(400);
  l_isc_attribute8          varchar2(400);
  l_isc_attribute9          varchar2(400);
  l_isc_attribute10         varchar2(400);
  l_isc_attribute11         varchar2(400);
  l_isc_attribute12         varchar2(400);
  l_isc_attribute13         varchar2(400);
  l_isc_attribute14         varchar2(400);
  l_isc_attribute15         varchar2(400);
  l_isc_attribute16         varchar2(400);
  l_isc_attribute17         varchar2(400);
  l_isc_attribute18         varchar2(400);
  l_isc_attribute19         varchar2(400);
  l_isc_attribute20         varchar2(400);
  l_isc_attribute21         varchar2(400);
  l_isc_attribute22         varchar2(400);
  l_isc_attribute23         varchar2(400);
  l_isc_attribute24         varchar2(400);
  l_isc_attribute25         varchar2(400);
  l_isc_attribute26         varchar2(400);
  l_isc_attribute27         varchar2(400);
  l_isc_attribute28         varchar2(400);
  l_isc_attribute29         varchar2(400);
  l_isc_attribute30         varchar2(400);

  l_id_flex_num             varchar2(100);
  l_segment1                varchar2(100);
  l_segment2                varchar2(100);
  l_segment3                varchar2(100);
  l_segment4                varchar2(100);
  l_segment5                varchar2(100);
  l_segment6                varchar2(100);
  l_segment7                varchar2(100);
  l_segment8                varchar2(100);
  l_segment9                varchar2(100);
  l_segment10               varchar2(100);
  l_segment11               varchar2(100);
  l_segment12               varchar2(100);
  l_segment13               varchar2(100);
  l_segment14               varchar2(100);
  l_segment15               varchar2(100);
  l_segment16               varchar2(100);
  l_segment17               varchar2(100);
  l_segment18               varchar2(100);
  l_segment19               varchar2(100);
  l_segment20               varchar2(100);
  l_segment21               varchar2(100);
  l_segment22               varchar2(100);
  l_segment23               varchar2(100);
  l_segment24               varchar2(100);
  l_segment25               varchar2(100);
  l_segment26               varchar2(100);
  l_segment27               varchar2(100);
  l_segment28               varchar2(100);
  l_segment29               varchar2(100);
  l_segment30               varchar2(100);

  l_attachments             xxas_attachment_table;
  l_competences             xxas_per_competence_ele_tbl;
  l_rec_activity            xxas_per_rec_activity_tbl;
  l_team_member             xxas_irc_rec_team_member_tbl;
  l_agency_vacancy          xxas_irc_agency_vacancy_tbl;
  l_variable_comp_ele       xxas_irc_variable_comp_ele_tbl;
  l_tasks                   xxas_vacancy_task_tbl;

  l_bg_name                 varchar2(400);
  l_pos_name                varchar2(400);
  l_job_name                varchar2(400);
  l_grade_name              varchar2(400);
  l_org_name                varchar2(400);
  l_location_code           varchar2(400);

  l_requisition_id          number;
  l_posting_content_id      number;
  l_search_criteria_id      number;
  l_vacancy                 xxas_per_all_vacancy;

  l_competence_xml          XMLType;
  l_var_comp_ele_xml        XMLType;
  l_team_members_xml        XMLType;
  l_agency_vac_xml          XMLType;
  l_rec_activities_xml      XMLType;
  l_attachments_xml         XMLType;
  l_is_draft                VARCHAR2(10) := 'Y';
  l_transaction_type        VARCHAR2(10) := 'UPDATE';
  l_vacancy_exists          VARCHAR2(1);
  
  CURSOR csr_data
  IS
  SELECT r.nam field_name
        ,nvl(r.val1, nvl(r.val2, r.val3)) val
        ,r.column_value field_xml
  FROM (SELECT x.column_value.getRootElement() nam
              ,extract(x.column_value, '/*/text()').getCLOBVal() val1
              ,extract(x.column_value, '/*/*/text()').getCLOBVal() val2
              ,extract(x.column_value, '/*/*/id/text()').getCLOBVal() val3
              ,x.column_value
        FROM  TABLE(XMLSequence
                     (extract
                       (p_form_data_xml
                       ,'//*'
                     || decode('Y'
                              ,'Y', '[not(ancestor-or-self::rows) '
                                  , '[not(self::rows) ')
                     || ' and not(self::root)
                          and not(self::postType)
                          and not(self::submittedFieldId)
                          and not(self::value)
                          and not(self::id)
                          and not(self::label)
                          and not(self::description)]'))) x) r;

  CURSOR csr_get_bg_name(p_bg_id NUMBER)
  IS
  SELECT name 
  FROM per_business_groups_perf
  WHERE business_group_id = p_bg_id;

  CURSOR csr_get_pos_name(p_pos_id NUMBER)
  IS
  SELECT name 
  FROM hr_all_positions_f_vl
  WHERE position_id = p_pos_id
  AND   trunc(sysdate) BETWEEN
        effective_start_date(+) AND effective_end_date(+);

  CURSOR csr_get_job_name(p_job_id NUMBER)
  IS
  SELECT name 
  FROM per_jobs_vl
  WHERE job_id = p_job_id;

  CURSOR csr_get_grade_name(p_grade_id NUMBER)
  IS
  SELECT name 
  FROM per_grades_vl
  WHERE grade_id = p_grade_id;

  CURSOR csr_get_org_name(p_org_id NUMBER)
  IS
  SELECT name 
  FROM hr_all_organization_units_vl
  WHERE organization_id = p_org_id;

  CURSOR csr_get_loc_code(p_loc_id NUMBER)
  IS
  SELECT location_code 
  FROM hr_locations_all_tl
  WHERE location_id = p_loc_id
  AND   language(+) = userenv('LANG');

  CURSOR csr_get_emp_full_name(p_person_id NUMBER) IS
  SELECT full_name, email_address, work_telephone
  FROM   per_all_people_f
  WHERE  person_id = p_person_id
  AND    trunc(sysdate) BETWEEN 
         effective_start_date AND effective_end_date;

  CURSOR csr_get_ids(p_vacancy_id NUMBER)
  IS
  SELECT v.requisition_id,
         v.primary_posting_id,
         isc.search_criteria_id
  FROM  per_all_vacancies v
       ,irc_search_criteria isc      
  WHERE  v.vacancy_id = p_vacancy_id
  AND    v.vacancy_id = isc.object_id(+)
  AND    isc.object_type(+) = 'VACANCY' ;

  CURSOR csr_get_trans_ids(cp_transaction_id NUMBER)
  IS
  SELECT xxas_xml_node_util_pkg.get_node_value(hat.transaction_id,'RequisitionId','//PerAllVacanciesEORow/'),
         xxas_xml_node_util_pkg.get_node_value(hat.transaction_id,'PrimaryPostingId','//PerAllVacanciesEORow/'),
         xxas_xml_node_util_pkg.get_node_value(hat.transaction_id,'SearchCriteriaId','//IrcVacancySearchCriteriaEORow/')
  FROM   hr_api_transactions hat
  WHERE  hat.transaction_id = cp_transaction_id;
  
  CURSOR csr_vacancy_exists(cp_vacancy_id number) IS
  SELECT 'Y'
  FROM  per_all_vacancies v
  WHERE  v.vacancy_id = cp_vacancy_id;
  
  CURSOR csr_var_comp IS
  SELECT replace(x.var_comp_code,'"') var_comp_code
  FROM   XMLTable
          ('//VARIABLE_COMP_ELEMENTS/rows'
          PASSING  l_var_comp_ele_xml
          COLUMNS  xml             XMLType          path '.'
                  ,var_comp_code   varchar2(30)     path '//VARIABLE_COMP_CODE/value/id'
          )x;

  CURSOR csr_per_comp IS
  SELECT x.comp_ele_id               comp_ele_id,
        replace(x.type_code,'"')     type_code,
        x.competence_id              competence_id,
		x.prof_level_id              prof_level_id,
        x.high_prof_level_id         high_prof_level_id,
		replace(x.mand_code,'"')     mand_code,
        replace(x.att_cat,'"')       att_cat,
		replace(x.att1,'"')          att1,
        replace(x.att2,'"')          att2,
		replace(x.att3,'"')          att3,
        replace(x.att4,'"')          att4,
		replace(x.att5,'"')          att5,
        replace(x.att6,'"')          att6,
		replace(x.att7,'"')          att7,
        replace(x.att8,'"')          att8,
		replace(x.att9,'"')          att9,
        replace(x.att10,'"')         att10,
		replace(x.att11,'"')         att11,
        replace(x.att12,'"')         att12,
		replace(x.att13,'"')         att13,
        replace(x.att14,'"')         att14,
		replace(x.att15,'"')         att15,
        replace(x.att16,'"')         att16,
		replace(x.att17,'"')         att17,
        replace(x.att18,'"')         att18,
		replace(x.att19,'"')         att19,
        replace(x.att20,'"')         att20
 FROM xmltable('//COMPETENCE_ELEMENTS/rows'
          PASSING l_competence_xml
		  COLUMNS  xml             XMLType          path '.'
		          ,comp_ele_id     number           path 'COMPETENCE_ELEMENT_ID/value'
		          ,type_code       VARCHAR2(100)    path 'TYPE_CODE/value'
				  ,competence_id   number           path 'COMPETENCE_ID/value/id'
				  ,prof_level_id   number           path 'PROFICIENCY_LEVEL_ID/value/id'
				  ,high_prof_level_id number        path 'HIGH_PROFICIENCY_LEVEL_ID/value/id'
				  ,mand_code       varchar2(10)     path 'MANDATORY_CODE/value/id'
				  ,att_cat         varchar2(100)    path 'ATTRIBUTE_CATEGORY/value'
				  ,att1            varchar2(100)    path 'ATTRIBUTE1/value'
				  ,att2            varchar2(100)    path 'ATTRIBUTE2/value'
				  ,att3            varchar2(100)    path 'ATTRIBUTE3/value'
				  ,att4            varchar2(100)    path 'ATTRIBUTE4/value'
				  ,att5            varchar2(100)    path 'ATTRIBUTE5/value'
				  ,att6            varchar2(100)    path 'ATTRIBUTE6/value'
				  ,att7            varchar2(100)    path 'ATTRIBUTE7/value'
				  ,att8            varchar2(100)    path 'ATTRIBUTE8/value'
				  ,att9            varchar2(100)    path 'ATTRIBUTE9/value'
				  ,att10           varchar2(100)    path 'ATTRIBUTE10/value'
				  ,att11           varchar2(100)    path 'ATTRIBUTE11/value'
				  ,att12           varchar2(100)    path 'ATTRIBUTE12/value'
				  ,att13           varchar2(100)    path 'ATTRIBUTE13/value'
				  ,att14           varchar2(100)    path 'ATTRIBUTE14/value'
				  ,att15           varchar2(100)    path 'ATTRIBUTE15/value'
				  ,att16           varchar2(100)    path 'ATTRIBUTE16/value'
				  ,att17           varchar2(100)    path 'ATTRIBUTE17/value'
				  ,att18           varchar2(100)    path 'ATTRIBUTE18/value'
				  ,att19           varchar2(100)    path 'ATTRIBUTE19/value'
				  ,att20           varchar2(100)    path 'ATTRIBUTE20/value'
		    )x;

  CURSOR csr_agency_vacancy IS
  SELECT x.agency_id                                           agency_id
        ,x.max_allowed_app                                     max_allowed_app
        ,fnd_date.canonical_to_date(replace(x.start_date,'"')) start_date
        ,fnd_date.canonical_to_date(replace(x.end_date,'"'))   end_date
        ,replace(x.manage_app_allowed,'"')                     manage_app_allowed
  FROM xmltable('//AGENCY_VACANCIES/rows'
          PASSING l_agency_vac_xml
		  COLUMNS  xml             XMLType          path '.'
                  ,agency_id       number           path 'AGENCY_ID/value/id'
                  ,max_allowed_app number           path 'MAX_ALLOWED_APPLICANTS/value'
                  ,start_date      varchar2(30)     path 'START_DATE/value'
                  ,end_date        varchar2(30)     path 'END_DATE/value'
                  ,manage_app_allowed  varchar2(10) path 'MANAGE_APPLICANTS_ALLOWED/value/id'                 
          )x;

  CURSOR csr_team_member IS
  SELECT x.person_id                                           person_id
        ,fnd_date.canonical_to_date(replace(x.start_date,'"')) start_date
        ,fnd_date.canonical_to_date(replace(x.end_date,'"'))   end_date
        ,replace(x.update_allowed,'"')                         update_allowed
        ,replace(x.delete_allowed,'"')                         delete_allowed
  FROM xmltable('//TEAM_MEMBERS/rows'
          PASSING l_team_members_xml
		  COLUMNS  xml             XMLType          path '.'
                  ,person_id       number           path 'PERSON_ID/value/id'
                  ,start_date      varchar2(30)     path 'START_DATE/value'
                  ,end_date        varchar2(30)     path 'END_DATE/value'
                  ,update_allowed  varchar2(10)     path 'UPDATE_ALLOWED/value/id'                 
                  ,delete_allowed  varchar2(10)     path 'DELETE_ALLOWED/value/id'
          )x;

  CURSOR csr_rec_activity IS
  SELECT trunc(sysdate) start_date
        ,fnd_date.canonical_to_date(replace(x.end_date,'"'))   end_date
        ,replace(x.rec_site_name,'"')                          site_name
  FROM xmltable('//RECRUITMENT_ACTIVITIES/rows'
          PASSING l_rec_activities_xml
		  COLUMNS  xml             XMLType          path '.'
                  ,start_date      varchar2(30)     path 'DATE_START/value'
                  ,end_date        varchar2(30)     path 'DATE_END/value'
                  ,rec_site_name   varchar2(400)    path 'RECRUITING_SITE_NAME/value'
          )x;
          
  CURSOR csr_vacancy_attachments IS
  SELECT replace(x.title,'"')                       title
        ,replace(x.description,'"')                 description
        ,x.file_id                                  file_id
        ,replace(x.entity_name,'"')                 entity_name
  FROM xmltable('//ATTACHMENTS/rows'
          PASSING l_attachments_xml
		  COLUMNS  xml             XMLType          path '.'
                  ,title           varchar2(100)    path 'TITLE/value'
                  ,description     varchar2(1000)   path 'DESCRIPTION/value'
                  ,file_id         number           path 'FILE_ID/value' 
                  ,entity_name     varchar2(100)     path 'ENTITY_NAME/value'           
          )x;

  CURSOR csr_attchment_exists
  IS
  Select 'Y'
  from fnd_attached_documents
  where entity_name in ('IRC_EXT_VAC_APPROVED', 'IRC_INT_VAC_APPROVED','IRC_EXT_VAC','IRC_INT_VAC')
  and  pk1_value = to_char(l_vacancy_id);

  l_var_comp_ele   xxas_irc_variable_comp_ele;
  l_per_comp_ele   XXAS_PER_COMPETENCE_ELE;
  l_irc_agency_vac XXAS_IRC_AGENCY_VACANCY;
  l_irc_team_mem   XXAS_IRC_REC_TEAM_MEMBER;
  l_per_rec_act    XXAS_PER_REC_ACTIVITY;
  l_attach_typ     xxas_attachment_type;
  l_recruitment_activity_id  per_recruitment_activities.recruitment_activity_id%TYPE;
  l_exists         varchar2(5) := 'N';

BEGIN
IF gc_log.proc=1 THEN gc_log.procedure_start(c_method); END IF;

l_data := xxas_emp_view_ui_pkg.transform_jsonxml_form_data
              (p_form_data_xml);

  log_statement(c_method,'form data'||substr(p_form_data_xml.getstringval(),1,3800));
  log_statement(c_method,'form data'||substr(p_form_data_xml.getstringval(),3801,7600));
  --
  -- Retrieve the form values
  --

  IF l_data.exists('PERSON_ID') THEN
    l_person_id := l_data('PERSON_ID');
  END IF;

  IF l_data.exists('VACANCY_ID') THEN
    l_vacancy_id := l_data('VACANCY_ID');
  END IF;

  IF l_data.exists('TRANSACTION_ID') THEN
    l_transaction_id := l_data('TRANSACTION_ID');
  END IF;

  IF l_data.exists('VACANCY_NAME') THEN
    l_vacancy_name := l_data('VACANCY_NAME');
  END IF;

  IF l_data.exists('DATE_FROM') THEN
    l_date_from := l_data('DATE_FROM');
  END IF;

  IF l_data.exists('DATE_TO') THEN
    l_date_to := l_data('DATE_TO');
  END IF;

  IF l_data.exists('NUMBER_OF_OPENINGS') THEN
    l_no_of_openings := l_data('NUMBER_OF_OPENINGS');
  END IF;

  IF l_data.exists('BUDGET_VALUE') THEN
    l_budget_value := l_data('BUDGET_VALUE');
  END IF;

  IF l_data.exists('BUSINESS_GROUP_ID') THEN
    l_bg_id := l_data('BUSINESS_GROUP_ID');
  END IF;

  IF l_data.exists('MANAGER_ID') THEN
    l_manager_id := l_data('MANAGER_ID');
  END IF;

  IF l_data.exists('POSITION_ID') THEN
    l_position_id := l_data('POSITION_ID');
  END IF;

  IF l_data.exists('JOB_ID') THEN
    l_job_id := l_data('JOB_ID');
  END IF;

  IF l_data.exists('LOCATION_ID') THEN
    l_location_id := l_data('LOCATION_ID');
  END IF;

  IF l_data.exists('ORGANIZATION_ID') THEN
    l_org_id := l_data('ORGANIZATION_ID');
  END IF;

  IF l_data.exists('GRADE_ID') THEN
    l_grade_id := l_data('GRADE_ID');
  END IF;

  IF l_data.exists('PROFESSIONAL_AREA') THEN
    l_prof_area := l_data('PROFESSIONAL_AREA');
  END IF;

  IF l_data.exists('EMPLOYEE_FLAG') THEN
    l_emp_flag := l_data('EMPLOYEE_FLAG');
  END IF;

  IF l_data.exists('CONTRACTOR_FLAG') THEN
    l_cont_flag := l_data('CONTRACTOR_FLAG');
  END IF;

  IF l_data.exists('EMPLOYMENT_CATEGORY') THEN
    l_emp_cat := l_data('EMPLOYMENT_CATEGORY');
  END IF;

  IF l_data.exists('ATTRIBUTE_CATEGORY') THEN
    l_attribute_category := l_data('ATTRIBUTE_CATEGORY');
  END IF;

  IF l_data.exists('ATTRIBUTE1') THEN
    l_attribute1 := l_data('ATTRIBUTE1');
  END IF;

  IF l_data.exists('ATTRIBUTE2') THEN
    l_attribute2 := l_data('ATTRIBUTE2');
  END IF;

  IF l_data.exists('ATTRIBUTE3') THEN
    l_attribute3 := l_data('ATTRIBUTE3');
  END IF;

  IF l_data.exists('ATTRIBUTE4') THEN
    l_attribute4 := l_data('ATTRIBUTE4');
  END IF;

  IF l_data.exists('ATTRIBUTE5') THEN
    l_attribute5 := l_data('ATTRIBUTE5');
  END IF;

  IF l_data.exists('ATTRIBUTE6') THEN
    l_attribute6 := l_data('ATTRIBUTE6');
  END IF;

  IF l_data.exists('ATTRIBUTE7') THEN
    l_attribute7 := l_data('ATTRIBUTE7');
  END IF;

  IF l_data.exists('ATTRIBUTE8') THEN
    l_attribute8 := l_data('ATTRIBUTE8');
  END IF;

  IF l_data.exists('ATTRIBUTE9') THEN
    l_attribute9 := l_data('ATTRIBUTE9');
  END IF;

  IF l_data.exists('ATTRIBUTE10') THEN
    l_attribute10 := l_data('ATTRIBUTE10');
  END IF;

  IF l_data.exists('ATTRIBUTE11') THEN
    l_attribute11 := l_data('ATTRIBUTE11');
  END IF;

  IF l_data.exists('ATTRIBUTE12') THEN
    l_attribute12 := l_data('ATTRIBUTE12');
  END IF;

  IF l_data.exists('ATTRIBUTE13') THEN
    l_attribute13 := l_data('ATTRIBUTE13');
  END IF;

  IF l_data.exists('ATTRIBUTE14') THEN
    l_attribute14 := l_data('ATTRIBUTE14');
  END IF;

  IF l_data.exists('ATTRIBUTE15') THEN
    l_attribute15 := l_data('ATTRIBUTE15');
  END IF;

  IF l_data.exists('ATTRIBUTE16') THEN
    l_attribute16 := l_data('ATTRIBUTE16');
  END IF;

  IF l_data.exists('ATTRIBUTE17') THEN
    l_attribute17 := l_data('ATTRIBUTE17');
  END IF;

  IF l_data.exists('ATTRIBUTE18') THEN
    l_attribute18 := l_data('ATTRIBUTE18');
  END IF;

  IF l_data.exists('ATTRIBUTE19') THEN
    l_attribute19 := l_data('ATTRIBUTE19');
  END IF;

  IF l_data.exists('ATTRIBUTE20') THEN
    l_attribute20 := l_data('ATTRIBUTE20');
  END IF;

  IF l_data.exists('ATTRIBUTE21') THEN
    l_attribute21 := l_data('ATTRIBUTE21');
  END IF;

  IF l_data.exists('ATTRIBUTE22') THEN
    l_attribute22 := l_data('ATTRIBUTE22');
  END IF;

  IF l_data.exists('ATTRIBUTE23') THEN
    l_attribute23 := l_data('ATTRIBUTE23');
  END IF;

  IF l_data.exists('ATTRIBUTE24') THEN
    l_attribute24 := l_data('ATTRIBUTE24');
  END IF;

  IF l_data.exists('ATTRIBUTE25') THEN
    l_attribute25 := l_data('ATTRIBUTE25');
  END IF;

  IF l_data.exists('ATTRIBUTE26') THEN
    l_attribute26 := l_data('ATTRIBUTE26');
  END IF;

  IF l_data.exists('ATTRIBUTE27') THEN
    l_attribute27 := l_data('ATTRIBUTE27');
  END IF;

  IF l_data.exists('ATTRIBUTE28') THEN
    l_attribute28 := l_data('ATTRIBUTE28');
  END IF;

  IF l_data.exists('ATTRIBUTE29') THEN
    l_attribute29 := l_data('ATTRIBUTE29');
  END IF;

  IF l_data.exists('ATTRIBUTE30') THEN
    l_attribute30 := l_data('ATTRIBUTE30');
  END IF;

  IF l_data.exists('MIN_SALARY') THEN
    l_min_sal := l_data('MIN_SALARY');
  END IF;

  IF l_data.exists('MAX_SALARY') THEN
    l_max_sal := l_data('MAX_SALARY');
  END IF;

  IF l_data.exists('SALARY_CURRENCY') THEN
    l_sal_currency := l_data('SALARY_CURRENCY');
  END IF;

  IF l_data.exists('BUDGET_TYPE_CODE') THEN
    l_budget_type_code := l_data('BUDGET_TYPE_CODE');
  END IF;

  IF l_data.exists('SALARY_PERIOD_CODE') THEN
    l_sal_per_code := l_data('SALARY_PERIOD_CODE');
  END IF;

  IF l_data.exists('TRAVEL_PERCENTAGE') THEN
    l_travel_per := l_data('TRAVEL_PERCENTAGE');
  END IF;

  IF l_data.exists('WORK_AT_HOME') THEN
    l_work_at_home := l_data('WORK_AT_HOME');
  END IF;

  IF l_data.exists('MIN_QUAL_LEVEL_ID') THEN
    l_min_qual_level := l_data('MIN_QUAL_LEVEL_ID');
  END IF;

  IF l_data.exists('MAX_QUAL_LEVEL_ID') THEN
    l_max_qual_level := l_data('MAX_QUAL_LEVEL_ID');
  END IF;

  IF l_data.exists('ASSESSMENT_ID') THEN
    l_assessment_id := l_data('ASSESSMENT_ID');
  END IF;

  IF l_data.exists('SECURITY_METHOD') THEN
    l_sec_method := l_data('SECURITY_METHOD');
  END IF;

  IF l_data.exists('RECRUITER_ID') THEN
    l_recruiter_id := l_data('RECRUITER_ID');
  END IF;

  IF l_data.exists('POSTING_NAME') THEN
    l_posting_name := l_data('POSTING_NAME');
  END IF;

  IF l_data.exists('POSTING_JOB_TITLE') THEN
    l_posting_job_title := l_data('POSTING_JOB_TITLE');
  END IF;

  IF l_data.exists('POSTING_ORG_NAME') THEN
    l_posting_org_name := l_data('POSTING_ORG_NAME');
  END IF;

  IF l_data.exists('POSTING_ORG_DESCRIPTION') THEN
    l_posting_org_desc := l_data('POSTING_ORG_DESCRIPTION');
  END IF;

  IF l_data.exists('POSTING_BRIEF_DESCRIPTION') THEN
    l_posting_brief_desc := l_data('POSTING_BRIEF_DESCRIPTION');
  END IF;

  IF l_data.exists('POSTING_DETAILED_DESCRIPTION') THEN
    l_posting_det_desc := l_data('POSTING_DETAILED_DESCRIPTION');
  END IF;

  IF l_data.exists('POSTING_JOB_REQUIREMENTS') THEN
    l_posting_job_req := l_data('POSTING_JOB_REQUIREMENTS');
  END IF;

  IF l_data.exists('POSTING_ADDITIONAL_DETAILS') THEN
    l_posting_addl_det := l_data('POSTING_ADDITIONAL_DETAILS');
  END IF;

  IF l_data.exists('POSTING_HOW_TO_APPLY') THEN
    l_posting_how_to_apply := l_data('POSTING_HOW_TO_APPLY');
  END IF;

  IF l_data.exists('IMAGE_URL') THEN
    l_image_url := l_data('IMAGE_URL');
  END IF;

  IF l_data.exists('DISPLAY_RECRUITER_INFO') THEN
    l_display_rec_info := l_data('DISPLAY_RECRUITER_INFO');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE_CATEGORY') THEN
    l_ipc_attribute_category := l_data('IPC_ATTRIBUTE_CATEGORY');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE1') THEN
    l_ipc_attribute1 := l_data('IPC_ATTRIBUTE1');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE2') THEN
    l_ipc_attribute2 := l_data('IPC_ATTRIBUTE2');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE3') THEN
    l_ipc_attribute3 := l_data('IPC_ATTRIBUTE3');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE4') THEN
    l_ipc_attribute4 := l_data('IPC_ATTRIBUTE4');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE5') THEN
    l_ipc_attribute5 := l_data('IPC_ATTRIBUTE5');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE6') THEN
    l_ipc_attribute6 := l_data('IPC_ATTRIBUTE6');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE7') THEN
    l_ipc_attribute7 := l_data('IPC_ATTRIBUTE7');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE8') THEN
    l_ipc_attribute8 := l_data('IPC_ATTRIBUTE8');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE9') THEN
    l_ipc_attribute9 := l_data('IPC_ATTRIBUTE9');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE10') THEN
    l_ipc_attribute10 := l_data('IPC_ATTRIBUTE10');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE11') THEN
    l_ipc_attribute11 := l_data('IPC_ATTRIBUTE11');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE12') THEN
    l_ipc_attribute12 := l_data('IPC_ATTRIBUTE12');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE13') THEN
    l_ipc_attribute13 := l_data('IPC_ATTRIBUTE13');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE14') THEN
    l_ipc_attribute14 := l_data('IPC_ATTRIBUTE14');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE15') THEN
    l_ipc_attribute15 := l_data('IPC_ATTRIBUTE15');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE16') THEN
    l_ipc_attribute16 := l_data('IPC_ATTRIBUTE16');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE17') THEN
    l_ipc_attribute17 := l_data('IPC_ATTRIBUTE17');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE18') THEN
    l_ipc_attribute18 := l_data('IPC_ATTRIBUTE18');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE19') THEN
    l_ipc_attribute19 := l_data('IPC_ATTRIBUTE19');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE20') THEN
    l_ipc_attribute20 := l_data('IPC_ATTRIBUTE20');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE21') THEN
    l_ipc_attribute21 := l_data('IPC_ATTRIBUTE21');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE22') THEN
    l_ipc_attribute22 := l_data('IPC_ATTRIBUTE22');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE23') THEN
    l_ipc_attribute23 := l_data('IPC_ATTRIBUTE23');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE24') THEN
    l_ipc_attribute24 := l_data('IPC_ATTRIBUTE24');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE25') THEN
    l_ipc_attribute25 := l_data('IPC_ATTRIBUTE25');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE26') THEN
    l_ipc_attribute26 := l_data('IPC_ATTRIBUTE26');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE27') THEN
    l_ipc_attribute27 := l_data('IPC_ATTRIBUTE27');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE28') THEN
    l_ipc_attribute28 := l_data('IPC_ATTRIBUTE28');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE29') THEN
    l_ipc_attribute29 := l_data('IPC_ATTRIBUTE29');
  END IF;

  IF l_data.exists('IPC_ATTRIBUTE30') THEN
    l_ipc_attribute30 := l_data('IPC_ATTRIBUTE30');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE_CATEGORY') THEN
    l_isc_attribute_category := l_data('ISC_ATTRIBUTE_CATEGORY');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE1') THEN
    l_isc_attribute1 := l_data('ISC_ATTRIBUTE1');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE2') THEN
    l_isc_attribute2 := l_data('ISC_ATTRIBUTE2');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE3') THEN
    l_isc_attribute3 := l_data('ISC_ATTRIBUTE3');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE4') THEN
    l_isc_attribute4 := l_data('ISC_ATTRIBUTE4');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE5') THEN
    l_isc_attribute5 := l_data('ISC_ATTRIBUTE5');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE6') THEN
    l_isc_attribute6 := l_data('ISC_ATTRIBUTE6');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE7') THEN
    l_isc_attribute7 := l_data('ISC_ATTRIBUTE7');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE8') THEN
    l_isc_attribute8 := l_data('ISC_ATTRIBUTE8');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE9') THEN
    l_isc_attribute9 := l_data('ISC_ATTRIBUTE9');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE10') THEN
    l_isc_attribute10 := l_data('ISC_ATTRIBUTE10');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE11') THEN
    l_isc_attribute11 := l_data('ISC_ATTRIBUTE11');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE12') THEN
    l_isc_attribute12 := l_data('ISC_ATTRIBUTE12');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE13') THEN
    l_isc_attribute13 := l_data('ISC_ATTRIBUTE13');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE14') THEN
    l_isc_attribute14 := l_data('ISC_ATTRIBUTE14');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE15') THEN
    l_isc_attribute15 := l_data('ISC_ATTRIBUTE15');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE16') THEN
    l_isc_attribute16 := l_data('ISC_ATTRIBUTE16');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE17') THEN
    l_isc_attribute17 := l_data('ISC_ATTRIBUTE17');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE18') THEN
    l_isc_attribute18 := l_data('ISC_ATTRIBUTE18');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE19') THEN
    l_isc_attribute19 := l_data('ISC_ATTRIBUTE19');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE20') THEN
    l_isc_attribute20 := l_data('ISC_ATTRIBUTE20');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE21') THEN
    l_isc_attribute21 := l_data('ISC_ATTRIBUTE21');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE22') THEN
    l_isc_attribute22 := l_data('ISC_ATTRIBUTE22');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE23') THEN
    l_isc_attribute23 := l_data('ISC_ATTRIBUTE23');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE24') THEN
    l_isc_attribute24 := l_data('ISC_ATTRIBUTE24');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE25') THEN
    l_isc_attribute25 := l_data('ISC_ATTRIBUTE25');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE26') THEN
    l_isc_attribute26 := l_data('ISC_ATTRIBUTE26');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE27') THEN
    l_isc_attribute27 := l_data('ISC_ATTRIBUTE27');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE28') THEN
    l_isc_attribute28 := l_data('ISC_ATTRIBUTE28');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE29') THEN
    l_isc_attribute29 := l_data('ISC_ATTRIBUTE29');
  END IF;

  IF l_data.exists('ISC_ATTRIBUTE30') THEN
    l_isc_attribute30 := l_data('ISC_ATTRIBUTE30');
  END IF;  

  IF l_data.exists('ID_FLEX_NUM') THEN
    l_id_flex_num := l_data('ID_FLEX_NUM');
  END IF;  

  IF l_data.exists('SEGMENT1') THEN
    l_segment1 := l_data('SEGMENT1');
  END IF;

  IF l_data.exists('SEGMENT2') THEN
    l_segment2 := l_data('SEGMENT2');
  END IF;

  IF l_data.exists('SEGMENT3') THEN
    l_segment3 := l_data('SEGMENT3');
  END IF;

  IF l_data.exists('SEGMENT4') THEN
    l_segment4 := l_data('SEGMENT4');
  END IF;

  IF l_data.exists('SEGMENT5') THEN
    l_segment5 := l_data('SEGMENT5');
  END IF;

  IF l_data.exists('SEGMENT6') THEN
    l_segment6 := l_data('SEGMENT6');
  END IF;

  IF l_data.exists('SEGMENT7') THEN
    l_segment7 := l_data('SEGMENT7');
  END IF;

  IF l_data.exists('SEGMENT8') THEN
    l_segment8 := l_data('SEGMENT8');
  END IF;

  IF l_data.exists('SEGMENT9') THEN
    l_segment9 := l_data('SEGMENT9');
  END IF;

  IF l_data.exists('SEGMENT10') THEN
    l_segment10 := l_data('SEGMENT10');
  END IF;

  IF l_data.exists('SEGMENT11') THEN
    l_segment11 := l_data('SEGMENT11');
  END IF;

  IF l_data.exists('SEGMENT12') THEN
    l_segment12 := l_data('SEGMENT12');
  END IF;

  IF l_data.exists('SEGMENT13') THEN
    l_segment13 := l_data('SEGMENT13');
  END IF;

  IF l_data.exists('SEGMENT14') THEN
    l_segment14 := l_data('SEGMENT14');
  END IF;

  IF l_data.exists('SEGMENT15') THEN
    l_segment15 := l_data('SEGMENT15');
  END IF;

  IF l_data.exists('SEGMENT16') THEN
    l_segment16 := l_data('SEGMENT16');
  END IF;

  IF l_data.exists('SEGMENT17') THEN
    l_segment17 := l_data('SEGMENT17');
  END IF;

  IF l_data.exists('SEGMENT18') THEN
    l_segment18 := l_data('SEGMENT18');
  END IF;

  IF l_data.exists('SEGMENT19') THEN
    l_segment19 := l_data('SEGMENT19');
  END IF;

  IF l_data.exists('SEGMENT20') THEN
    l_segment20 := l_data('SEGMENT20');
  END IF;

  IF l_data.exists('SEGMENT21') THEN
    l_segment21 := l_data('SEGMENT21');
  END IF;

  IF l_data.exists('SEGMENT22') THEN
    l_segment22 := l_data('SEGMENT22');
  END IF;

  IF l_data.exists('SEGMENT23') THEN
    l_segment23 := l_data('SEGMENT23');
  END IF;

  IF l_data.exists('SEGMENT24') THEN
    l_segment24 := l_data('SEGMENT24');
  END IF;

  IF l_data.exists('SEGMENT25') THEN
    l_segment25 := l_data('SEGMENT25');
  END IF;

  IF l_data.exists('SEGMENT26') THEN
    l_segment26 := l_data('SEGMENT26');
  END IF;

  IF l_data.exists('SEGMENT27') THEN
    l_segment27 := l_data('SEGMENT27');
  END IF;

  IF l_data.exists('SEGMENT28') THEN
    l_segment28 := l_data('SEGMENT28');
  END IF;

  IF l_data.exists('SEGMENT29') THEN
    l_segment29 := l_data('SEGMENT29');
  END IF;

  IF l_data.exists('SEGMENT30') THEN
    l_segment30 := l_data('SEGMENT30');
  END IF;

--  l_bg_id := xxas_per_pkg.get_business_group_id(l_person_id);

  OPEN  csr_get_emp_full_name(l_person_id);
  FETCH csr_get_emp_full_name INTO l_recruiter_full_name, l_recruiter_email, l_recruiter_work_telephone;
  CLOSE csr_get_emp_full_name;

  OPEN  csr_get_bg_name(l_bg_id);
  FETCH csr_get_bg_name INTO l_bg_name;
  CLOSE csr_get_bg_name;

  OPEN  csr_get_pos_name(l_position_id);
  FETCH csr_get_pos_name INTO l_pos_name;
  CLOSE csr_get_pos_name;

  OPEN  csr_get_job_name(l_job_id);
  FETCH csr_get_job_name INTO l_job_name;
  CLOSE csr_get_job_name;

  OPEN  csr_get_grade_name(l_grade_id);
  FETCH csr_get_grade_name INTO l_grade_name;
  CLOSE csr_get_grade_name;

  OPEN  csr_get_org_name(l_org_id);
  FETCH csr_get_org_name INTO l_org_name;
  CLOSE csr_get_org_name;

  OPEN  csr_get_loc_code(l_location_id);
  FETCH csr_get_loc_code INTO l_location_code;
  CLOSE csr_get_loc_code;

  IF l_transaction_id IS NOT NULL THEN
    OPEN csr_get_trans_ids(l_transaction_id);
    FETCH csr_get_trans_ids INTO l_requisition_id, l_posting_content_id, l_search_criteria_id;
    CLOSE csr_get_trans_ids;  
  ELSE
    OPEN csr_get_ids(l_vacancy_id);
    FETCH csr_get_ids INTO l_requisition_id, l_posting_content_id, l_search_criteria_id;
    CLOSE csr_get_ids;
  END IF;

  l_person_id  := l_manager_id;

  IF p_form_data_xml is not null THEN

    FOR csr_rec IN csr_data LOOP
      IF csr_rec.field_name is not null THEN
        IF csr_rec.field_name = 'COMPETENCE_ELEMENTS' THEN
           l_competence_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'VARIABLE_COMP_ELEMENTS' THEN 
           l_var_comp_ele_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'TEAM_MEMBERS' THEN
           l_team_members_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'AGENCY_VACANCIES' THEN
           l_agency_vac_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'RECRUITMENT_ACTIVITIES' THEN
           l_rec_activities_xml := csr_rec.field_xml;
        ELSIF csr_rec.field_name = 'ATTACHMENTS' THEN
           l_attachments_xml := csr_rec.field_xml;
        END IF;
      END IF;
    END LOOP;

  END IF;

  -- For REC Activity 
  l_rec_activity := xxas_per_rec_activity_tbl();

  IF l_rec_activity.count > 0 THEN
    l_rec_activity.DELETE;
  END IF;

  --
  -- Get all per rec activity details
  --
  FOR csr_rec_act IN csr_rec_activity LOOP

    SELECT  xxas_per_rec_activity
            (per_recruitment_activities_s.nextval
            ,per_recruitment_activity_for_s.nextval
            ,l_vacancy_id
            ,csr_rec_act.start_date
            ,csr_rec_act.end_date
            ,null
            ,l_posting_content_id
            ,irs.recruiting_site_id
            ,csr_rec_act.site_name
            ,1
            ,1
            ,trunc(sysdate)
            ,fnd_global.user_id
            ,fnd_global.login_id
            ,fnd_global.user_id
            ,trunc(sysdate)
              )
       INTO l_per_rec_act
    FROM irc_all_recruiting_sites_vl irs
  WHERE  irs.site_name                = csr_rec_act.site_name;

    l_rec_activity.extend;
    l_rec_activity(l_rec_activity.count) := l_per_rec_act;
    l_recruitment_activity_id := l_rec_activity(l_rec_activity.count).recruitment_activity_id;
    l_rec_activity(l_rec_activity.count).name := substr(l_vacancy_name,1,30-length(l_recruitment_activity_id)) || l_recruitment_activity_id;
  END LOOP;

  -- For Team Member
  l_team_member := xxas_irc_rec_team_member_tbl();

  IF l_team_member.count > 0 THEN
    l_team_member.DELETE;
  END IF;

  --
  -- Get all team details
  --
  FOR csr_rec_tm IN csr_team_member LOOP

    SELECT xxas_irc_rec_team_member
           (irc_rec_team_members_s.nextval
           ,csr_rec_tm.person_id
           ,pap.party_id
           ,l_vacancy_id
           ,null
           ,csr_rec_tm.start_date
           ,csr_rec_tm.end_date
           ,csr_rec_tm.update_allowed
           ,csr_rec_tm.delete_allowed
           ,1
           ,trunc(sysdate)
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,trunc(sysdate)
         )
        INTO l_irc_team_mem
    FROM per_all_people_f pap
    WHERE pap.person_id = csr_rec_tm.person_id
    AND   trunc(sysdate) between pap.effective_start_date and pap.effective_end_date;

    l_team_member.extend;
    l_team_member(l_team_member.count) := l_irc_team_mem;  
  END LOOP;

  -- For Agency Vacancy
  l_agency_vacancy := xxas_irc_agency_vacancy_tbl();

  IF l_agency_vacancy.count > 0 THEN
    l_agency_vacancy.DELETE;
  END IF;

  --
  -- Get all agency vac dettails
  --
  FOR csr_rec_vac IN csr_agency_vacancy LOOP

    SELECT xxas_irc_agency_vacancy
            (irc_agency_vacancies_s.nextval
            ,csr_rec_vac.agency_id
            ,pov.vendor_name
            ,l_vacancy_id
            ,csr_rec_vac.start_date
            ,csr_rec_vac.end_date
            ,csr_rec_vac.max_allowed_app
            ,null -- nvl(csr_rec_vac.manage_app_allowed,'N')
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,null
            ,1
            ,trunc(sysdate)
            ,fnd_global.user_id
            ,fnd_global.login_id
            ,fnd_global.user_id
            ,trunc(sysdate)
            ) INTO l_irc_agency_vac
    FROM po_vendors pov
  WHERE  csr_rec_vac.agency_id  = pov.vendor_id;

    l_agency_vacancy.extend;
    l_agency_vacancy(l_agency_vacancy.count) := l_irc_agency_vac;  
  END LOOP;

  -- For Variable Component Element
  l_variable_comp_ele := xxas_irc_variable_comp_ele_tbl();

  IF l_variable_comp_ele.count > 0 then
    l_variable_comp_ele.DELETE;
  END IF;
  --
  -- Get all variable_comp_ele
  --
  FOR csr_rec_comp IN csr_var_comp LOOP
    SELECT xxas_irc_variable_comp_ele
           (l_vacancy_id
           ,csr_rec_comp.var_comp_code
           ,hrlvc.meaning
           ,1
           ,trunc(sysdate)
           ,fnd_global.user_id
           ,fnd_global.login_id
           ,fnd_global.user_id
           ,trunc(sysdate)
           ) variable_comp_ele into l_var_comp_ele
    FROM   hr_lookups hrlvc
    WHERE  hrlvc.lookup_type = 'IRC_VARIABLE_COMP_ELEMENT'
    AND    hrlvc.lookup_code = csr_rec_comp.var_comp_code;

    l_variable_comp_ele.extend;
    l_variable_comp_ele(l_variable_comp_ele.count) := l_var_comp_ele;  
  END LOOP;

  -- For Competencies
  l_competences := xxas_per_competence_ele_tbl();

  IF l_competences.count > 0 THEN
    l_competences.DELETE;
  END IF;
  --
  -- Get all competences
  --
  FOR csr_rec IN csr_per_comp LOOP

    SELECT xxas_per_competence_ele
          (nvl(csr_rec.comp_ele_id,per_competence_elements_s.nextval)
          ,1
          ,csr_rec.type_code
          ,xxas_util_pkg.decode_lookup('COMPETENCE_ELEMENT_TYPE', csr_rec.type_code)
          ,csr_rec.competence_id
          ,pc.name
          ,csr_rec.prof_level_id
          ,null
          ,csr_rec.high_prof_level_id
          ,null
          ,csr_rec.mand_code
          ,xxas_util_pkg.decode_lookup('IRC_SKILL_DESIRABILITY', csr_rec.mand_code)
          ,csr_rec.att_cat
          ,csr_rec.att1
          ,csr_rec.att2
          ,csr_rec.att3
          ,csr_rec.att4
          ,csr_rec.att5
          ,csr_rec.att6
          ,csr_rec.att7
          ,csr_rec.att8
          ,csr_rec.att9
          ,csr_rec.att10
          ,csr_rec.att11
          ,csr_rec.att12
          ,csr_rec.att13
          ,csr_rec.att14
          ,csr_rec.att15
          ,csr_rec.att16
          ,csr_rec.att17
          ,csr_rec.att18
          ,csr_rec.att19
          ,csr_rec.att20
          ,trunc(sysdate)
          ,fnd_global.user_id
          ,fnd_global.login_id
          ,fnd_global.user_id
          ,trunc(sysdate)
          )
      INTO l_per_comp_ele
    FROM  per_competences pc
 --        ,per_rating_levels_vl prlmin
 --        ,per_rating_levels_vl prlmax
    WHERE  csr_rec.competence_id = pc.competence_id(+);
 --   AND    csr_rec.prof_level_id = prlmin.rating_level_id(+)
 --   AND    csr_rec.high_prof_level_id = prlmax.rating_level_id(+); 

    l_competences.extend;
    l_competences(l_competences.count) := l_per_comp_ele;  
  END LOOP;

  -- For Attachments

  l_attachments:= xxas_attachment_table();

  FOR csr_rec_attach in csr_vacancy_attachments
  LOOP
      SELECT xxas_attachment_type 
           (null
           ,null
           ,csr_rec_attach.entity_name 
           ,to_char (l_vacancy_id)  
           ,null
           ,null
           ,null
           ,null
           ,'MISC'  
           ,null  
           ,null
           ,null
           ,csr_rec_attach.title
           ,csr_rec_attach.description
           ,csr_rec_attach.file_id
           ,null
           ,null
           ,null
		       ,null
           ,sysdate
           ,fnd_global.user_id
           ,sysdate
           ,fnd_global.user_id
           ) INTO l_attach_typ
      FROM DUAL;
      
      l_attachments.extend;
      l_attachments(l_attachments.count) := l_attach_typ;  

  END LOOP;
  
  IF l_transaction_id IS NULL THEN
     SELECT hr_api_transactions_s.nextval
     INTO l_transaction_id
     FROM dual;
	 l_is_draft := 'N';
  END IF;

  --If vacancy does not exist(draft status) Transaction type is INSERT
  --Else Transaction type is UPDATE 
  OPEN csr_vacancy_exists(l_vacancy_id);
  FETCH csr_vacancy_exists INTO l_vacancy_exists;
  CLOSE csr_vacancy_exists;
  
  IF l_vacancy_exists IS NULL THEN 
    l_transaction_type := 'INSERT';
  END IF;
  
  l_vacancy := xxas_per_all_vacancy(
            vacancy_id                     => l_vacancy_id
           ,href                           => null         
           ,vacancy_name                   => l_vacancy_name
           ,edit_allowed                   => xxas_util_pkg.flag_to_boolean(xxas_com_sec_pkg.test_instance('XXAS_DATA_TRANS_NO_PEND_VIEW', to_char(l_transaction_id)))  --edit_allowed
           ,edit_label                     => xxas_com_msg_pkg.get_string('XXAS_RB_COM_EDIT') --edit_label
           ,delete_allowed                 => xxas_util_pkg.flag_to_boolean(xxas_com_sec_pkg.test_instance('XXAS_DATA_TRANS_NO_PEND_DEL', to_char(l_transaction_id)))  --delete_allowed
           ,delete_label                   => xxas_com_msg_pkg.get_string('XXAS_RB_COM_DEL') --delete_label
           ,num_total                      => 0 --num_total
           ,num_active                     => 0 --num_active
           ,num_term                       => 0 --num_term
           ,num_interview1                 => 0 --num_interview1
           ,num_interview2                 => 0 --num_interview2
           ,num_offer                      => 0 --num_offer
           ,num_accepted                   => 0 --num_accepted
           ,num_not_set                    => 0 --num_not_set
           ,num_no                         => 0 --num_no
           ,num_unsure                     => 0 --num_unsure
           ,num_yes                        => 0 --num_yes
           ,view_schema_href               => null --view_schema_href
           ,form_schema_href               => null --form_schema_href
           ,applicants_href                => xxas_vacancy_pkg.get_applicants_href(l_vacancy_id) --applicants_href
           ,applicants_export_href         => xxas_vacancy_pkg.get_applicants_export_href(l_vacancy_id) --applicants_export_href
           ,offers_href                    => xxas_vacancy_pkg.get_offers_href(l_vacancy_id) --offers_href
           ,ad_hoc_comms_href              => xxas_vacancy_pkg.get_ad_hoc_comms_href(l_vacancy_id) --ad_hoc_comms_href           
           ,transaction_id                 => l_transaction_id
           ,transaction_type               => l_transaction_type
           ,hr_api_status_code             => 'S'
           ,workflow_status_code           => xxas_selfservice_workflow_pkg.transcode_status('REVIEW')
           ,workflow_status_meaning        => xxas_selfservice_workflow_pkg.decode_status(xxas_selfservice_workflow_pkg.transcode_status('REVIEW'))
           ,item_type                      => g_item_type
           ,item_key                       => null
           ,business_group_id              => l_bg_id
           ,business_group_name            => l_bg_name
           ,position_id                    => l_position_id
           ,position_name                  => l_pos_name
           ,job_id                         => l_job_id
           ,job_name                       => l_job_name
           ,grade_id                       => l_grade_id
           ,grade_name                     => l_grade_name
           ,organization_id                => l_org_id
           ,organization_name              => l_org_name
           ,requisition_id                 => l_requisition_id
           ,req_object_version_number      => 1
           ,people_group_id                => null
           ,location_id                    => l_location_id
           ,location_name                  => l_location_code
           ,recruiter_id                   => l_recruiter_id
           ,date_from                      => nvl(xxas_xml_util_pkg.oafstring_to_date(l_date_from),trunc(sysdate))
           ,date_to                        => xxas_xml_util_pkg.oafstring_to_date(l_date_to)
           ,description                    => null
           ,number_of_openings             => l_budget_value   --l_no_of_openings
           ,status_code                    => 'UNAPPROVED'
           ,status_meaning                 => xxas_util_pkg.decode_lookup('VACANCY_STATUS', 'UNAPPROVED')
           ,status_color                   => null --status_color
           ,request_id                     => null --request_id
           ,program_application_id         => null --program_application_id
           ,program_id                     => null --program_id
           ,program_update_date            => null --program_update_date
           ,attribute_category             => l_attribute_category
           ,attribute1                     => l_attribute1
           ,attribute2                     => l_attribute2
           ,attribute3                     => l_attribute3
           ,attribute4                     => l_attribute4
           ,attribute5                     => l_attribute5
           ,attribute6                     => l_attribute6
           ,attribute7                     => l_attribute7
           ,attribute8                     => l_attribute8
           ,attribute9                     => l_attribute9
           ,attribute10                    => l_attribute10
           ,attribute11                    => l_attribute11
           ,attribute12                    => l_attribute12
           ,attribute13                    => l_attribute13
           ,attribute14                    => l_attribute14
           ,attribute15                    => l_attribute15
           ,attribute16                    => l_attribute16
           ,attribute17                    => l_attribute17
           ,attribute18                    => l_attribute18
           ,attribute19                    => l_attribute19
           ,attribute20                    => l_attribute20
           ,last_update_date               => trunc(sysdate)
           ,last_updated_by                => fnd_global.user_id
           ,last_update_login              => fnd_global.login_id
           ,created_by                     => fnd_global.user_id
           ,creation_date                  => trunc(sysdate)
           ,vacancy_category_code          => null
           ,vacancy_category_meaning       => null
           ,budget_type_code               => l_budget_type_code 
           ,budget_type_meaning            => xxas_util_pkg.decode_lookup('BUDGET_MEASUREMENT_TYPE', l_budget_type_code)
           ,budget_value                   => l_budget_value
           ,manager_id                     => l_person_id
           ,security_method                => l_sec_method  --'B'   --security_method
           ,object_version_number          => 1
           ,assessment_id                  => l_assessment_id
           ,primary_posting_id             => l_posting_content_id
           ,comments                       => null --comments
           ,comments_to_approver           => null
           ,display_manager_info           => nvl(xxas_util_pkg.flag_to_boolean(l_display_rec_info),'N')
           ,display_recruiter_info         => nvl(xxas_util_pkg.flag_to_boolean(l_display_rec_info),'N')
           ,posting_name                   => l_posting_name
           ,posting_org_name               => l_posting_org_name
		   ,posting_org_description        => l_posting_org_desc
		   ,posting_job_title              => l_posting_job_title
		   ,posting_brief_description      => l_posting_brief_desc
		   ,posting_detailed_description   => l_posting_det_desc
		   ,posting_job_requirements       => l_posting_job_req
		   ,posting_additional_details     => l_posting_addl_det
		   ,posting_how_to_apply           => l_posting_how_to_apply
		   ,image_url                      => l_image_url
		   ,image_url_alt                  => null
		   ,benefit_info                   => null   -- benefit_info
		   ,ipc_attribute_category         => l_ipc_attribute_category
		   ,ipc_attribute1                 => l_ipc_attribute1
		   ,ipc_attribute2                 => l_ipc_attribute2
		   ,ipc_attribute3                 => l_ipc_attribute3
		   ,ipc_attribute4                 => l_ipc_attribute4
		   ,ipc_attribute5                 => l_ipc_attribute5
		   ,ipc_attribute6                 => l_ipc_attribute6
		   ,ipc_attribute7                 => l_ipc_attribute7
		   ,ipc_attribute8                 => l_ipc_attribute8
		   ,ipc_attribute9                 => l_ipc_attribute9
		   ,ipc_attribute10                => l_ipc_attribute10
		   ,ipc_attribute11                => l_ipc_attribute11
		   ,ipc_attribute12                => l_ipc_attribute12
		   ,ipc_attribute13                => l_ipc_attribute13
		   ,ipc_attribute14                => l_ipc_attribute14
		   ,ipc_attribute15                => l_ipc_attribute15
		   ,ipc_attribute16                => l_ipc_attribute16
		   ,ipc_attribute17                => l_ipc_attribute17
		   ,ipc_attribute18                => l_ipc_attribute18
		   ,ipc_attribute19                => l_ipc_attribute19
		   ,ipc_attribute20                => l_ipc_attribute20
		   ,ipc_attribute21                => l_ipc_attribute21
		   ,ipc_attribute22                => l_ipc_attribute22
		   ,ipc_attribute23                => l_ipc_attribute23
		   ,ipc_attribute24                => l_ipc_attribute24
		   ,ipc_attribute25                => l_ipc_attribute25
		   ,ipc_attribute26                => l_ipc_attribute26
		   ,ipc_attribute27                => l_ipc_attribute27
		   ,ipc_attribute28                => l_ipc_attribute28
		   ,ipc_attribute29                => l_ipc_attribute29
		   ,ipc_attribute30                => l_ipc_attribute30
		   ,ipc_last_update_date           => sysdate
		   ,ipc_last_updated_by            => fnd_global.user_id
		   ,ipc_last_update_login          => fnd_global.login_id
		   ,ipc_created_by                 => fnd_global.user_id
		   ,ipc_creation_date              => sysdate
		   ,ipc_object_version_number      => 1
		   ,recruiter_full_name            => l_recruiter_full_name
		   ,recruiter_email                => l_recruiter_email
		   ,recruiter_work_telephone       => l_recruiter_work_telephone
		   ,manager_full_name              => l_recruiter_full_name
		   ,manager_email                  => l_recruiter_email
		   ,manager_work_telephone         => l_recruiter_work_telephone
		   ,search_criteria_id             => l_search_criteria_id
		   ,employee_flag                  => l_emp_flag
		   ,contractor_flag                => l_cont_flag
		   ,employment_category_code       => l_emp_cat
		   ,employment_category_meaning    => null
		   ,travel_percentage              => l_travel_per
		   ,travel_percentage_meaning      => null
		   ,min_salary                     => l_min_sal
		   ,max_salary                     => l_max_sal
		   ,salary_currency                => l_sal_currency
		   ,salary_period_code             => l_sal_per_code
		   ,salary_period_meaning          => null
		   ,professional_area_code         => l_prof_area
		   ,professional_area_meaning      => null
		   ,work_at_home_code              => l_work_at_home
		   ,work_at_home_meaning           => null
		   ,min_qual_level_id              => l_min_qual_level
		   ,min_qual_level_name            => null
		   ,max_qual_level_id              => l_max_qual_level
		   ,max_qual_level_name            => null
		   ,isc_attribute_category         => l_isc_attribute_category
		   ,isc_attribute1                 => l_isc_attribute1
		   ,isc_attribute2                 => l_isc_attribute2
		   ,isc_attribute3                 => l_isc_attribute3
		   ,isc_attribute4                 => l_isc_attribute4
		   ,isc_attribute5                 => l_isc_attribute5
		   ,isc_attribute6                 => l_isc_attribute6
		   ,isc_attribute7                 => l_isc_attribute7
		   ,isc_attribute8                 => l_isc_attribute8
		   ,isc_attribute9                 => l_isc_attribute9
		   ,isc_attribute10                => l_isc_attribute10
		   ,isc_attribute11                => l_isc_attribute11
		   ,isc_attribute12                => l_isc_attribute12
		   ,isc_attribute13                => l_isc_attribute13
		   ,isc_attribute14                => l_isc_attribute14
		   ,isc_attribute15                => l_isc_attribute15
		   ,isc_attribute16                => l_isc_attribute16
		   ,isc_attribute17                => l_isc_attribute17
		   ,isc_attribute18                => l_isc_attribute18
		   ,isc_attribute19                => l_isc_attribute19
		   ,isc_attribute20                => l_isc_attribute20
		   ,isc_attribute21                => l_isc_attribute21
		   ,isc_attribute22                => l_isc_attribute22
		   ,isc_attribute23                => l_isc_attribute23
		   ,isc_attribute24                => l_isc_attribute24
		   ,isc_attribute25                => l_isc_attribute25
		   ,isc_attribute26                => l_isc_attribute26
		   ,isc_attribute27                => l_isc_attribute27
		   ,isc_attribute28                => l_isc_attribute28
		   ,isc_attribute29                => l_isc_attribute29
		   ,isc_attribute30                => l_isc_attribute30
		   ,isc_object_version_number      => 1
		   ,pg_id_flex_num                 => l_id_flex_num
		   ,segment1                       => l_segment1
		   ,segment2                       => l_segment2
		   ,segment3                       => l_segment3
		   ,segment4                       => l_segment4
		   ,segment5                       => l_segment5
		   ,segment6                       => l_segment6
		   ,segment7                       => l_segment7
		   ,segment8                       => l_segment8
		   ,segment9                       => l_segment9
		   ,segment10                      => l_segment10
		   ,segment11                      => l_segment11
		   ,segment12                      => l_segment12
		   ,segment13                      => l_segment13
		   ,segment14                      => l_segment14
		   ,segment15                      => l_segment15
		   ,segment16                      => l_segment16
		   ,segment17                      => l_segment17
		   ,segment18                      => l_segment18
		   ,segment19                      => l_segment19
		   ,segment20                      => l_segment20
		   ,segment21                      => l_segment21
		   ,segment22                      => l_segment22
		   ,segment23                      => l_segment23
		   ,segment24                      => l_segment24
		   ,segment25                      => l_segment25
		   ,segment26                      => l_segment26
		   ,segment27                      => l_segment27
		   ,segment28                      => l_segment28
		   ,segment29                      => l_segment29
		   ,segment30                      => l_segment30
		   ,variable_comp_elements         => l_variable_comp_ele
		   ,competence_elements            => l_competences
		   ,agency_vacancies               => l_agency_vacancy
		   ,team_members                   => l_team_member
		   ,recruitment_activities         => l_rec_activity
		   ,vacancy_tasks                  => l_tasks
		   ,attachments                    => l_attachments
		   ,approvers                      => xxas_selfservice_workflow_pkg.get_approvers(l_transaction_id,g_AME_APP_ID,g_AME_TRANS_TYPE)
       );

--  default_posting(l_vacancy);  

  IF l_is_draft = 'Y' THEN
     update_hrapi_transaction(l_vacancy);
  ELSE
     create_hrapi_transaction(l_vacancy);
  END IF;

  OPEN csr_attchment_exists;
  FETCH csr_attchment_exists into l_exists;
  CLOSE csr_attchment_exists;
  IF l_exists = 'N' THEN
     save_attachments(l_vacancy.attachments
                     ,l_vacancy.vacancy_id);
  END IF;
	 
  p_response_xml := xxas_com_rest_util_pkg.app_route_redirect_response
                        (p_app_route         => 'review-vacancy/' ||
                                               to_char(l_vacancy_id) || '_' || to_char(l_transaction_id)
                        ,p_target            => 'modal'
                        ,p_persist_cache     => 'false'
                        ,p_delete_form_cache => 'false');

IF gc_log.proc=1 THEN gc_log.procedure_end(c_method); END IF;

END post_update_vacancy;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_vacancy_name
-- ------------------------------------------------------------------------ *
FUNCTION get_vacancy_name(p_page_id   IN varchar2
                         ,p_object_id IN varchar2) RETURN VARCHAR2
IS

  l_PROC CONSTANT varchar2(30) := 'get_vacancy_name';
  l_keys          xxas_com_keys_type;
  l_vac_id        number;
  
BEGIN
  l_keys := xxas_emp_menu_pkg.object_id_to_keys
    (p_object_id        => p_object_id
    ,p_object_id_format => '{VACANCY_ID}_{TRANSACTION_ID}');
    
  IF l_keys is not null THEN
    l_vac_id := l_keys.get_key('VACANCY_ID');
  END IF;
  
  RETURN 'New Vacancy: IRC'||to_char(l_vac_id);
  
END get_vacancy_name;

END xxas_vacancy_pkg;
/
COMMIT;
EXIT;